#ifndef _MYSQL_SUPPORT_BACKEND_PUBLIC_INTERFACE_H_
#define _MYSQL_SUPPORT_BACKEND_PUBLIC_INTERFACE_H_

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)

#ifdef MYSQLWBMYSQLSUPPORTBACKEND_EXPORTS
#define MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC __declspec(dllexport)
#else
#define MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC __declspec(dllimport)
#endif

#else
#define MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC
#endif

#endif // _MYSQL_SUPPORT_BACKEND_PUBLIC_INTERFACE_H_

