CREATE TABLE AllDatatypes (
  BigintColumn    bigint,
  BinaryColumn    binary,
  BitColumn    bit,
  CharColumn    char,
  DateColumn    date,
  DatetimeColumn    datetime,
  Datetime2Column    datetime2,
  DatetimeoffsetColumn    datetimeoffset,
  DecimalColumn    decimal,
  FloatColumn    float,
  GeographyColumn    geography,
  GeometryColumn    geometry,
  HierarchyidColumn    hierarchyid,
  ImageColumn    image,
  IntColumn    int,
  MoneyColumn    money,
  NcharColumn    nchar,
  NtextColumn    ntext,
  NumericColumn    numeric,
  NvarcharColumn    nvarchar,
  RealColumn    real,
  SmalldatetimeColumn    smalldatetime,
  SmallintColumn    smallint,
  SmallmoneyColumn    smallmoney,
  SqlvariantColumn    sql_variant,
  SysnameColumn    sysname,
  TextColumn    text,
  TimeColumn    time,
  TimestampColumn    timestamp,
  TinyintColumn    tinyint,
  UniqueidentifierColumn    uniqueidentifier,
  VarbinaryColumn    varbinary,
  VarcharColumn    varchar,
  XmlColumn    xml,
)
