CREATE  TABLE `TimeContainer` (
  `id` INT NULL DEFAULT NULL ,
  `date_data` DATE NULL DEFAULT NULL ,
  `time_data` TIME NULL DEFAULT NULL ,
  `timestamp_data` DATETIME NULL DEFAULT NULL ,
  PRIMARY KEY (`id`) );
