<html>
  <head>
     <link rel="stylesheet" type="text/css" media="screen" href="{{STYLE_NAME}}.css">
  </head>
  <body topmargin="0" leftmargin="0">
    <table border="0" cellpadding="0" cellspacing="0" width="100%">
      <tr valign="top">
        <td style="padding-left: 20px">
          <a href="http://www.mysql.com/" target="_top"><img src="images/logo.png" alt="MySQL Homepage" border="0"></a>
        </td>
        <td class="top-title">
          {{TITLE:h}}
        </td>
      </tr>
    </table>
  </body>
</html>
