#ifndef _STDAFX_H_
#define _STDAFX_H_

#ifdef _WIN32

#pragma once

#include <algorithm>
#include <sstream>
#include <pcre.h>
#include <stdio.h>
#include <cctype>
#include <map>

#include <windows.h>

#endif

#endif // _STDAFX_H_
