
//This file is auto-generated. Do not modify.


extern const char *modelXml = "<?xml version=\"1.0\" standalone=\"no\"?> <!--*- mode: xml -*-->\n"
"<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-2.0.dtd\">\n"
"\n"
"<glade-interface>\n"
"\n"
"<widget class=\"GtkWindow\" id=\"wb_main_window\">\n"
"  <property name=\"width_request\">800</property>\n"
"  <property name=\"height_request\">600</property>\n"
"  <property name=\"visible\">True</property>\n"
"  <property name=\"title\" translatable=\"yes\"></property>\n"
"  <property name=\"type\">GTK_WINDOW_TOPLEVEL</property>\n"
"  <property name=\"window_position\">GTK_WIN_POS_CENTER</property>\n"
"  <property name=\"modal\">False</property>\n"
"  <property name=\"default_width\">800</property>\n"
"  <property name=\"resizable\">True</property>\n"
"  <property name=\"destroy_with_parent\">False</property>\n"
"  <property name=\"decorated\">True</property>\n"
"  <property name=\"skip_taskbar_hint\">False</property>\n"
"  <property name=\"skip_pager_hint\">False</property>\n"
"  <property name=\"type_hint\">GDK_WINDOW_TYPE_HINT_NORMAL</property>\n"
"  <property name=\"gravity\">GDK_GRAVITY_NORTH_WEST</property>\n"
"  <property name=\"focus_on_map\">True</property>\n"
"  <property name=\"urgency_hint\">False</property>\n"
"\n"
"  <child>\n"
"    <widget class=\"GtkVBox\" id=\"top_vbox\">\n"
"      <property name=\"visible\">True</property>\n"
"      <property name=\"homogeneous\">False</property>\n"
"      <property name=\"spacing\">0</property>\n"
"\n"
"      <child>\n"
"	<widget class=\"GtkMenuBar\" id=\"main_menu\">\n"
"	  <property name=\"visible\">True</property>\n"
"	  <property name=\"pack_direction\">GTK_PACK_DIRECTION_LTR</property>\n"
"	  <property name=\"child_pack_direction\">GTK_PACK_DIRECTION_LTR</property>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkMenuItem\" id=\"menuitem1\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"label\" translatable=\"yes\">_File</property>\n"
"	      <property name=\"use_underline\">True</property>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkMenu\" id=\"menu1\">\n"
"		  <property name=\"visible\">True</property>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem1\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-new</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem2\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-open</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem3\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-save</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem4\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-save-as</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkSeparatorMenuItem\" id=\"separatormenuitem1\">\n"
"		      <property name=\"visible\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem5\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-quit</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"		</widget>\n"
"	      </child>\n"
"	    </widget>\n"
"	  </child>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkMenuItem\" id=\"menuitem2\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"label\" translatable=\"yes\">_Edit</property>\n"
"	      <property name=\"use_underline\">True</property>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkMenu\" id=\"menu2\">\n"
"		  <property name=\"visible\">True</property>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem6\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-cut</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem7\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-copy</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem8\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-paste</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem9\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-delete</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"		</widget>\n"
"	      </child>\n"
"	    </widget>\n"
"	  </child>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkMenuItem\" id=\"menuitem3\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"label\" translatable=\"yes\">_View</property>\n"
"	      <property name=\"use_underline\">True</property>\n"
"	    </widget>\n"
"	  </child>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkMenuItem\" id=\"menuitem4\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"label\" translatable=\"yes\">_Help</property>\n"
"	      <property name=\"use_underline\">True</property>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkMenu\" id=\"menu3\">\n"
"		  <property name=\"visible\">True</property>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkImageMenuItem\" id=\"imagemenuitem10\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"label\">gtk-about</property>\n"
"		      <property name=\"use_stock\">True</property>\n"
"		    </widget>\n"
"		  </child>\n"
"		</widget>\n"
"	      </child>\n"
"	    </widget>\n"
"	  </child>\n"
"	</widget>\n"
"	<packing>\n"
"	  <property name=\"padding\">0</property>\n"
"	  <property name=\"expand\">False</property>\n"
"	  <property name=\"fill\">True</property>\n"
"	</packing>\n"
"      </child>\n"
"\n"
"      <child>\n"
"	<widget class=\"GtkHandleBox\" id=\"handlebox1\">\n"
"	  <property name=\"visible\">True</property>\n"
"	  <property name=\"shadow_type\">GTK_SHADOW_OUT</property>\n"
"	  <property name=\"handle_position\">GTK_POS_LEFT</property>\n"
"	  <property name=\"snap_edge\">GTK_POS_TOP</property>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkToolbar\" id=\"main_toolbar\">\n"
"	      <property name=\"height_request\">24</property>\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"orientation\">GTK_ORIENTATION_HORIZONTAL</property>\n"
"	      <property name=\"toolbar_style\">GTK_TOOLBAR_BOTH</property>\n"
"	      <property name=\"tooltips\">True</property>\n"
"	      <property name=\"show_arrow\">True</property>\n"
"	    </widget>\n"
"	  </child>\n"
"	</widget>\n"
"	<packing>\n"
"	  <property name=\"padding\">0</property>\n"
"	  <property name=\"expand\">False</property>\n"
"	  <property name=\"fill\">True</property>\n"
"	</packing>\n"
"      </child>\n"
"\n"
"      <child>\n"
"	<widget class=\"GtkHBox\" id=\"hbox1\">\n"
"	  <property name=\"visible\">True</property>\n"
"	  <property name=\"homogeneous\">False</property>\n"
"	  <property name=\"spacing\">0</property>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkHandleBox\" id=\"handlebox2\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"shadow_type\">GTK_SHADOW_OUT</property>\n"
"	      <property name=\"handle_position\">GTK_POS_TOP</property>\n"
"	      <property name=\"snap_edge\">GTK_POS_TOP</property>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkToolbar\" id=\"toolbar1\">\n"
"		  <property name=\"width_request\">24</property>\n"
"		  <property name=\"visible\">True</property>\n"
"		  <property name=\"orientation\">GTK_ORIENTATION_VERTICAL</property>\n"
"		  <property name=\"toolbar_style\">GTK_TOOLBAR_BOTH</property>\n"
"		  <property name=\"tooltips\">True</property>\n"
"		  <property name=\"show_arrow\">True</property>\n"
"		</widget>\n"
"	      </child>\n"
"	    </widget>\n"
"	    <packing>\n"
"	      <property name=\"padding\">0</property>\n"
"	      <property name=\"expand\">False</property>\n"
"	      <property name=\"fill\">True</property>\n"
"	    </packing>\n"
"	  </child>\n"
"\n"
"	  <child>\n"
"	    <widget class=\"GtkHPaned\" id=\"top_panes\">\n"
"	      <property name=\"visible\">True</property>\n"
"	      <property name=\"can_focus\">True</property>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkVPaned\" id=\"model_eer_panes\">\n"
"		  <property name=\"visible\">True</property>\n"
"		  <property name=\"can_focus\">True</property>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkNotebook\" id=\"model_tabs\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"can_focus\">True</property>\n"
"		      <property name=\"show_tabs\">True</property>\n"
"		      <property name=\"show_border\">True</property>\n"
"		      <property name=\"tab_pos\">GTK_POS_TOP</property>\n"
"		      <property name=\"scrollable\">False</property>\n"
"		      <property name=\"enable_popup\">False</property>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkScrolledWindow\" id=\"model_scrolled_window\">\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"can_focus\">True</property>\n"
"			  <property name=\"hscrollbar_policy\">GTK_POLICY_AUTOMATIC</property>\n"
"			  <property name=\"vscrollbar_policy\">GTK_POLICY_AUTOMATIC</property>\n"
"			  <property name=\"shadow_type\">GTK_SHADOW_NONE</property>\n"
"			  <property name=\"window_placement\">GTK_CORNER_TOP_LEFT</property>\n"
"\n"
"			  <child>\n"
"			    <widget class=\"GtkViewport\" id=\"overview_viewport1\">\n"
"			      <property name=\"visible\">True</property>\n"
"			      <property name=\"shadow_type\">GTK_SHADOW_IN</property>\n"
"\n"
"			      <child>\n"
"				<widget class=\"GtkVBox\" id=\"mysql_model_vbox\">\n"
"				  <property name=\"visible\">True</property>\n"
"				  <property name=\"homogeneous\">False</property>\n"
"				  <property name=\"spacing\">0</property>\n"
"\n"
"				  <child>\n"
"				    <placeholder/>\n"
"				  </child>\n"
"				</widget>\n"
"			      </child>\n"
"			    </widget>\n"
"			  </child>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"tab_expand\">False</property>\n"
"			  <property name=\"tab_fill\">True</property>\n"
"			</packing>\n"
"		      </child>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkLabel\" id=\"mysql_model_tab_label\">\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"label\" translatable=\"yes\">MySQL Model</property>\n"
"			  <property name=\"use_underline\">False</property>\n"
"			  <property name=\"use_markup\">False</property>\n"
"			  <property name=\"justify\">GTK_JUSTIFY_LEFT</property>\n"
"			  <property name=\"wrap\">False</property>\n"
"			  <property name=\"selectable\">False</property>\n"
"			  <property name=\"xalign\">0.5</property>\n"
"			  <property name=\"yalign\">0.5</property>\n"
"			  <property name=\"xpad\">0</property>\n"
"			  <property name=\"ypad\">0</property>\n"
"			  <property name=\"ellipsize\">PANGO_ELLIPSIZE_NONE</property>\n"
"			  <property name=\"width_chars\">-1</property>\n"
"			  <property name=\"single_line_mode\">False</property>\n"
"			  <property name=\"angle\">0</property>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"type\">tab</property>\n"
"			</packing>\n"
"		      </child>\n"
"		    </widget>\n"
"		    <packing>\n"
"		      <property name=\"shrink\">True</property>\n"
"		      <property name=\"resize\">True</property>\n"
"		    </packing>\n"
"		  </child>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkNotebook\" id=\"plugins_tabs\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"show_tabs\">True</property>\n"
"		      <property name=\"show_border\">False</property>\n"
"		      <property name=\"tab_pos\">GTK_POS_TOP</property>\n"
"		      <property name=\"scrollable\">False</property>\n"
"		      <property name=\"enable_popup\">False</property>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkTextView\" id=\"textview1\">\n"
"			  <property name=\"height_request\">128</property>\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"can_focus\">True</property>\n"
"			  <property name=\"editable\">True</property>\n"
"			  <property name=\"overwrite\">False</property>\n"
"			  <property name=\"accepts_tab\">True</property>\n"
"			  <property name=\"justification\">GTK_JUSTIFY_LEFT</property>\n"
"			  <property name=\"wrap_mode\">GTK_WRAP_NONE</property>\n"
"			  <property name=\"cursor_visible\">True</property>\n"
"			  <property name=\"pixels_above_lines\">0</property>\n"
"			  <property name=\"pixels_below_lines\">0</property>\n"
"			  <property name=\"pixels_inside_wrap\">0</property>\n"
"			  <property name=\"left_margin\">0</property>\n"
"			  <property name=\"right_margin\">0</property>\n"
"			  <property name=\"indent\">0</property>\n"
"			  <property name=\"text\" translatable=\"yes\"></property>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"tab_expand\">False</property>\n"
"			  <property name=\"tab_fill\">True</property>\n"
"			</packing>\n"
"		      </child>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkLabel\" id=\"label1\">\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"label\" translatable=\"yes\">page 1</property>\n"
"			  <property name=\"use_underline\">False</property>\n"
"			  <property name=\"use_markup\">False</property>\n"
"			  <property name=\"justify\">GTK_JUSTIFY_LEFT</property>\n"
"			  <property name=\"wrap\">False</property>\n"
"			  <property name=\"selectable\">False</property>\n"
"			  <property name=\"xalign\">0.5</property>\n"
"			  <property name=\"yalign\">0.5</property>\n"
"			  <property name=\"xpad\">0</property>\n"
"			  <property name=\"ypad\">0</property>\n"
"			  <property name=\"ellipsize\">PANGO_ELLIPSIZE_NONE</property>\n"
"			  <property name=\"width_chars\">-1</property>\n"
"			  <property name=\"single_line_mode\">False</property>\n"
"			  <property name=\"angle\">0</property>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"type\">tab</property>\n"
"			</packing>\n"
"		      </child>\n"
"		    </widget>\n"
"		    <packing>\n"
"		      <property name=\"shrink\">False</property>\n"
"		      <property name=\"resize\">False</property>\n"
"		    </packing>\n"
"		  </child>\n"
"		</widget>\n"
"		<packing>\n"
"		  <property name=\"shrink\">True</property>\n"
"		  <property name=\"resize\">True</property>\n"
"		</packing>\n"
"	      </child>\n"
"\n"
"	      <child>\n"
"		<widget class=\"GtkFrame\" id=\"info_panes\">\n"
"		  <property name=\"visible\">True</property>\n"
"		  <property name=\"label_xalign\">0</property>\n"
"		  <property name=\"label_yalign\">0</property>\n"
"		  <property name=\"shadow_type\">GTK_SHADOW_IN</property>\n"
"\n"
"		  <child>\n"
"		    <widget class=\"GtkVPaned\" id=\"vpaned1\">\n"
"		      <property name=\"visible\">True</property>\n"
"		      <property name=\"can_focus\">True</property>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkTreeView\" id=\"test_treeview\">\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"can_focus\">True</property>\n"
"			  <property name=\"headers_visible\">True</property>\n"
"			  <property name=\"rules_hint\">False</property>\n"
"			  <property name=\"reorderable\">False</property>\n"
"			  <property name=\"enable_search\">True</property>\n"
"			  <property name=\"fixed_height_mode\">False</property>\n"
"			  <property name=\"hover_selection\">False</property>\n"
"			  <property name=\"hover_expand\">False</property>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"shrink\">True</property>\n"
"			  <property name=\"resize\">False</property>\n"
"			</packing>\n"
"		      </child>\n"
"\n"
"		      <child>\n"
"			<widget class=\"GtkIconView\" id=\"test_iconview\">\n"
"			  <property name=\"visible\">True</property>\n"
"			  <property name=\"can_focus\">True</property>\n"
"			  <property name=\"selection_mode\">GTK_SELECTION_SINGLE</property>\n"
"			  <property name=\"orientation\">GTK_ORIENTATION_VERTICAL</property>\n"
"			  <property name=\"reorderable\">False</property>\n"
"			</widget>\n"
"			<packing>\n"
"			  <property name=\"shrink\">True</property>\n"
"			  <property name=\"resize\">True</property>\n"
"			</packing>\n"
"		      </child>\n"
"		    </widget>\n"
"		  </child>\n"
"		</widget>\n"
"		<packing>\n"
"		  <property name=\"shrink\">False</property>\n"
"		  <property name=\"resize\">False</property>\n"
"		</packing>\n"
"	      </child>\n"
"	    </widget>\n"
"	    <packing>\n"
"	      <property name=\"padding\">0</property>\n"
"	      <property name=\"expand\">True</property>\n"
"	      <property name=\"fill\">True</property>\n"
"	    </packing>\n"
"	  </child>\n"
"	</widget>\n"
"	<packing>\n"
"	  <property name=\"padding\">0</property>\n"
"	  <property name=\"expand\">True</property>\n"
"	  <property name=\"fill\">True</property>\n"
"	</packing>\n"
"      </child>\n"
"\n"
"      <child>\n"
"	<widget class=\"GtkStatusbar\" id=\"statusbar1\">\n"
"	  <property name=\"visible\">True</property>\n"
"	  <property name=\"has_resize_grip\">True</property>\n"
"	</widget>\n"
"	<packing>\n"
"	  <property name=\"padding\">0</property>\n"
"	  <property name=\"expand\">False</property>\n"
"	  <property name=\"fill\">True</property>\n"
"	</packing>\n"
"      </child>\n"
"    </widget>\n"
"  </child>\n"
"</widget>\n"
"\n"
"</glade-interface>\n"
"";
