#ifndef _CPPDBC_H_
#define _CPPDBC_H_


#include "driver_manager.h"
#include "sql_batch_exec.h"

#include <cppconn/connection.h>
#include <cppconn/driver.h>
#include <cppconn/exception.h>
#include <cppconn/metadata.h>
#include <cppconn/parameter_metadata.h>
#include <cppconn/prepared_statement.h>
#include <cppconn/resultset.h>
#include <cppconn/resultset_metadata.h>
#include <cppconn/statement.h>


#endif // _CPPDBC_H_
