variable_groups = [ ('mysqlx_port', []),
  ('mysqlx_max_connections', []),
  ('mysqlx_initial_incoming_queue_bytes', []),
  ('Aborted_clients', ['Networking/Stats']),
  ('Aborted_connects', ['Networking/Stats']),
  ('Audit_log_current_size', ['General']),
  ('Audit_log_event_max_drop_size', ['General']),
  ('Audit_log_events', ['General']),
  ('Audit_log_events_filtered', ['General']),
  ('Audit_log_events_lost', ['General']),
  ('Audit_log_events_written', ['General']),
  ('Audit_log_total_size', ['General']),
  ('Audit_log_write_waits', ['General']),
  ('Binlog_cache_disk_use', ['Binlog']),
  ('Binlog_cache_use', ['Binlog']),
  ('Binlog_stmt_cache_disk_use', ['Binlog']),
  ('Binlog_stmt_cache_use', ['Binlog']),
  ('Bytes_received', ['Networking/Stats']),
  ('Bytes_sent', ['Networking/Stats']),
  ('Com_admin_commands', ['Commands/Admin']),
  ('Com_alter_db', ['Commands/DDL']),
  ('Com_alter_db_upgrade', ['Commands/DDL']),
  ('Com_alter_event', ['Commands/DDL']),
  ('Com_alter_function', ['Commands/DDL']),
  ('Com_alter_procedure', ['Commands/DDL']),
  ('Com_alter_server', ['Commands/DDL']),
  ('Com_alter_table', ['Commands/DDL']),
  ('Com_alter_tablespace', ['Commands/DDL']),
  ('Com_alter_user', ['Commands/Admin']),
  ('Com_analyze', ['Commands/General']),
  ('Com_assign_to_keycache', ['Commands/General']),
  ('Com_backup', []),
  ('Com_backup_table', []),
  ('Com_begin', ['Commands/Transaction']),
  ('Com_binlog', ['Binlog', 'Commands/General']),
  ('Com_call_procedure', ['Commands/DML']),
  ('Com_change_db', ['Commands/General']),
  ('Com_change_master', ['Commands/Admin']),
  ('Com_change_repl_filter', []),
  ('Com_check', ['Commands/Admin']),
  ('Com_checksum', ['Commands/General']),
  ('Com_commit', ['Commands/General']),
  ('Com_create_db', ['Commands/DDL']),
  ('Com_create_event', ['Commands/DDL']),
  ('Com_create_function', ['Commands/DDL']),
  ('Com_create_index', ['Commands/DDL']),
  ('Com_create_procedure', ['Commands/DDL']),
  ('Com_create_server', ['Commands/DDL']),
  ('Com_create_table', ['Commands/DDL']),
  ('Com_create_trigger', ['Commands/DDL']),
  ('Com_create_udf', ['Commands/Admin']),
  ('Com_create_user', ['Commands/Admin']),
  ('Com_create_view', ['Commands/DDL']),
  ('Com_dealloc_sql', ['Commands/DML']),
  ('Com_delete', ['Commands/DML']),
  ('Com_delete_multi', ['Commands/DML']),
  ('Com_do', ['Commands/DML']),
  ('Com_drop_db', ['Commands/DDL']),
  ('Com_drop_event', ['Commands/DDL']),
  ('Com_drop_function', ['Commands/DDL']),
  ('Com_drop_index', ['Commands/DDL']),
  ('Com_drop_procedure', ['Commands/DDL']),
  ('Com_drop_server', ['Commands/DDL']),
  ('Com_drop_table', ['Commands/DDL']),
  ('Com_drop_trigger', ['Commands/DDL']),
  ('Com_drop_user', ['Commands/DDL']),
  ('Com_drop_view', ['Commands/DDL']),
  ('Com_empty_query', ['Commands/General']),
  ('Com_execute_sql', ['Commands/General']),
  ('Com_explain_other', ['Commands/General']),
  ('Com_flush', ['Commands/General']),
  ('Com_get_diagnostics', ['Commands/Admin']),
  ('Com_grant', ['Commands/Admin']),
  ('Com_ha_close', ['Commands/Admin']),
  ('Com_ha_open', ['Commands/Admin']),
  ('Com_ha_read', ['Commands/Admin']),
  ('Com_help', ['Commands/General']),
  ('Com_insert', ['Commands/DML']),
  ('Com_insert_select', ['Commands/DML']),
  ('Com_install_plugin', ['Commands/Admin']),
  ('Com_kill', ['Commands/Admin']),
  ('Com_load', ['Commands/Admin']),
  ('Com_load_master_data', ['Commands/Admin']),
  ('Com_load_master_table', ['Commands/Admin']),
  ('Com_lock_tables', ['Commands/General']),
  ('Com_optimize', ['Commands/Admin']),
  ('Com_preload_keys', ['Commands/Admin']),
  ('Com_prepare_sql', ['Commands/DML']),
  ('Com_purge', ['Commands/Admin']),
  ('Com_purge_before_date', ['Commands/Admin']),
  ('Com_purge_bup_log', ['Commands/Admin']),
  ('Com_release_savepoint', ['Commands/Admin']),
  ('Com_rename_table', ['Commands/DDL']),
  ('Com_rename_user', ['Commands/Admin']),
  ('Com_repair', ['Commands/Admin']),
  ('Com_replace', ['Commands/DML']),
  ('Com_replace_select', ['Commands/DML']),
  ('Com_reset', ['Commands/Admin']),
  ('Com_resignal', ['Commands/Admin']),
  ('Com_restore', ['Commands/Admin']),
  ('Com_restore_table', ['Commands/Admin']),
  ('Com_revoke', ['Commands/Admin']),
  ('Com_revoke_all', ['Commands/Admin']),
  ('Com_rollback', ['Commands/Transaction']),
  ('Com_rollback_to_savepoint', ['Commands/Transaction']),
  ('Com_savepoint', ['Commands/Transaction']),
  ('Com_select', ['Commands/General']),
  ('Com_set_option', ['Commands/General']),
  ('Com_show_authors', ['Commands/Show']),
  ('Com_show_binlog_events', ['Commands/Show']),
  ('Com_show_binlogs', ['Commands/Show']),
  ('Com_show_charsets', ['Commands/Show']),
  ('Com_show_collations', ['Commands/Show']),
  ('Com_show_column_types', ['Commands/Show']),
  ('Com_show_contributors', ['Commands/Show']),
  ('Com_show_create_db', ['Commands/Show']),
  ('Com_show_create_event', ['Commands/Show']),
  ('Com_show_create_func', ['Commands/Show']),
  ('Com_show_create_proc', ['Commands/Show']),
  ('Com_show_create_table', ['Commands/Show']),
  ('Com_show_create_trigger', ['Commands/Show']),
  ('Com_show_create_user', ['Commands/Show']),
  ('Com_show_databases', ['Commands/Show']),
  ('Com_show_engine_logs', ['Commands/Show']),
  ('Com_show_engine_mutex', ['Commands/Show']),
  ('Com_show_engine_status', ['Commands/Show']),
  ('Com_show_errors', ['Commands/Show']),
  ('Com_show_events', ['Commands/Show']),
  ('Com_show_fields', ['Commands/Show']),
  ('Com_show_function_code', ['Commands/Show']),
  ('Com_show_function_status', ['Commands/Show']),
  ('Com_show_grants', ['Commands/Show']),
  ('Com_show_innodb_status', ['Commands/Show']),
  ('Com_show_keys', ['Commands/Show']),
  ('Com_show_logs', ['Commands/Show']),
  ('Com_show_master_status', ['Commands/Show']),
  ('Com_show_ndb_status', ['Commands/Show']),
  ('Com_show_new_master', ['Commands/Show']),
  ('Com_show_open_tables', ['Commands/Show']),
  ('Com_show_plugins', ['Commands/Show']),
  ('Com_show_privileges', ['Commands/Show']),
  ('Com_show_procedure_code', ['Commands/Show']),
  ('Com_show_procedure_status', ['Commands/Show']),
  ('Com_show_processlist', ['Commands/Show']),
  ('Com_show_profile', ['Commands/Show']),
  ('Com_show_profiles', ['Commands/Show']),
  ('Com_show_relaylog_events', ['Commands/Show']),
  ('Com_show_slave_hosts', ['Commands/Show']),
  ('Com_show_slave_status', ['Commands/Show']),
  ('Com_show_slave_status_nonblocking', ['Commands/Show']),
  ('Com_show_status', ['Commands/Show']),
  ('Com_show_storage_engines', ['Commands/Show']),
  ('Com_show_table_status', ['Commands/Show']),
  ('Com_show_tables', ['Commands/Show']),
  ('Com_show_triggers', ['Commands/Show']),
  ('Com_show_variables', ['Commands/Show']),
  ('Com_show_warnings', ['Commands/Show']),
  ('Com_shutdown', ['Commands/Show']),
  ('Com_signal', ['Commands/Admin']),
  ('Com_slave_start', ['Commands/Replication']),
  ('Com_slave_stop', ['Commands/Replication']),
  ('Com_stmt_close', ['Commands/Prepared Statement']),
  ('Com_stmt_execute', ['Commands/Prepared Statement']),
  ('Com_stmt_fetch', ['Commands/Prepared Statement']),
  ('Com_stmt_prepare', ['Commands/Prepared Statement']),
  ('Com_stmt_reprepare', ['Commands/Prepared Statement']),
  ('Com_stmt_reset', ['Commands/Prepared Statement']),
  ('Com_stmt_send_long_data', ['Commands/Prepared Statement']),
  ('Com_truncate', ['Commands/DML']),
  ('Com_uninstall_plugin', ['Commands/Admin']),
  ('Com_unlock_tables', ['Commands/Admin']),
  ('Com_update', ['Commands/DML']),
  ('Com_update_multi', ['Commands/DML']),
  ('Com_xa_commit', ['Commands/Transaction']),
  ('Com_xa_end', ['Commands/Transaction']),
  ('Com_xa_prepare', ['Commands/Transaction']),
  ('Com_xa_recover', ['Commands/Transaction']),
  ('Com_xa_rollback', ['Commands/Transaction']),
  ('Com_xa_start', ['Commands/Transaction']),
  ('Compression', ['General']),
  ('Connection_errors_accept', ['Networking/Errors']),
  ('Connection_errors_internal', ['Networking/Errors']),
  ('Connection_errors_max_connections', ['Networking/Errors']),
  ('Connection_errors_peer_addr', ['Networking/Errors']),
  ('Connection_errors_select', ['Networking/Errors']),
  ('Connection_errors_tcpwrap', ['Networking/Errors']),
  ('Connections', ['Networking/Stats']),
  ('Created_tmp_disk_tables', ['General']),
  ('Created_tmp_files', ['General']),
  ('Created_tmp_tables', ['General']),
  ('Delayed_errors', ['General']),
  ('Delayed_insert_threads', ['General']),
  ('Delayed_writes', ['General']),
  ('Firewall_access_denied', ['Firewall']),
  ('Firewall_access_granted', ['Firewall']),
  ('Firewall_cached_entries', ['Firewall']),
  ('Flush_commands', ['General']),
  ('Handler_commit', ['Handler']),
  ('Handler_delete', ['Handler']),
  ('Handler_discover', ['Handler']),
  ('Handler_external_lock', ['Handler']),
  ('Handler_mrr_init', ['Handler']),
  ('Handler_prepare', ['Handler']),
  ('Handler_read_first', ['Handler']),
  ('Handler_read_key', ['Handler']),
  ('Handler_read_last', ['Handler']),
  ('Handler_read_next', ['Handler']),
  ('Handler_read_prev', ['Handler']),
  ('Handler_read_rnd', ['Handler']),
  ('Handler_read_rnd_next', ['Handler']),
  ('Handler_rollback', ['Handler']),
  ('Handler_savepoint', ['Handler']),
  ('Handler_savepoint_rollback', ['Handler']),
  ('Handler_update', ['Handler']),
  ('Handler_write', ['Handler']),
  ('Innodb_available_undo_logs', ['InnoDB/Stats']),
  ('Innodb_buffer_pool_bytes_data', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_bytes_dirty', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_dump_status', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_load_status', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_data', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_dirty', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_flushed', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_free', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_latched', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_misc', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_pages_total', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_read_ahead', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_read_ahead_evicted', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_read_ahead_rnd', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_read_ahead_seq', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_read_requests', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_reads', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_resize_status', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_wait_free', ['InnoDB/Buffer pool']),
  ('Innodb_buffer_pool_write_requests', ['InnoDB/Buffer pool']),
  ('Innodb_data_fsyncs', ['InnoDB/Data']),
  ('Innodb_data_pending_fsyncs', ['InnoDB/Data']),
  ('Innodb_data_pending_reads', ['InnoDB/Data']),
  ('Innodb_data_pending_writes', ['InnoDB/Data']),
  ('Innodb_data_read', ['InnoDB/Data', 'InnoDB/Stats']),
  ('Innodb_data_reads', ['InnoDB/Data', 'InnoDB/Stats']),
  ('Innodb_data_writes', ['InnoDB/Data', 'InnoDB/Stats']),
  ('Innodb_data_written', ['InnoDB/Data', 'InnoDB/Stats']),
  ('Innodb_dblwr_pages_written', ['InnoDB/General']),
  ('Innodb_dblwr_writes', ['InnoDB/General']),
  ('Innodb_have_atomic_builtins', ['InnoDB/General']),
  ('Innodb_have_sync_atomic', ['InnoDB/General']),
  ('Innodb_heap_enabled', ['InnoDB/General']),
  ('Innodb_log_waits', ['InnoDB/Stats']),
  ('Innodb_log_write_requests', ['InnoDB/Stats']),
  ('Innodb_log_writes', ['InnoDB/Stats']),
  ('Innodb_num_open_files', ['InnoDB/Stats']),
  ('Innodb_os_log_fsyncs', ['InnoDB/Stats']),
  ('Innodb_os_log_pending_fsyncs', ['InnoDB/Stats']),
  ('Innodb_os_log_pending_writes', ['InnoDB/Stats']),
  ('Innodb_os_log_written', ['InnoDB/Stats']),
  ('Innodb_page_size', ['InnoDB/Stats']),
  ('Innodb_pages_created', ['InnoDB/Stats']),
  ('Innodb_pages_read', ['InnoDB/Stats']),
  ('Innodb_pages_written', ['InnoDB/Stats']),
  ('Innodb_row_lock_current_waits', ['InnoDB/Stats']),
  ('Innodb_row_lock_time', ['InnoDB/Stats']),
  ('Innodb_row_lock_time_avg', ['InnoDB/Stats']),
  ('Innodb_row_lock_time_max', ['InnoDB/Stats']),
  ('Innodb_row_lock_waits', ['InnoDB/Stats']),
  ('Innodb_rows_deleted', ['InnoDB/Stats']),
  ('Innodb_rows_inserted', ['InnoDB/Stats']),
  ('Innodb_rows_read', ['InnoDB/Stats']),
  ('Innodb_rows_updated', ['InnoDB/Stats']),
  ('Innodb_truncated_status_writes', ['InnoDB/Stats']),
  ('Innodb_wake_ups', ['InnoDB/Stats']),
  ('Key_blocks_not_flushed', ['Keycache']),
  ('Key_blocks_unused', ['Keycache']),
  ('Key_blocks_used', ['Keycache']),
  ('Key_read_requests', ['Keycache']),
  ('Key_reads', ['Keycache']),
  ('Key_write_requests', ['Keycache']),
  ('Key_writes', ['Keycache']),
  ('Last_query_cost', ['Performance']),
  ('Last_query_partial_plans', ['Performance']),
  ('Locked_connects', ['Networking/Stats']),
  ('Max_execution_time_exceeded', ['Networking/Stats']),
  ('Max_execution_time_set', ['Networking/Stats']),
  ('Max_execution_time_set_failed', ['Networking/Stats']),
  ('Max_statement_time_exceeded', ['Networking/Stats']),
  ('Max_statement_time_set', ['Networking/Stats']),
  ('Max_statement_time_set_failed', ['Networking/Stats']),
  ('Max_used_connections', ['Networking/Stats']),
  ('Max_used_connections_time', ['Networking/Stats']),
  ('mecab_charset', []),
  ('ndb-nodeid', ['NDB']),
  ('Ndb_api_bytes_received_count', ['NDB']),
  ('Ndb_api_bytes_received_count_session', ['NDB']),
  ('Ndb_api_bytes_received_count_slave', ['NDB']),
  ('Ndb_api_bytes_sent_count', ['NDB']),
  ('Ndb_api_bytes_sent_count_session', ['NDB']),
  ('Ndb_api_bytes_sent_count_slave', ['NDB']),
  ('Ndb_api_event_bytes_count', ['NDB']),
  ('Ndb_api_event_bytes_count_injector', ['NDB']),
  ('Ndb_api_event_data_count', ['NDB']),
  ('Ndb_api_event_data_count_injector', ['NDB']),
  ('Ndb_api_event_nondata_count', ['NDB']),
  ('Ndb_api_event_nondata_count_injector', ['NDB']),
  ('Ndb_api_pk_op_count', ['NDB']),
  ('Ndb_api_pk_op_count_session', ['NDB']),
  ('Ndb_api_pk_op_count_slave', ['NDB']),
  ('Ndb_api_pruned_scan_count', ['NDB']),
  ('Ndb_api_pruned_scan_count_session', ['NDB']),
  ('Ndb_api_pruned_scan_count_slave', ['NDB']),
  ('Ndb_api_range_scan_count', ['NDB']),
  ('Ndb_api_range_scan_count_session', ['NDB']),
  ('Ndb_api_range_scan_count_slave', ['NDB']),
  ('Ndb_api_read_row_count', ['NDB']),
  ('Ndb_api_read_row_count_session', ['NDB']),
  ('Ndb_api_read_row_count_slave', ['NDB']),
  ('Ndb_api_scan_batch_count', ['NDB']),
  ('Ndb_api_scan_batch_count_session', ['NDB']),
  ('Ndb_api_scan_batch_count_slave', ['NDB']),
  ('Ndb_api_table_scan_count', ['NDB']),
  ('Ndb_api_table_scan_count_session', ['NDB']),
  ('Ndb_api_table_scan_count_slave', ['NDB']),
  ('Ndb_api_trans_abort_count', ['NDB']),
  ('Ndb_api_trans_abort_count_session', ['NDB']),
  ('Ndb_api_trans_abort_count_slave', ['NDB']),
  ('Ndb_api_trans_close_count', ['NDB']),
  ('Ndb_api_trans_close_count_session', ['NDB']),
  ('Ndb_api_trans_close_count_slave', ['NDB']),
  ('Ndb_api_trans_commit_count', ['NDB']),
  ('Ndb_api_trans_commit_count_session', ['NDB']),
  ('Ndb_api_trans_commit_count_slave', ['NDB']),
  ('Ndb_api_trans_local_read_row_count', ['NDB']),
  ('Ndb_api_trans_local_read_row_count_session', ['NDB']),
  ('Ndb_api_trans_local_read_row_count_slave', ['NDB']),
  ('Ndb_api_trans_start_count', ['NDB']),
  ('Ndb_api_trans_start_count_session', ['NDB']),
  ('Ndb_api_trans_start_count_slave', ['NDB']),
  ('Ndb_api_uk_op_count', ['NDB']),
  ('Ndb_api_uk_op_count_session', ['NDB']),
  ('Ndb_api_uk_op_count_slave', ['NDB']),
  ('Ndb_api_wait_exec_complete_count', ['NDB']),
  ('Ndb_api_wait_exec_complete_count_session', ['NDB']),
  ('Ndb_api_wait_exec_complete_count_slave', ['NDB']),
  ('Ndb_api_wait_meta_request_count', ['NDB']),
  ('Ndb_api_wait_meta_request_count_session', ['NDB']),
  ('Ndb_api_wait_meta_request_count_slave', ['NDB']),
  ('Ndb_api_wait_nanos_count', ['NDB']),
  ('Ndb_api_wait_nanos_count_session', ['NDB']),
  ('Ndb_api_wait_nanos_count_slave', ['NDB']),
  ('Ndb_api_wait_scan_result_count', ['NDB']),
  ('Ndb_api_wait_scan_result_count_session', ['NDB']),
  ('Ndb_api_wait_scan_result_count_slave', ['NDB']),
  ('Ndb_cluster_node_id', ['NDB']),
  ('Ndb_config_from_host', ['NDB']),
  ('Ndb_config_from_port', ['NDB']),
  ('Ndb_conflict_fn_epoch', ['NDB']),
  ('Ndb_conflict_fn_epoch2', ['NDB']),
  ('Ndb_conflict_fn_epoch2_trans', ['NDB']),
  ('Ndb_conflict_fn_epoch_trans', ['NDB']),
  ('Ndb_conflict_fn_max', ['NDB']),
  ('Ndb_conflict_fn_max_del_win', ['NDB']),
  ('Ndb_conflict_fn_old', ['NDB']),
  ('Ndb_conflict_last_stable_epoch', ['NDB']),
  ('Ndb_conflict_reflected_op_discard_count', ['NDB']),
  ('Ndb_conflict_reflected_op_prepare_count', ['NDB']),
  ('Ndb_conflict_refresh_op_count', ['NDB']),
  ('Ndb_conflict_trans_conflict_commit_count', ['NDB']),
  ('Ndb_conflict_trans_detect_iter_count', ['NDB']),
  ('Ndb_conflict_trans_reject_count', ['NDB']),
  ('Ndb_conflict_trans_row_conflict_count', ['NDB']),
  ('Ndb_conflict_trans_row_reject_count', ['NDB']),
  ('Ndb_epoch_delete_delete_count', ['NDB']),
  ('Ndb_execute_count', ['NDB']),
  ('Ndb_last_commit_epoch_server', ['NDB']),
  ('Ndb_last_commit_epoch_session', ['NDB']),
  ('Ndb_number_of_data_nodes', ['NDB']),
  ('Ndb_pruned_scan_count', ['NDB']),
  ('Ndb_pushed_queries_defined', ['NDB']),
  ('Ndb_pushed_queries_dropped', ['NDB']),
  ('Ndb_pushed_queries_executed', ['NDB']),
  ('Ndb_pushed_reads', ['NDB']),
  ('ndb_recv_thread_activation_threshold', ['NDB']),
  ('Ndb_scan_count', ['NDB']),
  ('Not_flushed_delayed_rows', ['General']),
  ('Ongoing_anonymous_gtid_violating_transaction_count', []),
  ('Ongoing_anonymous_transaction_count', []),
  ('Ongoing_automatic_gtid_violating_transaction_count', []),
  ('Open_files', ['General']),
  ('Open_streams', ['General']),
  ('Open_table_definitions', ['General']),
  ('Open_tables', ['General']),
  ('Opened_files', ['General']),
  ('Opened_table_definitions', ['General']),
  ('Opened_tables', ['General']),
  ('Performance_schema_accounts_lost', ['Performance schema']),
  ('Performance_schema_cond_classes_lost', ['Performance schema']),
  ('Performance_schema_cond_instances_lost', ['Performance schema']),
  ('Performance_schema_digest_lost', ['Performance schema']),
  ('Performance_schema_file_classes_lost', ['Performance schema']),
  ('Performance_schema_file_handles_lost', ['Performance schema']),
  ('Performance_schema_file_instances_lost', ['Performance schema']),
  ('Performance_schema_hosts_lost', ['Performance schema']),
  ('Performance_schema_index_stat_lost', ['Performance schema']),
  ('Performance_schema_locker_lost', ['Performance schema']),
  ('Performance_schema_memory_classes_lost', ['Performance schema']),
  ('Performance_schema_metadata_lock_lost', ['Performance schema']),
  ('Performance_schema_mutex_classes_lost', ['Performance schema']),
  ('Performance_schema_mutex_instances_lost', ['Performance schema']),
  ('Performance_schema_nested_statement_lost', ['Performance schema']),
  ('Performance_schema_prepared_statements_lost', ['Performance schema']),
  ('Performance_schema_program_lost', ['Performance schema']),
  ('Performance_schema_rwlock_classes_lost', ['Performance schema']),
  ('Performance_schema_rwlock_instances_lost', ['Performance schema']),
  ('Performance_schema_session_connect_attrs_lost', ['Performance schema']),
  ('Performance_schema_socket_classes_lost', ['Performance schema']),
  ('Performance_schema_socket_instances_lost', ['Performance schema']),
  ('Performance_schema_stage_classes_lost', ['Performance schema']),
  ('Performance_schema_statement_classes_lost', ['Performance schema']),
  ('Performance_schema_table_handles_lost', ['Performance schema']),
  ('Performance_schema_table_instances_lost', ['Performance schema']),
  ('Performance_schema_table_lock_stat_lost', ['Performance schema']),
  ('Performance_schema_thread_classes_lost', ['Performance schema']),
  ('Performance_schema_thread_instances_lost', ['Performance schema']),
  ('Performance_schema_users_lost', ['Performance schema']),
  ('Prepared_stmt_count', ['General']),
  ('Qcache_free_blocks', ['Query cache']),
  ('Qcache_free_memory', ['Query cache']),
  ('Qcache_hits', ['Query cache']),
  ('Qcache_inserts', ['Query cache']),
  ('Qcache_lowmem_prunes', ['Query cache']),
  ('Qcache_not_cached', ['Query cache']),
  ('Qcache_queries_in_cache', ['Query cache']),
  ('Qcache_total_blocks', ['Query cache']),
  ('Queries', ['General']),
  ('Questions', ['General']),
  ('Rewriter_number_loaded_rules', []),
  ('Rewriter_number_reloads', []),
  ('Rewriter_number_rewritten_queries', []),
  ('Rewriter_reload_error', []),
  ('Rpl_semi_sync_master_clients', ['Commands/Replication']),
  ('Rpl_semi_sync_master_net_avg_wait_time', ['Commands/Replication']),
  ('Rpl_semi_sync_master_net_wait_time', ['Commands/Replication']),
  ('Rpl_semi_sync_master_net_waits', ['Commands/Replication']),
  ('Rpl_semi_sync_master_no_times', ['Commands/Replication']),
  ('Rpl_semi_sync_master_no_tx', ['Commands/Replication']),
  ('Rpl_semi_sync_master_status', ['Commands/Replication']),
  ('Rpl_semi_sync_master_timefunc_failures', ['Commands/Replication']),
  ('Rpl_semi_sync_master_tx_avg_wait_time', ['Commands/Replication']),
  ('Rpl_semi_sync_master_tx_wait_time', ['Commands/Replication']),
  ('Rpl_semi_sync_master_tx_waits', ['Commands/Replication']),
  ('Rpl_semi_sync_master_wait_pos_backtraverse', ['Commands/Replication']),
  ('Rpl_semi_sync_master_wait_sessions', ['Commands/Replication']),
  ('Rpl_semi_sync_master_yes_tx', ['Commands/Replication']),
  ('Rpl_semi_sync_slave_status', ['Commands/Replication']),
  ('Rpl_status', ['Commands/Replication']),
  ('Rsa_public_key', []),
  ('Select_full_join', ['General']),
  ('Select_full_range_join', ['General']),
  ('Select_range', ['General']),
  ('Select_range_check', ['General']),
  ('Select_scan', ['General']),
  ('Slave_heartbeat_period', ['Replication']),
  ('Slave_last_heartbeat', ['Replication']),
  ('Slave_open_temp_tables', ['Replication']),
  ('Slave_received_heartbeats', ['Replication']),
  ('Slave_retried_transactions', ['Replication']),
  ('Slave_running', ['Replication']),
  ('Slow_launch_threads', ['Threading']),
  ('Slow_queries', ['General']),
  ('Sort_merge_passes', ['General']),
  ('Sort_range', ['General']),
  ('Sort_rows', ['General']),
  ('Sort_scan', ['General']),
  ('Ssl_accept_renegotiates', ['SSL']),
  ('Ssl_accepts', ['SSL']),
  ('Ssl_callback_cache_hits', ['SSL']),
  ('Ssl_cipher', ['SSL']),
  ('Ssl_cipher_list', ['SSL']),
  ('Ssl_client_connects', ['SSL']),
  ('Ssl_connect_renegotiates', ['SSL']),
  ('Ssl_ctx_verify_depth', ['SSL']),
  ('Ssl_ctx_verify_mode', ['SSL']),
  ('Ssl_default_timeout', ['SSL']),
  ('Ssl_finished_accepts', ['SSL']),
  ('Ssl_finished_connects', ['SSL']),
  ('Ssl_server_not_after', ['SSL']),
  ('Ssl_server_not_before', ['SSL']),
  ('Ssl_session_cache_hits', ['SSL']),
  ('Ssl_session_cache_misses', ['SSL']),
  ('Ssl_session_cache_mode', ['SSL']),
  ('Ssl_session_cache_overflows', ['SSL']),
  ('Ssl_session_cache_size', ['SSL']),
  ('Ssl_session_cache_timeouts', ['SSL']),
  ('Ssl_sessions_reused', ['SSL']),
  ('Ssl_used_session_cache_entries', ['SSL']),
  ('Ssl_verify_depth', ['SSL']),
  ('Ssl_verify_mode', ['SSL']),
  ('Ssl_version', ['SSL']),
  ('Table_locks_immediate', ['General']),
  ('Table_locks_waited', ['General']),
  ('Table_open_cache_hits', ['General']),
  ('Table_open_cache_misses', ['General']),
  ('Table_open_cache_overflows', ['General']),
  ('Tc_log_max_pages_used', ['General']),
  ('Tc_log_page_size', ['General']),
  ('Tc_log_page_waits', ['General']),
  ('Threads_cached', ['Threading']),
  ('Threads_connected', ['Threading']),
  ('Threads_created', ['Threading']),
  ('Threads_running', ['Threading']),
  ('Uptime', ['General']),
  ('Uptime_since_flush_status', ['General']),
  ('validate_password_dictionary_file_last_parsed', []),
  ('validate_password_dictionary_file_words_count', [])]
