<html xmlns="http://www.w3.org/1999/xhtml">
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"  "http://www.w3.org/TR/html4/loose.dtd">
<head>
  <meta http-equiv="content-type" content="text/html; charset=UTF-8" />
  <title>{{TITLE:h}} - Overview</title>
  <link rel="stylesheet" type="text/css" media="screen" href="basic.css">
</head>

<body class="ov_overview_page">
<div class="ov_main">
  MySQL Workbench<br>
  <p class="small_text">{{TITLE:h}}</p>

{{#SCHEMATA}}
  <div class="ov_section_link">
  <a href="table_details.html#Schema_Nr_{{SCHEMA_ID}}" target="content">Schema {{SCHEMA_NAME}}</a>
  </div>
  <div class="ov_section_subitem">Schema Tables</div>
  <div class="ov_object_link">
{{#TABLES}}
    <a href="table_details.html#Table_{{SCHEMA_ID}}_{{TABLE_ID}}" target="content">{{TABLE_NAME}}</a>
{{/TABLES}}
  </div>

{{/SCHEMATA}}
</div>
</body>

</html>
