CREATE  TABLE IF NOT EXISTS `NumericContainer` (
  `id` INT NULL DEFAULT NULL ,
  `fdata` FLOAT NULL DEFAULT NULL ,
  `ddata` DOUBLE NULL DEFAULT NULL ,
  `ndata` DECIMAL NULL DEFAULT NULL ,
  `decdata` DECIMAL(10,5) NULL DEFAULT NULL ,
  PRIMARY KEY (`id`) );
