#ifndef _MYSQL_DB_DIFF_REPORTING_PUBLIC_INTERFACE_H_
#define _MYSQL_DB_DIFF_REPORTING_PUBLIC_INTERFACE_H_

#ifdef _WIN32

#ifdef DBMYSQLDIFFREPORTINGWBPBE_EXPORTS
#define DBMYSQLDIFFREPORTINGWBPBE_PUBLIC_FUNC __declspec(dllexport)
#else
#define DBMYSQLDIFFREPORTINGWBPBE_PUBLIC_FUNC __declspec(dllimport)
#endif

#else
#define DBMYSQLDIFFREPORTINGWBPBE_PUBLIC_FUNC
#endif

#endif // _MYSQL_DB_DIFF_REPORTING_PUBLIC_INTERFACE_H_

