#ifndef _STDAFX_H_
#define _STDAFX_H_

#ifdef _WIN32

#pragma once

#include <sigc++/sigc++.h>
#include <glib.h>
#include <string>
#include <vector>
#include <stdexcept>
#include <stdarg.h>
#include <algorithm>
#include <map>
#include <list>

#endif

#endif // _STDAFX_H_
