
#define APP_MAJOR_NUMBER 6
#define APP_MINOR_NUMBER 3
#define APP_RELEASE_NUMBER 8
#define APP_BUILD_NUMBER 1228
#define APP_RELEASE_TYPE "CE"

#define APP_LICENSE_TYPE "GPL"
#define APP_EDITION_NAME "Community"
