body {
	color: #000000;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 10px ;
	background : #FFFFFF;
	margin-bottom : 0px;
	margin-left : 0px;
	margin-right : 0px;
	margin-top : 0px;
}

.tbl_detail_page {
	margin-bottom : 10px;
	margin-left : 10px;
	margin-right : 10px;
	margin-top : 10px;
}

.schema_header {
	background: #33cdff;
	font-size : 12pt;
	font-weight : bold;
	padding-left : 10px;
	padding-bottom : 5px;
	padding-top : 5px;
	margin-bottom: 10px;
	border: 1px solid #888888;
	display : block;
}

.table_header {
	background: #caf2ff;
	font-size : 10pt;
	font-weight : bold;
	padding-left : 10px;
	padding-bottom : 5px;
	padding-top : 5px;
	border: 1px solid #BBBBBB;
	display : block;
}

.table_body {
	background: #EEEEEE;
	padding-left: 15px;
	padding-right: 15px;
	border-left: 1px solid #BBBBBB;
	border-bottom: 1px solid #BBBBBB;
	border-right: 1px solid #BBBBBB;
}

.subitem_header {
	color: #000000;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 9pt;
	font-weight : bold;
	padding-bottom : 5px;
	padding-top : 5px;
	visibility : visible;
}

.subitems_table {
	border:1px solid #CCCCCC;
	margin-bottom: 15px;
}

.subitem_table_head {
	color: #333333;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 8pt;
	font-weight : bold;
	text-align : left;
	border: 0pt solid #000000;
	padding-bottom : 3px;
	padding-left : 10px;
	padding-right : 2px;
	padding-top : 3px;
	background : #FFFFCC;
	visibility : visible;
}

.subitem_table_field {
	color: #333333;
	font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size: 8pt;
	font-weight: normal;
	text-align: left;
	border: 0pt solid #000000;
	padding-bottom: 3px;
	padding-left: 10px;
	padding-right: 2px;
	padding-top: 3px;
	vertical-align: top;
	background:#F8F8F8;
	visibility: visible;
}

.small_text {
	font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size: 7pt;
}

.ov_overview_page {
	background : #FFFFFF;
}

.ov_main {
	background: #BBBBBB;
	font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size: 12pt;
	font-weight: bold;
	display : block;
	text-align : center;
	padding-top: 8px;
	padding-bottom: 0px;
	padding-left : 0px;
	padding-right : 0px;
}

.ov_section_link {
	background: #EBEBEB;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 10pt;
	font-weight : normal;
	text-align : center;
	width : 100%;
	display : block;
	padding-top: 12px;
	padding-bottom: 12px;
	border-bottom: 1px solid #B7B7B7;
	visibility: visible;
}

.ov_section_link a {
	color: #333333;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 10pt;
	font-weight : bold;
	text-align : center;
	text-decoration: none;
	display : block;
	width: 100%;
	visibility : visible;
}

.ov_section_link a:hover {
	font-weight: bold;
	text-decoration: underline;
}

.ov_section_subitem {
	color: #555555;
	background: #EBEBEB;
	font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size: 7pt;
	font-weight: normal;
	text-align: center;
	width: 100%;
	padding-top: 2px;
	padding-bottom: 4px;
	display : block;
	height: 12px;
}

.ov_object_link {
	font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size: 8pt;
	font-weight: normal;
	text-align: left;
	text-indent: 15px;
	width: 100%;
	visibility: visible;
}

.ov_object_link a {
	color: #333333;
	background: #EBEBEB;
	font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
	font-size : 8pt;
	font-weight : normal;
	text-align : left;
	text-decoration : none;
	display : block;
	padding-top: 2px;
	padding-bottom: 2px;
	border-top: 1px solid #ffffff; 
	border-bottom: 1px solid #B7B7B7; 
	width: 100%;
	line-height: 12px;
	height: 12px;
	visibility : visible;
}


.ov_object_link a:link {}
.ov_object_link a:visited {}
.ov_object_link a:active {}
.ov_object_link a:hover {font-weight: bold; color: #000000; background: #ffffff;}

