variable_column_widths={
  'General' : 260,
  'Features' : 200,
  'Logging' : 300,
  'InnoDB' : 300,
  'Networking' : 200,
  'Advanced' : 250,
  'Other' : 300,
  'Security' : 250,
  'Replication' : 250,
  'MyISAM' : 200,
  'Performance' : 410
}