/* 
 * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
#ifndef _BE_PARSE_UTILS_H_
#define _BE_PARSE_UTILS_H_

#ifndef _WIN32
#include <glib.h>
#include <string>
#include <vector>
#include <list>
#include <stdexcept>
#include <stdarg.h>
#endif

#include "wbpublic_public_interface.h"

namespace bec {
  WBPUBLICBACKEND_PUBLIC_FUNC bool tokenize_string_list(const std::string &str, int quote_char, 
    bool quoted_only, std::list<std::string> &tokens);
};

#endif
