CREATE  TABLE IF NOT EXISTS `IntegerContainer` (
  `id` INT NULL DEFAULT NULL ,
  `tint` TINYINT NULL DEFAULT NULL ,
  `sint` SMALLINT NULL DEFAULT NULL ,
  `mint` MEDIUMINT NULL DEFAULT NULL ,
  `iint` INT NULL DEFAULT NULL ,
  `bint` BIGINT NULL DEFAULT NULL ,
  PRIMARY KEY (`id`) );
