/*
* Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; version 2 of the
* License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301  USA
*/

#pragma once

#define NOMINMAX
#define WIN32_LEAN_AND_MEAN 
#include <windows.h>

#include <glib.h>
#include <cctype>
#include <algorithm>
#include <sstream>
#include <map>
#include <set>
#include <vector>
#include <list>
#include <sstream>

#include <boost/optional.hpp>
#include <boost/cstdint.hpp>
#include <boost/shared_ptr.hpp>

#include <pcre.h>

#include <glib.h>
