<html xmlns="http://www.w3.org/1999/xhtml">
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"  "http://www.w3.org/TR/html4/loose.dtd">
<head>
  <meta http-equiv="content-type" content="text/html; charset=UTF-8" />
  <title>{{TITLE:h}} - Overview</title>
  <link rel="stylesheet" type="text/css" media="screen" href="{{STYLE_NAME}}.css">
</head>

<body>
  <h1 class="overview-head">Overview</h1>
  <table class="overview">
    <colgroup>
        <col width="30%" />
        <col width="*" />
     </colgroup>
    <tr>
      <td class="property-name">Document Title: </td>
      <td class="property-value">{{PROJECT_TITLE}}</td>
    </tr>
    <tr>
      <td class="property-name">Project Name: </td>
      <td class="property-value">{{PROJECT_NAME}}</td>
    </tr>
    <tr>
      <td class="property-name">Author: </td>
      <td class="property-value">{{PROJECT_AUTHOR}}</td>
    </tr>
    <tr>
      <td class="property-name">Version: </td>
      <td class="property-value">{{PROJECT_VERSION}}</td>
    </tr>
    <tr>
      <td class="property-name">Description: </td>
      <td class="property-value">{{PROJECT_DESCRIPTION}}</td>
    </tr>
    <tr>
      <td class="property-name">Created: </td>
      <td class="property-value">{{PROJECT_CREATED}}</td>
    </tr>
    <tr>
      <td class="property-name">Last Updated: </td>
      <td class="property-value">{{PROJECT_CHANGED}}</td>
    </tr>
    <tr>
      <td class="property-name">Schemata: </td>
      <td class="property-value">{{SCHEMA_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Tables: </td>
      <td class="property-value">{{TOTAL_TABLE_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Views: </td>
      <td class="property-value">{{TOTAL_VIEW_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Stored Routines: </td>
      <td class="property-value">{{TOTAL_ROUTINE_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Columns: </td>
      <td class="property-value">{{TOTAL_COLUMN_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Indices: </td>
      <td class="property-value">{{TOTAL_INDEX_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Foreign Keys: </td>
      <td class="property-value">{{TOTAL_FK_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Triggers: </td>
      <td class="property-value">{{TOTAL_TRIGGER_COUNT}}</td>
    </tr>
    <tr>
      <td class="property-name">Events: </td>
      <td class="property-value">not supported</td>
    </tr>
    <tr style="margin-top: 10px">
      <td class="property-name">Report Generated: </td>
      <td class="property-value">{{GENERATED}}</td>
    </tr>
  </table>
</body>

</html>
