SET NAMES 'utf8';

DROP DATABASE IF EXISTS Iñŧęřñⱥtìönal;

CREATE DATABASE Iñŧęřñⱥtìönal
  DEFAULT CHARACTER SET utf8
  DEFAULT COLLATE utf8_general_ci;

USE Iñŧęřñⱥtìönal;

CREATE TABLE México ( 
estado VARCHAR(50),
delegación VARCHAR(50)
);

CREATE TABLE Україна (
область VARCHAR(50),
район VARCHAR(50)
);

