/** \file
 *  This C source file was generated by $ANTLR version 3.4
 *
 *     -  From the grammar source file : MySQLSimpleParser.g
 *     -                            On : 2016-10-14 14:12:22
 *     -                for the parser : MySQLSimpleParserParser
 *
 * Editing it, at least manually, is not wise.
 *
 * C language generator and runtime by Jim Idle, jimi|hereisanat|idle|dotgoeshere|ws.
 *
 *
*/
// [The "BSD license"]
// Copyright (c) 2005-2009 Jim Idle, Temporal Wave LLC
// http://www.temporal-wave.com
// http://www.linkedin.com/in/jimidle
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The name of the author may not be used to endorse or promote products
//    derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


/* =============================================================================
 * This is what the grammar programmer asked us to put at the top of every file.
 */


#define ANTLR3_HUGE
#ifndef _WIN32
#pragma GCC diagnostic ignored "-Wunused-variable"
#pragma GCC diagnostic ignored "-Wparentheses"
#ifdef __APPLE__
// Comparison of unsigned expression >= 0 is always true.
#pragma GCC diagnostic ignored "-Wtautological-compare"
#else
#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 6 )
#pragma GCC diagnostic ignored "-Wtype-limits"
#endif
#endif
#endif

#include "MySQLLexer.h" // Not automatically included by the generator.

/* End of Header action.
 * =============================================================================
 */

/* -----------------------------------------
 * Include the ANTLR3 generated header file.
 */
#include    "MySQLSimpleParser.h"

#ifdef __cplusplus
extern "C" { 
#endif

  // Custom error reporting function.
  void onMySQLParseError(struct ANTLR3_BASE_RECOGNIZER_struct *recognizer, pANTLR3_UINT8 *tokenNames); 

#ifdef __cplusplus
};
#endif

/* ----------------------------------------- */





/* MACROS that hide the C interface implementations from the
 * generated code, which makes it a little more understandable to the human eye.
 * I am very much against using C pre-processor macros for function calls and bits
 * of code as you cannot see what is happening when single stepping in debuggers
 * and so on. The exception (in my book at least) is for generated code, where you are
 * not maintaining it, but may wish to read and understand it. If you single step it, you know that input()
 * hides some indirect calls, but is always referring to the input stream. This is
 * probably more readable than ctx->input->istream->input(snarfle0->blarg) and allows me to rejig
 * the runtime interfaces without changing the generated code too often, without
 * confusing the reader of the generated output, who may not wish to know the gory
 * details of the interface inheritance.
 */

#define		CTX	ctx

/* Aids in accessing scopes for grammar programmers
 */
#undef	SCOPE_TYPE
#undef	SCOPE_STACK
#undef	SCOPE_TOP
#define	SCOPE_TYPE(scope)   pMySQLSimpleParser_##scope##_SCOPE
#define SCOPE_STACK(scope)  pMySQLSimpleParser_##scope##Stack
#define	SCOPE_TOP(scope)    ctx->pMySQLSimpleParser_##scope##Top
#define	SCOPE_SIZE(scope)		ctx->pMySQLSimpleParser_##scope##Stack_limit
#define SCOPE_INSTANCE(scope, i)	(ctx->SCOPE_STACK(scope)->get(ctx->SCOPE_STACK(scope),i))

/* Macros for accessing things in the parser
 */

#undef	    PARSER
#undef	    RECOGNIZER
#undef	    HAVEPARSEDRULE
#undef		MEMOIZE
#undef	    INPUT
#undef	    STRSTREAM
#undef	    HASEXCEPTION
#undef	    EXCEPTION
#undef	    MATCHT
#undef	    MATCHANYT
#undef	    FOLLOWSTACK
#undef	    FOLLOWPUSH
#undef	    FOLLOWPOP
#undef	    PRECOVER
#undef	    PREPORTERROR
#undef	    LA
#undef	    LT
#undef	    CONSTRUCTEX
#undef	    CONSUME
#undef	    MARK
#undef	    REWIND
#undef	    REWINDLAST
#undef	    PERRORRECOVERY
#undef	    HASFAILED
#undef	    FAILEDFLAG
#undef	    RECOVERFROMMISMATCHEDSET
#undef	    RECOVERFROMMISMATCHEDELEMENT
#undef		INDEX
#undef      ADAPTOR
#undef		SEEK
#undef	    RULEMEMO
#undef		DBG

#define	    PARSER				ctx->pParser
#define	    RECOGNIZER				PARSER->rec
#define	    PSRSTATE				RECOGNIZER->state
#define	    HAVEPARSEDRULE(r)			RECOGNIZER->alreadyParsedRule(RECOGNIZER, r)
#define	    MEMOIZE(ri,si)			RECOGNIZER->memoize(RECOGNIZER, ri, si)
#define	    INPUT				PARSER->tstream
#define	    STRSTREAM				INPUT
#define	    ISTREAM				INPUT->istream
#define	    INDEX()				ISTREAM->index(INPUT->istream)
#define	    HASEXCEPTION()			(PSRSTATE->error == ANTLR3_TRUE)
#define	    EXCEPTION				PSRSTATE->exception
#define	    MATCHT(t, fs)			RECOGNIZER->match(RECOGNIZER, t, fs)
#define	    MATCHANYT()				RECOGNIZER->matchAny(RECOGNIZER)
#define	    FOLLOWSTACK				PSRSTATE->following
#ifdef  SKIP_FOLLOW_SETS
#define	    FOLLOWPUSH(x)
#define	    FOLLOWPOP()
#else
#define	    FOLLOWPUSH(x)			FOLLOWSTACK->push(FOLLOWSTACK, ((void *)(&(x))), NULL)
#define	    FOLLOWPOP()				FOLLOWSTACK->pop(FOLLOWSTACK)
#endif
#define	    PRECOVER()				RECOGNIZER->recover(RECOGNIZER)
#define	    PREPORTERROR()			RECOGNIZER->reportError(RECOGNIZER)
#define	    LA(n)				INPUT->istream->_LA(ISTREAM, n)
#define	    LT(n)				INPUT->_LT(INPUT, n)
#define	    CONSTRUCTEX()			RECOGNIZER->exConstruct(RECOGNIZER)
#define	    CONSUME()				ISTREAM->consume(ISTREAM)
#define	    MARK()				ISTREAM->mark(ISTREAM)
#define	    REWIND(m)				ISTREAM->rewind(ISTREAM, m)
#define	    REWINDLAST()			ISTREAM->rewindLast(ISTREAM)
#define	    SEEK(n)				ISTREAM->seek(ISTREAM, n)
#define	    PERRORRECOVERY			PSRSTATE->errorRecovery
#define	    FAILEDFLAG				PSRSTATE->failed
#define	    HASFAILED()				(FAILEDFLAG == ANTLR3_TRUE)
#define	    BACKTRACKING			PSRSTATE->backtracking
#define	    RECOVERFROMMISMATCHEDSET(s)		RECOGNIZER->recoverFromMismatchedSet(RECOGNIZER, s)
#define	    RECOVERFROMMISMATCHEDELEMENT(e)	RECOGNIZER->recoverFromMismatchedElement(RECOGNIZER, s)
#define     ADAPTOR                         ctx->adaptor
#define		RULEMEMO						PSRSTATE->ruleMemo
#define		DBG								RECOGNIZER->debugger


#define		TOKTEXT(tok, txt)				tok, (pANTLR3_UINT8)txt

/* The 4 tokens defined below may well clash with your own #defines or token types. If so
 * then for the present you must use different names for your defines as these are hard coded
 * in the code generator. It would be better not to use such names internally, and maybe
 * we can change this in a forthcoming release. I deliberately do not #undef these
 * here as this will at least give you a redefined error somewhere if they clash.
 */
#define	    UP	    ANTLR3_TOKEN_UP
#define	    DOWN    ANTLR3_TOKEN_DOWN
#define	    EOR	    ANTLR3_TOKEN_EOR
#define	    INVALID ANTLR3_TOKEN_INVALID


/* =============================================================================
 * Functions to create and destroy scopes. First come the rule scopes, followed
 * by the global declared scopes.
 */



/* ============================================================================= */

/* =============================================================================
 * Start of recognizer
 */


/** \brief Table of all token names in symbolic order, mainly used for
 *         error reporting.
 */
pANTLR3_UINT8   MySQLSimpleParserTokenNames[809+4]
     = {
        (pANTLR3_UINT8) "<invalid>",       /* String to print to indicate an invalid token */
        (pANTLR3_UINT8) "<EOR>",
        (pANTLR3_UINT8) "<DOWN>",
        (pANTLR3_UINT8) "<UP>",
        (pANTLR3_UINT8) "ACCOUNT_SYMBOL",
        (pANTLR3_UINT8) "ASCII_SYMBOL",
        (pANTLR3_UINT8) "ALWAYS_SYMBOL",
        (pANTLR3_UINT8) "BACKUP_SYMBOL",
        (pANTLR3_UINT8) "BEGIN_SYMBOL",
        (pANTLR3_UINT8) "BYTE_SYMBOL",
        (pANTLR3_UINT8) "CACHE_SYMBOL",
        (pANTLR3_UINT8) "CHARSET_SYMBOL",
        (pANTLR3_UINT8) "CHECKSUM_SYMBOL",
        (pANTLR3_UINT8) "CLOSE_SYMBOL",
        (pANTLR3_UINT8) "COMMENT_SYMBOL",
        (pANTLR3_UINT8) "COMMIT_SYMBOL",
        (pANTLR3_UINT8) "CONTAINS_SYMBOL",
        (pANTLR3_UINT8) "DEALLOCATE_SYMBOL",
        (pANTLR3_UINT8) "DO_SYMBOL",
        (pANTLR3_UINT8) "END_SYMBOL",
        (pANTLR3_UINT8) "EXECUTE_SYMBOL",
        (pANTLR3_UINT8) "FLUSH_SYMBOL",
        (pANTLR3_UINT8) "FOLLOWS_SYMBOL",
        (pANTLR3_UINT8) "FORMAT_SYMBOL",
        (pANTLR3_UINT8) "GROUP_REPLICATION_SYMBOL",
        (pANTLR3_UINT8) "HANDLER_SYMBOL",
        (pANTLR3_UINT8) "HELP_SYMBOL",
        (pANTLR3_UINT8) "HOST_SYMBOL",
        (pANTLR3_UINT8) "INSTALL_SYMBOL",
        (pANTLR3_UINT8) "LANGUAGE_SYMBOL",
        (pANTLR3_UINT8) "NO_SYMBOL",
        (pANTLR3_UINT8) "OPEN_SYMBOL",
        (pANTLR3_UINT8) "OPTIONS_SYMBOL",
        (pANTLR3_UINT8) "OWNER_SYMBOL",
        (pANTLR3_UINT8) "PARSER_SYMBOL",
        (pANTLR3_UINT8) "PARTITION_SYMBOL",
        (pANTLR3_UINT8) "PORT_SYMBOL",
        (pANTLR3_UINT8) "PRECEDES_SYMBOL",
        (pANTLR3_UINT8) "PREPARE_SYMBOL",
        (pANTLR3_UINT8) "REMOVE_SYMBOL",
        (pANTLR3_UINT8) "REPAIR_SYMBOL",
        (pANTLR3_UINT8) "RESET_SYMBOL",
        (pANTLR3_UINT8) "RESTORE_SYMBOL",
        (pANTLR3_UINT8) "ROLLBACK_SYMBOL",
        (pANTLR3_UINT8) "SAVEPOINT_SYMBOL",
        (pANTLR3_UINT8) "SECURITY_SYMBOL",
        (pANTLR3_UINT8) "SERVER_SYMBOL",
        (pANTLR3_UINT8) "SIGNED_SYMBOL",
        (pANTLR3_UINT8) "SLAVE_SYMBOL",
        (pANTLR3_UINT8) "SOCKET_SYMBOL",
        (pANTLR3_UINT8) "SONAME_SYMBOL",
        (pANTLR3_UINT8) "START_SYMBOL",
        (pANTLR3_UINT8) "STOP_SYMBOL",
        (pANTLR3_UINT8) "TRUNCATE_SYMBOL",
        (pANTLR3_UINT8) "UNICODE_SYMBOL",
        (pANTLR3_UINT8) "UNINSTALL_SYMBOL",
        (pANTLR3_UINT8) "UPGRADE_SYMBOL",
        (pANTLR3_UINT8) "WRAPPER_SYMBOL",
        (pANTLR3_UINT8) "XA_SYMBOL",
        (pANTLR3_UINT8) "ACTION_SYMBOL",
        (pANTLR3_UINT8) "ADDDATE_SYMBOL",
        (pANTLR3_UINT8) "AFTER_SYMBOL",
        (pANTLR3_UINT8) "AGAINST_SYMBOL",
        (pANTLR3_UINT8) "AGGREGATE_SYMBOL",
        (pANTLR3_UINT8) "ALGORITHM_SYMBOL",
        (pANTLR3_UINT8) "ANALYZE_SYMBOL",
        (pANTLR3_UINT8) "ANY_SYMBOL",
        (pANTLR3_UINT8) "AT_SYMBOL",
        (pANTLR3_UINT8) "AUTHORS_SYMBOL",
        (pANTLR3_UINT8) "AUTO_INCREMENT_SYMBOL",
        (pANTLR3_UINT8) "AUTOEXTEND_SIZE_SYMBOL",
        (pANTLR3_UINT8) "AVG_ROW_LENGTH_SYMBOL",
        (pANTLR3_UINT8) "AVG_SYMBOL",
        (pANTLR3_UINT8) "BINLOG_SYMBOL",
        (pANTLR3_UINT8) "BIT_SYMBOL",
        (pANTLR3_UINT8) "BLOCK_SYMBOL",
        (pANTLR3_UINT8) "BOOL_SYMBOL",
        (pANTLR3_UINT8) "BOOLEAN_SYMBOL",
        (pANTLR3_UINT8) "BTREE_SYMBOL",
        (pANTLR3_UINT8) "CASCADED_SYMBOL",
        (pANTLR3_UINT8) "CATALOG_NAME_SYMBOL",
        (pANTLR3_UINT8) "CHAIN_SYMBOL",
        (pANTLR3_UINT8) "CHANGED_SYMBOL",
        (pANTLR3_UINT8) "CHANNEL_SYMBOL",
        (pANTLR3_UINT8) "CIPHER_SYMBOL",
        (pANTLR3_UINT8) "CLIENT_SYMBOL",
        (pANTLR3_UINT8) "CLASS_ORIGIN_SYMBOL",
        (pANTLR3_UINT8) "COALESCE_SYMBOL",
        (pANTLR3_UINT8) "CODE_SYMBOL",
        (pANTLR3_UINT8) "COLLATION_SYMBOL",
        (pANTLR3_UINT8) "COLUMN_NAME_SYMBOL",
        (pANTLR3_UINT8) "COLUMN_FORMAT_SYMBOL",
        (pANTLR3_UINT8) "COLUMNS_SYMBOL",
        (pANTLR3_UINT8) "COMMITTED_SYMBOL",
        (pANTLR3_UINT8) "COMPACT_SYMBOL",
        (pANTLR3_UINT8) "COMPLETION_SYMBOL",
        (pANTLR3_UINT8) "COMPRESSED_SYMBOL",
        (pANTLR3_UINT8) "COMPRESSION_SYMBOL",
        (pANTLR3_UINT8) "CONCURRENT_SYMBOL",
        (pANTLR3_UINT8) "CONNECTION_SYMBOL",
        (pANTLR3_UINT8) "CONSISTENT_SYMBOL",
        (pANTLR3_UINT8) "CONSTRAINT_CATALOG_SYMBOL",
        (pANTLR3_UINT8) "CONSTRAINT_SCHEMA_SYMBOL",
        (pANTLR3_UINT8) "CONSTRAINT_NAME_SYMBOL",
        (pANTLR3_UINT8) "CONTEXT_SYMBOL",
        (pANTLR3_UINT8) "CONTRIBUTORS_SYMBOL",
        (pANTLR3_UINT8) "CPU_SYMBOL",
        (pANTLR3_UINT8) "CUBE_SYMBOL",
        (pANTLR3_UINT8) "CURRENT_SYMBOL",
        (pANTLR3_UINT8) "CURSOR_NAME_SYMBOL",
        (pANTLR3_UINT8) "DATA_SYMBOL",
        (pANTLR3_UINT8) "DATAFILE_SYMBOL",
        (pANTLR3_UINT8) "DATETIME_SYMBOL",
        (pANTLR3_UINT8) "DATE_SYMBOL",
        (pANTLR3_UINT8) "DAY_SYMBOL",
        (pANTLR3_UINT8) "DEFAULT_AUTH_SYMBOL",
        (pANTLR3_UINT8) "DEFINER_SYMBOL",
        (pANTLR3_UINT8) "DELAY_KEY_WRITE_SYMBOL",
        (pANTLR3_UINT8) "DES_KEY_FILE_SYMBOL",
        (pANTLR3_UINT8) "DIAGNOSTICS_SYMBOL",
        (pANTLR3_UINT8) "DIRECTORY_SYMBOL",
        (pANTLR3_UINT8) "DISABLE_SYMBOL",
        (pANTLR3_UINT8) "DISCARD_SYMBOL",
        (pANTLR3_UINT8) "DISK_SYMBOL",
        (pANTLR3_UINT8) "DUMPFILE_SYMBOL",
        (pANTLR3_UINT8) "DUPLICATE_SYMBOL",
        (pANTLR3_UINT8) "DYNAMIC_SYMBOL",
        (pANTLR3_UINT8) "ENDS_SYMBOL",
        (pANTLR3_UINT8) "ENUM_SYMBOL",
        (pANTLR3_UINT8) "ENGINE_SYMBOL",
        (pANTLR3_UINT8) "ENGINES_SYMBOL",
        (pANTLR3_UINT8) "ERROR_SYMBOL",
        (pANTLR3_UINT8) "ERRORS_SYMBOL",
        (pANTLR3_UINT8) "ESCAPE_SYMBOL",
        (pANTLR3_UINT8) "EVENT_SYMBOL",
        (pANTLR3_UINT8) "EVENTS_SYMBOL",
        (pANTLR3_UINT8) "EVERY_SYMBOL",
        (pANTLR3_UINT8) "EXPANSION_SYMBOL",
        (pANTLR3_UINT8) "EXPORT_SYMBOL",
        (pANTLR3_UINT8) "EXTENDED_SYMBOL",
        (pANTLR3_UINT8) "EXTENT_SIZE_SYMBOL",
        (pANTLR3_UINT8) "FAULTS_SYMBOL",
        (pANTLR3_UINT8) "FAST_SYMBOL",
        (pANTLR3_UINT8) "FOUND_SYMBOL",
        (pANTLR3_UINT8) "ENABLE_SYMBOL",
        (pANTLR3_UINT8) "FULL_SYMBOL",
        (pANTLR3_UINT8) "FILE_SYMBOL",
        (pANTLR3_UINT8) "FILE_BLOCK_SIZE_SYMBOL",
        (pANTLR3_UINT8) "FILTER_SYMBOL",
        (pANTLR3_UINT8) "FIRST_SYMBOL",
        (pANTLR3_UINT8) "FIXED_SYMBOL",
        (pANTLR3_UINT8) "GENERAL_SYMBOL",
        (pANTLR3_UINT8) "GEOMETRY_SYMBOL",
        (pANTLR3_UINT8) "GEOMETRYCOLLECTION_SYMBOL",
        (pANTLR3_UINT8) "GET_FORMAT_SYMBOL",
        (pANTLR3_UINT8) "GRANTS_SYMBOL",
        (pANTLR3_UINT8) "GLOBAL_SYMBOL",
        (pANTLR3_UINT8) "HASH_SYMBOL",
        (pANTLR3_UINT8) "HOSTS_SYMBOL",
        (pANTLR3_UINT8) "HOUR_SYMBOL",
        (pANTLR3_UINT8) "IDENTIFIED_SYMBOL",
        (pANTLR3_UINT8) "IGNORE_SERVER_IDS_SYMBOL",
        (pANTLR3_UINT8) "INVOKER_SYMBOL",
        (pANTLR3_UINT8) "IMPORT_SYMBOL",
        (pANTLR3_UINT8) "INDEXES_SYMBOL",
        (pANTLR3_UINT8) "INITIAL_SIZE_SYMBOL",
        (pANTLR3_UINT8) "INNODB_SYMBOL",
        (pANTLR3_UINT8) "IO_SYMBOL",
        (pANTLR3_UINT8) "IPC_SYMBOL",
        (pANTLR3_UINT8) "ISOLATION_SYMBOL",
        (pANTLR3_UINT8) "ISSUER_SYMBOL",
        (pANTLR3_UINT8) "INSERT_METHOD_SYMBOL",
        (pANTLR3_UINT8) "JSON_SYMBOL",
        (pANTLR3_UINT8) "KEY_BLOCK_SIZE_SYMBOL",
        (pANTLR3_UINT8) "LAST_SYMBOL",
        (pANTLR3_UINT8) "LEAVES_SYMBOL",
        (pANTLR3_UINT8) "LESS_SYMBOL",
        (pANTLR3_UINT8) "LEVEL_SYMBOL",
        (pANTLR3_UINT8) "LINESTRING_SYMBOL",
        (pANTLR3_UINT8) "LIST_SYMBOL",
        (pANTLR3_UINT8) "LOCAL_SYMBOL",
        (pANTLR3_UINT8) "LOCKS_SYMBOL",
        (pANTLR3_UINT8) "LOGFILE_SYMBOL",
        (pANTLR3_UINT8) "LOGS_SYMBOL",
        (pANTLR3_UINT8) "MAX_ROWS_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SYMBOL",
        (pANTLR3_UINT8) "MASTER_HEARTBEAT_PERIOD_SYMBOL",
        (pANTLR3_UINT8) "MASTER_HOST_SYMBOL",
        (pANTLR3_UINT8) "MASTER_PORT_SYMBOL",
        (pANTLR3_UINT8) "MASTER_LOG_FILE_SYMBOL",
        (pANTLR3_UINT8) "MASTER_LOG_POS_SYMBOL",
        (pANTLR3_UINT8) "MASTER_USER_SYMBOL",
        (pANTLR3_UINT8) "MASTER_PASSWORD_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SERVER_ID_SYMBOL",
        (pANTLR3_UINT8) "MASTER_CONNECT_RETRY_SYMBOL",
        (pANTLR3_UINT8) "MASTER_RETRY_COUNT_SYMBOL",
        (pANTLR3_UINT8) "MASTER_DELAY_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CA_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CAPATH_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CERT_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CIPHER_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CRL_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_CRLPATH_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_KEY_SYMBOL",
        (pANTLR3_UINT8) "MASTER_AUTO_POSITION_SYMBOL",
        (pANTLR3_UINT8) "MAX_CONNECTIONS_PER_HOUR_SYMBOL",
        (pANTLR3_UINT8) "MAX_QUERIES_PER_HOUR_SYMBOL",
        (pANTLR3_UINT8) "MAX_STATEMENT_TIME_SYMBOL",
        (pANTLR3_UINT8) "MAX_SIZE_SYMBOL",
        (pANTLR3_UINT8) "MAX_UPDATES_PER_HOUR_SYMBOL",
        (pANTLR3_UINT8) "MAX_USER_CONNECTIONS_SYMBOL",
        (pANTLR3_UINT8) "MEDIUM_SYMBOL",
        (pANTLR3_UINT8) "MEMORY_SYMBOL",
        (pANTLR3_UINT8) "MERGE_SYMBOL",
        (pANTLR3_UINT8) "MESSAGE_TEXT_SYMBOL",
        (pANTLR3_UINT8) "MICROSECOND_SYMBOL",
        (pANTLR3_UINT8) "MIGRATE_SYMBOL",
        (pANTLR3_UINT8) "MINUTE_SYMBOL",
        (pANTLR3_UINT8) "MIN_ROWS_SYMBOL",
        (pANTLR3_UINT8) "MODIFY_SYMBOL",
        (pANTLR3_UINT8) "MODE_SYMBOL",
        (pANTLR3_UINT8) "MONTH_SYMBOL",
        (pANTLR3_UINT8) "MULTILINESTRING_SYMBOL",
        (pANTLR3_UINT8) "MULTIPOINT_SYMBOL",
        (pANTLR3_UINT8) "MULTIPOLYGON_SYMBOL",
        (pANTLR3_UINT8) "MUTEX_SYMBOL",
        (pANTLR3_UINT8) "MYSQL_ERRNO_SYMBOL",
        (pANTLR3_UINT8) "NAME_SYMBOL",
        (pANTLR3_UINT8) "NAMES_SYMBOL",
        (pANTLR3_UINT8) "NATIONAL_SYMBOL",
        (pANTLR3_UINT8) "NCHAR_SYMBOL",
        (pANTLR3_UINT8) "NDBCLUSTER_SYMBOL",
        (pANTLR3_UINT8) "NEVER_SYMBOL",
        (pANTLR3_UINT8) "NEXT_SYMBOL",
        (pANTLR3_UINT8) "NEW_SYMBOL",
        (pANTLR3_UINT8) "NO_WAIT_SYMBOL",
        (pANTLR3_UINT8) "NODEGROUP_SYMBOL",
        (pANTLR3_UINT8) "NONE_SYMBOL",
        (pANTLR3_UINT8) "NUMBER_SYMBOL",
        (pANTLR3_UINT8) "NVARCHAR_SYMBOL",
        (pANTLR3_UINT8) "OFFSET_SYMBOL",
        (pANTLR3_UINT8) "OLD_PASSWORD_SYMBOL",
        (pANTLR3_UINT8) "ONE_SHOT_SYMBOL",
        (pANTLR3_UINT8) "ONE_SYMBOL",
        (pANTLR3_UINT8) "PACK_KEYS_SYMBOL",
        (pANTLR3_UINT8) "PAGE_SYMBOL",
        (pANTLR3_UINT8) "PARTIAL_SYMBOL",
        (pANTLR3_UINT8) "PARTITIONING_SYMBOL",
        (pANTLR3_UINT8) "PARTITIONS_SYMBOL",
        (pANTLR3_UINT8) "PASSWORD_SYMBOL",
        (pANTLR3_UINT8) "PHASE_SYMBOL",
        (pANTLR3_UINT8) "PLUGIN_DIR_SYMBOL",
        (pANTLR3_UINT8) "PLUGIN_SYMBOL",
        (pANTLR3_UINT8) "PLUGINS_SYMBOL",
        (pANTLR3_UINT8) "POINT_SYMBOL",
        (pANTLR3_UINT8) "POLYGON_SYMBOL",
        (pANTLR3_UINT8) "PRESERVE_SYMBOL",
        (pANTLR3_UINT8) "PREV_SYMBOL",
        (pANTLR3_UINT8) "PRIVILEGES_SYMBOL",
        (pANTLR3_UINT8) "PROCESS_SYMBOL",
        (pANTLR3_UINT8) "PROCESSLIST_SYMBOL",
        (pANTLR3_UINT8) "PROFILE_SYMBOL",
        (pANTLR3_UINT8) "PROFILES_SYMBOL",
        (pANTLR3_UINT8) "PROXY_SYMBOL",
        (pANTLR3_UINT8) "QUARTER_SYMBOL",
        (pANTLR3_UINT8) "QUERY_SYMBOL",
        (pANTLR3_UINT8) "QUICK_SYMBOL",
        (pANTLR3_UINT8) "READ_ONLY_SYMBOL",
        (pANTLR3_UINT8) "REBUILD_SYMBOL",
        (pANTLR3_UINT8) "RECOVER_SYMBOL",
        (pANTLR3_UINT8) "REDO_BUFFER_SIZE_SYMBOL",
        (pANTLR3_UINT8) "REDOFILE_SYMBOL",
        (pANTLR3_UINT8) "REDUNDANT_SYMBOL",
        (pANTLR3_UINT8) "RELAY_SYMBOL",
        (pANTLR3_UINT8) "RELAYLOG_SYMBOL",
        (pANTLR3_UINT8) "RELAY_LOG_FILE_SYMBOL",
        (pANTLR3_UINT8) "RELAY_LOG_POS_SYMBOL",
        (pANTLR3_UINT8) "RELAY_THREAD_SYMBOL",
        (pANTLR3_UINT8) "RELOAD_SYMBOL",
        (pANTLR3_UINT8) "REORGANIZE_SYMBOL",
        (pANTLR3_UINT8) "REPEATABLE_SYMBOL",
        (pANTLR3_UINT8) "REPLICATION_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_DO_DB_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_IGNORE_DB_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_DO_TABLE_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_IGNORE_TABLE_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_WILD_DO_TABLE_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_WILD_IGNORE_TABLE_SYMBOL",
        (pANTLR3_UINT8) "REPLICATE_REWRITE_DB_SYMBOL",
        (pANTLR3_UINT8) "RESUME_SYMBOL",
        (pANTLR3_UINT8) "RETURNED_SQLSTATE_SYMBOL",
        (pANTLR3_UINT8) "RETURNS_SYMBOL",
        (pANTLR3_UINT8) "REVERSE_SYMBOL",
        (pANTLR3_UINT8) "ROLLUP_SYMBOL",
        (pANTLR3_UINT8) "ROUTINE_SYMBOL",
        (pANTLR3_UINT8) "ROWS_SYMBOL",
        (pANTLR3_UINT8) "ROW_COUNT_SYMBOL",
        (pANTLR3_UINT8) "ROW_FORMAT_SYMBOL",
        (pANTLR3_UINT8) "ROW_SYMBOL",
        (pANTLR3_UINT8) "RTREE_SYMBOL",
        (pANTLR3_UINT8) "SCHEDULE_SYMBOL",
        (pANTLR3_UINT8) "SCHEMA_NAME_SYMBOL",
        (pANTLR3_UINT8) "SECOND_SYMBOL",
        (pANTLR3_UINT8) "SERIAL_SYMBOL",
        (pANTLR3_UINT8) "SERIALIZABLE_SYMBOL",
        (pANTLR3_UINT8) "SESSION_SYMBOL",
        (pANTLR3_UINT8) "SIMPLE_SYMBOL",
        (pANTLR3_UINT8) "SHARE_SYMBOL",
        (pANTLR3_UINT8) "SHUTDOWN_SYMBOL",
        (pANTLR3_UINT8) "SLOW_SYMBOL",
        (pANTLR3_UINT8) "SNAPSHOT_SYMBOL",
        (pANTLR3_UINT8) "SOUNDS_SYMBOL",
        (pANTLR3_UINT8) "SOURCE_SYMBOL",
        (pANTLR3_UINT8) "SQL_AFTER_GTIDS_SYMBOL",
        (pANTLR3_UINT8) "SQL_AFTER_MTS_GAPS_SYMBOL",
        (pANTLR3_UINT8) "SQL_BEFORE_GTIDS_SYMBOL",
        (pANTLR3_UINT8) "SQL_CACHE_SYMBOL",
        (pANTLR3_UINT8) "SQL_BUFFER_RESULT_SYMBOL",
        (pANTLR3_UINT8) "SQL_NO_CACHE_SYMBOL",
        (pANTLR3_UINT8) "SQL_THREAD_SYMBOL",
        (pANTLR3_UINT8) "STACKED_SYMBOL",
        (pANTLR3_UINT8) "STARTS_SYMBOL",
        (pANTLR3_UINT8) "STATS_AUTO_RECALC_SYMBOL",
        (pANTLR3_UINT8) "STATS_PERSISTENT_SYMBOL",
        (pANTLR3_UINT8) "STATS_SAMPLE_PAGES_SYMBOL",
        (pANTLR3_UINT8) "STATUS_SYMBOL",
        (pANTLR3_UINT8) "STORAGE_SYMBOL",
        (pANTLR3_UINT8) "STRING_SYMBOL",
        (pANTLR3_UINT8) "SUBCLASS_ORIGIN_SYMBOL",
        (pANTLR3_UINT8) "SUBDATE_SYMBOL",
        (pANTLR3_UINT8) "SUBJECT_SYMBOL",
        (pANTLR3_UINT8) "SUBPARTITION_SYMBOL",
        (pANTLR3_UINT8) "SUBPARTITIONS_SYMBOL",
        (pANTLR3_UINT8) "SUPER_SYMBOL",
        (pANTLR3_UINT8) "SUSPEND_SYMBOL",
        (pANTLR3_UINT8) "SWAPS_SYMBOL",
        (pANTLR3_UINT8) "SWITCHES_SYMBOL",
        (pANTLR3_UINT8) "TABLE_NAME_SYMBOL",
        (pANTLR3_UINT8) "TABLES_SYMBOL",
        (pANTLR3_UINT8) "TABLE_CHECKSUM_SYMBOL",
        (pANTLR3_UINT8) "TABLESPACE_SYMBOL",
        (pANTLR3_UINT8) "TEMPORARY_SYMBOL",
        (pANTLR3_UINT8) "TEMPTABLE_SYMBOL",
        (pANTLR3_UINT8) "TEXT_SYMBOL",
        (pANTLR3_UINT8) "THAN_SYMBOL",
        (pANTLR3_UINT8) "TRANSACTION_SYMBOL",
        (pANTLR3_UINT8) "TRIGGERS_SYMBOL",
        (pANTLR3_UINT8) "TIMESTAMP_SYMBOL",
        (pANTLR3_UINT8) "TIMESTAMP_ADD_SYMBOL",
        (pANTLR3_UINT8) "TIMESTAMP_DIFF_SYMBOL",
        (pANTLR3_UINT8) "TIME_SYMBOL",
        (pANTLR3_UINT8) "TYPES_SYMBOL",
        (pANTLR3_UINT8) "TYPE_SYMBOL",
        (pANTLR3_UINT8) "UDF_RETURNS_SYMBOL",
        (pANTLR3_UINT8) "FUNCTION_SYMBOL",
        (pANTLR3_UINT8) "UNCOMMITTED_SYMBOL",
        (pANTLR3_UINT8) "UNDEFINED_SYMBOL",
        (pANTLR3_UINT8) "UNDO_BUFFER_SIZE_SYMBOL",
        (pANTLR3_UINT8) "UNDOFILE_SYMBOL",
        (pANTLR3_UINT8) "UNKNOWN_SYMBOL",
        (pANTLR3_UINT8) "UNTIL_SYMBOL",
        (pANTLR3_UINT8) "USER_RESOURCES_SYMBOL",
        (pANTLR3_UINT8) "USER_SYMBOL",
        (pANTLR3_UINT8) "USE_FRM_SYMBOL",
        (pANTLR3_UINT8) "VARIABLES_SYMBOL",
        (pANTLR3_UINT8) "VIEW_SYMBOL",
        (pANTLR3_UINT8) "VALUE_SYMBOL",
        (pANTLR3_UINT8) "WARNINGS_SYMBOL",
        (pANTLR3_UINT8) "WAIT_SYMBOL",
        (pANTLR3_UINT8) "WEEK_SYMBOL",
        (pANTLR3_UINT8) "WORK_SYMBOL",
        (pANTLR3_UINT8) "WEIGHT_STRING_SYMBOL",
        (pANTLR3_UINT8) "X509_SYMBOL",
        (pANTLR3_UINT8) "XID_SYMBOL",
        (pANTLR3_UINT8) "XML_SYMBOL",
        (pANTLR3_UINT8) "YEAR_SYMBOL",
        (pANTLR3_UINT8) "ACCESSIBLE_SYMBOL",
        (pANTLR3_UINT8) "ADD_SYMBOL",
        (pANTLR3_UINT8) "ALL_SYMBOL",
        (pANTLR3_UINT8) "ALTER_SYMBOL",
        (pANTLR3_UINT8) "ANALYSE_SYMBOL",
        (pANTLR3_UINT8) "AND_SYMBOL",
        (pANTLR3_UINT8) "ASC_SYMBOL",
        (pANTLR3_UINT8) "ASENSITIVE_SYMBOL",
        (pANTLR3_UINT8) "BEFORE_SYMBOL",
        (pANTLR3_UINT8) "BETWEEN_SYMBOL",
        (pANTLR3_UINT8) "BIGINT_SYMBOL",
        (pANTLR3_UINT8) "BINARY_SYMBOL",
        (pANTLR3_UINT8) "BIN_NUM_SYMBOL",
        (pANTLR3_UINT8) "BLOB_SYMBOL",
        (pANTLR3_UINT8) "BLOCK_COMMENT",
        (pANTLR3_UINT8) "BOTH_SYMBOL",
        (pANTLR3_UINT8) "BY_SYMBOL",
        (pANTLR3_UINT8) "CALL_SYMBOL",
        (pANTLR3_UINT8) "CASCADE_SYMBOL",
        (pANTLR3_UINT8) "CASE_SYMBOL",
        (pANTLR3_UINT8) "CAST_SYMBOL",
        (pANTLR3_UINT8) "CHANGE_SYMBOL",
        (pANTLR3_UINT8) "CHARACTER_SYMBOL",
        (pANTLR3_UINT8) "CHAR_SYMBOL",
        (pANTLR3_UINT8) "CHECK_SYMBOL",
        (pANTLR3_UINT8) "COLLATE_SYMBOL",
        (pANTLR3_UINT8) "COLUMN_SYMBOL",
        (pANTLR3_UINT8) "COMMENT_RULE",
        (pANTLR3_UINT8) "CONDITION_SYMBOL",
        (pANTLR3_UINT8) "CONSTRAINT_SYMBOL",
        (pANTLR3_UINT8) "CONTINUE_SYMBOL",
        (pANTLR3_UINT8) "CONVERT_SYMBOL",
        (pANTLR3_UINT8) "COUNT_SYMBOL",
        (pANTLR3_UINT8) "CREATE_SYMBOL",
        (pANTLR3_UINT8) "CROSS_SYMBOL",
        (pANTLR3_UINT8) "CURDATE_SYMBOL",
        (pANTLR3_UINT8) "CURRENT_TIMESTAMP_SYMBOL",
        (pANTLR3_UINT8) "CURRENT_USER_SYMBOL",
        (pANTLR3_UINT8) "CURSOR_SYMBOL",
        (pANTLR3_UINT8) "CURTIME_SYMBOL",
        (pANTLR3_UINT8) "DASHDASH_COMMENT",
        (pANTLR3_UINT8) "DATABASES_SYMBOL",
        (pANTLR3_UINT8) "DATABASE_SYMBOL",
        (pANTLR3_UINT8) "DATE_ADD_INTERVAL_SYMBOL",
        (pANTLR3_UINT8) "DATE_ADD_SYMBOL",
        (pANTLR3_UINT8) "DATE_SUB_INTERVAL_SYMBOL",
        (pANTLR3_UINT8) "DATE_SUB_SYMBOL",
        (pANTLR3_UINT8) "DAYOFMONTH_SYMBOL",
        (pANTLR3_UINT8) "DAY_HOUR_SYMBOL",
        (pANTLR3_UINT8) "DAY_MICROSECOND_SYMBOL",
        (pANTLR3_UINT8) "DAY_MINUTE_SYMBOL",
        (pANTLR3_UINT8) "DAY_SECOND_SYMBOL",
        (pANTLR3_UINT8) "DECIMAL_NUM_SYMBOL",
        (pANTLR3_UINT8) "DECIMAL_SYMBOL",
        (pANTLR3_UINT8) "DECLARE_SYMBOL",
        (pANTLR3_UINT8) "DEC_SYMBOL",
        (pANTLR3_UINT8) "DEFAULT_SYMBOL",
        (pANTLR3_UINT8) "DELAYED_SYMBOL",
        (pANTLR3_UINT8) "DELETE_SYMBOL",
        (pANTLR3_UINT8) "DESCRIBE_SYMBOL",
        (pANTLR3_UINT8) "DESC_SYMBOL",
        (pANTLR3_UINT8) "DETERMINISTIC_SYMBOL",
        (pANTLR3_UINT8) "DISTINCTROW_SYMBOL",
        (pANTLR3_UINT8) "DISTINCT_SYMBOL",
        (pANTLR3_UINT8) "DROP_SYMBOL",
        (pANTLR3_UINT8) "DUAL_SYMBOL",
        (pANTLR3_UINT8) "EACH_SYMBOL",
        (pANTLR3_UINT8) "ELSEIF_SYMBOL",
        (pANTLR3_UINT8) "ELSE_SYMBOL",
        (pANTLR3_UINT8) "ENCLOSED_SYMBOL",
        (pANTLR3_UINT8) "END_OF_INPUT_SYMBOL",
        (pANTLR3_UINT8) "ESCAPED_SYMBOL",
        (pANTLR3_UINT8) "EXCHANGE_SYMBOL",
        (pANTLR3_UINT8) "EXISTS_SYMBOL",
        (pANTLR3_UINT8) "EXIT_SYMBOL",
        (pANTLR3_UINT8) "EXPIRE_SYMBOL",
        (pANTLR3_UINT8) "EXPLAIN_SYMBOL",
        (pANTLR3_UINT8) "EXTRACT_SYMBOL",
        (pANTLR3_UINT8) "FALSE_SYMBOL",
        (pANTLR3_UINT8) "FETCH_SYMBOL",
        (pANTLR3_UINT8) "FIELDS_SYMBOL",
        (pANTLR3_UINT8) "FLOAT4_SYMBOL",
        (pANTLR3_UINT8) "FLOAT8_SYMBOL",
        (pANTLR3_UINT8) "FLOAT_SYMBOL",
        (pANTLR3_UINT8) "FORCE_SYMBOL",
        (pANTLR3_UINT8) "FOREIGN_SYMBOL",
        (pANTLR3_UINT8) "FOR_SYMBOL",
        (pANTLR3_UINT8) "FRAC_SECOND_SYMBOL",
        (pANTLR3_UINT8) "FROM_SYMBOL",
        (pANTLR3_UINT8) "FULLTEXT_SYMBOL",
        (pANTLR3_UINT8) "GENERATED_SYMBOL",
        (pANTLR3_UINT8) "GET_SYMBOL",
        (pANTLR3_UINT8) "GRANT_SYMBOL",
        (pANTLR3_UINT8) "GROUP_CONCAT_SYMBOL",
        (pANTLR3_UINT8) "GROUP_SYMBOL",
        (pANTLR3_UINT8) "HAVING_SYMBOL",
        (pANTLR3_UINT8) "HIGH_PRIORITY_SYMBOL",
        (pANTLR3_UINT8) "HOUR_MICROSECOND_SYMBOL",
        (pANTLR3_UINT8) "HOUR_MINUTE_SYMBOL",
        (pANTLR3_UINT8) "HOUR_SECOND_SYMBOL",
        (pANTLR3_UINT8) "IF_SYMBOL",
        (pANTLR3_UINT8) "IGNORE_SYMBOL",
        (pANTLR3_UINT8) "INDEX_SYMBOL",
        (pANTLR3_UINT8) "INFILE_SYMBOL",
        (pANTLR3_UINT8) "INNER_SYMBOL",
        (pANTLR3_UINT8) "INOUT_SYMBOL",
        (pANTLR3_UINT8) "INSENSITIVE_SYMBOL",
        (pANTLR3_UINT8) "INSERT_SYMBOL",
        (pANTLR3_UINT8) "INT1_SYMBOL",
        (pANTLR3_UINT8) "INT2_SYMBOL",
        (pANTLR3_UINT8) "INT3_SYMBOL",
        (pANTLR3_UINT8) "INT4_SYMBOL",
        (pANTLR3_UINT8) "INT8_SYMBOL",
        (pANTLR3_UINT8) "INTEGER_SYMBOL",
        (pANTLR3_UINT8) "INTERVAL_SYMBOL",
        (pANTLR3_UINT8) "INTO_SYMBOL",
        (pANTLR3_UINT8) "INT_SYMBOL",
        (pANTLR3_UINT8) "IN_SYMBOL",
        (pANTLR3_UINT8) "IO_AFTER_GTIDS_SYMBOL",
        (pANTLR3_UINT8) "IO_BEFORE_GTIDS_SYMBOL",
        (pANTLR3_UINT8) "IO_THREAD_SYMBOL",
        (pANTLR3_UINT8) "IS_SYMBOL",
        (pANTLR3_UINT8) "ITERATE_SYMBOL",
        (pANTLR3_UINT8) "JOIN_SYMBOL",
        (pANTLR3_UINT8) "JSON_SEPARATOR_SYMBOL",
        (pANTLR3_UINT8) "KEYS_SYMBOL",
        (pANTLR3_UINT8) "KEY_SYMBOL",
        (pANTLR3_UINT8) "KILL_SYMBOL",
        (pANTLR3_UINT8) "LEADING_SYMBOL",
        (pANTLR3_UINT8) "LEAVE_SYMBOL",
        (pANTLR3_UINT8) "LEFT_SYMBOL",
        (pANTLR3_UINT8) "LIKE_SYMBOL",
        (pANTLR3_UINT8) "LIMIT_SYMBOL",
        (pANTLR3_UINT8) "LINEAR_SYMBOL",
        (pANTLR3_UINT8) "LINES_SYMBOL",
        (pANTLR3_UINT8) "LOAD_SYMBOL",
        (pANTLR3_UINT8) "LOCALTIMESTAMP_SYMBOL",
        (pANTLR3_UINT8) "LOCALTIME_SYMBOL",
        (pANTLR3_UINT8) "LOCATOR_SYMBOL",
        (pANTLR3_UINT8) "LOCK_SYMBOL",
        (pANTLR3_UINT8) "LONGBLOB_SYMBOL",
        (pANTLR3_UINT8) "LONGTEXT_SYMBOL",
        (pANTLR3_UINT8) "LONG_NUM_SYMBOL",
        (pANTLR3_UINT8) "LONG_SYMBOL",
        (pANTLR3_UINT8) "LOOP_SYMBOL",
        (pANTLR3_UINT8) "LOW_PRIORITY_SYMBOL",
        (pANTLR3_UINT8) "MASTER_BIND_SYMBOL",
        (pANTLR3_UINT8) "MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL",
        (pANTLR3_UINT8) "MATCH_SYMBOL",
        (pANTLR3_UINT8) "MAXVALUE_SYMBOL",
        (pANTLR3_UINT8) "MAX_SYMBOL",
        (pANTLR3_UINT8) "MEDIUMBLOB_SYMBOL",
        (pANTLR3_UINT8) "MEDIUMINT_SYMBOL",
        (pANTLR3_UINT8) "MEDIUMTEXT_SYMBOL",
        (pANTLR3_UINT8) "MIDDLEINT_SYMBOL",
        (pANTLR3_UINT8) "MID_SYMBOL",
        (pANTLR3_UINT8) "MINUTE_MICROSECOND_SYMBOL",
        (pANTLR3_UINT8) "MINUTE_SECOND_SYMBOL",
        (pANTLR3_UINT8) "MIN_SYMBOL",
        (pANTLR3_UINT8) "ML_COMMENT_END",
        (pANTLR3_UINT8) "ML_COMMENT_HEAD",
        (pANTLR3_UINT8) "MODIFIES_SYMBOL",
        (pANTLR3_UINT8) "MOD_SYMBOL",
        (pANTLR3_UINT8) "NATURAL_SYMBOL",
        (pANTLR3_UINT8) "NCHAR_STRING_SYMBOL",
        (pANTLR3_UINT8) "NCHAR_TEXT",
        (pANTLR3_UINT8) "NDB_SYMBOL",
        (pANTLR3_UINT8) "NEG_SYMBOL",
        (pANTLR3_UINT8) "NONBLOCKING_SYMBOL",
        (pANTLR3_UINT8) "NOT2_SYMBOL",
        (pANTLR3_UINT8) "NOT_SYMBOL",
        (pANTLR3_UINT8) "NOW_SYMBOL",
        (pANTLR3_UINT8) "NO_WRITE_TO_BINLOG_SYMBOL",
        (pANTLR3_UINT8) "NULL2_SYMBOL",
        (pANTLR3_UINT8) "NULL_SYMBOL",
        (pANTLR3_UINT8) "NUMERIC_SYMBOL",
        (pANTLR3_UINT8) "OFFLINE_SYMBOL",
        (pANTLR3_UINT8) "ONLINE_SYMBOL",
        (pANTLR3_UINT8) "ONLY_SYMBOL",
        (pANTLR3_UINT8) "ON_SYMBOL",
        (pANTLR3_UINT8) "OPEN_CURLY_SYMBOL",
        (pANTLR3_UINT8) "OPEN_PAR_SYMBOL",
        (pANTLR3_UINT8) "OPTIMIZER_COSTS_SYMBOL",
        (pANTLR3_UINT8) "OPTIMIZE_SYMBOL",
        (pANTLR3_UINT8) "OPTIONALLY_SYMBOL",
        (pANTLR3_UINT8) "OPTION_SYMBOL",
        (pANTLR3_UINT8) "ORDER_SYMBOL",
        (pANTLR3_UINT8) "OR_SYMBOL",
        (pANTLR3_UINT8) "OUTER_SYMBOL",
        (pANTLR3_UINT8) "OUTFILE_SYMBOL",
        (pANTLR3_UINT8) "OUT_SYMBOL",
        (pANTLR3_UINT8) "PARAM_MARKER",
        (pANTLR3_UINT8) "POSITION_SYMBOL",
        (pANTLR3_UINT8) "POUND_COMMENT",
        (pANTLR3_UINT8) "PRECISION_SYMBOL",
        (pANTLR3_UINT8) "PRIMARY_SYMBOL",
        (pANTLR3_UINT8) "PROCEDURE_SYMBOL",
        (pANTLR3_UINT8) "PURGE_SYMBOL",
        (pANTLR3_UINT8) "RANGE_SYMBOL",
        (pANTLR3_UINT8) "READS_SYMBOL",
        (pANTLR3_UINT8) "READ_SYMBOL",
        (pANTLR3_UINT8) "READ_WRITE_SYMBOL",
        (pANTLR3_UINT8) "REAL_SYMBOL",
        (pANTLR3_UINT8) "REFERENCES_SYMBOL",
        (pANTLR3_UINT8) "REGEXP_SYMBOL",
        (pANTLR3_UINT8) "RELEASE_SYMBOL",
        (pANTLR3_UINT8) "RENAME_SYMBOL",
        (pANTLR3_UINT8) "REPEAT_SYMBOL",
        (pANTLR3_UINT8) "REPLACE_SYMBOL",
        (pANTLR3_UINT8) "REQUIRE_SYMBOL",
        (pANTLR3_UINT8) "RESIGNAL_SYMBOL",
        (pANTLR3_UINT8) "RESTRICT_SYMBOL",
        (pANTLR3_UINT8) "RETURN_SYMBOL",
        (pANTLR3_UINT8) "REVOKE_SYMBOL",
        (pANTLR3_UINT8) "RIGHT_SYMBOL",
        (pANTLR3_UINT8) "RLIKE_SYMBOL",
        (pANTLR3_UINT8) "SCHEMAS_SYMBOL",
        (pANTLR3_UINT8) "SCHEMA_SYMBOL",
        (pANTLR3_UINT8) "SECOND_MICROSECOND_SYMBOL",
        (pANTLR3_UINT8) "SELECT_SYMBOL",
        (pANTLR3_UINT8) "SEMICOLON_SYMBOL",
        (pANTLR3_UINT8) "SENSITIVE_SYMBOL",
        (pANTLR3_UINT8) "SEPARATOR_SYMBOL",
        (pANTLR3_UINT8) "SERVER_OPTIONS_SYMBOL",
        (pANTLR3_UINT8) "SESSION_USER_SYMBOL",
        (pANTLR3_UINT8) "SET_SYMBOL",
        (pANTLR3_UINT8) "SET_VAR_SYMBOL",
        (pANTLR3_UINT8) "SHOW_SYMBOL",
        (pANTLR3_UINT8) "SIGNAL_SYMBOL",
        (pANTLR3_UINT8) "SINGLE_QUOTED_TEXT",
        (pANTLR3_UINT8) "SMALLINT_SYMBOL",
        (pANTLR3_UINT8) "SPATIAL_SYMBOL",
        (pANTLR3_UINT8) "SPECIFIC_SYMBOL",
        (pANTLR3_UINT8) "SQLEXCEPTION_SYMBOL",
        (pANTLR3_UINT8) "SQLSTATE_SYMBOL",
        (pANTLR3_UINT8) "SQLWARNING_SYMBOL",
        (pANTLR3_UINT8) "SQL_BIG_RESULT_SYMBOL",
        (pANTLR3_UINT8) "SQL_CALC_FOUND_ROWS_SYMBOL",
        (pANTLR3_UINT8) "SQL_SMALL_RESULT_SYMBOL",
        (pANTLR3_UINT8) "SQL_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_DAY_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_FRAC_SECOND_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_HOUR_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_MINUTE_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_MONTH_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_QUARTER_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_SECOND_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_WEEK_SYMBOL",
        (pANTLR3_UINT8) "SQL_TSI_YEAR_SYMBOL",
        (pANTLR3_UINT8) "SSL_SYMBOL",
        (pANTLR3_UINT8) "STARTING_SYMBOL",
        (pANTLR3_UINT8) "STDDEV_POP_SYMBOL",
        (pANTLR3_UINT8) "STDDEV_SAMP_SYMBOL",
        (pANTLR3_UINT8) "STDDEV_SYMBOL",
        (pANTLR3_UINT8) "STD_SYMBOL",
        (pANTLR3_UINT8) "STORED_SYMBOL",
        (pANTLR3_UINT8) "STRAIGHT_JOIN_SYMBOL",
        (pANTLR3_UINT8) "SUBSTRING_SYMBOL",
        (pANTLR3_UINT8) "SUBSTR_SYMBOL",
        (pANTLR3_UINT8) "SUM_SYMBOL",
        (pANTLR3_UINT8) "SYSDATE_SYMBOL",
        (pANTLR3_UINT8) "SYSTEM_USER_SYMBOL",
        (pANTLR3_UINT8) "TABLE_REF_PRIORITY_SYMBOL",
        (pANTLR3_UINT8) "TABLE_SYMBOL",
        (pANTLR3_UINT8) "TERMINATED_SYMBOL",
        (pANTLR3_UINT8) "THEN_SYMBOL",
        (pANTLR3_UINT8) "TINYBLOB_SYMBOL",
        (pANTLR3_UINT8) "TINYINT_SYMBOL",
        (pANTLR3_UINT8) "TINYTEXT_SYMBOL",
        (pANTLR3_UINT8) "TO_SYMBOL",
        (pANTLR3_UINT8) "TRAILING_SYMBOL",
        (pANTLR3_UINT8) "TRIGGER_SYMBOL",
        (pANTLR3_UINT8) "TRIM_SYMBOL",
        (pANTLR3_UINT8) "TRUE_SYMBOL",
        (pANTLR3_UINT8) "UNDERLINE_SYMBOL",
        (pANTLR3_UINT8) "UNDO_SYMBOL",
        (pANTLR3_UINT8) "UNION_SYMBOL",
        (pANTLR3_UINT8) "UNIQUE_SYMBOL",
        (pANTLR3_UINT8) "UNLOCK_SYMBOL",
        (pANTLR3_UINT8) "UNSIGNED_SYMBOL",
        (pANTLR3_UINT8) "UPDATE_SYMBOL",
        (pANTLR3_UINT8) "USAGE_SYMBOL",
        (pANTLR3_UINT8) "USE_SYMBOL",
        (pANTLR3_UINT8) "USING_SYMBOL",
        (pANTLR3_UINT8) "UTC_DATE_SYMBOL",
        (pANTLR3_UINT8) "UTC_TIMESTAMP_SYMBOL",
        (pANTLR3_UINT8) "UTC_TIME_SYMBOL",
        (pANTLR3_UINT8) "VALIDATION_SYMBOL",
        (pANTLR3_UINT8) "VALUES_SYMBOL",
        (pANTLR3_UINT8) "VARBINARY_SYMBOL",
        (pANTLR3_UINT8) "VARCHARACTER_SYMBOL",
        (pANTLR3_UINT8) "VARCHAR_SYMBOL",
        (pANTLR3_UINT8) "VARIANCE_SYMBOL",
        (pANTLR3_UINT8) "VARYING_SYMBOL",
        (pANTLR3_UINT8) "VAR_POP_SYMBOL",
        (pANTLR3_UINT8) "VAR_SAMP_SYMBOL",
        (pANTLR3_UINT8) "VIRTUAL_SYMBOL",
        (pANTLR3_UINT8) "WHEN_SYMBOL",
        (pANTLR3_UINT8) "WHERE_SYMBOL",
        (pANTLR3_UINT8) "WHILE_SYMBOL",
        (pANTLR3_UINT8) "WITHOUT_SYMBOL",
        (pANTLR3_UINT8) "WITH_CUBE_SYMBOL",
        (pANTLR3_UINT8) "WITH_ROLLUP_SYMBOL",
        (pANTLR3_UINT8) "WITH_SYMBOL",
        (pANTLR3_UINT8) "WRITE_SYMBOL",
        (pANTLR3_UINT8) "XOR_SYMBOL",
        (pANTLR3_UINT8) "YEAR_MONTH_SYMBOL",
        (pANTLR3_UINT8) "ZEROFILL_SYMBOL",
        (pANTLR3_UINT8) "ALTER_TABLE_ITEM_TOKEN",
        (pANTLR3_UINT8) "ASSIGN_OPERATOR",
        (pANTLR3_UINT8) "AS_SYMBOL",
        (pANTLR3_UINT8) "AT_AT_SIGN_SYMBOL",
        (pANTLR3_UINT8) "AT_SIGN_SYMBOL",
        (pANTLR3_UINT8) "AT_TEXT_SUFFIX",
        (pANTLR3_UINT8) "BACK_TICK",
        (pANTLR3_UINT8) "BACK_TICK_QUOTED_ID",
        (pANTLR3_UINT8) "BIN_NUMBER",
        (pANTLR3_UINT8) "BITWISE_AND_OPERATOR",
        (pANTLR3_UINT8) "BITWISE_NOT_OPERATOR",
        (pANTLR3_UINT8) "BITWISE_OR_OPERATOR",
        (pANTLR3_UINT8) "BITWISE_XOR_OPERATOR",
        (pANTLR3_UINT8) "BIT_AND_SYMBOL",
        (pANTLR3_UINT8) "BIT_OR_SYMBOL",
        (pANTLR3_UINT8) "BIT_XOR_SYMBOL",
        (pANTLR3_UINT8) "CHANGE_MASTER_OPTIONS_TOKEN",
        (pANTLR3_UINT8) "CLOSE_CURLY_SYMBOL",
        (pANTLR3_UINT8) "CLOSE_PAR_SYMBOL",
        (pANTLR3_UINT8) "COLON_SYMBOL",
        (pANTLR3_UINT8) "COLUMN_ASSIGNMENT_LIST_TOKEN",
        (pANTLR3_UINT8) "COLUMN_INTERNAL_REF_TOKEN",
        (pANTLR3_UINT8) "COLUMN_NAME_TOKEN",
        (pANTLR3_UINT8) "COLUMN_REF_TOKEN",
        (pANTLR3_UINT8) "COMMA_SYMBOL",
        (pANTLR3_UINT8) "CONCAT_PIPES_SYMBOL",
        (pANTLR3_UINT8) "CREATE_ITEM_TOKEN",
        (pANTLR3_UINT8) "CURRENT_DATE_SYMBOL",
        (pANTLR3_UINT8) "CURRENT_TIME_SYMBOL",
        (pANTLR3_UINT8) "DATA_TYPE_TOKEN",
        (pANTLR3_UINT8) "DECIMAL_NUMBER",
        (pANTLR3_UINT8) "DIGIT",
        (pANTLR3_UINT8) "DIGITS",
        (pANTLR3_UINT8) "DIV_OPERATOR",
        (pANTLR3_UINT8) "DIV_SYMBOL",
        (pANTLR3_UINT8) "DOT_IDENTIFIER",
        (pANTLR3_UINT8) "DOT_SYMBOL",
        (pANTLR3_UINT8) "DOUBLE_DASH",
        (pANTLR3_UINT8) "DOUBLE_QUOTE",
        (pANTLR3_UINT8) "DOUBLE_QUOTED_TEXT",
        (pANTLR3_UINT8) "DOUBLE_SYMBOL",
        (pANTLR3_UINT8) "ENGINE_REF_TOKEN",
        (pANTLR3_UINT8) "EQUAL_OPERATOR",
        (pANTLR3_UINT8) "ESCAPE_OPERATOR",
        (pANTLR3_UINT8) "EVENT_NAME_TOKEN",
        (pANTLR3_UINT8) "EVENT_REF_TOKEN",
        (pANTLR3_UINT8) "EXPRESSION_TOKEN",
        (pANTLR3_UINT8) "FLOAT_NUMBER",
        (pANTLR3_UINT8) "FUNCTION_CALL_TOKEN",
        (pANTLR3_UINT8) "FUNCTION_NAME_TOKEN",
        (pANTLR3_UINT8) "FUNCTION_REF_TOKEN",
        (pANTLR3_UINT8) "GREATER_OR_EQUAL_OPERATOR",
        (pANTLR3_UINT8) "GREATER_THAN_OPERATOR",
        (pANTLR3_UINT8) "HEXDIGIT",
        (pANTLR3_UINT8) "HEX_NUMBER",
        (pANTLR3_UINT8) "IDENTIFIER",
        (pANTLR3_UINT8) "INDEX_HINT_LIST_TOKEN",
        (pANTLR3_UINT8) "INDEX_NAME_TOKEN",
        (pANTLR3_UINT8) "INDEX_REF_TOKEN",
        (pANTLR3_UINT8) "INT_NUMBER",
        (pANTLR3_UINT8) "INVALID_INPUT",
        (pANTLR3_UINT8) "JOIN_EXPR_TOKEN",
        (pANTLR3_UINT8) "JSON_UNQUOTED_SEPARATOR_SYMBOL",
        (pANTLR3_UINT8) "KEY_CACHE_LIST_TOKEN",
        (pANTLR3_UINT8) "KEY_CACHE_PARTITION_TOKEN",
        (pANTLR3_UINT8) "LABEL_TOKEN",
        (pANTLR3_UINT8) "LESS_OR_EQUAL_OPERATOR",
        (pANTLR3_UINT8) "LESS_THAN_OPERATOR",
        (pANTLR3_UINT8) "LETTER_WHEN_UNQUOTED",
        (pANTLR3_UINT8) "LETTER_WHEN_UNQUOTED_NO_DIGIT",
        (pANTLR3_UINT8) "LINEBREAK",
        (pANTLR3_UINT8) "LOGFILE_GROUP_NAME_TOKEN",
        (pANTLR3_UINT8) "LOGFILE_GROUP_OPTIONS_TOKEN",
        (pANTLR3_UINT8) "LOGFILE_GROUP_REF_TOKEN",
        (pANTLR3_UINT8) "LOGICAL_AND_OPERATOR",
        (pANTLR3_UINT8) "LOGICAL_NOT_OPERATOR",
        (pANTLR3_UINT8) "LOGICAL_OR_OPERATOR",
        (pANTLR3_UINT8) "LONG_NUMBER",
        (pANTLR3_UINT8) "MINUS_OPERATOR",
        (pANTLR3_UINT8) "MOD_OPERATOR",
        (pANTLR3_UINT8) "MULTILINE_COMMENT",
        (pANTLR3_UINT8) "MULT_OPERATOR",
        (pANTLR3_UINT8) "NOT_EQUAL2_OPERATOR",
        (pANTLR3_UINT8) "NOT_EQUAL_OPERATOR",
        (pANTLR3_UINT8) "NULL_SAFE_EQUAL_OPERATOR",
        (pANTLR3_UINT8) "NUMBER",
        (pANTLR3_UINT8) "PAR_EXPRESSION_TOKEN",
        (pANTLR3_UINT8) "PLUS_OPERATOR",
        (pANTLR3_UINT8) "PRIVILEGE_TARGET_TOKEN",
        (pANTLR3_UINT8) "PROCEDURE_NAME_TOKEN",
        (pANTLR3_UINT8) "PROCEDURE_REF_TOKEN",
        (pANTLR3_UINT8) "ROUTINE_ALTER_OPTIONS",
        (pANTLR3_UINT8) "ROUTINE_CREATE_OPTIONS",
        (pANTLR3_UINT8) "RUNTIME_FUNCTION_TOKEN",
        (pANTLR3_UINT8) "SCHEMA_NAME_TOKEN",
        (pANTLR3_UINT8) "SCHEMA_REF_TOKEN",
        (pANTLR3_UINT8) "SELECT_EXPR_TOKEN",
        (pANTLR3_UINT8) "SERVER_NAME_TOKEN",
        (pANTLR3_UINT8) "SERVER_REF_TOKEN",
        (pANTLR3_UINT8) "SHIFT_LEFT_OPERATOR",
        (pANTLR3_UINT8) "SHIFT_RIGHT_OPERATOR",
        (pANTLR3_UINT8) "SIMPLE_IDENTIFIER",
        (pANTLR3_UINT8) "SINGLE_QUOTE",
        (pANTLR3_UINT8) "SLAVE_THREAD_OPTIONS_TOKEN",
        (pANTLR3_UINT8) "SOME_SYMBOL",
        (pANTLR3_UINT8) "STRING_NO_LINEBREAK_TOKEN",
        (pANTLR3_UINT8) "STRING_TOKEN",
        (pANTLR3_UINT8) "SUBQUERY_TOKEN",
        (pANTLR3_UINT8) "TABLESPACE_NAME_TOKEN",
        (pANTLR3_UINT8) "TABLESPACE_OPTIONS_TOKEN",
        (pANTLR3_UINT8) "TABLESPACE_REF_TOKEN",
        (pANTLR3_UINT8) "TABLE_NAME_TOKEN",
        (pANTLR3_UINT8) "TABLE_REF_TOKEN",
        (pANTLR3_UINT8) "TRIGGER_NAME_TOKEN",
        (pANTLR3_UINT8) "TRIGGER_REF_TOKEN",
        (pANTLR3_UINT8) "UDF_CALL_TOKEN",
        (pANTLR3_UINT8) "UDF_NAME_TOKEN",
        (pANTLR3_UINT8) "ULONGLONG_NUMBER",
        (pANTLR3_UINT8) "UNDERSCORE_CHARSET",
        (pANTLR3_UINT8) "VERSION_COMMENT",
        (pANTLR3_UINT8) "VERSION_COMMENT_END",
        (pANTLR3_UINT8) "VERSION_COMMENT_INTRODUCER",
        (pANTLR3_UINT8) "VERSION_COMMENT_START_TOKEN",
        (pANTLR3_UINT8) "VERSION_COMMENT_TAIL",
        (pANTLR3_UINT8) "VIEW_NAME_TOKEN",
        (pANTLR3_UINT8) "VIEW_REF_TOKEN",
        (pANTLR3_UINT8) "WHITESPACE",
        (pANTLR3_UINT8) "XA_ID_TOKEN"
       };



// Forward declare the locally static matching functions we have generated.
//
static 
 void
	query    (pMySQLSimpleParser ctx);
static 
 void
	statement    (pMySQLSimpleParser ctx);
static 
 void
	alter_statement    (pMySQLSimpleParser ctx);
static 
 void
	alter_database    (pMySQLSimpleParser ctx);
static 
 void
	alter_event    (pMySQLSimpleParser ctx);
static 
 void
	alter_log_file_group    (pMySQLSimpleParser ctx);
static 
 void
	alter_server    (pMySQLSimpleParser ctx);
static 
 void
	alter_table    (pMySQLSimpleParser ctx);
static 
 void
	alter_table_commands    (pMySQLSimpleParser ctx);
static 
 void
	alter_table_list_item    (pMySQLSimpleParser ctx);
static 
 void
	alter_table_list_entry    (pMySQLSimpleParser ctx);
static 
 void
	key_def    (pMySQLSimpleParser ctx);
static 
 void
	alter_order_by    (pMySQLSimpleParser ctx);
static 
 void
	alter_algorithm_option    (pMySQLSimpleParser ctx);
static 
 void
	alter_lock_option    (pMySQLSimpleParser ctx);
static 
 void
	index_lock_algorithm    (pMySQLSimpleParser ctx);
static 
 void
	alter_partition    (pMySQLSimpleParser ctx);
static 
 void
	validation    (pMySQLSimpleParser ctx);
static 
 void
	remove_partitioning    (pMySQLSimpleParser ctx);
static 
 void
	all_or_partition_name_list    (pMySQLSimpleParser ctx);
static 
 void
	alter_tablespace    (pMySQLSimpleParser ctx);
static 
 void
	alter_tablespace_option    (pMySQLSimpleParser ctx);
static 
 void
	change_tablespace_option    (pMySQLSimpleParser ctx);
static 
 void
	alter_view    (pMySQLSimpleParser ctx);
static 
 void
	create_statement    (pMySQLSimpleParser ctx);
static 
 void
	create_database    (pMySQLSimpleParser ctx);
static 
 void
	create_database_tail    (pMySQLSimpleParser ctx);
static 
 void
	create_with_definer    (pMySQLSimpleParser ctx);
static 
 void
	create_event    (pMySQLSimpleParser ctx);
static 
 void
	create_event_tail    (pMySQLSimpleParser ctx);
static 
 void
	create_routine    (pMySQLSimpleParser ctx);
static 
 void
	create_procedure    (pMySQLSimpleParser ctx);
static 
 void
	create_function    (pMySQLSimpleParser ctx);
static 
 void
	create_routine_or_udf    (pMySQLSimpleParser ctx);
static 
 void
	procedure_body    (pMySQLSimpleParser ctx);
static 
 void
	function_body    (pMySQLSimpleParser ctx);
static 
 void
	udf_tail    (pMySQLSimpleParser ctx);
static 
 void
	routine_create_options    (pMySQLSimpleParser ctx);
static 
 void
	routine_create_option    (pMySQLSimpleParser ctx);
static 
 void
	routine_alter_options    (pMySQLSimpleParser ctx);
static 
 void
	routine_option    (pMySQLSimpleParser ctx);
static 
 void
	create_index    (pMySQLSimpleParser ctx);
static 
 void
	create_index_tail    (pMySQLSimpleParser ctx);
static 
 void
	create_index_target    (pMySQLSimpleParser ctx);
static 
 void
	create_logfile_group    (pMySQLSimpleParser ctx);
static 
 void
	create_logfile_group_tail    (pMySQLSimpleParser ctx);
static 
 void
	logfile_group_options    (pMySQLSimpleParser ctx);
static 
 void
	logfile_group_option    (pMySQLSimpleParser ctx);
static 
 void
	create_server    (pMySQLSimpleParser ctx);
static 
 void
	create_server_tail    (pMySQLSimpleParser ctx);
static 
 void
	server_options    (pMySQLSimpleParser ctx);
static 
 void
	server_option    (pMySQLSimpleParser ctx);
static 
 void
	create_table    (pMySQLSimpleParser ctx);
static 
 void
	create_table_tail    (pMySQLSimpleParser ctx);
static 
 void
	create_field_list    (pMySQLSimpleParser ctx);
static 
 void
	create_item    (pMySQLSimpleParser ctx);
static 
 void
	create_field_list_item    (pMySQLSimpleParser ctx);
static 
 void
	table_creation_source    (pMySQLSimpleParser ctx);
static 
 void
	create_select    (pMySQLSimpleParser ctx);
static 
 void
	create_tablespace    (pMySQLSimpleParser ctx);
static 
 void
	create_tablespace_tail    (pMySQLSimpleParser ctx);
static 
 void
	tablespace_options    (pMySQLSimpleParser ctx);
static 
 void
	tablespace_option    (pMySQLSimpleParser ctx);
static 
 void
	create_trigger    (pMySQLSimpleParser ctx);
static 
 void
	create_trigger_tail    (pMySQLSimpleParser ctx);
static 
 void
	trigger_follows_precedes_clause    (pMySQLSimpleParser ctx);
static 
 void
	create_view    (pMySQLSimpleParser ctx);
static 
 void
	create_view_tail    (pMySQLSimpleParser ctx);
static 
 void
	view_replace_or_algorithm    (pMySQLSimpleParser ctx);
static 
 void
	view_algorithm    (pMySQLSimpleParser ctx);
static 
 void
	drop_statement    (pMySQLSimpleParser ctx);
static 
 void
	drop_logfile_group_option    (pMySQLSimpleParser ctx);
static 
 void
	rename_table_statement    (pMySQLSimpleParser ctx);
static 
 void
	truncate_table_statement    (pMySQLSimpleParser ctx);
static 
 void
	call_statement    (pMySQLSimpleParser ctx);
static 
 void
	delete_statement    (pMySQLSimpleParser ctx);
static 
 void
	partition_delete    (pMySQLSimpleParser ctx);
static 
 void
	delete_option    (pMySQLSimpleParser ctx);
static 
 void
	do_statement    (pMySQLSimpleParser ctx);
static 
 void
	handler_statement    (pMySQLSimpleParser ctx);
static 
 void
	handler_read_or_scan    (pMySQLSimpleParser ctx);
static 
 void
	insert_statement    (pMySQLSimpleParser ctx);
static 
 void
	insert_lock_option    (pMySQLSimpleParser ctx);
static 
 void
	insert_field_spec    (pMySQLSimpleParser ctx);
static 
 void
	fields    (pMySQLSimpleParser ctx);
static 
 void
	insert_values    (pMySQLSimpleParser ctx);
static 
 void
	insert_query_expression    (pMySQLSimpleParser ctx);
static 
 void
	insert_value_list    (pMySQLSimpleParser ctx);
static 
 void
	values    (pMySQLSimpleParser ctx);
static 
 void
	duplicate_key_update    (pMySQLSimpleParser ctx);
static 
 void
	load_statement    (pMySQLSimpleParser ctx);
static 
 void
	data_or_xml    (pMySQLSimpleParser ctx);
static 
 void
	xml_rows_identified_by    (pMySQLSimpleParser ctx);
static 
 void
	load_data_file_tail    (pMySQLSimpleParser ctx);
static 
 void
	load_data_file_target_list    (pMySQLSimpleParser ctx);
static 
 void
	field_or_variable_list    (pMySQLSimpleParser ctx);
static 
 void
	replace_statement    (pMySQLSimpleParser ctx);
static 
 void
	select_statement    (pMySQLSimpleParser ctx);
static 
 void
	select_paren    (pMySQLSimpleParser ctx);
static 
 void
	select_from    (pMySQLSimpleParser ctx);
static 
 void
	select_part2    (pMySQLSimpleParser ctx);
static 
 void
	table_expression    (pMySQLSimpleParser ctx);
static 
 void
	subquery    (pMySQLSimpleParser ctx);
static 
 void
	query_expression_body    (pMySQLSimpleParser ctx);
static 
 void
	select_part2_derived    (pMySQLSimpleParser ctx);
static 
 void
	select_option    (pMySQLSimpleParser ctx);
static 
 void
	query_spec_option    (pMySQLSimpleParser ctx);
static 
 void
	select_item_list    (pMySQLSimpleParser ctx);
static 
 void
	select_item    (pMySQLSimpleParser ctx);
static 
 void
	select_alias    (pMySQLSimpleParser ctx);
static 
 void
	limit_clause    (pMySQLSimpleParser ctx);
static 
 void
	simple_limit_clause    (pMySQLSimpleParser ctx);
static 
 void
	limit_options    (pMySQLSimpleParser ctx);
static 
 void
	limit_option    (pMySQLSimpleParser ctx);
static 
 void
	into_clause    (pMySQLSimpleParser ctx);
static 
 void
	procedure_analyse_clause    (pMySQLSimpleParser ctx);
static 
 void
	having_clause    (pMySQLSimpleParser ctx);
static 
 void
	group_by_clause    (pMySQLSimpleParser ctx);
static 
 void
	olap_option    (pMySQLSimpleParser ctx);
static 
 void
	order_by_clause    (pMySQLSimpleParser ctx);
static 
 void
	order_by_or_limit    (pMySQLSimpleParser ctx);
static 
 void
	direction    (pMySQLSimpleParser ctx);
static 
 void
	from_clause    (pMySQLSimpleParser ctx);
static 
 void
	where_clause    (pMySQLSimpleParser ctx);
static 
 void
	table_reference_list    (pMySQLSimpleParser ctx);
static 
 void
	join_table_list    (pMySQLSimpleParser ctx);
static 
 void
	escaped_table_reference    (pMySQLSimpleParser ctx);
static 
 void
	table_reference    (pMySQLSimpleParser ctx);
static 
 void
	join    (pMySQLSimpleParser ctx);
static 
 void
	table_factor    (pMySQLSimpleParser ctx);
static 
 void
	select_table_factor_union    (pMySQLSimpleParser ctx);
static 
 void
	query_specification    (pMySQLSimpleParser ctx);
static 
 void
	select_paren_derived    (pMySQLSimpleParser ctx);
static 
 void
	join_table    (pMySQLSimpleParser ctx);
static 
 void
	join_condition    (pMySQLSimpleParser ctx);
static 
 void
	union_clause    (pMySQLSimpleParser ctx);
static 
 void
	union_option    (pMySQLSimpleParser ctx);
static 
 void
	union_or_order_by_or_limit    (pMySQLSimpleParser ctx);
static 
 void
	select_lock_type    (pMySQLSimpleParser ctx);
static 
 void
	table_alias    (pMySQLSimpleParser ctx);
static 
 void
	index_hint_list    (pMySQLSimpleParser ctx);
static 
 void
	index_hint    (pMySQLSimpleParser ctx);
static 
 void
	index_hint_type    (pMySQLSimpleParser ctx);
static 
 void
	key_or_index    (pMySQLSimpleParser ctx);
static 
 void
	index_hint_clause    (pMySQLSimpleParser ctx);
static 
 void
	index_list    (pMySQLSimpleParser ctx);
static 
 void
	index_list_element    (pMySQLSimpleParser ctx);
static 
 void
	update_statement    (pMySQLSimpleParser ctx);
static 
 void
	transaction_or_locking_statement    (pMySQLSimpleParser ctx);
static 
 void
	transaction_statement    (pMySQLSimpleParser ctx);
static 
 void
	begin_work    (pMySQLSimpleParser ctx);
static 
 void
	transaction_characteristic    (pMySQLSimpleParser ctx);
static 
 void
	set_transaction_characteristic    (pMySQLSimpleParser ctx);
static 
 void
	isolation_level    (pMySQLSimpleParser ctx);
static 
 void
	savepoint_statement    (pMySQLSimpleParser ctx);
static 
 void
	lock_statement    (pMySQLSimpleParser ctx);
static 
 void
	lock_item    (pMySQLSimpleParser ctx);
static 
 void
	lock_option    (pMySQLSimpleParser ctx);
static 
 void
	xa_statement    (pMySQLSimpleParser ctx);
static 
 void
	xa_convert    (pMySQLSimpleParser ctx);
static 
 void
	xid    (pMySQLSimpleParser ctx);
static 
 void
	replication_statement    (pMySQLSimpleParser ctx);
static 
 void
	replication_load    (pMySQLSimpleParser ctx);
static 
 void
	change_master    (pMySQLSimpleParser ctx);
static 
 void
	change_master_options    (pMySQLSimpleParser ctx);
static 
 void
	master_option    (pMySQLSimpleParser ctx);
static 
 void
	master_file_def    (pMySQLSimpleParser ctx);
static 
 void
	server_id_list    (pMySQLSimpleParser ctx);
static 
 void
	change_replication    (pMySQLSimpleParser ctx);
static 
 void
	filter_definition    (pMySQLSimpleParser ctx);
static 
 void
	filter_db_list    (pMySQLSimpleParser ctx);
static 
 void
	filter_table_list    (pMySQLSimpleParser ctx);
static 
 void
	filter_string_list    (pMySQLSimpleParser ctx);
static 
 void
	filter_wild_db_table_string    (pMySQLSimpleParser ctx);
static 
 void
	filter_db_pair_list    (pMySQLSimpleParser ctx);
static 
 void
	slave    (pMySQLSimpleParser ctx);
static 
 void
	slave_until_options    (pMySQLSimpleParser ctx);
static 
 void
	slave_connection_options    (pMySQLSimpleParser ctx);
static 
 void
	slave_thread_options    (pMySQLSimpleParser ctx);
static 
 void
	slave_thread_option    (pMySQLSimpleParser ctx);
static 
 void
	group_replication    (pMySQLSimpleParser ctx);
static 
 void
	prepared_statement    (pMySQLSimpleParser ctx);
static 
 void
	execute_statement    (pMySQLSimpleParser ctx);
static 
 void
	execute_var_list    (pMySQLSimpleParser ctx);
static 
 void
	account_management_statement    (pMySQLSimpleParser ctx);
static 
 void
	alter_user    (pMySQLSimpleParser ctx);
static 
 void
	alter_user_tail    (pMySQLSimpleParser ctx);
static 
 void
	create_user    (pMySQLSimpleParser ctx);
static 
 void
	create_user_tail    (pMySQLSimpleParser ctx);
static 
 void
	require_clause    (pMySQLSimpleParser ctx);
static 
 void
	connect_options    (pMySQLSimpleParser ctx);
static 
 void
	account_lock_password_expire_options    (pMySQLSimpleParser ctx);
static 
 void
	drop_user    (pMySQLSimpleParser ctx);
static 
 void
	parse_grant    (pMySQLSimpleParser ctx);
static 
 void
	grant    (pMySQLSimpleParser ctx);
static 
 void
	grant_proxy    (pMySQLSimpleParser ctx);
static 
 void
	rename_user    (pMySQLSimpleParser ctx);
static 
 void
	revoke_statement    (pMySQLSimpleParser ctx);
static 
 void
	privilege_target    (pMySQLSimpleParser ctx);
static 
 void
	set_password    (pMySQLSimpleParser ctx);
static 
 void
	grant_object_type    (pMySQLSimpleParser ctx);
static 
 void
	grant_privileges    (pMySQLSimpleParser ctx);
static 
 void
	privilege_type    (pMySQLSimpleParser ctx);
static 
 void
	privilege_level    (pMySQLSimpleParser ctx);
static 
 void
	require_list    (pMySQLSimpleParser ctx);
static 
 void
	require_list_element    (pMySQLSimpleParser ctx);
static 
 void
	grant_option    (pMySQLSimpleParser ctx);
static 
 void
	table_administration_statement    (pMySQLSimpleParser ctx);
static 
 void
	check_option    (pMySQLSimpleParser ctx);
static 
 void
	repair_option    (pMySQLSimpleParser ctx);
static 
 void
	install_uninstall_statment    (pMySQLSimpleParser ctx);
static 
 void
	set_statement    (pMySQLSimpleParser ctx);
static 
 void
	option_value_no_option_type    (pMySQLSimpleParser ctx);
static 
 void
	option_value_following_option_type    (pMySQLSimpleParser ctx);
static 
 void
	set_expression_or_default    (pMySQLSimpleParser ctx);
static 
 void
	option_value_list    (pMySQLSimpleParser ctx);
static 
 void
	option_value    (pMySQLSimpleParser ctx);
static 
 void
	show_statement    (pMySQLSimpleParser ctx);
static 
 void
	non_blocking    (pMySQLSimpleParser ctx);
static 
 void
	from_or_in    (pMySQLSimpleParser ctx);
static 
 void
	in_db    (pMySQLSimpleParser ctx);
static 
 void
	profile_type    (pMySQLSimpleParser ctx);
static 
 void
	other_administrative_statement    (pMySQLSimpleParser ctx);
static 
 void
	key_cache_list_or_parts    (pMySQLSimpleParser ctx);
static 
 void
	key_cache_list    (pMySQLSimpleParser ctx);
static 
 void
	assign_to_keycache    (pMySQLSimpleParser ctx);
static 
 void
	assign_to_keycache_partition    (pMySQLSimpleParser ctx);
static 
 void
	cache_keys_spec    (pMySQLSimpleParser ctx);
static 
 void
	key_usage_element    (pMySQLSimpleParser ctx);
static 
 void
	flush_option    (pMySQLSimpleParser ctx);
static 
 void
	log_type    (pMySQLSimpleParser ctx);
static 
 void
	flush_tables    (pMySQLSimpleParser ctx);
static 
 void
	flush_tables_options    (pMySQLSimpleParser ctx);
static 
 void
	load_table_index_list    (pMySQLSimpleParser ctx);
static 
 void
	load_table_index_partion    (pMySQLSimpleParser ctx);
static 
 void
	reset_option    (pMySQLSimpleParser ctx);
static 
 void
	utility_statement    (pMySQLSimpleParser ctx);
static 
 void
	describe_command    (pMySQLSimpleParser ctx);
static 
 void
	explainable_statement    (pMySQLSimpleParser ctx);
static 
 void
	use_command    (pMySQLSimpleParser ctx);
static 
 void
	expression    (pMySQLSimpleParser ctx);
static 
 void
	logical_or_expression    (pMySQLSimpleParser ctx);
static 
 void
	logical_xor_expression    (pMySQLSimpleParser ctx);
static 
 void
	logical_and_expression    (pMySQLSimpleParser ctx);
static 
 void
	logical_not_expression    (pMySQLSimpleParser ctx);
static 
 void
	boolean_primary_expression    (pMySQLSimpleParser ctx);
static 
 void
	predicate    (pMySQLSimpleParser ctx);
static 
 void
	predicate_in    (pMySQLSimpleParser ctx);
static 
 void
	bitwise_or_expression    (pMySQLSimpleParser ctx);
static 
 void
	bitwise_and_expression    (pMySQLSimpleParser ctx);
static 
 void
	shift_expression    (pMySQLSimpleParser ctx);
static 
 void
	additive_expression    (pMySQLSimpleParser ctx);
static 
 void
	multiplicative_expression    (pMySQLSimpleParser ctx);
static 
 void
	bitwise_xor_expression    (pMySQLSimpleParser ctx);
static 
 void
	concat_expression    (pMySQLSimpleParser ctx);
static 
 void
	unary_expression    (pMySQLSimpleParser ctx);
static 
 void
	not_expression    (pMySQLSimpleParser ctx);
static 
 void
	interval_expression    (pMySQLSimpleParser ctx);
static 
 void
	interval_function    (pMySQLSimpleParser ctx);
static 
 void
	interval_time_span    (pMySQLSimpleParser ctx);
static 
 void
	primary    (pMySQLSimpleParser ctx);
static 
 void
	jsonOperator    (pMySQLSimpleParser ctx);
static 
 void
	expression_with_nested_parentheses    (pMySQLSimpleParser ctx);
static 
 void
	comparison_operator    (pMySQLSimpleParser ctx);
static 
 void
	multiplication_operator    (pMySQLSimpleParser ctx);
static 
 void
	runtime_function_call    (pMySQLSimpleParser ctx);
static 
 void
	runtime_function_call_expression    (pMySQLSimpleParser ctx);
static 
 void
	time_function_parameters    (pMySQLSimpleParser ctx);
static 
 void
	fractional_precision    (pMySQLSimpleParser ctx);
static 
 void
	weight_string_levels    (pMySQLSimpleParser ctx);
static 
 void
	weight_string_level_list_item    (pMySQLSimpleParser ctx);
static 
 void
	date_time_type    (pMySQLSimpleParser ctx);
static 
 void
	trim_function    (pMySQLSimpleParser ctx);
static 
 void
	substring_function    (pMySQLSimpleParser ctx);
static 
 void
	geometry_function    (pMySQLSimpleParser ctx);
static 
 void
	aggregate_function    (pMySQLSimpleParser ctx);
static 
 void
	in_aggregate_expression    (pMySQLSimpleParser ctx);
static 
 void
	count_function    (pMySQLSimpleParser ctx);
static 
 void
	function_call    (pMySQLSimpleParser ctx);
static 
 void
	function_call_expression    (pMySQLSimpleParser ctx);
static 
 void
	aliased_expression_list    (pMySQLSimpleParser ctx);
static 
 void
	aliased_expression    (pMySQLSimpleParser ctx);
static 
 void
	variable    (pMySQLSimpleParser ctx);
static 
 void
	user_variable    (pMySQLSimpleParser ctx);
static 
 void
	system_variable    (pMySQLSimpleParser ctx);
static 
 void
	variable_name    (pMySQLSimpleParser ctx);
static 
 void
	match_expression    (pMySQLSimpleParser ctx);
static 
 void
	case_expression    (pMySQLSimpleParser ctx);
static 
 void
	when_expression    (pMySQLSimpleParser ctx);
static 
 void
	then_expression    (pMySQLSimpleParser ctx);
static 
 void
	else_expression    (pMySQLSimpleParser ctx);
static 
 void
	cast_expression    (pMySQLSimpleParser ctx);
static 
 void
	cast_type    (pMySQLSimpleParser ctx);
static 
 void
	encoding    (pMySQLSimpleParser ctx);
static 
 void
	charset    (pMySQLSimpleParser ctx);
static 
 void
	not_rule    (pMySQLSimpleParser ctx);
static 
 void
	not2_rule    (pMySQLSimpleParser ctx);
static 
 void
	or_rule    (pMySQLSimpleParser ctx);
static 
 void
	xor_rule    (pMySQLSimpleParser ctx);
static 
 void
	and_rule    (pMySQLSimpleParser ctx);
static 
 void
	interval    (pMySQLSimpleParser ctx);
static 
 void
	interval_unit    (pMySQLSimpleParser ctx);
static 
 void
	interval_timestamp_unit    (pMySQLSimpleParser ctx);
static 
 void
	expression_list_with_parentheses    (pMySQLSimpleParser ctx);
static 
 void
	optional_expression_list_with_parentheses    (pMySQLSimpleParser ctx);
static 
 void
	expression_list    (pMySQLSimpleParser ctx);
static 
 void
	expression_list_with_direction    (pMySQLSimpleParser ctx);
static 
 void
	channel    (pMySQLSimpleParser ctx);
static 
 void
	compound_statement    (pMySQLSimpleParser ctx);
static 
 void
	return_statement    (pMySQLSimpleParser ctx);
static 
 void
	if_statement    (pMySQLSimpleParser ctx);
static 
 void
	if_body    (pMySQLSimpleParser ctx);
static 
 void
	then_statement    (pMySQLSimpleParser ctx);
static 
 void
	compound_statement_list    (pMySQLSimpleParser ctx);
static 
 void
	case_statement    (pMySQLSimpleParser ctx);
static 
 void
	else_statement    (pMySQLSimpleParser ctx);
static 
 void
	labeled_block    (pMySQLSimpleParser ctx);
static 
 void
	unlabeled_block    (pMySQLSimpleParser ctx);
static 
 void
	label    (pMySQLSimpleParser ctx);
static 
 void
	label_identifier    (pMySQLSimpleParser ctx);
static 
 void
	begin_end_block    (pMySQLSimpleParser ctx);
static 
 void
	labeled_control    (pMySQLSimpleParser ctx);
static 
 void
	unlabeled_control    (pMySQLSimpleParser ctx);
static 
 void
	loop_block    (pMySQLSimpleParser ctx);
static 
 void
	while_do_block    (pMySQLSimpleParser ctx);
static 
 void
	repeat_until_block    (pMySQLSimpleParser ctx);
static 
 void
	sp_declarations    (pMySQLSimpleParser ctx);
static 
 void
	sp_declaration    (pMySQLSimpleParser ctx);
static 
 void
	variable_declaration    (pMySQLSimpleParser ctx);
static 
 void
	condition_declaration    (pMySQLSimpleParser ctx);
static 
 void
	sp_condition    (pMySQLSimpleParser ctx);
static 
 void
	sqlstate    (pMySQLSimpleParser ctx);
static 
 void
	handler_declaration    (pMySQLSimpleParser ctx);
static 
 void
	handler_condition    (pMySQLSimpleParser ctx);
static 
 void
	cursor_declaration    (pMySQLSimpleParser ctx);
static 
 void
	iterate_statement    (pMySQLSimpleParser ctx);
static 
 void
	leave_statement    (pMySQLSimpleParser ctx);
static 
 void
	get_diagnostics    (pMySQLSimpleParser ctx);
static 
 void
	signal_allowed_expression    (pMySQLSimpleParser ctx);
static 
 void
	statement_information_item    (pMySQLSimpleParser ctx);
static 
 void
	condition_information_item    (pMySQLSimpleParser ctx);
static 
 void
	signal_information_item_name    (pMySQLSimpleParser ctx);
static 
 void
	signal_statement    (pMySQLSimpleParser ctx);
static 
 void
	resignal_statement    (pMySQLSimpleParser ctx);
static 
 void
	signal_information_item    (pMySQLSimpleParser ctx);
static 
 void
	cursor_open    (pMySQLSimpleParser ctx);
static 
 void
	cursor_close    (pMySQLSimpleParser ctx);
static 
 void
	cursor_fetch    (pMySQLSimpleParser ctx);
static 
 void
	schedule    (pMySQLSimpleParser ctx);
static 
 void
	database_option    (pMySQLSimpleParser ctx);
static 
 void
	column_definition    (pMySQLSimpleParser ctx);
static 
 void
	field_spec    (pMySQLSimpleParser ctx);
static 
 void
	field_def    (pMySQLSimpleParser ctx);
static 
 void
	field_def_tail    (pMySQLSimpleParser ctx);
static 
 void
	attribute    (pMySQLSimpleParser ctx);
static 
 void
	gcol_attribute    (pMySQLSimpleParser ctx);
static 
 void
	reference_definition    (pMySQLSimpleParser ctx);
static 
 void
	index_columns    (pMySQLSimpleParser ctx);
static 
 void
	index_column    (pMySQLSimpleParser ctx);
static 
 void
	index_type    (pMySQLSimpleParser ctx);
static 
 void
	normal_index_option    (pMySQLSimpleParser ctx);
static 
 void
	fulltext_index_option    (pMySQLSimpleParser ctx);
static 
 void
	spatial_index_option    (pMySQLSimpleParser ctx);
static 
 void
	all_key_option    (pMySQLSimpleParser ctx);
static 
 void
	reference_option    (pMySQLSimpleParser ctx);
static 
 void
	data_type    (pMySQLSimpleParser ctx);
static 
 void
	data_type_definition    (pMySQLSimpleParser ctx);
static 
 void
	data_type_elements    (pMySQLSimpleParser ctx);
static 
 void
	field_length    (pMySQLSimpleParser ctx);
static 
 void
	field_options    (pMySQLSimpleParser ctx);
static 
 void
	string_binary    (pMySQLSimpleParser ctx);
static 
 void
	ascii    (pMySQLSimpleParser ctx);
static 
 void
	unicode    (pMySQLSimpleParser ctx);
static 
 void
	type_datetime_precision    (pMySQLSimpleParser ctx);
static 
 void
	charset_name    (pMySQLSimpleParser ctx);
static 
 void
	charset_name_or_default    (pMySQLSimpleParser ctx);
static 
 void
	collation_name    (pMySQLSimpleParser ctx);
static 
 void
	collation_name_or_default    (pMySQLSimpleParser ctx);
static 
 void
	spatial_type    (pMySQLSimpleParser ctx);
static 
 void
	alter_table_options    (pMySQLSimpleParser ctx);
static 
 void
	create_table_options    (pMySQLSimpleParser ctx);
static 
 void
	create_table_option    (pMySQLSimpleParser ctx);
static 
 void
	partitioning    (pMySQLSimpleParser ctx);
static 
 void
	partition_key_algorithm    (pMySQLSimpleParser ctx);
static 
 void
	partition_definitions    (pMySQLSimpleParser ctx);
static 
 void
	partition_definition    (pMySQLSimpleParser ctx);
static 
 void
	partition_option    (pMySQLSimpleParser ctx);
static 
 void
	subpartition_definition    (pMySQLSimpleParser ctx);
static 
 void
	partition_value_list    (pMySQLSimpleParser ctx);
static 
 void
	definer_clause    (pMySQLSimpleParser ctx);
static 
 void
	if_exists    (pMySQLSimpleParser ctx);
static 
 void
	if_not_exists    (pMySQLSimpleParser ctx);
static 
 void
	procedure_parameter    (pMySQLSimpleParser ctx);
static 
 void
	function_parameter    (pMySQLSimpleParser ctx);
static 
 void
	schema_identifier_pair    (pMySQLSimpleParser ctx);
static 
 void
	view_identifier_list    (pMySQLSimpleParser ctx);
static 
 void
	field_name_list    (pMySQLSimpleParser ctx);
static 
 void
	column_assignment_list_with_default    (pMySQLSimpleParser ctx);
static 
 void
	column_assignment_with_default    (pMySQLSimpleParser ctx);
static 
 void
	charset_clause    (pMySQLSimpleParser ctx);
static 
 void
	fields_clause    (pMySQLSimpleParser ctx);
static 
 void
	field_term    (pMySQLSimpleParser ctx);
static 
 void
	lines_clause    (pMySQLSimpleParser ctx);
static 
 void
	line_term    (pMySQLSimpleParser ctx);
static 
 void
	user_list    (pMySQLSimpleParser ctx);
static 
 void
	grant_list    (pMySQLSimpleParser ctx);
static 
 void
	grant_user    (pMySQLSimpleParser ctx);
static 
 void
	user    (pMySQLSimpleParser ctx);
static 
 void
	like_clause    (pMySQLSimpleParser ctx);
static 
 void
	like_or_where    (pMySQLSimpleParser ctx);
static 
 void
	online_option    (pMySQLSimpleParser ctx);
static 
 void
	no_write_to_bin_log    (pMySQLSimpleParser ctx);
static 
 void
	use_partition    (pMySQLSimpleParser ctx);
static 
 void
	column_name    (pMySQLSimpleParser ctx);
static 
 void
	column_ref    (pMySQLSimpleParser ctx);
static 
 void
	column_ref_variants    (pMySQLSimpleParser ctx);
static 
 void
	column_internal_ref    (pMySQLSimpleParser ctx);
static 
 void
	column_ref_with_wildcard    (pMySQLSimpleParser ctx);
static 
 void
	column_ref_with_wildcard2    (pMySQLSimpleParser ctx);
static 
 void
	index_name    (pMySQLSimpleParser ctx);
static 
 void
	index_ref    (pMySQLSimpleParser ctx);
static 
 void
	table_wild    (pMySQLSimpleParser ctx);
static 
 void
	schema_name    (pMySQLSimpleParser ctx);
static 
 void
	schema_ref    (pMySQLSimpleParser ctx);
static 
 void
	procedure_name    (pMySQLSimpleParser ctx);
static 
 void
	procedure_ref    (pMySQLSimpleParser ctx);
static 
 void
	function_name    (pMySQLSimpleParser ctx);
static 
 void
	function_ref    (pMySQLSimpleParser ctx);
static 
 void
	trigger_name    (pMySQLSimpleParser ctx);
static 
 void
	trigger_ref    (pMySQLSimpleParser ctx);
static 
 void
	view_name    (pMySQLSimpleParser ctx);
static 
 void
	view_ref    (pMySQLSimpleParser ctx);
static 
 void
	tablespace_name    (pMySQLSimpleParser ctx);
static 
 void
	tablespace_ref    (pMySQLSimpleParser ctx);
static 
 void
	logfile_group_name    (pMySQLSimpleParser ctx);
static 
 void
	logfile_group_ref    (pMySQLSimpleParser ctx);
static 
 void
	event_name    (pMySQLSimpleParser ctx);
static 
 void
	event_ref    (pMySQLSimpleParser ctx);
static 
 void
	udf_name    (pMySQLSimpleParser ctx);
static 
 void
	server_name    (pMySQLSimpleParser ctx);
static 
 void
	server_ref    (pMySQLSimpleParser ctx);
static 
 void
	engine_ref    (pMySQLSimpleParser ctx);
static 
 void
	table_name    (pMySQLSimpleParser ctx);
static 
 void
	filter_table_ref    (pMySQLSimpleParser ctx);
static 
 void
	table_ref_with_wildcard    (pMySQLSimpleParser ctx);
static 
 void
	table_ref    (pMySQLSimpleParser ctx);
static 
 void
	table_ref_no_db    (pMySQLSimpleParser ctx);
static 
 void
	table_name_variants    (pMySQLSimpleParser ctx);
static 
 void
	table_ref_list    (pMySQLSimpleParser ctx);
static 
 void
	table_ref_list_with_wildcard    (pMySQLSimpleParser ctx);
static 
 void
	pure_identifier    (pMySQLSimpleParser ctx);
static 
 void
	identifier    (pMySQLSimpleParser ctx);
static 
 void
	identifier_list    (pMySQLSimpleParser ctx);
static 
 void
	identifier_list_with_parentheses    (pMySQLSimpleParser ctx);
static 
 void
	qualified_identifier    (pMySQLSimpleParser ctx);
static 
 void
	dot_identifier    (pMySQLSimpleParser ctx);
static 
 void
	ulong_number    (pMySQLSimpleParser ctx);
static 
 void
	real_ulong_number    (pMySQLSimpleParser ctx);
static 
 void
	ulonglong_number    (pMySQLSimpleParser ctx);
static 
 void
	real_ulonglong_number    (pMySQLSimpleParser ctx);
static 
 void
	string_list    (pMySQLSimpleParser ctx);
static 
 void
	text_string    (pMySQLSimpleParser ctx);
static 
 void
	text_string_no_linebreak    (pMySQLSimpleParser ctx);
static 
 void
	literal    (pMySQLSimpleParser ctx);
static 
 void
	signed_literal    (pMySQLSimpleParser ctx);
static 
 void
	string_literal    (pMySQLSimpleParser ctx);
static 
 void
	string    (pMySQLSimpleParser ctx);
static 
 void
	num_literal    (pMySQLSimpleParser ctx);
static 
 void
	bool_literal    (pMySQLSimpleParser ctx);
static 
 void
	null_literal    (pMySQLSimpleParser ctx);
static 
 void
	temporal_literal    (pMySQLSimpleParser ctx);
static 
 void
	integer_type    (pMySQLSimpleParser ctx);
static 
 void
	real_literal    (pMySQLSimpleParser ctx);
static 
 void
	float_options    (pMySQLSimpleParser ctx);
static 
 void
	precision    (pMySQLSimpleParser ctx);
static 
 void
	nchar_literal    (pMySQLSimpleParser ctx);
static 
 void
	varchar_literal    (pMySQLSimpleParser ctx);
static 
 void
	nvarchar_literal    (pMySQLSimpleParser ctx);
static 
 void
	text_or_identifier    (pMySQLSimpleParser ctx);
static 
 void
	size_number    (pMySQLSimpleParser ctx);
static 
 void
	parentheses    (pMySQLSimpleParser ctx);
static 
 void
	equal    (pMySQLSimpleParser ctx);
static 
 void
	option_type    (pMySQLSimpleParser ctx);
static 
 void
	keyword    (pMySQLSimpleParser ctx);
static 
 void
	keyword_sp    (pMySQLSimpleParser ctx);
static 
 ANTLR3_BOOLEAN
	synpred1_MySQLSimpleParser    (pMySQLSimpleParser ctx);
static 
 ANTLR3_BOOLEAN
	synpred2_MySQLSimpleParser    (pMySQLSimpleParser ctx);
static void	MySQLSimpleParserFree(pMySQLSimpleParser ctx);
static void     MySQLSimpleParserReset (pMySQLSimpleParser ctx);

/* For use in tree output where we are accumulating rule labels via label += ruleRef
 * we need a function that knows how to free a return scope when the list is destroyed.
 * We cannot just use ANTLR3_FREE because in debug tracking mode, this is a macro.
 */
static	void ANTLR3_CDECL freeScope(void * scope)
{
    ANTLR3_FREE(scope);
}

/** \brief Name of the grammar file that generated this code
 */
static const char fileName[] = "MySQLSimpleParser.g";

/** \brief Return the name of the grammar file that generated this code.
 */
static const char * getGrammarFileName()
{
	return fileName;
}
/** \brief Create a new MySQLSimpleParser parser and return a context for it.
 *
 * \param[in] instream Pointer to an input stream interface.
 *
 * \return Pointer to new parser context upon success.
 */
ANTLR3_API pMySQLSimpleParser
MySQLSimpleParserNew   (pANTLR3_COMMON_TOKEN_STREAM instream)
{
	// See if we can create a new parser with the standard constructor
	//
	return MySQLSimpleParserNewSSD(instream, NULL);
}

/** \brief Create a new MySQLSimpleParser parser and return a context for it.
 *
 * \param[in] instream Pointer to an input stream interface.
 *
 * \return Pointer to new parser context upon success.
 */
ANTLR3_API pMySQLSimpleParser
MySQLSimpleParserNewSSD   (pANTLR3_COMMON_TOKEN_STREAM instream, pANTLR3_RECOGNIZER_SHARED_STATE state)
{
    pMySQLSimpleParser ctx;	    /* Context structure we will build and return   */

    ctx	= (pMySQLSimpleParser) ANTLR3_CALLOC(1, sizeof(MySQLSimpleParser));

    if	(ctx == NULL)
    {
		// Failed to allocate memory for parser context
		//
        return  NULL;
    }

    /* -------------------------------------------------------------------
     * Memory for basic structure is allocated, now to fill in
     * the base ANTLR3 structures. We initialize the function pointers
     * for the standard ANTLR3 parser function set, but upon return
     * from here, the programmer may set the pointers to provide custom
     * implementations of each function.
     *
     * We don't use the macros defined in MySQLSimpleParser.h here, in order that you can get a sense
     * of what goes where.
     */

    /* Create a base parser/recognizer, using the supplied token stream
     */
    ctx->pParser	    = antlr3ParserNewStream(ANTLR3_SIZE_HINT, instream->tstream, state);
    /* Install the implementation of our MySQLSimpleParser interface
     */
    ctx->query	= query;
    ctx->statement	= statement;
    ctx->alter_statement	= alter_statement;
    ctx->alter_database	= alter_database;
    ctx->alter_event	= alter_event;
    ctx->alter_log_file_group	= alter_log_file_group;
    ctx->alter_server	= alter_server;
    ctx->alter_table	= alter_table;
    ctx->alter_table_commands	= alter_table_commands;
    ctx->alter_table_list_item	= alter_table_list_item;
    ctx->alter_table_list_entry	= alter_table_list_entry;
    ctx->key_def	= key_def;
    ctx->alter_order_by	= alter_order_by;
    ctx->alter_algorithm_option	= alter_algorithm_option;
    ctx->alter_lock_option	= alter_lock_option;
    ctx->index_lock_algorithm	= index_lock_algorithm;
    ctx->alter_partition	= alter_partition;
    ctx->validation	= validation;
    ctx->remove_partitioning	= remove_partitioning;
    ctx->all_or_partition_name_list	= all_or_partition_name_list;
    ctx->alter_tablespace	= alter_tablespace;
    ctx->alter_tablespace_option	= alter_tablespace_option;
    ctx->change_tablespace_option	= change_tablespace_option;
    ctx->alter_view	= alter_view;
    ctx->create_statement	= create_statement;
    ctx->create_database	= create_database;
    ctx->create_database_tail	= create_database_tail;
    ctx->create_with_definer	= create_with_definer;
    ctx->create_event	= create_event;
    ctx->create_event_tail	= create_event_tail;
    ctx->create_routine	= create_routine;
    ctx->create_procedure	= create_procedure;
    ctx->create_function	= create_function;
    ctx->create_routine_or_udf	= create_routine_or_udf;
    ctx->procedure_body	= procedure_body;
    ctx->function_body	= function_body;
    ctx->udf_tail	= udf_tail;
    ctx->routine_create_options	= routine_create_options;
    ctx->routine_create_option	= routine_create_option;
    ctx->routine_alter_options	= routine_alter_options;
    ctx->routine_option	= routine_option;
    ctx->create_index	= create_index;
    ctx->create_index_tail	= create_index_tail;
    ctx->create_index_target	= create_index_target;
    ctx->create_logfile_group	= create_logfile_group;
    ctx->create_logfile_group_tail	= create_logfile_group_tail;
    ctx->logfile_group_options	= logfile_group_options;
    ctx->logfile_group_option	= logfile_group_option;
    ctx->create_server	= create_server;
    ctx->create_server_tail	= create_server_tail;
    ctx->server_options	= server_options;
    ctx->server_option	= server_option;
    ctx->create_table	= create_table;
    ctx->create_table_tail	= create_table_tail;
    ctx->create_field_list	= create_field_list;
    ctx->create_item	= create_item;
    ctx->create_field_list_item	= create_field_list_item;
    ctx->table_creation_source	= table_creation_source;
    ctx->create_select	= create_select;
    ctx->create_tablespace	= create_tablespace;
    ctx->create_tablespace_tail	= create_tablespace_tail;
    ctx->tablespace_options	= tablespace_options;
    ctx->tablespace_option	= tablespace_option;
    ctx->create_trigger	= create_trigger;
    ctx->create_trigger_tail	= create_trigger_tail;
    ctx->trigger_follows_precedes_clause	= trigger_follows_precedes_clause;
    ctx->create_view	= create_view;
    ctx->create_view_tail	= create_view_tail;
    ctx->view_replace_or_algorithm	= view_replace_or_algorithm;
    ctx->view_algorithm	= view_algorithm;
    ctx->drop_statement	= drop_statement;
    ctx->drop_logfile_group_option	= drop_logfile_group_option;
    ctx->rename_table_statement	= rename_table_statement;
    ctx->truncate_table_statement	= truncate_table_statement;
    ctx->call_statement	= call_statement;
    ctx->delete_statement	= delete_statement;
    ctx->partition_delete	= partition_delete;
    ctx->delete_option	= delete_option;
    ctx->do_statement	= do_statement;
    ctx->handler_statement	= handler_statement;
    ctx->handler_read_or_scan	= handler_read_or_scan;
    ctx->insert_statement	= insert_statement;
    ctx->insert_lock_option	= insert_lock_option;
    ctx->insert_field_spec	= insert_field_spec;
    ctx->fields	= fields;
    ctx->insert_values	= insert_values;
    ctx->insert_query_expression	= insert_query_expression;
    ctx->insert_value_list	= insert_value_list;
    ctx->values	= values;
    ctx->duplicate_key_update	= duplicate_key_update;
    ctx->load_statement	= load_statement;
    ctx->data_or_xml	= data_or_xml;
    ctx->xml_rows_identified_by	= xml_rows_identified_by;
    ctx->load_data_file_tail	= load_data_file_tail;
    ctx->load_data_file_target_list	= load_data_file_target_list;
    ctx->field_or_variable_list	= field_or_variable_list;
    ctx->replace_statement	= replace_statement;
    ctx->select_statement	= select_statement;
    ctx->select_paren	= select_paren;
    ctx->select_from	= select_from;
    ctx->select_part2	= select_part2;
    ctx->table_expression	= table_expression;
    ctx->subquery	= subquery;
    ctx->query_expression_body	= query_expression_body;
    ctx->select_part2_derived	= select_part2_derived;
    ctx->select_option	= select_option;
    ctx->query_spec_option	= query_spec_option;
    ctx->select_item_list	= select_item_list;
    ctx->select_item	= select_item;
    ctx->select_alias	= select_alias;
    ctx->limit_clause	= limit_clause;
    ctx->simple_limit_clause	= simple_limit_clause;
    ctx->limit_options	= limit_options;
    ctx->limit_option	= limit_option;
    ctx->into_clause	= into_clause;
    ctx->procedure_analyse_clause	= procedure_analyse_clause;
    ctx->having_clause	= having_clause;
    ctx->group_by_clause	= group_by_clause;
    ctx->olap_option	= olap_option;
    ctx->order_by_clause	= order_by_clause;
    ctx->order_by_or_limit	= order_by_or_limit;
    ctx->direction	= direction;
    ctx->from_clause	= from_clause;
    ctx->where_clause	= where_clause;
    ctx->table_reference_list	= table_reference_list;
    ctx->join_table_list	= join_table_list;
    ctx->escaped_table_reference	= escaped_table_reference;
    ctx->table_reference	= table_reference;
    ctx->join	= join;
    ctx->table_factor	= table_factor;
    ctx->select_table_factor_union	= select_table_factor_union;
    ctx->query_specification	= query_specification;
    ctx->select_paren_derived	= select_paren_derived;
    ctx->join_table	= join_table;
    ctx->join_condition	= join_condition;
    ctx->union_clause	= union_clause;
    ctx->union_option	= union_option;
    ctx->union_or_order_by_or_limit	= union_or_order_by_or_limit;
    ctx->select_lock_type	= select_lock_type;
    ctx->table_alias	= table_alias;
    ctx->index_hint_list	= index_hint_list;
    ctx->index_hint	= index_hint;
    ctx->index_hint_type	= index_hint_type;
    ctx->key_or_index	= key_or_index;
    ctx->index_hint_clause	= index_hint_clause;
    ctx->index_list	= index_list;
    ctx->index_list_element	= index_list_element;
    ctx->update_statement	= update_statement;
    ctx->transaction_or_locking_statement	= transaction_or_locking_statement;
    ctx->transaction_statement	= transaction_statement;
    ctx->begin_work	= begin_work;
    ctx->transaction_characteristic	= transaction_characteristic;
    ctx->set_transaction_characteristic	= set_transaction_characteristic;
    ctx->isolation_level	= isolation_level;
    ctx->savepoint_statement	= savepoint_statement;
    ctx->lock_statement	= lock_statement;
    ctx->lock_item	= lock_item;
    ctx->lock_option	= lock_option;
    ctx->xa_statement	= xa_statement;
    ctx->xa_convert	= xa_convert;
    ctx->xid	= xid;
    ctx->replication_statement	= replication_statement;
    ctx->replication_load	= replication_load;
    ctx->change_master	= change_master;
    ctx->change_master_options	= change_master_options;
    ctx->master_option	= master_option;
    ctx->master_file_def	= master_file_def;
    ctx->server_id_list	= server_id_list;
    ctx->change_replication	= change_replication;
    ctx->filter_definition	= filter_definition;
    ctx->filter_db_list	= filter_db_list;
    ctx->filter_table_list	= filter_table_list;
    ctx->filter_string_list	= filter_string_list;
    ctx->filter_wild_db_table_string	= filter_wild_db_table_string;
    ctx->filter_db_pair_list	= filter_db_pair_list;
    ctx->slave	= slave;
    ctx->slave_until_options	= slave_until_options;
    ctx->slave_connection_options	= slave_connection_options;
    ctx->slave_thread_options	= slave_thread_options;
    ctx->slave_thread_option	= slave_thread_option;
    ctx->group_replication	= group_replication;
    ctx->prepared_statement	= prepared_statement;
    ctx->execute_statement	= execute_statement;
    ctx->execute_var_list	= execute_var_list;
    ctx->account_management_statement	= account_management_statement;
    ctx->alter_user	= alter_user;
    ctx->alter_user_tail	= alter_user_tail;
    ctx->create_user	= create_user;
    ctx->create_user_tail	= create_user_tail;
    ctx->require_clause	= require_clause;
    ctx->connect_options	= connect_options;
    ctx->account_lock_password_expire_options	= account_lock_password_expire_options;
    ctx->drop_user	= drop_user;
    ctx->parse_grant	= parse_grant;
    ctx->grant	= grant;
    ctx->grant_proxy	= grant_proxy;
    ctx->rename_user	= rename_user;
    ctx->revoke_statement	= revoke_statement;
    ctx->privilege_target	= privilege_target;
    ctx->set_password	= set_password;
    ctx->grant_object_type	= grant_object_type;
    ctx->grant_privileges	= grant_privileges;
    ctx->privilege_type	= privilege_type;
    ctx->privilege_level	= privilege_level;
    ctx->require_list	= require_list;
    ctx->require_list_element	= require_list_element;
    ctx->grant_option	= grant_option;
    ctx->table_administration_statement	= table_administration_statement;
    ctx->check_option	= check_option;
    ctx->repair_option	= repair_option;
    ctx->install_uninstall_statment	= install_uninstall_statment;
    ctx->set_statement	= set_statement;
    ctx->option_value_no_option_type	= option_value_no_option_type;
    ctx->option_value_following_option_type	= option_value_following_option_type;
    ctx->set_expression_or_default	= set_expression_or_default;
    ctx->option_value_list	= option_value_list;
    ctx->option_value	= option_value;
    ctx->show_statement	= show_statement;
    ctx->non_blocking	= non_blocking;
    ctx->from_or_in	= from_or_in;
    ctx->in_db	= in_db;
    ctx->profile_type	= profile_type;
    ctx->other_administrative_statement	= other_administrative_statement;
    ctx->key_cache_list_or_parts	= key_cache_list_or_parts;
    ctx->key_cache_list	= key_cache_list;
    ctx->assign_to_keycache	= assign_to_keycache;
    ctx->assign_to_keycache_partition	= assign_to_keycache_partition;
    ctx->cache_keys_spec	= cache_keys_spec;
    ctx->key_usage_element	= key_usage_element;
    ctx->flush_option	= flush_option;
    ctx->log_type	= log_type;
    ctx->flush_tables	= flush_tables;
    ctx->flush_tables_options	= flush_tables_options;
    ctx->load_table_index_list	= load_table_index_list;
    ctx->load_table_index_partion	= load_table_index_partion;
    ctx->reset_option	= reset_option;
    ctx->utility_statement	= utility_statement;
    ctx->describe_command	= describe_command;
    ctx->explainable_statement	= explainable_statement;
    ctx->use_command	= use_command;
    ctx->expression	= expression;
    ctx->logical_or_expression	= logical_or_expression;
    ctx->logical_xor_expression	= logical_xor_expression;
    ctx->logical_and_expression	= logical_and_expression;
    ctx->logical_not_expression	= logical_not_expression;
    ctx->boolean_primary_expression	= boolean_primary_expression;
    ctx->predicate	= predicate;
    ctx->predicate_in	= predicate_in;
    ctx->bitwise_or_expression	= bitwise_or_expression;
    ctx->bitwise_and_expression	= bitwise_and_expression;
    ctx->shift_expression	= shift_expression;
    ctx->additive_expression	= additive_expression;
    ctx->multiplicative_expression	= multiplicative_expression;
    ctx->bitwise_xor_expression	= bitwise_xor_expression;
    ctx->concat_expression	= concat_expression;
    ctx->unary_expression	= unary_expression;
    ctx->not_expression	= not_expression;
    ctx->interval_expression	= interval_expression;
    ctx->interval_function	= interval_function;
    ctx->interval_time_span	= interval_time_span;
    ctx->primary	= primary;
    ctx->jsonOperator	= jsonOperator;
    ctx->expression_with_nested_parentheses	= expression_with_nested_parentheses;
    ctx->comparison_operator	= comparison_operator;
    ctx->multiplication_operator	= multiplication_operator;
    ctx->runtime_function_call	= runtime_function_call;
    ctx->runtime_function_call_expression	= runtime_function_call_expression;
    ctx->time_function_parameters	= time_function_parameters;
    ctx->fractional_precision	= fractional_precision;
    ctx->weight_string_levels	= weight_string_levels;
    ctx->weight_string_level_list_item	= weight_string_level_list_item;
    ctx->date_time_type	= date_time_type;
    ctx->trim_function	= trim_function;
    ctx->substring_function	= substring_function;
    ctx->geometry_function	= geometry_function;
    ctx->aggregate_function	= aggregate_function;
    ctx->in_aggregate_expression	= in_aggregate_expression;
    ctx->count_function	= count_function;
    ctx->function_call	= function_call;
    ctx->function_call_expression	= function_call_expression;
    ctx->aliased_expression_list	= aliased_expression_list;
    ctx->aliased_expression	= aliased_expression;
    ctx->variable	= variable;
    ctx->user_variable	= user_variable;
    ctx->system_variable	= system_variable;
    ctx->variable_name	= variable_name;
    ctx->match_expression	= match_expression;
    ctx->case_expression	= case_expression;
    ctx->when_expression	= when_expression;
    ctx->then_expression	= then_expression;
    ctx->else_expression	= else_expression;
    ctx->cast_expression	= cast_expression;
    ctx->cast_type	= cast_type;
    ctx->encoding	= encoding;
    ctx->charset	= charset;
    ctx->not_rule	= not_rule;
    ctx->not2_rule	= not2_rule;
    ctx->or_rule	= or_rule;
    ctx->xor_rule	= xor_rule;
    ctx->and_rule	= and_rule;
    ctx->interval	= interval;
    ctx->interval_unit	= interval_unit;
    ctx->interval_timestamp_unit	= interval_timestamp_unit;
    ctx->expression_list_with_parentheses	= expression_list_with_parentheses;
    ctx->optional_expression_list_with_parentheses	= optional_expression_list_with_parentheses;
    ctx->expression_list	= expression_list;
    ctx->expression_list_with_direction	= expression_list_with_direction;
    ctx->channel	= channel;
    ctx->compound_statement	= compound_statement;
    ctx->return_statement	= return_statement;
    ctx->if_statement	= if_statement;
    ctx->if_body	= if_body;
    ctx->then_statement	= then_statement;
    ctx->compound_statement_list	= compound_statement_list;
    ctx->case_statement	= case_statement;
    ctx->else_statement	= else_statement;
    ctx->labeled_block	= labeled_block;
    ctx->unlabeled_block	= unlabeled_block;
    ctx->label	= label;
    ctx->label_identifier	= label_identifier;
    ctx->begin_end_block	= begin_end_block;
    ctx->labeled_control	= labeled_control;
    ctx->unlabeled_control	= unlabeled_control;
    ctx->loop_block	= loop_block;
    ctx->while_do_block	= while_do_block;
    ctx->repeat_until_block	= repeat_until_block;
    ctx->sp_declarations	= sp_declarations;
    ctx->sp_declaration	= sp_declaration;
    ctx->variable_declaration	= variable_declaration;
    ctx->condition_declaration	= condition_declaration;
    ctx->sp_condition	= sp_condition;
    ctx->sqlstate	= sqlstate;
    ctx->handler_declaration	= handler_declaration;
    ctx->handler_condition	= handler_condition;
    ctx->cursor_declaration	= cursor_declaration;
    ctx->iterate_statement	= iterate_statement;
    ctx->leave_statement	= leave_statement;
    ctx->get_diagnostics	= get_diagnostics;
    ctx->signal_allowed_expression	= signal_allowed_expression;
    ctx->statement_information_item	= statement_information_item;
    ctx->condition_information_item	= condition_information_item;
    ctx->signal_information_item_name	= signal_information_item_name;
    ctx->signal_statement	= signal_statement;
    ctx->resignal_statement	= resignal_statement;
    ctx->signal_information_item	= signal_information_item;
    ctx->cursor_open	= cursor_open;
    ctx->cursor_close	= cursor_close;
    ctx->cursor_fetch	= cursor_fetch;
    ctx->schedule	= schedule;
    ctx->database_option	= database_option;
    ctx->column_definition	= column_definition;
    ctx->field_spec	= field_spec;
    ctx->field_def	= field_def;
    ctx->field_def_tail	= field_def_tail;
    ctx->attribute	= attribute;
    ctx->gcol_attribute	= gcol_attribute;
    ctx->reference_definition	= reference_definition;
    ctx->index_columns	= index_columns;
    ctx->index_column	= index_column;
    ctx->index_type	= index_type;
    ctx->normal_index_option	= normal_index_option;
    ctx->fulltext_index_option	= fulltext_index_option;
    ctx->spatial_index_option	= spatial_index_option;
    ctx->all_key_option	= all_key_option;
    ctx->reference_option	= reference_option;
    ctx->data_type	= data_type;
    ctx->data_type_definition	= data_type_definition;
    ctx->data_type_elements	= data_type_elements;
    ctx->field_length	= field_length;
    ctx->field_options	= field_options;
    ctx->string_binary	= string_binary;
    ctx->ascii	= ascii;
    ctx->unicode	= unicode;
    ctx->type_datetime_precision	= type_datetime_precision;
    ctx->charset_name	= charset_name;
    ctx->charset_name_or_default	= charset_name_or_default;
    ctx->collation_name	= collation_name;
    ctx->collation_name_or_default	= collation_name_or_default;
    ctx->spatial_type	= spatial_type;
    ctx->alter_table_options	= alter_table_options;
    ctx->create_table_options	= create_table_options;
    ctx->create_table_option	= create_table_option;
    ctx->partitioning	= partitioning;
    ctx->partition_key_algorithm	= partition_key_algorithm;
    ctx->partition_definitions	= partition_definitions;
    ctx->partition_definition	= partition_definition;
    ctx->partition_option	= partition_option;
    ctx->subpartition_definition	= subpartition_definition;
    ctx->partition_value_list	= partition_value_list;
    ctx->definer_clause	= definer_clause;
    ctx->if_exists	= if_exists;
    ctx->if_not_exists	= if_not_exists;
    ctx->procedure_parameter	= procedure_parameter;
    ctx->function_parameter	= function_parameter;
    ctx->schema_identifier_pair	= schema_identifier_pair;
    ctx->view_identifier_list	= view_identifier_list;
    ctx->field_name_list	= field_name_list;
    ctx->column_assignment_list_with_default	= column_assignment_list_with_default;
    ctx->column_assignment_with_default	= column_assignment_with_default;
    ctx->charset_clause	= charset_clause;
    ctx->fields_clause	= fields_clause;
    ctx->field_term	= field_term;
    ctx->lines_clause	= lines_clause;
    ctx->line_term	= line_term;
    ctx->user_list	= user_list;
    ctx->grant_list	= grant_list;
    ctx->grant_user	= grant_user;
    ctx->user	= user;
    ctx->like_clause	= like_clause;
    ctx->like_or_where	= like_or_where;
    ctx->online_option	= online_option;
    ctx->no_write_to_bin_log	= no_write_to_bin_log;
    ctx->use_partition	= use_partition;
    ctx->column_name	= column_name;
    ctx->column_ref	= column_ref;
    ctx->column_ref_variants	= column_ref_variants;
    ctx->column_internal_ref	= column_internal_ref;
    ctx->column_ref_with_wildcard	= column_ref_with_wildcard;
    ctx->column_ref_with_wildcard2	= column_ref_with_wildcard2;
    ctx->index_name	= index_name;
    ctx->index_ref	= index_ref;
    ctx->table_wild	= table_wild;
    ctx->schema_name	= schema_name;
    ctx->schema_ref	= schema_ref;
    ctx->procedure_name	= procedure_name;
    ctx->procedure_ref	= procedure_ref;
    ctx->function_name	= function_name;
    ctx->function_ref	= function_ref;
    ctx->trigger_name	= trigger_name;
    ctx->trigger_ref	= trigger_ref;
    ctx->view_name	= view_name;
    ctx->view_ref	= view_ref;
    ctx->tablespace_name	= tablespace_name;
    ctx->tablespace_ref	= tablespace_ref;
    ctx->logfile_group_name	= logfile_group_name;
    ctx->logfile_group_ref	= logfile_group_ref;
    ctx->event_name	= event_name;
    ctx->event_ref	= event_ref;
    ctx->udf_name	= udf_name;
    ctx->server_name	= server_name;
    ctx->server_ref	= server_ref;
    ctx->engine_ref	= engine_ref;
    ctx->table_name	= table_name;
    ctx->filter_table_ref	= filter_table_ref;
    ctx->table_ref_with_wildcard	= table_ref_with_wildcard;
    ctx->table_ref	= table_ref;
    ctx->table_ref_no_db	= table_ref_no_db;
    ctx->table_name_variants	= table_name_variants;
    ctx->table_ref_list	= table_ref_list;
    ctx->table_ref_list_with_wildcard	= table_ref_list_with_wildcard;
    ctx->pure_identifier	= pure_identifier;
    ctx->identifier	= identifier;
    ctx->identifier_list	= identifier_list;
    ctx->identifier_list_with_parentheses	= identifier_list_with_parentheses;
    ctx->qualified_identifier	= qualified_identifier;
    ctx->dot_identifier	= dot_identifier;
    ctx->ulong_number	= ulong_number;
    ctx->real_ulong_number	= real_ulong_number;
    ctx->ulonglong_number	= ulonglong_number;
    ctx->real_ulonglong_number	= real_ulonglong_number;
    ctx->string_list	= string_list;
    ctx->text_string	= text_string;
    ctx->text_string_no_linebreak	= text_string_no_linebreak;
    ctx->literal	= literal;
    ctx->signed_literal	= signed_literal;
    ctx->string_literal	= string_literal;
    ctx->string	= string;
    ctx->num_literal	= num_literal;
    ctx->bool_literal	= bool_literal;
    ctx->null_literal	= null_literal;
    ctx->temporal_literal	= temporal_literal;
    ctx->integer_type	= integer_type;
    ctx->real_literal	= real_literal;
    ctx->float_options	= float_options;
    ctx->precision	= precision;
    ctx->nchar_literal	= nchar_literal;
    ctx->varchar_literal	= varchar_literal;
    ctx->nvarchar_literal	= nvarchar_literal;
    ctx->text_or_identifier	= text_or_identifier;
    ctx->size_number	= size_number;
    ctx->parentheses	= parentheses;
    ctx->equal	= equal;
    ctx->option_type	= option_type;
    ctx->keyword	= keyword;
    ctx->keyword_sp	= keyword_sp;
    ctx->synpred1_MySQLSimpleParser	= synpred1_MySQLSimpleParser;
    ctx->synpred2_MySQLSimpleParser	= synpred2_MySQLSimpleParser;
    ctx->free			= MySQLSimpleParserFree;
    ctx->reset			= MySQLSimpleParserReset;
    ctx->getGrammarFileName	= getGrammarFileName;

    /* Install the scope pushing methods.
     */


     // Install custom error collector for the front end.
     RECOGNIZER->displayRecognitionError = onMySQLParseError;

    /* Install the token table
     */
    PSRSTATE->tokenNames   = MySQLSimpleParserTokenNames;


    /* Return the newly built parser to the caller
     */
    return  ctx;
}

static void
MySQLSimpleParserReset (pMySQLSimpleParser ctx)
{
    RECOGNIZER->reset(RECOGNIZER);
}

/** Free the parser resources
 */
 static void
 MySQLSimpleParserFree(pMySQLSimpleParser ctx)
 {
    /* Free any scope memory
     */

	// Free this parser
	//
    ctx->pParser->free(ctx->pParser);


    ANTLR3_FREE(ctx);

    /* Everything is released, so we can return
     */
    return;
 }

/** Return token names used by this 
parser

 *
 * The returned pointer is used as an index into the token names table (using the token
 * number as the index).
 *
 * \return Pointer to first char * in the table.
 */
static pANTLR3_UINT8    *getTokenNames()
{
        return MySQLSimpleParserTokenNames;
}




/* Declare the bitsets
 */
/** Bitset defining follow set for error recovery in rule state: FOLLOW_statement_in_query96  */
static	ANTLR3_BITWORD FOLLOW_statement_in_query96_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_statement_in_query96	= { FOLLOW_statement_in_query96_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_begin_work_in_query100  */
static	ANTLR3_BITWORD FOLLOW_begin_work_in_query100_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_begin_work_in_query100	= { FOLLOW_begin_work_in_query100_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_query103  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_query103_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_query103	= { FOLLOW_SEMICOLON_SYMBOL_in_query103_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_query108  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_query108_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_query108	= { FOLLOW_EOF_in_query108_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_statement_in_statement119  */
static	ANTLR3_BITWORD FOLLOW_alter_statement_in_statement119_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_statement_in_statement119	= { FOLLOW_alter_statement_in_statement119_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_statement_in_statement124  */
static	ANTLR3_BITWORD FOLLOW_create_statement_in_statement124_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_statement_in_statement124	= { FOLLOW_create_statement_in_statement124_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_statement_in_statement129  */
static	ANTLR3_BITWORD FOLLOW_drop_statement_in_statement129_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_statement_in_statement129	= { FOLLOW_drop_statement_in_statement129_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_rename_table_statement_in_statement134  */
static	ANTLR3_BITWORD FOLLOW_rename_table_statement_in_statement134_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_rename_table_statement_in_statement134	= { FOLLOW_rename_table_statement_in_statement134_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_truncate_table_statement_in_statement139  */
static	ANTLR3_BITWORD FOLLOW_truncate_table_statement_in_statement139_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_truncate_table_statement_in_statement139	= { FOLLOW_truncate_table_statement_in_statement139_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_call_statement_in_statement148  */
static	ANTLR3_BITWORD FOLLOW_call_statement_in_statement148_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_call_statement_in_statement148	= { FOLLOW_call_statement_in_statement148_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_delete_statement_in_statement153  */
static	ANTLR3_BITWORD FOLLOW_delete_statement_in_statement153_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_delete_statement_in_statement153	= { FOLLOW_delete_statement_in_statement153_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_do_statement_in_statement158  */
static	ANTLR3_BITWORD FOLLOW_do_statement_in_statement158_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_do_statement_in_statement158	= { FOLLOW_do_statement_in_statement158_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_handler_statement_in_statement163  */
static	ANTLR3_BITWORD FOLLOW_handler_statement_in_statement163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_handler_statement_in_statement163	= { FOLLOW_handler_statement_in_statement163_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_statement_in_statement168  */
static	ANTLR3_BITWORD FOLLOW_insert_statement_in_statement168_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_statement_in_statement168	= { FOLLOW_insert_statement_in_statement168_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_load_statement_in_statement173  */
static	ANTLR3_BITWORD FOLLOW_load_statement_in_statement173_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_load_statement_in_statement173	= { FOLLOW_load_statement_in_statement173_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_replace_statement_in_statement178  */
static	ANTLR3_BITWORD FOLLOW_replace_statement_in_statement178_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_replace_statement_in_statement178	= { FOLLOW_replace_statement_in_statement178_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_statement183  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_statement183_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_statement183	= { FOLLOW_select_statement_in_statement183_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_update_statement_in_statement188  */
static	ANTLR3_BITWORD FOLLOW_update_statement_in_statement188_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_update_statement_in_statement188	= { FOLLOW_update_statement_in_statement188_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_statement196  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_statement196_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_statement196	= { FOLLOW_partitioning_in_statement196_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_transaction_or_locking_statement_in_statement206  */
static	ANTLR3_BITWORD FOLLOW_transaction_or_locking_statement_in_statement206_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_transaction_or_locking_statement_in_statement206	= { FOLLOW_transaction_or_locking_statement_in_statement206_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_replication_statement_in_statement213  */
static	ANTLR3_BITWORD FOLLOW_replication_statement_in_statement213_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_replication_statement_in_statement213	= { FOLLOW_replication_statement_in_statement213_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_prepared_statement_in_statement220  */
static	ANTLR3_BITWORD FOLLOW_prepared_statement_in_statement220_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_prepared_statement_in_statement220	= { FOLLOW_prepared_statement_in_statement220_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_account_management_statement_in_statement229  */
static	ANTLR3_BITWORD FOLLOW_account_management_statement_in_statement229_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_account_management_statement_in_statement229	= { FOLLOW_account_management_statement_in_statement229_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_administration_statement_in_statement234  */
static	ANTLR3_BITWORD FOLLOW_table_administration_statement_in_statement234_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_administration_statement_in_statement234	= { FOLLOW_table_administration_statement_in_statement234_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_install_uninstall_statment_in_statement239  */
static	ANTLR3_BITWORD FOLLOW_install_uninstall_statment_in_statement239_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_install_uninstall_statment_in_statement239	= { FOLLOW_install_uninstall_statment_in_statement239_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_statement_in_statement246  */
static	ANTLR3_BITWORD FOLLOW_set_statement_in_statement246_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_statement_in_statement246	= { FOLLOW_set_statement_in_statement246_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_show_statement_in_statement252  */
static	ANTLR3_BITWORD FOLLOW_show_statement_in_statement252_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_show_statement_in_statement252	= { FOLLOW_show_statement_in_statement252_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_other_administrative_statement_in_statement257  */
static	ANTLR3_BITWORD FOLLOW_other_administrative_statement_in_statement257_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_other_administrative_statement_in_statement257	= { FOLLOW_other_administrative_statement_in_statement257_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_utility_statement_in_statement266  */
static	ANTLR3_BITWORD FOLLOW_utility_statement_in_statement266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_utility_statement_in_statement266	= { FOLLOW_utility_statement_in_statement266_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_get_diagnostics_in_statement275  */
static	ANTLR3_BITWORD FOLLOW_get_diagnostics_in_statement275_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_get_diagnostics_in_statement275	= { FOLLOW_get_diagnostics_in_statement275_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_statement_in_statement282  */
static	ANTLR3_BITWORD FOLLOW_signal_statement_in_statement282_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_statement_in_statement282	= { FOLLOW_signal_statement_in_statement282_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_resignal_statement_in_statement289  */
static	ANTLR3_BITWORD FOLLOW_resignal_statement_in_statement289_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_resignal_statement_in_statement289	= { FOLLOW_resignal_statement_in_statement289_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALTER_SYMBOL_in_alter_statement301  */
static	ANTLR3_BITWORD FOLLOW_ALTER_SYMBOL_in_alter_statement301_bits[]	= { ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x0040000000000040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400800200000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x2000060000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_ALTER_SYMBOL_in_alter_statement301	= { FOLLOW_ALTER_SYMBOL_in_alter_statement301_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_database_in_alter_statement308  */
static	ANTLR3_BITWORD FOLLOW_alter_database_in_alter_statement308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_database_in_alter_statement308	= { FOLLOW_alter_database_in_alter_statement308_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_log_file_group_in_alter_statement313  */
static	ANTLR3_BITWORD FOLLOW_alter_log_file_group_in_alter_statement313_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_log_file_group_in_alter_statement313	= { FOLLOW_alter_log_file_group_in_alter_statement313_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_alter_statement318  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_alter_statement318_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_alter_statement318	= { FOLLOW_FUNCTION_SYMBOL_in_alter_statement318_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_ref_in_alter_statement320  */
static	ANTLR3_BITWORD FOLLOW_function_ref_in_alter_statement320_bits[]	= { ANTLR3_UINT64_LIT(0x0000000060014002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000804000000), ANTLR3_UINT64_LIT(0x0000010000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_function_ref_in_alter_statement320	= { FOLLOW_function_ref_in_alter_statement320_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_alter_options_in_alter_statement322  */
static	ANTLR3_BITWORD FOLLOW_routine_alter_options_in_alter_statement322_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_alter_options_in_alter_statement322	= { FOLLOW_routine_alter_options_in_alter_statement322_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_alter_statement328  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_alter_statement328_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_alter_statement328	= { FOLLOW_PROCEDURE_SYMBOL_in_alter_statement328_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_ref_in_alter_statement330  */
static	ANTLR3_BITWORD FOLLOW_procedure_ref_in_alter_statement330_bits[]	= { ANTLR3_UINT64_LIT(0x0000000060014002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000804000000), ANTLR3_UINT64_LIT(0x0000010000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_ref_in_alter_statement330	= { FOLLOW_procedure_ref_in_alter_statement330_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_alter_options_in_alter_statement332  */
static	ANTLR3_BITWORD FOLLOW_routine_alter_options_in_alter_statement332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_alter_options_in_alter_statement332	= { FOLLOW_routine_alter_options_in_alter_statement332_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_server_in_alter_statement338  */
static	ANTLR3_BITWORD FOLLOW_alter_server_in_alter_statement338_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_server_in_alter_statement338	= { FOLLOW_alter_server_in_alter_statement338_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_in_alter_statement343  */
static	ANTLR3_BITWORD FOLLOW_alter_table_in_alter_statement343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_in_alter_statement343	= { FOLLOW_alter_table_in_alter_statement343_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_tablespace_in_alter_statement348  */
static	ANTLR3_BITWORD FOLLOW_alter_tablespace_in_alter_statement348_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_tablespace_in_alter_statement348	= { FOLLOW_alter_tablespace_in_alter_statement348_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_event_in_alter_statement355  */
static	ANTLR3_BITWORD FOLLOW_alter_event_in_alter_statement355_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_event_in_alter_statement355	= { FOLLOW_alter_event_in_alter_statement355_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_view_in_alter_statement360  */
static	ANTLR3_BITWORD FOLLOW_alter_view_in_alter_statement360_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_view_in_alter_statement360	= { FOLLOW_alter_view_in_alter_statement360_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_alter_database373  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_alter_database373_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_alter_database373	= { FOLLOW_DATABASE_SYMBOL_in_alter_database373_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_alter_database375  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_alter_database375_bits[]	= { ANTLR3_UINT64_LIT(0x0100000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_alter_database375	= { FOLLOW_schema_ref_in_alter_database375_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_database_option_in_alter_database382  */
static	ANTLR3_BITWORD FOLLOW_database_option_in_alter_database382_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000802), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_database_option_in_alter_database382	= { FOLLOW_database_option_in_alter_database382_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPGRADE_SYMBOL_in_alter_database389  */
static	ANTLR3_BITWORD FOLLOW_UPGRADE_SYMBOL_in_alter_database389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPGRADE_SYMBOL_in_alter_database389	= { FOLLOW_UPGRADE_SYMBOL_in_alter_database389_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_alter_database391  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_alter_database391_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_alter_database391	= { FOLLOW_DATA_SYMBOL_in_alter_database391_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DIRECTORY_SYMBOL_in_alter_database393  */
static	ANTLR3_BITWORD FOLLOW_DIRECTORY_SYMBOL_in_alter_database393_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DIRECTORY_SYMBOL_in_alter_database393	= { FOLLOW_DIRECTORY_SYMBOL_in_alter_database393_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NAME_SYMBOL_in_alter_database395  */
static	ANTLR3_BITWORD FOLLOW_NAME_SYMBOL_in_alter_database395_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NAME_SYMBOL_in_alter_database395	= { FOLLOW_NAME_SYMBOL_in_alter_database395_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_alter_event407  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_alter_event407_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_alter_event407	= { FOLLOW_definer_clause_in_alter_event407_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENT_SYMBOL_in_alter_event412  */
static	ANTLR3_BITWORD FOLLOW_EVENT_SYMBOL_in_alter_event412_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENT_SYMBOL_in_alter_event412	= { FOLLOW_EVENT_SYMBOL_in_alter_event412_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_event_ref_in_alter_event414  */
static	ANTLR3_BITWORD FOLLOW_event_ref_in_alter_event414_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_event_ref_in_alter_event414	= { FOLLOW_event_ref_in_alter_event414_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_alter_event419  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_alter_event419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_alter_event419	= { FOLLOW_ON_SYMBOL_in_alter_event419_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SCHEDULE_SYMBOL_in_alter_event421  */
static	ANTLR3_BITWORD FOLLOW_SCHEDULE_SYMBOL_in_alter_event421_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000008), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_SCHEDULE_SYMBOL_in_alter_event421	= { FOLLOW_SCHEDULE_SYMBOL_in_alter_event421_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schedule_in_alter_event423  */
static	ANTLR3_BITWORD FOLLOW_schedule_in_alter_event423_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_schedule_in_alter_event423	= { FOLLOW_schedule_in_alter_event423_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_alter_event430  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_alter_event430_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_alter_event430	= { FOLLOW_ON_SYMBOL_in_alter_event430_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMPLETION_SYMBOL_in_alter_event432  */
static	ANTLR3_BITWORD FOLLOW_COMPLETION_SYMBOL_in_alter_event432_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMPLETION_SYMBOL_in_alter_event432	= { FOLLOW_COMPLETION_SYMBOL_in_alter_event432_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_alter_event434  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_alter_event434_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_alter_event434	= { FOLLOW_NOT_SYMBOL_in_alter_event434_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRESERVE_SYMBOL_in_alter_event437  */
static	ANTLR3_BITWORD FOLLOW_PRESERVE_SYMBOL_in_alter_event437_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_PRESERVE_SYMBOL_in_alter_event437	= { FOLLOW_PRESERVE_SYMBOL_in_alter_event437_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RENAME_SYMBOL_in_alter_event444  */
static	ANTLR3_BITWORD FOLLOW_RENAME_SYMBOL_in_alter_event444_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_RENAME_SYMBOL_in_alter_event444	= { FOLLOW_RENAME_SYMBOL_in_alter_event444_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_alter_event446  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_alter_event446_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_alter_event446	= { FOLLOW_TO_SYMBOL_in_alter_event446_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_event448  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_event448_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_event448	= { FOLLOW_identifier_in_alter_event448_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENABLE_SYMBOL_in_alter_event455  */
static	ANTLR3_BITWORD FOLLOW_ENABLE_SYMBOL_in_alter_event455_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002) };
static  ANTLR3_BITSET_LIST FOLLOW_ENABLE_SYMBOL_in_alter_event455	= { FOLLOW_ENABLE_SYMBOL_in_alter_event455_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISABLE_SYMBOL_in_alter_event459  */
static	ANTLR3_BITWORD FOLLOW_DISABLE_SYMBOL_in_alter_event459_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DISABLE_SYMBOL_in_alter_event459	= { FOLLOW_DISABLE_SYMBOL_in_alter_event459_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_alter_event462  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_alter_event462_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_alter_event462	= { FOLLOW_ON_SYMBOL_in_alter_event462_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_alter_event464  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_alter_event464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044002) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_alter_event464	= { FOLLOW_SLAVE_SYMBOL_in_alter_event464_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_alter_event473  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_alter_event473_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_alter_event473	= { FOLLOW_COMMENT_SYMBOL_in_alter_event473_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_alter_event475  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_alter_event475_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000040002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_alter_event475	= { FOLLOW_string_literal_in_alter_event475_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DO_SYMBOL_in_alter_event482  */
static	ANTLR3_BITWORD FOLLOW_DO_SYMBOL_in_alter_event482_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DO_SYMBOL_in_alter_event482	= { FOLLOW_DO_SYMBOL_in_alter_event482_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_alter_event484  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_alter_event484_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_alter_event484	= { FOLLOW_compound_statement_in_alter_event484_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGFILE_SYMBOL_in_alter_log_file_group495  */
static	ANTLR3_BITWORD FOLLOW_LOGFILE_SYMBOL_in_alter_log_file_group495_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGFILE_SYMBOL_in_alter_log_file_group495	= { FOLLOW_LOGFILE_SYMBOL_in_alter_log_file_group495_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_alter_log_file_group497  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_alter_log_file_group497_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_alter_log_file_group497	= { FOLLOW_GROUP_SYMBOL_in_alter_log_file_group497_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_ref_in_alter_log_file_group499  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_ref_in_alter_log_file_group499_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_ref_in_alter_log_file_group499	= { FOLLOW_logfile_group_ref_in_alter_log_file_group499_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_alter_log_file_group501  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_alter_log_file_group501_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000008000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_alter_log_file_group501	= { FOLLOW_ADD_SYMBOL_in_alter_log_file_group501_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNDOFILE_SYMBOL_in_alter_log_file_group503  */
static	ANTLR3_BITWORD FOLLOW_UNDOFILE_SYMBOL_in_alter_log_file_group503_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNDOFILE_SYMBOL_in_alter_log_file_group503	= { FOLLOW_UNDOFILE_SYMBOL_in_alter_log_file_group503_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_alter_log_file_group505  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_alter_log_file_group505_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_alter_log_file_group505	= { FOLLOW_string_literal_in_alter_log_file_group505_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_log_file_group510  */
static	ANTLR3_BITWORD FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_log_file_group510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_log_file_group510	= { FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_log_file_group510_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group512  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group512_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group512	= { FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group512_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_alter_log_file_group515  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_alter_log_file_group515_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_alter_log_file_group515	= { FOLLOW_size_number_in_alter_log_file_group515_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WAIT_SYMBOL_in_alter_log_file_group519  */
static	ANTLR3_BITWORD FOLLOW_WAIT_SYMBOL_in_alter_log_file_group519_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WAIT_SYMBOL_in_alter_log_file_group519	= { FOLLOW_WAIT_SYMBOL_in_alter_log_file_group519_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_alter_log_file_group522  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_alter_log_file_group522_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_alter_log_file_group522	= { FOLLOW_ENGINE_SYMBOL_in_alter_log_file_group522_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group524  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group524_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group524	= { FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group524_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_alter_log_file_group527  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_alter_log_file_group527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_alter_log_file_group527	= { FOLLOW_engine_ref_in_alter_log_file_group527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERVER_SYMBOL_in_alter_server536  */
static	ANTLR3_BITWORD FOLLOW_SERVER_SYMBOL_in_alter_server536_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SERVER_SYMBOL_in_alter_server536	= { FOLLOW_SERVER_SYMBOL_in_alter_server536_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_ref_in_alter_server538  */
static	ANTLR3_BITWORD FOLLOW_server_ref_in_alter_server538_bits[]	= { ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_server_ref_in_alter_server538	= { FOLLOW_server_ref_in_alter_server538_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_options_in_alter_server540  */
static	ANTLR3_BITWORD FOLLOW_server_options_in_alter_server540_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_server_options_in_alter_server540	= { FOLLOW_server_options_in_alter_server540_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_online_option_in_alter_table549  */
static	ANTLR3_BITWORD FOLLOW_online_option_in_alter_table549_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_online_option_in_alter_table549	= { FOLLOW_online_option_in_alter_table549_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SYMBOL_in_alter_table552  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SYMBOL_in_alter_table552_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SYMBOL_in_alter_table552	= { FOLLOW_IGNORE_SYMBOL_in_alter_table552_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_alter_table555  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_alter_table555_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_alter_table555	= { FOLLOW_TABLE_SYMBOL_in_alter_table555_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_alter_table557  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_alter_table557_bits[]	= { ANTLR3_UINT64_LIT(0x0120018800005802), ANTLR3_UINT64_LIT(0x0620400A008000A3), ANTLR3_UINT64_LIT(0x0100280800010002), ANTLR3_UINT64_LIT(0x0420000018000000), ANTLR3_UINT64_LIT(0x0000040001002000), ANTLR3_UINT64_LIT(0x14000002003000B8), ANTLR3_UINT64_LIT(0x0202000001074000), ANTLR3_UINT64_LIT(0x0000000080002002), ANTLR3_UINT64_LIT(0x0009000000000010), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_alter_table557	= { FOLLOW_table_ref_in_alter_table557_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_commands_in_alter_table559  */
static	ANTLR3_BITWORD FOLLOW_alter_table_commands_in_alter_table559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_commands_in_alter_table559	= { FOLLOW_alter_table_commands_in_alter_table559_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISCARD_SYMBOL_in_alter_table_commands569  */
static	ANTLR3_BITWORD FOLLOW_DISCARD_SYMBOL_in_alter_table_commands569_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_DISCARD_SYMBOL_in_alter_table_commands569	= { FOLLOW_DISCARD_SYMBOL_in_alter_table_commands569_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands571  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands571_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands571	= { FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands571_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IMPORT_SYMBOL_in_alter_table_commands576  */
static	ANTLR3_BITWORD FOLLOW_IMPORT_SYMBOL_in_alter_table_commands576_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_IMPORT_SYMBOL_in_alter_table_commands576	= { FOLLOW_IMPORT_SYMBOL_in_alter_table_commands576_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands578  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands578_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands578	= { FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands578_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_list_item_in_alter_table_commands583  */
static	ANTLR3_BITWORD FOLLOW_alter_table_list_item_in_alter_table_commands583_bits[]	= { ANTLR3_UINT64_LIT(0x0000008800000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_list_item_in_alter_table_commands583	= { FOLLOW_alter_table_list_item_in_alter_table_commands583_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_alter_table_commands586  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_alter_table_commands586_bits[]	= { ANTLR3_UINT64_LIT(0x0100000000005800), ANTLR3_UINT64_LIT(0x0220400A000000A1), ANTLR3_UINT64_LIT(0x0100280000010002), ANTLR3_UINT64_LIT(0x0420000018000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x14000002003000B8), ANTLR3_UINT64_LIT(0x0202000001054000), ANTLR3_UINT64_LIT(0x0000000080002000), ANTLR3_UINT64_LIT(0x0008000000000010), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_alter_table_commands586	= { FOLLOW_COMMA_SYMBOL_in_alter_table_commands586_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_list_item_in_alter_table_commands588  */
static	ANTLR3_BITWORD FOLLOW_alter_table_list_item_in_alter_table_commands588_bits[]	= { ANTLR3_UINT64_LIT(0x0000008800000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_list_item_in_alter_table_commands588	= { FOLLOW_alter_table_list_item_in_alter_table_commands588_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_alter_table_commands593  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_alter_table_commands593_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_alter_table_commands593	= { FOLLOW_partitioning_in_alter_table_commands593_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_remove_partitioning_in_alter_table_commands598  */
static	ANTLR3_BITWORD FOLLOW_remove_partitioning_in_alter_table_commands598_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_remove_partitioning_in_alter_table_commands598	= { FOLLOW_remove_partitioning_in_alter_table_commands598_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_remove_partitioning_in_alter_table_commands604  */
static	ANTLR3_BITWORD FOLLOW_remove_partitioning_in_alter_table_commands604_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_remove_partitioning_in_alter_table_commands604	= { FOLLOW_remove_partitioning_in_alter_table_commands604_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_alter_table_commands609  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_alter_table_commands609_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_alter_table_commands609	= { FOLLOW_partitioning_in_alter_table_commands609_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_partition_in_alter_table_commands616  */
static	ANTLR3_BITWORD FOLLOW_alter_partition_in_alter_table_commands616_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_partition_in_alter_table_commands616	= { FOLLOW_alter_partition_in_alter_table_commands616_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_list_entry_in_alter_table_list_item625  */
static	ANTLR3_BITWORD FOLLOW_alter_table_list_entry_in_alter_table_list_item625_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_list_entry_in_alter_table_list_item625	= { FOLLOW_alter_table_list_entry_in_alter_table_list_item625_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_table_options_in_alter_table_list_entry634  */
static	ANTLR3_BITWORD FOLLOW_alter_table_options_in_alter_table_list_entry634_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_table_options_in_alter_table_list_entry634	= { FOLLOW_alter_table_options_in_alter_table_list_entry634_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_alter_table_list_entry639  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_alter_table_list_entry639_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_alter_table_list_entry639	= { FOLLOW_ADD_SYMBOL_in_alter_table_list_entry639_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry641  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry641_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry641	= { FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry641_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_definition_in_alter_table_list_entry651  */
static	ANTLR3_BITWORD FOLLOW_column_definition_in_alter_table_list_entry651_bits[]	= { ANTLR3_UINT64_LIT(0x2000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_definition_in_alter_table_list_entry651	= { FOLLOW_column_definition_in_alter_table_list_entry651_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry654  */
static	ANTLR3_BITWORD FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry654_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry654	= { FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry654_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry658  */
static	ANTLR3_BITWORD FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry658_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry658	= { FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry658_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_table_list_entry660  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_table_list_entry660_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_table_list_entry660	= { FOLLOW_identifier_in_alter_table_list_entry660_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_alter_table_list_entry669  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_alter_table_list_entry669_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_alter_table_list_entry669	= { FOLLOW_OPEN_PAR_SYMBOL_in_alter_table_list_entry669_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_definition_in_alter_table_list_entry671  */
static	ANTLR3_BITWORD FOLLOW_column_definition_in_alter_table_list_entry671_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_definition_in_alter_table_list_entry671	= { FOLLOW_column_definition_in_alter_table_list_entry671_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_alter_table_list_entry674  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_alter_table_list_entry674_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_alter_table_list_entry674	= { FOLLOW_COMMA_SYMBOL_in_alter_table_list_entry674_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_definition_in_alter_table_list_entry676  */
static	ANTLR3_BITWORD FOLLOW_column_definition_in_alter_table_list_entry676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_definition_in_alter_table_list_entry676	= { FOLLOW_column_definition_in_alter_table_list_entry676_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_alter_table_list_entry680  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_alter_table_list_entry680_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_alter_table_list_entry680	= { FOLLOW_CLOSE_PAR_SYMBOL_in_alter_table_list_entry680_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_alter_table_list_entry689  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_alter_table_list_entry689_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000420000), ANTLR3_UINT64_LIT(0x0080000080044000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_alter_table_list_entry689	= { FOLLOW_ADD_SYMBOL_in_alter_table_list_entry689_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_def_in_alter_table_list_entry691  */
static	ANTLR3_BITWORD FOLLOW_key_def_in_alter_table_list_entry691_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_key_def_in_alter_table_list_entry691	= { FOLLOW_key_def_in_alter_table_list_entry691_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALTER_SYMBOL_in_alter_table_list_entry696  */
static	ANTLR3_BITWORD FOLLOW_ALTER_SYMBOL_in_alter_table_list_entry696_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALTER_SYMBOL_in_alter_table_list_entry696	= { FOLLOW_ALTER_SYMBOL_in_alter_table_list_entry696_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry698  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry698_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry698	= { FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry698_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_internal_ref_in_alter_table_list_entry701  */
static	ANTLR3_BITWORD FOLLOW_column_internal_ref_in_alter_table_list_entry701_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_internal_ref_in_alter_table_list_entry701	= { FOLLOW_column_internal_ref_in_alter_table_list_entry701_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_alter_table_list_entry704  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_alter_table_list_entry704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_alter_table_list_entry704	= { FOLLOW_SET_SYMBOL_in_alter_table_list_entry704_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry706  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry706_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000090000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000C040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000400), ANTLR3_UINT64_LIT(0x0C00010810100800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry706	= { FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry706_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signed_literal_in_alter_table_list_entry708  */
static	ANTLR3_BITWORD FOLLOW_signed_literal_in_alter_table_list_entry708_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_signed_literal_in_alter_table_list_entry708	= { FOLLOW_signed_literal_in_alter_table_list_entry708_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_alter_table_list_entry712  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_alter_table_list_entry712_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_alter_table_list_entry712	= { FOLLOW_DROP_SYMBOL_in_alter_table_list_entry712_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry714  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry714_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry714	= { FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry714_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANGE_SYMBOL_in_alter_table_list_entry720  */
static	ANTLR3_BITWORD FOLLOW_CHANGE_SYMBOL_in_alter_table_list_entry720_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANGE_SYMBOL_in_alter_table_list_entry720	= { FOLLOW_CHANGE_SYMBOL_in_alter_table_list_entry720_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry722  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry722_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry722	= { FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry722_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_internal_ref_in_alter_table_list_entry725  */
static	ANTLR3_BITWORD FOLLOW_column_internal_ref_in_alter_table_list_entry725_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_internal_ref_in_alter_table_list_entry725	= { FOLLOW_column_internal_ref_in_alter_table_list_entry725_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_spec_in_alter_table_list_entry727  */
static	ANTLR3_BITWORD FOLLOW_field_spec_in_alter_table_list_entry727_bits[]	= { ANTLR3_UINT64_LIT(0x2000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_spec_in_alter_table_list_entry727	= { FOLLOW_field_spec_in_alter_table_list_entry727_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry730  */
static	ANTLR3_BITWORD FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry730_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry730	= { FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry730_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry734  */
static	ANTLR3_BITWORD FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry734_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry734	= { FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry734_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_table_list_entry736  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_table_list_entry736_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_table_list_entry736	= { FOLLOW_identifier_in_alter_table_list_entry736_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MODIFY_SYMBOL_in_alter_table_list_entry743  */
static	ANTLR3_BITWORD FOLLOW_MODIFY_SYMBOL_in_alter_table_list_entry743_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_MODIFY_SYMBOL_in_alter_table_list_entry743	= { FOLLOW_MODIFY_SYMBOL_in_alter_table_list_entry743_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry745  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry745_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry745	= { FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry745_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_internal_ref_in_alter_table_list_entry748  */
static	ANTLR3_BITWORD FOLLOW_column_internal_ref_in_alter_table_list_entry748_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000003400), ANTLR3_UINT64_LIT(0x0004100003400001), ANTLR3_UINT64_LIT(0x800100C380000000), ANTLR3_UINT64_LIT(0x0001000000000001), ANTLR3_UINT64_LIT(0x0100000091000000), ANTLR3_UINT64_LIT(0x0000400000010058), ANTLR3_UINT64_LIT(0x0000200000001000), ANTLR3_UINT64_LIT(0x0000010000070160), ANTLR3_UINT64_LIT(0x0000000084000008), ANTLR3_UINT64_LIT(0x0000000014000038), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_internal_ref_in_alter_table_list_entry748	= { FOLLOW_column_internal_ref_in_alter_table_list_entry748_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_in_alter_table_list_entry750  */
static	ANTLR3_BITWORD FOLLOW_data_type_in_alter_table_list_entry750_bits[]	= { ANTLR3_UINT64_LIT(0x2000000000004002), ANTLR3_UINT64_LIT(0x0000000008000020), ANTLR3_UINT64_LIT(0x0000000000200000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0002000000040000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x100010C800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_in_alter_table_list_entry750	= { FOLLOW_data_type_in_alter_table_list_entry750_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_attribute_in_alter_table_list_entry752  */
static	ANTLR3_BITWORD FOLLOW_attribute_in_alter_table_list_entry752_bits[]	= { ANTLR3_UINT64_LIT(0x2000000000004002), ANTLR3_UINT64_LIT(0x0000000008000020), ANTLR3_UINT64_LIT(0x0000000000200000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0002000000040000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x100010C800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_attribute_in_alter_table_list_entry752	= { FOLLOW_attribute_in_alter_table_list_entry752_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry756  */
static	ANTLR3_BITWORD FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry756_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry756	= { FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry756_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry760  */
static	ANTLR3_BITWORD FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry760_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry760	= { FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry760_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_table_list_entry762  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_table_list_entry762_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_table_list_entry762	= { FOLLOW_identifier_in_alter_table_list_entry762_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_alter_table_list_entry769  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_alter_table_list_entry769_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0080000080004000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_alter_table_list_entry769	= { FOLLOW_DROP_SYMBOL_in_alter_table_list_entry769_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_table_list_entry778  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_table_list_entry778_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_table_list_entry778	= { FOLLOW_set_in_alter_table_list_entry778_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_alter_table_list_entry786  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_alter_table_list_entry786_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_alter_table_list_entry786	= { FOLLOW_column_ref_in_alter_table_list_entry786_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry793  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry793_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry793	= { FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry793_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_internal_ref_in_alter_table_list_entry796  */
static	ANTLR3_BITWORD FOLLOW_column_internal_ref_in_alter_table_list_entry796_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_internal_ref_in_alter_table_list_entry796	= { FOLLOW_column_internal_ref_in_alter_table_list_entry796_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIMARY_SYMBOL_in_alter_table_list_entry803  */
static	ANTLR3_BITWORD FOLLOW_PRIMARY_SYMBOL_in_alter_table_list_entry803_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIMARY_SYMBOL_in_alter_table_list_entry803	= { FOLLOW_PRIMARY_SYMBOL_in_alter_table_list_entry803_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_alter_table_list_entry805  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_alter_table_list_entry805_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_alter_table_list_entry805	= { FOLLOW_KEY_SYMBOL_in_alter_table_list_entry805_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOREIGN_SYMBOL_in_alter_table_list_entry812  */
static	ANTLR3_BITWORD FOLLOW_FOREIGN_SYMBOL_in_alter_table_list_entry812_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOREIGN_SYMBOL_in_alter_table_list_entry812	= { FOLLOW_FOREIGN_SYMBOL_in_alter_table_list_entry812_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_alter_table_list_entry814  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_alter_table_list_entry814_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_alter_table_list_entry814	= { FOLLOW_KEY_SYMBOL_in_alter_table_list_entry814_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_alter_table_list_entry837  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_alter_table_list_entry837_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_alter_table_list_entry837	= { FOLLOW_column_ref_in_alter_table_list_entry837_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_alter_table_list_entry850  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_alter_table_list_entry850_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_alter_table_list_entry850	= { FOLLOW_column_ref_in_alter_table_list_entry850_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISABLE_SYMBOL_in_alter_table_list_entry866  */
static	ANTLR3_BITWORD FOLLOW_DISABLE_SYMBOL_in_alter_table_list_entry866_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DISABLE_SYMBOL_in_alter_table_list_entry866	= { FOLLOW_DISABLE_SYMBOL_in_alter_table_list_entry866_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry868  */
static	ANTLR3_BITWORD FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry868_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry868	= { FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry868_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENABLE_SYMBOL_in_alter_table_list_entry873  */
static	ANTLR3_BITWORD FOLLOW_ENABLE_SYMBOL_in_alter_table_list_entry873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENABLE_SYMBOL_in_alter_table_list_entry873	= { FOLLOW_ENABLE_SYMBOL_in_alter_table_list_entry873_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry875  */
static	ANTLR3_BITWORD FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry875_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry875	= { FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry875_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry880  */
static	ANTLR3_BITWORD FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry880_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010800000000040), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry880	= { FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry880_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_alter_table_list_entry891  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_alter_table_list_entry891_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_alter_table_list_entry891	= { FOLLOW_table_ref_in_alter_table_list_entry891_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry898  */
static	ANTLR3_BITWORD FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry898_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry898	= { FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry898_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_table_list_entry900  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_table_list_entry900_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_table_list_entry900	= { FOLLOW_set_in_alter_table_list_entry900_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_alter_table_list_entry908  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_alter_table_list_entry908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_alter_table_list_entry908	= { FOLLOW_column_ref_in_alter_table_list_entry908_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_alter_table_list_entry910  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_alter_table_list_entry910_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_alter_table_list_entry910	= { FOLLOW_TO_SYMBOL_in_alter_table_list_entry910_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_alter_table_list_entry912  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_alter_table_list_entry912_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_alter_table_list_entry912	= { FOLLOW_column_ref_in_alter_table_list_entry912_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_order_by_in_alter_table_list_entry917  */
static	ANTLR3_BITWORD FOLLOW_alter_order_by_in_alter_table_list_entry917_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_order_by_in_alter_table_list_entry917	= { FOLLOW_alter_order_by_in_alter_table_list_entry917_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONVERT_SYMBOL_in_alter_table_list_entry922  */
static	ANTLR3_BITWORD FOLLOW_CONVERT_SYMBOL_in_alter_table_list_entry922_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_CONVERT_SYMBOL_in_alter_table_list_entry922	= { FOLLOW_CONVERT_SYMBOL_in_alter_table_list_entry922_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_alter_table_list_entry924  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_alter_table_list_entry924_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_alter_table_list_entry924	= { FOLLOW_TO_SYMBOL_in_alter_table_list_entry924_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_alter_table_list_entry926  */
static	ANTLR3_BITWORD FOLLOW_charset_in_alter_table_list_entry926_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_alter_table_list_entry926	= { FOLLOW_charset_in_alter_table_list_entry926_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_or_default_in_alter_table_list_entry928  */
static	ANTLR3_BITWORD FOLLOW_charset_name_or_default_in_alter_table_list_entry928_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_or_default_in_alter_table_list_entry928	= { FOLLOW_charset_name_or_default_in_alter_table_list_entry928_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_alter_table_list_entry931  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_alter_table_list_entry931_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_alter_table_list_entry931	= { FOLLOW_COLLATE_SYMBOL_in_alter_table_list_entry931_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_or_default_in_alter_table_list_entry933  */
static	ANTLR3_BITWORD FOLLOW_collation_name_or_default_in_alter_table_list_entry933_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_or_default_in_alter_table_list_entry933	= { FOLLOW_collation_name_or_default_in_alter_table_list_entry933_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FORCE_SYMBOL_in_alter_table_list_entry940  */
static	ANTLR3_BITWORD FOLLOW_FORCE_SYMBOL_in_alter_table_list_entry940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FORCE_SYMBOL_in_alter_table_list_entry940	= { FOLLOW_FORCE_SYMBOL_in_alter_table_list_entry940_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_algorithm_option_in_alter_table_list_entry947  */
static	ANTLR3_BITWORD FOLLOW_alter_algorithm_option_in_alter_table_list_entry947_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_algorithm_option_in_alter_table_list_entry947	= { FOLLOW_alter_algorithm_option_in_alter_table_list_entry947_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_lock_option_in_alter_table_list_entry954  */
static	ANTLR3_BITWORD FOLLOW_alter_lock_option_in_alter_table_list_entry954_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_lock_option_in_alter_table_list_entry954	= { FOLLOW_alter_lock_option_in_alter_table_list_entry954_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPGRADE_SYMBOL_in_alter_table_list_entry961  */
static	ANTLR3_BITWORD FOLLOW_UPGRADE_SYMBOL_in_alter_table_list_entry961_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPGRADE_SYMBOL_in_alter_table_list_entry961	= { FOLLOW_UPGRADE_SYMBOL_in_alter_table_list_entry961_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITIONING_SYMBOL_in_alter_table_list_entry963  */
static	ANTLR3_BITWORD FOLLOW_PARTITIONING_SYMBOL_in_alter_table_list_entry963_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITIONING_SYMBOL_in_alter_table_list_entry963	= { FOLLOW_PARTITIONING_SYMBOL_in_alter_table_list_entry963_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONSTRAINT_SYMBOL_in_key_def973  */
static	ANTLR3_BITWORD FOLLOW_CONSTRAINT_SYMBOL_in_key_def973_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000004000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONSTRAINT_SYMBOL_in_key_def973	= { FOLLOW_CONSTRAINT_SYMBOL_in_key_def973_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def975  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def975_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def975	= { FOLLOW_column_ref_in_key_def975_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIMARY_SYMBOL_in_key_def987  */
static	ANTLR3_BITWORD FOLLOW_PRIMARY_SYMBOL_in_key_def987_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIMARY_SYMBOL_in_key_def987	= { FOLLOW_PRIMARY_SYMBOL_in_key_def987_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_key_def989  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_key_def989_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000100000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_key_def989	= { FOLLOW_KEY_SYMBOL_in_key_def989_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def991  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def991_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def991	= { FOLLOW_column_ref_in_key_def991_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_type_in_key_def994  */
static	ANTLR3_BITWORD FOLLOW_index_type_in_key_def994_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_type_in_key_def994	= { FOLLOW_index_type_in_key_def994_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_key_def997  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_key_def997_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_key_def997	= { FOLLOW_index_columns_in_key_def997_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_normal_index_option_in_key_def999  */
static	ANTLR3_BITWORD FOLLOW_normal_index_option_in_key_def999_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_normal_index_option_in_key_def999	= { FOLLOW_normal_index_option_in_key_def999_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOREIGN_SYMBOL_in_key_def1007  */
static	ANTLR3_BITWORD FOLLOW_FOREIGN_SYMBOL_in_key_def1007_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOREIGN_SYMBOL_in_key_def1007	= { FOLLOW_FOREIGN_SYMBOL_in_key_def1007_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_key_def1009  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_key_def1009_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_key_def1009	= { FOLLOW_KEY_SYMBOL_in_key_def1009_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def1011  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def1011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def1011	= { FOLLOW_column_ref_in_key_def1011_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_key_def1014  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_key_def1014_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_key_def1014	= { FOLLOW_index_columns_in_key_def1014_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_definition_in_key_def1016  */
static	ANTLR3_BITWORD FOLLOW_reference_definition_in_key_def1016_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_definition_in_key_def1016	= { FOLLOW_reference_definition_in_key_def1016_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNIQUE_SYMBOL_in_key_def1023  */
static	ANTLR3_BITWORD FOLLOW_UNIQUE_SYMBOL_in_key_def1023_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000100000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNIQUE_SYMBOL_in_key_def1023	= { FOLLOW_UNIQUE_SYMBOL_in_key_def1023_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def1034  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def1034_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def1034	= { FOLLOW_column_ref_in_key_def1034_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_type_in_key_def1037  */
static	ANTLR3_BITWORD FOLLOW_index_type_in_key_def1037_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_type_in_key_def1037	= { FOLLOW_index_type_in_key_def1037_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_key_def1040  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_key_def1040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_key_def1040	= { FOLLOW_index_columns_in_key_def1040_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_normal_index_option_in_key_def1042  */
static	ANTLR3_BITWORD FOLLOW_normal_index_option_in_key_def1042_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_normal_index_option_in_key_def1042	= { FOLLOW_normal_index_option_in_key_def1042_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_key_def1050  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_key_def1050_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_key_def1050	= { FOLLOW_CHECK_SYMBOL_in_key_def1050_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_key_def1052  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_key_def1052_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_key_def1052	= { FOLLOW_OPEN_PAR_SYMBOL_in_key_def1052_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_key_def1054  */
static	ANTLR3_BITWORD FOLLOW_expression_in_key_def1054_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_key_def1054	= { FOLLOW_expression_in_key_def1054_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_key_def1056  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_key_def1056_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_key_def1056	= { FOLLOW_CLOSE_PAR_SYMBOL_in_key_def1056_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_key_def1065  */
static	ANTLR3_BITWORD FOLLOW_set_in_key_def1065_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000100000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_key_def1065	= { FOLLOW_set_in_key_def1065_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def1073  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def1073_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def1073	= { FOLLOW_column_ref_in_key_def1073_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_type_in_key_def1076  */
static	ANTLR3_BITWORD FOLLOW_index_type_in_key_def1076_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_type_in_key_def1076	= { FOLLOW_index_type_in_key_def1076_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_key_def1079  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_key_def1079_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_key_def1079	= { FOLLOW_index_columns_in_key_def1079_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_normal_index_option_in_key_def1081  */
static	ANTLR3_BITWORD FOLLOW_normal_index_option_in_key_def1081_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_normal_index_option_in_key_def1081	= { FOLLOW_normal_index_option_in_key_def1081_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_key_def1087  */
static	ANTLR3_BITWORD FOLLOW_set_in_key_def1087_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_key_def1087	= { FOLLOW_set_in_key_def1087_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_key_def1104  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_key_def1104_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_key_def1104	= { FOLLOW_column_ref_in_key_def1104_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_key_def1107  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_key_def1107_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_key_def1107	= { FOLLOW_index_columns_in_key_def1107_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_normal_index_option_in_key_def1109  */
static	ANTLR3_BITWORD FOLLOW_normal_index_option_in_key_def1109_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_normal_index_option_in_key_def1109	= { FOLLOW_normal_index_option_in_key_def1109_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ORDER_SYMBOL_in_alter_order_by1119  */
static	ANTLR3_BITWORD FOLLOW_ORDER_SYMBOL_in_alter_order_by1119_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_ORDER_SYMBOL_in_alter_order_by1119	= { FOLLOW_ORDER_SYMBOL_in_alter_order_by1119_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_alter_order_by1121  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_alter_order_by1121_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_alter_order_by1121	= { FOLLOW_BY_SYMBOL_in_alter_order_by1121_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_order_by1123  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_order_by1123_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_order_by1123	= { FOLLOW_identifier_in_alter_order_by1123_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_direction_in_alter_order_by1125  */
static	ANTLR3_BITWORD FOLLOW_direction_in_alter_order_by1125_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_direction_in_alter_order_by1125	= { FOLLOW_direction_in_alter_order_by1125_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_alter_order_by1140  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_alter_order_by1140_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_alter_order_by1140	= { FOLLOW_COMMA_SYMBOL_in_alter_order_by1140_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_order_by1142  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_order_by1142_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_order_by1142	= { FOLLOW_identifier_in_alter_order_by1142_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_direction_in_alter_order_by1144  */
static	ANTLR3_BITWORD FOLLOW_direction_in_alter_order_by1144_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_direction_in_alter_order_by1144	= { FOLLOW_direction_in_alter_order_by1144_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALGORITHM_SYMBOL_in_alter_algorithm_option1155  */
static	ANTLR3_BITWORD FOLLOW_ALGORITHM_SYMBOL_in_alter_algorithm_option1155_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALGORITHM_SYMBOL_in_alter_algorithm_option1155	= { FOLLOW_ALGORITHM_SYMBOL_in_alter_algorithm_option1155_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_algorithm_option1157  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_algorithm_option1157_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_algorithm_option1157	= { FOLLOW_EQUAL_OPERATOR_in_alter_algorithm_option1157_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_alter_algorithm_option1162  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_alter_algorithm_option1162_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_alter_algorithm_option1162	= { FOLLOW_DEFAULT_SYMBOL_in_alter_algorithm_option1162_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_algorithm_option1166  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_algorithm_option1166_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_algorithm_option1166	= { FOLLOW_identifier_in_alter_algorithm_option1166_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_alter_lock_option1177  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_alter_lock_option1177_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_alter_lock_option1177	= { FOLLOW_LOCK_SYMBOL_in_alter_lock_option1177_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_lock_option1179  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_lock_option1179_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_lock_option1179	= { FOLLOW_EQUAL_OPERATOR_in_alter_lock_option1179_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_alter_lock_option1183  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_alter_lock_option1183_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_alter_lock_option1183	= { FOLLOW_DEFAULT_SYMBOL_in_alter_lock_option1183_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_lock_option1187  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_lock_option1187_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_lock_option1187	= { FOLLOW_identifier_in_alter_lock_option1187_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_algorithm_option_in_index_lock_algorithm1209  */
static	ANTLR3_BITWORD FOLLOW_alter_algorithm_option_in_index_lock_algorithm1209_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_algorithm_option_in_index_lock_algorithm1209	= { FOLLOW_alter_algorithm_option_in_index_lock_algorithm1209_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_lock_option_in_index_lock_algorithm1211  */
static	ANTLR3_BITWORD FOLLOW_alter_lock_option_in_index_lock_algorithm1211_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_lock_option_in_index_lock_algorithm1211	= { FOLLOW_alter_lock_option_in_index_lock_algorithm1211_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_lock_option_in_index_lock_algorithm1219  */
static	ANTLR3_BITWORD FOLLOW_alter_lock_option_in_index_lock_algorithm1219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_lock_option_in_index_lock_algorithm1219	= { FOLLOW_alter_lock_option_in_index_lock_algorithm1219_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_algorithm_option_in_index_lock_algorithm1221  */
static	ANTLR3_BITWORD FOLLOW_alter_algorithm_option_in_index_lock_algorithm1221_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_algorithm_option_in_index_lock_algorithm1221	= { FOLLOW_alter_algorithm_option_in_index_lock_algorithm1221_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_alter_partition1239  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_alter_partition1239_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_alter_partition1239	= { FOLLOW_ADD_SYMBOL_in_alter_partition1239_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1241  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1241_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000402000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1241	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1241_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1243  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1243_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1243	= { FOLLOW_no_write_to_bin_log_in_alter_partition1243_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_alter_partition1253  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_alter_partition1253_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_alter_partition1253	= { FOLLOW_OPEN_PAR_SYMBOL_in_alter_partition1253_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_definition_in_alter_partition1255  */
static	ANTLR3_BITWORD FOLLOW_partition_definition_in_alter_partition1255_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_definition_in_alter_partition1255	= { FOLLOW_partition_definition_in_alter_partition1255_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_alter_partition1257  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_alter_partition1257_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_alter_partition1257	= { FOLLOW_CLOSE_PAR_SYMBOL_in_alter_partition1257_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITIONS_SYMBOL_in_alter_partition1264  */
static	ANTLR3_BITWORD FOLLOW_PARTITIONS_SYMBOL_in_alter_partition1264_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITIONS_SYMBOL_in_alter_partition1264	= { FOLLOW_PARTITIONS_SYMBOL_in_alter_partition1264_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_alter_partition1266  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_alter_partition1266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_alter_partition1266	= { FOLLOW_real_ulong_number_in_alter_partition1266_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_alter_partition1275  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_alter_partition1275_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_alter_partition1275	= { FOLLOW_DROP_SYMBOL_in_alter_partition1275_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1277  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1277_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1277	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1277_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_alter_partition1279  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_alter_partition1279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_alter_partition1279	= { FOLLOW_identifier_list_in_alter_partition1279_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REBUILD_SYMBOL_in_alter_partition1284  */
static	ANTLR3_BITWORD FOLLOW_REBUILD_SYMBOL_in_alter_partition1284_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REBUILD_SYMBOL_in_alter_partition1284	= { FOLLOW_REBUILD_SYMBOL_in_alter_partition1284_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1286  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1286_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1286	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1286_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1288  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1288_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1288	= { FOLLOW_no_write_to_bin_log_in_alter_partition1288_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1291  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1291_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1291	= { FOLLOW_all_or_partition_name_list_in_alter_partition1291_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTIMIZE_SYMBOL_in_alter_partition1296  */
static	ANTLR3_BITWORD FOLLOW_OPTIMIZE_SYMBOL_in_alter_partition1296_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTIMIZE_SYMBOL_in_alter_partition1296	= { FOLLOW_OPTIMIZE_SYMBOL_in_alter_partition1296_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1298  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1298_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1298	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1298_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1300  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1300_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1300	= { FOLLOW_no_write_to_bin_log_in_alter_partition1300_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1303  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1303_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1303	= { FOLLOW_all_or_partition_name_list_in_alter_partition1303_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1305  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1305_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1305	= { FOLLOW_no_write_to_bin_log_in_alter_partition1305_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ANALYZE_SYMBOL_in_alter_partition1312  */
static	ANTLR3_BITWORD FOLLOW_ANALYZE_SYMBOL_in_alter_partition1312_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ANALYZE_SYMBOL_in_alter_partition1312	= { FOLLOW_ANALYZE_SYMBOL_in_alter_partition1312_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1314  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1314_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1314	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1314_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1316  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1316_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1316	= { FOLLOW_no_write_to_bin_log_in_alter_partition1316_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1319  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1319_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1319	= { FOLLOW_all_or_partition_name_list_in_alter_partition1319_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_alter_partition1324  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_alter_partition1324_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_alter_partition1324	= { FOLLOW_CHECK_SYMBOL_in_alter_partition1324_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1326  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1326_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1326	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1326_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1328  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1328_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000040000), ANTLR3_UINT64_LIT(0x0000000000004800), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1328	= { FOLLOW_all_or_partition_name_list_in_alter_partition1328_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_check_option_in_alter_partition1330  */
static	ANTLR3_BITWORD FOLLOW_check_option_in_alter_partition1330_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000040000), ANTLR3_UINT64_LIT(0x0000000000004800), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_check_option_in_alter_partition1330	= { FOLLOW_check_option_in_alter_partition1330_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPAIR_SYMBOL_in_alter_partition1336  */
static	ANTLR3_BITWORD FOLLOW_REPAIR_SYMBOL_in_alter_partition1336_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPAIR_SYMBOL_in_alter_partition1336	= { FOLLOW_REPAIR_SYMBOL_in_alter_partition1336_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1338  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1338_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1338	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1338_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1340  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1340_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1340	= { FOLLOW_no_write_to_bin_log_in_alter_partition1340_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1343  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1343	= { FOLLOW_all_or_partition_name_list_in_alter_partition1343_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_repair_option_in_alter_partition1345  */
static	ANTLR3_BITWORD FOLLOW_repair_option_in_alter_partition1345_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_repair_option_in_alter_partition1345	= { FOLLOW_repair_option_in_alter_partition1345_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COALESCE_SYMBOL_in_alter_partition1351  */
static	ANTLR3_BITWORD FOLLOW_COALESCE_SYMBOL_in_alter_partition1351_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COALESCE_SYMBOL_in_alter_partition1351	= { FOLLOW_COALESCE_SYMBOL_in_alter_partition1351_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1353  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1353_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1353	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1353_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1355  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1355_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1355	= { FOLLOW_no_write_to_bin_log_in_alter_partition1355_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_alter_partition1358  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_alter_partition1358_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_alter_partition1358	= { FOLLOW_real_ulong_number_in_alter_partition1358_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRUNCATE_SYMBOL_in_alter_partition1365  */
static	ANTLR3_BITWORD FOLLOW_TRUNCATE_SYMBOL_in_alter_partition1365_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRUNCATE_SYMBOL_in_alter_partition1365	= { FOLLOW_TRUNCATE_SYMBOL_in_alter_partition1365_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1367  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1367_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1367	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1367_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1369  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1369_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1369	= { FOLLOW_all_or_partition_name_list_in_alter_partition1369_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXCHANGE_SYMBOL_in_alter_partition1374  */
static	ANTLR3_BITWORD FOLLOW_EXCHANGE_SYMBOL_in_alter_partition1374_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXCHANGE_SYMBOL_in_alter_partition1374	= { FOLLOW_EXCHANGE_SYMBOL_in_alter_partition1374_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1376  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1376_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1376	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1376_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_alter_partition1378  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_alter_partition1378_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_alter_partition1378	= { FOLLOW_identifier_in_alter_partition1378_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_alter_partition1380  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_alter_partition1380_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_alter_partition1380	= { FOLLOW_WITH_SYMBOL_in_alter_partition1380_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_alter_partition1382  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_alter_partition1382_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_alter_partition1382	= { FOLLOW_TABLE_SYMBOL_in_alter_partition1382_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_alter_partition1384  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_alter_partition1384_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000012000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_alter_partition1384	= { FOLLOW_table_ref_in_alter_partition1384_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_validation_in_alter_partition1386  */
static	ANTLR3_BITWORD FOLLOW_validation_in_alter_partition1386_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_validation_in_alter_partition1386	= { FOLLOW_validation_in_alter_partition1386_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REORGANIZE_SYMBOL_in_alter_partition1392  */
static	ANTLR3_BITWORD FOLLOW_REORGANIZE_SYMBOL_in_alter_partition1392_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REORGANIZE_SYMBOL_in_alter_partition1392	= { FOLLOW_REORGANIZE_SYMBOL_in_alter_partition1392_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1394  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1394_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1394	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1394_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_alter_partition1396  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_alter_partition1396_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_alter_partition1396	= { FOLLOW_no_write_to_bin_log_in_alter_partition1396_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_alter_partition1400  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_alter_partition1400_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_alter_partition1400	= { FOLLOW_identifier_list_in_alter_partition1400_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_alter_partition1402  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_alter_partition1402_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_alter_partition1402	= { FOLLOW_INTO_SYMBOL_in_alter_partition1402_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_definitions_in_alter_partition1404  */
static	ANTLR3_BITWORD FOLLOW_partition_definitions_in_alter_partition1404_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_definitions_in_alter_partition1404	= { FOLLOW_partition_definitions_in_alter_partition1404_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISCARD_SYMBOL_in_alter_partition1413  */
static	ANTLR3_BITWORD FOLLOW_DISCARD_SYMBOL_in_alter_partition1413_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DISCARD_SYMBOL_in_alter_partition1413	= { FOLLOW_DISCARD_SYMBOL_in_alter_partition1413_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1415  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1415_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1415	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1415_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1417  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1417_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1417	= { FOLLOW_all_or_partition_name_list_in_alter_partition1417_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1419  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1419	= { FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1419_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IMPORT_SYMBOL_in_alter_partition1426  */
static	ANTLR3_BITWORD FOLLOW_IMPORT_SYMBOL_in_alter_partition1426_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IMPORT_SYMBOL_in_alter_partition1426	= { FOLLOW_IMPORT_SYMBOL_in_alter_partition1426_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_alter_partition1428  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_alter_partition1428_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_alter_partition1428	= { FOLLOW_PARTITION_SYMBOL_in_alter_partition1428_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_or_partition_name_list_in_alter_partition1430  */
static	ANTLR3_BITWORD FOLLOW_all_or_partition_name_list_in_alter_partition1430_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_all_or_partition_name_list_in_alter_partition1430	= { FOLLOW_all_or_partition_name_list_in_alter_partition1430_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1432  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1432_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1432	= { FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1432_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_validation1445  */
static	ANTLR3_BITWORD FOLLOW_set_in_validation1445_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000001000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_validation1445	= { FOLLOW_set_in_validation1445_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALIDATION_SYMBOL_in_validation1453  */
static	ANTLR3_BITWORD FOLLOW_VALIDATION_SYMBOL_in_validation1453_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VALIDATION_SYMBOL_in_validation1453	= { FOLLOW_VALIDATION_SYMBOL_in_validation1453_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REMOVE_SYMBOL_in_remove_partitioning1466  */
static	ANTLR3_BITWORD FOLLOW_REMOVE_SYMBOL_in_remove_partitioning1466_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REMOVE_SYMBOL_in_remove_partitioning1466	= { FOLLOW_REMOVE_SYMBOL_in_remove_partitioning1466_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITIONING_SYMBOL_in_remove_partitioning1468  */
static	ANTLR3_BITWORD FOLLOW_PARTITIONING_SYMBOL_in_remove_partitioning1468_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITIONING_SYMBOL_in_remove_partitioning1468	= { FOLLOW_PARTITIONING_SYMBOL_in_remove_partitioning1468_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_all_or_partition_name_list1477  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_all_or_partition_name_list1477_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_all_or_partition_name_list1477	= { FOLLOW_ALL_SYMBOL_in_all_or_partition_name_list1477_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_all_or_partition_name_list1482  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_all_or_partition_name_list1482_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_all_or_partition_name_list1482	= { FOLLOW_identifier_list_in_all_or_partition_name_list1482_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_alter_tablespace1491  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_alter_tablespace1491_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_alter_tablespace1491	= { FOLLOW_TABLESPACE_SYMBOL_in_alter_tablespace1491_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_ref_in_alter_tablespace1493  */
static	ANTLR3_BITWORD FOLLOW_tablespace_ref_in_alter_tablespace1493_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0200000000004000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_ref_in_alter_tablespace1493	= { FOLLOW_tablespace_ref_in_alter_tablespace1493_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_tablespace1500  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_tablespace1500_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_tablespace1500	= { FOLLOW_set_in_alter_tablespace1500_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1508  */
static	ANTLR3_BITWORD FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1508_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1508	= { FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1508_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_alter_tablespace1510  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_alter_tablespace1510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000100000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_alter_tablespace1510	= { FOLLOW_string_literal_in_alter_tablespace1510_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_tablespace_option_in_alter_tablespace1513  */
static	ANTLR3_BITWORD FOLLOW_alter_tablespace_option_in_alter_tablespace1513_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000100000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_tablespace_option_in_alter_tablespace1513	= { FOLLOW_alter_tablespace_option_in_alter_tablespace1513_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_alter_tablespace1516  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_alter_tablespace1516_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000100000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_alter_tablespace1516	= { FOLLOW_COMMA_SYMBOL_in_alter_tablespace1516_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_tablespace_option_in_alter_tablespace1519  */
static	ANTLR3_BITWORD FOLLOW_alter_tablespace_option_in_alter_tablespace1519_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000100000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_tablespace_option_in_alter_tablespace1519	= { FOLLOW_alter_tablespace_option_in_alter_tablespace1519_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANGE_SYMBOL_in_alter_tablespace1532  */
static	ANTLR3_BITWORD FOLLOW_CHANGE_SYMBOL_in_alter_tablespace1532_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANGE_SYMBOL_in_alter_tablespace1532	= { FOLLOW_CHANGE_SYMBOL_in_alter_tablespace1532_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1534  */
static	ANTLR3_BITWORD FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1534_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1534	= { FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1534_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_alter_tablespace1536  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_alter_tablespace1536_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_alter_tablespace1536	= { FOLLOW_string_literal_in_alter_tablespace1536_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_change_tablespace_option_in_alter_tablespace1539  */
static	ANTLR3_BITWORD FOLLOW_change_tablespace_option_in_alter_tablespace1539_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_change_tablespace_option_in_alter_tablespace1539	= { FOLLOW_change_tablespace_option_in_alter_tablespace1539_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_alter_tablespace1542  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_alter_tablespace1542_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_alter_tablespace1542	= { FOLLOW_COMMA_SYMBOL_in_alter_tablespace1542_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_change_tablespace_option_in_alter_tablespace1545  */
static	ANTLR3_BITWORD FOLLOW_change_tablespace_option_in_alter_tablespace1545_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_change_tablespace_option_in_alter_tablespace1545	= { FOLLOW_change_tablespace_option_in_alter_tablespace1545_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_ONLY_SYMBOL_in_alter_tablespace1555  */
static	ANTLR3_BITWORD FOLLOW_READ_ONLY_SYMBOL_in_alter_tablespace1555_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_ONLY_SYMBOL_in_alter_tablespace1555	= { FOLLOW_READ_ONLY_SYMBOL_in_alter_tablespace1555_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_WRITE_SYMBOL_in_alter_tablespace1561  */
static	ANTLR3_BITWORD FOLLOW_READ_WRITE_SYMBOL_in_alter_tablespace1561_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_WRITE_SYMBOL_in_alter_tablespace1561	= { FOLLOW_READ_WRITE_SYMBOL_in_alter_tablespace1561_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_alter_tablespace1567  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_alter_tablespace1567_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_alter_tablespace1567	= { FOLLOW_NOT_SYMBOL_in_alter_tablespace1567_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ACCESSIBLE_SYMBOL_in_alter_tablespace1569  */
static	ANTLR3_BITWORD FOLLOW_ACCESSIBLE_SYMBOL_in_alter_tablespace1569_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ACCESSIBLE_SYMBOL_in_alter_tablespace1569	= { FOLLOW_ACCESSIBLE_SYMBOL_in_alter_tablespace1569_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_tablespace_option1581  */
static	ANTLR3_BITWORD FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_tablespace_option1581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_tablespace_option1581	= { FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_tablespace_option1581_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1583  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1583_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1583	= { FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1583_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_alter_tablespace_option1586  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_alter_tablespace_option1586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_alter_tablespace_option1586	= { FOLLOW_size_number_in_alter_tablespace_option1586_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_alter_tablespace_option1591  */
static	ANTLR3_BITWORD FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_alter_tablespace_option1591_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_alter_tablespace_option1591	= { FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_alter_tablespace_option1591_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1593  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1593_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1593	= { FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1593_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_alter_tablespace_option1596  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_alter_tablespace_option1596_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_alter_tablespace_option1596	= { FOLLOW_size_number_in_alter_tablespace_option1596_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_SIZE_SYMBOL_in_alter_tablespace_option1601  */
static	ANTLR3_BITWORD FOLLOW_MAX_SIZE_SYMBOL_in_alter_tablespace_option1601_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_SIZE_SYMBOL_in_alter_tablespace_option1601	= { FOLLOW_MAX_SIZE_SYMBOL_in_alter_tablespace_option1601_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1603  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1603_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1603	= { FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1603_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_alter_tablespace_option1606  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_alter_tablespace_option1606_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_alter_tablespace_option1606	= { FOLLOW_size_number_in_alter_tablespace_option1606_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_alter_tablespace_option1611  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_alter_tablespace_option1611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_alter_tablespace_option1611	= { FOLLOW_STORAGE_SYMBOL_in_alter_tablespace_option1611_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_alter_tablespace_option1614  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_alter_tablespace_option1614_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_alter_tablespace_option1614	= { FOLLOW_ENGINE_SYMBOL_in_alter_tablespace_option1614_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1616  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1616_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1616	= { FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1616_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_alter_tablespace_option1619  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_alter_tablespace_option1619_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_alter_tablespace_option1619	= { FOLLOW_engine_ref_in_alter_tablespace_option1619_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_tablespace_option1624  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_tablespace_option1624_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_tablespace_option1624	= { FOLLOW_set_in_alter_tablespace_option1624_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INITIAL_SIZE_SYMBOL_in_change_tablespace_option1639  */
static	ANTLR3_BITWORD FOLLOW_INITIAL_SIZE_SYMBOL_in_change_tablespace_option1639_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INITIAL_SIZE_SYMBOL_in_change_tablespace_option1639	= { FOLLOW_INITIAL_SIZE_SYMBOL_in_change_tablespace_option1639_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1641  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1641_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1641	= { FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1641_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_change_tablespace_option1644  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_change_tablespace_option1644_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_change_tablespace_option1644	= { FOLLOW_size_number_in_change_tablespace_option1644_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_change_tablespace_option1649  */
static	ANTLR3_BITWORD FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_change_tablespace_option1649_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_change_tablespace_option1649	= { FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_change_tablespace_option1649_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1651  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1651_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1651	= { FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1651_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_change_tablespace_option1654  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_change_tablespace_option1654_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_change_tablespace_option1654	= { FOLLOW_size_number_in_change_tablespace_option1654_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_SIZE_SYMBOL_in_change_tablespace_option1659  */
static	ANTLR3_BITWORD FOLLOW_MAX_SIZE_SYMBOL_in_change_tablespace_option1659_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_SIZE_SYMBOL_in_change_tablespace_option1659	= { FOLLOW_MAX_SIZE_SYMBOL_in_change_tablespace_option1659_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1661  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1661_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1661	= { FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1661_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_change_tablespace_option1664  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_change_tablespace_option1664_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_change_tablespace_option1664	= { FOLLOW_size_number_in_change_tablespace_option1664_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALGORITHM_SYMBOL_in_alter_view1674  */
static	ANTLR3_BITWORD FOLLOW_ALGORITHM_SYMBOL_in_alter_view1674_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALGORITHM_SYMBOL_in_alter_view1674	= { FOLLOW_ALGORITHM_SYMBOL_in_alter_view1674_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_alter_view1676  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_alter_view1676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_alter_view1676	= { FOLLOW_EQUAL_OPERATOR_in_alter_view1676_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_view1678  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_view1678_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_view1678	= { FOLLOW_set_in_alter_view1678_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_alter_view1694  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_alter_view1694_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_alter_view1694	= { FOLLOW_definer_clause_in_alter_view1694_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_alter_view1700  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_alter_view1700_bits[]	= { ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_alter_view1700	= { FOLLOW_SQL_SYMBOL_in_alter_view1700_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SECURITY_SYMBOL_in_alter_view1702  */
static	ANTLR3_BITWORD FOLLOW_SECURITY_SYMBOL_in_alter_view1702_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SECURITY_SYMBOL_in_alter_view1702	= { FOLLOW_SECURITY_SYMBOL_in_alter_view1702_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_alter_view1704  */
static	ANTLR3_BITWORD FOLLOW_set_in_alter_view1704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_alter_view1704	= { FOLLOW_set_in_alter_view1704_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_alter_view1716  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_alter_view1716_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_alter_view1716	= { FOLLOW_VIEW_SYMBOL_in_alter_view1716_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_ref_in_alter_view1718  */
static	ANTLR3_BITWORD FOLLOW_view_ref_in_alter_view1718_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_view_ref_in_alter_view1718	= { FOLLOW_view_ref_in_alter_view1718_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_alter_view1720  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_alter_view1720_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_alter_view1720	= { FOLLOW_identifier_list_with_parentheses_in_alter_view1720_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_alter_view1723  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_alter_view1723_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_alter_view1723	= { FOLLOW_AS_SYMBOL_in_alter_view1723_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_alter_view1725  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_alter_view1725_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_alter_view1725	= { FOLLOW_select_statement_in_alter_view1725_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_alter_view1730  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_alter_view1730_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_alter_view1730	= { FOLLOW_WITH_SYMBOL_in_alter_view1730_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_alter_view1741  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_alter_view1741_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_alter_view1741	= { FOLLOW_CHECK_SYMBOL_in_alter_view1741_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_alter_view1743  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_alter_view1743_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_alter_view1743	= { FOLLOW_OPTION_SYMBOL_in_alter_view1743_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_statement1759  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_statement1759_bits[]	= { ANTLR3_UINT64_LIT(0x8000400000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x0040000000000040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400800600000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000080040000), ANTLR3_UINT64_LIT(0x2010060000000000), ANTLR3_UINT64_LIT(0x0000010100000000), ANTLR3_UINT64_LIT(0x0000000000004101) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_statement1759	= { FOLLOW_CREATE_SYMBOL_in_create_statement1759_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_tail_in_create_statement1766  */
static	ANTLR3_BITWORD FOLLOW_create_table_tail_in_create_statement1766_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_tail_in_create_statement1766	= { FOLLOW_create_table_tail_in_create_statement1766_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_index_tail_in_create_statement1772  */
static	ANTLR3_BITWORD FOLLOW_create_index_tail_in_create_statement1772_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_index_tail_in_create_statement1772	= { FOLLOW_create_index_tail_in_create_statement1772_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_database_tail_in_create_statement1778  */
static	ANTLR3_BITWORD FOLLOW_create_database_tail_in_create_statement1778_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_database_tail_in_create_statement1778	= { FOLLOW_create_database_tail_in_create_statement1778_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_create_statement1784  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_create_statement1784_bits[]	= { ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_create_statement1784	= { FOLLOW_definer_clause_in_create_statement1784_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_event_tail_in_create_statement1798  */
static	ANTLR3_BITWORD FOLLOW_create_event_tail_in_create_statement1798_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_event_tail_in_create_statement1798	= { FOLLOW_create_event_tail_in_create_statement1798_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_view_tail_in_create_statement1806  */
static	ANTLR3_BITWORD FOLLOW_create_view_tail_in_create_statement1806_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_view_tail_in_create_statement1806	= { FOLLOW_create_view_tail_in_create_statement1806_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_routine_or_udf_in_create_statement1814  */
static	ANTLR3_BITWORD FOLLOW_create_routine_or_udf_in_create_statement1814_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_routine_or_udf_in_create_statement1814	= { FOLLOW_create_routine_or_udf_in_create_statement1814_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_trigger_tail_in_create_statement1822  */
static	ANTLR3_BITWORD FOLLOW_create_trigger_tail_in_create_statement1822_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_trigger_tail_in_create_statement1822	= { FOLLOW_create_trigger_tail_in_create_statement1822_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_replace_or_algorithm_in_create_statement1833  */
static	ANTLR3_BITWORD FOLLOW_view_replace_or_algorithm_in_create_statement1833_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_view_replace_or_algorithm_in_create_statement1833	= { FOLLOW_view_replace_or_algorithm_in_create_statement1833_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_create_statement1835  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_create_statement1835_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_create_statement1835	= { FOLLOW_definer_clause_in_create_statement1835_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_view_tail_in_create_statement1838  */
static	ANTLR3_BITWORD FOLLOW_create_view_tail_in_create_statement1838_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_view_tail_in_create_statement1838	= { FOLLOW_create_view_tail_in_create_statement1838_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_logfile_group_tail_in_create_statement1844  */
static	ANTLR3_BITWORD FOLLOW_create_logfile_group_tail_in_create_statement1844_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_logfile_group_tail_in_create_statement1844	= { FOLLOW_create_logfile_group_tail_in_create_statement1844_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_server_tail_in_create_statement1850  */
static	ANTLR3_BITWORD FOLLOW_create_server_tail_in_create_statement1850_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_server_tail_in_create_statement1850	= { FOLLOW_create_server_tail_in_create_statement1850_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_tablespace_tail_in_create_statement1856  */
static	ANTLR3_BITWORD FOLLOW_create_tablespace_tail_in_create_statement1856_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_tablespace_tail_in_create_statement1856	= { FOLLOW_create_tablespace_tail_in_create_statement1856_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_database1869  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_database1869_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_database1869	= { FOLLOW_CREATE_SYMBOL_in_create_database1869_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_database_tail_in_create_database1871  */
static	ANTLR3_BITWORD FOLLOW_create_database_tail_in_create_database1871_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_database_tail_in_create_database1871	= { FOLLOW_create_database_tail_in_create_database1871_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_database1873  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_database1873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_database1873	= { FOLLOW_SEMICOLON_SYMBOL_in_create_database1873_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_database1876  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_database1876_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_database1876	= { FOLLOW_EOF_in_create_database1876_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_create_database_tail1885  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_create_database_tail1885_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_create_database_tail1885	= { FOLLOW_DATABASE_SYMBOL_in_create_database_tail1885_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_not_exists_in_create_database_tail1887  */
static	ANTLR3_BITWORD FOLLOW_if_not_exists_in_create_database_tail1887_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_not_exists_in_create_database_tail1887	= { FOLLOW_if_not_exists_in_create_database_tail1887_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_name_in_create_database_tail1890  */
static	ANTLR3_BITWORD FOLLOW_schema_name_in_create_database_tail1890_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000802), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_name_in_create_database_tail1890	= { FOLLOW_schema_name_in_create_database_tail1890_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_database_option_in_create_database_tail1892  */
static	ANTLR3_BITWORD FOLLOW_database_option_in_create_database_tail1892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000802), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_database_option_in_create_database_tail1892	= { FOLLOW_database_option_in_create_database_tail1892_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_with_definer1902  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_with_definer1902_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_with_definer1902	= { FOLLOW_CREATE_SYMBOL_in_create_with_definer1902_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_create_with_definer1904  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_create_with_definer1904_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_create_with_definer1904	= { FOLLOW_definer_clause_in_create_with_definer1904_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_with_definer_in_create_event1915  */
static	ANTLR3_BITWORD FOLLOW_create_with_definer_in_create_event1915_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_create_with_definer_in_create_event1915	= { FOLLOW_create_with_definer_in_create_event1915_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_event_tail_in_create_event1917  */
static	ANTLR3_BITWORD FOLLOW_create_event_tail_in_create_event1917_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_event_tail_in_create_event1917	= { FOLLOW_create_event_tail_in_create_event1917_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_event1919  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_event1919_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_event1919	= { FOLLOW_SEMICOLON_SYMBOL_in_create_event1919_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_event1922  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_event1922_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_event1922	= { FOLLOW_EOF_in_create_event1922_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENT_SYMBOL_in_create_event_tail1931  */
static	ANTLR3_BITWORD FOLLOW_EVENT_SYMBOL_in_create_event_tail1931_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENT_SYMBOL_in_create_event_tail1931	= { FOLLOW_EVENT_SYMBOL_in_create_event_tail1931_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_not_exists_in_create_event_tail1933  */
static	ANTLR3_BITWORD FOLLOW_if_not_exists_in_create_event_tail1933_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_not_exists_in_create_event_tail1933	= { FOLLOW_if_not_exists_in_create_event_tail1933_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_event_name_in_create_event_tail1936  */
static	ANTLR3_BITWORD FOLLOW_event_name_in_create_event_tail1936_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_event_name_in_create_event_tail1936	= { FOLLOW_event_name_in_create_event_tail1936_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_create_event_tail1938  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_create_event_tail1938_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_create_event_tail1938	= { FOLLOW_ON_SYMBOL_in_create_event_tail1938_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SCHEDULE_SYMBOL_in_create_event_tail1940  */
static	ANTLR3_BITWORD FOLLOW_SCHEDULE_SYMBOL_in_create_event_tail1940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000008), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_SCHEDULE_SYMBOL_in_create_event_tail1940	= { FOLLOW_SCHEDULE_SYMBOL_in_create_event_tail1940_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schedule_in_create_event_tail1942  */
static	ANTLR3_BITWORD FOLLOW_schedule_in_create_event_tail1942_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_schedule_in_create_event_tail1942	= { FOLLOW_schedule_in_create_event_tail1942_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_create_event_tail1947  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_create_event_tail1947_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_create_event_tail1947	= { FOLLOW_ON_SYMBOL_in_create_event_tail1947_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMPLETION_SYMBOL_in_create_event_tail1949  */
static	ANTLR3_BITWORD FOLLOW_COMPLETION_SYMBOL_in_create_event_tail1949_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMPLETION_SYMBOL_in_create_event_tail1949	= { FOLLOW_COMPLETION_SYMBOL_in_create_event_tail1949_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_create_event_tail1951  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_create_event_tail1951_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_create_event_tail1951	= { FOLLOW_NOT_SYMBOL_in_create_event_tail1951_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRESERVE_SYMBOL_in_create_event_tail1954  */
static	ANTLR3_BITWORD FOLLOW_PRESERVE_SYMBOL_in_create_event_tail1954_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_PRESERVE_SYMBOL_in_create_event_tail1954	= { FOLLOW_PRESERVE_SYMBOL_in_create_event_tail1954_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENABLE_SYMBOL_in_create_event_tail1961  */
static	ANTLR3_BITWORD FOLLOW_ENABLE_SYMBOL_in_create_event_tail1961_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENABLE_SYMBOL_in_create_event_tail1961	= { FOLLOW_ENABLE_SYMBOL_in_create_event_tail1961_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISABLE_SYMBOL_in_create_event_tail1965  */
static	ANTLR3_BITWORD FOLLOW_DISABLE_SYMBOL_in_create_event_tail1965_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DISABLE_SYMBOL_in_create_event_tail1965	= { FOLLOW_DISABLE_SYMBOL_in_create_event_tail1965_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_create_event_tail1968  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_create_event_tail1968_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_create_event_tail1968	= { FOLLOW_ON_SYMBOL_in_create_event_tail1968_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_create_event_tail1970  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_create_event_tail1970_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000044000) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_create_event_tail1970	= { FOLLOW_SLAVE_SYMBOL_in_create_event_tail1970_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_create_event_tail1979  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_create_event_tail1979_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_create_event_tail1979	= { FOLLOW_COMMENT_SYMBOL_in_create_event_tail1979_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_create_event_tail1981  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_create_event_tail1981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_create_event_tail1981	= { FOLLOW_string_literal_in_create_event_tail1981_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DO_SYMBOL_in_create_event_tail1987  */
static	ANTLR3_BITWORD FOLLOW_DO_SYMBOL_in_create_event_tail1987_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DO_SYMBOL_in_create_event_tail1987	= { FOLLOW_DO_SYMBOL_in_create_event_tail1987_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_create_event_tail1989  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_create_event_tail1989_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_create_event_tail1989	= { FOLLOW_compound_statement_in_create_event_tail1989_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_with_definer_in_create_routine1999  */
static	ANTLR3_BITWORD FOLLOW_create_with_definer_in_create_routine1999_bits[]	= { ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_with_definer_in_create_routine1999	= { FOLLOW_create_with_definer_in_create_routine1999_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_routine_or_udf_in_create_routine2001  */
static	ANTLR3_BITWORD FOLLOW_create_routine_or_udf_in_create_routine2001_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_routine_or_udf_in_create_routine2001	= { FOLLOW_create_routine_or_udf_in_create_routine2001_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_routine2003  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_routine2003_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_routine2003	= { FOLLOW_SEMICOLON_SYMBOL_in_create_routine2003_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_routine2006  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_routine2006_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_routine2006	= { FOLLOW_EOF_in_create_routine2006_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_with_definer_in_create_procedure2016  */
static	ANTLR3_BITWORD FOLLOW_create_with_definer_in_create_procedure2016_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_with_definer_in_create_procedure2016	= { FOLLOW_create_with_definer_in_create_procedure2016_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_body_in_create_procedure2018  */
static	ANTLR3_BITWORD FOLLOW_procedure_body_in_create_procedure2018_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_body_in_create_procedure2018	= { FOLLOW_procedure_body_in_create_procedure2018_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_procedure2020  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_procedure2020_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_procedure2020	= { FOLLOW_SEMICOLON_SYMBOL_in_create_procedure2020_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_procedure2023  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_procedure2023_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_procedure2023	= { FOLLOW_EOF_in_create_procedure2023_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_with_definer_in_create_function2033  */
static	ANTLR3_BITWORD FOLLOW_create_with_definer_in_create_function2033_bits[]	= { ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_with_definer_in_create_function2033	= { FOLLOW_create_with_definer_in_create_function2033_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_body_in_create_function2035  */
static	ANTLR3_BITWORD FOLLOW_function_body_in_create_function2035_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_function_body_in_create_function2035	= { FOLLOW_function_body_in_create_function2035_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_function2037  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_function2037_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_function2037	= { FOLLOW_SEMICOLON_SYMBOL_in_create_function2037_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_function2040  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_function2040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_function2040	= { FOLLOW_EOF_in_create_function2040_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_body_in_create_routine_or_udf2049  */
static	ANTLR3_BITWORD FOLLOW_procedure_body_in_create_routine_or_udf2049_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_body_in_create_routine_or_udf2049	= { FOLLOW_procedure_body_in_create_routine_or_udf2049_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_body_in_create_routine_or_udf2054  */
static	ANTLR3_BITWORD FOLLOW_function_body_in_create_routine_or_udf2054_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_body_in_create_routine_or_udf2054	= { FOLLOW_function_body_in_create_routine_or_udf2054_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_procedure_body2063  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_procedure_body2063_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_procedure_body2063	= { FOLLOW_PROCEDURE_SYMBOL_in_procedure_body2063_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_name_in_procedure_body2065  */
static	ANTLR3_BITWORD FOLLOW_procedure_name_in_procedure_body2065_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_name_in_procedure_body2065	= { FOLLOW_procedure_name_in_procedure_body2065_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_procedure_body2067  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_procedure_body2067_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400400000000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_procedure_body2067	= { FOLLOW_OPEN_PAR_SYMBOL_in_procedure_body2067_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_parameter_in_procedure_body2070  */
static	ANTLR3_BITWORD FOLLOW_procedure_parameter_in_procedure_body2070_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_parameter_in_procedure_body2070	= { FOLLOW_procedure_parameter_in_procedure_body2070_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_procedure_body2073  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_procedure_body2073_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400400000000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_procedure_body2073	= { FOLLOW_COMMA_SYMBOL_in_procedure_body2073_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_parameter_in_procedure_body2075  */
static	ANTLR3_BITWORD FOLLOW_procedure_parameter_in_procedure_body2075_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_parameter_in_procedure_body2075	= { FOLLOW_procedure_parameter_in_procedure_body2075_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_body2081  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_body2081_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F48F637F580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0278000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400804000211), ANTLR3_UINT64_LIT(0x0000010034106BC1), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_body2081	= { FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_body2081_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_create_options_in_procedure_body2085  */
static	ANTLR3_BITWORD FOLLOW_routine_create_options_in_procedure_body2085_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_create_options_in_procedure_body2085	= { FOLLOW_routine_create_options_in_procedure_body2085_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_procedure_body2088  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_procedure_body2088_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_procedure_body2088	= { FOLLOW_compound_statement_in_procedure_body2088_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_function_body2098  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_function_body2098_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_function_body2098	= { FOLLOW_FUNCTION_SYMBOL_in_function_body2098_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_name_in_function_body2105  */
static	ANTLR3_BITWORD FOLLOW_function_name_in_function_body2105_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_function_name_in_function_body2105	= { FOLLOW_function_name_in_function_body2105_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_function_body2107  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_function_body2107_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_function_body2107	= { FOLLOW_OPEN_PAR_SYMBOL_in_function_body2107_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_parameter_in_function_body2110  */
static	ANTLR3_BITWORD FOLLOW_function_parameter_in_function_body2110_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_function_parameter_in_function_body2110	= { FOLLOW_function_parameter_in_function_body2110_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_function_body2113  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_function_body2113_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_function_body2113	= { FOLLOW_COMMA_SYMBOL_in_function_body2113_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_parameter_in_function_body2115  */
static	ANTLR3_BITWORD FOLLOW_function_parameter_in_function_body2115_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_function_parameter_in_function_body2115	= { FOLLOW_function_parameter_in_function_body2115_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_function_body2121  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_function_body2121_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_function_body2121	= { FOLLOW_CLOSE_PAR_SYMBOL_in_function_body2121_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RETURNS_SYMBOL_in_function_body2123  */
static	ANTLR3_BITWORD FOLLOW_RETURNS_SYMBOL_in_function_body2123_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000003400), ANTLR3_UINT64_LIT(0x0004100003400001), ANTLR3_UINT64_LIT(0x800100C380000000), ANTLR3_UINT64_LIT(0x0001000000000001), ANTLR3_UINT64_LIT(0x0100000091000000), ANTLR3_UINT64_LIT(0x0000400000010058), ANTLR3_UINT64_LIT(0x0000200000001000), ANTLR3_UINT64_LIT(0x0000010000070160), ANTLR3_UINT64_LIT(0x0000000084000008), ANTLR3_UINT64_LIT(0x0000000014000038), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_RETURNS_SYMBOL_in_function_body2123	= { FOLLOW_RETURNS_SYMBOL_in_function_body2123_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_in_function_body2128  */
static	ANTLR3_BITWORD FOLLOW_data_type_in_function_body2128_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F48F637F580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0278000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400804000211), ANTLR3_UINT64_LIT(0x0000010034106BC1), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_in_function_body2128	= { FOLLOW_data_type_in_function_body2128_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_create_options_in_function_body2130  */
static	ANTLR3_BITWORD FOLLOW_routine_create_options_in_function_body2130_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_create_options_in_function_body2130	= { FOLLOW_routine_create_options_in_function_body2130_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_function_body2133  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_function_body2133_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_function_body2133	= { FOLLOW_compound_statement_in_function_body2133_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_udf_tail_in_function_body2139  */
static	ANTLR3_BITWORD FOLLOW_udf_tail_in_function_body2139_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_udf_tail_in_function_body2139	= { FOLLOW_udf_tail_in_function_body2139_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AGGREGATE_SYMBOL_in_function_body2147  */
static	ANTLR3_BITWORD FOLLOW_AGGREGATE_SYMBOL_in_function_body2147_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AGGREGATE_SYMBOL_in_function_body2147	= { FOLLOW_AGGREGATE_SYMBOL_in_function_body2147_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_function_body2149  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_function_body2149_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_function_body2149	= { FOLLOW_FUNCTION_SYMBOL_in_function_body2149_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_udf_tail_in_function_body2151  */
static	ANTLR3_BITWORD FOLLOW_udf_tail_in_function_body2151_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_udf_tail_in_function_body2151	= { FOLLOW_udf_tail_in_function_body2151_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_udf_name_in_udf_tail2161  */
static	ANTLR3_BITWORD FOLLOW_udf_name_in_udf_tail2161_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_udf_name_in_udf_tail2161	= { FOLLOW_udf_name_in_udf_tail2161_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RETURNS_SYMBOL_in_udf_tail2163  */
static	ANTLR3_BITWORD FOLLOW_RETURNS_SYMBOL_in_udf_tail2163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000100), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000008) };
static  ANTLR3_BITSET_LIST FOLLOW_RETURNS_SYMBOL_in_udf_tail2163	= { FOLLOW_RETURNS_SYMBOL_in_udf_tail2163_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_udf_tail2165  */
static	ANTLR3_BITWORD FOLLOW_set_in_udf_tail2165_bits[]	= { ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_udf_tail2165	= { FOLLOW_set_in_udf_tail2165_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SONAME_SYMBOL_in_udf_tail2181  */
static	ANTLR3_BITWORD FOLLOW_SONAME_SYMBOL_in_udf_tail2181_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SONAME_SYMBOL_in_udf_tail2181	= { FOLLOW_SONAME_SYMBOL_in_udf_tail2181_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_udf_tail2183  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_udf_tail2183_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_udf_tail2183	= { FOLLOW_string_literal_in_udf_tail2183_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_create_option_in_routine_create_options2192  */
static	ANTLR3_BITWORD FOLLOW_routine_create_option_in_routine_create_options2192_bits[]	= { ANTLR3_UINT64_LIT(0x0000000060014002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000804000000), ANTLR3_UINT64_LIT(0x0000010000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_create_option_in_routine_create_options2192	= { FOLLOW_routine_create_option_in_routine_create_options2192_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_option_in_routine_create_option2203  */
static	ANTLR3_BITWORD FOLLOW_routine_option_in_routine_create_option2203_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_option_in_routine_create_option2203	= { FOLLOW_routine_option_in_routine_create_option2203_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_routine_create_option2208  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_routine_create_option2208_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_routine_create_option2208	= { FOLLOW_NOT_SYMBOL_in_routine_create_option2208_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DETERMINISTIC_SYMBOL_in_routine_create_option2211  */
static	ANTLR3_BITWORD FOLLOW_DETERMINISTIC_SYMBOL_in_routine_create_option2211_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DETERMINISTIC_SYMBOL_in_routine_create_option2211	= { FOLLOW_DETERMINISTIC_SYMBOL_in_routine_create_option2211_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_routine_create_option_in_routine_alter_options2220  */
static	ANTLR3_BITWORD FOLLOW_routine_create_option_in_routine_alter_options2220_bits[]	= { ANTLR3_UINT64_LIT(0x0000000060014002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000804000000), ANTLR3_UINT64_LIT(0x0000010000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_routine_create_option_in_routine_alter_options2220	= { FOLLOW_routine_create_option_in_routine_alter_options2220_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_routine_option2231  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_routine_option2231_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_routine_option2231	= { FOLLOW_COMMENT_SYMBOL_in_routine_option2231_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_routine_option2233  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_routine_option2233_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_routine_option2233	= { FOLLOW_string_literal_in_routine_option2233_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LANGUAGE_SYMBOL_in_routine_option2238  */
static	ANTLR3_BITWORD FOLLOW_LANGUAGE_SYMBOL_in_routine_option2238_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LANGUAGE_SYMBOL_in_routine_option2238	= { FOLLOW_LANGUAGE_SYMBOL_in_routine_option2238_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2240  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2240_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2240	= { FOLLOW_SQL_SYMBOL_in_routine_option2240_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_routine_option2245  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_routine_option2245_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_routine_option2245	= { FOLLOW_NO_SYMBOL_in_routine_option2245_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2247  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2247_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2247	= { FOLLOW_SQL_SYMBOL_in_routine_option2247_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONTAINS_SYMBOL_in_routine_option2252  */
static	ANTLR3_BITWORD FOLLOW_CONTAINS_SYMBOL_in_routine_option2252_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONTAINS_SYMBOL_in_routine_option2252	= { FOLLOW_CONTAINS_SYMBOL_in_routine_option2252_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2254  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2254_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2254	= { FOLLOW_SQL_SYMBOL_in_routine_option2254_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READS_SYMBOL_in_routine_option2259  */
static	ANTLR3_BITWORD FOLLOW_READS_SYMBOL_in_routine_option2259_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_READS_SYMBOL_in_routine_option2259	= { FOLLOW_READS_SYMBOL_in_routine_option2259_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2261  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2261_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2261	= { FOLLOW_SQL_SYMBOL_in_routine_option2261_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_routine_option2263  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_routine_option2263_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_routine_option2263	= { FOLLOW_DATA_SYMBOL_in_routine_option2263_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MODIFIES_SYMBOL_in_routine_option2268  */
static	ANTLR3_BITWORD FOLLOW_MODIFIES_SYMBOL_in_routine_option2268_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MODIFIES_SYMBOL_in_routine_option2268	= { FOLLOW_MODIFIES_SYMBOL_in_routine_option2268_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2270  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2270_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2270	= { FOLLOW_SQL_SYMBOL_in_routine_option2270_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_routine_option2272  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_routine_option2272_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_routine_option2272	= { FOLLOW_DATA_SYMBOL_in_routine_option2272_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_routine_option2277  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_routine_option2277_bits[]	= { ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_routine_option2277	= { FOLLOW_SQL_SYMBOL_in_routine_option2277_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SECURITY_SYMBOL_in_routine_option2279  */
static	ANTLR3_BITWORD FOLLOW_SECURITY_SYMBOL_in_routine_option2279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SECURITY_SYMBOL_in_routine_option2279	= { FOLLOW_SECURITY_SYMBOL_in_routine_option2279_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_routine_option2281  */
static	ANTLR3_BITWORD FOLLOW_set_in_routine_option2281_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_routine_option2281	= { FOLLOW_set_in_routine_option2281_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_index2297  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_index2297_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080040000), ANTLR3_UINT64_LIT(0x0000060000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_index2297	= { FOLLOW_CREATE_SYMBOL_in_create_index2297_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_index_tail_in_create_index2299  */
static	ANTLR3_BITWORD FOLLOW_create_index_tail_in_create_index2299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_index_tail_in_create_index2299	= { FOLLOW_create_index_tail_in_create_index2299_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_index2301  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_index2301_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_index2301	= { FOLLOW_SEMICOLON_SYMBOL_in_create_index2301_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_index2304  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_index2304_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_index2304	= { FOLLOW_EOF_in_create_index2304_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_online_option_in_create_index_tail2313  */
static	ANTLR3_BITWORD FOLLOW_online_option_in_create_index_tail2313_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080040000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_online_option_in_create_index_tail2313	= { FOLLOW_online_option_in_create_index_tail2313_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNIQUE_SYMBOL_in_create_index_tail2321  */
static	ANTLR3_BITWORD FOLLOW_UNIQUE_SYMBOL_in_create_index_tail2321_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNIQUE_SYMBOL_in_create_index_tail2321	= { FOLLOW_UNIQUE_SYMBOL_in_create_index_tail2321_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_create_index_tail2324  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_create_index_tail2324_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_create_index_tail2324	= { FOLLOW_INDEX_SYMBOL_in_create_index_tail2324_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_name_in_create_index_tail2326  */
static	ANTLR3_BITWORD FOLLOW_index_name_in_create_index_tail2326_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_name_in_create_index_tail2326	= { FOLLOW_index_name_in_create_index_tail2326_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_type_in_create_index_tail2328  */
static	ANTLR3_BITWORD FOLLOW_index_type_in_create_index_tail2328_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_type_in_create_index_tail2328	= { FOLLOW_index_type_in_create_index_tail2328_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_index_target_in_create_index_tail2331  */
static	ANTLR3_BITWORD FOLLOW_create_index_target_in_create_index_tail2331_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_index_target_in_create_index_tail2331	= { FOLLOW_create_index_target_in_create_index_tail2331_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_normal_index_option_in_create_index_tail2333  */
static	ANTLR3_BITWORD FOLLOW_normal_index_option_in_create_index_tail2333_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000200000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_normal_index_option_in_create_index_tail2333	= { FOLLOW_normal_index_option_in_create_index_tail2333_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_lock_algorithm_in_create_index_tail2336  */
static	ANTLR3_BITWORD FOLLOW_index_lock_algorithm_in_create_index_tail2336_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_lock_algorithm_in_create_index_tail2336	= { FOLLOW_index_lock_algorithm_in_create_index_tail2336_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FULLTEXT_SYMBOL_in_create_index_tail2343  */
static	ANTLR3_BITWORD FOLLOW_FULLTEXT_SYMBOL_in_create_index_tail2343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FULLTEXT_SYMBOL_in_create_index_tail2343	= { FOLLOW_FULLTEXT_SYMBOL_in_create_index_tail2343_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_create_index_tail2345  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_create_index_tail2345_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_create_index_tail2345	= { FOLLOW_INDEX_SYMBOL_in_create_index_tail2345_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_name_in_create_index_tail2347  */
static	ANTLR3_BITWORD FOLLOW_index_name_in_create_index_tail2347_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_name_in_create_index_tail2347	= { FOLLOW_index_name_in_create_index_tail2347_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_index_target_in_create_index_tail2349  */
static	ANTLR3_BITWORD FOLLOW_create_index_target_in_create_index_tail2349_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_index_target_in_create_index_tail2349	= { FOLLOW_create_index_target_in_create_index_tail2349_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_fulltext_index_option_in_create_index_tail2351  */
static	ANTLR3_BITWORD FOLLOW_fulltext_index_option_in_create_index_tail2351_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_fulltext_index_option_in_create_index_tail2351	= { FOLLOW_fulltext_index_option_in_create_index_tail2351_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_lock_algorithm_in_create_index_tail2354  */
static	ANTLR3_BITWORD FOLLOW_index_lock_algorithm_in_create_index_tail2354_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_lock_algorithm_in_create_index_tail2354	= { FOLLOW_index_lock_algorithm_in_create_index_tail2354_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SPATIAL_SYMBOL_in_create_index_tail2361  */
static	ANTLR3_BITWORD FOLLOW_SPATIAL_SYMBOL_in_create_index_tail2361_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SPATIAL_SYMBOL_in_create_index_tail2361	= { FOLLOW_SPATIAL_SYMBOL_in_create_index_tail2361_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_create_index_tail2363  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_create_index_tail2363_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_create_index_tail2363	= { FOLLOW_INDEX_SYMBOL_in_create_index_tail2363_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_name_in_create_index_tail2365  */
static	ANTLR3_BITWORD FOLLOW_index_name_in_create_index_tail2365_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_name_in_create_index_tail2365	= { FOLLOW_index_name_in_create_index_tail2365_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_index_target_in_create_index_tail2367  */
static	ANTLR3_BITWORD FOLLOW_create_index_target_in_create_index_tail2367_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_create_index_target_in_create_index_tail2367	= { FOLLOW_create_index_target_in_create_index_tail2367_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_spatial_index_option_in_create_index_tail2369  */
static	ANTLR3_BITWORD FOLLOW_spatial_index_option_in_create_index_tail2369_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000200000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_spatial_index_option_in_create_index_tail2369	= { FOLLOW_spatial_index_option_in_create_index_tail2369_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_lock_algorithm_in_create_index_tail2372  */
static	ANTLR3_BITWORD FOLLOW_index_lock_algorithm_in_create_index_tail2372_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_lock_algorithm_in_create_index_tail2372	= { FOLLOW_index_lock_algorithm_in_create_index_tail2372_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_create_index_target2385  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_create_index_target2385_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_create_index_target2385	= { FOLLOW_ON_SYMBOL_in_create_index_target2385_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_create_index_target2387  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_create_index_target2387_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_create_index_target2387	= { FOLLOW_table_ref_in_create_index_target2387_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_create_index_target2389  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_create_index_target2389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_create_index_target2389	= { FOLLOW_index_columns_in_create_index_target2389_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_logfile_group2399  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_logfile_group2399_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_logfile_group2399	= { FOLLOW_CREATE_SYMBOL_in_create_logfile_group2399_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_logfile_group_tail_in_create_logfile_group2401  */
static	ANTLR3_BITWORD FOLLOW_create_logfile_group_tail_in_create_logfile_group2401_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_logfile_group_tail_in_create_logfile_group2401	= { FOLLOW_create_logfile_group_tail_in_create_logfile_group2401_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_logfile_group2403  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_logfile_group2403_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_logfile_group2403	= { FOLLOW_SEMICOLON_SYMBOL_in_create_logfile_group2403_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_logfile_group2406  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_logfile_group2406_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_logfile_group2406	= { FOLLOW_EOF_in_create_logfile_group2406_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGFILE_SYMBOL_in_create_logfile_group_tail2415  */
static	ANTLR3_BITWORD FOLLOW_LOGFILE_SYMBOL_in_create_logfile_group_tail2415_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGFILE_SYMBOL_in_create_logfile_group_tail2415	= { FOLLOW_LOGFILE_SYMBOL_in_create_logfile_group_tail2415_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_create_logfile_group_tail2417  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_create_logfile_group_tail2417_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_create_logfile_group_tail2417	= { FOLLOW_GROUP_SYMBOL_in_create_logfile_group_tail2417_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_name_in_create_logfile_group_tail2419  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_name_in_create_logfile_group_tail2419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_name_in_create_logfile_group_tail2419	= { FOLLOW_logfile_group_name_in_create_logfile_group_tail2419_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_create_logfile_group_tail2421  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_create_logfile_group_tail2421_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x0000008000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_create_logfile_group_tail2421	= { FOLLOW_ADD_SYMBOL_in_create_logfile_group_tail2421_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_logfile_group_tail2423  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_logfile_group_tail2423_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_logfile_group_tail2423	= { FOLLOW_set_in_create_logfile_group_tail2423_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_create_logfile_group_tail2431  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_create_logfile_group_tail2431_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000300000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0002004000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_create_logfile_group_tail2431	= { FOLLOW_string_literal_in_create_logfile_group_tail2431_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_options_in_create_logfile_group_tail2435  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_options_in_create_logfile_group_tail2435_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_options_in_create_logfile_group_tail2435	= { FOLLOW_logfile_group_options_in_create_logfile_group_tail2435_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_option_in_logfile_group_options2446  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_option_in_logfile_group_options2446_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000300000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0002004000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_option_in_logfile_group_options2446	= { FOLLOW_logfile_group_option_in_logfile_group_options2446_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_logfile_group_options2449  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_logfile_group_options2449_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000300000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0002004000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_logfile_group_options2449	= { FOLLOW_COMMA_SYMBOL_in_logfile_group_options2449_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_option_in_logfile_group_options2452  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_option_in_logfile_group_options2452_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000002), ANTLR3_UINT64_LIT(0x0000300000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0002004000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_option_in_logfile_group_options2452	= { FOLLOW_logfile_group_option_in_logfile_group_options2452_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INITIAL_SIZE_SYMBOL_in_logfile_group_option2464  */
static	ANTLR3_BITWORD FOLLOW_INITIAL_SIZE_SYMBOL_in_logfile_group_option2464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INITIAL_SIZE_SYMBOL_in_logfile_group_option2464	= { FOLLOW_INITIAL_SIZE_SYMBOL_in_logfile_group_option2464_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2466  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2466_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2466	= { FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2466_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_logfile_group_option2469  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_logfile_group_option2469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_logfile_group_option2469	= { FOLLOW_size_number_in_logfile_group_option2469_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_logfile_group_option2474  */
static	ANTLR3_BITWORD FOLLOW_set_in_logfile_group_option2474_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_logfile_group_option2474	= { FOLLOW_set_in_logfile_group_option2474_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2482  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2482_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2482	= { FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2482_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_logfile_group_option2485  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_logfile_group_option2485_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_logfile_group_option2485	= { FOLLOW_size_number_in_logfile_group_option2485_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NODEGROUP_SYMBOL_in_logfile_group_option2490  */
static	ANTLR3_BITWORD FOLLOW_NODEGROUP_SYMBOL_in_logfile_group_option2490_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800800000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NODEGROUP_SYMBOL_in_logfile_group_option2490	= { FOLLOW_NODEGROUP_SYMBOL_in_logfile_group_option2490_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2492  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2492_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2492	= { FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2492_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_logfile_group_option2495  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_logfile_group_option2495_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_logfile_group_option2495	= { FOLLOW_real_ulong_number_in_logfile_group_option2495_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_logfile_group_option2500  */
static	ANTLR3_BITWORD FOLLOW_set_in_logfile_group_option2500_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_logfile_group_option2500	= { FOLLOW_set_in_logfile_group_option2500_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_logfile_group_option2511  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_logfile_group_option2511_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000900000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_logfile_group_option2511	= { FOLLOW_COMMENT_SYMBOL_in_logfile_group_option2511_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2513  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2513_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2513	= { FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2513_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_logfile_group_option2516  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_logfile_group_option2516_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_logfile_group_option2516	= { FOLLOW_string_literal_in_logfile_group_option2516_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_logfile_group_option2521  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_logfile_group_option2521_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_logfile_group_option2521	= { FOLLOW_STORAGE_SYMBOL_in_logfile_group_option2521_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_logfile_group_option2524  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_logfile_group_option2524_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_logfile_group_option2524	= { FOLLOW_ENGINE_SYMBOL_in_logfile_group_option2524_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2526  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2526_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2526	= { FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2526_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_logfile_group_option2529  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_logfile_group_option2529_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_logfile_group_option2529	= { FOLLOW_engine_ref_in_logfile_group_option2529_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_server2540  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_server2540_bits[]	= { ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_server2540	= { FOLLOW_CREATE_SYMBOL_in_create_server2540_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_server_tail_in_create_server2542  */
static	ANTLR3_BITWORD FOLLOW_create_server_tail_in_create_server2542_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_server_tail_in_create_server2542	= { FOLLOW_create_server_tail_in_create_server2542_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_server2544  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_server2544_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_server2544	= { FOLLOW_SEMICOLON_SYMBOL_in_create_server2544_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_server2547  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_server2547_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_server2547	= { FOLLOW_EOF_in_create_server2547_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERVER_SYMBOL_in_create_server_tail2556  */
static	ANTLR3_BITWORD FOLLOW_SERVER_SYMBOL_in_create_server_tail2556_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SERVER_SYMBOL_in_create_server_tail2556	= { FOLLOW_SERVER_SYMBOL_in_create_server_tail2556_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_name_in_create_server_tail2558  */
static	ANTLR3_BITWORD FOLLOW_server_name_in_create_server_tail2558_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_server_name_in_create_server_tail2558	= { FOLLOW_server_name_in_create_server_tail2558_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOREIGN_SYMBOL_in_create_server_tail2562  */
static	ANTLR3_BITWORD FOLLOW_FOREIGN_SYMBOL_in_create_server_tail2562_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOREIGN_SYMBOL_in_create_server_tail2562	= { FOLLOW_FOREIGN_SYMBOL_in_create_server_tail2562_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_create_server_tail2564  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_create_server_tail2564_bits[]	= { ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_create_server_tail2564	= { FOLLOW_DATA_SYMBOL_in_create_server_tail2564_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WRAPPER_SYMBOL_in_create_server_tail2566  */
static	ANTLR3_BITWORD FOLLOW_WRAPPER_SYMBOL_in_create_server_tail2566_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_WRAPPER_SYMBOL_in_create_server_tail2566	= { FOLLOW_WRAPPER_SYMBOL_in_create_server_tail2566_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_create_server_tail2568  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_create_server_tail2568_bits[]	= { ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_create_server_tail2568	= { FOLLOW_text_or_identifier_in_create_server_tail2568_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_options_in_create_server_tail2570  */
static	ANTLR3_BITWORD FOLLOW_server_options_in_create_server_tail2570_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_server_options_in_create_server_tail2570	= { FOLLOW_server_options_in_create_server_tail2570_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTIONS_SYMBOL_in_server_options2579  */
static	ANTLR3_BITWORD FOLLOW_OPTIONS_SYMBOL_in_server_options2579_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTIONS_SYMBOL_in_server_options2579	= { FOLLOW_OPTIONS_SYMBOL_in_server_options2579_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_server_options2581  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_server_options2581_bits[]	= { ANTLR3_UINT64_LIT(0x0002001208000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_server_options2581	= { FOLLOW_OPEN_PAR_SYMBOL_in_server_options2581_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_option_in_server_options2583  */
static	ANTLR3_BITWORD FOLLOW_server_option_in_server_options2583_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_server_option_in_server_options2583	= { FOLLOW_server_option_in_server_options2583_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_server_options2586  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_server_options2586_bits[]	= { ANTLR3_UINT64_LIT(0x0002001208000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_server_options2586	= { FOLLOW_COMMA_SYMBOL_in_server_options2586_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_option_in_server_options2588  */
static	ANTLR3_BITWORD FOLLOW_server_option_in_server_options2588_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_server_option_in_server_options2588	= { FOLLOW_server_option_in_server_options2588_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_server_options2592  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_server_options2592_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_server_options2592	= { FOLLOW_CLOSE_PAR_SYMBOL_in_server_options2592_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOST_SYMBOL_in_server_option2602  */
static	ANTLR3_BITWORD FOLLOW_HOST_SYMBOL_in_server_option2602_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_HOST_SYMBOL_in_server_option2602	= { FOLLOW_HOST_SYMBOL_in_server_option2602_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2604  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2604_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2604	= { FOLLOW_string_literal_in_server_option2604_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_server_option2609  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_server_option2609_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_server_option2609	= { FOLLOW_DATABASE_SYMBOL_in_server_option2609_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2611  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2611	= { FOLLOW_string_literal_in_server_option2611_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_server_option2616  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_server_option2616_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_server_option2616	= { FOLLOW_USER_SYMBOL_in_server_option2616_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2618  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2618_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2618	= { FOLLOW_string_literal_in_server_option2618_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_server_option2623  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_server_option2623_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_server_option2623	= { FOLLOW_PASSWORD_SYMBOL_in_server_option2623_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2625  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2625_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2625	= { FOLLOW_string_literal_in_server_option2625_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SOCKET_SYMBOL_in_server_option2630  */
static	ANTLR3_BITWORD FOLLOW_SOCKET_SYMBOL_in_server_option2630_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SOCKET_SYMBOL_in_server_option2630	= { FOLLOW_SOCKET_SYMBOL_in_server_option2630_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2632  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2632_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2632	= { FOLLOW_string_literal_in_server_option2632_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OWNER_SYMBOL_in_server_option2637  */
static	ANTLR3_BITWORD FOLLOW_OWNER_SYMBOL_in_server_option2637_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OWNER_SYMBOL_in_server_option2637	= { FOLLOW_OWNER_SYMBOL_in_server_option2637_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_server_option2639  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_server_option2639_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_server_option2639	= { FOLLOW_string_literal_in_server_option2639_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PORT_SYMBOL_in_server_option2644  */
static	ANTLR3_BITWORD FOLLOW_PORT_SYMBOL_in_server_option2644_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PORT_SYMBOL_in_server_option2644	= { FOLLOW_PORT_SYMBOL_in_server_option2644_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_server_option2646  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_server_option2646_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_server_option2646	= { FOLLOW_ulong_number_in_server_option2646_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_table2656  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_table2656_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_table2656	= { FOLLOW_CREATE_SYMBOL_in_create_table2656_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_tail_in_create_table2658  */
static	ANTLR3_BITWORD FOLLOW_create_table_tail_in_create_table2658_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_tail_in_create_table2658	= { FOLLOW_create_table_tail_in_create_table2658_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_table2660  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_table2660_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_table2660	= { FOLLOW_SEMICOLON_SYMBOL_in_create_table2660_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_table2663  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_table2663_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_table2663	= { FOLLOW_EOF_in_create_table2663_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TEMPORARY_SYMBOL_in_create_table_tail2672  */
static	ANTLR3_BITWORD FOLLOW_TEMPORARY_SYMBOL_in_create_table_tail2672_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_TEMPORARY_SYMBOL_in_create_table_tail2672	= { FOLLOW_TEMPORARY_SYMBOL_in_create_table_tail2672_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_create_table_tail2675  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_create_table_tail2675_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_create_table_tail2675	= { FOLLOW_TABLE_SYMBOL_in_create_table_tail2675_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_not_exists_in_create_table_tail2677  */
static	ANTLR3_BITWORD FOLLOW_if_not_exists_in_create_table_tail2677_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_not_exists_in_create_table_tail2677	= { FOLLOW_if_not_exists_in_create_table_tail2677_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_in_create_table_tail2680  */
static	ANTLR3_BITWORD FOLLOW_table_name_in_create_table_tail2680_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800005800), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x10000000C0000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_in_create_table_tail2680	= { FOLLOW_table_name_in_create_table_tail2680_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2689  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2689_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000420000), ANTLR3_UINT64_LIT(0x0080000080044000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0010000000004000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2689	= { FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2689_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_field_list_in_create_table_tail2702  */
static	ANTLR3_BITWORD FOLLOW_create_field_list_in_create_table_tail2702_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_field_list_in_create_table_tail2702	= { FOLLOW_create_field_list_in_create_table_tail2702_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2704  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800005802), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x00000000C0000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2704	= { FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2704_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_options_in_create_table_tail2706  */
static	ANTLR3_BITWORD FOLLOW_create_table_options_in_create_table_tail2706_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_options_in_create_table_tail2706	= { FOLLOW_create_table_options_in_create_table_tail2706_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_create_table_tail2709  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_create_table_tail2709_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_create_table_tail2709	= { FOLLOW_partitioning_in_create_table_tail2709_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_creation_source_in_create_table_tail2712  */
static	ANTLR3_BITWORD FOLLOW_table_creation_source_in_create_table_tail2712_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_creation_source_in_create_table_tail2712	= { FOLLOW_table_creation_source_in_create_table_tail2712_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_create_table_tail2722  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_create_table_tail2722_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_create_table_tail2722	= { FOLLOW_partitioning_in_create_table_tail2722_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_select_in_create_table_tail2724  */
static	ANTLR3_BITWORD FOLLOW_create_select_in_create_table_tail2724_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_select_in_create_table_tail2724	= { FOLLOW_create_select_in_create_table_tail2724_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2726  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2726_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2726	= { FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2726_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_clause_in_create_table_tail2728  */
static	ANTLR3_BITWORD FOLLOW_union_clause_in_create_table_tail2728_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_clause_in_create_table_tail2728	= { FOLLOW_union_clause_in_create_table_tail2728_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_options_in_create_table_tail2742  */
static	ANTLR3_BITWORD FOLLOW_create_table_options_in_create_table_tail2742_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_options_in_create_table_tail2742	= { FOLLOW_create_table_options_in_create_table_tail2742_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partitioning_in_create_table_tail2745  */
static	ANTLR3_BITWORD FOLLOW_partitioning_in_create_table_tail2745_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partitioning_in_create_table_tail2745	= { FOLLOW_partitioning_in_create_table_tail2745_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_creation_source_in_create_table_tail2748  */
static	ANTLR3_BITWORD FOLLOW_table_creation_source_in_create_table_tail2748_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_creation_source_in_create_table_tail2748	= { FOLLOW_table_creation_source_in_create_table_tail2748_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIKE_SYMBOL_in_create_table_tail2755  */
static	ANTLR3_BITWORD FOLLOW_LIKE_SYMBOL_in_create_table_tail2755_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_LIKE_SYMBOL_in_create_table_tail2755	= { FOLLOW_LIKE_SYMBOL_in_create_table_tail2755_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_create_table_tail2757  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_create_table_tail2757_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_create_table_tail2757	= { FOLLOW_table_ref_in_create_table_tail2757_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2764  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2764_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2764	= { FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2764_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIKE_SYMBOL_in_create_table_tail2766  */
static	ANTLR3_BITWORD FOLLOW_LIKE_SYMBOL_in_create_table_tail2766_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_LIKE_SYMBOL_in_create_table_tail2766	= { FOLLOW_LIKE_SYMBOL_in_create_table_tail2766_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_create_table_tail2768  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_create_table_tail2768_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_create_table_tail2768	= { FOLLOW_table_ref_in_create_table_tail2768_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2770  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2770_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2770	= { FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2770_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_item_in_create_field_list2783  */
static	ANTLR3_BITWORD FOLLOW_create_item_in_create_field_list2783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_create_item_in_create_field_list2783	= { FOLLOW_create_item_in_create_field_list2783_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_create_field_list2786  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_create_field_list2786_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000420000), ANTLR3_UINT64_LIT(0x0080000080044000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0010000000004000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_create_field_list2786	= { FOLLOW_COMMA_SYMBOL_in_create_field_list2786_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_item_in_create_field_list2788  */
static	ANTLR3_BITWORD FOLLOW_create_item_in_create_field_list2788_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_create_item_in_create_field_list2788	= { FOLLOW_create_item_in_create_field_list2788_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_field_list_item_in_create_item2799  */
static	ANTLR3_BITWORD FOLLOW_create_field_list_item_in_create_item2799_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_field_list_item_in_create_item2799	= { FOLLOW_create_field_list_item_in_create_item2799_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_definition_in_create_field_list_item2808  */
static	ANTLR3_BITWORD FOLLOW_column_definition_in_create_field_list_item2808_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_definition_in_create_field_list_item2808	= { FOLLOW_column_definition_in_create_field_list_item2808_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_def_in_create_field_list_item2813  */
static	ANTLR3_BITWORD FOLLOW_key_def_in_create_field_list_item2813_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_key_def_in_create_field_list_item2813	= { FOLLOW_key_def_in_create_field_list_item2813_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_table_creation_source2832  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_table_creation_source2832_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_table_creation_source2832	= { FOLLOW_AS_SYMBOL_in_table_creation_source2832_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_select_in_table_creation_source2840  */
static	ANTLR3_BITWORD FOLLOW_create_select_in_table_creation_source2840_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_select_in_table_creation_source2840	= { FOLLOW_create_select_in_table_creation_source2840_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_clause_in_table_creation_source2842  */
static	ANTLR3_BITWORD FOLLOW_union_clause_in_table_creation_source2842_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_clause_in_table_creation_source2842	= { FOLLOW_union_clause_in_table_creation_source2842_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_table_creation_source2849  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_table_creation_source2849_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_table_creation_source2849	= { FOLLOW_OPEN_PAR_SYMBOL_in_table_creation_source2849_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_select_in_table_creation_source2851  */
static	ANTLR3_BITWORD FOLLOW_create_select_in_table_creation_source2851_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_select_in_table_creation_source2851	= { FOLLOW_create_select_in_table_creation_source2851_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_table_creation_source2853  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_table_creation_source2853_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_table_creation_source2853	= { FOLLOW_CLOSE_PAR_SYMBOL_in_table_creation_source2853_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_or_order_by_or_limit_in_table_creation_source2855  */
static	ANTLR3_BITWORD FOLLOW_union_or_order_by_or_limit_in_table_creation_source2855_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_or_order_by_or_limit_in_table_creation_source2855	= { FOLLOW_union_or_order_by_or_limit_in_table_creation_source2855_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_create_select2869  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_create_select2869_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_create_select2869	= { FOLLOW_SELECT_SYMBOL_in_create_select2869_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_option_in_create_select2885  */
static	ANTLR3_BITWORD FOLLOW_select_option_in_create_select2885_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_option_in_create_select2885	= { FOLLOW_select_option_in_create_select2885_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_list_in_create_select2889  */
static	ANTLR3_BITWORD FOLLOW_select_item_list_in_create_select2889_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001828000), ANTLR3_UINT64_LIT(0x2008000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_list_in_create_select2889	= { FOLLOW_select_item_list_in_create_select2889_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_expression_in_create_select2891  */
static	ANTLR3_BITWORD FOLLOW_table_expression_in_create_select2891_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_expression_in_create_select2891	= { FOLLOW_table_expression_in_create_select2891_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_tablespace2901  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_tablespace2901_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_tablespace2901	= { FOLLOW_CREATE_SYMBOL_in_create_tablespace2901_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_tablespace_tail_in_create_tablespace2903  */
static	ANTLR3_BITWORD FOLLOW_create_tablespace_tail_in_create_tablespace2903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_tablespace_tail_in_create_tablespace2903	= { FOLLOW_create_tablespace_tail_in_create_tablespace2903_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_tablespace2905  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_tablespace2905_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_tablespace2905	= { FOLLOW_SEMICOLON_SYMBOL_in_create_tablespace2905_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_tablespace2908  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_tablespace2908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_tablespace2908	= { FOLLOW_EOF_in_create_tablespace2908_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_create_tablespace_tail2917  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_create_tablespace_tail2917_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_create_tablespace_tail2917	= { FOLLOW_TABLESPACE_SYMBOL_in_create_tablespace_tail2917_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_name_in_create_tablespace_tail2919  */
static	ANTLR3_BITWORD FOLLOW_tablespace_name_in_create_tablespace_tail2919_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_name_in_create_tablespace_tail2919	= { FOLLOW_tablespace_name_in_create_tablespace_tail2919_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ADD_SYMBOL_in_create_tablespace_tail2921  */
static	ANTLR3_BITWORD FOLLOW_ADD_SYMBOL_in_create_tablespace_tail2921_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ADD_SYMBOL_in_create_tablespace_tail2921	= { FOLLOW_ADD_SYMBOL_in_create_tablespace_tail2921_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATAFILE_SYMBOL_in_create_tablespace_tail2923  */
static	ANTLR3_BITWORD FOLLOW_DATAFILE_SYMBOL_in_create_tablespace_tail2923_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATAFILE_SYMBOL_in_create_tablespace_tail2923	= { FOLLOW_DATAFILE_SYMBOL_in_create_tablespace_tail2923_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_create_tablespace_tail2925  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_create_tablespace_tail2925_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000081002), ANTLR3_UINT64_LIT(0x0000300000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_create_tablespace_tail2925	= { FOLLOW_string_literal_in_create_tablespace_tail2925_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USE_SYMBOL_in_create_tablespace_tail2930  */
static	ANTLR3_BITWORD FOLLOW_USE_SYMBOL_in_create_tablespace_tail2930_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USE_SYMBOL_in_create_tablespace_tail2930	= { FOLLOW_USE_SYMBOL_in_create_tablespace_tail2930_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGFILE_SYMBOL_in_create_tablespace_tail2932  */
static	ANTLR3_BITWORD FOLLOW_LOGFILE_SYMBOL_in_create_tablespace_tail2932_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGFILE_SYMBOL_in_create_tablespace_tail2932	= { FOLLOW_LOGFILE_SYMBOL_in_create_tablespace_tail2932_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_create_tablespace_tail2934  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_create_tablespace_tail2934_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_create_tablespace_tail2934	= { FOLLOW_GROUP_SYMBOL_in_create_tablespace_tail2934_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_ref_in_create_tablespace_tail2936  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_ref_in_create_tablespace_tail2936_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000081002), ANTLR3_UINT64_LIT(0x0000300000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_ref_in_create_tablespace_tail2936	= { FOLLOW_logfile_group_ref_in_create_tablespace_tail2936_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_options_in_create_tablespace_tail2940  */
static	ANTLR3_BITWORD FOLLOW_tablespace_options_in_create_tablespace_tail2940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_options_in_create_tablespace_tail2940	= { FOLLOW_tablespace_options_in_create_tablespace_tail2940_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_option_in_tablespace_options2951  */
static	ANTLR3_BITWORD FOLLOW_tablespace_option_in_tablespace_options2951_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000081002), ANTLR3_UINT64_LIT(0x0000300000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_option_in_tablespace_options2951	= { FOLLOW_tablespace_option_in_tablespace_options2951_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_tablespace_options2954  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_tablespace_options2954_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000081002), ANTLR3_UINT64_LIT(0x0000300000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_tablespace_options2954	= { FOLLOW_COMMA_SYMBOL_in_tablespace_options2954_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_option_in_tablespace_options2957  */
static	ANTLR3_BITWORD FOLLOW_tablespace_option_in_tablespace_options2957_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000002000081002), ANTLR3_UINT64_LIT(0x0000300000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_option_in_tablespace_options2957	= { FOLLOW_tablespace_option_in_tablespace_options2957_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INITIAL_SIZE_SYMBOL_in_tablespace_option2969  */
static	ANTLR3_BITWORD FOLLOW_INITIAL_SIZE_SYMBOL_in_tablespace_option2969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INITIAL_SIZE_SYMBOL_in_tablespace_option2969	= { FOLLOW_INITIAL_SIZE_SYMBOL_in_tablespace_option2969_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option2971  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option2971_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option2971	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option2971_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_tablespace_option2974  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_tablespace_option2974_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_tablespace_option2974	= { FOLLOW_size_number_in_tablespace_option2974_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_tablespace_option2979  */
static	ANTLR3_BITWORD FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_tablespace_option2979_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_tablespace_option2979	= { FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_tablespace_option2979_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option2981  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option2981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option2981	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option2981_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_tablespace_option2984  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_tablespace_option2984_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_tablespace_option2984	= { FOLLOW_size_number_in_tablespace_option2984_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_SIZE_SYMBOL_in_tablespace_option2989  */
static	ANTLR3_BITWORD FOLLOW_MAX_SIZE_SYMBOL_in_tablespace_option2989_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_SIZE_SYMBOL_in_tablespace_option2989	= { FOLLOW_MAX_SIZE_SYMBOL_in_tablespace_option2989_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option2991  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option2991_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option2991	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option2991_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_tablespace_option2994  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_tablespace_option2994_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_tablespace_option2994	= { FOLLOW_size_number_in_tablespace_option2994_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXTENT_SIZE_SYMBOL_in_tablespace_option2999  */
static	ANTLR3_BITWORD FOLLOW_EXTENT_SIZE_SYMBOL_in_tablespace_option2999_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXTENT_SIZE_SYMBOL_in_tablespace_option2999	= { FOLLOW_EXTENT_SIZE_SYMBOL_in_tablespace_option2999_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option3001  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option3001_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option3001	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option3001_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_tablespace_option3004  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_tablespace_option3004_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_tablespace_option3004	= { FOLLOW_size_number_in_tablespace_option3004_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NODEGROUP_SYMBOL_in_tablespace_option3009  */
static	ANTLR3_BITWORD FOLLOW_NODEGROUP_SYMBOL_in_tablespace_option3009_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800800000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NODEGROUP_SYMBOL_in_tablespace_option3009	= { FOLLOW_NODEGROUP_SYMBOL_in_tablespace_option3009_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option3011  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option3011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option3011	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option3011_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_tablespace_option3014  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_tablespace_option3014_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_tablespace_option3014	= { FOLLOW_real_ulong_number_in_tablespace_option3014_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_tablespace_option3019  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_tablespace_option3019_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_tablespace_option3019	= { FOLLOW_STORAGE_SYMBOL_in_tablespace_option3019_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_tablespace_option3022  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_tablespace_option3022_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_tablespace_option3022	= { FOLLOW_ENGINE_SYMBOL_in_tablespace_option3022_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option3024  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option3024_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option3024	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option3024_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_tablespace_option3027  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_tablespace_option3027_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_tablespace_option3027	= { FOLLOW_engine_ref_in_tablespace_option3027_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_tablespace_option3032  */
static	ANTLR3_BITWORD FOLLOW_set_in_tablespace_option3032_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_tablespace_option3032	= { FOLLOW_set_in_tablespace_option3032_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_tablespace_option3043  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_tablespace_option3043_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000900000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_tablespace_option3043	= { FOLLOW_COMMENT_SYMBOL_in_tablespace_option3043_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option3045  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option3045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option3045	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option3045_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_tablespace_option3048  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_tablespace_option3048_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_tablespace_option3048	= { FOLLOW_string_literal_in_tablespace_option3048_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FILE_BLOCK_SIZE_SYMBOL_in_tablespace_option3055  */
static	ANTLR3_BITWORD FOLLOW_FILE_BLOCK_SIZE_SYMBOL_in_tablespace_option3055_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000900000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FILE_BLOCK_SIZE_SYMBOL_in_tablespace_option3055	= { FOLLOW_FILE_BLOCK_SIZE_SYMBOL_in_tablespace_option3055_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_tablespace_option3057  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_tablespace_option3057_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_tablespace_option3057	= { FOLLOW_EQUAL_OPERATOR_in_tablespace_option3057_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_size_number_in_tablespace_option3060  */
static	ANTLR3_BITWORD FOLLOW_size_number_in_tablespace_option3060_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_size_number_in_tablespace_option3060	= { FOLLOW_size_number_in_tablespace_option3060_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_with_definer_in_create_trigger3070  */
static	ANTLR3_BITWORD FOLLOW_create_with_definer_in_create_trigger3070_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_create_with_definer_in_create_trigger3070	= { FOLLOW_create_with_definer_in_create_trigger3070_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_trigger_tail_in_create_trigger3072  */
static	ANTLR3_BITWORD FOLLOW_create_trigger_tail_in_create_trigger3072_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_trigger_tail_in_create_trigger3072	= { FOLLOW_create_trigger_tail_in_create_trigger3072_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_trigger3074  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_trigger3074_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_trigger3074	= { FOLLOW_SEMICOLON_SYMBOL_in_create_trigger3074_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_trigger3077  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_trigger3077_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_trigger3077	= { FOLLOW_EOF_in_create_trigger3077_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIGGER_SYMBOL_in_create_trigger_tail3086  */
static	ANTLR3_BITWORD FOLLOW_TRIGGER_SYMBOL_in_create_trigger_tail3086_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIGGER_SYMBOL_in_create_trigger_tail3086	= { FOLLOW_TRIGGER_SYMBOL_in_create_trigger_tail3086_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_trigger_name_in_create_trigger_tail3088  */
static	ANTLR3_BITWORD FOLLOW_trigger_name_in_create_trigger_tail3088_bits[]	= { ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_trigger_name_in_create_trigger_tail3088	= { FOLLOW_trigger_name_in_create_trigger_tail3088_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_trigger_tail3090  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_trigger_tail3090_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000001000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_trigger_tail3090	= { FOLLOW_set_in_create_trigger_tail3090_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_trigger_tail3098  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_trigger_tail3098_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_trigger_tail3098	= { FOLLOW_set_in_create_trigger_tail3098_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_create_trigger_tail3112  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_create_trigger_tail3112_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_create_trigger_tail3112	= { FOLLOW_ON_SYMBOL_in_create_trigger_tail3112_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_create_trigger_tail3114  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_create_trigger_tail3114_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_create_trigger_tail3114	= { FOLLOW_table_ref_in_create_trigger_tail3114_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_create_trigger_tail3116  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_create_trigger_tail3116_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_create_trigger_tail3116	= { FOLLOW_FOR_SYMBOL_in_create_trigger_tail3116_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EACH_SYMBOL_in_create_trigger_tail3118  */
static	ANTLR3_BITWORD FOLLOW_EACH_SYMBOL_in_create_trigger_tail3118_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EACH_SYMBOL_in_create_trigger_tail3118	= { FOLLOW_EACH_SYMBOL_in_create_trigger_tail3118_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROW_SYMBOL_in_create_trigger_tail3120  */
static	ANTLR3_BITWORD FOLLOW_ROW_SYMBOL_in_create_trigger_tail3120_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F689676B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROW_SYMBOL_in_create_trigger_tail3120	= { FOLLOW_ROW_SYMBOL_in_create_trigger_tail3120_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_trigger_follows_precedes_clause_in_create_trigger_tail3122  */
static	ANTLR3_BITWORD FOLLOW_trigger_follows_precedes_clause_in_create_trigger_tail3122_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_trigger_follows_precedes_clause_in_create_trigger_tail3122	= { FOLLOW_trigger_follows_precedes_clause_in_create_trigger_tail3122_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_create_trigger_tail3125  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_create_trigger_tail3125_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_create_trigger_tail3125	= { FOLLOW_compound_statement_in_create_trigger_tail3125_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_trigger_follows_precedes_clause3136  */
static	ANTLR3_BITWORD FOLLOW_set_in_trigger_follows_precedes_clause3136_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_trigger_follows_precedes_clause3136	= { FOLLOW_set_in_trigger_follows_precedes_clause3136_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_trigger_follows_precedes_clause3144  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_trigger_follows_precedes_clause3144_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_trigger_follows_precedes_clause3144	= { FOLLOW_text_or_identifier_in_trigger_follows_precedes_clause3144_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_view3156  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_view3156_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_view3156	= { FOLLOW_CREATE_SYMBOL_in_create_view3156_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OR_SYMBOL_in_create_view3159  */
static	ANTLR3_BITWORD FOLLOW_OR_SYMBOL_in_create_view3159_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_OR_SYMBOL_in_create_view3159	= { FOLLOW_OR_SYMBOL_in_create_view3159_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLACE_SYMBOL_in_create_view3161  */
static	ANTLR3_BITWORD FOLLOW_REPLACE_SYMBOL_in_create_view3161_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLACE_SYMBOL_in_create_view3161	= { FOLLOW_REPLACE_SYMBOL_in_create_view3161_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_algorithm_in_create_view3165  */
static	ANTLR3_BITWORD FOLLOW_view_algorithm_in_create_view3165_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_view_algorithm_in_create_view3165	= { FOLLOW_view_algorithm_in_create_view3165_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_definer_clause_in_create_view3169  */
static	ANTLR3_BITWORD FOLLOW_definer_clause_in_create_view3169_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_definer_clause_in_create_view3169	= { FOLLOW_definer_clause_in_create_view3169_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_view_tail_in_create_view3172  */
static	ANTLR3_BITWORD FOLLOW_create_view_tail_in_create_view3172_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_view_tail_in_create_view3172	= { FOLLOW_create_view_tail_in_create_view3172_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_create_view3174  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_create_view3174_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_create_view3174	= { FOLLOW_SEMICOLON_SYMBOL_in_create_view3174_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_create_view3177  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_create_view3177_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_create_view3177	= { FOLLOW_EOF_in_create_view3177_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_SYMBOL_in_create_view_tail3187  */
static	ANTLR3_BITWORD FOLLOW_SQL_SYMBOL_in_create_view_tail3187_bits[]	= { ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_SYMBOL_in_create_view_tail3187	= { FOLLOW_SQL_SYMBOL_in_create_view_tail3187_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SECURITY_SYMBOL_in_create_view_tail3189  */
static	ANTLR3_BITWORD FOLLOW_SECURITY_SYMBOL_in_create_view_tail3189_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SECURITY_SYMBOL_in_create_view_tail3189	= { FOLLOW_SECURITY_SYMBOL_in_create_view_tail3189_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_view_tail3191  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_view_tail3191_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_view_tail3191	= { FOLLOW_set_in_create_view_tail3191_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_create_view_tail3203  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_create_view_tail3203_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_create_view_tail3203	= { FOLLOW_VIEW_SYMBOL_in_create_view_tail3203_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_name_in_create_view_tail3205  */
static	ANTLR3_BITWORD FOLLOW_view_name_in_create_view_tail3205_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_view_name_in_create_view_tail3205	= { FOLLOW_view_name_in_create_view_tail3205_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_create_view_tail3207  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_create_view_tail3207_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_create_view_tail3207	= { FOLLOW_identifier_list_with_parentheses_in_create_view_tail3207_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_create_view_tail3212  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_create_view_tail3212_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_create_view_tail3212	= { FOLLOW_AS_SYMBOL_in_create_view_tail3212_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_create_view_tail3214  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_create_view_tail3214_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_create_view_tail3214	= { FOLLOW_select_statement_in_create_view_tail3214_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_create_view_tail3219  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_create_view_tail3219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_create_view_tail3219	= { FOLLOW_WITH_SYMBOL_in_create_view_tail3219_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_create_view_tail3230  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_create_view_tail3230_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_create_view_tail3230	= { FOLLOW_CHECK_SYMBOL_in_create_view_tail3230_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_create_view_tail3232  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_create_view_tail3232_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_create_view_tail3232	= { FOLLOW_OPTION_SYMBOL_in_create_view_tail3232_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OR_SYMBOL_in_view_replace_or_algorithm3243  */
static	ANTLR3_BITWORD FOLLOW_OR_SYMBOL_in_view_replace_or_algorithm3243_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_OR_SYMBOL_in_view_replace_or_algorithm3243	= { FOLLOW_OR_SYMBOL_in_view_replace_or_algorithm3243_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLACE_SYMBOL_in_view_replace_or_algorithm3245  */
static	ANTLR3_BITWORD FOLLOW_REPLACE_SYMBOL_in_view_replace_or_algorithm3245_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLACE_SYMBOL_in_view_replace_or_algorithm3245	= { FOLLOW_REPLACE_SYMBOL_in_view_replace_or_algorithm3245_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_algorithm_in_view_replace_or_algorithm3247  */
static	ANTLR3_BITWORD FOLLOW_view_algorithm_in_view_replace_or_algorithm3247_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_view_algorithm_in_view_replace_or_algorithm3247	= { FOLLOW_view_algorithm_in_view_replace_or_algorithm3247_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_algorithm_in_view_replace_or_algorithm3253  */
static	ANTLR3_BITWORD FOLLOW_view_algorithm_in_view_replace_or_algorithm3253_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_view_algorithm_in_view_replace_or_algorithm3253	= { FOLLOW_view_algorithm_in_view_replace_or_algorithm3253_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALGORITHM_SYMBOL_in_view_algorithm3262  */
static	ANTLR3_BITWORD FOLLOW_ALGORITHM_SYMBOL_in_view_algorithm3262_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALGORITHM_SYMBOL_in_view_algorithm3262	= { FOLLOW_ALGORITHM_SYMBOL_in_view_algorithm3262_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_view_algorithm3264  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_view_algorithm3264_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_view_algorithm3264	= { FOLLOW_EQUAL_OPERATOR_in_view_algorithm3264_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_view_algorithm3266  */
static	ANTLR3_BITWORD FOLLOW_set_in_view_algorithm3266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_view_algorithm3266	= { FOLLOW_set_in_view_algorithm3266_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_drop_statement3287  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_drop_statement3287_bits[]	= { ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400800680000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x2000060000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000101) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_drop_statement3287	= { FOLLOW_DROP_SYMBOL_in_drop_statement3287_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_drop_statement3294  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_drop_statement3294_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_drop_statement3294	= { FOLLOW_DATABASE_SYMBOL_in_drop_statement3294_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3296  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3296_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3296	= { FOLLOW_if_exists_in_drop_statement3296_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_drop_statement3299  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_drop_statement3299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_drop_statement3299	= { FOLLOW_schema_ref_in_drop_statement3299_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENT_SYMBOL_in_drop_statement3307  */
static	ANTLR3_BITWORD FOLLOW_EVENT_SYMBOL_in_drop_statement3307_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENT_SYMBOL_in_drop_statement3307	= { FOLLOW_EVENT_SYMBOL_in_drop_statement3307_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3309  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3309_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3309	= { FOLLOW_if_exists_in_drop_statement3309_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_event_ref_in_drop_statement3312  */
static	ANTLR3_BITWORD FOLLOW_event_ref_in_drop_statement3312_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_event_ref_in_drop_statement3312	= { FOLLOW_event_ref_in_drop_statement3312_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_drop_statement3327  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_drop_statement3327_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_drop_statement3327	= { FOLLOW_FUNCTION_SYMBOL_in_drop_statement3327_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3329  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3329_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3329	= { FOLLOW_if_exists_in_drop_statement3329_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_ref_in_drop_statement3332  */
static	ANTLR3_BITWORD FOLLOW_function_ref_in_drop_statement3332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_ref_in_drop_statement3332	= { FOLLOW_function_ref_in_drop_statement3332_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_drop_statement3341  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_drop_statement3341_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_drop_statement3341	= { FOLLOW_PROCEDURE_SYMBOL_in_drop_statement3341_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3343  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3343_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3343	= { FOLLOW_if_exists_in_drop_statement3343_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_ref_in_drop_statement3346  */
static	ANTLR3_BITWORD FOLLOW_procedure_ref_in_drop_statement3346_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_ref_in_drop_statement3346	= { FOLLOW_procedure_ref_in_drop_statement3346_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_online_option_in_drop_statement3357  */
static	ANTLR3_BITWORD FOLLOW_online_option_in_drop_statement3357_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_online_option_in_drop_statement3357	= { FOLLOW_online_option_in_drop_statement3357_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_drop_statement3360  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_drop_statement3360_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_drop_statement3360	= { FOLLOW_INDEX_SYMBOL_in_drop_statement3360_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_ref_in_drop_statement3362  */
static	ANTLR3_BITWORD FOLLOW_index_ref_in_drop_statement3362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_ref_in_drop_statement3362	= { FOLLOW_index_ref_in_drop_statement3362_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_drop_statement3364  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_drop_statement3364_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_drop_statement3364	= { FOLLOW_ON_SYMBOL_in_drop_statement3364_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_drop_statement3366  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_drop_statement3366_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_drop_statement3366	= { FOLLOW_table_ref_in_drop_statement3366_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_lock_algorithm_in_drop_statement3368  */
static	ANTLR3_BITWORD FOLLOW_index_lock_algorithm_in_drop_statement3368_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_lock_algorithm_in_drop_statement3368	= { FOLLOW_index_lock_algorithm_in_drop_statement3368_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGFILE_SYMBOL_in_drop_statement3375  */
static	ANTLR3_BITWORD FOLLOW_LOGFILE_SYMBOL_in_drop_statement3375_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGFILE_SYMBOL_in_drop_statement3375	= { FOLLOW_LOGFILE_SYMBOL_in_drop_statement3375_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_drop_statement3377  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_drop_statement3377_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_drop_statement3377	= { FOLLOW_GROUP_SYMBOL_in_drop_statement3377_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logfile_group_ref_in_drop_statement3379  */
static	ANTLR3_BITWORD FOLLOW_logfile_group_ref_in_drop_statement3379_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_logfile_group_ref_in_drop_statement3379	= { FOLLOW_logfile_group_ref_in_drop_statement3379_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_logfile_group_option_in_drop_statement3382  */
static	ANTLR3_BITWORD FOLLOW_drop_logfile_group_option_in_drop_statement3382_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_logfile_group_option_in_drop_statement3382	= { FOLLOW_drop_logfile_group_option_in_drop_statement3382_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_drop_statement3385  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_drop_statement3385_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_drop_statement3385	= { FOLLOW_COMMA_SYMBOL_in_drop_statement3385_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_logfile_group_option_in_drop_statement3388  */
static	ANTLR3_BITWORD FOLLOW_drop_logfile_group_option_in_drop_statement3388_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_logfile_group_option_in_drop_statement3388	= { FOLLOW_drop_logfile_group_option_in_drop_statement3388_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERVER_SYMBOL_in_drop_statement3398  */
static	ANTLR3_BITWORD FOLLOW_SERVER_SYMBOL_in_drop_statement3398_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SERVER_SYMBOL_in_drop_statement3398	= { FOLLOW_SERVER_SYMBOL_in_drop_statement3398_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3400  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3400_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3400	= { FOLLOW_if_exists_in_drop_statement3400_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_ref_in_drop_statement3403  */
static	ANTLR3_BITWORD FOLLOW_server_ref_in_drop_statement3403_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_server_ref_in_drop_statement3403	= { FOLLOW_server_ref_in_drop_statement3403_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TEMPORARY_SYMBOL_in_drop_statement3409  */
static	ANTLR3_BITWORD FOLLOW_TEMPORARY_SYMBOL_in_drop_statement3409_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_TEMPORARY_SYMBOL_in_drop_statement3409	= { FOLLOW_TEMPORARY_SYMBOL_in_drop_statement3409_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_drop_statement3412  */
static	ANTLR3_BITWORD FOLLOW_set_in_drop_statement3412_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_drop_statement3412	= { FOLLOW_set_in_drop_statement3412_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3420  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3420_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3420	= { FOLLOW_if_exists_in_drop_statement3420_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_drop_statement3423  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_drop_statement3423_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_drop_statement3423	= { FOLLOW_table_ref_list_in_drop_statement3423_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_drop_statement3438  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_drop_statement3438_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_drop_statement3438	= { FOLLOW_TABLESPACE_SYMBOL_in_drop_statement3438_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_tablespace_ref_in_drop_statement3440  */
static	ANTLR3_BITWORD FOLLOW_tablespace_ref_in_drop_statement3440_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_tablespace_ref_in_drop_statement3440	= { FOLLOW_tablespace_ref_in_drop_statement3440_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_logfile_group_option_in_drop_statement3443  */
static	ANTLR3_BITWORD FOLLOW_drop_logfile_group_option_in_drop_statement3443_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_logfile_group_option_in_drop_statement3443	= { FOLLOW_drop_logfile_group_option_in_drop_statement3443_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_drop_statement3446  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_drop_statement3446_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_drop_statement3446	= { FOLLOW_COMMA_SYMBOL_in_drop_statement3446_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_logfile_group_option_in_drop_statement3449  */
static	ANTLR3_BITWORD FOLLOW_drop_logfile_group_option_in_drop_statement3449_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_logfile_group_option_in_drop_statement3449	= { FOLLOW_drop_logfile_group_option_in_drop_statement3449_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIGGER_SYMBOL_in_drop_statement3459  */
static	ANTLR3_BITWORD FOLLOW_TRIGGER_SYMBOL_in_drop_statement3459_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIGGER_SYMBOL_in_drop_statement3459	= { FOLLOW_TRIGGER_SYMBOL_in_drop_statement3459_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3461  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3461_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3461	= { FOLLOW_if_exists_in_drop_statement3461_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_trigger_ref_in_drop_statement3464  */
static	ANTLR3_BITWORD FOLLOW_trigger_ref_in_drop_statement3464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_trigger_ref_in_drop_statement3464	= { FOLLOW_trigger_ref_in_drop_statement3464_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_drop_statement3470  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_drop_statement3470_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_drop_statement3470	= { FOLLOW_VIEW_SYMBOL_in_drop_statement3470_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_statement3472  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_statement3472_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_statement3472	= { FOLLOW_if_exists_in_drop_statement3472_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_identifier_list_in_drop_statement3475  */
static	ANTLR3_BITWORD FOLLOW_view_identifier_list_in_drop_statement3475_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000) };
static  ANTLR3_BITSET_LIST FOLLOW_view_identifier_list_in_drop_statement3475	= { FOLLOW_view_identifier_list_in_drop_statement3475_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_drop_logfile_group_option3496  */
static	ANTLR3_BITWORD FOLLOW_set_in_drop_logfile_group_option3496_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_drop_logfile_group_option3496	= { FOLLOW_set_in_drop_logfile_group_option3496_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_drop_logfile_group_option3507  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_drop_logfile_group_option3507_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_drop_logfile_group_option3507	= { FOLLOW_STORAGE_SYMBOL_in_drop_logfile_group_option3507_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_drop_logfile_group_option3510  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_drop_logfile_group_option3510_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_drop_logfile_group_option3510	= { FOLLOW_ENGINE_SYMBOL_in_drop_logfile_group_option3510_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_drop_logfile_group_option3512  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_drop_logfile_group_option3512_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_drop_logfile_group_option3512	= { FOLLOW_EQUAL_OPERATOR_in_drop_logfile_group_option3512_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_drop_logfile_group_option3515  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_drop_logfile_group_option3515_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_drop_logfile_group_option3515	= { FOLLOW_engine_ref_in_drop_logfile_group_option3515_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RENAME_SYMBOL_in_rename_table_statement3525  */
static	ANTLR3_BITWORD FOLLOW_RENAME_SYMBOL_in_rename_table_statement3525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_RENAME_SYMBOL_in_rename_table_statement3525	= { FOLLOW_RENAME_SYMBOL_in_rename_table_statement3525_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_rename_table_statement3527  */
static	ANTLR3_BITWORD FOLLOW_set_in_rename_table_statement3527_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_rename_table_statement3527	= { FOLLOW_set_in_rename_table_statement3527_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_rename_table_statement3537  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_rename_table_statement3537_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_rename_table_statement3537	= { FOLLOW_table_ref_in_rename_table_statement3537_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_rename_table_statement3539  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_rename_table_statement3539_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_rename_table_statement3539	= { FOLLOW_TO_SYMBOL_in_rename_table_statement3539_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_in_rename_table_statement3541  */
static	ANTLR3_BITWORD FOLLOW_table_name_in_rename_table_statement3541_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_in_rename_table_statement3541	= { FOLLOW_table_name_in_rename_table_statement3541_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_rename_table_statement3544  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_rename_table_statement3544_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_rename_table_statement3544	= { FOLLOW_COMMA_SYMBOL_in_rename_table_statement3544_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_rename_table_statement3546  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_rename_table_statement3546_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_rename_table_statement3546	= { FOLLOW_table_ref_in_rename_table_statement3546_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_rename_table_statement3548  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_rename_table_statement3548_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_rename_table_statement3548	= { FOLLOW_TO_SYMBOL_in_rename_table_statement3548_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_in_rename_table_statement3550  */
static	ANTLR3_BITWORD FOLLOW_table_name_in_rename_table_statement3550_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_in_rename_table_statement3550	= { FOLLOW_table_name_in_rename_table_statement3550_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRUNCATE_SYMBOL_in_truncate_table_statement3564  */
static	ANTLR3_BITWORD FOLLOW_TRUNCATE_SYMBOL_in_truncate_table_statement3564_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRUNCATE_SYMBOL_in_truncate_table_statement3564	= { FOLLOW_TRUNCATE_SYMBOL_in_truncate_table_statement3564_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_truncate_table_statement3566  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_truncate_table_statement3566_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_truncate_table_statement3566	= { FOLLOW_TABLE_SYMBOL_in_truncate_table_statement3566_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_truncate_table_statement3569  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_truncate_table_statement3569_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_truncate_table_statement3569	= { FOLLOW_table_ref_in_truncate_table_statement3569_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CALL_SYMBOL_in_call_statement3580  */
static	ANTLR3_BITWORD FOLLOW_CALL_SYMBOL_in_call_statement3580_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_CALL_SYMBOL_in_call_statement3580	= { FOLLOW_CALL_SYMBOL_in_call_statement3580_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_ref_in_call_statement3582  */
static	ANTLR3_BITWORD FOLLOW_procedure_ref_in_call_statement3582_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_ref_in_call_statement3582	= { FOLLOW_procedure_ref_in_call_statement3582_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_optional_expression_list_with_parentheses_in_call_statement3584  */
static	ANTLR3_BITWORD FOLLOW_optional_expression_list_with_parentheses_in_call_statement3584_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_optional_expression_list_with_parentheses_in_call_statement3584	= { FOLLOW_optional_expression_list_with_parentheses_in_call_statement3584_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DELETE_SYMBOL_in_delete_statement3594  */
static	ANTLR3_BITWORD FOLLOW_DELETE_SYMBOL_in_delete_statement3594_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040020000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DELETE_SYMBOL_in_delete_statement3594	= { FOLLOW_DELETE_SYMBOL_in_delete_statement3594_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_delete_option_in_delete_statement3610  */
static	ANTLR3_BITWORD FOLLOW_delete_option_in_delete_statement3610_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040020000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_delete_option_in_delete_statement3610	= { FOLLOW_delete_option_in_delete_statement3610_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_delete_statement3621  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_delete_statement3621_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_delete_statement3621	= { FOLLOW_FROM_SYMBOL_in_delete_statement3621_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_with_wildcard_in_delete_statement3652  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_with_wildcard_in_delete_statement3652_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_with_wildcard_in_delete_statement3652	= { FOLLOW_table_ref_list_with_wildcard_in_delete_statement3652_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USING_SYMBOL_in_delete_statement3654  */
static	ANTLR3_BITWORD FOLLOW_USING_SYMBOL_in_delete_statement3654_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_USING_SYMBOL_in_delete_statement3654	= { FOLLOW_USING_SYMBOL_in_delete_statement3654_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_table_list_in_delete_statement3656  */
static	ANTLR3_BITWORD FOLLOW_join_table_list_in_delete_statement3656_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_join_table_list_in_delete_statement3656	= { FOLLOW_join_table_list_in_delete_statement3656_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_delete_statement3658  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_delete_statement3658_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_delete_statement3658	= { FOLLOW_where_clause_in_delete_statement3658_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_delete_statement3669  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_delete_statement3669_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_delete_statement3669	= { FOLLOW_table_ref_in_delete_statement3669_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_delete_in_delete_statement3671  */
static	ANTLR3_BITWORD FOLLOW_partition_delete_in_delete_statement3671_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_delete_in_delete_statement3671	= { FOLLOW_partition_delete_in_delete_statement3671_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_delete_statement3674  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_delete_statement3674_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_delete_statement3674	= { FOLLOW_where_clause_in_delete_statement3674_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_delete_statement3677  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_delete_statement3677_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_delete_statement3677	= { FOLLOW_order_by_clause_in_delete_statement3677_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_simple_limit_clause_in_delete_statement3680  */
static	ANTLR3_BITWORD FOLLOW_simple_limit_clause_in_delete_statement3680_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_simple_limit_clause_in_delete_statement3680	= { FOLLOW_simple_limit_clause_in_delete_statement3680_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_with_wildcard_in_delete_statement3696  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_with_wildcard_in_delete_statement3696_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_with_wildcard_in_delete_statement3696	= { FOLLOW_table_ref_list_with_wildcard_in_delete_statement3696_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_delete_statement3698  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_delete_statement3698_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_delete_statement3698	= { FOLLOW_FROM_SYMBOL_in_delete_statement3698_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_table_list_in_delete_statement3700  */
static	ANTLR3_BITWORD FOLLOW_join_table_list_in_delete_statement3700_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_join_table_list_in_delete_statement3700	= { FOLLOW_join_table_list_in_delete_statement3700_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_delete_statement3702  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_delete_statement3702_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_delete_statement3702	= { FOLLOW_where_clause_in_delete_statement3702_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_partition_delete3721  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_partition_delete3721_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_partition_delete3721	= { FOLLOW_PARTITION_SYMBOL_in_partition_delete3721_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partition_delete3723  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partition_delete3723_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partition_delete3723	= { FOLLOW_OPEN_PAR_SYMBOL_in_partition_delete3723_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_partition_delete3725  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_partition_delete3725_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_partition_delete3725	= { FOLLOW_identifier_list_in_partition_delete3725_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partition_delete3727  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partition_delete3727_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partition_delete3727	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partition_delete3727_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DO_SYMBOL_in_do_statement3753  */
static	ANTLR3_BITWORD FOLLOW_DO_SYMBOL_in_do_statement3753_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DO_SYMBOL_in_do_statement3753	= { FOLLOW_DO_SYMBOL_in_do_statement3753_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_do_statement3765  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_do_statement3765_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_do_statement3765	= { FOLLOW_expression_list_in_do_statement3765_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_list_in_do_statement3775  */
static	ANTLR3_BITWORD FOLLOW_select_item_list_in_do_statement3775_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_list_in_do_statement3775	= { FOLLOW_select_item_list_in_do_statement3775_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HANDLER_SYMBOL_in_handler_statement3787  */
static	ANTLR3_BITWORD FOLLOW_HANDLER_SYMBOL_in_handler_statement3787_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_HANDLER_SYMBOL_in_handler_statement3787	= { FOLLOW_HANDLER_SYMBOL_in_handler_statement3787_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_handler_statement3807  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_handler_statement3807_bits[]	= { ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_handler_statement3807	= { FOLLOW_table_ref_in_handler_statement3807_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_SYMBOL_in_handler_statement3809  */
static	ANTLR3_BITWORD FOLLOW_OPEN_SYMBOL_in_handler_statement3809_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010800000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_SYMBOL_in_handler_statement3809	= { FOLLOW_OPEN_SYMBOL_in_handler_statement3809_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_handler_statement3812  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_handler_statement3812_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_handler_statement3812	= { FOLLOW_AS_SYMBOL_in_handler_statement3812_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_handler_statement3815  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_handler_statement3815_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_handler_statement3815	= { FOLLOW_identifier_in_handler_statement3815_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_no_db_in_handler_statement3823  */
static	ANTLR3_BITWORD FOLLOW_table_ref_no_db_in_handler_statement3823_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000002002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_no_db_in_handler_statement3823	= { FOLLOW_table_ref_no_db_in_handler_statement3823_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_SYMBOL_in_handler_statement3836  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_SYMBOL_in_handler_statement3836_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_SYMBOL_in_handler_statement3836	= { FOLLOW_CLOSE_SYMBOL_in_handler_statement3836_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_handler_statement3844  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_handler_statement3844_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_handler_statement3844	= { FOLLOW_READ_SYMBOL_in_handler_statement3844_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_handler_read_or_scan_in_handler_statement3846  */
static	ANTLR3_BITWORD FOLLOW_handler_read_or_scan_in_handler_statement3846_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_handler_read_or_scan_in_handler_statement3846	= { FOLLOW_handler_read_or_scan_in_handler_statement3846_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_handler_statement3848  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_handler_statement3848_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_handler_statement3848	= { FOLLOW_where_clause_in_handler_statement3848_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_handler_statement3851  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_handler_statement3851_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_handler_statement3851	= { FOLLOW_limit_clause_in_handler_statement3851_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_handler_read_or_scan3869  */
static	ANTLR3_BITWORD FOLLOW_set_in_handler_read_or_scan3869_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_handler_read_or_scan3869	= { FOLLOW_set_in_handler_read_or_scan3869_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_handler_read_or_scan3881  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_handler_read_or_scan3881_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000200000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x0000000000000004), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001800300800000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_handler_read_or_scan3881	= { FOLLOW_identifier_in_handler_read_or_scan3881_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FIRST_SYMBOL_in_handler_read_or_scan3890  */
static	ANTLR3_BITWORD FOLLOW_FIRST_SYMBOL_in_handler_read_or_scan3890_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FIRST_SYMBOL_in_handler_read_or_scan3890	= { FOLLOW_FIRST_SYMBOL_in_handler_read_or_scan3890_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NEXT_SYMBOL_in_handler_read_or_scan3897  */
static	ANTLR3_BITWORD FOLLOW_NEXT_SYMBOL_in_handler_read_or_scan3897_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NEXT_SYMBOL_in_handler_read_or_scan3897	= { FOLLOW_NEXT_SYMBOL_in_handler_read_or_scan3897_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PREV_SYMBOL_in_handler_read_or_scan3904  */
static	ANTLR3_BITWORD FOLLOW_PREV_SYMBOL_in_handler_read_or_scan3904_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PREV_SYMBOL_in_handler_read_or_scan3904	= { FOLLOW_PREV_SYMBOL_in_handler_read_or_scan3904_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LAST_SYMBOL_in_handler_read_or_scan3911  */
static	ANTLR3_BITWORD FOLLOW_LAST_SYMBOL_in_handler_read_or_scan3911_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LAST_SYMBOL_in_handler_read_or_scan3911	= { FOLLOW_LAST_SYMBOL_in_handler_read_or_scan3911_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_handler_read_or_scan3918  */
static	ANTLR3_BITWORD FOLLOW_set_in_handler_read_or_scan3918_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_handler_read_or_scan3918	= { FOLLOW_set_in_handler_read_or_scan3918_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_handler_read_or_scan3942  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_handler_read_or_scan3942_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_handler_read_or_scan3942	= { FOLLOW_OPEN_PAR_SYMBOL_in_handler_read_or_scan3942_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_values_in_handler_read_or_scan3944  */
static	ANTLR3_BITWORD FOLLOW_values_in_handler_read_or_scan3944_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_values_in_handler_read_or_scan3944	= { FOLLOW_values_in_handler_read_or_scan3944_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_handler_read_or_scan3946  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_handler_read_or_scan3946_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_handler_read_or_scan3946	= { FOLLOW_CLOSE_PAR_SYMBOL_in_handler_read_or_scan3946_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INSERT_SYMBOL_in_insert_statement3961  */
static	ANTLR3_BITWORD FOLLOW_INSERT_SYMBOL_in_insert_statement3961_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000100042000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_INSERT_SYMBOL_in_insert_statement3961	= { FOLLOW_INSERT_SYMBOL_in_insert_statement3961_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_lock_option_in_insert_statement3963  */
static	ANTLR3_BITWORD FOLLOW_insert_lock_option_in_insert_statement3963_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_lock_option_in_insert_statement3963	= { FOLLOW_insert_lock_option_in_insert_statement3963_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SYMBOL_in_insert_statement3966  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SYMBOL_in_insert_statement3966_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SYMBOL_in_insert_statement3966	= { FOLLOW_IGNORE_SYMBOL_in_insert_statement3966_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_insert_statement3969  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_insert_statement3969_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_insert_statement3969	= { FOLLOW_INTO_SYMBOL_in_insert_statement3969_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_insert_statement3972  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_insert_statement3972_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004100000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_insert_statement3972	= { FOLLOW_table_ref_in_insert_statement3972_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_use_partition_in_insert_statement3974  */
static	ANTLR3_BITWORD FOLLOW_use_partition_in_insert_statement3974_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004100000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_use_partition_in_insert_statement3974	= { FOLLOW_use_partition_in_insert_statement3974_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_field_spec_in_insert_statement3979  */
static	ANTLR3_BITWORD FOLLOW_insert_field_spec_in_insert_statement3979_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_field_spec_in_insert_statement3979	= { FOLLOW_insert_field_spec_in_insert_statement3979_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_duplicate_key_update_in_insert_statement3981  */
static	ANTLR3_BITWORD FOLLOW_duplicate_key_update_in_insert_statement3981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_duplicate_key_update_in_insert_statement3981	= { FOLLOW_duplicate_key_update_in_insert_statement3981_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_insert_field_spec4013  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_insert_field_spec4013_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_insert_field_spec4013	= { FOLLOW_OPEN_PAR_SYMBOL_in_insert_field_spec4013_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_fields_in_insert_field_spec4015  */
static	ANTLR3_BITWORD FOLLOW_fields_in_insert_field_spec4015_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_fields_in_insert_field_spec4015	= { FOLLOW_fields_in_insert_field_spec4015_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_insert_field_spec4018  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_insert_field_spec4018_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_insert_field_spec4018	= { FOLLOW_CLOSE_PAR_SYMBOL_in_insert_field_spec4018_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_values_in_insert_field_spec4030  */
static	ANTLR3_BITWORD FOLLOW_insert_values_in_insert_field_spec4030_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_values_in_insert_field_spec4030	= { FOLLOW_insert_values_in_insert_field_spec4030_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_query_expression_in_insert_field_spec4037  */
static	ANTLR3_BITWORD FOLLOW_insert_query_expression_in_insert_field_spec4037_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_query_expression_in_insert_field_spec4037	= { FOLLOW_insert_query_expression_in_insert_field_spec4037_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_insert_field_spec4046  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_insert_field_spec4046_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_insert_field_spec4046	= { FOLLOW_SET_SYMBOL_in_insert_field_spec4046_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_list_with_default_in_insert_field_spec4048  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_list_with_default_in_insert_field_spec4048_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_list_with_default_in_insert_field_spec4048	= { FOLLOW_column_assignment_list_with_default_in_insert_field_spec4048_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_with_wildcard_in_fields4057  */
static	ANTLR3_BITWORD FOLLOW_column_ref_with_wildcard_in_fields4057_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_with_wildcard_in_fields4057	= { FOLLOW_column_ref_with_wildcard_in_fields4057_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_fields4060  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_fields4060_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_fields4060	= { FOLLOW_COMMA_SYMBOL_in_fields4060_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_with_wildcard_in_fields4062  */
static	ANTLR3_BITWORD FOLLOW_column_ref_with_wildcard_in_fields4062_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_with_wildcard_in_fields4062	= { FOLLOW_column_ref_with_wildcard_in_fields4062_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_insert_values4073  */
static	ANTLR3_BITWORD FOLLOW_set_in_insert_values4073_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_insert_values4073	= { FOLLOW_set_in_insert_values4073_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_value_list_in_insert_values4081  */
static	ANTLR3_BITWORD FOLLOW_insert_value_list_in_insert_values4081_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_value_list_in_insert_values4081	= { FOLLOW_insert_value_list_in_insert_values4081_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_select_in_insert_query_expression4090  */
static	ANTLR3_BITWORD FOLLOW_create_select_in_insert_query_expression4090_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_select_in_insert_query_expression4090	= { FOLLOW_create_select_in_insert_query_expression4090_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_clause_in_insert_query_expression4092  */
static	ANTLR3_BITWORD FOLLOW_union_clause_in_insert_query_expression4092_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_clause_in_insert_query_expression4092	= { FOLLOW_union_clause_in_insert_query_expression4092_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_insert_query_expression4098  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_insert_query_expression4098_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_insert_query_expression4098	= { FOLLOW_OPEN_PAR_SYMBOL_in_insert_query_expression4098_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_select_in_insert_query_expression4100  */
static	ANTLR3_BITWORD FOLLOW_create_select_in_insert_query_expression4100_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_select_in_insert_query_expression4100	= { FOLLOW_create_select_in_insert_query_expression4100_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_insert_query_expression4102  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_insert_query_expression4102_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_insert_query_expression4102	= { FOLLOW_CLOSE_PAR_SYMBOL_in_insert_query_expression4102_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_or_order_by_or_limit_in_insert_query_expression4104  */
static	ANTLR3_BITWORD FOLLOW_union_or_order_by_or_limit_in_insert_query_expression4104_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_or_order_by_or_limit_in_insert_query_expression4104	= { FOLLOW_union_or_order_by_or_limit_in_insert_query_expression4104_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4114  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4114_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x9CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4114	= { FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4114_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_values_in_insert_value_list4116  */
static	ANTLR3_BITWORD FOLLOW_values_in_insert_value_list4116_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_values_in_insert_value_list4116	= { FOLLOW_values_in_insert_value_list4116_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4119  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4119_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4119	= { FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4119_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_insert_value_list4122  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_insert_value_list4122_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_insert_value_list4122	= { FOLLOW_COMMA_SYMBOL_in_insert_value_list4122_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4124  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4124_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x9CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4124	= { FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4124_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_values_in_insert_value_list4126  */
static	ANTLR3_BITWORD FOLLOW_values_in_insert_value_list4126_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_values_in_insert_value_list4126	= { FOLLOW_values_in_insert_value_list4126_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4129  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4129_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4129	= { FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4129_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_values4141  */
static	ANTLR3_BITWORD FOLLOW_expression_in_values4141_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_values4141	= { FOLLOW_expression_in_values4141_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_values4145  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_values4145_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_values4145	= { FOLLOW_DEFAULT_SYMBOL_in_values4145_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_values4149  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_values4149_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_values4149	= { FOLLOW_COMMA_SYMBOL_in_values4149_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_values4152  */
static	ANTLR3_BITWORD FOLLOW_expression_in_values4152_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_values4152	= { FOLLOW_expression_in_values4152_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_values4156  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_values4156_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_values4156	= { FOLLOW_DEFAULT_SYMBOL_in_values4156_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_duplicate_key_update4168  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_duplicate_key_update4168_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_duplicate_key_update4168	= { FOLLOW_ON_SYMBOL_in_duplicate_key_update4168_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DUPLICATE_SYMBOL_in_duplicate_key_update4170  */
static	ANTLR3_BITWORD FOLLOW_DUPLICATE_SYMBOL_in_duplicate_key_update4170_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DUPLICATE_SYMBOL_in_duplicate_key_update4170	= { FOLLOW_DUPLICATE_SYMBOL_in_duplicate_key_update4170_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_duplicate_key_update4172  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_duplicate_key_update4172_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_duplicate_key_update4172	= { FOLLOW_KEY_SYMBOL_in_duplicate_key_update4172_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_duplicate_key_update4174  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_duplicate_key_update4174_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_duplicate_key_update4174	= { FOLLOW_UPDATE_SYMBOL_in_duplicate_key_update4174_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_list_with_default_in_duplicate_key_update4176  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_list_with_default_in_duplicate_key_update4176_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_list_with_default_in_duplicate_key_update4176	= { FOLLOW_column_assignment_list_with_default_in_duplicate_key_update4176_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOAD_SYMBOL_in_load_statement4187  */
static	ANTLR3_BITWORD FOLLOW_LOAD_SYMBOL_in_load_statement4187_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOAD_SYMBOL_in_load_statement4187	= { FOLLOW_LOAD_SYMBOL_in_load_statement4187_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_or_xml_in_load_statement4189  */
static	ANTLR3_BITWORD FOLLOW_data_or_xml_in_load_statement4189_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_data_or_xml_in_load_statement4189	= { FOLLOW_data_or_xml_in_load_statement4189_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCAL_SYMBOL_in_load_statement4200  */
static	ANTLR3_BITWORD FOLLOW_LOCAL_SYMBOL_in_load_statement4200_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCAL_SYMBOL_in_load_statement4200	= { FOLLOW_LOCAL_SYMBOL_in_load_statement4200_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INFILE_SYMBOL_in_load_statement4203  */
static	ANTLR3_BITWORD FOLLOW_INFILE_SYMBOL_in_load_statement4203_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INFILE_SYMBOL_in_load_statement4203	= { FOLLOW_INFILE_SYMBOL_in_load_statement4203_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_load_statement4205  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_load_statement4205_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_load_statement4205	= { FOLLOW_string_literal_in_load_statement4205_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_load_statement4218  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_load_statement4218_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_load_statement4218	= { FOLLOW_INTO_SYMBOL_in_load_statement4218_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_load_statement4220  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_load_statement4220_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_load_statement4220	= { FOLLOW_TABLE_SYMBOL_in_load_statement4220_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_load_statement4222  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_load_statement4222_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000800), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x8000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_load_statement4222	= { FOLLOW_table_ref_in_load_statement4222_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_use_partition_in_load_statement4226  */
static	ANTLR3_BITWORD FOLLOW_use_partition_in_load_statement4226_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x8000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_use_partition_in_load_statement4226	= { FOLLOW_use_partition_in_load_statement4226_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_clause_in_load_statement4229  */
static	ANTLR3_BITWORD FOLLOW_charset_clause_in_load_statement4229_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_clause_in_load_statement4229	= { FOLLOW_charset_clause_in_load_statement4229_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xml_rows_identified_by_in_load_statement4234  */
static	ANTLR3_BITWORD FOLLOW_xml_rows_identified_by_in_load_statement4234_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_xml_rows_identified_by_in_load_statement4234	= { FOLLOW_xml_rows_identified_by_in_load_statement4234_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_fields_clause_in_load_statement4239  */
static	ANTLR3_BITWORD FOLLOW_fields_clause_in_load_statement4239_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_fields_clause_in_load_statement4239	= { FOLLOW_fields_clause_in_load_statement4239_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lines_clause_in_load_statement4242  */
static	ANTLR3_BITWORD FOLLOW_lines_clause_in_load_statement4242_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_lines_clause_in_load_statement4242	= { FOLLOW_lines_clause_in_load_statement4242_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_load_data_file_tail_in_load_statement4247  */
static	ANTLR3_BITWORD FOLLOW_load_data_file_tail_in_load_statement4247_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_load_data_file_tail_in_load_statement4247	= { FOLLOW_load_data_file_tail_in_load_statement4247_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_data_or_xml4256  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_data_or_xml4256_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_data_or_xml4256	= { FOLLOW_DATA_SYMBOL_in_data_or_xml4256_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_XML_SYMBOL_in_data_or_xml4263  */
static	ANTLR3_BITWORD FOLLOW_XML_SYMBOL_in_data_or_xml4263_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_XML_SYMBOL_in_data_or_xml4263	= { FOLLOW_XML_SYMBOL_in_data_or_xml4263_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROWS_SYMBOL_in_xml_rows_identified_by4276  */
static	ANTLR3_BITWORD FOLLOW_ROWS_SYMBOL_in_xml_rows_identified_by4276_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROWS_SYMBOL_in_xml_rows_identified_by4276	= { FOLLOW_ROWS_SYMBOL_in_xml_rows_identified_by4276_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IDENTIFIED_SYMBOL_in_xml_rows_identified_by4278  */
static	ANTLR3_BITWORD FOLLOW_IDENTIFIED_SYMBOL_in_xml_rows_identified_by4278_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_IDENTIFIED_SYMBOL_in_xml_rows_identified_by4278	= { FOLLOW_IDENTIFIED_SYMBOL_in_xml_rows_identified_by4278_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_xml_rows_identified_by4280  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_xml_rows_identified_by4280_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_xml_rows_identified_by4280	= { FOLLOW_BY_SYMBOL_in_xml_rows_identified_by4280_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_xml_rows_identified_by4282  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_xml_rows_identified_by4282_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_xml_rows_identified_by4282	= { FOLLOW_text_string_in_xml_rows_identified_by4282_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SYMBOL_in_load_data_file_tail4292  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SYMBOL_in_load_data_file_tail4292_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SYMBOL_in_load_data_file_tail4292	= { FOLLOW_IGNORE_SYMBOL_in_load_data_file_tail4292_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_load_data_file_tail4294  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_load_data_file_tail4294_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_load_data_file_tail4294	= { FOLLOW_INT_NUMBER_in_load_data_file_tail4294_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_load_data_file_tail4296  */
static	ANTLR3_BITWORD FOLLOW_set_in_load_data_file_tail4296_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_load_data_file_tail4296	= { FOLLOW_set_in_load_data_file_tail4296_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_load_data_file_target_list_in_load_data_file_tail4306  */
static	ANTLR3_BITWORD FOLLOW_load_data_file_target_list_in_load_data_file_tail4306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_load_data_file_target_list_in_load_data_file_tail4306	= { FOLLOW_load_data_file_target_list_in_load_data_file_tail4306_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_load_data_file_tail4310  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_load_data_file_tail4310_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_load_data_file_tail4310	= { FOLLOW_SET_SYMBOL_in_load_data_file_tail4310_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_list_with_default_in_load_data_file_tail4312  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_list_with_default_in_load_data_file_tail4312_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_list_with_default_in_load_data_file_tail4312	= { FOLLOW_column_assignment_list_with_default_in_load_data_file_tail4312_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_load_data_file_target_list4323  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_load_data_file_target_list4323_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8016000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_load_data_file_target_list4323	= { FOLLOW_OPEN_PAR_SYMBOL_in_load_data_file_target_list4323_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_or_variable_list_in_load_data_file_target_list4325  */
static	ANTLR3_BITWORD FOLLOW_field_or_variable_list_in_load_data_file_target_list4325_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_or_variable_list_in_load_data_file_target_list4325	= { FOLLOW_field_or_variable_list_in_load_data_file_target_list4325_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_load_data_file_target_list4328  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_load_data_file_target_list4328_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_load_data_file_target_list4328	= { FOLLOW_CLOSE_PAR_SYMBOL_in_load_data_file_target_list4328_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_field_or_variable_list4338  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_field_or_variable_list4338_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_field_or_variable_list4338	= { FOLLOW_column_ref_in_field_or_variable_list4338_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_field_or_variable_list4342  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_field_or_variable_list4342_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_field_or_variable_list4342	= { FOLLOW_user_variable_in_field_or_variable_list4342_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_field_or_variable_list4346  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_field_or_variable_list4346_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0016000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_field_or_variable_list4346	= { FOLLOW_COMMA_SYMBOL_in_field_or_variable_list4346_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_field_or_variable_list4349  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_field_or_variable_list4349_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_field_or_variable_list4349	= { FOLLOW_column_ref_in_field_or_variable_list4349_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_field_or_variable_list4353  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_field_or_variable_list4353_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_field_or_variable_list4353	= { FOLLOW_user_variable_in_field_or_variable_list4353_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLACE_SYMBOL_in_replace_statement4367  */
static	ANTLR3_BITWORD FOLLOW_REPLACE_SYMBOL_in_replace_statement4367_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLACE_SYMBOL_in_replace_statement4367	= { FOLLOW_REPLACE_SYMBOL_in_replace_statement4367_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_replace_statement4378  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_replace_statement4378_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_replace_statement4378	= { FOLLOW_INTO_SYMBOL_in_replace_statement4378_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_replace_statement4381  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_replace_statement4381_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004100000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_replace_statement4381	= { FOLLOW_table_ref_in_replace_statement4381_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_use_partition_in_replace_statement4385  */
static	ANTLR3_BITWORD FOLLOW_use_partition_in_replace_statement4385_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000004100000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_use_partition_in_replace_statement4385	= { FOLLOW_use_partition_in_replace_statement4385_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_field_spec_in_replace_statement4388  */
static	ANTLR3_BITWORD FOLLOW_insert_field_spec_in_replace_statement4388_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_field_spec_in_replace_statement4388	= { FOLLOW_insert_field_spec_in_replace_statement4388_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_select_statement4399  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_select_statement4399_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_select_statement4399	= { FOLLOW_SELECT_SYMBOL_in_select_statement4399_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_part2_in_select_statement4401  */
static	ANTLR3_BITWORD FOLLOW_select_part2_in_select_statement4401_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_part2_in_select_statement4401	= { FOLLOW_select_part2_in_select_statement4401_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_clause_in_select_statement4403  */
static	ANTLR3_BITWORD FOLLOW_union_clause_in_select_statement4403_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_clause_in_select_statement4403	= { FOLLOW_union_clause_in_select_statement4403_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_select_statement4409  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_select_statement4409_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_select_statement4409	= { FOLLOW_OPEN_PAR_SYMBOL_in_select_statement4409_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_paren_in_select_statement4411  */
static	ANTLR3_BITWORD FOLLOW_select_paren_in_select_statement4411_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_paren_in_select_statement4411	= { FOLLOW_select_paren_in_select_statement4411_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_select_statement4413  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_select_statement4413_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_select_statement4413	= { FOLLOW_CLOSE_PAR_SYMBOL_in_select_statement4413_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_or_order_by_or_limit_in_select_statement4415  */
static	ANTLR3_BITWORD FOLLOW_union_or_order_by_or_limit_in_select_statement4415_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_or_order_by_or_limit_in_select_statement4415	= { FOLLOW_union_or_order_by_or_limit_in_select_statement4415_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_select_paren4425  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_select_paren4425_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_select_paren4425	= { FOLLOW_SELECT_SYMBOL_in_select_paren4425_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_part2_in_select_paren4427  */
static	ANTLR3_BITWORD FOLLOW_select_part2_in_select_paren4427_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_part2_in_select_paren4427	= { FOLLOW_select_part2_in_select_paren4427_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_select_paren4432  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_select_paren4432_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_select_paren4432	= { FOLLOW_OPEN_PAR_SYMBOL_in_select_paren4432_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_paren_in_select_paren4434  */
static	ANTLR3_BITWORD FOLLOW_select_paren_in_select_paren4434_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_paren_in_select_paren4434	= { FOLLOW_select_paren_in_select_paren4434_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren4436  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren4436_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren4436	= { FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren4436_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_from_clause_in_select_from4445  */
static	ANTLR3_BITWORD FOLLOW_from_clause_in_select_from4445_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001800000), ANTLR3_UINT64_LIT(0x2008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_from_clause_in_select_from4445	= { FOLLOW_from_clause_in_select_from4445_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_select_from4447  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_select_from4447_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001800000), ANTLR3_UINT64_LIT(0x2008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_select_from4447	= { FOLLOW_where_clause_in_select_from4447_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_group_by_clause_in_select_from4450  */
static	ANTLR3_BITWORD FOLLOW_group_by_clause_in_select_from4450_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001000000), ANTLR3_UINT64_LIT(0x2008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_group_by_clause_in_select_from4450	= { FOLLOW_group_by_clause_in_select_from4450_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_having_clause_in_select_from4453  */
static	ANTLR3_BITWORD FOLLOW_having_clause_in_select_from4453_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x2008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_having_clause_in_select_from4453	= { FOLLOW_having_clause_in_select_from4453_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_select_from4456  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_select_from4456_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_select_from4456	= { FOLLOW_order_by_clause_in_select_from4456_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_select_from4459  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_select_from4459_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_select_from4459	= { FOLLOW_limit_clause_in_select_from4459_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_analyse_clause_in_select_from4462  */
static	ANTLR3_BITWORD FOLLOW_procedure_analyse_clause_in_select_from4462_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_analyse_clause_in_select_from4462	= { FOLLOW_procedure_analyse_clause_in_select_from4462_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_option_in_select_part24486  */
static	ANTLR3_BITWORD FOLLOW_select_option_in_select_part24486_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_option_in_select_part24486	= { FOLLOW_select_option_in_select_part24486_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_list_in_select_part24490  */
static	ANTLR3_BITWORD FOLLOW_select_item_list_in_select_part24490_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000100000028000), ANTLR3_UINT64_LIT(0x0008000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_list_in_select_part24490	= { FOLLOW_select_item_list_in_select_part24490_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_select_part24499  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_select_part24499_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_select_part24499	= { FOLLOW_order_by_clause_in_select_part24499_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_select_part24502  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_select_part24502_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_select_part24502	= { FOLLOW_limit_clause_in_select_part24502_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_into_clause_in_select_part24510  */
static	ANTLR3_BITWORD FOLLOW_into_clause_in_select_part24510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000028000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_into_clause_in_select_part24510	= { FOLLOW_into_clause_in_select_part24510_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_from_in_select_part24512  */
static	ANTLR3_BITWORD FOLLOW_select_from_in_select_part24512_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_from_in_select_part24512	= { FOLLOW_select_from_in_select_part24512_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_from_in_select_part24520  */
static	ANTLR3_BITWORD FOLLOW_select_from_in_select_part24520_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_from_in_select_part24520	= { FOLLOW_select_from_in_select_part24520_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_into_clause_in_select_part24522  */
static	ANTLR3_BITWORD FOLLOW_into_clause_in_select_part24522_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_into_clause_in_select_part24522	= { FOLLOW_into_clause_in_select_part24522_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_lock_type_in_select_part24531  */
static	ANTLR3_BITWORD FOLLOW_select_lock_type_in_select_part24531_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_lock_type_in_select_part24531	= { FOLLOW_select_lock_type_in_select_part24531_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_from_clause_in_table_expression4541  */
static	ANTLR3_BITWORD FOLLOW_from_clause_in_table_expression4541_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001808000), ANTLR3_UINT64_LIT(0x2008000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_from_clause_in_table_expression4541	= { FOLLOW_from_clause_in_table_expression4541_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_table_expression4559  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_table_expression4559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001808000), ANTLR3_UINT64_LIT(0x2008000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_table_expression4559	= { FOLLOW_where_clause_in_table_expression4559_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_group_by_clause_in_table_expression4578  */
static	ANTLR3_BITWORD FOLLOW_group_by_clause_in_table_expression4578_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001008000), ANTLR3_UINT64_LIT(0x2008000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_group_by_clause_in_table_expression4578	= { FOLLOW_group_by_clause_in_table_expression4578_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_having_clause_in_table_expression4597  */
static	ANTLR3_BITWORD FOLLOW_having_clause_in_table_expression4597_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000), ANTLR3_UINT64_LIT(0x2008000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_having_clause_in_table_expression4597	= { FOLLOW_having_clause_in_table_expression4597_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_table_expression4616  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_table_expression4616_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000), ANTLR3_UINT64_LIT(0x2000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_table_expression4616	= { FOLLOW_order_by_clause_in_table_expression4616_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_table_expression4635  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_table_expression4635_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x2000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_table_expression4635	= { FOLLOW_limit_clause_in_table_expression4635_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_analyse_clause_in_table_expression4654  */
static	ANTLR3_BITWORD FOLLOW_procedure_analyse_clause_in_table_expression4654_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_analyse_clause_in_table_expression4654	= { FOLLOW_procedure_analyse_clause_in_table_expression4654_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_lock_type_in_table_expression4673  */
static	ANTLR3_BITWORD FOLLOW_select_lock_type_in_table_expression4673_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_lock_type_in_table_expression4673	= { FOLLOW_select_lock_type_in_table_expression4673_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_subquery4685  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_subquery4685_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_subquery4685	= { FOLLOW_OPEN_PAR_SYMBOL_in_subquery4685_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_expression_body_in_subquery4687  */
static	ANTLR3_BITWORD FOLLOW_query_expression_body_in_subquery4687_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_query_expression_body_in_subquery4687	= { FOLLOW_query_expression_body_in_subquery4687_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_subquery4689  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_subquery4689_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_subquery4689	= { FOLLOW_CLOSE_PAR_SYMBOL_in_subquery4689_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_specification_in_query_expression_body4698  */
static	ANTLR3_BITWORD FOLLOW_query_specification_in_query_expression_body4698_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_query_specification_in_query_expression_body4698	= { FOLLOW_query_specification_in_query_expression_body4698_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNION_SYMBOL_in_query_expression_body4701  */
static	ANTLR3_BITWORD FOLLOW_UNION_SYMBOL_in_query_expression_body4701_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNION_SYMBOL_in_query_expression_body4701	= { FOLLOW_UNION_SYMBOL_in_query_expression_body4701_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_option_in_query_expression_body4703  */
static	ANTLR3_BITWORD FOLLOW_union_option_in_query_expression_body4703_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_union_option_in_query_expression_body4703	= { FOLLOW_union_option_in_query_expression_body4703_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_specification_in_query_expression_body4705  */
static	ANTLR3_BITWORD FOLLOW_query_specification_in_query_expression_body4705_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_query_specification_in_query_expression_body4705	= { FOLLOW_query_specification_in_query_expression_body4705_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_spec_option_in_select_part2_derived4733  */
static	ANTLR3_BITWORD FOLLOW_query_spec_option_in_select_part2_derived4733_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_query_spec_option_in_select_part2_derived4733	= { FOLLOW_query_spec_option_in_select_part2_derived4733_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_select_part2_derived4741  */
static	ANTLR3_BITWORD FOLLOW_set_in_select_part2_derived4741_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_select_part2_derived4741	= { FOLLOW_set_in_select_part2_derived4741_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_list_in_select_part2_derived4753  */
static	ANTLR3_BITWORD FOLLOW_select_item_list_in_select_part2_derived4753_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_list_in_select_part2_derived4753	= { FOLLOW_select_item_list_in_select_part2_derived4753_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_spec_option_in_select_option4762  */
static	ANTLR3_BITWORD FOLLOW_query_spec_option_in_select_option4762_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_query_spec_option_in_select_option4762	= { FOLLOW_query_spec_option_in_select_option4762_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_NO_CACHE_SYMBOL_in_select_option4767  */
static	ANTLR3_BITWORD FOLLOW_SQL_NO_CACHE_SYMBOL_in_select_option4767_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_NO_CACHE_SYMBOL_in_select_option4767	= { FOLLOW_SQL_NO_CACHE_SYMBOL_in_select_option4767_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_CACHE_SYMBOL_in_select_option4772  */
static	ANTLR3_BITWORD FOLLOW_SQL_CACHE_SYMBOL_in_select_option4772_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_CACHE_SYMBOL_in_select_option4772	= { FOLLOW_SQL_CACHE_SYMBOL_in_select_option4772_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_STATEMENT_TIME_SYMBOL_in_select_option4779  */
static	ANTLR3_BITWORD FOLLOW_MAX_STATEMENT_TIME_SYMBOL_in_select_option4779_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_STATEMENT_TIME_SYMBOL_in_select_option4779	= { FOLLOW_MAX_STATEMENT_TIME_SYMBOL_in_select_option4779_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_select_option4781  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_select_option4781_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_select_option4781	= { FOLLOW_EQUAL_OPERATOR_in_select_option4781_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_select_option4783  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_select_option4783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_select_option4783	= { FOLLOW_real_ulong_number_in_select_option4783_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_in_select_item_list4838  */
static	ANTLR3_BITWORD FOLLOW_select_item_in_select_item_list4838_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_in_select_item_list4838	= { FOLLOW_select_item_in_select_item_list4838_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_select_item_list4842  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_select_item_list4842_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_select_item_list4842	= { FOLLOW_MULT_OPERATOR_in_select_item_list4842_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_select_item_list4859  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_select_item_list4859_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_select_item_list4859	= { FOLLOW_COMMA_SYMBOL_in_select_item_list4859_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_in_select_item_list4861  */
static	ANTLR3_BITWORD FOLLOW_select_item_in_select_item_list4861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_in_select_item_list4861	= { FOLLOW_select_item_in_select_item_list4861_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_wild_in_select_item4878  */
static	ANTLR3_BITWORD FOLLOW_table_wild_in_select_item4878_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_wild_in_select_item4878	= { FOLLOW_table_wild_in_select_item4878_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_select_item4883  */
static	ANTLR3_BITWORD FOLLOW_expression_in_select_item4883_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0030800000000000), ANTLR3_UINT64_LIT(0x0000001800100000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_select_item4883	= { FOLLOW_expression_in_select_item4883_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_alias_in_select_item4885  */
static	ANTLR3_BITWORD FOLLOW_select_alias_in_select_item4885_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_alias_in_select_item4885	= { FOLLOW_select_alias_in_select_item4885_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_select_alias4895  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_select_alias4895_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0030000000000000), ANTLR3_UINT64_LIT(0x0000001800100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_select_alias4895	= { FOLLOW_AS_SYMBOL_in_select_alias4895_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_select_alias4899  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_select_alias4899_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_select_alias4899	= { FOLLOW_identifier_in_select_alias4899_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_select_alias4903  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_select_alias4903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_select_alias4903	= { FOLLOW_text_string_in_select_alias4903_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIMIT_SYMBOL_in_limit_clause4923  */
static	ANTLR3_BITWORD FOLLOW_LIMIT_SYMBOL_in_limit_clause4923_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LIMIT_SYMBOL_in_limit_clause4923	= { FOLLOW_LIMIT_SYMBOL_in_limit_clause4923_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_options_in_limit_clause4925  */
static	ANTLR3_BITWORD FOLLOW_limit_options_in_limit_clause4925_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_options_in_limit_clause4925	= { FOLLOW_limit_options_in_limit_clause4925_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIMIT_SYMBOL_in_simple_limit_clause4934  */
static	ANTLR3_BITWORD FOLLOW_LIMIT_SYMBOL_in_simple_limit_clause4934_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LIMIT_SYMBOL_in_simple_limit_clause4934	= { FOLLOW_LIMIT_SYMBOL_in_simple_limit_clause4934_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_option_in_simple_limit_clause4936  */
static	ANTLR3_BITWORD FOLLOW_limit_option_in_simple_limit_clause4936_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_option_in_simple_limit_clause4936	= { FOLLOW_limit_option_in_simple_limit_clause4936_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_option_in_limit_options4945  */
static	ANTLR3_BITWORD FOLLOW_limit_option_in_limit_options4945_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_option_in_limit_options4945	= { FOLLOW_limit_option_in_limit_options4945_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_limit_options4961  */
static	ANTLR3_BITWORD FOLLOW_set_in_limit_options4961_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011000100000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_limit_options4961	= { FOLLOW_set_in_limit_options4961_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_option_in_limit_options4969  */
static	ANTLR3_BITWORD FOLLOW_limit_option_in_limit_options4969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_option_in_limit_options4969	= { FOLLOW_limit_option_in_limit_options4969_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_limit_option4980  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_limit_option4980_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_limit_option4980	= { FOLLOW_identifier_in_limit_option4980_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARAM_MARKER_in_limit_option4985  */
static	ANTLR3_BITWORD FOLLOW_PARAM_MARKER_in_limit_option4985_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PARAM_MARKER_in_limit_option4985	= { FOLLOW_PARAM_MARKER_in_limit_option4985_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ULONGLONG_NUMBER_in_limit_option4990  */
static	ANTLR3_BITWORD FOLLOW_ULONGLONG_NUMBER_in_limit_option4990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ULONGLONG_NUMBER_in_limit_option4990	= { FOLLOW_ULONGLONG_NUMBER_in_limit_option4990_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LONG_NUMBER_in_limit_option4995  */
static	ANTLR3_BITWORD FOLLOW_LONG_NUMBER_in_limit_option4995_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LONG_NUMBER_in_limit_option4995	= { FOLLOW_LONG_NUMBER_in_limit_option4995_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_limit_option5000  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_limit_option5000_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_limit_option5000	= { FOLLOW_INT_NUMBER_in_limit_option5000_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_into_clause5009  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_into_clause5009_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0016000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_into_clause5009	= { FOLLOW_INTO_SYMBOL_in_into_clause5009_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OUTFILE_SYMBOL_in_into_clause5016  */
static	ANTLR3_BITWORD FOLLOW_OUTFILE_SYMBOL_in_into_clause5016_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OUTFILE_SYMBOL_in_into_clause5016	= { FOLLOW_OUTFILE_SYMBOL_in_into_clause5016_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_into_clause5018  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_into_clause5018_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000802), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_into_clause5018	= { FOLLOW_string_literal_in_into_clause5018_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_clause_in_into_clause5020  */
static	ANTLR3_BITWORD FOLLOW_charset_clause_in_into_clause5020_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_clause_in_into_clause5020	= { FOLLOW_charset_clause_in_into_clause5020_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_fields_clause_in_into_clause5023  */
static	ANTLR3_BITWORD FOLLOW_fields_clause_in_into_clause5023_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_fields_clause_in_into_clause5023	= { FOLLOW_fields_clause_in_into_clause5023_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lines_clause_in_into_clause5026  */
static	ANTLR3_BITWORD FOLLOW_lines_clause_in_into_clause5026_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_lines_clause_in_into_clause5026	= { FOLLOW_lines_clause_in_into_clause5026_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DUMPFILE_SYMBOL_in_into_clause5033  */
static	ANTLR3_BITWORD FOLLOW_DUMPFILE_SYMBOL_in_into_clause5033_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DUMPFILE_SYMBOL_in_into_clause5033	= { FOLLOW_DUMPFILE_SYMBOL_in_into_clause5033_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_into_clause5035  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_into_clause5035_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_into_clause5035	= { FOLLOW_string_literal_in_into_clause5035_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_SIGN_SYMBOL_in_into_clause5041  */
static	ANTLR3_BITWORD FOLLOW_AT_SIGN_SYMBOL_in_into_clause5041_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0014000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_SIGN_SYMBOL_in_into_clause5041	= { FOLLOW_AT_SIGN_SYMBOL_in_into_clause5041_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_into_clause5045  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_into_clause5045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_into_clause5045	= { FOLLOW_text_or_identifier_in_into_clause5045_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_TEXT_SUFFIX_in_into_clause5049  */
static	ANTLR3_BITWORD FOLLOW_AT_TEXT_SUFFIX_in_into_clause5049_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_TEXT_SUFFIX_in_into_clause5049	= { FOLLOW_AT_TEXT_SUFFIX_in_into_clause5049_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_into_clause5053  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_into_clause5053_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0016000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_into_clause5053	= { FOLLOW_COMMA_SYMBOL_in_into_clause5053_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_SIGN_SYMBOL_in_into_clause5055  */
static	ANTLR3_BITWORD FOLLOW_AT_SIGN_SYMBOL_in_into_clause5055_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0014000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_SIGN_SYMBOL_in_into_clause5055	= { FOLLOW_AT_SIGN_SYMBOL_in_into_clause5055_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_into_clause5059  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_into_clause5059_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_into_clause5059	= { FOLLOW_text_or_identifier_in_into_clause5059_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_TEXT_SUFFIX_in_into_clause5063  */
static	ANTLR3_BITWORD FOLLOW_AT_TEXT_SUFFIX_in_into_clause5063_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_TEXT_SUFFIX_in_into_clause5063	= { FOLLOW_AT_TEXT_SUFFIX_in_into_clause5063_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_procedure_analyse_clause5078  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_procedure_analyse_clause5078_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_procedure_analyse_clause5078	= { FOLLOW_PROCEDURE_SYMBOL_in_procedure_analyse_clause5078_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ANALYSE_SYMBOL_in_procedure_analyse_clause5080  */
static	ANTLR3_BITWORD FOLLOW_ANALYSE_SYMBOL_in_procedure_analyse_clause5080_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ANALYSE_SYMBOL_in_procedure_analyse_clause5080	= { FOLLOW_ANALYSE_SYMBOL_in_procedure_analyse_clause5080_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_procedure_analyse_clause5082  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_procedure_analyse_clause5082_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_procedure_analyse_clause5082	= { FOLLOW_OPEN_PAR_SYMBOL_in_procedure_analyse_clause5082_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_procedure_analyse_clause5085  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_procedure_analyse_clause5085_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_procedure_analyse_clause5085	= { FOLLOW_INT_NUMBER_in_procedure_analyse_clause5085_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_procedure_analyse_clause5088  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_procedure_analyse_clause5088_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_procedure_analyse_clause5088	= { FOLLOW_COMMA_SYMBOL_in_procedure_analyse_clause5088_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_procedure_analyse_clause5090  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_procedure_analyse_clause5090_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_procedure_analyse_clause5090	= { FOLLOW_INT_NUMBER_in_procedure_analyse_clause5090_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_analyse_clause5096  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_analyse_clause5096_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_analyse_clause5096	= { FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_analyse_clause5096_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HAVING_SYMBOL_in_having_clause5105  */
static	ANTLR3_BITWORD FOLLOW_HAVING_SYMBOL_in_having_clause5105_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_HAVING_SYMBOL_in_having_clause5105	= { FOLLOW_HAVING_SYMBOL_in_having_clause5105_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_having_clause5107  */
static	ANTLR3_BITWORD FOLLOW_expression_in_having_clause5107_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_having_clause5107	= { FOLLOW_expression_in_having_clause5107_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_group_by_clause5116  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_group_by_clause5116_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_group_by_clause5116	= { FOLLOW_GROUP_SYMBOL_in_group_by_clause5116_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_group_by_clause5118  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_group_by_clause5118_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_group_by_clause5118	= { FOLLOW_BY_SYMBOL_in_group_by_clause5118_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_direction_in_group_by_clause5120  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_direction_in_group_by_clause5120_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_direction_in_group_by_clause5120	= { FOLLOW_expression_list_with_direction_in_group_by_clause5120_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_olap_option_in_group_by_clause5122  */
static	ANTLR3_BITWORD FOLLOW_olap_option_in_group_by_clause5122_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_olap_option_in_group_by_clause5122	= { FOLLOW_olap_option_in_group_by_clause5122_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_olap_option5132  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_olap_option5132_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_olap_option5132	= { FOLLOW_WITH_SYMBOL_in_olap_option5132_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CUBE_SYMBOL_in_olap_option5134  */
static	ANTLR3_BITWORD FOLLOW_CUBE_SYMBOL_in_olap_option5134_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CUBE_SYMBOL_in_olap_option5134	= { FOLLOW_CUBE_SYMBOL_in_olap_option5134_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_olap_option5139  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_olap_option5139_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000004000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_olap_option5139	= { FOLLOW_WITH_SYMBOL_in_olap_option5139_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROLLUP_SYMBOL_in_olap_option5141  */
static	ANTLR3_BITWORD FOLLOW_ROLLUP_SYMBOL_in_olap_option5141_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ROLLUP_SYMBOL_in_olap_option5141	= { FOLLOW_ROLLUP_SYMBOL_in_olap_option5141_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ORDER_SYMBOL_in_order_by_clause5150  */
static	ANTLR3_BITWORD FOLLOW_ORDER_SYMBOL_in_order_by_clause5150_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_ORDER_SYMBOL_in_order_by_clause5150	= { FOLLOW_ORDER_SYMBOL_in_order_by_clause5150_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_order_by_clause5152  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_order_by_clause5152_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_order_by_clause5152	= { FOLLOW_BY_SYMBOL_in_order_by_clause5152_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_direction_in_order_by_clause5154  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_direction_in_order_by_clause5154_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_direction_in_order_by_clause5154	= { FOLLOW_expression_list_with_direction_in_order_by_clause5154_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_order_by_or_limit5163  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_order_by_or_limit5163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_order_by_or_limit5163	= { FOLLOW_order_by_clause_in_order_by_or_limit5163_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_order_by_or_limit5165  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_order_by_or_limit5165_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_order_by_or_limit5165	= { FOLLOW_limit_clause_in_order_by_or_limit5165_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_order_by_or_limit5171  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_order_by_or_limit5171_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_order_by_or_limit5171	= { FOLLOW_limit_clause_in_order_by_or_limit5171_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_from_clause5194  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_from_clause5194_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_from_clause5194	= { FOLLOW_FROM_SYMBOL_in_from_clause5194_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_list_in_from_clause5196  */
static	ANTLR3_BITWORD FOLLOW_table_reference_list_in_from_clause5196_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_list_in_from_clause5196	= { FOLLOW_table_reference_list_in_from_clause5196_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WHERE_SYMBOL_in_where_clause5205  */
static	ANTLR3_BITWORD FOLLOW_WHERE_SYMBOL_in_where_clause5205_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_WHERE_SYMBOL_in_where_clause5205	= { FOLLOW_WHERE_SYMBOL_in_where_clause5205_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_where_clause5207  */
static	ANTLR3_BITWORD FOLLOW_expression_in_where_clause5207_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_where_clause5207	= { FOLLOW_expression_in_where_clause5207_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_table_list_in_table_reference_list5216  */
static	ANTLR3_BITWORD FOLLOW_join_table_list_in_table_reference_list5216_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_join_table_list_in_table_reference_list5216	= { FOLLOW_join_table_list_in_table_reference_list5216_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DUAL_SYMBOL_in_table_reference_list5221  */
static	ANTLR3_BITWORD FOLLOW_DUAL_SYMBOL_in_table_reference_list5221_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DUAL_SYMBOL_in_table_reference_list5221	= { FOLLOW_DUAL_SYMBOL_in_table_reference_list5221_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_escaped_table_reference_in_join_table_list5232  */
static	ANTLR3_BITWORD FOLLOW_escaped_table_reference_in_join_table_list5232_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_escaped_table_reference_in_join_table_list5232	= { FOLLOW_escaped_table_reference_in_join_table_list5232_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_join_table_list5248  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_join_table_list5248_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_join_table_list5248	= { FOLLOW_COMMA_SYMBOL_in_join_table_list5248_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_escaped_table_reference_in_join_table_list5250  */
static	ANTLR3_BITWORD FOLLOW_escaped_table_reference_in_join_table_list5250_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_escaped_table_reference_in_join_table_list5250	= { FOLLOW_escaped_table_reference_in_join_table_list5250_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_in_escaped_table_reference5267  */
static	ANTLR3_BITWORD FOLLOW_table_reference_in_escaped_table_reference5267_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_in_escaped_table_reference5267	= { FOLLOW_table_reference_in_escaped_table_reference5267_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_CURLY_SYMBOL_in_escaped_table_reference5272  */
static	ANTLR3_BITWORD FOLLOW_OPEN_CURLY_SYMBOL_in_escaped_table_reference5272_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_CURLY_SYMBOL_in_escaped_table_reference5272	= { FOLLOW_OPEN_CURLY_SYMBOL_in_escaped_table_reference5272_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_escaped_table_reference5274  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_escaped_table_reference5274_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_escaped_table_reference5274	= { FOLLOW_identifier_in_escaped_table_reference5274_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_in_escaped_table_reference5276  */
static	ANTLR3_BITWORD FOLLOW_table_reference_in_escaped_table_reference5276_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_in_escaped_table_reference5276	= { FOLLOW_table_reference_in_escaped_table_reference5276_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_CURLY_SYMBOL_in_escaped_table_reference5278  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_CURLY_SYMBOL_in_escaped_table_reference5278_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_CURLY_SYMBOL_in_escaped_table_reference5278	= { FOLLOW_CLOSE_CURLY_SYMBOL_in_escaped_table_reference5278_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_factor_in_table_reference5288  */
static	ANTLR3_BITWORD FOLLOW_table_factor_in_table_reference5288_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000008000000), ANTLR3_UINT64_LIT(0x0810000200000000), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0200000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_factor_in_table_reference5288	= { FOLLOW_table_factor_in_table_reference5288_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_in_table_reference5304  */
static	ANTLR3_BITWORD FOLLOW_join_in_table_reference5304_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000008000000), ANTLR3_UINT64_LIT(0x0810000200000000), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0200000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_join_in_table_reference5304	= { FOLLOW_join_in_table_reference5304_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_table_in_join5315  */
static	ANTLR3_BITWORD FOLLOW_join_table_in_join5315_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_join_table_in_join5315	= { FOLLOW_join_table_in_join5315_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_table_factor5324  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_table_factor5324_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_table_factor5324	= { FOLLOW_SELECT_SYMBOL_in_table_factor5324_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_option_in_table_factor5340  */
static	ANTLR3_BITWORD FOLLOW_select_option_in_table_factor5340_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_select_option_in_table_factor5340	= { FOLLOW_select_option_in_table_factor5340_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_item_list_in_table_factor5344  */
static	ANTLR3_BITWORD FOLLOW_select_item_list_in_table_factor5344_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001828000), ANTLR3_UINT64_LIT(0x2008000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_item_list_in_table_factor5344	= { FOLLOW_select_item_list_in_table_factor5344_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_expression_in_table_factor5346  */
static	ANTLR3_BITWORD FOLLOW_table_expression_in_table_factor5346_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_expression_in_table_factor5346	= { FOLLOW_table_expression_in_table_factor5346_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_table_factor5351  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_table_factor5351_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_table_factor5351	= { FOLLOW_OPEN_PAR_SYMBOL_in_table_factor5351_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_table_factor_union_in_table_factor5353  */
static	ANTLR3_BITWORD FOLLOW_select_table_factor_union_in_table_factor5353_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_table_factor_union_in_table_factor5353	= { FOLLOW_select_table_factor_union_in_table_factor5353_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_table_factor5355  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_table_factor5355_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010800000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_table_factor5355	= { FOLLOW_CLOSE_PAR_SYMBOL_in_table_factor5355_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_alias_in_table_factor5357  */
static	ANTLR3_BITWORD FOLLOW_table_alias_in_table_factor5357_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_alias_in_table_factor5357	= { FOLLOW_table_alias_in_table_factor5357_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_table_factor5363  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_table_factor5363_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010800000080000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_table_factor5363	= { FOLLOW_table_ref_in_table_factor5363_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_use_partition_in_table_factor5365  */
static	ANTLR3_BITWORD FOLLOW_use_partition_in_table_factor5365_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010800000080000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_use_partition_in_table_factor5365	= { FOLLOW_use_partition_in_table_factor5365_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_alias_in_table_factor5368  */
static	ANTLR3_BITWORD FOLLOW_table_alias_in_table_factor5368_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_alias_in_table_factor5368	= { FOLLOW_table_alias_in_table_factor5368_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_list_in_table_factor5371  */
static	ANTLR3_BITWORD FOLLOW_index_hint_list_in_table_factor5371_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_list_in_table_factor5371	= { FOLLOW_index_hint_list_in_table_factor5371_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_list_in_select_table_factor_union5382  */
static	ANTLR3_BITWORD FOLLOW_table_reference_list_in_select_table_factor_union5382_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_list_in_select_table_factor_union5382	= { FOLLOW_table_reference_list_in_select_table_factor_union5382_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_or_limit_in_select_table_factor_union5384  */
static	ANTLR3_BITWORD FOLLOW_order_by_or_limit_in_select_table_factor_union5384_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_or_limit_in_select_table_factor_union5384	= { FOLLOW_order_by_or_limit_in_select_table_factor_union5384_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNION_SYMBOL_in_select_table_factor_union5389  */
static	ANTLR3_BITWORD FOLLOW_UNION_SYMBOL_in_select_table_factor_union5389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNION_SYMBOL_in_select_table_factor_union5389	= { FOLLOW_UNION_SYMBOL_in_select_table_factor_union5389_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_option_in_select_table_factor_union5391  */
static	ANTLR3_BITWORD FOLLOW_union_option_in_select_table_factor_union5391_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_union_option_in_select_table_factor_union5391	= { FOLLOW_union_option_in_select_table_factor_union5391_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_query_specification_in_select_table_factor_union5394  */
static	ANTLR3_BITWORD FOLLOW_query_specification_in_select_table_factor_union5394_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_query_specification_in_select_table_factor_union5394	= { FOLLOW_query_specification_in_select_table_factor_union5394_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_query_specification5405  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_query_specification5405_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_query_specification5405	= { FOLLOW_SELECT_SYMBOL_in_query_specification5405_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_part2_derived_in_query_specification5407  */
static	ANTLR3_BITWORD FOLLOW_select_part2_derived_in_query_specification5407_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001828000), ANTLR3_UINT64_LIT(0x2008000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_part2_derived_in_query_specification5407	= { FOLLOW_select_part2_derived_in_query_specification5407_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_expression_in_query_specification5409  */
static	ANTLR3_BITWORD FOLLOW_table_expression_in_query_specification5409_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_expression_in_query_specification5409	= { FOLLOW_table_expression_in_query_specification5409_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_query_specification5414  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_query_specification5414_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_query_specification5414	= { FOLLOW_OPEN_PAR_SYMBOL_in_query_specification5414_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_paren_derived_in_query_specification5416  */
static	ANTLR3_BITWORD FOLLOW_select_paren_derived_in_query_specification5416_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_paren_derived_in_query_specification5416	= { FOLLOW_select_paren_derived_in_query_specification5416_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_query_specification5418  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_query_specification5418_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_query_specification5418	= { FOLLOW_CLOSE_PAR_SYMBOL_in_query_specification5418_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_or_limit_in_query_specification5420  */
static	ANTLR3_BITWORD FOLLOW_order_by_or_limit_in_query_specification5420_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_or_limit_in_query_specification5420	= { FOLLOW_order_by_or_limit_in_query_specification5420_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_select_paren_derived5430  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_select_paren_derived5430_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810224000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x36A000E040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_select_paren_derived5430	= { FOLLOW_SELECT_SYMBOL_in_select_paren_derived5430_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_part2_derived_in_select_paren_derived5432  */
static	ANTLR3_BITWORD FOLLOW_select_part2_derived_in_select_paren_derived5432_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000001828000), ANTLR3_UINT64_LIT(0x2008000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_part2_derived_in_select_paren_derived5432	= { FOLLOW_select_part2_derived_in_select_paren_derived5432_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_expression_in_select_paren_derived5434  */
static	ANTLR3_BITWORD FOLLOW_table_expression_in_select_paren_derived5434_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_expression_in_select_paren_derived5434	= { FOLLOW_table_expression_in_select_paren_derived5434_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_select_paren_derived5439  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_select_paren_derived5439_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_select_paren_derived5439	= { FOLLOW_OPEN_PAR_SYMBOL_in_select_paren_derived5439_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_paren_derived_in_select_paren_derived5441  */
static	ANTLR3_BITWORD FOLLOW_select_paren_derived_in_select_paren_derived5441_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_select_paren_derived_in_select_paren_derived5441	= { FOLLOW_select_paren_derived_in_select_paren_derived5441_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren_derived5443  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren_derived5443_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren_derived5443	= { FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren_derived5443_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JOIN_SYMBOL_in_join_table5462  */
static	ANTLR3_BITWORD FOLLOW_JOIN_SYMBOL_in_join_table5462_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_JOIN_SYMBOL_in_join_table5462	= { FOLLOW_JOIN_SYMBOL_in_join_table5462_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_in_join_table5464  */
static	ANTLR3_BITWORD FOLLOW_table_reference_in_join_table5464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_in_join_table5464	= { FOLLOW_table_reference_in_join_table5464_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_condition_in_join_table5478  */
static	ANTLR3_BITWORD FOLLOW_join_condition_in_join_table5478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_join_condition_in_join_table5478	= { FOLLOW_join_condition_in_join_table5478_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STRAIGHT_JOIN_SYMBOL_in_join_table5485  */
static	ANTLR3_BITWORD FOLLOW_STRAIGHT_JOIN_SYMBOL_in_join_table5485_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_STRAIGHT_JOIN_SYMBOL_in_join_table5485	= { FOLLOW_STRAIGHT_JOIN_SYMBOL_in_join_table5485_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_factor_in_join_table5487  */
static	ANTLR3_BITWORD FOLLOW_table_factor_in_join_table5487_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_factor_in_join_table5487	= { FOLLOW_table_factor_in_join_table5487_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_join_table5501  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_join_table5501_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_join_table5501	= { FOLLOW_ON_SYMBOL_in_join_table5501_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_join_table5503  */
static	ANTLR3_BITWORD FOLLOW_expression_in_join_table5503_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_join_table5503	= { FOLLOW_expression_in_join_table5503_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_join_table5510  */
static	ANTLR3_BITWORD FOLLOW_set_in_join_table5510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_join_table5510	= { FOLLOW_set_in_join_table5510_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OUTER_SYMBOL_in_join_table5518  */
static	ANTLR3_BITWORD FOLLOW_OUTER_SYMBOL_in_join_table5518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OUTER_SYMBOL_in_join_table5518	= { FOLLOW_OUTER_SYMBOL_in_join_table5518_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JOIN_SYMBOL_in_join_table5521  */
static	ANTLR3_BITWORD FOLLOW_JOIN_SYMBOL_in_join_table5521_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_JOIN_SYMBOL_in_join_table5521	= { FOLLOW_JOIN_SYMBOL_in_join_table5521_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_reference_in_join_table5523  */
static	ANTLR3_BITWORD FOLLOW_table_reference_in_join_table5523_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_reference_in_join_table5523	= { FOLLOW_table_reference_in_join_table5523_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_condition_in_join_table5525  */
static	ANTLR3_BITWORD FOLLOW_join_condition_in_join_table5525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_join_condition_in_join_table5525	= { FOLLOW_join_condition_in_join_table5525_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NATURAL_SYMBOL_in_join_table5530  */
static	ANTLR3_BITWORD FOLLOW_NATURAL_SYMBOL_in_join_table5530_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0810000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_NATURAL_SYMBOL_in_join_table5530	= { FOLLOW_NATURAL_SYMBOL_in_join_table5530_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_join_table5533  */
static	ANTLR3_BITWORD FOLLOW_set_in_join_table5533_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_join_table5533	= { FOLLOW_set_in_join_table5533_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OUTER_SYMBOL_in_join_table5541  */
static	ANTLR3_BITWORD FOLLOW_OUTER_SYMBOL_in_join_table5541_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OUTER_SYMBOL_in_join_table5541	= { FOLLOW_OUTER_SYMBOL_in_join_table5541_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JOIN_SYMBOL_in_join_table5546  */
static	ANTLR3_BITWORD FOLLOW_JOIN_SYMBOL_in_join_table5546_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_JOIN_SYMBOL_in_join_table5546	= { FOLLOW_JOIN_SYMBOL_in_join_table5546_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_factor_in_join_table5548  */
static	ANTLR3_BITWORD FOLLOW_table_factor_in_join_table5548_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_factor_in_join_table5548	= { FOLLOW_table_factor_in_join_table5548_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_join_condition5557  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_join_condition5557_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_join_condition5557	= { FOLLOW_ON_SYMBOL_in_join_condition5557_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_join_condition5559  */
static	ANTLR3_BITWORD FOLLOW_expression_in_join_condition5559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_join_condition5559	= { FOLLOW_expression_in_join_condition5559_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USING_SYMBOL_in_join_condition5564  */
static	ANTLR3_BITWORD FOLLOW_USING_SYMBOL_in_join_condition5564_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USING_SYMBOL_in_join_condition5564	= { FOLLOW_USING_SYMBOL_in_join_condition5564_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_join_condition5566  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_join_condition5566_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_join_condition5566	= { FOLLOW_identifier_list_with_parentheses_in_join_condition5566_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNION_SYMBOL_in_union_clause5576  */
static	ANTLR3_BITWORD FOLLOW_UNION_SYMBOL_in_union_clause5576_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNION_SYMBOL_in_union_clause5576	= { FOLLOW_UNION_SYMBOL_in_union_clause5576_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_option_in_union_clause5578  */
static	ANTLR3_BITWORD FOLLOW_union_option_in_union_clause5578_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_union_option_in_union_clause5578	= { FOLLOW_union_option_in_union_clause5578_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_union_clause5581  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_union_clause5581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_union_clause5581	= { FOLLOW_select_statement_in_union_clause5581_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_union_clause_in_union_or_order_by_or_limit5606  */
static	ANTLR3_BITWORD FOLLOW_union_clause_in_union_or_order_by_or_limit5606_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_union_clause_in_union_or_order_by_or_limit5606	= { FOLLOW_union_clause_in_union_or_order_by_or_limit5606_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_or_limit_in_union_or_order_by_or_limit5611  */
static	ANTLR3_BITWORD FOLLOW_order_by_or_limit_in_union_or_order_by_or_limit5611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_or_limit_in_union_or_order_by_or_limit5611	= { FOLLOW_order_by_or_limit_in_union_or_order_by_or_limit5611_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_select_lock_type5620  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_select_lock_type5620_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_select_lock_type5620	= { FOLLOW_FOR_SYMBOL_in_select_lock_type5620_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_select_lock_type5622  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_select_lock_type5622_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_select_lock_type5622	= { FOLLOW_UPDATE_SYMBOL_in_select_lock_type5622_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_select_lock_type5627  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_select_lock_type5627_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_select_lock_type5627	= { FOLLOW_LOCK_SYMBOL_in_select_lock_type5627_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_select_lock_type5629  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_select_lock_type5629_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_select_lock_type5629	= { FOLLOW_IN_SYMBOL_in_select_lock_type5629_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHARE_SYMBOL_in_select_lock_type5631  */
static	ANTLR3_BITWORD FOLLOW_SHARE_SYMBOL_in_select_lock_type5631_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SHARE_SYMBOL_in_select_lock_type5631	= { FOLLOW_SHARE_SYMBOL_in_select_lock_type5631_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MODE_SYMBOL_in_select_lock_type5633  */
static	ANTLR3_BITWORD FOLLOW_MODE_SYMBOL_in_select_lock_type5633_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MODE_SYMBOL_in_select_lock_type5633	= { FOLLOW_MODE_SYMBOL_in_select_lock_type5633_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_table_alias5651  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_table_alias5651_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_table_alias5651	= { FOLLOW_identifier_in_table_alias5651_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_in_index_hint_list5660  */
static	ANTLR3_BITWORD FOLLOW_index_hint_in_index_hint_list5660_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_in_index_hint_list5660	= { FOLLOW_index_hint_in_index_hint_list5660_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_index_hint_list5663  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_index_hint_list5663_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_index_hint_list5663	= { FOLLOW_COMMA_SYMBOL_in_index_hint_list5663_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_in_index_hint_list5665  */
static	ANTLR3_BITWORD FOLLOW_index_hint_in_index_hint_list5665_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_in_index_hint_list5665	= { FOLLOW_index_hint_in_index_hint_list5665_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_type_in_index_hint5676  */
static	ANTLR3_BITWORD FOLLOW_index_hint_type_in_index_hint5676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_type_in_index_hint5676	= { FOLLOW_index_hint_type_in_index_hint5676_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_or_index_in_index_hint5678  */
static	ANTLR3_BITWORD FOLLOW_key_or_index_in_index_hint5678_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_key_or_index_in_index_hint5678	= { FOLLOW_key_or_index_in_index_hint5678_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_clause_in_index_hint5680  */
static	ANTLR3_BITWORD FOLLOW_index_hint_clause_in_index_hint5680_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_clause_in_index_hint5680	= { FOLLOW_index_hint_clause_in_index_hint5680_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5683  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5683_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5683	= { FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5683_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_list_in_index_hint5685  */
static	ANTLR3_BITWORD FOLLOW_index_list_in_index_hint5685_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_list_in_index_hint5685	= { FOLLOW_index_list_in_index_hint5685_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5687  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5687_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5687	= { FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5687_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USE_SYMBOL_in_index_hint5692  */
static	ANTLR3_BITWORD FOLLOW_USE_SYMBOL_in_index_hint5692_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USE_SYMBOL_in_index_hint5692	= { FOLLOW_USE_SYMBOL_in_index_hint5692_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_or_index_in_index_hint5694  */
static	ANTLR3_BITWORD FOLLOW_key_or_index_in_index_hint5694_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_key_or_index_in_index_hint5694	= { FOLLOW_key_or_index_in_index_hint5694_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_hint_clause_in_index_hint5696  */
static	ANTLR3_BITWORD FOLLOW_index_hint_clause_in_index_hint5696_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_hint_clause_in_index_hint5696	= { FOLLOW_index_hint_clause_in_index_hint5696_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5699  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5699_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5699	= { FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5699_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_list_in_index_hint5701  */
static	ANTLR3_BITWORD FOLLOW_index_list_in_index_hint5701_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_list_in_index_hint5701	= { FOLLOW_index_list_in_index_hint5701_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5704  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5704	= { FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5704_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_index_hint_clause5741  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_index_hint_clause5741_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000800000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_index_hint_clause5741	= { FOLLOW_FOR_SYMBOL_in_index_hint_clause5741_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JOIN_SYMBOL_in_index_hint_clause5744  */
static	ANTLR3_BITWORD FOLLOW_JOIN_SYMBOL_in_index_hint_clause5744_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_JOIN_SYMBOL_in_index_hint_clause5744	= { FOLLOW_JOIN_SYMBOL_in_index_hint_clause5744_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ORDER_SYMBOL_in_index_hint_clause5748  */
static	ANTLR3_BITWORD FOLLOW_ORDER_SYMBOL_in_index_hint_clause5748_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_ORDER_SYMBOL_in_index_hint_clause5748	= { FOLLOW_ORDER_SYMBOL_in_index_hint_clause5748_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_index_hint_clause5750  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_index_hint_clause5750_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_index_hint_clause5750	= { FOLLOW_BY_SYMBOL_in_index_hint_clause5750_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_SYMBOL_in_index_hint_clause5754  */
static	ANTLR3_BITWORD FOLLOW_GROUP_SYMBOL_in_index_hint_clause5754_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_SYMBOL_in_index_hint_clause5754	= { FOLLOW_GROUP_SYMBOL_in_index_hint_clause5754_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_index_hint_clause5756  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_index_hint_clause5756_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_index_hint_clause5756	= { FOLLOW_BY_SYMBOL_in_index_hint_clause5756_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_list_element_in_index_list5766  */
static	ANTLR3_BITWORD FOLLOW_index_list_element_in_index_list5766_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_list_element_in_index_list5766	= { FOLLOW_index_list_element_in_index_list5766_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_index_list5769  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_index_list5769_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_index_list5769	= { FOLLOW_COMMA_SYMBOL_in_index_list5769_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_list_element_in_index_list5771  */
static	ANTLR3_BITWORD FOLLOW_index_list_element_in_index_list5771_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_list_element_in_index_list5771	= { FOLLOW_index_list_element_in_index_list5771_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_index_list_element5782  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_index_list_element5782_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_index_list_element5782	= { FOLLOW_identifier_in_index_list_element5782_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIMARY_SYMBOL_in_index_list_element5787  */
static	ANTLR3_BITWORD FOLLOW_PRIMARY_SYMBOL_in_index_list_element5787_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIMARY_SYMBOL_in_index_list_element5787	= { FOLLOW_PRIMARY_SYMBOL_in_index_list_element5787_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_update_statement5798  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_update_statement5798_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000600000000400), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_update_statement5798	= { FOLLOW_UPDATE_SYMBOL_in_update_statement5798_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOW_PRIORITY_SYMBOL_in_update_statement5800  */
static	ANTLR3_BITWORD FOLLOW_LOW_PRIORITY_SYMBOL_in_update_statement5800_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOW_PRIORITY_SYMBOL_in_update_statement5800	= { FOLLOW_LOW_PRIORITY_SYMBOL_in_update_statement5800_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SYMBOL_in_update_statement5803  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SYMBOL_in_update_statement5803_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000600000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SYMBOL_in_update_statement5803	= { FOLLOW_IGNORE_SYMBOL_in_update_statement5803_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_join_table_list_in_update_statement5806  */
static	ANTLR3_BITWORD FOLLOW_join_table_list_in_update_statement5806_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_join_table_list_in_update_statement5806	= { FOLLOW_join_table_list_in_update_statement5806_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_update_statement5810  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_update_statement5810_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_update_statement5810	= { FOLLOW_SET_SYMBOL_in_update_statement5810_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_list_with_default_in_update_statement5812  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_list_with_default_in_update_statement5812_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_list_with_default_in_update_statement5812	= { FOLLOW_column_assignment_list_with_default_in_update_statement5812_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_update_statement5814  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_update_statement5814_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_update_statement5814	= { FOLLOW_where_clause_in_update_statement5814_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_update_statement5817  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_update_statement5817_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_update_statement5817	= { FOLLOW_order_by_clause_in_update_statement5817_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_simple_limit_clause_in_update_statement5820  */
static	ANTLR3_BITWORD FOLLOW_simple_limit_clause_in_update_statement5820_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_simple_limit_clause_in_update_statement5820	= { FOLLOW_simple_limit_clause_in_update_statement5820_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_transaction_statement_in_transaction_or_locking_statement5832  */
static	ANTLR3_BITWORD FOLLOW_transaction_statement_in_transaction_or_locking_statement5832_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_transaction_statement_in_transaction_or_locking_statement5832	= { FOLLOW_transaction_statement_in_transaction_or_locking_statement5832_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_savepoint_statement_in_transaction_or_locking_statement5837  */
static	ANTLR3_BITWORD FOLLOW_savepoint_statement_in_transaction_or_locking_statement5837_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_savepoint_statement_in_transaction_or_locking_statement5837	= { FOLLOW_savepoint_statement_in_transaction_or_locking_statement5837_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lock_statement_in_transaction_or_locking_statement5842  */
static	ANTLR3_BITWORD FOLLOW_lock_statement_in_transaction_or_locking_statement5842_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_lock_statement_in_transaction_or_locking_statement5842	= { FOLLOW_lock_statement_in_transaction_or_locking_statement5842_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xa_statement_in_transaction_or_locking_statement5847  */
static	ANTLR3_BITWORD FOLLOW_xa_statement_in_transaction_or_locking_statement5847_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_xa_statement_in_transaction_or_locking_statement5847	= { FOLLOW_xa_statement_in_transaction_or_locking_statement5847_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_START_SYMBOL_in_transaction_statement5856  */
static	ANTLR3_BITWORD FOLLOW_START_SYMBOL_in_transaction_statement5856_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_START_SYMBOL_in_transaction_statement5856	= { FOLLOW_START_SYMBOL_in_transaction_statement5856_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRANSACTION_SYMBOL_in_transaction_statement5858  */
static	ANTLR3_BITWORD FOLLOW_TRANSACTION_SYMBOL_in_transaction_statement5858_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRANSACTION_SYMBOL_in_transaction_statement5858	= { FOLLOW_TRANSACTION_SYMBOL_in_transaction_statement5858_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_transaction_characteristic_in_transaction_statement5860  */
static	ANTLR3_BITWORD FOLLOW_transaction_characteristic_in_transaction_statement5860_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_transaction_characteristic_in_transaction_statement5860	= { FOLLOW_transaction_characteristic_in_transaction_statement5860_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMIT_SYMBOL_in_transaction_statement5866  */
static	ANTLR3_BITWORD FOLLOW_COMMIT_SYMBOL_in_transaction_statement5866_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMIT_SYMBOL_in_transaction_statement5866	= { FOLLOW_COMMIT_SYMBOL_in_transaction_statement5866_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WORK_SYMBOL_in_transaction_statement5868  */
static	ANTLR3_BITWORD FOLLOW_WORK_SYMBOL_in_transaction_statement5868_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_WORK_SYMBOL_in_transaction_statement5868	= { FOLLOW_WORK_SYMBOL_in_transaction_statement5868_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AND_SYMBOL_in_transaction_statement5872  */
static	ANTLR3_BITWORD FOLLOW_AND_SYMBOL_in_transaction_statement5872_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_AND_SYMBOL_in_transaction_statement5872	= { FOLLOW_AND_SYMBOL_in_transaction_statement5872_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_transaction_statement5874  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_transaction_statement5874_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_transaction_statement5874	= { FOLLOW_NO_SYMBOL_in_transaction_statement5874_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAIN_SYMBOL_in_transaction_statement5877  */
static	ANTLR3_BITWORD FOLLOW_CHAIN_SYMBOL_in_transaction_statement5877_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAIN_SYMBOL_in_transaction_statement5877	= { FOLLOW_CHAIN_SYMBOL_in_transaction_statement5877_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_transaction_statement5882  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_transaction_statement5882_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_transaction_statement5882	= { FOLLOW_NO_SYMBOL_in_transaction_statement5882_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELEASE_SYMBOL_in_transaction_statement5885  */
static	ANTLR3_BITWORD FOLLOW_RELEASE_SYMBOL_in_transaction_statement5885_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_RELEASE_SYMBOL_in_transaction_statement5885	= { FOLLOW_RELEASE_SYMBOL_in_transaction_statement5885_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROLLBACK_SYMBOL_in_transaction_statement5892  */
static	ANTLR3_BITWORD FOLLOW_ROLLBACK_SYMBOL_in_transaction_statement5892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_ROLLBACK_SYMBOL_in_transaction_statement5892	= { FOLLOW_ROLLBACK_SYMBOL_in_transaction_statement5892_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WORK_SYMBOL_in_transaction_statement5894  */
static	ANTLR3_BITWORD FOLLOW_WORK_SYMBOL_in_transaction_statement5894_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_WORK_SYMBOL_in_transaction_statement5894	= { FOLLOW_WORK_SYMBOL_in_transaction_statement5894_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AND_SYMBOL_in_transaction_statement5905  */
static	ANTLR3_BITWORD FOLLOW_AND_SYMBOL_in_transaction_statement5905_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_AND_SYMBOL_in_transaction_statement5905	= { FOLLOW_AND_SYMBOL_in_transaction_statement5905_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_transaction_statement5907  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_transaction_statement5907_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_transaction_statement5907	= { FOLLOW_NO_SYMBOL_in_transaction_statement5907_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAIN_SYMBOL_in_transaction_statement5910  */
static	ANTLR3_BITWORD FOLLOW_CHAIN_SYMBOL_in_transaction_statement5910_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAIN_SYMBOL_in_transaction_statement5910	= { FOLLOW_CHAIN_SYMBOL_in_transaction_statement5910_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_transaction_statement5915  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_transaction_statement5915_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_transaction_statement5915	= { FOLLOW_NO_SYMBOL_in_transaction_statement5915_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELEASE_SYMBOL_in_transaction_statement5918  */
static	ANTLR3_BITWORD FOLLOW_RELEASE_SYMBOL_in_transaction_statement5918_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_RELEASE_SYMBOL_in_transaction_statement5918	= { FOLLOW_RELEASE_SYMBOL_in_transaction_statement5918_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_transaction_statement5927  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_transaction_statement5927_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_transaction_statement5927	= { FOLLOW_TO_SYMBOL_in_transaction_statement5927_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SAVEPOINT_SYMBOL_in_transaction_statement5929  */
static	ANTLR3_BITWORD FOLLOW_SAVEPOINT_SYMBOL_in_transaction_statement5929_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SAVEPOINT_SYMBOL_in_transaction_statement5929	= { FOLLOW_SAVEPOINT_SYMBOL_in_transaction_statement5929_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_transaction_statement5932  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_transaction_statement5932_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_transaction_statement5932	= { FOLLOW_identifier_in_transaction_statement5932_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BEGIN_SYMBOL_in_begin_work5953  */
static	ANTLR3_BITWORD FOLLOW_BEGIN_SYMBOL_in_begin_work5953_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BEGIN_SYMBOL_in_begin_work5953	= { FOLLOW_BEGIN_SYMBOL_in_begin_work5953_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WORK_SYMBOL_in_begin_work5955  */
static	ANTLR3_BITWORD FOLLOW_WORK_SYMBOL_in_begin_work5955_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WORK_SYMBOL_in_begin_work5955	= { FOLLOW_WORK_SYMBOL_in_begin_work5955_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_transaction_characteristic5965  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_transaction_characteristic5965_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_transaction_characteristic5965	= { FOLLOW_WITH_SYMBOL_in_transaction_characteristic5965_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONSISTENT_SYMBOL_in_transaction_characteristic5967  */
static	ANTLR3_BITWORD FOLLOW_CONSISTENT_SYMBOL_in_transaction_characteristic5967_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONSISTENT_SYMBOL_in_transaction_characteristic5967	= { FOLLOW_CONSISTENT_SYMBOL_in_transaction_characteristic5967_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SNAPSHOT_SYMBOL_in_transaction_characteristic5969  */
static	ANTLR3_BITWORD FOLLOW_SNAPSHOT_SYMBOL_in_transaction_characteristic5969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SNAPSHOT_SYMBOL_in_transaction_characteristic5969	= { FOLLOW_SNAPSHOT_SYMBOL_in_transaction_characteristic5969_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_transaction_characteristic5976  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_transaction_characteristic5976_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_transaction_characteristic5976	= { FOLLOW_READ_SYMBOL_in_transaction_characteristic5976_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_transaction_characteristic5978  */
static	ANTLR3_BITWORD FOLLOW_set_in_transaction_characteristic5978_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_transaction_characteristic5978	= { FOLLOW_set_in_transaction_characteristic5978_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ISOLATION_SYMBOL_in_set_transaction_characteristic5993  */
static	ANTLR3_BITWORD FOLLOW_ISOLATION_SYMBOL_in_set_transaction_characteristic5993_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ISOLATION_SYMBOL_in_set_transaction_characteristic5993	= { FOLLOW_ISOLATION_SYMBOL_in_set_transaction_characteristic5993_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEVEL_SYMBOL_in_set_transaction_characteristic5995  */
static	ANTLR3_BITWORD FOLLOW_LEVEL_SYMBOL_in_set_transaction_characteristic5995_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000002000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LEVEL_SYMBOL_in_set_transaction_characteristic5995	= { FOLLOW_LEVEL_SYMBOL_in_set_transaction_characteristic5995_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_isolation_level_in_set_transaction_characteristic5997  */
static	ANTLR3_BITWORD FOLLOW_isolation_level_in_set_transaction_characteristic5997_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_isolation_level_in_set_transaction_characteristic5997	= { FOLLOW_isolation_level_in_set_transaction_characteristic5997_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_set_transaction_characteristic6004  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_set_transaction_characteristic6004_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_set_transaction_characteristic6004	= { FOLLOW_READ_SYMBOL_in_set_transaction_characteristic6004_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_set_transaction_characteristic6006  */
static	ANTLR3_BITWORD FOLLOW_set_in_set_transaction_characteristic6006_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_set_transaction_characteristic6006	= { FOLLOW_set_in_set_transaction_characteristic6006_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPEATABLE_SYMBOL_in_isolation_level6021  */
static	ANTLR3_BITWORD FOLLOW_REPEATABLE_SYMBOL_in_isolation_level6021_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_REPEATABLE_SYMBOL_in_isolation_level6021	= { FOLLOW_REPEATABLE_SYMBOL_in_isolation_level6021_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_isolation_level6023  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_isolation_level6023_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_isolation_level6023	= { FOLLOW_READ_SYMBOL_in_isolation_level6023_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_isolation_level6028  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_isolation_level6028_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_isolation_level6028	= { FOLLOW_READ_SYMBOL_in_isolation_level6028_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_isolation_level6030  */
static	ANTLR3_BITWORD FOLLOW_set_in_isolation_level6030_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_isolation_level6030	= { FOLLOW_set_in_isolation_level6030_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERIALIZABLE_SYMBOL_in_isolation_level6041  */
static	ANTLR3_BITWORD FOLLOW_SERIALIZABLE_SYMBOL_in_isolation_level6041_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SERIALIZABLE_SYMBOL_in_isolation_level6041	= { FOLLOW_SERIALIZABLE_SYMBOL_in_isolation_level6041_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6050  */
static	ANTLR3_BITWORD FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6050_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6050	= { FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6050_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_savepoint_statement6052  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_savepoint_statement6052_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_savepoint_statement6052	= { FOLLOW_identifier_in_savepoint_statement6052_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELEASE_SYMBOL_in_savepoint_statement6057  */
static	ANTLR3_BITWORD FOLLOW_RELEASE_SYMBOL_in_savepoint_statement6057_bits[]	= { ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RELEASE_SYMBOL_in_savepoint_statement6057	= { FOLLOW_RELEASE_SYMBOL_in_savepoint_statement6057_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6059  */
static	ANTLR3_BITWORD FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6059_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6059	= { FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6059_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_savepoint_statement6061  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_savepoint_statement6061_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_savepoint_statement6061	= { FOLLOW_identifier_in_savepoint_statement6061_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_lock_statement6070  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_lock_statement6070_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_lock_statement6070	= { FOLLOW_LOCK_SYMBOL_in_lock_statement6070_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_lock_statement6072  */
static	ANTLR3_BITWORD FOLLOW_set_in_lock_statement6072_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_lock_statement6072	= { FOLLOW_set_in_lock_statement6072_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lock_item_in_lock_statement6080  */
static	ANTLR3_BITWORD FOLLOW_lock_item_in_lock_statement6080_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_lock_item_in_lock_statement6080	= { FOLLOW_lock_item_in_lock_statement6080_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_lock_statement6083  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_lock_statement6083_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_lock_statement6083	= { FOLLOW_COMMA_SYMBOL_in_lock_statement6083_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lock_item_in_lock_statement6085  */
static	ANTLR3_BITWORD FOLLOW_lock_item_in_lock_statement6085_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_lock_item_in_lock_statement6085	= { FOLLOW_lock_item_in_lock_statement6085_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNLOCK_SYMBOL_in_lock_statement6092  */
static	ANTLR3_BITWORD FOLLOW_UNLOCK_SYMBOL_in_lock_statement6092_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_UNLOCK_SYMBOL_in_lock_statement6092	= { FOLLOW_UNLOCK_SYMBOL_in_lock_statement6092_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_lock_statement6094  */
static	ANTLR3_BITWORD FOLLOW_set_in_lock_statement6094_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_lock_statement6094	= { FOLLOW_set_in_lock_statement6094_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_lock_item6109  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_lock_item6109_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0010820000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_lock_item6109	= { FOLLOW_table_ref_in_lock_item6109_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_alias_in_lock_item6111  */
static	ANTLR3_BITWORD FOLLOW_table_alias_in_lock_item6111_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_alias_in_lock_item6111	= { FOLLOW_table_alias_in_lock_item6111_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_lock_option_in_lock_item6114  */
static	ANTLR3_BITWORD FOLLOW_lock_option_in_lock_item6114_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_lock_option_in_lock_item6114	= { FOLLOW_lock_option_in_lock_item6114_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_lock_option6123  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_lock_option6123_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_lock_option6123	= { FOLLOW_READ_SYMBOL_in_lock_option6123_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCAL_SYMBOL_in_lock_option6125  */
static	ANTLR3_BITWORD FOLLOW_LOCAL_SYMBOL_in_lock_option6125_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCAL_SYMBOL_in_lock_option6125	= { FOLLOW_LOCAL_SYMBOL_in_lock_option6125_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOW_PRIORITY_SYMBOL_in_lock_option6131  */
static	ANTLR3_BITWORD FOLLOW_LOW_PRIORITY_SYMBOL_in_lock_option6131_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOW_PRIORITY_SYMBOL_in_lock_option6131	= { FOLLOW_LOW_PRIORITY_SYMBOL_in_lock_option6131_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WRITE_SYMBOL_in_lock_option6134  */
static	ANTLR3_BITWORD FOLLOW_WRITE_SYMBOL_in_lock_option6134_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WRITE_SYMBOL_in_lock_option6134	= { FOLLOW_WRITE_SYMBOL_in_lock_option6134_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_XA_SYMBOL_in_xa_statement6144  */
static	ANTLR3_BITWORD FOLLOW_XA_SYMBOL_in_xa_statement6144_bits[]	= { ANTLR3_UINT64_LIT(0x0008084000088100), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_XA_SYMBOL_in_xa_statement6144	= { FOLLOW_XA_SYMBOL_in_xa_statement6144_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_xa_statement6153  */
static	ANTLR3_BITWORD FOLLOW_set_in_xa_statement6153_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_xa_statement6153	= { FOLLOW_set_in_xa_statement6153_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xid_in_xa_statement6161  */
static	ANTLR3_BITWORD FOLLOW_xid_in_xa_statement6161_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_xid_in_xa_statement6161	= { FOLLOW_xid_in_xa_statement6161_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_xa_statement6177  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_xa_statement6177_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_xa_statement6177	= { FOLLOW_END_SYMBOL_in_xa_statement6177_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xid_in_xa_statement6179  */
static	ANTLR3_BITWORD FOLLOW_xid_in_xa_statement6179_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_xid_in_xa_statement6179	= { FOLLOW_xid_in_xa_statement6179_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUSPEND_SYMBOL_in_xa_statement6182  */
static	ANTLR3_BITWORD FOLLOW_SUSPEND_SYMBOL_in_xa_statement6182_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUSPEND_SYMBOL_in_xa_statement6182	= { FOLLOW_SUSPEND_SYMBOL_in_xa_statement6182_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_xa_statement6185  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_xa_statement6185_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_xa_statement6185	= { FOLLOW_FOR_SYMBOL_in_xa_statement6185_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MIGRATE_SYMBOL_in_xa_statement6187  */
static	ANTLR3_BITWORD FOLLOW_MIGRATE_SYMBOL_in_xa_statement6187_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MIGRATE_SYMBOL_in_xa_statement6187	= { FOLLOW_MIGRATE_SYMBOL_in_xa_statement6187_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PREPARE_SYMBOL_in_xa_statement6198  */
static	ANTLR3_BITWORD FOLLOW_PREPARE_SYMBOL_in_xa_statement6198_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PREPARE_SYMBOL_in_xa_statement6198	= { FOLLOW_PREPARE_SYMBOL_in_xa_statement6198_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xid_in_xa_statement6200  */
static	ANTLR3_BITWORD FOLLOW_xid_in_xa_statement6200_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_xid_in_xa_statement6200	= { FOLLOW_xid_in_xa_statement6200_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMIT_SYMBOL_in_xa_statement6207  */
static	ANTLR3_BITWORD FOLLOW_COMMIT_SYMBOL_in_xa_statement6207_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMIT_SYMBOL_in_xa_statement6207	= { FOLLOW_COMMIT_SYMBOL_in_xa_statement6207_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xid_in_xa_statement6209  */
static	ANTLR3_BITWORD FOLLOW_xid_in_xa_statement6209_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_xid_in_xa_statement6209	= { FOLLOW_xid_in_xa_statement6209_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ONE_SYMBOL_in_xa_statement6212  */
static	ANTLR3_BITWORD FOLLOW_ONE_SYMBOL_in_xa_statement6212_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ONE_SYMBOL_in_xa_statement6212	= { FOLLOW_ONE_SYMBOL_in_xa_statement6212_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PHASE_SYMBOL_in_xa_statement6214  */
static	ANTLR3_BITWORD FOLLOW_PHASE_SYMBOL_in_xa_statement6214_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PHASE_SYMBOL_in_xa_statement6214	= { FOLLOW_PHASE_SYMBOL_in_xa_statement6214_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROLLBACK_SYMBOL_in_xa_statement6223  */
static	ANTLR3_BITWORD FOLLOW_ROLLBACK_SYMBOL_in_xa_statement6223_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROLLBACK_SYMBOL_in_xa_statement6223	= { FOLLOW_ROLLBACK_SYMBOL_in_xa_statement6223_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xid_in_xa_statement6225  */
static	ANTLR3_BITWORD FOLLOW_xid_in_xa_statement6225_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_xid_in_xa_statement6225	= { FOLLOW_xid_in_xa_statement6225_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RECOVER_SYMBOL_in_xa_statement6232  */
static	ANTLR3_BITWORD FOLLOW_RECOVER_SYMBOL_in_xa_statement6232_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000001000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RECOVER_SYMBOL_in_xa_statement6232	= { FOLLOW_RECOVER_SYMBOL_in_xa_statement6232_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_xa_convert_in_xa_statement6234  */
static	ANTLR3_BITWORD FOLLOW_xa_convert_in_xa_statement6234_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_xa_convert_in_xa_statement6234	= { FOLLOW_xa_convert_in_xa_statement6234_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONVERT_SYMBOL_in_xa_convert6250  */
static	ANTLR3_BITWORD FOLLOW_CONVERT_SYMBOL_in_xa_convert6250_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONVERT_SYMBOL_in_xa_convert6250	= { FOLLOW_CONVERT_SYMBOL_in_xa_convert6250_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_XID_SYMBOL_in_xa_convert6252  */
static	ANTLR3_BITWORD FOLLOW_XID_SYMBOL_in_xa_convert6252_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_XID_SYMBOL_in_xa_convert6252	= { FOLLOW_XID_SYMBOL_in_xa_convert6252_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_xid6268  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_xid6268_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_xid6268	= { FOLLOW_text_string_in_xid6268_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_xid6271  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_xid6271_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_xid6271	= { FOLLOW_COMMA_SYMBOL_in_xid6271_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_xid6273  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_xid6273_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_xid6273	= { FOLLOW_text_string_in_xid6273_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_xid6276  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_xid6276_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_xid6276	= { FOLLOW_COMMA_SYMBOL_in_xid6276_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_xid6278  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_xid6278_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_xid6278	= { FOLLOW_ulong_number_in_xid6278_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PURGE_SYMBOL_in_replication_statement6293  */
static	ANTLR3_BITWORD FOLLOW_PURGE_SYMBOL_in_replication_statement6293_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_PURGE_SYMBOL_in_replication_statement6293	= { FOLLOW_PURGE_SYMBOL_in_replication_statement6293_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_replication_statement6295  */
static	ANTLR3_BITWORD FOLLOW_set_in_replication_statement6295_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_replication_statement6295	= { FOLLOW_set_in_replication_statement6295_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGS_SYMBOL_in_replication_statement6303  */
static	ANTLR3_BITWORD FOLLOW_LOGS_SYMBOL_in_replication_statement6303_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGS_SYMBOL_in_replication_statement6303	= { FOLLOW_LOGS_SYMBOL_in_replication_statement6303_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_replication_statement6306  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_replication_statement6306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_replication_statement6306	= { FOLLOW_TO_SYMBOL_in_replication_statement6306_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_replication_statement6308  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_replication_statement6308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_replication_statement6308	= { FOLLOW_string_literal_in_replication_statement6308_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BEFORE_SYMBOL_in_replication_statement6312  */
static	ANTLR3_BITWORD FOLLOW_BEFORE_SYMBOL_in_replication_statement6312_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BEFORE_SYMBOL_in_replication_statement6312	= { FOLLOW_BEFORE_SYMBOL_in_replication_statement6312_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_replication_statement6314  */
static	ANTLR3_BITWORD FOLLOW_expression_in_replication_statement6314_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_replication_statement6314	= { FOLLOW_expression_in_replication_statement6314_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_change_master_in_replication_statement6320  */
static	ANTLR3_BITWORD FOLLOW_change_master_in_replication_statement6320_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_change_master_in_replication_statement6320	= { FOLLOW_change_master_in_replication_statement6320_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_change_replication_in_replication_statement6327  */
static	ANTLR3_BITWORD FOLLOW_change_replication_in_replication_statement6327_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_change_replication_in_replication_statement6327	= { FOLLOW_change_replication_in_replication_statement6327_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_in_replication_statement6335  */
static	ANTLR3_BITWORD FOLLOW_slave_in_replication_statement6335_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_in_replication_statement6335	= { FOLLOW_slave_in_replication_statement6335_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_replication_load_in_replication_statement6342  */
static	ANTLR3_BITWORD FOLLOW_replication_load_in_replication_statement6342_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_replication_load_in_replication_statement6342	= { FOLLOW_replication_load_in_replication_statement6342_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_group_replication_in_replication_statement6349  */
static	ANTLR3_BITWORD FOLLOW_group_replication_in_replication_statement6349_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_group_replication_in_replication_statement6349	= { FOLLOW_group_replication_in_replication_statement6349_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOAD_SYMBOL_in_replication_load6358  */
static	ANTLR3_BITWORD FOLLOW_LOAD_SYMBOL_in_replication_load6358_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_LOAD_SYMBOL_in_replication_load6358	= { FOLLOW_LOAD_SYMBOL_in_replication_load6358_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_replication_load6361  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_replication_load6361_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_replication_load6361	= { FOLLOW_DATA_SYMBOL_in_replication_load6361_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_replication_load6365  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_replication_load6365_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_replication_load6365	= { FOLLOW_TABLE_SYMBOL_in_replication_load6365_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_replication_load6367  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_replication_load6367_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_replication_load6367	= { FOLLOW_table_ref_in_replication_load6367_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_replication_load6370  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_replication_load6370_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_replication_load6370	= { FOLLOW_FROM_SYMBOL_in_replication_load6370_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SYMBOL_in_replication_load6372  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SYMBOL_in_replication_load6372_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SYMBOL_in_replication_load6372	= { FOLLOW_MASTER_SYMBOL_in_replication_load6372_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANGE_SYMBOL_in_change_master6381  */
static	ANTLR3_BITWORD FOLLOW_CHANGE_SYMBOL_in_change_master6381_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANGE_SYMBOL_in_change_master6381	= { FOLLOW_CHANGE_SYMBOL_in_change_master6381_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SYMBOL_in_change_master6383  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SYMBOL_in_change_master6383_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SYMBOL_in_change_master6383	= { FOLLOW_MASTER_SYMBOL_in_change_master6383_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_change_master6385  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_change_master6385_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0xFC00000200000000), ANTLR3_UINT64_LIT(0x0000000000003FFD), ANTLR3_UINT64_LIT(0x0000000000300000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001800) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_change_master6385	= { FOLLOW_TO_SYMBOL_in_change_master6385_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_change_master_options_in_change_master6387  */
static	ANTLR3_BITWORD FOLLOW_change_master_options_in_change_master6387_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_change_master_options_in_change_master6387	= { FOLLOW_change_master_options_in_change_master6387_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_change_master6389  */
static	ANTLR3_BITWORD FOLLOW_channel_in_change_master6389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_change_master6389	= { FOLLOW_channel_in_change_master6389_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_master_option_in_change_master_options6399  */
static	ANTLR3_BITWORD FOLLOW_master_option_in_change_master_options6399_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_master_option_in_change_master_options6399	= { FOLLOW_master_option_in_change_master_options6399_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_change_master_options6402  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_change_master_options6402_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0xFC00000200000000), ANTLR3_UINT64_LIT(0x0000000000003FFD), ANTLR3_UINT64_LIT(0x0000000000300000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001800) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_change_master_options6402	= { FOLLOW_COMMA_SYMBOL_in_change_master_options6402_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_master_option_in_change_master_options6404  */
static	ANTLR3_BITWORD FOLLOW_master_option_in_change_master_options6404_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_master_option_in_change_master_options6404	= { FOLLOW_master_option_in_change_master_options6404_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_HOST_SYMBOL_in_master_option6415  */
static	ANTLR3_BITWORD FOLLOW_MASTER_HOST_SYMBOL_in_master_option6415_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_HOST_SYMBOL_in_master_option6415	= { FOLLOW_MASTER_HOST_SYMBOL_in_master_option6415_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6417  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6417_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6417	= { FOLLOW_EQUAL_OPERATOR_in_master_option6417_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6419  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6419	= { FOLLOW_text_string_no_linebreak_in_master_option6419_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_BIND_SYMBOL_in_master_option6424  */
static	ANTLR3_BITWORD FOLLOW_MASTER_BIND_SYMBOL_in_master_option6424_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_BIND_SYMBOL_in_master_option6424	= { FOLLOW_MASTER_BIND_SYMBOL_in_master_option6424_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6426  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6426_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6426	= { FOLLOW_EQUAL_OPERATOR_in_master_option6426_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6428  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6428_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6428	= { FOLLOW_text_string_no_linebreak_in_master_option6428_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_USER_SYMBOL_in_master_option6433  */
static	ANTLR3_BITWORD FOLLOW_MASTER_USER_SYMBOL_in_master_option6433_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_USER_SYMBOL_in_master_option6433	= { FOLLOW_MASTER_USER_SYMBOL_in_master_option6433_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6435  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6435_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6435	= { FOLLOW_EQUAL_OPERATOR_in_master_option6435_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6437  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6437_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6437	= { FOLLOW_text_string_no_linebreak_in_master_option6437_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_PASSWORD_SYMBOL_in_master_option6442  */
static	ANTLR3_BITWORD FOLLOW_MASTER_PASSWORD_SYMBOL_in_master_option6442_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_PASSWORD_SYMBOL_in_master_option6442	= { FOLLOW_MASTER_PASSWORD_SYMBOL_in_master_option6442_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6444  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6444_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6444	= { FOLLOW_EQUAL_OPERATOR_in_master_option6444_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6446  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6446_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6446	= { FOLLOW_text_string_no_linebreak_in_master_option6446_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_PORT_SYMBOL_in_master_option6451  */
static	ANTLR3_BITWORD FOLLOW_MASTER_PORT_SYMBOL_in_master_option6451_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_PORT_SYMBOL_in_master_option6451	= { FOLLOW_MASTER_PORT_SYMBOL_in_master_option6451_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6453  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6453_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6453	= { FOLLOW_EQUAL_OPERATOR_in_master_option6453_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6455  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6455_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6455	= { FOLLOW_ulong_number_in_master_option6455_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_CONNECT_RETRY_SYMBOL_in_master_option6460  */
static	ANTLR3_BITWORD FOLLOW_MASTER_CONNECT_RETRY_SYMBOL_in_master_option6460_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_CONNECT_RETRY_SYMBOL_in_master_option6460	= { FOLLOW_MASTER_CONNECT_RETRY_SYMBOL_in_master_option6460_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6462  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6462_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6462	= { FOLLOW_EQUAL_OPERATOR_in_master_option6462_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6464  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6464	= { FOLLOW_ulong_number_in_master_option6464_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_RETRY_COUNT_SYMBOL_in_master_option6469  */
static	ANTLR3_BITWORD FOLLOW_MASTER_RETRY_COUNT_SYMBOL_in_master_option6469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_RETRY_COUNT_SYMBOL_in_master_option6469	= { FOLLOW_MASTER_RETRY_COUNT_SYMBOL_in_master_option6469_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6471  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6471	= { FOLLOW_EQUAL_OPERATOR_in_master_option6471_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6473  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6473_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6473	= { FOLLOW_ulong_number_in_master_option6473_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_DELAY_SYMBOL_in_master_option6478  */
static	ANTLR3_BITWORD FOLLOW_MASTER_DELAY_SYMBOL_in_master_option6478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_DELAY_SYMBOL_in_master_option6478	= { FOLLOW_MASTER_DELAY_SYMBOL_in_master_option6478_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6480  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6480_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6480	= { FOLLOW_EQUAL_OPERATOR_in_master_option6480_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6482  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6482_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6482	= { FOLLOW_ulong_number_in_master_option6482_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_SYMBOL_in_master_option6487  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_SYMBOL_in_master_option6487_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_SYMBOL_in_master_option6487	= { FOLLOW_MASTER_SSL_SYMBOL_in_master_option6487_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6489  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6489_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6489	= { FOLLOW_EQUAL_OPERATOR_in_master_option6489_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6491  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6491_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6491	= { FOLLOW_ulong_number_in_master_option6491_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CA_SYMBOL_in_master_option6496  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CA_SYMBOL_in_master_option6496_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CA_SYMBOL_in_master_option6496	= { FOLLOW_MASTER_SSL_CA_SYMBOL_in_master_option6496_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6498  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6498_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6498	= { FOLLOW_EQUAL_OPERATOR_in_master_option6498_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6500  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6500_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6500	= { FOLLOW_text_string_no_linebreak_in_master_option6500_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CAPATH_SYMBOL_in_master_option6505  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CAPATH_SYMBOL_in_master_option6505_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CAPATH_SYMBOL_in_master_option6505	= { FOLLOW_MASTER_SSL_CAPATH_SYMBOL_in_master_option6505_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6507  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6507_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6507	= { FOLLOW_EQUAL_OPERATOR_in_master_option6507_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6509  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6509_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6509	= { FOLLOW_text_string_no_linebreak_in_master_option6509_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CERT_SYMBOL_in_master_option6514  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CERT_SYMBOL_in_master_option6514_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CERT_SYMBOL_in_master_option6514	= { FOLLOW_MASTER_SSL_CERT_SYMBOL_in_master_option6514_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6516  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6516_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6516	= { FOLLOW_EQUAL_OPERATOR_in_master_option6516_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6518  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6518	= { FOLLOW_text_string_no_linebreak_in_master_option6518_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CIPHER_SYMBOL_in_master_option6523  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CIPHER_SYMBOL_in_master_option6523_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CIPHER_SYMBOL_in_master_option6523	= { FOLLOW_MASTER_SSL_CIPHER_SYMBOL_in_master_option6523_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6525  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6525	= { FOLLOW_EQUAL_OPERATOR_in_master_option6525_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6527  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6527	= { FOLLOW_text_string_no_linebreak_in_master_option6527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_KEY_SYMBOL_in_master_option6532  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_KEY_SYMBOL_in_master_option6532_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_KEY_SYMBOL_in_master_option6532	= { FOLLOW_MASTER_SSL_KEY_SYMBOL_in_master_option6532_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6534  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6534_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6534	= { FOLLOW_EQUAL_OPERATOR_in_master_option6534_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6536  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6536_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6536	= { FOLLOW_text_string_no_linebreak_in_master_option6536_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL_in_master_option6541  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL_in_master_option6541_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL_in_master_option6541	= { FOLLOW_MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL_in_master_option6541_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6543  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6543_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6543	= { FOLLOW_EQUAL_OPERATOR_in_master_option6543_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6545  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6545_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6545	= { FOLLOW_ulong_number_in_master_option6545_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CRL_SYMBOL_in_master_option6550  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CRL_SYMBOL_in_master_option6550_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CRL_SYMBOL_in_master_option6550	= { FOLLOW_MASTER_SSL_CRL_SYMBOL_in_master_option6550_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6552  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6552_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6552	= { FOLLOW_EQUAL_OPERATOR_in_master_option6552_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_master_option6554  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_master_option6554_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_master_option6554	= { FOLLOW_string_literal_in_master_option6554_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SSL_CRLPATH_SYMBOL_in_master_option6559  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SSL_CRLPATH_SYMBOL_in_master_option6559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SSL_CRLPATH_SYMBOL_in_master_option6559	= { FOLLOW_MASTER_SSL_CRLPATH_SYMBOL_in_master_option6559_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6561  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6561_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6561	= { FOLLOW_EQUAL_OPERATOR_in_master_option6561_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_option6563  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_option6563_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_option6563	= { FOLLOW_text_string_no_linebreak_in_master_option6563_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_HEARTBEAT_PERIOD_SYMBOL_in_master_option6568  */
static	ANTLR3_BITWORD FOLLOW_MASTER_HEARTBEAT_PERIOD_SYMBOL_in_master_option6568_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_HEARTBEAT_PERIOD_SYMBOL_in_master_option6568	= { FOLLOW_MASTER_HEARTBEAT_PERIOD_SYMBOL_in_master_option6568_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6570  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6570_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6570	= { FOLLOW_EQUAL_OPERATOR_in_master_option6570_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6572  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6572_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6572	= { FOLLOW_ulong_number_in_master_option6572_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SERVER_IDS_SYMBOL_in_master_option6577  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SERVER_IDS_SYMBOL_in_master_option6577_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SERVER_IDS_SYMBOL_in_master_option6577	= { FOLLOW_IGNORE_SERVER_IDS_SYMBOL_in_master_option6577_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6579  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6579_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6579	= { FOLLOW_EQUAL_OPERATOR_in_master_option6579_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_server_id_list_in_master_option6581  */
static	ANTLR3_BITWORD FOLLOW_server_id_list_in_master_option6581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_server_id_list_in_master_option6581	= { FOLLOW_server_id_list_in_master_option6581_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_AUTO_POSITION_SYMBOL_in_master_option6586  */
static	ANTLR3_BITWORD FOLLOW_MASTER_AUTO_POSITION_SYMBOL_in_master_option6586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_AUTO_POSITION_SYMBOL_in_master_option6586	= { FOLLOW_MASTER_AUTO_POSITION_SYMBOL_in_master_option6586_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_option6588  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_option6588_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_option6588	= { FOLLOW_EQUAL_OPERATOR_in_master_option6588_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_option6590  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_option6590_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_option6590	= { FOLLOW_ulong_number_in_master_option6590_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_master_file_def_in_master_option6595  */
static	ANTLR3_BITWORD FOLLOW_master_file_def_in_master_option6595_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_master_file_def_in_master_option6595	= { FOLLOW_master_file_def_in_master_option6595_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_LOG_FILE_SYMBOL_in_master_file_def6604  */
static	ANTLR3_BITWORD FOLLOW_MASTER_LOG_FILE_SYMBOL_in_master_file_def6604_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_LOG_FILE_SYMBOL_in_master_file_def6604	= { FOLLOW_MASTER_LOG_FILE_SYMBOL_in_master_file_def6604_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_file_def6606  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_file_def6606_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_file_def6606	= { FOLLOW_EQUAL_OPERATOR_in_master_file_def6606_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_file_def6608  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_file_def6608_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_file_def6608	= { FOLLOW_text_string_no_linebreak_in_master_file_def6608_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_LOG_POS_SYMBOL_in_master_file_def6613  */
static	ANTLR3_BITWORD FOLLOW_MASTER_LOG_POS_SYMBOL_in_master_file_def6613_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_LOG_POS_SYMBOL_in_master_file_def6613	= { FOLLOW_MASTER_LOG_POS_SYMBOL_in_master_file_def6613_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_file_def6615  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_file_def6615_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_file_def6615	= { FOLLOW_EQUAL_OPERATOR_in_master_file_def6615_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulonglong_number_in_master_file_def6617  */
static	ANTLR3_BITWORD FOLLOW_ulonglong_number_in_master_file_def6617_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulonglong_number_in_master_file_def6617	= { FOLLOW_ulonglong_number_in_master_file_def6617_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELAY_LOG_FILE_SYMBOL_in_master_file_def6622  */
static	ANTLR3_BITWORD FOLLOW_RELAY_LOG_FILE_SYMBOL_in_master_file_def6622_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_RELAY_LOG_FILE_SYMBOL_in_master_file_def6622	= { FOLLOW_RELAY_LOG_FILE_SYMBOL_in_master_file_def6622_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_file_def6624  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_file_def6624_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_file_def6624	= { FOLLOW_EQUAL_OPERATOR_in_master_file_def6624_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_master_file_def6626  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_master_file_def6626_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_master_file_def6626	= { FOLLOW_text_string_no_linebreak_in_master_file_def6626_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELAY_LOG_POS_SYMBOL_in_master_file_def6631  */
static	ANTLR3_BITWORD FOLLOW_RELAY_LOG_POS_SYMBOL_in_master_file_def6631_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_RELAY_LOG_POS_SYMBOL_in_master_file_def6631	= { FOLLOW_RELAY_LOG_POS_SYMBOL_in_master_file_def6631_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_master_file_def6633  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_master_file_def6633_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_master_file_def6633	= { FOLLOW_EQUAL_OPERATOR_in_master_file_def6633_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_master_file_def6635  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_master_file_def6635_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_master_file_def6635	= { FOLLOW_ulong_number_in_master_file_def6635_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_server_id_list6644  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_server_id_list6644_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_server_id_list6644	= { FOLLOW_OPEN_PAR_SYMBOL_in_server_id_list6644_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_server_id_list6647  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_server_id_list6647_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_server_id_list6647	= { FOLLOW_ulong_number_in_server_id_list6647_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_server_id_list6650  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_server_id_list6650_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_server_id_list6650	= { FOLLOW_COMMA_SYMBOL_in_server_id_list6650_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_server_id_list6652  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_server_id_list6652_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_server_id_list6652	= { FOLLOW_ulong_number_in_server_id_list6652_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_server_id_list6658  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_server_id_list6658_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_server_id_list6658	= { FOLLOW_CLOSE_PAR_SYMBOL_in_server_id_list6658_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANGE_SYMBOL_in_change_replication6667  */
static	ANTLR3_BITWORD FOLLOW_CHANGE_SYMBOL_in_change_replication6667_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANGE_SYMBOL_in_change_replication6667	= { FOLLOW_CHANGE_SYMBOL_in_change_replication6667_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATION_SYMBOL_in_change_replication6669  */
static	ANTLR3_BITWORD FOLLOW_REPLICATION_SYMBOL_in_change_replication6669_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATION_SYMBOL_in_change_replication6669	= { FOLLOW_REPLICATION_SYMBOL_in_change_replication6669_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FILTER_SYMBOL_in_change_replication6671  */
static	ANTLR3_BITWORD FOLLOW_FILTER_SYMBOL_in_change_replication6671_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000003F8000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FILTER_SYMBOL_in_change_replication6671	= { FOLLOW_FILTER_SYMBOL_in_change_replication6671_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_definition_in_change_replication6673  */
static	ANTLR3_BITWORD FOLLOW_filter_definition_in_change_replication6673_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_definition_in_change_replication6673	= { FOLLOW_filter_definition_in_change_replication6673_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_change_replication6676  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_change_replication6676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000003F8000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_change_replication6676	= { FOLLOW_COMMA_SYMBOL_in_change_replication6676_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_definition_in_change_replication6678  */
static	ANTLR3_BITWORD FOLLOW_filter_definition_in_change_replication6678_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_definition_in_change_replication6678	= { FOLLOW_filter_definition_in_change_replication6678_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_DO_DB_SYMBOL_in_filter_definition6689  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_DO_DB_SYMBOL_in_filter_definition6689_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_DO_DB_SYMBOL_in_filter_definition6689	= { FOLLOW_REPLICATE_DO_DB_SYMBOL_in_filter_definition6689_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6691  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6691_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6691	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6691_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6693  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6693_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6693	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6693_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_db_list_in_filter_definition6695  */
static	ANTLR3_BITWORD FOLLOW_filter_db_list_in_filter_definition6695_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_db_list_in_filter_definition6695	= { FOLLOW_filter_db_list_in_filter_definition6695_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6698  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6698_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6698	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6698_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_IGNORE_DB_SYMBOL_in_filter_definition6703  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_IGNORE_DB_SYMBOL_in_filter_definition6703_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_IGNORE_DB_SYMBOL_in_filter_definition6703	= { FOLLOW_REPLICATE_IGNORE_DB_SYMBOL_in_filter_definition6703_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6705  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6705_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6705	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6705_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6707  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6707_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6707	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6707_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_db_list_in_filter_definition6709  */
static	ANTLR3_BITWORD FOLLOW_filter_db_list_in_filter_definition6709_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_db_list_in_filter_definition6709	= { FOLLOW_filter_db_list_in_filter_definition6709_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6712  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6712_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6712	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6712_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_DO_TABLE_SYMBOL_in_filter_definition6717  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_DO_TABLE_SYMBOL_in_filter_definition6717_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_DO_TABLE_SYMBOL_in_filter_definition6717	= { FOLLOW_REPLICATE_DO_TABLE_SYMBOL_in_filter_definition6717_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6719  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6719_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6719	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6719_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6721  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6721_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6721	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6721_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_table_list_in_filter_definition6723  */
static	ANTLR3_BITWORD FOLLOW_filter_table_list_in_filter_definition6723_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_table_list_in_filter_definition6723	= { FOLLOW_filter_table_list_in_filter_definition6723_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6726  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6726_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6726	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6726_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_IGNORE_TABLE_SYMBOL_in_filter_definition6731  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_IGNORE_TABLE_SYMBOL_in_filter_definition6731_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_IGNORE_TABLE_SYMBOL_in_filter_definition6731	= { FOLLOW_REPLICATE_IGNORE_TABLE_SYMBOL_in_filter_definition6731_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6733  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6733_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6733	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6733_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6735  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6735_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6735	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6735_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_table_list_in_filter_definition6737  */
static	ANTLR3_BITWORD FOLLOW_filter_table_list_in_filter_definition6737_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_table_list_in_filter_definition6737	= { FOLLOW_filter_table_list_in_filter_definition6737_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6740  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6740_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6740	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6740_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_WILD_DO_TABLE_SYMBOL_in_filter_definition6745  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_WILD_DO_TABLE_SYMBOL_in_filter_definition6745_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_WILD_DO_TABLE_SYMBOL_in_filter_definition6745	= { FOLLOW_REPLICATE_WILD_DO_TABLE_SYMBOL_in_filter_definition6745_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6747  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6747_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6747	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6747_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6749  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6749_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x8020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6749	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6749_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_string_list_in_filter_definition6751  */
static	ANTLR3_BITWORD FOLLOW_filter_string_list_in_filter_definition6751_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_string_list_in_filter_definition6751	= { FOLLOW_filter_string_list_in_filter_definition6751_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6754  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6754_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6754	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6754_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_WILD_IGNORE_TABLE_SYMBOL_in_filter_definition6759  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_WILD_IGNORE_TABLE_SYMBOL_in_filter_definition6759_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_WILD_IGNORE_TABLE_SYMBOL_in_filter_definition6759	= { FOLLOW_REPLICATE_WILD_IGNORE_TABLE_SYMBOL_in_filter_definition6759_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6761  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6761_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6761	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6761_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6763  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6763_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x8020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6763	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6763_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_string_list_in_filter_definition6765  */
static	ANTLR3_BITWORD FOLLOW_filter_string_list_in_filter_definition6765_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_string_list_in_filter_definition6765	= { FOLLOW_filter_string_list_in_filter_definition6765_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6768  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6768_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6768	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6768_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATE_REWRITE_DB_SYMBOL_in_filter_definition6773  */
static	ANTLR3_BITWORD FOLLOW_REPLICATE_REWRITE_DB_SYMBOL_in_filter_definition6773_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATE_REWRITE_DB_SYMBOL_in_filter_definition6773	= { FOLLOW_REPLICATE_REWRITE_DB_SYMBOL_in_filter_definition6773_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_filter_definition6775  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_filter_definition6775_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_filter_definition6775	= { FOLLOW_EQUAL_OPERATOR_in_filter_definition6775_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6777  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6777_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6777	= { FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6777_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_db_pair_list_in_filter_definition6779  */
static	ANTLR3_BITWORD FOLLOW_filter_db_pair_list_in_filter_definition6779_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_db_pair_list_in_filter_definition6779	= { FOLLOW_filter_db_pair_list_in_filter_definition6779_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6782  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6782_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6782	= { FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6782_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_filter_db_list6791  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_filter_db_list6791_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_filter_db_list6791	= { FOLLOW_schema_ref_in_filter_db_list6791_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_filter_db_list6794  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_filter_db_list6794_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_filter_db_list6794	= { FOLLOW_COMMA_SYMBOL_in_filter_db_list6794_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_filter_db_list6796  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_filter_db_list6796_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_filter_db_list6796	= { FOLLOW_schema_ref_in_filter_db_list6796_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_table_ref_in_filter_table_list6807  */
static	ANTLR3_BITWORD FOLLOW_filter_table_ref_in_filter_table_list6807_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_table_ref_in_filter_table_list6807	= { FOLLOW_filter_table_ref_in_filter_table_list6807_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_filter_table_list6810  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_filter_table_list6810_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_filter_table_list6810	= { FOLLOW_COMMA_SYMBOL_in_filter_table_list6810_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_table_ref_in_filter_table_list6812  */
static	ANTLR3_BITWORD FOLLOW_filter_table_ref_in_filter_table_list6812_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_table_ref_in_filter_table_list6812	= { FOLLOW_filter_table_ref_in_filter_table_list6812_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_wild_db_table_string_in_filter_string_list6823  */
static	ANTLR3_BITWORD FOLLOW_filter_wild_db_table_string_in_filter_string_list6823_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_wild_db_table_string_in_filter_string_list6823	= { FOLLOW_filter_wild_db_table_string_in_filter_string_list6823_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_filter_string_list6826  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_filter_string_list6826_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_filter_string_list6826	= { FOLLOW_COMMA_SYMBOL_in_filter_string_list6826_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_filter_wild_db_table_string_in_filter_string_list6828  */
static	ANTLR3_BITWORD FOLLOW_filter_wild_db_table_string_in_filter_string_list6828_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_filter_wild_db_table_string_in_filter_string_list6828	= { FOLLOW_filter_wild_db_table_string_in_filter_string_list6828_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_filter_wild_db_table_string6839  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_filter_wild_db_table_string6839_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_filter_wild_db_table_string6839	= { FOLLOW_text_string_no_linebreak_in_filter_wild_db_table_string6839_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_identifier_pair_in_filter_db_pair_list6849  */
static	ANTLR3_BITWORD FOLLOW_schema_identifier_pair_in_filter_db_pair_list6849_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_identifier_pair_in_filter_db_pair_list6849	= { FOLLOW_schema_identifier_pair_in_filter_db_pair_list6849_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_filter_db_pair_list6852  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_filter_db_pair_list6852_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_filter_db_pair_list6852	= { FOLLOW_COMMA_SYMBOL_in_filter_db_pair_list6852_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_identifier_pair_in_filter_db_pair_list6854  */
static	ANTLR3_BITWORD FOLLOW_schema_identifier_pair_in_filter_db_pair_list6854_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_identifier_pair_in_filter_db_pair_list6854	= { FOLLOW_schema_identifier_pair_in_filter_db_pair_list6854_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_START_SYMBOL_in_slave6865  */
static	ANTLR3_BITWORD FOLLOW_START_SYMBOL_in_slave6865_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_START_SYMBOL_in_slave6865	= { FOLLOW_START_SYMBOL_in_slave6865_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_slave6867  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_slave6867_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1400000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x00000A0000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_slave6867	= { FOLLOW_SLAVE_SYMBOL_in_slave6867_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_thread_options_in_slave6869  */
static	ANTLR3_BITWORD FOLLOW_slave_thread_options_in_slave6869_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000A0000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_thread_options_in_slave6869	= { FOLLOW_slave_thread_options_in_slave6869_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNTIL_SYMBOL_in_slave6873  */
static	ANTLR3_BITWORD FOLLOW_UNTIL_SYMBOL_in_slave6873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x6000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1C00000000300000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNTIL_SYMBOL_in_slave6873	= { FOLLOW_UNTIL_SYMBOL_in_slave6873_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_until_options_in_slave6875  */
static	ANTLR3_BITWORD FOLLOW_slave_until_options_in_slave6875_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_until_options_in_slave6875	= { FOLLOW_slave_until_options_in_slave6875_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_connection_options_in_slave6879  */
static	ANTLR3_BITWORD FOLLOW_slave_connection_options_in_slave6879_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_connection_options_in_slave6879	= { FOLLOW_slave_connection_options_in_slave6879_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_slave6881  */
static	ANTLR3_BITWORD FOLLOW_channel_in_slave6881_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_slave6881	= { FOLLOW_channel_in_slave6881_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STOP_SYMBOL_in_slave6887  */
static	ANTLR3_BITWORD FOLLOW_STOP_SYMBOL_in_slave6887_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STOP_SYMBOL_in_slave6887	= { FOLLOW_STOP_SYMBOL_in_slave6887_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_slave6889  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_slave6889_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_slave6889	= { FOLLOW_SLAVE_SYMBOL_in_slave6889_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_thread_options_in_slave6891  */
static	ANTLR3_BITWORD FOLLOW_slave_thread_options_in_slave6891_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_thread_options_in_slave6891	= { FOLLOW_slave_thread_options_in_slave6891_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_slave6894  */
static	ANTLR3_BITWORD FOLLOW_channel_in_slave6894_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_slave6894	= { FOLLOW_channel_in_slave6894_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_master_file_def_in_slave_until_options6908  */
static	ANTLR3_BITWORD FOLLOW_master_file_def_in_slave_until_options6908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_master_file_def_in_slave_until_options6908	= { FOLLOW_master_file_def_in_slave_until_options6908_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_slave_until_options6916  */
static	ANTLR3_BITWORD FOLLOW_set_in_slave_until_options6916_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_slave_until_options6916	= { FOLLOW_set_in_slave_until_options6916_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_slave_until_options6924  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_slave_until_options6924_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_slave_until_options6924	= { FOLLOW_EQUAL_OPERATOR_in_slave_until_options6924_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_slave_until_options6926  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_slave_until_options6926_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_slave_until_options6926	= { FOLLOW_text_string_in_slave_until_options6926_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQL_AFTER_MTS_GAPS_SYMBOL_in_slave_until_options6934  */
static	ANTLR3_BITWORD FOLLOW_SQL_AFTER_MTS_GAPS_SYMBOL_in_slave_until_options6934_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_SQL_AFTER_MTS_GAPS_SYMBOL_in_slave_until_options6934	= { FOLLOW_SQL_AFTER_MTS_GAPS_SYMBOL_in_slave_until_options6934_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_slave_until_options6941  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_slave_until_options6941_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x6000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000300000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_slave_until_options6941	= { FOLLOW_COMMA_SYMBOL_in_slave_until_options6941_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_master_file_def_in_slave_until_options6943  */
static	ANTLR3_BITWORD FOLLOW_master_file_def_in_slave_until_options6943_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_master_file_def_in_slave_until_options6943	= { FOLLOW_master_file_def_in_slave_until_options6943_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_slave_connection_options6957  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_slave_connection_options6957_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_slave_connection_options6957	= { FOLLOW_USER_SYMBOL_in_slave_connection_options6957_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6959  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6959_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6959	= { FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6959_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_slave_connection_options6961  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_slave_connection_options6961_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_slave_connection_options6961	= { FOLLOW_text_string_in_slave_connection_options6961_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_slave_connection_options6966  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_slave_connection_options6966_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_slave_connection_options6966	= { FOLLOW_PASSWORD_SYMBOL_in_slave_connection_options6966_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6968  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6968_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6968	= { FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6968_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_slave_connection_options6970  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_slave_connection_options6970_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_slave_connection_options6970	= { FOLLOW_text_string_in_slave_connection_options6970_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_AUTH_SYMBOL_in_slave_connection_options6977  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_AUTH_SYMBOL_in_slave_connection_options6977_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_AUTH_SYMBOL_in_slave_connection_options6977	= { FOLLOW_DEFAULT_AUTH_SYMBOL_in_slave_connection_options6977_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6979  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6979_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6979	= { FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6979_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_slave_connection_options6981  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_slave_connection_options6981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_slave_connection_options6981	= { FOLLOW_text_string_in_slave_connection_options6981_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUGIN_DIR_SYMBOL_in_slave_connection_options6986  */
static	ANTLR3_BITWORD FOLLOW_PLUGIN_DIR_SYMBOL_in_slave_connection_options6986_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUGIN_DIR_SYMBOL_in_slave_connection_options6986	= { FOLLOW_PLUGIN_DIR_SYMBOL_in_slave_connection_options6986_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6988  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6988_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6988	= { FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6988_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_slave_connection_options6990  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_slave_connection_options6990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_slave_connection_options6990	= { FOLLOW_text_string_in_slave_connection_options6990_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_thread_option_in_slave_thread_options7006  */
static	ANTLR3_BITWORD FOLLOW_slave_thread_option_in_slave_thread_options7006_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_thread_option_in_slave_thread_options7006	= { FOLLOW_slave_thread_option_in_slave_thread_options7006_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_slave_thread_options7009  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_slave_thread_options7009_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_slave_thread_options7009	= { FOLLOW_COMMA_SYMBOL_in_slave_thread_options7009_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_slave_thread_option_in_slave_thread_options7011  */
static	ANTLR3_BITWORD FOLLOW_slave_thread_option_in_slave_thread_options7011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_slave_thread_option_in_slave_thread_options7011	= { FOLLOW_slave_thread_option_in_slave_thread_options7011_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_group_replication7035  */
static	ANTLR3_BITWORD FOLLOW_set_in_group_replication7035_bits[]	= { ANTLR3_UINT64_LIT(0x0000000001000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_group_replication7035	= { FOLLOW_set_in_group_replication7035_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_REPLICATION_SYMBOL_in_group_replication7043  */
static	ANTLR3_BITWORD FOLLOW_GROUP_REPLICATION_SYMBOL_in_group_replication7043_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_REPLICATION_SYMBOL_in_group_replication7043	= { FOLLOW_GROUP_REPLICATION_SYMBOL_in_group_replication7043_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PREPARE_SYMBOL_in_prepared_statement7054  */
static	ANTLR3_BITWORD FOLLOW_PREPARE_SYMBOL_in_prepared_statement7054_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PREPARE_SYMBOL_in_prepared_statement7054	= { FOLLOW_PREPARE_SYMBOL_in_prepared_statement7054_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_prepared_statement7056  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_prepared_statement7056_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_prepared_statement7056	= { FOLLOW_identifier_in_prepared_statement7056_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_prepared_statement7058  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_prepared_statement7058_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0006000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_prepared_statement7058	= { FOLLOW_FROM_SYMBOL_in_prepared_statement7058_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_prepared_statement7061  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_prepared_statement7061_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_prepared_statement7061	= { FOLLOW_string_literal_in_prepared_statement7061_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_prepared_statement7065  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_prepared_statement7065_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_prepared_statement7065	= { FOLLOW_user_variable_in_prepared_statement7065_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_execute_statement_in_prepared_statement7071  */
static	ANTLR3_BITWORD FOLLOW_execute_statement_in_prepared_statement7071_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_execute_statement_in_prepared_statement7071	= { FOLLOW_execute_statement_in_prepared_statement7071_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_prepared_statement7076  */
static	ANTLR3_BITWORD FOLLOW_set_in_prepared_statement7076_bits[]	= { ANTLR3_UINT64_LIT(0x0000004000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_prepared_statement7076	= { FOLLOW_set_in_prepared_statement7076_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PREPARE_SYMBOL_in_prepared_statement7084  */
static	ANTLR3_BITWORD FOLLOW_PREPARE_SYMBOL_in_prepared_statement7084_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PREPARE_SYMBOL_in_prepared_statement7084	= { FOLLOW_PREPARE_SYMBOL_in_prepared_statement7084_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_prepared_statement7086  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_prepared_statement7086_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_prepared_statement7086	= { FOLLOW_identifier_in_prepared_statement7086_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXECUTE_SYMBOL_in_execute_statement7095  */
static	ANTLR3_BITWORD FOLLOW_EXECUTE_SYMBOL_in_execute_statement7095_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXECUTE_SYMBOL_in_execute_statement7095	= { FOLLOW_EXECUTE_SYMBOL_in_execute_statement7095_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_execute_statement7097  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_execute_statement7097_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_execute_statement7097	= { FOLLOW_identifier_in_execute_statement7097_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USING_SYMBOL_in_execute_statement7100  */
static	ANTLR3_BITWORD FOLLOW_USING_SYMBOL_in_execute_statement7100_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0006000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USING_SYMBOL_in_execute_statement7100	= { FOLLOW_USING_SYMBOL_in_execute_statement7100_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_execute_var_list_in_execute_statement7102  */
static	ANTLR3_BITWORD FOLLOW_execute_var_list_in_execute_statement7102_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_execute_var_list_in_execute_statement7102	= { FOLLOW_execute_var_list_in_execute_statement7102_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_execute_var_list7113  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_execute_var_list7113_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_execute_var_list7113	= { FOLLOW_user_variable_in_execute_var_list7113_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_execute_var_list7116  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_execute_var_list7116_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0006000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_execute_var_list7116	= { FOLLOW_COMMA_SYMBOL_in_execute_var_list7116_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_execute_var_list7118  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_execute_var_list7118_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_execute_var_list7118	= { FOLLOW_user_variable_in_execute_var_list7118_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_user_in_account_management_statement7133  */
static	ANTLR3_BITWORD FOLLOW_alter_user_in_account_management_statement7133_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_user_in_account_management_statement7133	= { FOLLOW_alter_user_in_account_management_statement7133_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_user_in_account_management_statement7138  */
static	ANTLR3_BITWORD FOLLOW_create_user_in_account_management_statement7138_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_user_in_account_management_statement7138	= { FOLLOW_create_user_in_account_management_statement7138_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_drop_user_in_account_management_statement7143  */
static	ANTLR3_BITWORD FOLLOW_drop_user_in_account_management_statement7143_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_drop_user_in_account_management_statement7143	= { FOLLOW_drop_user_in_account_management_statement7143_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_proxy_in_account_management_statement7150  */
static	ANTLR3_BITWORD FOLLOW_grant_proxy_in_account_management_statement7150_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_proxy_in_account_management_statement7150	= { FOLLOW_grant_proxy_in_account_management_statement7150_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_in_account_management_statement7155  */
static	ANTLR3_BITWORD FOLLOW_grant_in_account_management_statement7155_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_in_account_management_statement7155	= { FOLLOW_grant_in_account_management_statement7155_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_rename_user_in_account_management_statement7160  */
static	ANTLR3_BITWORD FOLLOW_rename_user_in_account_management_statement7160_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_rename_user_in_account_management_statement7160	= { FOLLOW_rename_user_in_account_management_statement7160_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_revoke_statement_in_account_management_statement7165  */
static	ANTLR3_BITWORD FOLLOW_revoke_statement_in_account_management_statement7165_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_revoke_statement_in_account_management_statement7165	= { FOLLOW_revoke_statement_in_account_management_statement7165_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_password_in_account_management_statement7170  */
static	ANTLR3_BITWORD FOLLOW_set_password_in_account_management_statement7170_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_password_in_account_management_statement7170	= { FOLLOW_set_password_in_account_management_statement7170_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALTER_SYMBOL_in_alter_user7179  */
static	ANTLR3_BITWORD FOLLOW_ALTER_SYMBOL_in_alter_user7179_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALTER_SYMBOL_in_alter_user7179	= { FOLLOW_ALTER_SYMBOL_in_alter_user7179_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_alter_user7181  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_alter_user7181_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_alter_user7181	= { FOLLOW_USER_SYMBOL_in_alter_user7181_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_alter_user7186  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_alter_user7186_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_alter_user7186	= { FOLLOW_if_exists_in_alter_user7186_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_alter_user_tail_in_alter_user7193  */
static	ANTLR3_BITWORD FOLLOW_alter_user_tail_in_alter_user7193_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_alter_user_tail_in_alter_user7193	= { FOLLOW_alter_user_tail_in_alter_user7193_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_list_in_alter_user_tail7202  */
static	ANTLR3_BITWORD FOLLOW_grant_list_in_alter_user_tail7202_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_list_in_alter_user_tail7202	= { FOLLOW_grant_list_in_alter_user_tail7202_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_user_tail_in_alter_user_tail7204  */
static	ANTLR3_BITWORD FOLLOW_create_user_tail_in_alter_user_tail7204_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_user_tail_in_alter_user_tail7204	= { FOLLOW_create_user_tail_in_alter_user_tail7204_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_alter_user_tail7211  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_alter_user_tail7211_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_alter_user_tail7211	= { FOLLOW_USER_SYMBOL_in_alter_user_tail7211_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_alter_user_tail7213  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_alter_user_tail7213_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_alter_user_tail7213	= { FOLLOW_parentheses_in_alter_user_tail7213_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IDENTIFIED_SYMBOL_in_alter_user_tail7215  */
static	ANTLR3_BITWORD FOLLOW_IDENTIFIED_SYMBOL_in_alter_user_tail7215_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_IDENTIFIED_SYMBOL_in_alter_user_tail7215	= { FOLLOW_IDENTIFIED_SYMBOL_in_alter_user_tail7215_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_alter_user_tail7217  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_alter_user_tail7217_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_alter_user_tail7217	= { FOLLOW_BY_SYMBOL_in_alter_user_tail7217_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_alter_user_tail7219  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_alter_user_tail7219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_alter_user_tail7219	= { FOLLOW_text_string_in_alter_user_tail7219_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_create_user7228  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_create_user7228_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_create_user7228	= { FOLLOW_CREATE_SYMBOL_in_create_user7228_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_create_user7230  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_create_user7230_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_create_user7230	= { FOLLOW_USER_SYMBOL_in_create_user7230_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_not_exists_in_create_user7235  */
static	ANTLR3_BITWORD FOLLOW_if_not_exists_in_create_user7235_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_not_exists_in_create_user7235	= { FOLLOW_if_not_exists_in_create_user7235_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_list_in_create_user7242  */
static	ANTLR3_BITWORD FOLLOW_grant_list_in_create_user7242_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_list_in_create_user7242	= { FOLLOW_grant_list_in_create_user7242_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_user_tail_in_create_user7244  */
static	ANTLR3_BITWORD FOLLOW_create_user_tail_in_create_user7244_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_create_user_tail_in_create_user7244	= { FOLLOW_create_user_tail_in_create_user7244_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_require_clause_in_create_user_tail7255  */
static	ANTLR3_BITWORD FOLLOW_require_clause_in_create_user_tail7255_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000012), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_require_clause_in_create_user_tail7255	= { FOLLOW_require_clause_in_create_user_tail7255_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_connect_options_in_create_user_tail7258  */
static	ANTLR3_BITWORD FOLLOW_connect_options_in_create_user_tail7258_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000012), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_connect_options_in_create_user_tail7258	= { FOLLOW_connect_options_in_create_user_tail7258_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_account_lock_password_expire_options_in_create_user_tail7261  */
static	ANTLR3_BITWORD FOLLOW_account_lock_password_expire_options_in_create_user_tail7261_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_account_lock_password_expire_options_in_create_user_tail7261	= { FOLLOW_account_lock_password_expire_options_in_create_user_tail7261_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REQUIRE_SYMBOL_in_require_clause7276  */
static	ANTLR3_BITWORD FOLLOW_REQUIRE_SYMBOL_in_require_clause7276_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0020000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REQUIRE_SYMBOL_in_require_clause7276	= { FOLLOW_REQUIRE_SYMBOL_in_require_clause7276_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_require_list_in_require_clause7279  */
static	ANTLR3_BITWORD FOLLOW_require_list_in_require_clause7279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_require_list_in_require_clause7279	= { FOLLOW_require_list_in_require_clause7279_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SSL_SYMBOL_in_require_clause7283  */
static	ANTLR3_BITWORD FOLLOW_SSL_SYMBOL_in_require_clause7283_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SSL_SYMBOL_in_require_clause7283	= { FOLLOW_SSL_SYMBOL_in_require_clause7283_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_X509_SYMBOL_in_require_clause7287  */
static	ANTLR3_BITWORD FOLLOW_X509_SYMBOL_in_require_clause7287_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_X509_SYMBOL_in_require_clause7287	= { FOLLOW_X509_SYMBOL_in_require_clause7287_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NONE_SYMBOL_in_require_clause7291  */
static	ANTLR3_BITWORD FOLLOW_NONE_SYMBOL_in_require_clause7291_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NONE_SYMBOL_in_require_clause7291	= { FOLLOW_NONE_SYMBOL_in_require_clause7291_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_connect_options7301  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_connect_options7301_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_connect_options7301	= { FOLLOW_WITH_SYMBOL_in_connect_options7301_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_connect_options7308  */
static	ANTLR3_BITWORD FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_connect_options7308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_connect_options7308	= { FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_connect_options7308_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_connect_options7310  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_connect_options7310_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_connect_options7310	= { FOLLOW_ulong_number_in_connect_options7310_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_connect_options7316  */
static	ANTLR3_BITWORD FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_connect_options7316_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_connect_options7316	= { FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_connect_options7316_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_connect_options7318  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_connect_options7318_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_connect_options7318	= { FOLLOW_ulong_number_in_connect_options7318_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_connect_options7324  */
static	ANTLR3_BITWORD FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_connect_options7324_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_connect_options7324	= { FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_connect_options7324_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_connect_options7326  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_connect_options7326_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_connect_options7326	= { FOLLOW_ulong_number_in_connect_options7326_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_connect_options7332  */
static	ANTLR3_BITWORD FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_connect_options7332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_connect_options7332	= { FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_connect_options7332_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_connect_options7334  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_connect_options7334_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_connect_options7334	= { FOLLOW_ulong_number_in_connect_options7334_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ACCOUNT_SYMBOL_in_account_lock_password_expire_options7347  */
static	ANTLR3_BITWORD FOLLOW_ACCOUNT_SYMBOL_in_account_lock_password_expire_options7347_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_ACCOUNT_SYMBOL_in_account_lock_password_expire_options7347	= { FOLLOW_ACCOUNT_SYMBOL_in_account_lock_password_expire_options7347_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_account_lock_password_expire_options7349  */
static	ANTLR3_BITWORD FOLLOW_set_in_account_lock_password_expire_options7349_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_account_lock_password_expire_options7349	= { FOLLOW_set_in_account_lock_password_expire_options7349_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_account_lock_password_expire_options7360  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_account_lock_password_expire_options7360_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_account_lock_password_expire_options7360	= { FOLLOW_PASSWORD_SYMBOL_in_account_lock_password_expire_options7360_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXPIRE_SYMBOL_in_account_lock_password_expire_options7362  */
static	ANTLR3_BITWORD FOLLOW_EXPIRE_SYMBOL_in_account_lock_password_expire_options7362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXPIRE_SYMBOL_in_account_lock_password_expire_options7362	= { FOLLOW_EXPIRE_SYMBOL_in_account_lock_password_expire_options7362_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTERVAL_SYMBOL_in_account_lock_password_expire_options7371  */
static	ANTLR3_BITWORD FOLLOW_INTERVAL_SYMBOL_in_account_lock_password_expire_options7371_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTERVAL_SYMBOL_in_account_lock_password_expire_options7371	= { FOLLOW_INTERVAL_SYMBOL_in_account_lock_password_expire_options7371_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_account_lock_password_expire_options7373  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_account_lock_password_expire_options7373_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_account_lock_password_expire_options7373	= { FOLLOW_real_ulong_number_in_account_lock_password_expire_options7373_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_SYMBOL_in_account_lock_password_expire_options7375  */
static	ANTLR3_BITWORD FOLLOW_DAY_SYMBOL_in_account_lock_password_expire_options7375_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_SYMBOL_in_account_lock_password_expire_options7375	= { FOLLOW_DAY_SYMBOL_in_account_lock_password_expire_options7375_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NEVER_SYMBOL_in_account_lock_password_expire_options7382  */
static	ANTLR3_BITWORD FOLLOW_NEVER_SYMBOL_in_account_lock_password_expire_options7382_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NEVER_SYMBOL_in_account_lock_password_expire_options7382	= { FOLLOW_NEVER_SYMBOL_in_account_lock_password_expire_options7382_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_account_lock_password_expire_options7389  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_account_lock_password_expire_options7389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_account_lock_password_expire_options7389	= { FOLLOW_DEFAULT_SYMBOL_in_account_lock_password_expire_options7389_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_drop_user7403  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_drop_user7403_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_drop_user7403	= { FOLLOW_DROP_SYMBOL_in_drop_user7403_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_drop_user7405  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_drop_user7405_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_drop_user7405	= { FOLLOW_USER_SYMBOL_in_drop_user7405_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_exists_in_drop_user7410  */
static	ANTLR3_BITWORD FOLLOW_if_exists_in_drop_user7410_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_exists_in_drop_user7410	= { FOLLOW_if_exists_in_drop_user7410_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_list_in_drop_user7417  */
static	ANTLR3_BITWORD FOLLOW_user_list_in_drop_user7417_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_list_in_drop_user7417	= { FOLLOW_user_list_in_drop_user7417_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_in_parse_grant7427  */
static	ANTLR3_BITWORD FOLLOW_grant_in_parse_grant7427_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_in_parse_grant7427	= { FOLLOW_grant_in_parse_grant7427_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_parse_grant7429  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_parse_grant7429_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_parse_grant7429	= { FOLLOW_EOF_in_parse_grant7429_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_grant7438  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_grant7438_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0020000004800110), ANTLR3_UINT64_LIT(0x1800000000004000), ANTLR3_UINT64_LIT(0x0208000004000000), ANTLR3_UINT64_LIT(0x0000001080200000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000010100010), ANTLR3_UINT64_LIT(0x0000000000060100) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_grant7438	= { FOLLOW_GRANT_SYMBOL_in_grant7438_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_privileges_in_grant7440  */
static	ANTLR3_BITWORD FOLLOW_grant_privileges_in_grant7440_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_privileges_in_grant7440	= { FOLLOW_grant_privileges_in_grant7440_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_privilege_target_in_grant7442  */
static	ANTLR3_BITWORD FOLLOW_privilege_target_in_grant7442_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_privilege_target_in_grant7442	= { FOLLOW_privilege_target_in_grant7442_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_grant7446  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_grant7446_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_grant7446	= { FOLLOW_TO_SYMBOL_in_grant7446_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_list_in_grant7448  */
static	ANTLR3_BITWORD FOLLOW_grant_list_in_grant7448_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_list_in_grant7448	= { FOLLOW_grant_list_in_grant7448_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_require_clause_in_grant7450  */
static	ANTLR3_BITWORD FOLLOW_require_clause_in_grant7450_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_require_clause_in_grant7450	= { FOLLOW_require_clause_in_grant7450_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_grant7454  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_grant7454_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_grant7454	= { FOLLOW_WITH_SYMBOL_in_grant7454_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_option_in_grant7456  */
static	ANTLR3_BITWORD FOLLOW_grant_option_in_grant7456_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00000000000CC000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_option_in_grant7456	= { FOLLOW_grant_option_in_grant7456_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_grant_proxy7468  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_grant_proxy7468_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_grant_proxy7468	= { FOLLOW_GRANT_SYMBOL_in_grant_proxy7468_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROXY_SYMBOL_in_grant_proxy7470  */
static	ANTLR3_BITWORD FOLLOW_PROXY_SYMBOL_in_grant_proxy7470_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROXY_SYMBOL_in_grant_proxy7470	= { FOLLOW_PROXY_SYMBOL_in_grant_proxy7470_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_grant_proxy7472  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_grant_proxy7472_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_grant_proxy7472	= { FOLLOW_ON_SYMBOL_in_grant_proxy7472_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_user_in_grant_proxy7474  */
static	ANTLR3_BITWORD FOLLOW_grant_user_in_grant_proxy7474_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_user_in_grant_proxy7474	= { FOLLOW_grant_user_in_grant_proxy7474_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_grant_proxy7476  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_grant_proxy7476_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_grant_proxy7476	= { FOLLOW_TO_SYMBOL_in_grant_proxy7476_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_user_in_grant_proxy7478  */
static	ANTLR3_BITWORD FOLLOW_grant_user_in_grant_proxy7478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_user_in_grant_proxy7478	= { FOLLOW_grant_user_in_grant_proxy7478_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_grant_proxy7481  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_grant_proxy7481_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_grant_proxy7481	= { FOLLOW_COMMA_SYMBOL_in_grant_proxy7481_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_user_in_grant_proxy7483  */
static	ANTLR3_BITWORD FOLLOW_grant_user_in_grant_proxy7483_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_user_in_grant_proxy7483	= { FOLLOW_grant_user_in_grant_proxy7483_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_grant_proxy7490  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_grant_proxy7490_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_grant_proxy7490	= { FOLLOW_WITH_SYMBOL_in_grant_proxy7490_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_grant_proxy7492  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_grant_proxy7492_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_grant_proxy7492	= { FOLLOW_GRANT_SYMBOL_in_grant_proxy7492_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_grant_proxy7494  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_grant_proxy7494_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_grant_proxy7494	= { FOLLOW_OPTION_SYMBOL_in_grant_proxy7494_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RENAME_SYMBOL_in_rename_user7505  */
static	ANTLR3_BITWORD FOLLOW_RENAME_SYMBOL_in_rename_user7505_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RENAME_SYMBOL_in_rename_user7505	= { FOLLOW_RENAME_SYMBOL_in_rename_user7505_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_rename_user7507  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_rename_user7507_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_rename_user7507	= { FOLLOW_USER_SYMBOL_in_rename_user7507_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_rename_user7509  */
static	ANTLR3_BITWORD FOLLOW_user_in_rename_user7509_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_rename_user7509	= { FOLLOW_user_in_rename_user7509_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_rename_user7511  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_rename_user7511_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_rename_user7511	= { FOLLOW_TO_SYMBOL_in_rename_user7511_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_rename_user7513  */
static	ANTLR3_BITWORD FOLLOW_user_in_rename_user7513_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_rename_user7513	= { FOLLOW_user_in_rename_user7513_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_rename_user7516  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_rename_user7516_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_rename_user7516	= { FOLLOW_COMMA_SYMBOL_in_rename_user7516_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_rename_user7518  */
static	ANTLR3_BITWORD FOLLOW_user_in_rename_user7518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_rename_user7518	= { FOLLOW_user_in_rename_user7518_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_rename_user7520  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_rename_user7520_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_rename_user7520	= { FOLLOW_TO_SYMBOL_in_rename_user7520_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_rename_user7522  */
static	ANTLR3_BITWORD FOLLOW_user_in_rename_user7522_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_rename_user7522	= { FOLLOW_user_in_rename_user7522_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REVOKE_SYMBOL_in_revoke_statement7533  */
static	ANTLR3_BITWORD FOLLOW_REVOKE_SYMBOL_in_revoke_statement7533_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0020000004800110), ANTLR3_UINT64_LIT(0x1800000000004000), ANTLR3_UINT64_LIT(0x0208000004000000), ANTLR3_UINT64_LIT(0x0000001080200000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000010100010), ANTLR3_UINT64_LIT(0x0000000000060100) };
static  ANTLR3_BITSET_LIST FOLLOW_REVOKE_SYMBOL_in_revoke_statement7533	= { FOLLOW_REVOKE_SYMBOL_in_revoke_statement7533_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_revoke_statement7544  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_revoke_statement7544_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000008), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_revoke_statement7544	= { FOLLOW_ALL_SYMBOL_in_revoke_statement7544_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIVILEGES_SYMBOL_in_revoke_statement7546  */
static	ANTLR3_BITWORD FOLLOW_PRIVILEGES_SYMBOL_in_revoke_statement7546_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIVILEGES_SYMBOL_in_revoke_statement7546	= { FOLLOW_PRIVILEGES_SYMBOL_in_revoke_statement7546_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_revoke_statement7549  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_revoke_statement7549_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_revoke_statement7549	= { FOLLOW_COMMA_SYMBOL_in_revoke_statement7549_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_revoke_statement7551  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_revoke_statement7551_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_revoke_statement7551	= { FOLLOW_GRANT_SYMBOL_in_revoke_statement7551_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_revoke_statement7553  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_revoke_statement7553_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_revoke_statement7553	= { FOLLOW_OPTION_SYMBOL_in_revoke_statement7553_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_revoke_statement7555  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_revoke_statement7555_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_revoke_statement7555	= { FOLLOW_FROM_SYMBOL_in_revoke_statement7555_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_list_in_revoke_statement7557  */
static	ANTLR3_BITWORD FOLLOW_user_list_in_revoke_statement7557_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_list_in_revoke_statement7557	= { FOLLOW_user_list_in_revoke_statement7557_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_privileges_in_revoke_statement7563  */
static	ANTLR3_BITWORD FOLLOW_grant_privileges_in_revoke_statement7563_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_privileges_in_revoke_statement7563	= { FOLLOW_grant_privileges_in_revoke_statement7563_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_privilege_target_in_revoke_statement7565  */
static	ANTLR3_BITWORD FOLLOW_privilege_target_in_revoke_statement7565_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_privilege_target_in_revoke_statement7565	= { FOLLOW_privilege_target_in_revoke_statement7565_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_revoke_statement7567  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_revoke_statement7567_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_revoke_statement7567	= { FOLLOW_FROM_SYMBOL_in_revoke_statement7567_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_list_in_revoke_statement7569  */
static	ANTLR3_BITWORD FOLLOW_user_list_in_revoke_statement7569_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_list_in_revoke_statement7569	= { FOLLOW_user_list_in_revoke_statement7569_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROXY_SYMBOL_in_revoke_statement7577  */
static	ANTLR3_BITWORD FOLLOW_PROXY_SYMBOL_in_revoke_statement7577_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROXY_SYMBOL_in_revoke_statement7577	= { FOLLOW_PROXY_SYMBOL_in_revoke_statement7577_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_revoke_statement7579  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_revoke_statement7579_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_revoke_statement7579	= { FOLLOW_ON_SYMBOL_in_revoke_statement7579_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_revoke_statement7581  */
static	ANTLR3_BITWORD FOLLOW_user_in_revoke_statement7581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_revoke_statement7581	= { FOLLOW_user_in_revoke_statement7581_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_revoke_statement7583  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_revoke_statement7583_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_revoke_statement7583	= { FOLLOW_FROM_SYMBOL_in_revoke_statement7583_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_list_in_revoke_statement7585  */
static	ANTLR3_BITWORD FOLLOW_user_list_in_revoke_statement7585_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_list_in_revoke_statement7585	= { FOLLOW_user_list_in_revoke_statement7585_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_privilege_target7597  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_privilege_target7597_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000001), ANTLR3_UINT64_LIT(0x4000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_privilege_target7597	= { FOLLOW_ON_SYMBOL_in_privilege_target7597_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_object_type_in_privilege_target7599  */
static	ANTLR3_BITWORD FOLLOW_grant_object_type_in_privilege_target7599_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x4000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_object_type_in_privilege_target7599	= { FOLLOW_grant_object_type_in_privilege_target7599_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_privilege_level_in_privilege_target7601  */
static	ANTLR3_BITWORD FOLLOW_privilege_level_in_privilege_target7601_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_privilege_level_in_privilege_target7601	= { FOLLOW_privilege_level_in_privilege_target7601_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_set_password7610  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_set_password7610_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_set_password7610	= { FOLLOW_SET_SYMBOL_in_set_password7610_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_set_password7612  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_set_password7612_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_set_password7612	= { FOLLOW_PASSWORD_SYMBOL_in_set_password7612_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_set_password7615  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_set_password7615_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_set_password7615	= { FOLLOW_FOR_SYMBOL_in_set_password7615_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_set_password7617  */
static	ANTLR3_BITWORD FOLLOW_user_in_set_password7617_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_set_password7617	= { FOLLOW_user_in_set_password7617_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_set_password7621  */
static	ANTLR3_BITWORD FOLLOW_equal_in_set_password7621_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0404000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_set_password7621	= { FOLLOW_equal_in_set_password7621_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_set_password7628  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_set_password7628_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_set_password7628	= { FOLLOW_PASSWORD_SYMBOL_in_set_password7628_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_set_password7630  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_set_password7630_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_set_password7630	= { FOLLOW_OPEN_PAR_SYMBOL_in_set_password7630_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_set_password7632  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_set_password7632_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_set_password7632	= { FOLLOW_text_string_in_set_password7632_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7634  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7634_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7634	= { FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7634_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OLD_PASSWORD_SYMBOL_in_set_password7642  */
static	ANTLR3_BITWORD FOLLOW_OLD_PASSWORD_SYMBOL_in_set_password7642_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OLD_PASSWORD_SYMBOL_in_set_password7642	= { FOLLOW_OLD_PASSWORD_SYMBOL_in_set_password7642_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_set_password7644  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_set_password7644_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_set_password7644	= { FOLLOW_OPEN_PAR_SYMBOL_in_set_password7644_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_set_password7646  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_set_password7646_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_set_password7646	= { FOLLOW_text_string_in_set_password7646_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7648  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7648_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7648	= { FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7648_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_set_password7654  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_set_password7654_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_set_password7654	= { FOLLOW_text_string_in_set_password7654_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_grant_object_type7667  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_grant_object_type7667_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_grant_object_type7667	= { FOLLOW_TABLE_SYMBOL_in_grant_object_type7667_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_grant_object_type7672  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_grant_object_type7672_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_grant_object_type7672	= { FOLLOW_FUNCTION_SYMBOL_in_grant_object_type7672_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_grant_object_type7676  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_grant_object_type7676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_grant_object_type7676	= { FOLLOW_PROCEDURE_SYMBOL_in_grant_object_type7676_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_grant_privileges7686  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_grant_privileges7686_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000008) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_grant_privileges7686	= { FOLLOW_ALL_SYMBOL_in_grant_privileges7686_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIVILEGES_SYMBOL_in_grant_privileges7688  */
static	ANTLR3_BITWORD FOLLOW_PRIVILEGES_SYMBOL_in_grant_privileges7688_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIVILEGES_SYMBOL_in_grant_privileges7688	= { FOLLOW_PRIVILEGES_SYMBOL_in_grant_privileges7688_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_privilege_type_in_grant_privileges7694  */
static	ANTLR3_BITWORD FOLLOW_privilege_type_in_grant_privileges7694_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_privilege_type_in_grant_privileges7694	= { FOLLOW_privilege_type_in_grant_privileges7694_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_grant_privileges7697  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_grant_privileges7697_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0020000004800110), ANTLR3_UINT64_LIT(0x1000000000004000), ANTLR3_UINT64_LIT(0x0208000004000000), ANTLR3_UINT64_LIT(0x0000001080200000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000010100010), ANTLR3_UINT64_LIT(0x0000000000060100) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_grant_privileges7697	= { FOLLOW_COMMA_SYMBOL_in_grant_privileges7697_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_privilege_type_in_grant_privileges7699  */
static	ANTLR3_BITWORD FOLLOW_privilege_type_in_grant_privileges7699_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_privilege_type_in_grant_privileges7699	= { FOLLOW_privilege_type_in_grant_privileges7699_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALTER_SYMBOL_in_privilege_type7710  */
static	ANTLR3_BITWORD FOLLOW_ALTER_SYMBOL_in_privilege_type7710_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000008000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALTER_SYMBOL_in_privilege_type7710	= { FOLLOW_ALTER_SYMBOL_in_privilege_type7710_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROUTINE_SYMBOL_in_privilege_type7712  */
static	ANTLR3_BITWORD FOLLOW_ROUTINE_SYMBOL_in_privilege_type7712_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ROUTINE_SYMBOL_in_privilege_type7712	= { FOLLOW_ROUTINE_SYMBOL_in_privilege_type7712_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_privilege_type7718  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_privilege_type7718_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000008000000000), ANTLR3_UINT64_LIT(0x0000480000600000) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_privilege_type7718	= { FOLLOW_CREATE_SYMBOL_in_privilege_type7718_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROUTINE_SYMBOL_in_privilege_type7721  */
static	ANTLR3_BITWORD FOLLOW_ROUTINE_SYMBOL_in_privilege_type7721_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ROUTINE_SYMBOL_in_privilege_type7721	= { FOLLOW_ROUTINE_SYMBOL_in_privilege_type7721_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_privilege_type7725  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_privilege_type7725_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_privilege_type7725	= { FOLLOW_TABLESPACE_SYMBOL_in_privilege_type7725_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TEMPORARY_SYMBOL_in_privilege_type7729  */
static	ANTLR3_BITWORD FOLLOW_TEMPORARY_SYMBOL_in_privilege_type7729_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_TEMPORARY_SYMBOL_in_privilege_type7729	= { FOLLOW_TEMPORARY_SYMBOL_in_privilege_type7729_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLES_SYMBOL_in_privilege_type7731  */
static	ANTLR3_BITWORD FOLLOW_TABLES_SYMBOL_in_privilege_type7731_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLES_SYMBOL_in_privilege_type7731	= { FOLLOW_TABLES_SYMBOL_in_privilege_type7731_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_privilege_type7735  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_privilege_type7735_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_privilege_type7735	= { FOLLOW_USER_SYMBOL_in_privilege_type7735_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_privilege_type7739  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_privilege_type7739_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_privilege_type7739	= { FOLLOW_VIEW_SYMBOL_in_privilege_type7739_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DELETE_SYMBOL_in_privilege_type7746  */
static	ANTLR3_BITWORD FOLLOW_DELETE_SYMBOL_in_privilege_type7746_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DELETE_SYMBOL_in_privilege_type7746	= { FOLLOW_DELETE_SYMBOL_in_privilege_type7746_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DROP_SYMBOL_in_privilege_type7751  */
static	ANTLR3_BITWORD FOLLOW_DROP_SYMBOL_in_privilege_type7751_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DROP_SYMBOL_in_privilege_type7751	= { FOLLOW_DROP_SYMBOL_in_privilege_type7751_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENT_SYMBOL_in_privilege_type7756  */
static	ANTLR3_BITWORD FOLLOW_EVENT_SYMBOL_in_privilege_type7756_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENT_SYMBOL_in_privilege_type7756	= { FOLLOW_EVENT_SYMBOL_in_privilege_type7756_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXECUTE_SYMBOL_in_privilege_type7761  */
static	ANTLR3_BITWORD FOLLOW_EXECUTE_SYMBOL_in_privilege_type7761_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EXECUTE_SYMBOL_in_privilege_type7761	= { FOLLOW_EXECUTE_SYMBOL_in_privilege_type7761_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FILE_SYMBOL_in_privilege_type7766  */
static	ANTLR3_BITWORD FOLLOW_FILE_SYMBOL_in_privilege_type7766_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FILE_SYMBOL_in_privilege_type7766	= { FOLLOW_FILE_SYMBOL_in_privilege_type7766_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_privilege_type7771  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_privilege_type7771_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_privilege_type7771	= { FOLLOW_GRANT_SYMBOL_in_privilege_type7771_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_privilege_type7773  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_privilege_type7773_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_privilege_type7773	= { FOLLOW_OPTION_SYMBOL_in_privilege_type7773_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_privilege_type7778  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_privilege_type7778_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_privilege_type7778	= { FOLLOW_INDEX_SYMBOL_in_privilege_type7778_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INSERT_SYMBOL_in_privilege_type7783  */
static	ANTLR3_BITWORD FOLLOW_INSERT_SYMBOL_in_privilege_type7783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INSERT_SYMBOL_in_privilege_type7783	= { FOLLOW_INSERT_SYMBOL_in_privilege_type7783_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_privilege_type7785  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_privilege_type7785_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_privilege_type7785	= { FOLLOW_identifier_list_with_parentheses_in_privilege_type7785_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_privilege_type7791  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_privilege_type7791_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_privilege_type7791	= { FOLLOW_LOCK_SYMBOL_in_privilege_type7791_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLES_SYMBOL_in_privilege_type7793  */
static	ANTLR3_BITWORD FOLLOW_TABLES_SYMBOL_in_privilege_type7793_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLES_SYMBOL_in_privilege_type7793	= { FOLLOW_TABLES_SYMBOL_in_privilege_type7793_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCESS_SYMBOL_in_privilege_type7798  */
static	ANTLR3_BITWORD FOLLOW_PROCESS_SYMBOL_in_privilege_type7798_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCESS_SYMBOL_in_privilege_type7798	= { FOLLOW_PROCESS_SYMBOL_in_privilege_type7798_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROXY_SYMBOL_in_privilege_type7803  */
static	ANTLR3_BITWORD FOLLOW_PROXY_SYMBOL_in_privilege_type7803_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PROXY_SYMBOL_in_privilege_type7803	= { FOLLOW_PROXY_SYMBOL_in_privilege_type7803_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REFERENCES_SYMBOL_in_privilege_type7808  */
static	ANTLR3_BITWORD FOLLOW_REFERENCES_SYMBOL_in_privilege_type7808_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REFERENCES_SYMBOL_in_privilege_type7808	= { FOLLOW_REFERENCES_SYMBOL_in_privilege_type7808_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_privilege_type7810  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_privilege_type7810_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_privilege_type7810	= { FOLLOW_identifier_list_with_parentheses_in_privilege_type7810_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELOAD_SYMBOL_in_privilege_type7816  */
static	ANTLR3_BITWORD FOLLOW_RELOAD_SYMBOL_in_privilege_type7816_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_RELOAD_SYMBOL_in_privilege_type7816	= { FOLLOW_RELOAD_SYMBOL_in_privilege_type7816_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLICATION_SYMBOL_in_privilege_type7821  */
static	ANTLR3_BITWORD FOLLOW_REPLICATION_SYMBOL_in_privilege_type7821_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLICATION_SYMBOL_in_privilege_type7821	= { FOLLOW_REPLICATION_SYMBOL_in_privilege_type7821_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_privilege_type7823  */
static	ANTLR3_BITWORD FOLLOW_set_in_privilege_type7823_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_privilege_type7823	= { FOLLOW_set_in_privilege_type7823_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SELECT_SYMBOL_in_privilege_type7834  */
static	ANTLR3_BITWORD FOLLOW_SELECT_SYMBOL_in_privilege_type7834_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SELECT_SYMBOL_in_privilege_type7834	= { FOLLOW_SELECT_SYMBOL_in_privilege_type7834_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_privilege_type7836  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_privilege_type7836_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_privilege_type7836	= { FOLLOW_identifier_list_with_parentheses_in_privilege_type7836_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHOW_SYMBOL_in_privilege_type7842  */
static	ANTLR3_BITWORD FOLLOW_SHOW_SYMBOL_in_privilege_type7842_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SHOW_SYMBOL_in_privilege_type7842	= { FOLLOW_SHOW_SYMBOL_in_privilege_type7842_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASES_SYMBOL_in_privilege_type7844  */
static	ANTLR3_BITWORD FOLLOW_DATABASES_SYMBOL_in_privilege_type7844_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASES_SYMBOL_in_privilege_type7844	= { FOLLOW_DATABASES_SYMBOL_in_privilege_type7844_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHOW_SYMBOL_in_privilege_type7849  */
static	ANTLR3_BITWORD FOLLOW_SHOW_SYMBOL_in_privilege_type7849_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SHOW_SYMBOL_in_privilege_type7849	= { FOLLOW_SHOW_SYMBOL_in_privilege_type7849_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_privilege_type7851  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_privilege_type7851_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_privilege_type7851	= { FOLLOW_VIEW_SYMBOL_in_privilege_type7851_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHUTDOWN_SYMBOL_in_privilege_type7856  */
static	ANTLR3_BITWORD FOLLOW_SHUTDOWN_SYMBOL_in_privilege_type7856_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SHUTDOWN_SYMBOL_in_privilege_type7856	= { FOLLOW_SHUTDOWN_SYMBOL_in_privilege_type7856_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUPER_SYMBOL_in_privilege_type7861  */
static	ANTLR3_BITWORD FOLLOW_SUPER_SYMBOL_in_privilege_type7861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SUPER_SYMBOL_in_privilege_type7861	= { FOLLOW_SUPER_SYMBOL_in_privilege_type7861_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIGGER_SYMBOL_in_privilege_type7866  */
static	ANTLR3_BITWORD FOLLOW_TRIGGER_SYMBOL_in_privilege_type7866_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIGGER_SYMBOL_in_privilege_type7866	= { FOLLOW_TRIGGER_SYMBOL_in_privilege_type7866_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_privilege_type7871  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_privilege_type7871_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_privilege_type7871	= { FOLLOW_UPDATE_SYMBOL_in_privilege_type7871_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_privilege_type7873  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_privilege_type7873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_privilege_type7873	= { FOLLOW_identifier_list_with_parentheses_in_privilege_type7873_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USAGE_SYMBOL_in_privilege_type7879  */
static	ANTLR3_BITWORD FOLLOW_USAGE_SYMBOL_in_privilege_type7879_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_USAGE_SYMBOL_in_privilege_type7879	= { FOLLOW_USAGE_SYMBOL_in_privilege_type7879_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_privilege_level7888  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_privilege_level7888_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_privilege_level7888	= { FOLLOW_MULT_OPERATOR_in_privilege_level7888_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_privilege_level7891  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_privilege_level7891_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_privilege_level7891	= { FOLLOW_DOT_SYMBOL_in_privilege_level7891_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_privilege_level7893  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_privilege_level7893_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_privilege_level7893	= { FOLLOW_MULT_OPERATOR_in_privilege_level7893_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_privilege_level7900  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_privilege_level7900_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_privilege_level7900	= { FOLLOW_identifier_in_privilege_level7900_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_privilege_level7903  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_privilege_level7903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_privilege_level7903	= { FOLLOW_DOT_SYMBOL_in_privilege_level7903_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_privilege_level7905  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_privilege_level7905_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_privilege_level7905	= { FOLLOW_MULT_OPERATOR_in_privilege_level7905_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_privilege_level7909  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_privilege_level7909_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_privilege_level7909	= { FOLLOW_dot_identifier_in_privilege_level7909_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_require_list_element_in_require_list7920  */
static	ANTLR3_BITWORD FOLLOW_require_list_element_in_require_list7920_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000800) };
static  ANTLR3_BITSET_LIST FOLLOW_require_list_element_in_require_list7920	= { FOLLOW_require_list_element_in_require_list7920_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AND_SYMBOL_in_require_list7923  */
static	ANTLR3_BITWORD FOLLOW_AND_SYMBOL_in_require_list7923_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800) };
static  ANTLR3_BITSET_LIST FOLLOW_AND_SYMBOL_in_require_list7923	= { FOLLOW_AND_SYMBOL_in_require_list7923_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_require_list_element_in_require_list7926  */
static	ANTLR3_BITWORD FOLLOW_require_list_element_in_require_list7926_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000800) };
static  ANTLR3_BITSET_LIST FOLLOW_require_list_element_in_require_list7926	= { FOLLOW_require_list_element_in_require_list7926_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CIPHER_SYMBOL_in_require_list_element7937  */
static	ANTLR3_BITWORD FOLLOW_CIPHER_SYMBOL_in_require_list_element7937_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CIPHER_SYMBOL_in_require_list_element7937	= { FOLLOW_CIPHER_SYMBOL_in_require_list_element7937_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_require_list_element7939  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_require_list_element7939_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_require_list_element7939	= { FOLLOW_text_string_in_require_list_element7939_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ISSUER_SYMBOL_in_require_list_element7944  */
static	ANTLR3_BITWORD FOLLOW_ISSUER_SYMBOL_in_require_list_element7944_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ISSUER_SYMBOL_in_require_list_element7944	= { FOLLOW_ISSUER_SYMBOL_in_require_list_element7944_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_require_list_element7946  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_require_list_element7946_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_require_list_element7946	= { FOLLOW_text_string_in_require_list_element7946_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUBJECT_SYMBOL_in_require_list_element7951  */
static	ANTLR3_BITWORD FOLLOW_SUBJECT_SYMBOL_in_require_list_element7951_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUBJECT_SYMBOL_in_require_list_element7951	= { FOLLOW_SUBJECT_SYMBOL_in_require_list_element7951_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_require_list_element7953  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_require_list_element7953_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_require_list_element7953	= { FOLLOW_text_string_in_require_list_element7953_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANT_SYMBOL_in_grant_option7962  */
static	ANTLR3_BITWORD FOLLOW_GRANT_SYMBOL_in_grant_option7962_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANT_SYMBOL_in_grant_option7962	= { FOLLOW_GRANT_SYMBOL_in_grant_option7962_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTION_SYMBOL_in_grant_option7964  */
static	ANTLR3_BITWORD FOLLOW_OPTION_SYMBOL_in_grant_option7964_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTION_SYMBOL_in_grant_option7964	= { FOLLOW_OPTION_SYMBOL_in_grant_option7964_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_grant_option7969  */
static	ANTLR3_BITWORD FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_grant_option7969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_grant_option7969	= { FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_grant_option7969_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_grant_option7971  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_grant_option7971_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_grant_option7971	= { FOLLOW_ulong_number_in_grant_option7971_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_grant_option7976  */
static	ANTLR3_BITWORD FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_grant_option7976_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_grant_option7976	= { FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_grant_option7976_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_grant_option7978  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_grant_option7978_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_grant_option7978	= { FOLLOW_ulong_number_in_grant_option7978_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_grant_option7983  */
static	ANTLR3_BITWORD FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_grant_option7983_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_grant_option7983	= { FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_grant_option7983_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_grant_option7985  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_grant_option7985_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_grant_option7985	= { FOLLOW_ulong_number_in_grant_option7985_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_grant_option7990  */
static	ANTLR3_BITWORD FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_grant_option7990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_grant_option7990	= { FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_grant_option7990_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_grant_option7992  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_grant_option7992_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_grant_option7992	= { FOLLOW_ulong_number_in_grant_option7992_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ANALYZE_SYMBOL_in_table_administration_statement8003  */
static	ANTLR3_BITWORD FOLLOW_ANALYZE_SYMBOL_in_table_administration_statement8003_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_ANALYZE_SYMBOL_in_table_administration_statement8003	= { FOLLOW_ANALYZE_SYMBOL_in_table_administration_statement8003_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_table_administration_statement8005  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_table_administration_statement8005_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_table_administration_statement8005	= { FOLLOW_no_write_to_bin_log_in_table_administration_statement8005_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8008  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8008_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8008	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8008_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8010  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8010_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8010	= { FOLLOW_table_ref_list_in_table_administration_statement8010_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_table_administration_statement8015  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_table_administration_statement8015_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_table_administration_statement8015	= { FOLLOW_CHECK_SYMBOL_in_table_administration_statement8015_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8017  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8017_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8017	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8017_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8019  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8019_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000040000), ANTLR3_UINT64_LIT(0x0000000000004800), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8019	= { FOLLOW_table_ref_list_in_table_administration_statement8019_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_check_option_in_table_administration_statement8021  */
static	ANTLR3_BITWORD FOLLOW_check_option_in_table_administration_statement8021_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000040000), ANTLR3_UINT64_LIT(0x0000000000004800), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_check_option_in_table_administration_statement8021	= { FOLLOW_check_option_in_table_administration_statement8021_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECKSUM_SYMBOL_in_table_administration_statement8027  */
static	ANTLR3_BITWORD FOLLOW_CHECKSUM_SYMBOL_in_table_administration_statement8027_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECKSUM_SYMBOL_in_table_administration_statement8027	= { FOLLOW_CHECKSUM_SYMBOL_in_table_administration_statement8027_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8029  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8029_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8029	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8029_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8031  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8031_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8031	= { FOLLOW_table_ref_list_in_table_administration_statement8031_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTIMIZE_SYMBOL_in_table_administration_statement8045  */
static	ANTLR3_BITWORD FOLLOW_OPTIMIZE_SYMBOL_in_table_administration_statement8045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTIMIZE_SYMBOL_in_table_administration_statement8045	= { FOLLOW_OPTIMIZE_SYMBOL_in_table_administration_statement8045_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_table_administration_statement8047  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_table_administration_statement8047_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_table_administration_statement8047	= { FOLLOW_no_write_to_bin_log_in_table_administration_statement8047_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8050  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8050_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8050	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8050_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8052  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8052_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8052	= { FOLLOW_table_ref_list_in_table_administration_statement8052_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPAIR_SYMBOL_in_table_administration_statement8057  */
static	ANTLR3_BITWORD FOLLOW_REPAIR_SYMBOL_in_table_administration_statement8057_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_REPAIR_SYMBOL_in_table_administration_statement8057	= { FOLLOW_REPAIR_SYMBOL_in_table_administration_statement8057_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_table_administration_statement8059  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_table_administration_statement8059_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_table_administration_statement8059	= { FOLLOW_no_write_to_bin_log_in_table_administration_statement8059_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8062  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8062_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8062	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8062_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8064  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8064_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8064	= { FOLLOW_table_ref_list_in_table_administration_statement8064_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_repair_option_in_table_administration_statement8066  */
static	ANTLR3_BITWORD FOLLOW_repair_option_in_table_administration_statement8066_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_repair_option_in_table_administration_statement8066	= { FOLLOW_repair_option_in_table_administration_statement8066_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BACKUP_SYMBOL_in_table_administration_statement8074  */
static	ANTLR3_BITWORD FOLLOW_BACKUP_SYMBOL_in_table_administration_statement8074_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_BACKUP_SYMBOL_in_table_administration_statement8074	= { FOLLOW_BACKUP_SYMBOL_in_table_administration_statement8074_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8076  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8076_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8076	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8076_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8078  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8078_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8078	= { FOLLOW_table_ref_list_in_table_administration_statement8078_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TO_SYMBOL_in_table_administration_statement8080  */
static	ANTLR3_BITWORD FOLLOW_TO_SYMBOL_in_table_administration_statement8080_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TO_SYMBOL_in_table_administration_statement8080	= { FOLLOW_TO_SYMBOL_in_table_administration_statement8080_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_table_administration_statement8082  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_table_administration_statement8082_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_table_administration_statement8082	= { FOLLOW_string_literal_in_table_administration_statement8082_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RESTORE_SYMBOL_in_table_administration_statement8089  */
static	ANTLR3_BITWORD FOLLOW_RESTORE_SYMBOL_in_table_administration_statement8089_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_RESTORE_SYMBOL_in_table_administration_statement8089	= { FOLLOW_RESTORE_SYMBOL_in_table_administration_statement8089_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_table_administration_statement8091  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_table_administration_statement8091_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_table_administration_statement8091	= { FOLLOW_TABLE_SYMBOL_in_table_administration_statement8091_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_table_administration_statement8093  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_table_administration_statement8093_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_table_administration_statement8093	= { FOLLOW_table_ref_list_in_table_administration_statement8093_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_table_administration_statement8095  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_table_administration_statement8095_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_table_administration_statement8095	= { FOLLOW_FROM_SYMBOL_in_table_administration_statement8095_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_table_administration_statement8097  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_table_administration_statement8097_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_table_administration_statement8097	= { FOLLOW_string_literal_in_table_administration_statement8097_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_check_option8106  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_check_option8106_bits[]	= { ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_check_option8106	= { FOLLOW_FOR_SYMBOL_in_check_option8106_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPGRADE_SYMBOL_in_check_option8108  */
static	ANTLR3_BITWORD FOLLOW_UPGRADE_SYMBOL_in_check_option8108_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_UPGRADE_SYMBOL_in_check_option8108	= { FOLLOW_UPGRADE_SYMBOL_in_check_option8108_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUICK_SYMBOL_in_check_option8112  */
static	ANTLR3_BITWORD FOLLOW_QUICK_SYMBOL_in_check_option8112_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_QUICK_SYMBOL_in_check_option8112	= { FOLLOW_QUICK_SYMBOL_in_check_option8112_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FAST_SYMBOL_in_check_option8116  */
static	ANTLR3_BITWORD FOLLOW_FAST_SYMBOL_in_check_option8116_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FAST_SYMBOL_in_check_option8116	= { FOLLOW_FAST_SYMBOL_in_check_option8116_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MEDIUM_SYMBOL_in_check_option8120  */
static	ANTLR3_BITWORD FOLLOW_MEDIUM_SYMBOL_in_check_option8120_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MEDIUM_SYMBOL_in_check_option8120	= { FOLLOW_MEDIUM_SYMBOL_in_check_option8120_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXTENDED_SYMBOL_in_check_option8124  */
static	ANTLR3_BITWORD FOLLOW_EXTENDED_SYMBOL_in_check_option8124_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EXTENDED_SYMBOL_in_check_option8124	= { FOLLOW_EXTENDED_SYMBOL_in_check_option8124_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANGED_SYMBOL_in_check_option8128  */
static	ANTLR3_BITWORD FOLLOW_CHANGED_SYMBOL_in_check_option8128_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANGED_SYMBOL_in_check_option8128	= { FOLLOW_CHANGED_SYMBOL_in_check_option8128_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INSTALL_SYMBOL_in_install_uninstall_statment8156  */
static	ANTLR3_BITWORD FOLLOW_INSTALL_SYMBOL_in_install_uninstall_statment8156_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INSTALL_SYMBOL_in_install_uninstall_statment8156	= { FOLLOW_INSTALL_SYMBOL_in_install_uninstall_statment8156_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8158  */
static	ANTLR3_BITWORD FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8158_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8158	= { FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8158_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_install_uninstall_statment8160  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_install_uninstall_statment8160_bits[]	= { ANTLR3_UINT64_LIT(0x0004000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_install_uninstall_statment8160	= { FOLLOW_identifier_in_install_uninstall_statment8160_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SONAME_SYMBOL_in_install_uninstall_statment8162  */
static	ANTLR3_BITWORD FOLLOW_SONAME_SYMBOL_in_install_uninstall_statment8162_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SONAME_SYMBOL_in_install_uninstall_statment8162	= { FOLLOW_SONAME_SYMBOL_in_install_uninstall_statment8162_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_install_uninstall_statment8164  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_install_uninstall_statment8164_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_install_uninstall_statment8164	= { FOLLOW_string_literal_in_install_uninstall_statment8164_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNINSTALL_SYMBOL_in_install_uninstall_statment8169  */
static	ANTLR3_BITWORD FOLLOW_UNINSTALL_SYMBOL_in_install_uninstall_statment8169_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNINSTALL_SYMBOL_in_install_uninstall_statment8169	= { FOLLOW_UNINSTALL_SYMBOL_in_install_uninstall_statment8169_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8171  */
static	ANTLR3_BITWORD FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8171_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8171	= { FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8171_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_install_uninstall_statment8173  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_install_uninstall_statment8173_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_install_uninstall_statment8173	= { FOLLOW_identifier_in_install_uninstall_statment8173_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_set_statement8184  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_set_statement8184_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_set_statement8184	= { FOLLOW_SET_SYMBOL_in_set_statement8184_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_type_in_set_statement8206  */
static	ANTLR3_BITWORD FOLLOW_option_type_in_set_statement8206_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_option_type_in_set_statement8206	= { FOLLOW_option_type_in_set_statement8206_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRANSACTION_SYMBOL_in_set_statement8209  */
static	ANTLR3_BITWORD FOLLOW_TRANSACTION_SYMBOL_in_set_statement8209_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TRANSACTION_SYMBOL_in_set_statement8209	= { FOLLOW_TRANSACTION_SYMBOL_in_set_statement8209_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_transaction_characteristic_in_set_statement8211  */
static	ANTLR3_BITWORD FOLLOW_set_transaction_characteristic_in_set_statement8211_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_transaction_characteristic_in_set_statement8211	= { FOLLOW_set_transaction_characteristic_in_set_statement8211_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ONE_SHOT_SYMBOL_in_set_statement8222  */
static	ANTLR3_BITWORD FOLLOW_ONE_SHOT_SYMBOL_in_set_statement8222_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ONE_SHOT_SYMBOL_in_set_statement8222	= { FOLLOW_ONE_SHOT_SYMBOL_in_set_statement8222_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_no_option_type_in_set_statement8225  */
static	ANTLR3_BITWORD FOLLOW_option_value_no_option_type_in_set_statement8225_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_no_option_type_in_set_statement8225	= { FOLLOW_option_value_no_option_type_in_set_statement8225_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_set_statement8228  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_set_statement8228_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_set_statement8228	= { FOLLOW_COMMA_SYMBOL_in_set_statement8228_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_list_in_set_statement8230  */
static	ANTLR3_BITWORD FOLLOW_option_value_list_in_set_statement8230_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_list_in_set_statement8230	= { FOLLOW_option_value_list_in_set_statement8230_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_type_in_set_statement8239  */
static	ANTLR3_BITWORD FOLLOW_option_type_in_set_statement8239_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_option_type_in_set_statement8239	= { FOLLOW_option_type_in_set_statement8239_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_following_option_type_in_set_statement8241  */
static	ANTLR3_BITWORD FOLLOW_option_value_following_option_type_in_set_statement8241_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_following_option_type_in_set_statement8241	= { FOLLOW_option_value_following_option_type_in_set_statement8241_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_set_statement8244  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_set_statement8244_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_set_statement8244	= { FOLLOW_COMMA_SYMBOL_in_set_statement8244_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_list_in_set_statement8246  */
static	ANTLR3_BITWORD FOLLOW_option_value_list_in_set_statement8246_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_list_in_set_statement8246	= { FOLLOW_option_value_list_in_set_statement8246_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NAMES_SYMBOL_in_option_value_no_option_type8271  */
static	ANTLR3_BITWORD FOLLOW_NAMES_SYMBOL_in_option_value_no_option_type8271_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010400000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_NAMES_SYMBOL_in_option_value_no_option_type8271	= { FOLLOW_NAMES_SYMBOL_in_option_value_no_option_type8271_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value_no_option_type8280  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value_no_option_type8280_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value_no_option_type8280	= { FOLLOW_equal_in_option_value_no_option_type8280_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_option_value_no_option_type8282  */
static	ANTLR3_BITWORD FOLLOW_expression_in_option_value_no_option_type8282_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_option_value_no_option_type8282	= { FOLLOW_expression_in_option_value_no_option_type8282_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_or_default_in_option_value_no_option_type8289  */
static	ANTLR3_BITWORD FOLLOW_charset_name_or_default_in_option_value_no_option_type8289_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_or_default_in_option_value_no_option_type8289	= { FOLLOW_charset_name_or_default_in_option_value_no_option_type8289_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_option_value_no_option_type8292  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_option_value_no_option_type8292_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_option_value_no_option_type8292	= { FOLLOW_COLLATE_SYMBOL_in_option_value_no_option_type8292_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_or_default_in_option_value_no_option_type8294  */
static	ANTLR3_BITWORD FOLLOW_collation_name_or_default_in_option_value_no_option_type8294_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_or_default_in_option_value_no_option_type8294	= { FOLLOW_collation_name_or_default_in_option_value_no_option_type8294_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_name_in_option_value_no_option_type8305  */
static	ANTLR3_BITWORD FOLLOW_variable_name_in_option_value_no_option_type8305_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_name_in_option_value_no_option_type8305	= { FOLLOW_variable_name_in_option_value_no_option_type8305_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value_no_option_type8307  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value_no_option_type8307_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030070DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value_no_option_type8307	= { FOLLOW_equal_in_option_value_no_option_type8307_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_expression_or_default_in_option_value_no_option_type8309  */
static	ANTLR3_BITWORD FOLLOW_set_expression_or_default_in_option_value_no_option_type8309_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_expression_or_default_in_option_value_no_option_type8309	= { FOLLOW_set_expression_or_default_in_option_value_no_option_type8309_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_option_value_no_option_type8314  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_option_value_no_option_type8314_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_option_value_no_option_type8314	= { FOLLOW_user_variable_in_option_value_no_option_type8314_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value_no_option_type8316  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value_no_option_type8316_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value_no_option_type8316	= { FOLLOW_equal_in_option_value_no_option_type8316_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_option_value_no_option_type8318  */
static	ANTLR3_BITWORD FOLLOW_expression_in_option_value_no_option_type8318_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_option_value_no_option_type8318	= { FOLLOW_expression_in_option_value_no_option_type8318_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_system_variable_in_option_value_no_option_type8323  */
static	ANTLR3_BITWORD FOLLOW_system_variable_in_option_value_no_option_type8323_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_system_variable_in_option_value_no_option_type8323	= { FOLLOW_system_variable_in_option_value_no_option_type8323_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value_no_option_type8325  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value_no_option_type8325_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030070DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value_no_option_type8325	= { FOLLOW_equal_in_option_value_no_option_type8325_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_expression_or_default_in_option_value_no_option_type8327  */
static	ANTLR3_BITWORD FOLLOW_set_expression_or_default_in_option_value_no_option_type8327_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_expression_or_default_in_option_value_no_option_type8327	= { FOLLOW_set_expression_or_default_in_option_value_no_option_type8327_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_clause_in_option_value_no_option_type8332  */
static	ANTLR3_BITWORD FOLLOW_charset_clause_in_option_value_no_option_type8332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_clause_in_option_value_no_option_type8332	= { FOLLOW_charset_clause_in_option_value_no_option_type8332_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_name_in_option_value_following_option_type8341  */
static	ANTLR3_BITWORD FOLLOW_variable_name_in_option_value_following_option_type8341_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_name_in_option_value_following_option_type8341	= { FOLLOW_variable_name_in_option_value_following_option_type8341_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value_following_option_type8343  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value_following_option_type8343_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030070DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value_following_option_type8343	= { FOLLOW_equal_in_option_value_following_option_type8343_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_expression_or_default_in_option_value_following_option_type8345  */
static	ANTLR3_BITWORD FOLLOW_set_expression_or_default_in_option_value_following_option_type8345_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_expression_or_default_in_option_value_following_option_type8345	= { FOLLOW_set_expression_or_default_in_option_value_following_option_type8345_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_set_expression_or_default8355  */
static	ANTLR3_BITWORD FOLLOW_expression_in_set_expression_or_default8355_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_set_expression_or_default8355	= { FOLLOW_expression_in_set_expression_or_default8355_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_set_expression_or_default8360  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_set_expression_or_default8360_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_set_expression_or_default8360	= { FOLLOW_DEFAULT_SYMBOL_in_set_expression_or_default8360_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_set_expression_or_default8365  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_set_expression_or_default8365_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_set_expression_or_default8365	= { FOLLOW_ON_SYMBOL_in_set_expression_or_default8365_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_set_expression_or_default8370  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_set_expression_or_default8370_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_set_expression_or_default8370	= { FOLLOW_ALL_SYMBOL_in_set_expression_or_default8370_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_set_expression_or_default8375  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_set_expression_or_default8375_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_set_expression_or_default8375	= { FOLLOW_BINARY_SYMBOL_in_set_expression_or_default8375_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_in_option_value_list8384  */
static	ANTLR3_BITWORD FOLLOW_option_value_in_option_value_list8384_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_in_option_value_list8384	= { FOLLOW_option_value_in_option_value_list8384_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_option_value_list8387  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_option_value_list8387_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_option_value_list8387	= { FOLLOW_COMMA_SYMBOL_in_option_value_list8387_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_in_option_value_list8389  */
static	ANTLR3_BITWORD FOLLOW_option_value_in_option_value_list8389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_in_option_value_list8389	= { FOLLOW_option_value_in_option_value_list8389_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_type_in_option_value8400  */
static	ANTLR3_BITWORD FOLLOW_option_type_in_option_value8400_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_option_type_in_option_value8400	= { FOLLOW_option_type_in_option_value8400_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_name_in_option_value8402  */
static	ANTLR3_BITWORD FOLLOW_variable_name_in_option_value8402_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_name_in_option_value8402	= { FOLLOW_variable_name_in_option_value8402_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_equal_in_option_value8404  */
static	ANTLR3_BITWORD FOLLOW_equal_in_option_value8404_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030070DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_equal_in_option_value8404	= { FOLLOW_equal_in_option_value8404_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_expression_or_default_in_option_value8406  */
static	ANTLR3_BITWORD FOLLOW_set_expression_or_default_in_option_value8406_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_expression_or_default_in_option_value8406	= { FOLLOW_set_expression_or_default_in_option_value8406_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_value_no_option_type_in_option_value8411  */
static	ANTLR3_BITWORD FOLLOW_option_value_no_option_type_in_option_value8411_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_option_value_no_option_type_in_option_value8411	= { FOLLOW_option_value_no_option_type_in_option_value8411_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHOW_SYMBOL_in_show_statement8422  */
static	ANTLR3_BITWORD FOLLOW_SHOW_SYMBOL_in_show_statement8422_bits[]	= { ANTLR3_UINT64_LIT(0x0001000080000800), ANTLR3_UINT64_LIT(0x0000020012000210), ANTLR3_UINT64_LIT(0x0210005018020096), ANTLR3_UINT64_LIT(0x6000000000000000), ANTLR3_UINT64_LIT(0x00040000000800E8), ANTLR3_UINT64_LIT(0x00012008080800C0), ANTLR3_UINT64_LIT(0x0000000406010010), ANTLR3_UINT64_LIT(0x0040000080000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_SHOW_SYMBOL_in_show_statement8422	= { FOLLOW_SHOW_SYMBOL_in_show_statement8422_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTHORS_SYMBOL_in_show_statement8431  */
static	ANTLR3_BITWORD FOLLOW_AUTHORS_SYMBOL_in_show_statement8431_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTHORS_SYMBOL_in_show_statement8431	= { FOLLOW_AUTHORS_SYMBOL_in_show_statement8431_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASES_SYMBOL_in_show_statement8437  */
static	ANTLR3_BITWORD FOLLOW_DATABASES_SYMBOL_in_show_statement8437_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASES_SYMBOL_in_show_statement8437	= { FOLLOW_DATABASES_SYMBOL_in_show_statement8437_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8439  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8439_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8439	= { FOLLOW_like_or_where_in_show_statement8439_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FULL_SYMBOL_in_show_statement8446  */
static	ANTLR3_BITWORD FOLLOW_FULL_SYMBOL_in_show_statement8446_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_FULL_SYMBOL_in_show_statement8446	= { FOLLOW_FULL_SYMBOL_in_show_statement8446_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLES_SYMBOL_in_show_statement8449  */
static	ANTLR3_BITWORD FOLLOW_TABLES_SYMBOL_in_show_statement8449_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLES_SYMBOL_in_show_statement8449	= { FOLLOW_TABLES_SYMBOL_in_show_statement8449_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8451  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8451_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8451	= { FOLLOW_in_db_in_show_statement8451_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8454  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8454_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8454	= { FOLLOW_like_or_where_in_show_statement8454_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FULL_SYMBOL_in_show_statement8461  */
static	ANTLR3_BITWORD FOLLOW_FULL_SYMBOL_in_show_statement8461_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000008000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FULL_SYMBOL_in_show_statement8461	= { FOLLOW_FULL_SYMBOL_in_show_statement8461_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIGGERS_SYMBOL_in_show_statement8464  */
static	ANTLR3_BITWORD FOLLOW_TRIGGERS_SYMBOL_in_show_statement8464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIGGERS_SYMBOL_in_show_statement8464	= { FOLLOW_TRIGGERS_SYMBOL_in_show_statement8464_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8466  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8466_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8466	= { FOLLOW_in_db_in_show_statement8466_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8469  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8469	= { FOLLOW_like_or_where_in_show_statement8469_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENTS_SYMBOL_in_show_statement8476  */
static	ANTLR3_BITWORD FOLLOW_EVENTS_SYMBOL_in_show_statement8476_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENTS_SYMBOL_in_show_statement8476	= { FOLLOW_EVENTS_SYMBOL_in_show_statement8476_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8478  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8478	= { FOLLOW_in_db_in_show_statement8478_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8481  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8481_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8481	= { FOLLOW_like_or_where_in_show_statement8481_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_show_statement8488  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_show_statement8488_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_show_statement8488	= { FOLLOW_TABLE_SYMBOL_in_show_statement8488_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8490  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8490_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8490	= { FOLLOW_STATUS_SYMBOL_in_show_statement8490_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8492  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8492_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8492	= { FOLLOW_in_db_in_show_statement8492_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8495  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8495_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8495	= { FOLLOW_like_or_where_in_show_statement8495_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_SYMBOL_in_show_statement8502  */
static	ANTLR3_BITWORD FOLLOW_OPEN_SYMBOL_in_show_statement8502_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_SYMBOL_in_show_statement8502	= { FOLLOW_OPEN_SYMBOL_in_show_statement8502_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLES_SYMBOL_in_show_statement8504  */
static	ANTLR3_BITWORD FOLLOW_TABLES_SYMBOL_in_show_statement8504_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLES_SYMBOL_in_show_statement8504	= { FOLLOW_TABLES_SYMBOL_in_show_statement8504_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8506  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8506_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8506	= { FOLLOW_in_db_in_show_statement8506_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8509  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8509_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8509	= { FOLLOW_like_or_where_in_show_statement8509_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUGIN_SYMBOL_in_show_statement8518  */
static	ANTLR3_BITWORD FOLLOW_PLUGIN_SYMBOL_in_show_statement8518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUGIN_SYMBOL_in_show_statement8518	= { FOLLOW_PLUGIN_SYMBOL_in_show_statement8518_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUGINS_SYMBOL_in_show_statement8527  */
static	ANTLR3_BITWORD FOLLOW_PLUGINS_SYMBOL_in_show_statement8527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUGINS_SYMBOL_in_show_statement8527	= { FOLLOW_PLUGINS_SYMBOL_in_show_statement8527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_show_statement8533  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_show_statement8533_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_show_statement8533	= { FOLLOW_ENGINE_SYMBOL_in_show_statement8533_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_show_statement8535  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_show_statement8535_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_show_statement8535	= { FOLLOW_engine_ref_in_show_statement8535_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8537  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8537_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8537	= { FOLLOW_set_in_show_statement8537_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FULL_SYMBOL_in_show_statement8549  */
static	ANTLR3_BITWORD FOLLOW_FULL_SYMBOL_in_show_statement8549_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FULL_SYMBOL_in_show_statement8549	= { FOLLOW_FULL_SYMBOL_in_show_statement8549_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMNS_SYMBOL_in_show_statement8552  */
static	ANTLR3_BITWORD FOLLOW_COLUMNS_SYMBOL_in_show_statement8552_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMNS_SYMBOL_in_show_statement8552	= { FOLLOW_COLUMNS_SYMBOL_in_show_statement8552_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8554  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8554_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8554	= { FOLLOW_set_in_show_statement8554_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_show_statement8562  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_show_statement8562_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_show_statement8562	= { FOLLOW_table_ref_in_show_statement8562_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8564  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8564_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8564	= { FOLLOW_in_db_in_show_statement8564_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8567  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8567_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8567	= { FOLLOW_like_or_where_in_show_statement8567_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8574  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8574_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8574	= { FOLLOW_set_in_show_statement8574_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGS_SYMBOL_in_show_statement8582  */
static	ANTLR3_BITWORD FOLLOW_LOGS_SYMBOL_in_show_statement8582_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGS_SYMBOL_in_show_statement8582	= { FOLLOW_LOGS_SYMBOL_in_show_statement8582_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_show_statement8588  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_show_statement8588_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_show_statement8588	= { FOLLOW_SLAVE_SYMBOL_in_show_statement8588_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOSTS_SYMBOL_in_show_statement8599  */
static	ANTLR3_BITWORD FOLLOW_HOSTS_SYMBOL_in_show_statement8599_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HOSTS_SYMBOL_in_show_statement8599	= { FOLLOW_HOSTS_SYMBOL_in_show_statement8599_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8607  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8607_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000200000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8607	= { FOLLOW_STATUS_SYMBOL_in_show_statement8607_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_non_blocking_in_show_statement8609  */
static	ANTLR3_BITWORD FOLLOW_non_blocking_in_show_statement8609_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_non_blocking_in_show_statement8609	= { FOLLOW_non_blocking_in_show_statement8609_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_show_statement8611  */
static	ANTLR3_BITWORD FOLLOW_channel_in_show_statement8611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_show_statement8611	= { FOLLOW_channel_in_show_statement8611_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8623  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8623_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8623	= { FOLLOW_set_in_show_statement8623_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENTS_SYMBOL_in_show_statement8631  */
static	ANTLR3_BITWORD FOLLOW_EVENTS_SYMBOL_in_show_statement8631_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000400000028000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENTS_SYMBOL_in_show_statement8631	= { FOLLOW_EVENTS_SYMBOL_in_show_statement8631_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_show_statement8634  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_show_statement8634_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_show_statement8634	= { FOLLOW_IN_SYMBOL_in_show_statement8634_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_show_statement8636  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_show_statement8636_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000028000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_show_statement8636	= { FOLLOW_text_string_in_show_statement8636_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_show_statement8641  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_show_statement8641_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_show_statement8641	= { FOLLOW_FROM_SYMBOL_in_show_statement8641_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulonglong_number_in_show_statement8643  */
static	ANTLR3_BITWORD FOLLOW_ulonglong_number_in_show_statement8643_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulonglong_number_in_show_statement8643	= { FOLLOW_ulonglong_number_in_show_statement8643_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_show_statement8647  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_show_statement8647_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_show_statement8647	= { FOLLOW_limit_clause_in_show_statement8647_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_show_statement8650  */
static	ANTLR3_BITWORD FOLLOW_channel_in_show_statement8650_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_show_statement8650	= { FOLLOW_channel_in_show_statement8650_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8657  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8657_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8657	= { FOLLOW_set_in_show_statement8657_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_from_or_in_in_show_statement8669  */
static	ANTLR3_BITWORD FOLLOW_from_or_in_in_show_statement8669_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_from_or_in_in_show_statement8669	= { FOLLOW_from_or_in_in_show_statement8669_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_show_statement8671  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_show_statement8671_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_show_statement8671	= { FOLLOW_table_ref_in_show_statement8671_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_db_in_show_statement8673  */
static	ANTLR3_BITWORD FOLLOW_in_db_in_show_statement8673_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_db_in_show_statement8673	= { FOLLOW_in_db_in_show_statement8673_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_show_statement8676  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_show_statement8676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_show_statement8676	= { FOLLOW_where_clause_in_show_statement8676_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_show_statement8683  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_show_statement8683_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_show_statement8683	= { FOLLOW_STORAGE_SYMBOL_in_show_statement8683_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINES_SYMBOL_in_show_statement8686  */
static	ANTLR3_BITWORD FOLLOW_ENGINES_SYMBOL_in_show_statement8686_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINES_SYMBOL_in_show_statement8686	= { FOLLOW_ENGINES_SYMBOL_in_show_statement8686_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIVILEGES_SYMBOL_in_show_statement8692  */
static	ANTLR3_BITWORD FOLLOW_PRIVILEGES_SYMBOL_in_show_statement8692_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIVILEGES_SYMBOL_in_show_statement8692	= { FOLLOW_PRIVILEGES_SYMBOL_in_show_statement8692_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COUNT_SYMBOL_in_show_statement8698  */
static	ANTLR3_BITWORD FOLLOW_COUNT_SYMBOL_in_show_statement8698_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COUNT_SYMBOL_in_show_statement8698	= { FOLLOW_COUNT_SYMBOL_in_show_statement8698_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_show_statement8700  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_show_statement8700_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_show_statement8700	= { FOLLOW_OPEN_PAR_SYMBOL_in_show_statement8700_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_show_statement8702  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_show_statement8702_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_show_statement8702	= { FOLLOW_MULT_OPERATOR_in_show_statement8702_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_show_statement8704  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_show_statement8704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_show_statement8704	= { FOLLOW_CLOSE_PAR_SYMBOL_in_show_statement8704_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8706  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8706_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8706	= { FOLLOW_set_in_show_statement8706_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WARNINGS_SYMBOL_in_show_statement8718  */
static	ANTLR3_BITWORD FOLLOW_WARNINGS_SYMBOL_in_show_statement8718_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WARNINGS_SYMBOL_in_show_statement8718	= { FOLLOW_WARNINGS_SYMBOL_in_show_statement8718_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_show_statement8720  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_show_statement8720_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_show_statement8720	= { FOLLOW_limit_clause_in_show_statement8720_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ERRORS_SYMBOL_in_show_statement8727  */
static	ANTLR3_BITWORD FOLLOW_ERRORS_SYMBOL_in_show_statement8727_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ERRORS_SYMBOL_in_show_statement8727	= { FOLLOW_ERRORS_SYMBOL_in_show_statement8727_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_show_statement8729  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_show_statement8729_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_show_statement8729	= { FOLLOW_limit_clause_in_show_statement8729_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROFILES_SYMBOL_in_show_statement8736  */
static	ANTLR3_BITWORD FOLLOW_PROFILES_SYMBOL_in_show_statement8736_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PROFILES_SYMBOL_in_show_statement8736	= { FOLLOW_PROFILES_SYMBOL_in_show_statement8736_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROFILE_SYMBOL_in_show_statement8742  */
static	ANTLR3_BITWORD FOLLOW_PROFILE_SYMBOL_in_show_statement8742_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000050000000800), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0040000000200000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0800000000010000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROFILE_SYMBOL_in_show_statement8742	= { FOLLOW_PROFILE_SYMBOL_in_show_statement8742_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_profile_type_in_show_statement8745  */
static	ANTLR3_BITWORD FOLLOW_profile_type_in_show_statement8745_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_profile_type_in_show_statement8745	= { FOLLOW_profile_type_in_show_statement8745_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_show_statement8748  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_show_statement8748_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000050000000800), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0040000000200000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0800000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_show_statement8748	= { FOLLOW_COMMA_SYMBOL_in_show_statement8748_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_profile_type_in_show_statement8750  */
static	ANTLR3_BITWORD FOLLOW_profile_type_in_show_statement8750_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000008000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_profile_type_in_show_statement8750	= { FOLLOW_profile_type_in_show_statement8750_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_show_statement8757  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_show_statement8757_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_show_statement8757	= { FOLLOW_FOR_SYMBOL_in_show_statement8757_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUERY_SYMBOL_in_show_statement8759  */
static	ANTLR3_BITWORD FOLLOW_QUERY_SYMBOL_in_show_statement8759_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_QUERY_SYMBOL_in_show_statement8759	= { FOLLOW_QUERY_SYMBOL_in_show_statement8759_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_show_statement8761  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_show_statement8761_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_show_statement8761	= { FOLLOW_INT_NUMBER_in_show_statement8761_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_limit_clause_in_show_statement8765  */
static	ANTLR3_BITWORD FOLLOW_limit_clause_in_show_statement8765_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_limit_clause_in_show_statement8765	= { FOLLOW_limit_clause_in_show_statement8765_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_type_in_show_statement8772  */
static	ANTLR3_BITWORD FOLLOW_option_type_in_show_statement8772_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_option_type_in_show_statement8772	= { FOLLOW_option_type_in_show_statement8772_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_show_statement8775  */
static	ANTLR3_BITWORD FOLLOW_set_in_show_statement8775_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_show_statement8775	= { FOLLOW_set_in_show_statement8775_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8783  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8783	= { FOLLOW_like_or_where_in_show_statement8783_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FULL_SYMBOL_in_show_statement8790  */
static	ANTLR3_BITWORD FOLLOW_FULL_SYMBOL_in_show_statement8790_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_FULL_SYMBOL_in_show_statement8790	= { FOLLOW_FULL_SYMBOL_in_show_statement8790_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCESSLIST_SYMBOL_in_show_statement8793  */
static	ANTLR3_BITWORD FOLLOW_PROCESSLIST_SYMBOL_in_show_statement8793_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCESSLIST_SYMBOL_in_show_statement8793	= { FOLLOW_PROCESSLIST_SYMBOL_in_show_statement8793_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_show_statement8799  */
static	ANTLR3_BITWORD FOLLOW_charset_in_show_statement8799_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_show_statement8799	= { FOLLOW_charset_in_show_statement8799_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8801  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8801_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8801	= { FOLLOW_like_or_where_in_show_statement8801_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATION_SYMBOL_in_show_statement8808  */
static	ANTLR3_BITWORD FOLLOW_COLLATION_SYMBOL_in_show_statement8808_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATION_SYMBOL_in_show_statement8808	= { FOLLOW_COLLATION_SYMBOL_in_show_statement8808_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8810  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8810_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8810	= { FOLLOW_like_or_where_in_show_statement8810_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONTRIBUTORS_SYMBOL_in_show_statement8819  */
static	ANTLR3_BITWORD FOLLOW_CONTRIBUTORS_SYMBOL_in_show_statement8819_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CONTRIBUTORS_SYMBOL_in_show_statement8819	= { FOLLOW_CONTRIBUTORS_SYMBOL_in_show_statement8819_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GRANTS_SYMBOL_in_show_statement8825  */
static	ANTLR3_BITWORD FOLLOW_GRANTS_SYMBOL_in_show_statement8825_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_GRANTS_SYMBOL_in_show_statement8825	= { FOLLOW_GRANTS_SYMBOL_in_show_statement8825_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_show_statement8828  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_show_statement8828_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_show_statement8828	= { FOLLOW_FOR_SYMBOL_in_show_statement8828_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_show_statement8830  */
static	ANTLR3_BITWORD FOLLOW_user_in_show_statement8830_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_show_statement8830	= { FOLLOW_user_in_show_statement8830_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SYMBOL_in_show_statement8838  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SYMBOL_in_show_statement8838_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SYMBOL_in_show_statement8838	= { FOLLOW_MASTER_SYMBOL_in_show_statement8838_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8840  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8840_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8840	= { FOLLOW_STATUS_SYMBOL_in_show_statement8840_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CREATE_SYMBOL_in_show_statement8846  */
static	ANTLR3_BITWORD FOLLOW_CREATE_SYMBOL_in_show_statement8846_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000480800000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000101) };
static  ANTLR3_BITSET_LIST FOLLOW_CREATE_SYMBOL_in_show_statement8846	= { FOLLOW_CREATE_SYMBOL_in_show_statement8846_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_show_statement8857  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_show_statement8857_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_show_statement8857	= { FOLLOW_DATABASE_SYMBOL_in_show_statement8857_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_not_exists_in_show_statement8859  */
static	ANTLR3_BITWORD FOLLOW_if_not_exists_in_show_statement8859_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_not_exists_in_show_statement8859	= { FOLLOW_if_not_exists_in_show_statement8859_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_show_statement8862  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_show_statement8862_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_show_statement8862	= { FOLLOW_schema_ref_in_show_statement8862_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVENT_SYMBOL_in_show_statement8870  */
static	ANTLR3_BITWORD FOLLOW_EVENT_SYMBOL_in_show_statement8870_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EVENT_SYMBOL_in_show_statement8870	= { FOLLOW_EVENT_SYMBOL_in_show_statement8870_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_event_ref_in_show_statement8872  */
static	ANTLR3_BITWORD FOLLOW_event_ref_in_show_statement8872_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_event_ref_in_show_statement8872	= { FOLLOW_event_ref_in_show_statement8872_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_show_statement8880  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_show_statement8880_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_show_statement8880	= { FOLLOW_FUNCTION_SYMBOL_in_show_statement8880_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_ref_in_show_statement8882  */
static	ANTLR3_BITWORD FOLLOW_function_ref_in_show_statement8882_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_ref_in_show_statement8882	= { FOLLOW_function_ref_in_show_statement8882_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_show_statement8890  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_show_statement8890_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_show_statement8890	= { FOLLOW_PROCEDURE_SYMBOL_in_show_statement8890_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_ref_in_show_statement8892  */
static	ANTLR3_BITWORD FOLLOW_procedure_ref_in_show_statement8892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_ref_in_show_statement8892	= { FOLLOW_procedure_ref_in_show_statement8892_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLE_SYMBOL_in_show_statement8900  */
static	ANTLR3_BITWORD FOLLOW_TABLE_SYMBOL_in_show_statement8900_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLE_SYMBOL_in_show_statement8900	= { FOLLOW_TABLE_SYMBOL_in_show_statement8900_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_show_statement8902  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_show_statement8902_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_show_statement8902	= { FOLLOW_table_ref_in_show_statement8902_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIGGER_SYMBOL_in_show_statement8910  */
static	ANTLR3_BITWORD FOLLOW_TRIGGER_SYMBOL_in_show_statement8910_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIGGER_SYMBOL_in_show_statement8910	= { FOLLOW_TRIGGER_SYMBOL_in_show_statement8910_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_trigger_ref_in_show_statement8912  */
static	ANTLR3_BITWORD FOLLOW_trigger_ref_in_show_statement8912_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_trigger_ref_in_show_statement8912	= { FOLLOW_trigger_ref_in_show_statement8912_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VIEW_SYMBOL_in_show_statement8920  */
static	ANTLR3_BITWORD FOLLOW_VIEW_SYMBOL_in_show_statement8920_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_VIEW_SYMBOL_in_show_statement8920	= { FOLLOW_VIEW_SYMBOL_in_show_statement8920_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_ref_in_show_statement8922  */
static	ANTLR3_BITWORD FOLLOW_view_ref_in_show_statement8922_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_view_ref_in_show_statement8922	= { FOLLOW_view_ref_in_show_statement8922_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_show_statement8932  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_show_statement8932_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_show_statement8932	= { FOLLOW_USER_SYMBOL_in_show_statement8932_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_show_statement8934  */
static	ANTLR3_BITWORD FOLLOW_user_in_show_statement8934_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_show_statement8934	= { FOLLOW_user_in_show_statement8934_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_show_statement8945  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_show_statement8945_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_show_statement8945	= { FOLLOW_PROCEDURE_SYMBOL_in_show_statement8945_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8947  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8947_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8947	= { FOLLOW_STATUS_SYMBOL_in_show_statement8947_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8949  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8949_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8949	= { FOLLOW_like_or_where_in_show_statement8949_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_show_statement8956  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_show_statement8956_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_show_statement8956	= { FOLLOW_FUNCTION_SYMBOL_in_show_statement8956_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8958  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8958_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8958	= { FOLLOW_STATUS_SYMBOL_in_show_statement8958_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_or_where_in_show_statement8960  */
static	ANTLR3_BITWORD FOLLOW_like_or_where_in_show_statement8960_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_or_where_in_show_statement8960	= { FOLLOW_like_or_where_in_show_statement8960_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PROCEDURE_SYMBOL_in_show_statement8967  */
static	ANTLR3_BITWORD FOLLOW_PROCEDURE_SYMBOL_in_show_statement8967_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000001000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PROCEDURE_SYMBOL_in_show_statement8967	= { FOLLOW_PROCEDURE_SYMBOL_in_show_statement8967_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CODE_SYMBOL_in_show_statement8969  */
static	ANTLR3_BITWORD FOLLOW_CODE_SYMBOL_in_show_statement8969_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_CODE_SYMBOL_in_show_statement8969	= { FOLLOW_CODE_SYMBOL_in_show_statement8969_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_procedure_ref_in_show_statement8971  */
static	ANTLR3_BITWORD FOLLOW_procedure_ref_in_show_statement8971_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_procedure_ref_in_show_statement8971	= { FOLLOW_procedure_ref_in_show_statement8971_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FUNCTION_SYMBOL_in_show_statement8977  */
static	ANTLR3_BITWORD FOLLOW_FUNCTION_SYMBOL_in_show_statement8977_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000001000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FUNCTION_SYMBOL_in_show_statement8977	= { FOLLOW_FUNCTION_SYMBOL_in_show_statement8977_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CODE_SYMBOL_in_show_statement8979  */
static	ANTLR3_BITWORD FOLLOW_CODE_SYMBOL_in_show_statement8979_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_CODE_SYMBOL_in_show_statement8979	= { FOLLOW_CODE_SYMBOL_in_show_statement8979_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_ref_in_show_statement8981  */
static	ANTLR3_BITWORD FOLLOW_function_ref_in_show_statement8981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_ref_in_show_statement8981	= { FOLLOW_function_ref_in_show_statement8981_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INNODB_SYMBOL_in_show_statement8989  */
static	ANTLR3_BITWORD FOLLOW_INNODB_SYMBOL_in_show_statement8989_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_INNODB_SYMBOL_in_show_statement8989	= { FOLLOW_INNODB_SYMBOL_in_show_statement8989_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_show_statement8991  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_show_statement8991_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_show_statement8991	= { FOLLOW_STATUS_SYMBOL_in_show_statement8991_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NONBLOCKING_SYMBOL_in_non_blocking9006  */
static	ANTLR3_BITWORD FOLLOW_NONBLOCKING_SYMBOL_in_non_blocking9006_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NONBLOCKING_SYMBOL_in_non_blocking9006	= { FOLLOW_NONBLOCKING_SYMBOL_in_non_blocking9006_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_from_or_in_in_in_db9034  */
static	ANTLR3_BITWORD FOLLOW_from_or_in_in_in_db9034_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_from_or_in_in_in_db9034	= { FOLLOW_from_or_in_in_in_db9034_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_in_db9036  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_in_db9036_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_in_db9036	= { FOLLOW_identifier_in_in_db9036_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_profile_type9045  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_profile_type9045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_profile_type9045	= { FOLLOW_ALL_SYMBOL_in_profile_type9045_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BLOCK_SYMBOL_in_profile_type9050  */
static	ANTLR3_BITWORD FOLLOW_BLOCK_SYMBOL_in_profile_type9050_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000008000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BLOCK_SYMBOL_in_profile_type9050	= { FOLLOW_BLOCK_SYMBOL_in_profile_type9050_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IO_SYMBOL_in_profile_type9052  */
static	ANTLR3_BITWORD FOLLOW_IO_SYMBOL_in_profile_type9052_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_IO_SYMBOL_in_profile_type9052	= { FOLLOW_IO_SYMBOL_in_profile_type9052_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONTEXT_SYMBOL_in_profile_type9057  */
static	ANTLR3_BITWORD FOLLOW_CONTEXT_SYMBOL_in_profile_type9057_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONTEXT_SYMBOL_in_profile_type9057	= { FOLLOW_CONTEXT_SYMBOL_in_profile_type9057_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SWITCHES_SYMBOL_in_profile_type9059  */
static	ANTLR3_BITWORD FOLLOW_SWITCHES_SYMBOL_in_profile_type9059_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SWITCHES_SYMBOL_in_profile_type9059	= { FOLLOW_SWITCHES_SYMBOL_in_profile_type9059_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CPU_SYMBOL_in_profile_type9064  */
static	ANTLR3_BITWORD FOLLOW_CPU_SYMBOL_in_profile_type9064_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CPU_SYMBOL_in_profile_type9064	= { FOLLOW_CPU_SYMBOL_in_profile_type9064_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IPC_SYMBOL_in_profile_type9069  */
static	ANTLR3_BITWORD FOLLOW_IPC_SYMBOL_in_profile_type9069_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_IPC_SYMBOL_in_profile_type9069	= { FOLLOW_IPC_SYMBOL_in_profile_type9069_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MEMORY_SYMBOL_in_profile_type9074  */
static	ANTLR3_BITWORD FOLLOW_MEMORY_SYMBOL_in_profile_type9074_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MEMORY_SYMBOL_in_profile_type9074	= { FOLLOW_MEMORY_SYMBOL_in_profile_type9074_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PAGE_SYMBOL_in_profile_type9079  */
static	ANTLR3_BITWORD FOLLOW_PAGE_SYMBOL_in_profile_type9079_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_PAGE_SYMBOL_in_profile_type9079	= { FOLLOW_PAGE_SYMBOL_in_profile_type9079_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FAULTS_SYMBOL_in_profile_type9081  */
static	ANTLR3_BITWORD FOLLOW_FAULTS_SYMBOL_in_profile_type9081_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FAULTS_SYMBOL_in_profile_type9081	= { FOLLOW_FAULTS_SYMBOL_in_profile_type9081_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SOURCE_SYMBOL_in_profile_type9086  */
static	ANTLR3_BITWORD FOLLOW_SOURCE_SYMBOL_in_profile_type9086_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SOURCE_SYMBOL_in_profile_type9086	= { FOLLOW_SOURCE_SYMBOL_in_profile_type9086_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SWAPS_SYMBOL_in_profile_type9091  */
static	ANTLR3_BITWORD FOLLOW_SWAPS_SYMBOL_in_profile_type9091_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SWAPS_SYMBOL_in_profile_type9091	= { FOLLOW_SWAPS_SYMBOL_in_profile_type9091_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINLOG_SYMBOL_in_other_administrative_statement9102  */
static	ANTLR3_BITWORD FOLLOW_BINLOG_SYMBOL_in_other_administrative_statement9102_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINLOG_SYMBOL_in_other_administrative_statement9102	= { FOLLOW_BINLOG_SYMBOL_in_other_administrative_statement9102_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_other_administrative_statement9104  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_other_administrative_statement9104_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_other_administrative_statement9104	= { FOLLOW_string_literal_in_other_administrative_statement9104_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9109  */
static	ANTLR3_BITWORD FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9109_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9109	= { FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9109_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9111  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9111_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9111	= { FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9111_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_cache_list_or_parts_in_other_administrative_statement9113  */
static	ANTLR3_BITWORD FOLLOW_key_cache_list_or_parts_in_other_administrative_statement9113_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_key_cache_list_or_parts_in_other_administrative_statement9113	= { FOLLOW_key_cache_list_or_parts_in_other_administrative_statement9113_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_other_administrative_statement9115  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_other_administrative_statement9115_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_other_administrative_statement9115	= { FOLLOW_IN_SYMBOL_in_other_administrative_statement9115_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_other_administrative_statement9118  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_other_administrative_statement9118_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_other_administrative_statement9118	= { FOLLOW_identifier_in_other_administrative_statement9118_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_other_administrative_statement9122  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_other_administrative_statement9122_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_other_administrative_statement9122	= { FOLLOW_DEFAULT_SYMBOL_in_other_administrative_statement9122_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FLUSH_SYMBOL_in_other_administrative_statement9128  */
static	ANTLR3_BITWORD FOLLOW_FLUSH_SYMBOL_in_other_administrative_statement9128_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x009000004080000A), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000040408), ANTLR3_UINT64_LIT(0x0000040000080040), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000802000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_FLUSH_SYMBOL_in_other_administrative_statement9128	= { FOLLOW_FLUSH_SYMBOL_in_other_administrative_statement9128_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_no_write_to_bin_log_in_other_administrative_statement9130  */
static	ANTLR3_BITWORD FOLLOW_no_write_to_bin_log_in_other_administrative_statement9130_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x008000004080000A), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000040408), ANTLR3_UINT64_LIT(0x0000040000080040), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_no_write_to_bin_log_in_other_administrative_statement9130	= { FOLLOW_no_write_to_bin_log_in_other_administrative_statement9130_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_flush_tables_in_other_administrative_statement9140  */
static	ANTLR3_BITWORD FOLLOW_flush_tables_in_other_administrative_statement9140_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_flush_tables_in_other_administrative_statement9140	= { FOLLOW_flush_tables_in_other_administrative_statement9140_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_flush_option_in_other_administrative_statement9147  */
static	ANTLR3_BITWORD FOLLOW_flush_option_in_other_administrative_statement9147_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_flush_option_in_other_administrative_statement9147	= { FOLLOW_flush_option_in_other_administrative_statement9147_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9150  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9150_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000), ANTLR3_UINT64_LIT(0x008000004080000A), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000040408), ANTLR3_UINT64_LIT(0x0000040000000040), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9150	= { FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9150_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_flush_option_in_other_administrative_statement9152  */
static	ANTLR3_BITWORD FOLLOW_flush_option_in_other_administrative_statement9152_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_flush_option_in_other_administrative_statement9152	= { FOLLOW_flush_option_in_other_administrative_statement9152_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KILL_SYMBOL_in_other_administrative_statement9163  */
static	ANTLR3_BITWORD FOLLOW_KILL_SYMBOL_in_other_administrative_statement9163_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_KILL_SYMBOL_in_other_administrative_statement9163	= { FOLLOW_KILL_SYMBOL_in_other_administrative_statement9163_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_other_administrative_statement9179  */
static	ANTLR3_BITWORD FOLLOW_set_in_other_administrative_statement9179_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_other_administrative_statement9179	= { FOLLOW_set_in_other_administrative_statement9179_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_other_administrative_statement9189  */
static	ANTLR3_BITWORD FOLLOW_expression_in_other_administrative_statement9189_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_other_administrative_statement9189	= { FOLLOW_expression_in_other_administrative_statement9189_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOAD_SYMBOL_in_other_administrative_statement9194  */
static	ANTLR3_BITWORD FOLLOW_LOAD_SYMBOL_in_other_administrative_statement9194_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOAD_SYMBOL_in_other_administrative_statement9194	= { FOLLOW_LOAD_SYMBOL_in_other_administrative_statement9194_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9196  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9196_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9196	= { FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9196_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_other_administrative_statement9198  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_other_administrative_statement9198_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_other_administrative_statement9198	= { FOLLOW_INTO_SYMBOL_in_other_administrative_statement9198_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9200  */
static	ANTLR3_BITWORD FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9200_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9200	= { FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9200_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_load_table_index_list_in_other_administrative_statement9202  */
static	ANTLR3_BITWORD FOLLOW_load_table_index_list_in_other_administrative_statement9202_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_load_table_index_list_in_other_administrative_statement9202	= { FOLLOW_load_table_index_list_in_other_administrative_statement9202_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RESET_SYMBOL_in_other_administrative_statement9207  */
static	ANTLR3_BITWORD FOLLOW_RESET_SYMBOL_in_other_administrative_statement9207_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_RESET_SYMBOL_in_other_administrative_statement9207	= { FOLLOW_RESET_SYMBOL_in_other_administrative_statement9207_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reset_option_in_other_administrative_statement9209  */
static	ANTLR3_BITWORD FOLLOW_reset_option_in_other_administrative_statement9209_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_reset_option_in_other_administrative_statement9209	= { FOLLOW_reset_option_in_other_administrative_statement9209_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9212  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9212_bits[]	= { ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9212	= { FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9212_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reset_option_in_other_administrative_statement9214  */
static	ANTLR3_BITWORD FOLLOW_reset_option_in_other_administrative_statement9214_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_reset_option_in_other_administrative_statement9214	= { FOLLOW_reset_option_in_other_administrative_statement9214_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SHUTDOWN_SYMBOL_in_other_administrative_statement9223  */
static	ANTLR3_BITWORD FOLLOW_SHUTDOWN_SYMBOL_in_other_administrative_statement9223_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SHUTDOWN_SYMBOL_in_other_administrative_statement9223	= { FOLLOW_SHUTDOWN_SYMBOL_in_other_administrative_statement9223_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_cache_list_in_key_cache_list_or_parts9243  */
static	ANTLR3_BITWORD FOLLOW_key_cache_list_in_key_cache_list_or_parts9243_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_key_cache_list_in_key_cache_list_or_parts9243	= { FOLLOW_key_cache_list_in_key_cache_list_or_parts9243_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_assign_to_keycache_partition_in_key_cache_list_or_parts9248  */
static	ANTLR3_BITWORD FOLLOW_assign_to_keycache_partition_in_key_cache_list_or_parts9248_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_assign_to_keycache_partition_in_key_cache_list_or_parts9248	= { FOLLOW_assign_to_keycache_partition_in_key_cache_list_or_parts9248_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_assign_to_keycache_in_key_cache_list9257  */
static	ANTLR3_BITWORD FOLLOW_assign_to_keycache_in_key_cache_list9257_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_assign_to_keycache_in_key_cache_list9257	= { FOLLOW_assign_to_keycache_in_key_cache_list9257_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_key_cache_list9260  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_key_cache_list9260_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_key_cache_list9260	= { FOLLOW_COMMA_SYMBOL_in_key_cache_list9260_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_assign_to_keycache_in_key_cache_list9262  */
static	ANTLR3_BITWORD FOLLOW_assign_to_keycache_in_key_cache_list9262_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_assign_to_keycache_in_key_cache_list9262	= { FOLLOW_assign_to_keycache_in_key_cache_list9262_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_assign_to_keycache9273  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_assign_to_keycache9273_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_assign_to_keycache9273	= { FOLLOW_table_ref_in_assign_to_keycache9273_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cache_keys_spec_in_assign_to_keycache9275  */
static	ANTLR3_BITWORD FOLLOW_cache_keys_spec_in_assign_to_keycache9275_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cache_keys_spec_in_assign_to_keycache9275	= { FOLLOW_cache_keys_spec_in_assign_to_keycache9275_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_assign_to_keycache_partition9285  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_assign_to_keycache_partition9285_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_assign_to_keycache_partition9285	= { FOLLOW_table_ref_in_assign_to_keycache_partition9285_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_assign_to_keycache_partition9287  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_assign_to_keycache_partition9287_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_assign_to_keycache_partition9287	= { FOLLOW_PARTITION_SYMBOL_in_assign_to_keycache_partition9287_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_assign_to_keycache_partition9289  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_assign_to_keycache_partition9289_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_assign_to_keycache_partition9289	= { FOLLOW_OPEN_PAR_SYMBOL_in_assign_to_keycache_partition9289_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_assign_to_keycache_partition9292  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_assign_to_keycache_partition9292_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_assign_to_keycache_partition9292	= { FOLLOW_ALL_SYMBOL_in_assign_to_keycache_partition9292_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_assign_to_keycache_partition9296  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_assign_to_keycache_partition9296_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_assign_to_keycache_partition9296	= { FOLLOW_identifier_list_in_assign_to_keycache_partition9296_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_assign_to_keycache_partition9299  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_assign_to_keycache_partition9299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_assign_to_keycache_partition9299	= { FOLLOW_CLOSE_PAR_SYMBOL_in_assign_to_keycache_partition9299_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cache_keys_spec_in_assign_to_keycache_partition9301  */
static	ANTLR3_BITWORD FOLLOW_cache_keys_spec_in_assign_to_keycache_partition9301_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cache_keys_spec_in_assign_to_keycache_partition9301	= { FOLLOW_cache_keys_spec_in_assign_to_keycache_partition9301_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_cache_keys_spec9311  */
static	ANTLR3_BITWORD FOLLOW_set_in_cache_keys_spec9311_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_cache_keys_spec9311	= { FOLLOW_set_in_cache_keys_spec9311_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_cache_keys_spec9319  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_cache_keys_spec9319_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_cache_keys_spec9319	= { FOLLOW_OPEN_PAR_SYMBOL_in_cache_keys_spec9319_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_usage_element_in_cache_keys_spec9322  */
static	ANTLR3_BITWORD FOLLOW_key_usage_element_in_cache_keys_spec9322_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_key_usage_element_in_cache_keys_spec9322	= { FOLLOW_key_usage_element_in_cache_keys_spec9322_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_cache_keys_spec9325  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_cache_keys_spec9325_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_cache_keys_spec9325	= { FOLLOW_COMMA_SYMBOL_in_cache_keys_spec9325_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_key_usage_element_in_cache_keys_spec9327  */
static	ANTLR3_BITWORD FOLLOW_key_usage_element_in_cache_keys_spec9327_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_key_usage_element_in_cache_keys_spec9327	= { FOLLOW_key_usage_element_in_cache_keys_spec9327_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_cache_keys_spec9333  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_cache_keys_spec9333_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_cache_keys_spec9333	= { FOLLOW_CLOSE_PAR_SYMBOL_in_cache_keys_spec9333_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_key_usage_element9342  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_key_usage_element9342_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_key_usage_element9342	= { FOLLOW_identifier_in_key_usage_element9342_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIMARY_SYMBOL_in_key_usage_element9347  */
static	ANTLR3_BITWORD FOLLOW_PRIMARY_SYMBOL_in_key_usage_element9347_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIMARY_SYMBOL_in_key_usage_element9347	= { FOLLOW_PRIMARY_SYMBOL_in_key_usage_element9347_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DES_KEY_FILE_SYMBOL_in_flush_option9356  */
static	ANTLR3_BITWORD FOLLOW_DES_KEY_FILE_SYMBOL_in_flush_option9356_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DES_KEY_FILE_SYMBOL_in_flush_option9356	= { FOLLOW_DES_KEY_FILE_SYMBOL_in_flush_option9356_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOSTS_SYMBOL_in_flush_option9361  */
static	ANTLR3_BITWORD FOLLOW_HOSTS_SYMBOL_in_flush_option9361_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HOSTS_SYMBOL_in_flush_option9361	= { FOLLOW_HOSTS_SYMBOL_in_flush_option9361_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_log_type_in_flush_option9366  */
static	ANTLR3_BITWORD FOLLOW_log_type_in_flush_option9366_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_log_type_in_flush_option9366	= { FOLLOW_log_type_in_flush_option9366_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGS_SYMBOL_in_flush_option9369  */
static	ANTLR3_BITWORD FOLLOW_LOGS_SYMBOL_in_flush_option9369_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGS_SYMBOL_in_flush_option9369	= { FOLLOW_LOGS_SYMBOL_in_flush_option9369_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RELAY_SYMBOL_in_flush_option9374  */
static	ANTLR3_BITWORD FOLLOW_RELAY_SYMBOL_in_flush_option9374_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RELAY_SYMBOL_in_flush_option9374	= { FOLLOW_RELAY_SYMBOL_in_flush_option9374_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOGS_SYMBOL_in_flush_option9376  */
static	ANTLR3_BITWORD FOLLOW_LOGS_SYMBOL_in_flush_option9376_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOGS_SYMBOL_in_flush_option9376	= { FOLLOW_LOGS_SYMBOL_in_flush_option9376_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_flush_option9378  */
static	ANTLR3_BITWORD FOLLOW_channel_in_flush_option9378_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_flush_option9378	= { FOLLOW_channel_in_flush_option9378_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIVILEGES_SYMBOL_in_flush_option9384  */
static	ANTLR3_BITWORD FOLLOW_PRIVILEGES_SYMBOL_in_flush_option9384_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIVILEGES_SYMBOL_in_flush_option9384	= { FOLLOW_PRIVILEGES_SYMBOL_in_flush_option9384_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUERY_SYMBOL_in_flush_option9389  */
static	ANTLR3_BITWORD FOLLOW_QUERY_SYMBOL_in_flush_option9389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_QUERY_SYMBOL_in_flush_option9389	= { FOLLOW_QUERY_SYMBOL_in_flush_option9389_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CACHE_SYMBOL_in_flush_option9391  */
static	ANTLR3_BITWORD FOLLOW_CACHE_SYMBOL_in_flush_option9391_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CACHE_SYMBOL_in_flush_option9391	= { FOLLOW_CACHE_SYMBOL_in_flush_option9391_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STATUS_SYMBOL_in_flush_option9396  */
static	ANTLR3_BITWORD FOLLOW_STATUS_SYMBOL_in_flush_option9396_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STATUS_SYMBOL_in_flush_option9396	= { FOLLOW_STATUS_SYMBOL_in_flush_option9396_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_RESOURCES_SYMBOL_in_flush_option9401  */
static	ANTLR3_BITWORD FOLLOW_USER_RESOURCES_SYMBOL_in_flush_option9401_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_RESOURCES_SYMBOL_in_flush_option9401	= { FOLLOW_USER_RESOURCES_SYMBOL_in_flush_option9401_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTIMIZER_COSTS_SYMBOL_in_flush_option9408  */
static	ANTLR3_BITWORD FOLLOW_OPTIMIZER_COSTS_SYMBOL_in_flush_option9408_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTIMIZER_COSTS_SYMBOL_in_flush_option9408	= { FOLLOW_OPTIMIZER_COSTS_SYMBOL_in_flush_option9408_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_flush_tables9446  */
static	ANTLR3_BITWORD FOLLOW_set_in_flush_tables9446_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010010000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_flush_tables9446	= { FOLLOW_set_in_flush_tables9446_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_flush_tables9459  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_flush_tables9459_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_flush_tables9459	= { FOLLOW_WITH_SYMBOL_in_flush_tables9459_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_flush_tables9461  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_flush_tables9461_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_flush_tables9461	= { FOLLOW_READ_SYMBOL_in_flush_tables9461_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_flush_tables9463  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_flush_tables9463_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_flush_tables9463	= { FOLLOW_LOCK_SYMBOL_in_flush_tables9463_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_flush_tables9469  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_flush_tables9469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_flush_tables9469	= { FOLLOW_identifier_list_in_flush_tables9469_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_flush_tables_options_in_flush_tables9471  */
static	ANTLR3_BITWORD FOLLOW_flush_tables_options_in_flush_tables9471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_flush_tables_options_in_flush_tables9471	= { FOLLOW_flush_tables_options_in_flush_tables9471_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_flush_tables_options9487  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_flush_tables_options9487_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_flush_tables_options9487	= { FOLLOW_FOR_SYMBOL_in_flush_tables_options9487_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXPORT_SYMBOL_in_flush_tables_options9489  */
static	ANTLR3_BITWORD FOLLOW_EXPORT_SYMBOL_in_flush_tables_options9489_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EXPORT_SYMBOL_in_flush_tables_options9489	= { FOLLOW_EXPORT_SYMBOL_in_flush_tables_options9489_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_flush_tables_options9494  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_flush_tables_options9494_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_flush_tables_options9494	= { FOLLOW_WITH_SYMBOL_in_flush_tables_options9494_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_READ_SYMBOL_in_flush_tables_options9496  */
static	ANTLR3_BITWORD FOLLOW_READ_SYMBOL_in_flush_tables_options9496_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_READ_SYMBOL_in_flush_tables_options9496	= { FOLLOW_READ_SYMBOL_in_flush_tables_options9496_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCK_SYMBOL_in_flush_tables_options9498  */
static	ANTLR3_BITWORD FOLLOW_LOCK_SYMBOL_in_flush_tables_options9498_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCK_SYMBOL_in_flush_tables_options9498	= { FOLLOW_LOCK_SYMBOL_in_flush_tables_options9498_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_load_table_index_list9507  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_load_table_index_list9507_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00800000C0000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_load_table_index_list9507	= { FOLLOW_table_ref_in_load_table_index_list9507_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_load_table_index_partion_in_load_table_index_list9509  */
static	ANTLR3_BITWORD FOLLOW_load_table_index_partion_in_load_table_index_list9509_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x00800000C0000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_load_table_index_partion_in_load_table_index_list9509	= { FOLLOW_load_table_index_partion_in_load_table_index_list9509_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_load_table_index_list9524  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_load_table_index_list9524_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_load_table_index_list9524	= { FOLLOW_identifier_list_with_parentheses_in_load_table_index_list9524_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IGNORE_SYMBOL_in_load_table_index_list9529  */
static	ANTLR3_BITWORD FOLLOW_IGNORE_SYMBOL_in_load_table_index_list9529_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IGNORE_SYMBOL_in_load_table_index_list9529	= { FOLLOW_IGNORE_SYMBOL_in_load_table_index_list9529_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEAVES_SYMBOL_in_load_table_index_list9531  */
static	ANTLR3_BITWORD FOLLOW_LEAVES_SYMBOL_in_load_table_index_list9531_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LEAVES_SYMBOL_in_load_table_index_list9531	= { FOLLOW_LEAVES_SYMBOL_in_load_table_index_list9531_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_load_table_index_partion9547  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_load_table_index_partion9547_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_load_table_index_partion9547	= { FOLLOW_PARTITION_SYMBOL_in_load_table_index_partion9547_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_load_table_index_partion9549  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_load_table_index_partion9549_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_load_table_index_partion9549	= { FOLLOW_OPEN_PAR_SYMBOL_in_load_table_index_partion9549_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_load_table_index_partion9552  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_load_table_index_partion9552_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_load_table_index_partion9552	= { FOLLOW_identifier_list_in_load_table_index_partion9552_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_load_table_index_partion9556  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_load_table_index_partion9556_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_load_table_index_partion9556	= { FOLLOW_ALL_SYMBOL_in_load_table_index_partion9556_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_load_table_index_partion9559  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_load_table_index_partion9559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_load_table_index_partion9559	= { FOLLOW_CLOSE_PAR_SYMBOL_in_load_table_index_partion9559_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MASTER_SYMBOL_in_reset_option9569  */
static	ANTLR3_BITWORD FOLLOW_MASTER_SYMBOL_in_reset_option9569_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MASTER_SYMBOL_in_reset_option9569	= { FOLLOW_MASTER_SYMBOL_in_reset_option9569_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUERY_SYMBOL_in_reset_option9574  */
static	ANTLR3_BITWORD FOLLOW_QUERY_SYMBOL_in_reset_option9574_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_QUERY_SYMBOL_in_reset_option9574	= { FOLLOW_QUERY_SYMBOL_in_reset_option9574_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CACHE_SYMBOL_in_reset_option9576  */
static	ANTLR3_BITWORD FOLLOW_CACHE_SYMBOL_in_reset_option9576_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CACHE_SYMBOL_in_reset_option9576	= { FOLLOW_CACHE_SYMBOL_in_reset_option9576_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SLAVE_SYMBOL_in_reset_option9581  */
static	ANTLR3_BITWORD FOLLOW_SLAVE_SYMBOL_in_reset_option9581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_SLAVE_SYMBOL_in_reset_option9581	= { FOLLOW_SLAVE_SYMBOL_in_reset_option9581_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_reset_option9583  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_reset_option9583_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_reset_option9583	= { FOLLOW_ALL_SYMBOL_in_reset_option9583_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_channel_in_reset_option9586  */
static	ANTLR3_BITWORD FOLLOW_channel_in_reset_option9586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_channel_in_reset_option9586	= { FOLLOW_channel_in_reset_option9586_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_describe_command_in_utility_statement9598  */
static	ANTLR3_BITWORD FOLLOW_describe_command_in_utility_statement9598_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000001000008000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0010000000020000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_describe_command_in_utility_statement9598	= { FOLLOW_describe_command_in_utility_statement9598_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_utility_statement9607  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_utility_statement9607_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0030000000000000), ANTLR3_UINT64_LIT(0x0000001800100000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_utility_statement9607	= { FOLLOW_table_ref_in_utility_statement9607_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_utility_statement9610  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_utility_statement9610_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_utility_statement9610	= { FOLLOW_text_string_in_utility_statement9610_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_utility_statement9614  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_utility_statement9614_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_utility_statement9614	= { FOLLOW_identifier_in_utility_statement9614_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXTENDED_SYMBOL_in_utility_statement9646  */
static	ANTLR3_BITWORD FOLLOW_EXTENDED_SYMBOL_in_utility_statement9646_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000001000008000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXTENDED_SYMBOL_in_utility_statement9646	= { FOLLOW_EXTENDED_SYMBOL_in_utility_statement9646_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITIONS_SYMBOL_in_utility_statement9658  */
static	ANTLR3_BITWORD FOLLOW_PARTITIONS_SYMBOL_in_utility_statement9658_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000001000008000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITIONS_SYMBOL_in_utility_statement9658	= { FOLLOW_PARTITIONS_SYMBOL_in_utility_statement9658_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FORMAT_SYMBOL_in_utility_statement9670  */
static	ANTLR3_BITWORD FOLLOW_FORMAT_SYMBOL_in_utility_statement9670_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_FORMAT_SYMBOL_in_utility_statement9670	= { FOLLOW_FORMAT_SYMBOL_in_utility_statement9670_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_utility_statement9672  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_utility_statement9672_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_utility_statement9672	= { FOLLOW_EQUAL_OPERATOR_in_utility_statement9672_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_utility_statement9674  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_utility_statement9674_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000001000008000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100200), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_utility_statement9674	= { FOLLOW_text_or_identifier_in_utility_statement9674_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_explainable_statement_in_utility_statement9683  */
static	ANTLR3_BITWORD FOLLOW_explainable_statement_in_utility_statement9683_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_explainable_statement_in_utility_statement9683	= { FOLLOW_explainable_statement_in_utility_statement9683_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HELP_SYMBOL_in_utility_statement9692  */
static	ANTLR3_BITWORD FOLLOW_HELP_SYMBOL_in_utility_statement9692_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_HELP_SYMBOL_in_utility_statement9692	= { FOLLOW_HELP_SYMBOL_in_utility_statement9692_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_utility_statement9694  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_utility_statement9694_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_utility_statement9694	= { FOLLOW_text_or_identifier_in_utility_statement9694_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_use_command_in_utility_statement9699  */
static	ANTLR3_BITWORD FOLLOW_use_command_in_utility_statement9699_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_use_command_in_utility_statement9699	= { FOLLOW_use_command_in_utility_statement9699_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_explainable_statement9722  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_explainable_statement9722_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_explainable_statement9722	= { FOLLOW_select_statement_in_explainable_statement9722_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_delete_statement_in_explainable_statement9736  */
static	ANTLR3_BITWORD FOLLOW_delete_statement_in_explainable_statement9736_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_delete_statement_in_explainable_statement9736	= { FOLLOW_delete_statement_in_explainable_statement9736_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_insert_statement_in_explainable_statement9743  */
static	ANTLR3_BITWORD FOLLOW_insert_statement_in_explainable_statement9743_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_insert_statement_in_explainable_statement9743	= { FOLLOW_insert_statement_in_explainable_statement9743_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_replace_statement_in_explainable_statement9750  */
static	ANTLR3_BITWORD FOLLOW_replace_statement_in_explainable_statement9750_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_replace_statement_in_explainable_statement9750	= { FOLLOW_replace_statement_in_explainable_statement9750_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_update_statement_in_explainable_statement9757  */
static	ANTLR3_BITWORD FOLLOW_update_statement_in_explainable_statement9757_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_update_statement_in_explainable_statement9757	= { FOLLOW_update_statement_in_explainable_statement9757_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_explainable_statement9768  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_explainable_statement9768_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_explainable_statement9768	= { FOLLOW_FOR_SYMBOL_in_explainable_statement9768_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONNECTION_SYMBOL_in_explainable_statement9770  */
static	ANTLR3_BITWORD FOLLOW_CONNECTION_SYMBOL_in_explainable_statement9770_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONNECTION_SYMBOL_in_explainable_statement9770	= { FOLLOW_CONNECTION_SYMBOL_in_explainable_statement9770_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_explainable_statement9772  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_explainable_statement9772_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_explainable_statement9772	= { FOLLOW_real_ulong_number_in_explainable_statement9772_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USE_SYMBOL_in_use_command9781  */
static	ANTLR3_BITWORD FOLLOW_USE_SYMBOL_in_use_command9781_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USE_SYMBOL_in_use_command9781	= { FOLLOW_USE_SYMBOL_in_use_command9781_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_use_command9783  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_use_command9783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_use_command9783	= { FOLLOW_identifier_in_use_command9783_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_or_expression_in_expression9797  */
static	ANTLR3_BITWORD FOLLOW_logical_or_expression_in_expression9797_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_or_expression_in_expression9797	= { FOLLOW_logical_or_expression_in_expression9797_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_xor_expression_in_logical_or_expression9806  */
static	ANTLR3_BITWORD FOLLOW_logical_xor_expression_in_logical_or_expression9806_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_xor_expression_in_logical_or_expression9806	= { FOLLOW_logical_xor_expression_in_logical_or_expression9806_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_logical_or_expression9822  */
static	ANTLR3_BITWORD FOLLOW_set_in_logical_or_expression9822_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_logical_or_expression9822	= { FOLLOW_set_in_logical_or_expression9822_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_xor_expression_in_logical_or_expression9830  */
static	ANTLR3_BITWORD FOLLOW_logical_xor_expression_in_logical_or_expression9830_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_xor_expression_in_logical_or_expression9830	= { FOLLOW_logical_xor_expression_in_logical_or_expression9830_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_and_expression_in_logical_xor_expression9841  */
static	ANTLR3_BITWORD FOLLOW_logical_and_expression_in_logical_xor_expression9841_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000040000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_and_expression_in_logical_xor_expression9841	= { FOLLOW_logical_and_expression_in_logical_xor_expression9841_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_XOR_SYMBOL_in_logical_xor_expression9857  */
static	ANTLR3_BITWORD FOLLOW_XOR_SYMBOL_in_logical_xor_expression9857_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_XOR_SYMBOL_in_logical_xor_expression9857	= { FOLLOW_XOR_SYMBOL_in_logical_xor_expression9857_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_and_expression_in_logical_xor_expression9859  */
static	ANTLR3_BITWORD FOLLOW_logical_and_expression_in_logical_xor_expression9859_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000040000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_and_expression_in_logical_xor_expression9859	= { FOLLOW_logical_and_expression_in_logical_xor_expression9859_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_not_expression_in_logical_and_expression9870  */
static	ANTLR3_BITWORD FOLLOW_logical_not_expression_in_logical_and_expression9870_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_not_expression_in_logical_and_expression9870	= { FOLLOW_logical_not_expression_in_logical_and_expression9870_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_logical_and_expression9886  */
static	ANTLR3_BITWORD FOLLOW_set_in_logical_and_expression9886_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_logical_and_expression9886	= { FOLLOW_set_in_logical_and_expression9886_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_not_expression_in_logical_and_expression9894  */
static	ANTLR3_BITWORD FOLLOW_logical_not_expression_in_logical_and_expression9894_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_not_expression_in_logical_and_expression9894	= { FOLLOW_logical_not_expression_in_logical_and_expression9894_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_logical_not_expression9905  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_logical_not_expression9905_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_logical_not_expression9905	= { FOLLOW_NOT_SYMBOL_in_logical_not_expression9905_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_logical_not_expression_in_logical_not_expression9907  */
static	ANTLR3_BITWORD FOLLOW_logical_not_expression_in_logical_not_expression9907_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_logical_not_expression_in_logical_not_expression9907	= { FOLLOW_logical_not_expression_in_logical_not_expression9907_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_boolean_primary_expression_in_logical_not_expression9912  */
static	ANTLR3_BITWORD FOLLOW_boolean_primary_expression_in_logical_not_expression9912_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_boolean_primary_expression_in_logical_not_expression9912	= { FOLLOW_boolean_primary_expression_in_logical_not_expression9912_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_predicate_in_boolean_primary_expression9921  */
static	ANTLR3_BITWORD FOLLOW_predicate_in_boolean_primary_expression9921_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_predicate_in_boolean_primary_expression9921	= { FOLLOW_predicate_in_boolean_primary_expression9921_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_comparison_operator_in_boolean_primary_expression9942  */
static	ANTLR3_BITWORD FOLLOW_comparison_operator_in_boolean_primary_expression9942_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_comparison_operator_in_boolean_primary_expression9942	= { FOLLOW_comparison_operator_in_boolean_primary_expression9942_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_boolean_primary_expression9955  */
static	ANTLR3_BITWORD FOLLOW_set_in_boolean_primary_expression9955_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_boolean_primary_expression9955	= { FOLLOW_set_in_boolean_primary_expression9955_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subquery_in_boolean_primary_expression9963  */
static	ANTLR3_BITWORD FOLLOW_subquery_in_boolean_primary_expression9963_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_subquery_in_boolean_primary_expression9963	= { FOLLOW_subquery_in_boolean_primary_expression9963_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_predicate_in_boolean_primary_expression9971  */
static	ANTLR3_BITWORD FOLLOW_predicate_in_boolean_primary_expression9971_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_predicate_in_boolean_primary_expression9971	= { FOLLOW_predicate_in_boolean_primary_expression9971_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IS_SYMBOL_in_boolean_primary_expression10000  */
static	ANTLR3_BITWORD FOLLOW_IS_SYMBOL_in_boolean_primary_expression10000_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000C800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_IS_SYMBOL_in_boolean_primary_expression10000	= { FOLLOW_IS_SYMBOL_in_boolean_primary_expression10000_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_boolean_primary_expression10002  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_boolean_primary_expression10002_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000C000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_boolean_primary_expression10002	= { FOLLOW_NOT_SYMBOL_in_boolean_primary_expression10002_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_null_literal_in_boolean_primary_expression10007  */
static	ANTLR3_BITWORD FOLLOW_null_literal_in_boolean_primary_expression10007_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_null_literal_in_boolean_primary_expression10007	= { FOLLOW_null_literal_in_boolean_primary_expression10007_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FALSE_SYMBOL_in_boolean_primary_expression10011  */
static	ANTLR3_BITWORD FOLLOW_FALSE_SYMBOL_in_boolean_primary_expression10011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_FALSE_SYMBOL_in_boolean_primary_expression10011	= { FOLLOW_FALSE_SYMBOL_in_boolean_primary_expression10011_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRUE_SYMBOL_in_boolean_primary_expression10015  */
static	ANTLR3_BITWORD FOLLOW_TRUE_SYMBOL_in_boolean_primary_expression10015_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_TRUE_SYMBOL_in_boolean_primary_expression10015	= { FOLLOW_TRUE_SYMBOL_in_boolean_primary_expression10015_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNKNOWN_SYMBOL_in_boolean_primary_expression10019  */
static	ANTLR3_BITWORD FOLLOW_UNKNOWN_SYMBOL_in_boolean_primary_expression10019_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8001800300800000), ANTLR3_UINT64_LIT(0x0000000000000003) };
static  ANTLR3_BITSET_LIST FOLLOW_UNKNOWN_SYMBOL_in_boolean_primary_expression10019	= { FOLLOW_UNKNOWN_SYMBOL_in_boolean_primary_expression10019_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_predicate10036  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_predicate10036_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004), ANTLR3_UINT64_LIT(0x1000400000000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_predicate10036	= { FOLLOW_bitwise_or_expression_in_predicate10036_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_predicate10057  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_predicate10057_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004), ANTLR3_UINT64_LIT(0x1000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_predicate10057	= { FOLLOW_NOT_SYMBOL_in_predicate10057_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BETWEEN_SYMBOL_in_predicate10071  */
static	ANTLR3_BITWORD FOLLOW_BETWEEN_SYMBOL_in_predicate10071_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BETWEEN_SYMBOL_in_predicate10071	= { FOLLOW_BETWEEN_SYMBOL_in_predicate10071_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_predicate10073  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_predicate10073_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_predicate10073	= { FOLLOW_bitwise_or_expression_in_predicate10073_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AND_SYMBOL_in_predicate10075  */
static	ANTLR3_BITWORD FOLLOW_AND_SYMBOL_in_predicate10075_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_AND_SYMBOL_in_predicate10075	= { FOLLOW_AND_SYMBOL_in_predicate10075_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_predicate_in_predicate10077  */
static	ANTLR3_BITWORD FOLLOW_predicate_in_predicate10077_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_predicate_in_predicate10077	= { FOLLOW_predicate_in_predicate10077_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIKE_SYMBOL_in_predicate10086  */
static	ANTLR3_BITWORD FOLLOW_LIKE_SYMBOL_in_predicate10086_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_LIKE_SYMBOL_in_predicate10086	= { FOLLOW_LIKE_SYMBOL_in_predicate10086_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unary_expression_in_predicate10088  */
static	ANTLR3_BITWORD FOLLOW_unary_expression_in_predicate10088_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_unary_expression_in_predicate10088	= { FOLLOW_unary_expression_in_predicate10088_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ESCAPE_SYMBOL_in_predicate10104  */
static	ANTLR3_BITWORD FOLLOW_ESCAPE_SYMBOL_in_predicate10104_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000010204000C4), ANTLR3_UINT64_LIT(0x030060D04880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1C37000122E00600), ANTLR3_UINT64_LIT(0x0400011810120800), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ESCAPE_SYMBOL_in_predicate10104	= { FOLLOW_ESCAPE_SYMBOL_in_predicate10104_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_primary_in_predicate10106  */
static	ANTLR3_BITWORD FOLLOW_primary_in_predicate10106_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_primary_in_predicate10106	= { FOLLOW_primary_in_predicate10106_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REGEXP_SYMBOL_in_predicate10117  */
static	ANTLR3_BITWORD FOLLOW_REGEXP_SYMBOL_in_predicate10117_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_REGEXP_SYMBOL_in_predicate10117	= { FOLLOW_REGEXP_SYMBOL_in_predicate10117_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_predicate10119  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_predicate10119_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_predicate10119	= { FOLLOW_bitwise_or_expression_in_predicate10119_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_predicate10128  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_predicate10128_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_predicate10128	= { FOLLOW_IN_SYMBOL_in_predicate10128_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_predicate_in_in_predicate10130  */
static	ANTLR3_BITWORD FOLLOW_predicate_in_in_predicate10130_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_predicate_in_in_predicate10130	= { FOLLOW_predicate_in_in_predicate10130_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SOUNDS_SYMBOL_in_predicate10149  */
static	ANTLR3_BITWORD FOLLOW_SOUNDS_SYMBOL_in_predicate10149_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x1000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SOUNDS_SYMBOL_in_predicate10149	= { FOLLOW_SOUNDS_SYMBOL_in_predicate10149_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIKE_SYMBOL_in_predicate10151  */
static	ANTLR3_BITWORD FOLLOW_LIKE_SYMBOL_in_predicate10151_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_LIKE_SYMBOL_in_predicate10151	= { FOLLOW_LIKE_SYMBOL_in_predicate10151_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_predicate10153  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_predicate10153_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_predicate10153	= { FOLLOW_bitwise_or_expression_in_predicate10153_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subquery_in_predicate_in10172  */
static	ANTLR3_BITWORD FOLLOW_subquery_in_predicate_in10172_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_subquery_in_predicate_in10172	= { FOLLOW_subquery_in_predicate_in10172_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_predicate_in10177  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_predicate_in10177_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_predicate_in10177	= { FOLLOW_expression_list_with_parentheses_in_predicate_in10177_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_and_expression_in_bitwise_or_expression10186  */
static	ANTLR3_BITWORD FOLLOW_bitwise_and_expression_in_bitwise_or_expression10186_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_and_expression_in_bitwise_or_expression10186	= { FOLLOW_bitwise_and_expression_in_bitwise_or_expression10186_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BITWISE_OR_OPERATOR_in_bitwise_or_expression10202  */
static	ANTLR3_BITWORD FOLLOW_BITWISE_OR_OPERATOR_in_bitwise_or_expression10202_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BITWISE_OR_OPERATOR_in_bitwise_or_expression10202	= { FOLLOW_BITWISE_OR_OPERATOR_in_bitwise_or_expression10202_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_and_expression_in_bitwise_or_expression10204  */
static	ANTLR3_BITWORD FOLLOW_bitwise_and_expression_in_bitwise_or_expression10204_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_and_expression_in_bitwise_or_expression10204	= { FOLLOW_bitwise_and_expression_in_bitwise_or_expression10204_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_shift_expression_in_bitwise_and_expression10215  */
static	ANTLR3_BITWORD FOLLOW_shift_expression_in_bitwise_and_expression10215_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_shift_expression_in_bitwise_and_expression10215	= { FOLLOW_shift_expression_in_bitwise_and_expression10215_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BITWISE_AND_OPERATOR_in_bitwise_and_expression10231  */
static	ANTLR3_BITWORD FOLLOW_BITWISE_AND_OPERATOR_in_bitwise_and_expression10231_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BITWISE_AND_OPERATOR_in_bitwise_and_expression10231	= { FOLLOW_BITWISE_AND_OPERATOR_in_bitwise_and_expression10231_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_shift_expression_in_bitwise_and_expression10233  */
static	ANTLR3_BITWORD FOLLOW_shift_expression_in_bitwise_and_expression10233_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_shift_expression_in_bitwise_and_expression10233	= { FOLLOW_shift_expression_in_bitwise_and_expression10233_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_additive_expression_in_shift_expression10244  */
static	ANTLR3_BITWORD FOLLOW_additive_expression_in_shift_expression10244_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000030000) };
static  ANTLR3_BITSET_LIST FOLLOW_additive_expression_in_shift_expression10244	= { FOLLOW_additive_expression_in_shift_expression10244_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_shift_expression10260  */
static	ANTLR3_BITWORD FOLLOW_set_in_shift_expression10260_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_shift_expression10260	= { FOLLOW_set_in_shift_expression10260_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_additive_expression_in_shift_expression10268  */
static	ANTLR3_BITWORD FOLLOW_additive_expression_in_shift_expression10268_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000030000) };
static  ANTLR3_BITSET_LIST FOLLOW_additive_expression_in_shift_expression10268	= { FOLLOW_additive_expression_in_shift_expression10268_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_multiplicative_expression_in_additive_expression10279  */
static	ANTLR3_BITWORD FOLLOW_multiplicative_expression_in_additive_expression10279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_multiplicative_expression_in_additive_expression10279	= { FOLLOW_multiplicative_expression_in_additive_expression10279_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_additive_expression10295  */
static	ANTLR3_BITWORD FOLLOW_set_in_additive_expression10295_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_additive_expression10295	= { FOLLOW_set_in_additive_expression10295_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_multiplicative_expression_in_additive_expression10303  */
static	ANTLR3_BITWORD FOLLOW_multiplicative_expression_in_additive_expression10303_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_multiplicative_expression_in_additive_expression10303	= { FOLLOW_multiplicative_expression_in_additive_expression10303_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_xor_expression_in_multiplicative_expression10314  */
static	ANTLR3_BITWORD FOLLOW_bitwise_xor_expression_in_multiplicative_expression10314_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x500000000000C000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_xor_expression_in_multiplicative_expression10314	= { FOLLOW_bitwise_xor_expression_in_multiplicative_expression10314_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_multiplication_operator_in_multiplicative_expression10330  */
static	ANTLR3_BITWORD FOLLOW_multiplication_operator_in_multiplicative_expression10330_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_multiplication_operator_in_multiplicative_expression10330	= { FOLLOW_multiplication_operator_in_multiplicative_expression10330_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_xor_expression_in_multiplicative_expression10332  */
static	ANTLR3_BITWORD FOLLOW_bitwise_xor_expression_in_multiplicative_expression10332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x500000000000C000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_xor_expression_in_multiplicative_expression10332	= { FOLLOW_bitwise_xor_expression_in_multiplicative_expression10332_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_concat_expression_in_bitwise_xor_expression10343  */
static	ANTLR3_BITWORD FOLLOW_concat_expression_in_bitwise_xor_expression10343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_concat_expression_in_bitwise_xor_expression10343	= { FOLLOW_concat_expression_in_bitwise_xor_expression10343_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BITWISE_XOR_OPERATOR_in_bitwise_xor_expression10359  */
static	ANTLR3_BITWORD FOLLOW_BITWISE_XOR_OPERATOR_in_bitwise_xor_expression10359_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BITWISE_XOR_OPERATOR_in_bitwise_xor_expression10359	= { FOLLOW_BITWISE_XOR_OPERATOR_in_bitwise_xor_expression10359_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_concat_expression_in_bitwise_xor_expression10361  */
static	ANTLR3_BITWORD FOLLOW_concat_expression_in_bitwise_xor_expression10361_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_concat_expression_in_bitwise_xor_expression10361	= { FOLLOW_concat_expression_in_bitwise_xor_expression10361_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unary_expression_in_concat_expression10373  */
static	ANTLR3_BITWORD FOLLOW_unary_expression_in_concat_expression10373_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_unary_expression_in_concat_expression10373	= { FOLLOW_unary_expression_in_concat_expression10373_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONCAT_PIPES_SYMBOL_in_concat_expression10389  */
static	ANTLR3_BITWORD FOLLOW_CONCAT_PIPES_SYMBOL_in_concat_expression10389_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_CONCAT_PIPES_SYMBOL_in_concat_expression10389	= { FOLLOW_CONCAT_PIPES_SYMBOL_in_concat_expression10389_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unary_expression_in_concat_expression10391  */
static	ANTLR3_BITWORD FOLLOW_unary_expression_in_concat_expression10391_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_unary_expression_in_concat_expression10391	= { FOLLOW_unary_expression_in_concat_expression10391_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_unary_expression10402  */
static	ANTLR3_BITWORD FOLLOW_set_in_unary_expression10402_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_unary_expression10402	= { FOLLOW_set_in_unary_expression10402_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unary_expression_in_unary_expression10414  */
static	ANTLR3_BITWORD FOLLOW_unary_expression_in_unary_expression10414_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_unary_expression_in_unary_expression10414	= { FOLLOW_unary_expression_in_unary_expression10414_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_not_expression_in_unary_expression10419  */
static	ANTLR3_BITWORD FOLLOW_not_expression_in_unary_expression10419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_not_expression_in_unary_expression10419	= { FOLLOW_not_expression_in_unary_expression10419_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_not2_rule_in_not_expression10428  */
static	ANTLR3_BITWORD FOLLOW_not2_rule_in_not_expression10428_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D04880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1C37000122E00600), ANTLR3_UINT64_LIT(0x0400011810120800), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_not2_rule_in_not_expression10428	= { FOLLOW_not2_rule_in_not_expression10428_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_expression_in_not_expression10431  */
static	ANTLR3_BITWORD FOLLOW_interval_expression_in_not_expression10431_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_expression_in_not_expression10431	= { FOLLOW_interval_expression_in_not_expression10431_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTERVAL_SYMBOL_in_interval_expression10440  */
static	ANTLR3_BITWORD FOLLOW_INTERVAL_SYMBOL_in_interval_expression10440_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_INTERVAL_SYMBOL_in_interval_expression10440	= { FOLLOW_INTERVAL_SYMBOL_in_interval_expression10440_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_function_in_interval_expression10471  */
static	ANTLR3_BITWORD FOLLOW_interval_function_in_interval_expression10471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_function_in_interval_expression10471	= { FOLLOW_interval_function_in_interval_expression10471_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_time_span_in_interval_expression10478  */
static	ANTLR3_BITWORD FOLLOW_interval_time_span_in_interval_expression10478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_time_span_in_interval_expression10478	= { FOLLOW_interval_time_span_in_interval_expression10478_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_primary_in_interval_expression10487  */
static	ANTLR3_BITWORD FOLLOW_primary_in_interval_expression10487_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_primary_in_interval_expression10487	= { FOLLOW_primary_in_interval_expression10487_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_interval_function10496  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_interval_function10496_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_interval_function10496	= { FOLLOW_OPEN_PAR_SYMBOL_in_interval_function10496_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_interval_function10498  */
static	ANTLR3_BITWORD FOLLOW_expression_in_interval_function10498_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_interval_function10498	= { FOLLOW_expression_in_interval_function10498_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_interval_function10501  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_interval_function10501_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_interval_function10501	= { FOLLOW_COMMA_SYMBOL_in_interval_function10501_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_interval_function10503  */
static	ANTLR3_BITWORD FOLLOW_expression_in_interval_function10503_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_interval_function10503	= { FOLLOW_expression_in_interval_function10503_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_interval_function10507  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_interval_function10507_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_interval_function10507	= { FOLLOW_CLOSE_PAR_SYMBOL_in_interval_function10507_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_in_interval_time_span10516  */
static	ANTLR3_BITWORD FOLLOW_interval_in_interval_time_span10516_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_in_interval_time_span10516	= { FOLLOW_interval_in_interval_time_span10516_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_literal_in_primary10544  */
static	ANTLR3_BITWORD FOLLOW_literal_in_primary10544_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_literal_in_primary10544	= { FOLLOW_literal_in_primary10544_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_call_in_primary10550  */
static	ANTLR3_BITWORD FOLLOW_function_call_in_primary10550_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_function_call_in_primary10550	= { FOLLOW_function_call_in_primary10550_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_runtime_function_call_in_primary10556  */
static	ANTLR3_BITWORD FOLLOW_runtime_function_call_in_primary10556_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_runtime_function_call_in_primary10556	= { FOLLOW_runtime_function_call_in_primary10556_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_primary10563  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_primary10563_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_primary10563	= { FOLLOW_column_ref_in_primary10563_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_jsonOperator_in_primary10565  */
static	ANTLR3_BITWORD FOLLOW_jsonOperator_in_primary10565_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_jsonOperator_in_primary10565	= { FOLLOW_jsonOperator_in_primary10565_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARAM_MARKER_in_primary10572  */
static	ANTLR3_BITWORD FOLLOW_PARAM_MARKER_in_primary10572_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARAM_MARKER_in_primary10572	= { FOLLOW_PARAM_MARKER_in_primary10572_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_in_primary10578  */
static	ANTLR3_BITWORD FOLLOW_variable_in_primary10578_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_in_primary10578	= { FOLLOW_variable_in_primary10578_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXISTS_SYMBOL_in_primary10584  */
static	ANTLR3_BITWORD FOLLOW_EXISTS_SYMBOL_in_primary10584_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXISTS_SYMBOL_in_primary10584	= { FOLLOW_EXISTS_SYMBOL_in_primary10584_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subquery_in_primary10586  */
static	ANTLR3_BITWORD FOLLOW_subquery_in_primary10586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_subquery_in_primary10586	= { FOLLOW_subquery_in_primary10586_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_with_nested_parentheses_in_primary10592  */
static	ANTLR3_BITWORD FOLLOW_expression_with_nested_parentheses_in_primary10592_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_with_nested_parentheses_in_primary10592	= { FOLLOW_expression_with_nested_parentheses_in_primary10592_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROW_SYMBOL_in_primary10598  */
static	ANTLR3_BITWORD FOLLOW_ROW_SYMBOL_in_primary10598_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROW_SYMBOL_in_primary10598	= { FOLLOW_ROW_SYMBOL_in_primary10598_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_primary10600  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_primary10600_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_primary10600	= { FOLLOW_OPEN_PAR_SYMBOL_in_primary10600_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_primary10602  */
static	ANTLR3_BITWORD FOLLOW_expression_in_primary10602_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_primary10602	= { FOLLOW_expression_in_primary10602_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_primary10605  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_primary10605_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_primary10605	= { FOLLOW_COMMA_SYMBOL_in_primary10605_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_primary10607  */
static	ANTLR3_BITWORD FOLLOW_expression_in_primary10607_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_primary10607	= { FOLLOW_expression_in_primary10607_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_primary10611  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_primary10611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_primary10611	= { FOLLOW_CLOSE_PAR_SYMBOL_in_primary10611_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_CURLY_SYMBOL_in_primary10617  */
static	ANTLR3_BITWORD FOLLOW_OPEN_CURLY_SYMBOL_in_primary10617_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_CURLY_SYMBOL_in_primary10617	= { FOLLOW_OPEN_CURLY_SYMBOL_in_primary10617_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_primary10619  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_primary10619_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_primary10619	= { FOLLOW_identifier_in_primary10619_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_primary10621  */
static	ANTLR3_BITWORD FOLLOW_expression_in_primary10621_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_primary10621	= { FOLLOW_expression_in_primary10621_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_CURLY_SYMBOL_in_primary10623  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_CURLY_SYMBOL_in_primary10623_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_CURLY_SYMBOL_in_primary10623	= { FOLLOW_CLOSE_CURLY_SYMBOL_in_primary10623_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_match_expression_in_primary10629  */
static	ANTLR3_BITWORD FOLLOW_match_expression_in_primary10629_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_match_expression_in_primary10629	= { FOLLOW_match_expression_in_primary10629_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_case_expression_in_primary10635  */
static	ANTLR3_BITWORD FOLLOW_case_expression_in_primary10635_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_case_expression_in_primary10635	= { FOLLOW_case_expression_in_primary10635_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cast_expression_in_primary10641  */
static	ANTLR3_BITWORD FOLLOW_cast_expression_in_primary10641_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_cast_expression_in_primary10641	= { FOLLOW_cast_expression_in_primary10641_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_primary10663  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_primary10663_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_primary10663	= { FOLLOW_COLLATE_SYMBOL_in_primary10663_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_in_primary10665  */
static	ANTLR3_BITWORD FOLLOW_collation_name_in_primary10665_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_in_primary10665	= { FOLLOW_collation_name_in_primary10665_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JSON_SEPARATOR_SYMBOL_in_jsonOperator10679  */
static	ANTLR3_BITWORD FOLLOW_JSON_SEPARATOR_SYMBOL_in_jsonOperator10679_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_JSON_SEPARATOR_SYMBOL_in_jsonOperator10679	= { FOLLOW_JSON_SEPARATOR_SYMBOL_in_jsonOperator10679_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_jsonOperator10681  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_jsonOperator10681_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_jsonOperator10681	= { FOLLOW_text_string_in_jsonOperator10681_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JSON_UNQUOTED_SEPARATOR_SYMBOL_in_jsonOperator10689  */
static	ANTLR3_BITWORD FOLLOW_JSON_UNQUOTED_SEPARATOR_SYMBOL_in_jsonOperator10689_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_JSON_UNQUOTED_SEPARATOR_SYMBOL_in_jsonOperator10689	= { FOLLOW_JSON_UNQUOTED_SEPARATOR_SYMBOL_in_jsonOperator10689_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_jsonOperator10691  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_jsonOperator10691_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_jsonOperator10691	= { FOLLOW_text_string_in_jsonOperator10691_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subquery_in_expression_with_nested_parentheses10704  */
static	ANTLR3_BITWORD FOLLOW_subquery_in_expression_with_nested_parentheses10704_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_subquery_in_expression_with_nested_parentheses10704	= { FOLLOW_subquery_in_expression_with_nested_parentheses10704_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_expression_with_nested_parentheses10709  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_expression_with_nested_parentheses10709_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_expression_with_nested_parentheses10709	= { FOLLOW_expression_list_with_parentheses_in_expression_with_nested_parentheses10709_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_runtime_function_call_expression_in_runtime_function_call10787  */
static	ANTLR3_BITWORD FOLLOW_runtime_function_call_expression_in_runtime_function_call10787_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_runtime_function_call_expression_in_runtime_function_call10787	= { FOLLOW_runtime_function_call_expression_in_runtime_function_call10787_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression10798  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression10798_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression10798	= { FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression10798_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10800  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10800_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10800	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10800_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_runtime_function_call_expression10802  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_runtime_function_call_expression10802_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_runtime_function_call_expression10802	= { FOLLOW_expression_list_in_runtime_function_call_expression10802_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USING_SYMBOL_in_runtime_function_call_expression10805  */
static	ANTLR3_BITWORD FOLLOW_USING_SYMBOL_in_runtime_function_call_expression10805_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USING_SYMBOL_in_runtime_function_call_expression10805	= { FOLLOW_USING_SYMBOL_in_runtime_function_call_expression10805_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_runtime_function_call_expression10807  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_runtime_function_call_expression10807_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_runtime_function_call_expression10807	= { FOLLOW_charset_name_in_runtime_function_call_expression10807_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10811  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10811_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10811	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10811_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURRENT_USER_SYMBOL_in_runtime_function_call_expression10816  */
static	ANTLR3_BITWORD FOLLOW_CURRENT_USER_SYMBOL_in_runtime_function_call_expression10816_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURRENT_USER_SYMBOL_in_runtime_function_call_expression10816	= { FOLLOW_CURRENT_USER_SYMBOL_in_runtime_function_call_expression10816_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression10818  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression10818_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression10818	= { FOLLOW_parentheses_in_runtime_function_call_expression10818_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATE_SYMBOL_in_runtime_function_call_expression10824  */
static	ANTLR3_BITWORD FOLLOW_DATE_SYMBOL_in_runtime_function_call_expression10824_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATE_SYMBOL_in_runtime_function_call_expression10824	= { FOLLOW_DATE_SYMBOL_in_runtime_function_call_expression10824_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10826  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10826_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10826	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10826_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10828  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10828_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10828	= { FOLLOW_expression_in_runtime_function_call_expression10828_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10830  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10830_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10830	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10830_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_SYMBOL_in_runtime_function_call_expression10835  */
static	ANTLR3_BITWORD FOLLOW_DAY_SYMBOL_in_runtime_function_call_expression10835_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_SYMBOL_in_runtime_function_call_expression10835	= { FOLLOW_DAY_SYMBOL_in_runtime_function_call_expression10835_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10837  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10837_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10837	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10837_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10839  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10839_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10839	= { FOLLOW_expression_in_runtime_function_call_expression10839_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10841  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10841_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10841	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10841_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOUR_SYMBOL_in_runtime_function_call_expression10846  */
static	ANTLR3_BITWORD FOLLOW_HOUR_SYMBOL_in_runtime_function_call_expression10846_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_HOUR_SYMBOL_in_runtime_function_call_expression10846	= { FOLLOW_HOUR_SYMBOL_in_runtime_function_call_expression10846_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10848  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10848_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10848	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10848_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10850  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10850_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10850	= { FOLLOW_expression_in_runtime_function_call_expression10850_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10852  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10852_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10852	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10852_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INSERT_SYMBOL_in_runtime_function_call_expression10857  */
static	ANTLR3_BITWORD FOLLOW_INSERT_SYMBOL_in_runtime_function_call_expression10857_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INSERT_SYMBOL_in_runtime_function_call_expression10857	= { FOLLOW_INSERT_SYMBOL_in_runtime_function_call_expression10857_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10859  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10859_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10859	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10859_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10861  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10861	= { FOLLOW_expression_in_runtime_function_call_expression10861_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10863  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10863_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10863	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10863_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10865  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10865_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10865	= { FOLLOW_expression_in_runtime_function_call_expression10865_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10867  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10867_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10867	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10867_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10869  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10869_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10869	= { FOLLOW_expression_in_runtime_function_call_expression10869_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10871  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10871_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10871	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10871_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10873  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10873	= { FOLLOW_expression_in_runtime_function_call_expression10873_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10875  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10875_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10875	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10875_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEFT_SYMBOL_in_runtime_function_call_expression10882  */
static	ANTLR3_BITWORD FOLLOW_LEFT_SYMBOL_in_runtime_function_call_expression10882_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LEFT_SYMBOL_in_runtime_function_call_expression10882	= { FOLLOW_LEFT_SYMBOL_in_runtime_function_call_expression10882_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10884  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10884_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10884	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10884_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10886  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10886_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10886	= { FOLLOW_expression_in_runtime_function_call_expression10886_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10888  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10888_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10888	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10888_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10890  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10890_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10890	= { FOLLOW_expression_in_runtime_function_call_expression10890_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10892  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10892	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10892_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MINUTE_SYMBOL_in_runtime_function_call_expression10897  */
static	ANTLR3_BITWORD FOLLOW_MINUTE_SYMBOL_in_runtime_function_call_expression10897_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MINUTE_SYMBOL_in_runtime_function_call_expression10897	= { FOLLOW_MINUTE_SYMBOL_in_runtime_function_call_expression10897_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10899  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10899_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10899	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10899_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10901  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10901_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10901	= { FOLLOW_expression_in_runtime_function_call_expression10901_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10903  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10903	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10903_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MONTH_SYMBOL_in_runtime_function_call_expression10908  */
static	ANTLR3_BITWORD FOLLOW_MONTH_SYMBOL_in_runtime_function_call_expression10908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MONTH_SYMBOL_in_runtime_function_call_expression10908	= { FOLLOW_MONTH_SYMBOL_in_runtime_function_call_expression10908_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10910  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10910_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10910	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10910_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10912  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10912_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10912	= { FOLLOW_expression_in_runtime_function_call_expression10912_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10914  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10914_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10914	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10914_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RIGHT_SYMBOL_in_runtime_function_call_expression10919  */
static	ANTLR3_BITWORD FOLLOW_RIGHT_SYMBOL_in_runtime_function_call_expression10919_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RIGHT_SYMBOL_in_runtime_function_call_expression10919	= { FOLLOW_RIGHT_SYMBOL_in_runtime_function_call_expression10919_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10921  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10921_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10921	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10921_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10923  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10923_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10923	= { FOLLOW_expression_in_runtime_function_call_expression10923_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10925  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10925_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10925	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10925_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10927  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10927_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10927	= { FOLLOW_expression_in_runtime_function_call_expression10927_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10929  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10929_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10929	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10929_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SECOND_SYMBOL_in_runtime_function_call_expression10934  */
static	ANTLR3_BITWORD FOLLOW_SECOND_SYMBOL_in_runtime_function_call_expression10934_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SECOND_SYMBOL_in_runtime_function_call_expression10934	= { FOLLOW_SECOND_SYMBOL_in_runtime_function_call_expression10934_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10936  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10936_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10936	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10936_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10938  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10938_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10938	= { FOLLOW_expression_in_runtime_function_call_expression10938_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10940  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10940	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10940_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIME_SYMBOL_in_runtime_function_call_expression10945  */
static	ANTLR3_BITWORD FOLLOW_TIME_SYMBOL_in_runtime_function_call_expression10945_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIME_SYMBOL_in_runtime_function_call_expression10945	= { FOLLOW_TIME_SYMBOL_in_runtime_function_call_expression10945_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10947  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10947_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10947	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10947_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10949  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10949_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10949	= { FOLLOW_expression_in_runtime_function_call_expression10949_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10951  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10951_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10951	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10951_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIMESTAMP_SYMBOL_in_runtime_function_call_expression10956  */
static	ANTLR3_BITWORD FOLLOW_TIMESTAMP_SYMBOL_in_runtime_function_call_expression10956_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIMESTAMP_SYMBOL_in_runtime_function_call_expression10956	= { FOLLOW_TIMESTAMP_SYMBOL_in_runtime_function_call_expression10956_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10958  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10958_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10958	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10958_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10960  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10960_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10960	= { FOLLOW_expression_in_runtime_function_call_expression10960_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10963  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10963_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10963	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10963_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10965  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10965_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10965	= { FOLLOW_expression_in_runtime_function_call_expression10965_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10969  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10969	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10969_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_trim_function_in_runtime_function_call_expression10974  */
static	ANTLR3_BITWORD FOLLOW_trim_function_in_runtime_function_call_expression10974_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_trim_function_in_runtime_function_call_expression10974	= { FOLLOW_trim_function_in_runtime_function_call_expression10974_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USER_SYMBOL_in_runtime_function_call_expression10979  */
static	ANTLR3_BITWORD FOLLOW_USER_SYMBOL_in_runtime_function_call_expression10979_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_USER_SYMBOL_in_runtime_function_call_expression10979	= { FOLLOW_USER_SYMBOL_in_runtime_function_call_expression10979_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression10981  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression10981_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression10981	= { FOLLOW_parentheses_in_runtime_function_call_expression10981_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALUES_SYMBOL_in_runtime_function_call_expression10986  */
static	ANTLR3_BITWORD FOLLOW_VALUES_SYMBOL_in_runtime_function_call_expression10986_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VALUES_SYMBOL_in_runtime_function_call_expression10986	= { FOLLOW_VALUES_SYMBOL_in_runtime_function_call_expression10986_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10988  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10988_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10988	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10988_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression10990  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression10990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression10990	= { FOLLOW_expression_in_runtime_function_call_expression10990_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10992  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10992_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10992	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10992_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_YEAR_SYMBOL_in_runtime_function_call_expression10997  */
static	ANTLR3_BITWORD FOLLOW_YEAR_SYMBOL_in_runtime_function_call_expression10997_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_YEAR_SYMBOL_in_runtime_function_call_expression10997	= { FOLLOW_YEAR_SYMBOL_in_runtime_function_call_expression10997_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10999  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10999_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10999	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10999_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11001  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11001_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11001	= { FOLLOW_expression_in_runtime_function_call_expression11001_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11003  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11003_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11003	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11003_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_runtime_function_call_expression11011  */
static	ANTLR3_BITWORD FOLLOW_set_in_runtime_function_call_expression11011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_runtime_function_call_expression11011	= { FOLLOW_set_in_runtime_function_call_expression11011_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11019  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11019_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11019	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11019_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11021  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11021_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11021	= { FOLLOW_expression_in_runtime_function_call_expression11021_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11023  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11023_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11023	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11023_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11025  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11025_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11025	= { FOLLOW_expression_in_runtime_function_call_expression11025_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11027  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11027_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11027	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11027_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURDATE_SYMBOL_in_runtime_function_call_expression11032  */
static	ANTLR3_BITWORD FOLLOW_CURDATE_SYMBOL_in_runtime_function_call_expression11032_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURDATE_SYMBOL_in_runtime_function_call_expression11032	= { FOLLOW_CURDATE_SYMBOL_in_runtime_function_call_expression11032_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression11034  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression11034_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression11034	= { FOLLOW_parentheses_in_runtime_function_call_expression11034_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURTIME_SYMBOL_in_runtime_function_call_expression11040  */
static	ANTLR3_BITWORD FOLLOW_CURTIME_SYMBOL_in_runtime_function_call_expression11040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURTIME_SYMBOL_in_runtime_function_call_expression11040	= { FOLLOW_CURTIME_SYMBOL_in_runtime_function_call_expression11040_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_runtime_function_call_expression11042  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_runtime_function_call_expression11042_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_runtime_function_call_expression11042	= { FOLLOW_time_function_parameters_in_runtime_function_call_expression11042_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_runtime_function_call_expression11048  */
static	ANTLR3_BITWORD FOLLOW_set_in_runtime_function_call_expression11048_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_runtime_function_call_expression11048	= { FOLLOW_set_in_runtime_function_call_expression11048_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11056  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11056_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11056	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11056_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11058  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11058_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11058	= { FOLLOW_expression_in_runtime_function_call_expression11058_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11060  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11060_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11060	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11060_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTERVAL_SYMBOL_in_runtime_function_call_expression11062  */
static	ANTLR3_BITWORD FOLLOW_INTERVAL_SYMBOL_in_runtime_function_call_expression11062_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_INTERVAL_SYMBOL_in_runtime_function_call_expression11062	= { FOLLOW_INTERVAL_SYMBOL_in_runtime_function_call_expression11062_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_in_runtime_function_call_expression11064  */
static	ANTLR3_BITWORD FOLLOW_interval_in_runtime_function_call_expression11064_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_in_runtime_function_call_expression11064	= { FOLLOW_interval_in_runtime_function_call_expression11064_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11066  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11066_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11066	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11066_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXTRACT_SYMBOL_in_runtime_function_call_expression11071  */
static	ANTLR3_BITWORD FOLLOW_EXTRACT_SYMBOL_in_runtime_function_call_expression11071_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXTRACT_SYMBOL_in_runtime_function_call_expression11071	= { FOLLOW_EXTRACT_SYMBOL_in_runtime_function_call_expression11071_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11073  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11073_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000045000000), ANTLR3_UINT64_LIT(0x0000800000000200), ANTLR3_UINT64_LIT(0x0104000000000000), ANTLR3_UINT64_LIT(0x00001E0000000000), ANTLR3_UINT64_LIT(0x000000001C010000), ANTLR3_UINT64_LIT(0x0000000000600000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11073	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11073_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_unit_in_runtime_function_call_expression11075  */
static	ANTLR3_BITWORD FOLLOW_interval_unit_in_runtime_function_call_expression11075_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_unit_in_runtime_function_call_expression11075	= { FOLLOW_interval_unit_in_runtime_function_call_expression11075_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_runtime_function_call_expression11077  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_runtime_function_call_expression11077_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_runtime_function_call_expression11077	= { FOLLOW_FROM_SYMBOL_in_runtime_function_call_expression11077_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11079  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11079_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11079	= { FOLLOW_expression_in_runtime_function_call_expression11079_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11081  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11081_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11081	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11081_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GET_FORMAT_SYMBOL_in_runtime_function_call_expression11086  */
static	ANTLR3_BITWORD FOLLOW_GET_FORMAT_SYMBOL_in_runtime_function_call_expression11086_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GET_FORMAT_SYMBOL_in_runtime_function_call_expression11086	= { FOLLOW_GET_FORMAT_SYMBOL_in_runtime_function_call_expression11086_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11088  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11088_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000090000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11088	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11088_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_date_time_type_in_runtime_function_call_expression11090  */
static	ANTLR3_BITWORD FOLLOW_date_time_type_in_runtime_function_call_expression11090_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_date_time_type_in_runtime_function_call_expression11090	= { FOLLOW_date_time_type_in_runtime_function_call_expression11090_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11093  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11093_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11093	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11093_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11095  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11095_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11095	= { FOLLOW_expression_in_runtime_function_call_expression11095_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11097  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11097_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11097	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11097_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOW_SYMBOL_in_runtime_function_call_expression11102  */
static	ANTLR3_BITWORD FOLLOW_NOW_SYMBOL_in_runtime_function_call_expression11102_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOW_SYMBOL_in_runtime_function_call_expression11102	= { FOLLOW_NOW_SYMBOL_in_runtime_function_call_expression11102_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_runtime_function_call_expression11104  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_runtime_function_call_expression11104_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_runtime_function_call_expression11104	= { FOLLOW_time_function_parameters_in_runtime_function_call_expression11104_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_POSITION_SYMBOL_in_runtime_function_call_expression11110  */
static	ANTLR3_BITWORD FOLLOW_POSITION_SYMBOL_in_runtime_function_call_expression11110_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_POSITION_SYMBOL_in_runtime_function_call_expression11110	= { FOLLOW_POSITION_SYMBOL_in_runtime_function_call_expression11110_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11112  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11112_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11112	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11112_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_runtime_function_call_expression11114  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_runtime_function_call_expression11114_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_runtime_function_call_expression11114	= { FOLLOW_bitwise_or_expression_in_runtime_function_call_expression11114_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_runtime_function_call_expression11116  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_runtime_function_call_expression11116_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_runtime_function_call_expression11116	= { FOLLOW_IN_SYMBOL_in_runtime_function_call_expression11116_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11118  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11118_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11118	= { FOLLOW_expression_in_runtime_function_call_expression11118_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11120  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11120_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11120	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11120_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_substring_function_in_runtime_function_call_expression11125  */
static	ANTLR3_BITWORD FOLLOW_substring_function_in_runtime_function_call_expression11125_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_substring_function_in_runtime_function_call_expression11125	= { FOLLOW_substring_function_in_runtime_function_call_expression11125_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SYSDATE_SYMBOL_in_runtime_function_call_expression11130  */
static	ANTLR3_BITWORD FOLLOW_SYSDATE_SYMBOL_in_runtime_function_call_expression11130_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SYSDATE_SYMBOL_in_runtime_function_call_expression11130	= { FOLLOW_SYSDATE_SYMBOL_in_runtime_function_call_expression11130_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_runtime_function_call_expression11132  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_runtime_function_call_expression11132_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_runtime_function_call_expression11132	= { FOLLOW_time_function_parameters_in_runtime_function_call_expression11132_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_runtime_function_call_expression11138  */
static	ANTLR3_BITWORD FOLLOW_set_in_runtime_function_call_expression11138_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_runtime_function_call_expression11138	= { FOLLOW_set_in_runtime_function_call_expression11138_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11146  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11146_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000045000000), ANTLR3_UINT64_LIT(0x0000800000000200), ANTLR3_UINT64_LIT(0x0104000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11146	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11146_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_timestamp_unit_in_runtime_function_call_expression11148  */
static	ANTLR3_BITWORD FOLLOW_interval_timestamp_unit_in_runtime_function_call_expression11148_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_timestamp_unit_in_runtime_function_call_expression11148	= { FOLLOW_interval_timestamp_unit_in_runtime_function_call_expression11148_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11150  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11150_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11150	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11150_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11152  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11152_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11152	= { FOLLOW_expression_in_runtime_function_call_expression11152_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11154  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11154_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11154	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11154_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11156  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11156_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11156	= { FOLLOW_expression_in_runtime_function_call_expression11156_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11158  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11158_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11158	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11158_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UTC_DATE_SYMBOL_in_runtime_function_call_expression11163  */
static	ANTLR3_BITWORD FOLLOW_UTC_DATE_SYMBOL_in_runtime_function_call_expression11163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UTC_DATE_SYMBOL_in_runtime_function_call_expression11163	= { FOLLOW_UTC_DATE_SYMBOL_in_runtime_function_call_expression11163_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression11165  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression11165_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression11165	= { FOLLOW_parentheses_in_runtime_function_call_expression11165_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UTC_TIME_SYMBOL_in_runtime_function_call_expression11171  */
static	ANTLR3_BITWORD FOLLOW_UTC_TIME_SYMBOL_in_runtime_function_call_expression11171_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UTC_TIME_SYMBOL_in_runtime_function_call_expression11171	= { FOLLOW_UTC_TIME_SYMBOL_in_runtime_function_call_expression11171_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_runtime_function_call_expression11173  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_runtime_function_call_expression11173_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_runtime_function_call_expression11173	= { FOLLOW_time_function_parameters_in_runtime_function_call_expression11173_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UTC_TIMESTAMP_SYMBOL_in_runtime_function_call_expression11179  */
static	ANTLR3_BITWORD FOLLOW_UTC_TIMESTAMP_SYMBOL_in_runtime_function_call_expression11179_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UTC_TIMESTAMP_SYMBOL_in_runtime_function_call_expression11179	= { FOLLOW_UTC_TIMESTAMP_SYMBOL_in_runtime_function_call_expression11179_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_runtime_function_call_expression11181  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_runtime_function_call_expression11181_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_runtime_function_call_expression11181	= { FOLLOW_time_function_parameters_in_runtime_function_call_expression11181_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASCII_SYMBOL_in_runtime_function_call_expression11190  */
static	ANTLR3_BITWORD FOLLOW_ASCII_SYMBOL_in_runtime_function_call_expression11190_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ASCII_SYMBOL_in_runtime_function_call_expression11190	= { FOLLOW_ASCII_SYMBOL_in_runtime_function_call_expression11190_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11192  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11192_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11192	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11192_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11194  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11194_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11194	= { FOLLOW_expression_in_runtime_function_call_expression11194_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11196  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11196_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11196	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11196_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHARSET_SYMBOL_in_runtime_function_call_expression11201  */
static	ANTLR3_BITWORD FOLLOW_CHARSET_SYMBOL_in_runtime_function_call_expression11201_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHARSET_SYMBOL_in_runtime_function_call_expression11201	= { FOLLOW_CHARSET_SYMBOL_in_runtime_function_call_expression11201_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11203  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11203_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11203	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11203_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11205  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11205_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11205	= { FOLLOW_expression_in_runtime_function_call_expression11205_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11207  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11207_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11207	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11207_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COALESCE_SYMBOL_in_runtime_function_call_expression11212  */
static	ANTLR3_BITWORD FOLLOW_COALESCE_SYMBOL_in_runtime_function_call_expression11212_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COALESCE_SYMBOL_in_runtime_function_call_expression11212	= { FOLLOW_COALESCE_SYMBOL_in_runtime_function_call_expression11212_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_runtime_function_call_expression11214  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_runtime_function_call_expression11214_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_runtime_function_call_expression11214	= { FOLLOW_expression_list_with_parentheses_in_runtime_function_call_expression11214_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATION_SYMBOL_in_runtime_function_call_expression11219  */
static	ANTLR3_BITWORD FOLLOW_COLLATION_SYMBOL_in_runtime_function_call_expression11219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATION_SYMBOL_in_runtime_function_call_expression11219	= { FOLLOW_COLLATION_SYMBOL_in_runtime_function_call_expression11219_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11221  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11221_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11221	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11221_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11223  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11223_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11223	= { FOLLOW_expression_in_runtime_function_call_expression11223_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11225  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11225_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11225	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11225_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATABASE_SYMBOL_in_runtime_function_call_expression11230  */
static	ANTLR3_BITWORD FOLLOW_DATABASE_SYMBOL_in_runtime_function_call_expression11230_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATABASE_SYMBOL_in_runtime_function_call_expression11230	= { FOLLOW_DATABASE_SYMBOL_in_runtime_function_call_expression11230_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression11232  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression11232_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression11232	= { FOLLOW_parentheses_in_runtime_function_call_expression11232_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IF_SYMBOL_in_runtime_function_call_expression11237  */
static	ANTLR3_BITWORD FOLLOW_IF_SYMBOL_in_runtime_function_call_expression11237_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IF_SYMBOL_in_runtime_function_call_expression11237	= { FOLLOW_IF_SYMBOL_in_runtime_function_call_expression11237_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11239  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11239_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11239	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11239_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11241  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11241_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11241	= { FOLLOW_expression_in_runtime_function_call_expression11241_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11243  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11243_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11243	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11243_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11245  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11245_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11245	= { FOLLOW_expression_in_runtime_function_call_expression11245_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11247  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11247_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11247	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11247_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11249  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11249_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11249	= { FOLLOW_expression_in_runtime_function_call_expression11249_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11251  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11251_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11251	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11251_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FORMAT_SYMBOL_in_runtime_function_call_expression11256  */
static	ANTLR3_BITWORD FOLLOW_FORMAT_SYMBOL_in_runtime_function_call_expression11256_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FORMAT_SYMBOL_in_runtime_function_call_expression11256	= { FOLLOW_FORMAT_SYMBOL_in_runtime_function_call_expression11256_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11258  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11258_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11258	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11258_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11260  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11260_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11260	= { FOLLOW_expression_in_runtime_function_call_expression11260_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11262  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11262_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11262	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11262_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11264  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11264_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11264	= { FOLLOW_expression_in_runtime_function_call_expression11264_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11267  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11267_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11267	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11267_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11269  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11269_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11269	= { FOLLOW_expression_in_runtime_function_call_expression11269_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11273  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11273_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11273	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11273_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MICROSECOND_SYMBOL_in_runtime_function_call_expression11278  */
static	ANTLR3_BITWORD FOLLOW_MICROSECOND_SYMBOL_in_runtime_function_call_expression11278_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MICROSECOND_SYMBOL_in_runtime_function_call_expression11278	= { FOLLOW_MICROSECOND_SYMBOL_in_runtime_function_call_expression11278_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11280  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11280_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11280	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11280_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11282  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11282_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11282	= { FOLLOW_expression_in_runtime_function_call_expression11282_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11284  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11284_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11284	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11284_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MOD_SYMBOL_in_runtime_function_call_expression11289  */
static	ANTLR3_BITWORD FOLLOW_MOD_SYMBOL_in_runtime_function_call_expression11289_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MOD_SYMBOL_in_runtime_function_call_expression11289	= { FOLLOW_MOD_SYMBOL_in_runtime_function_call_expression11289_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11291  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11291_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11291	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11291_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11293  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11293_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11293	= { FOLLOW_expression_in_runtime_function_call_expression11293_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11295  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11295_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11295	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11295_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11297  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11297_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11297	= { FOLLOW_expression_in_runtime_function_call_expression11297_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11299  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11299	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11299_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OLD_PASSWORD_SYMBOL_in_runtime_function_call_expression11306  */
static	ANTLR3_BITWORD FOLLOW_OLD_PASSWORD_SYMBOL_in_runtime_function_call_expression11306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OLD_PASSWORD_SYMBOL_in_runtime_function_call_expression11306	= { FOLLOW_OLD_PASSWORD_SYMBOL_in_runtime_function_call_expression11306_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11308  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11308	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11308_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_runtime_function_call_expression11310  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_runtime_function_call_expression11310_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_runtime_function_call_expression11310	= { FOLLOW_string_literal_in_runtime_function_call_expression11310_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11312  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11312_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11312	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11312_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_runtime_function_call_expression11317  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_runtime_function_call_expression11317_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_runtime_function_call_expression11317	= { FOLLOW_PASSWORD_SYMBOL_in_runtime_function_call_expression11317_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11319  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11319_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11319	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11319_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11321  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11321_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11321	= { FOLLOW_expression_in_runtime_function_call_expression11321_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11323  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11323_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11323	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11323_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUARTER_SYMBOL_in_runtime_function_call_expression11328  */
static	ANTLR3_BITWORD FOLLOW_QUARTER_SYMBOL_in_runtime_function_call_expression11328_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_QUARTER_SYMBOL_in_runtime_function_call_expression11328	= { FOLLOW_QUARTER_SYMBOL_in_runtime_function_call_expression11328_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11330  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11330_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11330	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11330_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11332  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11332_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11332	= { FOLLOW_expression_in_runtime_function_call_expression11332_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11334  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11334_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11334	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11334_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPEAT_SYMBOL_in_runtime_function_call_expression11339  */
static	ANTLR3_BITWORD FOLLOW_REPEAT_SYMBOL_in_runtime_function_call_expression11339_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPEAT_SYMBOL_in_runtime_function_call_expression11339	= { FOLLOW_REPEAT_SYMBOL_in_runtime_function_call_expression11339_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11341  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11341_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11341	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11341_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11343  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11343	= { FOLLOW_expression_in_runtime_function_call_expression11343_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11345  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11345_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11345	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11345_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11347  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11347_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11347	= { FOLLOW_expression_in_runtime_function_call_expression11347_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11349  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11349_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11349	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11349_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPLACE_SYMBOL_in_runtime_function_call_expression11354  */
static	ANTLR3_BITWORD FOLLOW_REPLACE_SYMBOL_in_runtime_function_call_expression11354_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPLACE_SYMBOL_in_runtime_function_call_expression11354	= { FOLLOW_REPLACE_SYMBOL_in_runtime_function_call_expression11354_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11356  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11356_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11356	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11356_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11358  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11358_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11358	= { FOLLOW_expression_in_runtime_function_call_expression11358_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11360  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11360_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11360	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11360_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11362  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11362	= { FOLLOW_expression_in_runtime_function_call_expression11362_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11364  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11364_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11364	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11364_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11366  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11366_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11366	= { FOLLOW_expression_in_runtime_function_call_expression11366_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11368  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11368_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11368	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11368_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REVERSE_SYMBOL_in_runtime_function_call_expression11373  */
static	ANTLR3_BITWORD FOLLOW_REVERSE_SYMBOL_in_runtime_function_call_expression11373_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_REVERSE_SYMBOL_in_runtime_function_call_expression11373	= { FOLLOW_REVERSE_SYMBOL_in_runtime_function_call_expression11373_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11375  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11375_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11375	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11375_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11377  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11377_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11377	= { FOLLOW_expression_in_runtime_function_call_expression11377_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11379  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11379_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11379	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11379_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROW_COUNT_SYMBOL_in_runtime_function_call_expression11384  */
static	ANTLR3_BITWORD FOLLOW_ROW_COUNT_SYMBOL_in_runtime_function_call_expression11384_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROW_COUNT_SYMBOL_in_runtime_function_call_expression11384	= { FOLLOW_ROW_COUNT_SYMBOL_in_runtime_function_call_expression11384_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_runtime_function_call_expression11386  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_runtime_function_call_expression11386_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_runtime_function_call_expression11386	= { FOLLOW_parentheses_in_runtime_function_call_expression11386_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRUNCATE_SYMBOL_in_runtime_function_call_expression11391  */
static	ANTLR3_BITWORD FOLLOW_TRUNCATE_SYMBOL_in_runtime_function_call_expression11391_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRUNCATE_SYMBOL_in_runtime_function_call_expression11391	= { FOLLOW_TRUNCATE_SYMBOL_in_runtime_function_call_expression11391_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11393  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11393_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11393	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11393_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11395  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11395_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11395	= { FOLLOW_expression_in_runtime_function_call_expression11395_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11397  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11397_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11397	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11397_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11399  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11399_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11399	= { FOLLOW_expression_in_runtime_function_call_expression11399_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11401  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11401_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11401	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11401_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WEEK_SYMBOL_in_runtime_function_call_expression11406  */
static	ANTLR3_BITWORD FOLLOW_WEEK_SYMBOL_in_runtime_function_call_expression11406_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WEEK_SYMBOL_in_runtime_function_call_expression11406	= { FOLLOW_WEEK_SYMBOL_in_runtime_function_call_expression11406_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11408  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11408_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11408	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11408_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11410  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11410_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11410	= { FOLLOW_expression_in_runtime_function_call_expression11410_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11413  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11413_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11413	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11413_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11415  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11415_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11415	= { FOLLOW_expression_in_runtime_function_call_expression11415_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11419  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11419_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11419	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11419_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WEIGHT_STRING_SYMBOL_in_runtime_function_call_expression11426  */
static	ANTLR3_BITWORD FOLLOW_WEIGHT_STRING_SYMBOL_in_runtime_function_call_expression11426_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WEIGHT_STRING_SYMBOL_in_runtime_function_call_expression11426	= { FOLLOW_WEIGHT_STRING_SYMBOL_in_runtime_function_call_expression11426_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11428  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11428_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11428	= { FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11428_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_runtime_function_call_expression11430  */
static	ANTLR3_BITWORD FOLLOW_expression_in_runtime_function_call_expression11430_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000800000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_runtime_function_call_expression11430	= { FOLLOW_expression_in_runtime_function_call_expression11430_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11440  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11440_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11440	= { FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11440_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression11442  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression11442_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression11442	= { FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression11442_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_runtime_function_call_expression11444  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_runtime_function_call_expression11444_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_runtime_function_call_expression11444	= { FOLLOW_field_length_in_runtime_function_call_expression11444_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_weight_string_levels_in_runtime_function_call_expression11448  */
static	ANTLR3_BITWORD FOLLOW_weight_string_levels_in_runtime_function_call_expression11448_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_weight_string_levels_in_runtime_function_call_expression11448	= { FOLLOW_weight_string_levels_in_runtime_function_call_expression11448_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11456  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11456_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11456	= { FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11456_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_runtime_function_call_expression11458  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_runtime_function_call_expression11458_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_runtime_function_call_expression11458	= { FOLLOW_BINARY_SYMBOL_in_runtime_function_call_expression11458_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_runtime_function_call_expression11460  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_runtime_function_call_expression11460_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_runtime_function_call_expression11460	= { FOLLOW_field_length_in_runtime_function_call_expression11460_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11467  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11467_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11467	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11467_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_runtime_function_call_expression11469  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_runtime_function_call_expression11469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_runtime_function_call_expression11469	= { FOLLOW_ulong_number_in_runtime_function_call_expression11469_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11471  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11471	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11471_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_runtime_function_call_expression11473  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_runtime_function_call_expression11473_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_runtime_function_call_expression11473	= { FOLLOW_ulong_number_in_runtime_function_call_expression11473_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11475  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11475_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11475	= { FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11475_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_runtime_function_call_expression11477  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_runtime_function_call_expression11477_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_runtime_function_call_expression11477	= { FOLLOW_ulong_number_in_runtime_function_call_expression11477_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11485  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11485_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11485	= { FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11485_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_geometry_function_in_runtime_function_call_expression11490  */
static	ANTLR3_BITWORD FOLLOW_geometry_function_in_runtime_function_call_expression11490_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_geometry_function_in_runtime_function_call_expression11490	= { FOLLOW_geometry_function_in_runtime_function_call_expression11490_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_aggregate_function_in_runtime_function_call_expression11495  */
static	ANTLR3_BITWORD FOLLOW_aggregate_function_in_runtime_function_call_expression11495_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_aggregate_function_in_runtime_function_call_expression11495	= { FOLLOW_aggregate_function_in_runtime_function_call_expression11495_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_time_function_parameters11504  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_time_function_parameters11504_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_time_function_parameters11504	= { FOLLOW_OPEN_PAR_SYMBOL_in_time_function_parameters11504_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_fractional_precision_in_time_function_parameters11506  */
static	ANTLR3_BITWORD FOLLOW_fractional_precision_in_time_function_parameters11506_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_fractional_precision_in_time_function_parameters11506	= { FOLLOW_fractional_precision_in_time_function_parameters11506_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_time_function_parameters11509  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_time_function_parameters11509_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_time_function_parameters11509	= { FOLLOW_CLOSE_PAR_SYMBOL_in_time_function_parameters11509_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_fractional_precision11522  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_fractional_precision11522_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_fractional_precision11522	= { FOLLOW_INT_NUMBER_in_fractional_precision11522_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEVEL_SYMBOL_in_weight_string_levels11531  */
static	ANTLR3_BITWORD FOLLOW_LEVEL_SYMBOL_in_weight_string_levels11531_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LEVEL_SYMBOL_in_weight_string_levels11531	= { FOLLOW_LEVEL_SYMBOL_in_weight_string_levels11531_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_weight_string_levels11538  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_weight_string_levels11538_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_weight_string_levels11538	= { FOLLOW_real_ulong_number_in_weight_string_levels11538_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MINUS_OPERATOR_in_weight_string_levels11540  */
static	ANTLR3_BITWORD FOLLOW_MINUS_OPERATOR_in_weight_string_levels11540_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MINUS_OPERATOR_in_weight_string_levels11540	= { FOLLOW_MINUS_OPERATOR_in_weight_string_levels11540_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_weight_string_levels11542  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_weight_string_levels11542_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_weight_string_levels11542	= { FOLLOW_real_ulong_number_in_weight_string_levels11542_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_weight_string_level_list_item_in_weight_string_levels11548  */
static	ANTLR3_BITWORD FOLLOW_weight_string_level_list_item_in_weight_string_levels11548_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_weight_string_level_list_item_in_weight_string_levels11548	= { FOLLOW_weight_string_level_list_item_in_weight_string_levels11548_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_weight_string_levels11551  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_weight_string_levels11551_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_weight_string_levels11551	= { FOLLOW_COMMA_SYMBOL_in_weight_string_levels11551_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_weight_string_level_list_item_in_weight_string_levels11553  */
static	ANTLR3_BITWORD FOLLOW_weight_string_level_list_item_in_weight_string_levels11553_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_weight_string_level_list_item_in_weight_string_levels11553	= { FOLLOW_weight_string_level_list_item_in_weight_string_levels11553_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_weight_string_level_list_item11567  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_weight_string_level_list_item11567_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_weight_string_level_list_item11567	= { FOLLOW_real_ulong_number_in_weight_string_level_list_item11567_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_weight_string_level_list_item11574  */
static	ANTLR3_BITWORD FOLLOW_set_in_weight_string_level_list_item11574_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000002000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_weight_string_level_list_item11574	= { FOLLOW_set_in_weight_string_level_list_item11574_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11582  */
static	ANTLR3_BITWORD FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11582_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11582	= { FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11582_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11589  */
static	ANTLR3_BITWORD FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11589_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11589	= { FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11589_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRIM_SYMBOL_in_trim_function11626  */
static	ANTLR3_BITWORD FOLLOW_TRIM_SYMBOL_in_trim_function11626_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TRIM_SYMBOL_in_trim_function11626	= { FOLLOW_TRIM_SYMBOL_in_trim_function11626_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_trim_function11628  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_trim_function11628_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013110), ANTLR3_UINT64_LIT(0x0A000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00680), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_trim_function11628	= { FOLLOW_OPEN_PAR_SYMBOL_in_trim_function11628_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11635  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11635_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11635	= { FOLLOW_expression_in_trim_function11635_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_trim_function11638  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_trim_function11638_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_trim_function11638	= { FOLLOW_FROM_SYMBOL_in_trim_function11638_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11640  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11640_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11640	= { FOLLOW_expression_in_trim_function11640_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEADING_SYMBOL_in_trim_function11648  */
static	ANTLR3_BITWORD FOLLOW_LEADING_SYMBOL_in_trim_function11648_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204200C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_LEADING_SYMBOL_in_trim_function11648	= { FOLLOW_LEADING_SYMBOL_in_trim_function11648_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11650  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11650_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11650	= { FOLLOW_expression_in_trim_function11650_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_trim_function11653  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_trim_function11653_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_trim_function11653	= { FOLLOW_FROM_SYMBOL_in_trim_function11653_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11655  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11655_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11655	= { FOLLOW_expression_in_trim_function11655_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TRAILING_SYMBOL_in_trim_function11661  */
static	ANTLR3_BITWORD FOLLOW_TRAILING_SYMBOL_in_trim_function11661_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204200C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_TRAILING_SYMBOL_in_trim_function11661	= { FOLLOW_TRAILING_SYMBOL_in_trim_function11661_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11663  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11663_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11663	= { FOLLOW_expression_in_trim_function11663_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_trim_function11666  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_trim_function11666_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_trim_function11666	= { FOLLOW_FROM_SYMBOL_in_trim_function11666_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11668  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11668_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11668	= { FOLLOW_expression_in_trim_function11668_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BOTH_SYMBOL_in_trim_function11674  */
static	ANTLR3_BITWORD FOLLOW_BOTH_SYMBOL_in_trim_function11674_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204200C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_BOTH_SYMBOL_in_trim_function11674	= { FOLLOW_BOTH_SYMBOL_in_trim_function11674_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11676  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11676	= { FOLLOW_expression_in_trim_function11676_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_trim_function11679  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_trim_function11679_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_trim_function11679	= { FOLLOW_FROM_SYMBOL_in_trim_function11679_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_trim_function11681  */
static	ANTLR3_BITWORD FOLLOW_expression_in_trim_function11681_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_trim_function11681	= { FOLLOW_expression_in_trim_function11681_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_trim_function11687  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_trim_function11687_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_trim_function11687	= { FOLLOW_CLOSE_PAR_SYMBOL_in_trim_function11687_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUBSTRING_SYMBOL_in_substring_function11696  */
static	ANTLR3_BITWORD FOLLOW_SUBSTRING_SYMBOL_in_substring_function11696_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUBSTRING_SYMBOL_in_substring_function11696	= { FOLLOW_SUBSTRING_SYMBOL_in_substring_function11696_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_substring_function11698  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_substring_function11698_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_substring_function11698	= { FOLLOW_OPEN_PAR_SYMBOL_in_substring_function11698_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_substring_function11700  */
static	ANTLR3_BITWORD FOLLOW_expression_in_substring_function11700_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_substring_function11700	= { FOLLOW_expression_in_substring_function11700_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_substring_function11707  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_substring_function11707_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_substring_function11707	= { FOLLOW_COMMA_SYMBOL_in_substring_function11707_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_substring_function11709  */
static	ANTLR3_BITWORD FOLLOW_expression_in_substring_function11709_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_substring_function11709	= { FOLLOW_expression_in_substring_function11709_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_substring_function11712  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_substring_function11712_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_substring_function11712	= { FOLLOW_COMMA_SYMBOL_in_substring_function11712_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_substring_function11714  */
static	ANTLR3_BITWORD FOLLOW_expression_in_substring_function11714_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_substring_function11714	= { FOLLOW_expression_in_substring_function11714_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FROM_SYMBOL_in_substring_function11722  */
static	ANTLR3_BITWORD FOLLOW_FROM_SYMBOL_in_substring_function11722_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FROM_SYMBOL_in_substring_function11722	= { FOLLOW_FROM_SYMBOL_in_substring_function11722_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_substring_function11724  */
static	ANTLR3_BITWORD FOLLOW_expression_in_substring_function11724_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_substring_function11724	= { FOLLOW_expression_in_substring_function11724_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_substring_function11727  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_substring_function11727_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_substring_function11727	= { FOLLOW_FOR_SYMBOL_in_substring_function11727_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_substring_function11729  */
static	ANTLR3_BITWORD FOLLOW_expression_in_substring_function11729_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_substring_function11729	= { FOLLOW_expression_in_substring_function11729_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_substring_function11737  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_substring_function11737_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_substring_function11737	= { FOLLOW_CLOSE_PAR_SYMBOL_in_substring_function11737_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GEOMETRYCOLLECTION_SYMBOL_in_geometry_function11747  */
static	ANTLR3_BITWORD FOLLOW_GEOMETRYCOLLECTION_SYMBOL_in_geometry_function11747_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GEOMETRYCOLLECTION_SYMBOL_in_geometry_function11747	= { FOLLOW_GEOMETRYCOLLECTION_SYMBOL_in_geometry_function11747_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_optional_expression_list_with_parentheses_in_geometry_function11749  */
static	ANTLR3_BITWORD FOLLOW_optional_expression_list_with_parentheses_in_geometry_function11749_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_optional_expression_list_with_parentheses_in_geometry_function11749	= { FOLLOW_optional_expression_list_with_parentheses_in_geometry_function11749_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LINESTRING_SYMBOL_in_geometry_function11754  */
static	ANTLR3_BITWORD FOLLOW_LINESTRING_SYMBOL_in_geometry_function11754_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LINESTRING_SYMBOL_in_geometry_function11754	= { FOLLOW_LINESTRING_SYMBOL_in_geometry_function11754_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_geometry_function11756  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_geometry_function11756_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_geometry_function11756	= { FOLLOW_expression_list_with_parentheses_in_geometry_function11756_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULTILINESTRING_SYMBOL_in_geometry_function11761  */
static	ANTLR3_BITWORD FOLLOW_MULTILINESTRING_SYMBOL_in_geometry_function11761_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULTILINESTRING_SYMBOL_in_geometry_function11761	= { FOLLOW_MULTILINESTRING_SYMBOL_in_geometry_function11761_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_geometry_function11763  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_geometry_function11763_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_geometry_function11763	= { FOLLOW_expression_list_with_parentheses_in_geometry_function11763_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULTIPOINT_SYMBOL_in_geometry_function11768  */
static	ANTLR3_BITWORD FOLLOW_MULTIPOINT_SYMBOL_in_geometry_function11768_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULTIPOINT_SYMBOL_in_geometry_function11768	= { FOLLOW_MULTIPOINT_SYMBOL_in_geometry_function11768_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_geometry_function11770  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_geometry_function11770_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_geometry_function11770	= { FOLLOW_expression_list_with_parentheses_in_geometry_function11770_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULTIPOLYGON_SYMBOL_in_geometry_function11775  */
static	ANTLR3_BITWORD FOLLOW_MULTIPOLYGON_SYMBOL_in_geometry_function11775_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULTIPOLYGON_SYMBOL_in_geometry_function11775	= { FOLLOW_MULTIPOLYGON_SYMBOL_in_geometry_function11775_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_geometry_function11777  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_geometry_function11777_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_geometry_function11777	= { FOLLOW_expression_list_with_parentheses_in_geometry_function11777_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_POINT_SYMBOL_in_geometry_function11782  */
static	ANTLR3_BITWORD FOLLOW_POINT_SYMBOL_in_geometry_function11782_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_POINT_SYMBOL_in_geometry_function11782	= { FOLLOW_POINT_SYMBOL_in_geometry_function11782_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11784  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11784_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11784	= { FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11784_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_geometry_function11786  */
static	ANTLR3_BITWORD FOLLOW_expression_in_geometry_function11786_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_geometry_function11786	= { FOLLOW_expression_in_geometry_function11786_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_geometry_function11788  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_geometry_function11788_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_geometry_function11788	= { FOLLOW_COMMA_SYMBOL_in_geometry_function11788_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_geometry_function11790  */
static	ANTLR3_BITWORD FOLLOW_expression_in_geometry_function11790_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_geometry_function11790	= { FOLLOW_expression_in_geometry_function11790_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11792  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11792_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11792	= { FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11792_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_POLYGON_SYMBOL_in_geometry_function11797  */
static	ANTLR3_BITWORD FOLLOW_POLYGON_SYMBOL_in_geometry_function11797_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_POLYGON_SYMBOL_in_geometry_function11797	= { FOLLOW_POLYGON_SYMBOL_in_geometry_function11797_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_with_parentheses_in_geometry_function11799  */
static	ANTLR3_BITWORD FOLLOW_expression_list_with_parentheses_in_geometry_function11799_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_with_parentheses_in_geometry_function11799	= { FOLLOW_expression_list_with_parentheses_in_geometry_function11799_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONTAINS_SYMBOL_in_geometry_function11806  */
static	ANTLR3_BITWORD FOLLOW_CONTAINS_SYMBOL_in_geometry_function11806_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONTAINS_SYMBOL_in_geometry_function11806	= { FOLLOW_CONTAINS_SYMBOL_in_geometry_function11806_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11808  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11808_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11808	= { FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11808_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_geometry_function11810  */
static	ANTLR3_BITWORD FOLLOW_expression_in_geometry_function11810_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_geometry_function11810	= { FOLLOW_expression_in_geometry_function11810_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_geometry_function11812  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_geometry_function11812_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_geometry_function11812	= { FOLLOW_COMMA_SYMBOL_in_geometry_function11812_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_geometry_function11814  */
static	ANTLR3_BITWORD FOLLOW_expression_in_geometry_function11814_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_geometry_function11814	= { FOLLOW_expression_in_geometry_function11814_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11816  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11816_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11816	= { FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11816_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AVG_SYMBOL_in_aggregate_function11825  */
static	ANTLR3_BITWORD FOLLOW_AVG_SYMBOL_in_aggregate_function11825_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AVG_SYMBOL_in_aggregate_function11825	= { FOLLOW_AVG_SYMBOL_in_aggregate_function11825_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11827  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11827_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11827	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11827_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11829  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11829_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11829	= { FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11829_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11832  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11832_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11832	= { FOLLOW_in_aggregate_expression_in_aggregate_function11832_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11834  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11834_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11834	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11834_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BIT_AND_SYMBOL_in_aggregate_function11839  */
static	ANTLR3_BITWORD FOLLOW_BIT_AND_SYMBOL_in_aggregate_function11839_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BIT_AND_SYMBOL_in_aggregate_function11839	= { FOLLOW_BIT_AND_SYMBOL_in_aggregate_function11839_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11841  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11841_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11841	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11841_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11843  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11843_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11843	= { FOLLOW_in_aggregate_expression_in_aggregate_function11843_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11845  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11845_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11845	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11845_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BIT_OR_SYMBOL_in_aggregate_function11850  */
static	ANTLR3_BITWORD FOLLOW_BIT_OR_SYMBOL_in_aggregate_function11850_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BIT_OR_SYMBOL_in_aggregate_function11850	= { FOLLOW_BIT_OR_SYMBOL_in_aggregate_function11850_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11852  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11852_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11852	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11852_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11854  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11854_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11854	= { FOLLOW_in_aggregate_expression_in_aggregate_function11854_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11856  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11856_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11856	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11856_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BIT_XOR_SYMBOL_in_aggregate_function11861  */
static	ANTLR3_BITWORD FOLLOW_BIT_XOR_SYMBOL_in_aggregate_function11861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BIT_XOR_SYMBOL_in_aggregate_function11861	= { FOLLOW_BIT_XOR_SYMBOL_in_aggregate_function11861_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11863  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11863_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11863	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11863_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11865  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11865_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11865	= { FOLLOW_in_aggregate_expression_in_aggregate_function11865_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11867  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11867_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11867	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11867_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_count_function_in_aggregate_function11872  */
static	ANTLR3_BITWORD FOLLOW_count_function_in_aggregate_function11872_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_count_function_in_aggregate_function11872	= { FOLLOW_count_function_in_aggregate_function11872_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MIN_SYMBOL_in_aggregate_function11877  */
static	ANTLR3_BITWORD FOLLOW_MIN_SYMBOL_in_aggregate_function11877_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MIN_SYMBOL_in_aggregate_function11877	= { FOLLOW_MIN_SYMBOL_in_aggregate_function11877_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11879  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11879_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11879	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11879_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11881  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11881_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11881	= { FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11881_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11884  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11884_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11884	= { FOLLOW_in_aggregate_expression_in_aggregate_function11884_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11886  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11886_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11886	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11886_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_SYMBOL_in_aggregate_function11891  */
static	ANTLR3_BITWORD FOLLOW_MAX_SYMBOL_in_aggregate_function11891_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_SYMBOL_in_aggregate_function11891	= { FOLLOW_MAX_SYMBOL_in_aggregate_function11891_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11893  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11893_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11893	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11893_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11895  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11895_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11895	= { FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11895_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11898  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11898_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11898	= { FOLLOW_in_aggregate_expression_in_aggregate_function11898_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11900  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11900_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11900	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11900_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STD_SYMBOL_in_aggregate_function11905  */
static	ANTLR3_BITWORD FOLLOW_STD_SYMBOL_in_aggregate_function11905_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STD_SYMBOL_in_aggregate_function11905	= { FOLLOW_STD_SYMBOL_in_aggregate_function11905_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11907  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11907_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11907	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11907_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11909  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11909_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11909	= { FOLLOW_in_aggregate_expression_in_aggregate_function11909_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11911  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11911_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11911	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11911_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARIANCE_SYMBOL_in_aggregate_function11916  */
static	ANTLR3_BITWORD FOLLOW_VARIANCE_SYMBOL_in_aggregate_function11916_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VARIANCE_SYMBOL_in_aggregate_function11916	= { FOLLOW_VARIANCE_SYMBOL_in_aggregate_function11916_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11918  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11918_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11918	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11918_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11920  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11920_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11920	= { FOLLOW_in_aggregate_expression_in_aggregate_function11920_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11922  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11922_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11922	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11922_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STDDEV_SAMP_SYMBOL_in_aggregate_function11927  */
static	ANTLR3_BITWORD FOLLOW_STDDEV_SAMP_SYMBOL_in_aggregate_function11927_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STDDEV_SAMP_SYMBOL_in_aggregate_function11927	= { FOLLOW_STDDEV_SAMP_SYMBOL_in_aggregate_function11927_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11929  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11929_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11929	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11929_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11931  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11931_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11931	= { FOLLOW_in_aggregate_expression_in_aggregate_function11931_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11933  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11933_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11933	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11933_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VAR_SAMP_SYMBOL_in_aggregate_function11938  */
static	ANTLR3_BITWORD FOLLOW_VAR_SAMP_SYMBOL_in_aggregate_function11938_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VAR_SAMP_SYMBOL_in_aggregate_function11938	= { FOLLOW_VAR_SAMP_SYMBOL_in_aggregate_function11938_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11940  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11940_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11940	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11940_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11942  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11942_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11942	= { FOLLOW_in_aggregate_expression_in_aggregate_function11942_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11944  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11944_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11944	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11944_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUM_SYMBOL_in_aggregate_function11949  */
static	ANTLR3_BITWORD FOLLOW_SUM_SYMBOL_in_aggregate_function11949_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUM_SYMBOL_in_aggregate_function11949	= { FOLLOW_SUM_SYMBOL_in_aggregate_function11949_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11951  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11951_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11951	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11951_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11953  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11953_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11953	= { FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11953_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_aggregate_function11956  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_aggregate_function11956_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_aggregate_function11956	= { FOLLOW_in_aggregate_expression_in_aggregate_function11956_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11958  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11958_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11958	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11958_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GROUP_CONCAT_SYMBOL_in_aggregate_function11963  */
static	ANTLR3_BITWORD FOLLOW_GROUP_CONCAT_SYMBOL_in_aggregate_function11963_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_GROUP_CONCAT_SYMBOL_in_aggregate_function11963	= { FOLLOW_GROUP_CONCAT_SYMBOL_in_aggregate_function11963_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11965  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11965_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11965	= { FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11965_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11967  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11967_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11967	= { FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11967_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_aggregate_function11970  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_aggregate_function11970_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000800000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_aggregate_function11970	= { FOLLOW_expression_list_in_aggregate_function11970_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_order_by_clause_in_aggregate_function11972  */
static	ANTLR3_BITWORD FOLLOW_order_by_clause_in_aggregate_function11972_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_order_by_clause_in_aggregate_function11972	= { FOLLOW_order_by_clause_in_aggregate_function11972_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEPARATOR_SYMBOL_in_aggregate_function11976  */
static	ANTLR3_BITWORD FOLLOW_SEPARATOR_SYMBOL_in_aggregate_function11976_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEPARATOR_SYMBOL_in_aggregate_function11976	= { FOLLOW_SEPARATOR_SYMBOL_in_aggregate_function11976_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_aggregate_function11978  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_aggregate_function11978_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_aggregate_function11978	= { FOLLOW_text_string_in_aggregate_function11978_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11982  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11982_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11982	= { FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11982_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_in_aggregate_expression11991  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_in_aggregate_expression11991_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_in_aggregate_expression11991	= { FOLLOW_ALL_SYMBOL_in_in_aggregate_expression11991_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_in_aggregate_expression11994  */
static	ANTLR3_BITWORD FOLLOW_expression_in_in_aggregate_expression11994_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_in_aggregate_expression11994	= { FOLLOW_expression_in_in_aggregate_expression11994_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COUNT_SYMBOL_in_count_function12003  */
static	ANTLR3_BITWORD FOLLOW_COUNT_SYMBOL_in_count_function12003_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COUNT_SYMBOL_in_count_function12003	= { FOLLOW_COUNT_SYMBOL_in_count_function12003_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_count_function12005  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_count_function12005_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x09FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x010000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x4D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_count_function12005	= { FOLLOW_OPEN_PAR_SYMBOL_in_count_function12005_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALL_SYMBOL_in_count_function12012  */
static	ANTLR3_BITWORD FOLLOW_ALL_SYMBOL_in_count_function12012_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALL_SYMBOL_in_count_function12012	= { FOLLOW_ALL_SYMBOL_in_count_function12012_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_count_function12015  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_count_function12015_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_count_function12015	= { FOLLOW_MULT_OPERATOR_in_count_function12015_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_in_aggregate_expression_in_count_function12021  */
static	ANTLR3_BITWORD FOLLOW_in_aggregate_expression_in_count_function12021_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_in_aggregate_expression_in_count_function12021	= { FOLLOW_in_aggregate_expression_in_count_function12021_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DISTINCT_SYMBOL_in_count_function12027  */
static	ANTLR3_BITWORD FOLLOW_DISTINCT_SYMBOL_in_count_function12027_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DISTINCT_SYMBOL_in_count_function12027	= { FOLLOW_DISTINCT_SYMBOL_in_count_function12027_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_count_function12029  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_count_function12029_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_count_function12029	= { FOLLOW_expression_list_in_count_function12029_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_count_function12035  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_count_function12035_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_count_function12035	= { FOLLOW_CLOSE_PAR_SYMBOL_in_count_function12035_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_call_expression_in_function_call12044  */
static	ANTLR3_BITWORD FOLLOW_function_call_expression_in_function_call12044_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_call_expression_in_function_call12044	= { FOLLOW_function_call_expression_in_function_call12044_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_pure_identifier_in_function_call_expression12053  */
static	ANTLR3_BITWORD FOLLOW_pure_identifier_in_function_call_expression12053_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_pure_identifier_in_function_call_expression12053	= { FOLLOW_pure_identifier_in_function_call_expression12053_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12060  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12060_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x9CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12060	= { FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12060_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_aliased_expression_list_in_function_call_expression12062  */
static	ANTLR3_BITWORD FOLLOW_aliased_expression_list_in_function_call_expression12062_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_aliased_expression_list_in_function_call_expression12062	= { FOLLOW_aliased_expression_list_in_function_call_expression12062_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12065  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12065_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12065	= { FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12065_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_function_call_expression12072  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_function_call_expression12072_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_function_call_expression12072	= { FOLLOW_dot_identifier_in_function_call_expression12072_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12074  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12074_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x9CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12074	= { FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12074_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_function_call_expression12076  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_function_call_expression12076_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_function_call_expression12076	= { FOLLOW_expression_list_in_function_call_expression12076_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12079  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12079_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12079	= { FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12079_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_aliased_expression_in_aliased_expression_list12092  */
static	ANTLR3_BITWORD FOLLOW_aliased_expression_in_aliased_expression_list12092_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_aliased_expression_in_aliased_expression_list12092	= { FOLLOW_aliased_expression_in_aliased_expression_list12092_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_aliased_expression_list12095  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_aliased_expression_list12095_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_aliased_expression_list12095	= { FOLLOW_COMMA_SYMBOL_in_aliased_expression_list12095_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_aliased_expression_in_aliased_expression_list12097  */
static	ANTLR3_BITWORD FOLLOW_aliased_expression_in_aliased_expression_list12097_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_aliased_expression_in_aliased_expression_list12097	= { FOLLOW_aliased_expression_in_aliased_expression_list12097_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_aliased_expression12108  */
static	ANTLR3_BITWORD FOLLOW_expression_in_aliased_expression12108_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF2), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0030800000000000), ANTLR3_UINT64_LIT(0x0000001800100000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_aliased_expression12108	= { FOLLOW_expression_in_aliased_expression12108_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_alias_in_aliased_expression12110  */
static	ANTLR3_BITWORD FOLLOW_select_alias_in_aliased_expression12110_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_alias_in_aliased_expression12110	= { FOLLOW_select_alias_in_aliased_expression12110_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_variable_in_variable12131  */
static	ANTLR3_BITWORD FOLLOW_user_variable_in_variable12131_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_user_variable_in_variable12131	= { FOLLOW_user_variable_in_variable12131_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASSIGN_OPERATOR_in_variable12134  */
static	ANTLR3_BITWORD FOLLOW_ASSIGN_OPERATOR_in_variable12134_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ASSIGN_OPERATOR_in_variable12134	= { FOLLOW_ASSIGN_OPERATOR_in_variable12134_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_variable12136  */
static	ANTLR3_BITWORD FOLLOW_expression_in_variable12136_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_variable12136	= { FOLLOW_expression_in_variable12136_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_system_variable_in_variable12143  */
static	ANTLR3_BITWORD FOLLOW_system_variable_in_variable12143_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_system_variable_in_variable12143	= { FOLLOW_system_variable_in_variable12143_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_SIGN_SYMBOL_in_user_variable12153  */
static	ANTLR3_BITWORD FOLLOW_AT_SIGN_SYMBOL_in_user_variable12153_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_SIGN_SYMBOL_in_user_variable12153	= { FOLLOW_AT_SIGN_SYMBOL_in_user_variable12153_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_user_variable12155  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_user_variable12155_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_user_variable12155	= { FOLLOW_text_or_identifier_in_user_variable12155_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_TEXT_SUFFIX_in_user_variable12161  */
static	ANTLR3_BITWORD FOLLOW_AT_TEXT_SUFFIX_in_user_variable12161_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_TEXT_SUFFIX_in_user_variable12161	= { FOLLOW_AT_TEXT_SUFFIX_in_user_variable12161_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_AT_SIGN_SYMBOL_in_system_variable12171  */
static	ANTLR3_BITWORD FOLLOW_AT_AT_SIGN_SYMBOL_in_system_variable12171_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_AT_SIGN_SYMBOL_in_system_variable12171	= { FOLLOW_AT_AT_SIGN_SYMBOL_in_system_variable12171_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_option_type_in_system_variable12187  */
static	ANTLR3_BITWORD FOLLOW_option_type_in_system_variable12187_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_option_type_in_system_variable12187	= { FOLLOW_option_type_in_system_variable12187_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_system_variable12189  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_system_variable12189_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_system_variable12189	= { FOLLOW_DOT_SYMBOL_in_system_variable12189_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_name_in_system_variable12193  */
static	ANTLR3_BITWORD FOLLOW_variable_name_in_system_variable12193_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_name_in_system_variable12193	= { FOLLOW_variable_name_in_system_variable12193_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_variable_name12202  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_variable_name12202_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_variable_name12202	= { FOLLOW_identifier_in_variable_name12202_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_variable_name12204  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_variable_name12204_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_variable_name12204	= { FOLLOW_dot_identifier_in_variable_name12204_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_variable_name12214  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_variable_name12214_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_variable_name12214	= { FOLLOW_DEFAULT_SYMBOL_in_variable_name12214_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_variable_name12216  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_variable_name12216_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_variable_name12216	= { FOLLOW_dot_identifier_in_variable_name12216_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MATCH_SYMBOL_in_match_expression12225  */
static	ANTLR3_BITWORD FOLLOW_MATCH_SYMBOL_in_match_expression12225_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MATCH_SYMBOL_in_match_expression12225	= { FOLLOW_MATCH_SYMBOL_in_match_expression12225_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12227  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12227_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12227	= { FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12227_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_name_list_in_match_expression12229  */
static	ANTLR3_BITWORD FOLLOW_field_name_list_in_match_expression12229_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_name_list_in_match_expression12229	= { FOLLOW_field_name_list_in_match_expression12229_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12231  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12231_bits[]	= { ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12231	= { FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12231_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AGAINST_SYMBOL_in_match_expression12233  */
static	ANTLR3_BITWORD FOLLOW_AGAINST_SYMBOL_in_match_expression12233_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AGAINST_SYMBOL_in_match_expression12233	= { FOLLOW_AGAINST_SYMBOL_in_match_expression12233_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12235  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12235_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12235	= { FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12235_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_match_expression12237  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_match_expression12237_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_match_expression12237	= { FOLLOW_bitwise_or_expression_in_match_expression12237_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_match_expression12246  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_match_expression12246_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_match_expression12246	= { FOLLOW_IN_SYMBOL_in_match_expression12246_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BOOLEAN_SYMBOL_in_match_expression12248  */
static	ANTLR3_BITWORD FOLLOW_BOOLEAN_SYMBOL_in_match_expression12248_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BOOLEAN_SYMBOL_in_match_expression12248	= { FOLLOW_BOOLEAN_SYMBOL_in_match_expression12248_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MODE_SYMBOL_in_match_expression12250  */
static	ANTLR3_BITWORD FOLLOW_MODE_SYMBOL_in_match_expression12250_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MODE_SYMBOL_in_match_expression12250	= { FOLLOW_MODE_SYMBOL_in_match_expression12250_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_match_expression12258  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_match_expression12258_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_match_expression12258	= { FOLLOW_IN_SYMBOL_in_match_expression12258_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NATURAL_SYMBOL_in_match_expression12260  */
static	ANTLR3_BITWORD FOLLOW_NATURAL_SYMBOL_in_match_expression12260_bits[]	= { ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NATURAL_SYMBOL_in_match_expression12260	= { FOLLOW_NATURAL_SYMBOL_in_match_expression12260_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LANGUAGE_SYMBOL_in_match_expression12262  */
static	ANTLR3_BITWORD FOLLOW_LANGUAGE_SYMBOL_in_match_expression12262_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LANGUAGE_SYMBOL_in_match_expression12262	= { FOLLOW_LANGUAGE_SYMBOL_in_match_expression12262_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MODE_SYMBOL_in_match_expression12264  */
static	ANTLR3_BITWORD FOLLOW_MODE_SYMBOL_in_match_expression12264_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MODE_SYMBOL_in_match_expression12264	= { FOLLOW_MODE_SYMBOL_in_match_expression12264_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_match_expression12269  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_match_expression12269_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000400) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_match_expression12269	= { FOLLOW_WITH_SYMBOL_in_match_expression12269_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_QUERY_SYMBOL_in_match_expression12271  */
static	ANTLR3_BITWORD FOLLOW_QUERY_SYMBOL_in_match_expression12271_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_QUERY_SYMBOL_in_match_expression12271	= { FOLLOW_QUERY_SYMBOL_in_match_expression12271_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXPANSION_SYMBOL_in_match_expression12273  */
static	ANTLR3_BITWORD FOLLOW_EXPANSION_SYMBOL_in_match_expression12273_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EXPANSION_SYMBOL_in_match_expression12273	= { FOLLOW_EXPANSION_SYMBOL_in_match_expression12273_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12283  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12283_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12283	= { FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12283_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CASE_SYMBOL_in_case_expression12293  */
static	ANTLR3_BITWORD FOLLOW_CASE_SYMBOL_in_case_expression12293_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000522E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_CASE_SYMBOL_in_case_expression12293	= { FOLLOW_CASE_SYMBOL_in_case_expression12293_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_case_expression12295  */
static	ANTLR3_BITWORD FOLLOW_expression_in_case_expression12295_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_case_expression12295	= { FOLLOW_expression_in_case_expression12295_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_when_expression_in_case_expression12299  */
static	ANTLR3_BITWORD FOLLOW_when_expression_in_case_expression12299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_when_expression_in_case_expression12299	= { FOLLOW_when_expression_in_case_expression12299_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_then_expression_in_case_expression12301  */
static	ANTLR3_BITWORD FOLLOW_then_expression_in_case_expression12301_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_then_expression_in_case_expression12301	= { FOLLOW_then_expression_in_case_expression12301_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_else_expression_in_case_expression12305  */
static	ANTLR3_BITWORD FOLLOW_else_expression_in_case_expression12305_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_else_expression_in_case_expression12305	= { FOLLOW_else_expression_in_case_expression12305_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_case_expression12308  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_case_expression12308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_case_expression12308	= { FOLLOW_END_SYMBOL_in_case_expression12308_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WHEN_SYMBOL_in_when_expression12317  */
static	ANTLR3_BITWORD FOLLOW_WHEN_SYMBOL_in_when_expression12317_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_WHEN_SYMBOL_in_when_expression12317	= { FOLLOW_WHEN_SYMBOL_in_when_expression12317_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_when_expression12319  */
static	ANTLR3_BITWORD FOLLOW_expression_in_when_expression12319_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_when_expression12319	= { FOLLOW_expression_in_when_expression12319_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_THEN_SYMBOL_in_then_expression12328  */
static	ANTLR3_BITWORD FOLLOW_THEN_SYMBOL_in_then_expression12328_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_THEN_SYMBOL_in_then_expression12328	= { FOLLOW_THEN_SYMBOL_in_then_expression12328_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_then_expression12330  */
static	ANTLR3_BITWORD FOLLOW_expression_in_then_expression12330_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_then_expression12330	= { FOLLOW_expression_in_then_expression12330_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ELSE_SYMBOL_in_else_expression12339  */
static	ANTLR3_BITWORD FOLLOW_ELSE_SYMBOL_in_else_expression12339_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ELSE_SYMBOL_in_else_expression12339	= { FOLLOW_ELSE_SYMBOL_in_else_expression12339_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_else_expression12341  */
static	ANTLR3_BITWORD FOLLOW_expression_in_else_expression12341_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_else_expression12341	= { FOLLOW_expression_in_else_expression12341_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_cast_expression12350  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_cast_expression12350_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000010204000C4), ANTLR3_UINT64_LIT(0x030060D04880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1C37000122E00600), ANTLR3_UINT64_LIT(0x0400011810120800), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_cast_expression12350	= { FOLLOW_BINARY_SYMBOL_in_cast_expression12350_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_primary_in_cast_expression12352  */
static	ANTLR3_BITWORD FOLLOW_primary_in_cast_expression12352_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_primary_in_cast_expression12352	= { FOLLOW_primary_in_cast_expression12352_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CAST_SYMBOL_in_cast_expression12357  */
static	ANTLR3_BITWORD FOLLOW_CAST_SYMBOL_in_cast_expression12357_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CAST_SYMBOL_in_cast_expression12357	= { FOLLOW_CAST_SYMBOL_in_cast_expression12357_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12359  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12359_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12359	= { FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12359_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_cast_expression12361  */
static	ANTLR3_BITWORD FOLLOW_expression_in_cast_expression12361_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_cast_expression12361	= { FOLLOW_expression_in_cast_expression12361_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_cast_expression12363  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_cast_expression12363_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0003000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000008000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_cast_expression12363	= { FOLLOW_AS_SYMBOL_in_cast_expression12363_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cast_type_in_cast_expression12365  */
static	ANTLR3_BITWORD FOLLOW_cast_type_in_cast_expression12365_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_cast_type_in_cast_expression12365	= { FOLLOW_cast_type_in_cast_expression12365_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12367  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12367_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12367	= { FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12367_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONVERT_SYMBOL_in_cast_expression12372  */
static	ANTLR3_BITWORD FOLLOW_CONVERT_SYMBOL_in_cast_expression12372_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONVERT_SYMBOL_in_cast_expression12372	= { FOLLOW_CONVERT_SYMBOL_in_cast_expression12372_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12374  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12374_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12374	= { FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12374_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_cast_expression12376  */
static	ANTLR3_BITWORD FOLLOW_expression_in_cast_expression12376_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_cast_expression12376	= { FOLLOW_expression_in_cast_expression12376_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_cast_expression12379  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_cast_expression12379_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0003000000000000), ANTLR3_UINT64_LIT(0x0000100000000000), ANTLR3_UINT64_LIT(0x0000008000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_cast_expression12379	= { FOLLOW_COMMA_SYMBOL_in_cast_expression12379_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cast_type_in_cast_expression12381  */
static	ANTLR3_BITWORD FOLLOW_cast_type_in_cast_expression12381_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_cast_type_in_cast_expression12381	= { FOLLOW_cast_type_in_cast_expression12381_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_USING_SYMBOL_in_cast_expression12385  */
static	ANTLR3_BITWORD FOLLOW_USING_SYMBOL_in_cast_expression12385_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_USING_SYMBOL_in_cast_expression12385	= { FOLLOW_USING_SYMBOL_in_cast_expression12385_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_cast_expression12387  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_cast_expression12387_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_cast_expression12387	= { FOLLOW_charset_name_in_cast_expression12387_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12390  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12390_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12390	= { FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12390_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_cast_type12399  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_cast_type12399_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_cast_type12399	= { FOLLOW_BINARY_SYMBOL_in_cast_type12399_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_cast_type12401  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_cast_type12401_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_cast_type12401	= { FOLLOW_field_length_in_cast_type12401_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_cast_type12407  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_cast_type12407_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_cast_type12407	= { FOLLOW_CHAR_SYMBOL_in_cast_type12407_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_cast_type12409  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_cast_type12409_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_cast_type12409	= { FOLLOW_field_length_in_cast_type12409_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_encoding_in_cast_type12412  */
static	ANTLR3_BITWORD FOLLOW_encoding_in_cast_type12412_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_encoding_in_cast_type12412	= { FOLLOW_encoding_in_cast_type12412_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NCHAR_SYMBOL_in_cast_type12418  */
static	ANTLR3_BITWORD FOLLOW_NCHAR_SYMBOL_in_cast_type12418_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NCHAR_SYMBOL_in_cast_type12418	= { FOLLOW_NCHAR_SYMBOL_in_cast_type12418_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_cast_type12420  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_cast_type12420_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_cast_type12420	= { FOLLOW_field_length_in_cast_type12420_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SIGNED_SYMBOL_in_cast_type12426  */
static	ANTLR3_BITWORD FOLLOW_SIGNED_SYMBOL_in_cast_type12426_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SIGNED_SYMBOL_in_cast_type12426	= { FOLLOW_SIGNED_SYMBOL_in_cast_type12426_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_SYMBOL_in_cast_type12428  */
static	ANTLR3_BITWORD FOLLOW_INT_SYMBOL_in_cast_type12428_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_SYMBOL_in_cast_type12428	= { FOLLOW_INT_SYMBOL_in_cast_type12428_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNSIGNED_SYMBOL_in_cast_type12434  */
static	ANTLR3_BITWORD FOLLOW_UNSIGNED_SYMBOL_in_cast_type12434_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000200000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNSIGNED_SYMBOL_in_cast_type12434	= { FOLLOW_UNSIGNED_SYMBOL_in_cast_type12434_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_SYMBOL_in_cast_type12436  */
static	ANTLR3_BITWORD FOLLOW_INT_SYMBOL_in_cast_type12436_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_SYMBOL_in_cast_type12436	= { FOLLOW_INT_SYMBOL_in_cast_type12436_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATE_SYMBOL_in_cast_type12442  */
static	ANTLR3_BITWORD FOLLOW_DATE_SYMBOL_in_cast_type12442_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATE_SYMBOL_in_cast_type12442	= { FOLLOW_DATE_SYMBOL_in_cast_type12442_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIME_SYMBOL_in_cast_type12447  */
static	ANTLR3_BITWORD FOLLOW_TIME_SYMBOL_in_cast_type12447_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIME_SYMBOL_in_cast_type12447	= { FOLLOW_TIME_SYMBOL_in_cast_type12447_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_type_datetime_precision_in_cast_type12449  */
static	ANTLR3_BITWORD FOLLOW_type_datetime_precision_in_cast_type12449_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_type_datetime_precision_in_cast_type12449	= { FOLLOW_type_datetime_precision_in_cast_type12449_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATETIME_SYMBOL_in_cast_type12455  */
static	ANTLR3_BITWORD FOLLOW_DATETIME_SYMBOL_in_cast_type12455_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATETIME_SYMBOL_in_cast_type12455	= { FOLLOW_DATETIME_SYMBOL_in_cast_type12455_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_type_datetime_precision_in_cast_type12457  */
static	ANTLR3_BITWORD FOLLOW_type_datetime_precision_in_cast_type12457_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_type_datetime_precision_in_cast_type12457	= { FOLLOW_type_datetime_precision_in_cast_type12457_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DECIMAL_SYMBOL_in_cast_type12463  */
static	ANTLR3_BITWORD FOLLOW_DECIMAL_SYMBOL_in_cast_type12463_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DECIMAL_SYMBOL_in_cast_type12463	= { FOLLOW_DECIMAL_SYMBOL_in_cast_type12463_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_float_options_in_cast_type12465  */
static	ANTLR3_BITWORD FOLLOW_float_options_in_cast_type12465_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_float_options_in_cast_type12465	= { FOLLOW_float_options_in_cast_type12465_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JSON_SYMBOL_in_cast_type12473  */
static	ANTLR3_BITWORD FOLLOW_JSON_SYMBOL_in_cast_type12473_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_JSON_SYMBOL_in_cast_type12473	= { FOLLOW_JSON_SYMBOL_in_cast_type12473_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASCII_SYMBOL_in_encoding12482  */
static	ANTLR3_BITWORD FOLLOW_ASCII_SYMBOL_in_encoding12482_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ASCII_SYMBOL_in_encoding12482	= { FOLLOW_ASCII_SYMBOL_in_encoding12482_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_encoding12484  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_encoding12484_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_encoding12484	= { FOLLOW_BINARY_SYMBOL_in_encoding12484_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_encoding12490  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_encoding12490_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000822), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_encoding12490	= { FOLLOW_BINARY_SYMBOL_in_encoding12490_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASCII_SYMBOL_in_encoding12493  */
static	ANTLR3_BITWORD FOLLOW_ASCII_SYMBOL_in_encoding12493_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ASCII_SYMBOL_in_encoding12493	= { FOLLOW_ASCII_SYMBOL_in_encoding12493_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNICODE_SYMBOL_in_encoding12497  */
static	ANTLR3_BITWORD FOLLOW_UNICODE_SYMBOL_in_encoding12497_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_UNICODE_SYMBOL_in_encoding12497	= { FOLLOW_UNICODE_SYMBOL_in_encoding12497_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_encoding12501  */
static	ANTLR3_BITWORD FOLLOW_charset_in_encoding12501_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_encoding12501	= { FOLLOW_charset_in_encoding12501_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_encoding12503  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_encoding12503_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_encoding12503	= { FOLLOW_charset_name_in_encoding12503_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNICODE_SYMBOL_in_encoding12510  */
static	ANTLR3_BITWORD FOLLOW_UNICODE_SYMBOL_in_encoding12510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_UNICODE_SYMBOL_in_encoding12510	= { FOLLOW_UNICODE_SYMBOL_in_encoding12510_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_encoding12512  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_encoding12512_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_encoding12512	= { FOLLOW_BINARY_SYMBOL_in_encoding12512_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BYTE_SYMBOL_in_encoding12518  */
static	ANTLR3_BITWORD FOLLOW_BYTE_SYMBOL_in_encoding12518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BYTE_SYMBOL_in_encoding12518	= { FOLLOW_BYTE_SYMBOL_in_encoding12518_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_encoding12523  */
static	ANTLR3_BITWORD FOLLOW_charset_in_encoding12523_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_encoding12523	= { FOLLOW_charset_in_encoding12523_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_encoding12525  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_encoding12525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_encoding12525	= { FOLLOW_charset_name_in_encoding12525_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_encoding12527  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_encoding12527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_encoding12527	= { FOLLOW_BINARY_SYMBOL_in_encoding12527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_charset12537  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_charset12537_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_charset12537	= { FOLLOW_CHAR_SYMBOL_in_charset12537_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_charset12539  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_charset12539_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_charset12539	= { FOLLOW_SET_SYMBOL_in_charset12539_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHARSET_SYMBOL_in_charset12543  */
static	ANTLR3_BITWORD FOLLOW_CHARSET_SYMBOL_in_charset12543_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CHARSET_SYMBOL_in_charset12543	= { FOLLOW_CHARSET_SYMBOL_in_charset12543_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_interval12624  */
static	ANTLR3_BITWORD FOLLOW_expression_in_interval12624_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0004000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000045000000), ANTLR3_UINT64_LIT(0x0000800000000200), ANTLR3_UINT64_LIT(0x0104000000000000), ANTLR3_UINT64_LIT(0x00001E0000000000), ANTLR3_UINT64_LIT(0x000000001C010000), ANTLR3_UINT64_LIT(0x0000000000600000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000080000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_interval12624	= { FOLLOW_expression_in_interval12624_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_unit_in_interval12626  */
static	ANTLR3_BITWORD FOLLOW_interval_unit_in_interval12626_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_unit_in_interval12626	= { FOLLOW_interval_unit_in_interval12626_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_timestamp_unit_in_interval_unit12636  */
static	ANTLR3_BITWORD FOLLOW_interval_timestamp_unit_in_interval_unit12636_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_timestamp_unit_in_interval_unit12636	= { FOLLOW_interval_timestamp_unit_in_interval_unit12636_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SECOND_MICROSECOND_SYMBOL_in_interval_unit12641  */
static	ANTLR3_BITWORD FOLLOW_SECOND_MICROSECOND_SYMBOL_in_interval_unit12641_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SECOND_MICROSECOND_SYMBOL_in_interval_unit12641	= { FOLLOW_SECOND_MICROSECOND_SYMBOL_in_interval_unit12641_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MINUTE_MICROSECOND_SYMBOL_in_interval_unit12646  */
static	ANTLR3_BITWORD FOLLOW_MINUTE_MICROSECOND_SYMBOL_in_interval_unit12646_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MINUTE_MICROSECOND_SYMBOL_in_interval_unit12646	= { FOLLOW_MINUTE_MICROSECOND_SYMBOL_in_interval_unit12646_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MINUTE_SECOND_SYMBOL_in_interval_unit12651  */
static	ANTLR3_BITWORD FOLLOW_MINUTE_SECOND_SYMBOL_in_interval_unit12651_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MINUTE_SECOND_SYMBOL_in_interval_unit12651	= { FOLLOW_MINUTE_SECOND_SYMBOL_in_interval_unit12651_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOUR_MICROSECOND_SYMBOL_in_interval_unit12656  */
static	ANTLR3_BITWORD FOLLOW_HOUR_MICROSECOND_SYMBOL_in_interval_unit12656_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HOUR_MICROSECOND_SYMBOL_in_interval_unit12656	= { FOLLOW_HOUR_MICROSECOND_SYMBOL_in_interval_unit12656_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOUR_SECOND_SYMBOL_in_interval_unit12661  */
static	ANTLR3_BITWORD FOLLOW_HOUR_SECOND_SYMBOL_in_interval_unit12661_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HOUR_SECOND_SYMBOL_in_interval_unit12661	= { FOLLOW_HOUR_SECOND_SYMBOL_in_interval_unit12661_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HOUR_MINUTE_SYMBOL_in_interval_unit12666  */
static	ANTLR3_BITWORD FOLLOW_HOUR_MINUTE_SYMBOL_in_interval_unit12666_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HOUR_MINUTE_SYMBOL_in_interval_unit12666	= { FOLLOW_HOUR_MINUTE_SYMBOL_in_interval_unit12666_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_MICROSECOND_SYMBOL_in_interval_unit12671  */
static	ANTLR3_BITWORD FOLLOW_DAY_MICROSECOND_SYMBOL_in_interval_unit12671_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_MICROSECOND_SYMBOL_in_interval_unit12671	= { FOLLOW_DAY_MICROSECOND_SYMBOL_in_interval_unit12671_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_SECOND_SYMBOL_in_interval_unit12676  */
static	ANTLR3_BITWORD FOLLOW_DAY_SECOND_SYMBOL_in_interval_unit12676_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_SECOND_SYMBOL_in_interval_unit12676	= { FOLLOW_DAY_SECOND_SYMBOL_in_interval_unit12676_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_MINUTE_SYMBOL_in_interval_unit12681  */
static	ANTLR3_BITWORD FOLLOW_DAY_MINUTE_SYMBOL_in_interval_unit12681_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_MINUTE_SYMBOL_in_interval_unit12681	= { FOLLOW_DAY_MINUTE_SYMBOL_in_interval_unit12681_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DAY_HOUR_SYMBOL_in_interval_unit12686  */
static	ANTLR3_BITWORD FOLLOW_DAY_HOUR_SYMBOL_in_interval_unit12686_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DAY_HOUR_SYMBOL_in_interval_unit12686	= { FOLLOW_DAY_HOUR_SYMBOL_in_interval_unit12686_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_YEAR_MONTH_SYMBOL_in_interval_unit12691  */
static	ANTLR3_BITWORD FOLLOW_YEAR_MONTH_SYMBOL_in_interval_unit12691_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_YEAR_MONTH_SYMBOL_in_interval_unit12691	= { FOLLOW_YEAR_MONTH_SYMBOL_in_interval_unit12691_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_expression_list_with_parentheses12759  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_expression_list_with_parentheses12759_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_expression_list_with_parentheses12759	= { FOLLOW_OPEN_PAR_SYMBOL_in_expression_list_with_parentheses12759_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_expression_list_with_parentheses12761  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_expression_list_with_parentheses12761_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_expression_list_with_parentheses12761	= { FOLLOW_expression_list_in_expression_list_with_parentheses12761_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_expression_list_with_parentheses12763  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_expression_list_with_parentheses12763_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_expression_list_with_parentheses12763	= { FOLLOW_CLOSE_PAR_SYMBOL_in_expression_list_with_parentheses12763_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_optional_expression_list_with_parentheses12772  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_optional_expression_list_with_parentheses12772_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x9CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_optional_expression_list_with_parentheses12772	= { FOLLOW_OPEN_PAR_SYMBOL_in_optional_expression_list_with_parentheses12772_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_list_in_optional_expression_list_with_parentheses12774  */
static	ANTLR3_BITWORD FOLLOW_expression_list_in_optional_expression_list_with_parentheses12774_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_list_in_optional_expression_list_with_parentheses12774	= { FOLLOW_expression_list_in_optional_expression_list_with_parentheses12774_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_optional_expression_list_with_parentheses12777  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_optional_expression_list_with_parentheses12777_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_optional_expression_list_with_parentheses12777	= { FOLLOW_CLOSE_PAR_SYMBOL_in_optional_expression_list_with_parentheses12777_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_expression_list12786  */
static	ANTLR3_BITWORD FOLLOW_expression_in_expression_list12786_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_expression_list12786	= { FOLLOW_expression_in_expression_list12786_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_expression_list12789  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_expression_list12789_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_expression_list12789	= { FOLLOW_COMMA_SYMBOL_in_expression_list12789_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_expression_list12791  */
static	ANTLR3_BITWORD FOLLOW_expression_in_expression_list12791_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_expression_list12791	= { FOLLOW_expression_in_expression_list12791_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_expression_list_with_direction12802  */
static	ANTLR3_BITWORD FOLLOW_expression_in_expression_list_with_direction12802_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_expression_list_with_direction12802	= { FOLLOW_expression_in_expression_list_with_direction12802_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_direction_in_expression_list_with_direction12804  */
static	ANTLR3_BITWORD FOLLOW_direction_in_expression_list_with_direction12804_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_direction_in_expression_list_with_direction12804	= { FOLLOW_direction_in_expression_list_with_direction12804_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_expression_list_with_direction12821  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_expression_list_with_direction12821_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_expression_list_with_direction12821	= { FOLLOW_COMMA_SYMBOL_in_expression_list_with_direction12821_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_expression_list_with_direction12823  */
static	ANTLR3_BITWORD FOLLOW_expression_in_expression_list_with_direction12823_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_expression_list_with_direction12823	= { FOLLOW_expression_in_expression_list_with_direction12823_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_direction_in_expression_list_with_direction12825  */
static	ANTLR3_BITWORD FOLLOW_direction_in_expression_list_with_direction12825_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_direction_in_expression_list_with_direction12825	= { FOLLOW_direction_in_expression_list_with_direction12825_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_channel12841  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_channel12841_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_channel12841	= { FOLLOW_FOR_SYMBOL_in_channel12841_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHANNEL_SYMBOL_in_channel12843  */
static	ANTLR3_BITWORD FOLLOW_CHANNEL_SYMBOL_in_channel12843_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHANNEL_SYMBOL_in_channel12843	= { FOLLOW_CHANNEL_SYMBOL_in_channel12843_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_no_linebreak_in_channel12845  */
static	ANTLR3_BITWORD FOLLOW_text_string_no_linebreak_in_channel12845_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_no_linebreak_in_channel12845	= { FOLLOW_text_string_no_linebreak_in_channel12845_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_statement_in_compound_statement12869  */
static	ANTLR3_BITWORD FOLLOW_statement_in_compound_statement12869_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_statement_in_compound_statement12869	= { FOLLOW_statement_in_compound_statement12869_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_return_statement_in_compound_statement12874  */
static	ANTLR3_BITWORD FOLLOW_return_statement_in_compound_statement12874_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_return_statement_in_compound_statement12874	= { FOLLOW_return_statement_in_compound_statement12874_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_statement_in_compound_statement12879  */
static	ANTLR3_BITWORD FOLLOW_if_statement_in_compound_statement12879_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_if_statement_in_compound_statement12879	= { FOLLOW_if_statement_in_compound_statement12879_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_case_statement_in_compound_statement12884  */
static	ANTLR3_BITWORD FOLLOW_case_statement_in_compound_statement12884_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_case_statement_in_compound_statement12884	= { FOLLOW_case_statement_in_compound_statement12884_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_labeled_block_in_compound_statement12889  */
static	ANTLR3_BITWORD FOLLOW_labeled_block_in_compound_statement12889_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_labeled_block_in_compound_statement12889	= { FOLLOW_labeled_block_in_compound_statement12889_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unlabeled_block_in_compound_statement12894  */
static	ANTLR3_BITWORD FOLLOW_unlabeled_block_in_compound_statement12894_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_unlabeled_block_in_compound_statement12894	= { FOLLOW_unlabeled_block_in_compound_statement12894_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_labeled_control_in_compound_statement12899  */
static	ANTLR3_BITWORD FOLLOW_labeled_control_in_compound_statement12899_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_labeled_control_in_compound_statement12899	= { FOLLOW_labeled_control_in_compound_statement12899_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unlabeled_control_in_compound_statement12904  */
static	ANTLR3_BITWORD FOLLOW_unlabeled_control_in_compound_statement12904_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_unlabeled_control_in_compound_statement12904	= { FOLLOW_unlabeled_control_in_compound_statement12904_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_leave_statement_in_compound_statement12909  */
static	ANTLR3_BITWORD FOLLOW_leave_statement_in_compound_statement12909_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_leave_statement_in_compound_statement12909	= { FOLLOW_leave_statement_in_compound_statement12909_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_iterate_statement_in_compound_statement12914  */
static	ANTLR3_BITWORD FOLLOW_iterate_statement_in_compound_statement12914_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_iterate_statement_in_compound_statement12914	= { FOLLOW_iterate_statement_in_compound_statement12914_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cursor_open_in_compound_statement12921  */
static	ANTLR3_BITWORD FOLLOW_cursor_open_in_compound_statement12921_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cursor_open_in_compound_statement12921	= { FOLLOW_cursor_open_in_compound_statement12921_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cursor_fetch_in_compound_statement12926  */
static	ANTLR3_BITWORD FOLLOW_cursor_fetch_in_compound_statement12926_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cursor_fetch_in_compound_statement12926	= { FOLLOW_cursor_fetch_in_compound_statement12926_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cursor_close_in_compound_statement12931  */
static	ANTLR3_BITWORD FOLLOW_cursor_close_in_compound_statement12931_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cursor_close_in_compound_statement12931	= { FOLLOW_cursor_close_in_compound_statement12931_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RETURN_SYMBOL_in_return_statement12940  */
static	ANTLR3_BITWORD FOLLOW_RETURN_SYMBOL_in_return_statement12940_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_RETURN_SYMBOL_in_return_statement12940	= { FOLLOW_RETURN_SYMBOL_in_return_statement12940_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_return_statement12942  */
static	ANTLR3_BITWORD FOLLOW_expression_in_return_statement12942_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_return_statement12942	= { FOLLOW_expression_in_return_statement12942_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IF_SYMBOL_in_if_statement12951  */
static	ANTLR3_BITWORD FOLLOW_IF_SYMBOL_in_if_statement12951_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_IF_SYMBOL_in_if_statement12951	= { FOLLOW_IF_SYMBOL_in_if_statement12951_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_body_in_if_statement12953  */
static	ANTLR3_BITWORD FOLLOW_if_body_in_if_statement12953_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_if_body_in_if_statement12953	= { FOLLOW_if_body_in_if_statement12953_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_if_statement12955  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_if_statement12955_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_if_statement12955	= { FOLLOW_END_SYMBOL_in_if_statement12955_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IF_SYMBOL_in_if_statement12957  */
static	ANTLR3_BITWORD FOLLOW_IF_SYMBOL_in_if_statement12957_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_IF_SYMBOL_in_if_statement12957	= { FOLLOW_IF_SYMBOL_in_if_statement12957_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_if_body12967  */
static	ANTLR3_BITWORD FOLLOW_expression_in_if_body12967_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_if_body12967	= { FOLLOW_expression_in_if_body12967_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_then_statement_in_if_body12969  */
static	ANTLR3_BITWORD FOLLOW_then_statement_in_if_body12969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x3000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_then_statement_in_if_body12969	= { FOLLOW_then_statement_in_if_body12969_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ELSEIF_SYMBOL_in_if_body12972  */
static	ANTLR3_BITWORD FOLLOW_ELSEIF_SYMBOL_in_if_body12972_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ELSEIF_SYMBOL_in_if_body12972	= { FOLLOW_ELSEIF_SYMBOL_in_if_body12972_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_if_body_in_if_body12974  */
static	ANTLR3_BITWORD FOLLOW_if_body_in_if_body12974_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_if_body_in_if_body12974	= { FOLLOW_if_body_in_if_body12974_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ELSE_SYMBOL_in_if_body12978  */
static	ANTLR3_BITWORD FOLLOW_ELSE_SYMBOL_in_if_body12978_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ELSE_SYMBOL_in_if_body12978	= { FOLLOW_ELSE_SYMBOL_in_if_body12978_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_if_body12980  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_if_body12980_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_if_body12980	= { FOLLOW_compound_statement_list_in_if_body12980_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_THEN_SYMBOL_in_then_statement12991  */
static	ANTLR3_BITWORD FOLLOW_THEN_SYMBOL_in_then_statement12991_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_THEN_SYMBOL_in_then_statement12991	= { FOLLOW_THEN_SYMBOL_in_then_statement12991_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_then_statement12993  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_then_statement12993_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_then_statement12993	= { FOLLOW_compound_statement_list_in_then_statement12993_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_compound_statement_list13003  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_compound_statement_list13003_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_compound_statement_list13003	= { FOLLOW_compound_statement_in_compound_statement_list13003_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_compound_statement_list13005  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_compound_statement_list13005_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B582), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_compound_statement_list13005	= { FOLLOW_SEMICOLON_SYMBOL_in_compound_statement_list13005_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CASE_SYMBOL_in_case_statement13017  */
static	ANTLR3_BITWORD FOLLOW_CASE_SYMBOL_in_case_statement13017_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000522E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_CASE_SYMBOL_in_case_statement13017	= { FOLLOW_CASE_SYMBOL_in_case_statement13017_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_case_statement13019  */
static	ANTLR3_BITWORD FOLLOW_expression_in_case_statement13019_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_case_statement13019	= { FOLLOW_expression_in_case_statement13019_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_when_expression_in_case_statement13023  */
static	ANTLR3_BITWORD FOLLOW_when_expression_in_case_statement13023_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_when_expression_in_case_statement13023	= { FOLLOW_when_expression_in_case_statement13023_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_then_statement_in_case_statement13025  */
static	ANTLR3_BITWORD FOLLOW_then_statement_in_case_statement13025_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x2000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_then_statement_in_case_statement13025	= { FOLLOW_then_statement_in_case_statement13025_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_else_statement_in_case_statement13029  */
static	ANTLR3_BITWORD FOLLOW_else_statement_in_case_statement13029_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_else_statement_in_case_statement13029	= { FOLLOW_else_statement_in_case_statement13029_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_case_statement13032  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_case_statement13032_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_case_statement13032	= { FOLLOW_END_SYMBOL_in_case_statement13032_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CASE_SYMBOL_in_case_statement13034  */
static	ANTLR3_BITWORD FOLLOW_CASE_SYMBOL_in_case_statement13034_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CASE_SYMBOL_in_case_statement13034	= { FOLLOW_CASE_SYMBOL_in_case_statement13034_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ELSE_SYMBOL_in_else_statement13043  */
static	ANTLR3_BITWORD FOLLOW_ELSE_SYMBOL_in_else_statement13043_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ELSE_SYMBOL_in_else_statement13043	= { FOLLOW_ELSE_SYMBOL_in_else_statement13043_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_else_statement13045  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_else_statement13045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_else_statement13045	= { FOLLOW_compound_statement_list_in_else_statement13045_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_in_labeled_block13054  */
static	ANTLR3_BITWORD FOLLOW_label_in_labeled_block13054_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_label_in_labeled_block13054	= { FOLLOW_label_in_labeled_block13054_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_begin_end_block_in_labeled_block13056  */
static	ANTLR3_BITWORD FOLLOW_begin_end_block_in_labeled_block13056_bits[]	= { ANTLR3_UINT64_LIT(0xF800000000000002), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFDFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_begin_end_block_in_labeled_block13056	= { FOLLOW_begin_end_block_in_labeled_block13056_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_identifier_in_labeled_block13058  */
static	ANTLR3_BITWORD FOLLOW_label_identifier_in_labeled_block13058_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_label_identifier_in_labeled_block13058	= { FOLLOW_label_identifier_in_labeled_block13058_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_begin_end_block_in_unlabeled_block13069  */
static	ANTLR3_BITWORD FOLLOW_begin_end_block_in_unlabeled_block13069_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_begin_end_block_in_unlabeled_block13069	= { FOLLOW_begin_end_block_in_unlabeled_block13069_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_identifier_in_label13081  */
static	ANTLR3_BITWORD FOLLOW_label_identifier_in_label13081_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001) };
static  ANTLR3_BITSET_LIST FOLLOW_label_identifier_in_label13081	= { FOLLOW_label_identifier_in_label13081_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLON_SYMBOL_in_label13083  */
static	ANTLR3_BITWORD FOLLOW_COLON_SYMBOL_in_label13083_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_COLON_SYMBOL_in_label13083	= { FOLLOW_COLON_SYMBOL_in_label13083_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_pure_identifier_in_label_identifier13092  */
static	ANTLR3_BITWORD FOLLOW_pure_identifier_in_label_identifier13092_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_pure_identifier_in_label_identifier13092	= { FOLLOW_pure_identifier_in_label_identifier13092_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_keyword_sp_in_label_identifier13096  */
static	ANTLR3_BITWORD FOLLOW_keyword_sp_in_label_identifier13096_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_keyword_sp_in_label_identifier13096	= { FOLLOW_keyword_sp_in_label_identifier13096_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BEGIN_SYMBOL_in_begin_end_block13106  */
static	ANTLR3_BITWORD FOLLOW_BEGIN_SYMBOL_in_begin_end_block13106_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F48963EB580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238800004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_BEGIN_SYMBOL_in_begin_end_block13106	= { FOLLOW_BEGIN_SYMBOL_in_begin_end_block13106_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sp_declarations_in_begin_end_block13108  */
static	ANTLR3_BITWORD FOLLOW_sp_declarations_in_begin_end_block13108_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F48963EB580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_sp_declarations_in_begin_end_block13108	= { FOLLOW_sp_declarations_in_begin_end_block13108_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_begin_end_block13111  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_begin_end_block13111_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_begin_end_block13111	= { FOLLOW_compound_statement_list_in_begin_end_block13111_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_begin_end_block13114  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_begin_end_block13114_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_begin_end_block13114	= { FOLLOW_END_SYMBOL_in_begin_end_block13114_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_in_labeled_control13123  */
static	ANTLR3_BITWORD FOLLOW_label_in_labeled_control13123_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200), ANTLR3_UINT64_LIT(0x0000000000000100), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_label_in_labeled_control13123	= { FOLLOW_label_in_labeled_control13123_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unlabeled_control_in_labeled_control13125  */
static	ANTLR3_BITWORD FOLLOW_unlabeled_control_in_labeled_control13125_bits[]	= { ANTLR3_UINT64_LIT(0xF800000000000002), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFDFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_unlabeled_control_in_labeled_control13125	= { FOLLOW_unlabeled_control_in_labeled_control13125_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_identifier_in_labeled_control13127  */
static	ANTLR3_BITWORD FOLLOW_label_identifier_in_labeled_control13127_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_label_identifier_in_labeled_control13127	= { FOLLOW_label_identifier_in_labeled_control13127_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_loop_block_in_unlabeled_control13137  */
static	ANTLR3_BITWORD FOLLOW_loop_block_in_unlabeled_control13137_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_loop_block_in_unlabeled_control13137	= { FOLLOW_loop_block_in_unlabeled_control13137_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_while_do_block_in_unlabeled_control13142  */
static	ANTLR3_BITWORD FOLLOW_while_do_block_in_unlabeled_control13142_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_while_do_block_in_unlabeled_control13142	= { FOLLOW_while_do_block_in_unlabeled_control13142_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_repeat_until_block_in_unlabeled_control13147  */
static	ANTLR3_BITWORD FOLLOW_repeat_until_block_in_unlabeled_control13147_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_repeat_until_block_in_unlabeled_control13147	= { FOLLOW_repeat_until_block_in_unlabeled_control13147_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOOP_SYMBOL_in_loop_block13156  */
static	ANTLR3_BITWORD FOLLOW_LOOP_SYMBOL_in_loop_block13156_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_LOOP_SYMBOL_in_loop_block13156	= { FOLLOW_LOOP_SYMBOL_in_loop_block13156_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_loop_block13158  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_loop_block13158_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_loop_block13158	= { FOLLOW_compound_statement_list_in_loop_block13158_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_loop_block13160  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_loop_block13160_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_loop_block13160	= { FOLLOW_END_SYMBOL_in_loop_block13160_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOOP_SYMBOL_in_loop_block13162  */
static	ANTLR3_BITWORD FOLLOW_LOOP_SYMBOL_in_loop_block13162_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOOP_SYMBOL_in_loop_block13162	= { FOLLOW_LOOP_SYMBOL_in_loop_block13162_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WHILE_SYMBOL_in_while_do_block13171  */
static	ANTLR3_BITWORD FOLLOW_WHILE_SYMBOL_in_while_do_block13171_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_WHILE_SYMBOL_in_while_do_block13171	= { FOLLOW_WHILE_SYMBOL_in_while_do_block13171_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_while_do_block13173  */
static	ANTLR3_BITWORD FOLLOW_expression_in_while_do_block13173_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_while_do_block13173	= { FOLLOW_expression_in_while_do_block13173_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DO_SYMBOL_in_while_do_block13175  */
static	ANTLR3_BITWORD FOLLOW_DO_SYMBOL_in_while_do_block13175_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DO_SYMBOL_in_while_do_block13175	= { FOLLOW_DO_SYMBOL_in_while_do_block13175_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_while_do_block13177  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_while_do_block13177_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_while_do_block13177	= { FOLLOW_compound_statement_list_in_while_do_block13177_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_while_do_block13179  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_while_do_block13179_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_while_do_block13179	= { FOLLOW_END_SYMBOL_in_while_do_block13179_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WHILE_SYMBOL_in_while_do_block13181  */
static	ANTLR3_BITWORD FOLLOW_WHILE_SYMBOL_in_while_do_block13181_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_WHILE_SYMBOL_in_while_do_block13181	= { FOLLOW_WHILE_SYMBOL_in_while_do_block13181_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13190  */
static	ANTLR3_BITWORD FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13190_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13190	= { FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13190_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_list_in_repeat_until_block13192  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_list_in_repeat_until_block13192_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_list_in_repeat_until_block13192	= { FOLLOW_compound_statement_list_in_repeat_until_block13192_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNTIL_SYMBOL_in_repeat_until_block13194  */
static	ANTLR3_BITWORD FOLLOW_UNTIL_SYMBOL_in_repeat_until_block13194_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_UNTIL_SYMBOL_in_repeat_until_block13194	= { FOLLOW_UNTIL_SYMBOL_in_repeat_until_block13194_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_repeat_until_block13196  */
static	ANTLR3_BITWORD FOLLOW_expression_in_repeat_until_block13196_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000080000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_repeat_until_block13196	= { FOLLOW_expression_in_repeat_until_block13196_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_END_SYMBOL_in_repeat_until_block13198  */
static	ANTLR3_BITWORD FOLLOW_END_SYMBOL_in_repeat_until_block13198_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000100) };
static  ANTLR3_BITSET_LIST FOLLOW_END_SYMBOL_in_repeat_until_block13198	= { FOLLOW_END_SYMBOL_in_repeat_until_block13198_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13200  */
static	ANTLR3_BITWORD FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13200_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13200	= { FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13200_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sp_declaration_in_sp_declarations13210  */
static	ANTLR3_BITWORD FOLLOW_sp_declaration_in_sp_declarations13210_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_sp_declaration_in_sp_declarations13210	= { FOLLOW_sp_declaration_in_sp_declarations13210_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SEMICOLON_SYMBOL_in_sp_declarations13212  */
static	ANTLR3_BITWORD FOLLOW_SEMICOLON_SYMBOL_in_sp_declarations13212_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SEMICOLON_SYMBOL_in_sp_declarations13212	= { FOLLOW_SEMICOLON_SYMBOL_in_sp_declarations13212_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DECLARE_SYMBOL_in_sp_declaration13223  */
static	ANTLR3_BITWORD FOLLOW_DECLARE_SYMBOL_in_sp_declaration13223_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000800000), ANTLR3_UINT64_LIT(0x0000000000000008), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000001000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DECLARE_SYMBOL_in_sp_declaration13223	= { FOLLOW_DECLARE_SYMBOL_in_sp_declaration13223_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_declaration_in_sp_declaration13230  */
static	ANTLR3_BITWORD FOLLOW_variable_declaration_in_sp_declaration13230_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_declaration_in_sp_declaration13230	= { FOLLOW_variable_declaration_in_sp_declaration13230_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_condition_declaration_in_sp_declaration13236  */
static	ANTLR3_BITWORD FOLLOW_condition_declaration_in_sp_declaration13236_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_condition_declaration_in_sp_declaration13236	= { FOLLOW_condition_declaration_in_sp_declaration13236_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_handler_declaration_in_sp_declaration13242  */
static	ANTLR3_BITWORD FOLLOW_handler_declaration_in_sp_declaration13242_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_handler_declaration_in_sp_declaration13242	= { FOLLOW_handler_declaration_in_sp_declaration13242_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_cursor_declaration_in_sp_declaration13248  */
static	ANTLR3_BITWORD FOLLOW_cursor_declaration_in_sp_declaration13248_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_cursor_declaration_in_sp_declaration13248	= { FOLLOW_cursor_declaration_in_sp_declaration13248_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_variable_declaration13261  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_variable_declaration13261_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000003400), ANTLR3_UINT64_LIT(0x0004100003400001), ANTLR3_UINT64_LIT(0x800100C380000000), ANTLR3_UINT64_LIT(0x0001000000000001), ANTLR3_UINT64_LIT(0x0100000091000000), ANTLR3_UINT64_LIT(0x0000400000010058), ANTLR3_UINT64_LIT(0x0000200000001000), ANTLR3_UINT64_LIT(0x0000010000070160), ANTLR3_UINT64_LIT(0x0000000084000008), ANTLR3_UINT64_LIT(0x0000000014000038), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_variable_declaration13261	= { FOLLOW_identifier_list_in_variable_declaration13261_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_in_variable_declaration13263  */
static	ANTLR3_BITWORD FOLLOW_data_type_in_variable_declaration13263_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000040000) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_in_variable_declaration13263	= { FOLLOW_data_type_in_variable_declaration13263_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_variable_declaration13266  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_variable_declaration13266_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_variable_declaration13266	= { FOLLOW_COLLATE_SYMBOL_in_variable_declaration13266_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_or_default_in_variable_declaration13268  */
static	ANTLR3_BITWORD FOLLOW_collation_name_or_default_in_variable_declaration13268_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_or_default_in_variable_declaration13268	= { FOLLOW_collation_name_or_default_in_variable_declaration13268_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_variable_declaration13273  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_variable_declaration13273_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_variable_declaration13273	= { FOLLOW_DEFAULT_SYMBOL_in_variable_declaration13273_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_variable_declaration13275  */
static	ANTLR3_BITWORD FOLLOW_expression_in_variable_declaration13275_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_variable_declaration13275	= { FOLLOW_expression_in_variable_declaration13275_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_condition_declaration13286  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_condition_declaration13286_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_condition_declaration13286	= { FOLLOW_identifier_in_condition_declaration13286_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONDITION_SYMBOL_in_condition_declaration13288  */
static	ANTLR3_BITWORD FOLLOW_CONDITION_SYMBOL_in_condition_declaration13288_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONDITION_SYMBOL_in_condition_declaration13288	= { FOLLOW_CONDITION_SYMBOL_in_condition_declaration13288_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_condition_declaration13290  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_condition_declaration13290_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_condition_declaration13290	= { FOLLOW_FOR_SYMBOL_in_condition_declaration13290_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sp_condition_in_condition_declaration13292  */
static	ANTLR3_BITWORD FOLLOW_sp_condition_in_condition_declaration13292_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_sp_condition_in_condition_declaration13292	= { FOLLOW_sp_condition_in_condition_declaration13292_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_sp_condition13301  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_sp_condition13301_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_sp_condition13301	= { FOLLOW_ulong_number_in_sp_condition13301_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sqlstate_in_sp_condition13306  */
static	ANTLR3_BITWORD FOLLOW_sqlstate_in_sp_condition13306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_sqlstate_in_sp_condition13306	= { FOLLOW_sqlstate_in_sp_condition13306_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQLSTATE_SYMBOL_in_sqlstate13315  */
static	ANTLR3_BITWORD FOLLOW_SQLSTATE_SYMBOL_in_sqlstate13315_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQLSTATE_SYMBOL_in_sqlstate13315	= { FOLLOW_SQLSTATE_SYMBOL_in_sqlstate13315_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALUE_SYMBOL_in_sqlstate13317  */
static	ANTLR3_BITWORD FOLLOW_VALUE_SYMBOL_in_sqlstate13317_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VALUE_SYMBOL_in_sqlstate13317	= { FOLLOW_VALUE_SYMBOL_in_sqlstate13317_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_sqlstate13320  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_sqlstate13320_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_sqlstate13320	= { FOLLOW_string_literal_in_sqlstate13320_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_handler_declaration13329  */
static	ANTLR3_BITWORD FOLLOW_set_in_handler_declaration13329_bits[]	= { ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_handler_declaration13329	= { FOLLOW_set_in_handler_declaration13329_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HANDLER_SYMBOL_in_handler_declaration13341  */
static	ANTLR3_BITWORD FOLLOW_HANDLER_SYMBOL_in_handler_declaration13341_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_HANDLER_SYMBOL_in_handler_declaration13341	= { FOLLOW_HANDLER_SYMBOL_in_handler_declaration13341_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_handler_declaration13345  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_handler_declaration13345_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000C00000000), ANTLR3_UINT64_LIT(0x0000001C00000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011810100800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_handler_declaration13345	= { FOLLOW_FOR_SYMBOL_in_handler_declaration13345_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_handler_condition_in_handler_declaration13347  */
static	ANTLR3_BITWORD FOLLOW_handler_condition_in_handler_declaration13347_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100020) };
static  ANTLR3_BITSET_LIST FOLLOW_handler_condition_in_handler_declaration13347	= { FOLLOW_handler_condition_in_handler_declaration13347_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_handler_declaration13350  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_handler_declaration13350_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000C00000000), ANTLR3_UINT64_LIT(0x0000001C00000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0400011810100800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_handler_declaration13350	= { FOLLOW_COMMA_SYMBOL_in_handler_declaration13350_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_handler_condition_in_handler_declaration13352  */
static	ANTLR3_BITWORD FOLLOW_handler_condition_in_handler_declaration13352_bits[]	= { ANTLR3_UINT64_LIT(0xFCB81F489636B580), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x11FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0238000004025400), ANTLR3_UINT64_LIT(0x0508001020300100), ANTLR3_UINT64_LIT(0x4001400000000211), ANTLR3_UINT64_LIT(0x0000000034106BC0), ANTLR3_UINT64_LIT(0x00100010000A8000), ANTLR3_UINT64_LIT(0x0000001000100020) };
static  ANTLR3_BITSET_LIST FOLLOW_handler_condition_in_handler_declaration13352	= { FOLLOW_handler_condition_in_handler_declaration13352_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_compound_statement_in_handler_declaration13356  */
static	ANTLR3_BITWORD FOLLOW_compound_statement_in_handler_declaration13356_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_compound_statement_in_handler_declaration13356	= { FOLLOW_compound_statement_in_handler_declaration13356_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sp_condition_in_handler_condition13365  */
static	ANTLR3_BITWORD FOLLOW_sp_condition_in_handler_condition13365_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_sp_condition_in_handler_condition13365	= { FOLLOW_sp_condition_in_handler_condition13365_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_handler_condition13370  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_handler_condition13370_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_handler_condition13370	= { FOLLOW_identifier_in_handler_condition13370_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQLWARNING_SYMBOL_in_handler_condition13375  */
static	ANTLR3_BITWORD FOLLOW_SQLWARNING_SYMBOL_in_handler_condition13375_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQLWARNING_SYMBOL_in_handler_condition13375	= { FOLLOW_SQLWARNING_SYMBOL_in_handler_condition13375_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_not_rule_in_handler_condition13380  */
static	ANTLR3_BITWORD FOLLOW_not_rule_in_handler_condition13380_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_not_rule_in_handler_condition13380	= { FOLLOW_not_rule_in_handler_condition13380_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOUND_SYMBOL_in_handler_condition13382  */
static	ANTLR3_BITWORD FOLLOW_FOUND_SYMBOL_in_handler_condition13382_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_FOUND_SYMBOL_in_handler_condition13382	= { FOLLOW_FOUND_SYMBOL_in_handler_condition13382_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQLEXCEPTION_SYMBOL_in_handler_condition13387  */
static	ANTLR3_BITWORD FOLLOW_SQLEXCEPTION_SYMBOL_in_handler_condition13387_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SQLEXCEPTION_SYMBOL_in_handler_condition13387	= { FOLLOW_SQLEXCEPTION_SYMBOL_in_handler_condition13387_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_cursor_declaration13396  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_cursor_declaration13396_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_cursor_declaration13396	= { FOLLOW_identifier_in_cursor_declaration13396_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURSOR_SYMBOL_in_cursor_declaration13398  */
static	ANTLR3_BITWORD FOLLOW_CURSOR_SYMBOL_in_cursor_declaration13398_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000008000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURSOR_SYMBOL_in_cursor_declaration13398	= { FOLLOW_CURSOR_SYMBOL_in_cursor_declaration13398_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FOR_SYMBOL_in_cursor_declaration13400  */
static	ANTLR3_BITWORD FOLLOW_FOR_SYMBOL_in_cursor_declaration13400_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FOR_SYMBOL_in_cursor_declaration13400	= { FOLLOW_FOR_SYMBOL_in_cursor_declaration13400_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_select_statement_in_cursor_declaration13402  */
static	ANTLR3_BITWORD FOLLOW_select_statement_in_cursor_declaration13402_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_select_statement_in_cursor_declaration13402	= { FOLLOW_select_statement_in_cursor_declaration13402_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ITERATE_SYMBOL_in_iterate_statement13411  */
static	ANTLR3_BITWORD FOLLOW_ITERATE_SYMBOL_in_iterate_statement13411_bits[]	= { ANTLR3_UINT64_LIT(0xF800000000000000), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFDFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_ITERATE_SYMBOL_in_iterate_statement13411	= { FOLLOW_ITERATE_SYMBOL_in_iterate_statement13411_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_identifier_in_iterate_statement13413  */
static	ANTLR3_BITWORD FOLLOW_label_identifier_in_iterate_statement13413_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_label_identifier_in_iterate_statement13413	= { FOLLOW_label_identifier_in_iterate_statement13413_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LEAVE_SYMBOL_in_leave_statement13422  */
static	ANTLR3_BITWORD FOLLOW_LEAVE_SYMBOL_in_leave_statement13422_bits[]	= { ANTLR3_UINT64_LIT(0xF800000000000000), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFDFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_LEAVE_SYMBOL_in_leave_statement13422	= { FOLLOW_LEAVE_SYMBOL_in_leave_statement13422_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_label_identifier_in_leave_statement13424  */
static	ANTLR3_BITWORD FOLLOW_label_identifier_in_leave_statement13424_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_label_identifier_in_leave_statement13424	= { FOLLOW_label_identifier_in_leave_statement13424_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GET_SYMBOL_in_get_diagnostics13433  */
static	ANTLR3_BITWORD FOLLOW_GET_SYMBOL_in_get_diagnostics13433_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080100000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_GET_SYMBOL_in_get_diagnostics13433	= { FOLLOW_GET_SYMBOL_in_get_diagnostics13433_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURRENT_SYMBOL_in_get_diagnostics13440  */
static	ANTLR3_BITWORD FOLLOW_CURRENT_SYMBOL_in_get_diagnostics13440_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURRENT_SYMBOL_in_get_diagnostics13440	= { FOLLOW_CURRENT_SYMBOL_in_get_diagnostics13440_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STACKED_SYMBOL_in_get_diagnostics13448  */
static	ANTLR3_BITWORD FOLLOW_STACKED_SYMBOL_in_get_diagnostics13448_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STACKED_SYMBOL_in_get_diagnostics13448	= { FOLLOW_STACKED_SYMBOL_in_get_diagnostics13448_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DIAGNOSTICS_SYMBOL_in_get_diagnostics13454  */
static	ANTLR3_BITWORD FOLLOW_DIAGNOSTICS_SYMBOL_in_get_diagnostics13454_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000200000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DIAGNOSTICS_SYMBOL_in_get_diagnostics13454	= { FOLLOW_DIAGNOSTICS_SYMBOL_in_get_diagnostics13454_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_statement_information_item_in_get_diagnostics13461  */
static	ANTLR3_BITWORD FOLLOW_statement_information_item_in_get_diagnostics13461_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_statement_information_item_in_get_diagnostics13461	= { FOLLOW_statement_information_item_in_get_diagnostics13461_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_get_diagnostics13464  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_get_diagnostics13464_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_get_diagnostics13464	= { FOLLOW_COMMA_SYMBOL_in_get_diagnostics13464_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_statement_information_item_in_get_diagnostics13466  */
static	ANTLR3_BITWORD FOLLOW_statement_information_item_in_get_diagnostics13466_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_statement_information_item_in_get_diagnostics13466	= { FOLLOW_statement_information_item_in_get_diagnostics13466_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONDITION_SYMBOL_in_get_diagnostics13474  */
static	ANTLR3_BITWORD FOLLOW_CONDITION_SYMBOL_in_get_diagnostics13474_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000C040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0037000000000400), ANTLR3_UINT64_LIT(0x0400011810100800), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONDITION_SYMBOL_in_get_diagnostics13474	= { FOLLOW_CONDITION_SYMBOL_in_get_diagnostics13474_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_allowed_expression_in_get_diagnostics13476  */
static	ANTLR3_BITWORD FOLLOW_signal_allowed_expression_in_get_diagnostics13476_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_allowed_expression_in_get_diagnostics13476	= { FOLLOW_signal_allowed_expression_in_get_diagnostics13476_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_condition_information_item_in_get_diagnostics13478  */
static	ANTLR3_BITWORD FOLLOW_condition_information_item_in_get_diagnostics13478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_condition_information_item_in_get_diagnostics13478	= { FOLLOW_condition_information_item_in_get_diagnostics13478_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_get_diagnostics13481  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_get_diagnostics13481_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0017000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_get_diagnostics13481	= { FOLLOW_COMMA_SYMBOL_in_get_diagnostics13481_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_condition_information_item_in_get_diagnostics13483  */
static	ANTLR3_BITWORD FOLLOW_condition_information_item_in_get_diagnostics13483_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_condition_information_item_in_get_diagnostics13483	= { FOLLOW_condition_information_item_in_get_diagnostics13483_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_literal_in_signal_allowed_expression13498  */
static	ANTLR3_BITWORD FOLLOW_literal_in_signal_allowed_expression13498_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_literal_in_signal_allowed_expression13498	= { FOLLOW_literal_in_signal_allowed_expression13498_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_in_signal_allowed_expression13503  */
static	ANTLR3_BITWORD FOLLOW_variable_in_signal_allowed_expression13503_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_in_signal_allowed_expression13503	= { FOLLOW_variable_in_signal_allowed_expression13503_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_signal_allowed_expression13508  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_signal_allowed_expression13508_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_signal_allowed_expression13508	= { FOLLOW_qualified_identifier_in_signal_allowed_expression13508_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_in_statement_information_item13518  */
static	ANTLR3_BITWORD FOLLOW_variable_in_statement_information_item13518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_in_statement_information_item13518	= { FOLLOW_variable_in_statement_information_item13518_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_statement_information_item13522  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_statement_information_item13522_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_statement_information_item13522	= { FOLLOW_identifier_in_statement_information_item13522_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_statement_information_item13525  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_statement_information_item13525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000), ANTLR3_UINT64_LIT(0x0000020000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_statement_information_item13525	= { FOLLOW_EQUAL_OPERATOR_in_statement_information_item13525_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_statement_information_item13527  */
static	ANTLR3_BITWORD FOLLOW_set_in_statement_information_item13527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_statement_information_item13527	= { FOLLOW_set_in_statement_information_item13527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_variable_in_condition_information_item13543  */
static	ANTLR3_BITWORD FOLLOW_variable_in_condition_information_item13543_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_variable_in_condition_information_item13543	= { FOLLOW_variable_in_condition_information_item13543_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_condition_information_item13547  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_condition_information_item13547_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_condition_information_item13547	= { FOLLOW_identifier_in_condition_information_item13547_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_condition_information_item13550  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_condition_information_item13550_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000020E004410000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800800000), ANTLR3_UINT64_LIT(0x0000400800000000), ANTLR3_UINT64_LIT(0x0000000000040200) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_condition_information_item13550	= { FOLLOW_EQUAL_OPERATOR_in_condition_information_item13550_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_name_in_condition_information_item13553  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_name_in_condition_information_item13553_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_name_in_condition_information_item13553	= { FOLLOW_signal_information_item_name_in_condition_information_item13553_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RETURNED_SQLSTATE_SYMBOL_in_condition_information_item13557  */
static	ANTLR3_BITWORD FOLLOW_RETURNED_SQLSTATE_SYMBOL_in_condition_information_item13557_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_RETURNED_SQLSTATE_SYMBOL_in_condition_information_item13557	= { FOLLOW_RETURNED_SQLSTATE_SYMBOL_in_condition_information_item13557_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SIGNAL_SYMBOL_in_signal_statement13631  */
static	ANTLR3_BITWORD FOLLOW_SIGNAL_SYMBOL_in_signal_statement13631_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SIGNAL_SYMBOL_in_signal_statement13631	= { FOLLOW_SIGNAL_SYMBOL_in_signal_statement13631_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_signal_statement13634  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_signal_statement13634_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_signal_statement13634	= { FOLLOW_identifier_in_signal_statement13634_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_sqlstate_in_signal_statement13638  */
static	ANTLR3_BITWORD FOLLOW_sqlstate_in_signal_statement13638_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_sqlstate_in_signal_statement13638	= { FOLLOW_sqlstate_in_signal_statement13638_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_signal_statement13642  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_signal_statement13642_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000020E004410000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800800000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000040200) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_signal_statement13642	= { FOLLOW_SET_SYMBOL_in_signal_statement13642_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_in_signal_statement13644  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_in_signal_statement13644_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_in_signal_statement13644	= { FOLLOW_signal_information_item_in_signal_statement13644_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_signal_statement13647  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_signal_statement13647_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000020E004410000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800800000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000040200) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_signal_statement13647	= { FOLLOW_COMMA_SYMBOL_in_signal_statement13647_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_in_signal_statement13649  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_in_signal_statement13649_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_in_signal_statement13649	= { FOLLOW_signal_information_item_in_signal_statement13649_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RESIGNAL_SYMBOL_in_resignal_statement13662  */
static	ANTLR3_BITWORD FOLLOW_RESIGNAL_SYMBOL_in_resignal_statement13662_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000804000000) };
static  ANTLR3_BITSET_LIST FOLLOW_RESIGNAL_SYMBOL_in_resignal_statement13662	= { FOLLOW_RESIGNAL_SYMBOL_in_resignal_statement13662_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SQLSTATE_SYMBOL_in_resignal_statement13665  */
static	ANTLR3_BITWORD FOLLOW_SQLSTATE_SYMBOL_in_resignal_statement13665_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SQLSTATE_SYMBOL_in_resignal_statement13665	= { FOLLOW_SQLSTATE_SYMBOL_in_resignal_statement13665_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALUE_SYMBOL_in_resignal_statement13667  */
static	ANTLR3_BITWORD FOLLOW_VALUE_SYMBOL_in_resignal_statement13667_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_VALUE_SYMBOL_in_resignal_statement13667	= { FOLLOW_VALUE_SYMBOL_in_resignal_statement13667_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_resignal_statement13670  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_resignal_statement13670_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_resignal_statement13670	= { FOLLOW_text_or_identifier_in_resignal_statement13670_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_resignal_statement13677  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_resignal_statement13677_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000020E004410000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800800000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000040200) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_resignal_statement13677	= { FOLLOW_SET_SYMBOL_in_resignal_statement13677_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_in_resignal_statement13679  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_in_resignal_statement13679_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_in_resignal_statement13679	= { FOLLOW_signal_information_item_in_resignal_statement13679_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_resignal_statement13682  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_resignal_statement13682_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000020E004410000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800800000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000040200) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_resignal_statement13682	= { FOLLOW_COMMA_SYMBOL_in_resignal_statement13682_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_in_resignal_statement13684  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_in_resignal_statement13684_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_in_resignal_statement13684	= { FOLLOW_signal_information_item_in_resignal_statement13684_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_information_item_name_in_signal_information_item13697  */
static	ANTLR3_BITWORD FOLLOW_signal_information_item_name_in_signal_information_item13697_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_information_item_name_in_signal_information_item13697	= { FOLLOW_signal_information_item_name_in_signal_information_item13697_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_signal_information_item13699  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_signal_information_item13699_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000C040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0037000000000400), ANTLR3_UINT64_LIT(0x0400011810100800), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_signal_information_item13699	= { FOLLOW_EQUAL_OPERATOR_in_signal_information_item13699_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signal_allowed_expression_in_signal_information_item13701  */
static	ANTLR3_BITWORD FOLLOW_signal_allowed_expression_in_signal_information_item13701_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_signal_allowed_expression_in_signal_information_item13701	= { FOLLOW_signal_allowed_expression_in_signal_information_item13701_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_SYMBOL_in_cursor_open13710  */
static	ANTLR3_BITWORD FOLLOW_OPEN_SYMBOL_in_cursor_open13710_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_SYMBOL_in_cursor_open13710	= { FOLLOW_OPEN_SYMBOL_in_cursor_open13710_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_cursor_open13712  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_cursor_open13712_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_cursor_open13712	= { FOLLOW_identifier_in_cursor_open13712_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_SYMBOL_in_cursor_close13721  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_SYMBOL_in_cursor_close13721_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_SYMBOL_in_cursor_close13721	= { FOLLOW_CLOSE_SYMBOL_in_cursor_close13721_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_cursor_close13723  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_cursor_close13723_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_cursor_close13723	= { FOLLOW_identifier_in_cursor_close13723_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FETCH_SYMBOL_in_cursor_fetch13732  */
static	ANTLR3_BITWORD FOLLOW_FETCH_SYMBOL_in_cursor_fetch13732_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_FETCH_SYMBOL_in_cursor_fetch13732	= { FOLLOW_FETCH_SYMBOL_in_cursor_fetch13732_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_cursor_fetch13734  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_cursor_fetch13734_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_cursor_fetch13734	= { FOLLOW_identifier_in_cursor_fetch13734_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INTO_SYMBOL_in_cursor_fetch13736  */
static	ANTLR3_BITWORD FOLLOW_INTO_SYMBOL_in_cursor_fetch13736_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_INTO_SYMBOL_in_cursor_fetch13736	= { FOLLOW_INTO_SYMBOL_in_cursor_fetch13736_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_cursor_fetch13738  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_cursor_fetch13738_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_cursor_fetch13738	= { FOLLOW_identifier_list_in_cursor_fetch13738_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_SYMBOL_in_schedule13750  */
static	ANTLR3_BITWORD FOLLOW_AT_SYMBOL_in_schedule13750_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_SYMBOL_in_schedule13750	= { FOLLOW_AT_SYMBOL_in_schedule13750_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_schedule13752  */
static	ANTLR3_BITWORD FOLLOW_expression_in_schedule13752_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_schedule13752	= { FOLLOW_expression_in_schedule13752_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EVERY_SYMBOL_in_schedule13757  */
static	ANTLR3_BITWORD FOLLOW_EVERY_SYMBOL_in_schedule13757_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_EVERY_SYMBOL_in_schedule13757	= { FOLLOW_EVERY_SYMBOL_in_schedule13757_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_interval_in_schedule13759  */
static	ANTLR3_BITWORD FOLLOW_interval_in_schedule13759_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_interval_in_schedule13759	= { FOLLOW_interval_in_schedule13759_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STARTS_SYMBOL_in_schedule13762  */
static	ANTLR3_BITWORD FOLLOW_STARTS_SYMBOL_in_schedule13762_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_STARTS_SYMBOL_in_schedule13762	= { FOLLOW_STARTS_SYMBOL_in_schedule13762_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_schedule13764  */
static	ANTLR3_BITWORD FOLLOW_expression_in_schedule13764_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_schedule13764	= { FOLLOW_expression_in_schedule13764_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENDS_SYMBOL_in_schedule13769  */
static	ANTLR3_BITWORD FOLLOW_ENDS_SYMBOL_in_schedule13769_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ENDS_SYMBOL_in_schedule13769	= { FOLLOW_ENDS_SYMBOL_in_schedule13769_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_schedule13771  */
static	ANTLR3_BITWORD FOLLOW_expression_in_schedule13771_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_schedule13771	= { FOLLOW_expression_in_schedule13771_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_database_option13782  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_database_option13782_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_database_option13782	= { FOLLOW_DEFAULT_SYMBOL_in_database_option13782_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_database_option13792  */
static	ANTLR3_BITWORD FOLLOW_charset_in_database_option13792_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_database_option13792	= { FOLLOW_charset_in_database_option13792_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_database_option13794  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_database_option13794_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_database_option13794	= { FOLLOW_EQUAL_OPERATOR_in_database_option13794_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_or_default_in_database_option13797  */
static	ANTLR3_BITWORD FOLLOW_charset_name_or_default_in_database_option13797_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_or_default_in_database_option13797	= { FOLLOW_charset_name_or_default_in_database_option13797_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_database_option13804  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_database_option13804_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_database_option13804	= { FOLLOW_COLLATE_SYMBOL_in_database_option13804_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_database_option13806  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_database_option13806_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_database_option13806	= { FOLLOW_EQUAL_OPERATOR_in_database_option13806_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_or_default_in_database_option13809  */
static	ANTLR3_BITWORD FOLLOW_collation_name_or_default_in_database_option13809_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_or_default_in_database_option13809	= { FOLLOW_collation_name_or_default_in_database_option13809_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_spec_in_column_definition13822  */
static	ANTLR3_BITWORD FOLLOW_field_spec_in_column_definition13822_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_spec_in_column_definition13822	= { FOLLOW_field_spec_in_column_definition13822_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_definition_in_column_definition13825  */
static	ANTLR3_BITWORD FOLLOW_reference_definition_in_column_definition13825_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_definition_in_column_definition13825	= { FOLLOW_reference_definition_in_column_definition13825_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHECK_SYMBOL_in_column_definition13830  */
static	ANTLR3_BITWORD FOLLOW_CHECK_SYMBOL_in_column_definition13830_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHECK_SYMBOL_in_column_definition13830	= { FOLLOW_CHECK_SYMBOL_in_column_definition13830_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_column_definition13832  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_column_definition13832_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_column_definition13832	= { FOLLOW_OPEN_PAR_SYMBOL_in_column_definition13832_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_column_definition13834  */
static	ANTLR3_BITWORD FOLLOW_expression_in_column_definition13834_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_column_definition13834	= { FOLLOW_expression_in_column_definition13834_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_column_definition13836  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_column_definition13836_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_column_definition13836	= { FOLLOW_CLOSE_PAR_SYMBOL_in_column_definition13836_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_name_in_field_spec13848  */
static	ANTLR3_BITWORD FOLLOW_column_name_in_field_spec13848_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000003400), ANTLR3_UINT64_LIT(0x0004100003400001), ANTLR3_UINT64_LIT(0x800100C380000000), ANTLR3_UINT64_LIT(0x0001000000000001), ANTLR3_UINT64_LIT(0x0100000091000000), ANTLR3_UINT64_LIT(0x0000400000010058), ANTLR3_UINT64_LIT(0x0000200000001000), ANTLR3_UINT64_LIT(0x0000010000070160), ANTLR3_UINT64_LIT(0x0000000084000008), ANTLR3_UINT64_LIT(0x0000000014000038), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_name_in_field_spec13848	= { FOLLOW_column_name_in_field_spec13848_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_def_in_field_spec13850  */
static	ANTLR3_BITWORD FOLLOW_field_def_in_field_spec13850_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_def_in_field_spec13850	= { FOLLOW_field_def_in_field_spec13850_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_in_field_def13859  */
static	ANTLR3_BITWORD FOLLOW_data_type_in_field_def13859_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x0000000008000020), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0002000000040000), ANTLR3_UINT64_LIT(0x0080000000080000), ANTLR3_UINT64_LIT(0x100010C800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_in_field_def13859	= { FOLLOW_data_type_in_field_def13859_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_def_tail_in_field_def13861  */
static	ANTLR3_BITWORD FOLLOW_field_def_tail_in_field_def13861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_def_tail_in_field_def13861	= { FOLLOW_field_def_tail_in_field_def13861_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_attribute_in_field_def_tail13881  */
static	ANTLR3_BITWORD FOLLOW_attribute_in_field_def_tail13881_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000008000020), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x0002000000040000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x100010C800000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_attribute_in_field_def_tail13881	= { FOLLOW_attribute_in_field_def_tail13881_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_field_def_tail13890  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_field_def_tail13890_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_field_def_tail13890	= { FOLLOW_COLLATE_SYMBOL_in_field_def_tail13890_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_in_field_def_tail13892  */
static	ANTLR3_BITWORD FOLLOW_collation_name_in_field_def_tail13892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000080000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_in_field_def_tail13892	= { FOLLOW_collation_name_in_field_def_tail13892_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_GENERATED_SYMBOL_in_field_def_tail13897  */
static	ANTLR3_BITWORD FOLLOW_GENERATED_SYMBOL_in_field_def_tail13897_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000040) };
static  ANTLR3_BITSET_LIST FOLLOW_GENERATED_SYMBOL_in_field_def_tail13897	= { FOLLOW_GENERATED_SYMBOL_in_field_def_tail13897_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALWAYS_SYMBOL_in_field_def_tail13899  */
static	ANTLR3_BITWORD FOLLOW_ALWAYS_SYMBOL_in_field_def_tail13899_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALWAYS_SYMBOL_in_field_def_tail13899	= { FOLLOW_ALWAYS_SYMBOL_in_field_def_tail13899_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_field_def_tail13903  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_field_def_tail13903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_field_def_tail13903	= { FOLLOW_AS_SYMBOL_in_field_def_tail13903_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_field_def_tail13907  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_field_def_tail13907_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_field_def_tail13907	= { FOLLOW_OPEN_PAR_SYMBOL_in_field_def_tail13907_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_field_def_tail13909  */
static	ANTLR3_BITWORD FOLLOW_expression_in_field_def_tail13909_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_field_def_tail13909	= { FOLLOW_expression_in_field_def_tail13909_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_field_def_tail13911  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_field_def_tail13911_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x1000008C00000000), ANTLR3_UINT64_LIT(0x0100000000000000), ANTLR3_UINT64_LIT(0x0000000200004000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_field_def_tail13911	= { FOLLOW_CLOSE_PAR_SYMBOL_in_field_def_tail13911_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_gcol_attribute_in_field_def_tail13922  */
static	ANTLR3_BITWORD FOLLOW_gcol_attribute_in_field_def_tail13922_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x1000008C00000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_gcol_attribute_in_field_def_tail13922	= { FOLLOW_gcol_attribute_in_field_def_tail13922_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOT_SYMBOL_in_attribute13932  */
static	ANTLR3_BITWORD FOLLOW_NOT_SYMBOL_in_attribute13932_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000000C000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOT_SYMBOL_in_attribute13932	= { FOLLOW_NOT_SYMBOL_in_attribute13932_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_null_literal_in_attribute13935  */
static	ANTLR3_BITWORD FOLLOW_null_literal_in_attribute13935_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_null_literal_in_attribute13935	= { FOLLOW_null_literal_in_attribute13935_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_attribute13940  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_attribute13940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000090000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000080), ANTLR3_UINT64_LIT(0x000000D040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000400), ANTLR3_UINT64_LIT(0x0C00010810100800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_attribute13940	= { FOLLOW_DEFAULT_SYMBOL_in_attribute13940_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_signed_literal_in_attribute13943  */
static	ANTLR3_BITWORD FOLLOW_signed_literal_in_attribute13943_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_signed_literal_in_attribute13943	= { FOLLOW_signed_literal_in_attribute13943_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOW_SYMBOL_in_attribute13947  */
static	ANTLR3_BITWORD FOLLOW_NOW_SYMBOL_in_attribute13947_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOW_SYMBOL_in_attribute13947	= { FOLLOW_NOW_SYMBOL_in_attribute13947_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_attribute13949  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_attribute13949_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_attribute13949	= { FOLLOW_time_function_parameters_in_attribute13949_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_attribute13956  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_attribute13956_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_attribute13956	= { FOLLOW_ON_SYMBOL_in_attribute13956_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_attribute13958  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_attribute13958_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000001000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_attribute13958	= { FOLLOW_UPDATE_SYMBOL_in_attribute13958_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NOW_SYMBOL_in_attribute13960  */
static	ANTLR3_BITWORD FOLLOW_NOW_SYMBOL_in_attribute13960_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NOW_SYMBOL_in_attribute13960	= { FOLLOW_NOW_SYMBOL_in_attribute13960_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_time_function_parameters_in_attribute13962  */
static	ANTLR3_BITWORD FOLLOW_time_function_parameters_in_attribute13962_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_time_function_parameters_in_attribute13962	= { FOLLOW_time_function_parameters_in_attribute13962_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTO_INCREMENT_SYMBOL_in_attribute13968  */
static	ANTLR3_BITWORD FOLLOW_AUTO_INCREMENT_SYMBOL_in_attribute13968_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTO_INCREMENT_SYMBOL_in_attribute13968	= { FOLLOW_AUTO_INCREMENT_SYMBOL_in_attribute13968_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERIAL_SYMBOL_in_attribute13973  */
static	ANTLR3_BITWORD FOLLOW_SERIAL_SYMBOL_in_attribute13973_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SERIAL_SYMBOL_in_attribute13973	= { FOLLOW_SERIAL_SYMBOL_in_attribute13973_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_attribute13975  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_attribute13975_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_attribute13975	= { FOLLOW_DEFAULT_SYMBOL_in_attribute13975_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALUE_SYMBOL_in_attribute13977  */
static	ANTLR3_BITWORD FOLLOW_VALUE_SYMBOL_in_attribute13977_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VALUE_SYMBOL_in_attribute13977	= { FOLLOW_VALUE_SYMBOL_in_attribute13977_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNIQUE_SYMBOL_in_attribute13984  */
static	ANTLR3_BITWORD FOLLOW_UNIQUE_SYMBOL_in_attribute13984_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_UNIQUE_SYMBOL_in_attribute13984	= { FOLLOW_UNIQUE_SYMBOL_in_attribute13984_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_attribute13989  */
static	ANTLR3_BITWORD FOLLOW_set_in_attribute13989_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_attribute13989	= { FOLLOW_set_in_attribute13989_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_attribute13997  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_attribute13997_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_attribute13997	= { FOLLOW_KEY_SYMBOL_in_attribute13997_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_attribute14002  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_attribute14002_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_attribute14002	= { FOLLOW_KEY_SYMBOL_in_attribute14002_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_attribute14007  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_attribute14007_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_attribute14007	= { FOLLOW_COMMENT_SYMBOL_in_attribute14007_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_attribute14009  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_attribute14009_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_attribute14009	= { FOLLOW_string_literal_in_attribute14009_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_attribute14014  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_attribute14014_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_attribute14014	= { FOLLOW_COLLATE_SYMBOL_in_attribute14014_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_in_attribute14016  */
static	ANTLR3_BITWORD FOLLOW_collation_name_in_attribute14016_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_in_attribute14016	= { FOLLOW_collation_name_in_attribute14016_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMN_FORMAT_SYMBOL_in_attribute14021  */
static	ANTLR3_BITWORD FOLLOW_COLUMN_FORMAT_SYMBOL_in_attribute14021_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMN_FORMAT_SYMBOL_in_attribute14021	= { FOLLOW_COLUMN_FORMAT_SYMBOL_in_attribute14021_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_attribute14023  */
static	ANTLR3_BITWORD FOLLOW_set_in_attribute14023_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_attribute14023	= { FOLLOW_set_in_attribute14023_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_attribute14038  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_attribute14038_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_attribute14038	= { FOLLOW_STORAGE_SYMBOL_in_attribute14038_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_attribute14040  */
static	ANTLR3_BITWORD FOLLOW_set_in_attribute14040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_attribute14040	= { FOLLOW_set_in_attribute14040_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNIQUE_SYMBOL_in_gcol_attribute14060  */
static	ANTLR3_BITWORD FOLLOW_UNIQUE_SYMBOL_in_gcol_attribute14060_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNIQUE_SYMBOL_in_gcol_attribute14060	= { FOLLOW_UNIQUE_SYMBOL_in_gcol_attribute14060_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_gcol_attribute14075  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_gcol_attribute14075_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_gcol_attribute14075	= { FOLLOW_KEY_SYMBOL_in_gcol_attribute14075_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_gcol_attribute14083  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_gcol_attribute14083_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_gcol_attribute14083	= { FOLLOW_COMMENT_SYMBOL_in_gcol_attribute14083_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_gcol_attribute14085  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_gcol_attribute14085_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_gcol_attribute14085	= { FOLLOW_text_string_in_gcol_attribute14085_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_not_rule_in_gcol_attribute14090  */
static	ANTLR3_BITWORD FOLLOW_not_rule_in_gcol_attribute14090_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000008000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_not_rule_in_gcol_attribute14090	= { FOLLOW_not_rule_in_gcol_attribute14090_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NULL_SYMBOL_in_gcol_attribute14093  */
static	ANTLR3_BITWORD FOLLOW_NULL_SYMBOL_in_gcol_attribute14093_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NULL_SYMBOL_in_gcol_attribute14093	= { FOLLOW_NULL_SYMBOL_in_gcol_attribute14093_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRIMARY_SYMBOL_in_gcol_attribute14098  */
static	ANTLR3_BITWORD FOLLOW_PRIMARY_SYMBOL_in_gcol_attribute14098_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PRIMARY_SYMBOL_in_gcol_attribute14098	= { FOLLOW_PRIMARY_SYMBOL_in_gcol_attribute14098_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_gcol_attribute14101  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_gcol_attribute14101_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_gcol_attribute14101	= { FOLLOW_KEY_SYMBOL_in_gcol_attribute14101_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REFERENCES_SYMBOL_in_reference_definition14113  */
static	ANTLR3_BITWORD FOLLOW_REFERENCES_SYMBOL_in_reference_definition14113_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_REFERENCES_SYMBOL_in_reference_definition14113	= { FOLLOW_REFERENCES_SYMBOL_in_reference_definition14113_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_reference_definition14115  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_reference_definition14115_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000500000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_reference_definition14115	= { FOLLOW_table_ref_in_reference_definition14115_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_columns_in_reference_definition14117  */
static	ANTLR3_BITWORD FOLLOW_index_columns_in_reference_definition14117_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_index_columns_in_reference_definition14117	= { FOLLOW_index_columns_in_reference_definition14117_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MATCH_SYMBOL_in_reference_definition14129  */
static	ANTLR3_BITWORD FOLLOW_MATCH_SYMBOL_in_reference_definition14129_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MATCH_SYMBOL_in_reference_definition14129	= { FOLLOW_MATCH_SYMBOL_in_reference_definition14129_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_reference_definition14131  */
static	ANTLR3_BITWORD FOLLOW_set_in_reference_definition14131_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_reference_definition14131	= { FOLLOW_set_in_reference_definition14131_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_reference_definition14152  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_reference_definition14152_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_reference_definition14152	= { FOLLOW_ON_SYMBOL_in_reference_definition14152_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DELETE_SYMBOL_in_reference_definition14154  */
static	ANTLR3_BITWORD FOLLOW_DELETE_SYMBOL_in_reference_definition14154_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004001000) };
static  ANTLR3_BITSET_LIST FOLLOW_DELETE_SYMBOL_in_reference_definition14154	= { FOLLOW_DELETE_SYMBOL_in_reference_definition14154_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_option_in_reference_definition14156  */
static	ANTLR3_BITWORD FOLLOW_reference_option_in_reference_definition14156_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_option_in_reference_definition14156	= { FOLLOW_reference_option_in_reference_definition14156_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_reference_definition14159  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_reference_definition14159_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_reference_definition14159	= { FOLLOW_ON_SYMBOL_in_reference_definition14159_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_reference_definition14161  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_reference_definition14161_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004001000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_reference_definition14161	= { FOLLOW_UPDATE_SYMBOL_in_reference_definition14161_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_option_in_reference_definition14163  */
static	ANTLR3_BITWORD FOLLOW_reference_option_in_reference_definition14163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_option_in_reference_definition14163	= { FOLLOW_reference_option_in_reference_definition14163_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_reference_definition14172  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_reference_definition14172_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_reference_definition14172	= { FOLLOW_ON_SYMBOL_in_reference_definition14172_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UPDATE_SYMBOL_in_reference_definition14174  */
static	ANTLR3_BITWORD FOLLOW_UPDATE_SYMBOL_in_reference_definition14174_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004001000) };
static  ANTLR3_BITSET_LIST FOLLOW_UPDATE_SYMBOL_in_reference_definition14174	= { FOLLOW_UPDATE_SYMBOL_in_reference_definition14174_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_option_in_reference_definition14176  */
static	ANTLR3_BITWORD FOLLOW_reference_option_in_reference_definition14176_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_option_in_reference_definition14176	= { FOLLOW_reference_option_in_reference_definition14176_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ON_SYMBOL_in_reference_definition14179  */
static	ANTLR3_BITWORD FOLLOW_ON_SYMBOL_in_reference_definition14179_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ON_SYMBOL_in_reference_definition14179	= { FOLLOW_ON_SYMBOL_in_reference_definition14179_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DELETE_SYMBOL_in_reference_definition14181  */
static	ANTLR3_BITWORD FOLLOW_DELETE_SYMBOL_in_reference_definition14181_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004001000) };
static  ANTLR3_BITSET_LIST FOLLOW_DELETE_SYMBOL_in_reference_definition14181	= { FOLLOW_DELETE_SYMBOL_in_reference_definition14181_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_reference_option_in_reference_definition14183  */
static	ANTLR3_BITWORD FOLLOW_reference_option_in_reference_definition14183_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_reference_option_in_reference_definition14183	= { FOLLOW_reference_option_in_reference_definition14183_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_index_columns14199  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_index_columns14199_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_index_columns14199	= { FOLLOW_OPEN_PAR_SYMBOL_in_index_columns14199_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_column_in_index_columns14201  */
static	ANTLR3_BITWORD FOLLOW_index_column_in_index_columns14201_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_column_in_index_columns14201	= { FOLLOW_index_column_in_index_columns14201_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_index_columns14204  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_index_columns14204_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_index_columns14204	= { FOLLOW_COMMA_SYMBOL_in_index_columns14204_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_column_in_index_columns14206  */
static	ANTLR3_BITWORD FOLLOW_index_column_in_index_columns14206_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_index_column_in_index_columns14206	= { FOLLOW_index_column_in_index_columns14206_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_index_columns14210  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_index_columns14210_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_index_columns14210	= { FOLLOW_CLOSE_PAR_SYMBOL_in_index_columns14210_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_index_column14219  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_index_column14219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_index_column14219	= { FOLLOW_identifier_in_index_column14219_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_index_column14221  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_index_column14221_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_index_column14221	= { FOLLOW_field_length_in_index_column14221_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_direction_in_index_column14224  */
static	ANTLR3_BITWORD FOLLOW_direction_in_index_column14224_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_direction_in_index_column14224	= { FOLLOW_direction_in_index_column14224_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_index_type14234  */
static	ANTLR3_BITWORD FOLLOW_set_in_index_type14234_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000004000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_index_type14234	= { FOLLOW_set_in_index_type14234_bits, 5	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_index_type14242  */
static	ANTLR3_BITWORD FOLLOW_set_in_index_type14242_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_index_type14242	= { FOLLOW_set_in_index_type14242_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_index_type_in_normal_index_option14261  */
static	ANTLR3_BITWORD FOLLOW_index_type_in_normal_index_option14261_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_index_type_in_normal_index_option14261	= { FOLLOW_index_type_in_normal_index_option14261_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_key_option_in_normal_index_option14266  */
static	ANTLR3_BITWORD FOLLOW_all_key_option_in_normal_index_option14266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_key_option_in_normal_index_option14266	= { FOLLOW_all_key_option_in_normal_index_option14266_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_fulltext_index_option14275  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_fulltext_index_option14275_bits[]	= { ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_fulltext_index_option14275	= { FOLLOW_WITH_SYMBOL_in_fulltext_index_option14275_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARSER_SYMBOL_in_fulltext_index_option14277  */
static	ANTLR3_BITWORD FOLLOW_PARSER_SYMBOL_in_fulltext_index_option14277_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARSER_SYMBOL_in_fulltext_index_option14277	= { FOLLOW_PARSER_SYMBOL_in_fulltext_index_option14277_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_fulltext_index_option14279  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_fulltext_index_option14279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_fulltext_index_option14279	= { FOLLOW_identifier_in_fulltext_index_option14279_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_key_option_in_fulltext_index_option14284  */
static	ANTLR3_BITWORD FOLLOW_all_key_option_in_fulltext_index_option14284_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_key_option_in_fulltext_index_option14284	= { FOLLOW_all_key_option_in_fulltext_index_option14284_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_all_key_option_in_spatial_index_option14293  */
static	ANTLR3_BITWORD FOLLOW_all_key_option_in_spatial_index_option14293_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_all_key_option_in_spatial_index_option14293	= { FOLLOW_all_key_option_in_spatial_index_option14293_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_all_key_option14302  */
static	ANTLR3_BITWORD FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_all_key_option14302_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_all_key_option14302	= { FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_all_key_option14302_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_all_key_option14304  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_all_key_option14304_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_all_key_option14304	= { FOLLOW_EQUAL_OPERATOR_in_all_key_option14304_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_all_key_option14307  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_all_key_option14307_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_all_key_option14307	= { FOLLOW_ulong_number_in_all_key_option14307_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_all_key_option14314  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_all_key_option14314_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_all_key_option14314	= { FOLLOW_COMMENT_SYMBOL_in_all_key_option14314_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_all_key_option14316  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_all_key_option14316_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_all_key_option14316	= { FOLLOW_string_literal_in_all_key_option14316_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_RESTRICT_SYMBOL_in_reference_option14325  */
static	ANTLR3_BITWORD FOLLOW_RESTRICT_SYMBOL_in_reference_option14325_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_RESTRICT_SYMBOL_in_reference_option14325	= { FOLLOW_RESTRICT_SYMBOL_in_reference_option14325_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CASCADE_SYMBOL_in_reference_option14329  */
static	ANTLR3_BITWORD FOLLOW_CASCADE_SYMBOL_in_reference_option14329_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CASCADE_SYMBOL_in_reference_option14329	= { FOLLOW_CASCADE_SYMBOL_in_reference_option14329_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_reference_option14333  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_reference_option14333_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x000000C000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_reference_option14333	= { FOLLOW_SET_SYMBOL_in_reference_option14333_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_null_literal_in_reference_option14335  */
static	ANTLR3_BITWORD FOLLOW_null_literal_in_reference_option14335_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_null_literal_in_reference_option14335	= { FOLLOW_null_literal_in_reference_option14335_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_SYMBOL_in_reference_option14339  */
static	ANTLR3_BITWORD FOLLOW_NO_SYMBOL_in_reference_option14339_bits[]	= { ANTLR3_UINT64_LIT(0x0800000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_SYMBOL_in_reference_option14339	= { FOLLOW_NO_SYMBOL_in_reference_option14339_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ACTION_SYMBOL_in_reference_option14341  */
static	ANTLR3_BITWORD FOLLOW_ACTION_SYMBOL_in_reference_option14341_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ACTION_SYMBOL_in_reference_option14341	= { FOLLOW_ACTION_SYMBOL_in_reference_option14341_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_elements_in_data_type14350  */
static	ANTLR3_BITWORD FOLLOW_data_type_elements_in_data_type14350_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_elements_in_data_type14350	= { FOLLOW_data_type_elements_in_data_type14350_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_elements_in_data_type_definition14360  */
static	ANTLR3_BITWORD FOLLOW_data_type_elements_in_data_type_definition14360_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_elements_in_data_type_definition14360	= { FOLLOW_data_type_elements_in_data_type_definition14360_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EOF_in_data_type_definition14362  */
static	ANTLR3_BITWORD FOLLOW_EOF_in_data_type_definition14362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EOF_in_data_type_definition14362	= { FOLLOW_EOF_in_data_type_definition14362_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_integer_type_in_data_type_elements14371  */
static	ANTLR3_BITWORD FOLLOW_integer_type_in_data_type_elements14371_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_integer_type_in_data_type_elements14371	= { FOLLOW_integer_type_in_data_type_elements14371_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14373  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14373_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14373	= { FOLLOW_field_length_in_data_type_elements14373_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14376  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14376_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14376	= { FOLLOW_field_options_in_data_type_elements14376_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_literal_in_data_type_elements14383  */
static	ANTLR3_BITWORD FOLLOW_real_literal_in_data_type_elements14383_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_literal_in_data_type_elements14383	= { FOLLOW_real_literal_in_data_type_elements14383_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_precision_in_data_type_elements14385  */
static	ANTLR3_BITWORD FOLLOW_precision_in_data_type_elements14385_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_precision_in_data_type_elements14385	= { FOLLOW_precision_in_data_type_elements14385_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14388  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14388_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14388	= { FOLLOW_field_options_in_data_type_elements14388_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FLOAT_SYMBOL_in_data_type_elements14394  */
static	ANTLR3_BITWORD FOLLOW_FLOAT_SYMBOL_in_data_type_elements14394_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_FLOAT_SYMBOL_in_data_type_elements14394	= { FOLLOW_FLOAT_SYMBOL_in_data_type_elements14394_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_float_options_in_data_type_elements14396  */
static	ANTLR3_BITWORD FOLLOW_float_options_in_data_type_elements14396_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_float_options_in_data_type_elements14396	= { FOLLOW_float_options_in_data_type_elements14396_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14399  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14399_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14399	= { FOLLOW_field_options_in_data_type_elements14399_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BIT_SYMBOL_in_data_type_elements14407  */
static	ANTLR3_BITWORD FOLLOW_BIT_SYMBOL_in_data_type_elements14407_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BIT_SYMBOL_in_data_type_elements14407	= { FOLLOW_BIT_SYMBOL_in_data_type_elements14407_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14409  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14409_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14409	= { FOLLOW_field_length_in_data_type_elements14409_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BOOL_SYMBOL_in_data_type_elements14415  */
static	ANTLR3_BITWORD FOLLOW_BOOL_SYMBOL_in_data_type_elements14415_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BOOL_SYMBOL_in_data_type_elements14415	= { FOLLOW_BOOL_SYMBOL_in_data_type_elements14415_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BOOLEAN_SYMBOL_in_data_type_elements14420  */
static	ANTLR3_BITWORD FOLLOW_BOOLEAN_SYMBOL_in_data_type_elements14420_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BOOLEAN_SYMBOL_in_data_type_elements14420	= { FOLLOW_BOOLEAN_SYMBOL_in_data_type_elements14420_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_data_type_elements14427  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_data_type_elements14427_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_data_type_elements14427	= { FOLLOW_CHAR_SYMBOL_in_data_type_elements14427_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14429  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14429_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14429	= { FOLLOW_field_length_in_data_type_elements14429_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14432  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14432_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14432	= { FOLLOW_string_binary_in_data_type_elements14432_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_nchar_literal_in_data_type_elements14438  */
static	ANTLR3_BITWORD FOLLOW_nchar_literal_in_data_type_elements14438_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_nchar_literal_in_data_type_elements14438	= { FOLLOW_nchar_literal_in_data_type_elements14438_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14440  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14440_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14440	= { FOLLOW_field_length_in_data_type_elements14440_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_data_type_elements14443  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_data_type_elements14443_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_data_type_elements14443	= { FOLLOW_BINARY_SYMBOL_in_data_type_elements14443_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_data_type_elements14449  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_data_type_elements14449_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_data_type_elements14449	= { FOLLOW_BINARY_SYMBOL_in_data_type_elements14449_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14451  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14451_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14451	= { FOLLOW_field_length_in_data_type_elements14451_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_varchar_literal_in_data_type_elements14457  */
static	ANTLR3_BITWORD FOLLOW_varchar_literal_in_data_type_elements14457_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_varchar_literal_in_data_type_elements14457	= { FOLLOW_varchar_literal_in_data_type_elements14457_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14459  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14459_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14459	= { FOLLOW_field_length_in_data_type_elements14459_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14461  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14461_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14461	= { FOLLOW_string_binary_in_data_type_elements14461_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_nvarchar_literal_in_data_type_elements14467  */
static	ANTLR3_BITWORD FOLLOW_nvarchar_literal_in_data_type_elements14467_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_nvarchar_literal_in_data_type_elements14467	= { FOLLOW_nvarchar_literal_in_data_type_elements14467_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14469  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14469_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14469	= { FOLLOW_field_length_in_data_type_elements14469_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_data_type_elements14471  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_data_type_elements14471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_data_type_elements14471	= { FOLLOW_BINARY_SYMBOL_in_data_type_elements14471_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14478  */
static	ANTLR3_BITWORD FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14478_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14478	= { FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14478_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14480  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14480_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14480	= { FOLLOW_field_length_in_data_type_elements14480_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_YEAR_SYMBOL_in_data_type_elements14486  */
static	ANTLR3_BITWORD FOLLOW_YEAR_SYMBOL_in_data_type_elements14486_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_YEAR_SYMBOL_in_data_type_elements14486	= { FOLLOW_YEAR_SYMBOL_in_data_type_elements14486_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14488  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14488_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14488	= { FOLLOW_field_length_in_data_type_elements14488_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14491  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14491_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14491	= { FOLLOW_field_options_in_data_type_elements14491_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATE_SYMBOL_in_data_type_elements14497  */
static	ANTLR3_BITWORD FOLLOW_DATE_SYMBOL_in_data_type_elements14497_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DATE_SYMBOL_in_data_type_elements14497	= { FOLLOW_DATE_SYMBOL_in_data_type_elements14497_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIME_SYMBOL_in_data_type_elements14502  */
static	ANTLR3_BITWORD FOLLOW_TIME_SYMBOL_in_data_type_elements14502_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIME_SYMBOL_in_data_type_elements14502	= { FOLLOW_TIME_SYMBOL_in_data_type_elements14502_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_type_datetime_precision_in_data_type_elements14504  */
static	ANTLR3_BITWORD FOLLOW_type_datetime_precision_in_data_type_elements14504_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_type_datetime_precision_in_data_type_elements14504	= { FOLLOW_type_datetime_precision_in_data_type_elements14504_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIMESTAMP_SYMBOL_in_data_type_elements14510  */
static	ANTLR3_BITWORD FOLLOW_TIMESTAMP_SYMBOL_in_data_type_elements14510_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIMESTAMP_SYMBOL_in_data_type_elements14510	= { FOLLOW_TIMESTAMP_SYMBOL_in_data_type_elements14510_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_type_datetime_precision_in_data_type_elements14512  */
static	ANTLR3_BITWORD FOLLOW_type_datetime_precision_in_data_type_elements14512_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_type_datetime_precision_in_data_type_elements14512	= { FOLLOW_type_datetime_precision_in_data_type_elements14512_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATETIME_SYMBOL_in_data_type_elements14518  */
static	ANTLR3_BITWORD FOLLOW_DATETIME_SYMBOL_in_data_type_elements14518_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATETIME_SYMBOL_in_data_type_elements14518	= { FOLLOW_DATETIME_SYMBOL_in_data_type_elements14518_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_type_datetime_precision_in_data_type_elements14520  */
static	ANTLR3_BITWORD FOLLOW_type_datetime_precision_in_data_type_elements14520_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_type_datetime_precision_in_data_type_elements14520	= { FOLLOW_type_datetime_precision_in_data_type_elements14520_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TINYBLOB_SYMBOL_in_data_type_elements14527  */
static	ANTLR3_BITWORD FOLLOW_TINYBLOB_SYMBOL_in_data_type_elements14527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_TINYBLOB_SYMBOL_in_data_type_elements14527	= { FOLLOW_TINYBLOB_SYMBOL_in_data_type_elements14527_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BLOB_SYMBOL_in_data_type_elements14532  */
static	ANTLR3_BITWORD FOLLOW_BLOB_SYMBOL_in_data_type_elements14532_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BLOB_SYMBOL_in_data_type_elements14532	= { FOLLOW_BLOB_SYMBOL_in_data_type_elements14532_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14534  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14534_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14534	= { FOLLOW_field_length_in_data_type_elements14534_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MEDIUMBLOB_SYMBOL_in_data_type_elements14540  */
static	ANTLR3_BITWORD FOLLOW_MEDIUMBLOB_SYMBOL_in_data_type_elements14540_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MEDIUMBLOB_SYMBOL_in_data_type_elements14540	= { FOLLOW_MEDIUMBLOB_SYMBOL_in_data_type_elements14540_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LONGBLOB_SYMBOL_in_data_type_elements14545  */
static	ANTLR3_BITWORD FOLLOW_LONGBLOB_SYMBOL_in_data_type_elements14545_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LONGBLOB_SYMBOL_in_data_type_elements14545	= { FOLLOW_LONGBLOB_SYMBOL_in_data_type_elements14545_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LONG_SYMBOL_in_data_type_elements14550  */
static	ANTLR3_BITWORD FOLLOW_LONG_SYMBOL_in_data_type_elements14550_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000004000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LONG_SYMBOL_in_data_type_elements14550	= { FOLLOW_LONG_SYMBOL_in_data_type_elements14550_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14552  */
static	ANTLR3_BITWORD FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14552_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14552	= { FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14552_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LONG_SYMBOL_in_data_type_elements14557  */
static	ANTLR3_BITWORD FOLLOW_LONG_SYMBOL_in_data_type_elements14557_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LONG_SYMBOL_in_data_type_elements14557	= { FOLLOW_LONG_SYMBOL_in_data_type_elements14557_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_varchar_literal_in_data_type_elements14559  */
static	ANTLR3_BITWORD FOLLOW_varchar_literal_in_data_type_elements14559_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_varchar_literal_in_data_type_elements14559	= { FOLLOW_varchar_literal_in_data_type_elements14559_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14562  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14562_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14562	= { FOLLOW_string_binary_in_data_type_elements14562_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TINYTEXT_SYMBOL_in_data_type_elements14569  */
static	ANTLR3_BITWORD FOLLOW_TINYTEXT_SYMBOL_in_data_type_elements14569_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_TINYTEXT_SYMBOL_in_data_type_elements14569	= { FOLLOW_TINYTEXT_SYMBOL_in_data_type_elements14569_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14571  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14571_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14571	= { FOLLOW_string_binary_in_data_type_elements14571_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TEXT_SYMBOL_in_data_type_elements14577  */
static	ANTLR3_BITWORD FOLLOW_TEXT_SYMBOL_in_data_type_elements14577_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TEXT_SYMBOL_in_data_type_elements14577	= { FOLLOW_TEXT_SYMBOL_in_data_type_elements14577_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_length_in_data_type_elements14579  */
static	ANTLR3_BITWORD FOLLOW_field_length_in_data_type_elements14579_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_field_length_in_data_type_elements14579	= { FOLLOW_field_length_in_data_type_elements14579_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14582  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14582_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14582	= { FOLLOW_string_binary_in_data_type_elements14582_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MEDIUMTEXT_SYMBOL_in_data_type_elements14588  */
static	ANTLR3_BITWORD FOLLOW_MEDIUMTEXT_SYMBOL_in_data_type_elements14588_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_MEDIUMTEXT_SYMBOL_in_data_type_elements14588	= { FOLLOW_MEDIUMTEXT_SYMBOL_in_data_type_elements14588_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14590  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14590_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14590	= { FOLLOW_string_binary_in_data_type_elements14590_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LONGTEXT_SYMBOL_in_data_type_elements14596  */
static	ANTLR3_BITWORD FOLLOW_LONGTEXT_SYMBOL_in_data_type_elements14596_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_LONGTEXT_SYMBOL_in_data_type_elements14596	= { FOLLOW_LONGTEXT_SYMBOL_in_data_type_elements14596_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14598  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14598_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14598	= { FOLLOW_string_binary_in_data_type_elements14598_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DECIMAL_SYMBOL_in_data_type_elements14606  */
static	ANTLR3_BITWORD FOLLOW_DECIMAL_SYMBOL_in_data_type_elements14606_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_DECIMAL_SYMBOL_in_data_type_elements14606	= { FOLLOW_DECIMAL_SYMBOL_in_data_type_elements14606_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_float_options_in_data_type_elements14608  */
static	ANTLR3_BITWORD FOLLOW_float_options_in_data_type_elements14608_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_float_options_in_data_type_elements14608	= { FOLLOW_float_options_in_data_type_elements14608_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14611  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14611_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14611	= { FOLLOW_field_options_in_data_type_elements14611_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NUMERIC_SYMBOL_in_data_type_elements14617  */
static	ANTLR3_BITWORD FOLLOW_NUMERIC_SYMBOL_in_data_type_elements14617_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_NUMERIC_SYMBOL_in_data_type_elements14617	= { FOLLOW_NUMERIC_SYMBOL_in_data_type_elements14617_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_float_options_in_data_type_elements14619  */
static	ANTLR3_BITWORD FOLLOW_float_options_in_data_type_elements14619_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_float_options_in_data_type_elements14619	= { FOLLOW_float_options_in_data_type_elements14619_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14622  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14622_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14622	= { FOLLOW_field_options_in_data_type_elements14622_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_FIXED_SYMBOL_in_data_type_elements14628  */
static	ANTLR3_BITWORD FOLLOW_FIXED_SYMBOL_in_data_type_elements14628_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_FIXED_SYMBOL_in_data_type_elements14628	= { FOLLOW_FIXED_SYMBOL_in_data_type_elements14628_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_float_options_in_data_type_elements14630  */
static	ANTLR3_BITWORD FOLLOW_float_options_in_data_type_elements14630_bits[]	= { ANTLR3_UINT64_LIT(0x0000800000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000100000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_float_options_in_data_type_elements14630	= { FOLLOW_float_options_in_data_type_elements14630_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_options_in_data_type_elements14633  */
static	ANTLR3_BITWORD FOLLOW_field_options_in_data_type_elements14633_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_options_in_data_type_elements14633	= { FOLLOW_field_options_in_data_type_elements14633_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENUM_SYMBOL_in_data_type_elements14641  */
static	ANTLR3_BITWORD FOLLOW_ENUM_SYMBOL_in_data_type_elements14641_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENUM_SYMBOL_in_data_type_elements14641	= { FOLLOW_ENUM_SYMBOL_in_data_type_elements14641_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_list_in_data_type_elements14643  */
static	ANTLR3_BITWORD FOLLOW_string_list_in_data_type_elements14643_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_string_list_in_data_type_elements14643	= { FOLLOW_string_list_in_data_type_elements14643_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14645  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14645_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14645	= { FOLLOW_string_binary_in_data_type_elements14645_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SET_SYMBOL_in_data_type_elements14651  */
static	ANTLR3_BITWORD FOLLOW_SET_SYMBOL_in_data_type_elements14651_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SET_SYMBOL_in_data_type_elements14651	= { FOLLOW_SET_SYMBOL_in_data_type_elements14651_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_list_in_data_type_elements14653  */
static	ANTLR3_BITWORD FOLLOW_string_list_in_data_type_elements14653_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000A22), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010010) };
static  ANTLR3_BITSET_LIST FOLLOW_string_list_in_data_type_elements14653	= { FOLLOW_string_list_in_data_type_elements14653_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_binary_in_data_type_elements14655  */
static	ANTLR3_BITWORD FOLLOW_string_binary_in_data_type_elements14655_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_binary_in_data_type_elements14655	= { FOLLOW_string_binary_in_data_type_elements14655_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SERIAL_SYMBOL_in_data_type_elements14661  */
static	ANTLR3_BITWORD FOLLOW_SERIAL_SYMBOL_in_data_type_elements14661_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SERIAL_SYMBOL_in_data_type_elements14661	= { FOLLOW_SERIAL_SYMBOL_in_data_type_elements14661_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_JSON_SYMBOL_in_data_type_elements14668  */
static	ANTLR3_BITWORD FOLLOW_JSON_SYMBOL_in_data_type_elements14668_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_JSON_SYMBOL_in_data_type_elements14668	= { FOLLOW_JSON_SYMBOL_in_data_type_elements14668_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_spatial_type_in_data_type_elements14673  */
static	ANTLR3_BITWORD FOLLOW_spatial_type_in_data_type_elements14673_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_spatial_type_in_data_type_elements14673	= { FOLLOW_spatial_type_in_data_type_elements14673_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_field_length14682  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_field_length14682_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010000000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_field_length14682	= { FOLLOW_OPEN_PAR_SYMBOL_in_field_length14682_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulonglong_number_in_field_length14685  */
static	ANTLR3_BITWORD FOLLOW_real_ulonglong_number_in_field_length14685_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulonglong_number_in_field_length14685	= { FOLLOW_real_ulonglong_number_in_field_length14685_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DECIMAL_NUMBER_in_field_length14689  */
static	ANTLR3_BITWORD FOLLOW_DECIMAL_NUMBER_in_field_length14689_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DECIMAL_NUMBER_in_field_length14689	= { FOLLOW_DECIMAL_NUMBER_in_field_length14689_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_field_length14692  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_field_length14692_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_field_length14692	= { FOLLOW_CLOSE_PAR_SYMBOL_in_field_length14692_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ascii_in_string_binary14721  */
static	ANTLR3_BITWORD FOLLOW_ascii_in_string_binary14721_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ascii_in_string_binary14721	= { FOLLOW_ascii_in_string_binary14721_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_unicode_in_string_binary14726  */
static	ANTLR3_BITWORD FOLLOW_unicode_in_string_binary14726_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_unicode_in_string_binary14726	= { FOLLOW_unicode_in_string_binary14726_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BYTE_SYMBOL_in_string_binary14731  */
static	ANTLR3_BITWORD FOLLOW_BYTE_SYMBOL_in_string_binary14731_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BYTE_SYMBOL_in_string_binary14731	= { FOLLOW_BYTE_SYMBOL_in_string_binary14731_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_string_binary14736  */
static	ANTLR3_BITWORD FOLLOW_charset_in_string_binary14736_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_string_binary14736	= { FOLLOW_charset_in_string_binary14736_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_string_binary14738  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_string_binary14738_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_string_binary14738	= { FOLLOW_charset_name_in_string_binary14738_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_string_binary14740  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_string_binary14740_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_string_binary14740	= { FOLLOW_BINARY_SYMBOL_in_string_binary14740_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_string_binary14746  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_string_binary14746_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000802), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_string_binary14746	= { FOLLOW_BINARY_SYMBOL_in_string_binary14746_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_string_binary14749  */
static	ANTLR3_BITWORD FOLLOW_charset_in_string_binary14749_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_string_binary14749	= { FOLLOW_charset_in_string_binary14749_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_string_binary14751  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_string_binary14751_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_string_binary14751	= { FOLLOW_charset_name_in_string_binary14751_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASCII_SYMBOL_in_ascii14762  */
static	ANTLR3_BITWORD FOLLOW_ASCII_SYMBOL_in_ascii14762_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_ASCII_SYMBOL_in_ascii14762	= { FOLLOW_ASCII_SYMBOL_in_ascii14762_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_ascii14764  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_ascii14764_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_ascii14764	= { FOLLOW_BINARY_SYMBOL_in_ascii14764_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_ascii14772  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_ascii14772_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_ascii14772	= { FOLLOW_BINARY_SYMBOL_in_ascii14772_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ASCII_SYMBOL_in_ascii14774  */
static	ANTLR3_BITWORD FOLLOW_ASCII_SYMBOL_in_ascii14774_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ASCII_SYMBOL_in_ascii14774	= { FOLLOW_ASCII_SYMBOL_in_ascii14774_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNICODE_SYMBOL_in_unicode14783  */
static	ANTLR3_BITWORD FOLLOW_UNICODE_SYMBOL_in_unicode14783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000010) };
static  ANTLR3_BITSET_LIST FOLLOW_UNICODE_SYMBOL_in_unicode14783	= { FOLLOW_UNICODE_SYMBOL_in_unicode14783_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_unicode14785  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_unicode14785_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_unicode14785	= { FOLLOW_BINARY_SYMBOL_in_unicode14785_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_unicode14793  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_unicode14793_bits[]	= { ANTLR3_UINT64_LIT(0x0040000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_unicode14793	= { FOLLOW_BINARY_SYMBOL_in_unicode14793_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNICODE_SYMBOL_in_unicode14795  */
static	ANTLR3_BITWORD FOLLOW_UNICODE_SYMBOL_in_unicode14795_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_UNICODE_SYMBOL_in_unicode14795	= { FOLLOW_UNICODE_SYMBOL_in_unicode14795_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_type_datetime_precision14806  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_type_datetime_precision14806_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_type_datetime_precision14806	= { FOLLOW_OPEN_PAR_SYMBOL_in_type_datetime_precision14806_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_type_datetime_precision14808  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_type_datetime_precision14808_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_type_datetime_precision14808	= { FOLLOW_INT_NUMBER_in_type_datetime_precision14808_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_type_datetime_precision14810  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_type_datetime_precision14810_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_type_datetime_precision14810	= { FOLLOW_CLOSE_PAR_SYMBOL_in_type_datetime_precision14810_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_charset_name14819  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_charset_name14819_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_charset_name14819	= { FOLLOW_text_or_identifier_in_charset_name14819_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_charset_name14824  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_charset_name14824_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_charset_name14824	= { FOLLOW_BINARY_SYMBOL_in_charset_name14824_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_in_charset_name_or_default14833  */
static	ANTLR3_BITWORD FOLLOW_charset_name_in_charset_name_or_default14833_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_in_charset_name_or_default14833	= { FOLLOW_charset_name_in_charset_name_or_default14833_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_charset_name_or_default14838  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_charset_name_or_default14838_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_charset_name_or_default14838	= { FOLLOW_DEFAULT_SYMBOL_in_charset_name_or_default14838_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_collation_name14847  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_collation_name14847_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_collation_name14847	= { FOLLOW_text_or_identifier_in_collation_name14847_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_in_collation_name_or_default14856  */
static	ANTLR3_BITWORD FOLLOW_collation_name_in_collation_name_or_default14856_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_in_collation_name_or_default14856	= { FOLLOW_collation_name_in_collation_name_or_default14856_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_collation_name_or_default14861  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_collation_name_or_default14861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_collation_name_or_default14861	= { FOLLOW_DEFAULT_SYMBOL_in_collation_name_or_default14861_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_option_in_alter_table_options14914  */
static	ANTLR3_BITWORD FOLLOW_create_table_option_in_alter_table_options14914_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000005802), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_option_in_alter_table_options14914	= { FOLLOW_create_table_option_in_alter_table_options14914_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_option_in_create_table_options14924  */
static	ANTLR3_BITWORD FOLLOW_create_table_option_in_create_table_options14924_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000005802), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_option_in_create_table_options14924	= { FOLLOW_create_table_option_in_create_table_options14924_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_create_table_options14927  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_create_table_options14927_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000005800), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_create_table_options14927	= { FOLLOW_COMMA_SYMBOL_in_create_table_options14927_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_create_table_option_in_create_table_options14930  */
static	ANTLR3_BITWORD FOLLOW_create_table_option_in_create_table_options14930_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000005802), ANTLR3_UINT64_LIT(0x0020400A000000A0), ANTLR3_UINT64_LIT(0x0100280000000002), ANTLR3_UINT64_LIT(0x0420000008000000), ANTLR3_UINT64_LIT(0x0000040000000000), ANTLR3_UINT64_LIT(0x00000002003000B8), ANTLR3_UINT64_LIT(0x0002000000050000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_create_table_option_in_create_table_options14930	= { FOLLOW_create_table_option_in_create_table_options14930_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_create_table_option14943  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_create_table_option14943_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_create_table_option14943	= { FOLLOW_ENGINE_SYMBOL_in_create_table_option14943_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TYPE_SYMBOL_in_create_table_option14949  */
static	ANTLR3_BITWORD FOLLOW_TYPE_SYMBOL_in_create_table_option14949_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_TYPE_SYMBOL_in_create_table_option14949	= { FOLLOW_TYPE_SYMBOL_in_create_table_option14949_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option14952  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option14952_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option14952	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option14952_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_create_table_option14955  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_create_table_option14955_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_create_table_option14955	= { FOLLOW_engine_ref_in_create_table_option14955_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAX_ROWS_SYMBOL_in_create_table_option14960  */
static	ANTLR3_BITWORD FOLLOW_MAX_ROWS_SYMBOL_in_create_table_option14960_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAX_ROWS_SYMBOL_in_create_table_option14960	= { FOLLOW_MAX_ROWS_SYMBOL_in_create_table_option14960_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option14962  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option14962_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option14962	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option14962_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulonglong_number_in_create_table_option14965  */
static	ANTLR3_BITWORD FOLLOW_ulonglong_number_in_create_table_option14965_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulonglong_number_in_create_table_option14965	= { FOLLOW_ulonglong_number_in_create_table_option14965_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MIN_ROWS_SYMBOL_in_create_table_option14970  */
static	ANTLR3_BITWORD FOLLOW_MIN_ROWS_SYMBOL_in_create_table_option14970_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MIN_ROWS_SYMBOL_in_create_table_option14970	= { FOLLOW_MIN_ROWS_SYMBOL_in_create_table_option14970_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option14972  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option14972_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option14972	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option14972_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulonglong_number_in_create_table_option14975  */
static	ANTLR3_BITWORD FOLLOW_ulonglong_number_in_create_table_option14975_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulonglong_number_in_create_table_option14975	= { FOLLOW_ulonglong_number_in_create_table_option14975_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AVG_ROW_LENGTH_SYMBOL_in_create_table_option14980  */
static	ANTLR3_BITWORD FOLLOW_AVG_ROW_LENGTH_SYMBOL_in_create_table_option14980_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AVG_ROW_LENGTH_SYMBOL_in_create_table_option14980	= { FOLLOW_AVG_ROW_LENGTH_SYMBOL_in_create_table_option14980_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option14982  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option14982_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option14982	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option14982_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option14985  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option14985_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option14985	= { FOLLOW_ulong_number_in_create_table_option14985_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_create_table_option14990  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_create_table_option14990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_create_table_option14990	= { FOLLOW_PASSWORD_SYMBOL_in_create_table_option14990_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option14992  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option14992_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option14992	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option14992_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option14995  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option14995_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option14995	= { FOLLOW_text_string_in_create_table_option14995_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_create_table_option15000  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_create_table_option15000_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_create_table_option15000	= { FOLLOW_COMMENT_SYMBOL_in_create_table_option15000_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15002  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15002_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15002	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15002_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option15005  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option15005_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option15005	= { FOLLOW_text_string_in_create_table_option15005_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMPRESSION_SYMBOL_in_create_table_option15010  */
static	ANTLR3_BITWORD FOLLOW_COMPRESSION_SYMBOL_in_create_table_option15010_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMPRESSION_SYMBOL_in_create_table_option15010	= { FOLLOW_COMPRESSION_SYMBOL_in_create_table_option15010_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15012  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15012_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15012	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15012_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option15015  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option15015_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option15015	= { FOLLOW_text_string_in_create_table_option15015_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AUTO_INCREMENT_SYMBOL_in_create_table_option15020  */
static	ANTLR3_BITWORD FOLLOW_AUTO_INCREMENT_SYMBOL_in_create_table_option15020_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AUTO_INCREMENT_SYMBOL_in_create_table_option15020	= { FOLLOW_AUTO_INCREMENT_SYMBOL_in_create_table_option15020_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15022  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15022_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010010000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15022	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15022_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulonglong_number_in_create_table_option15025  */
static	ANTLR3_BITWORD FOLLOW_ulonglong_number_in_create_table_option15025_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulonglong_number_in_create_table_option15025	= { FOLLOW_ulonglong_number_in_create_table_option15025_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PACK_KEYS_SYMBOL_in_create_table_option15030  */
static	ANTLR3_BITWORD FOLLOW_PACK_KEYS_SYMBOL_in_create_table_option15030_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PACK_KEYS_SYMBOL_in_create_table_option15030	= { FOLLOW_PACK_KEYS_SYMBOL_in_create_table_option15030_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15032  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15032_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15032	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15032_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option15036  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option15036_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option15036	= { FOLLOW_ulong_number_in_create_table_option15036_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_create_table_option15040  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_create_table_option15040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_create_table_option15040	= { FOLLOW_DEFAULT_SYMBOL_in_create_table_option15040_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_table_option15048  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_table_option15048_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_table_option15048	= { FOLLOW_set_in_create_table_option15048_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15062  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15062_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15062	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15062_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option15066  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option15066_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option15066	= { FOLLOW_ulong_number_in_create_table_option15066_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_create_table_option15070  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_create_table_option15070_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_create_table_option15070	= { FOLLOW_DEFAULT_SYMBOL_in_create_table_option15070_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_table_option15076  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_table_option15076_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_table_option15076	= { FOLLOW_set_in_create_table_option15076_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15084  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15084_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15084	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15084_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option15087  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option15087_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option15087	= { FOLLOW_ulong_number_in_create_table_option15087_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DELAY_KEY_WRITE_SYMBOL_in_create_table_option15092  */
static	ANTLR3_BITWORD FOLLOW_DELAY_KEY_WRITE_SYMBOL_in_create_table_option15092_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DELAY_KEY_WRITE_SYMBOL_in_create_table_option15092	= { FOLLOW_DELAY_KEY_WRITE_SYMBOL_in_create_table_option15092_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15094  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15094_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15094	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15094_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option15097  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option15097_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option15097	= { FOLLOW_ulong_number_in_create_table_option15097_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ROW_FORMAT_SYMBOL_in_create_table_option15102  */
static	ANTLR3_BITWORD FOLLOW_ROW_FORMAT_SYMBOL_in_create_table_option15102_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000140000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_ROW_FORMAT_SYMBOL_in_create_table_option15102	= { FOLLOW_ROW_FORMAT_SYMBOL_in_create_table_option15102_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15104  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15104_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000140000000), ANTLR3_UINT64_LIT(0x0000000000400000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0002000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15104	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15104_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_table_option15107  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_table_option15107_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_table_option15107	= { FOLLOW_set_in_create_table_option15107_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNION_SYMBOL_in_create_table_option15134  */
static	ANTLR3_BITWORD FOLLOW_UNION_SYMBOL_in_create_table_option15134_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNION_SYMBOL_in_create_table_option15134	= { FOLLOW_UNION_SYMBOL_in_create_table_option15134_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15136  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15136_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15136	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15136_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_create_table_option15139  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_create_table_option15139_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_create_table_option15139	= { FOLLOW_OPEN_PAR_SYMBOL_in_create_table_option15139_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_list_in_create_table_option15141  */
static	ANTLR3_BITWORD FOLLOW_table_ref_list_in_create_table_option15141_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_list_in_create_table_option15141	= { FOLLOW_table_ref_list_in_create_table_option15141_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_option15143  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_option15143_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_option15143	= { FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_option15143_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_create_table_option15148  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_create_table_option15148_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000800), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000050000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_create_table_option15148	= { FOLLOW_DEFAULT_SYMBOL_in_create_table_option15148_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLLATE_SYMBOL_in_create_table_option15158  */
static	ANTLR3_BITWORD FOLLOW_COLLATE_SYMBOL_in_create_table_option15158_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLLATE_SYMBOL_in_create_table_option15158	= { FOLLOW_COLLATE_SYMBOL_in_create_table_option15158_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15160  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15160_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15160	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15160_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_collation_name_or_default_in_create_table_option15163  */
static	ANTLR3_BITWORD FOLLOW_collation_name_or_default_in_create_table_option15163_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_collation_name_or_default_in_create_table_option15163	= { FOLLOW_collation_name_or_default_in_create_table_option15163_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_create_table_option15170  */
static	ANTLR3_BITWORD FOLLOW_charset_in_create_table_option15170_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_create_table_option15170	= { FOLLOW_charset_in_create_table_option15170_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15172  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15172_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15172	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15172_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_name_or_default_in_create_table_option15175  */
static	ANTLR3_BITWORD FOLLOW_charset_name_or_default_in_create_table_option15175_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_name_or_default_in_create_table_option15175	= { FOLLOW_charset_name_or_default_in_create_table_option15175_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INSERT_METHOD_SYMBOL_in_create_table_option15185  */
static	ANTLR3_BITWORD FOLLOW_INSERT_METHOD_SYMBOL_in_create_table_option15185_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000200000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_INSERT_METHOD_SYMBOL_in_create_table_option15185	= { FOLLOW_INSERT_METHOD_SYMBOL_in_create_table_option15185_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15187  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15187_bits[]	= { ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15187	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15187_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_table_option15190  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_table_option15190_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_table_option15190	= { FOLLOW_set_in_create_table_option15190_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATA_SYMBOL_in_create_table_option15205  */
static	ANTLR3_BITWORD FOLLOW_DATA_SYMBOL_in_create_table_option15205_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATA_SYMBOL_in_create_table_option15205	= { FOLLOW_DATA_SYMBOL_in_create_table_option15205_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15207  */
static	ANTLR3_BITWORD FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15207_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15207	= { FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15207_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15209  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15209_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15209	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15209_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option15212  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option15212_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option15212	= { FOLLOW_text_string_in_create_table_option15212_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INDEX_SYMBOL_in_create_table_option15217  */
static	ANTLR3_BITWORD FOLLOW_INDEX_SYMBOL_in_create_table_option15217_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INDEX_SYMBOL_in_create_table_option15217	= { FOLLOW_INDEX_SYMBOL_in_create_table_option15217_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15219  */
static	ANTLR3_BITWORD FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15219_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15219	= { FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15219_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15221  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15221_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15221	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15221_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option15224  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option15224_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option15224	= { FOLLOW_text_string_in_create_table_option15224_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_create_table_option15229  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_create_table_option15229_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_create_table_option15229	= { FOLLOW_TABLESPACE_SYMBOL_in_create_table_option15229_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15234  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15234_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15234	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15234_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_create_table_option15243  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_create_table_option15243_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_create_table_option15243	= { FOLLOW_identifier_in_create_table_option15243_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_create_table_option15248  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_create_table_option15248_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_create_table_option15248	= { FOLLOW_STORAGE_SYMBOL_in_create_table_option15248_bits, 4	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_create_table_option15250  */
static	ANTLR3_BITWORD FOLLOW_set_in_create_table_option15250_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_create_table_option15250	= { FOLLOW_set_in_create_table_option15250_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CONNECTION_SYMBOL_in_create_table_option15261  */
static	ANTLR3_BITWORD FOLLOW_CONNECTION_SYMBOL_in_create_table_option15261_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800800000) };
static  ANTLR3_BITSET_LIST FOLLOW_CONNECTION_SYMBOL_in_create_table_option15261	= { FOLLOW_CONNECTION_SYMBOL_in_create_table_option15261_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15263  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15263_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15263	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15263_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_create_table_option15266  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_create_table_option15266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_create_table_option15266	= { FOLLOW_text_string_in_create_table_option15266_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_create_table_option15271  */
static	ANTLR3_BITWORD FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_create_table_option15271_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810800800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_create_table_option15271	= { FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_create_table_option15271_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_create_table_option15273  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_create_table_option15273_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_create_table_option15273	= { FOLLOW_EQUAL_OPERATOR_in_create_table_option15273_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_create_table_option15276  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_create_table_option15276_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_create_table_option15276	= { FOLLOW_ulong_number_in_create_table_option15276_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_partitioning15290  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_partitioning15290_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_partitioning15290	= { FOLLOW_PARTITION_SYMBOL_in_partitioning15290_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_partitioning15292  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_partitioning15292_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4080000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_partitioning15292	= { FOLLOW_BY_SYMBOL_in_partitioning15292_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LINEAR_SYMBOL_in_partitioning15299  */
static	ANTLR3_BITWORD FOLLOW_LINEAR_SYMBOL_in_partitioning15299_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LINEAR_SYMBOL_in_partitioning15299	= { FOLLOW_LINEAR_SYMBOL_in_partitioning15299_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HASH_SYMBOL_in_partitioning15302  */
static	ANTLR3_BITWORD FOLLOW_HASH_SYMBOL_in_partitioning15302_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_HASH_SYMBOL_in_partitioning15302	= { FOLLOW_HASH_SYMBOL_in_partitioning15302_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15304  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15304_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15304	= { FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15304_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_partitioning15306  */
static	ANTLR3_BITWORD FOLLOW_expression_in_partitioning15306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_partitioning15306	= { FOLLOW_expression_in_partitioning15306_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15308  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15308_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15308	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15308_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LINEAR_SYMBOL_in_partitioning15314  */
static	ANTLR3_BITWORD FOLLOW_LINEAR_SYMBOL_in_partitioning15314_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LINEAR_SYMBOL_in_partitioning15314	= { FOLLOW_LINEAR_SYMBOL_in_partitioning15314_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_partitioning15317  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_partitioning15317_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_partitioning15317	= { FOLLOW_KEY_SYMBOL_in_partitioning15317_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_key_algorithm_in_partitioning15319  */
static	ANTLR3_BITWORD FOLLOW_partition_key_algorithm_in_partitioning15319_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_key_algorithm_in_partitioning15319	= { FOLLOW_partition_key_algorithm_in_partitioning15319_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_partitioning15322  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_partitioning15322_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_partitioning15322	= { FOLLOW_identifier_list_with_parentheses_in_partitioning15322_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_partitioning15328  */
static	ANTLR3_BITWORD FOLLOW_set_in_partitioning15328_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_partitioning15328	= { FOLLOW_set_in_partitioning15328_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15346  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15346_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15346	= { FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15346_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_partitioning15348  */
static	ANTLR3_BITWORD FOLLOW_expression_in_partitioning15348_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_partitioning15348	= { FOLLOW_expression_in_partitioning15348_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15350  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15350_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15350	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15350_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMNS_SYMBOL_in_partitioning15360  */
static	ANTLR3_BITWORD FOLLOW_COLUMNS_SYMBOL_in_partitioning15360_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMNS_SYMBOL_in_partitioning15360	= { FOLLOW_COLUMNS_SYMBOL_in_partitioning15360_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_partitioning15362  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_partitioning15362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0200000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_partitioning15362	= { FOLLOW_identifier_list_with_parentheses_in_partitioning15362_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITIONS_SYMBOL_in_partitioning15374  */
static	ANTLR3_BITWORD FOLLOW_PARTITIONS_SYMBOL_in_partitioning15374_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITIONS_SYMBOL_in_partitioning15374	= { FOLLOW_PARTITIONS_SYMBOL_in_partitioning15374_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_partitioning15376  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_partitioning15376_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_partitioning15376	= { FOLLOW_real_ulong_number_in_partitioning15376_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUBPARTITION_SYMBOL_in_partitioning15385  */
static	ANTLR3_BITWORD FOLLOW_SUBPARTITION_SYMBOL_in_partitioning15385_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_SUBPARTITION_SYMBOL_in_partitioning15385	= { FOLLOW_SUBPARTITION_SYMBOL_in_partitioning15385_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_partitioning15387  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_partitioning15387_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4080000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_partitioning15387	= { FOLLOW_BY_SYMBOL_in_partitioning15387_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LINEAR_SYMBOL_in_partitioning15389  */
static	ANTLR3_BITWORD FOLLOW_LINEAR_SYMBOL_in_partitioning15389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000020000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0080000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LINEAR_SYMBOL_in_partitioning15389	= { FOLLOW_LINEAR_SYMBOL_in_partitioning15389_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HASH_SYMBOL_in_partitioning15406  */
static	ANTLR3_BITWORD FOLLOW_HASH_SYMBOL_in_partitioning15406_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_HASH_SYMBOL_in_partitioning15406	= { FOLLOW_HASH_SYMBOL_in_partitioning15406_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15408  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15408_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060D44880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15408	= { FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15408_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bitwise_or_expression_in_partitioning15410  */
static	ANTLR3_BITWORD FOLLOW_bitwise_or_expression_in_partitioning15410_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_bitwise_or_expression_in_partitioning15410	= { FOLLOW_bitwise_or_expression_in_partitioning15410_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15412  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15412_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15412	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15412_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_KEY_SYMBOL_in_partitioning15420  */
static	ANTLR3_BITWORD FOLLOW_KEY_SYMBOL_in_partitioning15420_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_KEY_SYMBOL_in_partitioning15420	= { FOLLOW_KEY_SYMBOL_in_partitioning15420_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_key_algorithm_in_partitioning15422  */
static	ANTLR3_BITWORD FOLLOW_partition_key_algorithm_in_partitioning15422_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_key_algorithm_in_partitioning15422	= { FOLLOW_partition_key_algorithm_in_partitioning15422_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_partitioning15425  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_partitioning15425_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000002000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_partitioning15425	= { FOLLOW_identifier_list_with_parentheses_in_partitioning15425_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUBPARTITIONS_SYMBOL_in_partitioning15436  */
static	ANTLR3_BITWORD FOLLOW_SUBPARTITIONS_SYMBOL_in_partitioning15436_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUBPARTITIONS_SYMBOL_in_partitioning15436	= { FOLLOW_SUBPARTITIONS_SYMBOL_in_partitioning15436_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_partitioning15438  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_partitioning15438_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_partitioning15438	= { FOLLOW_real_ulong_number_in_partitioning15438_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_definitions_in_partitioning15458  */
static	ANTLR3_BITWORD FOLLOW_partition_definitions_in_partitioning15458_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_definitions_in_partitioning15458	= { FOLLOW_partition_definitions_in_partitioning15458_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ALGORITHM_SYMBOL_in_partition_key_algorithm15471  */
static	ANTLR3_BITWORD FOLLOW_ALGORITHM_SYMBOL_in_partition_key_algorithm15471_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_ALGORITHM_SYMBOL_in_partition_key_algorithm15471	= { FOLLOW_ALGORITHM_SYMBOL_in_partition_key_algorithm15471_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_key_algorithm15473  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_key_algorithm15473_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_key_algorithm15473	= { FOLLOW_EQUAL_OPERATOR_in_partition_key_algorithm15473_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_partition_key_algorithm15475  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_partition_key_algorithm15475_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_partition_key_algorithm15475	= { FOLLOW_real_ulong_number_in_partition_key_algorithm15475_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partition_definitions15484  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partition_definitions15484_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partition_definitions15484	= { FOLLOW_OPEN_PAR_SYMBOL_in_partition_definitions15484_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_definition_in_partition_definitions15486  */
static	ANTLR3_BITWORD FOLLOW_partition_definition_in_partition_definitions15486_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_definition_in_partition_definitions15486	= { FOLLOW_partition_definition_in_partition_definitions15486_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_partition_definitions15489  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_partition_definitions15489_bits[]	= { ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_partition_definitions15489	= { FOLLOW_COMMA_SYMBOL_in_partition_definitions15489_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_definition_in_partition_definitions15491  */
static	ANTLR3_BITWORD FOLLOW_partition_definition_in_partition_definitions15491_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_definition_in_partition_definitions15491	= { FOLLOW_partition_definition_in_partition_definitions15491_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definitions15495  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definitions15495_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definitions15495	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definitions15495_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_partition_definition15504  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_partition_definition15504_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_partition_definition15504	= { FOLLOW_PARTITION_SYMBOL_in_partition_definition15504_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_partition_definition15506  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_partition_definition15506_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_partition_definition15506	= { FOLLOW_identifier_in_partition_definition15506_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VALUES_SYMBOL_in_partition_definition15513  */
static	ANTLR3_BITWORD FOLLOW_VALUES_SYMBOL_in_partition_definition15513_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0001000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_VALUES_SYMBOL_in_partition_definition15513	= { FOLLOW_VALUES_SYMBOL_in_partition_definition15513_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LESS_SYMBOL_in_partition_definition15522  */
static	ANTLR3_BITWORD FOLLOW_LESS_SYMBOL_in_partition_definition15522_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000002000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LESS_SYMBOL_in_partition_definition15522	= { FOLLOW_LESS_SYMBOL_in_partition_definition15522_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_THAN_SYMBOL_in_partition_definition15524  */
static	ANTLR3_BITWORD FOLLOW_THAN_SYMBOL_in_partition_definition15524_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000004000) };
static  ANTLR3_BITSET_LIST FOLLOW_THAN_SYMBOL_in_partition_definition15524	= { FOLLOW_THAN_SYMBOL_in_partition_definition15524_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_value_list_in_partition_definition15527  */
static	ANTLR3_BITWORD FOLLOW_partition_value_list_in_partition_definition15527_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_value_list_in_partition_definition15527	= { FOLLOW_partition_value_list_in_partition_definition15527_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAXVALUE_SYMBOL_in_partition_definition15531  */
static	ANTLR3_BITWORD FOLLOW_MAXVALUE_SYMBOL_in_partition_definition15531_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MAXVALUE_SYMBOL_in_partition_definition15531	= { FOLLOW_MAXVALUE_SYMBOL_in_partition_definition15531_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IN_SYMBOL_in_partition_definition15539  */
static	ANTLR3_BITWORD FOLLOW_IN_SYMBOL_in_partition_definition15539_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IN_SYMBOL_in_partition_definition15539	= { FOLLOW_IN_SYMBOL_in_partition_definition15539_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_value_list_in_partition_definition15541  */
static	ANTLR3_BITWORD FOLLOW_partition_value_list_in_partition_definition15541_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_value_list_in_partition_definition15541	= { FOLLOW_partition_value_list_in_partition_definition15541_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_option_in_partition_definition15553  */
static	ANTLR3_BITWORD FOLLOW_partition_option_in_partition_definition15553_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_option_in_partition_definition15553	= { FOLLOW_partition_option_in_partition_definition15553_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partition_definition15559  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partition_definition15559_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partition_definition15559	= { FOLLOW_OPEN_PAR_SYMBOL_in_partition_definition15559_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subpartition_definition_in_partition_definition15561  */
static	ANTLR3_BITWORD FOLLOW_subpartition_definition_in_partition_definition15561_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_subpartition_definition_in_partition_definition15561	= { FOLLOW_subpartition_definition_in_partition_definition15561_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_partition_definition15564  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_partition_definition15564_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000001000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_partition_definition15564	= { FOLLOW_COMMA_SYMBOL_in_partition_definition15564_bits, 6	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_subpartition_definition_in_partition_definition15566  */
static	ANTLR3_BITWORD FOLLOW_subpartition_definition_in_partition_definition15566_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_subpartition_definition_in_partition_definition15566	= { FOLLOW_subpartition_definition_in_partition_definition15566_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definition15570  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definition15570_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definition15570	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definition15570_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TABLESPACE_SYMBOL_in_partition_option15581  */
static	ANTLR3_BITWORD FOLLOW_TABLESPACE_SYMBOL_in_partition_option15581_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_TABLESPACE_SYMBOL_in_partition_option15581	= { FOLLOW_TABLESPACE_SYMBOL_in_partition_option15581_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15583  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15583_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15583	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15583_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_partition_option15586  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_partition_option15586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_partition_option15586	= { FOLLOW_identifier_in_partition_option15586_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_STORAGE_SYMBOL_in_partition_option15591  */
static	ANTLR3_BITWORD FOLLOW_STORAGE_SYMBOL_in_partition_option15591_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_STORAGE_SYMBOL_in_partition_option15591	= { FOLLOW_STORAGE_SYMBOL_in_partition_option15591_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENGINE_SYMBOL_in_partition_option15594  */
static	ANTLR3_BITWORD FOLLOW_ENGINE_SYMBOL_in_partition_option15594_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000900000) };
static  ANTLR3_BITSET_LIST FOLLOW_ENGINE_SYMBOL_in_partition_option15594	= { FOLLOW_ENGINE_SYMBOL_in_partition_option15594_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15596  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15596_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15596	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15596_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_engine_ref_in_partition_option15599  */
static	ANTLR3_BITWORD FOLLOW_engine_ref_in_partition_option15599_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_engine_ref_in_partition_option15599	= { FOLLOW_engine_ref_in_partition_option15599_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NODEGROUP_SYMBOL_in_partition_option15604  */
static	ANTLR3_BITWORD FOLLOW_NODEGROUP_SYMBOL_in_partition_option15604_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800800000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NODEGROUP_SYMBOL_in_partition_option15604	= { FOLLOW_NODEGROUP_SYMBOL_in_partition_option15604_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15606  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15606_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15606	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15606_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_partition_option15609  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_partition_option15609_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_partition_option15609	= { FOLLOW_real_ulong_number_in_partition_option15609_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_partition_option15614  */
static	ANTLR3_BITWORD FOLLOW_set_in_partition_option15614_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800800000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_partition_option15614	= { FOLLOW_set_in_partition_option15614_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15622  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15622_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010800000000), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15622	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15622_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulong_number_in_partition_option15625  */
static	ANTLR3_BITWORD FOLLOW_real_ulong_number_in_partition_option15625_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulong_number_in_partition_option15625	= { FOLLOW_real_ulong_number_in_partition_option15625_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_partition_option15630  */
static	ANTLR3_BITWORD FOLLOW_set_in_partition_option15630_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0100000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_partition_option15630	= { FOLLOW_set_in_partition_option15630_bits, 2	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DIRECTORY_SYMBOL_in_partition_option15638  */
static	ANTLR3_BITWORD FOLLOW_DIRECTORY_SYMBOL_in_partition_option15638_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000900000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DIRECTORY_SYMBOL_in_partition_option15638	= { FOLLOW_DIRECTORY_SYMBOL_in_partition_option15638_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15640  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15640_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15640	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15640_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_partition_option15643  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_partition_option15643_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_partition_option15643	= { FOLLOW_string_literal_in_partition_option15643_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMENT_SYMBOL_in_partition_option15648  */
static	ANTLR3_BITWORD FOLLOW_COMMENT_SYMBOL_in_partition_option15648_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000900000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMENT_SYMBOL_in_partition_option15648	= { FOLLOW_COMMENT_SYMBOL_in_partition_option15648_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_partition_option15650  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_partition_option15650_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_partition_option15650	= { FOLLOW_EQUAL_OPERATOR_in_partition_option15650_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_partition_option15653  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_partition_option15653_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_partition_option15653	= { FOLLOW_string_literal_in_partition_option15653_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SUBPARTITION_SYMBOL_in_subpartition_definition15662  */
static	ANTLR3_BITWORD FOLLOW_SUBPARTITION_SYMBOL_in_subpartition_definition15662_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SUBPARTITION_SYMBOL_in_subpartition_definition15662	= { FOLLOW_SUBPARTITION_SYMBOL_in_subpartition_definition15662_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_subpartition_definition15664  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_subpartition_definition15664_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_subpartition_definition15664	= { FOLLOW_identifier_in_subpartition_definition15664_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_partition_option_in_subpartition_definition15667  */
static	ANTLR3_BITWORD FOLLOW_partition_option_in_subpartition_definition15667_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000004002), ANTLR3_UINT64_LIT(0x0000400000000000), ANTLR3_UINT64_LIT(0x0100000000000002), ANTLR3_UINT64_LIT(0x0000200008000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000200080), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000080000000) };
static  ANTLR3_BITSET_LIST FOLLOW_partition_option_in_subpartition_definition15667	= { FOLLOW_partition_option_in_subpartition_definition15667_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_partition_value_list15678  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_partition_value_list15678_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880E000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_partition_value_list15678	= { FOLLOW_OPEN_PAR_SYMBOL_in_partition_value_list15678_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_partition_value_list15681  */
static	ANTLR3_BITWORD FOLLOW_expression_in_partition_value_list15681_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_partition_value_list15681	= { FOLLOW_expression_in_partition_value_list15681_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15685  */
static	ANTLR3_BITWORD FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15685_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15685	= { FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15685_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_partition_value_list15689  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_partition_value_list15689_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880E000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_partition_value_list15689	= { FOLLOW_COMMA_SYMBOL_in_partition_value_list15689_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_partition_value_list15692  */
static	ANTLR3_BITWORD FOLLOW_expression_in_partition_value_list15692_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_partition_value_list15692	= { FOLLOW_expression_in_partition_value_list15692_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15696  */
static	ANTLR3_BITWORD FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15696_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15696	= { FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15696_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_partition_value_list15701  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_partition_value_list15701_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_partition_value_list15701	= { FOLLOW_CLOSE_PAR_SYMBOL_in_partition_value_list15701_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFINER_SYMBOL_in_definer_clause15710  */
static	ANTLR3_BITWORD FOLLOW_DEFINER_SYMBOL_in_definer_clause15710_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFINER_SYMBOL_in_definer_clause15710	= { FOLLOW_DEFINER_SYMBOL_in_definer_clause15710_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_definer_clause15712  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_definer_clause15712_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_definer_clause15712	= { FOLLOW_EQUAL_OPERATOR_in_definer_clause15712_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_definer_clause15714  */
static	ANTLR3_BITWORD FOLLOW_user_in_definer_clause15714_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_definer_clause15714	= { FOLLOW_user_in_definer_clause15714_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IF_SYMBOL_in_if_exists15723  */
static	ANTLR3_BITWORD FOLLOW_IF_SYMBOL_in_if_exists15723_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_IF_SYMBOL_in_if_exists15723	= { FOLLOW_IF_SYMBOL_in_if_exists15723_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXISTS_SYMBOL_in_if_exists15725  */
static	ANTLR3_BITWORD FOLLOW_EXISTS_SYMBOL_in_if_exists15725_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EXISTS_SYMBOL_in_if_exists15725	= { FOLLOW_EXISTS_SYMBOL_in_if_exists15725_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IF_SYMBOL_in_if_not_exists15734  */
static	ANTLR3_BITWORD FOLLOW_IF_SYMBOL_in_if_not_exists15734_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000C00000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IF_SYMBOL_in_if_not_exists15734	= { FOLLOW_IF_SYMBOL_in_if_not_exists15734_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_not_rule_in_if_not_exists15736  */
static	ANTLR3_BITWORD FOLLOW_not_rule_in_if_not_exists15736_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000004) };
static  ANTLR3_BITSET_LIST FOLLOW_not_rule_in_if_not_exists15736	= { FOLLOW_not_rule_in_if_not_exists15736_bits, 8	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EXISTS_SYMBOL_in_if_not_exists15738  */
static	ANTLR3_BITWORD FOLLOW_EXISTS_SYMBOL_in_if_not_exists15738_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_EXISTS_SYMBOL_in_if_not_exists15738	= { FOLLOW_EXISTS_SYMBOL_in_if_not_exists15738_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_function_parameter_in_procedure_parameter15760  */
static	ANTLR3_BITWORD FOLLOW_function_parameter_in_procedure_parameter15760_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_function_parameter_in_procedure_parameter15760	= { FOLLOW_function_parameter_in_procedure_parameter15760_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_function_parameter15769  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_function_parameter15769_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0003000000003400), ANTLR3_UINT64_LIT(0x0004100003400001), ANTLR3_UINT64_LIT(0x800100C380000000), ANTLR3_UINT64_LIT(0x0001000000000001), ANTLR3_UINT64_LIT(0x0100000091000000), ANTLR3_UINT64_LIT(0x0000400000010058), ANTLR3_UINT64_LIT(0x0000200000001000), ANTLR3_UINT64_LIT(0x0000010000070160), ANTLR3_UINT64_LIT(0x0000000084000008), ANTLR3_UINT64_LIT(0x0000000014000038), ANTLR3_UINT64_LIT(0x0000000000200000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_function_parameter15769	= { FOLLOW_identifier_in_function_parameter15769_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_data_type_in_function_parameter15771  */
static	ANTLR3_BITWORD FOLLOW_data_type_in_function_parameter15771_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_data_type_in_function_parameter15771	= { FOLLOW_data_type_in_function_parameter15771_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_schema_identifier_pair15780  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_schema_identifier_pair15780_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_schema_identifier_pair15780	= { FOLLOW_OPEN_PAR_SYMBOL_in_schema_identifier_pair15780_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_schema_identifier_pair15782  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_schema_identifier_pair15782_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_schema_identifier_pair15782	= { FOLLOW_schema_ref_in_schema_identifier_pair15782_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_schema_identifier_pair15784  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_schema_identifier_pair15784_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_schema_identifier_pair15784	= { FOLLOW_COMMA_SYMBOL_in_schema_identifier_pair15784_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_schema_ref_in_schema_identifier_pair15786  */
static	ANTLR3_BITWORD FOLLOW_schema_ref_in_schema_identifier_pair15786_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_schema_ref_in_schema_identifier_pair15786	= { FOLLOW_schema_ref_in_schema_identifier_pair15786_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_schema_identifier_pair15788  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_schema_identifier_pair15788_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_schema_identifier_pair15788	= { FOLLOW_CLOSE_PAR_SYMBOL_in_schema_identifier_pair15788_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_ref_in_view_identifier_list15797  */
static	ANTLR3_BITWORD FOLLOW_view_ref_in_view_identifier_list15797_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_view_ref_in_view_identifier_list15797	= { FOLLOW_view_ref_in_view_identifier_list15797_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_view_identifier_list15800  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_view_identifier_list15800_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_view_identifier_list15800	= { FOLLOW_COMMA_SYMBOL_in_view_identifier_list15800_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_view_ref_in_view_identifier_list15802  */
static	ANTLR3_BITWORD FOLLOW_view_ref_in_view_identifier_list15802_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_view_ref_in_view_identifier_list15802	= { FOLLOW_view_ref_in_view_identifier_list15802_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_field_name_list15820  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_field_name_list15820_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_field_name_list15820	= { FOLLOW_column_ref_in_field_name_list15820_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_field_name_list15823  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_field_name_list15823_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_field_name_list15823	= { FOLLOW_COMMA_SYMBOL_in_field_name_list15823_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_field_name_list15825  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_field_name_list15825_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_field_name_list15825	= { FOLLOW_column_ref_in_field_name_list15825_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15836  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15836_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15836	= { FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15836_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_column_assignment_list_with_default15839  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_column_assignment_list_with_default15839_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_column_assignment_list_with_default15839	= { FOLLOW_COMMA_SYMBOL_in_column_assignment_list_with_default15839_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15841  */
static	ANTLR3_BITWORD FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15841_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15841	= { FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15841_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_in_column_assignment_with_default15852  */
static	ANTLR3_BITWORD FOLLOW_column_ref_in_column_assignment_with_default15852_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000800000) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_in_column_assignment_with_default15852	= { FOLLOW_column_ref_in_column_assignment_with_default15852_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_EQUAL_OPERATOR_in_column_assignment_with_default15854  */
static	ANTLR3_BITWORD FOLLOW_EQUAL_OPERATOR_in_column_assignment_with_default15854_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000200A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_EQUAL_OPERATOR_in_column_assignment_with_default15854	= { FOLLOW_EQUAL_OPERATOR_in_column_assignment_with_default15854_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_column_assignment_with_default15857  */
static	ANTLR3_BITWORD FOLLOW_expression_in_column_assignment_with_default15857_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_column_assignment_with_default15857	= { FOLLOW_expression_in_column_assignment_with_default15857_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_column_assignment_with_default15861  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_column_assignment_with_default15861_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_column_assignment_with_default15861	= { FOLLOW_DEFAULT_SYMBOL_in_column_assignment_with_default15861_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_charset_in_charset_clause15871  */
static	ANTLR3_BITWORD FOLLOW_charset_in_charset_clause15871_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0002000000000010), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_charset_in_charset_clause15871	= { FOLLOW_charset_in_charset_clause15871_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_charset_clause15874  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_charset_clause15874_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_charset_clause15874	= { FOLLOW_text_or_identifier_in_charset_clause15874_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DEFAULT_SYMBOL_in_charset_clause15878  */
static	ANTLR3_BITWORD FOLLOW_DEFAULT_SYMBOL_in_charset_clause15878_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DEFAULT_SYMBOL_in_charset_clause15878	= { FOLLOW_DEFAULT_SYMBOL_in_charset_clause15878_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BINARY_SYMBOL_in_charset_clause15882  */
static	ANTLR3_BITWORD FOLLOW_BINARY_SYMBOL_in_charset_clause15882_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BINARY_SYMBOL_in_charset_clause15882	= { FOLLOW_BINARY_SYMBOL_in_charset_clause15882_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COLUMNS_SYMBOL_in_fields_clause15892  */
static	ANTLR3_BITWORD FOLLOW_COLUMNS_SYMBOL_in_fields_clause15892_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_COLUMNS_SYMBOL_in_fields_clause15892	= { FOLLOW_COLUMNS_SYMBOL_in_fields_clause15892_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_field_term_in_fields_clause15894  */
static	ANTLR3_BITWORD FOLLOW_field_term_in_fields_clause15894_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000), ANTLR3_UINT64_LIT(0x0000000000000001), ANTLR3_UINT64_LIT(0x0002000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_field_term_in_fields_clause15894	= { FOLLOW_field_term_in_fields_clause15894_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TERMINATED_SYMBOL_in_field_term15904  */
static	ANTLR3_BITWORD FOLLOW_TERMINATED_SYMBOL_in_field_term15904_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_TERMINATED_SYMBOL_in_field_term15904	= { FOLLOW_TERMINATED_SYMBOL_in_field_term15904_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_field_term15906  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_field_term15906_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_field_term15906	= { FOLLOW_BY_SYMBOL_in_field_term15906_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_field_term15908  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_field_term15908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_field_term15908	= { FOLLOW_text_string_in_field_term15908_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPTIONALLY_SYMBOL_in_field_term15913  */
static	ANTLR3_BITWORD FOLLOW_OPTIONALLY_SYMBOL_in_field_term15913_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPTIONALLY_SYMBOL_in_field_term15913	= { FOLLOW_OPTIONALLY_SYMBOL_in_field_term15913_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ENCLOSED_SYMBOL_in_field_term15916  */
static	ANTLR3_BITWORD FOLLOW_ENCLOSED_SYMBOL_in_field_term15916_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_ENCLOSED_SYMBOL_in_field_term15916	= { FOLLOW_ENCLOSED_SYMBOL_in_field_term15916_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_field_term15918  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_field_term15918_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_field_term15918	= { FOLLOW_BY_SYMBOL_in_field_term15918_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_field_term15920  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_field_term15920_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_field_term15920	= { FOLLOW_text_string_in_field_term15920_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ESCAPED_SYMBOL_in_field_term15925  */
static	ANTLR3_BITWORD FOLLOW_ESCAPED_SYMBOL_in_field_term15925_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_ESCAPED_SYMBOL_in_field_term15925	= { FOLLOW_ESCAPED_SYMBOL_in_field_term15925_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_field_term15927  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_field_term15927_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_field_term15927	= { FOLLOW_BY_SYMBOL_in_field_term15927_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_field_term15929  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_field_term15929_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_field_term15929	= { FOLLOW_text_string_in_field_term15929_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LINES_SYMBOL_in_lines_clause15938  */
static	ANTLR3_BITWORD FOLLOW_LINES_SYMBOL_in_lines_clause15938_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LINES_SYMBOL_in_lines_clause15938	= { FOLLOW_LINES_SYMBOL_in_lines_clause15938_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_line_term_in_lines_clause15940  */
static	ANTLR3_BITWORD FOLLOW_line_term_in_lines_clause15940_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0008000000000000), ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_line_term_in_lines_clause15940	= { FOLLOW_line_term_in_lines_clause15940_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_line_term15950  */
static	ANTLR3_BITWORD FOLLOW_set_in_line_term15950_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_line_term15950	= { FOLLOW_set_in_line_term15950_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_line_term15958  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_line_term15958_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_line_term15958	= { FOLLOW_BY_SYMBOL_in_line_term15958_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_line_term15960  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_line_term15960_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_line_term15960	= { FOLLOW_text_string_in_line_term15960_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_user_list15969  */
static	ANTLR3_BITWORD FOLLOW_user_in_user_list15969_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_user_list15969	= { FOLLOW_user_in_user_list15969_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_user_list15972  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_user_list15972_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_user_list15972	= { FOLLOW_COMMA_SYMBOL_in_user_list15972_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_user_list15974  */
static	ANTLR3_BITWORD FOLLOW_user_in_user_list15974_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_user_list15974	= { FOLLOW_user_in_user_list15974_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_user_in_grant_list15985  */
static	ANTLR3_BITWORD FOLLOW_grant_user_in_grant_list15985_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_user_in_grant_list15985	= { FOLLOW_grant_user_in_grant_list15985_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_grant_list15988  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_grant_list15988_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_grant_list15988	= { FOLLOW_COMMA_SYMBOL_in_grant_list15988_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_grant_user_in_grant_list15990  */
static	ANTLR3_BITWORD FOLLOW_grant_user_in_grant_list15990_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_grant_user_in_grant_list15990	= { FOLLOW_grant_user_in_grant_list15990_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_user_in_grant_user16001  */
static	ANTLR3_BITWORD FOLLOW_user_in_grant_user16001_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000100000000) };
static  ANTLR3_BITSET_LIST FOLLOW_user_in_grant_user16001	= { FOLLOW_user_in_grant_user16001_bits, 3	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IDENTIFIED_SYMBOL_in_grant_user16008  */
static	ANTLR3_BITWORD FOLLOW_IDENTIFIED_SYMBOL_in_grant_user16008_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_IDENTIFIED_SYMBOL_in_grant_user16008	= { FOLLOW_IDENTIFIED_SYMBOL_in_grant_user16008_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_grant_user16017  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_grant_user16017_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_grant_user16017	= { FOLLOW_BY_SYMBOL_in_grant_user16017_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PASSWORD_SYMBOL_in_grant_user16019  */
static	ANTLR3_BITWORD FOLLOW_PASSWORD_SYMBOL_in_grant_user16019_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PASSWORD_SYMBOL_in_grant_user16019	= { FOLLOW_PASSWORD_SYMBOL_in_grant_user16019_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_grant_user16022  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_grant_user16022_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_grant_user16022	= { FOLLOW_text_string_in_grant_user16022_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_WITH_SYMBOL_in_grant_user16031  */
static	ANTLR3_BITWORD FOLLOW_WITH_SYMBOL_in_grant_user16031_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_WITH_SYMBOL_in_grant_user16031	= { FOLLOW_WITH_SYMBOL_in_grant_user16031_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_grant_user16033  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_grant_user16033_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000200), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000800000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_grant_user16033	= { FOLLOW_text_or_identifier_in_grant_user16033_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AS_SYMBOL_in_grant_user16042  */
static	ANTLR3_BITWORD FOLLOW_AS_SYMBOL_in_grant_user16042_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_AS_SYMBOL_in_grant_user16042	= { FOLLOW_AS_SYMBOL_in_grant_user16042_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BY_SYMBOL_in_grant_user16048  */
static	ANTLR3_BITWORD FOLLOW_BY_SYMBOL_in_grant_user16048_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_BY_SYMBOL_in_grant_user16048	= { FOLLOW_BY_SYMBOL_in_grant_user16048_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_grant_user16051  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_grant_user16051_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_grant_user16051	= { FOLLOW_text_string_in_grant_user16051_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_user16070  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_user16070_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0006000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_user16070	= { FOLLOW_text_or_identifier_in_user16070_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_SIGN_SYMBOL_in_user16073  */
static	ANTLR3_BITWORD FOLLOW_AT_SIGN_SYMBOL_in_user16073_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_SIGN_SYMBOL_in_user16073	= { FOLLOW_AT_SIGN_SYMBOL_in_user16073_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_user16075  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_user16075_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_user16075	= { FOLLOW_text_or_identifier_in_user16075_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_AT_TEXT_SUFFIX_in_user16079  */
static	ANTLR3_BITWORD FOLLOW_AT_TEXT_SUFFIX_in_user16079_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_AT_TEXT_SUFFIX_in_user16079	= { FOLLOW_AT_TEXT_SUFFIX_in_user16079_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CURRENT_USER_SYMBOL_in_user16086  */
static	ANTLR3_BITWORD FOLLOW_CURRENT_USER_SYMBOL_in_user16086_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CURRENT_USER_SYMBOL_in_user16086	= { FOLLOW_CURRENT_USER_SYMBOL_in_user16086_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_parentheses_in_user16088  */
static	ANTLR3_BITWORD FOLLOW_parentheses_in_user16088_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_parentheses_in_user16088	= { FOLLOW_parentheses_in_user16088_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LIKE_SYMBOL_in_like_clause16098  */
static	ANTLR3_BITWORD FOLLOW_LIKE_SYMBOL_in_like_clause16098_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_LIKE_SYMBOL_in_like_clause16098	= { FOLLOW_LIKE_SYMBOL_in_like_clause16098_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_like_clause16100  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_like_clause16100_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_like_clause16100	= { FOLLOW_text_string_in_like_clause16100_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_like_clause_in_like_or_where16109  */
static	ANTLR3_BITWORD FOLLOW_like_clause_in_like_or_where16109_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_like_clause_in_like_or_where16109	= { FOLLOW_like_clause_in_like_or_where16109_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_where_clause_in_like_or_where16113  */
static	ANTLR3_BITWORD FOLLOW_where_clause_in_like_or_where16113_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_where_clause_in_like_or_where16113	= { FOLLOW_where_clause_in_like_or_where16113_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_online_option16124  */
static	ANTLR3_BITWORD FOLLOW_set_in_online_option16124_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_online_option16124	= { FOLLOW_set_in_online_option16124_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_LOCAL_SYMBOL_in_no_write_to_bin_log16143  */
static	ANTLR3_BITWORD FOLLOW_LOCAL_SYMBOL_in_no_write_to_bin_log16143_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_LOCAL_SYMBOL_in_no_write_to_bin_log16143	= { FOLLOW_LOCAL_SYMBOL_in_no_write_to_bin_log16143_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NO_WRITE_TO_BINLOG_SYMBOL_in_no_write_to_bin_log16149  */
static	ANTLR3_BITWORD FOLLOW_NO_WRITE_TO_BINLOG_SYMBOL_in_no_write_to_bin_log16149_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NO_WRITE_TO_BINLOG_SYMBOL_in_no_write_to_bin_log16149	= { FOLLOW_NO_WRITE_TO_BINLOG_SYMBOL_in_no_write_to_bin_log16149_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PARTITION_SYMBOL_in_use_partition16162  */
static	ANTLR3_BITWORD FOLLOW_PARTITION_SYMBOL_in_use_partition16162_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000400000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PARTITION_SYMBOL_in_use_partition16162	= { FOLLOW_PARTITION_SYMBOL_in_use_partition16162_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_with_parentheses_in_use_partition16164  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_with_parentheses_in_use_partition16164_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_with_parentheses_in_use_partition16164	= { FOLLOW_identifier_list_with_parentheses_in_use_partition16164_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_column_name16181  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_column_name16181_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_column_name16181	= { FOLLOW_identifier_in_column_name16181_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_variants_in_column_ref16190  */
static	ANTLR3_BITWORD FOLLOW_column_ref_variants_in_column_ref16190_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_variants_in_column_ref16190	= { FOLLOW_column_ref_variants_in_column_ref16190_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_column_ref_variants16200  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_column_ref_variants16200_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_column_ref_variants16200	= { FOLLOW_dot_identifier_in_column_ref_variants16200_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_column_ref_variants16205  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_column_ref_variants16205_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_column_ref_variants16205	= { FOLLOW_qualified_identifier_in_column_ref_variants16205_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_column_ref_variants16220  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_column_ref_variants16220_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_column_ref_variants16220	= { FOLLOW_dot_identifier_in_column_ref_variants16220_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_variants_in_column_internal_ref16231  */
static	ANTLR3_BITWORD FOLLOW_column_ref_variants_in_column_internal_ref16231_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_variants_in_column_internal_ref16231	= { FOLLOW_column_ref_variants_in_column_internal_ref16231_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_column_ref_with_wildcard2_in_column_ref_with_wildcard16240  */
static	ANTLR3_BITWORD FOLLOW_column_ref_with_wildcard2_in_column_ref_with_wildcard16240_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_column_ref_with_wildcard2_in_column_ref_with_wildcard16240	= { FOLLOW_column_ref_with_wildcard2_in_column_ref_with_wildcard16240_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_column_ref_with_wildcard216249  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_column_ref_with_wildcard216249_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_column_ref_with_wildcard216249	= { FOLLOW_identifier_in_column_ref_with_wildcard216249_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216258  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216258_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216258	= { FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216258_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216260  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216260_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216260	= { FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216260_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_column_ref_with_wildcard216266  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_column_ref_with_wildcard216266_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_column_ref_with_wildcard216266	= { FOLLOW_dot_identifier_in_column_ref_with_wildcard216266_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216277  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216277_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216277	= { FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216277_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216279  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216279_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216279	= { FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216279_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_column_ref_with_wildcard216286  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_column_ref_with_wildcard216286_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_column_ref_with_wildcard216286	= { FOLLOW_dot_identifier_in_column_ref_with_wildcard216286_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_index_name16304  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_index_name16304_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_index_name16304	= { FOLLOW_identifier_in_index_name16304_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_index_ref16313  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_index_ref16313_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_index_ref16313	= { FOLLOW_identifier_in_index_ref16313_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_table_wild16322  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_table_wild16322_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_table_wild16322	= { FOLLOW_identifier_in_table_wild16322_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_table_wild16331  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_table_wild16331_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_table_wild16331	= { FOLLOW_DOT_SYMBOL_in_table_wild16331_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_table_wild16333  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_table_wild16333_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_table_wild16333	= { FOLLOW_MULT_OPERATOR_in_table_wild16333_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_table_wild16339  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_table_wild16339_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_table_wild16339	= { FOLLOW_dot_identifier_in_table_wild16339_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_table_wild16341  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_table_wild16341_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_table_wild16341	= { FOLLOW_DOT_SYMBOL_in_table_wild16341_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_table_wild16343  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_table_wild16343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_table_wild16343	= { FOLLOW_MULT_OPERATOR_in_table_wild16343_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_schema_name16362  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_schema_name16362_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_schema_name16362	= { FOLLOW_identifier_in_schema_name16362_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_schema_ref16371  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_schema_ref16371_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_schema_ref16371	= { FOLLOW_identifier_in_schema_ref16371_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_procedure_name16380  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_procedure_name16380_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_procedure_name16380	= { FOLLOW_qualified_identifier_in_procedure_name16380_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_procedure_ref16389  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_procedure_ref16389_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_procedure_ref16389	= { FOLLOW_qualified_identifier_in_procedure_ref16389_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_function_name16398  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_function_name16398_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_function_name16398	= { FOLLOW_qualified_identifier_in_function_name16398_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_function_ref16407  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_function_ref16407_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_function_ref16407	= { FOLLOW_qualified_identifier_in_function_ref16407_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_trigger_name16416  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_trigger_name16416_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_trigger_name16416	= { FOLLOW_qualified_identifier_in_trigger_name16416_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_trigger_ref16425  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_trigger_ref16425_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_trigger_ref16425	= { FOLLOW_qualified_identifier_in_trigger_ref16425_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_variants_in_view_name16434  */
static	ANTLR3_BITWORD FOLLOW_table_name_variants_in_view_name16434_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_variants_in_view_name16434	= { FOLLOW_table_name_variants_in_view_name16434_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_view_ref16443  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_view_ref16443_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_view_ref16443	= { FOLLOW_qualified_identifier_in_view_ref16443_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_tablespace_name16452  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_tablespace_name16452_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_tablespace_name16452	= { FOLLOW_identifier_in_tablespace_name16452_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_tablespace_ref16461  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_tablespace_ref16461_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_tablespace_ref16461	= { FOLLOW_identifier_in_tablespace_ref16461_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_logfile_group_name16470  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_logfile_group_name16470_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_logfile_group_name16470	= { FOLLOW_identifier_in_logfile_group_name16470_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_logfile_group_ref16479  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_logfile_group_ref16479_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_logfile_group_ref16479	= { FOLLOW_identifier_in_logfile_group_ref16479_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_event_name16488  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_event_name16488_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_event_name16488	= { FOLLOW_qualified_identifier_in_event_name16488_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_event_ref16497  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_event_ref16497_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_event_ref16497	= { FOLLOW_qualified_identifier_in_event_ref16497_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_udf_name16507  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_udf_name16507_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_udf_name16507	= { FOLLOW_identifier_in_udf_name16507_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_server_name16516  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_server_name16516_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_server_name16516	= { FOLLOW_text_or_identifier_in_server_name16516_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_server_ref16525  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_server_ref16525_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_server_ref16525	= { FOLLOW_text_or_identifier_in_server_ref16525_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_or_identifier_in_engine_ref16534  */
static	ANTLR3_BITWORD FOLLOW_text_or_identifier_in_engine_ref16534_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_or_identifier_in_engine_ref16534	= { FOLLOW_text_or_identifier_in_engine_ref16534_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_variants_in_table_name16543  */
static	ANTLR3_BITWORD FOLLOW_table_name_variants_in_table_name16543_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_variants_in_table_name16543	= { FOLLOW_table_name_variants_in_table_name16543_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_filter_table_ref16553  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_filter_table_ref16553_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_filter_table_ref16553	= { FOLLOW_identifier_in_filter_table_ref16553_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_filter_table_ref16555  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_filter_table_ref16555_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_filter_table_ref16555	= { FOLLOW_dot_identifier_in_filter_table_ref16555_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_table_ref_with_wildcard16564  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_table_ref_with_wildcard16564_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_table_ref_with_wildcard16564	= { FOLLOW_identifier_in_table_ref_with_wildcard16564_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16573  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16573_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16573	= { FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16573_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16575  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16575_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16575	= { FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16575_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_table_ref_with_wildcard16581  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_table_ref_with_wildcard16581_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_table_ref_with_wildcard16581	= { FOLLOW_dot_identifier_in_table_ref_with_wildcard16581_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16584  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16584_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x4000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16584	= { FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16584_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16586  */
static	ANTLR3_BITWORD FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16586_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16586	= { FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16586_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_name_variants_in_table_ref16601  */
static	ANTLR3_BITWORD FOLLOW_table_name_variants_in_table_ref16601_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_name_variants_in_table_ref16601	= { FOLLOW_table_name_variants_in_table_ref16601_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_table_ref_no_db16610  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_table_ref_no_db16610_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_table_ref_no_db16610	= { FOLLOW_identifier_in_table_ref_no_db16610_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_qualified_identifier_in_table_name_variants16619  */
static	ANTLR3_BITWORD FOLLOW_qualified_identifier_in_table_name_variants16619_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_qualified_identifier_in_table_name_variants16619	= { FOLLOW_qualified_identifier_in_table_name_variants16619_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_table_name_variants16624  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_table_name_variants16624_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_table_name_variants16624	= { FOLLOW_dot_identifier_in_table_name_variants16624_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_table_ref_list16633  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_table_ref_list16633_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_table_ref_list16633	= { FOLLOW_table_ref_in_table_ref_list16633_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_table_ref_list16636  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_table_ref_list16636_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000120000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_table_ref_list16636	= { FOLLOW_COMMA_SYMBOL_in_table_ref_list16636_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_in_table_ref_list16638  */
static	ANTLR3_BITWORD FOLLOW_table_ref_in_table_ref_list16638_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_in_table_ref_list16638	= { FOLLOW_table_ref_in_table_ref_list16638_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16656  */
static	ANTLR3_BITWORD FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16656_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16656	= { FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16656_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_table_ref_list_with_wildcard16659  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_table_ref_list_with_wildcard16659_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_table_ref_list_with_wildcard16659	= { FOLLOW_COMMA_SYMBOL_in_table_ref_list_with_wildcard16659_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16661  */
static	ANTLR3_BITWORD FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16661_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16661	= { FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16661_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_IDENTIFIER_in_pure_identifier16675  */
static	ANTLR3_BITWORD FOLLOW_IDENTIFIER_in_pure_identifier16675_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_IDENTIFIER_in_pure_identifier16675	= { FOLLOW_IDENTIFIER_in_pure_identifier16675_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BACK_TICK_QUOTED_ID_in_pure_identifier16680  */
static	ANTLR3_BITWORD FOLLOW_BACK_TICK_QUOTED_ID_in_pure_identifier16680_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BACK_TICK_QUOTED_ID_in_pure_identifier16680	= { FOLLOW_BACK_TICK_QUOTED_ID_in_pure_identifier16680_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOUBLE_QUOTED_TEXT_in_pure_identifier16687  */
static	ANTLR3_BITWORD FOLLOW_DOUBLE_QUOTED_TEXT_in_pure_identifier16687_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_DOUBLE_QUOTED_TEXT_in_pure_identifier16687	= { FOLLOW_DOUBLE_QUOTED_TEXT_in_pure_identifier16687_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_pure_identifier_in_identifier16697  */
static	ANTLR3_BITWORD FOLLOW_pure_identifier_in_identifier16697_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_pure_identifier_in_identifier16697	= { FOLLOW_pure_identifier_in_identifier16697_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_keyword_in_identifier16702  */
static	ANTLR3_BITWORD FOLLOW_keyword_in_identifier16702_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_keyword_in_identifier16702	= { FOLLOW_keyword_in_identifier16702_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_identifier_list16711  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_identifier_list16711_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_identifier_list16711	= { FOLLOW_identifier_in_identifier_list16711_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_identifier_list16727  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_identifier_list16727_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_identifier_list16727	= { FOLLOW_COMMA_SYMBOL_in_identifier_list16727_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_identifier_list16729  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_identifier_list16729_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_identifier_list16729	= { FOLLOW_identifier_in_identifier_list16729_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_identifier_list_with_parentheses16740  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_identifier_list_with_parentheses16740_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_identifier_list_with_parentheses16740	= { FOLLOW_OPEN_PAR_SYMBOL_in_identifier_list_with_parentheses16740_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_list_in_identifier_list_with_parentheses16742  */
static	ANTLR3_BITWORD FOLLOW_identifier_list_in_identifier_list_with_parentheses16742_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_list_in_identifier_list_with_parentheses16742	= { FOLLOW_identifier_list_in_identifier_list_with_parentheses16742_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_identifier_list_with_parentheses16744  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_identifier_list_with_parentheses16744_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_identifier_list_with_parentheses16744	= { FOLLOW_CLOSE_PAR_SYMBOL_in_identifier_list_with_parentheses16744_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_qualified_identifier16753  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_qualified_identifier16753_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000020000) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_qualified_identifier16753	= { FOLLOW_identifier_in_qualified_identifier16753_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_dot_identifier_in_qualified_identifier16768  */
static	ANTLR3_BITWORD FOLLOW_dot_identifier_in_qualified_identifier16768_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_dot_identifier_in_qualified_identifier16768	= { FOLLOW_dot_identifier_in_qualified_identifier16768_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOT_SYMBOL_in_dot_identifier16781  */
static	ANTLR3_BITWORD FOLLOW_DOT_SYMBOL_in_dot_identifier16781_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0010000000000000), ANTLR3_UINT64_LIT(0x0000001000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOT_SYMBOL_in_dot_identifier16781	= { FOLLOW_DOT_SYMBOL_in_dot_identifier16781_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_dot_identifier16783  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_dot_identifier16783_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_dot_identifier16783	= { FOLLOW_identifier_in_dot_identifier16783_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_string_list16901  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_string_list16901_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_string_list16901	= { FOLLOW_OPEN_PAR_SYMBOL_in_string_list16901_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_string_list16903  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_string_list16903_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_string_list16903	= { FOLLOW_text_string_in_string_list16903_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_string_list16906  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_string_list16906_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0020000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_string_list16906	= { FOLLOW_COMMA_SYMBOL_in_string_list16906_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_string_list16908  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_string_list16908_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_string_list16908	= { FOLLOW_text_string_in_string_list16908_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_string_list16912  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_string_list16912_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_string_list16912	= { FOLLOW_CLOSE_PAR_SYMBOL_in_string_list16912_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_text_string_in_text_string_no_linebreak16942  */
static	ANTLR3_BITWORD FOLLOW_text_string_in_text_string_no_linebreak16942_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_text_string_in_text_string_no_linebreak16942	= { FOLLOW_text_string_in_text_string_no_linebreak16942_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_literal_in_literal16951  */
static	ANTLR3_BITWORD FOLLOW_string_literal_in_literal16951_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_literal_in_literal16951	= { FOLLOW_string_literal_in_literal16951_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_num_literal_in_literal16956  */
static	ANTLR3_BITWORD FOLLOW_num_literal_in_literal16956_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_num_literal_in_literal16956	= { FOLLOW_num_literal_in_literal16956_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_temporal_literal_in_literal16968  */
static	ANTLR3_BITWORD FOLLOW_temporal_literal_in_literal16968_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_temporal_literal_in_literal16968	= { FOLLOW_temporal_literal_in_literal16968_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_null_literal_in_literal16973  */
static	ANTLR3_BITWORD FOLLOW_null_literal_in_literal16973_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_null_literal_in_literal16973	= { FOLLOW_null_literal_in_literal16973_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_bool_literal_in_literal16978  */
static	ANTLR3_BITWORD FOLLOW_bool_literal_in_literal16978_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_bool_literal_in_literal16978	= { FOLLOW_bool_literal_in_literal16978_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNDERSCORE_CHARSET_in_literal16983  */
static	ANTLR3_BITWORD FOLLOW_UNDERSCORE_CHARSET_in_literal16983_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000800000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNDERSCORE_CHARSET_in_literal16983	= { FOLLOW_UNDERSCORE_CHARSET_in_literal16983_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_HEX_NUMBER_in_literal16986  */
static	ANTLR3_BITWORD FOLLOW_HEX_NUMBER_in_literal16986_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_HEX_NUMBER_in_literal16986	= { FOLLOW_HEX_NUMBER_in_literal16986_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNDERSCORE_CHARSET_in_literal16993  */
static	ANTLR3_BITWORD FOLLOW_UNDERSCORE_CHARSET_in_literal16993_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0020000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNDERSCORE_CHARSET_in_literal16993	= { FOLLOW_UNDERSCORE_CHARSET_in_literal16993_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_BIN_NUMBER_in_literal16996  */
static	ANTLR3_BITWORD FOLLOW_BIN_NUMBER_in_literal16996_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_BIN_NUMBER_in_literal16996	= { FOLLOW_BIN_NUMBER_in_literal16996_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_literal_in_signed_literal17006  */
static	ANTLR3_BITWORD FOLLOW_literal_in_signed_literal17006_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_literal_in_signed_literal17006	= { FOLLOW_literal_in_signed_literal17006_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PLUS_OPERATOR_in_signed_literal17011  */
static	ANTLR3_BITWORD FOLLOW_PLUS_OPERATOR_in_signed_literal17011_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_PLUS_OPERATOR_in_signed_literal17011	= { FOLLOW_PLUS_OPERATOR_in_signed_literal17011_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_signed_literal17013  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_signed_literal17013_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_signed_literal17013	= { FOLLOW_ulong_number_in_signed_literal17013_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_MINUS_OPERATOR_in_signed_literal17018  */
static	ANTLR3_BITWORD FOLLOW_MINUS_OPERATOR_in_signed_literal17018_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0400010810000800), ANTLR3_UINT64_LIT(0x0000000400000000) };
static  ANTLR3_BITSET_LIST FOLLOW_MINUS_OPERATOR_in_signed_literal17018	= { FOLLOW_MINUS_OPERATOR_in_signed_literal17018_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_ulong_number_in_signed_literal17020  */
static	ANTLR3_BITWORD FOLLOW_ulong_number_in_signed_literal17020_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_ulong_number_in_signed_literal17020	= { FOLLOW_ulong_number_in_signed_literal17020_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_string_in_string_literal17031  */
static	ANTLR3_BITWORD FOLLOW_string_in_string_literal17031_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_string_in_string_literal17031	= { FOLLOW_string_in_string_literal17031_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NCHAR_TEXT_in_string17040  */
static	ANTLR3_BITWORD FOLLOW_NCHAR_TEXT_in_string17040_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NCHAR_TEXT_in_string17040	= { FOLLOW_NCHAR_TEXT_in_string17040_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_UNDERSCORE_CHARSET_in_string17045  */
static	ANTLR3_BITWORD FOLLOW_UNDERSCORE_CHARSET_in_string17045_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_UNDERSCORE_CHARSET_in_string17045	= { FOLLOW_UNDERSCORE_CHARSET_in_string17045_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SINGLE_QUOTED_TEXT_in_string17062  */
static	ANTLR3_BITWORD FOLLOW_SINGLE_QUOTED_TEXT_in_string17062_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_SINGLE_QUOTED_TEXT_in_string17062	= { FOLLOW_SINGLE_QUOTED_TEXT_in_string17062_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOUBLE_QUOTED_TEXT_in_string17068  */
static	ANTLR3_BITWORD FOLLOW_DOUBLE_QUOTED_TEXT_in_string17068_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000100000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOUBLE_QUOTED_TEXT_in_string17068	= { FOLLOW_DOUBLE_QUOTED_TEXT_in_string17068_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DATE_SYMBOL_in_temporal_literal17137  */
static	ANTLR3_BITWORD FOLLOW_DATE_SYMBOL_in_temporal_literal17137_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DATE_SYMBOL_in_temporal_literal17137	= { FOLLOW_DATE_SYMBOL_in_temporal_literal17137_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17139  */
static	ANTLR3_BITWORD FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17139_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17139	= { FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17139_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIME_SYMBOL_in_temporal_literal17144  */
static	ANTLR3_BITWORD FOLLOW_TIME_SYMBOL_in_temporal_literal17144_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIME_SYMBOL_in_temporal_literal17144	= { FOLLOW_TIME_SYMBOL_in_temporal_literal17144_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17146  */
static	ANTLR3_BITWORD FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17146_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17146	= { FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17146_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_TIMESTAMP_SYMBOL_in_temporal_literal17151  */
static	ANTLR3_BITWORD FOLLOW_TIMESTAMP_SYMBOL_in_temporal_literal17151_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_TIMESTAMP_SYMBOL_in_temporal_literal17151	= { FOLLOW_TIMESTAMP_SYMBOL_in_temporal_literal17151_bits, 10	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17153  */
static	ANTLR3_BITWORD FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17153_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17153	= { FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17153_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_REAL_SYMBOL_in_real_literal17193  */
static	ANTLR3_BITWORD FOLLOW_REAL_SYMBOL_in_real_literal17193_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_REAL_SYMBOL_in_real_literal17193	= { FOLLOW_REAL_SYMBOL_in_real_literal17193_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_DOUBLE_SYMBOL_in_real_literal17197  */
static	ANTLR3_BITWORD FOLLOW_DOUBLE_SYMBOL_in_real_literal17197_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0800000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_DOUBLE_SYMBOL_in_real_literal17197	= { FOLLOW_DOUBLE_SYMBOL_in_real_literal17197_bits, 9	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_PRECISION_SYMBOL_in_real_literal17199  */
static	ANTLR3_BITWORD FOLLOW_PRECISION_SYMBOL_in_real_literal17199_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_PRECISION_SYMBOL_in_real_literal17199	= { FOLLOW_PRECISION_SYMBOL_in_real_literal17199_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_float_options17211  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_float_options17211_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_float_options17211	= { FOLLOW_OPEN_PAR_SYMBOL_in_float_options17211_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_float_options17213  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_float_options17213_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_float_options17213	= { FOLLOW_INT_NUMBER_in_float_options17213_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_float_options17216  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_float_options17216_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_float_options17216	= { FOLLOW_COMMA_SYMBOL_in_float_options17216_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_float_options17218  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_float_options17218_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_float_options17218	= { FOLLOW_INT_NUMBER_in_float_options17218_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_float_options17222  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_float_options17222_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_float_options17222	= { FOLLOW_CLOSE_PAR_SYMBOL_in_float_options17222_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_precision17231  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_precision17231_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_precision17231	= { FOLLOW_OPEN_PAR_SYMBOL_in_precision17231_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_precision17233  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_precision17233_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_precision17233	= { FOLLOW_INT_NUMBER_in_precision17233_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_precision17235  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_precision17235_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000010000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_precision17235	= { FOLLOW_COMMA_SYMBOL_in_precision17235_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_INT_NUMBER_in_precision17237  */
static	ANTLR3_BITWORD FOLLOW_INT_NUMBER_in_precision17237_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_INT_NUMBER_in_precision17237	= { FOLLOW_INT_NUMBER_in_precision17237_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_precision17239  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_precision17239_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_precision17239	= { FOLLOW_CLOSE_PAR_SYMBOL_in_precision17239_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NCHAR_SYMBOL_in_nchar_literal17248  */
static	ANTLR3_BITWORD FOLLOW_NCHAR_SYMBOL_in_nchar_literal17248_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NCHAR_SYMBOL_in_nchar_literal17248	= { FOLLOW_NCHAR_SYMBOL_in_nchar_literal17248_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NATIONAL_SYMBOL_in_nchar_literal17255  */
static	ANTLR3_BITWORD FOLLOW_NATIONAL_SYMBOL_in_nchar_literal17255_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_NATIONAL_SYMBOL_in_nchar_literal17255	= { FOLLOW_NATIONAL_SYMBOL_in_nchar_literal17255_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_nchar_literal17257  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_nchar_literal17257_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_nchar_literal17257	= { FOLLOW_CHAR_SYMBOL_in_nchar_literal17257_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_varchar_literal17267  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_varchar_literal17267_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_varchar_literal17267	= { FOLLOW_CHAR_SYMBOL_in_varchar_literal17267_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARYING_SYMBOL_in_varchar_literal17269  */
static	ANTLR3_BITWORD FOLLOW_VARYING_SYMBOL_in_varchar_literal17269_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARYING_SYMBOL_in_varchar_literal17269	= { FOLLOW_VARYING_SYMBOL_in_varchar_literal17269_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARCHAR_SYMBOL_in_varchar_literal17274  */
static	ANTLR3_BITWORD FOLLOW_VARCHAR_SYMBOL_in_varchar_literal17274_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARCHAR_SYMBOL_in_varchar_literal17274	= { FOLLOW_VARCHAR_SYMBOL_in_varchar_literal17274_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17283  */
static	ANTLR3_BITWORD FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17283_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17283	= { FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17283_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17285  */
static	ANTLR3_BITWORD FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17285_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17285	= { FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17285_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NVARCHAR_SYMBOL_in_nvarchar_literal17290  */
static	ANTLR3_BITWORD FOLLOW_NVARCHAR_SYMBOL_in_nvarchar_literal17290_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_NVARCHAR_SYMBOL_in_nvarchar_literal17290	= { FOLLOW_NVARCHAR_SYMBOL_in_nvarchar_literal17290_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17295  */
static	ANTLR3_BITWORD FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17295_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000010000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17295	= { FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17295_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17297  */
static	ANTLR3_BITWORD FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17297_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17297	= { FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17297_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17302  */
static	ANTLR3_BITWORD FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17302_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000010000) };
static  ANTLR3_BITSET_LIST FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17302	= { FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17302_bits, 7	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CHAR_SYMBOL_in_nvarchar_literal17304  */
static	ANTLR3_BITWORD FOLLOW_CHAR_SYMBOL_in_nvarchar_literal17304_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_CHAR_SYMBOL_in_nvarchar_literal17304	= { FOLLOW_CHAR_SYMBOL_in_nvarchar_literal17304_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17306  */
static	ANTLR3_BITWORD FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17306_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17306	= { FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17306_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17311  */
static	ANTLR3_BITWORD FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17311_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000040000000) };
static  ANTLR3_BITSET_LIST FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17311	= { FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17311_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17313  */
static	ANTLR3_BITWORD FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17313_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17313	= { FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17313_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_SINGLE_QUOTED_TEXT_in_text_or_identifier17322  */
static	ANTLR3_BITWORD FOLLOW_SINGLE_QUOTED_TEXT_in_text_or_identifier17322_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_SINGLE_QUOTED_TEXT_in_text_or_identifier17322	= { FOLLOW_SINGLE_QUOTED_TEXT_in_text_or_identifier17322_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_identifier_in_text_or_identifier17327  */
static	ANTLR3_BITWORD FOLLOW_identifier_in_text_or_identifier17327_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_identifier_in_text_or_identifier17327	= { FOLLOW_identifier_in_text_or_identifier17327_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_real_ulonglong_number_in_size_number17338  */
static	ANTLR3_BITWORD FOLLOW_real_ulonglong_number_in_size_number17338_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_real_ulonglong_number_in_size_number17338	= { FOLLOW_real_ulonglong_number_in_size_number17338_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_pure_identifier_in_size_number17343  */
static	ANTLR3_BITWORD FOLLOW_pure_identifier_in_size_number17343_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_pure_identifier_in_size_number17343	= { FOLLOW_pure_identifier_in_size_number17343_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_parentheses17353  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_parentheses17353_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x8000000000000000) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_parentheses17353	= { FOLLOW_OPEN_PAR_SYMBOL_in_parentheses17353_bits, 11	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_CLOSE_PAR_SYMBOL_in_parentheses17355  */
static	ANTLR3_BITWORD FOLLOW_CLOSE_PAR_SYMBOL_in_parentheses17355_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_CLOSE_PAR_SYMBOL_in_parentheses17355	= { FOLLOW_CLOSE_PAR_SYMBOL_in_parentheses17355_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_keyword_sp_in_keyword17397  */
static	ANTLR3_BITWORD FOLLOW_keyword_sp_in_keyword17397_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_keyword_sp_in_keyword17397	= { FOLLOW_keyword_sp_in_keyword17397_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_set_in_keyword17402  */
static	ANTLR3_BITWORD FOLLOW_set_in_keyword17402_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_set_in_keyword17402	= { FOLLOW_set_in_keyword17402_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_table_wild_in_synpred1_MySQLSimpleParser4873  */
static	ANTLR3_BITWORD FOLLOW_table_wild_in_synpred1_MySQLSimpleParser4873_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_table_wild_in_synpred1_MySQLSimpleParser4873	= { FOLLOW_table_wild_in_synpred1_MySQLSimpleParser4873_bits, 1	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_OPEN_PAR_SYMBOL_in_synpred2_MySQLSimpleParser10462  */
static	ANTLR3_BITWORD FOLLOW_OPEN_PAR_SYMBOL_in_synpred2_MySQLSimpleParser10462_bits[]	= { ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFF0), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0xFFFFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x01FFFFFFFFFFFFFF), ANTLR3_UINT64_LIT(0x000000A953013010), ANTLR3_UINT64_LIT(0x08000810204000C4), ANTLR3_UINT64_LIT(0x030060DC4880A000), ANTLR3_UINT64_LIT(0x34A0000040008300), ANTLR3_UINT64_LIT(0x1CB7000122E00600), ANTLR3_UINT64_LIT(0x0D00011810120800), ANTLR3_UINT64_LIT(0x0000000C00000010) };
static  ANTLR3_BITSET_LIST FOLLOW_OPEN_PAR_SYMBOL_in_synpred2_MySQLSimpleParser10462	= { FOLLOW_OPEN_PAR_SYMBOL_in_synpred2_MySQLSimpleParser10462_bits, 13	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_expression_in_synpred2_MySQLSimpleParser10464  */
static	ANTLR3_BITWORD FOLLOW_expression_in_synpred2_MySQLSimpleParser10464_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000000), ANTLR3_UINT64_LIT(0x0000000000000020) };
static  ANTLR3_BITSET_LIST FOLLOW_expression_in_synpred2_MySQLSimpleParser10464	= { FOLLOW_expression_in_synpred2_MySQLSimpleParser10464_bits, 12	};
/** Bitset defining follow set for error recovery in rule state: FOLLOW_COMMA_SYMBOL_in_synpred2_MySQLSimpleParser10466  */
static	ANTLR3_BITWORD FOLLOW_COMMA_SYMBOL_in_synpred2_MySQLSimpleParser10466_bits[]	= { ANTLR3_UINT64_LIT(0x0000000000000002) };
static  ANTLR3_BITSET_LIST FOLLOW_COMMA_SYMBOL_in_synpred2_MySQLSimpleParser10466	= { FOLLOW_COMMA_SYMBOL_in_synpred2_MySQLSimpleParser10466_bits, 1	};


/* ==============================================
 * Parsing rules
 */
/**
 * $ANTLR start query
 * MySQLSimpleParser.g:87:1: query : ( ( statement | begin_work ) ( SEMICOLON_SYMBOL )? )? EOF ;
 */
static void
query(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:87:6: ( ( ( statement | begin_work ) ( SEMICOLON_SYMBOL )? )? EOF )
        // MySQLSimpleParser.g:88:2: ( ( statement | begin_work ) ( SEMICOLON_SYMBOL )? )? EOF
        {
            // MySQLSimpleParser.g:88:2: ( ( statement | begin_work ) ( SEMICOLON_SYMBOL )? )?
            {
                int alt3=2;
                {
                    int LA3_0 = LA(1);
                    if ( (LA3_0 == DO_SYMBOL || LA3_0 == HANDLER_SYMBOL || LA3_0 == TRUNCATE_SYMBOL || LA3_0 == ALTER_SYMBOL || LA3_0 == CALL_SYMBOL || LA3_0 == CREATE_SYMBOL || LA3_0 == DELETE_SYMBOL || LA3_0 == DROP_SYMBOL || LA3_0 == INSERT_SYMBOL || LA3_0 == LOAD_SYMBOL || LA3_0 == OPEN_PAR_SYMBOL || LA3_0 == RENAME_SYMBOL || LA3_0 == REPLACE_SYMBOL || LA3_0 == SELECT_SYMBOL || LA3_0 == UPDATE_SYMBOL))
                    {
                        alt3=1;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA3_0 == PARTITION_SYMBOL))
                    {
                        alt3=1;
                    }
                    else if ( (((LA3_0 >= BACKUP_SYMBOL) && (LA3_0 <= BEGIN_SYMBOL)) || LA3_0 == CACHE_SYMBOL || LA3_0 == CHECKSUM_SYMBOL || LA3_0 == COMMIT_SYMBOL || LA3_0 == DEALLOCATE_SYMBOL || ((LA3_0 >= EXECUTE_SYMBOL) && (LA3_0 <= FLUSH_SYMBOL)) || LA3_0 == HELP_SYMBOL || LA3_0 == INSTALL_SYMBOL || LA3_0 == PREPARE_SYMBOL || ((LA3_0 >= REPAIR_SYMBOL) && (LA3_0 <= SAVEPOINT_SYMBOL)) || ((LA3_0 >= START_SYMBOL) && (LA3_0 <= STOP_SYMBOL)) || LA3_0 == UNINSTALL_SYMBOL || LA3_0 == XA_SYMBOL || LA3_0 == ANALYZE_SYMBOL || LA3_0 == BINLOG_SYMBOL || LA3_0 == SHUTDOWN_SYMBOL || LA3_0 == CHANGE_SYMBOL || LA3_0 == CHECK_SYMBOL || ((LA3_0 >= DESCRIBE_SYMBOL) && (LA3_0 <= DESC_SYMBOL)) || ((LA3_0 >= GET_SYMBOL) && (LA3_0 <= GRANT_SYMBOL)) || LA3_0 == KILL_SYMBOL || LA3_0 == LOCK_SYMBOL || LA3_0 == OPTIMIZE_SYMBOL || LA3_0 == PURGE_SYMBOL || LA3_0 == RELEASE_SYMBOL || LA3_0 == RESIGNAL_SYMBOL || LA3_0 == REVOKE_SYMBOL || LA3_0 == SET_SYMBOL || ((LA3_0 >= SHOW_SYMBOL) && (LA3_0 <= SIGNAL_SYMBOL)) || LA3_0 == UNLOCK_SYMBOL || LA3_0 == USE_SYMBOL))
                    {
                        alt3=1;
                    }
                }
                switch (alt3)
                {
            	case 1:
            	    // MySQLSimpleParser.g:88:3: ( statement | begin_work ) ( SEMICOLON_SYMBOL )?
            	    {
            	        // MySQLSimpleParser.g:88:3: ( statement | begin_work )
            	        {
            	            int alt1=2;
            	            {
            	                int LA1_0 = LA(1);
            	                if ( (LA1_0 == DO_SYMBOL || LA1_0 == HANDLER_SYMBOL || LA1_0 == TRUNCATE_SYMBOL || LA1_0 == ALTER_SYMBOL || LA1_0 == CALL_SYMBOL || LA1_0 == CREATE_SYMBOL || LA1_0 == DELETE_SYMBOL || LA1_0 == DROP_SYMBOL || LA1_0 == INSERT_SYMBOL || LA1_0 == LOAD_SYMBOL || LA1_0 == OPEN_PAR_SYMBOL || LA1_0 == RENAME_SYMBOL || LA1_0 == REPLACE_SYMBOL || LA1_0 == SELECT_SYMBOL || LA1_0 == UPDATE_SYMBOL))
            	                {
            	                    alt1=1;
            	                }
            	                else if ( ((SERVER_VERSION >= 50100)) && (LA1_0 == PARTITION_SYMBOL))
            	                {
            	                    alt1=1;
            	                }
            	                else if ( (LA1_0 == BACKUP_SYMBOL || LA1_0 == CACHE_SYMBOL || LA1_0 == CHECKSUM_SYMBOL || LA1_0 == COMMIT_SYMBOL || LA1_0 == DEALLOCATE_SYMBOL || ((LA1_0 >= EXECUTE_SYMBOL) && (LA1_0 <= FLUSH_SYMBOL)) || LA1_0 == HELP_SYMBOL || LA1_0 == INSTALL_SYMBOL || LA1_0 == PREPARE_SYMBOL || ((LA1_0 >= REPAIR_SYMBOL) && (LA1_0 <= SAVEPOINT_SYMBOL)) || ((LA1_0 >= START_SYMBOL) && (LA1_0 <= STOP_SYMBOL)) || LA1_0 == UNINSTALL_SYMBOL || LA1_0 == XA_SYMBOL || LA1_0 == ANALYZE_SYMBOL || LA1_0 == BINLOG_SYMBOL || LA1_0 == SHUTDOWN_SYMBOL || LA1_0 == CHANGE_SYMBOL || LA1_0 == CHECK_SYMBOL || ((LA1_0 >= DESCRIBE_SYMBOL) && (LA1_0 <= DESC_SYMBOL)) || ((LA1_0 >= GET_SYMBOL) && (LA1_0 <= GRANT_SYMBOL)) || LA1_0 == KILL_SYMBOL || LA1_0 == LOCK_SYMBOL || LA1_0 == OPTIMIZE_SYMBOL || LA1_0 == PURGE_SYMBOL || LA1_0 == RELEASE_SYMBOL || LA1_0 == RESIGNAL_SYMBOL || LA1_0 == REVOKE_SYMBOL || LA1_0 == SET_SYMBOL || ((LA1_0 >= SHOW_SYMBOL) && (LA1_0 <= SIGNAL_SYMBOL)) || LA1_0 == UNLOCK_SYMBOL || LA1_0 == USE_SYMBOL))
            	                {
            	                    alt1=1;
            	                }
            	                else if ( (LA1_0 == BEGIN_SYMBOL))
            	                {
            	                    alt1=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 1;
            	                    EXCEPTION->state        = 0;


            	                    goto rulequeryEx;

            	                }
            	            }
            	            switch (alt1)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:88:4: statement
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_statement_in_query96);
            	        	        statement(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulequeryEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:88:16: begin_work
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_begin_work_in_query100);
            	        	        begin_work(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulequeryEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:88:28: ( SEMICOLON_SYMBOL )?
            	        {
            	            int alt2=2;
            	            switch ( LA(1) )
            	            {
            	                case SEMICOLON_SYMBOL:
            	                	{
            	                		alt2=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt2)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:88:28: SEMICOLON_SYMBOL
            	        	    {
            	        	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_query103);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulequeryEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_query108);
            if  (HASEXCEPTION())
            {
                goto rulequeryEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulequeryEx; /* Prevent compiler warnings */
    rulequeryEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end query */

/**
 * $ANTLR start statement
 * MySQLSimpleParser.g:91:1: statement : ( alter_statement | create_statement | drop_statement | rename_table_statement | truncate_table_statement | call_statement | delete_statement | do_statement | handler_statement | insert_statement | load_statement | replace_statement | select_statement | update_statement | partitioning | transaction_or_locking_statement | replication_statement | prepared_statement | account_management_statement | table_administration_statement | install_uninstall_statment |{...}? set_statement | show_statement | other_administrative_statement | utility_statement |{...}? get_diagnostics |{...}? signal_statement |{...}? resignal_statement );
 */
static void
statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:91:10: ( alter_statement | create_statement | drop_statement | rename_table_statement | truncate_table_statement | call_statement | delete_statement | do_statement | handler_statement | insert_statement | load_statement | replace_statement | select_statement | update_statement | partitioning | transaction_or_locking_statement | replication_statement | prepared_statement | account_management_statement | table_administration_statement | install_uninstall_statment |{...}? set_statement | show_statement | other_administrative_statement | utility_statement |{...}? get_diagnostics |{...}? signal_statement |{...}? resignal_statement )

            ANTLR3_UINT32 alt4;

            alt4=28;

            {
                int LA4_0 = LA(1);
                if ( (LA4_0 == ALTER_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case USER_SYMBOL:
                    	{
                    		alt4=19;
                    	}
                        break;
                    case SERVER_SYMBOL:
                    case ALGORITHM_SYMBOL:
                    case DEFINER_SYMBOL:
                    case EVENT_SYMBOL:
                    case LOGFILE_SYMBOL:
                    case TABLESPACE_SYMBOL:
                    case FUNCTION_SYMBOL:
                    case VIEW_SYMBOL:
                    case DATABASE_SYMBOL:
                    case IGNORE_SYMBOL:
                    case OFFLINE_SYMBOL:
                    case ONLINE_SYMBOL:
                    case PROCEDURE_SYMBOL:
                    case SQL_SYMBOL:
                    case TABLE_SYMBOL:
                    	{
                    		alt4=1;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 1;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == CREATE_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case USER_SYMBOL:
                    	{
                    		alt4=19;
                    	}
                        break;
                    case SERVER_SYMBOL:
                    case AGGREGATE_SYMBOL:
                    case ALGORITHM_SYMBOL:
                    case DEFINER_SYMBOL:
                    case EVENT_SYMBOL:
                    case LOGFILE_SYMBOL:
                    case TABLESPACE_SYMBOL:
                    case TEMPORARY_SYMBOL:
                    case FUNCTION_SYMBOL:
                    case VIEW_SYMBOL:
                    case DATABASE_SYMBOL:
                    case FULLTEXT_SYMBOL:
                    case INDEX_SYMBOL:
                    case OFFLINE_SYMBOL:
                    case ONLINE_SYMBOL:
                    case OR_SYMBOL:
                    case PROCEDURE_SYMBOL:
                    case SPATIAL_SYMBOL:
                    case SQL_SYMBOL:
                    case TABLE_SYMBOL:
                    case TRIGGER_SYMBOL:
                    case UNIQUE_SYMBOL:
                    	{
                    		alt4=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 2;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == DROP_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case PREPARE_SYMBOL:
                    	{
                    		alt4=18;
                    	}
                        break;
                    case USER_SYMBOL:
                    	{
                    		alt4=19;
                    	}
                        break;
                    case SERVER_SYMBOL:
                    case EVENT_SYMBOL:
                    case LOGFILE_SYMBOL:
                    case TABLES_SYMBOL:
                    case TABLESPACE_SYMBOL:
                    case TEMPORARY_SYMBOL:
                    case FUNCTION_SYMBOL:
                    case VIEW_SYMBOL:
                    case DATABASE_SYMBOL:
                    case INDEX_SYMBOL:
                    case OFFLINE_SYMBOL:
                    case ONLINE_SYMBOL:
                    case PROCEDURE_SYMBOL:
                    case TABLE_SYMBOL:
                    case TRIGGER_SYMBOL:
                    	{
                    		alt4=3;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 3;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == RENAME_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case TABLES_SYMBOL:
                    case TABLE_SYMBOL:
                    	{
                    		alt4=4;
                    	}
                        break;
                    case USER_SYMBOL:
                    	{
                    		alt4=19;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 4;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == TRUNCATE_SYMBOL))
                {
                    alt4=5;
                }
                else if ( (LA4_0 == CALL_SYMBOL))
                {
                    alt4=6;
                }
                else if ( (LA4_0 == DELETE_SYMBOL))
                {
                    alt4=7;
                }
                else if ( (LA4_0 == DO_SYMBOL))
                {
                    alt4=8;
                }
                else if ( (LA4_0 == HANDLER_SYMBOL))
                {
                    alt4=9;
                }
                else if ( (LA4_0 == INSERT_SYMBOL))
                {
                    alt4=10;
                }
                else if ( (LA4_0 == LOAD_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case INDEX_SYMBOL:
                    	{
                    		alt4=24;
                    	}
                        break;
                    case DATA_SYMBOL:
                    	{
                    		{
                    		    int LA4_111 = LA(3);
                    		    if ( (!( ((SERVER_VERSION < 50500)) )))
                    		    {
                    		        alt4=11;
                    		    }
                    		    else if ( ((SERVER_VERSION < 50500)))
                    		    {
                    		        alt4=17;
                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 4;
                    		        EXCEPTION->state        = 111;


                    		        goto rulestatementEx;

                    		    }
                    		}
                    	}
                        break;
                    case XML_SYMBOL:
                    	{
                    		alt4=11;
                    	}
                        break;
                    case TABLE_SYMBOL:
                    	{
                    		alt4=17;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 11;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == REPLACE_SYMBOL))
                {
                    alt4=12;
                }
                else if ( (LA4_0 == OPEN_PAR_SYMBOL || LA4_0 == SELECT_SYMBOL))
                {
                    alt4=13;
                }
                else if ( (LA4_0 == UPDATE_SYMBOL))
                {
                    alt4=14;
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA4_0 == PARTITION_SYMBOL))
                {
                    alt4=15;
                }
                else if ( (LA4_0 == START_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case TRANSACTION_SYMBOL:
                    	{
                    		alt4=16;
                    	}
                        break;
                    case GROUP_REPLICATION_SYMBOL:
                    case SLAVE_SYMBOL:
                    	{
                    		alt4=17;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 4;
                        EXCEPTION->state        = 17;


                        goto rulestatementEx;

                    }

                }
                else if ( (LA4_0 == COMMIT_SYMBOL || ((LA4_0 >= ROLLBACK_SYMBOL) && (LA4_0 <= SAVEPOINT_SYMBOL)) || LA4_0 == XA_SYMBOL || LA4_0 == LOCK_SYMBOL || LA4_0 == RELEASE_SYMBOL || LA4_0 == UNLOCK_SYMBOL))
                {
                    alt4=16;
                }
                else if ( (LA4_0 == STOP_SYMBOL || LA4_0 == CHANGE_SYMBOL || LA4_0 == PURGE_SYMBOL))
                {
                    alt4=17;
                }
                else if ( (LA4_0 == DEALLOCATE_SYMBOL || LA4_0 == EXECUTE_SYMBOL || LA4_0 == PREPARE_SYMBOL))
                {
                    alt4=18;
                }
                else if ( (LA4_0 == GRANT_SYMBOL || LA4_0 == REVOKE_SYMBOL))
                {
                    alt4=19;
                }
                else if ( (LA4_0 == SET_SYMBOL))
                {
                    {
                        int LA4_33 = LA(2);
                        if ( (LA4_33 == PASSWORD_SYMBOL))
                        {
                            {
                                int LA4_117 = LA(3);
                                if ( (!( ((LA(1) == SET_SYMBOL && LA(2) != PASSWORD_SYMBOL)) )))
                                {
                                    alt4=19;
                                }
                                else if ( ((LA(1) == SET_SYMBOL && LA(2) != PASSWORD_SYMBOL)))
                                {
                                    alt4=22;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 4;
                                    EXCEPTION->state        = 117;


                                    goto rulestatementEx;

                                }
                            }
                        }
                        else if ( (((LA4_33 >= ACCOUNT_SYMBOL) && (LA4_33 <= PARTITIONS_SYMBOL)) || ((LA4_33 >= PHASE_SYMBOL) && (LA4_33 <= YEAR_SYMBOL)) || LA4_33 == CHAR_SYMBOL || LA4_33 == DEFAULT_SYMBOL || ((LA4_33 >= AT_AT_SIGN_SYMBOL) && (LA4_33 <= AT_TEXT_SUFFIX)) || LA4_33 == BACK_TICK_QUOTED_ID || LA4_33 == DOUBLE_QUOTED_TEXT || LA4_33 == IDENTIFIER))
                        {
                            alt4=22;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 4;
                            EXCEPTION->state        = 33;


                            goto rulestatementEx;

                        }
                    }
                }
                else if ( (LA4_0 == BACKUP_SYMBOL || LA4_0 == CHECKSUM_SYMBOL || LA4_0 == REPAIR_SYMBOL || LA4_0 == RESTORE_SYMBOL || LA4_0 == ANALYZE_SYMBOL || LA4_0 == CHECK_SYMBOL || LA4_0 == OPTIMIZE_SYMBOL))
                {
                    alt4=20;
                }
                else if ( (LA4_0 == INSTALL_SYMBOL || LA4_0 == UNINSTALL_SYMBOL))
                {
                    alt4=21;
                }
                else if ( (LA4_0 == SHOW_SYMBOL))
                {
                    alt4=23;
                }
                else if ( (LA4_0 == CACHE_SYMBOL || LA4_0 == FLUSH_SYMBOL || LA4_0 == RESET_SYMBOL || LA4_0 == BINLOG_SYMBOL || LA4_0 == SHUTDOWN_SYMBOL || LA4_0 == KILL_SYMBOL))
                {
                    alt4=24;
                }
                else if ( (LA4_0 == HELP_SYMBOL || ((LA4_0 >= DESCRIBE_SYMBOL) && (LA4_0 <= DESC_SYMBOL)) || LA4_0 == USE_SYMBOL))
                {
                    alt4=25;
                }
                else if ( (LA4_0 == GET_SYMBOL))
                {
                    alt4=26;
                }
                else if ( (LA4_0 == SIGNAL_SYMBOL))
                {
                    alt4=27;
                }
                else if ( (LA4_0 == RESIGNAL_SYMBOL))
                {
                    alt4=28;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 4;
                    EXCEPTION->state        = 0;


                    goto rulestatementEx;

                }
            }
            switch (alt4)
            {
        	case 1:
        	    // MySQLSimpleParser.g:93:2: alter_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_alter_statement_in_statement119);
        	        alter_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:94:4: create_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_create_statement_in_statement124);
        	        create_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:95:4: drop_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_drop_statement_in_statement129);
        	        drop_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:96:4: rename_table_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_rename_table_statement_in_statement134);
        	        rename_table_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:97:4: truncate_table_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_truncate_table_statement_in_statement139);
        	        truncate_table_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:100:4: call_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_call_statement_in_statement148);
        	        call_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:101:4: delete_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_delete_statement_in_statement153);
        	        delete_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:102:4: do_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_do_statement_in_statement158);
        	        do_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:103:4: handler_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_handler_statement_in_statement163);
        	        handler_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:104:4: insert_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_insert_statement_in_statement168);
        	        insert_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:105:4: load_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_load_statement_in_statement173);
        	        load_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:106:4: replace_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_replace_statement_in_statement178);
        	        replace_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:107:4: select_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_select_statement_in_statement183);
        	        select_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:108:4: update_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_update_statement_in_statement188);
        	        update_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:111:4: partitioning
        	    {
        	        FOLLOWPUSH(FOLLOW_partitioning_in_statement196);
        	        partitioning(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:114:4: transaction_or_locking_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_transaction_or_locking_statement_in_statement206);
        	        transaction_or_locking_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:116:4: replication_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_replication_statement_in_statement213);
        	        replication_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:118:4: prepared_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_prepared_statement_in_statement220);
        	        prepared_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:121:4: account_management_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_account_management_statement_in_statement229);
        	        account_management_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:122:4: table_administration_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_table_administration_statement_in_statement234);
        	        table_administration_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:123:4: install_uninstall_statment
        	    {
        	        FOLLOWPUSH(FOLLOW_install_uninstall_statment_in_statement239);
        	        install_uninstall_statment(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 22:
        	    // MySQLSimpleParser.g:124:4: {...}? set_statement
        	    {
        	        if ( !((LA(1) == SET_SYMBOL && LA(2) != PASSWORD_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == SET_SYMBOL && LA(2) != PASSWORD_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_set_statement_in_statement246);
        	        set_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 23:
        	    // MySQLSimpleParser.g:125:4: show_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_show_statement_in_statement252);
        	        show_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 24:
        	    // MySQLSimpleParser.g:126:4: other_administrative_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_other_administrative_statement_in_statement257);
        	        other_administrative_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 25:
        	    // MySQLSimpleParser.g:129:4: utility_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_utility_statement_in_statement266);
        	        utility_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 26:
        	    // MySQLSimpleParser.g:131:4: {...}? get_diagnostics
        	    {
        	        if ( !((SERVER_VERSION >= 50604)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50604";
        	                EXCEPTION->ruleName	 = (void *)"statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_get_diagnostics_in_statement275);
        	        get_diagnostics(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 27:
        	    // MySQLSimpleParser.g:132:4: {...}? signal_statement
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_signal_statement_in_statement282);
        	        signal_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 28:
        	    // MySQLSimpleParser.g:133:4: {...}? resignal_statement
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_resignal_statement_in_statement289);
        	        resignal_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestatementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulestatementEx; /* Prevent compiler warnings */
    rulestatementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end statement */

/**
 * $ANTLR start alter_statement
 * MySQLSimpleParser.g:138:1: alter_statement : ALTER_SYMBOL ( alter_database | alter_log_file_group | FUNCTION_SYMBOL function_ref ( routine_alter_options )? | PROCEDURE_SYMBOL procedure_ref ( routine_alter_options )? | alter_server | alter_table | alter_tablespace |{...}? alter_event | alter_view ) ;
 */
static void
alter_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:138:16: ( ALTER_SYMBOL ( alter_database | alter_log_file_group | FUNCTION_SYMBOL function_ref ( routine_alter_options )? | PROCEDURE_SYMBOL procedure_ref ( routine_alter_options )? | alter_server | alter_table | alter_tablespace |{...}? alter_event | alter_view ) )
        // MySQLSimpleParser.g:139:3: ALTER_SYMBOL ( alter_database | alter_log_file_group | FUNCTION_SYMBOL function_ref ( routine_alter_options )? | PROCEDURE_SYMBOL procedure_ref ( routine_alter_options )? | alter_server | alter_table | alter_tablespace |{...}? alter_event | alter_view )
        {
             MATCHT(ALTER_SYMBOL, &FOLLOW_ALTER_SYMBOL_in_alter_statement301);
            if  (HASEXCEPTION())
            {
                goto rulealter_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:140:3: ( alter_database | alter_log_file_group | FUNCTION_SYMBOL function_ref ( routine_alter_options )? | PROCEDURE_SYMBOL procedure_ref ( routine_alter_options )? | alter_server | alter_table | alter_tablespace |{...}? alter_event | alter_view )
            {
                int alt7=9;
                switch ( LA(1) )
                {
                case DATABASE_SYMBOL:
                	{
                		alt7=1;
                	}
                    break;
                case LOGFILE_SYMBOL:
                	{
                		alt7=2;
                	}
                    break;
                case FUNCTION_SYMBOL:
                	{
                		alt7=3;
                	}
                    break;
                case PROCEDURE_SYMBOL:
                	{
                		alt7=4;
                	}
                    break;
                case SERVER_SYMBOL:
                	{
                		alt7=5;
                	}
                    break;
                case IGNORE_SYMBOL:
                case OFFLINE_SYMBOL:
                case ONLINE_SYMBOL:
                case TABLE_SYMBOL:
                	{
                		alt7=6;
                	}
                    break;
                case TABLESPACE_SYMBOL:
                	{
                		alt7=7;
                	}
                    break;
                case DEFINER_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case EQUAL_OPERATOR:
                			{
                				{
                				    int LA7_15 = LA(3);
                				    if ( ((SERVER_VERSION >= 50100)))
                				    {
                				        alt7=8;
                				    }
                				    else if ( (ANTLR3_TRUE))
                				    {
                				        alt7=9;
                				    }
                				    else
                				    {
                				        if (BACKTRACKING>0)
                				        {
                				            FAILEDFLAG = ANTLR3_TRUE;
                				            return ;
                				        }


                				        CONSTRUCTEX();
                				        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                				        EXCEPTION->message      = (void *)"";
                				        EXCEPTION->decisionNum  = 7;
                				        EXCEPTION->state        = 15;


                				        goto rulealter_statementEx;

                				    }
                				}
                			}
                		    break;

                		default:
                		    if (BACKTRACKING>0)
                		    {
                		        FAILEDFLAG = ANTLR3_TRUE;
                		        return ;
                		    }

                		    CONSTRUCTEX();
                		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		    EXCEPTION->message      = (void *)"";
                		    EXCEPTION->decisionNum  = 7;
                		    EXCEPTION->state        = 10;


                		    goto rulealter_statementEx;

                		}

                	}
                    break;
                case EVENT_SYMBOL:
                	{
                		alt7=8;
                	}
                    break;
                case ALGORITHM_SYMBOL:
                case VIEW_SYMBOL:
                case SQL_SYMBOL:
                	{
                		alt7=9;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 7;
                    EXCEPTION->state        = 0;


                    goto rulealter_statementEx;

                }

                switch (alt7)
                {
            	case 1:
            	    // MySQLSimpleParser.g:141:2: alter_database
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_database_in_alter_statement308);
            	        alter_database(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:142:4: alter_log_file_group
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_log_file_group_in_alter_statement313);
            	        alter_log_file_group(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:143:4: FUNCTION_SYMBOL function_ref ( routine_alter_options )?
            	    {
            	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_alter_statement318);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_function_ref_in_alter_statement320);
            	        function_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:143:33: ( routine_alter_options )?
            	        {
            	            int alt5=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMENT_SYMBOL:
            	                case CONTAINS_SYMBOL:
            	                case LANGUAGE_SYMBOL:
            	                case NO_SYMBOL:
            	                case DETERMINISTIC_SYMBOL:
            	                case MODIFIES_SYMBOL:
            	                case NOT_SYMBOL:
            	                case READS_SYMBOL:
            	                case SQL_SYMBOL:
            	                	{
            	                		alt5=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt5)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:143:33: routine_alter_options
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_routine_alter_options_in_alter_statement322);
            	        	        routine_alter_options(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:144:4: PROCEDURE_SYMBOL procedure_ref ( routine_alter_options )?
            	    {
            	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_alter_statement328);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_procedure_ref_in_alter_statement330);
            	        procedure_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:144:35: ( routine_alter_options )?
            	        {
            	            int alt6=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMENT_SYMBOL:
            	                case CONTAINS_SYMBOL:
            	                case LANGUAGE_SYMBOL:
            	                case NO_SYMBOL:
            	                case DETERMINISTIC_SYMBOL:
            	                case MODIFIES_SYMBOL:
            	                case NOT_SYMBOL:
            	                case READS_SYMBOL:
            	                case SQL_SYMBOL:
            	                	{
            	                		alt6=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt6)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:144:35: routine_alter_options
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_routine_alter_options_in_alter_statement332);
            	        	        routine_alter_options(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:145:4: alter_server
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_server_in_alter_statement338);
            	        alter_server(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:146:4: alter_table
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_table_in_alter_statement343);
            	        alter_table(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 7:
            	    // MySQLSimpleParser.g:147:4: alter_tablespace
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_tablespace_in_alter_statement348);
            	        alter_tablespace(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 8:
            	    // MySQLSimpleParser.g:148:4: {...}? alter_event
            	    {
            	        if ( !((SERVER_VERSION >= 50100)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
            	                EXCEPTION->ruleName	 = (void *)"alter_statement";


            	        }

            	        FOLLOWPUSH(FOLLOW_alter_event_in_alter_statement355);
            	        alter_event(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 9:
            	    // MySQLSimpleParser.g:149:4: alter_view
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_view_in_alter_statement360);
            	        alter_view(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_statementEx; /* Prevent compiler warnings */
    rulealter_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_statement */

/**
 * $ANTLR start alter_database
 * MySQLSimpleParser.g:153:1: alter_database : DATABASE_SYMBOL schema_ref ( ( database_option )+ | UPGRADE_SYMBOL DATA_SYMBOL DIRECTORY_SYMBOL NAME_SYMBOL ) ;
 */
static void
alter_database(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:153:15: ( DATABASE_SYMBOL schema_ref ( ( database_option )+ | UPGRADE_SYMBOL DATA_SYMBOL DIRECTORY_SYMBOL NAME_SYMBOL ) )
        // MySQLSimpleParser.g:154:2: DATABASE_SYMBOL schema_ref ( ( database_option )+ | UPGRADE_SYMBOL DATA_SYMBOL DIRECTORY_SYMBOL NAME_SYMBOL )
        {
             MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_alter_database373);
            if  (HASEXCEPTION())
            {
                goto rulealter_databaseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_schema_ref_in_alter_database375);
            schema_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_databaseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:155:2: ( ( database_option )+ | UPGRADE_SYMBOL DATA_SYMBOL DIRECTORY_SYMBOL NAME_SYMBOL )
            {
                int alt9=2;
                switch ( LA(1) )
                {
                case CHARSET_SYMBOL:
                case CHAR_SYMBOL:
                case COLLATE_SYMBOL:
                case DEFAULT_SYMBOL:
                	{
                		alt9=1;
                	}
                    break;
                case UPGRADE_SYMBOL:
                	{
                		alt9=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 9;
                    EXCEPTION->state        = 0;


                    goto rulealter_databaseEx;

                }

                switch (alt9)
                {
            	case 1:
            	    // MySQLSimpleParser.g:156:3: ( database_option )+
            	    {
            	        // MySQLSimpleParser.g:156:3: ( database_option )+
            	        {
            	            int cnt8=0;

            	            for (;;)
            	            {
            	                int alt8=2;
            	        	switch ( LA(1) )
            	        	{
            	        	case CHARSET_SYMBOL:
            	        	case CHAR_SYMBOL:
            	        	case COLLATE_SYMBOL:
            	        	case DEFAULT_SYMBOL:
            	        		{
            	        			alt8=1;
            	        		}
            	        	    break;

            	        	}

            	        	switch (alt8)
            	        	{
            	        	    case 1:
            	        	        // MySQLSimpleParser.g:156:3: database_option
            	        	        {
            	        	            FOLLOWPUSH(FOLLOW_database_option_in_alter_database382);
            	        	            database_option(ctx);

            	        	            FOLLOWPOP();
            	        	            if  (HASEXCEPTION())
            	        	            {
            	        	                goto rulealter_databaseEx;
            	        	            }
            	        	            if (HASFAILED())
            	        	            {
            	        	                return ;
            	        	            }


            	        	        }
            	        	        break;

            	        	    default:

            	        		if ( cnt8 >= 1 )
            	        		{
            	        		    goto loop8;
            	        		}
            	        		if (BACKTRACKING>0)
            	        		{
            	        		    FAILEDFLAG = ANTLR3_TRUE;
            	        		    return ;
            	        		}

            	        		/* mismatchedSetEx()
            	        		 */
            	        		CONSTRUCTEX();
            	        		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            	        		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            	        		goto rulealter_databaseEx;
            	        	}
            	        	cnt8++;
            	            }
            	            loop8: ;	/* Jump to here if this rule does not match */
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:157:5: UPGRADE_SYMBOL DATA_SYMBOL DIRECTORY_SYMBOL NAME_SYMBOL
            	    {
            	         MATCHT(UPGRADE_SYMBOL, &FOLLOW_UPGRADE_SYMBOL_in_alter_database389);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_databaseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_alter_database391);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_databaseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DIRECTORY_SYMBOL, &FOLLOW_DIRECTORY_SYMBOL_in_alter_database393);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_databaseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(NAME_SYMBOL, &FOLLOW_NAME_SYMBOL_in_alter_database395);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_databaseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_databaseEx; /* Prevent compiler warnings */
    rulealter_databaseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_database */

/**
 * $ANTLR start alter_event
 * MySQLSimpleParser.g:161:1: alter_event : ( definer_clause )? EVENT_SYMBOL event_ref ( ON_SYMBOL SCHEDULE_SYMBOL schedule )? ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( RENAME_SYMBOL TO_SYMBOL identifier )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? ( DO_SYMBOL compound_statement )? ;
 */
static void
alter_event(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:161:12: ( ( definer_clause )? EVENT_SYMBOL event_ref ( ON_SYMBOL SCHEDULE_SYMBOL schedule )? ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( RENAME_SYMBOL TO_SYMBOL identifier )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? ( DO_SYMBOL compound_statement )? )
        // MySQLSimpleParser.g:162:2: ( definer_clause )? EVENT_SYMBOL event_ref ( ON_SYMBOL SCHEDULE_SYMBOL schedule )? ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( RENAME_SYMBOL TO_SYMBOL identifier )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? ( DO_SYMBOL compound_statement )?
        {
            // MySQLSimpleParser.g:162:2: ( definer_clause )?
            {
                int alt10=2;
                switch ( LA(1) )
                {
                    case DEFINER_SYMBOL:
                    	{
                    		alt10=1;
                    	}
                        break;
                }

                switch (alt10)
                {
            	case 1:
            	    // MySQLSimpleParser.g:162:2: definer_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_definer_clause_in_alter_event407);
            	        definer_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EVENT_SYMBOL, &FOLLOW_EVENT_SYMBOL_in_alter_event412);
            if  (HASEXCEPTION())
            {
                goto rulealter_eventEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_event_ref_in_alter_event414);
            event_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_eventEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:164:3: ( ON_SYMBOL SCHEDULE_SYMBOL schedule )?
            {
                int alt11=2;
                switch ( LA(1) )
                {
                    case ON_SYMBOL:
                    	{
                    		switch ( LA(2) )
                    		{
                    		    case SCHEDULE_SYMBOL:
                    		    	{
                    		    		alt11=1;
                    		    	}
                    		        break;
                    		}

                    	}
                        break;
                }

                switch (alt11)
                {
            	case 1:
            	    // MySQLSimpleParser.g:164:4: ON_SYMBOL SCHEDULE_SYMBOL schedule
            	    {
            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_alter_event419);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(SCHEDULE_SYMBOL, &FOLLOW_SCHEDULE_SYMBOL_in_alter_event421);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_schedule_in_alter_event423);
            	        schedule(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:165:3: ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )?
            {
                int alt13=2;
                switch ( LA(1) )
                {
                    case ON_SYMBOL:
                    	{
                    		alt13=1;
                    	}
                        break;
                }

                switch (alt13)
                {
            	case 1:
            	    // MySQLSimpleParser.g:165:4: ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL
            	    {
            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_alter_event430);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(COMPLETION_SYMBOL, &FOLLOW_COMPLETION_SYMBOL_in_alter_event432);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:165:32: ( NOT_SYMBOL )?
            	        {
            	            int alt12=2;
            	            switch ( LA(1) )
            	            {
            	                case NOT_SYMBOL:
            	                	{
            	                		alt12=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt12)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:165:32: NOT_SYMBOL
            	        	    {
            	        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_alter_event434);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_eventEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(PRESERVE_SYMBOL, &FOLLOW_PRESERVE_SYMBOL_in_alter_event437);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:166:3: ( RENAME_SYMBOL TO_SYMBOL identifier )?
            {
                int alt14=2;
                switch ( LA(1) )
                {
                    case RENAME_SYMBOL:
                    	{
                    		alt14=1;
                    	}
                        break;
                }

                switch (alt14)
                {
            	case 1:
            	    // MySQLSimpleParser.g:166:4: RENAME_SYMBOL TO_SYMBOL identifier
            	    {
            	         MATCHT(RENAME_SYMBOL, &FOLLOW_RENAME_SYMBOL_in_alter_event444);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_alter_event446);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_identifier_in_alter_event448);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:167:3: ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )?
            {
                int alt16=3;
                switch ( LA(1) )
                {
                    case ENABLE_SYMBOL:
                    	{
                    		alt16=1;
                    	}
                        break;
                    case DISABLE_SYMBOL:
                    	{
                    		alt16=2;
                    	}
                        break;
                }

                switch (alt16)
                {
            	case 1:
            	    // MySQLSimpleParser.g:167:4: ENABLE_SYMBOL
            	    {
            	         MATCHT(ENABLE_SYMBOL, &FOLLOW_ENABLE_SYMBOL_in_alter_event455);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:167:20: DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )?
            	    {
            	         MATCHT(DISABLE_SYMBOL, &FOLLOW_DISABLE_SYMBOL_in_alter_event459);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:167:35: ( ON_SYMBOL SLAVE_SYMBOL )?
            	        {
            	            int alt15=2;
            	            switch ( LA(1) )
            	            {
            	                case ON_SYMBOL:
            	                	{
            	                		alt15=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt15)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:167:36: ON_SYMBOL SLAVE_SYMBOL
            	        	    {
            	        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_alter_event462);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_eventEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_alter_event464);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_eventEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:168:3: ( COMMENT_SYMBOL string_literal )?
            {
                int alt17=2;
                switch ( LA(1) )
                {
                    case COMMENT_SYMBOL:
                    	{
                    		alt17=1;
                    	}
                        break;
                }

                switch (alt17)
                {
            	case 1:
            	    // MySQLSimpleParser.g:168:4: COMMENT_SYMBOL string_literal
            	    {
            	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_alter_event473);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_alter_event475);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:169:3: ( DO_SYMBOL compound_statement )?
            {
                int alt18=2;
                switch ( LA(1) )
                {
                    case DO_SYMBOL:
                    	{
                    		alt18=1;
                    	}
                        break;
                }

                switch (alt18)
                {
            	case 1:
            	    // MySQLSimpleParser.g:169:4: DO_SYMBOL compound_statement
            	    {
            	         MATCHT(DO_SYMBOL, &FOLLOW_DO_SYMBOL_in_alter_event482);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_compound_statement_in_alter_event484);
            	        compound_statement(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_eventEx; /* Prevent compiler warnings */
    rulealter_eventEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_event */

/**
 * $ANTLR start alter_log_file_group
 * MySQLSimpleParser.g:172:1: alter_log_file_group : LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ADD_SYMBOL UNDOFILE_SYMBOL string_literal ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )? ( WAIT_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref ;
 */
static void
alter_log_file_group(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:172:21: ( LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ADD_SYMBOL UNDOFILE_SYMBOL string_literal ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )? ( WAIT_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref )
        // MySQLSimpleParser.g:173:2: LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ADD_SYMBOL UNDOFILE_SYMBOL string_literal ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )? ( WAIT_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        {
             MATCHT(LOGFILE_SYMBOL, &FOLLOW_LOGFILE_SYMBOL_in_alter_log_file_group495);
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_alter_log_file_group497);
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_logfile_group_ref_in_alter_log_file_group499);
            logfile_group_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_alter_log_file_group501);
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(UNDOFILE_SYMBOL, &FOLLOW_UNDOFILE_SYMBOL_in_alter_log_file_group503);
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_string_literal_in_alter_log_file_group505);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:174:3: ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )?
            {
                int alt20=2;
                switch ( LA(1) )
                {
                    case INITIAL_SIZE_SYMBOL:
                    	{
                    		alt20=1;
                    	}
                        break;
                }

                switch (alt20)
                {
            	case 1:
            	    // MySQLSimpleParser.g:174:4: INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
            	    {
            	         MATCHT(INITIAL_SIZE_SYMBOL, &FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_log_file_group510);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_log_file_groupEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:174:24: ( EQUAL_OPERATOR )?
            	        {
            	            int alt19=2;
            	            switch ( LA(1) )
            	            {
            	                case EQUAL_OPERATOR:
            	                	{
            	                		alt19=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt19)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:174:24: EQUAL_OPERATOR
            	        	    {
            	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group512);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_log_file_groupEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_size_number_in_alter_log_file_group515);
            	        size_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_log_file_groupEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:174:54: ( WAIT_SYMBOL )?
            {
                int alt21=2;
                switch ( LA(1) )
                {
                    case WAIT_SYMBOL:
                    	{
                    		alt21=1;
                    	}
                        break;
                }

                switch (alt21)
                {
            	case 1:
            	    // MySQLSimpleParser.g:174:54: WAIT_SYMBOL
            	    {
            	         MATCHT(WAIT_SYMBOL, &FOLLOW_WAIT_SYMBOL_in_alter_log_file_group519);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_log_file_groupEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_alter_log_file_group522);
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:174:81: ( EQUAL_OPERATOR )?
            {
                int alt22=2;
                switch ( LA(1) )
                {
                    case EQUAL_OPERATOR:
                    	{
                    		alt22=1;
                    	}
                        break;
                }

                switch (alt22)
                {
            	case 1:
            	    // MySQLSimpleParser.g:174:81: EQUAL_OPERATOR
            	    {
            	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_log_file_group524);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_log_file_groupEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_engine_ref_in_alter_log_file_group527);
            engine_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_log_file_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_log_file_groupEx; /* Prevent compiler warnings */
    rulealter_log_file_groupEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_log_file_group */

/**
 * $ANTLR start alter_server
 * MySQLSimpleParser.g:177:1: alter_server : SERVER_SYMBOL server_ref server_options ;
 */
static void
alter_server(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:177:13: ( SERVER_SYMBOL server_ref server_options )
        // MySQLSimpleParser.g:178:2: SERVER_SYMBOL server_ref server_options
        {
             MATCHT(SERVER_SYMBOL, &FOLLOW_SERVER_SYMBOL_in_alter_server536);
            if  (HASEXCEPTION())
            {
                goto rulealter_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_server_ref_in_alter_server538);
            server_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_server_options_in_alter_server540);
            server_options(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_serverEx; /* Prevent compiler warnings */
    rulealter_serverEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_server */

/**
 * $ANTLR start alter_table
 * MySQLSimpleParser.g:181:1: alter_table : ( online_option )? ( IGNORE_SYMBOL )? TABLE_SYMBOL table_ref ( alter_table_commands )? ;
 */
static void
alter_table(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:181:12: ( ( online_option )? ( IGNORE_SYMBOL )? TABLE_SYMBOL table_ref ( alter_table_commands )? )
        // MySQLSimpleParser.g:182:2: ( online_option )? ( IGNORE_SYMBOL )? TABLE_SYMBOL table_ref ( alter_table_commands )?
        {
            // MySQLSimpleParser.g:182:2: ( online_option )?
            {
                int alt23=2;
                switch ( LA(1) )
                {
                    case OFFLINE_SYMBOL:
                    case ONLINE_SYMBOL:
                    	{
                    		alt23=1;
                    	}
                        break;
                }

                switch (alt23)
                {
            	case 1:
            	    // MySQLSimpleParser.g:182:2: online_option
            	    {
            	        FOLLOWPUSH(FOLLOW_online_option_in_alter_table549);
            	        online_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:182:17: ( IGNORE_SYMBOL )?
            {
                int alt24=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    	{
                    		alt24=1;
                    	}
                        break;
                }

                switch (alt24)
                {
            	case 1:
            	    // MySQLSimpleParser.g:182:17: IGNORE_SYMBOL
            	    {
            	         MATCHT(IGNORE_SYMBOL, &FOLLOW_IGNORE_SYMBOL_in_alter_table552);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_alter_table555);
            if  (HASEXCEPTION())
            {
                goto rulealter_tableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_alter_table557);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_tableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:182:55: ( alter_table_commands )?
            {
                int alt25=2;
                {
                    int LA25_0 = LA(1);
                    if ( (((LA25_0 >= CHARSET_SYMBOL) && (LA25_0 <= CHECKSUM_SYMBOL)) || LA25_0 == COMMENT_SYMBOL || LA25_0 == UPGRADE_SYMBOL || LA25_0 == ALGORITHM_SYMBOL || LA25_0 == AUTO_INCREMENT_SYMBOL || LA25_0 == AVG_ROW_LENGTH_SYMBOL || LA25_0 == COMPRESSION_SYMBOL || LA25_0 == CONNECTION_SYMBOL || LA25_0 == DATA_SYMBOL || LA25_0 == DELAY_KEY_WRITE_SYMBOL || ((LA25_0 >= DISABLE_SYMBOL) && (LA25_0 <= DISCARD_SYMBOL)) || LA25_0 == ENGINE_SYMBOL || LA25_0 == ENABLE_SYMBOL || LA25_0 == IMPORT_SYMBOL || LA25_0 == INSERT_METHOD_SYMBOL || LA25_0 == KEY_BLOCK_SIZE_SYMBOL || LA25_0 == MAX_ROWS_SYMBOL || ((LA25_0 >= MIN_ROWS_SYMBOL) && (LA25_0 <= MODIFY_SYMBOL)) || LA25_0 == PACK_KEYS_SYMBOL || LA25_0 == PASSWORD_SYMBOL || LA25_0 == ROW_FORMAT_SYMBOL || ((LA25_0 >= STATS_AUTO_RECALC_SYMBOL) && (LA25_0 <= STATS_SAMPLE_PAGES_SYMBOL)) || LA25_0 == STORAGE_SYMBOL || ((LA25_0 >= TABLE_CHECKSUM_SYMBOL) && (LA25_0 <= TABLESPACE_SYMBOL)) || LA25_0 == TYPE_SYMBOL || LA25_0 == ADD_SYMBOL || LA25_0 == ALTER_SYMBOL || LA25_0 == CHANGE_SYMBOL || LA25_0 == CHAR_SYMBOL || LA25_0 == COLLATE_SYMBOL || LA25_0 == CONVERT_SYMBOL || LA25_0 == DEFAULT_SYMBOL || LA25_0 == DROP_SYMBOL || LA25_0 == FORCE_SYMBOL || LA25_0 == INDEX_SYMBOL || LA25_0 == LOCK_SYMBOL || LA25_0 == ORDER_SYMBOL || LA25_0 == RENAME_SYMBOL || LA25_0 == UNION_SYMBOL))
                    {
                        alt25=1;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA25_0 == PARTITION_SYMBOL || LA25_0 == REMOVE_SYMBOL))
                    {
                        alt25=1;
                    }
                    else if ( (LA25_0 == REPAIR_SYMBOL || LA25_0 == TRUNCATE_SYMBOL || LA25_0 == ANALYZE_SYMBOL || LA25_0 == COALESCE_SYMBOL || LA25_0 == REBUILD_SYMBOL || LA25_0 == REORGANIZE_SYMBOL || LA25_0 == CHECK_SYMBOL || LA25_0 == EXCHANGE_SYMBOL || LA25_0 == OPTIMIZE_SYMBOL))
                    {
                        alt25=1;
                    }
                }
                switch (alt25)
                {
            	case 1:
            	    // MySQLSimpleParser.g:182:55: alter_table_commands
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_table_commands_in_alter_table559);
            	        alter_table_commands(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_tableEx; /* Prevent compiler warnings */
    rulealter_tableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_table */

/**
 * $ANTLR start alter_table_commands
 * MySQLSimpleParser.g:185:1: alter_table_commands : ( DISCARD_SYMBOL TABLESPACE_SYMBOL | IMPORT_SYMBOL TABLESPACE_SYMBOL | alter_table_list_item ( COMMA_SYMBOL alter_table_list_item )* ( ( partitioning )? | remove_partitioning ) | remove_partitioning | partitioning |{...}? alter_partition );
 */
static void
alter_table_commands(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:185:21: ( DISCARD_SYMBOL TABLESPACE_SYMBOL | IMPORT_SYMBOL TABLESPACE_SYMBOL | alter_table_list_item ( COMMA_SYMBOL alter_table_list_item )* ( ( partitioning )? | remove_partitioning ) | remove_partitioning | partitioning |{...}? alter_partition )

            ANTLR3_UINT32 alt29;

            alt29=6;

            {
                int LA29_0 = LA(1);
                if ( (LA29_0 == DISCARD_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case TABLESPACE_SYMBOL:
                    	{
                    		alt29=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt29=6;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 29;
                        EXCEPTION->state        = 1;


                        goto rulealter_table_commandsEx;

                    }

                }
                else if ( (LA29_0 == IMPORT_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case TABLESPACE_SYMBOL:
                    	{
                    		alt29=2;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt29=6;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 29;
                        EXCEPTION->state        = 2;


                        goto rulealter_table_commandsEx;

                    }

                }
                else if ( (((LA29_0 >= CHARSET_SYMBOL) && (LA29_0 <= CHECKSUM_SYMBOL)) || LA29_0 == COMMENT_SYMBOL || LA29_0 == UPGRADE_SYMBOL || LA29_0 == ALGORITHM_SYMBOL || LA29_0 == AUTO_INCREMENT_SYMBOL || LA29_0 == AVG_ROW_LENGTH_SYMBOL || LA29_0 == COMPRESSION_SYMBOL || LA29_0 == CONNECTION_SYMBOL || LA29_0 == DATA_SYMBOL || LA29_0 == DELAY_KEY_WRITE_SYMBOL || LA29_0 == DISABLE_SYMBOL || LA29_0 == ENGINE_SYMBOL || LA29_0 == ENABLE_SYMBOL || LA29_0 == INSERT_METHOD_SYMBOL || LA29_0 == KEY_BLOCK_SIZE_SYMBOL || LA29_0 == MAX_ROWS_SYMBOL || ((LA29_0 >= MIN_ROWS_SYMBOL) && (LA29_0 <= MODIFY_SYMBOL)) || LA29_0 == PACK_KEYS_SYMBOL || LA29_0 == PASSWORD_SYMBOL || LA29_0 == ROW_FORMAT_SYMBOL || ((LA29_0 >= STATS_AUTO_RECALC_SYMBOL) && (LA29_0 <= STATS_SAMPLE_PAGES_SYMBOL)) || LA29_0 == STORAGE_SYMBOL || ((LA29_0 >= TABLE_CHECKSUM_SYMBOL) && (LA29_0 <= TABLESPACE_SYMBOL)) || LA29_0 == TYPE_SYMBOL || LA29_0 == ALTER_SYMBOL || LA29_0 == CHANGE_SYMBOL || LA29_0 == CHAR_SYMBOL || LA29_0 == COLLATE_SYMBOL || LA29_0 == CONVERT_SYMBOL || LA29_0 == DEFAULT_SYMBOL || LA29_0 == FORCE_SYMBOL || LA29_0 == INDEX_SYMBOL || LA29_0 == LOCK_SYMBOL || LA29_0 == ORDER_SYMBOL || LA29_0 == RENAME_SYMBOL || LA29_0 == UNION_SYMBOL))
                {
                    alt29=3;
                }
                else if ( (LA29_0 == ADD_SYMBOL))
                {
                    {
                        int LA29_29 = LA(2);
                        if ( (LA29_29 == PARTITION_SYMBOL))
                        {
                            {
                                int LA29_58 = LA(3);
                                if ( (!( (( (( ((SERVER_VERSION >= 50100)) && ((SERVER_VERSION >= 50100)) )) && ((LA(1) == ADD_SYMBOL && LA(2) == PARTITION_SYMBOL)) )) )))
                                {
                                    alt29=3;
                                }
                                else if ( (( (( ((SERVER_VERSION >= 50100)) && ((SERVER_VERSION >= 50100)) )) && ((LA(1) == ADD_SYMBOL && LA(2) == PARTITION_SYMBOL)) )))
                                {
                                    alt29=6;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 29;
                                    EXCEPTION->state        = 58;


                                    goto rulealter_table_commandsEx;

                                }
                            }
                        }
                        else if ( (((LA29_29 >= ACCOUNT_SYMBOL) && (LA29_29 <= PARSER_SYMBOL)) || ((LA29_29 >= PORT_SYMBOL) && (LA29_29 <= YEAR_SYMBOL)) || LA29_29 == CHECK_SYMBOL || LA29_29 == COLUMN_SYMBOL || LA29_29 == CONSTRAINT_SYMBOL || LA29_29 == FOREIGN_SYMBOL || LA29_29 == FULLTEXT_SYMBOL || LA29_29 == INDEX_SYMBOL || LA29_29 == KEY_SYMBOL || LA29_29 == OPEN_PAR_SYMBOL || LA29_29 == PRIMARY_SYMBOL || LA29_29 == SPATIAL_SYMBOL || LA29_29 == UNIQUE_SYMBOL || LA29_29 == BACK_TICK_QUOTED_ID || LA29_29 == DOUBLE_QUOTED_TEXT || LA29_29 == IDENTIFIER))
                        {
                            alt29=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 29;
                            EXCEPTION->state        = 29;


                            goto rulealter_table_commandsEx;

                        }
                    }
                }
                else if ( (LA29_0 == DROP_SYMBOL))
                {
                    {
                        int LA29_33 = LA(2);
                        if ( (LA29_33 == PARTITION_SYMBOL))
                        {
                            {
                                int LA29_73 = LA(3);
                                if ( (!( ((SERVER_VERSION >= 50100)) )))
                                {
                                    alt29=3;
                                }
                                else if ( ((SERVER_VERSION >= 50100)))
                                {
                                    alt29=6;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 29;
                                    EXCEPTION->state        = 73;


                                    goto rulealter_table_commandsEx;

                                }
                            }
                        }
                        else if ( (((LA29_33 >= ACCOUNT_SYMBOL) && (LA29_33 <= PARSER_SYMBOL)) || ((LA29_33 >= PORT_SYMBOL) && (LA29_33 <= YEAR_SYMBOL)) || LA29_33 == COLUMN_SYMBOL || LA29_33 == FOREIGN_SYMBOL || LA29_33 == INDEX_SYMBOL || LA29_33 == KEY_SYMBOL || LA29_33 == PRIMARY_SYMBOL || LA29_33 == BACK_TICK_QUOTED_ID || LA29_33 == DOT_SYMBOL || LA29_33 == DOUBLE_QUOTED_TEXT || LA29_33 == IDENTIFIER))
                        {
                            alt29=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 29;
                            EXCEPTION->state        = 33;


                            goto rulealter_table_commandsEx;

                        }
                    }
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA29_0 == REMOVE_SYMBOL))
                {
                    alt29=4;
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA29_0 == PARTITION_SYMBOL))
                {
                    alt29=5;
                }
                else if ( (LA29_0 == REPAIR_SYMBOL || LA29_0 == TRUNCATE_SYMBOL || LA29_0 == ANALYZE_SYMBOL || LA29_0 == COALESCE_SYMBOL || LA29_0 == REBUILD_SYMBOL || LA29_0 == REORGANIZE_SYMBOL || LA29_0 == CHECK_SYMBOL || LA29_0 == EXCHANGE_SYMBOL || LA29_0 == OPTIMIZE_SYMBOL))
                {
                    alt29=6;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 29;
                    EXCEPTION->state        = 0;


                    goto rulealter_table_commandsEx;

                }
            }
            switch (alt29)
            {
        	case 1:
        	    // MySQLSimpleParser.g:186:2: DISCARD_SYMBOL TABLESPACE_SYMBOL
        	    {
        	         MATCHT(DISCARD_SYMBOL, &FOLLOW_DISCARD_SYMBOL_in_alter_table_commands569);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands571);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:187:4: IMPORT_SYMBOL TABLESPACE_SYMBOL
        	    {
        	         MATCHT(IMPORT_SYMBOL, &FOLLOW_IMPORT_SYMBOL_in_alter_table_commands576);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_alter_table_commands578);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:188:4: alter_table_list_item ( COMMA_SYMBOL alter_table_list_item )* ( ( partitioning )? | remove_partitioning )
        	    {
        	        FOLLOWPUSH(FOLLOW_alter_table_list_item_in_alter_table_commands583);
        	        alter_table_list_item(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:188:26: ( COMMA_SYMBOL alter_table_list_item )*

        	        for (;;)
        	        {
        	            int alt26=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt26=1;
        	            	}
        	                break;

        	            }

        	            switch (alt26)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:188:27: COMMA_SYMBOL alter_table_list_item
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_alter_table_commands586);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_commandsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_alter_table_list_item_in_alter_table_commands588);
        	        	        alter_table_list_item(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_commandsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop26;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop26: ; /* Jump out to here if this rule does not match */


        	        // MySQLSimpleParser.g:188:64: ( ( partitioning )? | remove_partitioning )
        	        {
        	            int alt28=2;
        	            {
        	                int LA28_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50100)) && (LA28_0 == PARTITION_SYMBOL))
        	                {
        	                    alt28=1;
        	                }
        	                else if ( (LA28_0 == EOF || LA28_0 == SEMICOLON_SYMBOL))
        	                {
        	                    alt28=1;
        	                }
        	                else if ( ((SERVER_VERSION >= 50100)) && (LA28_0 == REMOVE_SYMBOL))
        	                {
        	                    alt28=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 28;
        	                    EXCEPTION->state        = 0;


        	                    goto rulealter_table_commandsEx;

        	                }
        	            }
        	            switch (alt28)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:188:65: ( partitioning )?
        	        	    {
        	        	        // MySQLSimpleParser.g:188:65: ( partitioning )?
        	        	        {
        	        	            int alt27=2;
        	        	            {
        	        	                int LA27_0 = LA(1);
        	        	                if ( ((SERVER_VERSION >= 50100)) && (LA27_0 == PARTITION_SYMBOL))
        	        	                {
        	        	                    alt27=1;
        	        	                }
        	        	            }
        	        	            switch (alt27)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:188:65: partitioning
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_partitioning_in_alter_table_commands593);
        	        	        	        partitioning(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_commandsEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:188:81: remove_partitioning
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_remove_partitioning_in_alter_table_commands598);
        	        	        remove_partitioning(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_commandsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:189:4: remove_partitioning
        	    {
        	        FOLLOWPUSH(FOLLOW_remove_partitioning_in_alter_table_commands604);
        	        remove_partitioning(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:190:4: partitioning
        	    {
        	        FOLLOWPUSH(FOLLOW_partitioning_in_alter_table_commands609);
        	        partitioning(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:191:4: {...}? alter_partition
        	    {
        	        if ( !((SERVER_VERSION >= 50100)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
        	                EXCEPTION->ruleName	 = (void *)"alter_table_commands";


        	        }

        	        FOLLOWPUSH(FOLLOW_alter_partition_in_alter_table_commands616);
        	        alter_partition(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_commandsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulealter_table_commandsEx; /* Prevent compiler warnings */
    rulealter_table_commandsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_table_commands */

/**
 * $ANTLR start alter_table_list_item
 * MySQLSimpleParser.g:194:1: alter_table_list_item : alter_table_list_entry ;
 */
static void
alter_table_list_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:194:22: ( alter_table_list_entry )
        // MySQLSimpleParser.g:195:2: alter_table_list_entry
        {
            FOLLOWPUSH(FOLLOW_alter_table_list_entry_in_alter_table_list_item625);
            alter_table_list_entry(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_table_list_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_table_list_itemEx; /* Prevent compiler warnings */
    rulealter_table_list_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_table_list_item */

/**
 * $ANTLR start alter_table_list_entry
 * MySQLSimpleParser.g:198:1: alter_table_list_entry : ( alter_table_options | ADD_SYMBOL ( COLUMN_SYMBOL )? ( column_definition ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | OPEN_PAR_SYMBOL column_definition ( COMMA_SYMBOL column_definition )* CLOSE_PAR_SYMBOL ) | ADD_SYMBOL key_def | ALTER_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref ( SET_SYMBOL DEFAULT_SYMBOL signed_literal | DROP_SYMBOL DEFAULT_SYMBOL ) | CHANGE_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref field_spec ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | MODIFY_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref data_type ( attribute )* ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | DROP_SYMBOL ( ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref | ( COLUMN_SYMBOL )? column_internal_ref | PRIMARY_SYMBOL KEY_SYMBOL | FOREIGN_SYMBOL KEY_SYMBOL ({...}? => column_ref |{...}? => ( column_ref )? ) ) | DISABLE_SYMBOL KEYS_SYMBOL | ENABLE_SYMBOL KEYS_SYMBOL | RENAME_SYMBOL ( TO_SYMBOL | AS_SYMBOL )? table_ref |{...}? RENAME_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref TO_SYMBOL column_ref | alter_order_by | CONVERT_SYMBOL TO_SYMBOL charset charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? | FORCE_SYMBOL |{...}? alter_algorithm_option |{...}? alter_lock_option |{...}? UPGRADE_SYMBOL PARTITIONING_SYMBOL );
 */
static void
alter_table_list_entry(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:198:23: ( alter_table_options | ADD_SYMBOL ( COLUMN_SYMBOL )? ( column_definition ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | OPEN_PAR_SYMBOL column_definition ( COMMA_SYMBOL column_definition )* CLOSE_PAR_SYMBOL ) | ADD_SYMBOL key_def | ALTER_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref ( SET_SYMBOL DEFAULT_SYMBOL signed_literal | DROP_SYMBOL DEFAULT_SYMBOL ) | CHANGE_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref field_spec ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | MODIFY_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref data_type ( attribute )* ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | DROP_SYMBOL ( ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref | ( COLUMN_SYMBOL )? column_internal_ref | PRIMARY_SYMBOL KEY_SYMBOL | FOREIGN_SYMBOL KEY_SYMBOL ({...}? => column_ref |{...}? => ( column_ref )? ) ) | DISABLE_SYMBOL KEYS_SYMBOL | ENABLE_SYMBOL KEYS_SYMBOL | RENAME_SYMBOL ( TO_SYMBOL | AS_SYMBOL )? table_ref |{...}? RENAME_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref TO_SYMBOL column_ref | alter_order_by | CONVERT_SYMBOL TO_SYMBOL charset charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? | FORCE_SYMBOL |{...}? alter_algorithm_option |{...}? alter_lock_option |{...}? UPGRADE_SYMBOL PARTITIONING_SYMBOL )

            ANTLR3_UINT32 alt47;

            alt47=17;

            {
                int LA47_0 = LA(1);
                if ( (((LA47_0 >= CHARSET_SYMBOL) && (LA47_0 <= CHECKSUM_SYMBOL)) || LA47_0 == COMMENT_SYMBOL || LA47_0 == AUTO_INCREMENT_SYMBOL || LA47_0 == AVG_ROW_LENGTH_SYMBOL || LA47_0 == COMPRESSION_SYMBOL || LA47_0 == CONNECTION_SYMBOL || LA47_0 == DATA_SYMBOL || LA47_0 == DELAY_KEY_WRITE_SYMBOL || LA47_0 == ENGINE_SYMBOL || LA47_0 == INSERT_METHOD_SYMBOL || LA47_0 == KEY_BLOCK_SIZE_SYMBOL || LA47_0 == MAX_ROWS_SYMBOL || LA47_0 == MIN_ROWS_SYMBOL || LA47_0 == PACK_KEYS_SYMBOL || LA47_0 == PASSWORD_SYMBOL || LA47_0 == ROW_FORMAT_SYMBOL || ((LA47_0 >= STATS_AUTO_RECALC_SYMBOL) && (LA47_0 <= STATS_SAMPLE_PAGES_SYMBOL)) || LA47_0 == STORAGE_SYMBOL || ((LA47_0 >= TABLE_CHECKSUM_SYMBOL) && (LA47_0 <= TABLESPACE_SYMBOL)) || LA47_0 == TYPE_SYMBOL || LA47_0 == CHAR_SYMBOL || LA47_0 == COLLATE_SYMBOL || LA47_0 == DEFAULT_SYMBOL || LA47_0 == INDEX_SYMBOL || LA47_0 == UNION_SYMBOL))
                {
                    alt47=1;
                }
                else if ( (LA47_0 == ADD_SYMBOL))
                {
                    {
                        int LA47_27 = LA(2);
                        if ( (((LA47_27 >= ACCOUNT_SYMBOL) && (LA47_27 <= YEAR_SYMBOL)) || LA47_27 == COLUMN_SYMBOL || LA47_27 == OPEN_PAR_SYMBOL || LA47_27 == BACK_TICK_QUOTED_ID || LA47_27 == DOUBLE_QUOTED_TEXT || LA47_27 == IDENTIFIER))
                        {
                            alt47=2;
                        }
                        else if ( (LA47_27 == CHECK_SYMBOL || LA47_27 == CONSTRAINT_SYMBOL || LA47_27 == FOREIGN_SYMBOL || LA47_27 == FULLTEXT_SYMBOL || LA47_27 == INDEX_SYMBOL || LA47_27 == KEY_SYMBOL || LA47_27 == PRIMARY_SYMBOL || LA47_27 == SPATIAL_SYMBOL || LA47_27 == UNIQUE_SYMBOL))
                        {
                            alt47=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 47;
                            EXCEPTION->state        = 27;


                            goto rulealter_table_list_entryEx;

                        }
                    }
                }
                else if ( (LA47_0 == ALTER_SYMBOL))
                {
                    alt47=4;
                }
                else if ( (LA47_0 == CHANGE_SYMBOL))
                {
                    alt47=5;
                }
                else if ( (LA47_0 == MODIFY_SYMBOL))
                {
                    alt47=6;
                }
                else if ( (LA47_0 == DROP_SYMBOL))
                {
                    alt47=7;
                }
                else if ( (LA47_0 == DISABLE_SYMBOL))
                {
                    alt47=8;
                }
                else if ( (LA47_0 == ENABLE_SYMBOL))
                {
                    alt47=9;
                }
                else if ( (LA47_0 == RENAME_SYMBOL))
                {
                    {
                        int LA47_34 = LA(2);
                        if ( (LA47_34 == INDEX_SYMBOL || LA47_34 == KEY_SYMBOL))
                        {
                            alt47=11;
                        }
                        else if ( (((LA47_34 >= ACCOUNT_SYMBOL) && (LA47_34 <= YEAR_SYMBOL)) || LA47_34 == TO_SYMBOL || LA47_34 == AS_SYMBOL || LA47_34 == BACK_TICK_QUOTED_ID || LA47_34 == DOT_SYMBOL || LA47_34 == DOUBLE_QUOTED_TEXT || LA47_34 == IDENTIFIER))
                        {
                            alt47=10;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 47;
                            EXCEPTION->state        = 34;


                            goto rulealter_table_list_entryEx;

                        }
                    }
                }
                else if ( (LA47_0 == ORDER_SYMBOL))
                {
                    alt47=12;
                }
                else if ( (LA47_0 == CONVERT_SYMBOL))
                {
                    alt47=13;
                }
                else if ( (LA47_0 == FORCE_SYMBOL))
                {
                    alt47=14;
                }
                else if ( (LA47_0 == ALGORITHM_SYMBOL))
                {
                    alt47=15;
                }
                else if ( (LA47_0 == LOCK_SYMBOL))
                {
                    alt47=16;
                }
                else if ( (LA47_0 == UPGRADE_SYMBOL))
                {
                    alt47=17;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 47;
                    EXCEPTION->state        = 0;


                    goto rulealter_table_list_entryEx;

                }
            }
            switch (alt47)
            {
        	case 1:
        	    // MySQLSimpleParser.g:199:2: alter_table_options
        	    {
        	        FOLLOWPUSH(FOLLOW_alter_table_options_in_alter_table_list_entry634);
        	        alter_table_options(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:200:4: ADD_SYMBOL ( COLUMN_SYMBOL )? ( column_definition ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | OPEN_PAR_SYMBOL column_definition ( COMMA_SYMBOL column_definition )* CLOSE_PAR_SYMBOL )
        	    {
        	         MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_alter_table_list_entry639);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:200:15: ( COLUMN_SYMBOL )?
        	        {
        	            int alt30=2;
        	            switch ( LA(1) )
        	            {
        	                case COLUMN_SYMBOL:
        	                	{
        	                		alt30=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt30)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:200:15: COLUMN_SYMBOL
        	        	    {
        	        	         MATCHT(COLUMN_SYMBOL, &FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry641);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:201:3: ( column_definition ( FIRST_SYMBOL | AFTER_SYMBOL identifier )? | OPEN_PAR_SYMBOL column_definition ( COMMA_SYMBOL column_definition )* CLOSE_PAR_SYMBOL )
        	        {
        	            int alt33=2;
        	            {
        	                int LA33_0 = LA(1);
        	                if ( (((LA33_0 >= ACCOUNT_SYMBOL) && (LA33_0 <= YEAR_SYMBOL)) || LA33_0 == BACK_TICK_QUOTED_ID || LA33_0 == DOUBLE_QUOTED_TEXT || LA33_0 == IDENTIFIER))
        	                {
        	                    alt33=1;
        	                }
        	                else if ( (LA33_0 == OPEN_PAR_SYMBOL))
        	                {
        	                    alt33=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 33;
        	                    EXCEPTION->state        = 0;


        	                    goto rulealter_table_list_entryEx;

        	                }
        	            }
        	            switch (alt33)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:202:4: column_definition ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_column_definition_in_alter_table_list_entry651);
        	        	        column_definition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:202:22: ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	        	        {
        	        	            int alt31=3;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case FIRST_SYMBOL:
        	        	                	{
        	        	                		alt31=1;
        	        	                	}
        	        	                    break;
        	        	                case AFTER_SYMBOL:
        	        	                	{
        	        	                		alt31=2;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt31)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:202:23: FIRST_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(FIRST_SYMBOL, &FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry654);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;
        	        	        	case 2:
        	        	        	    // MySQLSimpleParser.g:202:38: AFTER_SYMBOL identifier
        	        	        	    {
        	        	        	         MATCHT(AFTER_SYMBOL, &FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry658);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_identifier_in_alter_table_list_entry660);
        	        	        	        identifier(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:203:6: OPEN_PAR_SYMBOL column_definition ( COMMA_SYMBOL column_definition )* CLOSE_PAR_SYMBOL
        	        	    {
        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_alter_table_list_entry669);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_column_definition_in_alter_table_list_entry671);
        	        	        column_definition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:203:40: ( COMMA_SYMBOL column_definition )*

        	        	        for (;;)
        	        	        {
        	        	            int alt32=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	            case COMMA_SYMBOL:
        	        	            	{
        	        	            		alt32=1;
        	        	            	}
        	        	                break;

        	        	            }

        	        	            switch (alt32)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:203:41: COMMA_SYMBOL column_definition
        	        	        	    {
        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_alter_table_list_entry674);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_column_definition_in_alter_table_list_entry676);
        	        	        	        column_definition(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	        	default:
        	        	        	    goto loop32;	/* break out of the loop */
        	        	        	    break;
        	        	            }
        	        	        }
        	        	        loop32: ; /* Jump out to here if this rule does not match */


        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_alter_table_list_entry680);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:205:4: ADD_SYMBOL key_def
        	    {
        	         MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_alter_table_list_entry689);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_key_def_in_alter_table_list_entry691);
        	        key_def(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:206:4: ALTER_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref ( SET_SYMBOL DEFAULT_SYMBOL signed_literal | DROP_SYMBOL DEFAULT_SYMBOL )
        	    {
        	         MATCHT(ALTER_SYMBOL, &FOLLOW_ALTER_SYMBOL_in_alter_table_list_entry696);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:206:17: ( COLUMN_SYMBOL )?
        	        {
        	            int alt34=2;
        	            switch ( LA(1) )
        	            {
        	                case COLUMN_SYMBOL:
        	                	{
        	                		alt34=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt34)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:206:17: COLUMN_SYMBOL
        	        	    {
        	        	         MATCHT(COLUMN_SYMBOL, &FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry698);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_column_internal_ref_in_alter_table_list_entry701);
        	        column_internal_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:206:52: ( SET_SYMBOL DEFAULT_SYMBOL signed_literal | DROP_SYMBOL DEFAULT_SYMBOL )
        	        {
        	            int alt35=2;
        	            switch ( LA(1) )
        	            {
        	            case SET_SYMBOL:
        	            	{
        	            		alt35=1;
        	            	}
        	                break;
        	            case DROP_SYMBOL:
        	            	{
        	            		alt35=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 35;
        	                EXCEPTION->state        = 0;


        	                goto rulealter_table_list_entryEx;

        	            }

        	            switch (alt35)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:206:53: SET_SYMBOL DEFAULT_SYMBOL signed_literal
        	        	    {
        	        	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_alter_table_list_entry704);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry706);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_signed_literal_in_alter_table_list_entry708);
        	        	        signed_literal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:206:96: DROP_SYMBOL DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_alter_table_list_entry712);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_alter_table_list_entry714);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:207:4: CHANGE_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref field_spec ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	    {
        	         MATCHT(CHANGE_SYMBOL, &FOLLOW_CHANGE_SYMBOL_in_alter_table_list_entry720);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:207:18: ( COLUMN_SYMBOL )?
        	        {
        	            int alt36=2;
        	            switch ( LA(1) )
        	            {
        	                case COLUMN_SYMBOL:
        	                	{
        	                		alt36=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt36)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:207:18: COLUMN_SYMBOL
        	        	    {
        	        	         MATCHT(COLUMN_SYMBOL, &FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry722);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_column_internal_ref_in_alter_table_list_entry725);
        	        column_internal_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_field_spec_in_alter_table_list_entry727);
        	        field_spec(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:207:64: ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	        {
        	            int alt37=3;
        	            switch ( LA(1) )
        	            {
        	                case FIRST_SYMBOL:
        	                	{
        	                		alt37=1;
        	                	}
        	                    break;
        	                case AFTER_SYMBOL:
        	                	{
        	                		alt37=2;
        	                	}
        	                    break;
        	            }

        	            switch (alt37)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:207:65: FIRST_SYMBOL
        	        	    {
        	        	         MATCHT(FIRST_SYMBOL, &FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry730);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:207:80: AFTER_SYMBOL identifier
        	        	    {
        	        	         MATCHT(AFTER_SYMBOL, &FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry734);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_identifier_in_alter_table_list_entry736);
        	        	        identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:208:4: MODIFY_SYMBOL ( COLUMN_SYMBOL )? column_internal_ref data_type ( attribute )* ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	    {
        	         MATCHT(MODIFY_SYMBOL, &FOLLOW_MODIFY_SYMBOL_in_alter_table_list_entry743);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:208:18: ( COLUMN_SYMBOL )?
        	        {
        	            int alt38=2;
        	            switch ( LA(1) )
        	            {
        	                case COLUMN_SYMBOL:
        	                	{
        	                		alt38=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt38)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:208:18: COLUMN_SYMBOL
        	        	    {
        	        	         MATCHT(COLUMN_SYMBOL, &FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry745);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_column_internal_ref_in_alter_table_list_entry748);
        	        column_internal_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_data_type_in_alter_table_list_entry750);
        	        data_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:208:63: ( attribute )*

        	        for (;;)
        	        {
        	            int alt39=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMENT_SYMBOL:
        	            case AUTO_INCREMENT_SYMBOL:
        	            case COLUMN_FORMAT_SYMBOL:
        	            case SERIAL_SYMBOL:
        	            case STORAGE_SYMBOL:
        	            case COLLATE_SYMBOL:
        	            case DEFAULT_SYMBOL:
        	            case KEY_SYMBOL:
        	            case NOT_SYMBOL:
        	            case NULL2_SYMBOL:
        	            case NULL_SYMBOL:
        	            case ON_SYMBOL:
        	            case PRIMARY_SYMBOL:
        	            case UNIQUE_SYMBOL:
        	            	{
        	            		alt39=1;
        	            	}
        	                break;

        	            }

        	            switch (alt39)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:208:63: attribute
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_attribute_in_alter_table_list_entry752);
        	        	        attribute(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop39;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop39: ; /* Jump out to here if this rule does not match */


        	        // MySQLSimpleParser.g:208:74: ( FIRST_SYMBOL | AFTER_SYMBOL identifier )?
        	        {
        	            int alt40=3;
        	            switch ( LA(1) )
        	            {
        	                case FIRST_SYMBOL:
        	                	{
        	                		alt40=1;
        	                	}
        	                    break;
        	                case AFTER_SYMBOL:
        	                	{
        	                		alt40=2;
        	                	}
        	                    break;
        	            }

        	            switch (alt40)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:208:75: FIRST_SYMBOL
        	        	    {
        	        	         MATCHT(FIRST_SYMBOL, &FOLLOW_FIRST_SYMBOL_in_alter_table_list_entry756);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:208:90: AFTER_SYMBOL identifier
        	        	    {
        	        	         MATCHT(AFTER_SYMBOL, &FOLLOW_AFTER_SYMBOL_in_alter_table_list_entry760);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_identifier_in_alter_table_list_entry762);
        	        	        identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:209:4: DROP_SYMBOL ( ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref | ( COLUMN_SYMBOL )? column_internal_ref | PRIMARY_SYMBOL KEY_SYMBOL | FOREIGN_SYMBOL KEY_SYMBOL ({...}? => column_ref |{...}? => ( column_ref )? ) )
        	    {
        	         MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_alter_table_list_entry769);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:210:3: ( ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref | ( COLUMN_SYMBOL )? column_internal_ref | PRIMARY_SYMBOL KEY_SYMBOL | FOREIGN_SYMBOL KEY_SYMBOL ({...}? => column_ref |{...}? => ( column_ref )? ) )
        	        {
        	            int alt44=4;
        	            {
        	                int LA44_0 = LA(1);
        	                if ( (LA44_0 == INDEX_SYMBOL || LA44_0 == KEY_SYMBOL))
        	                {
        	                    alt44=1;
        	                }
        	                else if ( (((LA44_0 >= ACCOUNT_SYMBOL) && (LA44_0 <= YEAR_SYMBOL)) || LA44_0 == COLUMN_SYMBOL || LA44_0 == BACK_TICK_QUOTED_ID || LA44_0 == DOT_SYMBOL || LA44_0 == DOUBLE_QUOTED_TEXT || LA44_0 == IDENTIFIER))
        	                {
        	                    alt44=2;
        	                }
        	                else if ( (LA44_0 == PRIMARY_SYMBOL))
        	                {
        	                    alt44=3;
        	                }
        	                else if ( (LA44_0 == FOREIGN_SYMBOL))
        	                {
        	                    alt44=4;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 44;
        	                    EXCEPTION->state        = 0;


        	                    goto rulealter_table_list_entryEx;

        	                }
        	            }
        	            switch (alt44)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:211:4: ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref
        	        	    {
        	        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulealter_table_list_entryEx;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_alter_table_list_entry786);
        	        	        column_ref(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:212:6: ( COLUMN_SYMBOL )? column_internal_ref
        	        	    {
        	        	        // MySQLSimpleParser.g:212:6: ( COLUMN_SYMBOL )?
        	        	        {
        	        	            int alt41=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case COLUMN_SYMBOL:
        	        	                	{
        	        	                		alt41=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt41)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:212:6: COLUMN_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(COLUMN_SYMBOL, &FOLLOW_COLUMN_SYMBOL_in_alter_table_list_entry793);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_column_internal_ref_in_alter_table_list_entry796);
        	        	        column_internal_ref(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:213:6: PRIMARY_SYMBOL KEY_SYMBOL
        	        	    {
        	        	         MATCHT(PRIMARY_SYMBOL, &FOLLOW_PRIMARY_SYMBOL_in_alter_table_list_entry803);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_alter_table_list_entry805);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 4:
        	        	    // MySQLSimpleParser.g:214:6: FOREIGN_SYMBOL KEY_SYMBOL ({...}? => column_ref |{...}? => ( column_ref )? )
        	        	    {
        	        	         MATCHT(FOREIGN_SYMBOL, &FOLLOW_FOREIGN_SYMBOL_in_alter_table_list_entry812);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_alter_table_list_entry814);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:215:5: ({...}? => column_ref |{...}? => ( column_ref )? )
        	        	        {
        	        	            int alt43=2;
        	        	            {
        	        	                int LA43_0 = LA(1);
        	        	                if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_0 == DOT_SYMBOL))
        	        	                {
        	        	                    {
        	        	                        int LA43_1 = LA(2);
        	        	                        if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_1 == IDENTIFIER))
        	        	                        {
        	        	                            {
        	        	                                int LA43_12 = LA(3);
        	        	                                if ( ((SERVER_VERSION >= 50700)))
        	        	                                {
        	        	                                    alt43=1;
        	        	                                }
        	        	                                else if ( ((SERVER_VERSION < 50700)))
        	        	                                {
        	        	                                    alt43=2;
        	        	                                }
        	        	                                else
        	        	                                {
        	        	                                    if (BACKTRACKING>0)
        	        	                                    {
        	        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                                        return ;
        	        	                                    }


        	        	                                    CONSTRUCTEX();
        	        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                                    EXCEPTION->message      = (void *)"";
        	        	                                    EXCEPTION->decisionNum  = 43;
        	        	                                    EXCEPTION->state        = 12;


        	        	                                    goto rulealter_table_list_entryEx;

        	        	                                }
        	        	                            }
        	        	                        }
        	        	                        else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_1 == BACK_TICK_QUOTED_ID))
        	        	                        {
        	        	                            {
        	        	                                int LA43_13 = LA(3);
        	        	                                if ( ((SERVER_VERSION >= 50700)))
        	        	                                {
        	        	                                    alt43=1;
        	        	                                }
        	        	                                else if ( ((SERVER_VERSION < 50700)))
        	        	                                {
        	        	                                    alt43=2;
        	        	                                }
        	        	                                else
        	        	                                {
        	        	                                    if (BACKTRACKING>0)
        	        	                                    {
        	        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                                        return ;
        	        	                                    }


        	        	                                    CONSTRUCTEX();
        	        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                                    EXCEPTION->message      = (void *)"";
        	        	                                    EXCEPTION->decisionNum  = 43;
        	        	                                    EXCEPTION->state        = 13;


        	        	                                    goto rulealter_table_list_entryEx;

        	        	                                }
        	        	                            }
        	        	                        }
        	        	                        else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_1 == DOUBLE_QUOTED_TEXT))
        	        	                        {
        	        	                            {
        	        	                                int LA43_14 = LA(3);
        	        	                                if ( ((SERVER_VERSION >= 50700)))
        	        	                                {
        	        	                                    alt43=1;
        	        	                                }
        	        	                                else if ( ((SERVER_VERSION < 50700)))
        	        	                                {
        	        	                                    alt43=2;
        	        	                                }
        	        	                                else
        	        	                                {
        	        	                                    if (BACKTRACKING>0)
        	        	                                    {
        	        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                                        return ;
        	        	                                    }


        	        	                                    CONSTRUCTEX();
        	        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                                    EXCEPTION->message      = (void *)"";
        	        	                                    EXCEPTION->decisionNum  = 43;
        	        	                                    EXCEPTION->state        = 14;


        	        	                                    goto rulealter_table_list_entryEx;

        	        	                                }
        	        	                            }
        	        	                        }
        	        	                        else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (((LA43_1 >= ACTION_SYMBOL) && (LA43_1 <= SHARE_SYMBOL)) || ((LA43_1 >= SLOW_SYMBOL) && (LA43_1 <= YEAR_SYMBOL))))
        	        	                        {
        	        	                            {
        	        	                                int LA43_15 = LA(3);
        	        	                                if ( ((SERVER_VERSION >= 50700)))
        	        	                                {
        	        	                                    alt43=1;
        	        	                                }
        	        	                                else if ( ((SERVER_VERSION < 50700)))
        	        	                                {
        	        	                                    alt43=2;
        	        	                                }
        	        	                                else
        	        	                                {
        	        	                                    if (BACKTRACKING>0)
        	        	                                    {
        	        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                                        return ;
        	        	                                    }


        	        	                                    CONSTRUCTEX();
        	        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                                    EXCEPTION->message      = (void *)"";
        	        	                                    EXCEPTION->decisionNum  = 43;
        	        	                                    EXCEPTION->state        = 15;


        	        	                                    goto rulealter_table_list_entryEx;

        	        	                                }
        	        	                            }
        	        	                        }
        	        	                        else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (((LA43_1 >= ACCOUNT_SYMBOL) && (LA43_1 <= XA_SYMBOL)) || LA43_1 == SHUTDOWN_SYMBOL))
        	        	                        {
        	        	                            {
        	        	                                int LA43_16 = LA(3);
        	        	                                if ( ((SERVER_VERSION >= 50700)))
        	        	                                {
        	        	                                    alt43=1;
        	        	                                }
        	        	                                else if ( ((SERVER_VERSION < 50700)))
        	        	                                {
        	        	                                    alt43=2;
        	        	                                }
        	        	                                else
        	        	                                {
        	        	                                    if (BACKTRACKING>0)
        	        	                                    {
        	        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                                        return ;
        	        	                                    }


        	        	                                    CONSTRUCTEX();
        	        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                                    EXCEPTION->message      = (void *)"";
        	        	                                    EXCEPTION->decisionNum  = 43;
        	        	                                    EXCEPTION->state        = 16;


        	        	                                    goto rulealter_table_list_entryEx;

        	        	                                }
        	        	                            }
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 1;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_0 == IDENTIFIER))
        	        	                {
        	        	                    {
        	        	                        int LA43_2 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 2;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_0 == BACK_TICK_QUOTED_ID))
        	        	                {
        	        	                    {
        	        	                        int LA43_3 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 3;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (LA43_0 == DOUBLE_QUOTED_TEXT))
        	        	                {
        	        	                    {
        	        	                        int LA43_4 = LA(2);
        	        	                        if ( (( (( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) && ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )) && ((SERVER_VERSION >= 50700)) )))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( (( (( ((SERVER_VERSION < 50700)) && ((SERVER_VERSION < 50700)) )) && ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 4;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (((LA43_0 >= ACTION_SYMBOL) && (LA43_0 <= SHARE_SYMBOL)) || ((LA43_0 >= SLOW_SYMBOL) && (LA43_0 <= YEAR_SYMBOL))))
        	        	                {
        	        	                    {
        	        	                        int LA43_5 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 5;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||(( (( ((SERVER_VERSION < 50700)) && ((SERVER_VERSION < 50700)) )) && ((SERVER_VERSION >= 50100)) ))||((SERVER_VERSION >= 50700)))) && (LA43_0 == PARTITION_SYMBOL))
        	        	                {
        	        	                    {
        	        	                        int LA43_6 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 6;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((SERVER_VERSION < 50700)) && (LA43_0 == EOF || LA43_0 == SEMICOLON_SYMBOL || LA43_0 == COMMA_SYMBOL))
        	        	                {
        	        	                    alt43=2;
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||(( (( ((SERVER_VERSION < 50700)) && ((SERVER_VERSION < 50700)) )) && ((SERVER_VERSION >= 50100)) ))||((SERVER_VERSION >= 50700)))) && (LA43_0 == REMOVE_SYMBOL))
        	        	                {
        	        	                    {
        	        	                        int LA43_8 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 8;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else if ( ((((SERVER_VERSION < 50700))||((SERVER_VERSION >= 50700)))) && (((LA43_0 >= ACCOUNT_SYMBOL) && (LA43_0 <= PARSER_SYMBOL)) || ((LA43_0 >= PORT_SYMBOL) && (LA43_0 <= PREPARE_SYMBOL)) || ((LA43_0 >= REPAIR_SYMBOL) && (LA43_0 <= XA_SYMBOL)) || LA43_0 == SHUTDOWN_SYMBOL))
        	        	                {
        	        	                    {
        	        	                        int LA43_11 = LA(2);
        	        	                        if ( ((SERVER_VERSION >= 50700)))
        	        	                        {
        	        	                            alt43=1;
        	        	                        }
        	        	                        else if ( ((SERVER_VERSION < 50700)))
        	        	                        {
        	        	                            alt43=2;
        	        	                        }
        	        	                        else
        	        	                        {
        	        	                            if (BACKTRACKING>0)
        	        	                            {
        	        	                                FAILEDFLAG = ANTLR3_TRUE;
        	        	                                return ;
        	        	                            }


        	        	                            CONSTRUCTEX();
        	        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                            EXCEPTION->message      = (void *)"";
        	        	                            EXCEPTION->decisionNum  = 43;
        	        	                            EXCEPTION->state        = 11;


        	        	                            goto rulealter_table_list_entryEx;

        	        	                        }
        	        	                    }
        	        	                }
        	        	                else
        	        	                {
        	        	                    if (BACKTRACKING>0)
        	        	                    {
        	        	                        FAILEDFLAG = ANTLR3_TRUE;
        	        	                        return ;
        	        	                    }


        	        	                    CONSTRUCTEX();
        	        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	        	                    EXCEPTION->message      = (void *)"";
        	        	                    EXCEPTION->decisionNum  = 43;
        	        	                    EXCEPTION->state        = 0;


        	        	                    goto rulealter_table_list_entryEx;

        	        	                }
        	        	            }
        	        	            switch (alt43)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:217:6: {...}? => column_ref
        	        	        	    {
        	        	        	        if ( !((SERVER_VERSION >= 50700)) )
        	        	        	        {
        	        	        	            if (BACKTRACKING>0)
        	        	        	            {
        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	        	                return ;
        	        	        	            }

        	        	        	                CONSTRUCTEX();
        	        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
        	        	        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        	        	        }

        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_alter_table_list_entry837);
        	        	        	        column_ref(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;
        	        	        	case 2:
        	        	        	    // MySQLSimpleParser.g:218:8: {...}? => ( column_ref )?
        	        	        	    {
        	        	        	        if ( !((SERVER_VERSION < 50700)) )
        	        	        	        {
        	        	        	            if (BACKTRACKING>0)
        	        	        	            {
        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	        	                return ;
        	        	        	            }

        	        	        	                CONSTRUCTEX();
        	        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50700";
        	        	        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        	        	        }

        	        	        	        // MySQLSimpleParser.g:218:37: ( column_ref )?
        	        	        	        {
        	        	        	            int alt42=2;
        	        	        	            {
        	        	        	                int LA42_0 = LA(1);
        	        	        	                if ( (((LA42_0 >= ACCOUNT_SYMBOL) && (LA42_0 <= PARSER_SYMBOL)) || ((LA42_0 >= PORT_SYMBOL) && (LA42_0 <= PREPARE_SYMBOL)) || ((LA42_0 >= REPAIR_SYMBOL) && (LA42_0 <= YEAR_SYMBOL)) || LA42_0 == BACK_TICK_QUOTED_ID || LA42_0 == DOT_SYMBOL || LA42_0 == DOUBLE_QUOTED_TEXT || LA42_0 == IDENTIFIER))
        	        	        	                {
        	        	        	                    alt42=1;
        	        	        	                }
        	        	        	                else if ( (LA42_0 == PARTITION_SYMBOL))
        	        	        	                {
        	        	        	                    switch ( LA(2) )
        	        	        	                    {
        	        	        	                        case EOF:
        	        	        	                        case PARTITION_SYMBOL:
        	        	        	                        case REMOVE_SYMBOL:
        	        	        	                        case SEMICOLON_SYMBOL:
        	        	        	                        case COMMA_SYMBOL:
        	        	        	                        case DOT_SYMBOL:
        	        	        	                        	{
        	        	        	                        		alt42=1;
        	        	        	                        	}
        	        	        	                            break;
        	        	        	                    }

        	        	        	                }
        	        	        	                else if ( (LA42_0 == REMOVE_SYMBOL))
        	        	        	                {
        	        	        	                    switch ( LA(2) )
        	        	        	                    {
        	        	        	                        case EOF:
        	        	        	                        case PARTITION_SYMBOL:
        	        	        	                        case REMOVE_SYMBOL:
        	        	        	                        case SEMICOLON_SYMBOL:
        	        	        	                        case COMMA_SYMBOL:
        	        	        	                        case DOT_SYMBOL:
        	        	        	                        	{
        	        	        	                        		alt42=1;
        	        	        	                        	}
        	        	        	                            break;
        	        	        	                    }

        	        	        	                }
        	        	        	            }
        	        	        	            switch (alt42)
        	        	        	            {
        	        	        	        	case 1:
        	        	        	        	    // MySQLSimpleParser.g:218:37: column_ref
        	        	        	        	    {
        	        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_alter_table_list_entry850);
        	        	        	        	        column_ref(ctx);

        	        	        	        	        FOLLOWPOP();
        	        	        	        	        if  (HASEXCEPTION())
        	        	        	        	        {
        	        	        	        	            goto rulealter_table_list_entryEx;
        	        	        	        	        }
        	        	        	        	        if (HASFAILED())
        	        	        	        	        {
        	        	        	        	            return ;
        	        	        	        	        }


        	        	        	        	    }
        	        	        	        	    break;

        	        	        	            }
        	        	        	        }

        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:221:4: DISABLE_SYMBOL KEYS_SYMBOL
        	    {
        	         MATCHT(DISABLE_SYMBOL, &FOLLOW_DISABLE_SYMBOL_in_alter_table_list_entry866);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(KEYS_SYMBOL, &FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry868);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:222:4: ENABLE_SYMBOL KEYS_SYMBOL
        	    {
        	         MATCHT(ENABLE_SYMBOL, &FOLLOW_ENABLE_SYMBOL_in_alter_table_list_entry873);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(KEYS_SYMBOL, &FOLLOW_KEYS_SYMBOL_in_alter_table_list_entry875);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:223:4: RENAME_SYMBOL ( TO_SYMBOL | AS_SYMBOL )? table_ref
        	    {
        	         MATCHT(RENAME_SYMBOL, &FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry880);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:223:18: ( TO_SYMBOL | AS_SYMBOL )?
        	        {
        	            int alt45=2;
        	            switch ( LA(1) )
        	            {
        	                case TO_SYMBOL:
        	                case AS_SYMBOL:
        	                	{
        	                		alt45=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt45)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:
        	        	    {
        	        	        if ( LA(1) == TO_SYMBOL || LA(1) == AS_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulealter_table_list_entryEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_table_ref_in_alter_table_list_entry891);
        	        table_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:224:4: {...}? RENAME_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL ) column_ref TO_SYMBOL column_ref
        	    {
        	        if ( !((SERVER_VERSION >= 50700)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        }

        	         MATCHT(RENAME_SYMBOL, &FOLLOW_RENAME_SYMBOL_in_alter_table_list_entry898);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulealter_table_list_entryEx;
        	        }


        	        FOLLOWPUSH(FOLLOW_column_ref_in_alter_table_list_entry908);
        	        column_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_alter_table_list_entry910);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_column_ref_in_alter_table_list_entry912);
        	        column_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:225:4: alter_order_by
        	    {
        	        FOLLOWPUSH(FOLLOW_alter_order_by_in_alter_table_list_entry917);
        	        alter_order_by(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:226:4: CONVERT_SYMBOL TO_SYMBOL charset charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )?
        	    {
        	         MATCHT(CONVERT_SYMBOL, &FOLLOW_CONVERT_SYMBOL_in_alter_table_list_entry922);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_alter_table_list_entry924);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_charset_in_alter_table_list_entry926);
        	        charset(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_charset_name_or_default_in_alter_table_list_entry928);
        	        charset_name_or_default(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:226:61: ( COLLATE_SYMBOL collation_name_or_default )?
        	        {
        	            int alt46=2;
        	            switch ( LA(1) )
        	            {
        	                case COLLATE_SYMBOL:
        	                	{
        	                		alt46=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt46)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:226:62: COLLATE_SYMBOL collation_name_or_default
        	        	    {
        	        	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_alter_table_list_entry931);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_collation_name_or_default_in_alter_table_list_entry933);
        	        	        collation_name_or_default(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_table_list_entryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:227:4: FORCE_SYMBOL
        	    {
        	         MATCHT(FORCE_SYMBOL, &FOLLOW_FORCE_SYMBOL_in_alter_table_list_entry940);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:228:4: {...}? alter_algorithm_option
        	    {
        	        if ( !((SERVER_VERSION >= 50600)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        }

        	        FOLLOWPUSH(FOLLOW_alter_algorithm_option_in_alter_table_list_entry947);
        	        alter_algorithm_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:229:4: {...}? alter_lock_option
        	    {
        	        if ( !((SERVER_VERSION >= 50600)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        }

        	        FOLLOWPUSH(FOLLOW_alter_lock_option_in_alter_table_list_entry954);
        	        alter_lock_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:230:4: {...}? UPGRADE_SYMBOL PARTITIONING_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50708)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50708";
        	                EXCEPTION->ruleName	 = (void *)"alter_table_list_entry";


        	        }

        	         MATCHT(UPGRADE_SYMBOL, &FOLLOW_UPGRADE_SYMBOL_in_alter_table_list_entry961);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITIONING_SYMBOL, &FOLLOW_PARTITIONING_SYMBOL_in_alter_table_list_entry963);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_table_list_entryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulealter_table_list_entryEx; /* Prevent compiler warnings */
    rulealter_table_list_entryEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_table_list_entry */

/**
 * $ANTLR start key_def
 * MySQLSimpleParser.g:233:1: key_def : ( ( CONSTRAINT_SYMBOL ( column_ref )? )? ( PRIMARY_SYMBOL KEY_SYMBOL ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | FOREIGN_SYMBOL KEY_SYMBOL ( column_ref )? index_columns reference_definition | UNIQUE_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL ) | ( INDEX_SYMBOL | KEY_SYMBOL ) ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | ( FULLTEXT_SYMBOL | SPATIAL_SYMBOL ) ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? index_columns ( normal_index_option )* );
 */
static void
key_def(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:233:8: ( ( CONSTRAINT_SYMBOL ( column_ref )? )? ( PRIMARY_SYMBOL KEY_SYMBOL ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | FOREIGN_SYMBOL KEY_SYMBOL ( column_ref )? index_columns reference_definition | UNIQUE_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL ) | ( INDEX_SYMBOL | KEY_SYMBOL ) ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | ( FULLTEXT_SYMBOL | SPATIAL_SYMBOL ) ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? index_columns ( normal_index_option )* )

            ANTLR3_UINT32 alt65;

            alt65=3;

            switch ( LA(1) )
            {
            case CHECK_SYMBOL:
            case CONSTRAINT_SYMBOL:
            case FOREIGN_SYMBOL:
            case PRIMARY_SYMBOL:
            case UNIQUE_SYMBOL:
            	{
            		alt65=1;
            	}
                break;
            case INDEX_SYMBOL:
            case KEY_SYMBOL:
            	{
            		alt65=2;
            	}
                break;
            case FULLTEXT_SYMBOL:
            case SPATIAL_SYMBOL:
            	{
            		alt65=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 65;
                EXCEPTION->state        = 0;


                goto rulekey_defEx;

            }

            switch (alt65)
            {
        	case 1:
        	    // MySQLSimpleParser.g:234:2: ( CONSTRAINT_SYMBOL ( column_ref )? )? ( PRIMARY_SYMBOL KEY_SYMBOL ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | FOREIGN_SYMBOL KEY_SYMBOL ( column_ref )? index_columns reference_definition | UNIQUE_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )
        	    {
        	        // MySQLSimpleParser.g:234:2: ( CONSTRAINT_SYMBOL ( column_ref )? )?
        	        {
        	            int alt49=2;
        	            switch ( LA(1) )
        	            {
        	                case CONSTRAINT_SYMBOL:
        	                	{
        	                		alt49=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt49)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:234:3: CONSTRAINT_SYMBOL ( column_ref )?
        	        	    {
        	        	         MATCHT(CONSTRAINT_SYMBOL, &FOLLOW_CONSTRAINT_SYMBOL_in_key_def973);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:234:21: ( column_ref )?
        	        	        {
        	        	            int alt48=2;
        	        	            {
        	        	                int LA48_0 = LA(1);
        	        	                if ( (((LA48_0 >= ACCOUNT_SYMBOL) && (LA48_0 <= YEAR_SYMBOL)) || LA48_0 == BACK_TICK_QUOTED_ID || LA48_0 == DOT_SYMBOL || LA48_0 == DOUBLE_QUOTED_TEXT || LA48_0 == IDENTIFIER))
        	        	                {
        	        	                    alt48=1;
        	        	                }
        	        	            }
        	        	            switch (alt48)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:234:21: column_ref
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def975);
        	        	        	        column_ref(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:235:3: ( PRIMARY_SYMBOL KEY_SYMBOL ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | FOREIGN_SYMBOL KEY_SYMBOL ( column_ref )? index_columns reference_definition | UNIQUE_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? ( index_type )? index_columns ( normal_index_option )* | CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )
        	        {
        	            int alt58=4;
        	            switch ( LA(1) )
        	            {
        	            case PRIMARY_SYMBOL:
        	            	{
        	            		alt58=1;
        	            	}
        	                break;
        	            case FOREIGN_SYMBOL:
        	            	{
        	            		alt58=2;
        	            	}
        	                break;
        	            case UNIQUE_SYMBOL:
        	            	{
        	            		alt58=3;
        	            	}
        	                break;
        	            case CHECK_SYMBOL:
        	            	{
        	            		alt58=4;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 58;
        	                EXCEPTION->state        = 0;


        	                goto rulekey_defEx;

        	            }

        	            switch (alt58)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:236:4: PRIMARY_SYMBOL KEY_SYMBOL ( column_ref )? ( index_type )? index_columns ( normal_index_option )*
        	        	    {
        	        	         MATCHT(PRIMARY_SYMBOL, &FOLLOW_PRIMARY_SYMBOL_in_key_def987);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_key_def989);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:236:30: ( column_ref )?
        	        	        {
        	        	            int alt50=2;
        	        	            {
        	        	                int LA50_0 = LA(1);
        	        	                if ( (((LA50_0 >= ACCOUNT_SYMBOL) && (LA50_0 <= TYPES_SYMBOL)) || ((LA50_0 >= UDF_RETURNS_SYMBOL) && (LA50_0 <= YEAR_SYMBOL)) || LA50_0 == BACK_TICK_QUOTED_ID || LA50_0 == DOT_SYMBOL || LA50_0 == DOUBLE_QUOTED_TEXT || LA50_0 == IDENTIFIER))
        	        	                {
        	        	                    alt50=1;
        	        	                }
        	        	                else if ( (LA50_0 == TYPE_SYMBOL))
        	        	                {
        	        	                    switch ( LA(2) )
        	        	                    {
        	        	                        case TYPE_SYMBOL:
        	        	                        case OPEN_PAR_SYMBOL:
        	        	                        case USING_SYMBOL:
        	        	                        case DOT_SYMBOL:
        	        	                        	{
        	        	                        		alt50=1;
        	        	                        	}
        	        	                            break;
        	        	                    }

        	        	                }
        	        	            }
        	        	            switch (alt50)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:236:30: column_ref
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def991);
        	        	        	        column_ref(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        // MySQLSimpleParser.g:236:42: ( index_type )?
        	        	        {
        	        	            int alt51=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case TYPE_SYMBOL:
        	        	                case USING_SYMBOL:
        	        	                	{
        	        	                		alt51=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt51)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:236:42: index_type
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_index_type_in_key_def994);
        	        	        	        index_type(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_index_columns_in_key_def997);
        	        	        index_columns(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:236:68: ( normal_index_option )*

        	        	        for (;;)
        	        	        {
        	        	            int alt52=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	            case COMMENT_SYMBOL:
        	        	            case KEY_BLOCK_SIZE_SYMBOL:
        	        	            case TYPE_SYMBOL:
        	        	            case USING_SYMBOL:
        	        	            	{
        	        	            		alt52=1;
        	        	            	}
        	        	                break;

        	        	            }

        	        	            switch (alt52)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:236:68: normal_index_option
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_normal_index_option_in_key_def999);
        	        	        	        normal_index_option(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	        	default:
        	        	        	    goto loop52;	/* break out of the loop */
        	        	        	    break;
        	        	            }
        	        	        }
        	        	        loop52: ; /* Jump out to here if this rule does not match */


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:237:6: FOREIGN_SYMBOL KEY_SYMBOL ( column_ref )? index_columns reference_definition
        	        	    {
        	        	         MATCHT(FOREIGN_SYMBOL, &FOLLOW_FOREIGN_SYMBOL_in_key_def1007);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_key_def1009);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:237:32: ( column_ref )?
        	        	        {
        	        	            int alt53=2;
        	        	            {
        	        	                int LA53_0 = LA(1);
        	        	                if ( (((LA53_0 >= ACCOUNT_SYMBOL) && (LA53_0 <= YEAR_SYMBOL)) || LA53_0 == BACK_TICK_QUOTED_ID || LA53_0 == DOT_SYMBOL || LA53_0 == DOUBLE_QUOTED_TEXT || LA53_0 == IDENTIFIER))
        	        	                {
        	        	                    alt53=1;
        	        	                }
        	        	            }
        	        	            switch (alt53)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:237:32: column_ref
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def1011);
        	        	        	        column_ref(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_index_columns_in_key_def1014);
        	        	        index_columns(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_reference_definition_in_key_def1016);
        	        	        reference_definition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:238:6: UNIQUE_SYMBOL ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? ( index_type )? index_columns ( normal_index_option )*
        	        	    {
        	        	         MATCHT(UNIQUE_SYMBOL, &FOLLOW_UNIQUE_SYMBOL_in_key_def1023);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:238:20: ( INDEX_SYMBOL | KEY_SYMBOL )?
        	        	        {
        	        	            int alt54=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case INDEX_SYMBOL:
        	        	                case KEY_SYMBOL:
        	        	                	{
        	        	                		alt54=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt54)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:
        	        	        	    {
        	        	        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
        	        	        	        {
        	        	        	            CONSUME();
        	        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        	        }
        	        	        	        else
        	        	        	        {
        	        	        	            if (BACKTRACKING>0)
        	        	        	            {
        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	        	                return ;
        	        	        	            }

        	        	        	            CONSTRUCTEX();
        	        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	        	            EXCEPTION->expectingSet = NULL;

        	        	        	            goto rulekey_defEx;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        // MySQLSimpleParser.g:238:49: ( column_ref )?
        	        	        {
        	        	            int alt55=2;
        	        	            {
        	        	                int LA55_0 = LA(1);
        	        	                if ( (((LA55_0 >= ACCOUNT_SYMBOL) && (LA55_0 <= TYPES_SYMBOL)) || ((LA55_0 >= UDF_RETURNS_SYMBOL) && (LA55_0 <= YEAR_SYMBOL)) || LA55_0 == BACK_TICK_QUOTED_ID || LA55_0 == DOT_SYMBOL || LA55_0 == DOUBLE_QUOTED_TEXT || LA55_0 == IDENTIFIER))
        	        	                {
        	        	                    alt55=1;
        	        	                }
        	        	                else if ( (LA55_0 == TYPE_SYMBOL))
        	        	                {
        	        	                    switch ( LA(2) )
        	        	                    {
        	        	                        case TYPE_SYMBOL:
        	        	                        case OPEN_PAR_SYMBOL:
        	        	                        case USING_SYMBOL:
        	        	                        case DOT_SYMBOL:
        	        	                        	{
        	        	                        		alt55=1;
        	        	                        	}
        	        	                            break;
        	        	                    }

        	        	                }
        	        	            }
        	        	            switch (alt55)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:238:49: column_ref
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def1034);
        	        	        	        column_ref(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        // MySQLSimpleParser.g:238:61: ( index_type )?
        	        	        {
        	        	            int alt56=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case TYPE_SYMBOL:
        	        	                case USING_SYMBOL:
        	        	                	{
        	        	                		alt56=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt56)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:238:61: index_type
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_index_type_in_key_def1037);
        	        	        	        index_type(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_index_columns_in_key_def1040);
        	        	        index_columns(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:238:87: ( normal_index_option )*

        	        	        for (;;)
        	        	        {
        	        	            int alt57=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	            case COMMENT_SYMBOL:
        	        	            case KEY_BLOCK_SIZE_SYMBOL:
        	        	            case TYPE_SYMBOL:
        	        	            case USING_SYMBOL:
        	        	            	{
        	        	            		alt57=1;
        	        	            	}
        	        	                break;

        	        	            }

        	        	            switch (alt57)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:238:87: normal_index_option
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_normal_index_option_in_key_def1042);
        	        	        	        normal_index_option(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulekey_defEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	        	default:
        	        	        	    goto loop57;	/* break out of the loop */
        	        	        	    break;
        	        	            }
        	        	        }
        	        	        loop57: ; /* Jump out to here if this rule does not match */


        	        	    }
        	        	    break;
        	        	case 4:
        	        	    // MySQLSimpleParser.g:239:6: CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	        	    {
        	        	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_key_def1050);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_key_def1052);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_key_def1054);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_key_def1056);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:241:4: ( INDEX_SYMBOL | KEY_SYMBOL ) ( column_ref )? ( index_type )? index_columns ( normal_index_option )*
        	    {
        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulekey_defEx;
        	        }


        	        // MySQLSimpleParser.g:241:32: ( column_ref )?
        	        {
        	            int alt59=2;
        	            {
        	                int LA59_0 = LA(1);
        	                if ( (((LA59_0 >= ACCOUNT_SYMBOL) && (LA59_0 <= TYPES_SYMBOL)) || ((LA59_0 >= UDF_RETURNS_SYMBOL) && (LA59_0 <= YEAR_SYMBOL)) || LA59_0 == BACK_TICK_QUOTED_ID || LA59_0 == DOT_SYMBOL || LA59_0 == DOUBLE_QUOTED_TEXT || LA59_0 == IDENTIFIER))
        	                {
        	                    alt59=1;
        	                }
        	                else if ( (LA59_0 == TYPE_SYMBOL))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                        case TYPE_SYMBOL:
        	                        case OPEN_PAR_SYMBOL:
        	                        case USING_SYMBOL:
        	                        case DOT_SYMBOL:
        	                        	{
        	                        		alt59=1;
        	                        	}
        	                            break;
        	                    }

        	                }
        	            }
        	            switch (alt59)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:241:32: column_ref
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def1073);
        	        	        column_ref(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:241:44: ( index_type )?
        	        {
        	            int alt60=2;
        	            switch ( LA(1) )
        	            {
        	                case TYPE_SYMBOL:
        	                case USING_SYMBOL:
        	                	{
        	                		alt60=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt60)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:241:44: index_type
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_index_type_in_key_def1076);
        	        	        index_type(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_index_columns_in_key_def1079);
        	        index_columns(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:241:70: ( normal_index_option )*

        	        for (;;)
        	        {
        	            int alt61=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMENT_SYMBOL:
        	            case KEY_BLOCK_SIZE_SYMBOL:
        	            case TYPE_SYMBOL:
        	            case USING_SYMBOL:
        	            	{
        	            		alt61=1;
        	            	}
        	                break;

        	            }

        	            switch (alt61)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:241:70: normal_index_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_normal_index_option_in_key_def1081);
        	        	        normal_index_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop61;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop61: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:242:4: ( FULLTEXT_SYMBOL | SPATIAL_SYMBOL ) ( INDEX_SYMBOL | KEY_SYMBOL )? ( column_ref )? index_columns ( normal_index_option )*
        	    {
        	        if ( LA(1) == FULLTEXT_SYMBOL || LA(1) == SPATIAL_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulekey_defEx;
        	        }


        	        // MySQLSimpleParser.g:242:39: ( INDEX_SYMBOL | KEY_SYMBOL )?
        	        {
        	            int alt62=2;
        	            switch ( LA(1) )
        	            {
        	                case INDEX_SYMBOL:
        	                case KEY_SYMBOL:
        	                	{
        	                		alt62=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt62)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:
        	        	    {
        	        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulekey_defEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:242:68: ( column_ref )?
        	        {
        	            int alt63=2;
        	            {
        	                int LA63_0 = LA(1);
        	                if ( (((LA63_0 >= ACCOUNT_SYMBOL) && (LA63_0 <= YEAR_SYMBOL)) || LA63_0 == BACK_TICK_QUOTED_ID || LA63_0 == DOT_SYMBOL || LA63_0 == DOUBLE_QUOTED_TEXT || LA63_0 == IDENTIFIER))
        	                {
        	                    alt63=1;
        	                }
        	            }
        	            switch (alt63)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:242:68: column_ref
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_column_ref_in_key_def1104);
        	        	        column_ref(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_index_columns_in_key_def1107);
        	        index_columns(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:242:94: ( normal_index_option )*

        	        for (;;)
        	        {
        	            int alt64=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMENT_SYMBOL:
        	            case KEY_BLOCK_SIZE_SYMBOL:
        	            case TYPE_SYMBOL:
        	            case USING_SYMBOL:
        	            	{
        	            		alt64=1;
        	            	}
        	                break;

        	            }

        	            switch (alt64)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:242:94: normal_index_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_normal_index_option_in_key_def1109);
        	        	        normal_index_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulekey_defEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop64;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop64: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulekey_defEx; /* Prevent compiler warnings */
    rulekey_defEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end key_def */

/**
 * $ANTLR start alter_order_by
 * MySQLSimpleParser.g:245:1: alter_order_by : ORDER_SYMBOL BY_SYMBOL identifier direction ( options {greedy=true; } : COMMA_SYMBOL identifier direction )* ;
 */
static void
alter_order_by(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:245:15: ( ORDER_SYMBOL BY_SYMBOL identifier direction ( options {greedy=true; } : COMMA_SYMBOL identifier direction )* )
        // MySQLSimpleParser.g:246:2: ORDER_SYMBOL BY_SYMBOL identifier direction ( options {greedy=true; } : COMMA_SYMBOL identifier direction )*
        {
             MATCHT(ORDER_SYMBOL, &FOLLOW_ORDER_SYMBOL_in_alter_order_by1119);
            if  (HASEXCEPTION())
            {
                goto rulealter_order_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_alter_order_by1121);
            if  (HASEXCEPTION())
            {
                goto rulealter_order_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_alter_order_by1123);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_order_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_direction_in_alter_order_by1125);
            direction(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_order_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:246:46: ( options {greedy=true; } : COMMA_SYMBOL identifier direction )*

            for (;;)
            {
                int alt66=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA66_1 = LA(2);
                		    if ( (LA66_1 == ENGINE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == TYPE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == MAX_ROWS_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == MIN_ROWS_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == AVG_ROW_LENGTH_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == PASSWORD_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == COMMENT_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == COMPRESSION_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == AUTO_INCREMENT_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == PACK_KEYS_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (((LA66_1 >= STATS_AUTO_RECALC_SYMBOL) && (LA66_1 <= STATS_SAMPLE_PAGES_SYMBOL))))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == TABLE_CHECKSUM_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == DELAY_KEY_WRITE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == ROW_FORMAT_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == CHARSET_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == INSERT_METHOD_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == DATA_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == TABLESPACE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == STORAGE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == CONNECTION_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == KEY_BLOCK_SIZE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == MODIFY_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == DISABLE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == ENABLE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == ALGORITHM_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == UPGRADE_SYMBOL))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (((LA66_1 >= ACCOUNT_SYMBOL) && (LA66_1 <= CACHE_SYMBOL)) || LA66_1 == CLOSE_SYMBOL || ((LA66_1 >= COMMIT_SYMBOL) && (LA66_1 <= UNINSTALL_SYMBOL)) || ((LA66_1 >= WRAPPER_SYMBOL) && (LA66_1 <= AGGREGATE_SYMBOL)) || ((LA66_1 >= ANALYZE_SYMBOL) && (LA66_1 <= AUTHORS_SYMBOL)) || LA66_1 == AUTOEXTEND_SIZE_SYMBOL || ((LA66_1 >= AVG_SYMBOL) && (LA66_1 <= COMPRESSED_SYMBOL)) || LA66_1 == CONCURRENT_SYMBOL || ((LA66_1 >= CONSISTENT_SYMBOL) && (LA66_1 <= CURSOR_NAME_SYMBOL)) || ((LA66_1 >= DATAFILE_SYMBOL) && (LA66_1 <= DEFINER_SYMBOL)) || ((LA66_1 >= DES_KEY_FILE_SYMBOL) && (LA66_1 <= DIRECTORY_SYMBOL)) || ((LA66_1 >= DISCARD_SYMBOL) && (LA66_1 <= ENUM_SYMBOL)) || ((LA66_1 >= ENGINES_SYMBOL) && (LA66_1 <= FOUND_SYMBOL)) || ((LA66_1 >= FULL_SYMBOL) && (LA66_1 <= ISSUER_SYMBOL)) || LA66_1 == JSON_SYMBOL || ((LA66_1 >= LAST_SYMBOL) && (LA66_1 <= LOGS_SYMBOL)) || ((LA66_1 >= MASTER_SYMBOL) && (LA66_1 <= MINUTE_SYMBOL)) || ((LA66_1 >= MODE_SYMBOL) && (LA66_1 <= ONE_SYMBOL)) || ((LA66_1 >= PAGE_SYMBOL) && (LA66_1 <= PARTITIONS_SYMBOL)) || ((LA66_1 >= PHASE_SYMBOL) && (LA66_1 <= ROW_COUNT_SYMBOL)) || ((LA66_1 >= ROW_SYMBOL) && (LA66_1 <= STARTS_SYMBOL)) || LA66_1 == STATUS_SYMBOL || ((LA66_1 >= STRING_SYMBOL) && (LA66_1 <= TABLES_SYMBOL)) || ((LA66_1 >= TEMPORARY_SYMBOL) && (LA66_1 <= TYPES_SYMBOL)) || ((LA66_1 >= UDF_RETURNS_SYMBOL) && (LA66_1 <= YEAR_SYMBOL)) || LA66_1 == BACK_TICK_QUOTED_ID || LA66_1 == DOUBLE_QUOTED_TEXT || LA66_1 == IDENTIFIER))
                		    {
                		        alt66=1;
                		    }
                		    else if ( (LA66_1 == CHECKSUM_SYMBOL))
                		    {
                		        alt66=1;
                		    }

                		}
                	}
                    break;

                }

                switch (alt66)
                {
            	case 1:
            	    // MySQLSimpleParser.g:246:75: COMMA_SYMBOL identifier direction
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_alter_order_by1140);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_order_byEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_identifier_in_alter_order_by1142);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_order_byEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_direction_in_alter_order_by1144);
            	        direction(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_order_byEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop66;	/* break out of the loop */
            	    break;
                }
            }
            loop66: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_order_byEx; /* Prevent compiler warnings */
    rulealter_order_byEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_order_by */

/**
 * $ANTLR start alter_algorithm_option
 * MySQLSimpleParser.g:249:1: alter_algorithm_option : ALGORITHM_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier ) ;
 */
static void
alter_algorithm_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:249:23: ( ALGORITHM_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier ) )
        // MySQLSimpleParser.g:250:2: ALGORITHM_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier )
        {
             MATCHT(ALGORITHM_SYMBOL, &FOLLOW_ALGORITHM_SYMBOL_in_alter_algorithm_option1155);
            if  (HASEXCEPTION())
            {
                goto rulealter_algorithm_optionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:250:19: ( EQUAL_OPERATOR )?
            {
                int alt67=2;
                switch ( LA(1) )
                {
                    case EQUAL_OPERATOR:
                    	{
                    		alt67=1;
                    	}
                        break;
                }

                switch (alt67)
                {
            	case 1:
            	    // MySQLSimpleParser.g:250:19: EQUAL_OPERATOR
            	    {
            	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_algorithm_option1157);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_algorithm_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:250:35: ( DEFAULT_SYMBOL | identifier )
            {
                int alt68=2;
                {
                    int LA68_0 = LA(1);
                    if ( (LA68_0 == DEFAULT_SYMBOL))
                    {
                        alt68=1;
                    }
                    else if ( (((LA68_0 >= ACCOUNT_SYMBOL) && (LA68_0 <= YEAR_SYMBOL)) || LA68_0 == BACK_TICK_QUOTED_ID || LA68_0 == DOUBLE_QUOTED_TEXT || LA68_0 == IDENTIFIER))
                    {
                        alt68=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 68;
                        EXCEPTION->state        = 0;


                        goto rulealter_algorithm_optionEx;

                    }
                }
                switch (alt68)
                {
            	case 1:
            	    // MySQLSimpleParser.g:250:37: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_alter_algorithm_option1162);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_algorithm_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:250:54: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_alter_algorithm_option1166);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_algorithm_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_algorithm_optionEx; /* Prevent compiler warnings */
    rulealter_algorithm_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_algorithm_option */

/**
 * $ANTLR start alter_lock_option
 * MySQLSimpleParser.g:253:1: alter_lock_option : LOCK_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier ) ;
 */
static void
alter_lock_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:253:18: ( LOCK_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier ) )
        // MySQLSimpleParser.g:254:2: LOCK_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | identifier )
        {
             MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_alter_lock_option1177);
            if  (HASEXCEPTION())
            {
                goto rulealter_lock_optionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:254:14: ( EQUAL_OPERATOR )?
            {
                int alt69=2;
                switch ( LA(1) )
                {
                    case EQUAL_OPERATOR:
                    	{
                    		alt69=1;
                    	}
                        break;
                }

                switch (alt69)
                {
            	case 1:
            	    // MySQLSimpleParser.g:254:14: EQUAL_OPERATOR
            	    {
            	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_lock_option1179);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_lock_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:254:30: ( DEFAULT_SYMBOL | identifier )
            {
                int alt70=2;
                {
                    int LA70_0 = LA(1);
                    if ( (LA70_0 == DEFAULT_SYMBOL))
                    {
                        alt70=1;
                    }
                    else if ( (((LA70_0 >= ACCOUNT_SYMBOL) && (LA70_0 <= YEAR_SYMBOL)) || LA70_0 == BACK_TICK_QUOTED_ID || LA70_0 == DOUBLE_QUOTED_TEXT || LA70_0 == IDENTIFIER))
                    {
                        alt70=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 70;
                        EXCEPTION->state        = 0;


                        goto rulealter_lock_optionEx;

                    }
                }
                switch (alt70)
                {
            	case 1:
            	    // MySQLSimpleParser.g:254:31: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_alter_lock_option1183);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_lock_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:254:48: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_alter_lock_option1187);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_lock_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_lock_optionEx; /* Prevent compiler warnings */
    rulealter_lock_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_lock_option */

/**
 * $ANTLR start index_lock_algorithm
 * MySQLSimpleParser.g:257:1: index_lock_algorithm :{...}? => ( alter_algorithm_option ( alter_lock_option )? | alter_lock_option ( alter_algorithm_option )? ) ;
 */
static void
index_lock_algorithm(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:257:21: ({...}? => ( alter_algorithm_option ( alter_lock_option )? | alter_lock_option ( alter_algorithm_option )? ) )
        // MySQLSimpleParser.g:258:2: {...}? => ( alter_algorithm_option ( alter_lock_option )? | alter_lock_option ( alter_algorithm_option )? )
        {
            if ( !((SERVER_VERSION >= 50600)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
                    EXCEPTION->ruleName	 = (void *)"index_lock_algorithm";


            }

            // MySQLSimpleParser.g:259:3: ( alter_algorithm_option ( alter_lock_option )? | alter_lock_option ( alter_algorithm_option )? )
            {
                int alt73=2;
                switch ( LA(1) )
                {
                case ALGORITHM_SYMBOL:
                	{
                		alt73=1;
                	}
                    break;
                case LOCK_SYMBOL:
                	{
                		alt73=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 73;
                    EXCEPTION->state        = 0;


                    goto ruleindex_lock_algorithmEx;

                }

                switch (alt73)
                {
            	case 1:
            	    // MySQLSimpleParser.g:260:4: alter_algorithm_option ( alter_lock_option )?
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_algorithm_option_in_index_lock_algorithm1209);
            	        alter_algorithm_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_lock_algorithmEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:260:27: ( alter_lock_option )?
            	        {
            	            int alt71=2;
            	            switch ( LA(1) )
            	            {
            	                case LOCK_SYMBOL:
            	                	{
            	                		alt71=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt71)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:260:27: alter_lock_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_alter_lock_option_in_index_lock_algorithm1211);
            	        	        alter_lock_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleindex_lock_algorithmEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:261:6: alter_lock_option ( alter_algorithm_option )?
            	    {
            	        FOLLOWPUSH(FOLLOW_alter_lock_option_in_index_lock_algorithm1219);
            	        alter_lock_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_lock_algorithmEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:261:24: ( alter_algorithm_option )?
            	        {
            	            int alt72=2;
            	            switch ( LA(1) )
            	            {
            	                case ALGORITHM_SYMBOL:
            	                	{
            	                		alt72=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt72)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:261:24: alter_algorithm_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_alter_algorithm_option_in_index_lock_algorithm1221);
            	        	        alter_algorithm_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleindex_lock_algorithmEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_lock_algorithmEx; /* Prevent compiler warnings */
    ruleindex_lock_algorithmEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_lock_algorithm */

/**
 * $ANTLR start alter_partition
 * MySQLSimpleParser.g:265:1: alter_partition : ({...}? => ADD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( OPEN_PAR_SYMBOL partition_definition CLOSE_PAR_SYMBOL | PARTITIONS_SYMBOL real_ulong_number ) | DROP_SYMBOL PARTITION_SYMBOL identifier_list | REBUILD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list | OPTIMIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( no_write_to_bin_log )? | ANALYZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list | CHECK_SYMBOL PARTITION_SYMBOL all_or_partition_name_list ( check_option )* | REPAIR_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( repair_option )* | COALESCE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? real_ulong_number |{...}? TRUNCATE_SYMBOL PARTITION_SYMBOL all_or_partition_name_list | EXCHANGE_SYMBOL PARTITION_SYMBOL identifier WITH_SYMBOL TABLE_SYMBOL table_ref ( validation )? | REORGANIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( identifier_list INTO_SYMBOL partition_definitions )? |{...}? DISCARD_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL |{...}? IMPORT_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL );
 */
static void
alter_partition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:265:16: ({...}? => ADD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( OPEN_PAR_SYMBOL partition_definition CLOSE_PAR_SYMBOL | PARTITIONS_SYMBOL real_ulong_number ) | DROP_SYMBOL PARTITION_SYMBOL identifier_list | REBUILD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list | OPTIMIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( no_write_to_bin_log )? | ANALYZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list | CHECK_SYMBOL PARTITION_SYMBOL all_or_partition_name_list ( check_option )* | REPAIR_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( repair_option )* | COALESCE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? real_ulong_number |{...}? TRUNCATE_SYMBOL PARTITION_SYMBOL all_or_partition_name_list | EXCHANGE_SYMBOL PARTITION_SYMBOL identifier WITH_SYMBOL TABLE_SYMBOL table_ref ( validation )? | REORGANIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( identifier_list INTO_SYMBOL partition_definitions )? |{...}? DISCARD_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL |{...}? IMPORT_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL )

            ANTLR3_UINT32 alt87;

            alt87=13;

            {
                int LA87_0 = LA(1);
                if ( ((LA(1) == ADD_SYMBOL && LA(2) == PARTITION_SYMBOL)) && (LA87_0 == ADD_SYMBOL))
                {
                    alt87=1;
                }
                else if ( (LA87_0 == DROP_SYMBOL))
                {
                    alt87=2;
                }
                else if ( (LA87_0 == REBUILD_SYMBOL))
                {
                    alt87=3;
                }
                else if ( (LA87_0 == OPTIMIZE_SYMBOL))
                {
                    alt87=4;
                }
                else if ( (LA87_0 == ANALYZE_SYMBOL))
                {
                    alt87=5;
                }
                else if ( (LA87_0 == CHECK_SYMBOL))
                {
                    alt87=6;
                }
                else if ( (LA87_0 == REPAIR_SYMBOL))
                {
                    alt87=7;
                }
                else if ( (LA87_0 == COALESCE_SYMBOL))
                {
                    alt87=8;
                }
                else if ( (LA87_0 == TRUNCATE_SYMBOL))
                {
                    alt87=9;
                }
                else if ( (LA87_0 == EXCHANGE_SYMBOL))
                {
                    alt87=10;
                }
                else if ( (LA87_0 == REORGANIZE_SYMBOL))
                {
                    alt87=11;
                }
                else if ( (LA87_0 == DISCARD_SYMBOL))
                {
                    alt87=12;
                }
                else if ( (LA87_0 == IMPORT_SYMBOL))
                {
                    alt87=13;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 87;
                    EXCEPTION->state        = 0;


                    goto rulealter_partitionEx;

                }
            }
            switch (alt87)
            {
        	case 1:
        	    // MySQLSimpleParser.g:266:2: {...}? => ADD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( OPEN_PAR_SYMBOL partition_definition CLOSE_PAR_SYMBOL | PARTITIONS_SYMBOL real_ulong_number )
        	    {
        	        if ( !((LA(1) == ADD_SYMBOL && LA(2) == PARTITION_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == ADD_SYMBOL && LA(2) == PARTITION_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"alter_partition";


        	        }

        	         MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_alter_partition1239);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1241);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:266:85: ( no_write_to_bin_log )?
        	        {
        	            int alt74=2;
        	            {
        	                int LA74_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA74_0 == LOCAL_SYMBOL))
        	                {
        	                    alt74=1;
        	                }
        	                else if ( (LA74_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt74=1;
        	                }
        	            }
        	            switch (alt74)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:266:85: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1243);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:267:3: ( OPEN_PAR_SYMBOL partition_definition CLOSE_PAR_SYMBOL | PARTITIONS_SYMBOL real_ulong_number )
        	        {
        	            int alt75=2;
        	            switch ( LA(1) )
        	            {
        	            case OPEN_PAR_SYMBOL:
        	            	{
        	            		alt75=1;
        	            	}
        	                break;
        	            case PARTITIONS_SYMBOL:
        	            	{
        	            		alt75=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 75;
        	                EXCEPTION->state        = 0;


        	                goto rulealter_partitionEx;

        	            }

        	            switch (alt75)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:268:4: OPEN_PAR_SYMBOL partition_definition CLOSE_PAR_SYMBOL
        	        	    {
        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_alter_partition1253);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_partition_definition_in_alter_partition1255);
        	        	        partition_definition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_alter_partition1257);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:269:6: PARTITIONS_SYMBOL real_ulong_number
        	        	    {
        	        	         MATCHT(PARTITIONS_SYMBOL, &FOLLOW_PARTITIONS_SYMBOL_in_alter_partition1264);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_alter_partition1266);
        	        	        real_ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:271:4: DROP_SYMBOL PARTITION_SYMBOL identifier_list
        	    {
        	         MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_alter_partition1275);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1277);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_list_in_alter_partition1279);
        	        identifier_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:272:4: REBUILD_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list
        	    {
        	         MATCHT(REBUILD_SYMBOL, &FOLLOW_REBUILD_SYMBOL_in_alter_partition1284);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1286);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:272:36: ( no_write_to_bin_log )?
        	        {
        	            int alt76=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		{
        	                		    int LA76_1 = LA(2);
        	                		    if ( ((LA(1) == LOCAL_SYMBOL)) && (((LA76_1 >= ACCOUNT_SYMBOL) && (LA76_1 <= YEAR_SYMBOL)) || LA76_1 == ALL_SYMBOL || LA76_1 == BACK_TICK_QUOTED_ID || LA76_1 == DOUBLE_QUOTED_TEXT || LA76_1 == IDENTIFIER))
        	                		    {
        	                		        alt76=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	                case NO_WRITE_TO_BINLOG_SYMBOL:
        	                	{
        	                		alt76=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt76)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:272:36: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1288);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1291);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:273:4: OPTIMIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( no_write_to_bin_log )?
        	    {
        	         MATCHT(OPTIMIZE_SYMBOL, &FOLLOW_OPTIMIZE_SYMBOL_in_alter_partition1296);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1298);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:273:37: ( no_write_to_bin_log )?
        	        {
        	            int alt77=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		{
        	                		    int LA77_1 = LA(2);
        	                		    if ( ((LA(1) == LOCAL_SYMBOL)) && (((LA77_1 >= ACCOUNT_SYMBOL) && (LA77_1 <= LIST_SYMBOL)) || ((LA77_1 >= LOCKS_SYMBOL) && (LA77_1 <= YEAR_SYMBOL)) || LA77_1 == ALL_SYMBOL || LA77_1 == BACK_TICK_QUOTED_ID || LA77_1 == DOUBLE_QUOTED_TEXT || LA77_1 == IDENTIFIER))
        	                		    {
        	                		        alt77=1;
        	                		    }
        	                		    else if ( (LA77_1 == LOCAL_SYMBOL))
        	                		    {
        	                		        {
        	                		            int LA77_13 = LA(3);
        	                		            if ( ((LA(1) == LOCAL_SYMBOL)))
        	                		            {
        	                		                alt77=1;
        	                		            }
        	                		        }
        	                		    }
        	                		}
        	                	}
        	                    break;
        	                case NO_WRITE_TO_BINLOG_SYMBOL:
        	                	{
        	                		alt77=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt77)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:273:37: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1300);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1303);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:273:85: ( no_write_to_bin_log )?
        	        {
        	            int alt78=2;
        	            {
        	                int LA78_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA78_0 == LOCAL_SYMBOL))
        	                {
        	                    alt78=1;
        	                }
        	                else if ( (LA78_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt78=1;
        	                }
        	            }
        	            switch (alt78)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:273:85: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1305);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:274:4: ANALYZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list
        	    {
        	         MATCHT(ANALYZE_SYMBOL, &FOLLOW_ANALYZE_SYMBOL_in_alter_partition1312);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1314);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:274:36: ( no_write_to_bin_log )?
        	        {
        	            int alt79=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		{
        	                		    int LA79_1 = LA(2);
        	                		    if ( ((LA(1) == LOCAL_SYMBOL)) && (((LA79_1 >= ACCOUNT_SYMBOL) && (LA79_1 <= YEAR_SYMBOL)) || LA79_1 == ALL_SYMBOL || LA79_1 == BACK_TICK_QUOTED_ID || LA79_1 == DOUBLE_QUOTED_TEXT || LA79_1 == IDENTIFIER))
        	                		    {
        	                		        alt79=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	                case NO_WRITE_TO_BINLOG_SYMBOL:
        	                	{
        	                		alt79=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt79)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:274:36: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1316);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1319);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:275:4: CHECK_SYMBOL PARTITION_SYMBOL all_or_partition_name_list ( check_option )*
        	    {
        	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_alter_partition1324);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1326);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1328);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:275:61: ( check_option )*

        	        for (;;)
        	        {
        	            int alt80=2;
        	            switch ( LA(1) )
        	            {
        	            case CHANGED_SYMBOL:
        	            case EXTENDED_SYMBOL:
        	            case FAST_SYMBOL:
        	            case MEDIUM_SYMBOL:
        	            case QUICK_SYMBOL:
        	            case FOR_SYMBOL:
        	            	{
        	            		alt80=1;
        	            	}
        	                break;

        	            }

        	            switch (alt80)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:275:61: check_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_check_option_in_alter_partition1330);
        	        	        check_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop80;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop80: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:276:4: REPAIR_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? all_or_partition_name_list ( repair_option )*
        	    {
        	         MATCHT(REPAIR_SYMBOL, &FOLLOW_REPAIR_SYMBOL_in_alter_partition1336);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1338);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:276:35: ( no_write_to_bin_log )?
        	        {
        	            int alt81=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		{
        	                		    int LA81_1 = LA(2);
        	                		    if ( ((LA(1) == LOCAL_SYMBOL)) && (((LA81_1 >= ACCOUNT_SYMBOL) && (LA81_1 <= EXPORT_SYMBOL)) || ((LA81_1 >= EXTENT_SIZE_SYMBOL) && (LA81_1 <= QUERY_SYMBOL)) || ((LA81_1 >= READ_ONLY_SYMBOL) && (LA81_1 <= USER_SYMBOL)) || ((LA81_1 >= VARIABLES_SYMBOL) && (LA81_1 <= YEAR_SYMBOL)) || LA81_1 == ALL_SYMBOL || LA81_1 == BACK_TICK_QUOTED_ID || LA81_1 == DOUBLE_QUOTED_TEXT || LA81_1 == IDENTIFIER))
        	                		    {
        	                		        alt81=1;
        	                		    }
        	                		    else if ( (LA81_1 == EXTENDED_SYMBOL || LA81_1 == QUICK_SYMBOL || LA81_1 == USE_FRM_SYMBOL))
        	                		    {
        	                		        {
        	                		            int LA81_13 = LA(3);
        	                		            if ( ((LA(1) == LOCAL_SYMBOL)))
        	                		            {
        	                		                alt81=1;
        	                		            }
        	                		        }
        	                		    }
        	                		}
        	                	}
        	                    break;
        	                case NO_WRITE_TO_BINLOG_SYMBOL:
        	                	{
        	                		alt81=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt81)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:276:35: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1340);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1343);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:276:83: ( repair_option )*

        	        for (;;)
        	        {
        	            int alt82=2;
        	            switch ( LA(1) )
        	            {
        	            case EXTENDED_SYMBOL:
        	            case QUICK_SYMBOL:
        	            case USE_FRM_SYMBOL:
        	            	{
        	            		alt82=1;
        	            	}
        	                break;

        	            }

        	            switch (alt82)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:276:83: repair_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_repair_option_in_alter_partition1345);
        	        	        repair_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop82;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop82: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:277:4: COALESCE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? real_ulong_number
        	    {
        	         MATCHT(COALESCE_SYMBOL, &FOLLOW_COALESCE_SYMBOL_in_alter_partition1351);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1353);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:277:37: ( no_write_to_bin_log )?
        	        {
        	            int alt83=2;
        	            {
        	                int LA83_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA83_0 == LOCAL_SYMBOL))
        	                {
        	                    alt83=1;
        	                }
        	                else if ( (LA83_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt83=1;
        	                }
        	            }
        	            switch (alt83)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:277:37: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1355);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_alter_partition1358);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:278:4: {...}? TRUNCATE_SYMBOL PARTITION_SYMBOL all_or_partition_name_list
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"alter_partition";


        	        }

        	         MATCHT(TRUNCATE_SYMBOL, &FOLLOW_TRUNCATE_SYMBOL_in_alter_partition1365);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1367);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1369);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:279:4: EXCHANGE_SYMBOL PARTITION_SYMBOL identifier WITH_SYMBOL TABLE_SYMBOL table_ref ( validation )?
        	    {
        	         MATCHT(EXCHANGE_SYMBOL, &FOLLOW_EXCHANGE_SYMBOL_in_alter_partition1374);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1376);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_alter_partition1378);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_alter_partition1380);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_alter_partition1382);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_in_alter_partition1384);
        	        table_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:279:83: ( validation )?
        	        {
        	            int alt84=2;
        	            {
        	                int LA84_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50706)) && (LA84_0 == WITHOUT_SYMBOL || LA84_0 == WITH_SYMBOL))
        	                {
        	                    alt84=1;
        	                }
        	            }
        	            switch (alt84)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:279:83: validation
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_validation_in_alter_partition1386);
        	        	        validation(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:280:4: REORGANIZE_SYMBOL PARTITION_SYMBOL ( no_write_to_bin_log )? ( identifier_list INTO_SYMBOL partition_definitions )?
        	    {
        	         MATCHT(REORGANIZE_SYMBOL, &FOLLOW_REORGANIZE_SYMBOL_in_alter_partition1392);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1394);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:280:39: ( no_write_to_bin_log )?
        	        {
        	            int alt85=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		{
        	                		    int LA85_1 = LA(2);
        	                		    if ( ((LA(1) == LOCAL_SYMBOL)) && (LA85_1 == EOF || ((LA85_1 >= ACCOUNT_SYMBOL) && (LA85_1 <= YEAR_SYMBOL)) || LA85_1 == SEMICOLON_SYMBOL || LA85_1 == BACK_TICK_QUOTED_ID || LA85_1 == DOUBLE_QUOTED_TEXT || LA85_1 == IDENTIFIER))
        	                		    {
        	                		        alt85=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	                case NO_WRITE_TO_BINLOG_SYMBOL:
        	                	{
        	                		alt85=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt85)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:280:39: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_alter_partition1396);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:280:60: ( identifier_list INTO_SYMBOL partition_definitions )?
        	        {
        	            int alt86=2;
        	            {
        	                int LA86_0 = LA(1);
        	                if ( (((LA86_0 >= ACCOUNT_SYMBOL) && (LA86_0 <= YEAR_SYMBOL)) || LA86_0 == BACK_TICK_QUOTED_ID || LA86_0 == DOUBLE_QUOTED_TEXT || LA86_0 == IDENTIFIER))
        	                {
        	                    alt86=1;
        	                }
        	            }
        	            switch (alt86)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:280:61: identifier_list INTO_SYMBOL partition_definitions
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_list_in_alter_partition1400);
        	        	        identifier_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_alter_partition1402);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_partition_definitions_in_alter_partition1404);
        	        	        partition_definitions(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_partitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:281:4: {...}? DISCARD_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50704)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50704";
        	                EXCEPTION->ruleName	 = (void *)"alter_partition";


        	        }

        	         MATCHT(DISCARD_SYMBOL, &FOLLOW_DISCARD_SYMBOL_in_alter_partition1413);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1415);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1417);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1419);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:282:4: {...}? IMPORT_SYMBOL PARTITION_SYMBOL all_or_partition_name_list TABLESPACE_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50704)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50704";
        	                EXCEPTION->ruleName	 = (void *)"alter_partition";


        	        }

        	         MATCHT(IMPORT_SYMBOL, &FOLLOW_IMPORT_SYMBOL_in_alter_partition1426);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_alter_partition1428);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_all_or_partition_name_list_in_alter_partition1430);
        	        all_or_partition_name_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_alter_partition1432);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_partitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulealter_partitionEx; /* Prevent compiler warnings */
    rulealter_partitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_partition */

/**
 * $ANTLR start validation
 * MySQLSimpleParser.g:285:1: validation :{...}? => ( WITH_SYMBOL | WITHOUT_SYMBOL ) VALIDATION_SYMBOL ;
 */
static void
validation(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:285:11: ({...}? => ( WITH_SYMBOL | WITHOUT_SYMBOL ) VALIDATION_SYMBOL )
        // MySQLSimpleParser.g:286:2: {...}? => ( WITH_SYMBOL | WITHOUT_SYMBOL ) VALIDATION_SYMBOL
        {
            if ( !((SERVER_VERSION >= 50706)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
                    EXCEPTION->ruleName	 = (void *)"validation";


            }

            if ( LA(1) == WITHOUT_SYMBOL || LA(1) == WITH_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulevalidationEx;
            }


             MATCHT(VALIDATION_SYMBOL, &FOLLOW_VALIDATION_SYMBOL_in_validation1453);
            if  (HASEXCEPTION())
            {
                goto rulevalidationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulevalidationEx; /* Prevent compiler warnings */
    rulevalidationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end validation */

/**
 * $ANTLR start remove_partitioning
 * MySQLSimpleParser.g:289:1: remove_partitioning :{...}? => REMOVE_SYMBOL PARTITIONING_SYMBOL ;
 */
static void
remove_partitioning(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:289:20: ({...}? => REMOVE_SYMBOL PARTITIONING_SYMBOL )
        // MySQLSimpleParser.g:290:2: {...}? => REMOVE_SYMBOL PARTITIONING_SYMBOL
        {
            if ( !((SERVER_VERSION >= 50100)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
                    EXCEPTION->ruleName	 = (void *)"remove_partitioning";


            }

             MATCHT(REMOVE_SYMBOL, &FOLLOW_REMOVE_SYMBOL_in_remove_partitioning1466);
            if  (HASEXCEPTION())
            {
                goto ruleremove_partitioningEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(PARTITIONING_SYMBOL, &FOLLOW_PARTITIONING_SYMBOL_in_remove_partitioning1468);
            if  (HASEXCEPTION())
            {
                goto ruleremove_partitioningEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleremove_partitioningEx; /* Prevent compiler warnings */
    ruleremove_partitioningEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end remove_partitioning */

/**
 * $ANTLR start all_or_partition_name_list
 * MySQLSimpleParser.g:293:1: all_or_partition_name_list : ( ALL_SYMBOL | identifier_list );
 */
static void
all_or_partition_name_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:293:27: ( ALL_SYMBOL | identifier_list )

            ANTLR3_UINT32 alt88;

            alt88=2;

            {
                int LA88_0 = LA(1);
                if ( (LA88_0 == ALL_SYMBOL))
                {
                    alt88=1;
                }
                else if ( (((LA88_0 >= ACCOUNT_SYMBOL) && (LA88_0 <= YEAR_SYMBOL)) || LA88_0 == BACK_TICK_QUOTED_ID || LA88_0 == DOUBLE_QUOTED_TEXT || LA88_0 == IDENTIFIER))
                {
                    alt88=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 88;
                    EXCEPTION->state        = 0;


                    goto ruleall_or_partition_name_listEx;

                }
            }
            switch (alt88)
            {
        	case 1:
        	    // MySQLSimpleParser.g:294:2: ALL_SYMBOL
        	    {
        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_all_or_partition_name_list1477);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_or_partition_name_listEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:295:4: identifier_list
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_list_in_all_or_partition_name_list1482);
        	        identifier_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_or_partition_name_listEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleall_or_partition_name_listEx; /* Prevent compiler warnings */
    ruleall_or_partition_name_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end all_or_partition_name_list */

/**
 * $ANTLR start alter_tablespace
 * MySQLSimpleParser.g:298:1: alter_tablespace : TABLESPACE_SYMBOL tablespace_ref ( ( ADD_SYMBOL | DROP_SYMBOL ) DATAFILE_SYMBOL string_literal ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )? | CHANGE_SYMBOL DATAFILE_SYMBOL string_literal ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )? | READ_ONLY_SYMBOL | READ_WRITE_SYMBOL | NOT_SYMBOL ACCESSIBLE_SYMBOL ) ;
 */
static void
alter_tablespace(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:298:17: ( TABLESPACE_SYMBOL tablespace_ref ( ( ADD_SYMBOL | DROP_SYMBOL ) DATAFILE_SYMBOL string_literal ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )? | CHANGE_SYMBOL DATAFILE_SYMBOL string_literal ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )? | READ_ONLY_SYMBOL | READ_WRITE_SYMBOL | NOT_SYMBOL ACCESSIBLE_SYMBOL ) )
        // MySQLSimpleParser.g:299:2: TABLESPACE_SYMBOL tablespace_ref ( ( ADD_SYMBOL | DROP_SYMBOL ) DATAFILE_SYMBOL string_literal ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )? | CHANGE_SYMBOL DATAFILE_SYMBOL string_literal ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )? | READ_ONLY_SYMBOL | READ_WRITE_SYMBOL | NOT_SYMBOL ACCESSIBLE_SYMBOL )
        {
             MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_alter_tablespace1491);
            if  (HASEXCEPTION())
            {
                goto rulealter_tablespaceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_tablespace_ref_in_alter_tablespace1493);
            tablespace_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_tablespaceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:300:2: ( ( ADD_SYMBOL | DROP_SYMBOL ) DATAFILE_SYMBOL string_literal ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )? | CHANGE_SYMBOL DATAFILE_SYMBOL string_literal ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )? | READ_ONLY_SYMBOL | READ_WRITE_SYMBOL | NOT_SYMBOL ACCESSIBLE_SYMBOL )
            {
                int alt95=5;
                switch ( LA(1) )
                {
                case ADD_SYMBOL:
                case DROP_SYMBOL:
                	{
                		alt95=1;
                	}
                    break;
                case CHANGE_SYMBOL:
                	{
                		alt95=2;
                	}
                    break;
                case READ_ONLY_SYMBOL:
                	{
                		alt95=3;
                	}
                    break;
                case READ_WRITE_SYMBOL:
                	{
                		alt95=4;
                	}
                    break;
                case NOT_SYMBOL:
                	{
                		alt95=5;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 95;
                    EXCEPTION->state        = 0;


                    goto rulealter_tablespaceEx;

                }

                switch (alt95)
                {
            	case 1:
            	    // MySQLSimpleParser.g:301:3: ( ADD_SYMBOL | DROP_SYMBOL ) DATAFILE_SYMBOL string_literal ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )?
            	    {
            	        if ( LA(1) == ADD_SYMBOL || LA(1) == DROP_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulealter_tablespaceEx;
            	        }


            	         MATCHT(DATAFILE_SYMBOL, &FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1508);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_alter_tablespace1510);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:301:61: ( alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )* )?
            	        {
            	            int alt91=2;
            	            switch ( LA(1) )
            	            {
            	                case AUTOEXTEND_SIZE_SYMBOL:
            	                case ENGINE_SYMBOL:
            	                case INITIAL_SIZE_SYMBOL:
            	                case MAX_SIZE_SYMBOL:
            	                case NO_WAIT_SYMBOL:
            	                case STORAGE_SYMBOL:
            	                case WAIT_SYMBOL:
            	                	{
            	                		alt91=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt91)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:301:62: alter_tablespace_option ( ( COMMA_SYMBOL )? alter_tablespace_option )*
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_alter_tablespace_option_in_alter_tablespace1513);
            	        	        alter_tablespace_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_tablespaceEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:301:86: ( ( COMMA_SYMBOL )? alter_tablespace_option )*

            	        	        for (;;)
            	        	        {
            	        	            int alt90=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case AUTOEXTEND_SIZE_SYMBOL:
            	        	            case ENGINE_SYMBOL:
            	        	            case INITIAL_SIZE_SYMBOL:
            	        	            case MAX_SIZE_SYMBOL:
            	        	            case NO_WAIT_SYMBOL:
            	        	            case STORAGE_SYMBOL:
            	        	            case WAIT_SYMBOL:
            	        	            case COMMA_SYMBOL:
            	        	            	{
            	        	            		alt90=1;
            	        	            	}
            	        	                break;

            	        	            }

            	        	            switch (alt90)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:301:87: ( COMMA_SYMBOL )? alter_tablespace_option
            	        	        	    {
            	        	        	        // MySQLSimpleParser.g:301:87: ( COMMA_SYMBOL )?
            	        	        	        {
            	        	        	            int alt89=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	                case COMMA_SYMBOL:
            	        	        	                	{
            	        	        	                		alt89=1;
            	        	        	                	}
            	        	        	                    break;
            	        	        	            }

            	        	        	            switch (alt89)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:301:87: COMMA_SYMBOL
            	        	        	        	    {
            	        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_alter_tablespace1516);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto rulealter_tablespaceEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_alter_tablespace_option_in_alter_tablespace1519);
            	        	        	        alter_tablespace_option(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulealter_tablespaceEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	        	default:
            	        	        	    goto loop90;	/* break out of the loop */
            	        	        	    break;
            	        	            }
            	        	        }
            	        	        loop90: ; /* Jump out to here if this rule does not match */


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:303:5: CHANGE_SYMBOL DATAFILE_SYMBOL string_literal ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )?
            	    {
            	         MATCHT(CHANGE_SYMBOL, &FOLLOW_CHANGE_SYMBOL_in_alter_tablespace1532);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DATAFILE_SYMBOL, &FOLLOW_DATAFILE_SYMBOL_in_alter_tablespace1534);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_alter_tablespace1536);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:303:50: ( change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )* )?
            	        {
            	            int alt94=2;
            	            switch ( LA(1) )
            	            {
            	                case AUTOEXTEND_SIZE_SYMBOL:
            	                case INITIAL_SIZE_SYMBOL:
            	                case MAX_SIZE_SYMBOL:
            	                	{
            	                		alt94=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt94)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:303:51: change_tablespace_option ( ( COMMA_SYMBOL )? change_tablespace_option )*
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_change_tablespace_option_in_alter_tablespace1539);
            	        	        change_tablespace_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulealter_tablespaceEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:303:76: ( ( COMMA_SYMBOL )? change_tablespace_option )*

            	        	        for (;;)
            	        	        {
            	        	            int alt93=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case AUTOEXTEND_SIZE_SYMBOL:
            	        	            case INITIAL_SIZE_SYMBOL:
            	        	            case MAX_SIZE_SYMBOL:
            	        	            case COMMA_SYMBOL:
            	        	            	{
            	        	            		alt93=1;
            	        	            	}
            	        	                break;

            	        	            }

            	        	            switch (alt93)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:303:77: ( COMMA_SYMBOL )? change_tablespace_option
            	        	        	    {
            	        	        	        // MySQLSimpleParser.g:303:77: ( COMMA_SYMBOL )?
            	        	        	        {
            	        	        	            int alt92=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	                case COMMA_SYMBOL:
            	        	        	                	{
            	        	        	                		alt92=1;
            	        	        	                	}
            	        	        	                    break;
            	        	        	            }

            	        	        	            switch (alt92)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:303:77: COMMA_SYMBOL
            	        	        	        	    {
            	        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_alter_tablespace1542);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto rulealter_tablespaceEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_change_tablespace_option_in_alter_tablespace1545);
            	        	        	        change_tablespace_option(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulealter_tablespaceEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	        	default:
            	        	        	    goto loop93;	/* break out of the loop */
            	        	        	    break;
            	        	            }
            	        	        }
            	        	        loop93: ; /* Jump out to here if this rule does not match */


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:304:5: READ_ONLY_SYMBOL
            	    {
            	         MATCHT(READ_ONLY_SYMBOL, &FOLLOW_READ_ONLY_SYMBOL_in_alter_tablespace1555);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:305:5: READ_WRITE_SYMBOL
            	    {
            	         MATCHT(READ_WRITE_SYMBOL, &FOLLOW_READ_WRITE_SYMBOL_in_alter_tablespace1561);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:306:5: NOT_SYMBOL ACCESSIBLE_SYMBOL
            	    {
            	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_alter_tablespace1567);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(ACCESSIBLE_SYMBOL, &FOLLOW_ACCESSIBLE_SYMBOL_in_alter_tablespace1569);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_tablespaceEx; /* Prevent compiler warnings */
    rulealter_tablespaceEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_tablespace */

/**
 * $ANTLR start alter_tablespace_option
 * MySQLSimpleParser.g:310:1: alter_tablespace_option : ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) );
 */
static void
alter_tablespace_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:310:24: ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) )

            ANTLR3_UINT32 alt101;

            alt101=5;

            switch ( LA(1) )
            {
            case INITIAL_SIZE_SYMBOL:
            	{
            		alt101=1;
            	}
                break;
            case AUTOEXTEND_SIZE_SYMBOL:
            	{
            		alt101=2;
            	}
                break;
            case MAX_SIZE_SYMBOL:
            	{
            		alt101=3;
            	}
                break;
            case ENGINE_SYMBOL:
            case STORAGE_SYMBOL:
            	{
            		alt101=4;
            	}
                break;
            case NO_WAIT_SYMBOL:
            case WAIT_SYMBOL:
            	{
            		alt101=5;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 101;
                EXCEPTION->state        = 0;


                goto rulealter_tablespace_optionEx;

            }

            switch (alt101)
            {
        	case 1:
        	    // MySQLSimpleParser.g:311:2: INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(INITIAL_SIZE_SYMBOL, &FOLLOW_INITIAL_SIZE_SYMBOL_in_alter_tablespace_option1581);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:311:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt96=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt96=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt96)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:311:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1583);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_alter_tablespace_option1586);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:312:4: AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(AUTOEXTEND_SIZE_SYMBOL, &FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_alter_tablespace_option1591);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:312:27: ( EQUAL_OPERATOR )?
        	        {
        	            int alt97=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt97=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt97)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:312:27: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1593);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_alter_tablespace_option1596);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:313:4: MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(MAX_SIZE_SYMBOL, &FOLLOW_MAX_SIZE_SYMBOL_in_alter_tablespace_option1601);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:313:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt98=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt98=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt98)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:313:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1603);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_alter_tablespace_option1606);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:314:4: ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:314:4: ( STORAGE_SYMBOL )?
        	        {
        	            int alt99=2;
        	            switch ( LA(1) )
        	            {
        	                case STORAGE_SYMBOL:
        	                	{
        	                		alt99=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt99)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:314:4: STORAGE_SYMBOL
        	        	    {
        	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_alter_tablespace_option1611);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_alter_tablespace_option1614);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:314:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt100=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt100=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt100)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:314:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_tablespace_option1616);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulealter_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_alter_tablespace_option1619);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:315:4: ( WAIT_SYMBOL | NO_WAIT_SYMBOL )
        	    {
        	        if ( LA(1) == NO_WAIT_SYMBOL || LA(1) == WAIT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulealter_tablespace_optionEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulealter_tablespace_optionEx; /* Prevent compiler warnings */
    rulealter_tablespace_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_tablespace_option */

/**
 * $ANTLR start change_tablespace_option
 * MySQLSimpleParser.g:318:1: change_tablespace_option : ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number );
 */
static void
change_tablespace_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:318:25: ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )

            ANTLR3_UINT32 alt105;

            alt105=3;

            switch ( LA(1) )
            {
            case INITIAL_SIZE_SYMBOL:
            	{
            		alt105=1;
            	}
                break;
            case AUTOEXTEND_SIZE_SYMBOL:
            	{
            		alt105=2;
            	}
                break;
            case MAX_SIZE_SYMBOL:
            	{
            		alt105=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 105;
                EXCEPTION->state        = 0;


                goto rulechange_tablespace_optionEx;

            }

            switch (alt105)
            {
        	case 1:
        	    // MySQLSimpleParser.g:319:2: INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(INITIAL_SIZE_SYMBOL, &FOLLOW_INITIAL_SIZE_SYMBOL_in_change_tablespace_option1639);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:319:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt102=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt102=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt102)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:319:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1641);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulechange_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_change_tablespace_option1644);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:320:4: AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(AUTOEXTEND_SIZE_SYMBOL, &FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_change_tablespace_option1649);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:320:27: ( EQUAL_OPERATOR )?
        	        {
        	            int alt103=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt103=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt103)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:320:27: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1651);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulechange_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_change_tablespace_option1654);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:321:4: MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(MAX_SIZE_SYMBOL, &FOLLOW_MAX_SIZE_SYMBOL_in_change_tablespace_option1659);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:321:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt104=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt104=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt104)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:321:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_change_tablespace_option1661);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulechange_tablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_change_tablespace_option1664);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulechange_tablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulechange_tablespace_optionEx; /* Prevent compiler warnings */
    rulechange_tablespace_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end change_tablespace_option */

/**
 * $ANTLR start alter_view
 * MySQLSimpleParser.g:324:1: alter_view : ( ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) )? ( definer_clause )? ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_ref ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )? ;
 */
static void
alter_view(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:324:11: ( ( ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) )? ( definer_clause )? ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_ref ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )? )
        // MySQLSimpleParser.g:325:2: ( ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) )? ( definer_clause )? ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_ref ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )?
        {
            // MySQLSimpleParser.g:325:2: ( ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) )?
            {
                int alt106=2;
                switch ( LA(1) )
                {
                    case ALGORITHM_SYMBOL:
                    	{
                    		alt106=1;
                    	}
                        break;
                }

                switch (alt106)
                {
            	case 1:
            	    // MySQLSimpleParser.g:325:3: ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL )
            	    {
            	         MATCHT(ALGORITHM_SYMBOL, &FOLLOW_ALGORITHM_SYMBOL_in_alter_view1674);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_alter_view1676);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == MERGE_SYMBOL || LA(1) == TEMPTABLE_SYMBOL || LA(1) == UNDEFINED_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulealter_viewEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:326:3: ( definer_clause )?
            {
                int alt107=2;
                switch ( LA(1) )
                {
                    case DEFINER_SYMBOL:
                    	{
                    		alt107=1;
                    	}
                        break;
                }

                switch (alt107)
                {
            	case 1:
            	    // MySQLSimpleParser.g:326:3: definer_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_definer_clause_in_alter_view1694);
            	        definer_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:327:3: ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )?
            {
                int alt108=2;
                switch ( LA(1) )
                {
                    case SQL_SYMBOL:
                    	{
                    		alt108=1;
                    	}
                        break;
                }

                switch (alt108)
                {
            	case 1:
            	    // MySQLSimpleParser.g:327:4: SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL )
            	    {
            	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_alter_view1700);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(SECURITY_SYMBOL, &FOLLOW_SECURITY_SYMBOL_in_alter_view1702);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == DEFINER_SYMBOL || LA(1) == INVOKER_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulealter_viewEx;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_alter_view1716);
            if  (HASEXCEPTION())
            {
                goto rulealter_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_view_ref_in_alter_view1718);
            view_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:328:24: ( identifier_list_with_parentheses )?
            {
                int alt109=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt109=1;
                    	}
                        break;
                }

                switch (alt109)
                {
            	case 1:
            	    // MySQLSimpleParser.g:328:24: identifier_list_with_parentheses
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_alter_view1720);
            	        identifier_list_with_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_alter_view1723);
            if  (HASEXCEPTION())
            {
                goto rulealter_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_select_statement_in_alter_view1725);
            select_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:329:3: ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )?
            {
                int alt111=2;
                switch ( LA(1) )
                {
                    case WITH_SYMBOL:
                    	{
                    		alt111=1;
                    	}
                        break;
                }

                switch (alt111)
                {
            	case 1:
            	    // MySQLSimpleParser.g:329:4: WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL
            	    {
            	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_alter_view1730);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:329:16: ( CASCADED_SYMBOL | LOCAL_SYMBOL )?
            	        {
            	            int alt110=2;
            	            switch ( LA(1) )
            	            {
            	                case CASCADED_SYMBOL:
            	                case LOCAL_SYMBOL:
            	                	{
            	                		alt110=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt110)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == CASCADED_SYMBOL || LA(1) == LOCAL_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto rulealter_viewEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_alter_view1741);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_alter_view1743);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_viewEx; /* Prevent compiler warnings */
    rulealter_viewEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_view */

/**
 * $ANTLR start create_statement
 * MySQLSimpleParser.g:337:1: create_statement : CREATE_SYMBOL ( create_table_tail | create_index_tail | create_database_tail | ( definer_clause )? ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail ) | view_replace_or_algorithm ( definer_clause )? create_view_tail | create_logfile_group_tail | create_server_tail | create_tablespace_tail ) ;
 */
static void
create_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:337:17: ( CREATE_SYMBOL ( create_table_tail | create_index_tail | create_database_tail | ( definer_clause )? ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail ) | view_replace_or_algorithm ( definer_clause )? create_view_tail | create_logfile_group_tail | create_server_tail | create_tablespace_tail ) )
        // MySQLSimpleParser.g:338:2: CREATE_SYMBOL ( create_table_tail | create_index_tail | create_database_tail | ( definer_clause )? ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail ) | view_replace_or_algorithm ( definer_clause )? create_view_tail | create_logfile_group_tail | create_server_tail | create_tablespace_tail )
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_statement1759);
            if  (HASEXCEPTION())
            {
                goto rulecreate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:339:2: ( create_table_tail | create_index_tail | create_database_tail | ( definer_clause )? ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail ) | view_replace_or_algorithm ( definer_clause )? create_view_tail | create_logfile_group_tail | create_server_tail | create_tablespace_tail )
            {
                int alt115=8;
                switch ( LA(1) )
                {
                case TEMPORARY_SYMBOL:
                case TABLE_SYMBOL:
                	{
                		alt115=1;
                	}
                    break;
                case FULLTEXT_SYMBOL:
                case INDEX_SYMBOL:
                case OFFLINE_SYMBOL:
                case ONLINE_SYMBOL:
                case SPATIAL_SYMBOL:
                case UNIQUE_SYMBOL:
                	{
                		alt115=2;
                	}
                    break;
                case DATABASE_SYMBOL:
                	{
                		alt115=3;
                	}
                    break;
                case AGGREGATE_SYMBOL:
                case DEFINER_SYMBOL:
                case EVENT_SYMBOL:
                case FUNCTION_SYMBOL:
                case VIEW_SYMBOL:
                case PROCEDURE_SYMBOL:
                case SQL_SYMBOL:
                case TRIGGER_SYMBOL:
                	{
                		alt115=4;
                	}
                    break;
                case ALGORITHM_SYMBOL:
                case OR_SYMBOL:
                	{
                		alt115=5;
                	}
                    break;
                case LOGFILE_SYMBOL:
                	{
                		alt115=6;
                	}
                    break;
                case SERVER_SYMBOL:
                	{
                		alt115=7;
                	}
                    break;
                case TABLESPACE_SYMBOL:
                	{
                		alt115=8;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 115;
                    EXCEPTION->state        = 0;


                    goto rulecreate_statementEx;

                }

                switch (alt115)
                {
            	case 1:
            	    // MySQLSimpleParser.g:340:3: create_table_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_table_tail_in_create_statement1766);
            	        create_table_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:341:5: create_index_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_index_tail_in_create_statement1772);
            	        create_index_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:342:5: create_database_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_database_tail_in_create_statement1778);
            	        create_database_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:343:5: ( definer_clause )? ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail )
            	    {
            	        // MySQLSimpleParser.g:343:5: ( definer_clause )?
            	        {
            	            int alt112=2;
            	            switch ( LA(1) )
            	            {
            	                case DEFINER_SYMBOL:
            	                	{
            	                		alt112=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt112)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:343:5: definer_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_definer_clause_in_create_statement1784);
            	        	        definer_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:344:4: ({...}? create_event_tail | create_view_tail | create_routine_or_udf | create_trigger_tail )
            	        {
            	            int alt113=4;
            	            switch ( LA(1) )
            	            {
            	            case EVENT_SYMBOL:
            	            	{
            	            		alt113=1;
            	            	}
            	                break;
            	            case VIEW_SYMBOL:
            	            case SQL_SYMBOL:
            	            	{
            	            		alt113=2;
            	            	}
            	                break;
            	            case AGGREGATE_SYMBOL:
            	            case FUNCTION_SYMBOL:
            	            case PROCEDURE_SYMBOL:
            	            	{
            	            		alt113=3;
            	            	}
            	                break;
            	            case TRIGGER_SYMBOL:
            	            	{
            	            		alt113=4;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 113;
            	                EXCEPTION->state        = 0;


            	                goto rulecreate_statementEx;

            	            }

            	            switch (alt113)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:345:5: {...}? create_event_tail
            	        	    {
            	        	        if ( !((SERVER_VERSION >= 50100)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
            	        	                EXCEPTION->ruleName	 = (void *)"create_statement";


            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_create_event_tail_in_create_statement1798);
            	        	        create_event_tail(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:346:7: create_view_tail
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_create_view_tail_in_create_statement1806);
            	        	        create_view_tail(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 3:
            	        	    // MySQLSimpleParser.g:347:7: create_routine_or_udf
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_create_routine_or_udf_in_create_statement1814);
            	        	        create_routine_or_udf(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 4:
            	        	    // MySQLSimpleParser.g:348:7: create_trigger_tail
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_create_trigger_tail_in_create_statement1822);
            	        	        create_trigger_tail(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:350:5: view_replace_or_algorithm ( definer_clause )? create_view_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_view_replace_or_algorithm_in_create_statement1833);
            	        view_replace_or_algorithm(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:350:31: ( definer_clause )?
            	        {
            	            int alt114=2;
            	            switch ( LA(1) )
            	            {
            	                case DEFINER_SYMBOL:
            	                	{
            	                		alt114=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt114)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:350:31: definer_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_definer_clause_in_create_statement1835);
            	        	        definer_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_create_view_tail_in_create_statement1838);
            	        create_view_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:351:5: create_logfile_group_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_logfile_group_tail_in_create_statement1844);
            	        create_logfile_group_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 7:
            	    // MySQLSimpleParser.g:352:5: create_server_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_server_tail_in_create_statement1850);
            	        create_server_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 8:
            	    // MySQLSimpleParser.g:353:5: create_tablespace_tail
            	    {
            	        FOLLOWPUSH(FOLLOW_create_tablespace_tail_in_create_statement1856);
            	        create_tablespace_tail(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_statementEx; /* Prevent compiler warnings */
    rulecreate_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_statement */

/**
 * $ANTLR start create_database
 * MySQLSimpleParser.g:357:1: create_database : CREATE_SYMBOL create_database_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_database(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:357:16: ( CREATE_SYMBOL create_database_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:358:2: CREATE_SYMBOL create_database_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_database1869);
            if  (HASEXCEPTION())
            {
                goto rulecreate_databaseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_database_tail_in_create_database1871);
            create_database_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_databaseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:358:37: ( SEMICOLON_SYMBOL )?
            {
                int alt116=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt116=1;
                    	}
                        break;
                }

                switch (alt116)
                {
            	case 1:
            	    // MySQLSimpleParser.g:358:37: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_database1873);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_databaseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_database1876);
            if  (HASEXCEPTION())
            {
                goto rulecreate_databaseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_databaseEx; /* Prevent compiler warnings */
    rulecreate_databaseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_database */

/**
 * $ANTLR start create_database_tail
 * MySQLSimpleParser.g:361:1: create_database_tail : DATABASE_SYMBOL ( if_not_exists )? schema_name ( database_option )* ;
 */
static void
create_database_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:361:21: ( DATABASE_SYMBOL ( if_not_exists )? schema_name ( database_option )* )
        // MySQLSimpleParser.g:362:2: DATABASE_SYMBOL ( if_not_exists )? schema_name ( database_option )*
        {
             MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_create_database_tail1885);
            if  (HASEXCEPTION())
            {
                goto rulecreate_database_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:362:18: ( if_not_exists )?
            {
                int alt117=2;
                switch ( LA(1) )
                {
                    case IF_SYMBOL:
                    	{
                    		alt117=1;
                    	}
                        break;
                }

                switch (alt117)
                {
            	case 1:
            	    // MySQLSimpleParser.g:362:18: if_not_exists
            	    {
            	        FOLLOWPUSH(FOLLOW_if_not_exists_in_create_database_tail1887);
            	        if_not_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_database_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_schema_name_in_create_database_tail1890);
            schema_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_database_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:362:45: ( database_option )*

            for (;;)
            {
                int alt118=2;
                switch ( LA(1) )
                {
                case CHARSET_SYMBOL:
                case CHAR_SYMBOL:
                case COLLATE_SYMBOL:
                case DEFAULT_SYMBOL:
                	{
                		alt118=1;
                	}
                    break;

                }

                switch (alt118)
                {
            	case 1:
            	    // MySQLSimpleParser.g:362:45: database_option
            	    {
            	        FOLLOWPUSH(FOLLOW_database_option_in_create_database_tail1892);
            	        database_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_database_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop118;	/* break out of the loop */
            	    break;
                }
            }
            loop118: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_database_tailEx; /* Prevent compiler warnings */
    rulecreate_database_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_database_tail */

/**
 * $ANTLR start create_with_definer
 * MySQLSimpleParser.g:365:1: create_with_definer : CREATE_SYMBOL ( definer_clause )? ;
 */
static void
create_with_definer(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:365:20: ( CREATE_SYMBOL ( definer_clause )? )
        // MySQLSimpleParser.g:366:2: CREATE_SYMBOL ( definer_clause )?
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_with_definer1902);
            if  (HASEXCEPTION())
            {
                goto rulecreate_with_definerEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:366:16: ( definer_clause )?
            {
                int alt119=2;
                switch ( LA(1) )
                {
                    case DEFINER_SYMBOL:
                    	{
                    		alt119=1;
                    	}
                        break;
                }

                switch (alt119)
                {
            	case 1:
            	    // MySQLSimpleParser.g:366:16: definer_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_definer_clause_in_create_with_definer1904);
            	        definer_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_with_definerEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_with_definerEx; /* Prevent compiler warnings */
    rulecreate_with_definerEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_with_definer */

/**
 * $ANTLR start create_event
 * MySQLSimpleParser.g:369:1: create_event : create_with_definer create_event_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_event(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:369:13: ( create_with_definer create_event_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:370:2: create_with_definer create_event_tail ( SEMICOLON_SYMBOL )? EOF
        {
            FOLLOWPUSH(FOLLOW_create_with_definer_in_create_event1915);
            create_with_definer(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_eventEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_event_tail_in_create_event1917);
            create_event_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_eventEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:370:40: ( SEMICOLON_SYMBOL )?
            {
                int alt120=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt120=1;
                    	}
                        break;
                }

                switch (alt120)
                {
            	case 1:
            	    // MySQLSimpleParser.g:370:40: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_event1919);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_eventEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_event1922);
            if  (HASEXCEPTION())
            {
                goto rulecreate_eventEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_eventEx; /* Prevent compiler warnings */
    rulecreate_eventEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_event */

/**
 * $ANTLR start create_event_tail
 * MySQLSimpleParser.g:373:1: create_event_tail : EVENT_SYMBOL ( if_not_exists )? event_name ON_SYMBOL SCHEDULE_SYMBOL schedule ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? DO_SYMBOL compound_statement ;
 */
static void
create_event_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:373:18: ( EVENT_SYMBOL ( if_not_exists )? event_name ON_SYMBOL SCHEDULE_SYMBOL schedule ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? DO_SYMBOL compound_statement )
        // MySQLSimpleParser.g:374:2: EVENT_SYMBOL ( if_not_exists )? event_name ON_SYMBOL SCHEDULE_SYMBOL schedule ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )? ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )? ( COMMENT_SYMBOL string_literal )? DO_SYMBOL compound_statement
        {
             MATCHT(EVENT_SYMBOL, &FOLLOW_EVENT_SYMBOL_in_create_event_tail1931);
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:374:15: ( if_not_exists )?
            {
                int alt121=2;
                switch ( LA(1) )
                {
                    case IF_SYMBOL:
                    	{
                    		alt121=1;
                    	}
                        break;
                }

                switch (alt121)
                {
            	case 1:
            	    // MySQLSimpleParser.g:374:15: if_not_exists
            	    {
            	        FOLLOWPUSH(FOLLOW_if_not_exists_in_create_event_tail1933);
            	        if_not_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_event_name_in_create_event_tail1936);
            event_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_create_event_tail1938);
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(SCHEDULE_SYMBOL, &FOLLOW_SCHEDULE_SYMBOL_in_create_event_tail1940);
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_schedule_in_create_event_tail1942);
            schedule(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:375:3: ( ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL )?
            {
                int alt123=2;
                switch ( LA(1) )
                {
                    case ON_SYMBOL:
                    	{
                    		alt123=1;
                    	}
                        break;
                }

                switch (alt123)
                {
            	case 1:
            	    // MySQLSimpleParser.g:375:4: ON_SYMBOL COMPLETION_SYMBOL ( NOT_SYMBOL )? PRESERVE_SYMBOL
            	    {
            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_create_event_tail1947);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(COMPLETION_SYMBOL, &FOLLOW_COMPLETION_SYMBOL_in_create_event_tail1949);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:375:32: ( NOT_SYMBOL )?
            	        {
            	            int alt122=2;
            	            switch ( LA(1) )
            	            {
            	                case NOT_SYMBOL:
            	                	{
            	                		alt122=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt122)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:375:32: NOT_SYMBOL
            	        	    {
            	        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_create_event_tail1951);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_event_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(PRESERVE_SYMBOL, &FOLLOW_PRESERVE_SYMBOL_in_create_event_tail1954);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:376:3: ( ENABLE_SYMBOL | DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )? )?
            {
                int alt125=3;
                switch ( LA(1) )
                {
                    case ENABLE_SYMBOL:
                    	{
                    		alt125=1;
                    	}
                        break;
                    case DISABLE_SYMBOL:
                    	{
                    		alt125=2;
                    	}
                        break;
                }

                switch (alt125)
                {
            	case 1:
            	    // MySQLSimpleParser.g:376:4: ENABLE_SYMBOL
            	    {
            	         MATCHT(ENABLE_SYMBOL, &FOLLOW_ENABLE_SYMBOL_in_create_event_tail1961);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:376:20: DISABLE_SYMBOL ( ON_SYMBOL SLAVE_SYMBOL )?
            	    {
            	         MATCHT(DISABLE_SYMBOL, &FOLLOW_DISABLE_SYMBOL_in_create_event_tail1965);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:376:35: ( ON_SYMBOL SLAVE_SYMBOL )?
            	        {
            	            int alt124=2;
            	            switch ( LA(1) )
            	            {
            	                case ON_SYMBOL:
            	                	{
            	                		alt124=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt124)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:376:36: ON_SYMBOL SLAVE_SYMBOL
            	        	    {
            	        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_create_event_tail1968);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_event_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_create_event_tail1970);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_event_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:377:3: ( COMMENT_SYMBOL string_literal )?
            {
                int alt126=2;
                switch ( LA(1) )
                {
                    case COMMENT_SYMBOL:
                    	{
                    		alt126=1;
                    	}
                        break;
                }

                switch (alt126)
                {
            	case 1:
            	    // MySQLSimpleParser.g:377:4: COMMENT_SYMBOL string_literal
            	    {
            	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_create_event_tail1979);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_create_event_tail1981);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_event_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(DO_SYMBOL, &FOLLOW_DO_SYMBOL_in_create_event_tail1987);
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_in_create_event_tail1989);
            compound_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_event_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_event_tailEx; /* Prevent compiler warnings */
    rulecreate_event_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_event_tail */

/**
 * $ANTLR start create_routine
 * MySQLSimpleParser.g:381:1: create_routine : create_with_definer create_routine_or_udf ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_routine(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:381:15: ( create_with_definer create_routine_or_udf ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:382:2: create_with_definer create_routine_or_udf ( SEMICOLON_SYMBOL )? EOF
        {
            FOLLOWPUSH(FOLLOW_create_with_definer_in_create_routine1999);
            create_with_definer(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_routineEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_routine_or_udf_in_create_routine2001);
            create_routine_or_udf(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_routineEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:382:44: ( SEMICOLON_SYMBOL )?
            {
                int alt127=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt127=1;
                    	}
                        break;
                }

                switch (alt127)
                {
            	case 1:
            	    // MySQLSimpleParser.g:382:44: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_routine2003);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_routineEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_routine2006);
            if  (HASEXCEPTION())
            {
                goto rulecreate_routineEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_routineEx; /* Prevent compiler warnings */
    rulecreate_routineEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_routine */

/**
 * $ANTLR start create_procedure
 * MySQLSimpleParser.g:385:1: create_procedure : create_with_definer procedure_body ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_procedure(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:385:17: ( create_with_definer procedure_body ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:386:2: create_with_definer procedure_body ( SEMICOLON_SYMBOL )? EOF
        {
            FOLLOWPUSH(FOLLOW_create_with_definer_in_create_procedure2016);
            create_with_definer(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_procedureEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_procedure_body_in_create_procedure2018);
            procedure_body(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_procedureEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:386:37: ( SEMICOLON_SYMBOL )?
            {
                int alt128=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt128=1;
                    	}
                        break;
                }

                switch (alt128)
                {
            	case 1:
            	    // MySQLSimpleParser.g:386:37: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_procedure2020);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_procedureEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_procedure2023);
            if  (HASEXCEPTION())
            {
                goto rulecreate_procedureEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_procedureEx; /* Prevent compiler warnings */
    rulecreate_procedureEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_procedure */

/**
 * $ANTLR start create_function
 * MySQLSimpleParser.g:389:1: create_function : create_with_definer function_body ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:389:16: ( create_with_definer function_body ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:390:2: create_with_definer function_body ( SEMICOLON_SYMBOL )? EOF
        {
            FOLLOWPUSH(FOLLOW_create_with_definer_in_create_function2033);
            create_with_definer(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_function_body_in_create_function2035);
            function_body(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:390:36: ( SEMICOLON_SYMBOL )?
            {
                int alt129=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt129=1;
                    	}
                        break;
                }

                switch (alt129)
                {
            	case 1:
            	    // MySQLSimpleParser.g:390:36: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_function2037);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_function2040);
            if  (HASEXCEPTION())
            {
                goto rulecreate_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_functionEx; /* Prevent compiler warnings */
    rulecreate_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_function */

/**
 * $ANTLR start create_routine_or_udf
 * MySQLSimpleParser.g:393:1: create_routine_or_udf : ( procedure_body | function_body );
 */
static void
create_routine_or_udf(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:393:22: ( procedure_body | function_body )

            ANTLR3_UINT32 alt130;

            alt130=2;

            switch ( LA(1) )
            {
            case PROCEDURE_SYMBOL:
            	{
            		alt130=1;
            	}
                break;
            case AGGREGATE_SYMBOL:
            case FUNCTION_SYMBOL:
            	{
            		alt130=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 130;
                EXCEPTION->state        = 0;


                goto rulecreate_routine_or_udfEx;

            }

            switch (alt130)
            {
        	case 1:
        	    // MySQLSimpleParser.g:394:2: procedure_body
        	    {
        	        FOLLOWPUSH(FOLLOW_procedure_body_in_create_routine_or_udf2049);
        	        procedure_body(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_routine_or_udfEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:395:4: function_body
        	    {
        	        FOLLOWPUSH(FOLLOW_function_body_in_create_routine_or_udf2054);
        	        function_body(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_routine_or_udfEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecreate_routine_or_udfEx; /* Prevent compiler warnings */
    rulecreate_routine_or_udfEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_routine_or_udf */

/**
 * $ANTLR start procedure_body
 * MySQLSimpleParser.g:398:1: procedure_body : PROCEDURE_SYMBOL procedure_name OPEN_PAR_SYMBOL ( procedure_parameter ( COMMA_SYMBOL procedure_parameter )* )? CLOSE_PAR_SYMBOL ( routine_create_options )? compound_statement ;
 */
static void
procedure_body(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:398:15: ( PROCEDURE_SYMBOL procedure_name OPEN_PAR_SYMBOL ( procedure_parameter ( COMMA_SYMBOL procedure_parameter )* )? CLOSE_PAR_SYMBOL ( routine_create_options )? compound_statement )
        // MySQLSimpleParser.g:399:2: PROCEDURE_SYMBOL procedure_name OPEN_PAR_SYMBOL ( procedure_parameter ( COMMA_SYMBOL procedure_parameter )* )? CLOSE_PAR_SYMBOL ( routine_create_options )? compound_statement
        {
             MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_procedure_body2063);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_procedure_name_in_procedure_body2065);
            procedure_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_procedure_body2067);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:399:50: ( procedure_parameter ( COMMA_SYMBOL procedure_parameter )* )?
            {
                int alt132=2;
                {
                    int LA132_0 = LA(1);
                    if ( (((LA132_0 >= ACCOUNT_SYMBOL) && (LA132_0 <= YEAR_SYMBOL)) || LA132_0 == INOUT_SYMBOL || LA132_0 == IN_SYMBOL || LA132_0 == OUT_SYMBOL || LA132_0 == BACK_TICK_QUOTED_ID || LA132_0 == DOUBLE_QUOTED_TEXT || LA132_0 == IDENTIFIER))
                    {
                        alt132=1;
                    }
                }
                switch (alt132)
                {
            	case 1:
            	    // MySQLSimpleParser.g:399:51: procedure_parameter ( COMMA_SYMBOL procedure_parameter )*
            	    {
            	        FOLLOWPUSH(FOLLOW_procedure_parameter_in_procedure_body2070);
            	        procedure_parameter(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprocedure_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:399:71: ( COMMA_SYMBOL procedure_parameter )*

            	        for (;;)
            	        {
            	            int alt131=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt131=1;
            	            	}
            	                break;

            	            }

            	            switch (alt131)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:399:72: COMMA_SYMBOL procedure_parameter
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_procedure_body2073);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleprocedure_bodyEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_procedure_parameter_in_procedure_body2075);
            	        	        procedure_parameter(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleprocedure_bodyEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop131;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop131: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_body2081);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:400:3: ( routine_create_options )?
            {
                int alt133=2;
                switch ( LA(1) )
                {
                    case COMMENT_SYMBOL:
                    case CONTAINS_SYMBOL:
                    case LANGUAGE_SYMBOL:
                    case NO_SYMBOL:
                    case DETERMINISTIC_SYMBOL:
                    case MODIFIES_SYMBOL:
                    case NOT_SYMBOL:
                    case READS_SYMBOL:
                    case SQL_SYMBOL:
                    	{
                    		alt133=1;
                    	}
                        break;
                }

                switch (alt133)
                {
            	case 1:
            	    // MySQLSimpleParser.g:400:3: routine_create_options
            	    {
            	        FOLLOWPUSH(FOLLOW_routine_create_options_in_procedure_body2085);
            	        routine_create_options(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprocedure_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_compound_statement_in_procedure_body2088);
            compound_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprocedure_bodyEx; /* Prevent compiler warnings */
    ruleprocedure_bodyEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end procedure_body */

/**
 * $ANTLR start function_body
 * MySQLSimpleParser.g:403:1: function_body : ( FUNCTION_SYMBOL ( function_name OPEN_PAR_SYMBOL ( function_parameter ( COMMA_SYMBOL function_parameter )* )? CLOSE_PAR_SYMBOL RETURNS_SYMBOL data_type ( routine_create_options )? compound_statement | udf_tail ) | AGGREGATE_SYMBOL FUNCTION_SYMBOL udf_tail );
 */
static void
function_body(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:403:14: ( FUNCTION_SYMBOL ( function_name OPEN_PAR_SYMBOL ( function_parameter ( COMMA_SYMBOL function_parameter )* )? CLOSE_PAR_SYMBOL RETURNS_SYMBOL data_type ( routine_create_options )? compound_statement | udf_tail ) | AGGREGATE_SYMBOL FUNCTION_SYMBOL udf_tail )

            ANTLR3_UINT32 alt138;

            alt138=2;

            switch ( LA(1) )
            {
            case FUNCTION_SYMBOL:
            	{
            		alt138=1;
            	}
                break;
            case AGGREGATE_SYMBOL:
            	{
            		alt138=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 138;
                EXCEPTION->state        = 0;


                goto rulefunction_bodyEx;

            }

            switch (alt138)
            {
        	case 1:
        	    // MySQLSimpleParser.g:404:2: FUNCTION_SYMBOL ( function_name OPEN_PAR_SYMBOL ( function_parameter ( COMMA_SYMBOL function_parameter )* )? CLOSE_PAR_SYMBOL RETURNS_SYMBOL data_type ( routine_create_options )? compound_statement | udf_tail )
        	    {
        	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_function_body2098);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefunction_bodyEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:405:2: ( function_name OPEN_PAR_SYMBOL ( function_parameter ( COMMA_SYMBOL function_parameter )* )? CLOSE_PAR_SYMBOL RETURNS_SYMBOL data_type ( routine_create_options )? compound_statement | udf_tail )
        	        {
        	            int alt137=2;
        	            {
        	                int LA137_0 = LA(1);
        	                if ( (LA137_0 == IDENTIFIER))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                    case OPEN_PAR_SYMBOL:
        	                    case DOT_SYMBOL:
        	                    	{
        	                    		alt137=1;
        	                    	}
        	                        break;
        	                    case RETURNS_SYMBOL:
        	                    	{
        	                    		alt137=2;
        	                    	}
        	                        break;

        	                    default:
        	                        if (BACKTRACKING>0)
        	                        {
        	                            FAILEDFLAG = ANTLR3_TRUE;
        	                            return ;
        	                        }

        	                        CONSTRUCTEX();
        	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                        EXCEPTION->message      = (void *)"";
        	                        EXCEPTION->decisionNum  = 137;
        	                        EXCEPTION->state        = 1;


        	                        goto rulefunction_bodyEx;

        	                    }

        	                }
        	                else if ( (LA137_0 == BACK_TICK_QUOTED_ID))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                    case OPEN_PAR_SYMBOL:
        	                    case DOT_SYMBOL:
        	                    	{
        	                    		alt137=1;
        	                    	}
        	                        break;
        	                    case RETURNS_SYMBOL:
        	                    	{
        	                    		alt137=2;
        	                    	}
        	                        break;

        	                    default:
        	                        if (BACKTRACKING>0)
        	                        {
        	                            FAILEDFLAG = ANTLR3_TRUE;
        	                            return ;
        	                        }

        	                        CONSTRUCTEX();
        	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                        EXCEPTION->message      = (void *)"";
        	                        EXCEPTION->decisionNum  = 137;
        	                        EXCEPTION->state        = 2;


        	                        goto rulefunction_bodyEx;

        	                    }

        	                }
        	                else if ( (LA137_0 == DOUBLE_QUOTED_TEXT))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                    case OPEN_PAR_SYMBOL:
        	                    case DOT_SYMBOL:
        	                    	{
        	                    		alt137=1;
        	                    	}
        	                        break;
        	                    case RETURNS_SYMBOL:
        	                    	{
        	                    		alt137=2;
        	                    	}
        	                        break;

        	                    default:
        	                        if (BACKTRACKING>0)
        	                        {
        	                            FAILEDFLAG = ANTLR3_TRUE;
        	                            return ;
        	                        }

        	                        CONSTRUCTEX();
        	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                        EXCEPTION->message      = (void *)"";
        	                        EXCEPTION->decisionNum  = 137;
        	                        EXCEPTION->state        = 3;


        	                        goto rulefunction_bodyEx;

        	                    }

        	                }
        	                else if ( (((LA137_0 >= ACTION_SYMBOL) && (LA137_0 <= SHARE_SYMBOL)) || ((LA137_0 >= SLOW_SYMBOL) && (LA137_0 <= YEAR_SYMBOL))))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                    case OPEN_PAR_SYMBOL:
        	                    case DOT_SYMBOL:
        	                    	{
        	                    		alt137=1;
        	                    	}
        	                        break;
        	                    case RETURNS_SYMBOL:
        	                    	{
        	                    		alt137=2;
        	                    	}
        	                        break;

        	                    default:
        	                        if (BACKTRACKING>0)
        	                        {
        	                            FAILEDFLAG = ANTLR3_TRUE;
        	                            return ;
        	                        }

        	                        CONSTRUCTEX();
        	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                        EXCEPTION->message      = (void *)"";
        	                        EXCEPTION->decisionNum  = 137;
        	                        EXCEPTION->state        = 4;


        	                        goto rulefunction_bodyEx;

        	                    }

        	                }
        	                else if ( (((LA137_0 >= ACCOUNT_SYMBOL) && (LA137_0 <= XA_SYMBOL)) || LA137_0 == SHUTDOWN_SYMBOL))
        	                {
        	                    switch ( LA(2) )
        	                    {
        	                    case OPEN_PAR_SYMBOL:
        	                    case DOT_SYMBOL:
        	                    	{
        	                    		alt137=1;
        	                    	}
        	                        break;
        	                    case RETURNS_SYMBOL:
        	                    	{
        	                    		alt137=2;
        	                    	}
        	                        break;

        	                    default:
        	                        if (BACKTRACKING>0)
        	                        {
        	                            FAILEDFLAG = ANTLR3_TRUE;
        	                            return ;
        	                        }

        	                        CONSTRUCTEX();
        	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                        EXCEPTION->message      = (void *)"";
        	                        EXCEPTION->decisionNum  = 137;
        	                        EXCEPTION->state        = 5;


        	                        goto rulefunction_bodyEx;

        	                    }

        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 137;
        	                    EXCEPTION->state        = 0;


        	                    goto rulefunction_bodyEx;

        	                }
        	            }
        	            switch (alt137)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:406:3: function_name OPEN_PAR_SYMBOL ( function_parameter ( COMMA_SYMBOL function_parameter )* )? CLOSE_PAR_SYMBOL RETURNS_SYMBOL data_type ( routine_create_options )? compound_statement
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_function_name_in_function_body2105);
        	        	        function_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_function_body2107);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:406:33: ( function_parameter ( COMMA_SYMBOL function_parameter )* )?
        	        	        {
        	        	            int alt135=2;
        	        	            {
        	        	                int LA135_0 = LA(1);
        	        	                if ( (((LA135_0 >= ACCOUNT_SYMBOL) && (LA135_0 <= YEAR_SYMBOL)) || LA135_0 == BACK_TICK_QUOTED_ID || LA135_0 == DOUBLE_QUOTED_TEXT || LA135_0 == IDENTIFIER))
        	        	                {
        	        	                    alt135=1;
        	        	                }
        	        	            }
        	        	            switch (alt135)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:406:34: function_parameter ( COMMA_SYMBOL function_parameter )*
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_function_parameter_in_function_body2110);
        	        	        	        function_parameter(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulefunction_bodyEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        // MySQLSimpleParser.g:406:53: ( COMMA_SYMBOL function_parameter )*

        	        	        	        for (;;)
        	        	        	        {
        	        	        	            int alt134=2;
        	        	        	            switch ( LA(1) )
        	        	        	            {
        	        	        	            case COMMA_SYMBOL:
        	        	        	            	{
        	        	        	            		alt134=1;
        	        	        	            	}
        	        	        	                break;

        	        	        	            }

        	        	        	            switch (alt134)
        	        	        	            {
        	        	        	        	case 1:
        	        	        	        	    // MySQLSimpleParser.g:406:54: COMMA_SYMBOL function_parameter
        	        	        	        	    {
        	        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_function_body2113);
        	        	        	        	        if  (HASEXCEPTION())
        	        	        	        	        {
        	        	        	        	            goto rulefunction_bodyEx;
        	        	        	        	        }
        	        	        	        	        if (HASFAILED())
        	        	        	        	        {
        	        	        	        	            return ;
        	        	        	        	        }


        	        	        	        	        FOLLOWPUSH(FOLLOW_function_parameter_in_function_body2115);
        	        	        	        	        function_parameter(ctx);

        	        	        	        	        FOLLOWPOP();
        	        	        	        	        if  (HASEXCEPTION())
        	        	        	        	        {
        	        	        	        	            goto rulefunction_bodyEx;
        	        	        	        	        }
        	        	        	        	        if (HASFAILED())
        	        	        	        	        {
        	        	        	        	            return ;
        	        	        	        	        }


        	        	        	        	    }
        	        	        	        	    break;

        	        	        	        	default:
        	        	        	        	    goto loop134;	/* break out of the loop */
        	        	        	        	    break;
        	        	        	            }
        	        	        	        }
        	        	        	        loop134: ; /* Jump out to here if this rule does not match */


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_function_body2121);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(RETURNS_SYMBOL, &FOLLOW_RETURNS_SYMBOL_in_function_body2123);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_data_type_in_function_body2128);
        	        	        data_type(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:407:14: ( routine_create_options )?
        	        	        {
        	        	            int alt136=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case COMMENT_SYMBOL:
        	        	                case CONTAINS_SYMBOL:
        	        	                case LANGUAGE_SYMBOL:
        	        	                case NO_SYMBOL:
        	        	                case DETERMINISTIC_SYMBOL:
        	        	                case MODIFIES_SYMBOL:
        	        	                case NOT_SYMBOL:
        	        	                case READS_SYMBOL:
        	        	                case SQL_SYMBOL:
        	        	                	{
        	        	                		alt136=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt136)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:407:14: routine_create_options
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_routine_create_options_in_function_body2130);
        	        	        	        routine_create_options(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulefunction_bodyEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_compound_statement_in_function_body2133);
        	        	        compound_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:408:5: udf_tail
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_udf_tail_in_function_body2139);
        	        	        udf_tail(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefunction_bodyEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:410:4: AGGREGATE_SYMBOL FUNCTION_SYMBOL udf_tail
        	    {
        	         MATCHT(AGGREGATE_SYMBOL, &FOLLOW_AGGREGATE_SYMBOL_in_function_body2147);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefunction_bodyEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_function_body2149);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefunction_bodyEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_udf_tail_in_function_body2151);
        	        udf_tail(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefunction_bodyEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulefunction_bodyEx; /* Prevent compiler warnings */
    rulefunction_bodyEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_body */

/**
 * $ANTLR start udf_tail
 * MySQLSimpleParser.g:413:1: udf_tail : udf_name RETURNS_SYMBOL ( STRING_SYMBOL | INT_SYMBOL | REAL_SYMBOL | DECIMAL_SYMBOL ) SONAME_SYMBOL string_literal ;
 */
static void
udf_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:413:9: ( udf_name RETURNS_SYMBOL ( STRING_SYMBOL | INT_SYMBOL | REAL_SYMBOL | DECIMAL_SYMBOL ) SONAME_SYMBOL string_literal )
        // MySQLSimpleParser.g:414:2: udf_name RETURNS_SYMBOL ( STRING_SYMBOL | INT_SYMBOL | REAL_SYMBOL | DECIMAL_SYMBOL ) SONAME_SYMBOL string_literal
        {
            FOLLOWPUSH(FOLLOW_udf_name_in_udf_tail2161);
            udf_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleudf_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(RETURNS_SYMBOL, &FOLLOW_RETURNS_SYMBOL_in_udf_tail2163);
            if  (HASEXCEPTION())
            {
                goto ruleudf_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == STRING_SYMBOL || LA(1) == DECIMAL_SYMBOL || LA(1) == INT_SYMBOL || LA(1) == REAL_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleudf_tailEx;
            }


             MATCHT(SONAME_SYMBOL, &FOLLOW_SONAME_SYMBOL_in_udf_tail2181);
            if  (HASEXCEPTION())
            {
                goto ruleudf_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_string_literal_in_udf_tail2183);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleudf_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleudf_tailEx; /* Prevent compiler warnings */
    ruleudf_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end udf_tail */

/**
 * $ANTLR start routine_create_options
 * MySQLSimpleParser.g:417:1: routine_create_options : ( routine_create_option )+ ;
 */
static void
routine_create_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:417:23: ( ( routine_create_option )+ )
        // MySQLSimpleParser.g:418:2: ( routine_create_option )+
        {
            // MySQLSimpleParser.g:418:2: ( routine_create_option )+
            {
                int cnt139=0;

                for (;;)
                {
                    int alt139=2;
            	switch ( LA(1) )
            	{
            	case COMMENT_SYMBOL:
            	case CONTAINS_SYMBOL:
            	case LANGUAGE_SYMBOL:
            	case NO_SYMBOL:
            	case DETERMINISTIC_SYMBOL:
            	case MODIFIES_SYMBOL:
            	case NOT_SYMBOL:
            	case READS_SYMBOL:
            	case SQL_SYMBOL:
            		{
            			alt139=1;
            		}
            	    break;

            	}

            	switch (alt139)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:418:2: routine_create_option
            	        {
            	            FOLLOWPUSH(FOLLOW_routine_create_option_in_routine_create_options2192);
            	            routine_create_option(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleroutine_create_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt139 >= 1 )
            		{
            		    goto loop139;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto ruleroutine_create_optionsEx;
            	}
            	cnt139++;
                }
                loop139: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleroutine_create_optionsEx; /* Prevent compiler warnings */
    ruleroutine_create_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end routine_create_options */

/**
 * $ANTLR start routine_create_option
 * MySQLSimpleParser.g:421:1: routine_create_option : ( routine_option | ( NOT_SYMBOL )? DETERMINISTIC_SYMBOL );
 */
static void
routine_create_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:421:22: ( routine_option | ( NOT_SYMBOL )? DETERMINISTIC_SYMBOL )

            ANTLR3_UINT32 alt141;

            alt141=2;

            switch ( LA(1) )
            {
            case COMMENT_SYMBOL:
            case CONTAINS_SYMBOL:
            case LANGUAGE_SYMBOL:
            case NO_SYMBOL:
            case MODIFIES_SYMBOL:
            case READS_SYMBOL:
            case SQL_SYMBOL:
            	{
            		alt141=1;
            	}
                break;
            case DETERMINISTIC_SYMBOL:
            case NOT_SYMBOL:
            	{
            		alt141=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 141;
                EXCEPTION->state        = 0;


                goto ruleroutine_create_optionEx;

            }

            switch (alt141)
            {
        	case 1:
        	    // MySQLSimpleParser.g:422:2: routine_option
        	    {
        	        FOLLOWPUSH(FOLLOW_routine_option_in_routine_create_option2203);
        	        routine_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_create_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:423:4: ( NOT_SYMBOL )? DETERMINISTIC_SYMBOL
        	    {
        	        // MySQLSimpleParser.g:423:4: ( NOT_SYMBOL )?
        	        {
        	            int alt140=2;
        	            switch ( LA(1) )
        	            {
        	                case NOT_SYMBOL:
        	                	{
        	                		alt140=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt140)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:423:4: NOT_SYMBOL
        	        	    {
        	        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_routine_create_option2208);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleroutine_create_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(DETERMINISTIC_SYMBOL, &FOLLOW_DETERMINISTIC_SYMBOL_in_routine_create_option2211);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_create_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleroutine_create_optionEx; /* Prevent compiler warnings */
    ruleroutine_create_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end routine_create_option */

/**
 * $ANTLR start routine_alter_options
 * MySQLSimpleParser.g:426:1: routine_alter_options : ( routine_create_option )+ ;
 */
static void
routine_alter_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:426:22: ( ( routine_create_option )+ )
        // MySQLSimpleParser.g:427:2: ( routine_create_option )+
        {
            // MySQLSimpleParser.g:427:2: ( routine_create_option )+
            {
                int cnt142=0;

                for (;;)
                {
                    int alt142=2;
            	switch ( LA(1) )
            	{
            	case COMMENT_SYMBOL:
            	case CONTAINS_SYMBOL:
            	case LANGUAGE_SYMBOL:
            	case NO_SYMBOL:
            	case DETERMINISTIC_SYMBOL:
            	case MODIFIES_SYMBOL:
            	case NOT_SYMBOL:
            	case READS_SYMBOL:
            	case SQL_SYMBOL:
            		{
            			alt142=1;
            		}
            	    break;

            	}

            	switch (alt142)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:427:2: routine_create_option
            	        {
            	            FOLLOWPUSH(FOLLOW_routine_create_option_in_routine_alter_options2220);
            	            routine_create_option(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleroutine_alter_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt142 >= 1 )
            		{
            		    goto loop142;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto ruleroutine_alter_optionsEx;
            	}
            	cnt142++;
                }
                loop142: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleroutine_alter_optionsEx; /* Prevent compiler warnings */
    ruleroutine_alter_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end routine_alter_options */

/**
 * $ANTLR start routine_option
 * MySQLSimpleParser.g:430:1: routine_option : ( COMMENT_SYMBOL string_literal | LANGUAGE_SYMBOL SQL_SYMBOL | NO_SYMBOL SQL_SYMBOL | CONTAINS_SYMBOL SQL_SYMBOL | READS_SYMBOL SQL_SYMBOL DATA_SYMBOL | MODIFIES_SYMBOL SQL_SYMBOL DATA_SYMBOL | SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) );
 */
static void
routine_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:430:15: ( COMMENT_SYMBOL string_literal | LANGUAGE_SYMBOL SQL_SYMBOL | NO_SYMBOL SQL_SYMBOL | CONTAINS_SYMBOL SQL_SYMBOL | READS_SYMBOL SQL_SYMBOL DATA_SYMBOL | MODIFIES_SYMBOL SQL_SYMBOL DATA_SYMBOL | SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )

            ANTLR3_UINT32 alt143;

            alt143=7;

            switch ( LA(1) )
            {
            case COMMENT_SYMBOL:
            	{
            		alt143=1;
            	}
                break;
            case LANGUAGE_SYMBOL:
            	{
            		alt143=2;
            	}
                break;
            case NO_SYMBOL:
            	{
            		alt143=3;
            	}
                break;
            case CONTAINS_SYMBOL:
            	{
            		alt143=4;
            	}
                break;
            case READS_SYMBOL:
            	{
            		alt143=5;
            	}
                break;
            case MODIFIES_SYMBOL:
            	{
            		alt143=6;
            	}
                break;
            case SQL_SYMBOL:
            	{
            		alt143=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 143;
                EXCEPTION->state        = 0;


                goto ruleroutine_optionEx;

            }

            switch (alt143)
            {
        	case 1:
        	    // MySQLSimpleParser.g:431:2: COMMENT_SYMBOL string_literal
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_routine_option2231);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_routine_option2233);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:432:4: LANGUAGE_SYMBOL SQL_SYMBOL
        	    {
        	         MATCHT(LANGUAGE_SYMBOL, &FOLLOW_LANGUAGE_SYMBOL_in_routine_option2238);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2240);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:433:4: NO_SYMBOL SQL_SYMBOL
        	    {
        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_routine_option2245);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2247);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:434:4: CONTAINS_SYMBOL SQL_SYMBOL
        	    {
        	         MATCHT(CONTAINS_SYMBOL, &FOLLOW_CONTAINS_SYMBOL_in_routine_option2252);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2254);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:435:4: READS_SYMBOL SQL_SYMBOL DATA_SYMBOL
        	    {
        	         MATCHT(READS_SYMBOL, &FOLLOW_READS_SYMBOL_in_routine_option2259);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2261);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_routine_option2263);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:436:4: MODIFIES_SYMBOL SQL_SYMBOL DATA_SYMBOL
        	    {
        	         MATCHT(MODIFIES_SYMBOL, &FOLLOW_MODIFIES_SYMBOL_in_routine_option2268);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2270);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_routine_option2272);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:437:4: SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL )
        	    {
        	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_routine_option2277);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SECURITY_SYMBOL, &FOLLOW_SECURITY_SYMBOL_in_routine_option2279);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleroutine_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == DEFINER_SYMBOL || LA(1) == INVOKER_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleroutine_optionEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleroutine_optionEx; /* Prevent compiler warnings */
    ruleroutine_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end routine_option */

/**
 * $ANTLR start create_index
 * MySQLSimpleParser.g:440:1: create_index : CREATE_SYMBOL create_index_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_index(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:440:13: ( CREATE_SYMBOL create_index_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:441:2: CREATE_SYMBOL create_index_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_index2297);
            if  (HASEXCEPTION())
            {
                goto rulecreate_indexEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_index_tail_in_create_index2299);
            create_index_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_indexEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:441:34: ( SEMICOLON_SYMBOL )?
            {
                int alt144=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt144=1;
                    	}
                        break;
                }

                switch (alt144)
                {
            	case 1:
            	    // MySQLSimpleParser.g:441:34: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_index2301);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_indexEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_index2304);
            if  (HASEXCEPTION())
            {
                goto rulecreate_indexEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_indexEx; /* Prevent compiler warnings */
    rulecreate_indexEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_index */

/**
 * $ANTLR start create_index_tail
 * MySQLSimpleParser.g:444:1: create_index_tail : ( online_option )? ( ( UNIQUE_SYMBOL )? INDEX_SYMBOL index_name ( index_type )? create_index_target ( normal_index_option )* ( index_lock_algorithm )? | FULLTEXT_SYMBOL INDEX_SYMBOL index_name create_index_target ( fulltext_index_option )* ( index_lock_algorithm )? | SPATIAL_SYMBOL INDEX_SYMBOL index_name create_index_target ( spatial_index_option )* ( index_lock_algorithm )? ) ;
 */
static void
create_index_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:444:18: ( ( online_option )? ( ( UNIQUE_SYMBOL )? INDEX_SYMBOL index_name ( index_type )? create_index_target ( normal_index_option )* ( index_lock_algorithm )? | FULLTEXT_SYMBOL INDEX_SYMBOL index_name create_index_target ( fulltext_index_option )* ( index_lock_algorithm )? | SPATIAL_SYMBOL INDEX_SYMBOL index_name create_index_target ( spatial_index_option )* ( index_lock_algorithm )? ) )
        // MySQLSimpleParser.g:445:2: ( online_option )? ( ( UNIQUE_SYMBOL )? INDEX_SYMBOL index_name ( index_type )? create_index_target ( normal_index_option )* ( index_lock_algorithm )? | FULLTEXT_SYMBOL INDEX_SYMBOL index_name create_index_target ( fulltext_index_option )* ( index_lock_algorithm )? | SPATIAL_SYMBOL INDEX_SYMBOL index_name create_index_target ( spatial_index_option )* ( index_lock_algorithm )? )
        {
            // MySQLSimpleParser.g:445:2: ( online_option )?
            {
                int alt145=2;
                switch ( LA(1) )
                {
                    case OFFLINE_SYMBOL:
                    case ONLINE_SYMBOL:
                    	{
                    		alt145=1;
                    	}
                        break;
                }

                switch (alt145)
                {
            	case 1:
            	    // MySQLSimpleParser.g:445:2: online_option
            	    {
            	        FOLLOWPUSH(FOLLOW_online_option_in_create_index_tail2313);
            	        online_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:446:2: ( ( UNIQUE_SYMBOL )? INDEX_SYMBOL index_name ( index_type )? create_index_target ( normal_index_option )* ( index_lock_algorithm )? | FULLTEXT_SYMBOL INDEX_SYMBOL index_name create_index_target ( fulltext_index_option )* ( index_lock_algorithm )? | SPATIAL_SYMBOL INDEX_SYMBOL index_name create_index_target ( spatial_index_option )* ( index_lock_algorithm )? )
            {
                int alt154=3;
                switch ( LA(1) )
                {
                case INDEX_SYMBOL:
                case UNIQUE_SYMBOL:
                	{
                		alt154=1;
                	}
                    break;
                case FULLTEXT_SYMBOL:
                	{
                		alt154=2;
                	}
                    break;
                case SPATIAL_SYMBOL:
                	{
                		alt154=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 154;
                    EXCEPTION->state        = 0;


                    goto rulecreate_index_tailEx;

                }

                switch (alt154)
                {
            	case 1:
            	    // MySQLSimpleParser.g:447:3: ( UNIQUE_SYMBOL )? INDEX_SYMBOL index_name ( index_type )? create_index_target ( normal_index_option )* ( index_lock_algorithm )?
            	    {
            	        // MySQLSimpleParser.g:447:3: ( UNIQUE_SYMBOL )?
            	        {
            	            int alt146=2;
            	            switch ( LA(1) )
            	            {
            	                case UNIQUE_SYMBOL:
            	                	{
            	                		alt146=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt146)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:447:3: UNIQUE_SYMBOL
            	        	    {
            	        	         MATCHT(UNIQUE_SYMBOL, &FOLLOW_UNIQUE_SYMBOL_in_create_index_tail2321);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_create_index_tail2324);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_name_in_create_index_tail2326);
            	        index_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:447:42: ( index_type )?
            	        {
            	            int alt147=2;
            	            switch ( LA(1) )
            	            {
            	                case TYPE_SYMBOL:
            	                case USING_SYMBOL:
            	                	{
            	                		alt147=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt147)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:447:42: index_type
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_index_type_in_create_index_tail2328);
            	        	        index_type(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_create_index_target_in_create_index_tail2331);
            	        create_index_target(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:447:74: ( normal_index_option )*

            	        for (;;)
            	        {
            	            int alt148=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMENT_SYMBOL:
            	            case KEY_BLOCK_SIZE_SYMBOL:
            	            case TYPE_SYMBOL:
            	            case USING_SYMBOL:
            	            	{
            	            		alt148=1;
            	            	}
            	                break;

            	            }

            	            switch (alt148)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:447:74: normal_index_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_normal_index_option_in_create_index_tail2333);
            	        	        normal_index_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop148;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop148: ; /* Jump out to here if this rule does not match */


            	        // MySQLSimpleParser.g:447:95: ( index_lock_algorithm )?
            	        {
            	            int alt149=2;
            	            {
            	                int LA149_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50600)) && (LA149_0 == ALGORITHM_SYMBOL || LA149_0 == LOCK_SYMBOL))
            	                {
            	                    alt149=1;
            	                }
            	            }
            	            switch (alt149)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:447:95: index_lock_algorithm
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_index_lock_algorithm_in_create_index_tail2336);
            	        	        index_lock_algorithm(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:448:5: FULLTEXT_SYMBOL INDEX_SYMBOL index_name create_index_target ( fulltext_index_option )* ( index_lock_algorithm )?
            	    {
            	         MATCHT(FULLTEXT_SYMBOL, &FOLLOW_FULLTEXT_SYMBOL_in_create_index_tail2343);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_create_index_tail2345);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_name_in_create_index_tail2347);
            	        index_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_create_index_target_in_create_index_tail2349);
            	        create_index_target(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:448:65: ( fulltext_index_option )*

            	        for (;;)
            	        {
            	            int alt150=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMENT_SYMBOL:
            	            case KEY_BLOCK_SIZE_SYMBOL:
            	            case WITH_SYMBOL:
            	            	{
            	            		alt150=1;
            	            	}
            	                break;

            	            }

            	            switch (alt150)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:448:65: fulltext_index_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_fulltext_index_option_in_create_index_tail2351);
            	        	        fulltext_index_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop150;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop150: ; /* Jump out to here if this rule does not match */


            	        // MySQLSimpleParser.g:448:88: ( index_lock_algorithm )?
            	        {
            	            int alt151=2;
            	            {
            	                int LA151_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50600)) && (LA151_0 == ALGORITHM_SYMBOL || LA151_0 == LOCK_SYMBOL))
            	                {
            	                    alt151=1;
            	                }
            	            }
            	            switch (alt151)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:448:88: index_lock_algorithm
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_index_lock_algorithm_in_create_index_tail2354);
            	        	        index_lock_algorithm(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:449:5: SPATIAL_SYMBOL INDEX_SYMBOL index_name create_index_target ( spatial_index_option )* ( index_lock_algorithm )?
            	    {
            	         MATCHT(SPATIAL_SYMBOL, &FOLLOW_SPATIAL_SYMBOL_in_create_index_tail2361);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_create_index_tail2363);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_name_in_create_index_tail2365);
            	        index_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_create_index_target_in_create_index_tail2367);
            	        create_index_target(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_index_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:449:64: ( spatial_index_option )*

            	        for (;;)
            	        {
            	            int alt152=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMENT_SYMBOL:
            	            case KEY_BLOCK_SIZE_SYMBOL:
            	            	{
            	            		alt152=1;
            	            	}
            	                break;

            	            }

            	            switch (alt152)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:449:64: spatial_index_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_spatial_index_option_in_create_index_tail2369);
            	        	        spatial_index_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop152;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop152: ; /* Jump out to here if this rule does not match */


            	        // MySQLSimpleParser.g:449:86: ( index_lock_algorithm )?
            	        {
            	            int alt153=2;
            	            {
            	                int LA153_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50600)) && (LA153_0 == ALGORITHM_SYMBOL || LA153_0 == LOCK_SYMBOL))
            	                {
            	                    alt153=1;
            	                }
            	            }
            	            switch (alt153)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:449:86: index_lock_algorithm
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_index_lock_algorithm_in_create_index_tail2372);
            	        	        index_lock_algorithm(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_index_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_index_tailEx; /* Prevent compiler warnings */
    rulecreate_index_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_index_tail */

/**
 * $ANTLR start create_index_target
 * MySQLSimpleParser.g:453:1: create_index_target : ON_SYMBOL table_ref index_columns ;
 */
static void
create_index_target(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:453:20: ( ON_SYMBOL table_ref index_columns )
        // MySQLSimpleParser.g:454:2: ON_SYMBOL table_ref index_columns
        {
             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_create_index_target2385);
            if  (HASEXCEPTION())
            {
                goto rulecreate_index_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_create_index_target2387);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_index_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_index_columns_in_create_index_target2389);
            index_columns(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_index_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_index_targetEx; /* Prevent compiler warnings */
    rulecreate_index_targetEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_index_target */

/**
 * $ANTLR start create_logfile_group
 * MySQLSimpleParser.g:457:1: create_logfile_group : CREATE_SYMBOL create_logfile_group_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_logfile_group(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:457:21: ( CREATE_SYMBOL create_logfile_group_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:458:2: CREATE_SYMBOL create_logfile_group_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_logfile_group2399);
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_logfile_group_tail_in_create_logfile_group2401);
            create_logfile_group_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:458:42: ( SEMICOLON_SYMBOL )?
            {
                int alt155=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt155=1;
                    	}
                        break;
                }

                switch (alt155)
                {
            	case 1:
            	    // MySQLSimpleParser.g:458:42: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_logfile_group2403);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_logfile_groupEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_logfile_group2406);
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_groupEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_logfile_groupEx; /* Prevent compiler warnings */
    rulecreate_logfile_groupEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_logfile_group */

/**
 * $ANTLR start create_logfile_group_tail
 * MySQLSimpleParser.g:461:1: create_logfile_group_tail : LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_name ADD_SYMBOL ( UNDOFILE_SYMBOL | REDOFILE_SYMBOL ) string_literal ( logfile_group_options )? ;
 */
static void
create_logfile_group_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:461:26: ( LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_name ADD_SYMBOL ( UNDOFILE_SYMBOL | REDOFILE_SYMBOL ) string_literal ( logfile_group_options )? )
        // MySQLSimpleParser.g:462:2: LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_name ADD_SYMBOL ( UNDOFILE_SYMBOL | REDOFILE_SYMBOL ) string_literal ( logfile_group_options )?
        {
             MATCHT(LOGFILE_SYMBOL, &FOLLOW_LOGFILE_SYMBOL_in_create_logfile_group_tail2415);
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_group_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_create_logfile_group_tail2417);
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_group_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_logfile_group_name_in_create_logfile_group_tail2419);
            logfile_group_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_group_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_create_logfile_group_tail2421);
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_group_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == REDOFILE_SYMBOL || LA(1) == UNDOFILE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulecreate_logfile_group_tailEx;
            }


            FOLLOWPUSH(FOLLOW_string_literal_in_create_logfile_group_tail2431);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_logfile_group_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:463:3: ( logfile_group_options )?
            {
                int alt156=2;
                switch ( LA(1) )
                {
                    case COMMENT_SYMBOL:
                    case ENGINE_SYMBOL:
                    case INITIAL_SIZE_SYMBOL:
                    case NO_WAIT_SYMBOL:
                    case NODEGROUP_SYMBOL:
                    case REDO_BUFFER_SIZE_SYMBOL:
                    case STORAGE_SYMBOL:
                    case UNDO_BUFFER_SIZE_SYMBOL:
                    case WAIT_SYMBOL:
                    	{
                    		alt156=1;
                    	}
                        break;
                }

                switch (alt156)
                {
            	case 1:
            	    // MySQLSimpleParser.g:463:3: logfile_group_options
            	    {
            	        FOLLOWPUSH(FOLLOW_logfile_group_options_in_create_logfile_group_tail2435);
            	        logfile_group_options(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_logfile_group_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_logfile_group_tailEx; /* Prevent compiler warnings */
    rulecreate_logfile_group_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_logfile_group_tail */

/**
 * $ANTLR start logfile_group_options
 * MySQLSimpleParser.g:466:1: logfile_group_options : ( logfile_group_option ( ( COMMA_SYMBOL )? logfile_group_option )* ) ;
 */
static void
logfile_group_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:466:22: ( ( logfile_group_option ( ( COMMA_SYMBOL )? logfile_group_option )* ) )
        // MySQLSimpleParser.g:467:2: ( logfile_group_option ( ( COMMA_SYMBOL )? logfile_group_option )* )
        {
            // MySQLSimpleParser.g:467:2: ( logfile_group_option ( ( COMMA_SYMBOL )? logfile_group_option )* )
            // MySQLSimpleParser.g:467:3: logfile_group_option ( ( COMMA_SYMBOL )? logfile_group_option )*
            {
                FOLLOWPUSH(FOLLOW_logfile_group_option_in_logfile_group_options2446);
                logfile_group_option(ctx);

                FOLLOWPOP();
                if  (HASEXCEPTION())
                {
                    goto rulelogfile_group_optionsEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


                // MySQLSimpleParser.g:467:24: ( ( COMMA_SYMBOL )? logfile_group_option )*

                for (;;)
                {
                    int alt158=2;
                    switch ( LA(1) )
                    {
                    case COMMENT_SYMBOL:
                    case ENGINE_SYMBOL:
                    case INITIAL_SIZE_SYMBOL:
                    case NO_WAIT_SYMBOL:
                    case NODEGROUP_SYMBOL:
                    case REDO_BUFFER_SIZE_SYMBOL:
                    case STORAGE_SYMBOL:
                    case UNDO_BUFFER_SIZE_SYMBOL:
                    case WAIT_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt158=1;
                    	}
                        break;

                    }

                    switch (alt158)
                    {
                	case 1:
                	    // MySQLSimpleParser.g:467:25: ( COMMA_SYMBOL )? logfile_group_option
                	    {
                	        // MySQLSimpleParser.g:467:25: ( COMMA_SYMBOL )?
                	        {
                	            int alt157=2;
                	            switch ( LA(1) )
                	            {
                	                case COMMA_SYMBOL:
                	                	{
                	                		alt157=1;
                	                	}
                	                    break;
                	            }

                	            switch (alt157)
                	            {
                	        	case 1:
                	        	    // MySQLSimpleParser.g:467:25: COMMA_SYMBOL
                	        	    {
                	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_logfile_group_options2449);
                	        	        if  (HASEXCEPTION())
                	        	        {
                	        	            goto rulelogfile_group_optionsEx;
                	        	        }
                	        	        if (HASFAILED())
                	        	        {
                	        	            return ;
                	        	        }


                	        	    }
                	        	    break;

                	            }
                	        }

                	        FOLLOWPUSH(FOLLOW_logfile_group_option_in_logfile_group_options2452);
                	        logfile_group_option(ctx);

                	        FOLLOWPOP();
                	        if  (HASEXCEPTION())
                	        {
                	            goto rulelogfile_group_optionsEx;
                	        }
                	        if (HASFAILED())
                	        {
                	            return ;
                	        }


                	    }
                	    break;

                	default:
                	    goto loop158;	/* break out of the loop */
                	    break;
                    }
                }
                loop158: ; /* Jump out to here if this rule does not match */


            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogfile_group_optionsEx; /* Prevent compiler warnings */
    rulelogfile_group_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logfile_group_options */

/**
 * $ANTLR start logfile_group_option
 * MySQLSimpleParser.g:470:1: logfile_group_option : ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | ( UNDO_BUFFER_SIZE_SYMBOL | REDO_BUFFER_SIZE_SYMBOL ) ( EQUAL_OPERATOR )? size_number | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref );
 */
static void
logfile_group_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:470:21: ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | ( UNDO_BUFFER_SIZE_SYMBOL | REDO_BUFFER_SIZE_SYMBOL ) ( EQUAL_OPERATOR )? size_number | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref )

            ANTLR3_UINT32 alt165;

            alt165=6;

            switch ( LA(1) )
            {
            case INITIAL_SIZE_SYMBOL:
            	{
            		alt165=1;
            	}
                break;
            case REDO_BUFFER_SIZE_SYMBOL:
            case UNDO_BUFFER_SIZE_SYMBOL:
            	{
            		alt165=2;
            	}
                break;
            case NODEGROUP_SYMBOL:
            	{
            		alt165=3;
            	}
                break;
            case NO_WAIT_SYMBOL:
            case WAIT_SYMBOL:
            	{
            		alt165=4;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt165=5;
            	}
                break;
            case ENGINE_SYMBOL:
            case STORAGE_SYMBOL:
            	{
            		alt165=6;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 165;
                EXCEPTION->state        = 0;


                goto rulelogfile_group_optionEx;

            }

            switch (alt165)
            {
        	case 1:
        	    // MySQLSimpleParser.g:471:2: INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(INITIAL_SIZE_SYMBOL, &FOLLOW_INITIAL_SIZE_SYMBOL_in_logfile_group_option2464);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:471:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt159=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt159=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt159)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:471:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2466);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_logfile_group_option2469);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:472:4: ( UNDO_BUFFER_SIZE_SYMBOL | REDO_BUFFER_SIZE_SYMBOL ) ( EQUAL_OPERATOR )? size_number
        	    {
        	        if ( LA(1) == REDO_BUFFER_SIZE_SYMBOL || LA(1) == UNDO_BUFFER_SIZE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulelogfile_group_optionEx;
        	        }


        	        // MySQLSimpleParser.g:472:56: ( EQUAL_OPERATOR )?
        	        {
        	            int alt160=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt160=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt160)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:472:56: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2482);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_logfile_group_option2485);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:473:4: NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number
        	    {
        	         MATCHT(NODEGROUP_SYMBOL, &FOLLOW_NODEGROUP_SYMBOL_in_logfile_group_option2490);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:473:21: ( EQUAL_OPERATOR )?
        	        {
        	            int alt161=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt161=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt161)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:473:21: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2492);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_logfile_group_option2495);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:474:4: ( WAIT_SYMBOL | NO_WAIT_SYMBOL )
        	    {
        	        if ( LA(1) == NO_WAIT_SYMBOL || LA(1) == WAIT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulelogfile_group_optionEx;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:475:4: COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_logfile_group_option2511);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:475:19: ( EQUAL_OPERATOR )?
        	        {
        	            int alt162=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt162=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt162)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:475:19: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2513);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_string_literal_in_logfile_group_option2516);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:476:4: ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:476:4: ( STORAGE_SYMBOL )?
        	        {
        	            int alt163=2;
        	            switch ( LA(1) )
        	            {
        	                case STORAGE_SYMBOL:
        	                	{
        	                		alt163=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt163)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:476:4: STORAGE_SYMBOL
        	        	    {
        	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_logfile_group_option2521);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_logfile_group_option2524);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:476:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt164=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt164=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt164)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:476:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_logfile_group_option2526);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelogfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_logfile_group_option2529);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelogfile_group_optionEx; /* Prevent compiler warnings */
    rulelogfile_group_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logfile_group_option */

/**
 * $ANTLR start create_server
 * MySQLSimpleParser.g:479:1: create_server : CREATE_SYMBOL create_server_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_server(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:479:14: ( CREATE_SYMBOL create_server_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:480:2: CREATE_SYMBOL create_server_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_server2540);
            if  (HASEXCEPTION())
            {
                goto rulecreate_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_server_tail_in_create_server2542);
            create_server_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:480:35: ( SEMICOLON_SYMBOL )?
            {
                int alt166=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt166=1;
                    	}
                        break;
                }

                switch (alt166)
                {
            	case 1:
            	    // MySQLSimpleParser.g:480:35: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_server2544);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_serverEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_server2547);
            if  (HASEXCEPTION())
            {
                goto rulecreate_serverEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_serverEx; /* Prevent compiler warnings */
    rulecreate_serverEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_server */

/**
 * $ANTLR start create_server_tail
 * MySQLSimpleParser.g:483:1: create_server_tail : SERVER_SYMBOL server_name FOREIGN_SYMBOL DATA_SYMBOL WRAPPER_SYMBOL text_or_identifier server_options ;
 */
static void
create_server_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:483:19: ( SERVER_SYMBOL server_name FOREIGN_SYMBOL DATA_SYMBOL WRAPPER_SYMBOL text_or_identifier server_options )
        // MySQLSimpleParser.g:484:2: SERVER_SYMBOL server_name FOREIGN_SYMBOL DATA_SYMBOL WRAPPER_SYMBOL text_or_identifier server_options
        {
             MATCHT(SERVER_SYMBOL, &FOLLOW_SERVER_SYMBOL_in_create_server_tail2556);
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_server_name_in_create_server_tail2558);
            server_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FOREIGN_SYMBOL, &FOLLOW_FOREIGN_SYMBOL_in_create_server_tail2562);
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_create_server_tail2564);
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(WRAPPER_SYMBOL, &FOLLOW_WRAPPER_SYMBOL_in_create_server_tail2566);
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_or_identifier_in_create_server_tail2568);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_server_options_in_create_server_tail2570);
            server_options(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_server_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_server_tailEx; /* Prevent compiler warnings */
    rulecreate_server_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_server_tail */

/**
 * $ANTLR start server_options
 * MySQLSimpleParser.g:488:1: server_options : OPTIONS_SYMBOL OPEN_PAR_SYMBOL server_option ( COMMA_SYMBOL server_option )* CLOSE_PAR_SYMBOL ;
 */
static void
server_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:488:15: ( OPTIONS_SYMBOL OPEN_PAR_SYMBOL server_option ( COMMA_SYMBOL server_option )* CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:489:2: OPTIONS_SYMBOL OPEN_PAR_SYMBOL server_option ( COMMA_SYMBOL server_option )* CLOSE_PAR_SYMBOL
        {
             MATCHT(OPTIONS_SYMBOL, &FOLLOW_OPTIONS_SYMBOL_in_server_options2579);
            if  (HASEXCEPTION())
            {
                goto ruleserver_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_server_options2581);
            if  (HASEXCEPTION())
            {
                goto ruleserver_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_server_option_in_server_options2583);
            server_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleserver_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:489:47: ( COMMA_SYMBOL server_option )*

            for (;;)
            {
                int alt167=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt167=1;
                	}
                    break;

                }

                switch (alt167)
                {
            	case 1:
            	    // MySQLSimpleParser.g:489:48: COMMA_SYMBOL server_option
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_server_options2586);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleserver_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_server_option_in_server_options2588);
            	        server_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleserver_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop167;	/* break out of the loop */
            	    break;
                }
            }
            loop167: ; /* Jump out to here if this rule does not match */


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_server_options2592);
            if  (HASEXCEPTION())
            {
                goto ruleserver_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleserver_optionsEx; /* Prevent compiler warnings */
    ruleserver_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end server_options */

/**
 * $ANTLR start server_option
 * MySQLSimpleParser.g:493:1: server_option : ( HOST_SYMBOL string_literal | DATABASE_SYMBOL string_literal | USER_SYMBOL string_literal | PASSWORD_SYMBOL string_literal | SOCKET_SYMBOL string_literal | OWNER_SYMBOL string_literal | PORT_SYMBOL ulong_number );
 */
static void
server_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:493:14: ( HOST_SYMBOL string_literal | DATABASE_SYMBOL string_literal | USER_SYMBOL string_literal | PASSWORD_SYMBOL string_literal | SOCKET_SYMBOL string_literal | OWNER_SYMBOL string_literal | PORT_SYMBOL ulong_number )

            ANTLR3_UINT32 alt168;

            alt168=7;

            switch ( LA(1) )
            {
            case HOST_SYMBOL:
            	{
            		alt168=1;
            	}
                break;
            case DATABASE_SYMBOL:
            	{
            		alt168=2;
            	}
                break;
            case USER_SYMBOL:
            	{
            		alt168=3;
            	}
                break;
            case PASSWORD_SYMBOL:
            	{
            		alt168=4;
            	}
                break;
            case SOCKET_SYMBOL:
            	{
            		alt168=5;
            	}
                break;
            case OWNER_SYMBOL:
            	{
            		alt168=6;
            	}
                break;
            case PORT_SYMBOL:
            	{
            		alt168=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 168;
                EXCEPTION->state        = 0;


                goto ruleserver_optionEx;

            }

            switch (alt168)
            {
        	case 1:
        	    // MySQLSimpleParser.g:494:2: HOST_SYMBOL string_literal
        	    {
        	         MATCHT(HOST_SYMBOL, &FOLLOW_HOST_SYMBOL_in_server_option2602);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2604);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:495:4: DATABASE_SYMBOL string_literal
        	    {
        	         MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_server_option2609);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2611);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:496:4: USER_SYMBOL string_literal
        	    {
        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_server_option2616);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2618);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:497:4: PASSWORD_SYMBOL string_literal
        	    {
        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_server_option2623);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2625);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:498:4: SOCKET_SYMBOL string_literal
        	    {
        	         MATCHT(SOCKET_SYMBOL, &FOLLOW_SOCKET_SYMBOL_in_server_option2630);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2632);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:499:4: OWNER_SYMBOL string_literal
        	    {
        	         MATCHT(OWNER_SYMBOL, &FOLLOW_OWNER_SYMBOL_in_server_option2637);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_server_option2639);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:500:4: PORT_SYMBOL ulong_number
        	    {
        	         MATCHT(PORT_SYMBOL, &FOLLOW_PORT_SYMBOL_in_server_option2644);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_server_option2646);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleserver_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleserver_optionEx; /* Prevent compiler warnings */
    ruleserver_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end server_option */

/**
 * $ANTLR start create_table
 * MySQLSimpleParser.g:503:1: create_table : CREATE_SYMBOL create_table_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_table(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:503:13: ( CREATE_SYMBOL create_table_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:504:2: CREATE_SYMBOL create_table_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_table2656);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_table_tail_in_create_table2658);
            create_table_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_tableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:504:34: ( SEMICOLON_SYMBOL )?
            {
                int alt169=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt169=1;
                    	}
                        break;
                }

                switch (alt169)
                {
            	case 1:
            	    // MySQLSimpleParser.g:504:34: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_table2660);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_table2663);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_tableEx; /* Prevent compiler warnings */
    rulecreate_tableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_table */

/**
 * $ANTLR start create_table_tail
 * MySQLSimpleParser.g:507:1: create_table_tail : ( TEMPORARY_SYMBOL )? TABLE_SYMBOL ( if_not_exists )? table_name ( OPEN_PAR_SYMBOL ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? ) | ( create_table_options )? ( partitioning )? table_creation_source | LIKE_SYMBOL table_ref | OPEN_PAR_SYMBOL LIKE_SYMBOL table_ref CLOSE_PAR_SYMBOL ) ;
 */
static void
create_table_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:507:18: ( ( TEMPORARY_SYMBOL )? TABLE_SYMBOL ( if_not_exists )? table_name ( OPEN_PAR_SYMBOL ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? ) | ( create_table_options )? ( partitioning )? table_creation_source | LIKE_SYMBOL table_ref | OPEN_PAR_SYMBOL LIKE_SYMBOL table_ref CLOSE_PAR_SYMBOL ) )
        // MySQLSimpleParser.g:508:2: ( TEMPORARY_SYMBOL )? TABLE_SYMBOL ( if_not_exists )? table_name ( OPEN_PAR_SYMBOL ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? ) | ( create_table_options )? ( partitioning )? table_creation_source | LIKE_SYMBOL table_ref | OPEN_PAR_SYMBOL LIKE_SYMBOL table_ref CLOSE_PAR_SYMBOL )
        {
            // MySQLSimpleParser.g:508:2: ( TEMPORARY_SYMBOL )?
            {
                int alt170=2;
                switch ( LA(1) )
                {
                    case TEMPORARY_SYMBOL:
                    	{
                    		alt170=1;
                    	}
                        break;
                }

                switch (alt170)
                {
            	case 1:
            	    // MySQLSimpleParser.g:508:2: TEMPORARY_SYMBOL
            	    {
            	         MATCHT(TEMPORARY_SYMBOL, &FOLLOW_TEMPORARY_SYMBOL_in_create_table_tail2672);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_create_table_tail2675);
            if  (HASEXCEPTION())
            {
                goto rulecreate_table_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:508:33: ( if_not_exists )?
            {
                int alt171=2;
                switch ( LA(1) )
                {
                    case IF_SYMBOL:
                    	{
                    		alt171=1;
                    	}
                        break;
                }

                switch (alt171)
                {
            	case 1:
            	    // MySQLSimpleParser.g:508:33: if_not_exists
            	    {
            	        FOLLOWPUSH(FOLLOW_if_not_exists_in_create_table_tail2677);
            	        if_not_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_table_name_in_create_table_tail2680);
            table_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_table_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:509:3: ( OPEN_PAR_SYMBOL ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? ) | ( create_table_options )? ( partitioning )? table_creation_source | LIKE_SYMBOL table_ref | OPEN_PAR_SYMBOL LIKE_SYMBOL table_ref CLOSE_PAR_SYMBOL )
            {
                int alt179=4;
                {
                    int LA179_0 = LA(1);
                    if ( (LA179_0 == OPEN_PAR_SYMBOL))
                    {
                        {
                            int LA179_1 = LA(2);
                            if ( (LA179_1 == LIKE_SYMBOL))
                            {
                                alt179=4;
                            }
                            else if ( (((LA179_1 >= ACCOUNT_SYMBOL) && (LA179_1 <= YEAR_SYMBOL)) || LA179_1 == CHECK_SYMBOL || LA179_1 == CONSTRAINT_SYMBOL || LA179_1 == FOREIGN_SYMBOL || LA179_1 == FULLTEXT_SYMBOL || LA179_1 == INDEX_SYMBOL || LA179_1 == KEY_SYMBOL || LA179_1 == PRIMARY_SYMBOL || LA179_1 == SPATIAL_SYMBOL || LA179_1 == UNIQUE_SYMBOL || LA179_1 == BACK_TICK_QUOTED_ID || LA179_1 == DOUBLE_QUOTED_TEXT || LA179_1 == IDENTIFIER))
                            {
                                alt179=1;
                            }
                            else if ( (LA179_1 == SELECT_SYMBOL))
                            {
                                alt179=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 179;
                                EXCEPTION->state        = 1;


                                goto rulecreate_table_tailEx;

                            }
                        }
                    }
                    else if ( (((LA179_0 >= CHARSET_SYMBOL) && (LA179_0 <= CHECKSUM_SYMBOL)) || LA179_0 == COMMENT_SYMBOL || LA179_0 == AUTO_INCREMENT_SYMBOL || LA179_0 == AVG_ROW_LENGTH_SYMBOL || LA179_0 == COMPRESSION_SYMBOL || LA179_0 == CONNECTION_SYMBOL || LA179_0 == DATA_SYMBOL || LA179_0 == DELAY_KEY_WRITE_SYMBOL || LA179_0 == ENGINE_SYMBOL || LA179_0 == INSERT_METHOD_SYMBOL || LA179_0 == KEY_BLOCK_SIZE_SYMBOL || LA179_0 == MAX_ROWS_SYMBOL || LA179_0 == MIN_ROWS_SYMBOL || LA179_0 == PACK_KEYS_SYMBOL || LA179_0 == PASSWORD_SYMBOL || LA179_0 == ROW_FORMAT_SYMBOL || ((LA179_0 >= STATS_AUTO_RECALC_SYMBOL) && (LA179_0 <= STATS_SAMPLE_PAGES_SYMBOL)) || LA179_0 == STORAGE_SYMBOL || ((LA179_0 >= TABLE_CHECKSUM_SYMBOL) && (LA179_0 <= TABLESPACE_SYMBOL)) || LA179_0 == TYPE_SYMBOL || LA179_0 == CHAR_SYMBOL || LA179_0 == COLLATE_SYMBOL || LA179_0 == DEFAULT_SYMBOL || LA179_0 == INDEX_SYMBOL || LA179_0 == UNION_SYMBOL))
                    {
                        alt179=2;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA179_0 == PARTITION_SYMBOL))
                    {
                        alt179=2;
                    }
                    else if ( (LA179_0 == IGNORE_SYMBOL || LA179_0 == REPLACE_SYMBOL || LA179_0 == SELECT_SYMBOL || LA179_0 == AS_SYMBOL))
                    {
                        alt179=2;
                    }
                    else if ( (LA179_0 == LIKE_SYMBOL))
                    {
                        alt179=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 179;
                        EXCEPTION->state        = 0;


                        goto rulecreate_table_tailEx;

                    }
                }
                switch (alt179)
                {
            	case 1:
            	    // MySQLSimpleParser.g:510:4: OPEN_PAR_SYMBOL ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? )
            	    {
            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2689);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:511:5: ( create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )? | partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )? )
            	        {
            	            int alt176=2;
            	            {
            	                int LA176_0 = LA(1);
            	                if ( (((LA176_0 >= ACCOUNT_SYMBOL) && (LA176_0 <= PARSER_SYMBOL)) || ((LA176_0 >= PORT_SYMBOL) && (LA176_0 <= YEAR_SYMBOL)) || LA176_0 == CHECK_SYMBOL || LA176_0 == CONSTRAINT_SYMBOL || LA176_0 == FOREIGN_SYMBOL || LA176_0 == FULLTEXT_SYMBOL || LA176_0 == INDEX_SYMBOL || LA176_0 == KEY_SYMBOL || LA176_0 == PRIMARY_SYMBOL || LA176_0 == SPATIAL_SYMBOL || LA176_0 == UNIQUE_SYMBOL || LA176_0 == BACK_TICK_QUOTED_ID || LA176_0 == DOUBLE_QUOTED_TEXT || LA176_0 == IDENTIFIER))
            	                {
            	                    alt176=1;
            	                }
            	                else if ( (LA176_0 == PARTITION_SYMBOL))
            	                {
            	                    {
            	                        int LA176_5 = LA(2);
            	                        if ( ((SERVER_VERSION >= 50100)) && (LA176_5 == BY_SYMBOL))
            	                        {
            	                            alt176=2;
            	                        }
            	                        else if ( (LA176_5 == BIT_SYMBOL || ((LA176_5 >= BOOL_SYMBOL) && (LA176_5 <= BOOLEAN_SYMBOL)) || ((LA176_5 >= DATETIME_SYMBOL) && (LA176_5 <= DATE_SYMBOL)) || LA176_5 == ENUM_SYMBOL || LA176_5 == FIXED_SYMBOL || ((LA176_5 >= GEOMETRY_SYMBOL) && (LA176_5 <= GEOMETRYCOLLECTION_SYMBOL)) || LA176_5 == JSON_SYMBOL || LA176_5 == LINESTRING_SYMBOL || ((LA176_5 >= MULTILINESTRING_SYMBOL) && (LA176_5 <= MULTIPOLYGON_SYMBOL)) || ((LA176_5 >= NATIONAL_SYMBOL) && (LA176_5 <= NCHAR_SYMBOL)) || LA176_5 == NVARCHAR_SYMBOL || ((LA176_5 >= POINT_SYMBOL) && (LA176_5 <= POLYGON_SYMBOL)) || LA176_5 == SERIAL_SYMBOL || LA176_5 == TEXT_SYMBOL || LA176_5 == TIMESTAMP_SYMBOL || LA176_5 == TIME_SYMBOL || LA176_5 == YEAR_SYMBOL || ((LA176_5 >= BIGINT_SYMBOL) && (LA176_5 <= BINARY_SYMBOL)) || LA176_5 == BLOB_SYMBOL || LA176_5 == CHAR_SYMBOL || LA176_5 == DECIMAL_SYMBOL || LA176_5 == FLOAT_SYMBOL || LA176_5 == INT_SYMBOL || ((LA176_5 >= LONGBLOB_SYMBOL) && (LA176_5 <= LONGTEXT_SYMBOL)) || LA176_5 == LONG_SYMBOL || ((LA176_5 >= MEDIUMBLOB_SYMBOL) && (LA176_5 <= MEDIUMTEXT_SYMBOL)) || LA176_5 == NUMERIC_SYMBOL || LA176_5 == REAL_SYMBOL || LA176_5 == SET_SYMBOL || LA176_5 == SMALLINT_SYMBOL || ((LA176_5 >= TINYBLOB_SYMBOL) && (LA176_5 <= TINYTEXT_SYMBOL)) || LA176_5 == VARBINARY_SYMBOL || LA176_5 == VARCHAR_SYMBOL || LA176_5 == DOUBLE_SYMBOL))
            	                        {
            	                            alt176=1;
            	                        }
            	                        else
            	                        {
            	                            if (BACKTRACKING>0)
            	                            {
            	                                FAILEDFLAG = ANTLR3_TRUE;
            	                                return ;
            	                            }


            	                            CONSTRUCTEX();
            	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                            EXCEPTION->message      = (void *)"";
            	                            EXCEPTION->decisionNum  = 176;
            	                            EXCEPTION->state        = 5;


            	                            goto rulecreate_table_tailEx;

            	                        }
            	                    }
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 176;
            	                    EXCEPTION->state        = 0;


            	                    goto rulecreate_table_tailEx;

            	                }
            	            }
            	            switch (alt176)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:512:6: create_field_list CLOSE_PAR_SYMBOL ( create_table_options )? ( partitioning )? ( table_creation_source )?
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_create_field_list_in_create_table_tail2702);
            	        	        create_field_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2704);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:512:41: ( create_table_options )?
            	        	        {
            	        	            int alt172=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case CHARSET_SYMBOL:
            	        	                case CHECKSUM_SYMBOL:
            	        	                case COMMENT_SYMBOL:
            	        	                case AUTO_INCREMENT_SYMBOL:
            	        	                case AVG_ROW_LENGTH_SYMBOL:
            	        	                case COMPRESSION_SYMBOL:
            	        	                case CONNECTION_SYMBOL:
            	        	                case DATA_SYMBOL:
            	        	                case DELAY_KEY_WRITE_SYMBOL:
            	        	                case ENGINE_SYMBOL:
            	        	                case INSERT_METHOD_SYMBOL:
            	        	                case KEY_BLOCK_SIZE_SYMBOL:
            	        	                case MAX_ROWS_SYMBOL:
            	        	                case MIN_ROWS_SYMBOL:
            	        	                case PACK_KEYS_SYMBOL:
            	        	                case PASSWORD_SYMBOL:
            	        	                case ROW_FORMAT_SYMBOL:
            	        	                case STATS_AUTO_RECALC_SYMBOL:
            	        	                case STATS_PERSISTENT_SYMBOL:
            	        	                case STATS_SAMPLE_PAGES_SYMBOL:
            	        	                case STORAGE_SYMBOL:
            	        	                case TABLE_CHECKSUM_SYMBOL:
            	        	                case TABLESPACE_SYMBOL:
            	        	                case TYPE_SYMBOL:
            	        	                case CHAR_SYMBOL:
            	        	                case COLLATE_SYMBOL:
            	        	                case DEFAULT_SYMBOL:
            	        	                case INDEX_SYMBOL:
            	        	                case UNION_SYMBOL:
            	        	                	{
            	        	                		alt172=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt172)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:512:41: create_table_options
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_create_table_options_in_create_table_tail2706);
            	        	        	        create_table_options(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulecreate_table_tailEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:512:63: ( partitioning )?
            	        	        {
            	        	            int alt173=2;
            	        	            {
            	        	                int LA173_0 = LA(1);
            	        	                if ( ((SERVER_VERSION >= 50100)) && (LA173_0 == PARTITION_SYMBOL))
            	        	                {
            	        	                    alt173=1;
            	        	                }
            	        	            }
            	        	            switch (alt173)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:512:63: partitioning
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_partitioning_in_create_table_tail2709);
            	        	        	        partitioning(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulecreate_table_tailEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:512:77: ( table_creation_source )?
            	        	        {
            	        	            int alt174=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case IGNORE_SYMBOL:
            	        	                case OPEN_PAR_SYMBOL:
            	        	                case REPLACE_SYMBOL:
            	        	                case SELECT_SYMBOL:
            	        	                case AS_SYMBOL:
            	        	                	{
            	        	                		alt174=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt174)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:512:77: table_creation_source
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_table_creation_source_in_create_table_tail2712);
            	        	        	        table_creation_source(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulecreate_table_tailEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:513:8: partitioning create_select CLOSE_PAR_SYMBOL ( union_clause )?
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_partitioning_in_create_table_tail2722);
            	        	        partitioning(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_create_select_in_create_table_tail2724);
            	        	        create_select(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2726);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:513:52: ( union_clause )?
            	        	        {
            	        	            int alt175=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case UNION_SYMBOL:
            	        	                	{
            	        	                		alt175=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt175)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:513:52: union_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_union_clause_in_create_table_tail2728);
            	        	        	        union_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulecreate_table_tailEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:515:6: ( create_table_options )? ( partitioning )? table_creation_source
            	    {
            	        // MySQLSimpleParser.g:515:6: ( create_table_options )?
            	        {
            	            int alt177=2;
            	            switch ( LA(1) )
            	            {
            	                case CHARSET_SYMBOL:
            	                case CHECKSUM_SYMBOL:
            	                case COMMENT_SYMBOL:
            	                case AUTO_INCREMENT_SYMBOL:
            	                case AVG_ROW_LENGTH_SYMBOL:
            	                case COMPRESSION_SYMBOL:
            	                case CONNECTION_SYMBOL:
            	                case DATA_SYMBOL:
            	                case DELAY_KEY_WRITE_SYMBOL:
            	                case ENGINE_SYMBOL:
            	                case INSERT_METHOD_SYMBOL:
            	                case KEY_BLOCK_SIZE_SYMBOL:
            	                case MAX_ROWS_SYMBOL:
            	                case MIN_ROWS_SYMBOL:
            	                case PACK_KEYS_SYMBOL:
            	                case PASSWORD_SYMBOL:
            	                case ROW_FORMAT_SYMBOL:
            	                case STATS_AUTO_RECALC_SYMBOL:
            	                case STATS_PERSISTENT_SYMBOL:
            	                case STATS_SAMPLE_PAGES_SYMBOL:
            	                case STORAGE_SYMBOL:
            	                case TABLE_CHECKSUM_SYMBOL:
            	                case TABLESPACE_SYMBOL:
            	                case TYPE_SYMBOL:
            	                case CHAR_SYMBOL:
            	                case COLLATE_SYMBOL:
            	                case DEFAULT_SYMBOL:
            	                case INDEX_SYMBOL:
            	                case UNION_SYMBOL:
            	                	{
            	                		alt177=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt177)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:515:6: create_table_options
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_create_table_options_in_create_table_tail2742);
            	        	        create_table_options(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:515:28: ( partitioning )?
            	        {
            	            int alt178=2;
            	            {
            	                int LA178_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50100)) && (LA178_0 == PARTITION_SYMBOL))
            	                {
            	                    alt178=1;
            	                }
            	            }
            	            switch (alt178)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:515:28: partitioning
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_partitioning_in_create_table_tail2745);
            	        	        partitioning(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_tailEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_table_creation_source_in_create_table_tail2748);
            	        table_creation_source(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:516:6: LIKE_SYMBOL table_ref
            	    {
            	         MATCHT(LIKE_SYMBOL, &FOLLOW_LIKE_SYMBOL_in_create_table_tail2755);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_create_table_tail2757);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:517:6: OPEN_PAR_SYMBOL LIKE_SYMBOL table_ref CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_create_table_tail2764);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(LIKE_SYMBOL, &FOLLOW_LIKE_SYMBOL_in_create_table_tail2766);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_create_table_tail2768);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_tail2770);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_table_tailEx; /* Prevent compiler warnings */
    rulecreate_table_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_table_tail */

/**
 * $ANTLR start create_field_list
 * MySQLSimpleParser.g:521:1: create_field_list : create_item ( COMMA_SYMBOL create_item )* ;
 */
static void
create_field_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:521:18: ( create_item ( COMMA_SYMBOL create_item )* )
        // MySQLSimpleParser.g:522:2: create_item ( COMMA_SYMBOL create_item )*
        {
            FOLLOWPUSH(FOLLOW_create_item_in_create_field_list2783);
            create_item(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_field_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:522:14: ( COMMA_SYMBOL create_item )*

            for (;;)
            {
                int alt180=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt180=1;
                	}
                    break;

                }

                switch (alt180)
                {
            	case 1:
            	    // MySQLSimpleParser.g:522:15: COMMA_SYMBOL create_item
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_create_field_list2786);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_field_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_create_item_in_create_field_list2788);
            	        create_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_field_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop180;	/* break out of the loop */
            	    break;
                }
            }
            loop180: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_field_listEx; /* Prevent compiler warnings */
    rulecreate_field_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_field_list */

/**
 * $ANTLR start create_item
 * MySQLSimpleParser.g:525:1: create_item : create_field_list_item ;
 */
static void
create_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:525:12: ( create_field_list_item )
        // MySQLSimpleParser.g:526:2: create_field_list_item
        {
            FOLLOWPUSH(FOLLOW_create_field_list_item_in_create_item2799);
            create_field_list_item(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_itemEx; /* Prevent compiler warnings */
    rulecreate_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_item */

/**
 * $ANTLR start create_field_list_item
 * MySQLSimpleParser.g:529:1: create_field_list_item : ( column_definition | key_def );
 */
static void
create_field_list_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:529:23: ( column_definition | key_def )

            ANTLR3_UINT32 alt181;

            alt181=2;

            {
                int LA181_0 = LA(1);
                if ( (((LA181_0 >= ACCOUNT_SYMBOL) && (LA181_0 <= YEAR_SYMBOL)) || LA181_0 == BACK_TICK_QUOTED_ID || LA181_0 == DOUBLE_QUOTED_TEXT || LA181_0 == IDENTIFIER))
                {
                    alt181=1;
                }
                else if ( (LA181_0 == CHECK_SYMBOL || LA181_0 == CONSTRAINT_SYMBOL || LA181_0 == FOREIGN_SYMBOL || LA181_0 == FULLTEXT_SYMBOL || LA181_0 == INDEX_SYMBOL || LA181_0 == KEY_SYMBOL || LA181_0 == PRIMARY_SYMBOL || LA181_0 == SPATIAL_SYMBOL || LA181_0 == UNIQUE_SYMBOL))
                {
                    alt181=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 181;
                    EXCEPTION->state        = 0;


                    goto rulecreate_field_list_itemEx;

                }
            }
            switch (alt181)
            {
        	case 1:
        	    // MySQLSimpleParser.g:530:2: column_definition
        	    {
        	        FOLLOWPUSH(FOLLOW_column_definition_in_create_field_list_item2808);
        	        column_definition(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_field_list_itemEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:531:4: key_def
        	    {
        	        FOLLOWPUSH(FOLLOW_key_def_in_create_field_list_item2813);
        	        key_def(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_field_list_itemEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecreate_field_list_itemEx; /* Prevent compiler warnings */
    rulecreate_field_list_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_field_list_item */

/**
 * $ANTLR start table_creation_source
 * MySQLSimpleParser.g:534:1: table_creation_source : ( REPLACE_SYMBOL | IGNORE_SYMBOL )? ( AS_SYMBOL )? ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? ) ;
 */
static void
table_creation_source(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:534:22: ( ( REPLACE_SYMBOL | IGNORE_SYMBOL )? ( AS_SYMBOL )? ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? ) )
        // MySQLSimpleParser.g:535:2: ( REPLACE_SYMBOL | IGNORE_SYMBOL )? ( AS_SYMBOL )? ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? )
        {
            // MySQLSimpleParser.g:535:2: ( REPLACE_SYMBOL | IGNORE_SYMBOL )?
            {
                int alt182=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    case REPLACE_SYMBOL:
                    	{
                    		alt182=1;
                    	}
                        break;
                }

                switch (alt182)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == IGNORE_SYMBOL || LA(1) == REPLACE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruletable_creation_sourceEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:535:36: ( AS_SYMBOL )?
            {
                int alt183=2;
                switch ( LA(1) )
                {
                    case AS_SYMBOL:
                    	{
                    		alt183=1;
                    	}
                        break;
                }

                switch (alt183)
                {
            	case 1:
            	    // MySQLSimpleParser.g:535:36: AS_SYMBOL
            	    {
            	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_table_creation_source2832);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_creation_sourceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:536:2: ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? )
            {
                int alt186=2;
                switch ( LA(1) )
                {
                case SELECT_SYMBOL:
                	{
                		alt186=1;
                	}
                    break;
                case OPEN_PAR_SYMBOL:
                	{
                		alt186=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 186;
                    EXCEPTION->state        = 0;


                    goto ruletable_creation_sourceEx;

                }

                switch (alt186)
                {
            	case 1:
            	    // MySQLSimpleParser.g:537:3: create_select ( union_clause )?
            	    {
            	        FOLLOWPUSH(FOLLOW_create_select_in_table_creation_source2840);
            	        create_select(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_creation_sourceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:537:17: ( union_clause )?
            	        {
            	            int alt184=2;
            	            switch ( LA(1) )
            	            {
            	                case UNION_SYMBOL:
            	                	{
            	                		alt184=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt184)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:537:17: union_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_union_clause_in_table_creation_source2842);
            	        	        union_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletable_creation_sourceEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:538:5: OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )?
            	    {
            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_table_creation_source2849);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_creation_sourceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_create_select_in_table_creation_source2851);
            	        create_select(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_creation_sourceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_table_creation_source2853);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_creation_sourceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:538:52: ( union_or_order_by_or_limit )?
            	        {
            	            int alt185=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                case ORDER_SYMBOL:
            	                case UNION_SYMBOL:
            	                	{
            	                		alt185=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt185)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:538:52: union_or_order_by_or_limit
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_union_or_order_by_or_limit_in_table_creation_source2855);
            	        	        union_or_order_by_or_limit(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletable_creation_sourceEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_creation_sourceEx; /* Prevent compiler warnings */
    ruletable_creation_sourceEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_creation_source */

/**
 * $ANTLR start create_select
 * MySQLSimpleParser.g:543:1: create_select : SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression ;
 */
static void
create_select(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:543:14: ( SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression )
        // MySQLSimpleParser.g:544:2: SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression
        {
             MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_create_select2869);
            if  (HASEXCEPTION())
            {
                goto rulecreate_selectEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:544:16: ( options {greedy=true; } : select_option )*

            for (;;)
            {
                int alt187=2;
                switch ( LA(1) )
                {
                case SQL_BUFFER_RESULT_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA187_96 = LA(2);
                		    if ( (LA187_96 == DOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MOD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MINUS_OPERATOR || LA187_96 == PLUS_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == NOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == SOUNDS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == IDENTIFIER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == DATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ASCII_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == INTERVAL_SYMBOL || LA187_96 == NCHAR_TEXT || LA187_96 == NOT2_SYMBOL || LA187_96 == SINGLE_QUOTED_TEXT || LA187_96 == BITWISE_NOT_OPERATOR || LA187_96 == LOGICAL_NOT_OPERATOR || LA187_96 == UNDERSCORE_CHARSET))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == DECIMAL_NUMBER || LA187_96 == FLOAT_NUMBER || LA187_96 == HEX_NUMBER || LA187_96 == INT_NUMBER || LA187_96 == LONG_NUMBER || LA187_96 == ULONGLONG_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == TIMESTAMP_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == DAY_SYMBOL || LA187_96 == FALSE_SYMBOL || ((LA187_96 >= NULL2_SYMBOL) && (LA187_96 <= NULL_SYMBOL)) || LA187_96 == TRUE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == CHAR_SYMBOL || LA187_96 == CURRENT_USER_SYMBOL || LA187_96 == BIN_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == HOUR_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MINUTE_SYMBOL || LA187_96 == INSERT_SYMBOL || LA187_96 == LEFT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MONTH_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == SECOND_SYMBOL || LA187_96 == RIGHT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == USER_SYMBOL || LA187_96 == TRIM_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == YEAR_SYMBOL || LA187_96 == VALUES_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ADDDATE_SYMBOL || LA187_96 == SUBDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == GET_FORMAT_SYMBOL || LA187_96 == CURDATE_SYMBOL || LA187_96 == CURTIME_SYMBOL || LA187_96 == DATE_ADD_SYMBOL || LA187_96 == DATE_SUB_SYMBOL || LA187_96 == EXTRACT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (((LA187_96 >= TIMESTAMP_ADD_SYMBOL) && (LA187_96 <= TIMESTAMP_DIFF_SYMBOL)) || LA187_96 == NOW_SYMBOL || LA187_96 == POSITION_SYMBOL || LA187_96 == SUBSTRING_SYMBOL || LA187_96 == SYSDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == COALESCE_SYMBOL || ((LA187_96 >= UTC_DATE_SYMBOL) && (LA187_96 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == CHARSET_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == FORMAT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == COLLATION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MICROSECOND_SYMBOL || LA187_96 == DATABASE_SYMBOL || LA187_96 == IF_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == TRUNCATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MULT_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == QUARTER_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == REVERSE_SYMBOL || ((LA187_96 >= REPEAT_SYMBOL) && (LA187_96 <= REPLACE_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ROW_COUNT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == WEEK_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == CONTAINS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == LINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MULTIPOINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == POINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == POLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == AVG_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ACCOUNT_SYMBOL || ((LA187_96 >= ALWAYS_SYMBOL) && (LA187_96 <= CACHE_SYMBOL)) || ((LA187_96 >= CHECKSUM_SYMBOL) && (LA187_96 <= COMMIT_SYMBOL)) || ((LA187_96 >= DEALLOCATE_SYMBOL) && (LA187_96 <= FOLLOWS_SYMBOL)) || ((LA187_96 >= GROUP_REPLICATION_SYMBOL) && (LA187_96 <= STOP_SYMBOL)) || ((LA187_96 >= UNICODE_SYMBOL) && (LA187_96 <= XA_SYMBOL)) || LA187_96 == SHUTDOWN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ROW_SYMBOL || LA187_96 == COUNT_SYMBOL || LA187_96 == EXISTS_SYMBOL || LA187_96 == GROUP_CONCAT_SYMBOL || LA187_96 == MAX_SYMBOL || LA187_96 == MIN_SYMBOL || LA187_96 == OPEN_PAR_SYMBOL || LA187_96 == PARAM_MARKER || LA187_96 == STDDEV_SAMP_SYMBOL || LA187_96 == STD_SYMBOL || LA187_96 == SUM_SYMBOL || LA187_96 == VARIANCE_SYMBOL || LA187_96 == VAR_SAMP_SYMBOL || ((LA187_96 >= AT_AT_SIGN_SYMBOL) && (LA187_96 <= AT_TEXT_SUFFIX)) || ((LA187_96 >= BIT_AND_SYMBOL) && (LA187_96 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == SQL_BUFFER_RESULT_SYMBOL || LA187_96 == BINARY_SYMBOL || ((LA187_96 >= CASE_SYMBOL) && (LA187_96 <= CAST_SYMBOL)) || LA187_96 == CONVERT_SYMBOL || LA187_96 == MATCH_SYMBOL || LA187_96 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == SQL_NO_CACHE_SYMBOL || LA187_96 == ALL_SYMBOL || LA187_96 == DISTINCT_SYMBOL || LA187_96 == HIGH_PRIORITY_SYMBOL || ((LA187_96 >= SQL_BIG_RESULT_SYMBOL) && (LA187_96 <= SQL_SMALL_RESULT_SYMBOL)) || LA187_96 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == SQL_CACHE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_96 == ACTION_SYMBOL || ((LA187_96 >= AFTER_SYMBOL) && (LA187_96 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA187_96 >= BINLOG_SYMBOL) && (LA187_96 <= CLASS_ORIGIN_SYMBOL)) || LA187_96 == CODE_SYMBOL || ((LA187_96 >= COLUMN_NAME_SYMBOL) && (LA187_96 <= DATETIME_SYMBOL)) || ((LA187_96 >= DEFAULT_AUTH_SYMBOL) && (LA187_96 <= GEOMETRY_SYMBOL)) || ((LA187_96 >= GRANTS_SYMBOL) && (LA187_96 <= HOSTS_SYMBOL)) || ((LA187_96 >= IDENTIFIED_SYMBOL) && (LA187_96 <= LEVEL_SYMBOL)) || ((LA187_96 >= LIST_SYMBOL) && (LA187_96 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA187_96 >= MAX_SIZE_SYMBOL) && (LA187_96 <= MESSAGE_TEXT_SYMBOL)) || LA187_96 == MIGRATE_SYMBOL || ((LA187_96 >= MIN_ROWS_SYMBOL) && (LA187_96 <= MODE_SYMBOL)) || ((LA187_96 >= MUTEX_SYMBOL) && (LA187_96 <= OFFSET_SYMBOL)) || ((LA187_96 >= ONE_SHOT_SYMBOL) && (LA187_96 <= PARTITIONS_SYMBOL)) || ((LA187_96 >= PHASE_SYMBOL) && (LA187_96 <= PLUGINS_SYMBOL)) || ((LA187_96 >= PRESERVE_SYMBOL) && (LA187_96 <= PROXY_SYMBOL)) || ((LA187_96 >= QUERY_SYMBOL) && (LA187_96 <= RETURNS_SYMBOL)) || ((LA187_96 >= ROLLUP_SYMBOL) && (LA187_96 <= ROWS_SYMBOL)) || LA187_96 == ROW_FORMAT_SYMBOL || ((LA187_96 >= RTREE_SYMBOL) && (LA187_96 <= SCHEMA_NAME_SYMBOL)) || ((LA187_96 >= SERIAL_SYMBOL) && (LA187_96 <= SHARE_SYMBOL)) || ((LA187_96 >= SLOW_SYMBOL) && (LA187_96 <= SNAPSHOT_SYMBOL)) || ((LA187_96 >= SOURCE_SYMBOL) && (LA187_96 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA187_96 >= SQL_THREAD_SYMBOL) && (LA187_96 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA187_96 >= SUBJECT_SYMBOL) && (LA187_96 <= TRIGGERS_SYMBOL)) || ((LA187_96 >= TYPES_SYMBOL) && (LA187_96 <= USER_RESOURCES_SYMBOL)) || ((LA187_96 >= USE_FRM_SYMBOL) && (LA187_96 <= WAIT_SYMBOL)) || LA187_96 == WORK_SYMBOL || ((LA187_96 >= X509_SYMBOL) && (LA187_96 <= XML_SYMBOL))))
                		    {
                		        alt187=1;
                		    }

                		}
                	}
                    break;
                case SQL_NO_CACHE_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA187_104 = LA(2);
                		    if ( (LA187_104 == DOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MOD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MINUS_OPERATOR || LA187_104 == PLUS_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == NOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == SOUNDS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == IDENTIFIER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == DATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ASCII_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == INTERVAL_SYMBOL || LA187_104 == NCHAR_TEXT || LA187_104 == NOT2_SYMBOL || LA187_104 == SINGLE_QUOTED_TEXT || LA187_104 == BITWISE_NOT_OPERATOR || LA187_104 == LOGICAL_NOT_OPERATOR || LA187_104 == UNDERSCORE_CHARSET))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == DECIMAL_NUMBER || LA187_104 == FLOAT_NUMBER || LA187_104 == HEX_NUMBER || LA187_104 == INT_NUMBER || LA187_104 == LONG_NUMBER || LA187_104 == ULONGLONG_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == TIMESTAMP_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == DAY_SYMBOL || LA187_104 == FALSE_SYMBOL || ((LA187_104 >= NULL2_SYMBOL) && (LA187_104 <= NULL_SYMBOL)) || LA187_104 == TRUE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == CHAR_SYMBOL || LA187_104 == CURRENT_USER_SYMBOL || LA187_104 == BIN_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == HOUR_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MINUTE_SYMBOL || LA187_104 == INSERT_SYMBOL || LA187_104 == LEFT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MONTH_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == SECOND_SYMBOL || LA187_104 == RIGHT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == USER_SYMBOL || LA187_104 == TRIM_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == YEAR_SYMBOL || LA187_104 == VALUES_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ADDDATE_SYMBOL || LA187_104 == SUBDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == GET_FORMAT_SYMBOL || LA187_104 == CURDATE_SYMBOL || LA187_104 == CURTIME_SYMBOL || LA187_104 == DATE_ADD_SYMBOL || LA187_104 == DATE_SUB_SYMBOL || LA187_104 == EXTRACT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (((LA187_104 >= TIMESTAMP_ADD_SYMBOL) && (LA187_104 <= TIMESTAMP_DIFF_SYMBOL)) || LA187_104 == NOW_SYMBOL || LA187_104 == POSITION_SYMBOL || LA187_104 == SUBSTRING_SYMBOL || LA187_104 == SYSDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == COALESCE_SYMBOL || ((LA187_104 >= UTC_DATE_SYMBOL) && (LA187_104 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == CHARSET_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == FORMAT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == COLLATION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MICROSECOND_SYMBOL || LA187_104 == DATABASE_SYMBOL || LA187_104 == IF_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == TRUNCATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MULT_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == QUARTER_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == REVERSE_SYMBOL || ((LA187_104 >= REPEAT_SYMBOL) && (LA187_104 <= REPLACE_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ROW_COUNT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == WEEK_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == CONTAINS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == LINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MULTIPOINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == POINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == POLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == AVG_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ACCOUNT_SYMBOL || ((LA187_104 >= ALWAYS_SYMBOL) && (LA187_104 <= CACHE_SYMBOL)) || ((LA187_104 >= CHECKSUM_SYMBOL) && (LA187_104 <= COMMIT_SYMBOL)) || ((LA187_104 >= DEALLOCATE_SYMBOL) && (LA187_104 <= FOLLOWS_SYMBOL)) || ((LA187_104 >= GROUP_REPLICATION_SYMBOL) && (LA187_104 <= STOP_SYMBOL)) || ((LA187_104 >= UNICODE_SYMBOL) && (LA187_104 <= XA_SYMBOL)) || LA187_104 == SHUTDOWN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ROW_SYMBOL || LA187_104 == COUNT_SYMBOL || LA187_104 == EXISTS_SYMBOL || LA187_104 == GROUP_CONCAT_SYMBOL || LA187_104 == MAX_SYMBOL || LA187_104 == MIN_SYMBOL || LA187_104 == OPEN_PAR_SYMBOL || LA187_104 == PARAM_MARKER || LA187_104 == STDDEV_SAMP_SYMBOL || LA187_104 == STD_SYMBOL || LA187_104 == SUM_SYMBOL || LA187_104 == VARIANCE_SYMBOL || LA187_104 == VAR_SAMP_SYMBOL || ((LA187_104 >= AT_AT_SIGN_SYMBOL) && (LA187_104 <= AT_TEXT_SUFFIX)) || ((LA187_104 >= BIT_AND_SYMBOL) && (LA187_104 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == SQL_BUFFER_RESULT_SYMBOL || LA187_104 == BINARY_SYMBOL || ((LA187_104 >= CASE_SYMBOL) && (LA187_104 <= CAST_SYMBOL)) || LA187_104 == CONVERT_SYMBOL || LA187_104 == MATCH_SYMBOL || LA187_104 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == SQL_NO_CACHE_SYMBOL || LA187_104 == ALL_SYMBOL || LA187_104 == DISTINCT_SYMBOL || LA187_104 == HIGH_PRIORITY_SYMBOL || ((LA187_104 >= SQL_BIG_RESULT_SYMBOL) && (LA187_104 <= SQL_SMALL_RESULT_SYMBOL)) || LA187_104 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == SQL_CACHE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_104 == ACTION_SYMBOL || ((LA187_104 >= AFTER_SYMBOL) && (LA187_104 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA187_104 >= BINLOG_SYMBOL) && (LA187_104 <= CLASS_ORIGIN_SYMBOL)) || LA187_104 == CODE_SYMBOL || ((LA187_104 >= COLUMN_NAME_SYMBOL) && (LA187_104 <= DATETIME_SYMBOL)) || ((LA187_104 >= DEFAULT_AUTH_SYMBOL) && (LA187_104 <= GEOMETRY_SYMBOL)) || ((LA187_104 >= GRANTS_SYMBOL) && (LA187_104 <= HOSTS_SYMBOL)) || ((LA187_104 >= IDENTIFIED_SYMBOL) && (LA187_104 <= LEVEL_SYMBOL)) || ((LA187_104 >= LIST_SYMBOL) && (LA187_104 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA187_104 >= MAX_SIZE_SYMBOL) && (LA187_104 <= MESSAGE_TEXT_SYMBOL)) || LA187_104 == MIGRATE_SYMBOL || ((LA187_104 >= MIN_ROWS_SYMBOL) && (LA187_104 <= MODE_SYMBOL)) || ((LA187_104 >= MUTEX_SYMBOL) && (LA187_104 <= OFFSET_SYMBOL)) || ((LA187_104 >= ONE_SHOT_SYMBOL) && (LA187_104 <= PARTITIONS_SYMBOL)) || ((LA187_104 >= PHASE_SYMBOL) && (LA187_104 <= PLUGINS_SYMBOL)) || ((LA187_104 >= PRESERVE_SYMBOL) && (LA187_104 <= PROXY_SYMBOL)) || ((LA187_104 >= QUERY_SYMBOL) && (LA187_104 <= RETURNS_SYMBOL)) || ((LA187_104 >= ROLLUP_SYMBOL) && (LA187_104 <= ROWS_SYMBOL)) || LA187_104 == ROW_FORMAT_SYMBOL || ((LA187_104 >= RTREE_SYMBOL) && (LA187_104 <= SCHEMA_NAME_SYMBOL)) || ((LA187_104 >= SERIAL_SYMBOL) && (LA187_104 <= SHARE_SYMBOL)) || ((LA187_104 >= SLOW_SYMBOL) && (LA187_104 <= SNAPSHOT_SYMBOL)) || ((LA187_104 >= SOURCE_SYMBOL) && (LA187_104 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA187_104 >= SQL_THREAD_SYMBOL) && (LA187_104 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA187_104 >= SUBJECT_SYMBOL) && (LA187_104 <= TRIGGERS_SYMBOL)) || ((LA187_104 >= TYPES_SYMBOL) && (LA187_104 <= USER_RESOURCES_SYMBOL)) || ((LA187_104 >= USE_FRM_SYMBOL) && (LA187_104 <= WAIT_SYMBOL)) || LA187_104 == WORK_SYMBOL || ((LA187_104 >= X509_SYMBOL) && (LA187_104 <= XML_SYMBOL))))
                		    {
                		        alt187=1;
                		    }

                		}
                	}
                    break;
                case ALL_SYMBOL:
                case DISTINCT_SYMBOL:
                case HIGH_PRIORITY_SYMBOL:
                case SQL_BIG_RESULT_SYMBOL:
                case SQL_CALC_FOUND_ROWS_SYMBOL:
                case SQL_SMALL_RESULT_SYMBOL:
                case STRAIGHT_JOIN_SYMBOL:
                	{
                		alt187=1;
                	}
                    break;
                case SQL_CACHE_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA187_106 = LA(2);
                		    if ( (LA187_106 == DOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MOD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MINUS_OPERATOR || LA187_106 == PLUS_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == NOT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == SOUNDS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == IDENTIFIER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == DATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ASCII_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == INTERVAL_SYMBOL || LA187_106 == NCHAR_TEXT || LA187_106 == NOT2_SYMBOL || LA187_106 == SINGLE_QUOTED_TEXT || LA187_106 == BITWISE_NOT_OPERATOR || LA187_106 == LOGICAL_NOT_OPERATOR || LA187_106 == UNDERSCORE_CHARSET))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == DECIMAL_NUMBER || LA187_106 == FLOAT_NUMBER || LA187_106 == HEX_NUMBER || LA187_106 == INT_NUMBER || LA187_106 == LONG_NUMBER || LA187_106 == ULONGLONG_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == TIMESTAMP_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == DAY_SYMBOL || LA187_106 == FALSE_SYMBOL || ((LA187_106 >= NULL2_SYMBOL) && (LA187_106 <= NULL_SYMBOL)) || LA187_106 == TRUE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == CHAR_SYMBOL || LA187_106 == CURRENT_USER_SYMBOL || LA187_106 == BIN_NUMBER))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == HOUR_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MINUTE_SYMBOL || LA187_106 == INSERT_SYMBOL || LA187_106 == LEFT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MONTH_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == SECOND_SYMBOL || LA187_106 == RIGHT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == USER_SYMBOL || LA187_106 == TRIM_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == YEAR_SYMBOL || LA187_106 == VALUES_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ADDDATE_SYMBOL || LA187_106 == SUBDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == GET_FORMAT_SYMBOL || LA187_106 == CURDATE_SYMBOL || LA187_106 == CURTIME_SYMBOL || LA187_106 == DATE_ADD_SYMBOL || LA187_106 == DATE_SUB_SYMBOL || LA187_106 == EXTRACT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (((LA187_106 >= TIMESTAMP_ADD_SYMBOL) && (LA187_106 <= TIMESTAMP_DIFF_SYMBOL)) || LA187_106 == NOW_SYMBOL || LA187_106 == POSITION_SYMBOL || LA187_106 == SUBSTRING_SYMBOL || LA187_106 == SYSDATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == COALESCE_SYMBOL || ((LA187_106 >= UTC_DATE_SYMBOL) && (LA187_106 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == CHARSET_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == FORMAT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == COLLATION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MICROSECOND_SYMBOL || LA187_106 == DATABASE_SYMBOL || LA187_106 == IF_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == TRUNCATE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MULT_OPERATOR))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == PASSWORD_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == QUARTER_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == REVERSE_SYMBOL || ((LA187_106 >= REPEAT_SYMBOL) && (LA187_106 <= REPLACE_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ROW_COUNT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == WEEK_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == CONTAINS_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == LINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MULTIPOINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == POINT_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == POLYGON_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == AVG_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ACCOUNT_SYMBOL || ((LA187_106 >= ALWAYS_SYMBOL) && (LA187_106 <= CACHE_SYMBOL)) || ((LA187_106 >= CHECKSUM_SYMBOL) && (LA187_106 <= COMMIT_SYMBOL)) || ((LA187_106 >= DEALLOCATE_SYMBOL) && (LA187_106 <= FOLLOWS_SYMBOL)) || ((LA187_106 >= GROUP_REPLICATION_SYMBOL) && (LA187_106 <= STOP_SYMBOL)) || ((LA187_106 >= UNICODE_SYMBOL) && (LA187_106 <= XA_SYMBOL)) || LA187_106 == SHUTDOWN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ROW_SYMBOL || LA187_106 == COUNT_SYMBOL || LA187_106 == EXISTS_SYMBOL || LA187_106 == GROUP_CONCAT_SYMBOL || LA187_106 == MAX_SYMBOL || LA187_106 == MIN_SYMBOL || LA187_106 == OPEN_PAR_SYMBOL || LA187_106 == PARAM_MARKER || LA187_106 == STDDEV_SAMP_SYMBOL || LA187_106 == STD_SYMBOL || LA187_106 == SUM_SYMBOL || LA187_106 == VARIANCE_SYMBOL || LA187_106 == VAR_SAMP_SYMBOL || ((LA187_106 >= AT_AT_SIGN_SYMBOL) && (LA187_106 <= AT_TEXT_SUFFIX)) || ((LA187_106 >= BIT_AND_SYMBOL) && (LA187_106 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == SQL_BUFFER_RESULT_SYMBOL || LA187_106 == BINARY_SYMBOL || ((LA187_106 >= CASE_SYMBOL) && (LA187_106 <= CAST_SYMBOL)) || LA187_106 == CONVERT_SYMBOL || LA187_106 == MATCH_SYMBOL || LA187_106 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == SQL_NO_CACHE_SYMBOL || LA187_106 == ALL_SYMBOL || LA187_106 == DISTINCT_SYMBOL || LA187_106 == HIGH_PRIORITY_SYMBOL || ((LA187_106 >= SQL_BIG_RESULT_SYMBOL) && (LA187_106 <= SQL_SMALL_RESULT_SYMBOL)) || LA187_106 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == SQL_CACHE_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt187=1;
                		    }
                		    else if ( (LA187_106 == ACTION_SYMBOL || ((LA187_106 >= AFTER_SYMBOL) && (LA187_106 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA187_106 >= BINLOG_SYMBOL) && (LA187_106 <= CLASS_ORIGIN_SYMBOL)) || LA187_106 == CODE_SYMBOL || ((LA187_106 >= COLUMN_NAME_SYMBOL) && (LA187_106 <= DATETIME_SYMBOL)) || ((LA187_106 >= DEFAULT_AUTH_SYMBOL) && (LA187_106 <= GEOMETRY_SYMBOL)) || ((LA187_106 >= GRANTS_SYMBOL) && (LA187_106 <= HOSTS_SYMBOL)) || ((LA187_106 >= IDENTIFIED_SYMBOL) && (LA187_106 <= LEVEL_SYMBOL)) || ((LA187_106 >= LIST_SYMBOL) && (LA187_106 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA187_106 >= MAX_SIZE_SYMBOL) && (LA187_106 <= MESSAGE_TEXT_SYMBOL)) || LA187_106 == MIGRATE_SYMBOL || ((LA187_106 >= MIN_ROWS_SYMBOL) && (LA187_106 <= MODE_SYMBOL)) || ((LA187_106 >= MUTEX_SYMBOL) && (LA187_106 <= OFFSET_SYMBOL)) || ((LA187_106 >= ONE_SHOT_SYMBOL) && (LA187_106 <= PARTITIONS_SYMBOL)) || ((LA187_106 >= PHASE_SYMBOL) && (LA187_106 <= PLUGINS_SYMBOL)) || ((LA187_106 >= PRESERVE_SYMBOL) && (LA187_106 <= PROXY_SYMBOL)) || ((LA187_106 >= QUERY_SYMBOL) && (LA187_106 <= RETURNS_SYMBOL)) || ((LA187_106 >= ROLLUP_SYMBOL) && (LA187_106 <= ROWS_SYMBOL)) || LA187_106 == ROW_FORMAT_SYMBOL || ((LA187_106 >= RTREE_SYMBOL) && (LA187_106 <= SCHEMA_NAME_SYMBOL)) || ((LA187_106 >= SERIAL_SYMBOL) && (LA187_106 <= SHARE_SYMBOL)) || ((LA187_106 >= SLOW_SYMBOL) && (LA187_106 <= SNAPSHOT_SYMBOL)) || ((LA187_106 >= SOURCE_SYMBOL) && (LA187_106 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA187_106 >= SQL_THREAD_SYMBOL) && (LA187_106 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA187_106 >= SUBJECT_SYMBOL) && (LA187_106 <= TRIGGERS_SYMBOL)) || ((LA187_106 >= TYPES_SYMBOL) && (LA187_106 <= USER_RESOURCES_SYMBOL)) || ((LA187_106 >= USE_FRM_SYMBOL) && (LA187_106 <= WAIT_SYMBOL)) || LA187_106 == WORK_SYMBOL || ((LA187_106 >= X509_SYMBOL) && (LA187_106 <= XML_SYMBOL))))
                		    {
                		        alt187=1;
                		    }

                		}
                	}
                    break;
                case MAX_STATEMENT_TIME_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case EQUAL_OPERATOR:
                			{
                				{
                				   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                				    */
                				    int LA187_541 = LA(3);
                				    if ( ((SERVER_VERSION >= 50704 && SERVER_VERSION < 50708)))
                				    {
                				        alt187=1;
                				    }

                				}
                			}
                		    break;

                		}

                	}
                    break;

                }

                switch (alt187)
                {
            	case 1:
            	    // MySQLSimpleParser.g:544:46: select_option
            	    {
            	        FOLLOWPUSH(FOLLOW_select_option_in_create_select2885);
            	        select_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_selectEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop187;	/* break out of the loop */
            	    break;
                }
            }
            loop187: ; /* Jump out to here if this rule does not match */


            FOLLOWPUSH(FOLLOW_select_item_list_in_create_select2889);
            select_item_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_selectEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_expression_in_create_select2891);
            table_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_selectEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_selectEx; /* Prevent compiler warnings */
    rulecreate_selectEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_select */

/**
 * $ANTLR start create_tablespace
 * MySQLSimpleParser.g:547:1: create_tablespace : CREATE_SYMBOL create_tablespace_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_tablespace(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:547:18: ( CREATE_SYMBOL create_tablespace_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:548:2: CREATE_SYMBOL create_tablespace_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_tablespace2901);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespaceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_tablespace_tail_in_create_tablespace2903);
            create_tablespace_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespaceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:548:39: ( SEMICOLON_SYMBOL )?
            {
                int alt188=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt188=1;
                    	}
                        break;
                }

                switch (alt188)
                {
            	case 1:
            	    // MySQLSimpleParser.g:548:39: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_tablespace2905);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespaceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_tablespace2908);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespaceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_tablespaceEx; /* Prevent compiler warnings */
    rulecreate_tablespaceEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_tablespace */

/**
 * $ANTLR start create_tablespace_tail
 * MySQLSimpleParser.g:551:1: create_tablespace_tail : TABLESPACE_SYMBOL tablespace_name ADD_SYMBOL DATAFILE_SYMBOL string_literal ( USE_SYMBOL LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref )? ( tablespace_options )? ;
 */
static void
create_tablespace_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:551:23: ( TABLESPACE_SYMBOL tablespace_name ADD_SYMBOL DATAFILE_SYMBOL string_literal ( USE_SYMBOL LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref )? ( tablespace_options )? )
        // MySQLSimpleParser.g:552:2: TABLESPACE_SYMBOL tablespace_name ADD_SYMBOL DATAFILE_SYMBOL string_literal ( USE_SYMBOL LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref )? ( tablespace_options )?
        {
             MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_create_tablespace_tail2917);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespace_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_tablespace_name_in_create_tablespace_tail2919);
            tablespace_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespace_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ADD_SYMBOL, &FOLLOW_ADD_SYMBOL_in_create_tablespace_tail2921);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespace_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(DATAFILE_SYMBOL, &FOLLOW_DATAFILE_SYMBOL_in_create_tablespace_tail2923);
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespace_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_string_literal_in_create_tablespace_tail2925);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_tablespace_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:553:3: ( USE_SYMBOL LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref )?
            {
                int alt189=2;
                switch ( LA(1) )
                {
                    case USE_SYMBOL:
                    	{
                    		alt189=1;
                    	}
                        break;
                }

                switch (alt189)
                {
            	case 1:
            	    // MySQLSimpleParser.g:553:4: USE_SYMBOL LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref
            	    {
            	         MATCHT(USE_SYMBOL, &FOLLOW_USE_SYMBOL_in_create_tablespace_tail2930);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespace_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(LOGFILE_SYMBOL, &FOLLOW_LOGFILE_SYMBOL_in_create_tablespace_tail2932);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespace_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_create_tablespace_tail2934);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespace_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_logfile_group_ref_in_create_tablespace_tail2936);
            	        logfile_group_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespace_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:553:63: ( tablespace_options )?
            {
                int alt190=2;
                switch ( LA(1) )
                {
                    case COMMENT_SYMBOL:
                    case AUTOEXTEND_SIZE_SYMBOL:
                    case ENGINE_SYMBOL:
                    case EXTENT_SIZE_SYMBOL:
                    case FILE_BLOCK_SIZE_SYMBOL:
                    case INITIAL_SIZE_SYMBOL:
                    case MAX_SIZE_SYMBOL:
                    case NO_WAIT_SYMBOL:
                    case NODEGROUP_SYMBOL:
                    case STORAGE_SYMBOL:
                    case WAIT_SYMBOL:
                    	{
                    		alt190=1;
                    	}
                        break;
                }

                switch (alt190)
                {
            	case 1:
            	    // MySQLSimpleParser.g:553:63: tablespace_options
            	    {
            	        FOLLOWPUSH(FOLLOW_tablespace_options_in_create_tablespace_tail2940);
            	        tablespace_options(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_tablespace_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_tablespace_tailEx; /* Prevent compiler warnings */
    rulecreate_tablespace_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_tablespace_tail */

/**
 * $ANTLR start tablespace_options
 * MySQLSimpleParser.g:556:1: tablespace_options : ( tablespace_option ( ( COMMA_SYMBOL )? tablespace_option )* ) ;
 */
static void
tablespace_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:556:19: ( ( tablespace_option ( ( COMMA_SYMBOL )? tablespace_option )* ) )
        // MySQLSimpleParser.g:557:2: ( tablespace_option ( ( COMMA_SYMBOL )? tablespace_option )* )
        {
            // MySQLSimpleParser.g:557:2: ( tablespace_option ( ( COMMA_SYMBOL )? tablespace_option )* )
            // MySQLSimpleParser.g:557:3: tablespace_option ( ( COMMA_SYMBOL )? tablespace_option )*
            {
                FOLLOWPUSH(FOLLOW_tablespace_option_in_tablespace_options2951);
                tablespace_option(ctx);

                FOLLOWPOP();
                if  (HASEXCEPTION())
                {
                    goto ruletablespace_optionsEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


                // MySQLSimpleParser.g:557:21: ( ( COMMA_SYMBOL )? tablespace_option )*

                for (;;)
                {
                    int alt192=2;
                    switch ( LA(1) )
                    {
                    case COMMENT_SYMBOL:
                    case AUTOEXTEND_SIZE_SYMBOL:
                    case ENGINE_SYMBOL:
                    case EXTENT_SIZE_SYMBOL:
                    case FILE_BLOCK_SIZE_SYMBOL:
                    case INITIAL_SIZE_SYMBOL:
                    case MAX_SIZE_SYMBOL:
                    case NO_WAIT_SYMBOL:
                    case NODEGROUP_SYMBOL:
                    case STORAGE_SYMBOL:
                    case WAIT_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt192=1;
                    	}
                        break;

                    }

                    switch (alt192)
                    {
                	case 1:
                	    // MySQLSimpleParser.g:557:22: ( COMMA_SYMBOL )? tablespace_option
                	    {
                	        // MySQLSimpleParser.g:557:22: ( COMMA_SYMBOL )?
                	        {
                	            int alt191=2;
                	            switch ( LA(1) )
                	            {
                	                case COMMA_SYMBOL:
                	                	{
                	                		alt191=1;
                	                	}
                	                    break;
                	            }

                	            switch (alt191)
                	            {
                	        	case 1:
                	        	    // MySQLSimpleParser.g:557:22: COMMA_SYMBOL
                	        	    {
                	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_tablespace_options2954);
                	        	        if  (HASEXCEPTION())
                	        	        {
                	        	            goto ruletablespace_optionsEx;
                	        	        }
                	        	        if (HASFAILED())
                	        	        {
                	        	            return ;
                	        	        }


                	        	    }
                	        	    break;

                	            }
                	        }

                	        FOLLOWPUSH(FOLLOW_tablespace_option_in_tablespace_options2957);
                	        tablespace_option(ctx);

                	        FOLLOWPOP();
                	        if  (HASEXCEPTION())
                	        {
                	            goto ruletablespace_optionsEx;
                	        }
                	        if (HASFAILED())
                	        {
                	            return ;
                	        }


                	    }
                	    break;

                	default:
                	    goto loop192;	/* break out of the loop */
                	    break;
                    }
                }
                loop192: ; /* Jump out to here if this rule does not match */


            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletablespace_optionsEx; /* Prevent compiler warnings */
    ruletablespace_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end tablespace_options */

/**
 * $ANTLR start tablespace_option
 * MySQLSimpleParser.g:560:1: tablespace_option : ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | EXTENT_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal |{...}? FILE_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number );
 */
static void
tablespace_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:560:18: ( INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | EXTENT_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal |{...}? FILE_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number )

            ANTLR3_UINT32 alt202;

            alt202=9;

            switch ( LA(1) )
            {
            case INITIAL_SIZE_SYMBOL:
            	{
            		alt202=1;
            	}
                break;
            case AUTOEXTEND_SIZE_SYMBOL:
            	{
            		alt202=2;
            	}
                break;
            case MAX_SIZE_SYMBOL:
            	{
            		alt202=3;
            	}
                break;
            case EXTENT_SIZE_SYMBOL:
            	{
            		alt202=4;
            	}
                break;
            case NODEGROUP_SYMBOL:
            	{
            		alt202=5;
            	}
                break;
            case ENGINE_SYMBOL:
            case STORAGE_SYMBOL:
            	{
            		alt202=6;
            	}
                break;
            case NO_WAIT_SYMBOL:
            case WAIT_SYMBOL:
            	{
            		alt202=7;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt202=8;
            	}
                break;
            case FILE_BLOCK_SIZE_SYMBOL:
            	{
            		alt202=9;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 202;
                EXCEPTION->state        = 0;


                goto ruletablespace_optionEx;

            }

            switch (alt202)
            {
        	case 1:
        	    // MySQLSimpleParser.g:561:2: INITIAL_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(INITIAL_SIZE_SYMBOL, &FOLLOW_INITIAL_SIZE_SYMBOL_in_tablespace_option2969);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:561:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt193=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt193=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt193)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:561:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option2971);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_tablespace_option2974);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:562:4: AUTOEXTEND_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(AUTOEXTEND_SIZE_SYMBOL, &FOLLOW_AUTOEXTEND_SIZE_SYMBOL_in_tablespace_option2979);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:562:27: ( EQUAL_OPERATOR )?
        	        {
        	            int alt194=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt194=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt194)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:562:27: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option2981);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_tablespace_option2984);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:563:4: MAX_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(MAX_SIZE_SYMBOL, &FOLLOW_MAX_SIZE_SYMBOL_in_tablespace_option2989);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:563:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt195=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt195=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt195)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:563:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option2991);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_tablespace_option2994);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:564:4: EXTENT_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	         MATCHT(EXTENT_SIZE_SYMBOL, &FOLLOW_EXTENT_SIZE_SYMBOL_in_tablespace_option2999);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:564:23: ( EQUAL_OPERATOR )?
        	        {
        	            int alt196=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt196=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt196)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:564:23: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option3001);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_tablespace_option3004);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:565:4: NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number
        	    {
        	         MATCHT(NODEGROUP_SYMBOL, &FOLLOW_NODEGROUP_SYMBOL_in_tablespace_option3009);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:565:21: ( EQUAL_OPERATOR )?
        	        {
        	            int alt197=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt197=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt197)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:565:21: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option3011);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_tablespace_option3014);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:566:4: ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:566:4: ( STORAGE_SYMBOL )?
        	        {
        	            int alt198=2;
        	            switch ( LA(1) )
        	            {
        	                case STORAGE_SYMBOL:
        	                	{
        	                		alt198=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt198)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:566:4: STORAGE_SYMBOL
        	        	    {
        	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_tablespace_option3019);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_tablespace_option3022);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:566:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt199=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt199=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt199)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:566:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option3024);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_tablespace_option3027);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:567:4: ( WAIT_SYMBOL | NO_WAIT_SYMBOL )
        	    {
        	        if ( LA(1) == NO_WAIT_SYMBOL || LA(1) == WAIT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruletablespace_optionEx;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:568:4: COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_tablespace_option3043);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:568:19: ( EQUAL_OPERATOR )?
        	        {
        	            int alt200=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt200=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt200)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:568:19: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option3045);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_string_literal_in_tablespace_option3048);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:569:4: {...}? FILE_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? size_number
        	    {
        	        if ( !((SERVER_VERSION >= 50707)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50707";
        	                EXCEPTION->ruleName	 = (void *)"tablespace_option";


        	        }

        	         MATCHT(FILE_BLOCK_SIZE_SYMBOL, &FOLLOW_FILE_BLOCK_SIZE_SYMBOL_in_tablespace_option3055);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:569:54: ( EQUAL_OPERATOR )?
        	        {
        	            int alt201=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt201=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt201)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:569:54: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_tablespace_option3057);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletablespace_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_size_number_in_tablespace_option3060);
        	        size_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletablespace_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletablespace_optionEx; /* Prevent compiler warnings */
    ruletablespace_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end tablespace_option */

/**
 * $ANTLR start create_trigger
 * MySQLSimpleParser.g:572:1: create_trigger : create_with_definer create_trigger_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_trigger(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:572:15: ( create_with_definer create_trigger_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:573:2: create_with_definer create_trigger_tail ( SEMICOLON_SYMBOL )? EOF
        {
            FOLLOWPUSH(FOLLOW_create_with_definer_in_create_trigger3070);
            create_with_definer(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_triggerEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_trigger_tail_in_create_trigger3072);
            create_trigger_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_triggerEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:573:42: ( SEMICOLON_SYMBOL )?
            {
                int alt203=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt203=1;
                    	}
                        break;
                }

                switch (alt203)
                {
            	case 1:
            	    // MySQLSimpleParser.g:573:42: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_trigger3074);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_triggerEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_trigger3077);
            if  (HASEXCEPTION())
            {
                goto rulecreate_triggerEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_triggerEx; /* Prevent compiler warnings */
    rulecreate_triggerEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_trigger */

/**
 * $ANTLR start create_trigger_tail
 * MySQLSimpleParser.g:576:1: create_trigger_tail : TRIGGER_SYMBOL trigger_name ( BEFORE_SYMBOL | AFTER_SYMBOL ) ( INSERT_SYMBOL | UPDATE_SYMBOL | DELETE_SYMBOL ) ON_SYMBOL table_ref FOR_SYMBOL EACH_SYMBOL ROW_SYMBOL ( trigger_follows_precedes_clause )? compound_statement ;
 */
static void
create_trigger_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:576:20: ( TRIGGER_SYMBOL trigger_name ( BEFORE_SYMBOL | AFTER_SYMBOL ) ( INSERT_SYMBOL | UPDATE_SYMBOL | DELETE_SYMBOL ) ON_SYMBOL table_ref FOR_SYMBOL EACH_SYMBOL ROW_SYMBOL ( trigger_follows_precedes_clause )? compound_statement )
        // MySQLSimpleParser.g:577:2: TRIGGER_SYMBOL trigger_name ( BEFORE_SYMBOL | AFTER_SYMBOL ) ( INSERT_SYMBOL | UPDATE_SYMBOL | DELETE_SYMBOL ) ON_SYMBOL table_ref FOR_SYMBOL EACH_SYMBOL ROW_SYMBOL ( trigger_follows_precedes_clause )? compound_statement
        {
             MATCHT(TRIGGER_SYMBOL, &FOLLOW_TRIGGER_SYMBOL_in_create_trigger_tail3086);
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_trigger_name_in_create_trigger_tail3088);
            trigger_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == AFTER_SYMBOL || LA(1) == BEFORE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulecreate_trigger_tailEx;
            }


            if ( LA(1) == DELETE_SYMBOL || LA(1) == INSERT_SYMBOL || LA(1) == UPDATE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulecreate_trigger_tailEx;
            }


             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_create_trigger_tail3112);
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_create_trigger_tail3114);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_create_trigger_tail3116);
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EACH_SYMBOL, &FOLLOW_EACH_SYMBOL_in_create_trigger_tail3118);
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ROW_SYMBOL, &FOLLOW_ROW_SYMBOL_in_create_trigger_tail3120);
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:578:57: ( trigger_follows_precedes_clause )?
            {
                int alt204=2;
                switch ( LA(1) )
                {
                    case FOLLOWS_SYMBOL:
                    case PRECEDES_SYMBOL:
                    	{
                    		alt204=1;
                    	}
                        break;
                }

                switch (alt204)
                {
            	case 1:
            	    // MySQLSimpleParser.g:578:57: trigger_follows_precedes_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_trigger_follows_precedes_clause_in_create_trigger_tail3122);
            	        trigger_follows_precedes_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_trigger_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_compound_statement_in_create_trigger_tail3125);
            compound_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_trigger_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_trigger_tailEx; /* Prevent compiler warnings */
    rulecreate_trigger_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_trigger_tail */

/**
 * $ANTLR start trigger_follows_precedes_clause
 * MySQLSimpleParser.g:581:1: trigger_follows_precedes_clause :{...}? ( FOLLOWS_SYMBOL | PRECEDES_SYMBOL ) text_or_identifier ;
 */
static void
trigger_follows_precedes_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:581:32: ({...}? ( FOLLOWS_SYMBOL | PRECEDES_SYMBOL ) text_or_identifier )
        // MySQLSimpleParser.g:582:2: {...}? ( FOLLOWS_SYMBOL | PRECEDES_SYMBOL ) text_or_identifier
        {
            if ( !((SERVER_VERSION >= 50700)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
                    EXCEPTION->ruleName	 = (void *)"trigger_follows_precedes_clause";


            }

            if ( LA(1) == FOLLOWS_SYMBOL || LA(1) == PRECEDES_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruletrigger_follows_precedes_clauseEx;
            }


            FOLLOWPUSH(FOLLOW_text_or_identifier_in_trigger_follows_precedes_clause3144);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletrigger_follows_precedes_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletrigger_follows_precedes_clauseEx; /* Prevent compiler warnings */
    ruletrigger_follows_precedes_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end trigger_follows_precedes_clause */

/**
 * $ANTLR start create_view
 * MySQLSimpleParser.g:585:1: create_view : CREATE_SYMBOL ( OR_SYMBOL REPLACE_SYMBOL )? ( view_algorithm )? ( definer_clause )? create_view_tail ( SEMICOLON_SYMBOL )? EOF ;
 */
static void
create_view(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:585:12: ( CREATE_SYMBOL ( OR_SYMBOL REPLACE_SYMBOL )? ( view_algorithm )? ( definer_clause )? create_view_tail ( SEMICOLON_SYMBOL )? EOF )
        // MySQLSimpleParser.g:586:2: CREATE_SYMBOL ( OR_SYMBOL REPLACE_SYMBOL )? ( view_algorithm )? ( definer_clause )? create_view_tail ( SEMICOLON_SYMBOL )? EOF
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_view3156);
            if  (HASEXCEPTION())
            {
                goto rulecreate_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:586:16: ( OR_SYMBOL REPLACE_SYMBOL )?
            {
                int alt205=2;
                switch ( LA(1) )
                {
                    case OR_SYMBOL:
                    	{
                    		alt205=1;
                    	}
                        break;
                }

                switch (alt205)
                {
            	case 1:
            	    // MySQLSimpleParser.g:586:17: OR_SYMBOL REPLACE_SYMBOL
            	    {
            	         MATCHT(OR_SYMBOL, &FOLLOW_OR_SYMBOL_in_create_view3159);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(REPLACE_SYMBOL, &FOLLOW_REPLACE_SYMBOL_in_create_view3161);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:586:44: ( view_algorithm )?
            {
                int alt206=2;
                switch ( LA(1) )
                {
                    case ALGORITHM_SYMBOL:
                    	{
                    		alt206=1;
                    	}
                        break;
                }

                switch (alt206)
                {
            	case 1:
            	    // MySQLSimpleParser.g:586:44: view_algorithm
            	    {
            	        FOLLOWPUSH(FOLLOW_view_algorithm_in_create_view3165);
            	        view_algorithm(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:586:61: ( definer_clause )?
            {
                int alt207=2;
                switch ( LA(1) )
                {
                    case DEFINER_SYMBOL:
                    	{
                    		alt207=1;
                    	}
                        break;
                }

                switch (alt207)
                {
            	case 1:
            	    // MySQLSimpleParser.g:586:61: definer_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_definer_clause_in_create_view3169);
            	        definer_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_create_view_tail_in_create_view3172);
            create_view_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:586:94: ( SEMICOLON_SYMBOL )?
            {
                int alt208=2;
                switch ( LA(1) )
                {
                    case SEMICOLON_SYMBOL:
                    	{
                    		alt208=1;
                    	}
                        break;
                }

                switch (alt208)
                {
            	case 1:
            	    // MySQLSimpleParser.g:586:94: SEMICOLON_SYMBOL
            	    {
            	         MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_create_view3174);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_viewEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EOF, &FOLLOW_EOF_in_create_view3177);
            if  (HASEXCEPTION())
            {
                goto rulecreate_viewEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_viewEx; /* Prevent compiler warnings */
    rulecreate_viewEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_view */

/**
 * $ANTLR start create_view_tail
 * MySQLSimpleParser.g:589:1: create_view_tail : ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_name ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )? ;
 */
static void
create_view_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:589:17: ( ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_name ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )? )
        // MySQLSimpleParser.g:590:2: ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )? VIEW_SYMBOL view_name ( identifier_list_with_parentheses )? AS_SYMBOL select_statement ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )?
        {
            // MySQLSimpleParser.g:590:2: ( SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL ) )?
            {
                int alt209=2;
                switch ( LA(1) )
                {
                    case SQL_SYMBOL:
                    	{
                    		alt209=1;
                    	}
                        break;
                }

                switch (alt209)
                {
            	case 1:
            	    // MySQLSimpleParser.g:590:3: SQL_SYMBOL SECURITY_SYMBOL ( DEFINER_SYMBOL | INVOKER_SYMBOL )
            	    {
            	         MATCHT(SQL_SYMBOL, &FOLLOW_SQL_SYMBOL_in_create_view_tail3187);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(SECURITY_SYMBOL, &FOLLOW_SECURITY_SYMBOL_in_create_view_tail3189);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == DEFINER_SYMBOL || LA(1) == INVOKER_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulecreate_view_tailEx;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_create_view_tail3203);
            if  (HASEXCEPTION())
            {
                goto rulecreate_view_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_view_name_in_create_view_tail3205);
            view_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_view_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:591:25: ( identifier_list_with_parentheses )?
            {
                int alt210=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt210=1;
                    	}
                        break;
                }

                switch (alt210)
                {
            	case 1:
            	    // MySQLSimpleParser.g:591:25: identifier_list_with_parentheses
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_create_view_tail3207);
            	        identifier_list_with_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_create_view_tail3212);
            if  (HASEXCEPTION())
            {
                goto rulecreate_view_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_select_statement_in_create_view_tail3214);
            select_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_view_tailEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:593:3: ( WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL )?
            {
                int alt212=2;
                switch ( LA(1) )
                {
                    case WITH_SYMBOL:
                    	{
                    		alt212=1;
                    	}
                        break;
                }

                switch (alt212)
                {
            	case 1:
            	    // MySQLSimpleParser.g:593:4: WITH_SYMBOL ( CASCADED_SYMBOL | LOCAL_SYMBOL )? CHECK_SYMBOL OPTION_SYMBOL
            	    {
            	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_create_view_tail3219);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:593:16: ( CASCADED_SYMBOL | LOCAL_SYMBOL )?
            	        {
            	            int alt211=2;
            	            switch ( LA(1) )
            	            {
            	                case CASCADED_SYMBOL:
            	                case LOCAL_SYMBOL:
            	                	{
            	                		alt211=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt211)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == CASCADED_SYMBOL || LA(1) == LOCAL_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto rulecreate_view_tailEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_create_view_tail3230);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_create_view_tail3232);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_view_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_view_tailEx; /* Prevent compiler warnings */
    rulecreate_view_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_view_tail */

/**
 * $ANTLR start view_replace_or_algorithm
 * MySQLSimpleParser.g:596:1: view_replace_or_algorithm : ( OR_SYMBOL REPLACE_SYMBOL ( view_algorithm )? | view_algorithm );
 */
static void
view_replace_or_algorithm(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:596:26: ( OR_SYMBOL REPLACE_SYMBOL ( view_algorithm )? | view_algorithm )

            ANTLR3_UINT32 alt214;

            alt214=2;

            switch ( LA(1) )
            {
            case OR_SYMBOL:
            	{
            		alt214=1;
            	}
                break;
            case ALGORITHM_SYMBOL:
            	{
            		alt214=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 214;
                EXCEPTION->state        = 0;


                goto ruleview_replace_or_algorithmEx;

            }

            switch (alt214)
            {
        	case 1:
        	    // MySQLSimpleParser.g:597:2: OR_SYMBOL REPLACE_SYMBOL ( view_algorithm )?
        	    {
        	         MATCHT(OR_SYMBOL, &FOLLOW_OR_SYMBOL_in_view_replace_or_algorithm3243);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleview_replace_or_algorithmEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(REPLACE_SYMBOL, &FOLLOW_REPLACE_SYMBOL_in_view_replace_or_algorithm3245);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleview_replace_or_algorithmEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:597:27: ( view_algorithm )?
        	        {
        	            int alt213=2;
        	            switch ( LA(1) )
        	            {
        	                case ALGORITHM_SYMBOL:
        	                	{
        	                		alt213=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt213)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:597:27: view_algorithm
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_view_algorithm_in_view_replace_or_algorithm3247);
        	        	        view_algorithm(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleview_replace_or_algorithmEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:598:4: view_algorithm
        	    {
        	        FOLLOWPUSH(FOLLOW_view_algorithm_in_view_replace_or_algorithm3253);
        	        view_algorithm(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleview_replace_or_algorithmEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleview_replace_or_algorithmEx; /* Prevent compiler warnings */
    ruleview_replace_or_algorithmEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end view_replace_or_algorithm */

/**
 * $ANTLR start view_algorithm
 * MySQLSimpleParser.g:601:1: view_algorithm : ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) ;
 */
static void
view_algorithm(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:601:15: ( ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL ) )
        // MySQLSimpleParser.g:602:2: ALGORITHM_SYMBOL EQUAL_OPERATOR ( UNDEFINED_SYMBOL | MERGE_SYMBOL | TEMPTABLE_SYMBOL )
        {
             MATCHT(ALGORITHM_SYMBOL, &FOLLOW_ALGORITHM_SYMBOL_in_view_algorithm3262);
            if  (HASEXCEPTION())
            {
                goto ruleview_algorithmEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_view_algorithm3264);
            if  (HASEXCEPTION())
            {
                goto ruleview_algorithmEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == MERGE_SYMBOL || LA(1) == TEMPTABLE_SYMBOL || LA(1) == UNDEFINED_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleview_algorithmEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleview_algorithmEx; /* Prevent compiler warnings */
    ruleview_algorithmEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end view_algorithm */

/**
 * $ANTLR start drop_statement
 * MySQLSimpleParser.g:607:1: drop_statement : DROP_SYMBOL ( DATABASE_SYMBOL ( if_exists )? schema_ref |{...}? EVENT_SYMBOL ( if_exists )? event_ref | ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref ) | ( online_option )? INDEX_SYMBOL index_ref ON_SYMBOL table_ref ( index_lock_algorithm )? | LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | SERVER_SYMBOL ( if_exists )? server_ref | ( TEMPORARY_SYMBOL )? ( TABLE_SYMBOL | TABLES_SYMBOL ) ( if_exists )? table_ref_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? | TABLESPACE_SYMBOL tablespace_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | TRIGGER_SYMBOL ( if_exists )? trigger_ref | VIEW_SYMBOL ( if_exists )? view_identifier_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? ) ;
 */
static void
drop_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:607:15: ( DROP_SYMBOL ( DATABASE_SYMBOL ( if_exists )? schema_ref |{...}? EVENT_SYMBOL ( if_exists )? event_ref | ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref ) | ( online_option )? INDEX_SYMBOL index_ref ON_SYMBOL table_ref ( index_lock_algorithm )? | LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | SERVER_SYMBOL ( if_exists )? server_ref | ( TEMPORARY_SYMBOL )? ( TABLE_SYMBOL | TABLES_SYMBOL ) ( if_exists )? table_ref_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? | TABLESPACE_SYMBOL tablespace_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | TRIGGER_SYMBOL ( if_exists )? trigger_ref | VIEW_SYMBOL ( if_exists )? view_identifier_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? ) )
        // MySQLSimpleParser.g:608:2: DROP_SYMBOL ( DATABASE_SYMBOL ( if_exists )? schema_ref |{...}? EVENT_SYMBOL ( if_exists )? event_ref | ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref ) | ( online_option )? INDEX_SYMBOL index_ref ON_SYMBOL table_ref ( index_lock_algorithm )? | LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | SERVER_SYMBOL ( if_exists )? server_ref | ( TEMPORARY_SYMBOL )? ( TABLE_SYMBOL | TABLES_SYMBOL ) ( if_exists )? table_ref_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? | TABLESPACE_SYMBOL tablespace_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | TRIGGER_SYMBOL ( if_exists )? trigger_ref | VIEW_SYMBOL ( if_exists )? view_identifier_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? )
        {
             MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_drop_statement3287);
            if  (HASEXCEPTION())
            {
                goto ruledrop_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:609:2: ( DATABASE_SYMBOL ( if_exists )? schema_ref |{...}? EVENT_SYMBOL ( if_exists )? event_ref | ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref ) | ( online_option )? INDEX_SYMBOL index_ref ON_SYMBOL table_ref ( index_lock_algorithm )? | LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | SERVER_SYMBOL ( if_exists )? server_ref | ( TEMPORARY_SYMBOL )? ( TABLE_SYMBOL | TABLES_SYMBOL ) ( if_exists )? table_ref_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? | TABLESPACE_SYMBOL tablespace_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )? | TRIGGER_SYMBOL ( if_exists )? trigger_ref | VIEW_SYMBOL ( if_exists )? view_identifier_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )? )
            {
                int alt235=10;
                switch ( LA(1) )
                {
                case DATABASE_SYMBOL:
                	{
                		alt235=1;
                	}
                    break;
                case EVENT_SYMBOL:
                	{
                		alt235=2;
                	}
                    break;
                case FUNCTION_SYMBOL:
                case PROCEDURE_SYMBOL:
                	{
                		alt235=3;
                	}
                    break;
                case INDEX_SYMBOL:
                case OFFLINE_SYMBOL:
                case ONLINE_SYMBOL:
                	{
                		alt235=4;
                	}
                    break;
                case LOGFILE_SYMBOL:
                	{
                		alt235=5;
                	}
                    break;
                case SERVER_SYMBOL:
                	{
                		alt235=6;
                	}
                    break;
                case TABLES_SYMBOL:
                case TEMPORARY_SYMBOL:
                case TABLE_SYMBOL:
                	{
                		alt235=7;
                	}
                    break;
                case TABLESPACE_SYMBOL:
                	{
                		alt235=8;
                	}
                    break;
                case TRIGGER_SYMBOL:
                	{
                		alt235=9;
                	}
                    break;
                case VIEW_SYMBOL:
                	{
                		alt235=10;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 235;
                    EXCEPTION->state        = 0;


                    goto ruledrop_statementEx;

                }

                switch (alt235)
                {
            	case 1:
            	    // MySQLSimpleParser.g:610:3: DATABASE_SYMBOL ( if_exists )? schema_ref
            	    {
            	         MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_drop_statement3294);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:610:19: ( if_exists )?
            	        {
            	            int alt215=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt215=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt215)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:610:19: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3296);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_schema_ref_in_drop_statement3299);
            	        schema_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:611:5: {...}? EVENT_SYMBOL ( if_exists )? event_ref
            	    {
            	        if ( !((SERVER_VERSION >= 50100)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
            	                EXCEPTION->ruleName	 = (void *)"drop_statement";


            	        }

            	         MATCHT(EVENT_SYMBOL, &FOLLOW_EVENT_SYMBOL_in_drop_statement3307);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:611:45: ( if_exists )?
            	        {
            	            int alt216=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt216=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt216)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:611:45: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3309);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_event_ref_in_drop_statement3312);
            	        event_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:613:4: ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref )
            	    {
            	        // MySQLSimpleParser.g:613:4: ( FUNCTION_SYMBOL ( if_exists )? function_ref | PROCEDURE_SYMBOL ( if_exists )? procedure_ref )
            	        {
            	            int alt219=2;
            	            switch ( LA(1) )
            	            {
            	            case FUNCTION_SYMBOL:
            	            	{
            	            		alt219=1;
            	            	}
            	                break;
            	            case PROCEDURE_SYMBOL:
            	            	{
            	            		alt219=2;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 219;
            	                EXCEPTION->state        = 0;


            	                goto ruledrop_statementEx;

            	            }

            	            switch (alt219)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:614:5: FUNCTION_SYMBOL ( if_exists )? function_ref
            	        	    {
            	        	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_drop_statement3327);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:614:21: ( if_exists )?
            	        	        {
            	        	            int alt217=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case IF_SYMBOL:
            	        	                	{
            	        	                		alt217=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt217)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:614:21: if_exists
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3329);
            	        	        	        if_exists(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledrop_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_function_ref_in_drop_statement3332);
            	        	        function_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:615:7: PROCEDURE_SYMBOL ( if_exists )? procedure_ref
            	        	    {
            	        	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_drop_statement3341);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:615:24: ( if_exists )?
            	        	        {
            	        	            int alt218=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case IF_SYMBOL:
            	        	                	{
            	        	                		alt218=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt218)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:615:24: if_exists
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3343);
            	        	        	        if_exists(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledrop_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_procedure_ref_in_drop_statement3346);
            	        	        procedure_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:617:5: ( online_option )? INDEX_SYMBOL index_ref ON_SYMBOL table_ref ( index_lock_algorithm )?
            	    {
            	        // MySQLSimpleParser.g:617:5: ( online_option )?
            	        {
            	            int alt220=2;
            	            switch ( LA(1) )
            	            {
            	                case OFFLINE_SYMBOL:
            	                case ONLINE_SYMBOL:
            	                	{
            	                		alt220=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt220)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:617:5: online_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_online_option_in_drop_statement3357);
            	        	        online_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_drop_statement3360);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_ref_in_drop_statement3362);
            	        index_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_drop_statement3364);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_drop_statement3366);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:617:63: ( index_lock_algorithm )?
            	        {
            	            int alt221=2;
            	            {
            	                int LA221_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50600)) && (LA221_0 == ALGORITHM_SYMBOL || LA221_0 == LOCK_SYMBOL))
            	                {
            	                    alt221=1;
            	                }
            	            }
            	            switch (alt221)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:617:63: index_lock_algorithm
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_index_lock_algorithm_in_drop_statement3368);
            	        	        index_lock_algorithm(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:618:5: LOGFILE_SYMBOL GROUP_SYMBOL logfile_group_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )?
            	    {
            	         MATCHT(LOGFILE_SYMBOL, &FOLLOW_LOGFILE_SYMBOL_in_drop_statement3375);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_drop_statement3377);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_logfile_group_ref_in_drop_statement3379);
            	        logfile_group_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:618:51: ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )?
            	        {
            	            int alt224=2;
            	            switch ( LA(1) )
            	            {
            	                case ENGINE_SYMBOL:
            	                case NO_WAIT_SYMBOL:
            	                case STORAGE_SYMBOL:
            	                case WAIT_SYMBOL:
            	                	{
            	                		alt224=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt224)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:618:52: drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )*
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_drop_logfile_group_option_in_drop_statement3382);
            	        	        drop_logfile_group_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:618:78: ( ( COMMA_SYMBOL )? drop_logfile_group_option )*

            	        	        for (;;)
            	        	        {
            	        	            int alt223=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case ENGINE_SYMBOL:
            	        	            case NO_WAIT_SYMBOL:
            	        	            case STORAGE_SYMBOL:
            	        	            case WAIT_SYMBOL:
            	        	            case COMMA_SYMBOL:
            	        	            	{
            	        	            		alt223=1;
            	        	            	}
            	        	                break;

            	        	            }

            	        	            switch (alt223)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:618:79: ( COMMA_SYMBOL )? drop_logfile_group_option
            	        	        	    {
            	        	        	        // MySQLSimpleParser.g:618:79: ( COMMA_SYMBOL )?
            	        	        	        {
            	        	        	            int alt222=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	                case COMMA_SYMBOL:
            	        	        	                	{
            	        	        	                		alt222=1;
            	        	        	                	}
            	        	        	                    break;
            	        	        	            }

            	        	        	            switch (alt222)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:618:79: COMMA_SYMBOL
            	        	        	        	    {
            	        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_drop_statement3385);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto ruledrop_statementEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_drop_logfile_group_option_in_drop_statement3388);
            	        	        	        drop_logfile_group_option(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledrop_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	        	default:
            	        	        	    goto loop223;	/* break out of the loop */
            	        	        	    break;
            	        	            }
            	        	        }
            	        	        loop223: ; /* Jump out to here if this rule does not match */


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:619:5: SERVER_SYMBOL ( if_exists )? server_ref
            	    {
            	         MATCHT(SERVER_SYMBOL, &FOLLOW_SERVER_SYMBOL_in_drop_statement3398);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:619:19: ( if_exists )?
            	        {
            	            int alt225=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt225=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt225)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:619:19: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3400);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_server_ref_in_drop_statement3403);
            	        server_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 7:
            	    // MySQLSimpleParser.g:620:5: ( TEMPORARY_SYMBOL )? ( TABLE_SYMBOL | TABLES_SYMBOL ) ( if_exists )? table_ref_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )?
            	    {
            	        // MySQLSimpleParser.g:620:5: ( TEMPORARY_SYMBOL )?
            	        {
            	            int alt226=2;
            	            switch ( LA(1) )
            	            {
            	                case TEMPORARY_SYMBOL:
            	                	{
            	                		alt226=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt226)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:620:5: TEMPORARY_SYMBOL
            	        	    {
            	        	         MATCHT(TEMPORARY_SYMBOL, &FOLLOW_TEMPORARY_SYMBOL_in_drop_statement3409);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        if ( LA(1) == TABLES_SYMBOL || LA(1) == TABLE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruledrop_statementEx;
            	        }


            	        // MySQLSimpleParser.g:620:54: ( if_exists )?
            	        {
            	            int alt227=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt227=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt227)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:620:54: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3420);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_table_ref_list_in_drop_statement3423);
            	        table_ref_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:620:80: ( RESTRICT_SYMBOL | CASCADE_SYMBOL )?
            	        {
            	            int alt228=2;
            	            switch ( LA(1) )
            	            {
            	                case CASCADE_SYMBOL:
            	                case RESTRICT_SYMBOL:
            	                	{
            	                		alt228=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt228)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == CASCADE_SYMBOL || LA(1) == RESTRICT_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto ruledrop_statementEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 8:
            	    // MySQLSimpleParser.g:621:5: TABLESPACE_SYMBOL tablespace_ref ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )?
            	    {
            	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_drop_statement3438);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_tablespace_ref_in_drop_statement3440);
            	        tablespace_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:621:38: ( drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )* )?
            	        {
            	            int alt231=2;
            	            switch ( LA(1) )
            	            {
            	                case ENGINE_SYMBOL:
            	                case NO_WAIT_SYMBOL:
            	                case STORAGE_SYMBOL:
            	                case WAIT_SYMBOL:
            	                	{
            	                		alt231=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt231)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:621:39: drop_logfile_group_option ( ( COMMA_SYMBOL )? drop_logfile_group_option )*
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_drop_logfile_group_option_in_drop_statement3443);
            	        	        drop_logfile_group_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:621:65: ( ( COMMA_SYMBOL )? drop_logfile_group_option )*

            	        	        for (;;)
            	        	        {
            	        	            int alt230=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case ENGINE_SYMBOL:
            	        	            case NO_WAIT_SYMBOL:
            	        	            case STORAGE_SYMBOL:
            	        	            case WAIT_SYMBOL:
            	        	            case COMMA_SYMBOL:
            	        	            	{
            	        	            		alt230=1;
            	        	            	}
            	        	                break;

            	        	            }

            	        	            switch (alt230)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:621:66: ( COMMA_SYMBOL )? drop_logfile_group_option
            	        	        	    {
            	        	        	        // MySQLSimpleParser.g:621:66: ( COMMA_SYMBOL )?
            	        	        	        {
            	        	        	            int alt229=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	                case COMMA_SYMBOL:
            	        	        	                	{
            	        	        	                		alt229=1;
            	        	        	                	}
            	        	        	                    break;
            	        	        	            }

            	        	        	            switch (alt229)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:621:66: COMMA_SYMBOL
            	        	        	        	    {
            	        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_drop_statement3446);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto ruledrop_statementEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_drop_logfile_group_option_in_drop_statement3449);
            	        	        	        drop_logfile_group_option(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledrop_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	        	default:
            	        	        	    goto loop230;	/* break out of the loop */
            	        	        	    break;
            	        	            }
            	        	        }
            	        	        loop230: ; /* Jump out to here if this rule does not match */


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 9:
            	    // MySQLSimpleParser.g:622:5: TRIGGER_SYMBOL ( if_exists )? trigger_ref
            	    {
            	         MATCHT(TRIGGER_SYMBOL, &FOLLOW_TRIGGER_SYMBOL_in_drop_statement3459);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:622:20: ( if_exists )?
            	        {
            	            int alt232=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt232=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt232)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:622:20: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3461);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_trigger_ref_in_drop_statement3464);
            	        trigger_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 10:
            	    // MySQLSimpleParser.g:623:5: VIEW_SYMBOL ( if_exists )? view_identifier_list ( RESTRICT_SYMBOL | CASCADE_SYMBOL )?
            	    {
            	         MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_drop_statement3470);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:623:17: ( if_exists )?
            	        {
            	            int alt233=2;
            	            switch ( LA(1) )
            	            {
            	                case IF_SYMBOL:
            	                	{
            	                		alt233=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt233)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:623:17: if_exists
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_statement3472);
            	        	        if_exists(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledrop_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_view_identifier_list_in_drop_statement3475);
            	        view_identifier_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:623:49: ( RESTRICT_SYMBOL | CASCADE_SYMBOL )?
            	        {
            	            int alt234=2;
            	            switch ( LA(1) )
            	            {
            	                case CASCADE_SYMBOL:
            	                case RESTRICT_SYMBOL:
            	                	{
            	                		alt234=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt234)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == CASCADE_SYMBOL || LA(1) == RESTRICT_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto ruledrop_statementEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruledrop_statementEx; /* Prevent compiler warnings */
    ruledrop_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end drop_statement */

/**
 * $ANTLR start drop_logfile_group_option
 * MySQLSimpleParser.g:627:1: drop_logfile_group_option : ( ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref );
 */
static void
drop_logfile_group_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:627:26: ( ( WAIT_SYMBOL | NO_WAIT_SYMBOL ) | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref )

            ANTLR3_UINT32 alt238;

            alt238=2;

            switch ( LA(1) )
            {
            case NO_WAIT_SYMBOL:
            case WAIT_SYMBOL:
            	{
            		alt238=1;
            	}
                break;
            case ENGINE_SYMBOL:
            case STORAGE_SYMBOL:
            	{
            		alt238=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 238;
                EXCEPTION->state        = 0;


                goto ruledrop_logfile_group_optionEx;

            }

            switch (alt238)
            {
        	case 1:
        	    // MySQLSimpleParser.g:628:2: ( WAIT_SYMBOL | NO_WAIT_SYMBOL )
        	    {
        	        if ( LA(1) == NO_WAIT_SYMBOL || LA(1) == WAIT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruledrop_logfile_group_optionEx;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:629:4: ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:629:4: ( STORAGE_SYMBOL )?
        	        {
        	            int alt236=2;
        	            switch ( LA(1) )
        	            {
        	                case STORAGE_SYMBOL:
        	                	{
        	                		alt236=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt236)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:629:4: STORAGE_SYMBOL
        	        	    {
        	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_drop_logfile_group_option3507);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledrop_logfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_drop_logfile_group_option3510);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledrop_logfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:629:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt237=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt237=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt237)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:629:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_drop_logfile_group_option3512);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledrop_logfile_group_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_drop_logfile_group_option3515);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledrop_logfile_group_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruledrop_logfile_group_optionEx; /* Prevent compiler warnings */
    ruledrop_logfile_group_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end drop_logfile_group_option */

/**
 * $ANTLR start rename_table_statement
 * MySQLSimpleParser.g:633:1: rename_table_statement : RENAME_SYMBOL ( TABLE_SYMBOL | TABLES_SYMBOL ) table_ref TO_SYMBOL table_name ( COMMA_SYMBOL table_ref TO_SYMBOL table_name )* ;
 */
static void
rename_table_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:633:23: ( RENAME_SYMBOL ( TABLE_SYMBOL | TABLES_SYMBOL ) table_ref TO_SYMBOL table_name ( COMMA_SYMBOL table_ref TO_SYMBOL table_name )* )
        // MySQLSimpleParser.g:634:2: RENAME_SYMBOL ( TABLE_SYMBOL | TABLES_SYMBOL ) table_ref TO_SYMBOL table_name ( COMMA_SYMBOL table_ref TO_SYMBOL table_name )*
        {
             MATCHT(RENAME_SYMBOL, &FOLLOW_RENAME_SYMBOL_in_rename_table_statement3525);
            if  (HASEXCEPTION())
            {
                goto rulerename_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == TABLES_SYMBOL || LA(1) == TABLE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulerename_table_statementEx;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_rename_table_statement3537);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerename_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_rename_table_statement3539);
            if  (HASEXCEPTION())
            {
                goto rulerename_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_name_in_rename_table_statement3541);
            table_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerename_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:635:34: ( COMMA_SYMBOL table_ref TO_SYMBOL table_name )*

            for (;;)
            {
                int alt239=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt239=1;
                	}
                    break;

                }

                switch (alt239)
                {
            	case 1:
            	    // MySQLSimpleParser.g:635:35: COMMA_SYMBOL table_ref TO_SYMBOL table_name
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_rename_table_statement3544);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_table_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_rename_table_statement3546);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_table_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_rename_table_statement3548);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_table_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_name_in_rename_table_statement3550);
            	        table_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_table_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop239;	/* break out of the loop */
            	    break;
                }
            }
            loop239: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulerename_table_statementEx; /* Prevent compiler warnings */
    rulerename_table_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end rename_table_statement */

/**
 * $ANTLR start truncate_table_statement
 * MySQLSimpleParser.g:640:1: truncate_table_statement : TRUNCATE_SYMBOL ( TABLE_SYMBOL )? table_ref ;
 */
static void
truncate_table_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:640:25: ( TRUNCATE_SYMBOL ( TABLE_SYMBOL )? table_ref )
        // MySQLSimpleParser.g:641:2: TRUNCATE_SYMBOL ( TABLE_SYMBOL )? table_ref
        {
             MATCHT(TRUNCATE_SYMBOL, &FOLLOW_TRUNCATE_SYMBOL_in_truncate_table_statement3564);
            if  (HASEXCEPTION())
            {
                goto ruletruncate_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:641:18: ( TABLE_SYMBOL )?
            {
                int alt240=2;
                switch ( LA(1) )
                {
                    case TABLE_SYMBOL:
                    	{
                    		alt240=1;
                    	}
                        break;
                }

                switch (alt240)
                {
            	case 1:
            	    // MySQLSimpleParser.g:641:18: TABLE_SYMBOL
            	    {
            	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_truncate_table_statement3566);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletruncate_table_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_table_ref_in_truncate_table_statement3569);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletruncate_table_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletruncate_table_statementEx; /* Prevent compiler warnings */
    ruletruncate_table_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end truncate_table_statement */

/**
 * $ANTLR start call_statement
 * MySQLSimpleParser.g:646:1: call_statement : CALL_SYMBOL procedure_ref ( optional_expression_list_with_parentheses )? ;
 */
static void
call_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:646:15: ( CALL_SYMBOL procedure_ref ( optional_expression_list_with_parentheses )? )
        // MySQLSimpleParser.g:647:2: CALL_SYMBOL procedure_ref ( optional_expression_list_with_parentheses )?
        {
             MATCHT(CALL_SYMBOL, &FOLLOW_CALL_SYMBOL_in_call_statement3580);
            if  (HASEXCEPTION())
            {
                goto rulecall_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_procedure_ref_in_call_statement3582);
            procedure_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecall_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:647:28: ( optional_expression_list_with_parentheses )?
            {
                int alt241=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt241=1;
                    	}
                        break;
                }

                switch (alt241)
                {
            	case 1:
            	    // MySQLSimpleParser.g:647:28: optional_expression_list_with_parentheses
            	    {
            	        FOLLOWPUSH(FOLLOW_optional_expression_list_with_parentheses_in_call_statement3584);
            	        optional_expression_list_with_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecall_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecall_statementEx; /* Prevent compiler warnings */
    rulecall_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end call_statement */

/**
 * $ANTLR start delete_statement
 * MySQLSimpleParser.g:650:1: delete_statement : DELETE_SYMBOL ( options {greedy=true; } : delete_option )* ( FROM_SYMBOL ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? ) | table_ref_list_with_wildcard FROM_SYMBOL join_table_list ( where_clause )? ) ;
 */
static void
delete_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:650:17: ( DELETE_SYMBOL ( options {greedy=true; } : delete_option )* ( FROM_SYMBOL ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? ) | table_ref_list_with_wildcard FROM_SYMBOL join_table_list ( where_clause )? ) )
        // MySQLSimpleParser.g:651:2: DELETE_SYMBOL ( options {greedy=true; } : delete_option )* ( FROM_SYMBOL ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? ) | table_ref_list_with_wildcard FROM_SYMBOL join_table_list ( where_clause )? )
        {
             MATCHT(DELETE_SYMBOL, &FOLLOW_DELETE_SYMBOL_in_delete_statement3594);
            if  (HASEXCEPTION())
            {
                goto ruledelete_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:651:16: ( options {greedy=true; } : delete_option )*

            for (;;)
            {
                int alt242=2;
                switch ( LA(1) )
                {
                case QUICK_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA242_5 = LA(2);
                		    if ( (LA242_5 == FROM_SYMBOL))
                		    {
                		        alt242=1;
                		    }
                		    else if ( (((LA242_5 >= ACCOUNT_SYMBOL) && (LA242_5 <= YEAR_SYMBOL)) || LA242_5 == IGNORE_SYMBOL || LA242_5 == LOW_PRIORITY_SYMBOL || LA242_5 == BACK_TICK_QUOTED_ID || LA242_5 == DOUBLE_QUOTED_TEXT || LA242_5 == IDENTIFIER))
                		    {
                		        alt242=1;
                		    }

                		}
                	}
                    break;
                case IGNORE_SYMBOL:
                case LOW_PRIORITY_SYMBOL:
                	{
                		alt242=1;
                	}
                    break;

                }

                switch (alt242)
                {
            	case 1:
            	    // MySQLSimpleParser.g:651:46: delete_option
            	    {
            	        FOLLOWPUSH(FOLLOW_delete_option_in_delete_statement3610);
            	        delete_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledelete_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop242;	/* break out of the loop */
            	    break;
                }
            }
            loop242: ; /* Jump out to here if this rule does not match */


            // MySQLSimpleParser.g:652:3: ( FROM_SYMBOL ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? ) | table_ref_list_with_wildcard FROM_SYMBOL join_table_list ( where_clause )? )
            {
                int alt250=2;
                {
                    int LA250_0 = LA(1);
                    if ( (LA250_0 == FROM_SYMBOL))
                    {
                        alt250=1;
                    }
                    else if ( (((LA250_0 >= ACCOUNT_SYMBOL) && (LA250_0 <= YEAR_SYMBOL)) || LA250_0 == BACK_TICK_QUOTED_ID || LA250_0 == DOUBLE_QUOTED_TEXT || LA250_0 == IDENTIFIER))
                    {
                        alt250=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 250;
                        EXCEPTION->state        = 0;


                        goto ruledelete_statementEx;

                    }
                }
                switch (alt250)
                {
            	case 1:
            	    // MySQLSimpleParser.g:653:4: FROM_SYMBOL ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? )
            	    {
            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_delete_statement3621);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledelete_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:655:5: ( options {k=4; } : table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )? | table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? )
            	        {
            	            int alt248=2;
            	            {
            	                int LA248_0 = LA(1);
            	                if ( (LA248_0 == IDENTIFIER))
            	                {
            	                    switch ( LA(2) )
            	                    {
            	                    case DOT_SYMBOL:
            	                    	{
            	                    		{
            	                    		    int LA248_7 = LA(3);
            	                    		    if ( (LA248_7 == MULT_OPERATOR))
            	                    		    {
            	                    		        alt248=1;
            	                    		    }
            	                    		    else if ( (LA248_7 == IDENTIFIER))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 53;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_7 == BACK_TICK_QUOTED_ID))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 54;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_7 == DOUBLE_QUOTED_TEXT))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 55;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_7 >= ACTION_SYMBOL) && (LA248_7 <= SHARE_SYMBOL)) || ((LA248_7 >= SLOW_SYMBOL) && (LA248_7 <= YEAR_SYMBOL))))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 56;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_7 >= ACCOUNT_SYMBOL) && (LA248_7 <= XA_SYMBOL)) || LA248_7 == SHUTDOWN_SYMBOL))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 57;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else
            	                    		    {
            	                    		        if (BACKTRACKING>0)
            	                    		        {
            	                    		            FAILEDFLAG = ANTLR3_TRUE;
            	                    		            return ;
            	                    		        }


            	                    		        CONSTRUCTEX();
            	                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		        EXCEPTION->message      = (void *)"";
            	                    		        EXCEPTION->decisionNum  = 248;
            	                    		        EXCEPTION->state        = 7;


            	                    		        goto ruledelete_statementEx;

            	                    		    }
            	                    		}
            	                    	}
            	                        break;
            	                    case USING_SYMBOL:
            	                    case COMMA_SYMBOL:
            	                    	{
            	                    		alt248=1;
            	                    	}
            	                        break;
            	                    case EOF:
            	                    case PARTITION_SYMBOL:
            	                    case LIMIT_SYMBOL:
            	                    case ORDER_SYMBOL:
            	                    case SEMICOLON_SYMBOL:
            	                    case WHERE_SYMBOL:
            	                    	{
            	                    		alt248=2;
            	                    	}
            	                        break;

            	                    default:
            	                        if (BACKTRACKING>0)
            	                        {
            	                            FAILEDFLAG = ANTLR3_TRUE;
            	                            return ;
            	                        }

            	                        CONSTRUCTEX();
            	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                        EXCEPTION->message      = (void *)"";
            	                        EXCEPTION->decisionNum  = 248;
            	                        EXCEPTION->state        = 1;


            	                        goto ruledelete_statementEx;

            	                    }

            	                }
            	                else if ( (LA248_0 == BACK_TICK_QUOTED_ID))
            	                {
            	                    switch ( LA(2) )
            	                    {
            	                    case DOT_SYMBOL:
            	                    	{
            	                    		{
            	                    		    int LA248_16 = LA(3);
            	                    		    if ( (LA248_16 == MULT_OPERATOR))
            	                    		    {
            	                    		        alt248=1;
            	                    		    }
            	                    		    else if ( (LA248_16 == IDENTIFIER))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 59;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_16 == BACK_TICK_QUOTED_ID))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 60;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_16 == DOUBLE_QUOTED_TEXT))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 61;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_16 >= ACTION_SYMBOL) && (LA248_16 <= SHARE_SYMBOL)) || ((LA248_16 >= SLOW_SYMBOL) && (LA248_16 <= YEAR_SYMBOL))))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 62;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_16 >= ACCOUNT_SYMBOL) && (LA248_16 <= XA_SYMBOL)) || LA248_16 == SHUTDOWN_SYMBOL))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 63;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else
            	                    		    {
            	                    		        if (BACKTRACKING>0)
            	                    		        {
            	                    		            FAILEDFLAG = ANTLR3_TRUE;
            	                    		            return ;
            	                    		        }


            	                    		        CONSTRUCTEX();
            	                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		        EXCEPTION->message      = (void *)"";
            	                    		        EXCEPTION->decisionNum  = 248;
            	                    		        EXCEPTION->state        = 16;


            	                    		        goto ruledelete_statementEx;

            	                    		    }
            	                    		}
            	                    	}
            	                        break;
            	                    case USING_SYMBOL:
            	                    case COMMA_SYMBOL:
            	                    	{
            	                    		alt248=1;
            	                    	}
            	                        break;
            	                    case EOF:
            	                    case PARTITION_SYMBOL:
            	                    case LIMIT_SYMBOL:
            	                    case ORDER_SYMBOL:
            	                    case SEMICOLON_SYMBOL:
            	                    case WHERE_SYMBOL:
            	                    	{
            	                    		alt248=2;
            	                    	}
            	                        break;

            	                    default:
            	                        if (BACKTRACKING>0)
            	                        {
            	                            FAILEDFLAG = ANTLR3_TRUE;
            	                            return ;
            	                        }

            	                        CONSTRUCTEX();
            	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                        EXCEPTION->message      = (void *)"";
            	                        EXCEPTION->decisionNum  = 248;
            	                        EXCEPTION->state        = 2;


            	                        goto ruledelete_statementEx;

            	                    }

            	                }
            	                else if ( (LA248_0 == DOUBLE_QUOTED_TEXT))
            	                {
            	                    switch ( LA(2) )
            	                    {
            	                    case DOT_SYMBOL:
            	                    	{
            	                    		{
            	                    		    int LA248_25 = LA(3);
            	                    		    if ( (LA248_25 == MULT_OPERATOR))
            	                    		    {
            	                    		        alt248=1;
            	                    		    }
            	                    		    else if ( (LA248_25 == IDENTIFIER))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 65;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_25 == BACK_TICK_QUOTED_ID))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 66;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_25 == DOUBLE_QUOTED_TEXT))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 67;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_25 >= ACTION_SYMBOL) && (LA248_25 <= SHARE_SYMBOL)) || ((LA248_25 >= SLOW_SYMBOL) && (LA248_25 <= YEAR_SYMBOL))))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 68;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_25 >= ACCOUNT_SYMBOL) && (LA248_25 <= XA_SYMBOL)) || LA248_25 == SHUTDOWN_SYMBOL))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 69;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else
            	                    		    {
            	                    		        if (BACKTRACKING>0)
            	                    		        {
            	                    		            FAILEDFLAG = ANTLR3_TRUE;
            	                    		            return ;
            	                    		        }


            	                    		        CONSTRUCTEX();
            	                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		        EXCEPTION->message      = (void *)"";
            	                    		        EXCEPTION->decisionNum  = 248;
            	                    		        EXCEPTION->state        = 25;


            	                    		        goto ruledelete_statementEx;

            	                    		    }
            	                    		}
            	                    	}
            	                        break;
            	                    case USING_SYMBOL:
            	                    case COMMA_SYMBOL:
            	                    	{
            	                    		alt248=1;
            	                    	}
            	                        break;
            	                    case EOF:
            	                    case PARTITION_SYMBOL:
            	                    case LIMIT_SYMBOL:
            	                    case ORDER_SYMBOL:
            	                    case SEMICOLON_SYMBOL:
            	                    case WHERE_SYMBOL:
            	                    	{
            	                    		alt248=2;
            	                    	}
            	                        break;

            	                    default:
            	                        if (BACKTRACKING>0)
            	                        {
            	                            FAILEDFLAG = ANTLR3_TRUE;
            	                            return ;
            	                        }

            	                        CONSTRUCTEX();
            	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                        EXCEPTION->message      = (void *)"";
            	                        EXCEPTION->decisionNum  = 248;
            	                        EXCEPTION->state        = 3;


            	                        goto ruledelete_statementEx;

            	                    }

            	                }
            	                else if ( (((LA248_0 >= ACTION_SYMBOL) && (LA248_0 <= SHARE_SYMBOL)) || ((LA248_0 >= SLOW_SYMBOL) && (LA248_0 <= YEAR_SYMBOL))))
            	                {
            	                    switch ( LA(2) )
            	                    {
            	                    case DOT_SYMBOL:
            	                    	{
            	                    		{
            	                    		    int LA248_34 = LA(3);
            	                    		    if ( (LA248_34 == MULT_OPERATOR))
            	                    		    {
            	                    		        alt248=1;
            	                    		    }
            	                    		    else if ( (LA248_34 == IDENTIFIER))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 71;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_34 == BACK_TICK_QUOTED_ID))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 72;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_34 == DOUBLE_QUOTED_TEXT))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 73;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_34 >= ACTION_SYMBOL) && (LA248_34 <= SHARE_SYMBOL)) || ((LA248_34 >= SLOW_SYMBOL) && (LA248_34 <= YEAR_SYMBOL))))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 74;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_34 >= ACCOUNT_SYMBOL) && (LA248_34 <= XA_SYMBOL)) || LA248_34 == SHUTDOWN_SYMBOL))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 75;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else
            	                    		    {
            	                    		        if (BACKTRACKING>0)
            	                    		        {
            	                    		            FAILEDFLAG = ANTLR3_TRUE;
            	                    		            return ;
            	                    		        }


            	                    		        CONSTRUCTEX();
            	                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		        EXCEPTION->message      = (void *)"";
            	                    		        EXCEPTION->decisionNum  = 248;
            	                    		        EXCEPTION->state        = 34;


            	                    		        goto ruledelete_statementEx;

            	                    		    }
            	                    		}
            	                    	}
            	                        break;
            	                    case USING_SYMBOL:
            	                    case COMMA_SYMBOL:
            	                    	{
            	                    		alt248=1;
            	                    	}
            	                        break;
            	                    case EOF:
            	                    case PARTITION_SYMBOL:
            	                    case LIMIT_SYMBOL:
            	                    case ORDER_SYMBOL:
            	                    case SEMICOLON_SYMBOL:
            	                    case WHERE_SYMBOL:
            	                    	{
            	                    		alt248=2;
            	                    	}
            	                        break;

            	                    default:
            	                        if (BACKTRACKING>0)
            	                        {
            	                            FAILEDFLAG = ANTLR3_TRUE;
            	                            return ;
            	                        }

            	                        CONSTRUCTEX();
            	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                        EXCEPTION->message      = (void *)"";
            	                        EXCEPTION->decisionNum  = 248;
            	                        EXCEPTION->state        = 4;


            	                        goto ruledelete_statementEx;

            	                    }

            	                }
            	                else if ( (((LA248_0 >= ACCOUNT_SYMBOL) && (LA248_0 <= XA_SYMBOL)) || LA248_0 == SHUTDOWN_SYMBOL))
            	                {
            	                    switch ( LA(2) )
            	                    {
            	                    case DOT_SYMBOL:
            	                    	{
            	                    		{
            	                    		    int LA248_43 = LA(3);
            	                    		    if ( (LA248_43 == MULT_OPERATOR))
            	                    		    {
            	                    		        alt248=1;
            	                    		    }
            	                    		    else if ( (LA248_43 == IDENTIFIER))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 77;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_43 == BACK_TICK_QUOTED_ID))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 78;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (LA248_43 == DOUBLE_QUOTED_TEXT))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 79;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_43 >= ACTION_SYMBOL) && (LA248_43 <= SHARE_SYMBOL)) || ((LA248_43 >= SLOW_SYMBOL) && (LA248_43 <= YEAR_SYMBOL))))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 80;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else if ( (((LA248_43 >= ACCOUNT_SYMBOL) && (LA248_43 <= XA_SYMBOL)) || LA248_43 == SHUTDOWN_SYMBOL))
            	                    		    {
            	                    		        switch ( LA(4) )
            	                    		        {
            	                    		        case USING_SYMBOL:
            	                    		        case COMMA_SYMBOL:
            	                    		        case DOT_SYMBOL:
            	                    		        	{
            	                    		        		alt248=1;
            	                    		        	}
            	                    		            break;
            	                    		        case EOF:
            	                    		        case PARTITION_SYMBOL:
            	                    		        case LIMIT_SYMBOL:
            	                    		        case ORDER_SYMBOL:
            	                    		        case SEMICOLON_SYMBOL:
            	                    		        case WHERE_SYMBOL:
            	                    		        	{
            	                    		        		alt248=2;
            	                    		        	}
            	                    		            break;

            	                    		        default:
            	                    		            if (BACKTRACKING>0)
            	                    		            {
            	                    		                FAILEDFLAG = ANTLR3_TRUE;
            	                    		                return ;
            	                    		            }

            	                    		            CONSTRUCTEX();
            	                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		            EXCEPTION->message      = (void *)"";
            	                    		            EXCEPTION->decisionNum  = 248;
            	                    		            EXCEPTION->state        = 81;


            	                    		            goto ruledelete_statementEx;

            	                    		        }

            	                    		    }
            	                    		    else
            	                    		    {
            	                    		        if (BACKTRACKING>0)
            	                    		        {
            	                    		            FAILEDFLAG = ANTLR3_TRUE;
            	                    		            return ;
            	                    		        }


            	                    		        CONSTRUCTEX();
            	                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    		        EXCEPTION->message      = (void *)"";
            	                    		        EXCEPTION->decisionNum  = 248;
            	                    		        EXCEPTION->state        = 43;


            	                    		        goto ruledelete_statementEx;

            	                    		    }
            	                    		}
            	                    	}
            	                        break;
            	                    case USING_SYMBOL:
            	                    case COMMA_SYMBOL:
            	                    	{
            	                    		alt248=1;
            	                    	}
            	                        break;
            	                    case EOF:
            	                    case PARTITION_SYMBOL:
            	                    case LIMIT_SYMBOL:
            	                    case ORDER_SYMBOL:
            	                    case SEMICOLON_SYMBOL:
            	                    case WHERE_SYMBOL:
            	                    	{
            	                    		alt248=2;
            	                    	}
            	                        break;

            	                    default:
            	                        if (BACKTRACKING>0)
            	                        {
            	                            FAILEDFLAG = ANTLR3_TRUE;
            	                            return ;
            	                        }

            	                        CONSTRUCTEX();
            	                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                        EXCEPTION->message      = (void *)"";
            	                        EXCEPTION->decisionNum  = 248;
            	                        EXCEPTION->state        = 5;


            	                        goto ruledelete_statementEx;

            	                    }

            	                }
            	                else if ( (LA248_0 == DOT_SYMBOL))
            	                {
            	                    alt248=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 248;
            	                    EXCEPTION->state        = 0;


            	                    goto ruledelete_statementEx;

            	                }
            	            }
            	            switch (alt248)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:656:7: table_ref_list_with_wildcard USING_SYMBOL join_table_list ( where_clause )?
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_table_ref_list_with_wildcard_in_delete_statement3652);
            	        	        table_ref_list_with_wildcard(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledelete_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(USING_SYMBOL, &FOLLOW_USING_SYMBOL_in_delete_statement3654);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledelete_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_join_table_list_in_delete_statement3656);
            	        	        join_table_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledelete_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:656:65: ( where_clause )?
            	        	        {
            	        	            int alt243=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case WHERE_SYMBOL:
            	        	                	{
            	        	                		alt243=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt243)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:656:65: where_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_where_clause_in_delete_statement3658);
            	        	        	        where_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledelete_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:657:8: table_ref ( partition_delete )? ( where_clause )? ( order_by_clause )? ( simple_limit_clause )?
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_table_ref_in_delete_statement3669);
            	        	        table_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledelete_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:657:18: ( partition_delete )?
            	        	        {
            	        	            int alt244=2;
            	        	            {
            	        	                int LA244_0 = LA(1);
            	        	                if ( ((SERVER_VERSION >= 50602)) && (LA244_0 == PARTITION_SYMBOL))
            	        	                {
            	        	                    alt244=1;
            	        	                }
            	        	            }
            	        	            switch (alt244)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:657:18: partition_delete
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_partition_delete_in_delete_statement3671);
            	        	        	        partition_delete(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledelete_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:657:36: ( where_clause )?
            	        	        {
            	        	            int alt245=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case WHERE_SYMBOL:
            	        	                	{
            	        	                		alt245=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt245)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:657:36: where_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_where_clause_in_delete_statement3674);
            	        	        	        where_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledelete_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:657:50: ( order_by_clause )?
            	        	        {
            	        	            int alt246=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case ORDER_SYMBOL:
            	        	                	{
            	        	                		alt246=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt246)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:657:50: order_by_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_order_by_clause_in_delete_statement3677);
            	        	        	        order_by_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledelete_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:657:67: ( simple_limit_clause )?
            	        	        {
            	        	            int alt247=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case LIMIT_SYMBOL:
            	        	                	{
            	        	                		alt247=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt247)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:657:67: simple_limit_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_simple_limit_clause_in_delete_statement3680);
            	        	        	        simple_limit_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruledelete_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:659:7: table_ref_list_with_wildcard FROM_SYMBOL join_table_list ( where_clause )?
            	    {
            	        FOLLOWPUSH(FOLLOW_table_ref_list_with_wildcard_in_delete_statement3696);
            	        table_ref_list_with_wildcard(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledelete_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_delete_statement3698);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledelete_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_join_table_list_in_delete_statement3700);
            	        join_table_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledelete_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:659:64: ( where_clause )?
            	        {
            	            int alt249=2;
            	            switch ( LA(1) )
            	            {
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt249=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt249)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:659:64: where_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_where_clause_in_delete_statement3702);
            	        	        where_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledelete_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruledelete_statementEx; /* Prevent compiler warnings */
    ruledelete_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end delete_statement */

/**
 * $ANTLR start partition_delete
 * MySQLSimpleParser.g:663:1: partition_delete :{...}? => PARTITION_SYMBOL OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL ;
 */
static void
partition_delete(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:663:17: ({...}? => PARTITION_SYMBOL OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:664:2: {...}? => PARTITION_SYMBOL OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL
        {
            if ( !((SERVER_VERSION >= 50602)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50602";
                    EXCEPTION->ruleName	 = (void *)"partition_delete";


            }

             MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_partition_delete3721);
            if  (HASEXCEPTION())
            {
                goto rulepartition_deleteEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partition_delete3723);
            if  (HASEXCEPTION())
            {
                goto rulepartition_deleteEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_list_in_partition_delete3725);
            identifier_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulepartition_deleteEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partition_delete3727);
            if  (HASEXCEPTION())
            {
                goto rulepartition_deleteEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartition_deleteEx; /* Prevent compiler warnings */
    rulepartition_deleteEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_delete */

/**
 * $ANTLR start delete_option
 * MySQLSimpleParser.g:667:1: delete_option : ( LOW_PRIORITY_SYMBOL | QUICK_SYMBOL | IGNORE_SYMBOL );
 */
static void
delete_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:667:14: ( LOW_PRIORITY_SYMBOL | QUICK_SYMBOL | IGNORE_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == QUICK_SYMBOL || LA(1) == IGNORE_SYMBOL || LA(1) == LOW_PRIORITY_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruledelete_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledelete_optionEx; /* Prevent compiler warnings */
    ruledelete_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end delete_option */

/**
 * $ANTLR start do_statement
 * MySQLSimpleParser.g:671:1: do_statement : DO_SYMBOL ({...}? => expression_list |{...}? => select_item_list ) ;
 */
static void
do_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:671:13: ( DO_SYMBOL ({...}? => expression_list |{...}? => select_item_list ) )
        // MySQLSimpleParser.g:672:2: DO_SYMBOL ({...}? => expression_list |{...}? => select_item_list )
        {
             MATCHT(DO_SYMBOL, &FOLLOW_DO_SYMBOL_in_do_statement3753);
            if  (HASEXCEPTION())
            {
                goto ruledo_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:673:2: ({...}? => expression_list |{...}? => select_item_list )
            {
                int alt251=2;
                {
                    int LA251_0 = LA(1);
                    if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == NOT_SYMBOL))
                    {
                        {
                            int LA251_1 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == NOT_SYMBOL))
                            {
                                {
                                    int LA251_104 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 104;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BITWISE_NOT_OPERATOR || LA251_1 == MINUS_OPERATOR || LA251_1 == PLUS_OPERATOR))
                            {
                                {
                                    int LA251_105 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 105;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == NOT2_SYMBOL || LA251_1 == LOGICAL_NOT_OPERATOR))
                            {
                                {
                                    int LA251_106 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 106;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_107 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 107;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_108 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 108;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_109 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 109;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_110 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 110;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_111 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 111;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DECIMAL_NUMBER || LA251_1 == FLOAT_NUMBER || LA251_1 == INT_NUMBER || LA251_1 == LONG_NUMBER || LA251_1 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_112 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 112;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_113 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 113;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_114 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 114;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_115 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 115;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_1 >= NULL2_SYMBOL) && (LA251_1 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_116 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 116;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == FALSE_SYMBOL || LA251_1 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_117 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 117;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == HEX_NUMBER))
                            {
                                {
                                    int LA251_118 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 118;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BIN_NUMBER))
                            {
                                {
                                    int LA251_119 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 119;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == IDENTIFIER))
                            {
                                {
                                    int LA251_120 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 120;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_121 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 121;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_122 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 122;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_123 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 123;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_124 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 124;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_125 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 125;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_126 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 126;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_127 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 127;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_128 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 128;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_129 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 129;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_130 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 130;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_131 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 131;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_132 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 132;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == USER_SYMBOL))
                            {
                                {
                                    int LA251_133 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 133;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_134 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 134;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_135 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 135;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ADDDATE_SYMBOL || LA251_1 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_136 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 136;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_137 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 137;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_138 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 138;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DATE_ADD_SYMBOL || LA251_1 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_139 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 139;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_140 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 140;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_141 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 141;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_142 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 142;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_143 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 143;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_144 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 144;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_145 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 145;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_1 >= TIMESTAMP_ADD_SYMBOL) && (LA251_1 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_146 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 146;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_147 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 147;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_148 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 148;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_149 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 149;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_150 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 150;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_151 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 151;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_152 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 152;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_153 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 153;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_154 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 154;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == IF_SYMBOL))
                            {
                                {
                                    int LA251_155 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 155;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_156 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 156;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_157 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 157;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_158 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 158;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_159 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 159;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_160 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 160;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_161 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 161;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_162 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 162;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_163 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 163;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_164 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 164;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_165 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 165;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_166 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 166;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_167 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 167;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_168 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 168;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_169 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 169;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_170 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 170;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_171 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 171;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_172 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 172;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_173 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 173;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_174 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 174;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_175 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 175;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_176 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 176;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_177 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 177;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_178 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 178;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_179 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 179;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_180 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 180;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_181 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 181;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_182 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 182;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_183 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 183;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == STD_SYMBOL))
                            {
                                {
                                    int LA251_184 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 184;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_185 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 185;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_186 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 186;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_187 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 187;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_188 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 188;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_189 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 189;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_190 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 190;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_191 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 191;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ACCOUNT_SYMBOL || ((LA251_1 >= ALWAYS_SYMBOL) && (LA251_1 <= CACHE_SYMBOL)) || ((LA251_1 >= CHECKSUM_SYMBOL) && (LA251_1 <= COMMIT_SYMBOL)) || ((LA251_1 >= DEALLOCATE_SYMBOL) && (LA251_1 <= FOLLOWS_SYMBOL)) || ((LA251_1 >= GROUP_REPLICATION_SYMBOL) && (LA251_1 <= STOP_SYMBOL)) || ((LA251_1 >= UNICODE_SYMBOL) && (LA251_1 <= XA_SYMBOL)) || LA251_1 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_192 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 192;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == PARAM_MARKER))
                            {
                                {
                                    int LA251_193 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 193;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_194 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 194;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_195 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 195;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_196 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 196;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_197 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 197;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_198 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 198;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == ACTION_SYMBOL || ((LA251_1 >= AFTER_SYMBOL) && (LA251_1 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_1 >= BINLOG_SYMBOL) && (LA251_1 <= CLASS_ORIGIN_SYMBOL)) || LA251_1 == CODE_SYMBOL || ((LA251_1 >= COLUMN_NAME_SYMBOL) && (LA251_1 <= DATETIME_SYMBOL)) || ((LA251_1 >= DEFAULT_AUTH_SYMBOL) && (LA251_1 <= GEOMETRY_SYMBOL)) || ((LA251_1 >= GRANTS_SYMBOL) && (LA251_1 <= HOSTS_SYMBOL)) || ((LA251_1 >= IDENTIFIED_SYMBOL) && (LA251_1 <= LEVEL_SYMBOL)) || ((LA251_1 >= LIST_SYMBOL) && (LA251_1 <= MESSAGE_TEXT_SYMBOL)) || LA251_1 == MIGRATE_SYMBOL || ((LA251_1 >= MIN_ROWS_SYMBOL) && (LA251_1 <= MODE_SYMBOL)) || ((LA251_1 >= MUTEX_SYMBOL) && (LA251_1 <= OFFSET_SYMBOL)) || ((LA251_1 >= ONE_SHOT_SYMBOL) && (LA251_1 <= PARTITIONS_SYMBOL)) || ((LA251_1 >= PHASE_SYMBOL) && (LA251_1 <= PLUGINS_SYMBOL)) || ((LA251_1 >= PRESERVE_SYMBOL) && (LA251_1 <= PROXY_SYMBOL)) || ((LA251_1 >= QUERY_SYMBOL) && (LA251_1 <= RETURNS_SYMBOL)) || ((LA251_1 >= ROLLUP_SYMBOL) && (LA251_1 <= ROWS_SYMBOL)) || LA251_1 == ROW_FORMAT_SYMBOL || ((LA251_1 >= RTREE_SYMBOL) && (LA251_1 <= SCHEMA_NAME_SYMBOL)) || ((LA251_1 >= SERIAL_SYMBOL) && (LA251_1 <= SHARE_SYMBOL)) || ((LA251_1 >= SLOW_SYMBOL) && (LA251_1 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_1 >= SUBJECT_SYMBOL) && (LA251_1 <= TRIGGERS_SYMBOL)) || ((LA251_1 >= TYPES_SYMBOL) && (LA251_1 <= USER_RESOURCES_SYMBOL)) || ((LA251_1 >= USE_FRM_SYMBOL) && (LA251_1 <= WAIT_SYMBOL)) || LA251_1 == WORK_SYMBOL || ((LA251_1 >= X509_SYMBOL) && (LA251_1 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_199 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 199;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_200 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 200;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_201 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 201;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_202 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 202;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_203 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 203;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_204 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 204;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_1 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_205 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 205;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 1;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BITWISE_NOT_OPERATOR || LA251_0 == MINUS_OPERATOR || LA251_0 == PLUS_OPERATOR))
                    {
                        {
                            int LA251_2 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BITWISE_NOT_OPERATOR || LA251_2 == MINUS_OPERATOR || LA251_2 == PLUS_OPERATOR))
                            {
                                {
                                    int LA251_206 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 206;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == NOT2_SYMBOL || LA251_2 == LOGICAL_NOT_OPERATOR))
                            {
                                {
                                    int LA251_207 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 207;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_208 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 208;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_209 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 209;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_210 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 210;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_211 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 211;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_212 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 212;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DECIMAL_NUMBER || LA251_2 == FLOAT_NUMBER || LA251_2 == INT_NUMBER || LA251_2 == LONG_NUMBER || LA251_2 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_213 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 213;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_214 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 214;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_215 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 215;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_216 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 216;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_2 >= NULL2_SYMBOL) && (LA251_2 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_217 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 217;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == FALSE_SYMBOL || LA251_2 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_218 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 218;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == HEX_NUMBER))
                            {
                                {
                                    int LA251_219 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 219;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BIN_NUMBER))
                            {
                                {
                                    int LA251_220 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 220;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == IDENTIFIER))
                            {
                                {
                                    int LA251_221 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 221;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_222 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 222;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_223 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 223;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_224 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 224;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_225 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 225;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_226 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 226;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_227 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 227;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_228 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 228;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_229 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 229;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_230 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 230;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_231 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 231;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_232 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 232;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_233 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 233;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == USER_SYMBOL))
                            {
                                {
                                    int LA251_234 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 234;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_235 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 235;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_236 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 236;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ADDDATE_SYMBOL || LA251_2 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_237 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 237;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_238 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 238;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_239 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 239;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DATE_ADD_SYMBOL || LA251_2 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_240 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 240;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_241 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 241;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_242 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 242;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_243 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 243;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_244 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 244;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_245 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 245;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_246 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 246;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_2 >= TIMESTAMP_ADD_SYMBOL) && (LA251_2 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_247 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 247;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_248 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 248;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_249 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 249;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_250 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 250;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_251 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 251;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_252 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 252;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_253 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 253;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_254 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 254;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_255 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 255;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == IF_SYMBOL))
                            {
                                {
                                    int LA251_256 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 256;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_257 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 257;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_258 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 258;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_259 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 259;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_260 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 260;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_261 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 261;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_262 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 262;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_263 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 263;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_264 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 264;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_265 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 265;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_266 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 266;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_267 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 267;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_268 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 268;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_269 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 269;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_270 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 270;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_271 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 271;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_272 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 272;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_273 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 273;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_274 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 274;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_275 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 275;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_276 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 276;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_277 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 277;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_278 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 278;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_279 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 279;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_280 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 280;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_281 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 281;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_282 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 282;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_283 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 283;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_284 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 284;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == STD_SYMBOL))
                            {
                                {
                                    int LA251_285 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 285;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_286 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 286;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_287 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 287;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_288 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 288;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_289 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 289;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_290 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 290;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_291 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 291;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_292 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 292;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ACCOUNT_SYMBOL || ((LA251_2 >= ALWAYS_SYMBOL) && (LA251_2 <= CACHE_SYMBOL)) || ((LA251_2 >= CHECKSUM_SYMBOL) && (LA251_2 <= COMMIT_SYMBOL)) || ((LA251_2 >= DEALLOCATE_SYMBOL) && (LA251_2 <= FOLLOWS_SYMBOL)) || ((LA251_2 >= GROUP_REPLICATION_SYMBOL) && (LA251_2 <= STOP_SYMBOL)) || ((LA251_2 >= UNICODE_SYMBOL) && (LA251_2 <= XA_SYMBOL)) || LA251_2 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_293 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 293;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == PARAM_MARKER))
                            {
                                {
                                    int LA251_294 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 294;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_295 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 295;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_296 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 296;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_297 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 297;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_298 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 298;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_299 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 299;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == ACTION_SYMBOL || ((LA251_2 >= AFTER_SYMBOL) && (LA251_2 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_2 >= BINLOG_SYMBOL) && (LA251_2 <= CLASS_ORIGIN_SYMBOL)) || LA251_2 == CODE_SYMBOL || ((LA251_2 >= COLUMN_NAME_SYMBOL) && (LA251_2 <= DATETIME_SYMBOL)) || ((LA251_2 >= DEFAULT_AUTH_SYMBOL) && (LA251_2 <= GEOMETRY_SYMBOL)) || ((LA251_2 >= GRANTS_SYMBOL) && (LA251_2 <= HOSTS_SYMBOL)) || ((LA251_2 >= IDENTIFIED_SYMBOL) && (LA251_2 <= LEVEL_SYMBOL)) || ((LA251_2 >= LIST_SYMBOL) && (LA251_2 <= MESSAGE_TEXT_SYMBOL)) || LA251_2 == MIGRATE_SYMBOL || ((LA251_2 >= MIN_ROWS_SYMBOL) && (LA251_2 <= MODE_SYMBOL)) || ((LA251_2 >= MUTEX_SYMBOL) && (LA251_2 <= OFFSET_SYMBOL)) || ((LA251_2 >= ONE_SHOT_SYMBOL) && (LA251_2 <= PARTITIONS_SYMBOL)) || ((LA251_2 >= PHASE_SYMBOL) && (LA251_2 <= PLUGINS_SYMBOL)) || ((LA251_2 >= PRESERVE_SYMBOL) && (LA251_2 <= PROXY_SYMBOL)) || ((LA251_2 >= QUERY_SYMBOL) && (LA251_2 <= RETURNS_SYMBOL)) || ((LA251_2 >= ROLLUP_SYMBOL) && (LA251_2 <= ROWS_SYMBOL)) || LA251_2 == ROW_FORMAT_SYMBOL || ((LA251_2 >= RTREE_SYMBOL) && (LA251_2 <= SCHEMA_NAME_SYMBOL)) || ((LA251_2 >= SERIAL_SYMBOL) && (LA251_2 <= SHARE_SYMBOL)) || ((LA251_2 >= SLOW_SYMBOL) && (LA251_2 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_2 >= SUBJECT_SYMBOL) && (LA251_2 <= TRIGGERS_SYMBOL)) || ((LA251_2 >= TYPES_SYMBOL) && (LA251_2 <= USER_RESOURCES_SYMBOL)) || ((LA251_2 >= USE_FRM_SYMBOL) && (LA251_2 <= WAIT_SYMBOL)) || LA251_2 == WORK_SYMBOL || ((LA251_2 >= X509_SYMBOL) && (LA251_2 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_300 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 300;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_301 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 301;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_302 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 302;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_303 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 303;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_304 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 304;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_305 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 305;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_2 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_306 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 306;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 2;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == NOT2_SYMBOL || LA251_0 == LOGICAL_NOT_OPERATOR))
                    {
                        {
                            int LA251_3 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_307 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 307;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_308 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 308;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_309 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 309;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_310 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 310;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_311 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 311;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DECIMAL_NUMBER || LA251_3 == FLOAT_NUMBER || LA251_3 == INT_NUMBER || LA251_3 == LONG_NUMBER || LA251_3 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_312 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 312;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_313 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 313;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_314 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 314;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_315 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 315;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_3 >= NULL2_SYMBOL) && (LA251_3 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_316 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 316;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == FALSE_SYMBOL || LA251_3 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_317 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 317;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == HEX_NUMBER))
                            {
                                {
                                    int LA251_318 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 318;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BIN_NUMBER))
                            {
                                {
                                    int LA251_319 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 319;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == IDENTIFIER))
                            {
                                {
                                    int LA251_320 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 320;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_321 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 321;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_322 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 322;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_323 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 323;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_324 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 324;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_325 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 325;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_326 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 326;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_327 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 327;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_328 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 328;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_329 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 329;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_330 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 330;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_331 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 331;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_332 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 332;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == USER_SYMBOL))
                            {
                                {
                                    int LA251_333 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 333;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_334 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 334;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_335 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 335;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ADDDATE_SYMBOL || LA251_3 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_336 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 336;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_337 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 337;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_338 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 338;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DATE_ADD_SYMBOL || LA251_3 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_339 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 339;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_340 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 340;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_341 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 341;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_342 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 342;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_343 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 343;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_344 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 344;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_345 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 345;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_3 >= TIMESTAMP_ADD_SYMBOL) && (LA251_3 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_346 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 346;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_347 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 347;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_348 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 348;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_349 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 349;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_350 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 350;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_351 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 351;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_352 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 352;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_353 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 353;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_354 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 354;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == IF_SYMBOL))
                            {
                                {
                                    int LA251_355 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 355;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_356 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 356;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_357 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 357;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_358 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 358;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_359 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 359;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_360 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 360;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_361 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 361;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_362 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 362;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_363 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 363;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_364 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 364;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_365 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 365;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_366 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 366;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_367 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 367;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_368 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 368;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_369 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 369;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_370 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 370;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_371 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 371;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_372 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 372;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_373 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 373;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_374 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 374;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_375 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 375;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_376 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 376;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_377 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 377;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_378 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 378;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_379 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 379;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_380 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 380;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_381 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 381;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_382 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 382;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_383 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 383;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == STD_SYMBOL))
                            {
                                {
                                    int LA251_384 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 384;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_385 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 385;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_386 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 386;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_387 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 387;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_388 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 388;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_389 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 389;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_390 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 390;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_391 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 391;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ACCOUNT_SYMBOL || ((LA251_3 >= ALWAYS_SYMBOL) && (LA251_3 <= CACHE_SYMBOL)) || ((LA251_3 >= CHECKSUM_SYMBOL) && (LA251_3 <= COMMIT_SYMBOL)) || ((LA251_3 >= DEALLOCATE_SYMBOL) && (LA251_3 <= FOLLOWS_SYMBOL)) || ((LA251_3 >= GROUP_REPLICATION_SYMBOL) && (LA251_3 <= STOP_SYMBOL)) || ((LA251_3 >= UNICODE_SYMBOL) && (LA251_3 <= XA_SYMBOL)) || LA251_3 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_392 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 392;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == PARAM_MARKER))
                            {
                                {
                                    int LA251_393 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 393;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_394 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 394;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_395 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 395;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_396 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 396;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_397 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 397;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_398 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 398;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == ACTION_SYMBOL || ((LA251_3 >= AFTER_SYMBOL) && (LA251_3 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_3 >= BINLOG_SYMBOL) && (LA251_3 <= CLASS_ORIGIN_SYMBOL)) || LA251_3 == CODE_SYMBOL || ((LA251_3 >= COLUMN_NAME_SYMBOL) && (LA251_3 <= DATETIME_SYMBOL)) || ((LA251_3 >= DEFAULT_AUTH_SYMBOL) && (LA251_3 <= GEOMETRY_SYMBOL)) || ((LA251_3 >= GRANTS_SYMBOL) && (LA251_3 <= HOSTS_SYMBOL)) || ((LA251_3 >= IDENTIFIED_SYMBOL) && (LA251_3 <= LEVEL_SYMBOL)) || ((LA251_3 >= LIST_SYMBOL) && (LA251_3 <= MESSAGE_TEXT_SYMBOL)) || LA251_3 == MIGRATE_SYMBOL || ((LA251_3 >= MIN_ROWS_SYMBOL) && (LA251_3 <= MODE_SYMBOL)) || ((LA251_3 >= MUTEX_SYMBOL) && (LA251_3 <= OFFSET_SYMBOL)) || ((LA251_3 >= ONE_SHOT_SYMBOL) && (LA251_3 <= PARTITIONS_SYMBOL)) || ((LA251_3 >= PHASE_SYMBOL) && (LA251_3 <= PLUGINS_SYMBOL)) || ((LA251_3 >= PRESERVE_SYMBOL) && (LA251_3 <= PROXY_SYMBOL)) || ((LA251_3 >= QUERY_SYMBOL) && (LA251_3 <= RETURNS_SYMBOL)) || ((LA251_3 >= ROLLUP_SYMBOL) && (LA251_3 <= ROWS_SYMBOL)) || LA251_3 == ROW_FORMAT_SYMBOL || ((LA251_3 >= RTREE_SYMBOL) && (LA251_3 <= SCHEMA_NAME_SYMBOL)) || ((LA251_3 >= SERIAL_SYMBOL) && (LA251_3 <= SHARE_SYMBOL)) || ((LA251_3 >= SLOW_SYMBOL) && (LA251_3 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_3 >= SUBJECT_SYMBOL) && (LA251_3 <= TRIGGERS_SYMBOL)) || ((LA251_3 >= TYPES_SYMBOL) && (LA251_3 <= USER_RESOURCES_SYMBOL)) || ((LA251_3 >= USE_FRM_SYMBOL) && (LA251_3 <= WAIT_SYMBOL)) || LA251_3 == WORK_SYMBOL || ((LA251_3 >= X509_SYMBOL) && (LA251_3 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_399 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 399;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_400 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 400;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_401 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 401;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_402 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 402;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_403 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 403;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_404 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 404;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_3 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_405 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 405;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 3;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == INTERVAL_SYMBOL))
                    {
                        {
                            int LA251_4 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_406 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 406;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == NOT_SYMBOL))
                            {
                                {
                                    int LA251_407 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 407;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BITWISE_NOT_OPERATOR || LA251_4 == MINUS_OPERATOR || LA251_4 == PLUS_OPERATOR))
                            {
                                {
                                    int LA251_408 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 408;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == NOT2_SYMBOL || LA251_4 == LOGICAL_NOT_OPERATOR))
                            {
                                {
                                    int LA251_409 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 409;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_410 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 410;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_411 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 411;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_412 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 412;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_413 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 413;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_414 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 414;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DECIMAL_NUMBER || LA251_4 == FLOAT_NUMBER || LA251_4 == INT_NUMBER || LA251_4 == LONG_NUMBER || LA251_4 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_415 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 415;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_416 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 416;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_417 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 417;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_418 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 418;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_4 >= NULL2_SYMBOL) && (LA251_4 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_419 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 419;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == FALSE_SYMBOL || LA251_4 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_420 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 420;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == HEX_NUMBER))
                            {
                                {
                                    int LA251_421 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 421;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BIN_NUMBER))
                            {
                                {
                                    int LA251_422 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 422;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == IDENTIFIER))
                            {
                                {
                                    int LA251_423 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 423;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_424 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 424;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_425 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 425;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_426 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 426;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_427 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 427;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_428 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 428;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_429 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 429;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_430 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 430;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_431 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 431;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_432 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 432;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_433 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 433;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_434 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 434;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_435 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 435;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == USER_SYMBOL))
                            {
                                {
                                    int LA251_436 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 436;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_437 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 437;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_438 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 438;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ADDDATE_SYMBOL || LA251_4 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_439 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 439;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_440 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 440;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_441 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 441;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DATE_ADD_SYMBOL || LA251_4 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_442 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 442;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_443 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 443;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_444 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 444;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_445 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 445;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_446 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 446;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_447 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 447;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_448 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 448;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_4 >= TIMESTAMP_ADD_SYMBOL) && (LA251_4 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_449 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 449;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_450 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 450;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_451 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 451;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_452 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 452;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_453 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 453;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_454 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 454;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_455 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 455;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_456 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 456;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_457 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 457;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == IF_SYMBOL))
                            {
                                {
                                    int LA251_458 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 458;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_459 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 459;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_460 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 460;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_461 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 461;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_462 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 462;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_463 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 463;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_464 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 464;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_465 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 465;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_466 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 466;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_467 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 467;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_468 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 468;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_469 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 469;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_470 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 470;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_471 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 471;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_472 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 472;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_473 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 473;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_474 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 474;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_475 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 475;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_476 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 476;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_477 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 477;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_478 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 478;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_479 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 479;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_480 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 480;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_481 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 481;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_482 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 482;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_483 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 483;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_484 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 484;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_485 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 485;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_486 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 486;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == STD_SYMBOL))
                            {
                                {
                                    int LA251_487 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 487;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_488 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 488;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_489 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 489;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_490 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 490;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_491 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 491;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_492 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 492;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_493 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 493;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_494 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 494;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ACCOUNT_SYMBOL || ((LA251_4 >= ALWAYS_SYMBOL) && (LA251_4 <= CACHE_SYMBOL)) || ((LA251_4 >= CHECKSUM_SYMBOL) && (LA251_4 <= COMMIT_SYMBOL)) || ((LA251_4 >= DEALLOCATE_SYMBOL) && (LA251_4 <= FOLLOWS_SYMBOL)) || ((LA251_4 >= GROUP_REPLICATION_SYMBOL) && (LA251_4 <= STOP_SYMBOL)) || ((LA251_4 >= UNICODE_SYMBOL) && (LA251_4 <= XA_SYMBOL)) || LA251_4 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_495 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 495;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == PARAM_MARKER))
                            {
                                {
                                    int LA251_496 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 496;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_497 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 497;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_498 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 498;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_499 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 499;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_500 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 500;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == ACTION_SYMBOL || ((LA251_4 >= AFTER_SYMBOL) && (LA251_4 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_4 >= BINLOG_SYMBOL) && (LA251_4 <= CLASS_ORIGIN_SYMBOL)) || LA251_4 == CODE_SYMBOL || ((LA251_4 >= COLUMN_NAME_SYMBOL) && (LA251_4 <= DATETIME_SYMBOL)) || ((LA251_4 >= DEFAULT_AUTH_SYMBOL) && (LA251_4 <= GEOMETRY_SYMBOL)) || ((LA251_4 >= GRANTS_SYMBOL) && (LA251_4 <= HOSTS_SYMBOL)) || ((LA251_4 >= IDENTIFIED_SYMBOL) && (LA251_4 <= LEVEL_SYMBOL)) || ((LA251_4 >= LIST_SYMBOL) && (LA251_4 <= MESSAGE_TEXT_SYMBOL)) || LA251_4 == MIGRATE_SYMBOL || ((LA251_4 >= MIN_ROWS_SYMBOL) && (LA251_4 <= MODE_SYMBOL)) || ((LA251_4 >= MUTEX_SYMBOL) && (LA251_4 <= OFFSET_SYMBOL)) || ((LA251_4 >= ONE_SHOT_SYMBOL) && (LA251_4 <= PARTITIONS_SYMBOL)) || ((LA251_4 >= PHASE_SYMBOL) && (LA251_4 <= PLUGINS_SYMBOL)) || ((LA251_4 >= PRESERVE_SYMBOL) && (LA251_4 <= PROXY_SYMBOL)) || ((LA251_4 >= QUERY_SYMBOL) && (LA251_4 <= RETURNS_SYMBOL)) || ((LA251_4 >= ROLLUP_SYMBOL) && (LA251_4 <= ROWS_SYMBOL)) || LA251_4 == ROW_FORMAT_SYMBOL || ((LA251_4 >= RTREE_SYMBOL) && (LA251_4 <= SCHEMA_NAME_SYMBOL)) || ((LA251_4 >= SERIAL_SYMBOL) && (LA251_4 <= SHARE_SYMBOL)) || ((LA251_4 >= SLOW_SYMBOL) && (LA251_4 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_4 >= SUBJECT_SYMBOL) && (LA251_4 <= TRIGGERS_SYMBOL)) || ((LA251_4 >= TYPES_SYMBOL) && (LA251_4 <= USER_RESOURCES_SYMBOL)) || ((LA251_4 >= USE_FRM_SYMBOL) && (LA251_4 <= WAIT_SYMBOL)) || LA251_4 == WORK_SYMBOL || ((LA251_4 >= X509_SYMBOL) && (LA251_4 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_501 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 501;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_502 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 502;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_503 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 503;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_504 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 504;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_505 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 505;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_506 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 506;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_4 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_507 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 507;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 4;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == NCHAR_TEXT))
                    {
                        {
                            int LA251_5 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 5;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == UNDERSCORE_CHARSET))
                    {
                        {
                            int LA251_6 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_6 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_538 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 538;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_6 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_539 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 539;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_6 == HEX_NUMBER))
                            {
                                {
                                    int LA251_540 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 540;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_6 == BIN_NUMBER))
                            {
                                {
                                    int LA251_541 = LA(3);
                                    if ( (( (( ((SERVER_VERSION >= 50003)) && ((SERVER_VERSION >= 50003)) )) && ((SERVER_VERSION < 50709)) )))
                                    {
                                        alt251=1;
                                    }
                                    else if ( (( (( ((SERVER_VERSION >= 50709)) && ((SERVER_VERSION >= 50709)) )) && ((SERVER_VERSION >= 50003)) )))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 541;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 6;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == SINGLE_QUOTED_TEXT))
                    {
                        {
                            int LA251_7 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 7;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DOUBLE_QUOTED_TEXT))
                    {
                        {
                            int LA251_8 = LA(2);
                            if ( (( (( ((((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)))||((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))) && ((((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)))||((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))) )) && ((SERVER_VERSION < 50709)) )))
                            {
                                alt251=1;
                            }
                            else if ( (( (( ((SERVER_VERSION >= 50709)) && ((SERVER_VERSION >= 50709)) )) && ((((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)))||((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))) )))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 8;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DECIMAL_NUMBER || LA251_0 == FLOAT_NUMBER || LA251_0 == INT_NUMBER || LA251_0 == LONG_NUMBER || LA251_0 == ULONGLONG_NUMBER))
                    {
                        {
                            int LA251_9 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 9;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DATE_SYMBOL))
                    {
                        {
                            int LA251_10 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 10;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == TIME_SYMBOL))
                    {
                        {
                            int LA251_11 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 11;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == TIMESTAMP_SYMBOL))
                    {
                        {
                            int LA251_12 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 12;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_0 >= NULL2_SYMBOL) && (LA251_0 <= NULL_SYMBOL))))
                    {
                        {
                            int LA251_13 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 13;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == FALSE_SYMBOL || LA251_0 == TRUE_SYMBOL))
                    {
                        {
                            int LA251_14 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 14;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == HEX_NUMBER))
                    {
                        {
                            int LA251_15 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 15;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BIN_NUMBER))
                    {
                        {
                            int LA251_16 = LA(2);
                            if ( (( (( ((SERVER_VERSION >= 50003)) && ((SERVER_VERSION >= 50003)) )) && ((SERVER_VERSION < 50709)) )))
                            {
                                alt251=1;
                            }
                            else if ( (( (( ((SERVER_VERSION >= 50709)) && ((SERVER_VERSION >= 50709)) )) && ((SERVER_VERSION >= 50003)) )))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 16;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == IDENTIFIER))
                    {
                        {
                            int LA251_17 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 17;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BACK_TICK_QUOTED_ID))
                    {
                        {
                            int LA251_18 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 18;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CHAR_SYMBOL))
                    {
                        {
                            int LA251_19 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_19 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_919 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 919;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 19;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CURRENT_USER_SYMBOL))
                    {
                        {
                            int LA251_20 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 20;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DAY_SYMBOL))
                    {
                        {
                            int LA251_21 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 21;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == HOUR_SYMBOL))
                    {
                        {
                            int LA251_22 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 22;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == INSERT_SYMBOL))
                    {
                        {
                            int LA251_23 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_23 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1016 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1016;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 23;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == LEFT_SYMBOL))
                    {
                        {
                            int LA251_24 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_24 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1017 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1017;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 24;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MINUTE_SYMBOL))
                    {
                        {
                            int LA251_25 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 25;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MONTH_SYMBOL))
                    {
                        {
                            int LA251_26 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 26;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == RIGHT_SYMBOL))
                    {
                        {
                            int LA251_27 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_27 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1084 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1084;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 27;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == SECOND_SYMBOL))
                    {
                        {
                            int LA251_28 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 28;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == TRIM_SYMBOL))
                    {
                        {
                            int LA251_29 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_29 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1118 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1118;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 29;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == USER_SYMBOL))
                    {
                        {
                            int LA251_30 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 30;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == VALUES_SYMBOL))
                    {
                        {
                            int LA251_31 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_31 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1152 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1152;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 31;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == YEAR_SYMBOL))
                    {
                        {
                            int LA251_32 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 32;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ADDDATE_SYMBOL || LA251_0 == SUBDATE_SYMBOL))
                    {
                        {
                            int LA251_33 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 33;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CURDATE_SYMBOL))
                    {
                        {
                            int LA251_34 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 34;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CURTIME_SYMBOL))
                    {
                        {
                            int LA251_35 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 35;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DATE_ADD_SYMBOL || LA251_0 == DATE_SUB_SYMBOL))
                    {
                        {
                            int LA251_36 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_36 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1279 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1279;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 36;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == EXTRACT_SYMBOL))
                    {
                        {
                            int LA251_37 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_37 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1280 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1280;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 37;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == GET_FORMAT_SYMBOL))
                    {
                        {
                            int LA251_38 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 38;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == NOW_SYMBOL))
                    {
                        {
                            int LA251_39 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 39;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == POSITION_SYMBOL))
                    {
                        {
                            int LA251_40 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_40 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1344 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1344;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 40;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == SUBSTRING_SYMBOL))
                    {
                        {
                            int LA251_41 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_41 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1345 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1345;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 41;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == SYSDATE_SYMBOL))
                    {
                        {
                            int LA251_42 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 42;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_0 >= TIMESTAMP_ADD_SYMBOL) && (LA251_0 <= TIMESTAMP_DIFF_SYMBOL))))
                    {
                        {
                            int LA251_43 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 43;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == UTC_DATE_SYMBOL))
                    {
                        {
                            int LA251_44 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 44;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == UTC_TIME_SYMBOL))
                    {
                        {
                            int LA251_45 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 45;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == UTC_TIMESTAMP_SYMBOL))
                    {
                        {
                            int LA251_46 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 46;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ASCII_SYMBOL))
                    {
                        {
                            int LA251_47 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 47;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CHARSET_SYMBOL))
                    {
                        {
                            int LA251_48 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 48;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == COALESCE_SYMBOL))
                    {
                        {
                            int LA251_49 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 49;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == COLLATION_SYMBOL))
                    {
                        {
                            int LA251_50 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 50;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DATABASE_SYMBOL))
                    {
                        {
                            int LA251_51 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_51 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1631 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1631;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 51;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == IF_SYMBOL))
                    {
                        {
                            int LA251_52 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_52 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1632 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1632;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 52;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == FORMAT_SYMBOL))
                    {
                        {
                            int LA251_53 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 53;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MICROSECOND_SYMBOL))
                    {
                        {
                            int LA251_54 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 54;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MOD_SYMBOL))
                    {
                        {
                            int LA251_55 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_55 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1699 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1699;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 55;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == OLD_PASSWORD_SYMBOL))
                    {
                        {
                            int LA251_56 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 56;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == PASSWORD_SYMBOL))
                    {
                        {
                            int LA251_57 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 57;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == QUARTER_SYMBOL))
                    {
                        {
                            int LA251_58 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 58;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == REPEAT_SYMBOL))
                    {
                        {
                            int LA251_59 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_59 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1799 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1799;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 59;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == REPLACE_SYMBOL))
                    {
                        {
                            int LA251_60 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_60 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_1800 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 1800;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 60;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == REVERSE_SYMBOL))
                    {
                        {
                            int LA251_61 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 61;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ROW_COUNT_SYMBOL))
                    {
                        {
                            int LA251_62 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 62;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == TRUNCATE_SYMBOL))
                    {
                        {
                            int LA251_63 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 63;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == WEEK_SYMBOL))
                    {
                        {
                            int LA251_64 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 64;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == WEIGHT_STRING_SYMBOL))
                    {
                        {
                            int LA251_65 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 65;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == GEOMETRYCOLLECTION_SYMBOL))
                    {
                        {
                            int LA251_66 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 66;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == LINESTRING_SYMBOL))
                    {
                        {
                            int LA251_67 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 67;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MULTILINESTRING_SYMBOL))
                    {
                        {
                            int LA251_68 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 68;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MULTIPOINT_SYMBOL))
                    {
                        {
                            int LA251_69 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 69;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MULTIPOLYGON_SYMBOL))
                    {
                        {
                            int LA251_70 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 70;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == POINT_SYMBOL))
                    {
                        {
                            int LA251_71 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 71;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == POLYGON_SYMBOL))
                    {
                        {
                            int LA251_72 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 72;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CONTAINS_SYMBOL))
                    {
                        {
                            int LA251_73 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 73;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == AVG_SYMBOL))
                    {
                        {
                            int LA251_74 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 74;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BIT_AND_SYMBOL))
                    {
                        {
                            int LA251_75 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_75 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2263 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2263;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 75;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BIT_OR_SYMBOL))
                    {
                        {
                            int LA251_76 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_76 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2264 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2264;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 76;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BIT_XOR_SYMBOL))
                    {
                        {
                            int LA251_77 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_77 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2265 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2265;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 77;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == COUNT_SYMBOL))
                    {
                        {
                            int LA251_78 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_78 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2266 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2266;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 78;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MIN_SYMBOL))
                    {
                        {
                            int LA251_79 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_79 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2267 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2267;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 79;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MAX_SYMBOL))
                    {
                        {
                            int LA251_80 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_80 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2268 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2268;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 80;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == STD_SYMBOL))
                    {
                        {
                            int LA251_81 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_81 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2269 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2269;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 81;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == VARIANCE_SYMBOL))
                    {
                        {
                            int LA251_82 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_82 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2270 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2270;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 82;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == STDDEV_SAMP_SYMBOL))
                    {
                        {
                            int LA251_83 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_83 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2271 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2271;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 83;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == VAR_SAMP_SYMBOL))
                    {
                        {
                            int LA251_84 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_84 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2272 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2272;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 84;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == SUM_SYMBOL))
                    {
                        {
                            int LA251_85 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_85 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2273 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2273;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 85;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == GROUP_CONCAT_SYMBOL))
                    {
                        {
                            int LA251_86 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_86 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2274 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2274;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 86;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == DOT_SYMBOL))
                    {
                        {
                            int LA251_87 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_87 == IDENTIFIER))
                            {
                                {
                                    int LA251_2275 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2275;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_87 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2276 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2276;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_87 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2277 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2277;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_87 >= ACTION_SYMBOL) && (LA251_87 <= SHARE_SYMBOL)) || ((LA251_87 >= SLOW_SYMBOL) && (LA251_87 <= YEAR_SYMBOL))))
                            {
                                {
                                    int LA251_2278 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2278;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_87 >= ACCOUNT_SYMBOL) && (LA251_87 <= XA_SYMBOL)) || LA251_87 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2279 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2279;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 87;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ROW_SYMBOL))
                    {
                        {
                            int LA251_88 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 88;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ACCOUNT_SYMBOL || ((LA251_0 >= ALWAYS_SYMBOL) && (LA251_0 <= CACHE_SYMBOL)) || ((LA251_0 >= CHECKSUM_SYMBOL) && (LA251_0 <= COMMIT_SYMBOL)) || ((LA251_0 >= DEALLOCATE_SYMBOL) && (LA251_0 <= FOLLOWS_SYMBOL)) || ((LA251_0 >= GROUP_REPLICATION_SYMBOL) && (LA251_0 <= STOP_SYMBOL)) || ((LA251_0 >= UNICODE_SYMBOL) && (LA251_0 <= XA_SYMBOL)) || LA251_0 == SHUTDOWN_SYMBOL))
                    {
                        {
                            int LA251_89 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 89;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == PARAM_MARKER))
                    {
                        {
                            int LA251_90 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 90;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == AT_SIGN_SYMBOL))
                    {
                        {
                            int LA251_91 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_91 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2374 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2374;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_91 == IDENTIFIER))
                            {
                                {
                                    int LA251_2375 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2375;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_91 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2376 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2376;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_91 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2377 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2377;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_91 >= ACTION_SYMBOL) && (LA251_91 <= SHARE_SYMBOL)) || ((LA251_91 >= SLOW_SYMBOL) && (LA251_91 <= YEAR_SYMBOL))))
                            {
                                {
                                    int LA251_2378 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2378;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_91 >= ACCOUNT_SYMBOL) && (LA251_91 <= XA_SYMBOL)) || LA251_91 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2379 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2379;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 91;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == AT_TEXT_SUFFIX))
                    {
                        {
                            int LA251_92 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 92;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == AT_AT_SIGN_SYMBOL))
                    {
                        {
                            int LA251_93 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_93 == GLOBAL_SYMBOL || LA251_93 == LOCAL_SYMBOL || LA251_93 == SESSION_SYMBOL))
                            {
                                {
                                    int LA251_2410 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2410;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_93 == IDENTIFIER))
                            {
                                {
                                    int LA251_2411 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2411;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_93 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2412 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2412;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_93 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2413 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2413;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_93 >= ACTION_SYMBOL) && (LA251_93 <= GRANTS_SYMBOL)) || ((LA251_93 >= HASH_SYMBOL) && (LA251_93 <= LIST_SYMBOL)) || ((LA251_93 >= LOCKS_SYMBOL) && (LA251_93 <= SERIALIZABLE_SYMBOL)) || ((LA251_93 >= SIMPLE_SYMBOL) && (LA251_93 <= SHARE_SYMBOL)) || ((LA251_93 >= SLOW_SYMBOL) && (LA251_93 <= YEAR_SYMBOL))))
                            {
                                {
                                    int LA251_2414 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2414;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_93 >= ACCOUNT_SYMBOL) && (LA251_93 <= XA_SYMBOL)) || LA251_93 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2415 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2415;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_93 == DEFAULT_SYMBOL))
                            {
                                {
                                    int LA251_2416 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2416;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 93;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == EXISTS_SYMBOL))
                    {
                        {
                            int LA251_94 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_94 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2417 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2417;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 94;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == OPEN_PAR_SYMBOL))
                    {
                        {
                            int LA251_95 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SELECT_SYMBOL))
                            {
                                {
                                    int LA251_2418 = LA(3);
                                    if ( (( (( ((SERVER_VERSION < 50709)) && ((SERVER_VERSION < 50709)) )) && ((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)) )))
                                    {
                                        alt251=1;
                                    }
                                    else if ( (( (( ((SERVER_VERSION >= 50709)) && ((SERVER_VERSION >= 50709)) )) && ((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)) )))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2418;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2419 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2419;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == NOT_SYMBOL))
                            {
                                {
                                    int LA251_2420 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2420;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BITWISE_NOT_OPERATOR || LA251_95 == MINUS_OPERATOR || LA251_95 == PLUS_OPERATOR))
                            {
                                {
                                    int LA251_2421 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2421;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == NOT2_SYMBOL || LA251_95 == LOGICAL_NOT_OPERATOR))
                            {
                                {
                                    int LA251_2422 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2422;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_2423 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2423;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_2424 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2424;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_2425 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2425;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2426 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2426;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2427 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2427;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DECIMAL_NUMBER || LA251_95 == FLOAT_NUMBER || LA251_95 == INT_NUMBER || LA251_95 == LONG_NUMBER || LA251_95 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_2428 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2428;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_2429 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2429;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_2430 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2430;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2431 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2431;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_95 >= NULL2_SYMBOL) && (LA251_95 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_2432 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2432;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == FALSE_SYMBOL || LA251_95 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_2433 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2433;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == HEX_NUMBER))
                            {
                                {
                                    int LA251_2434 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2434;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BIN_NUMBER))
                            {
                                {
                                    int LA251_2435 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2435;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == IDENTIFIER))
                            {
                                {
                                    int LA251_2436 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2436;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2437 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2437;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_2438 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2438;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_2439 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2439;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_2440 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2440;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_2441 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2441;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_2442 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2442;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_2443 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2443;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_2444 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2444;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_2445 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2445;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_2446 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2446;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_2447 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2447;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_2448 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2448;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == USER_SYMBOL))
                            {
                                {
                                    int LA251_2449 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2449;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_2450 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2450;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_2451 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2451;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ADDDATE_SYMBOL || LA251_95 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_2452 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2452;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_2453 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2453;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_2454 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2454;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DATE_ADD_SYMBOL || LA251_95 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_2455 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2455;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_2456 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2456;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2457 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2457;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_2458 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2458;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_2459 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2459;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_2460 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2460;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_2461 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2461;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_95 >= TIMESTAMP_ADD_SYMBOL) && (LA251_95 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_2462 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2462;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_2463 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2463;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_2464 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2464;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2465 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2465;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_2466 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2466;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_2467 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2467;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_2468 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2468;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_2469 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2469;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_2470 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2470;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == IF_SYMBOL))
                            {
                                {
                                    int LA251_2471 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2471;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2472 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2472;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_2473 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2473;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_2474 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2474;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2475 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2475;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2476 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2476;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_2477 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2477;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_2478 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2478;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_2479 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2479;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_2480 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2480;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2481 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2481;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_2482 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2482;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_2483 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2483;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_2484 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2484;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_2485 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2485;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2486 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2486;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2487 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2487;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_2488 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2488;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2489 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2489;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_2490 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2490;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2491 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2491;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_2492 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2492;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_2493 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2493;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_2494 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2494;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_2495 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2495;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_2496 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2496;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2497 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2497;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_2498 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2498;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_2499 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2499;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == STD_SYMBOL))
                            {
                                {
                                    int LA251_2500 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2500;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_2501 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2501;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2502 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2502;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2503 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2503;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_2504 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2504;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_2505 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2505;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_2506 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2506;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_2507 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2507;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ACCOUNT_SYMBOL || ((LA251_95 >= ALWAYS_SYMBOL) && (LA251_95 <= CACHE_SYMBOL)) || ((LA251_95 >= CHECKSUM_SYMBOL) && (LA251_95 <= COMMIT_SYMBOL)) || ((LA251_95 >= DEALLOCATE_SYMBOL) && (LA251_95 <= FOLLOWS_SYMBOL)) || ((LA251_95 >= GROUP_REPLICATION_SYMBOL) && (LA251_95 <= STOP_SYMBOL)) || ((LA251_95 >= UNICODE_SYMBOL) && (LA251_95 <= XA_SYMBOL)) || LA251_95 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2508 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2508;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == PARAM_MARKER))
                            {
                                {
                                    int LA251_2509 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2509;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2510 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2510;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_2511 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2511;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2512 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2512;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_2513 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2513;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == ACTION_SYMBOL || ((LA251_95 >= AFTER_SYMBOL) && (LA251_95 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_95 >= BINLOG_SYMBOL) && (LA251_95 <= CLASS_ORIGIN_SYMBOL)) || LA251_95 == CODE_SYMBOL || ((LA251_95 >= COLUMN_NAME_SYMBOL) && (LA251_95 <= DATETIME_SYMBOL)) || ((LA251_95 >= DEFAULT_AUTH_SYMBOL) && (LA251_95 <= GEOMETRY_SYMBOL)) || ((LA251_95 >= GRANTS_SYMBOL) && (LA251_95 <= HOSTS_SYMBOL)) || ((LA251_95 >= IDENTIFIED_SYMBOL) && (LA251_95 <= LEVEL_SYMBOL)) || ((LA251_95 >= LIST_SYMBOL) && (LA251_95 <= MESSAGE_TEXT_SYMBOL)) || LA251_95 == MIGRATE_SYMBOL || ((LA251_95 >= MIN_ROWS_SYMBOL) && (LA251_95 <= MODE_SYMBOL)) || ((LA251_95 >= MUTEX_SYMBOL) && (LA251_95 <= OFFSET_SYMBOL)) || ((LA251_95 >= ONE_SHOT_SYMBOL) && (LA251_95 <= PARTITIONS_SYMBOL)) || ((LA251_95 >= PHASE_SYMBOL) && (LA251_95 <= PLUGINS_SYMBOL)) || ((LA251_95 >= PRESERVE_SYMBOL) && (LA251_95 <= PROXY_SYMBOL)) || ((LA251_95 >= QUERY_SYMBOL) && (LA251_95 <= RETURNS_SYMBOL)) || ((LA251_95 >= ROLLUP_SYMBOL) && (LA251_95 <= ROWS_SYMBOL)) || LA251_95 == ROW_FORMAT_SYMBOL || ((LA251_95 >= RTREE_SYMBOL) && (LA251_95 <= SCHEMA_NAME_SYMBOL)) || ((LA251_95 >= SERIAL_SYMBOL) && (LA251_95 <= SHARE_SYMBOL)) || ((LA251_95 >= SLOW_SYMBOL) && (LA251_95 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_95 >= SUBJECT_SYMBOL) && (LA251_95 <= TRIGGERS_SYMBOL)) || ((LA251_95 >= TYPES_SYMBOL) && (LA251_95 <= USER_RESOURCES_SYMBOL)) || ((LA251_95 >= USE_FRM_SYMBOL) && (LA251_95 <= WAIT_SYMBOL)) || LA251_95 == WORK_SYMBOL || ((LA251_95 >= X509_SYMBOL) && (LA251_95 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_2514 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2514;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_2515 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2515;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_2516 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2516;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_2517 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2517;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_2518 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2518;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_2519 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2519;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_95 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_2520 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2520;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 95;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == ACTION_SYMBOL || ((LA251_0 >= AFTER_SYMBOL) && (LA251_0 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_0 >= BINLOG_SYMBOL) && (LA251_0 <= CLASS_ORIGIN_SYMBOL)) || LA251_0 == CODE_SYMBOL || ((LA251_0 >= COLUMN_NAME_SYMBOL) && (LA251_0 <= DATETIME_SYMBOL)) || ((LA251_0 >= DEFAULT_AUTH_SYMBOL) && (LA251_0 <= GEOMETRY_SYMBOL)) || ((LA251_0 >= GRANTS_SYMBOL) && (LA251_0 <= HOSTS_SYMBOL)) || ((LA251_0 >= IDENTIFIED_SYMBOL) && (LA251_0 <= LEVEL_SYMBOL)) || ((LA251_0 >= LIST_SYMBOL) && (LA251_0 <= MESSAGE_TEXT_SYMBOL)) || LA251_0 == MIGRATE_SYMBOL || ((LA251_0 >= MIN_ROWS_SYMBOL) && (LA251_0 <= MODE_SYMBOL)) || ((LA251_0 >= MUTEX_SYMBOL) && (LA251_0 <= OFFSET_SYMBOL)) || ((LA251_0 >= ONE_SHOT_SYMBOL) && (LA251_0 <= PARTITIONS_SYMBOL)) || ((LA251_0 >= PHASE_SYMBOL) && (LA251_0 <= PLUGINS_SYMBOL)) || ((LA251_0 >= PRESERVE_SYMBOL) && (LA251_0 <= PROXY_SYMBOL)) || ((LA251_0 >= QUERY_SYMBOL) && (LA251_0 <= RETURNS_SYMBOL)) || ((LA251_0 >= ROLLUP_SYMBOL) && (LA251_0 <= ROWS_SYMBOL)) || LA251_0 == ROW_FORMAT_SYMBOL || ((LA251_0 >= RTREE_SYMBOL) && (LA251_0 <= SCHEMA_NAME_SYMBOL)) || ((LA251_0 >= SERIAL_SYMBOL) && (LA251_0 <= SHARE_SYMBOL)) || ((LA251_0 >= SLOW_SYMBOL) && (LA251_0 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_0 >= SUBJECT_SYMBOL) && (LA251_0 <= TRIGGERS_SYMBOL)) || ((LA251_0 >= TYPES_SYMBOL) && (LA251_0 <= USER_RESOURCES_SYMBOL)) || ((LA251_0 >= USE_FRM_SYMBOL) && (LA251_0 <= WAIT_SYMBOL)) || LA251_0 == WORK_SYMBOL || ((LA251_0 >= X509_SYMBOL) && (LA251_0 <= XML_SYMBOL))))
                    {
                        {
                            int LA251_96 = LA(2);
                            if ( ((SERVER_VERSION < 50709)))
                            {
                                alt251=1;
                            }
                            else if ( ((SERVER_VERSION >= 50709)))
                            {
                                alt251=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 96;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == OPEN_CURLY_SYMBOL))
                    {
                        {
                            int LA251_97 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_97 == IDENTIFIER))
                            {
                                {
                                    int LA251_2553 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2553;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_97 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2554 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2554;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_97 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2555 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2555;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_97 >= ACTION_SYMBOL) && (LA251_97 <= SHARE_SYMBOL)) || ((LA251_97 >= SLOW_SYMBOL) && (LA251_97 <= YEAR_SYMBOL))))
                            {
                                {
                                    int LA251_2556 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2556;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_97 >= ACCOUNT_SYMBOL) && (LA251_97 <= XA_SYMBOL)) || LA251_97 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2557 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2557;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 97;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == MATCH_SYMBOL))
                    {
                        {
                            int LA251_98 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_98 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2558 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2558;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 98;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CASE_SYMBOL))
                    {
                        {
                            int LA251_99 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == NOT_SYMBOL))
                            {
                                {
                                    int LA251_2559 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2559;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BITWISE_NOT_OPERATOR || LA251_99 == MINUS_OPERATOR || LA251_99 == PLUS_OPERATOR))
                            {
                                {
                                    int LA251_2560 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2560;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == NOT2_SYMBOL || LA251_99 == LOGICAL_NOT_OPERATOR))
                            {
                                {
                                    int LA251_2561 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2561;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == INTERVAL_SYMBOL))
                            {
                                {
                                    int LA251_2562 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2562;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_2563 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2563;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_2564 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2564;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2565 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2565;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2566 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2566;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DECIMAL_NUMBER || LA251_99 == FLOAT_NUMBER || LA251_99 == INT_NUMBER || LA251_99 == LONG_NUMBER || LA251_99 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_2567 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2567;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_2568 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2568;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_2569 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2569;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2570 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2570;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_99 >= NULL2_SYMBOL) && (LA251_99 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_2571 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2571;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == FALSE_SYMBOL || LA251_99 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_2572 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2572;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == HEX_NUMBER))
                            {
                                {
                                    int LA251_2573 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2573;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BIN_NUMBER))
                            {
                                {
                                    int LA251_2574 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2574;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == IDENTIFIER))
                            {
                                {
                                    int LA251_2575 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2575;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2576 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2576;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_2577 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2577;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_2578 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2578;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_2579 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2579;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_2580 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2580;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_2581 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2581;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_2582 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2582;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_2583 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2583;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_2584 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2584;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_2585 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2585;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_2586 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2586;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_2587 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2587;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == USER_SYMBOL))
                            {
                                {
                                    int LA251_2588 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2588;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_2589 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2589;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_2590 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2590;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ADDDATE_SYMBOL || LA251_99 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_2591 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2591;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_2592 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2592;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_2593 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2593;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DATE_ADD_SYMBOL || LA251_99 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_2594 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2594;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_2595 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2595;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2596 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2596;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_2597 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2597;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_2598 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2598;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_2599 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2599;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_2600 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2600;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_99 >= TIMESTAMP_ADD_SYMBOL) && (LA251_99 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_2601 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2601;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_2602 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2602;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_2603 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2603;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2604 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2604;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_2605 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2605;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_2606 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2606;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_2607 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2607;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_2608 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2608;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_2609 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2609;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == IF_SYMBOL))
                            {
                                {
                                    int LA251_2610 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2610;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2611 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2611;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_2612 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2612;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_2613 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2613;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2614 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2614;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2615 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2615;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_2616 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2616;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_2617 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2617;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_2618 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2618;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_2619 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2619;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2620 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2620;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_2621 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2621;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_2622 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2622;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_2623 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2623;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_2624 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2624;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2625 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2625;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2626 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2626;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_2627 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2627;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2628 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2628;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_2629 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2629;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2630 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2630;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_2631 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2631;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_2632 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2632;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_2633 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2633;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_2634 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2634;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_2635 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2635;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2636 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2636;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_2637 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2637;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_2638 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2638;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == STD_SYMBOL))
                            {
                                {
                                    int LA251_2639 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2639;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_2640 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2640;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2641 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2641;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2642 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2642;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_2643 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2643;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_2644 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2644;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_2645 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2645;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_2646 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2646;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ACCOUNT_SYMBOL || ((LA251_99 >= ALWAYS_SYMBOL) && (LA251_99 <= CACHE_SYMBOL)) || ((LA251_99 >= CHECKSUM_SYMBOL) && (LA251_99 <= COMMIT_SYMBOL)) || ((LA251_99 >= DEALLOCATE_SYMBOL) && (LA251_99 <= FOLLOWS_SYMBOL)) || ((LA251_99 >= GROUP_REPLICATION_SYMBOL) && (LA251_99 <= STOP_SYMBOL)) || ((LA251_99 >= UNICODE_SYMBOL) && (LA251_99 <= XA_SYMBOL)) || LA251_99 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2647 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2647;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == PARAM_MARKER))
                            {
                                {
                                    int LA251_2648 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2648;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2649 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2649;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_2650 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2650;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2651 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2651;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_2652 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2652;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2653 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2653;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == ACTION_SYMBOL || ((LA251_99 >= AFTER_SYMBOL) && (LA251_99 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_99 >= BINLOG_SYMBOL) && (LA251_99 <= CLASS_ORIGIN_SYMBOL)) || LA251_99 == CODE_SYMBOL || ((LA251_99 >= COLUMN_NAME_SYMBOL) && (LA251_99 <= DATETIME_SYMBOL)) || ((LA251_99 >= DEFAULT_AUTH_SYMBOL) && (LA251_99 <= GEOMETRY_SYMBOL)) || ((LA251_99 >= GRANTS_SYMBOL) && (LA251_99 <= HOSTS_SYMBOL)) || ((LA251_99 >= IDENTIFIED_SYMBOL) && (LA251_99 <= LEVEL_SYMBOL)) || ((LA251_99 >= LIST_SYMBOL) && (LA251_99 <= MESSAGE_TEXT_SYMBOL)) || LA251_99 == MIGRATE_SYMBOL || ((LA251_99 >= MIN_ROWS_SYMBOL) && (LA251_99 <= MODE_SYMBOL)) || ((LA251_99 >= MUTEX_SYMBOL) && (LA251_99 <= OFFSET_SYMBOL)) || ((LA251_99 >= ONE_SHOT_SYMBOL) && (LA251_99 <= PARTITIONS_SYMBOL)) || ((LA251_99 >= PHASE_SYMBOL) && (LA251_99 <= PLUGINS_SYMBOL)) || ((LA251_99 >= PRESERVE_SYMBOL) && (LA251_99 <= PROXY_SYMBOL)) || ((LA251_99 >= QUERY_SYMBOL) && (LA251_99 <= RETURNS_SYMBOL)) || ((LA251_99 >= ROLLUP_SYMBOL) && (LA251_99 <= ROWS_SYMBOL)) || LA251_99 == ROW_FORMAT_SYMBOL || ((LA251_99 >= RTREE_SYMBOL) && (LA251_99 <= SCHEMA_NAME_SYMBOL)) || ((LA251_99 >= SERIAL_SYMBOL) && (LA251_99 <= SHARE_SYMBOL)) || ((LA251_99 >= SLOW_SYMBOL) && (LA251_99 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_99 >= SUBJECT_SYMBOL) && (LA251_99 <= TRIGGERS_SYMBOL)) || ((LA251_99 >= TYPES_SYMBOL) && (LA251_99 <= USER_RESOURCES_SYMBOL)) || ((LA251_99 >= USE_FRM_SYMBOL) && (LA251_99 <= WAIT_SYMBOL)) || LA251_99 == WORK_SYMBOL || ((LA251_99 >= X509_SYMBOL) && (LA251_99 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_2654 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2654;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_2655 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2655;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_2656 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2656;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_2657 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2657;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_2658 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2658;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_2659 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2659;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_2660 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2660;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_99 == WHEN_SYMBOL))
                            {
                                {
                                    int LA251_2661 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2661;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 99;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == BINARY_SYMBOL))
                    {
                        {
                            int LA251_100 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == NCHAR_TEXT))
                            {
                                {
                                    int LA251_2662 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2662;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == UNDERSCORE_CHARSET))
                            {
                                {
                                    int LA251_2663 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2663;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2664 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2664;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DOUBLE_QUOTED_TEXT))
                            {
                                {
                                    int LA251_2665 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2665;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DECIMAL_NUMBER || LA251_100 == FLOAT_NUMBER || LA251_100 == INT_NUMBER || LA251_100 == LONG_NUMBER || LA251_100 == ULONGLONG_NUMBER))
                            {
                                {
                                    int LA251_2666 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2666;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DATE_SYMBOL))
                            {
                                {
                                    int LA251_2667 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2667;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == TIME_SYMBOL))
                            {
                                {
                                    int LA251_2668 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2668;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2669 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2669;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_100 >= NULL2_SYMBOL) && (LA251_100 <= NULL_SYMBOL))))
                            {
                                {
                                    int LA251_2670 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2670;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == FALSE_SYMBOL || LA251_100 == TRUE_SYMBOL))
                            {
                                {
                                    int LA251_2671 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2671;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == HEX_NUMBER))
                            {
                                {
                                    int LA251_2672 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2672;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BIN_NUMBER))
                            {
                                {
                                    int LA251_2673 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2673;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == IDENTIFIER))
                            {
                                {
                                    int LA251_2674 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2674;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BACK_TICK_QUOTED_ID))
                            {
                                {
                                    int LA251_2675 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2675;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CHAR_SYMBOL))
                            {
                                {
                                    int LA251_2676 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2676;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CURRENT_USER_SYMBOL))
                            {
                                {
                                    int LA251_2677 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2677;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DAY_SYMBOL))
                            {
                                {
                                    int LA251_2678 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2678;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == HOUR_SYMBOL))
                            {
                                {
                                    int LA251_2679 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2679;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == INSERT_SYMBOL))
                            {
                                {
                                    int LA251_2680 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2680;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == LEFT_SYMBOL))
                            {
                                {
                                    int LA251_2681 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2681;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MINUTE_SYMBOL))
                            {
                                {
                                    int LA251_2682 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2682;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MONTH_SYMBOL))
                            {
                                {
                                    int LA251_2683 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2683;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == RIGHT_SYMBOL))
                            {
                                {
                                    int LA251_2684 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2684;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == SECOND_SYMBOL))
                            {
                                {
                                    int LA251_2685 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2685;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == TRIM_SYMBOL))
                            {
                                {
                                    int LA251_2686 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2686;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == USER_SYMBOL))
                            {
                                {
                                    int LA251_2687 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2687;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == VALUES_SYMBOL))
                            {
                                {
                                    int LA251_2688 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2688;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == YEAR_SYMBOL))
                            {
                                {
                                    int LA251_2689 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2689;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ADDDATE_SYMBOL || LA251_100 == SUBDATE_SYMBOL))
                            {
                                {
                                    int LA251_2690 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2690;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CURDATE_SYMBOL))
                            {
                                {
                                    int LA251_2691 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2691;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CURTIME_SYMBOL))
                            {
                                {
                                    int LA251_2692 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2692;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DATE_ADD_SYMBOL || LA251_100 == DATE_SUB_SYMBOL))
                            {
                                {
                                    int LA251_2693 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2693;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == EXTRACT_SYMBOL))
                            {
                                {
                                    int LA251_2694 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2694;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == GET_FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2695 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2695;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == NOW_SYMBOL))
                            {
                                {
                                    int LA251_2696 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2696;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == POSITION_SYMBOL))
                            {
                                {
                                    int LA251_2697 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2697;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == SUBSTRING_SYMBOL))
                            {
                                {
                                    int LA251_2698 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2698;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == SYSDATE_SYMBOL))
                            {
                                {
                                    int LA251_2699 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2699;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (((LA251_100 >= TIMESTAMP_ADD_SYMBOL) && (LA251_100 <= TIMESTAMP_DIFF_SYMBOL))))
                            {
                                {
                                    int LA251_2700 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2700;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == UTC_DATE_SYMBOL))
                            {
                                {
                                    int LA251_2701 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2701;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == UTC_TIME_SYMBOL))
                            {
                                {
                                    int LA251_2702 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2702;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == UTC_TIMESTAMP_SYMBOL))
                            {
                                {
                                    int LA251_2703 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2703;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ASCII_SYMBOL))
                            {
                                {
                                    int LA251_2704 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2704;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CHARSET_SYMBOL))
                            {
                                {
                                    int LA251_2705 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2705;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == COALESCE_SYMBOL))
                            {
                                {
                                    int LA251_2706 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2706;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == COLLATION_SYMBOL))
                            {
                                {
                                    int LA251_2707 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2707;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DATABASE_SYMBOL))
                            {
                                {
                                    int LA251_2708 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2708;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == IF_SYMBOL))
                            {
                                {
                                    int LA251_2709 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2709;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == FORMAT_SYMBOL))
                            {
                                {
                                    int LA251_2710 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2710;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MICROSECOND_SYMBOL))
                            {
                                {
                                    int LA251_2711 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2711;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MOD_SYMBOL))
                            {
                                {
                                    int LA251_2712 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2712;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == OLD_PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2713 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2713;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == PASSWORD_SYMBOL))
                            {
                                {
                                    int LA251_2714 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2714;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == QUARTER_SYMBOL))
                            {
                                {
                                    int LA251_2715 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2715;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == REPEAT_SYMBOL))
                            {
                                {
                                    int LA251_2716 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2716;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == REPLACE_SYMBOL))
                            {
                                {
                                    int LA251_2717 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2717;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == REVERSE_SYMBOL))
                            {
                                {
                                    int LA251_2718 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2718;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ROW_COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2719 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2719;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == TRUNCATE_SYMBOL))
                            {
                                {
                                    int LA251_2720 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2720;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == WEEK_SYMBOL))
                            {
                                {
                                    int LA251_2721 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2721;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == WEIGHT_STRING_SYMBOL))
                            {
                                {
                                    int LA251_2722 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2722;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == GEOMETRYCOLLECTION_SYMBOL))
                            {
                                {
                                    int LA251_2723 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2723;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == LINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2724 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2724;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MULTILINESTRING_SYMBOL))
                            {
                                {
                                    int LA251_2725 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2725;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MULTIPOINT_SYMBOL))
                            {
                                {
                                    int LA251_2726 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2726;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MULTIPOLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2727 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2727;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == POINT_SYMBOL))
                            {
                                {
                                    int LA251_2728 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2728;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == POLYGON_SYMBOL))
                            {
                                {
                                    int LA251_2729 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2729;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CONTAINS_SYMBOL))
                            {
                                {
                                    int LA251_2730 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2730;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == AVG_SYMBOL))
                            {
                                {
                                    int LA251_2731 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2731;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BIT_AND_SYMBOL))
                            {
                                {
                                    int LA251_2732 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2732;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BIT_OR_SYMBOL))
                            {
                                {
                                    int LA251_2733 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2733;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BIT_XOR_SYMBOL))
                            {
                                {
                                    int LA251_2734 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2734;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == COUNT_SYMBOL))
                            {
                                {
                                    int LA251_2735 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2735;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MIN_SYMBOL))
                            {
                                {
                                    int LA251_2736 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2736;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MAX_SYMBOL))
                            {
                                {
                                    int LA251_2737 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2737;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == STD_SYMBOL))
                            {
                                {
                                    int LA251_2738 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2738;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == VARIANCE_SYMBOL))
                            {
                                {
                                    int LA251_2739 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2739;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == STDDEV_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2740 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2740;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == VAR_SAMP_SYMBOL))
                            {
                                {
                                    int LA251_2741 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2741;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == SUM_SYMBOL))
                            {
                                {
                                    int LA251_2742 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2742;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == GROUP_CONCAT_SYMBOL))
                            {
                                {
                                    int LA251_2743 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2743;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == DOT_SYMBOL))
                            {
                                {
                                    int LA251_2744 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2744;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ROW_SYMBOL))
                            {
                                {
                                    int LA251_2745 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2745;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ACCOUNT_SYMBOL || ((LA251_100 >= ALWAYS_SYMBOL) && (LA251_100 <= CACHE_SYMBOL)) || ((LA251_100 >= CHECKSUM_SYMBOL) && (LA251_100 <= COMMIT_SYMBOL)) || ((LA251_100 >= DEALLOCATE_SYMBOL) && (LA251_100 <= FOLLOWS_SYMBOL)) || ((LA251_100 >= GROUP_REPLICATION_SYMBOL) && (LA251_100 <= STOP_SYMBOL)) || ((LA251_100 >= UNICODE_SYMBOL) && (LA251_100 <= XA_SYMBOL)) || LA251_100 == SHUTDOWN_SYMBOL))
                            {
                                {
                                    int LA251_2746 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2746;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == PARAM_MARKER))
                            {
                                {
                                    int LA251_2747 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2747;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2748 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2748;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == AT_TEXT_SUFFIX))
                            {
                                {
                                    int LA251_2749 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2749;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == AT_AT_SIGN_SYMBOL))
                            {
                                {
                                    int LA251_2750 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2750;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == EXISTS_SYMBOL))
                            {
                                {
                                    int LA251_2751 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2751;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2752 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2752;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == ACTION_SYMBOL || ((LA251_100 >= AFTER_SYMBOL) && (LA251_100 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA251_100 >= BINLOG_SYMBOL) && (LA251_100 <= CLASS_ORIGIN_SYMBOL)) || LA251_100 == CODE_SYMBOL || ((LA251_100 >= COLUMN_NAME_SYMBOL) && (LA251_100 <= DATETIME_SYMBOL)) || ((LA251_100 >= DEFAULT_AUTH_SYMBOL) && (LA251_100 <= GEOMETRY_SYMBOL)) || ((LA251_100 >= GRANTS_SYMBOL) && (LA251_100 <= HOSTS_SYMBOL)) || ((LA251_100 >= IDENTIFIED_SYMBOL) && (LA251_100 <= LEVEL_SYMBOL)) || ((LA251_100 >= LIST_SYMBOL) && (LA251_100 <= MESSAGE_TEXT_SYMBOL)) || LA251_100 == MIGRATE_SYMBOL || ((LA251_100 >= MIN_ROWS_SYMBOL) && (LA251_100 <= MODE_SYMBOL)) || ((LA251_100 >= MUTEX_SYMBOL) && (LA251_100 <= OFFSET_SYMBOL)) || ((LA251_100 >= ONE_SHOT_SYMBOL) && (LA251_100 <= PARTITIONS_SYMBOL)) || ((LA251_100 >= PHASE_SYMBOL) && (LA251_100 <= PLUGINS_SYMBOL)) || ((LA251_100 >= PRESERVE_SYMBOL) && (LA251_100 <= PROXY_SYMBOL)) || ((LA251_100 >= QUERY_SYMBOL) && (LA251_100 <= RETURNS_SYMBOL)) || ((LA251_100 >= ROLLUP_SYMBOL) && (LA251_100 <= ROWS_SYMBOL)) || LA251_100 == ROW_FORMAT_SYMBOL || ((LA251_100 >= RTREE_SYMBOL) && (LA251_100 <= SCHEMA_NAME_SYMBOL)) || ((LA251_100 >= SERIAL_SYMBOL) && (LA251_100 <= SHARE_SYMBOL)) || ((LA251_100 >= SLOW_SYMBOL) && (LA251_100 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA251_100 >= SUBJECT_SYMBOL) && (LA251_100 <= TRIGGERS_SYMBOL)) || ((LA251_100 >= TYPES_SYMBOL) && (LA251_100 <= USER_RESOURCES_SYMBOL)) || ((LA251_100 >= USE_FRM_SYMBOL) && (LA251_100 <= WAIT_SYMBOL)) || LA251_100 == WORK_SYMBOL || ((LA251_100 >= X509_SYMBOL) && (LA251_100 <= XML_SYMBOL))))
                            {
                                {
                                    int LA251_2753 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2753;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == OPEN_CURLY_SYMBOL))
                            {
                                {
                                    int LA251_2754 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2754;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == MATCH_SYMBOL))
                            {
                                {
                                    int LA251_2755 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2755;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CASE_SYMBOL))
                            {
                                {
                                    int LA251_2756 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2756;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == BINARY_SYMBOL))
                            {
                                {
                                    int LA251_2757 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2757;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CAST_SYMBOL))
                            {
                                {
                                    int LA251_2758 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2758;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_100 == CONVERT_SYMBOL))
                            {
                                {
                                    int LA251_2759 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2759;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 100;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CAST_SYMBOL))
                    {
                        {
                            int LA251_101 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_101 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2760 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2760;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 101;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_0 == CONVERT_SYMBOL))
                    {
                        {
                            int LA251_102 = LA(2);
                            if ( ((((SERVER_VERSION >= 50709))||((SERVER_VERSION < 50709)))) && (LA251_102 == OPEN_PAR_SYMBOL))
                            {
                                {
                                    int LA251_2761 = LA(3);
                                    if ( ((SERVER_VERSION < 50709)))
                                    {
                                        alt251=1;
                                    }
                                    else if ( ((SERVER_VERSION >= 50709)))
                                    {
                                        alt251=2;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 251;
                                        EXCEPTION->state        = 2761;


                                        goto ruledo_statementEx;

                                    }
                                }
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 251;
                                EXCEPTION->state        = 102;


                                goto ruledo_statementEx;

                            }
                        }
                    }
                    else if ( ((SERVER_VERSION >= 50709)) && (LA251_0 == MULT_OPERATOR))
                    {
                        alt251=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 251;
                        EXCEPTION->state        = 0;


                        goto ruledo_statementEx;

                    }
                }
                switch (alt251)
                {
            	case 1:
            	    // MySQLSimpleParser.g:674:3: {...}? => expression_list
            	    {
            	        if ( !((SERVER_VERSION < 50709)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50709";
            	                EXCEPTION->ruleName	 = (void *)"do_statement";


            	        }

            	        FOLLOWPUSH(FOLLOW_expression_list_in_do_statement3765);
            	        expression_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledo_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:675:5: {...}? => select_item_list
            	    {
            	        if ( !((SERVER_VERSION >= 50709)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50709";
            	                EXCEPTION->ruleName	 = (void *)"do_statement";


            	        }

            	        FOLLOWPUSH(FOLLOW_select_item_list_in_do_statement3775);
            	        select_item_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledo_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruledo_statementEx; /* Prevent compiler warnings */
    ruledo_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end do_statement */

/**
 * $ANTLR start handler_statement
 * MySQLSimpleParser.g:679:1: handler_statement : HANDLER_SYMBOL ( options {k=4; } : table_ref OPEN_SYMBOL ( ( AS_SYMBOL )? identifier )? | table_ref_no_db (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? ) ) ;
 */
static void
handler_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:679:18: ( HANDLER_SYMBOL ( options {k=4; } : table_ref OPEN_SYMBOL ( ( AS_SYMBOL )? identifier )? | table_ref_no_db (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? ) ) )
        // MySQLSimpleParser.g:680:2: HANDLER_SYMBOL ( options {k=4; } : table_ref OPEN_SYMBOL ( ( AS_SYMBOL )? identifier )? | table_ref_no_db (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? ) )
        {
             MATCHT(HANDLER_SYMBOL, &FOLLOW_HANDLER_SYMBOL_in_handler_statement3787);
            if  (HASEXCEPTION())
            {
                goto rulehandler_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:681:2: ( options {k=4; } : table_ref OPEN_SYMBOL ( ( AS_SYMBOL )? identifier )? | table_ref_no_db (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? ) )
            {
                int alt257=2;
                {
                    int LA257_0 = LA(1);
                    if ( (LA257_0 == IDENTIFIER))
                    {
                        switch ( LA(2) )
                        {
                        case OPEN_SYMBOL:
                        case DOT_SYMBOL:
                        	{
                        		alt257=1;
                        	}
                            break;
                        case EOF:
                        case CLOSE_SYMBOL:
                        case READ_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        	{
                        		alt257=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 257;
                            EXCEPTION->state        = 1;


                            goto rulehandler_statementEx;

                        }

                    }
                    else if ( (LA257_0 == BACK_TICK_QUOTED_ID))
                    {
                        switch ( LA(2) )
                        {
                        case OPEN_SYMBOL:
                        case DOT_SYMBOL:
                        	{
                        		alt257=1;
                        	}
                            break;
                        case EOF:
                        case CLOSE_SYMBOL:
                        case READ_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        	{
                        		alt257=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 257;
                            EXCEPTION->state        = 2;


                            goto rulehandler_statementEx;

                        }

                    }
                    else if ( (LA257_0 == DOUBLE_QUOTED_TEXT))
                    {
                        switch ( LA(2) )
                        {
                        case OPEN_SYMBOL:
                        case DOT_SYMBOL:
                        	{
                        		alt257=1;
                        	}
                            break;
                        case EOF:
                        case CLOSE_SYMBOL:
                        case READ_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        	{
                        		alt257=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 257;
                            EXCEPTION->state        = 3;


                            goto rulehandler_statementEx;

                        }

                    }
                    else if ( (((LA257_0 >= ACTION_SYMBOL) && (LA257_0 <= SHARE_SYMBOL)) || ((LA257_0 >= SLOW_SYMBOL) && (LA257_0 <= YEAR_SYMBOL))))
                    {
                        switch ( LA(2) )
                        {
                        case OPEN_SYMBOL:
                        case DOT_SYMBOL:
                        	{
                        		alt257=1;
                        	}
                            break;
                        case EOF:
                        case CLOSE_SYMBOL:
                        case READ_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        	{
                        		alt257=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 257;
                            EXCEPTION->state        = 4;


                            goto rulehandler_statementEx;

                        }

                    }
                    else if ( (((LA257_0 >= ACCOUNT_SYMBOL) && (LA257_0 <= XA_SYMBOL)) || LA257_0 == SHUTDOWN_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case OPEN_SYMBOL:
                        case DOT_SYMBOL:
                        	{
                        		alt257=1;
                        	}
                            break;
                        case EOF:
                        case CLOSE_SYMBOL:
                        case READ_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        	{
                        		alt257=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 257;
                            EXCEPTION->state        = 5;


                            goto rulehandler_statementEx;

                        }

                    }
                    else if ( (LA257_0 == DOT_SYMBOL))
                    {
                        alt257=1;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 257;
                        EXCEPTION->state        = 0;


                        goto rulehandler_statementEx;

                    }
                }
                switch (alt257)
                {
            	case 1:
            	    // MySQLSimpleParser.g:682:3: table_ref OPEN_SYMBOL ( ( AS_SYMBOL )? identifier )?
            	    {
            	        FOLLOWPUSH(FOLLOW_table_ref_in_handler_statement3807);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulehandler_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_SYMBOL, &FOLLOW_OPEN_SYMBOL_in_handler_statement3809);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulehandler_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:682:25: ( ( AS_SYMBOL )? identifier )?
            	        {
            	            int alt253=2;
            	            {
            	                int LA253_0 = LA(1);
            	                if ( (((LA253_0 >= ACCOUNT_SYMBOL) && (LA253_0 <= YEAR_SYMBOL)) || LA253_0 == AS_SYMBOL || LA253_0 == BACK_TICK_QUOTED_ID || LA253_0 == DOUBLE_QUOTED_TEXT || LA253_0 == IDENTIFIER))
            	                {
            	                    alt253=1;
            	                }
            	            }
            	            switch (alt253)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:682:26: ( AS_SYMBOL )? identifier
            	        	    {
            	        	        // MySQLSimpleParser.g:682:26: ( AS_SYMBOL )?
            	        	        {
            	        	            int alt252=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case AS_SYMBOL:
            	        	                	{
            	        	                		alt252=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt252)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:682:26: AS_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_handler_statement3812);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulehandler_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_identifier_in_handler_statement3815);
            	        	        identifier(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulehandler_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:683:5: table_ref_no_db (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? )
            	    {
            	        FOLLOWPUSH(FOLLOW_table_ref_no_db_in_handler_statement3823);
            	        table_ref_no_db(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulehandler_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:684:4: (| CLOSE_SYMBOL | READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )? )
            	        {
            	            int alt256=3;
            	            switch ( LA(1) )
            	            {
            	            case EOF:
            	            case SEMICOLON_SYMBOL:
            	            	{
            	            		alt256=1;
            	            	}
            	                break;
            	            case CLOSE_SYMBOL:
            	            	{
            	            		alt256=2;
            	            	}
            	                break;
            	            case READ_SYMBOL:
            	            	{
            	            		alt256=3;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 256;
            	                EXCEPTION->state        = 0;


            	                goto rulehandler_statementEx;

            	            }

            	            switch (alt256)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:685:5: 
            	        	    {
            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:685:7: CLOSE_SYMBOL
            	        	    {
            	        	         MATCHT(CLOSE_SYMBOL, &FOLLOW_CLOSE_SYMBOL_in_handler_statement3836);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulehandler_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 3:
            	        	    // MySQLSimpleParser.g:686:7: READ_SYMBOL handler_read_or_scan ( where_clause )? ( limit_clause )?
            	        	    {
            	        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_handler_statement3844);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulehandler_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_handler_read_or_scan_in_handler_statement3846);
            	        	        handler_read_or_scan(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulehandler_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:686:40: ( where_clause )?
            	        	        {
            	        	            int alt254=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case WHERE_SYMBOL:
            	        	                	{
            	        	                		alt254=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt254)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:686:40: where_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_where_clause_in_handler_statement3848);
            	        	        	        where_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulehandler_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:686:54: ( limit_clause )?
            	        	        {
            	        	            int alt255=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case LIMIT_SYMBOL:
            	        	                	{
            	        	                		alt255=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt255)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:686:54: limit_clause
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_handler_statement3851);
            	        	        	        limit_clause(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulehandler_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulehandler_statementEx; /* Prevent compiler warnings */
    rulehandler_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end handler_statement */

/**
 * $ANTLR start handler_read_or_scan
 * MySQLSimpleParser.g:691:1: handler_read_or_scan : ( ( FIRST_SYMBOL | NEXT_SYMBOL ) | identifier ( FIRST_SYMBOL | NEXT_SYMBOL | PREV_SYMBOL | LAST_SYMBOL | ( EQUAL_OPERATOR | LESS_THAN_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR ) OPEN_PAR_SYMBOL values CLOSE_PAR_SYMBOL ) );
 */
static void
handler_read_or_scan(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:691:21: ( ( FIRST_SYMBOL | NEXT_SYMBOL ) | identifier ( FIRST_SYMBOL | NEXT_SYMBOL | PREV_SYMBOL | LAST_SYMBOL | ( EQUAL_OPERATOR | LESS_THAN_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR ) OPEN_PAR_SYMBOL values CLOSE_PAR_SYMBOL ) )

            ANTLR3_UINT32 alt259;

            alt259=2;

            {
                int LA259_0 = LA(1);
                if ( (LA259_0 == FIRST_SYMBOL || LA259_0 == NEXT_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case EOF:
                    case LIMIT_SYMBOL:
                    case SEMICOLON_SYMBOL:
                    case WHERE_SYMBOL:
                    	{
                    		alt259=1;
                    	}
                        break;
                    case FIRST_SYMBOL:
                    case LAST_SYMBOL:
                    case NEXT_SYMBOL:
                    case PREV_SYMBOL:
                    case EQUAL_OPERATOR:
                    case GREATER_OR_EQUAL_OPERATOR:
                    case GREATER_THAN_OPERATOR:
                    case LESS_OR_EQUAL_OPERATOR:
                    case LESS_THAN_OPERATOR:
                    	{
                    		alt259=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 259;
                        EXCEPTION->state        = 1;


                        goto rulehandler_read_or_scanEx;

                    }

                }
                else if ( (((LA259_0 >= ACCOUNT_SYMBOL) && (LA259_0 <= FILTER_SYMBOL)) || ((LA259_0 >= FIXED_SYMBOL) && (LA259_0 <= NEVER_SYMBOL)) || ((LA259_0 >= NEW_SYMBOL) && (LA259_0 <= YEAR_SYMBOL)) || LA259_0 == BACK_TICK_QUOTED_ID || LA259_0 == DOUBLE_QUOTED_TEXT || LA259_0 == IDENTIFIER))
                {
                    alt259=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 259;
                    EXCEPTION->state        = 0;


                    goto rulehandler_read_or_scanEx;

                }
            }
            switch (alt259)
            {
        	case 1:
        	    // MySQLSimpleParser.g:692:2: ( FIRST_SYMBOL | NEXT_SYMBOL )
        	    {
        	        if ( LA(1) == FIRST_SYMBOL || LA(1) == NEXT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulehandler_read_or_scanEx;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:693:4: identifier ( FIRST_SYMBOL | NEXT_SYMBOL | PREV_SYMBOL | LAST_SYMBOL | ( EQUAL_OPERATOR | LESS_THAN_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR ) OPEN_PAR_SYMBOL values CLOSE_PAR_SYMBOL )
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_handler_read_or_scan3881);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_read_or_scanEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:694:3: ( FIRST_SYMBOL | NEXT_SYMBOL | PREV_SYMBOL | LAST_SYMBOL | ( EQUAL_OPERATOR | LESS_THAN_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR ) OPEN_PAR_SYMBOL values CLOSE_PAR_SYMBOL )
        	        {
        	            int alt258=5;
        	            switch ( LA(1) )
        	            {
        	            case FIRST_SYMBOL:
        	            	{
        	            		alt258=1;
        	            	}
        	                break;
        	            case NEXT_SYMBOL:
        	            	{
        	            		alt258=2;
        	            	}
        	                break;
        	            case PREV_SYMBOL:
        	            	{
        	            		alt258=3;
        	            	}
        	                break;
        	            case LAST_SYMBOL:
        	            	{
        	            		alt258=4;
        	            	}
        	                break;
        	            case EQUAL_OPERATOR:
        	            case GREATER_OR_EQUAL_OPERATOR:
        	            case GREATER_THAN_OPERATOR:
        	            case LESS_OR_EQUAL_OPERATOR:
        	            case LESS_THAN_OPERATOR:
        	            	{
        	            		alt258=5;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 258;
        	                EXCEPTION->state        = 0;


        	                goto rulehandler_read_or_scanEx;

        	            }

        	            switch (alt258)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:695:4: FIRST_SYMBOL
        	        	    {
        	        	         MATCHT(FIRST_SYMBOL, &FOLLOW_FIRST_SYMBOL_in_handler_read_or_scan3890);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:696:6: NEXT_SYMBOL
        	        	    {
        	        	         MATCHT(NEXT_SYMBOL, &FOLLOW_NEXT_SYMBOL_in_handler_read_or_scan3897);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:697:6: PREV_SYMBOL
        	        	    {
        	        	         MATCHT(PREV_SYMBOL, &FOLLOW_PREV_SYMBOL_in_handler_read_or_scan3904);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 4:
        	        	    // MySQLSimpleParser.g:698:6: LAST_SYMBOL
        	        	    {
        	        	         MATCHT(LAST_SYMBOL, &FOLLOW_LAST_SYMBOL_in_handler_read_or_scan3911);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 5:
        	        	    // MySQLSimpleParser.g:699:6: ( EQUAL_OPERATOR | LESS_THAN_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR ) OPEN_PAR_SYMBOL values CLOSE_PAR_SYMBOL
        	        	    {
        	        	        if ( LA(1) == EQUAL_OPERATOR || ((LA(1) >= GREATER_OR_EQUAL_OPERATOR) && (LA(1) <= GREATER_THAN_OPERATOR)) || ((LA(1) >= LESS_OR_EQUAL_OPERATOR) && (LA(1) <= LESS_THAN_OPERATOR)) )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulehandler_read_or_scanEx;
        	        	        }


        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_handler_read_or_scan3942);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_values_in_handler_read_or_scan3944);
        	        	        values(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_handler_read_or_scan3946);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulehandler_read_or_scanEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulehandler_read_or_scanEx; /* Prevent compiler warnings */
    rulehandler_read_or_scanEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end handler_read_or_scan */

/**
 * $ANTLR start insert_statement
 * MySQLSimpleParser.g:706:1: insert_statement : INSERT_SYMBOL ( insert_lock_option )? ( IGNORE_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec ( duplicate_key_update )? ;
 */
static void
insert_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:706:17: ( INSERT_SYMBOL ( insert_lock_option )? ( IGNORE_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec ( duplicate_key_update )? )
        // MySQLSimpleParser.g:707:2: INSERT_SYMBOL ( insert_lock_option )? ( IGNORE_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec ( duplicate_key_update )?
        {
             MATCHT(INSERT_SYMBOL, &FOLLOW_INSERT_SYMBOL_in_insert_statement3961);
            if  (HASEXCEPTION())
            {
                goto ruleinsert_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:707:16: ( insert_lock_option )?
            {
                int alt260=2;
                switch ( LA(1) )
                {
                    case DELAYED_SYMBOL:
                    case HIGH_PRIORITY_SYMBOL:
                    case LOW_PRIORITY_SYMBOL:
                    	{
                    		alt260=1;
                    	}
                        break;
                }

                switch (alt260)
                {
            	case 1:
            	    // MySQLSimpleParser.g:707:16: insert_lock_option
            	    {
            	        FOLLOWPUSH(FOLLOW_insert_lock_option_in_insert_statement3963);
            	        insert_lock_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:707:36: ( IGNORE_SYMBOL )?
            {
                int alt261=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    	{
                    		alt261=1;
                    	}
                        break;
                }

                switch (alt261)
                {
            	case 1:
            	    // MySQLSimpleParser.g:707:36: IGNORE_SYMBOL
            	    {
            	         MATCHT(IGNORE_SYMBOL, &FOLLOW_IGNORE_SYMBOL_in_insert_statement3966);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:707:51: ( INTO_SYMBOL )?
            {
                int alt262=2;
                switch ( LA(1) )
                {
                    case INTO_SYMBOL:
                    	{
                    		alt262=1;
                    	}
                        break;
                }

                switch (alt262)
                {
            	case 1:
            	    // MySQLSimpleParser.g:707:51: INTO_SYMBOL
            	    {
            	         MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_insert_statement3969);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_table_ref_in_insert_statement3972);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleinsert_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:707:74: ( use_partition )?
            {
                int alt263=2;
                {
                    int LA263_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50602)) && (LA263_0 == PARTITION_SYMBOL))
                    {
                        alt263=1;
                    }
                }
                switch (alt263)
                {
            	case 1:
            	    // MySQLSimpleParser.g:707:74: use_partition
            	    {
            	        FOLLOWPUSH(FOLLOW_use_partition_in_insert_statement3974);
            	        use_partition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_insert_field_spec_in_insert_statement3979);
            insert_field_spec(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleinsert_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:708:21: ( duplicate_key_update )?
            {
                int alt264=2;
                switch ( LA(1) )
                {
                    case ON_SYMBOL:
                    	{
                    		alt264=1;
                    	}
                        break;
                }

                switch (alt264)
                {
            	case 1:
            	    // MySQLSimpleParser.g:708:21: duplicate_key_update
            	    {
            	        FOLLOWPUSH(FOLLOW_duplicate_key_update_in_insert_statement3981);
            	        duplicate_key_update(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_statementEx; /* Prevent compiler warnings */
    ruleinsert_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_statement */

/**
 * $ANTLR start insert_lock_option
 * MySQLSimpleParser.g:711:1: insert_lock_option : ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL | HIGH_PRIORITY_SYMBOL );
 */
static void
insert_lock_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:711:19: ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL | HIGH_PRIORITY_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == DELAYED_SYMBOL || LA(1) == HIGH_PRIORITY_SYMBOL || LA(1) == LOW_PRIORITY_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleinsert_lock_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_lock_optionEx; /* Prevent compiler warnings */
    ruleinsert_lock_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_lock_option */

/**
 * $ANTLR start insert_field_spec
 * MySQLSimpleParser.g:717:1: insert_field_spec : ( ( OPEN_PAR_SYMBOL ( fields )? CLOSE_PAR_SYMBOL )? ( insert_values | insert_query_expression ) | SET_SYMBOL column_assignment_list_with_default );
 */
static void
insert_field_spec(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:717:18: ( ( OPEN_PAR_SYMBOL ( fields )? CLOSE_PAR_SYMBOL )? ( insert_values | insert_query_expression ) | SET_SYMBOL column_assignment_list_with_default )

            ANTLR3_UINT32 alt268;

            alt268=2;

            switch ( LA(1) )
            {
            case VALUE_SYMBOL:
            case OPEN_PAR_SYMBOL:
            case SELECT_SYMBOL:
            case VALUES_SYMBOL:
            	{
            		alt268=1;
            	}
                break;
            case SET_SYMBOL:
            	{
            		alt268=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 268;
                EXCEPTION->state        = 0;


                goto ruleinsert_field_specEx;

            }

            switch (alt268)
            {
        	case 1:
        	    // MySQLSimpleParser.g:718:2: ( OPEN_PAR_SYMBOL ( fields )? CLOSE_PAR_SYMBOL )? ( insert_values | insert_query_expression )
        	    {
        	        // MySQLSimpleParser.g:718:2: ( OPEN_PAR_SYMBOL ( fields )? CLOSE_PAR_SYMBOL )?
        	        {
        	            int alt266=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		{
        	                		    int LA266_1 = LA(2);
        	                		    if ( (((LA266_1 >= ACCOUNT_SYMBOL) && (LA266_1 <= YEAR_SYMBOL)) || LA266_1 == BACK_TICK_QUOTED_ID || LA266_1 == CLOSE_PAR_SYMBOL || LA266_1 == DOUBLE_QUOTED_TEXT || LA266_1 == IDENTIFIER))
        	                		    {
        	                		        alt266=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	            }

        	            switch (alt266)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:718:3: OPEN_PAR_SYMBOL ( fields )? CLOSE_PAR_SYMBOL
        	        	    {
        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_insert_field_spec4013);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_field_specEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:718:19: ( fields )?
        	        	        {
        	        	            int alt265=2;
        	        	            {
        	        	                int LA265_0 = LA(1);
        	        	                if ( (((LA265_0 >= ACCOUNT_SYMBOL) && (LA265_0 <= YEAR_SYMBOL)) || LA265_0 == BACK_TICK_QUOTED_ID || LA265_0 == DOUBLE_QUOTED_TEXT || LA265_0 == IDENTIFIER))
        	        	                {
        	        	                    alt265=1;
        	        	                }
        	        	            }
        	        	            switch (alt265)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:718:19: fields
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_fields_in_insert_field_spec4015);
        	        	        	        fields(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleinsert_field_specEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_insert_field_spec4018);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_field_specEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:719:3: ( insert_values | insert_query_expression )
        	        {
        	            int alt267=2;
        	            switch ( LA(1) )
        	            {
        	            case VALUE_SYMBOL:
        	            case VALUES_SYMBOL:
        	            	{
        	            		alt267=1;
        	            	}
        	                break;
        	            case OPEN_PAR_SYMBOL:
        	            case SELECT_SYMBOL:
        	            	{
        	            		alt267=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 267;
        	                EXCEPTION->state        = 0;


        	                goto ruleinsert_field_specEx;

        	            }

        	            switch (alt267)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:720:4: insert_values
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_insert_values_in_insert_field_spec4030);
        	        	        insert_values(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_field_specEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:721:6: insert_query_expression
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_insert_query_expression_in_insert_field_spec4037);
        	        	        insert_query_expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_field_specEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:723:4: SET_SYMBOL column_assignment_list_with_default
        	    {
        	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_insert_field_spec4046);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_field_specEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_column_assignment_list_with_default_in_insert_field_spec4048);
        	        column_assignment_list_with_default(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_field_specEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_field_specEx; /* Prevent compiler warnings */
    ruleinsert_field_specEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_field_spec */

/**
 * $ANTLR start fields
 * MySQLSimpleParser.g:726:1: fields : column_ref_with_wildcard ( COMMA_SYMBOL column_ref_with_wildcard )* ;
 */
static void
fields(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:726:7: ( column_ref_with_wildcard ( COMMA_SYMBOL column_ref_with_wildcard )* )
        // MySQLSimpleParser.g:727:2: column_ref_with_wildcard ( COMMA_SYMBOL column_ref_with_wildcard )*
        {
            FOLLOWPUSH(FOLLOW_column_ref_with_wildcard_in_fields4057);
            column_ref_with_wildcard(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefieldsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:727:27: ( COMMA_SYMBOL column_ref_with_wildcard )*

            for (;;)
            {
                int alt269=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt269=1;
                	}
                    break;

                }

                switch (alt269)
                {
            	case 1:
            	    // MySQLSimpleParser.g:727:28: COMMA_SYMBOL column_ref_with_wildcard
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_fields4060);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefieldsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_column_ref_with_wildcard_in_fields4062);
            	        column_ref_with_wildcard(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefieldsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop269;	/* break out of the loop */
            	    break;
                }
            }
            loop269: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefieldsEx; /* Prevent compiler warnings */
    rulefieldsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end fields */

/**
 * $ANTLR start insert_values
 * MySQLSimpleParser.g:730:1: insert_values : ( VALUES_SYMBOL | VALUE_SYMBOL ) insert_value_list ;
 */
static void
insert_values(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:730:14: ( ( VALUES_SYMBOL | VALUE_SYMBOL ) insert_value_list )
        // MySQLSimpleParser.g:731:2: ( VALUES_SYMBOL | VALUE_SYMBOL ) insert_value_list
        {
            if ( LA(1) == VALUE_SYMBOL || LA(1) == VALUES_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleinsert_valuesEx;
            }


            FOLLOWPUSH(FOLLOW_insert_value_list_in_insert_values4081);
            insert_value_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleinsert_valuesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_valuesEx; /* Prevent compiler warnings */
    ruleinsert_valuesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_values */

/**
 * $ANTLR start insert_query_expression
 * MySQLSimpleParser.g:734:1: insert_query_expression : ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? );
 */
static void
insert_query_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:734:24: ( create_select ( union_clause )? | OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? )

            ANTLR3_UINT32 alt272;

            alt272=2;

            switch ( LA(1) )
            {
            case SELECT_SYMBOL:
            	{
            		alt272=1;
            	}
                break;
            case OPEN_PAR_SYMBOL:
            	{
            		alt272=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 272;
                EXCEPTION->state        = 0;


                goto ruleinsert_query_expressionEx;

            }

            switch (alt272)
            {
        	case 1:
        	    // MySQLSimpleParser.g:735:2: create_select ( union_clause )?
        	    {
        	        FOLLOWPUSH(FOLLOW_create_select_in_insert_query_expression4090);
        	        create_select(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_query_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:735:16: ( union_clause )?
        	        {
        	            int alt270=2;
        	            switch ( LA(1) )
        	            {
        	                case UNION_SYMBOL:
        	                	{
        	                		alt270=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt270)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:735:16: union_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_union_clause_in_insert_query_expression4092);
        	        	        union_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_query_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:736:4: OPEN_PAR_SYMBOL create_select CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )?
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_insert_query_expression4098);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_query_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_create_select_in_insert_query_expression4100);
        	        create_select(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_query_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_insert_query_expression4102);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinsert_query_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:736:51: ( union_or_order_by_or_limit )?
        	        {
        	            int alt271=2;
        	            switch ( LA(1) )
        	            {
        	                case LIMIT_SYMBOL:
        	                case ORDER_SYMBOL:
        	                case UNION_SYMBOL:
        	                	{
        	                		alt271=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt271)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:736:51: union_or_order_by_or_limit
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_union_or_order_by_or_limit_in_insert_query_expression4104);
        	        	        union_or_order_by_or_limit(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinsert_query_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_query_expressionEx; /* Prevent compiler warnings */
    ruleinsert_query_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_query_expression */

/**
 * $ANTLR start insert_value_list
 * MySQLSimpleParser.g:739:1: insert_value_list : OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL ( COMMA_SYMBOL OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL )* ;
 */
static void
insert_value_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:739:18: ( OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL ( COMMA_SYMBOL OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL )* )
        // MySQLSimpleParser.g:740:2: OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL ( COMMA_SYMBOL OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL )*
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4114);
            if  (HASEXCEPTION())
            {
                goto ruleinsert_value_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:740:18: ( values )?
            {
                int alt273=2;
                {
                    int LA273_0 = LA(1);
                    if ( (((LA273_0 >= ACCOUNT_SYMBOL) && (LA273_0 <= YEAR_SYMBOL)) || LA273_0 == BINARY_SYMBOL || ((LA273_0 >= CASE_SYMBOL) && (LA273_0 <= CAST_SYMBOL)) || LA273_0 == CHAR_SYMBOL || ((LA273_0 >= CONVERT_SYMBOL) && (LA273_0 <= COUNT_SYMBOL)) || LA273_0 == CURDATE_SYMBOL || LA273_0 == CURRENT_USER_SYMBOL || LA273_0 == CURTIME_SYMBOL || LA273_0 == DATABASE_SYMBOL || LA273_0 == DATE_ADD_SYMBOL || LA273_0 == DATE_SUB_SYMBOL || LA273_0 == DEFAULT_SYMBOL || LA273_0 == EXISTS_SYMBOL || ((LA273_0 >= EXTRACT_SYMBOL) && (LA273_0 <= FALSE_SYMBOL)) || LA273_0 == GROUP_CONCAT_SYMBOL || LA273_0 == IF_SYMBOL || LA273_0 == INSERT_SYMBOL || LA273_0 == INTERVAL_SYMBOL || LA273_0 == LEFT_SYMBOL || LA273_0 == MATCH_SYMBOL || LA273_0 == MAX_SYMBOL || LA273_0 == MIN_SYMBOL || LA273_0 == MOD_SYMBOL || LA273_0 == NCHAR_TEXT || ((LA273_0 >= NOT2_SYMBOL) && (LA273_0 <= NOW_SYMBOL)) || ((LA273_0 >= NULL2_SYMBOL) && (LA273_0 <= NULL_SYMBOL)) || ((LA273_0 >= OPEN_CURLY_SYMBOL) && (LA273_0 <= OPEN_PAR_SYMBOL)) || ((LA273_0 >= PARAM_MARKER) && (LA273_0 <= POSITION_SYMBOL)) || ((LA273_0 >= REPEAT_SYMBOL) && (LA273_0 <= REPLACE_SYMBOL)) || LA273_0 == RIGHT_SYMBOL || LA273_0 == SINGLE_QUOTED_TEXT || LA273_0 == STDDEV_SAMP_SYMBOL || LA273_0 == STD_SYMBOL || LA273_0 == SUBSTRING_SYMBOL || ((LA273_0 >= SUM_SYMBOL) && (LA273_0 <= SYSDATE_SYMBOL)) || ((LA273_0 >= TRIM_SYMBOL) && (LA273_0 <= TRUE_SYMBOL)) || ((LA273_0 >= UTC_DATE_SYMBOL) && (LA273_0 <= UTC_TIME_SYMBOL)) || LA273_0 == VALUES_SYMBOL || LA273_0 == VARIANCE_SYMBOL || LA273_0 == VAR_SAMP_SYMBOL || ((LA273_0 >= AT_AT_SIGN_SYMBOL) && (LA273_0 <= AT_TEXT_SUFFIX)) || ((LA273_0 >= BACK_TICK_QUOTED_ID) && (LA273_0 <= BIN_NUMBER)) || LA273_0 == BITWISE_NOT_OPERATOR || ((LA273_0 >= BIT_AND_SYMBOL) && (LA273_0 <= BIT_XOR_SYMBOL)) || LA273_0 == DECIMAL_NUMBER || LA273_0 == DOT_SYMBOL || LA273_0 == DOUBLE_QUOTED_TEXT || LA273_0 == FLOAT_NUMBER || ((LA273_0 >= HEX_NUMBER) && (LA273_0 <= IDENTIFIER)) || LA273_0 == INT_NUMBER || LA273_0 == LOGICAL_NOT_OPERATOR || ((LA273_0 >= LONG_NUMBER) && (LA273_0 <= MINUS_OPERATOR)) || LA273_0 == PLUS_OPERATOR || ((LA273_0 >= ULONGLONG_NUMBER) && (LA273_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt273=1;
                    }
                }
                switch (alt273)
                {
            	case 1:
            	    // MySQLSimpleParser.g:740:18: values
            	    {
            	        FOLLOWPUSH(FOLLOW_values_in_insert_value_list4116);
            	        values(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4119);
            if  (HASEXCEPTION())
            {
                goto ruleinsert_value_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:740:43: ( COMMA_SYMBOL OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL )*

            for (;;)
            {
                int alt275=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt275=1;
                	}
                    break;

                }

                switch (alt275)
                {
            	case 1:
            	    // MySQLSimpleParser.g:740:44: COMMA_SYMBOL OPEN_PAR_SYMBOL ( values )? CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_insert_value_list4122);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_insert_value_list4124);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:740:73: ( values )?
            	        {
            	            int alt274=2;
            	            {
            	                int LA274_0 = LA(1);
            	                if ( (((LA274_0 >= ACCOUNT_SYMBOL) && (LA274_0 <= YEAR_SYMBOL)) || LA274_0 == BINARY_SYMBOL || ((LA274_0 >= CASE_SYMBOL) && (LA274_0 <= CAST_SYMBOL)) || LA274_0 == CHAR_SYMBOL || ((LA274_0 >= CONVERT_SYMBOL) && (LA274_0 <= COUNT_SYMBOL)) || LA274_0 == CURDATE_SYMBOL || LA274_0 == CURRENT_USER_SYMBOL || LA274_0 == CURTIME_SYMBOL || LA274_0 == DATABASE_SYMBOL || LA274_0 == DATE_ADD_SYMBOL || LA274_0 == DATE_SUB_SYMBOL || LA274_0 == DEFAULT_SYMBOL || LA274_0 == EXISTS_SYMBOL || ((LA274_0 >= EXTRACT_SYMBOL) && (LA274_0 <= FALSE_SYMBOL)) || LA274_0 == GROUP_CONCAT_SYMBOL || LA274_0 == IF_SYMBOL || LA274_0 == INSERT_SYMBOL || LA274_0 == INTERVAL_SYMBOL || LA274_0 == LEFT_SYMBOL || LA274_0 == MATCH_SYMBOL || LA274_0 == MAX_SYMBOL || LA274_0 == MIN_SYMBOL || LA274_0 == MOD_SYMBOL || LA274_0 == NCHAR_TEXT || ((LA274_0 >= NOT2_SYMBOL) && (LA274_0 <= NOW_SYMBOL)) || ((LA274_0 >= NULL2_SYMBOL) && (LA274_0 <= NULL_SYMBOL)) || ((LA274_0 >= OPEN_CURLY_SYMBOL) && (LA274_0 <= OPEN_PAR_SYMBOL)) || ((LA274_0 >= PARAM_MARKER) && (LA274_0 <= POSITION_SYMBOL)) || ((LA274_0 >= REPEAT_SYMBOL) && (LA274_0 <= REPLACE_SYMBOL)) || LA274_0 == RIGHT_SYMBOL || LA274_0 == SINGLE_QUOTED_TEXT || LA274_0 == STDDEV_SAMP_SYMBOL || LA274_0 == STD_SYMBOL || LA274_0 == SUBSTRING_SYMBOL || ((LA274_0 >= SUM_SYMBOL) && (LA274_0 <= SYSDATE_SYMBOL)) || ((LA274_0 >= TRIM_SYMBOL) && (LA274_0 <= TRUE_SYMBOL)) || ((LA274_0 >= UTC_DATE_SYMBOL) && (LA274_0 <= UTC_TIME_SYMBOL)) || LA274_0 == VALUES_SYMBOL || LA274_0 == VARIANCE_SYMBOL || LA274_0 == VAR_SAMP_SYMBOL || ((LA274_0 >= AT_AT_SIGN_SYMBOL) && (LA274_0 <= AT_TEXT_SUFFIX)) || ((LA274_0 >= BACK_TICK_QUOTED_ID) && (LA274_0 <= BIN_NUMBER)) || LA274_0 == BITWISE_NOT_OPERATOR || ((LA274_0 >= BIT_AND_SYMBOL) && (LA274_0 <= BIT_XOR_SYMBOL)) || LA274_0 == DECIMAL_NUMBER || LA274_0 == DOT_SYMBOL || LA274_0 == DOUBLE_QUOTED_TEXT || LA274_0 == FLOAT_NUMBER || ((LA274_0 >= HEX_NUMBER) && (LA274_0 <= IDENTIFIER)) || LA274_0 == INT_NUMBER || LA274_0 == LOGICAL_NOT_OPERATOR || ((LA274_0 >= LONG_NUMBER) && (LA274_0 <= MINUS_OPERATOR)) || LA274_0 == PLUS_OPERATOR || ((LA274_0 >= ULONGLONG_NUMBER) && (LA274_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt274=1;
            	                }
            	            }
            	            switch (alt274)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:740:73: values
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_values_in_insert_value_list4126);
            	        	        values(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinsert_value_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_insert_value_list4129);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinsert_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop275;	/* break out of the loop */
            	    break;
                }
            }
            loop275: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinsert_value_listEx; /* Prevent compiler warnings */
    ruleinsert_value_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end insert_value_list */

/**
 * $ANTLR start values
 * MySQLSimpleParser.g:743:1: values : ( expression | DEFAULT_SYMBOL ) ( COMMA_SYMBOL ( expression | DEFAULT_SYMBOL ) )* ;
 */
static void
values(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:743:7: ( ( expression | DEFAULT_SYMBOL ) ( COMMA_SYMBOL ( expression | DEFAULT_SYMBOL ) )* )
        // MySQLSimpleParser.g:744:2: ( expression | DEFAULT_SYMBOL ) ( COMMA_SYMBOL ( expression | DEFAULT_SYMBOL ) )*
        {
            // MySQLSimpleParser.g:744:2: ( expression | DEFAULT_SYMBOL )
            {
                int alt276=2;
                {
                    int LA276_0 = LA(1);
                    if ( (((LA276_0 >= ACCOUNT_SYMBOL) && (LA276_0 <= YEAR_SYMBOL)) || LA276_0 == BINARY_SYMBOL || ((LA276_0 >= CASE_SYMBOL) && (LA276_0 <= CAST_SYMBOL)) || LA276_0 == CHAR_SYMBOL || ((LA276_0 >= CONVERT_SYMBOL) && (LA276_0 <= COUNT_SYMBOL)) || LA276_0 == CURDATE_SYMBOL || LA276_0 == CURRENT_USER_SYMBOL || LA276_0 == CURTIME_SYMBOL || LA276_0 == DATABASE_SYMBOL || LA276_0 == DATE_ADD_SYMBOL || LA276_0 == DATE_SUB_SYMBOL || LA276_0 == EXISTS_SYMBOL || ((LA276_0 >= EXTRACT_SYMBOL) && (LA276_0 <= FALSE_SYMBOL)) || LA276_0 == GROUP_CONCAT_SYMBOL || LA276_0 == IF_SYMBOL || LA276_0 == INSERT_SYMBOL || LA276_0 == INTERVAL_SYMBOL || LA276_0 == LEFT_SYMBOL || LA276_0 == MATCH_SYMBOL || LA276_0 == MAX_SYMBOL || LA276_0 == MIN_SYMBOL || LA276_0 == MOD_SYMBOL || LA276_0 == NCHAR_TEXT || ((LA276_0 >= NOT2_SYMBOL) && (LA276_0 <= NOW_SYMBOL)) || ((LA276_0 >= NULL2_SYMBOL) && (LA276_0 <= NULL_SYMBOL)) || ((LA276_0 >= OPEN_CURLY_SYMBOL) && (LA276_0 <= OPEN_PAR_SYMBOL)) || ((LA276_0 >= PARAM_MARKER) && (LA276_0 <= POSITION_SYMBOL)) || ((LA276_0 >= REPEAT_SYMBOL) && (LA276_0 <= REPLACE_SYMBOL)) || LA276_0 == RIGHT_SYMBOL || LA276_0 == SINGLE_QUOTED_TEXT || LA276_0 == STDDEV_SAMP_SYMBOL || LA276_0 == STD_SYMBOL || LA276_0 == SUBSTRING_SYMBOL || ((LA276_0 >= SUM_SYMBOL) && (LA276_0 <= SYSDATE_SYMBOL)) || ((LA276_0 >= TRIM_SYMBOL) && (LA276_0 <= TRUE_SYMBOL)) || ((LA276_0 >= UTC_DATE_SYMBOL) && (LA276_0 <= UTC_TIME_SYMBOL)) || LA276_0 == VALUES_SYMBOL || LA276_0 == VARIANCE_SYMBOL || LA276_0 == VAR_SAMP_SYMBOL || ((LA276_0 >= AT_AT_SIGN_SYMBOL) && (LA276_0 <= AT_TEXT_SUFFIX)) || ((LA276_0 >= BACK_TICK_QUOTED_ID) && (LA276_0 <= BIN_NUMBER)) || LA276_0 == BITWISE_NOT_OPERATOR || ((LA276_0 >= BIT_AND_SYMBOL) && (LA276_0 <= BIT_XOR_SYMBOL)) || LA276_0 == DECIMAL_NUMBER || LA276_0 == DOT_SYMBOL || LA276_0 == DOUBLE_QUOTED_TEXT || LA276_0 == FLOAT_NUMBER || ((LA276_0 >= HEX_NUMBER) && (LA276_0 <= IDENTIFIER)) || LA276_0 == INT_NUMBER || LA276_0 == LOGICAL_NOT_OPERATOR || ((LA276_0 >= LONG_NUMBER) && (LA276_0 <= MINUS_OPERATOR)) || LA276_0 == PLUS_OPERATOR || ((LA276_0 >= ULONGLONG_NUMBER) && (LA276_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt276=1;
                    }
                    else if ( (LA276_0 == DEFAULT_SYMBOL))
                    {
                        alt276=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 276;
                        EXCEPTION->state        = 0;


                        goto rulevaluesEx;

                    }
                }
                switch (alt276)
                {
            	case 1:
            	    // MySQLSimpleParser.g:744:3: expression
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_values4141);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevaluesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:744:16: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_values4145);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevaluesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:744:32: ( COMMA_SYMBOL ( expression | DEFAULT_SYMBOL ) )*

            for (;;)
            {
                int alt278=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt278=1;
                	}
                    break;

                }

                switch (alt278)
                {
            	case 1:
            	    // MySQLSimpleParser.g:744:33: COMMA_SYMBOL ( expression | DEFAULT_SYMBOL )
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_values4149);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevaluesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:744:46: ( expression | DEFAULT_SYMBOL )
            	        {
            	            int alt277=2;
            	            {
            	                int LA277_0 = LA(1);
            	                if ( (((LA277_0 >= ACCOUNT_SYMBOL) && (LA277_0 <= YEAR_SYMBOL)) || LA277_0 == BINARY_SYMBOL || ((LA277_0 >= CASE_SYMBOL) && (LA277_0 <= CAST_SYMBOL)) || LA277_0 == CHAR_SYMBOL || ((LA277_0 >= CONVERT_SYMBOL) && (LA277_0 <= COUNT_SYMBOL)) || LA277_0 == CURDATE_SYMBOL || LA277_0 == CURRENT_USER_SYMBOL || LA277_0 == CURTIME_SYMBOL || LA277_0 == DATABASE_SYMBOL || LA277_0 == DATE_ADD_SYMBOL || LA277_0 == DATE_SUB_SYMBOL || LA277_0 == EXISTS_SYMBOL || ((LA277_0 >= EXTRACT_SYMBOL) && (LA277_0 <= FALSE_SYMBOL)) || LA277_0 == GROUP_CONCAT_SYMBOL || LA277_0 == IF_SYMBOL || LA277_0 == INSERT_SYMBOL || LA277_0 == INTERVAL_SYMBOL || LA277_0 == LEFT_SYMBOL || LA277_0 == MATCH_SYMBOL || LA277_0 == MAX_SYMBOL || LA277_0 == MIN_SYMBOL || LA277_0 == MOD_SYMBOL || LA277_0 == NCHAR_TEXT || ((LA277_0 >= NOT2_SYMBOL) && (LA277_0 <= NOW_SYMBOL)) || ((LA277_0 >= NULL2_SYMBOL) && (LA277_0 <= NULL_SYMBOL)) || ((LA277_0 >= OPEN_CURLY_SYMBOL) && (LA277_0 <= OPEN_PAR_SYMBOL)) || ((LA277_0 >= PARAM_MARKER) && (LA277_0 <= POSITION_SYMBOL)) || ((LA277_0 >= REPEAT_SYMBOL) && (LA277_0 <= REPLACE_SYMBOL)) || LA277_0 == RIGHT_SYMBOL || LA277_0 == SINGLE_QUOTED_TEXT || LA277_0 == STDDEV_SAMP_SYMBOL || LA277_0 == STD_SYMBOL || LA277_0 == SUBSTRING_SYMBOL || ((LA277_0 >= SUM_SYMBOL) && (LA277_0 <= SYSDATE_SYMBOL)) || ((LA277_0 >= TRIM_SYMBOL) && (LA277_0 <= TRUE_SYMBOL)) || ((LA277_0 >= UTC_DATE_SYMBOL) && (LA277_0 <= UTC_TIME_SYMBOL)) || LA277_0 == VALUES_SYMBOL || LA277_0 == VARIANCE_SYMBOL || LA277_0 == VAR_SAMP_SYMBOL || ((LA277_0 >= AT_AT_SIGN_SYMBOL) && (LA277_0 <= AT_TEXT_SUFFIX)) || ((LA277_0 >= BACK_TICK_QUOTED_ID) && (LA277_0 <= BIN_NUMBER)) || LA277_0 == BITWISE_NOT_OPERATOR || ((LA277_0 >= BIT_AND_SYMBOL) && (LA277_0 <= BIT_XOR_SYMBOL)) || LA277_0 == DECIMAL_NUMBER || LA277_0 == DOT_SYMBOL || LA277_0 == DOUBLE_QUOTED_TEXT || LA277_0 == FLOAT_NUMBER || ((LA277_0 >= HEX_NUMBER) && (LA277_0 <= IDENTIFIER)) || LA277_0 == INT_NUMBER || LA277_0 == LOGICAL_NOT_OPERATOR || ((LA277_0 >= LONG_NUMBER) && (LA277_0 <= MINUS_OPERATOR)) || LA277_0 == PLUS_OPERATOR || ((LA277_0 >= ULONGLONG_NUMBER) && (LA277_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt277=1;
            	                }
            	                else if ( (LA277_0 == DEFAULT_SYMBOL))
            	                {
            	                    alt277=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 277;
            	                    EXCEPTION->state        = 0;


            	                    goto rulevaluesEx;

            	                }
            	            }
            	            switch (alt277)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:744:47: expression
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_in_values4152);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulevaluesEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:744:60: DEFAULT_SYMBOL
            	        	    {
            	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_values4156);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulevaluesEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

            	default:
            	    goto loop278;	/* break out of the loop */
            	    break;
                }
            }
            loop278: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulevaluesEx; /* Prevent compiler warnings */
    rulevaluesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end values */

/**
 * $ANTLR start duplicate_key_update
 * MySQLSimpleParser.g:747:1: duplicate_key_update : ON_SYMBOL DUPLICATE_SYMBOL KEY_SYMBOL UPDATE_SYMBOL column_assignment_list_with_default ;
 */
static void
duplicate_key_update(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:747:21: ( ON_SYMBOL DUPLICATE_SYMBOL KEY_SYMBOL UPDATE_SYMBOL column_assignment_list_with_default )
        // MySQLSimpleParser.g:748:2: ON_SYMBOL DUPLICATE_SYMBOL KEY_SYMBOL UPDATE_SYMBOL column_assignment_list_with_default
        {
             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_duplicate_key_update4168);
            if  (HASEXCEPTION())
            {
                goto ruleduplicate_key_updateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(DUPLICATE_SYMBOL, &FOLLOW_DUPLICATE_SYMBOL_in_duplicate_key_update4170);
            if  (HASEXCEPTION())
            {
                goto ruleduplicate_key_updateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_duplicate_key_update4172);
            if  (HASEXCEPTION())
            {
                goto ruleduplicate_key_updateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_duplicate_key_update4174);
            if  (HASEXCEPTION())
            {
                goto ruleduplicate_key_updateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_column_assignment_list_with_default_in_duplicate_key_update4176);
            column_assignment_list_with_default(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleduplicate_key_updateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleduplicate_key_updateEx; /* Prevent compiler warnings */
    ruleduplicate_key_updateEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end duplicate_key_update */

/**
 * $ANTLR start load_statement
 * MySQLSimpleParser.g:753:1: load_statement : LOAD_SYMBOL data_or_xml ( LOW_PRIORITY_SYMBOL | CONCURRENT_SYMBOL )? ( LOCAL_SYMBOL )? INFILE_SYMBOL string_literal ( REPLACE_SYMBOL | IGNORE_SYMBOL )? INTO_SYMBOL TABLE_SYMBOL table_ref ( use_partition )? ( charset_clause )? ( xml_rows_identified_by )? ( fields_clause )? ( lines_clause )? load_data_file_tail ;
 */
static void
load_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:753:15: ( LOAD_SYMBOL data_or_xml ( LOW_PRIORITY_SYMBOL | CONCURRENT_SYMBOL )? ( LOCAL_SYMBOL )? INFILE_SYMBOL string_literal ( REPLACE_SYMBOL | IGNORE_SYMBOL )? INTO_SYMBOL TABLE_SYMBOL table_ref ( use_partition )? ( charset_clause )? ( xml_rows_identified_by )? ( fields_clause )? ( lines_clause )? load_data_file_tail )
        // MySQLSimpleParser.g:754:2: LOAD_SYMBOL data_or_xml ( LOW_PRIORITY_SYMBOL | CONCURRENT_SYMBOL )? ( LOCAL_SYMBOL )? INFILE_SYMBOL string_literal ( REPLACE_SYMBOL | IGNORE_SYMBOL )? INTO_SYMBOL TABLE_SYMBOL table_ref ( use_partition )? ( charset_clause )? ( xml_rows_identified_by )? ( fields_clause )? ( lines_clause )? load_data_file_tail
        {
             MATCHT(LOAD_SYMBOL, &FOLLOW_LOAD_SYMBOL_in_load_statement4187);
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_data_or_xml_in_load_statement4189);
            data_or_xml(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:754:26: ( LOW_PRIORITY_SYMBOL | CONCURRENT_SYMBOL )?
            {
                int alt279=2;
                switch ( LA(1) )
                {
                    case CONCURRENT_SYMBOL:
                    case LOW_PRIORITY_SYMBOL:
                    	{
                    		alt279=1;
                    	}
                        break;
                }

                switch (alt279)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == CONCURRENT_SYMBOL || LA(1) == LOW_PRIORITY_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleload_statementEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:754:69: ( LOCAL_SYMBOL )?
            {
                int alt280=2;
                switch ( LA(1) )
                {
                    case LOCAL_SYMBOL:
                    	{
                    		alt280=1;
                    	}
                        break;
                }

                switch (alt280)
                {
            	case 1:
            	    // MySQLSimpleParser.g:754:69: LOCAL_SYMBOL
            	    {
            	         MATCHT(LOCAL_SYMBOL, &FOLLOW_LOCAL_SYMBOL_in_load_statement4200);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(INFILE_SYMBOL, &FOLLOW_INFILE_SYMBOL_in_load_statement4203);
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_string_literal_in_load_statement4205);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:755:3: ( REPLACE_SYMBOL | IGNORE_SYMBOL )?
            {
                int alt281=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    case REPLACE_SYMBOL:
                    	{
                    		alt281=1;
                    	}
                        break;
                }

                switch (alt281)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == IGNORE_SYMBOL || LA(1) == REPLACE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleload_statementEx;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_load_statement4218);
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_load_statement4220);
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_load_statement4222);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:756:3: ( use_partition )?
            {
                int alt282=2;
                {
                    int LA282_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50602)) && (LA282_0 == PARTITION_SYMBOL))
                    {
                        alt282=1;
                    }
                }
                switch (alt282)
                {
            	case 1:
            	    // MySQLSimpleParser.g:756:3: use_partition
            	    {
            	        FOLLOWPUSH(FOLLOW_use_partition_in_load_statement4226);
            	        use_partition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:756:18: ( charset_clause )?
            {
                int alt283=2;
                switch ( LA(1) )
                {
                    case CHARSET_SYMBOL:
                    case CHAR_SYMBOL:
                    	{
                    		alt283=1;
                    	}
                        break;
                }

                switch (alt283)
                {
            	case 1:
            	    // MySQLSimpleParser.g:756:18: charset_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_charset_clause_in_load_statement4229);
            	        charset_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:757:3: ( xml_rows_identified_by )?
            {
                int alt284=2;
                {
                    int LA284_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50500)) && (LA284_0 == ROWS_SYMBOL))
                    {
                        alt284=1;
                    }
                }
                switch (alt284)
                {
            	case 1:
            	    // MySQLSimpleParser.g:757:3: xml_rows_identified_by
            	    {
            	        FOLLOWPUSH(FOLLOW_xml_rows_identified_by_in_load_statement4234);
            	        xml_rows_identified_by(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:758:3: ( fields_clause )?
            {
                int alt285=2;
                switch ( LA(1) )
                {
                    case COLUMNS_SYMBOL:
                    	{
                    		alt285=1;
                    	}
                        break;
                }

                switch (alt285)
                {
            	case 1:
            	    // MySQLSimpleParser.g:758:3: fields_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_fields_clause_in_load_statement4239);
            	        fields_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:758:18: ( lines_clause )?
            {
                int alt286=2;
                switch ( LA(1) )
                {
                    case LINES_SYMBOL:
                    	{
                    		alt286=1;
                    	}
                        break;
                }

                switch (alt286)
                {
            	case 1:
            	    // MySQLSimpleParser.g:758:18: lines_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_lines_clause_in_load_statement4242);
            	        lines_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_load_data_file_tail_in_load_statement4247);
            load_data_file_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleload_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleload_statementEx; /* Prevent compiler warnings */
    ruleload_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end load_statement */

/**
 * $ANTLR start data_or_xml
 * MySQLSimpleParser.g:762:1: data_or_xml : ( DATA_SYMBOL |{...}? XML_SYMBOL );
 */
static void
data_or_xml(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:762:12: ( DATA_SYMBOL |{...}? XML_SYMBOL )

            ANTLR3_UINT32 alt287;

            alt287=2;

            switch ( LA(1) )
            {
            case DATA_SYMBOL:
            	{
            		alt287=1;
            	}
                break;
            case XML_SYMBOL:
            	{
            		alt287=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 287;
                EXCEPTION->state        = 0;


                goto ruledata_or_xmlEx;

            }

            switch (alt287)
            {
        	case 1:
        	    // MySQLSimpleParser.g:763:2: DATA_SYMBOL
        	    {
        	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_data_or_xml4256);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_or_xmlEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:764:4: {...}? XML_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"data_or_xml";


        	        }

        	         MATCHT(XML_SYMBOL, &FOLLOW_XML_SYMBOL_in_data_or_xml4263);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_or_xmlEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruledata_or_xmlEx; /* Prevent compiler warnings */
    ruledata_or_xmlEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end data_or_xml */

/**
 * $ANTLR start xml_rows_identified_by
 * MySQLSimpleParser.g:767:1: xml_rows_identified_by :{...}? => ROWS_SYMBOL IDENTIFIED_SYMBOL BY_SYMBOL text_string ;
 */
static void
xml_rows_identified_by(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:767:23: ({...}? => ROWS_SYMBOL IDENTIFIED_SYMBOL BY_SYMBOL text_string )
        // MySQLSimpleParser.g:768:2: {...}? => ROWS_SYMBOL IDENTIFIED_SYMBOL BY_SYMBOL text_string
        {
            if ( !((SERVER_VERSION >= 50500)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
                    EXCEPTION->ruleName	 = (void *)"xml_rows_identified_by";


            }

             MATCHT(ROWS_SYMBOL, &FOLLOW_ROWS_SYMBOL_in_xml_rows_identified_by4276);
            if  (HASEXCEPTION())
            {
                goto rulexml_rows_identified_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(IDENTIFIED_SYMBOL, &FOLLOW_IDENTIFIED_SYMBOL_in_xml_rows_identified_by4278);
            if  (HASEXCEPTION())
            {
                goto rulexml_rows_identified_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_xml_rows_identified_by4280);
            if  (HASEXCEPTION())
            {
                goto rulexml_rows_identified_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_string_in_xml_rows_identified_by4282);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulexml_rows_identified_byEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulexml_rows_identified_byEx; /* Prevent compiler warnings */
    rulexml_rows_identified_byEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end xml_rows_identified_by */

/**
 * $ANTLR start load_data_file_tail
 * MySQLSimpleParser.g:771:1: load_data_file_tail : ( IGNORE_SYMBOL INT_NUMBER ( LINES_SYMBOL | ROWS_SYMBOL ) )? ( load_data_file_target_list )? ( SET_SYMBOL column_assignment_list_with_default )? ;
 */
static void
load_data_file_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:771:20: ( ( IGNORE_SYMBOL INT_NUMBER ( LINES_SYMBOL | ROWS_SYMBOL ) )? ( load_data_file_target_list )? ( SET_SYMBOL column_assignment_list_with_default )? )
        // MySQLSimpleParser.g:772:2: ( IGNORE_SYMBOL INT_NUMBER ( LINES_SYMBOL | ROWS_SYMBOL ) )? ( load_data_file_target_list )? ( SET_SYMBOL column_assignment_list_with_default )?
        {
            // MySQLSimpleParser.g:772:2: ( IGNORE_SYMBOL INT_NUMBER ( LINES_SYMBOL | ROWS_SYMBOL ) )?
            {
                int alt288=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    	{
                    		alt288=1;
                    	}
                        break;
                }

                switch (alt288)
                {
            	case 1:
            	    // MySQLSimpleParser.g:772:3: IGNORE_SYMBOL INT_NUMBER ( LINES_SYMBOL | ROWS_SYMBOL )
            	    {
            	         MATCHT(IGNORE_SYMBOL, &FOLLOW_IGNORE_SYMBOL_in_load_data_file_tail4292);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_load_data_file_tail4294);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == ROWS_SYMBOL || LA(1) == LINES_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleload_data_file_tailEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:772:59: ( load_data_file_target_list )?
            {
                int alt289=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt289=1;
                    	}
                        break;
                }

                switch (alt289)
                {
            	case 1:
            	    // MySQLSimpleParser.g:772:59: load_data_file_target_list
            	    {
            	        FOLLOWPUSH(FOLLOW_load_data_file_target_list_in_load_data_file_tail4306);
            	        load_data_file_target_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:772:87: ( SET_SYMBOL column_assignment_list_with_default )?
            {
                int alt290=2;
                switch ( LA(1) )
                {
                    case SET_SYMBOL:
                    	{
                    		alt290=1;
                    	}
                        break;
                }

                switch (alt290)
                {
            	case 1:
            	    // MySQLSimpleParser.g:772:88: SET_SYMBOL column_assignment_list_with_default
            	    {
            	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_load_data_file_tail4310);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_column_assignment_list_with_default_in_load_data_file_tail4312);
            	        column_assignment_list_with_default(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_tailEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleload_data_file_tailEx; /* Prevent compiler warnings */
    ruleload_data_file_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end load_data_file_tail */

/**
 * $ANTLR start load_data_file_target_list
 * MySQLSimpleParser.g:775:1: load_data_file_target_list : OPEN_PAR_SYMBOL ( field_or_variable_list )? CLOSE_PAR_SYMBOL ;
 */
static void
load_data_file_target_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:775:27: ( OPEN_PAR_SYMBOL ( field_or_variable_list )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:776:2: OPEN_PAR_SYMBOL ( field_or_variable_list )? CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_load_data_file_target_list4323);
            if  (HASEXCEPTION())
            {
                goto ruleload_data_file_target_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:776:18: ( field_or_variable_list )?
            {
                int alt291=2;
                {
                    int LA291_0 = LA(1);
                    if ( (((LA291_0 >= ACCOUNT_SYMBOL) && (LA291_0 <= YEAR_SYMBOL)) || ((LA291_0 >= AT_SIGN_SYMBOL) && (LA291_0 <= AT_TEXT_SUFFIX)) || LA291_0 == BACK_TICK_QUOTED_ID || LA291_0 == DOT_SYMBOL || LA291_0 == DOUBLE_QUOTED_TEXT || LA291_0 == IDENTIFIER))
                    {
                        alt291=1;
                    }
                }
                switch (alt291)
                {
            	case 1:
            	    // MySQLSimpleParser.g:776:18: field_or_variable_list
            	    {
            	        FOLLOWPUSH(FOLLOW_field_or_variable_list_in_load_data_file_target_list4325);
            	        field_or_variable_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_data_file_target_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_load_data_file_target_list4328);
            if  (HASEXCEPTION())
            {
                goto ruleload_data_file_target_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleload_data_file_target_listEx; /* Prevent compiler warnings */
    ruleload_data_file_target_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end load_data_file_target_list */

/**
 * $ANTLR start field_or_variable_list
 * MySQLSimpleParser.g:779:1: field_or_variable_list : ( column_ref | user_variable ) ( COMMA_SYMBOL ( column_ref | user_variable ) )* ;
 */
static void
field_or_variable_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:779:23: ( ( column_ref | user_variable ) ( COMMA_SYMBOL ( column_ref | user_variable ) )* )
        // MySQLSimpleParser.g:780:2: ( column_ref | user_variable ) ( COMMA_SYMBOL ( column_ref | user_variable ) )*
        {
            // MySQLSimpleParser.g:780:2: ( column_ref | user_variable )
            {
                int alt292=2;
                {
                    int LA292_0 = LA(1);
                    if ( (((LA292_0 >= ACCOUNT_SYMBOL) && (LA292_0 <= YEAR_SYMBOL)) || LA292_0 == BACK_TICK_QUOTED_ID || LA292_0 == DOT_SYMBOL || LA292_0 == DOUBLE_QUOTED_TEXT || LA292_0 == IDENTIFIER))
                    {
                        alt292=1;
                    }
                    else if ( (((LA292_0 >= AT_SIGN_SYMBOL) && (LA292_0 <= AT_TEXT_SUFFIX))))
                    {
                        alt292=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 292;
                        EXCEPTION->state        = 0;


                        goto rulefield_or_variable_listEx;

                    }
                }
                switch (alt292)
                {
            	case 1:
            	    // MySQLSimpleParser.g:780:3: column_ref
            	    {
            	        FOLLOWPUSH(FOLLOW_column_ref_in_field_or_variable_list4338);
            	        column_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_or_variable_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:780:16: user_variable
            	    {
            	        FOLLOWPUSH(FOLLOW_user_variable_in_field_or_variable_list4342);
            	        user_variable(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_or_variable_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:780:31: ( COMMA_SYMBOL ( column_ref | user_variable ) )*

            for (;;)
            {
                int alt294=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt294=1;
                	}
                    break;

                }

                switch (alt294)
                {
            	case 1:
            	    // MySQLSimpleParser.g:780:32: COMMA_SYMBOL ( column_ref | user_variable )
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_field_or_variable_list4346);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_or_variable_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:780:45: ( column_ref | user_variable )
            	        {
            	            int alt293=2;
            	            {
            	                int LA293_0 = LA(1);
            	                if ( (((LA293_0 >= ACCOUNT_SYMBOL) && (LA293_0 <= YEAR_SYMBOL)) || LA293_0 == BACK_TICK_QUOTED_ID || LA293_0 == DOT_SYMBOL || LA293_0 == DOUBLE_QUOTED_TEXT || LA293_0 == IDENTIFIER))
            	                {
            	                    alt293=1;
            	                }
            	                else if ( (((LA293_0 >= AT_SIGN_SYMBOL) && (LA293_0 <= AT_TEXT_SUFFIX))))
            	                {
            	                    alt293=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 293;
            	                    EXCEPTION->state        = 0;


            	                    goto rulefield_or_variable_listEx;

            	                }
            	            }
            	            switch (alt293)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:780:46: column_ref
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_column_ref_in_field_or_variable_list4349);
            	        	        column_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulefield_or_variable_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:780:59: user_variable
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_user_variable_in_field_or_variable_list4353);
            	        	        user_variable(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulefield_or_variable_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

            	default:
            	    goto loop294;	/* break out of the loop */
            	    break;
                }
            }
            loop294: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_or_variable_listEx; /* Prevent compiler warnings */
    rulefield_or_variable_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_or_variable_list */

/**
 * $ANTLR start replace_statement
 * MySQLSimpleParser.g:785:1: replace_statement : REPLACE_SYMBOL ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec ;
 */
static void
replace_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:785:18: ( REPLACE_SYMBOL ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec )
        // MySQLSimpleParser.g:786:2: REPLACE_SYMBOL ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL )? ( INTO_SYMBOL )? table_ref ( use_partition )? insert_field_spec
        {
             MATCHT(REPLACE_SYMBOL, &FOLLOW_REPLACE_SYMBOL_in_replace_statement4367);
            if  (HASEXCEPTION())
            {
                goto rulereplace_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:786:17: ( LOW_PRIORITY_SYMBOL | DELAYED_SYMBOL )?
            {
                int alt295=2;
                switch ( LA(1) )
                {
                    case DELAYED_SYMBOL:
                    case LOW_PRIORITY_SYMBOL:
                    	{
                    		alt295=1;
                    	}
                        break;
                }

                switch (alt295)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == DELAYED_SYMBOL || LA(1) == LOW_PRIORITY_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulereplace_statementEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:786:57: ( INTO_SYMBOL )?
            {
                int alt296=2;
                switch ( LA(1) )
                {
                    case INTO_SYMBOL:
                    	{
                    		alt296=1;
                    	}
                        break;
                }

                switch (alt296)
                {
            	case 1:
            	    // MySQLSimpleParser.g:786:57: INTO_SYMBOL
            	    {
            	         MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_replace_statement4378);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereplace_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_table_ref_in_replace_statement4381);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulereplace_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:787:3: ( use_partition )?
            {
                int alt297=2;
                {
                    int LA297_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50602)) && (LA297_0 == PARTITION_SYMBOL))
                    {
                        alt297=1;
                    }
                }
                switch (alt297)
                {
            	case 1:
            	    // MySQLSimpleParser.g:787:3: use_partition
            	    {
            	        FOLLOWPUSH(FOLLOW_use_partition_in_replace_statement4385);
            	        use_partition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereplace_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_insert_field_spec_in_replace_statement4388);
            insert_field_spec(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulereplace_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulereplace_statementEx; /* Prevent compiler warnings */
    rulereplace_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end replace_statement */

/**
 * $ANTLR start select_statement
 * MySQLSimpleParser.g:792:1: select_statement : ( SELECT_SYMBOL select_part2 ( union_clause )? | OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? );
 */
static void
select_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:792:17: ( SELECT_SYMBOL select_part2 ( union_clause )? | OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )? )

            ANTLR3_UINT32 alt300;

            alt300=2;

            switch ( LA(1) )
            {
            case SELECT_SYMBOL:
            	{
            		alt300=1;
            	}
                break;
            case OPEN_PAR_SYMBOL:
            	{
            		alt300=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 300;
                EXCEPTION->state        = 0;


                goto ruleselect_statementEx;

            }

            switch (alt300)
            {
        	case 1:
        	    // MySQLSimpleParser.g:793:2: SELECT_SYMBOL select_part2 ( union_clause )?
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_select_statement4399);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_part2_in_select_statement4401);
        	        select_part2(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:793:29: ( union_clause )?
        	        {
        	            int alt298=2;
        	            switch ( LA(1) )
        	            {
        	                case UNION_SYMBOL:
        	                	{
        	                		alt298=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt298)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:793:29: union_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_union_clause_in_select_statement4403);
        	        	        union_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleselect_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:794:4: OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL ( union_or_order_by_or_limit )?
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_select_statement4409);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_paren_in_select_statement4411);
        	        select_paren(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_select_statement4413);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:794:50: ( union_or_order_by_or_limit )?
        	        {
        	            int alt299=2;
        	            switch ( LA(1) )
        	            {
        	                case LIMIT_SYMBOL:
        	                case ORDER_SYMBOL:
        	                case UNION_SYMBOL:
        	                	{
        	                		alt299=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt299)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:794:50: union_or_order_by_or_limit
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_union_or_order_by_or_limit_in_select_statement4415);
        	        	        union_or_order_by_or_limit(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleselect_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_statementEx; /* Prevent compiler warnings */
    ruleselect_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_statement */

/**
 * $ANTLR start select_paren
 * MySQLSimpleParser.g:797:1: select_paren : ( SELECT_SYMBOL select_part2 | OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL );
 */
static void
select_paren(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:797:13: ( SELECT_SYMBOL select_part2 | OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt301;

            alt301=2;

            switch ( LA(1) )
            {
            case SELECT_SYMBOL:
            	{
            		alt301=1;
            	}
                break;
            case OPEN_PAR_SYMBOL:
            	{
            		alt301=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 301;
                EXCEPTION->state        = 0;


                goto ruleselect_parenEx;

            }

            switch (alt301)
            {
        	case 1:
        	    // MySQLSimpleParser.g:798:2: SELECT_SYMBOL select_part2
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_select_paren4425);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_parenEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_part2_in_select_paren4427);
        	        select_part2(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_parenEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:799:4: OPEN_PAR_SYMBOL select_paren CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_select_paren4432);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_parenEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_paren_in_select_paren4434);
        	        select_paren(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_parenEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren4436);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_parenEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_parenEx; /* Prevent compiler warnings */
    ruleselect_parenEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_paren */

/**
 * $ANTLR start select_from
 * MySQLSimpleParser.g:802:1: select_from : from_clause ( where_clause )? ( group_by_clause )? ( having_clause )? ( order_by_clause )? ( limit_clause )? ( procedure_analyse_clause )? ;
 */
static void
select_from(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:802:12: ( from_clause ( where_clause )? ( group_by_clause )? ( having_clause )? ( order_by_clause )? ( limit_clause )? ( procedure_analyse_clause )? )
        // MySQLSimpleParser.g:803:2: from_clause ( where_clause )? ( group_by_clause )? ( having_clause )? ( order_by_clause )? ( limit_clause )? ( procedure_analyse_clause )?
        {
            FOLLOWPUSH(FOLLOW_from_clause_in_select_from4445);
            from_clause(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleselect_fromEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:803:14: ( where_clause )?
            {
                int alt302=2;
                switch ( LA(1) )
                {
                    case WHERE_SYMBOL:
                    	{
                    		alt302=1;
                    	}
                        break;
                }

                switch (alt302)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:14: where_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_where_clause_in_select_from4447);
            	        where_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:803:28: ( group_by_clause )?
            {
                int alt303=2;
                switch ( LA(1) )
                {
                    case GROUP_SYMBOL:
                    	{
                    		alt303=1;
                    	}
                        break;
                }

                switch (alt303)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:28: group_by_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_group_by_clause_in_select_from4450);
            	        group_by_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:803:45: ( having_clause )?
            {
                int alt304=2;
                switch ( LA(1) )
                {
                    case HAVING_SYMBOL:
                    	{
                    		alt304=1;
                    	}
                        break;
                }

                switch (alt304)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:45: having_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_having_clause_in_select_from4453);
            	        having_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:803:60: ( order_by_clause )?
            {
                int alt305=2;
                switch ( LA(1) )
                {
                    case ORDER_SYMBOL:
                    	{
                    		alt305=1;
                    	}
                        break;
                }

                switch (alt305)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:60: order_by_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_order_by_clause_in_select_from4456);
            	        order_by_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:803:77: ( limit_clause )?
            {
                int alt306=2;
                switch ( LA(1) )
                {
                    case LIMIT_SYMBOL:
                    	{
                    		alt306=1;
                    	}
                        break;
                }

                switch (alt306)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:77: limit_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_limit_clause_in_select_from4459);
            	        limit_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:803:91: ( procedure_analyse_clause )?
            {
                int alt307=2;
                switch ( LA(1) )
                {
                    case PROCEDURE_SYMBOL:
                    	{
                    		alt307=1;
                    	}
                        break;
                }

                switch (alt307)
                {
            	case 1:
            	    // MySQLSimpleParser.g:803:91: procedure_analyse_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_procedure_analyse_clause_in_select_from4462);
            	        procedure_analyse_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_fromEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_fromEx; /* Prevent compiler warnings */
    ruleselect_fromEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_from */

/**
 * $ANTLR start select_part2
 * MySQLSimpleParser.g:806:1: select_part2 : ( options {greedy=true; } : select_option )* select_item_list ( ( order_by_clause )? ( limit_clause )? | into_clause ( select_from )? | select_from ( into_clause )? ) ( select_lock_type )? ;
 */
static void
select_part2(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:806:13: ( ( options {greedy=true; } : select_option )* select_item_list ( ( order_by_clause )? ( limit_clause )? | into_clause ( select_from )? | select_from ( into_clause )? ) ( select_lock_type )? )
        // MySQLSimpleParser.g:807:2: ( options {greedy=true; } : select_option )* select_item_list ( ( order_by_clause )? ( limit_clause )? | into_clause ( select_from )? | select_from ( into_clause )? ) ( select_lock_type )?
        {
            // MySQLSimpleParser.g:807:2: ( options {greedy=true; } : select_option )*

            for (;;)
            {
                int alt308=2;
                switch ( LA(1) )
                {
                case SQL_BUFFER_RESULT_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA308_96 = LA(2);
                		    if ( (LA308_96 == DOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MOD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MINUS_OPERATOR || LA308_96 == PLUS_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == NOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == SOUNDS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == IDENTIFIER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == DATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ASCII_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == INTERVAL_SYMBOL || LA308_96 == NCHAR_TEXT || LA308_96 == NOT2_SYMBOL || LA308_96 == SINGLE_QUOTED_TEXT || LA308_96 == BITWISE_NOT_OPERATOR || LA308_96 == LOGICAL_NOT_OPERATOR || LA308_96 == UNDERSCORE_CHARSET))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == DECIMAL_NUMBER || LA308_96 == FLOAT_NUMBER || LA308_96 == HEX_NUMBER || LA308_96 == INT_NUMBER || LA308_96 == LONG_NUMBER || LA308_96 == ULONGLONG_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == TIMESTAMP_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == DAY_SYMBOL || LA308_96 == FALSE_SYMBOL || ((LA308_96 >= NULL2_SYMBOL) && (LA308_96 <= NULL_SYMBOL)) || LA308_96 == TRUE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == CHAR_SYMBOL || LA308_96 == CURRENT_USER_SYMBOL || LA308_96 == BIN_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == HOUR_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MINUTE_SYMBOL || LA308_96 == INSERT_SYMBOL || LA308_96 == LEFT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MONTH_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == SECOND_SYMBOL || LA308_96 == RIGHT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == USER_SYMBOL || LA308_96 == TRIM_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == YEAR_SYMBOL || LA308_96 == VALUES_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ADDDATE_SYMBOL || LA308_96 == SUBDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == GET_FORMAT_SYMBOL || LA308_96 == CURDATE_SYMBOL || LA308_96 == CURTIME_SYMBOL || LA308_96 == DATE_ADD_SYMBOL || LA308_96 == DATE_SUB_SYMBOL || LA308_96 == EXTRACT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (((LA308_96 >= TIMESTAMP_ADD_SYMBOL) && (LA308_96 <= TIMESTAMP_DIFF_SYMBOL)) || LA308_96 == NOW_SYMBOL || LA308_96 == POSITION_SYMBOL || LA308_96 == SUBSTRING_SYMBOL || LA308_96 == SYSDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == COALESCE_SYMBOL || ((LA308_96 >= UTC_DATE_SYMBOL) && (LA308_96 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == CHARSET_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == FORMAT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == COLLATION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MICROSECOND_SYMBOL || LA308_96 == DATABASE_SYMBOL || LA308_96 == IF_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == TRUNCATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MULT_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == QUARTER_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == REVERSE_SYMBOL || ((LA308_96 >= REPEAT_SYMBOL) && (LA308_96 <= REPLACE_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ROW_COUNT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == WEEK_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == CONTAINS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == LINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MULTIPOINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == POINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == POLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == AVG_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ACCOUNT_SYMBOL || ((LA308_96 >= ALWAYS_SYMBOL) && (LA308_96 <= CACHE_SYMBOL)) || ((LA308_96 >= CHECKSUM_SYMBOL) && (LA308_96 <= COMMIT_SYMBOL)) || ((LA308_96 >= DEALLOCATE_SYMBOL) && (LA308_96 <= FOLLOWS_SYMBOL)) || ((LA308_96 >= GROUP_REPLICATION_SYMBOL) && (LA308_96 <= STOP_SYMBOL)) || ((LA308_96 >= UNICODE_SYMBOL) && (LA308_96 <= XA_SYMBOL)) || LA308_96 == SHUTDOWN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ROW_SYMBOL || LA308_96 == COUNT_SYMBOL || LA308_96 == EXISTS_SYMBOL || LA308_96 == GROUP_CONCAT_SYMBOL || LA308_96 == MAX_SYMBOL || LA308_96 == MIN_SYMBOL || LA308_96 == OPEN_PAR_SYMBOL || LA308_96 == PARAM_MARKER || LA308_96 == STDDEV_SAMP_SYMBOL || LA308_96 == STD_SYMBOL || LA308_96 == SUM_SYMBOL || LA308_96 == VARIANCE_SYMBOL || LA308_96 == VAR_SAMP_SYMBOL || ((LA308_96 >= AT_AT_SIGN_SYMBOL) && (LA308_96 <= AT_TEXT_SUFFIX)) || ((LA308_96 >= BIT_AND_SYMBOL) && (LA308_96 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == SQL_BUFFER_RESULT_SYMBOL || LA308_96 == BINARY_SYMBOL || ((LA308_96 >= CASE_SYMBOL) && (LA308_96 <= CAST_SYMBOL)) || LA308_96 == CONVERT_SYMBOL || LA308_96 == MATCH_SYMBOL || LA308_96 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == SQL_NO_CACHE_SYMBOL || LA308_96 == ALL_SYMBOL || LA308_96 == DISTINCT_SYMBOL || LA308_96 == HIGH_PRIORITY_SYMBOL || ((LA308_96 >= SQL_BIG_RESULT_SYMBOL) && (LA308_96 <= SQL_SMALL_RESULT_SYMBOL)) || LA308_96 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == SQL_CACHE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_96 == ACTION_SYMBOL || ((LA308_96 >= AFTER_SYMBOL) && (LA308_96 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA308_96 >= BINLOG_SYMBOL) && (LA308_96 <= CLASS_ORIGIN_SYMBOL)) || LA308_96 == CODE_SYMBOL || ((LA308_96 >= COLUMN_NAME_SYMBOL) && (LA308_96 <= DATETIME_SYMBOL)) || ((LA308_96 >= DEFAULT_AUTH_SYMBOL) && (LA308_96 <= GEOMETRY_SYMBOL)) || ((LA308_96 >= GRANTS_SYMBOL) && (LA308_96 <= HOSTS_SYMBOL)) || ((LA308_96 >= IDENTIFIED_SYMBOL) && (LA308_96 <= LEVEL_SYMBOL)) || ((LA308_96 >= LIST_SYMBOL) && (LA308_96 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA308_96 >= MAX_SIZE_SYMBOL) && (LA308_96 <= MESSAGE_TEXT_SYMBOL)) || LA308_96 == MIGRATE_SYMBOL || ((LA308_96 >= MIN_ROWS_SYMBOL) && (LA308_96 <= MODE_SYMBOL)) || ((LA308_96 >= MUTEX_SYMBOL) && (LA308_96 <= OFFSET_SYMBOL)) || ((LA308_96 >= ONE_SHOT_SYMBOL) && (LA308_96 <= PARTITIONS_SYMBOL)) || ((LA308_96 >= PHASE_SYMBOL) && (LA308_96 <= PLUGINS_SYMBOL)) || ((LA308_96 >= PRESERVE_SYMBOL) && (LA308_96 <= PROXY_SYMBOL)) || ((LA308_96 >= QUERY_SYMBOL) && (LA308_96 <= RETURNS_SYMBOL)) || ((LA308_96 >= ROLLUP_SYMBOL) && (LA308_96 <= ROWS_SYMBOL)) || LA308_96 == ROW_FORMAT_SYMBOL || ((LA308_96 >= RTREE_SYMBOL) && (LA308_96 <= SCHEMA_NAME_SYMBOL)) || ((LA308_96 >= SERIAL_SYMBOL) && (LA308_96 <= SHARE_SYMBOL)) || ((LA308_96 >= SLOW_SYMBOL) && (LA308_96 <= SNAPSHOT_SYMBOL)) || ((LA308_96 >= SOURCE_SYMBOL) && (LA308_96 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA308_96 >= SQL_THREAD_SYMBOL) && (LA308_96 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA308_96 >= SUBJECT_SYMBOL) && (LA308_96 <= TRIGGERS_SYMBOL)) || ((LA308_96 >= TYPES_SYMBOL) && (LA308_96 <= USER_RESOURCES_SYMBOL)) || ((LA308_96 >= USE_FRM_SYMBOL) && (LA308_96 <= WAIT_SYMBOL)) || LA308_96 == WORK_SYMBOL || ((LA308_96 >= X509_SYMBOL) && (LA308_96 <= XML_SYMBOL))))
                		    {
                		        alt308=1;
                		    }

                		}
                	}
                    break;
                case SQL_NO_CACHE_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA308_104 = LA(2);
                		    if ( (LA308_104 == DOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MOD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MINUS_OPERATOR || LA308_104 == PLUS_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == NOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == SOUNDS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == IDENTIFIER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == DATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ASCII_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == INTERVAL_SYMBOL || LA308_104 == NCHAR_TEXT || LA308_104 == NOT2_SYMBOL || LA308_104 == SINGLE_QUOTED_TEXT || LA308_104 == BITWISE_NOT_OPERATOR || LA308_104 == LOGICAL_NOT_OPERATOR || LA308_104 == UNDERSCORE_CHARSET))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == DECIMAL_NUMBER || LA308_104 == FLOAT_NUMBER || LA308_104 == HEX_NUMBER || LA308_104 == INT_NUMBER || LA308_104 == LONG_NUMBER || LA308_104 == ULONGLONG_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == TIMESTAMP_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == DAY_SYMBOL || LA308_104 == FALSE_SYMBOL || ((LA308_104 >= NULL2_SYMBOL) && (LA308_104 <= NULL_SYMBOL)) || LA308_104 == TRUE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == CHAR_SYMBOL || LA308_104 == CURRENT_USER_SYMBOL || LA308_104 == BIN_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == HOUR_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MINUTE_SYMBOL || LA308_104 == INSERT_SYMBOL || LA308_104 == LEFT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MONTH_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == SECOND_SYMBOL || LA308_104 == RIGHT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == USER_SYMBOL || LA308_104 == TRIM_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == YEAR_SYMBOL || LA308_104 == VALUES_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ADDDATE_SYMBOL || LA308_104 == SUBDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == GET_FORMAT_SYMBOL || LA308_104 == CURDATE_SYMBOL || LA308_104 == CURTIME_SYMBOL || LA308_104 == DATE_ADD_SYMBOL || LA308_104 == DATE_SUB_SYMBOL || LA308_104 == EXTRACT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (((LA308_104 >= TIMESTAMP_ADD_SYMBOL) && (LA308_104 <= TIMESTAMP_DIFF_SYMBOL)) || LA308_104 == NOW_SYMBOL || LA308_104 == POSITION_SYMBOL || LA308_104 == SUBSTRING_SYMBOL || LA308_104 == SYSDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == COALESCE_SYMBOL || ((LA308_104 >= UTC_DATE_SYMBOL) && (LA308_104 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == CHARSET_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == FORMAT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == COLLATION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MICROSECOND_SYMBOL || LA308_104 == DATABASE_SYMBOL || LA308_104 == IF_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == TRUNCATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MULT_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == QUARTER_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == REVERSE_SYMBOL || ((LA308_104 >= REPEAT_SYMBOL) && (LA308_104 <= REPLACE_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ROW_COUNT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == WEEK_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == CONTAINS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == LINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MULTIPOINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == POINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == POLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == AVG_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ACCOUNT_SYMBOL || ((LA308_104 >= ALWAYS_SYMBOL) && (LA308_104 <= CACHE_SYMBOL)) || ((LA308_104 >= CHECKSUM_SYMBOL) && (LA308_104 <= COMMIT_SYMBOL)) || ((LA308_104 >= DEALLOCATE_SYMBOL) && (LA308_104 <= FOLLOWS_SYMBOL)) || ((LA308_104 >= GROUP_REPLICATION_SYMBOL) && (LA308_104 <= STOP_SYMBOL)) || ((LA308_104 >= UNICODE_SYMBOL) && (LA308_104 <= XA_SYMBOL)) || LA308_104 == SHUTDOWN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ROW_SYMBOL || LA308_104 == COUNT_SYMBOL || LA308_104 == EXISTS_SYMBOL || LA308_104 == GROUP_CONCAT_SYMBOL || LA308_104 == MAX_SYMBOL || LA308_104 == MIN_SYMBOL || LA308_104 == OPEN_PAR_SYMBOL || LA308_104 == PARAM_MARKER || LA308_104 == STDDEV_SAMP_SYMBOL || LA308_104 == STD_SYMBOL || LA308_104 == SUM_SYMBOL || LA308_104 == VARIANCE_SYMBOL || LA308_104 == VAR_SAMP_SYMBOL || ((LA308_104 >= AT_AT_SIGN_SYMBOL) && (LA308_104 <= AT_TEXT_SUFFIX)) || ((LA308_104 >= BIT_AND_SYMBOL) && (LA308_104 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == SQL_BUFFER_RESULT_SYMBOL || LA308_104 == BINARY_SYMBOL || ((LA308_104 >= CASE_SYMBOL) && (LA308_104 <= CAST_SYMBOL)) || LA308_104 == CONVERT_SYMBOL || LA308_104 == MATCH_SYMBOL || LA308_104 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == SQL_NO_CACHE_SYMBOL || LA308_104 == ALL_SYMBOL || LA308_104 == DISTINCT_SYMBOL || LA308_104 == HIGH_PRIORITY_SYMBOL || ((LA308_104 >= SQL_BIG_RESULT_SYMBOL) && (LA308_104 <= SQL_SMALL_RESULT_SYMBOL)) || LA308_104 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == SQL_CACHE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_104 == ACTION_SYMBOL || ((LA308_104 >= AFTER_SYMBOL) && (LA308_104 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA308_104 >= BINLOG_SYMBOL) && (LA308_104 <= CLASS_ORIGIN_SYMBOL)) || LA308_104 == CODE_SYMBOL || ((LA308_104 >= COLUMN_NAME_SYMBOL) && (LA308_104 <= DATETIME_SYMBOL)) || ((LA308_104 >= DEFAULT_AUTH_SYMBOL) && (LA308_104 <= GEOMETRY_SYMBOL)) || ((LA308_104 >= GRANTS_SYMBOL) && (LA308_104 <= HOSTS_SYMBOL)) || ((LA308_104 >= IDENTIFIED_SYMBOL) && (LA308_104 <= LEVEL_SYMBOL)) || ((LA308_104 >= LIST_SYMBOL) && (LA308_104 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA308_104 >= MAX_SIZE_SYMBOL) && (LA308_104 <= MESSAGE_TEXT_SYMBOL)) || LA308_104 == MIGRATE_SYMBOL || ((LA308_104 >= MIN_ROWS_SYMBOL) && (LA308_104 <= MODE_SYMBOL)) || ((LA308_104 >= MUTEX_SYMBOL) && (LA308_104 <= OFFSET_SYMBOL)) || ((LA308_104 >= ONE_SHOT_SYMBOL) && (LA308_104 <= PARTITIONS_SYMBOL)) || ((LA308_104 >= PHASE_SYMBOL) && (LA308_104 <= PLUGINS_SYMBOL)) || ((LA308_104 >= PRESERVE_SYMBOL) && (LA308_104 <= PROXY_SYMBOL)) || ((LA308_104 >= QUERY_SYMBOL) && (LA308_104 <= RETURNS_SYMBOL)) || ((LA308_104 >= ROLLUP_SYMBOL) && (LA308_104 <= ROWS_SYMBOL)) || LA308_104 == ROW_FORMAT_SYMBOL || ((LA308_104 >= RTREE_SYMBOL) && (LA308_104 <= SCHEMA_NAME_SYMBOL)) || ((LA308_104 >= SERIAL_SYMBOL) && (LA308_104 <= SHARE_SYMBOL)) || ((LA308_104 >= SLOW_SYMBOL) && (LA308_104 <= SNAPSHOT_SYMBOL)) || ((LA308_104 >= SOURCE_SYMBOL) && (LA308_104 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA308_104 >= SQL_THREAD_SYMBOL) && (LA308_104 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA308_104 >= SUBJECT_SYMBOL) && (LA308_104 <= TRIGGERS_SYMBOL)) || ((LA308_104 >= TYPES_SYMBOL) && (LA308_104 <= USER_RESOURCES_SYMBOL)) || ((LA308_104 >= USE_FRM_SYMBOL) && (LA308_104 <= WAIT_SYMBOL)) || LA308_104 == WORK_SYMBOL || ((LA308_104 >= X509_SYMBOL) && (LA308_104 <= XML_SYMBOL))))
                		    {
                		        alt308=1;
                		    }

                		}
                	}
                    break;
                case ALL_SYMBOL:
                case DISTINCT_SYMBOL:
                case HIGH_PRIORITY_SYMBOL:
                case SQL_BIG_RESULT_SYMBOL:
                case SQL_CALC_FOUND_ROWS_SYMBOL:
                case SQL_SMALL_RESULT_SYMBOL:
                case STRAIGHT_JOIN_SYMBOL:
                	{
                		alt308=1;
                	}
                    break;
                case SQL_CACHE_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA308_106 = LA(2);
                		    if ( (LA308_106 == DOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MOD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MINUS_OPERATOR || LA308_106 == PLUS_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == NOT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == SOUNDS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == IDENTIFIER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == DATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ASCII_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == INTERVAL_SYMBOL || LA308_106 == NCHAR_TEXT || LA308_106 == NOT2_SYMBOL || LA308_106 == SINGLE_QUOTED_TEXT || LA308_106 == BITWISE_NOT_OPERATOR || LA308_106 == LOGICAL_NOT_OPERATOR || LA308_106 == UNDERSCORE_CHARSET))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == DECIMAL_NUMBER || LA308_106 == FLOAT_NUMBER || LA308_106 == HEX_NUMBER || LA308_106 == INT_NUMBER || LA308_106 == LONG_NUMBER || LA308_106 == ULONGLONG_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == TIMESTAMP_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == DAY_SYMBOL || LA308_106 == FALSE_SYMBOL || ((LA308_106 >= NULL2_SYMBOL) && (LA308_106 <= NULL_SYMBOL)) || LA308_106 == TRUE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == CHAR_SYMBOL || LA308_106 == CURRENT_USER_SYMBOL || LA308_106 == BIN_NUMBER))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == HOUR_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MINUTE_SYMBOL || LA308_106 == INSERT_SYMBOL || LA308_106 == LEFT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MONTH_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == SECOND_SYMBOL || LA308_106 == RIGHT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == USER_SYMBOL || LA308_106 == TRIM_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == YEAR_SYMBOL || LA308_106 == VALUES_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ADDDATE_SYMBOL || LA308_106 == SUBDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == GET_FORMAT_SYMBOL || LA308_106 == CURDATE_SYMBOL || LA308_106 == CURTIME_SYMBOL || LA308_106 == DATE_ADD_SYMBOL || LA308_106 == DATE_SUB_SYMBOL || LA308_106 == EXTRACT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (((LA308_106 >= TIMESTAMP_ADD_SYMBOL) && (LA308_106 <= TIMESTAMP_DIFF_SYMBOL)) || LA308_106 == NOW_SYMBOL || LA308_106 == POSITION_SYMBOL || LA308_106 == SUBSTRING_SYMBOL || LA308_106 == SYSDATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == COALESCE_SYMBOL || ((LA308_106 >= UTC_DATE_SYMBOL) && (LA308_106 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == CHARSET_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == FORMAT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == COLLATION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MICROSECOND_SYMBOL || LA308_106 == DATABASE_SYMBOL || LA308_106 == IF_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == TRUNCATE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MULT_OPERATOR))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == PASSWORD_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == QUARTER_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == REVERSE_SYMBOL || ((LA308_106 >= REPEAT_SYMBOL) && (LA308_106 <= REPLACE_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ROW_COUNT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == WEEK_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == CONTAINS_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == LINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MULTIPOINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == POINT_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == POLYGON_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == AVG_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ACCOUNT_SYMBOL || ((LA308_106 >= ALWAYS_SYMBOL) && (LA308_106 <= CACHE_SYMBOL)) || ((LA308_106 >= CHECKSUM_SYMBOL) && (LA308_106 <= COMMIT_SYMBOL)) || ((LA308_106 >= DEALLOCATE_SYMBOL) && (LA308_106 <= FOLLOWS_SYMBOL)) || ((LA308_106 >= GROUP_REPLICATION_SYMBOL) && (LA308_106 <= STOP_SYMBOL)) || ((LA308_106 >= UNICODE_SYMBOL) && (LA308_106 <= XA_SYMBOL)) || LA308_106 == SHUTDOWN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ROW_SYMBOL || LA308_106 == COUNT_SYMBOL || LA308_106 == EXISTS_SYMBOL || LA308_106 == GROUP_CONCAT_SYMBOL || LA308_106 == MAX_SYMBOL || LA308_106 == MIN_SYMBOL || LA308_106 == OPEN_PAR_SYMBOL || LA308_106 == PARAM_MARKER || LA308_106 == STDDEV_SAMP_SYMBOL || LA308_106 == STD_SYMBOL || LA308_106 == SUM_SYMBOL || LA308_106 == VARIANCE_SYMBOL || LA308_106 == VAR_SAMP_SYMBOL || ((LA308_106 >= AT_AT_SIGN_SYMBOL) && (LA308_106 <= AT_TEXT_SUFFIX)) || ((LA308_106 >= BIT_AND_SYMBOL) && (LA308_106 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == SQL_BUFFER_RESULT_SYMBOL || LA308_106 == BINARY_SYMBOL || ((LA308_106 >= CASE_SYMBOL) && (LA308_106 <= CAST_SYMBOL)) || LA308_106 == CONVERT_SYMBOL || LA308_106 == MATCH_SYMBOL || LA308_106 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == SQL_NO_CACHE_SYMBOL || LA308_106 == ALL_SYMBOL || LA308_106 == DISTINCT_SYMBOL || LA308_106 == HIGH_PRIORITY_SYMBOL || ((LA308_106 >= SQL_BIG_RESULT_SYMBOL) && (LA308_106 <= SQL_SMALL_RESULT_SYMBOL)) || LA308_106 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == SQL_CACHE_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == MAX_STATEMENT_TIME_SYMBOL))
                		    {
                		        alt308=1;
                		    }
                		    else if ( (LA308_106 == ACTION_SYMBOL || ((LA308_106 >= AFTER_SYMBOL) && (LA308_106 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA308_106 >= BINLOG_SYMBOL) && (LA308_106 <= CLASS_ORIGIN_SYMBOL)) || LA308_106 == CODE_SYMBOL || ((LA308_106 >= COLUMN_NAME_SYMBOL) && (LA308_106 <= DATETIME_SYMBOL)) || ((LA308_106 >= DEFAULT_AUTH_SYMBOL) && (LA308_106 <= GEOMETRY_SYMBOL)) || ((LA308_106 >= GRANTS_SYMBOL) && (LA308_106 <= HOSTS_SYMBOL)) || ((LA308_106 >= IDENTIFIED_SYMBOL) && (LA308_106 <= LEVEL_SYMBOL)) || ((LA308_106 >= LIST_SYMBOL) && (LA308_106 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA308_106 >= MAX_SIZE_SYMBOL) && (LA308_106 <= MESSAGE_TEXT_SYMBOL)) || LA308_106 == MIGRATE_SYMBOL || ((LA308_106 >= MIN_ROWS_SYMBOL) && (LA308_106 <= MODE_SYMBOL)) || ((LA308_106 >= MUTEX_SYMBOL) && (LA308_106 <= OFFSET_SYMBOL)) || ((LA308_106 >= ONE_SHOT_SYMBOL) && (LA308_106 <= PARTITIONS_SYMBOL)) || ((LA308_106 >= PHASE_SYMBOL) && (LA308_106 <= PLUGINS_SYMBOL)) || ((LA308_106 >= PRESERVE_SYMBOL) && (LA308_106 <= PROXY_SYMBOL)) || ((LA308_106 >= QUERY_SYMBOL) && (LA308_106 <= RETURNS_SYMBOL)) || ((LA308_106 >= ROLLUP_SYMBOL) && (LA308_106 <= ROWS_SYMBOL)) || LA308_106 == ROW_FORMAT_SYMBOL || ((LA308_106 >= RTREE_SYMBOL) && (LA308_106 <= SCHEMA_NAME_SYMBOL)) || ((LA308_106 >= SERIAL_SYMBOL) && (LA308_106 <= SHARE_SYMBOL)) || ((LA308_106 >= SLOW_SYMBOL) && (LA308_106 <= SNAPSHOT_SYMBOL)) || ((LA308_106 >= SOURCE_SYMBOL) && (LA308_106 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA308_106 >= SQL_THREAD_SYMBOL) && (LA308_106 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA308_106 >= SUBJECT_SYMBOL) && (LA308_106 <= TRIGGERS_SYMBOL)) || ((LA308_106 >= TYPES_SYMBOL) && (LA308_106 <= USER_RESOURCES_SYMBOL)) || ((LA308_106 >= USE_FRM_SYMBOL) && (LA308_106 <= WAIT_SYMBOL)) || LA308_106 == WORK_SYMBOL || ((LA308_106 >= X509_SYMBOL) && (LA308_106 <= XML_SYMBOL))))
                		    {
                		        alt308=1;
                		    }

                		}
                	}
                    break;
                case MAX_STATEMENT_TIME_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case EQUAL_OPERATOR:
                			{
                				{
                				   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                				    */
                				    int LA308_535 = LA(3);
                				    if ( ((SERVER_VERSION >= 50704 && SERVER_VERSION < 50708)))
                				    {
                				        alt308=1;
                				    }

                				}
                			}
                		    break;

                		}

                	}
                    break;

                }

                switch (alt308)
                {
            	case 1:
            	    // MySQLSimpleParser.g:807:32: select_option
            	    {
            	        FOLLOWPUSH(FOLLOW_select_option_in_select_part24486);
            	        select_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_part2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop308;	/* break out of the loop */
            	    break;
                }
            }
            loop308: ; /* Jump out to here if this rule does not match */


            FOLLOWPUSH(FOLLOW_select_item_list_in_select_part24490);
            select_item_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleselect_part2Ex;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:808:3: ( ( order_by_clause )? ( limit_clause )? | into_clause ( select_from )? | select_from ( into_clause )? )
            {
                int alt313=3;
                switch ( LA(1) )
                {
                case EOF:
                case FOR_SYMBOL:
                case LIMIT_SYMBOL:
                case LOCK_SYMBOL:
                case ON_SYMBOL:
                case ORDER_SYMBOL:
                case SEMICOLON_SYMBOL:
                case UNION_SYMBOL:
                case WITH_SYMBOL:
                case CLOSE_PAR_SYMBOL:
                	{
                		alt313=1;
                	}
                    break;
                case INTO_SYMBOL:
                	{
                		alt313=2;
                	}
                    break;
                case FROM_SYMBOL:
                	{
                		alt313=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 313;
                    EXCEPTION->state        = 0;


                    goto ruleselect_part2Ex;

                }

                switch (alt313)
                {
            	case 1:
            	    // MySQLSimpleParser.g:809:4: ( order_by_clause )? ( limit_clause )?
            	    {
            	        // MySQLSimpleParser.g:809:4: ( order_by_clause )?
            	        {
            	            int alt309=2;
            	            switch ( LA(1) )
            	            {
            	                case ORDER_SYMBOL:
            	                	{
            	                		alt309=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt309)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:809:4: order_by_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_order_by_clause_in_select_part24499);
            	        	        order_by_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleselect_part2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:809:21: ( limit_clause )?
            	        {
            	            int alt310=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                	{
            	                		alt310=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt310)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:809:21: limit_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_select_part24502);
            	        	        limit_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleselect_part2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:810:6: into_clause ( select_from )?
            	    {
            	        FOLLOWPUSH(FOLLOW_into_clause_in_select_part24510);
            	        into_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_part2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:810:18: ( select_from )?
            	        {
            	            int alt311=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                	{
            	                		alt311=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt311)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:810:18: select_from
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_select_from_in_select_part24512);
            	        	        select_from(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleselect_part2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:811:6: select_from ( into_clause )?
            	    {
            	        FOLLOWPUSH(FOLLOW_select_from_in_select_part24520);
            	        select_from(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_part2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:811:18: ( into_clause )?
            	        {
            	            int alt312=2;
            	            switch ( LA(1) )
            	            {
            	                case INTO_SYMBOL:
            	                	{
            	                		alt312=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt312)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:811:18: into_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_into_clause_in_select_part24522);
            	        	        into_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleselect_part2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:813:3: ( select_lock_type )?
            {
                int alt314=2;
                switch ( LA(1) )
                {
                    case FOR_SYMBOL:
                    case LOCK_SYMBOL:
                    	{
                    		alt314=1;
                    	}
                        break;
                }

                switch (alt314)
                {
            	case 1:
            	    // MySQLSimpleParser.g:813:3: select_lock_type
            	    {
            	        FOLLOWPUSH(FOLLOW_select_lock_type_in_select_part24531);
            	        select_lock_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_part2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_part2Ex; /* Prevent compiler warnings */
    ruleselect_part2Ex: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_part2 */

/**
 * $ANTLR start table_expression
 * MySQLSimpleParser.g:816:1: table_expression : ( from_clause )? ( options {greedy=true; } : where_clause )? ( options {greedy=true; } : group_by_clause )? ( options {greedy=true; } : having_clause )? ( options {greedy=true; } : order_by_clause )? ( options {greedy=true; } : limit_clause )? ( options {greedy=true; } : procedure_analyse_clause )? ( options {greedy=true; } : select_lock_type )? ;
 */
static void
table_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:816:17: ( ( from_clause )? ( options {greedy=true; } : where_clause )? ( options {greedy=true; } : group_by_clause )? ( options {greedy=true; } : having_clause )? ( options {greedy=true; } : order_by_clause )? ( options {greedy=true; } : limit_clause )? ( options {greedy=true; } : procedure_analyse_clause )? ( options {greedy=true; } : select_lock_type )? )
        // MySQLSimpleParser.g:817:2: ( from_clause )? ( options {greedy=true; } : where_clause )? ( options {greedy=true; } : group_by_clause )? ( options {greedy=true; } : having_clause )? ( options {greedy=true; } : order_by_clause )? ( options {greedy=true; } : limit_clause )? ( options {greedy=true; } : procedure_analyse_clause )? ( options {greedy=true; } : select_lock_type )?
        {
            // MySQLSimpleParser.g:817:2: ( from_clause )?
            {
                int alt315=2;
                switch ( LA(1) )
                {
                    case FROM_SYMBOL:
                    	{
                    		alt315=1;
                    	}
                        break;
                }

                switch (alt315)
                {
            	case 1:
            	    // MySQLSimpleParser.g:817:2: from_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_from_clause_in_table_expression4541);
            	        from_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:818:2: ( options {greedy=true; } : where_clause )?
            {
                int alt316=2;
                switch ( LA(1) )
                {
                    case WHERE_SYMBOL:
                    	{
                    		alt316=1;
                    	}
                        break;
                }

                switch (alt316)
                {
            	case 1:
            	    // MySQLSimpleParser.g:818:32: where_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_where_clause_in_table_expression4559);
            	        where_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:819:2: ( options {greedy=true; } : group_by_clause )?
            {
                int alt317=2;
                switch ( LA(1) )
                {
                    case GROUP_SYMBOL:
                    	{
                    		alt317=1;
                    	}
                        break;
                }

                switch (alt317)
                {
            	case 1:
            	    // MySQLSimpleParser.g:819:32: group_by_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_group_by_clause_in_table_expression4578);
            	        group_by_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:820:2: ( options {greedy=true; } : having_clause )?
            {
                int alt318=2;
                switch ( LA(1) )
                {
                    case HAVING_SYMBOL:
                    	{
                    		alt318=1;
                    	}
                        break;
                }

                switch (alt318)
                {
            	case 1:
            	    // MySQLSimpleParser.g:820:32: having_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_having_clause_in_table_expression4597);
            	        having_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:821:2: ( options {greedy=true; } : order_by_clause )?
            {
                int alt319=2;
                switch ( LA(1) )
                {
                    case ORDER_SYMBOL:
                    	{
                    		alt319=1;
                    	}
                        break;
                }

                switch (alt319)
                {
            	case 1:
            	    // MySQLSimpleParser.g:821:32: order_by_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_order_by_clause_in_table_expression4616);
            	        order_by_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:822:2: ( options {greedy=true; } : limit_clause )?
            {
                int alt320=2;
                switch ( LA(1) )
                {
                    case LIMIT_SYMBOL:
                    	{
                    		alt320=1;
                    	}
                        break;
                }

                switch (alt320)
                {
            	case 1:
            	    // MySQLSimpleParser.g:822:32: limit_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_limit_clause_in_table_expression4635);
            	        limit_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:823:2: ( options {greedy=true; } : procedure_analyse_clause )?
            {
                int alt321=2;
                switch ( LA(1) )
                {
                    case PROCEDURE_SYMBOL:
                    	{
                    		alt321=1;
                    	}
                        break;
                }

                switch (alt321)
                {
            	case 1:
            	    // MySQLSimpleParser.g:823:32: procedure_analyse_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_procedure_analyse_clause_in_table_expression4654);
            	        procedure_analyse_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:824:2: ( options {greedy=true; } : select_lock_type )?
            {
                int alt322=2;
                switch ( LA(1) )
                {
                    case FOR_SYMBOL:
                    	{
                    		alt322=1;
                    	}
                        break;
                    case LOCK_SYMBOL:
                    	{
                    		alt322=1;
                    	}
                        break;
                }

                switch (alt322)
                {
            	case 1:
            	    // MySQLSimpleParser.g:824:32: select_lock_type
            	    {
            	        FOLLOWPUSH(FOLLOW_select_lock_type_in_table_expression4673);
            	        select_lock_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_expressionEx; /* Prevent compiler warnings */
    ruletable_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_expression */

/**
 * $ANTLR start subquery
 * MySQLSimpleParser.g:827:1: subquery : OPEN_PAR_SYMBOL query_expression_body CLOSE_PAR_SYMBOL ;
 */
static void
subquery(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:827:9: ( OPEN_PAR_SYMBOL query_expression_body CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:828:2: OPEN_PAR_SYMBOL query_expression_body CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_subquery4685);
            if  (HASEXCEPTION())
            {
                goto rulesubqueryEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_query_expression_body_in_subquery4687);
            query_expression_body(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesubqueryEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_subquery4689);
            if  (HASEXCEPTION())
            {
                goto rulesubqueryEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesubqueryEx; /* Prevent compiler warnings */
    rulesubqueryEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end subquery */

/**
 * $ANTLR start query_expression_body
 * MySQLSimpleParser.g:831:1: query_expression_body : query_specification ( UNION_SYMBOL union_option query_specification )* ;
 */
static void
query_expression_body(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:831:22: ( query_specification ( UNION_SYMBOL union_option query_specification )* )
        // MySQLSimpleParser.g:832:2: query_specification ( UNION_SYMBOL union_option query_specification )*
        {
            FOLLOWPUSH(FOLLOW_query_specification_in_query_expression_body4698);
            query_specification(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulequery_expression_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:832:22: ( UNION_SYMBOL union_option query_specification )*

            for (;;)
            {
                int alt323=2;
                switch ( LA(1) )
                {
                case UNION_SYMBOL:
                	{
                		alt323=1;
                	}
                    break;

                }

                switch (alt323)
                {
            	case 1:
            	    // MySQLSimpleParser.g:832:23: UNION_SYMBOL union_option query_specification
            	    {
            	         MATCHT(UNION_SYMBOL, &FOLLOW_UNION_SYMBOL_in_query_expression_body4701);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulequery_expression_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_union_option_in_query_expression_body4703);
            	        union_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulequery_expression_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_query_specification_in_query_expression_body4705);
            	        query_specification(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulequery_expression_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop323;	/* break out of the loop */
            	    break;
                }
            }
            loop323: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulequery_expression_bodyEx; /* Prevent compiler warnings */
    rulequery_expression_bodyEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end query_expression_body */

/**
 * $ANTLR start select_part2_derived
 * MySQLSimpleParser.g:835:1: select_part2_derived : ( options {greedy=true; } : query_spec_option |{...}? ( SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL ) )* select_item_list ;
 */
static void
select_part2_derived(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:835:21: ( ( options {greedy=true; } : query_spec_option |{...}? ( SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL ) )* select_item_list )
        // MySQLSimpleParser.g:836:2: ( options {greedy=true; } : query_spec_option |{...}? ( SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL ) )* select_item_list
        {
            // MySQLSimpleParser.g:836:2: ( options {greedy=true; } : query_spec_option |{...}? ( SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL ) )*

            for (;;)
            {
                int alt324=3;
                switch ( LA(1) )
                {
                case SQL_BUFFER_RESULT_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA324_96 = LA(2);
                		    if ( (LA324_96 == DOT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MOD_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MINUS_OPERATOR || LA324_96 == PLUS_OPERATOR))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == NOT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == SOUNDS_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == IDENTIFIER))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == DATE_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ASCII_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == INTERVAL_SYMBOL || LA324_96 == NCHAR_TEXT || LA324_96 == NOT2_SYMBOL || LA324_96 == SINGLE_QUOTED_TEXT || LA324_96 == BITWISE_NOT_OPERATOR || LA324_96 == LOGICAL_NOT_OPERATOR || LA324_96 == UNDERSCORE_CHARSET))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == DECIMAL_NUMBER || LA324_96 == FLOAT_NUMBER || LA324_96 == HEX_NUMBER || LA324_96 == INT_NUMBER || LA324_96 == LONG_NUMBER || LA324_96 == ULONGLONG_NUMBER))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == TIME_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == TIMESTAMP_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == DAY_SYMBOL || LA324_96 == FALSE_SYMBOL || ((LA324_96 >= NULL2_SYMBOL) && (LA324_96 <= NULL_SYMBOL)) || LA324_96 == TRUE_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == CHAR_SYMBOL || LA324_96 == CURRENT_USER_SYMBOL || LA324_96 == BIN_NUMBER))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == HOUR_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MINUTE_SYMBOL || LA324_96 == INSERT_SYMBOL || LA324_96 == LEFT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MONTH_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == SECOND_SYMBOL || LA324_96 == RIGHT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == USER_SYMBOL || LA324_96 == TRIM_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == YEAR_SYMBOL || LA324_96 == VALUES_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ADDDATE_SYMBOL || LA324_96 == SUBDATE_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == GET_FORMAT_SYMBOL || LA324_96 == CURDATE_SYMBOL || LA324_96 == CURTIME_SYMBOL || LA324_96 == DATE_ADD_SYMBOL || LA324_96 == DATE_SUB_SYMBOL || LA324_96 == EXTRACT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (((LA324_96 >= TIMESTAMP_ADD_SYMBOL) && (LA324_96 <= TIMESTAMP_DIFF_SYMBOL)) || LA324_96 == NOW_SYMBOL || LA324_96 == POSITION_SYMBOL || LA324_96 == SUBSTRING_SYMBOL || LA324_96 == SYSDATE_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == COALESCE_SYMBOL || ((LA324_96 >= UTC_DATE_SYMBOL) && (LA324_96 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == CHARSET_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == FORMAT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == COLLATION_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MICROSECOND_SYMBOL || LA324_96 == DATABASE_SYMBOL || LA324_96 == IF_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == TRUNCATE_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == OLD_PASSWORD_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MULT_OPERATOR))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == PASSWORD_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == QUARTER_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == REVERSE_SYMBOL || ((LA324_96 >= REPEAT_SYMBOL) && (LA324_96 <= REPLACE_SYMBOL))))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ROW_COUNT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == WEEK_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == CONTAINS_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == LINESTRING_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MULTIPOINT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == POINT_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == POLYGON_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == AVG_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ACCOUNT_SYMBOL || ((LA324_96 >= ALWAYS_SYMBOL) && (LA324_96 <= CACHE_SYMBOL)) || ((LA324_96 >= CHECKSUM_SYMBOL) && (LA324_96 <= COMMIT_SYMBOL)) || ((LA324_96 >= DEALLOCATE_SYMBOL) && (LA324_96 <= FOLLOWS_SYMBOL)) || ((LA324_96 >= GROUP_REPLICATION_SYMBOL) && (LA324_96 <= STOP_SYMBOL)) || ((LA324_96 >= UNICODE_SYMBOL) && (LA324_96 <= XA_SYMBOL)) || LA324_96 == SHUTDOWN_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ROW_SYMBOL || LA324_96 == COUNT_SYMBOL || LA324_96 == EXISTS_SYMBOL || LA324_96 == GROUP_CONCAT_SYMBOL || LA324_96 == MAX_SYMBOL || LA324_96 == MIN_SYMBOL || LA324_96 == OPEN_PAR_SYMBOL || LA324_96 == PARAM_MARKER || LA324_96 == STDDEV_SAMP_SYMBOL || LA324_96 == STD_SYMBOL || LA324_96 == SUM_SYMBOL || LA324_96 == VARIANCE_SYMBOL || LA324_96 == VAR_SAMP_SYMBOL || ((LA324_96 >= AT_AT_SIGN_SYMBOL) && (LA324_96 <= AT_TEXT_SUFFIX)) || ((LA324_96 >= BIT_AND_SYMBOL) && (LA324_96 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == SQL_BUFFER_RESULT_SYMBOL || LA324_96 == BINARY_SYMBOL || ((LA324_96 >= CASE_SYMBOL) && (LA324_96 <= CAST_SYMBOL)) || LA324_96 == CONVERT_SYMBOL || LA324_96 == MATCH_SYMBOL || LA324_96 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == SQL_CACHE_SYMBOL || LA324_96 == SQL_NO_CACHE_SYMBOL || LA324_96 == ALL_SYMBOL || LA324_96 == DISTINCT_SYMBOL || LA324_96 == HIGH_PRIORITY_SYMBOL || ((LA324_96 >= SQL_BIG_RESULT_SYMBOL) && (LA324_96 <= SQL_SMALL_RESULT_SYMBOL)) || LA324_96 == STRAIGHT_JOIN_SYMBOL))
                		    {
                		        alt324=1;
                		    }
                		    else if ( (LA324_96 == ACTION_SYMBOL || ((LA324_96 >= AFTER_SYMBOL) && (LA324_96 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA324_96 >= BINLOG_SYMBOL) && (LA324_96 <= CLASS_ORIGIN_SYMBOL)) || LA324_96 == CODE_SYMBOL || ((LA324_96 >= COLUMN_NAME_SYMBOL) && (LA324_96 <= DATETIME_SYMBOL)) || ((LA324_96 >= DEFAULT_AUTH_SYMBOL) && (LA324_96 <= GEOMETRY_SYMBOL)) || ((LA324_96 >= GRANTS_SYMBOL) && (LA324_96 <= HOSTS_SYMBOL)) || ((LA324_96 >= IDENTIFIED_SYMBOL) && (LA324_96 <= LEVEL_SYMBOL)) || ((LA324_96 >= LIST_SYMBOL) && (LA324_96 <= MESSAGE_TEXT_SYMBOL)) || LA324_96 == MIGRATE_SYMBOL || ((LA324_96 >= MIN_ROWS_SYMBOL) && (LA324_96 <= MODE_SYMBOL)) || ((LA324_96 >= MUTEX_SYMBOL) && (LA324_96 <= OFFSET_SYMBOL)) || ((LA324_96 >= ONE_SHOT_SYMBOL) && (LA324_96 <= PARTITIONS_SYMBOL)) || ((LA324_96 >= PHASE_SYMBOL) && (LA324_96 <= PLUGINS_SYMBOL)) || ((LA324_96 >= PRESERVE_SYMBOL) && (LA324_96 <= PROXY_SYMBOL)) || ((LA324_96 >= QUERY_SYMBOL) && (LA324_96 <= RETURNS_SYMBOL)) || ((LA324_96 >= ROLLUP_SYMBOL) && (LA324_96 <= ROWS_SYMBOL)) || LA324_96 == ROW_FORMAT_SYMBOL || ((LA324_96 >= RTREE_SYMBOL) && (LA324_96 <= SCHEMA_NAME_SYMBOL)) || ((LA324_96 >= SERIAL_SYMBOL) && (LA324_96 <= SHARE_SYMBOL)) || ((LA324_96 >= SLOW_SYMBOL) && (LA324_96 <= SNAPSHOT_SYMBOL)) || ((LA324_96 >= SOURCE_SYMBOL) && (LA324_96 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA324_96 >= SQL_THREAD_SYMBOL) && (LA324_96 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA324_96 >= SUBJECT_SYMBOL) && (LA324_96 <= TRIGGERS_SYMBOL)) || ((LA324_96 >= TYPES_SYMBOL) && (LA324_96 <= USER_RESOURCES_SYMBOL)) || ((LA324_96 >= USE_FRM_SYMBOL) && (LA324_96 <= WAIT_SYMBOL)) || LA324_96 == WORK_SYMBOL || ((LA324_96 >= X509_SYMBOL) && (LA324_96 <= XML_SYMBOL))))
                		    {
                		        alt324=1;
                		    }

                		}
                	}
                    break;
                case SQL_CACHE_SYMBOL:
                case SQL_NO_CACHE_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA324_104 = LA(2);
                		    if ( (LA324_104 == DOT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_246 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MOD_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_252 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MINUS_OPERATOR || LA324_104 == PLUS_OPERATOR))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_253 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == NOT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_257 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == SOUNDS_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_262 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == IDENTIFIER))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_269 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == BACK_TICK_QUOTED_ID))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_270 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == DOUBLE_QUOTED_TEXT))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_271 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == DATE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_272 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ASCII_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_273 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == SINGLE_QUOTED_TEXT))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_274 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ALL_SYMBOL || LA324_104 == BINARY_SYMBOL || ((LA324_104 >= CASE_SYMBOL) && (LA324_104 <= CAST_SYMBOL)) || LA324_104 == CHAR_SYMBOL || ((LA324_104 >= CONVERT_SYMBOL) && (LA324_104 <= COUNT_SYMBOL)) || LA324_104 == CURDATE_SYMBOL || LA324_104 == CURRENT_USER_SYMBOL || LA324_104 == CURTIME_SYMBOL || LA324_104 == DATABASE_SYMBOL || LA324_104 == DATE_ADD_SYMBOL || LA324_104 == DATE_SUB_SYMBOL || LA324_104 == DISTINCT_SYMBOL || LA324_104 == EXISTS_SYMBOL || ((LA324_104 >= EXTRACT_SYMBOL) && (LA324_104 <= FALSE_SYMBOL)) || LA324_104 == GROUP_CONCAT_SYMBOL || LA324_104 == HIGH_PRIORITY_SYMBOL || LA324_104 == IF_SYMBOL || LA324_104 == INSERT_SYMBOL || LA324_104 == INTERVAL_SYMBOL || LA324_104 == LEFT_SYMBOL || LA324_104 == MATCH_SYMBOL || LA324_104 == MAX_SYMBOL || LA324_104 == MIN_SYMBOL || LA324_104 == NCHAR_TEXT || LA324_104 == NOT2_SYMBOL || LA324_104 == NOW_SYMBOL || ((LA324_104 >= NULL2_SYMBOL) && (LA324_104 <= NULL_SYMBOL)) || ((LA324_104 >= OPEN_CURLY_SYMBOL) && (LA324_104 <= OPEN_PAR_SYMBOL)) || ((LA324_104 >= PARAM_MARKER) && (LA324_104 <= POSITION_SYMBOL)) || ((LA324_104 >= REPEAT_SYMBOL) && (LA324_104 <= REPLACE_SYMBOL)) || LA324_104 == RIGHT_SYMBOL || ((LA324_104 >= SQL_BIG_RESULT_SYMBOL) && (LA324_104 <= SQL_SMALL_RESULT_SYMBOL)) || LA324_104 == STDDEV_SAMP_SYMBOL || LA324_104 == STD_SYMBOL || ((LA324_104 >= STRAIGHT_JOIN_SYMBOL) && (LA324_104 <= SUBSTRING_SYMBOL)) || ((LA324_104 >= SUM_SYMBOL) && (LA324_104 <= SYSDATE_SYMBOL)) || ((LA324_104 >= TRIM_SYMBOL) && (LA324_104 <= TRUE_SYMBOL)) || ((LA324_104 >= UTC_DATE_SYMBOL) && (LA324_104 <= UTC_TIME_SYMBOL)) || LA324_104 == VALUES_SYMBOL || LA324_104 == VARIANCE_SYMBOL || LA324_104 == VAR_SAMP_SYMBOL || ((LA324_104 >= AT_AT_SIGN_SYMBOL) && (LA324_104 <= AT_TEXT_SUFFIX)) || LA324_104 == BITWISE_NOT_OPERATOR || ((LA324_104 >= BIT_AND_SYMBOL) && (LA324_104 <= BIT_XOR_SYMBOL)) || LA324_104 == DECIMAL_NUMBER || LA324_104 == FLOAT_NUMBER || LA324_104 == INT_NUMBER || LA324_104 == LOGICAL_NOT_OPERATOR || LA324_104 == LONG_NUMBER || ((LA324_104 >= ULONGLONG_NUMBER) && (LA324_104 <= UNDERSCORE_CHARSET))))
                		    {
                		        alt324=2;
                		    }
                		    else if ( (LA324_104 == HEX_NUMBER))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_292 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == TIME_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_294 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == TIMESTAMP_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_295 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == DAY_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_296 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == BIN_NUMBER))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_299 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == HOUR_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_302 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MINUTE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_303 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MONTH_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_306 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == SECOND_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_307 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == USER_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_309 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == YEAR_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_311 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ADDDATE_SYMBOL || LA324_104 == SUBDATE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_313 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == GET_FORMAT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_314 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (((LA324_104 >= TIMESTAMP_ADD_SYMBOL) && (LA324_104 <= TIMESTAMP_DIFF_SYMBOL))))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_319 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == COALESCE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_324 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == CHARSET_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_328 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == FORMAT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_329 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == COLLATION_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_330 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MICROSECOND_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_331 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == TRUNCATE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_334 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == OLD_PASSWORD_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_335 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MULT_OPERATOR))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_336 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == PASSWORD_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_337 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == QUARTER_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_338 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == REVERSE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_339 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ROW_COUNT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_342 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == WEEK_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_343 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == CONTAINS_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_344 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == WEIGHT_STRING_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_345 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_346 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == LINESTRING_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_347 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MULTILINESTRING_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_348 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MULTIPOINT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_349 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == MULTIPOLYGON_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_350 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == POINT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_351 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == POLYGON_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_352 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == AVG_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_353 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ACCOUNT_SYMBOL || ((LA324_104 >= ALWAYS_SYMBOL) && (LA324_104 <= CACHE_SYMBOL)) || ((LA324_104 >= CHECKSUM_SYMBOL) && (LA324_104 <= COMMIT_SYMBOL)) || ((LA324_104 >= DEALLOCATE_SYMBOL) && (LA324_104 <= FOLLOWS_SYMBOL)) || ((LA324_104 >= GROUP_REPLICATION_SYMBOL) && (LA324_104 <= STOP_SYMBOL)) || ((LA324_104 >= UNICODE_SYMBOL) && (LA324_104 <= XA_SYMBOL)) || LA324_104 == SHUTDOWN_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_354 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ROW_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_355 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == SQL_BUFFER_RESULT_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_374 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == SQL_CACHE_SYMBOL || LA324_104 == SQL_NO_CACHE_SYMBOL))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_382 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }
                		    else if ( (LA324_104 == ACTION_SYMBOL || ((LA324_104 >= AFTER_SYMBOL) && (LA324_104 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA324_104 >= BINLOG_SYMBOL) && (LA324_104 <= CLASS_ORIGIN_SYMBOL)) || LA324_104 == CODE_SYMBOL || ((LA324_104 >= COLUMN_NAME_SYMBOL) && (LA324_104 <= DATETIME_SYMBOL)) || ((LA324_104 >= DEFAULT_AUTH_SYMBOL) && (LA324_104 <= GEOMETRY_SYMBOL)) || ((LA324_104 >= GRANTS_SYMBOL) && (LA324_104 <= HOSTS_SYMBOL)) || ((LA324_104 >= IDENTIFIED_SYMBOL) && (LA324_104 <= LEVEL_SYMBOL)) || ((LA324_104 >= LIST_SYMBOL) && (LA324_104 <= MESSAGE_TEXT_SYMBOL)) || LA324_104 == MIGRATE_SYMBOL || ((LA324_104 >= MIN_ROWS_SYMBOL) && (LA324_104 <= MODE_SYMBOL)) || ((LA324_104 >= MUTEX_SYMBOL) && (LA324_104 <= OFFSET_SYMBOL)) || ((LA324_104 >= ONE_SHOT_SYMBOL) && (LA324_104 <= PARTITIONS_SYMBOL)) || ((LA324_104 >= PHASE_SYMBOL) && (LA324_104 <= PLUGINS_SYMBOL)) || ((LA324_104 >= PRESERVE_SYMBOL) && (LA324_104 <= PROXY_SYMBOL)) || ((LA324_104 >= QUERY_SYMBOL) && (LA324_104 <= RETURNS_SYMBOL)) || ((LA324_104 >= ROLLUP_SYMBOL) && (LA324_104 <= ROWS_SYMBOL)) || LA324_104 == ROW_FORMAT_SYMBOL || ((LA324_104 >= RTREE_SYMBOL) && (LA324_104 <= SCHEMA_NAME_SYMBOL)) || ((LA324_104 >= SERIAL_SYMBOL) && (LA324_104 <= SHARE_SYMBOL)) || ((LA324_104 >= SLOW_SYMBOL) && (LA324_104 <= SNAPSHOT_SYMBOL)) || ((LA324_104 >= SOURCE_SYMBOL) && (LA324_104 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA324_104 >= SQL_THREAD_SYMBOL) && (LA324_104 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA324_104 >= SUBJECT_SYMBOL) && (LA324_104 <= TRIGGERS_SYMBOL)) || ((LA324_104 >= TYPES_SYMBOL) && (LA324_104 <= USER_RESOURCES_SYMBOL)) || ((LA324_104 >= USE_FRM_SYMBOL) && (LA324_104 <= WAIT_SYMBOL)) || LA324_104 == WORK_SYMBOL || ((LA324_104 >= X509_SYMBOL) && (LA324_104 <= XML_SYMBOL))))
                		    {
                		        {
                		           /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		            */
                		            int LA324_384 = LA(3);
                		            if ( ((SERVER_VERSION <= 50100)))
                		            {
                		                alt324=2;
                		            }

                		        }
                		    }

                		}
                	}
                    break;
                case ALL_SYMBOL:
                case DISTINCT_SYMBOL:
                case HIGH_PRIORITY_SYMBOL:
                case SQL_BIG_RESULT_SYMBOL:
                case SQL_CALC_FOUND_ROWS_SYMBOL:
                case SQL_SMALL_RESULT_SYMBOL:
                case STRAIGHT_JOIN_SYMBOL:
                	{
                		alt324=1;
                	}
                    break;

                }

                switch (alt324)
                {
            	case 1:
            	    // MySQLSimpleParser.g:837:3: query_spec_option
            	    {
            	        FOLLOWPUSH(FOLLOW_query_spec_option_in_select_part2_derived4733);
            	        query_spec_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_part2_derivedEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:838:5: {...}? ( SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL )
            	    {
            	        if ( !((SERVER_VERSION <= 50100)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION <= 50100";
            	                EXCEPTION->ruleName	 = (void *)"select_part2_derived";


            	        }

            	        if ( LA(1) == SQL_CACHE_SYMBOL || LA(1) == SQL_NO_CACHE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleselect_part2_derivedEx;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop324;	/* break out of the loop */
            	    break;
                }
            }
            loop324: ; /* Jump out to here if this rule does not match */


            FOLLOWPUSH(FOLLOW_select_item_list_in_select_part2_derived4753);
            select_item_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleselect_part2_derivedEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_part2_derivedEx; /* Prevent compiler warnings */
    ruleselect_part2_derivedEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_part2_derived */

/**
 * $ANTLR start select_option
 * MySQLSimpleParser.g:842:1: select_option : ( query_spec_option | SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL |{...}? MAX_STATEMENT_TIME_SYMBOL EQUAL_OPERATOR real_ulong_number );
 */
static void
select_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:842:14: ( query_spec_option | SQL_NO_CACHE_SYMBOL | SQL_CACHE_SYMBOL |{...}? MAX_STATEMENT_TIME_SYMBOL EQUAL_OPERATOR real_ulong_number )

            ANTLR3_UINT32 alt325;

            alt325=4;

            switch ( LA(1) )
            {
            case SQL_BUFFER_RESULT_SYMBOL:
            case ALL_SYMBOL:
            case DISTINCT_SYMBOL:
            case HIGH_PRIORITY_SYMBOL:
            case SQL_BIG_RESULT_SYMBOL:
            case SQL_CALC_FOUND_ROWS_SYMBOL:
            case SQL_SMALL_RESULT_SYMBOL:
            case STRAIGHT_JOIN_SYMBOL:
            	{
            		alt325=1;
            	}
                break;
            case SQL_NO_CACHE_SYMBOL:
            	{
            		alt325=2;
            	}
                break;
            case SQL_CACHE_SYMBOL:
            	{
            		alt325=3;
            	}
                break;
            case MAX_STATEMENT_TIME_SYMBOL:
            	{
            		alt325=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 325;
                EXCEPTION->state        = 0;


                goto ruleselect_optionEx;

            }

            switch (alt325)
            {
        	case 1:
        	    // MySQLSimpleParser.g:843:2: query_spec_option
        	    {
        	        FOLLOWPUSH(FOLLOW_query_spec_option_in_select_option4762);
        	        query_spec_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:844:4: SQL_NO_CACHE_SYMBOL
        	    {
        	         MATCHT(SQL_NO_CACHE_SYMBOL, &FOLLOW_SQL_NO_CACHE_SYMBOL_in_select_option4767);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:845:4: SQL_CACHE_SYMBOL
        	    {
        	         MATCHT(SQL_CACHE_SYMBOL, &FOLLOW_SQL_CACHE_SYMBOL_in_select_option4772);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:846:4: {...}? MAX_STATEMENT_TIME_SYMBOL EQUAL_OPERATOR real_ulong_number
        	    {
        	        if ( !((SERVER_VERSION >= 50704 && SERVER_VERSION < 50708)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50704 && SERVER_VERSION < 50708";
        	                EXCEPTION->ruleName	 = (void *)"select_option";


        	        }

        	         MATCHT(MAX_STATEMENT_TIME_SYMBOL, &FOLLOW_MAX_STATEMENT_TIME_SYMBOL_in_select_option4779);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_select_option4781);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_select_option4783);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_optionEx; /* Prevent compiler warnings */
    ruleselect_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_option */

/**
 * $ANTLR start query_spec_option
 * MySQLSimpleParser.g:849:1: query_spec_option : ( ( ALL_SYMBOL | DISTINCT_SYMBOL ) | STRAIGHT_JOIN_SYMBOL | HIGH_PRIORITY_SYMBOL | SQL_SMALL_RESULT_SYMBOL | SQL_BIG_RESULT_SYMBOL | SQL_BUFFER_RESULT_SYMBOL | SQL_CALC_FOUND_ROWS_SYMBOL );
 */
static void
query_spec_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:849:18: ( ( ALL_SYMBOL | DISTINCT_SYMBOL ) | STRAIGHT_JOIN_SYMBOL | HIGH_PRIORITY_SYMBOL | SQL_SMALL_RESULT_SYMBOL | SQL_BIG_RESULT_SYMBOL | SQL_BUFFER_RESULT_SYMBOL | SQL_CALC_FOUND_ROWS_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == SQL_BUFFER_RESULT_SYMBOL || LA(1) == ALL_SYMBOL || LA(1) == DISTINCT_SYMBOL || LA(1) == HIGH_PRIORITY_SYMBOL || ((LA(1) >= SQL_BIG_RESULT_SYMBOL) && (LA(1) <= SQL_SMALL_RESULT_SYMBOL)) || LA(1) == STRAIGHT_JOIN_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulequery_spec_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulequery_spec_optionEx; /* Prevent compiler warnings */
    rulequery_spec_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end query_spec_option */

/**
 * $ANTLR start select_item_list
 * MySQLSimpleParser.g:859:1: select_item_list : ( select_item | MULT_OPERATOR ) ( options {greedy=true; } : COMMA_SYMBOL select_item )* ;
 */
static void
select_item_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:859:17: ( ( select_item | MULT_OPERATOR ) ( options {greedy=true; } : COMMA_SYMBOL select_item )* )
        // MySQLSimpleParser.g:860:2: ( select_item | MULT_OPERATOR ) ( options {greedy=true; } : COMMA_SYMBOL select_item )*
        {
            // MySQLSimpleParser.g:860:2: ( select_item | MULT_OPERATOR )
            {
                int alt326=2;
                {
                    int LA326_0 = LA(1);
                    if ( (((LA326_0 >= ACCOUNT_SYMBOL) && (LA326_0 <= YEAR_SYMBOL)) || LA326_0 == BINARY_SYMBOL || ((LA326_0 >= CASE_SYMBOL) && (LA326_0 <= CAST_SYMBOL)) || LA326_0 == CHAR_SYMBOL || ((LA326_0 >= CONVERT_SYMBOL) && (LA326_0 <= COUNT_SYMBOL)) || LA326_0 == CURDATE_SYMBOL || LA326_0 == CURRENT_USER_SYMBOL || LA326_0 == CURTIME_SYMBOL || LA326_0 == DATABASE_SYMBOL || LA326_0 == DATE_ADD_SYMBOL || LA326_0 == DATE_SUB_SYMBOL || LA326_0 == EXISTS_SYMBOL || ((LA326_0 >= EXTRACT_SYMBOL) && (LA326_0 <= FALSE_SYMBOL)) || LA326_0 == GROUP_CONCAT_SYMBOL || LA326_0 == IF_SYMBOL || LA326_0 == INSERT_SYMBOL || LA326_0 == INTERVAL_SYMBOL || LA326_0 == LEFT_SYMBOL || LA326_0 == MATCH_SYMBOL || LA326_0 == MAX_SYMBOL || LA326_0 == MIN_SYMBOL || LA326_0 == MOD_SYMBOL || LA326_0 == NCHAR_TEXT || ((LA326_0 >= NOT2_SYMBOL) && (LA326_0 <= NOW_SYMBOL)) || ((LA326_0 >= NULL2_SYMBOL) && (LA326_0 <= NULL_SYMBOL)) || ((LA326_0 >= OPEN_CURLY_SYMBOL) && (LA326_0 <= OPEN_PAR_SYMBOL)) || ((LA326_0 >= PARAM_MARKER) && (LA326_0 <= POSITION_SYMBOL)) || ((LA326_0 >= REPEAT_SYMBOL) && (LA326_0 <= REPLACE_SYMBOL)) || LA326_0 == RIGHT_SYMBOL || LA326_0 == SINGLE_QUOTED_TEXT || LA326_0 == STDDEV_SAMP_SYMBOL || LA326_0 == STD_SYMBOL || LA326_0 == SUBSTRING_SYMBOL || ((LA326_0 >= SUM_SYMBOL) && (LA326_0 <= SYSDATE_SYMBOL)) || ((LA326_0 >= TRIM_SYMBOL) && (LA326_0 <= TRUE_SYMBOL)) || ((LA326_0 >= UTC_DATE_SYMBOL) && (LA326_0 <= UTC_TIME_SYMBOL)) || LA326_0 == VALUES_SYMBOL || LA326_0 == VARIANCE_SYMBOL || LA326_0 == VAR_SAMP_SYMBOL || ((LA326_0 >= AT_AT_SIGN_SYMBOL) && (LA326_0 <= AT_TEXT_SUFFIX)) || ((LA326_0 >= BACK_TICK_QUOTED_ID) && (LA326_0 <= BIN_NUMBER)) || LA326_0 == BITWISE_NOT_OPERATOR || ((LA326_0 >= BIT_AND_SYMBOL) && (LA326_0 <= BIT_XOR_SYMBOL)) || LA326_0 == DECIMAL_NUMBER || LA326_0 == DOT_SYMBOL || LA326_0 == DOUBLE_QUOTED_TEXT || LA326_0 == FLOAT_NUMBER || ((LA326_0 >= HEX_NUMBER) && (LA326_0 <= IDENTIFIER)) || LA326_0 == INT_NUMBER || LA326_0 == LOGICAL_NOT_OPERATOR || ((LA326_0 >= LONG_NUMBER) && (LA326_0 <= MINUS_OPERATOR)) || LA326_0 == PLUS_OPERATOR || ((LA326_0 >= ULONGLONG_NUMBER) && (LA326_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt326=1;
                    }
                    else if ( (LA326_0 == MULT_OPERATOR))
                    {
                        alt326=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 326;
                        EXCEPTION->state        = 0;


                        goto ruleselect_item_listEx;

                    }
                }
                switch (alt326)
                {
            	case 1:
            	    // MySQLSimpleParser.g:860:3: select_item
            	    {
            	        FOLLOWPUSH(FOLLOW_select_item_in_select_item_list4838);
            	        select_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_item_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:860:17: MULT_OPERATOR
            	    {
            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_select_item_list4842);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_item_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:860:32: ( options {greedy=true; } : COMMA_SYMBOL select_item )*

            for (;;)
            {
                int alt327=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA327_22 = LA(2);
                		    if ( (LA327_22 == OPEN_PAR_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == IDENTIFIER))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == DATE_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ASCII_SYMBOL || LA327_22 == INTERVAL_SYMBOL || LA327_22 == NCHAR_TEXT || ((LA327_22 >= NOT2_SYMBOL) && (LA327_22 <= NOT_SYMBOL)) || LA327_22 == SINGLE_QUOTED_TEXT || LA327_22 == BITWISE_NOT_OPERATOR || LA327_22 == DECIMAL_NUMBER || LA327_22 == FLOAT_NUMBER || LA327_22 == INT_NUMBER || LA327_22 == LOGICAL_NOT_OPERATOR || ((LA327_22 >= LONG_NUMBER) && (LA327_22 <= MINUS_OPERATOR)) || LA327_22 == PLUS_OPERATOR || ((LA327_22 >= ULONGLONG_NUMBER) && (LA327_22 <= UNDERSCORE_CHARSET))))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == DOT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == TIME_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == TIMESTAMP_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == DAY_SYMBOL || LA327_22 == CHAR_SYMBOL || LA327_22 == CURRENT_USER_SYMBOL || LA327_22 == FALSE_SYMBOL || ((LA327_22 >= NULL2_SYMBOL) && (LA327_22 <= NULL_SYMBOL)) || LA327_22 == TRUE_SYMBOL || LA327_22 == BIN_NUMBER || LA327_22 == HEX_NUMBER))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == HOUR_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MINUTE_SYMBOL || LA327_22 == INSERT_SYMBOL || LA327_22 == LEFT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MONTH_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == SECOND_SYMBOL || LA327_22 == RIGHT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == USER_SYMBOL || LA327_22 == TRIM_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == YEAR_SYMBOL || LA327_22 == VALUES_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ADDDATE_SYMBOL || LA327_22 == SUBDATE_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == GET_FORMAT_SYMBOL || LA327_22 == CURDATE_SYMBOL || LA327_22 == CURTIME_SYMBOL || LA327_22 == DATE_ADD_SYMBOL || LA327_22 == DATE_SUB_SYMBOL || LA327_22 == EXTRACT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (((LA327_22 >= TIMESTAMP_ADD_SYMBOL) && (LA327_22 <= TIMESTAMP_DIFF_SYMBOL)) || LA327_22 == NOW_SYMBOL || LA327_22 == POSITION_SYMBOL || LA327_22 == SUBSTRING_SYMBOL || LA327_22 == SYSDATE_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == COALESCE_SYMBOL || ((LA327_22 >= UTC_DATE_SYMBOL) && (LA327_22 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == CHARSET_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == FORMAT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == COLLATION_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MICROSECOND_SYMBOL || LA327_22 == DATABASE_SYMBOL || LA327_22 == IF_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == TRUNCATE_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == OLD_PASSWORD_SYMBOL || LA327_22 == MOD_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == PASSWORD_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == QUARTER_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == REVERSE_SYMBOL || ((LA327_22 >= REPEAT_SYMBOL) && (LA327_22 <= REPLACE_SYMBOL))))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ROW_COUNT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == WEEK_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == CONTAINS_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == LINESTRING_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MULTIPOINT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == POINT_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == POLYGON_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == AVG_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ACCOUNT_SYMBOL || ((LA327_22 >= ALWAYS_SYMBOL) && (LA327_22 <= CACHE_SYMBOL)) || ((LA327_22 >= CHECKSUM_SYMBOL) && (LA327_22 <= COMMIT_SYMBOL)) || ((LA327_22 >= DEALLOCATE_SYMBOL) && (LA327_22 <= FOLLOWS_SYMBOL)) || ((LA327_22 >= GROUP_REPLICATION_SYMBOL) && (LA327_22 <= STOP_SYMBOL)) || ((LA327_22 >= UNICODE_SYMBOL) && (LA327_22 <= XA_SYMBOL)) || LA327_22 == SHUTDOWN_SYMBOL))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ROW_SYMBOL || LA327_22 == COUNT_SYMBOL || LA327_22 == EXISTS_SYMBOL || LA327_22 == GROUP_CONCAT_SYMBOL || LA327_22 == MAX_SYMBOL || LA327_22 == MIN_SYMBOL || LA327_22 == PARAM_MARKER || LA327_22 == STDDEV_SAMP_SYMBOL || LA327_22 == STD_SYMBOL || LA327_22 == SUM_SYMBOL || LA327_22 == VARIANCE_SYMBOL || LA327_22 == VAR_SAMP_SYMBOL || ((LA327_22 >= AT_AT_SIGN_SYMBOL) && (LA327_22 <= AT_TEXT_SUFFIX)) || ((LA327_22 >= BIT_AND_SYMBOL) && (LA327_22 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt327=1;
                		    }
                		    else if ( (LA327_22 == ACTION_SYMBOL || ((LA327_22 >= AFTER_SYMBOL) && (LA327_22 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA327_22 >= BINLOG_SYMBOL) && (LA327_22 <= CLASS_ORIGIN_SYMBOL)) || LA327_22 == CODE_SYMBOL || ((LA327_22 >= COLUMN_NAME_SYMBOL) && (LA327_22 <= DATETIME_SYMBOL)) || ((LA327_22 >= DEFAULT_AUTH_SYMBOL) && (LA327_22 <= GEOMETRY_SYMBOL)) || ((LA327_22 >= GRANTS_SYMBOL) && (LA327_22 <= HOSTS_SYMBOL)) || ((LA327_22 >= IDENTIFIED_SYMBOL) && (LA327_22 <= LEVEL_SYMBOL)) || ((LA327_22 >= LIST_SYMBOL) && (LA327_22 <= MESSAGE_TEXT_SYMBOL)) || LA327_22 == MIGRATE_SYMBOL || ((LA327_22 >= MIN_ROWS_SYMBOL) && (LA327_22 <= MODE_SYMBOL)) || ((LA327_22 >= MUTEX_SYMBOL) && (LA327_22 <= OFFSET_SYMBOL)) || ((LA327_22 >= ONE_SHOT_SYMBOL) && (LA327_22 <= PARTITIONS_SYMBOL)) || ((LA327_22 >= PHASE_SYMBOL) && (LA327_22 <= PLUGINS_SYMBOL)) || ((LA327_22 >= PRESERVE_SYMBOL) && (LA327_22 <= PROXY_SYMBOL)) || ((LA327_22 >= QUERY_SYMBOL) && (LA327_22 <= RETURNS_SYMBOL)) || ((LA327_22 >= ROLLUP_SYMBOL) && (LA327_22 <= ROWS_SYMBOL)) || LA327_22 == ROW_FORMAT_SYMBOL || ((LA327_22 >= RTREE_SYMBOL) && (LA327_22 <= SCHEMA_NAME_SYMBOL)) || ((LA327_22 >= SERIAL_SYMBOL) && (LA327_22 <= SHARE_SYMBOL)) || ((LA327_22 >= SLOW_SYMBOL) && (LA327_22 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA327_22 >= SUBJECT_SYMBOL) && (LA327_22 <= TRIGGERS_SYMBOL)) || ((LA327_22 >= TYPES_SYMBOL) && (LA327_22 <= USER_RESOURCES_SYMBOL)) || ((LA327_22 >= USE_FRM_SYMBOL) && (LA327_22 <= WAIT_SYMBOL)) || LA327_22 == WORK_SYMBOL || ((LA327_22 >= X509_SYMBOL) && (LA327_22 <= XML_SYMBOL)) || LA327_22 == BINARY_SYMBOL || ((LA327_22 >= CASE_SYMBOL) && (LA327_22 <= CAST_SYMBOL)) || LA327_22 == CONVERT_SYMBOL || LA327_22 == MATCH_SYMBOL))
                		    {
                		        alt327=1;
                		    }

                		}
                	}
                    break;

                }

                switch (alt327)
                {
            	case 1:
            	    // MySQLSimpleParser.g:860:62: COMMA_SYMBOL select_item
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_select_item_list4859);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_item_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_select_item_in_select_item_list4861);
            	        select_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_item_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop327;	/* break out of the loop */
            	    break;
                }
            }
            loop327: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_item_listEx; /* Prevent compiler warnings */
    ruleselect_item_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_item_list */

/**
 * $ANTLR start select_item
 * MySQLSimpleParser.g:863:1: select_item : ( ( table_wild )=> table_wild | expression ( select_alias )? );
 */
static void
select_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:863:12: ( ( table_wild )=> table_wild | expression ( select_alias )? )

            ANTLR3_UINT32 alt329;

            alt329=2;

            {
                int LA329_0 = LA(1);
                if ( (LA329_0 == IDENTIFIER))
                {
                    {
                        int LA329_1 = LA(2);
                        if ( (LA329_1 == DOT_SYMBOL))
                        {
                            {
                                int LA329_103 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 103;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_1 == EOF || ((LA329_1 >= ACCOUNT_SYMBOL) && (LA329_1 <= YEAR_SYMBOL)) || LA329_1 == AND_SYMBOL || LA329_1 == BETWEEN_SYMBOL || LA329_1 == COLLATE_SYMBOL || LA329_1 == CROSS_SYMBOL || LA329_1 == FOR_SYMBOL || LA329_1 == FROM_SYMBOL || ((LA329_1 >= GROUP_SYMBOL) && (LA329_1 <= HAVING_SYMBOL)) || LA329_1 == INNER_SYMBOL || LA329_1 == INTO_SYMBOL || LA329_1 == IN_SYMBOL || LA329_1 == IS_SYMBOL || ((LA329_1 >= JOIN_SYMBOL) && (LA329_1 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_1 >= LEFT_SYMBOL) && (LA329_1 <= LIMIT_SYMBOL)) || LA329_1 == LOCK_SYMBOL || ((LA329_1 >= MOD_SYMBOL) && (LA329_1 <= NATURAL_SYMBOL)) || LA329_1 == NOT_SYMBOL || LA329_1 == ON_SYMBOL || LA329_1 == OPEN_PAR_SYMBOL || ((LA329_1 >= ORDER_SYMBOL) && (LA329_1 <= OR_SYMBOL)) || LA329_1 == PROCEDURE_SYMBOL || LA329_1 == REGEXP_SYMBOL || LA329_1 == RIGHT_SYMBOL || LA329_1 == SEMICOLON_SYMBOL || LA329_1 == SET_SYMBOL || LA329_1 == SINGLE_QUOTED_TEXT || LA329_1 == STRAIGHT_JOIN_SYMBOL || LA329_1 == UNION_SYMBOL || LA329_1 == USING_SYMBOL || LA329_1 == WHERE_SYMBOL || LA329_1 == WITH_SYMBOL || LA329_1 == XOR_SYMBOL || LA329_1 == AS_SYMBOL || ((LA329_1 >= BACK_TICK_QUOTED_ID) && (LA329_1 <= BITWISE_AND_OPERATOR)) || ((LA329_1 >= BITWISE_OR_OPERATOR) && (LA329_1 <= BITWISE_XOR_OPERATOR)) || ((LA329_1 >= CLOSE_CURLY_SYMBOL) && (LA329_1 <= CLOSE_PAR_SYMBOL)) || ((LA329_1 >= COMMA_SYMBOL) && (LA329_1 <= CONCAT_PIPES_SYMBOL)) || ((LA329_1 >= DIV_OPERATOR) && (LA329_1 <= DIV_SYMBOL)) || LA329_1 == DOUBLE_QUOTED_TEXT || LA329_1 == EQUAL_OPERATOR || ((LA329_1 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_1 <= GREATER_THAN_OPERATOR)) || ((LA329_1 >= HEX_NUMBER) && (LA329_1 <= IDENTIFIER)) || LA329_1 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_1 >= LESS_OR_EQUAL_OPERATOR) && (LA329_1 <= LESS_THAN_OPERATOR)) || LA329_1 == LOGICAL_AND_OPERATOR || LA329_1 == LOGICAL_OR_OPERATOR || ((LA329_1 >= MINUS_OPERATOR) && (LA329_1 <= MOD_OPERATOR)) || ((LA329_1 >= MULT_OPERATOR) && (LA329_1 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_1 == PLUS_OPERATOR || ((LA329_1 >= SHIFT_LEFT_OPERATOR) && (LA329_1 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 1;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == BACK_TICK_QUOTED_ID))
                {
                    {
                        int LA329_2 = LA(2);
                        if ( (LA329_2 == DOT_SYMBOL))
                        {
                            {
                                int LA329_158 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 158;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_2 == EOF || ((LA329_2 >= ACCOUNT_SYMBOL) && (LA329_2 <= YEAR_SYMBOL)) || LA329_2 == AND_SYMBOL || LA329_2 == BETWEEN_SYMBOL || LA329_2 == COLLATE_SYMBOL || LA329_2 == CROSS_SYMBOL || LA329_2 == FOR_SYMBOL || LA329_2 == FROM_SYMBOL || ((LA329_2 >= GROUP_SYMBOL) && (LA329_2 <= HAVING_SYMBOL)) || LA329_2 == INNER_SYMBOL || LA329_2 == INTO_SYMBOL || LA329_2 == IN_SYMBOL || LA329_2 == IS_SYMBOL || ((LA329_2 >= JOIN_SYMBOL) && (LA329_2 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_2 >= LEFT_SYMBOL) && (LA329_2 <= LIMIT_SYMBOL)) || LA329_2 == LOCK_SYMBOL || ((LA329_2 >= MOD_SYMBOL) && (LA329_2 <= NATURAL_SYMBOL)) || LA329_2 == NOT_SYMBOL || LA329_2 == ON_SYMBOL || LA329_2 == OPEN_PAR_SYMBOL || ((LA329_2 >= ORDER_SYMBOL) && (LA329_2 <= OR_SYMBOL)) || LA329_2 == PROCEDURE_SYMBOL || LA329_2 == REGEXP_SYMBOL || LA329_2 == RIGHT_SYMBOL || LA329_2 == SEMICOLON_SYMBOL || LA329_2 == SET_SYMBOL || LA329_2 == SINGLE_QUOTED_TEXT || LA329_2 == STRAIGHT_JOIN_SYMBOL || LA329_2 == UNION_SYMBOL || LA329_2 == USING_SYMBOL || LA329_2 == WHERE_SYMBOL || LA329_2 == WITH_SYMBOL || LA329_2 == XOR_SYMBOL || LA329_2 == AS_SYMBOL || ((LA329_2 >= BACK_TICK_QUOTED_ID) && (LA329_2 <= BITWISE_AND_OPERATOR)) || ((LA329_2 >= BITWISE_OR_OPERATOR) && (LA329_2 <= BITWISE_XOR_OPERATOR)) || ((LA329_2 >= CLOSE_CURLY_SYMBOL) && (LA329_2 <= CLOSE_PAR_SYMBOL)) || ((LA329_2 >= COMMA_SYMBOL) && (LA329_2 <= CONCAT_PIPES_SYMBOL)) || ((LA329_2 >= DIV_OPERATOR) && (LA329_2 <= DIV_SYMBOL)) || LA329_2 == DOUBLE_QUOTED_TEXT || LA329_2 == EQUAL_OPERATOR || ((LA329_2 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_2 <= GREATER_THAN_OPERATOR)) || ((LA329_2 >= HEX_NUMBER) && (LA329_2 <= IDENTIFIER)) || LA329_2 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_2 >= LESS_OR_EQUAL_OPERATOR) && (LA329_2 <= LESS_THAN_OPERATOR)) || LA329_2 == LOGICAL_AND_OPERATOR || LA329_2 == LOGICAL_OR_OPERATOR || ((LA329_2 >= MINUS_OPERATOR) && (LA329_2 <= MOD_OPERATOR)) || ((LA329_2 >= MULT_OPERATOR) && (LA329_2 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_2 == PLUS_OPERATOR || ((LA329_2 >= SHIFT_LEFT_OPERATOR) && (LA329_2 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 2;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == DOUBLE_QUOTED_TEXT))
                {
                    {
                        int LA329_3 = LA(2);
                        if ( (LA329_3 == DOT_SYMBOL))
                        {
                            {
                                int LA329_213 = LA(3);
                                if ( (( (( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) && ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )) && (synpred1_MySQLSimpleParser(ctx)) )))
                                {
                                    alt329=1;
                                }
                                else if ( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 213;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_3 == EOF || ((LA329_3 >= ACCOUNT_SYMBOL) && (LA329_3 <= YEAR_SYMBOL)) || LA329_3 == AND_SYMBOL || LA329_3 == BETWEEN_SYMBOL || LA329_3 == COLLATE_SYMBOL || LA329_3 == CROSS_SYMBOL || LA329_3 == FOR_SYMBOL || LA329_3 == FROM_SYMBOL || ((LA329_3 >= GROUP_SYMBOL) && (LA329_3 <= HAVING_SYMBOL)) || LA329_3 == INNER_SYMBOL || LA329_3 == INTO_SYMBOL || LA329_3 == IN_SYMBOL || LA329_3 == IS_SYMBOL || ((LA329_3 >= JOIN_SYMBOL) && (LA329_3 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_3 >= LEFT_SYMBOL) && (LA329_3 <= LIMIT_SYMBOL)) || LA329_3 == LOCK_SYMBOL || ((LA329_3 >= MOD_SYMBOL) && (LA329_3 <= NATURAL_SYMBOL)) || LA329_3 == NOT_SYMBOL || LA329_3 == ON_SYMBOL || LA329_3 == OPEN_PAR_SYMBOL || ((LA329_3 >= ORDER_SYMBOL) && (LA329_3 <= OR_SYMBOL)) || LA329_3 == PROCEDURE_SYMBOL || LA329_3 == REGEXP_SYMBOL || LA329_3 == RIGHT_SYMBOL || LA329_3 == SEMICOLON_SYMBOL || LA329_3 == SET_SYMBOL || LA329_3 == SINGLE_QUOTED_TEXT || LA329_3 == STRAIGHT_JOIN_SYMBOL || LA329_3 == UNION_SYMBOL || LA329_3 == USING_SYMBOL || LA329_3 == WHERE_SYMBOL || LA329_3 == WITH_SYMBOL || LA329_3 == XOR_SYMBOL || LA329_3 == AS_SYMBOL || ((LA329_3 >= BACK_TICK_QUOTED_ID) && (LA329_3 <= BITWISE_AND_OPERATOR)) || ((LA329_3 >= BITWISE_OR_OPERATOR) && (LA329_3 <= BITWISE_XOR_OPERATOR)) || ((LA329_3 >= CLOSE_CURLY_SYMBOL) && (LA329_3 <= CLOSE_PAR_SYMBOL)) || ((LA329_3 >= COMMA_SYMBOL) && (LA329_3 <= CONCAT_PIPES_SYMBOL)) || ((LA329_3 >= DIV_OPERATOR) && (LA329_3 <= DIV_SYMBOL)) || LA329_3 == DOUBLE_QUOTED_TEXT || LA329_3 == EQUAL_OPERATOR || ((LA329_3 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_3 <= GREATER_THAN_OPERATOR)) || ((LA329_3 >= HEX_NUMBER) && (LA329_3 <= IDENTIFIER)) || LA329_3 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_3 >= LESS_OR_EQUAL_OPERATOR) && (LA329_3 <= LESS_THAN_OPERATOR)) || LA329_3 == LOGICAL_AND_OPERATOR || LA329_3 == LOGICAL_OR_OPERATOR || ((LA329_3 >= MINUS_OPERATOR) && (LA329_3 <= MOD_OPERATOR)) || ((LA329_3 >= MULT_OPERATOR) && (LA329_3 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_3 == PLUS_OPERATOR || ((LA329_3 >= SHIFT_LEFT_OPERATOR) && (LA329_3 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 3;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == DATE_SYMBOL))
                {
                    {
                        int LA329_4 = LA(2);
                        if ( (LA329_4 == EOF || ((LA329_4 >= ACCOUNT_SYMBOL) && (LA329_4 <= YEAR_SYMBOL)) || LA329_4 == AND_SYMBOL || LA329_4 == BETWEEN_SYMBOL || LA329_4 == COLLATE_SYMBOL || LA329_4 == CROSS_SYMBOL || LA329_4 == FOR_SYMBOL || LA329_4 == FROM_SYMBOL || ((LA329_4 >= GROUP_SYMBOL) && (LA329_4 <= HAVING_SYMBOL)) || LA329_4 == INNER_SYMBOL || LA329_4 == INTO_SYMBOL || LA329_4 == IN_SYMBOL || LA329_4 == IS_SYMBOL || ((LA329_4 >= JOIN_SYMBOL) && (LA329_4 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_4 >= LEFT_SYMBOL) && (LA329_4 <= LIMIT_SYMBOL)) || LA329_4 == LOCK_SYMBOL || ((LA329_4 >= MOD_SYMBOL) && (LA329_4 <= NATURAL_SYMBOL)) || LA329_4 == NOT_SYMBOL || LA329_4 == ON_SYMBOL || LA329_4 == OPEN_PAR_SYMBOL || ((LA329_4 >= ORDER_SYMBOL) && (LA329_4 <= OR_SYMBOL)) || LA329_4 == PROCEDURE_SYMBOL || LA329_4 == REGEXP_SYMBOL || LA329_4 == RIGHT_SYMBOL || LA329_4 == SEMICOLON_SYMBOL || LA329_4 == SET_SYMBOL || LA329_4 == SINGLE_QUOTED_TEXT || LA329_4 == STRAIGHT_JOIN_SYMBOL || LA329_4 == UNION_SYMBOL || LA329_4 == USING_SYMBOL || LA329_4 == WHERE_SYMBOL || LA329_4 == WITH_SYMBOL || LA329_4 == XOR_SYMBOL || LA329_4 == AS_SYMBOL || ((LA329_4 >= BACK_TICK_QUOTED_ID) && (LA329_4 <= BITWISE_AND_OPERATOR)) || ((LA329_4 >= BITWISE_OR_OPERATOR) && (LA329_4 <= BITWISE_XOR_OPERATOR)) || ((LA329_4 >= CLOSE_CURLY_SYMBOL) && (LA329_4 <= CLOSE_PAR_SYMBOL)) || ((LA329_4 >= COMMA_SYMBOL) && (LA329_4 <= CONCAT_PIPES_SYMBOL)) || ((LA329_4 >= DIV_OPERATOR) && (LA329_4 <= DIV_SYMBOL)) || LA329_4 == DOUBLE_QUOTED_TEXT || LA329_4 == EQUAL_OPERATOR || ((LA329_4 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_4 <= GREATER_THAN_OPERATOR)) || ((LA329_4 >= HEX_NUMBER) && (LA329_4 <= IDENTIFIER)) || LA329_4 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_4 >= LESS_OR_EQUAL_OPERATOR) && (LA329_4 <= LESS_THAN_OPERATOR)) || LA329_4 == LOGICAL_AND_OPERATOR || LA329_4 == LOGICAL_OR_OPERATOR || ((LA329_4 >= MINUS_OPERATOR) && (LA329_4 <= MOD_OPERATOR)) || ((LA329_4 >= MULT_OPERATOR) && (LA329_4 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_4 == PLUS_OPERATOR || ((LA329_4 >= SHIFT_LEFT_OPERATOR) && (LA329_4 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_4 == DOT_SYMBOL))
                        {
                            {
                                int LA329_271 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 271;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 4;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ASCII_SYMBOL))
                {
                    {
                        int LA329_5 = LA(2);
                        if ( (LA329_5 == EOF || ((LA329_5 >= ACCOUNT_SYMBOL) && (LA329_5 <= YEAR_SYMBOL)) || LA329_5 == AND_SYMBOL || LA329_5 == BETWEEN_SYMBOL || LA329_5 == COLLATE_SYMBOL || LA329_5 == CROSS_SYMBOL || LA329_5 == FOR_SYMBOL || LA329_5 == FROM_SYMBOL || ((LA329_5 >= GROUP_SYMBOL) && (LA329_5 <= HAVING_SYMBOL)) || LA329_5 == INNER_SYMBOL || LA329_5 == INTO_SYMBOL || LA329_5 == IN_SYMBOL || LA329_5 == IS_SYMBOL || ((LA329_5 >= JOIN_SYMBOL) && (LA329_5 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_5 >= LEFT_SYMBOL) && (LA329_5 <= LIMIT_SYMBOL)) || LA329_5 == LOCK_SYMBOL || ((LA329_5 >= MOD_SYMBOL) && (LA329_5 <= NATURAL_SYMBOL)) || LA329_5 == NOT_SYMBOL || LA329_5 == ON_SYMBOL || LA329_5 == OPEN_PAR_SYMBOL || ((LA329_5 >= ORDER_SYMBOL) && (LA329_5 <= OR_SYMBOL)) || LA329_5 == PROCEDURE_SYMBOL || LA329_5 == REGEXP_SYMBOL || LA329_5 == RIGHT_SYMBOL || LA329_5 == SEMICOLON_SYMBOL || LA329_5 == SET_SYMBOL || LA329_5 == SINGLE_QUOTED_TEXT || LA329_5 == STRAIGHT_JOIN_SYMBOL || LA329_5 == UNION_SYMBOL || LA329_5 == USING_SYMBOL || LA329_5 == WHERE_SYMBOL || LA329_5 == WITH_SYMBOL || LA329_5 == XOR_SYMBOL || LA329_5 == AS_SYMBOL || ((LA329_5 >= BACK_TICK_QUOTED_ID) && (LA329_5 <= BITWISE_AND_OPERATOR)) || ((LA329_5 >= BITWISE_OR_OPERATOR) && (LA329_5 <= BITWISE_XOR_OPERATOR)) || ((LA329_5 >= CLOSE_CURLY_SYMBOL) && (LA329_5 <= CLOSE_PAR_SYMBOL)) || ((LA329_5 >= COMMA_SYMBOL) && (LA329_5 <= CONCAT_PIPES_SYMBOL)) || ((LA329_5 >= DIV_OPERATOR) && (LA329_5 <= DIV_SYMBOL)) || LA329_5 == DOUBLE_QUOTED_TEXT || LA329_5 == EQUAL_OPERATOR || ((LA329_5 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_5 <= GREATER_THAN_OPERATOR)) || ((LA329_5 >= HEX_NUMBER) && (LA329_5 <= IDENTIFIER)) || LA329_5 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_5 >= LESS_OR_EQUAL_OPERATOR) && (LA329_5 <= LESS_THAN_OPERATOR)) || LA329_5 == LOGICAL_AND_OPERATOR || LA329_5 == LOGICAL_OR_OPERATOR || ((LA329_5 >= MINUS_OPERATOR) && (LA329_5 <= MOD_OPERATOR)) || ((LA329_5 >= MULT_OPERATOR) && (LA329_5 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_5 == PLUS_OPERATOR || ((LA329_5 >= SHIFT_LEFT_OPERATOR) && (LA329_5 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_5 == DOT_SYMBOL))
                        {
                            {
                                int LA329_326 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 326;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 5;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == BINARY_SYMBOL || ((LA329_0 >= CASE_SYMBOL) && (LA329_0 <= CAST_SYMBOL)) || LA329_0 == CHAR_SYMBOL || ((LA329_0 >= CONVERT_SYMBOL) && (LA329_0 <= COUNT_SYMBOL)) || LA329_0 == CURDATE_SYMBOL || LA329_0 == CURRENT_USER_SYMBOL || LA329_0 == CURTIME_SYMBOL || LA329_0 == DATABASE_SYMBOL || LA329_0 == DATE_ADD_SYMBOL || LA329_0 == DATE_SUB_SYMBOL || LA329_0 == EXISTS_SYMBOL || ((LA329_0 >= EXTRACT_SYMBOL) && (LA329_0 <= FALSE_SYMBOL)) || LA329_0 == GROUP_CONCAT_SYMBOL || LA329_0 == IF_SYMBOL || LA329_0 == INSERT_SYMBOL || LA329_0 == INTERVAL_SYMBOL || LA329_0 == LEFT_SYMBOL || LA329_0 == MATCH_SYMBOL || LA329_0 == MAX_SYMBOL || LA329_0 == MIN_SYMBOL || LA329_0 == MOD_SYMBOL || LA329_0 == NCHAR_TEXT || ((LA329_0 >= NOT2_SYMBOL) && (LA329_0 <= NOW_SYMBOL)) || ((LA329_0 >= NULL2_SYMBOL) && (LA329_0 <= NULL_SYMBOL)) || ((LA329_0 >= OPEN_CURLY_SYMBOL) && (LA329_0 <= OPEN_PAR_SYMBOL)) || ((LA329_0 >= PARAM_MARKER) && (LA329_0 <= POSITION_SYMBOL)) || ((LA329_0 >= REPEAT_SYMBOL) && (LA329_0 <= REPLACE_SYMBOL)) || LA329_0 == RIGHT_SYMBOL || LA329_0 == SINGLE_QUOTED_TEXT || LA329_0 == STDDEV_SAMP_SYMBOL || LA329_0 == STD_SYMBOL || LA329_0 == SUBSTRING_SYMBOL || ((LA329_0 >= SUM_SYMBOL) && (LA329_0 <= SYSDATE_SYMBOL)) || ((LA329_0 >= TRIM_SYMBOL) && (LA329_0 <= TRUE_SYMBOL)) || ((LA329_0 >= UTC_DATE_SYMBOL) && (LA329_0 <= UTC_TIME_SYMBOL)) || LA329_0 == VALUES_SYMBOL || LA329_0 == VARIANCE_SYMBOL || LA329_0 == VAR_SAMP_SYMBOL || ((LA329_0 >= AT_AT_SIGN_SYMBOL) && (LA329_0 <= AT_TEXT_SUFFIX)) || LA329_0 == BIN_NUMBER || LA329_0 == BITWISE_NOT_OPERATOR || ((LA329_0 >= BIT_AND_SYMBOL) && (LA329_0 <= BIT_XOR_SYMBOL)) || LA329_0 == DECIMAL_NUMBER || LA329_0 == DOT_SYMBOL || LA329_0 == FLOAT_NUMBER || LA329_0 == HEX_NUMBER || LA329_0 == INT_NUMBER || LA329_0 == LOGICAL_NOT_OPERATOR || ((LA329_0 >= LONG_NUMBER) && (LA329_0 <= MINUS_OPERATOR)) || LA329_0 == PLUS_OPERATOR || ((LA329_0 >= ULONGLONG_NUMBER) && (LA329_0 <= UNDERSCORE_CHARSET))))
                {
                    alt329=2;
                }
                else if ( (LA329_0 == TIME_SYMBOL))
                {
                    {
                        int LA329_14 = LA(2);
                        if ( (LA329_14 == EOF || ((LA329_14 >= ACCOUNT_SYMBOL) && (LA329_14 <= YEAR_SYMBOL)) || LA329_14 == AND_SYMBOL || LA329_14 == BETWEEN_SYMBOL || LA329_14 == COLLATE_SYMBOL || LA329_14 == CROSS_SYMBOL || LA329_14 == FOR_SYMBOL || LA329_14 == FROM_SYMBOL || ((LA329_14 >= GROUP_SYMBOL) && (LA329_14 <= HAVING_SYMBOL)) || LA329_14 == INNER_SYMBOL || LA329_14 == INTO_SYMBOL || LA329_14 == IN_SYMBOL || LA329_14 == IS_SYMBOL || ((LA329_14 >= JOIN_SYMBOL) && (LA329_14 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_14 >= LEFT_SYMBOL) && (LA329_14 <= LIMIT_SYMBOL)) || LA329_14 == LOCK_SYMBOL || ((LA329_14 >= MOD_SYMBOL) && (LA329_14 <= NATURAL_SYMBOL)) || LA329_14 == NOT_SYMBOL || LA329_14 == ON_SYMBOL || LA329_14 == OPEN_PAR_SYMBOL || ((LA329_14 >= ORDER_SYMBOL) && (LA329_14 <= OR_SYMBOL)) || LA329_14 == PROCEDURE_SYMBOL || LA329_14 == REGEXP_SYMBOL || LA329_14 == RIGHT_SYMBOL || LA329_14 == SEMICOLON_SYMBOL || LA329_14 == SET_SYMBOL || LA329_14 == SINGLE_QUOTED_TEXT || LA329_14 == STRAIGHT_JOIN_SYMBOL || LA329_14 == UNION_SYMBOL || LA329_14 == USING_SYMBOL || LA329_14 == WHERE_SYMBOL || LA329_14 == WITH_SYMBOL || LA329_14 == XOR_SYMBOL || LA329_14 == AS_SYMBOL || ((LA329_14 >= BACK_TICK_QUOTED_ID) && (LA329_14 <= BITWISE_AND_OPERATOR)) || ((LA329_14 >= BITWISE_OR_OPERATOR) && (LA329_14 <= BITWISE_XOR_OPERATOR)) || ((LA329_14 >= CLOSE_CURLY_SYMBOL) && (LA329_14 <= CLOSE_PAR_SYMBOL)) || ((LA329_14 >= COMMA_SYMBOL) && (LA329_14 <= CONCAT_PIPES_SYMBOL)) || ((LA329_14 >= DIV_OPERATOR) && (LA329_14 <= DIV_SYMBOL)) || LA329_14 == DOUBLE_QUOTED_TEXT || LA329_14 == EQUAL_OPERATOR || ((LA329_14 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_14 <= GREATER_THAN_OPERATOR)) || ((LA329_14 >= HEX_NUMBER) && (LA329_14 <= IDENTIFIER)) || LA329_14 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_14 >= LESS_OR_EQUAL_OPERATOR) && (LA329_14 <= LESS_THAN_OPERATOR)) || LA329_14 == LOGICAL_AND_OPERATOR || LA329_14 == LOGICAL_OR_OPERATOR || ((LA329_14 >= MINUS_OPERATOR) && (LA329_14 <= MOD_OPERATOR)) || ((LA329_14 >= MULT_OPERATOR) && (LA329_14 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_14 == PLUS_OPERATOR || ((LA329_14 >= SHIFT_LEFT_OPERATOR) && (LA329_14 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_14 == DOT_SYMBOL))
                        {
                            {
                                int LA329_382 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 382;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 14;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == TIMESTAMP_SYMBOL))
                {
                    {
                        int LA329_15 = LA(2);
                        if ( (LA329_15 == EOF || ((LA329_15 >= ACCOUNT_SYMBOL) && (LA329_15 <= YEAR_SYMBOL)) || LA329_15 == AND_SYMBOL || LA329_15 == BETWEEN_SYMBOL || LA329_15 == COLLATE_SYMBOL || LA329_15 == CROSS_SYMBOL || LA329_15 == FOR_SYMBOL || LA329_15 == FROM_SYMBOL || ((LA329_15 >= GROUP_SYMBOL) && (LA329_15 <= HAVING_SYMBOL)) || LA329_15 == INNER_SYMBOL || LA329_15 == INTO_SYMBOL || LA329_15 == IN_SYMBOL || LA329_15 == IS_SYMBOL || ((LA329_15 >= JOIN_SYMBOL) && (LA329_15 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_15 >= LEFT_SYMBOL) && (LA329_15 <= LIMIT_SYMBOL)) || LA329_15 == LOCK_SYMBOL || ((LA329_15 >= MOD_SYMBOL) && (LA329_15 <= NATURAL_SYMBOL)) || LA329_15 == NOT_SYMBOL || LA329_15 == ON_SYMBOL || LA329_15 == OPEN_PAR_SYMBOL || ((LA329_15 >= ORDER_SYMBOL) && (LA329_15 <= OR_SYMBOL)) || LA329_15 == PROCEDURE_SYMBOL || LA329_15 == REGEXP_SYMBOL || LA329_15 == RIGHT_SYMBOL || LA329_15 == SEMICOLON_SYMBOL || LA329_15 == SET_SYMBOL || LA329_15 == SINGLE_QUOTED_TEXT || LA329_15 == STRAIGHT_JOIN_SYMBOL || LA329_15 == UNION_SYMBOL || LA329_15 == USING_SYMBOL || LA329_15 == WHERE_SYMBOL || LA329_15 == WITH_SYMBOL || LA329_15 == XOR_SYMBOL || LA329_15 == AS_SYMBOL || ((LA329_15 >= BACK_TICK_QUOTED_ID) && (LA329_15 <= BITWISE_AND_OPERATOR)) || ((LA329_15 >= BITWISE_OR_OPERATOR) && (LA329_15 <= BITWISE_XOR_OPERATOR)) || ((LA329_15 >= CLOSE_CURLY_SYMBOL) && (LA329_15 <= CLOSE_PAR_SYMBOL)) || ((LA329_15 >= COMMA_SYMBOL) && (LA329_15 <= CONCAT_PIPES_SYMBOL)) || ((LA329_15 >= DIV_OPERATOR) && (LA329_15 <= DIV_SYMBOL)) || LA329_15 == DOUBLE_QUOTED_TEXT || LA329_15 == EQUAL_OPERATOR || ((LA329_15 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_15 <= GREATER_THAN_OPERATOR)) || ((LA329_15 >= HEX_NUMBER) && (LA329_15 <= IDENTIFIER)) || LA329_15 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_15 >= LESS_OR_EQUAL_OPERATOR) && (LA329_15 <= LESS_THAN_OPERATOR)) || LA329_15 == LOGICAL_AND_OPERATOR || LA329_15 == LOGICAL_OR_OPERATOR || ((LA329_15 >= MINUS_OPERATOR) && (LA329_15 <= MOD_OPERATOR)) || ((LA329_15 >= MULT_OPERATOR) && (LA329_15 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_15 == PLUS_OPERATOR || ((LA329_15 >= SHIFT_LEFT_OPERATOR) && (LA329_15 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_15 == DOT_SYMBOL))
                        {
                            {
                                int LA329_438 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 438;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 15;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == DAY_SYMBOL))
                {
                    {
                        int LA329_16 = LA(2);
                        if ( (LA329_16 == EOF || ((LA329_16 >= ACCOUNT_SYMBOL) && (LA329_16 <= YEAR_SYMBOL)) || LA329_16 == AND_SYMBOL || LA329_16 == BETWEEN_SYMBOL || LA329_16 == COLLATE_SYMBOL || LA329_16 == CROSS_SYMBOL || LA329_16 == FOR_SYMBOL || LA329_16 == FROM_SYMBOL || ((LA329_16 >= GROUP_SYMBOL) && (LA329_16 <= HAVING_SYMBOL)) || LA329_16 == INNER_SYMBOL || LA329_16 == INTO_SYMBOL || LA329_16 == IN_SYMBOL || LA329_16 == IS_SYMBOL || ((LA329_16 >= JOIN_SYMBOL) && (LA329_16 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_16 >= LEFT_SYMBOL) && (LA329_16 <= LIMIT_SYMBOL)) || LA329_16 == LOCK_SYMBOL || ((LA329_16 >= MOD_SYMBOL) && (LA329_16 <= NATURAL_SYMBOL)) || LA329_16 == NOT_SYMBOL || LA329_16 == ON_SYMBOL || LA329_16 == OPEN_PAR_SYMBOL || ((LA329_16 >= ORDER_SYMBOL) && (LA329_16 <= OR_SYMBOL)) || LA329_16 == PROCEDURE_SYMBOL || LA329_16 == REGEXP_SYMBOL || LA329_16 == RIGHT_SYMBOL || LA329_16 == SEMICOLON_SYMBOL || LA329_16 == SET_SYMBOL || LA329_16 == SINGLE_QUOTED_TEXT || LA329_16 == STRAIGHT_JOIN_SYMBOL || LA329_16 == UNION_SYMBOL || LA329_16 == USING_SYMBOL || LA329_16 == WHERE_SYMBOL || LA329_16 == WITH_SYMBOL || LA329_16 == XOR_SYMBOL || LA329_16 == AS_SYMBOL || ((LA329_16 >= BACK_TICK_QUOTED_ID) && (LA329_16 <= BITWISE_AND_OPERATOR)) || ((LA329_16 >= BITWISE_OR_OPERATOR) && (LA329_16 <= BITWISE_XOR_OPERATOR)) || ((LA329_16 >= CLOSE_CURLY_SYMBOL) && (LA329_16 <= CLOSE_PAR_SYMBOL)) || ((LA329_16 >= COMMA_SYMBOL) && (LA329_16 <= CONCAT_PIPES_SYMBOL)) || ((LA329_16 >= DIV_OPERATOR) && (LA329_16 <= DIV_SYMBOL)) || LA329_16 == DOUBLE_QUOTED_TEXT || LA329_16 == EQUAL_OPERATOR || ((LA329_16 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_16 <= GREATER_THAN_OPERATOR)) || ((LA329_16 >= HEX_NUMBER) && (LA329_16 <= IDENTIFIER)) || LA329_16 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_16 >= LESS_OR_EQUAL_OPERATOR) && (LA329_16 <= LESS_THAN_OPERATOR)) || LA329_16 == LOGICAL_AND_OPERATOR || LA329_16 == LOGICAL_OR_OPERATOR || ((LA329_16 >= MINUS_OPERATOR) && (LA329_16 <= MOD_OPERATOR)) || ((LA329_16 >= MULT_OPERATOR) && (LA329_16 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_16 == PLUS_OPERATOR || ((LA329_16 >= SHIFT_LEFT_OPERATOR) && (LA329_16 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_16 == DOT_SYMBOL))
                        {
                            {
                                int LA329_493 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 493;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 16;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == HOUR_SYMBOL))
                {
                    {
                        int LA329_23 = LA(2);
                        if ( (LA329_23 == EOF || ((LA329_23 >= ACCOUNT_SYMBOL) && (LA329_23 <= YEAR_SYMBOL)) || LA329_23 == AND_SYMBOL || LA329_23 == BETWEEN_SYMBOL || LA329_23 == COLLATE_SYMBOL || LA329_23 == CROSS_SYMBOL || LA329_23 == FOR_SYMBOL || LA329_23 == FROM_SYMBOL || ((LA329_23 >= GROUP_SYMBOL) && (LA329_23 <= HAVING_SYMBOL)) || LA329_23 == INNER_SYMBOL || LA329_23 == INTO_SYMBOL || LA329_23 == IN_SYMBOL || LA329_23 == IS_SYMBOL || ((LA329_23 >= JOIN_SYMBOL) && (LA329_23 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_23 >= LEFT_SYMBOL) && (LA329_23 <= LIMIT_SYMBOL)) || LA329_23 == LOCK_SYMBOL || ((LA329_23 >= MOD_SYMBOL) && (LA329_23 <= NATURAL_SYMBOL)) || LA329_23 == NOT_SYMBOL || LA329_23 == ON_SYMBOL || LA329_23 == OPEN_PAR_SYMBOL || ((LA329_23 >= ORDER_SYMBOL) && (LA329_23 <= OR_SYMBOL)) || LA329_23 == PROCEDURE_SYMBOL || LA329_23 == REGEXP_SYMBOL || LA329_23 == RIGHT_SYMBOL || LA329_23 == SEMICOLON_SYMBOL || LA329_23 == SET_SYMBOL || LA329_23 == SINGLE_QUOTED_TEXT || LA329_23 == STRAIGHT_JOIN_SYMBOL || LA329_23 == UNION_SYMBOL || LA329_23 == USING_SYMBOL || LA329_23 == WHERE_SYMBOL || LA329_23 == WITH_SYMBOL || LA329_23 == XOR_SYMBOL || LA329_23 == AS_SYMBOL || ((LA329_23 >= BACK_TICK_QUOTED_ID) && (LA329_23 <= BITWISE_AND_OPERATOR)) || ((LA329_23 >= BITWISE_OR_OPERATOR) && (LA329_23 <= BITWISE_XOR_OPERATOR)) || ((LA329_23 >= CLOSE_CURLY_SYMBOL) && (LA329_23 <= CLOSE_PAR_SYMBOL)) || ((LA329_23 >= COMMA_SYMBOL) && (LA329_23 <= CONCAT_PIPES_SYMBOL)) || ((LA329_23 >= DIV_OPERATOR) && (LA329_23 <= DIV_SYMBOL)) || LA329_23 == DOUBLE_QUOTED_TEXT || LA329_23 == EQUAL_OPERATOR || ((LA329_23 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_23 <= GREATER_THAN_OPERATOR)) || ((LA329_23 >= HEX_NUMBER) && (LA329_23 <= IDENTIFIER)) || LA329_23 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_23 >= LESS_OR_EQUAL_OPERATOR) && (LA329_23 <= LESS_THAN_OPERATOR)) || LA329_23 == LOGICAL_AND_OPERATOR || LA329_23 == LOGICAL_OR_OPERATOR || ((LA329_23 >= MINUS_OPERATOR) && (LA329_23 <= MOD_OPERATOR)) || ((LA329_23 >= MULT_OPERATOR) && (LA329_23 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_23 == PLUS_OPERATOR || ((LA329_23 >= SHIFT_LEFT_OPERATOR) && (LA329_23 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_23 == DOT_SYMBOL))
                        {
                            {
                                int LA329_548 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 548;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 23;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MINUTE_SYMBOL))
                {
                    {
                        int LA329_24 = LA(2);
                        if ( (LA329_24 == EOF || ((LA329_24 >= ACCOUNT_SYMBOL) && (LA329_24 <= YEAR_SYMBOL)) || LA329_24 == AND_SYMBOL || LA329_24 == BETWEEN_SYMBOL || LA329_24 == COLLATE_SYMBOL || LA329_24 == CROSS_SYMBOL || LA329_24 == FOR_SYMBOL || LA329_24 == FROM_SYMBOL || ((LA329_24 >= GROUP_SYMBOL) && (LA329_24 <= HAVING_SYMBOL)) || LA329_24 == INNER_SYMBOL || LA329_24 == INTO_SYMBOL || LA329_24 == IN_SYMBOL || LA329_24 == IS_SYMBOL || ((LA329_24 >= JOIN_SYMBOL) && (LA329_24 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_24 >= LEFT_SYMBOL) && (LA329_24 <= LIMIT_SYMBOL)) || LA329_24 == LOCK_SYMBOL || ((LA329_24 >= MOD_SYMBOL) && (LA329_24 <= NATURAL_SYMBOL)) || LA329_24 == NOT_SYMBOL || LA329_24 == ON_SYMBOL || LA329_24 == OPEN_PAR_SYMBOL || ((LA329_24 >= ORDER_SYMBOL) && (LA329_24 <= OR_SYMBOL)) || LA329_24 == PROCEDURE_SYMBOL || LA329_24 == REGEXP_SYMBOL || LA329_24 == RIGHT_SYMBOL || LA329_24 == SEMICOLON_SYMBOL || LA329_24 == SET_SYMBOL || LA329_24 == SINGLE_QUOTED_TEXT || LA329_24 == STRAIGHT_JOIN_SYMBOL || LA329_24 == UNION_SYMBOL || LA329_24 == USING_SYMBOL || LA329_24 == WHERE_SYMBOL || LA329_24 == WITH_SYMBOL || LA329_24 == XOR_SYMBOL || LA329_24 == AS_SYMBOL || ((LA329_24 >= BACK_TICK_QUOTED_ID) && (LA329_24 <= BITWISE_AND_OPERATOR)) || ((LA329_24 >= BITWISE_OR_OPERATOR) && (LA329_24 <= BITWISE_XOR_OPERATOR)) || ((LA329_24 >= CLOSE_CURLY_SYMBOL) && (LA329_24 <= CLOSE_PAR_SYMBOL)) || ((LA329_24 >= COMMA_SYMBOL) && (LA329_24 <= CONCAT_PIPES_SYMBOL)) || ((LA329_24 >= DIV_OPERATOR) && (LA329_24 <= DIV_SYMBOL)) || LA329_24 == DOUBLE_QUOTED_TEXT || LA329_24 == EQUAL_OPERATOR || ((LA329_24 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_24 <= GREATER_THAN_OPERATOR)) || ((LA329_24 >= HEX_NUMBER) && (LA329_24 <= IDENTIFIER)) || LA329_24 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_24 >= LESS_OR_EQUAL_OPERATOR) && (LA329_24 <= LESS_THAN_OPERATOR)) || LA329_24 == LOGICAL_AND_OPERATOR || LA329_24 == LOGICAL_OR_OPERATOR || ((LA329_24 >= MINUS_OPERATOR) && (LA329_24 <= MOD_OPERATOR)) || ((LA329_24 >= MULT_OPERATOR) && (LA329_24 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_24 == PLUS_OPERATOR || ((LA329_24 >= SHIFT_LEFT_OPERATOR) && (LA329_24 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_24 == DOT_SYMBOL))
                        {
                            {
                                int LA329_603 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 603;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 24;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MONTH_SYMBOL))
                {
                    {
                        int LA329_27 = LA(2);
                        if ( (LA329_27 == EOF || ((LA329_27 >= ACCOUNT_SYMBOL) && (LA329_27 <= YEAR_SYMBOL)) || LA329_27 == AND_SYMBOL || LA329_27 == BETWEEN_SYMBOL || LA329_27 == COLLATE_SYMBOL || LA329_27 == CROSS_SYMBOL || LA329_27 == FOR_SYMBOL || LA329_27 == FROM_SYMBOL || ((LA329_27 >= GROUP_SYMBOL) && (LA329_27 <= HAVING_SYMBOL)) || LA329_27 == INNER_SYMBOL || LA329_27 == INTO_SYMBOL || LA329_27 == IN_SYMBOL || LA329_27 == IS_SYMBOL || ((LA329_27 >= JOIN_SYMBOL) && (LA329_27 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_27 >= LEFT_SYMBOL) && (LA329_27 <= LIMIT_SYMBOL)) || LA329_27 == LOCK_SYMBOL || ((LA329_27 >= MOD_SYMBOL) && (LA329_27 <= NATURAL_SYMBOL)) || LA329_27 == NOT_SYMBOL || LA329_27 == ON_SYMBOL || LA329_27 == OPEN_PAR_SYMBOL || ((LA329_27 >= ORDER_SYMBOL) && (LA329_27 <= OR_SYMBOL)) || LA329_27 == PROCEDURE_SYMBOL || LA329_27 == REGEXP_SYMBOL || LA329_27 == RIGHT_SYMBOL || LA329_27 == SEMICOLON_SYMBOL || LA329_27 == SET_SYMBOL || LA329_27 == SINGLE_QUOTED_TEXT || LA329_27 == STRAIGHT_JOIN_SYMBOL || LA329_27 == UNION_SYMBOL || LA329_27 == USING_SYMBOL || LA329_27 == WHERE_SYMBOL || LA329_27 == WITH_SYMBOL || LA329_27 == XOR_SYMBOL || LA329_27 == AS_SYMBOL || ((LA329_27 >= BACK_TICK_QUOTED_ID) && (LA329_27 <= BITWISE_AND_OPERATOR)) || ((LA329_27 >= BITWISE_OR_OPERATOR) && (LA329_27 <= BITWISE_XOR_OPERATOR)) || ((LA329_27 >= CLOSE_CURLY_SYMBOL) && (LA329_27 <= CLOSE_PAR_SYMBOL)) || ((LA329_27 >= COMMA_SYMBOL) && (LA329_27 <= CONCAT_PIPES_SYMBOL)) || ((LA329_27 >= DIV_OPERATOR) && (LA329_27 <= DIV_SYMBOL)) || LA329_27 == DOUBLE_QUOTED_TEXT || LA329_27 == EQUAL_OPERATOR || ((LA329_27 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_27 <= GREATER_THAN_OPERATOR)) || ((LA329_27 >= HEX_NUMBER) && (LA329_27 <= IDENTIFIER)) || LA329_27 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_27 >= LESS_OR_EQUAL_OPERATOR) && (LA329_27 <= LESS_THAN_OPERATOR)) || LA329_27 == LOGICAL_AND_OPERATOR || LA329_27 == LOGICAL_OR_OPERATOR || ((LA329_27 >= MINUS_OPERATOR) && (LA329_27 <= MOD_OPERATOR)) || ((LA329_27 >= MULT_OPERATOR) && (LA329_27 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_27 == PLUS_OPERATOR || ((LA329_27 >= SHIFT_LEFT_OPERATOR) && (LA329_27 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_27 == DOT_SYMBOL))
                        {
                            {
                                int LA329_658 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 658;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 27;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == SECOND_SYMBOL))
                {
                    {
                        int LA329_28 = LA(2);
                        if ( (LA329_28 == EOF || ((LA329_28 >= ACCOUNT_SYMBOL) && (LA329_28 <= YEAR_SYMBOL)) || LA329_28 == AND_SYMBOL || LA329_28 == BETWEEN_SYMBOL || LA329_28 == COLLATE_SYMBOL || LA329_28 == CROSS_SYMBOL || LA329_28 == FOR_SYMBOL || LA329_28 == FROM_SYMBOL || ((LA329_28 >= GROUP_SYMBOL) && (LA329_28 <= HAVING_SYMBOL)) || LA329_28 == INNER_SYMBOL || LA329_28 == INTO_SYMBOL || LA329_28 == IN_SYMBOL || LA329_28 == IS_SYMBOL || ((LA329_28 >= JOIN_SYMBOL) && (LA329_28 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_28 >= LEFT_SYMBOL) && (LA329_28 <= LIMIT_SYMBOL)) || LA329_28 == LOCK_SYMBOL || ((LA329_28 >= MOD_SYMBOL) && (LA329_28 <= NATURAL_SYMBOL)) || LA329_28 == NOT_SYMBOL || LA329_28 == ON_SYMBOL || LA329_28 == OPEN_PAR_SYMBOL || ((LA329_28 >= ORDER_SYMBOL) && (LA329_28 <= OR_SYMBOL)) || LA329_28 == PROCEDURE_SYMBOL || LA329_28 == REGEXP_SYMBOL || LA329_28 == RIGHT_SYMBOL || LA329_28 == SEMICOLON_SYMBOL || LA329_28 == SET_SYMBOL || LA329_28 == SINGLE_QUOTED_TEXT || LA329_28 == STRAIGHT_JOIN_SYMBOL || LA329_28 == UNION_SYMBOL || LA329_28 == USING_SYMBOL || LA329_28 == WHERE_SYMBOL || LA329_28 == WITH_SYMBOL || LA329_28 == XOR_SYMBOL || LA329_28 == AS_SYMBOL || ((LA329_28 >= BACK_TICK_QUOTED_ID) && (LA329_28 <= BITWISE_AND_OPERATOR)) || ((LA329_28 >= BITWISE_OR_OPERATOR) && (LA329_28 <= BITWISE_XOR_OPERATOR)) || ((LA329_28 >= CLOSE_CURLY_SYMBOL) && (LA329_28 <= CLOSE_PAR_SYMBOL)) || ((LA329_28 >= COMMA_SYMBOL) && (LA329_28 <= CONCAT_PIPES_SYMBOL)) || ((LA329_28 >= DIV_OPERATOR) && (LA329_28 <= DIV_SYMBOL)) || LA329_28 == DOUBLE_QUOTED_TEXT || LA329_28 == EQUAL_OPERATOR || ((LA329_28 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_28 <= GREATER_THAN_OPERATOR)) || ((LA329_28 >= HEX_NUMBER) && (LA329_28 <= IDENTIFIER)) || LA329_28 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_28 >= LESS_OR_EQUAL_OPERATOR) && (LA329_28 <= LESS_THAN_OPERATOR)) || LA329_28 == LOGICAL_AND_OPERATOR || LA329_28 == LOGICAL_OR_OPERATOR || ((LA329_28 >= MINUS_OPERATOR) && (LA329_28 <= MOD_OPERATOR)) || ((LA329_28 >= MULT_OPERATOR) && (LA329_28 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_28 == PLUS_OPERATOR || ((LA329_28 >= SHIFT_LEFT_OPERATOR) && (LA329_28 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_28 == DOT_SYMBOL))
                        {
                            {
                                int LA329_713 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 713;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 28;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == USER_SYMBOL))
                {
                    {
                        int LA329_30 = LA(2);
                        if ( (LA329_30 == DOT_SYMBOL))
                        {
                            {
                                int LA329_767 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 767;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_30 == EOF || ((LA329_30 >= ACCOUNT_SYMBOL) && (LA329_30 <= YEAR_SYMBOL)) || LA329_30 == AND_SYMBOL || LA329_30 == BETWEEN_SYMBOL || LA329_30 == COLLATE_SYMBOL || LA329_30 == CROSS_SYMBOL || LA329_30 == FOR_SYMBOL || LA329_30 == FROM_SYMBOL || ((LA329_30 >= GROUP_SYMBOL) && (LA329_30 <= HAVING_SYMBOL)) || LA329_30 == INNER_SYMBOL || LA329_30 == INTO_SYMBOL || LA329_30 == IN_SYMBOL || LA329_30 == IS_SYMBOL || ((LA329_30 >= JOIN_SYMBOL) && (LA329_30 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_30 >= LEFT_SYMBOL) && (LA329_30 <= LIMIT_SYMBOL)) || LA329_30 == LOCK_SYMBOL || ((LA329_30 >= MOD_SYMBOL) && (LA329_30 <= NATURAL_SYMBOL)) || LA329_30 == NOT_SYMBOL || LA329_30 == ON_SYMBOL || LA329_30 == OPEN_PAR_SYMBOL || ((LA329_30 >= ORDER_SYMBOL) && (LA329_30 <= OR_SYMBOL)) || LA329_30 == PROCEDURE_SYMBOL || LA329_30 == REGEXP_SYMBOL || LA329_30 == RIGHT_SYMBOL || LA329_30 == SEMICOLON_SYMBOL || LA329_30 == SET_SYMBOL || LA329_30 == SINGLE_QUOTED_TEXT || LA329_30 == STRAIGHT_JOIN_SYMBOL || LA329_30 == UNION_SYMBOL || LA329_30 == USING_SYMBOL || LA329_30 == WHERE_SYMBOL || LA329_30 == WITH_SYMBOL || LA329_30 == XOR_SYMBOL || LA329_30 == AS_SYMBOL || ((LA329_30 >= BACK_TICK_QUOTED_ID) && (LA329_30 <= BITWISE_AND_OPERATOR)) || ((LA329_30 >= BITWISE_OR_OPERATOR) && (LA329_30 <= BITWISE_XOR_OPERATOR)) || ((LA329_30 >= CLOSE_CURLY_SYMBOL) && (LA329_30 <= CLOSE_PAR_SYMBOL)) || ((LA329_30 >= COMMA_SYMBOL) && (LA329_30 <= CONCAT_PIPES_SYMBOL)) || ((LA329_30 >= DIV_OPERATOR) && (LA329_30 <= DIV_SYMBOL)) || LA329_30 == DOUBLE_QUOTED_TEXT || LA329_30 == EQUAL_OPERATOR || ((LA329_30 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_30 <= GREATER_THAN_OPERATOR)) || ((LA329_30 >= HEX_NUMBER) && (LA329_30 <= IDENTIFIER)) || LA329_30 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_30 >= LESS_OR_EQUAL_OPERATOR) && (LA329_30 <= LESS_THAN_OPERATOR)) || LA329_30 == LOGICAL_AND_OPERATOR || LA329_30 == LOGICAL_OR_OPERATOR || ((LA329_30 >= MINUS_OPERATOR) && (LA329_30 <= MOD_OPERATOR)) || ((LA329_30 >= MULT_OPERATOR) && (LA329_30 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_30 == PLUS_OPERATOR || ((LA329_30 >= SHIFT_LEFT_OPERATOR) && (LA329_30 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 30;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == YEAR_SYMBOL))
                {
                    {
                        int LA329_32 = LA(2);
                        if ( (LA329_32 == EOF || ((LA329_32 >= ACCOUNT_SYMBOL) && (LA329_32 <= YEAR_SYMBOL)) || LA329_32 == AND_SYMBOL || LA329_32 == BETWEEN_SYMBOL || LA329_32 == COLLATE_SYMBOL || LA329_32 == CROSS_SYMBOL || LA329_32 == FOR_SYMBOL || LA329_32 == FROM_SYMBOL || ((LA329_32 >= GROUP_SYMBOL) && (LA329_32 <= HAVING_SYMBOL)) || LA329_32 == INNER_SYMBOL || LA329_32 == INTO_SYMBOL || LA329_32 == IN_SYMBOL || LA329_32 == IS_SYMBOL || ((LA329_32 >= JOIN_SYMBOL) && (LA329_32 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_32 >= LEFT_SYMBOL) && (LA329_32 <= LIMIT_SYMBOL)) || LA329_32 == LOCK_SYMBOL || ((LA329_32 >= MOD_SYMBOL) && (LA329_32 <= NATURAL_SYMBOL)) || LA329_32 == NOT_SYMBOL || LA329_32 == ON_SYMBOL || LA329_32 == OPEN_PAR_SYMBOL || ((LA329_32 >= ORDER_SYMBOL) && (LA329_32 <= OR_SYMBOL)) || LA329_32 == PROCEDURE_SYMBOL || LA329_32 == REGEXP_SYMBOL || LA329_32 == RIGHT_SYMBOL || LA329_32 == SEMICOLON_SYMBOL || LA329_32 == SET_SYMBOL || LA329_32 == SINGLE_QUOTED_TEXT || LA329_32 == STRAIGHT_JOIN_SYMBOL || LA329_32 == UNION_SYMBOL || LA329_32 == USING_SYMBOL || LA329_32 == WHERE_SYMBOL || LA329_32 == WITH_SYMBOL || LA329_32 == XOR_SYMBOL || LA329_32 == AS_SYMBOL || ((LA329_32 >= BACK_TICK_QUOTED_ID) && (LA329_32 <= BITWISE_AND_OPERATOR)) || ((LA329_32 >= BITWISE_OR_OPERATOR) && (LA329_32 <= BITWISE_XOR_OPERATOR)) || ((LA329_32 >= CLOSE_CURLY_SYMBOL) && (LA329_32 <= CLOSE_PAR_SYMBOL)) || ((LA329_32 >= COMMA_SYMBOL) && (LA329_32 <= CONCAT_PIPES_SYMBOL)) || ((LA329_32 >= DIV_OPERATOR) && (LA329_32 <= DIV_SYMBOL)) || LA329_32 == DOUBLE_QUOTED_TEXT || LA329_32 == EQUAL_OPERATOR || ((LA329_32 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_32 <= GREATER_THAN_OPERATOR)) || ((LA329_32 >= HEX_NUMBER) && (LA329_32 <= IDENTIFIER)) || LA329_32 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_32 >= LESS_OR_EQUAL_OPERATOR) && (LA329_32 <= LESS_THAN_OPERATOR)) || LA329_32 == LOGICAL_AND_OPERATOR || LA329_32 == LOGICAL_OR_OPERATOR || ((LA329_32 >= MINUS_OPERATOR) && (LA329_32 <= MOD_OPERATOR)) || ((LA329_32 >= MULT_OPERATOR) && (LA329_32 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_32 == PLUS_OPERATOR || ((LA329_32 >= SHIFT_LEFT_OPERATOR) && (LA329_32 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_32 == DOT_SYMBOL))
                        {
                            {
                                int LA329_823 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 823;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 32;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ADDDATE_SYMBOL || LA329_0 == SUBDATE_SYMBOL))
                {
                    {
                        int LA329_34 = LA(2);
                        if ( (LA329_34 == EOF || ((LA329_34 >= ACCOUNT_SYMBOL) && (LA329_34 <= YEAR_SYMBOL)) || LA329_34 == AND_SYMBOL || LA329_34 == BETWEEN_SYMBOL || LA329_34 == COLLATE_SYMBOL || LA329_34 == CROSS_SYMBOL || LA329_34 == FOR_SYMBOL || LA329_34 == FROM_SYMBOL || ((LA329_34 >= GROUP_SYMBOL) && (LA329_34 <= HAVING_SYMBOL)) || LA329_34 == INNER_SYMBOL || LA329_34 == INTO_SYMBOL || LA329_34 == IN_SYMBOL || LA329_34 == IS_SYMBOL || ((LA329_34 >= JOIN_SYMBOL) && (LA329_34 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_34 >= LEFT_SYMBOL) && (LA329_34 <= LIMIT_SYMBOL)) || LA329_34 == LOCK_SYMBOL || ((LA329_34 >= MOD_SYMBOL) && (LA329_34 <= NATURAL_SYMBOL)) || LA329_34 == NOT_SYMBOL || LA329_34 == ON_SYMBOL || LA329_34 == OPEN_PAR_SYMBOL || ((LA329_34 >= ORDER_SYMBOL) && (LA329_34 <= OR_SYMBOL)) || LA329_34 == PROCEDURE_SYMBOL || LA329_34 == REGEXP_SYMBOL || LA329_34 == RIGHT_SYMBOL || LA329_34 == SEMICOLON_SYMBOL || LA329_34 == SET_SYMBOL || LA329_34 == SINGLE_QUOTED_TEXT || LA329_34 == STRAIGHT_JOIN_SYMBOL || LA329_34 == UNION_SYMBOL || LA329_34 == USING_SYMBOL || LA329_34 == WHERE_SYMBOL || LA329_34 == WITH_SYMBOL || LA329_34 == XOR_SYMBOL || LA329_34 == AS_SYMBOL || ((LA329_34 >= BACK_TICK_QUOTED_ID) && (LA329_34 <= BITWISE_AND_OPERATOR)) || ((LA329_34 >= BITWISE_OR_OPERATOR) && (LA329_34 <= BITWISE_XOR_OPERATOR)) || ((LA329_34 >= CLOSE_CURLY_SYMBOL) && (LA329_34 <= CLOSE_PAR_SYMBOL)) || ((LA329_34 >= COMMA_SYMBOL) && (LA329_34 <= CONCAT_PIPES_SYMBOL)) || ((LA329_34 >= DIV_OPERATOR) && (LA329_34 <= DIV_SYMBOL)) || LA329_34 == DOUBLE_QUOTED_TEXT || LA329_34 == EQUAL_OPERATOR || ((LA329_34 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_34 <= GREATER_THAN_OPERATOR)) || ((LA329_34 >= HEX_NUMBER) && (LA329_34 <= IDENTIFIER)) || LA329_34 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_34 >= LESS_OR_EQUAL_OPERATOR) && (LA329_34 <= LESS_THAN_OPERATOR)) || LA329_34 == LOGICAL_AND_OPERATOR || LA329_34 == LOGICAL_OR_OPERATOR || ((LA329_34 >= MINUS_OPERATOR) && (LA329_34 <= MOD_OPERATOR)) || ((LA329_34 >= MULT_OPERATOR) && (LA329_34 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_34 == PLUS_OPERATOR || ((LA329_34 >= SHIFT_LEFT_OPERATOR) && (LA329_34 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_34 == DOT_SYMBOL))
                        {
                            {
                                int LA329_878 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 878;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 34;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == GET_FORMAT_SYMBOL))
                {
                    {
                        int LA329_35 = LA(2);
                        if ( (LA329_35 == EOF || ((LA329_35 >= ACCOUNT_SYMBOL) && (LA329_35 <= YEAR_SYMBOL)) || LA329_35 == AND_SYMBOL || LA329_35 == BETWEEN_SYMBOL || LA329_35 == COLLATE_SYMBOL || LA329_35 == CROSS_SYMBOL || LA329_35 == FOR_SYMBOL || LA329_35 == FROM_SYMBOL || ((LA329_35 >= GROUP_SYMBOL) && (LA329_35 <= HAVING_SYMBOL)) || LA329_35 == INNER_SYMBOL || LA329_35 == INTO_SYMBOL || LA329_35 == IN_SYMBOL || LA329_35 == IS_SYMBOL || ((LA329_35 >= JOIN_SYMBOL) && (LA329_35 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_35 >= LEFT_SYMBOL) && (LA329_35 <= LIMIT_SYMBOL)) || LA329_35 == LOCK_SYMBOL || ((LA329_35 >= MOD_SYMBOL) && (LA329_35 <= NATURAL_SYMBOL)) || LA329_35 == NOT_SYMBOL || LA329_35 == ON_SYMBOL || LA329_35 == OPEN_PAR_SYMBOL || ((LA329_35 >= ORDER_SYMBOL) && (LA329_35 <= OR_SYMBOL)) || LA329_35 == PROCEDURE_SYMBOL || LA329_35 == REGEXP_SYMBOL || LA329_35 == RIGHT_SYMBOL || LA329_35 == SEMICOLON_SYMBOL || LA329_35 == SET_SYMBOL || LA329_35 == SINGLE_QUOTED_TEXT || LA329_35 == STRAIGHT_JOIN_SYMBOL || LA329_35 == UNION_SYMBOL || LA329_35 == USING_SYMBOL || LA329_35 == WHERE_SYMBOL || LA329_35 == WITH_SYMBOL || LA329_35 == XOR_SYMBOL || LA329_35 == AS_SYMBOL || ((LA329_35 >= BACK_TICK_QUOTED_ID) && (LA329_35 <= BITWISE_AND_OPERATOR)) || ((LA329_35 >= BITWISE_OR_OPERATOR) && (LA329_35 <= BITWISE_XOR_OPERATOR)) || ((LA329_35 >= CLOSE_CURLY_SYMBOL) && (LA329_35 <= CLOSE_PAR_SYMBOL)) || ((LA329_35 >= COMMA_SYMBOL) && (LA329_35 <= CONCAT_PIPES_SYMBOL)) || ((LA329_35 >= DIV_OPERATOR) && (LA329_35 <= DIV_SYMBOL)) || LA329_35 == DOUBLE_QUOTED_TEXT || LA329_35 == EQUAL_OPERATOR || ((LA329_35 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_35 <= GREATER_THAN_OPERATOR)) || ((LA329_35 >= HEX_NUMBER) && (LA329_35 <= IDENTIFIER)) || LA329_35 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_35 >= LESS_OR_EQUAL_OPERATOR) && (LA329_35 <= LESS_THAN_OPERATOR)) || LA329_35 == LOGICAL_AND_OPERATOR || LA329_35 == LOGICAL_OR_OPERATOR || ((LA329_35 >= MINUS_OPERATOR) && (LA329_35 <= MOD_OPERATOR)) || ((LA329_35 >= MULT_OPERATOR) && (LA329_35 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_35 == PLUS_OPERATOR || ((LA329_35 >= SHIFT_LEFT_OPERATOR) && (LA329_35 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_35 == DOT_SYMBOL))
                        {
                            {
                                int LA329_933 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 933;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 35;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (((LA329_0 >= TIMESTAMP_ADD_SYMBOL) && (LA329_0 <= TIMESTAMP_DIFF_SYMBOL))))
                {
                    {
                        int LA329_40 = LA(2);
                        if ( (LA329_40 == EOF || ((LA329_40 >= ACCOUNT_SYMBOL) && (LA329_40 <= YEAR_SYMBOL)) || LA329_40 == AND_SYMBOL || LA329_40 == BETWEEN_SYMBOL || LA329_40 == COLLATE_SYMBOL || LA329_40 == CROSS_SYMBOL || LA329_40 == FOR_SYMBOL || LA329_40 == FROM_SYMBOL || ((LA329_40 >= GROUP_SYMBOL) && (LA329_40 <= HAVING_SYMBOL)) || LA329_40 == INNER_SYMBOL || LA329_40 == INTO_SYMBOL || LA329_40 == IN_SYMBOL || LA329_40 == IS_SYMBOL || ((LA329_40 >= JOIN_SYMBOL) && (LA329_40 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_40 >= LEFT_SYMBOL) && (LA329_40 <= LIMIT_SYMBOL)) || LA329_40 == LOCK_SYMBOL || ((LA329_40 >= MOD_SYMBOL) && (LA329_40 <= NATURAL_SYMBOL)) || LA329_40 == NOT_SYMBOL || LA329_40 == ON_SYMBOL || LA329_40 == OPEN_PAR_SYMBOL || ((LA329_40 >= ORDER_SYMBOL) && (LA329_40 <= OR_SYMBOL)) || LA329_40 == PROCEDURE_SYMBOL || LA329_40 == REGEXP_SYMBOL || LA329_40 == RIGHT_SYMBOL || LA329_40 == SEMICOLON_SYMBOL || LA329_40 == SET_SYMBOL || LA329_40 == SINGLE_QUOTED_TEXT || LA329_40 == STRAIGHT_JOIN_SYMBOL || LA329_40 == UNION_SYMBOL || LA329_40 == USING_SYMBOL || LA329_40 == WHERE_SYMBOL || LA329_40 == WITH_SYMBOL || LA329_40 == XOR_SYMBOL || LA329_40 == AS_SYMBOL || ((LA329_40 >= BACK_TICK_QUOTED_ID) && (LA329_40 <= BITWISE_AND_OPERATOR)) || ((LA329_40 >= BITWISE_OR_OPERATOR) && (LA329_40 <= BITWISE_XOR_OPERATOR)) || ((LA329_40 >= CLOSE_CURLY_SYMBOL) && (LA329_40 <= CLOSE_PAR_SYMBOL)) || ((LA329_40 >= COMMA_SYMBOL) && (LA329_40 <= CONCAT_PIPES_SYMBOL)) || ((LA329_40 >= DIV_OPERATOR) && (LA329_40 <= DIV_SYMBOL)) || LA329_40 == DOUBLE_QUOTED_TEXT || LA329_40 == EQUAL_OPERATOR || ((LA329_40 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_40 <= GREATER_THAN_OPERATOR)) || ((LA329_40 >= HEX_NUMBER) && (LA329_40 <= IDENTIFIER)) || LA329_40 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_40 >= LESS_OR_EQUAL_OPERATOR) && (LA329_40 <= LESS_THAN_OPERATOR)) || LA329_40 == LOGICAL_AND_OPERATOR || LA329_40 == LOGICAL_OR_OPERATOR || ((LA329_40 >= MINUS_OPERATOR) && (LA329_40 <= MOD_OPERATOR)) || ((LA329_40 >= MULT_OPERATOR) && (LA329_40 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_40 == PLUS_OPERATOR || ((LA329_40 >= SHIFT_LEFT_OPERATOR) && (LA329_40 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_40 == DOT_SYMBOL))
                        {
                            {
                                int LA329_988 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 988;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 40;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == COALESCE_SYMBOL))
                {
                    {
                        int LA329_45 = LA(2);
                        if ( (LA329_45 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1042 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1042;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_45 == EOF || ((LA329_45 >= ACCOUNT_SYMBOL) && (LA329_45 <= YEAR_SYMBOL)) || LA329_45 == AND_SYMBOL || LA329_45 == BETWEEN_SYMBOL || LA329_45 == COLLATE_SYMBOL || LA329_45 == CROSS_SYMBOL || LA329_45 == FOR_SYMBOL || LA329_45 == FROM_SYMBOL || ((LA329_45 >= GROUP_SYMBOL) && (LA329_45 <= HAVING_SYMBOL)) || LA329_45 == INNER_SYMBOL || LA329_45 == INTO_SYMBOL || LA329_45 == IN_SYMBOL || LA329_45 == IS_SYMBOL || ((LA329_45 >= JOIN_SYMBOL) && (LA329_45 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_45 >= LEFT_SYMBOL) && (LA329_45 <= LIMIT_SYMBOL)) || LA329_45 == LOCK_SYMBOL || ((LA329_45 >= MOD_SYMBOL) && (LA329_45 <= NATURAL_SYMBOL)) || LA329_45 == NOT_SYMBOL || LA329_45 == ON_SYMBOL || LA329_45 == OPEN_PAR_SYMBOL || ((LA329_45 >= ORDER_SYMBOL) && (LA329_45 <= OR_SYMBOL)) || LA329_45 == PROCEDURE_SYMBOL || LA329_45 == REGEXP_SYMBOL || LA329_45 == RIGHT_SYMBOL || LA329_45 == SEMICOLON_SYMBOL || LA329_45 == SET_SYMBOL || LA329_45 == SINGLE_QUOTED_TEXT || LA329_45 == STRAIGHT_JOIN_SYMBOL || LA329_45 == UNION_SYMBOL || LA329_45 == USING_SYMBOL || LA329_45 == WHERE_SYMBOL || LA329_45 == WITH_SYMBOL || LA329_45 == XOR_SYMBOL || LA329_45 == AS_SYMBOL || ((LA329_45 >= BACK_TICK_QUOTED_ID) && (LA329_45 <= BITWISE_AND_OPERATOR)) || ((LA329_45 >= BITWISE_OR_OPERATOR) && (LA329_45 <= BITWISE_XOR_OPERATOR)) || ((LA329_45 >= CLOSE_CURLY_SYMBOL) && (LA329_45 <= CLOSE_PAR_SYMBOL)) || ((LA329_45 >= COMMA_SYMBOL) && (LA329_45 <= CONCAT_PIPES_SYMBOL)) || ((LA329_45 >= DIV_OPERATOR) && (LA329_45 <= DIV_SYMBOL)) || LA329_45 == DOUBLE_QUOTED_TEXT || LA329_45 == EQUAL_OPERATOR || ((LA329_45 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_45 <= GREATER_THAN_OPERATOR)) || ((LA329_45 >= HEX_NUMBER) && (LA329_45 <= IDENTIFIER)) || LA329_45 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_45 >= LESS_OR_EQUAL_OPERATOR) && (LA329_45 <= LESS_THAN_OPERATOR)) || LA329_45 == LOGICAL_AND_OPERATOR || LA329_45 == LOGICAL_OR_OPERATOR || ((LA329_45 >= MINUS_OPERATOR) && (LA329_45 <= MOD_OPERATOR)) || ((LA329_45 >= MULT_OPERATOR) && (LA329_45 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_45 == PLUS_OPERATOR || ((LA329_45 >= SHIFT_LEFT_OPERATOR) && (LA329_45 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 45;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == CHARSET_SYMBOL))
                {
                    {
                        int LA329_49 = LA(2);
                        if ( (LA329_49 == EOF || ((LA329_49 >= ACCOUNT_SYMBOL) && (LA329_49 <= YEAR_SYMBOL)) || LA329_49 == AND_SYMBOL || LA329_49 == BETWEEN_SYMBOL || LA329_49 == COLLATE_SYMBOL || LA329_49 == CROSS_SYMBOL || LA329_49 == FOR_SYMBOL || LA329_49 == FROM_SYMBOL || ((LA329_49 >= GROUP_SYMBOL) && (LA329_49 <= HAVING_SYMBOL)) || LA329_49 == INNER_SYMBOL || LA329_49 == INTO_SYMBOL || LA329_49 == IN_SYMBOL || LA329_49 == IS_SYMBOL || ((LA329_49 >= JOIN_SYMBOL) && (LA329_49 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_49 >= LEFT_SYMBOL) && (LA329_49 <= LIMIT_SYMBOL)) || LA329_49 == LOCK_SYMBOL || ((LA329_49 >= MOD_SYMBOL) && (LA329_49 <= NATURAL_SYMBOL)) || LA329_49 == NOT_SYMBOL || LA329_49 == ON_SYMBOL || LA329_49 == OPEN_PAR_SYMBOL || ((LA329_49 >= ORDER_SYMBOL) && (LA329_49 <= OR_SYMBOL)) || LA329_49 == PROCEDURE_SYMBOL || LA329_49 == REGEXP_SYMBOL || LA329_49 == RIGHT_SYMBOL || LA329_49 == SEMICOLON_SYMBOL || LA329_49 == SET_SYMBOL || LA329_49 == SINGLE_QUOTED_TEXT || LA329_49 == STRAIGHT_JOIN_SYMBOL || LA329_49 == UNION_SYMBOL || LA329_49 == USING_SYMBOL || LA329_49 == WHERE_SYMBOL || LA329_49 == WITH_SYMBOL || LA329_49 == XOR_SYMBOL || LA329_49 == AS_SYMBOL || ((LA329_49 >= BACK_TICK_QUOTED_ID) && (LA329_49 <= BITWISE_AND_OPERATOR)) || ((LA329_49 >= BITWISE_OR_OPERATOR) && (LA329_49 <= BITWISE_XOR_OPERATOR)) || ((LA329_49 >= CLOSE_CURLY_SYMBOL) && (LA329_49 <= CLOSE_PAR_SYMBOL)) || ((LA329_49 >= COMMA_SYMBOL) && (LA329_49 <= CONCAT_PIPES_SYMBOL)) || ((LA329_49 >= DIV_OPERATOR) && (LA329_49 <= DIV_SYMBOL)) || LA329_49 == DOUBLE_QUOTED_TEXT || LA329_49 == EQUAL_OPERATOR || ((LA329_49 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_49 <= GREATER_THAN_OPERATOR)) || ((LA329_49 >= HEX_NUMBER) && (LA329_49 <= IDENTIFIER)) || LA329_49 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_49 >= LESS_OR_EQUAL_OPERATOR) && (LA329_49 <= LESS_THAN_OPERATOR)) || LA329_49 == LOGICAL_AND_OPERATOR || LA329_49 == LOGICAL_OR_OPERATOR || ((LA329_49 >= MINUS_OPERATOR) && (LA329_49 <= MOD_OPERATOR)) || ((LA329_49 >= MULT_OPERATOR) && (LA329_49 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_49 == PLUS_OPERATOR || ((LA329_49 >= SHIFT_LEFT_OPERATOR) && (LA329_49 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_49 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1098 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1098;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 49;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == FORMAT_SYMBOL))
                {
                    {
                        int LA329_50 = LA(2);
                        if ( (LA329_50 == EOF || ((LA329_50 >= ACCOUNT_SYMBOL) && (LA329_50 <= YEAR_SYMBOL)) || LA329_50 == AND_SYMBOL || LA329_50 == BETWEEN_SYMBOL || LA329_50 == COLLATE_SYMBOL || LA329_50 == CROSS_SYMBOL || LA329_50 == FOR_SYMBOL || LA329_50 == FROM_SYMBOL || ((LA329_50 >= GROUP_SYMBOL) && (LA329_50 <= HAVING_SYMBOL)) || LA329_50 == INNER_SYMBOL || LA329_50 == INTO_SYMBOL || LA329_50 == IN_SYMBOL || LA329_50 == IS_SYMBOL || ((LA329_50 >= JOIN_SYMBOL) && (LA329_50 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_50 >= LEFT_SYMBOL) && (LA329_50 <= LIMIT_SYMBOL)) || LA329_50 == LOCK_SYMBOL || ((LA329_50 >= MOD_SYMBOL) && (LA329_50 <= NATURAL_SYMBOL)) || LA329_50 == NOT_SYMBOL || LA329_50 == ON_SYMBOL || LA329_50 == OPEN_PAR_SYMBOL || ((LA329_50 >= ORDER_SYMBOL) && (LA329_50 <= OR_SYMBOL)) || LA329_50 == PROCEDURE_SYMBOL || LA329_50 == REGEXP_SYMBOL || LA329_50 == RIGHT_SYMBOL || LA329_50 == SEMICOLON_SYMBOL || LA329_50 == SET_SYMBOL || LA329_50 == SINGLE_QUOTED_TEXT || LA329_50 == STRAIGHT_JOIN_SYMBOL || LA329_50 == UNION_SYMBOL || LA329_50 == USING_SYMBOL || LA329_50 == WHERE_SYMBOL || LA329_50 == WITH_SYMBOL || LA329_50 == XOR_SYMBOL || LA329_50 == AS_SYMBOL || ((LA329_50 >= BACK_TICK_QUOTED_ID) && (LA329_50 <= BITWISE_AND_OPERATOR)) || ((LA329_50 >= BITWISE_OR_OPERATOR) && (LA329_50 <= BITWISE_XOR_OPERATOR)) || ((LA329_50 >= CLOSE_CURLY_SYMBOL) && (LA329_50 <= CLOSE_PAR_SYMBOL)) || ((LA329_50 >= COMMA_SYMBOL) && (LA329_50 <= CONCAT_PIPES_SYMBOL)) || ((LA329_50 >= DIV_OPERATOR) && (LA329_50 <= DIV_SYMBOL)) || LA329_50 == DOUBLE_QUOTED_TEXT || LA329_50 == EQUAL_OPERATOR || ((LA329_50 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_50 <= GREATER_THAN_OPERATOR)) || ((LA329_50 >= HEX_NUMBER) && (LA329_50 <= IDENTIFIER)) || LA329_50 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_50 >= LESS_OR_EQUAL_OPERATOR) && (LA329_50 <= LESS_THAN_OPERATOR)) || LA329_50 == LOGICAL_AND_OPERATOR || LA329_50 == LOGICAL_OR_OPERATOR || ((LA329_50 >= MINUS_OPERATOR) && (LA329_50 <= MOD_OPERATOR)) || ((LA329_50 >= MULT_OPERATOR) && (LA329_50 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_50 == PLUS_OPERATOR || ((LA329_50 >= SHIFT_LEFT_OPERATOR) && (LA329_50 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_50 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1153 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1153;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 50;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == COLLATION_SYMBOL))
                {
                    {
                        int LA329_51 = LA(2);
                        if ( (LA329_51 == EOF || ((LA329_51 >= ACCOUNT_SYMBOL) && (LA329_51 <= YEAR_SYMBOL)) || LA329_51 == AND_SYMBOL || LA329_51 == BETWEEN_SYMBOL || LA329_51 == COLLATE_SYMBOL || LA329_51 == CROSS_SYMBOL || LA329_51 == FOR_SYMBOL || LA329_51 == FROM_SYMBOL || ((LA329_51 >= GROUP_SYMBOL) && (LA329_51 <= HAVING_SYMBOL)) || LA329_51 == INNER_SYMBOL || LA329_51 == INTO_SYMBOL || LA329_51 == IN_SYMBOL || LA329_51 == IS_SYMBOL || ((LA329_51 >= JOIN_SYMBOL) && (LA329_51 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_51 >= LEFT_SYMBOL) && (LA329_51 <= LIMIT_SYMBOL)) || LA329_51 == LOCK_SYMBOL || ((LA329_51 >= MOD_SYMBOL) && (LA329_51 <= NATURAL_SYMBOL)) || LA329_51 == NOT_SYMBOL || LA329_51 == ON_SYMBOL || LA329_51 == OPEN_PAR_SYMBOL || ((LA329_51 >= ORDER_SYMBOL) && (LA329_51 <= OR_SYMBOL)) || LA329_51 == PROCEDURE_SYMBOL || LA329_51 == REGEXP_SYMBOL || LA329_51 == RIGHT_SYMBOL || LA329_51 == SEMICOLON_SYMBOL || LA329_51 == SET_SYMBOL || LA329_51 == SINGLE_QUOTED_TEXT || LA329_51 == STRAIGHT_JOIN_SYMBOL || LA329_51 == UNION_SYMBOL || LA329_51 == USING_SYMBOL || LA329_51 == WHERE_SYMBOL || LA329_51 == WITH_SYMBOL || LA329_51 == XOR_SYMBOL || LA329_51 == AS_SYMBOL || ((LA329_51 >= BACK_TICK_QUOTED_ID) && (LA329_51 <= BITWISE_AND_OPERATOR)) || ((LA329_51 >= BITWISE_OR_OPERATOR) && (LA329_51 <= BITWISE_XOR_OPERATOR)) || ((LA329_51 >= CLOSE_CURLY_SYMBOL) && (LA329_51 <= CLOSE_PAR_SYMBOL)) || ((LA329_51 >= COMMA_SYMBOL) && (LA329_51 <= CONCAT_PIPES_SYMBOL)) || ((LA329_51 >= DIV_OPERATOR) && (LA329_51 <= DIV_SYMBOL)) || LA329_51 == DOUBLE_QUOTED_TEXT || LA329_51 == EQUAL_OPERATOR || ((LA329_51 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_51 <= GREATER_THAN_OPERATOR)) || ((LA329_51 >= HEX_NUMBER) && (LA329_51 <= IDENTIFIER)) || LA329_51 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_51 >= LESS_OR_EQUAL_OPERATOR) && (LA329_51 <= LESS_THAN_OPERATOR)) || LA329_51 == LOGICAL_AND_OPERATOR || LA329_51 == LOGICAL_OR_OPERATOR || ((LA329_51 >= MINUS_OPERATOR) && (LA329_51 <= MOD_OPERATOR)) || ((LA329_51 >= MULT_OPERATOR) && (LA329_51 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_51 == PLUS_OPERATOR || ((LA329_51 >= SHIFT_LEFT_OPERATOR) && (LA329_51 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_51 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1208 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1208;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 51;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MICROSECOND_SYMBOL))
                {
                    {
                        int LA329_52 = LA(2);
                        if ( (LA329_52 == EOF || ((LA329_52 >= ACCOUNT_SYMBOL) && (LA329_52 <= YEAR_SYMBOL)) || LA329_52 == AND_SYMBOL || LA329_52 == BETWEEN_SYMBOL || LA329_52 == COLLATE_SYMBOL || LA329_52 == CROSS_SYMBOL || LA329_52 == FOR_SYMBOL || LA329_52 == FROM_SYMBOL || ((LA329_52 >= GROUP_SYMBOL) && (LA329_52 <= HAVING_SYMBOL)) || LA329_52 == INNER_SYMBOL || LA329_52 == INTO_SYMBOL || LA329_52 == IN_SYMBOL || LA329_52 == IS_SYMBOL || ((LA329_52 >= JOIN_SYMBOL) && (LA329_52 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_52 >= LEFT_SYMBOL) && (LA329_52 <= LIMIT_SYMBOL)) || LA329_52 == LOCK_SYMBOL || ((LA329_52 >= MOD_SYMBOL) && (LA329_52 <= NATURAL_SYMBOL)) || LA329_52 == NOT_SYMBOL || LA329_52 == ON_SYMBOL || LA329_52 == OPEN_PAR_SYMBOL || ((LA329_52 >= ORDER_SYMBOL) && (LA329_52 <= OR_SYMBOL)) || LA329_52 == PROCEDURE_SYMBOL || LA329_52 == REGEXP_SYMBOL || LA329_52 == RIGHT_SYMBOL || LA329_52 == SEMICOLON_SYMBOL || LA329_52 == SET_SYMBOL || LA329_52 == SINGLE_QUOTED_TEXT || LA329_52 == STRAIGHT_JOIN_SYMBOL || LA329_52 == UNION_SYMBOL || LA329_52 == USING_SYMBOL || LA329_52 == WHERE_SYMBOL || LA329_52 == WITH_SYMBOL || LA329_52 == XOR_SYMBOL || LA329_52 == AS_SYMBOL || ((LA329_52 >= BACK_TICK_QUOTED_ID) && (LA329_52 <= BITWISE_AND_OPERATOR)) || ((LA329_52 >= BITWISE_OR_OPERATOR) && (LA329_52 <= BITWISE_XOR_OPERATOR)) || ((LA329_52 >= CLOSE_CURLY_SYMBOL) && (LA329_52 <= CLOSE_PAR_SYMBOL)) || ((LA329_52 >= COMMA_SYMBOL) && (LA329_52 <= CONCAT_PIPES_SYMBOL)) || ((LA329_52 >= DIV_OPERATOR) && (LA329_52 <= DIV_SYMBOL)) || LA329_52 == DOUBLE_QUOTED_TEXT || LA329_52 == EQUAL_OPERATOR || ((LA329_52 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_52 <= GREATER_THAN_OPERATOR)) || ((LA329_52 >= HEX_NUMBER) && (LA329_52 <= IDENTIFIER)) || LA329_52 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_52 >= LESS_OR_EQUAL_OPERATOR) && (LA329_52 <= LESS_THAN_OPERATOR)) || LA329_52 == LOGICAL_AND_OPERATOR || LA329_52 == LOGICAL_OR_OPERATOR || ((LA329_52 >= MINUS_OPERATOR) && (LA329_52 <= MOD_OPERATOR)) || ((LA329_52 >= MULT_OPERATOR) && (LA329_52 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_52 == PLUS_OPERATOR || ((LA329_52 >= SHIFT_LEFT_OPERATOR) && (LA329_52 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_52 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1263 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1263;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 52;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == TRUNCATE_SYMBOL))
                {
                    {
                        int LA329_55 = LA(2);
                        if ( (LA329_55 == EOF || ((LA329_55 >= ACCOUNT_SYMBOL) && (LA329_55 <= YEAR_SYMBOL)) || LA329_55 == AND_SYMBOL || LA329_55 == BETWEEN_SYMBOL || LA329_55 == COLLATE_SYMBOL || LA329_55 == CROSS_SYMBOL || LA329_55 == FOR_SYMBOL || LA329_55 == FROM_SYMBOL || ((LA329_55 >= GROUP_SYMBOL) && (LA329_55 <= HAVING_SYMBOL)) || LA329_55 == INNER_SYMBOL || LA329_55 == INTO_SYMBOL || LA329_55 == IN_SYMBOL || LA329_55 == IS_SYMBOL || ((LA329_55 >= JOIN_SYMBOL) && (LA329_55 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_55 >= LEFT_SYMBOL) && (LA329_55 <= LIMIT_SYMBOL)) || LA329_55 == LOCK_SYMBOL || ((LA329_55 >= MOD_SYMBOL) && (LA329_55 <= NATURAL_SYMBOL)) || LA329_55 == NOT_SYMBOL || LA329_55 == ON_SYMBOL || LA329_55 == OPEN_PAR_SYMBOL || ((LA329_55 >= ORDER_SYMBOL) && (LA329_55 <= OR_SYMBOL)) || LA329_55 == PROCEDURE_SYMBOL || LA329_55 == REGEXP_SYMBOL || LA329_55 == RIGHT_SYMBOL || LA329_55 == SEMICOLON_SYMBOL || LA329_55 == SET_SYMBOL || LA329_55 == SINGLE_QUOTED_TEXT || LA329_55 == STRAIGHT_JOIN_SYMBOL || LA329_55 == UNION_SYMBOL || LA329_55 == USING_SYMBOL || LA329_55 == WHERE_SYMBOL || LA329_55 == WITH_SYMBOL || LA329_55 == XOR_SYMBOL || LA329_55 == AS_SYMBOL || ((LA329_55 >= BACK_TICK_QUOTED_ID) && (LA329_55 <= BITWISE_AND_OPERATOR)) || ((LA329_55 >= BITWISE_OR_OPERATOR) && (LA329_55 <= BITWISE_XOR_OPERATOR)) || ((LA329_55 >= CLOSE_CURLY_SYMBOL) && (LA329_55 <= CLOSE_PAR_SYMBOL)) || ((LA329_55 >= COMMA_SYMBOL) && (LA329_55 <= CONCAT_PIPES_SYMBOL)) || ((LA329_55 >= DIV_OPERATOR) && (LA329_55 <= DIV_SYMBOL)) || LA329_55 == DOUBLE_QUOTED_TEXT || LA329_55 == EQUAL_OPERATOR || ((LA329_55 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_55 <= GREATER_THAN_OPERATOR)) || ((LA329_55 >= HEX_NUMBER) && (LA329_55 <= IDENTIFIER)) || LA329_55 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_55 >= LESS_OR_EQUAL_OPERATOR) && (LA329_55 <= LESS_THAN_OPERATOR)) || LA329_55 == LOGICAL_AND_OPERATOR || LA329_55 == LOGICAL_OR_OPERATOR || ((LA329_55 >= MINUS_OPERATOR) && (LA329_55 <= MOD_OPERATOR)) || ((LA329_55 >= MULT_OPERATOR) && (LA329_55 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_55 == PLUS_OPERATOR || ((LA329_55 >= SHIFT_LEFT_OPERATOR) && (LA329_55 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_55 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1318 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1318;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 55;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == OLD_PASSWORD_SYMBOL))
                {
                    {
                        int LA329_56 = LA(2);
                        if ( (LA329_56 == EOF || ((LA329_56 >= ACCOUNT_SYMBOL) && (LA329_56 <= YEAR_SYMBOL)) || LA329_56 == AND_SYMBOL || LA329_56 == BETWEEN_SYMBOL || LA329_56 == COLLATE_SYMBOL || LA329_56 == CROSS_SYMBOL || LA329_56 == FOR_SYMBOL || LA329_56 == FROM_SYMBOL || ((LA329_56 >= GROUP_SYMBOL) && (LA329_56 <= HAVING_SYMBOL)) || LA329_56 == INNER_SYMBOL || LA329_56 == INTO_SYMBOL || LA329_56 == IN_SYMBOL || LA329_56 == IS_SYMBOL || ((LA329_56 >= JOIN_SYMBOL) && (LA329_56 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_56 >= LEFT_SYMBOL) && (LA329_56 <= LIMIT_SYMBOL)) || LA329_56 == LOCK_SYMBOL || ((LA329_56 >= MOD_SYMBOL) && (LA329_56 <= NATURAL_SYMBOL)) || LA329_56 == NOT_SYMBOL || LA329_56 == ON_SYMBOL || LA329_56 == OPEN_PAR_SYMBOL || ((LA329_56 >= ORDER_SYMBOL) && (LA329_56 <= OR_SYMBOL)) || LA329_56 == PROCEDURE_SYMBOL || LA329_56 == REGEXP_SYMBOL || LA329_56 == RIGHT_SYMBOL || LA329_56 == SEMICOLON_SYMBOL || LA329_56 == SET_SYMBOL || LA329_56 == SINGLE_QUOTED_TEXT || LA329_56 == STRAIGHT_JOIN_SYMBOL || LA329_56 == UNION_SYMBOL || LA329_56 == USING_SYMBOL || LA329_56 == WHERE_SYMBOL || LA329_56 == WITH_SYMBOL || LA329_56 == XOR_SYMBOL || LA329_56 == AS_SYMBOL || ((LA329_56 >= BACK_TICK_QUOTED_ID) && (LA329_56 <= BITWISE_AND_OPERATOR)) || ((LA329_56 >= BITWISE_OR_OPERATOR) && (LA329_56 <= BITWISE_XOR_OPERATOR)) || ((LA329_56 >= CLOSE_CURLY_SYMBOL) && (LA329_56 <= CLOSE_PAR_SYMBOL)) || ((LA329_56 >= COMMA_SYMBOL) && (LA329_56 <= CONCAT_PIPES_SYMBOL)) || ((LA329_56 >= DIV_OPERATOR) && (LA329_56 <= DIV_SYMBOL)) || LA329_56 == DOUBLE_QUOTED_TEXT || LA329_56 == EQUAL_OPERATOR || ((LA329_56 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_56 <= GREATER_THAN_OPERATOR)) || ((LA329_56 >= HEX_NUMBER) && (LA329_56 <= IDENTIFIER)) || LA329_56 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_56 >= LESS_OR_EQUAL_OPERATOR) && (LA329_56 <= LESS_THAN_OPERATOR)) || LA329_56 == LOGICAL_AND_OPERATOR || LA329_56 == LOGICAL_OR_OPERATOR || ((LA329_56 >= MINUS_OPERATOR) && (LA329_56 <= MOD_OPERATOR)) || ((LA329_56 >= MULT_OPERATOR) && (LA329_56 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_56 == PLUS_OPERATOR || ((LA329_56 >= SHIFT_LEFT_OPERATOR) && (LA329_56 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_56 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1373 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1373;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 56;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == PASSWORD_SYMBOL))
                {
                    {
                        int LA329_58 = LA(2);
                        if ( (LA329_58 == EOF || ((LA329_58 >= ACCOUNT_SYMBOL) && (LA329_58 <= YEAR_SYMBOL)) || LA329_58 == AND_SYMBOL || LA329_58 == BETWEEN_SYMBOL || LA329_58 == COLLATE_SYMBOL || LA329_58 == CROSS_SYMBOL || LA329_58 == FOR_SYMBOL || LA329_58 == FROM_SYMBOL || ((LA329_58 >= GROUP_SYMBOL) && (LA329_58 <= HAVING_SYMBOL)) || LA329_58 == INNER_SYMBOL || LA329_58 == INTO_SYMBOL || LA329_58 == IN_SYMBOL || LA329_58 == IS_SYMBOL || ((LA329_58 >= JOIN_SYMBOL) && (LA329_58 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_58 >= LEFT_SYMBOL) && (LA329_58 <= LIMIT_SYMBOL)) || LA329_58 == LOCK_SYMBOL || ((LA329_58 >= MOD_SYMBOL) && (LA329_58 <= NATURAL_SYMBOL)) || LA329_58 == NOT_SYMBOL || LA329_58 == ON_SYMBOL || LA329_58 == OPEN_PAR_SYMBOL || ((LA329_58 >= ORDER_SYMBOL) && (LA329_58 <= OR_SYMBOL)) || LA329_58 == PROCEDURE_SYMBOL || LA329_58 == REGEXP_SYMBOL || LA329_58 == RIGHT_SYMBOL || LA329_58 == SEMICOLON_SYMBOL || LA329_58 == SET_SYMBOL || LA329_58 == SINGLE_QUOTED_TEXT || LA329_58 == STRAIGHT_JOIN_SYMBOL || LA329_58 == UNION_SYMBOL || LA329_58 == USING_SYMBOL || LA329_58 == WHERE_SYMBOL || LA329_58 == WITH_SYMBOL || LA329_58 == XOR_SYMBOL || LA329_58 == AS_SYMBOL || ((LA329_58 >= BACK_TICK_QUOTED_ID) && (LA329_58 <= BITWISE_AND_OPERATOR)) || ((LA329_58 >= BITWISE_OR_OPERATOR) && (LA329_58 <= BITWISE_XOR_OPERATOR)) || ((LA329_58 >= CLOSE_CURLY_SYMBOL) && (LA329_58 <= CLOSE_PAR_SYMBOL)) || ((LA329_58 >= COMMA_SYMBOL) && (LA329_58 <= CONCAT_PIPES_SYMBOL)) || ((LA329_58 >= DIV_OPERATOR) && (LA329_58 <= DIV_SYMBOL)) || LA329_58 == DOUBLE_QUOTED_TEXT || LA329_58 == EQUAL_OPERATOR || ((LA329_58 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_58 <= GREATER_THAN_OPERATOR)) || ((LA329_58 >= HEX_NUMBER) && (LA329_58 <= IDENTIFIER)) || LA329_58 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_58 >= LESS_OR_EQUAL_OPERATOR) && (LA329_58 <= LESS_THAN_OPERATOR)) || LA329_58 == LOGICAL_AND_OPERATOR || LA329_58 == LOGICAL_OR_OPERATOR || ((LA329_58 >= MINUS_OPERATOR) && (LA329_58 <= MOD_OPERATOR)) || ((LA329_58 >= MULT_OPERATOR) && (LA329_58 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_58 == PLUS_OPERATOR || ((LA329_58 >= SHIFT_LEFT_OPERATOR) && (LA329_58 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_58 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1428 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1428;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 58;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == QUARTER_SYMBOL))
                {
                    {
                        int LA329_59 = LA(2);
                        if ( (LA329_59 == EOF || ((LA329_59 >= ACCOUNT_SYMBOL) && (LA329_59 <= YEAR_SYMBOL)) || LA329_59 == AND_SYMBOL || LA329_59 == BETWEEN_SYMBOL || LA329_59 == COLLATE_SYMBOL || LA329_59 == CROSS_SYMBOL || LA329_59 == FOR_SYMBOL || LA329_59 == FROM_SYMBOL || ((LA329_59 >= GROUP_SYMBOL) && (LA329_59 <= HAVING_SYMBOL)) || LA329_59 == INNER_SYMBOL || LA329_59 == INTO_SYMBOL || LA329_59 == IN_SYMBOL || LA329_59 == IS_SYMBOL || ((LA329_59 >= JOIN_SYMBOL) && (LA329_59 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_59 >= LEFT_SYMBOL) && (LA329_59 <= LIMIT_SYMBOL)) || LA329_59 == LOCK_SYMBOL || ((LA329_59 >= MOD_SYMBOL) && (LA329_59 <= NATURAL_SYMBOL)) || LA329_59 == NOT_SYMBOL || LA329_59 == ON_SYMBOL || LA329_59 == OPEN_PAR_SYMBOL || ((LA329_59 >= ORDER_SYMBOL) && (LA329_59 <= OR_SYMBOL)) || LA329_59 == PROCEDURE_SYMBOL || LA329_59 == REGEXP_SYMBOL || LA329_59 == RIGHT_SYMBOL || LA329_59 == SEMICOLON_SYMBOL || LA329_59 == SET_SYMBOL || LA329_59 == SINGLE_QUOTED_TEXT || LA329_59 == STRAIGHT_JOIN_SYMBOL || LA329_59 == UNION_SYMBOL || LA329_59 == USING_SYMBOL || LA329_59 == WHERE_SYMBOL || LA329_59 == WITH_SYMBOL || LA329_59 == XOR_SYMBOL || LA329_59 == AS_SYMBOL || ((LA329_59 >= BACK_TICK_QUOTED_ID) && (LA329_59 <= BITWISE_AND_OPERATOR)) || ((LA329_59 >= BITWISE_OR_OPERATOR) && (LA329_59 <= BITWISE_XOR_OPERATOR)) || ((LA329_59 >= CLOSE_CURLY_SYMBOL) && (LA329_59 <= CLOSE_PAR_SYMBOL)) || ((LA329_59 >= COMMA_SYMBOL) && (LA329_59 <= CONCAT_PIPES_SYMBOL)) || ((LA329_59 >= DIV_OPERATOR) && (LA329_59 <= DIV_SYMBOL)) || LA329_59 == DOUBLE_QUOTED_TEXT || LA329_59 == EQUAL_OPERATOR || ((LA329_59 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_59 <= GREATER_THAN_OPERATOR)) || ((LA329_59 >= HEX_NUMBER) && (LA329_59 <= IDENTIFIER)) || LA329_59 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_59 >= LESS_OR_EQUAL_OPERATOR) && (LA329_59 <= LESS_THAN_OPERATOR)) || LA329_59 == LOGICAL_AND_OPERATOR || LA329_59 == LOGICAL_OR_OPERATOR || ((LA329_59 >= MINUS_OPERATOR) && (LA329_59 <= MOD_OPERATOR)) || ((LA329_59 >= MULT_OPERATOR) && (LA329_59 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_59 == PLUS_OPERATOR || ((LA329_59 >= SHIFT_LEFT_OPERATOR) && (LA329_59 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_59 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1483 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1483;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 59;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == REVERSE_SYMBOL))
                {
                    {
                        int LA329_60 = LA(2);
                        if ( (LA329_60 == EOF || ((LA329_60 >= ACCOUNT_SYMBOL) && (LA329_60 <= YEAR_SYMBOL)) || LA329_60 == AND_SYMBOL || LA329_60 == BETWEEN_SYMBOL || LA329_60 == COLLATE_SYMBOL || LA329_60 == CROSS_SYMBOL || LA329_60 == FOR_SYMBOL || LA329_60 == FROM_SYMBOL || ((LA329_60 >= GROUP_SYMBOL) && (LA329_60 <= HAVING_SYMBOL)) || LA329_60 == INNER_SYMBOL || LA329_60 == INTO_SYMBOL || LA329_60 == IN_SYMBOL || LA329_60 == IS_SYMBOL || ((LA329_60 >= JOIN_SYMBOL) && (LA329_60 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_60 >= LEFT_SYMBOL) && (LA329_60 <= LIMIT_SYMBOL)) || LA329_60 == LOCK_SYMBOL || ((LA329_60 >= MOD_SYMBOL) && (LA329_60 <= NATURAL_SYMBOL)) || LA329_60 == NOT_SYMBOL || LA329_60 == ON_SYMBOL || LA329_60 == OPEN_PAR_SYMBOL || ((LA329_60 >= ORDER_SYMBOL) && (LA329_60 <= OR_SYMBOL)) || LA329_60 == PROCEDURE_SYMBOL || LA329_60 == REGEXP_SYMBOL || LA329_60 == RIGHT_SYMBOL || LA329_60 == SEMICOLON_SYMBOL || LA329_60 == SET_SYMBOL || LA329_60 == SINGLE_QUOTED_TEXT || LA329_60 == STRAIGHT_JOIN_SYMBOL || LA329_60 == UNION_SYMBOL || LA329_60 == USING_SYMBOL || LA329_60 == WHERE_SYMBOL || LA329_60 == WITH_SYMBOL || LA329_60 == XOR_SYMBOL || LA329_60 == AS_SYMBOL || ((LA329_60 >= BACK_TICK_QUOTED_ID) && (LA329_60 <= BITWISE_AND_OPERATOR)) || ((LA329_60 >= BITWISE_OR_OPERATOR) && (LA329_60 <= BITWISE_XOR_OPERATOR)) || ((LA329_60 >= CLOSE_CURLY_SYMBOL) && (LA329_60 <= CLOSE_PAR_SYMBOL)) || ((LA329_60 >= COMMA_SYMBOL) && (LA329_60 <= CONCAT_PIPES_SYMBOL)) || ((LA329_60 >= DIV_OPERATOR) && (LA329_60 <= DIV_SYMBOL)) || LA329_60 == DOUBLE_QUOTED_TEXT || LA329_60 == EQUAL_OPERATOR || ((LA329_60 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_60 <= GREATER_THAN_OPERATOR)) || ((LA329_60 >= HEX_NUMBER) && (LA329_60 <= IDENTIFIER)) || LA329_60 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_60 >= LESS_OR_EQUAL_OPERATOR) && (LA329_60 <= LESS_THAN_OPERATOR)) || LA329_60 == LOGICAL_AND_OPERATOR || LA329_60 == LOGICAL_OR_OPERATOR || ((LA329_60 >= MINUS_OPERATOR) && (LA329_60 <= MOD_OPERATOR)) || ((LA329_60 >= MULT_OPERATOR) && (LA329_60 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_60 == PLUS_OPERATOR || ((LA329_60 >= SHIFT_LEFT_OPERATOR) && (LA329_60 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_60 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1538 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1538;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 60;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ROW_COUNT_SYMBOL))
                {
                    {
                        int LA329_63 = LA(2);
                        if ( (LA329_63 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1592 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1592;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_63 == EOF || ((LA329_63 >= ACCOUNT_SYMBOL) && (LA329_63 <= YEAR_SYMBOL)) || LA329_63 == AND_SYMBOL || LA329_63 == BETWEEN_SYMBOL || LA329_63 == COLLATE_SYMBOL || LA329_63 == CROSS_SYMBOL || LA329_63 == FOR_SYMBOL || LA329_63 == FROM_SYMBOL || ((LA329_63 >= GROUP_SYMBOL) && (LA329_63 <= HAVING_SYMBOL)) || LA329_63 == INNER_SYMBOL || LA329_63 == INTO_SYMBOL || LA329_63 == IN_SYMBOL || LA329_63 == IS_SYMBOL || ((LA329_63 >= JOIN_SYMBOL) && (LA329_63 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_63 >= LEFT_SYMBOL) && (LA329_63 <= LIMIT_SYMBOL)) || LA329_63 == LOCK_SYMBOL || ((LA329_63 >= MOD_SYMBOL) && (LA329_63 <= NATURAL_SYMBOL)) || LA329_63 == NOT_SYMBOL || LA329_63 == ON_SYMBOL || LA329_63 == OPEN_PAR_SYMBOL || ((LA329_63 >= ORDER_SYMBOL) && (LA329_63 <= OR_SYMBOL)) || LA329_63 == PROCEDURE_SYMBOL || LA329_63 == REGEXP_SYMBOL || LA329_63 == RIGHT_SYMBOL || LA329_63 == SEMICOLON_SYMBOL || LA329_63 == SET_SYMBOL || LA329_63 == SINGLE_QUOTED_TEXT || LA329_63 == STRAIGHT_JOIN_SYMBOL || LA329_63 == UNION_SYMBOL || LA329_63 == USING_SYMBOL || LA329_63 == WHERE_SYMBOL || LA329_63 == WITH_SYMBOL || LA329_63 == XOR_SYMBOL || LA329_63 == AS_SYMBOL || ((LA329_63 >= BACK_TICK_QUOTED_ID) && (LA329_63 <= BITWISE_AND_OPERATOR)) || ((LA329_63 >= BITWISE_OR_OPERATOR) && (LA329_63 <= BITWISE_XOR_OPERATOR)) || ((LA329_63 >= CLOSE_CURLY_SYMBOL) && (LA329_63 <= CLOSE_PAR_SYMBOL)) || ((LA329_63 >= COMMA_SYMBOL) && (LA329_63 <= CONCAT_PIPES_SYMBOL)) || ((LA329_63 >= DIV_OPERATOR) && (LA329_63 <= DIV_SYMBOL)) || LA329_63 == DOUBLE_QUOTED_TEXT || LA329_63 == EQUAL_OPERATOR || ((LA329_63 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_63 <= GREATER_THAN_OPERATOR)) || ((LA329_63 >= HEX_NUMBER) && (LA329_63 <= IDENTIFIER)) || LA329_63 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_63 >= LESS_OR_EQUAL_OPERATOR) && (LA329_63 <= LESS_THAN_OPERATOR)) || LA329_63 == LOGICAL_AND_OPERATOR || LA329_63 == LOGICAL_OR_OPERATOR || ((LA329_63 >= MINUS_OPERATOR) && (LA329_63 <= MOD_OPERATOR)) || ((LA329_63 >= MULT_OPERATOR) && (LA329_63 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_63 == PLUS_OPERATOR || ((LA329_63 >= SHIFT_LEFT_OPERATOR) && (LA329_63 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 63;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == WEEK_SYMBOL))
                {
                    {
                        int LA329_64 = LA(2);
                        if ( (LA329_64 == EOF || ((LA329_64 >= ACCOUNT_SYMBOL) && (LA329_64 <= YEAR_SYMBOL)) || LA329_64 == AND_SYMBOL || LA329_64 == BETWEEN_SYMBOL || LA329_64 == COLLATE_SYMBOL || LA329_64 == CROSS_SYMBOL || LA329_64 == FOR_SYMBOL || LA329_64 == FROM_SYMBOL || ((LA329_64 >= GROUP_SYMBOL) && (LA329_64 <= HAVING_SYMBOL)) || LA329_64 == INNER_SYMBOL || LA329_64 == INTO_SYMBOL || LA329_64 == IN_SYMBOL || LA329_64 == IS_SYMBOL || ((LA329_64 >= JOIN_SYMBOL) && (LA329_64 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_64 >= LEFT_SYMBOL) && (LA329_64 <= LIMIT_SYMBOL)) || LA329_64 == LOCK_SYMBOL || ((LA329_64 >= MOD_SYMBOL) && (LA329_64 <= NATURAL_SYMBOL)) || LA329_64 == NOT_SYMBOL || LA329_64 == ON_SYMBOL || LA329_64 == OPEN_PAR_SYMBOL || ((LA329_64 >= ORDER_SYMBOL) && (LA329_64 <= OR_SYMBOL)) || LA329_64 == PROCEDURE_SYMBOL || LA329_64 == REGEXP_SYMBOL || LA329_64 == RIGHT_SYMBOL || LA329_64 == SEMICOLON_SYMBOL || LA329_64 == SET_SYMBOL || LA329_64 == SINGLE_QUOTED_TEXT || LA329_64 == STRAIGHT_JOIN_SYMBOL || LA329_64 == UNION_SYMBOL || LA329_64 == USING_SYMBOL || LA329_64 == WHERE_SYMBOL || LA329_64 == WITH_SYMBOL || LA329_64 == XOR_SYMBOL || LA329_64 == AS_SYMBOL || ((LA329_64 >= BACK_TICK_QUOTED_ID) && (LA329_64 <= BITWISE_AND_OPERATOR)) || ((LA329_64 >= BITWISE_OR_OPERATOR) && (LA329_64 <= BITWISE_XOR_OPERATOR)) || ((LA329_64 >= CLOSE_CURLY_SYMBOL) && (LA329_64 <= CLOSE_PAR_SYMBOL)) || ((LA329_64 >= COMMA_SYMBOL) && (LA329_64 <= CONCAT_PIPES_SYMBOL)) || ((LA329_64 >= DIV_OPERATOR) && (LA329_64 <= DIV_SYMBOL)) || LA329_64 == DOUBLE_QUOTED_TEXT || LA329_64 == EQUAL_OPERATOR || ((LA329_64 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_64 <= GREATER_THAN_OPERATOR)) || ((LA329_64 >= HEX_NUMBER) && (LA329_64 <= IDENTIFIER)) || LA329_64 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_64 >= LESS_OR_EQUAL_OPERATOR) && (LA329_64 <= LESS_THAN_OPERATOR)) || LA329_64 == LOGICAL_AND_OPERATOR || LA329_64 == LOGICAL_OR_OPERATOR || ((LA329_64 >= MINUS_OPERATOR) && (LA329_64 <= MOD_OPERATOR)) || ((LA329_64 >= MULT_OPERATOR) && (LA329_64 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_64 == PLUS_OPERATOR || ((LA329_64 >= SHIFT_LEFT_OPERATOR) && (LA329_64 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_64 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1648 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1648;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 64;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == CONTAINS_SYMBOL))
                {
                    {
                        int LA329_65 = LA(2);
                        if ( (LA329_65 == EOF || ((LA329_65 >= ACCOUNT_SYMBOL) && (LA329_65 <= YEAR_SYMBOL)) || LA329_65 == AND_SYMBOL || LA329_65 == BETWEEN_SYMBOL || LA329_65 == COLLATE_SYMBOL || LA329_65 == CROSS_SYMBOL || LA329_65 == FOR_SYMBOL || LA329_65 == FROM_SYMBOL || ((LA329_65 >= GROUP_SYMBOL) && (LA329_65 <= HAVING_SYMBOL)) || LA329_65 == INNER_SYMBOL || LA329_65 == INTO_SYMBOL || LA329_65 == IN_SYMBOL || LA329_65 == IS_SYMBOL || ((LA329_65 >= JOIN_SYMBOL) && (LA329_65 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_65 >= LEFT_SYMBOL) && (LA329_65 <= LIMIT_SYMBOL)) || LA329_65 == LOCK_SYMBOL || ((LA329_65 >= MOD_SYMBOL) && (LA329_65 <= NATURAL_SYMBOL)) || LA329_65 == NOT_SYMBOL || LA329_65 == ON_SYMBOL || LA329_65 == OPEN_PAR_SYMBOL || ((LA329_65 >= ORDER_SYMBOL) && (LA329_65 <= OR_SYMBOL)) || LA329_65 == PROCEDURE_SYMBOL || LA329_65 == REGEXP_SYMBOL || LA329_65 == RIGHT_SYMBOL || LA329_65 == SEMICOLON_SYMBOL || LA329_65 == SET_SYMBOL || LA329_65 == SINGLE_QUOTED_TEXT || LA329_65 == STRAIGHT_JOIN_SYMBOL || LA329_65 == UNION_SYMBOL || LA329_65 == USING_SYMBOL || LA329_65 == WHERE_SYMBOL || LA329_65 == WITH_SYMBOL || LA329_65 == XOR_SYMBOL || LA329_65 == AS_SYMBOL || ((LA329_65 >= BACK_TICK_QUOTED_ID) && (LA329_65 <= BITWISE_AND_OPERATOR)) || ((LA329_65 >= BITWISE_OR_OPERATOR) && (LA329_65 <= BITWISE_XOR_OPERATOR)) || ((LA329_65 >= CLOSE_CURLY_SYMBOL) && (LA329_65 <= CLOSE_PAR_SYMBOL)) || ((LA329_65 >= COMMA_SYMBOL) && (LA329_65 <= CONCAT_PIPES_SYMBOL)) || ((LA329_65 >= DIV_OPERATOR) && (LA329_65 <= DIV_SYMBOL)) || LA329_65 == DOUBLE_QUOTED_TEXT || LA329_65 == EQUAL_OPERATOR || ((LA329_65 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_65 <= GREATER_THAN_OPERATOR)) || ((LA329_65 >= HEX_NUMBER) && (LA329_65 <= IDENTIFIER)) || LA329_65 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_65 >= LESS_OR_EQUAL_OPERATOR) && (LA329_65 <= LESS_THAN_OPERATOR)) || LA329_65 == LOGICAL_AND_OPERATOR || LA329_65 == LOGICAL_OR_OPERATOR || ((LA329_65 >= MINUS_OPERATOR) && (LA329_65 <= MOD_OPERATOR)) || ((LA329_65 >= MULT_OPERATOR) && (LA329_65 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_65 == PLUS_OPERATOR || ((LA329_65 >= SHIFT_LEFT_OPERATOR) && (LA329_65 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_65 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1703 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1703;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 65;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == WEIGHT_STRING_SYMBOL))
                {
                    {
                        int LA329_66 = LA(2);
                        if ( (LA329_66 == EOF || ((LA329_66 >= ACCOUNT_SYMBOL) && (LA329_66 <= YEAR_SYMBOL)) || LA329_66 == AND_SYMBOL || LA329_66 == BETWEEN_SYMBOL || LA329_66 == COLLATE_SYMBOL || LA329_66 == CROSS_SYMBOL || LA329_66 == FOR_SYMBOL || LA329_66 == FROM_SYMBOL || ((LA329_66 >= GROUP_SYMBOL) && (LA329_66 <= HAVING_SYMBOL)) || LA329_66 == INNER_SYMBOL || LA329_66 == INTO_SYMBOL || LA329_66 == IN_SYMBOL || LA329_66 == IS_SYMBOL || ((LA329_66 >= JOIN_SYMBOL) && (LA329_66 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_66 >= LEFT_SYMBOL) && (LA329_66 <= LIMIT_SYMBOL)) || LA329_66 == LOCK_SYMBOL || ((LA329_66 >= MOD_SYMBOL) && (LA329_66 <= NATURAL_SYMBOL)) || LA329_66 == NOT_SYMBOL || LA329_66 == ON_SYMBOL || LA329_66 == OPEN_PAR_SYMBOL || ((LA329_66 >= ORDER_SYMBOL) && (LA329_66 <= OR_SYMBOL)) || LA329_66 == PROCEDURE_SYMBOL || LA329_66 == REGEXP_SYMBOL || LA329_66 == RIGHT_SYMBOL || LA329_66 == SEMICOLON_SYMBOL || LA329_66 == SET_SYMBOL || LA329_66 == SINGLE_QUOTED_TEXT || LA329_66 == STRAIGHT_JOIN_SYMBOL || LA329_66 == UNION_SYMBOL || LA329_66 == USING_SYMBOL || LA329_66 == WHERE_SYMBOL || LA329_66 == WITH_SYMBOL || LA329_66 == XOR_SYMBOL || LA329_66 == AS_SYMBOL || ((LA329_66 >= BACK_TICK_QUOTED_ID) && (LA329_66 <= BITWISE_AND_OPERATOR)) || ((LA329_66 >= BITWISE_OR_OPERATOR) && (LA329_66 <= BITWISE_XOR_OPERATOR)) || ((LA329_66 >= CLOSE_CURLY_SYMBOL) && (LA329_66 <= CLOSE_PAR_SYMBOL)) || ((LA329_66 >= COMMA_SYMBOL) && (LA329_66 <= CONCAT_PIPES_SYMBOL)) || ((LA329_66 >= DIV_OPERATOR) && (LA329_66 <= DIV_SYMBOL)) || LA329_66 == DOUBLE_QUOTED_TEXT || LA329_66 == EQUAL_OPERATOR || ((LA329_66 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_66 <= GREATER_THAN_OPERATOR)) || ((LA329_66 >= HEX_NUMBER) && (LA329_66 <= IDENTIFIER)) || LA329_66 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_66 >= LESS_OR_EQUAL_OPERATOR) && (LA329_66 <= LESS_THAN_OPERATOR)) || LA329_66 == LOGICAL_AND_OPERATOR || LA329_66 == LOGICAL_OR_OPERATOR || ((LA329_66 >= MINUS_OPERATOR) && (LA329_66 <= MOD_OPERATOR)) || ((LA329_66 >= MULT_OPERATOR) && (LA329_66 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_66 == PLUS_OPERATOR || ((LA329_66 >= SHIFT_LEFT_OPERATOR) && (LA329_66 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_66 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1758 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1758;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 66;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == GEOMETRYCOLLECTION_SYMBOL))
                {
                    {
                        int LA329_67 = LA(2);
                        if ( (LA329_67 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1812 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1812;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_67 == EOF || ((LA329_67 >= ACCOUNT_SYMBOL) && (LA329_67 <= YEAR_SYMBOL)) || LA329_67 == AND_SYMBOL || LA329_67 == BETWEEN_SYMBOL || LA329_67 == COLLATE_SYMBOL || LA329_67 == CROSS_SYMBOL || LA329_67 == FOR_SYMBOL || LA329_67 == FROM_SYMBOL || ((LA329_67 >= GROUP_SYMBOL) && (LA329_67 <= HAVING_SYMBOL)) || LA329_67 == INNER_SYMBOL || LA329_67 == INTO_SYMBOL || LA329_67 == IN_SYMBOL || LA329_67 == IS_SYMBOL || ((LA329_67 >= JOIN_SYMBOL) && (LA329_67 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_67 >= LEFT_SYMBOL) && (LA329_67 <= LIMIT_SYMBOL)) || LA329_67 == LOCK_SYMBOL || ((LA329_67 >= MOD_SYMBOL) && (LA329_67 <= NATURAL_SYMBOL)) || LA329_67 == NOT_SYMBOL || LA329_67 == ON_SYMBOL || LA329_67 == OPEN_PAR_SYMBOL || ((LA329_67 >= ORDER_SYMBOL) && (LA329_67 <= OR_SYMBOL)) || LA329_67 == PROCEDURE_SYMBOL || LA329_67 == REGEXP_SYMBOL || LA329_67 == RIGHT_SYMBOL || LA329_67 == SEMICOLON_SYMBOL || LA329_67 == SET_SYMBOL || LA329_67 == SINGLE_QUOTED_TEXT || LA329_67 == STRAIGHT_JOIN_SYMBOL || LA329_67 == UNION_SYMBOL || LA329_67 == USING_SYMBOL || LA329_67 == WHERE_SYMBOL || LA329_67 == WITH_SYMBOL || LA329_67 == XOR_SYMBOL || LA329_67 == AS_SYMBOL || ((LA329_67 >= BACK_TICK_QUOTED_ID) && (LA329_67 <= BITWISE_AND_OPERATOR)) || ((LA329_67 >= BITWISE_OR_OPERATOR) && (LA329_67 <= BITWISE_XOR_OPERATOR)) || ((LA329_67 >= CLOSE_CURLY_SYMBOL) && (LA329_67 <= CLOSE_PAR_SYMBOL)) || ((LA329_67 >= COMMA_SYMBOL) && (LA329_67 <= CONCAT_PIPES_SYMBOL)) || ((LA329_67 >= DIV_OPERATOR) && (LA329_67 <= DIV_SYMBOL)) || LA329_67 == DOUBLE_QUOTED_TEXT || LA329_67 == EQUAL_OPERATOR || ((LA329_67 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_67 <= GREATER_THAN_OPERATOR)) || ((LA329_67 >= HEX_NUMBER) && (LA329_67 <= IDENTIFIER)) || LA329_67 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_67 >= LESS_OR_EQUAL_OPERATOR) && (LA329_67 <= LESS_THAN_OPERATOR)) || LA329_67 == LOGICAL_AND_OPERATOR || LA329_67 == LOGICAL_OR_OPERATOR || ((LA329_67 >= MINUS_OPERATOR) && (LA329_67 <= MOD_OPERATOR)) || ((LA329_67 >= MULT_OPERATOR) && (LA329_67 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_67 == PLUS_OPERATOR || ((LA329_67 >= SHIFT_LEFT_OPERATOR) && (LA329_67 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 67;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == LINESTRING_SYMBOL))
                {
                    {
                        int LA329_68 = LA(2);
                        if ( (LA329_68 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1867 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1867;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_68 == EOF || ((LA329_68 >= ACCOUNT_SYMBOL) && (LA329_68 <= YEAR_SYMBOL)) || LA329_68 == AND_SYMBOL || LA329_68 == BETWEEN_SYMBOL || LA329_68 == COLLATE_SYMBOL || LA329_68 == CROSS_SYMBOL || LA329_68 == FOR_SYMBOL || LA329_68 == FROM_SYMBOL || ((LA329_68 >= GROUP_SYMBOL) && (LA329_68 <= HAVING_SYMBOL)) || LA329_68 == INNER_SYMBOL || LA329_68 == INTO_SYMBOL || LA329_68 == IN_SYMBOL || LA329_68 == IS_SYMBOL || ((LA329_68 >= JOIN_SYMBOL) && (LA329_68 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_68 >= LEFT_SYMBOL) && (LA329_68 <= LIMIT_SYMBOL)) || LA329_68 == LOCK_SYMBOL || ((LA329_68 >= MOD_SYMBOL) && (LA329_68 <= NATURAL_SYMBOL)) || LA329_68 == NOT_SYMBOL || LA329_68 == ON_SYMBOL || LA329_68 == OPEN_PAR_SYMBOL || ((LA329_68 >= ORDER_SYMBOL) && (LA329_68 <= OR_SYMBOL)) || LA329_68 == PROCEDURE_SYMBOL || LA329_68 == REGEXP_SYMBOL || LA329_68 == RIGHT_SYMBOL || LA329_68 == SEMICOLON_SYMBOL || LA329_68 == SET_SYMBOL || LA329_68 == SINGLE_QUOTED_TEXT || LA329_68 == STRAIGHT_JOIN_SYMBOL || LA329_68 == UNION_SYMBOL || LA329_68 == USING_SYMBOL || LA329_68 == WHERE_SYMBOL || LA329_68 == WITH_SYMBOL || LA329_68 == XOR_SYMBOL || LA329_68 == AS_SYMBOL || ((LA329_68 >= BACK_TICK_QUOTED_ID) && (LA329_68 <= BITWISE_AND_OPERATOR)) || ((LA329_68 >= BITWISE_OR_OPERATOR) && (LA329_68 <= BITWISE_XOR_OPERATOR)) || ((LA329_68 >= CLOSE_CURLY_SYMBOL) && (LA329_68 <= CLOSE_PAR_SYMBOL)) || ((LA329_68 >= COMMA_SYMBOL) && (LA329_68 <= CONCAT_PIPES_SYMBOL)) || ((LA329_68 >= DIV_OPERATOR) && (LA329_68 <= DIV_SYMBOL)) || LA329_68 == DOUBLE_QUOTED_TEXT || LA329_68 == EQUAL_OPERATOR || ((LA329_68 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_68 <= GREATER_THAN_OPERATOR)) || ((LA329_68 >= HEX_NUMBER) && (LA329_68 <= IDENTIFIER)) || LA329_68 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_68 >= LESS_OR_EQUAL_OPERATOR) && (LA329_68 <= LESS_THAN_OPERATOR)) || LA329_68 == LOGICAL_AND_OPERATOR || LA329_68 == LOGICAL_OR_OPERATOR || ((LA329_68 >= MINUS_OPERATOR) && (LA329_68 <= MOD_OPERATOR)) || ((LA329_68 >= MULT_OPERATOR) && (LA329_68 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_68 == PLUS_OPERATOR || ((LA329_68 >= SHIFT_LEFT_OPERATOR) && (LA329_68 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 68;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MULTILINESTRING_SYMBOL))
                {
                    {
                        int LA329_69 = LA(2);
                        if ( (LA329_69 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1922 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1922;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_69 == EOF || ((LA329_69 >= ACCOUNT_SYMBOL) && (LA329_69 <= YEAR_SYMBOL)) || LA329_69 == AND_SYMBOL || LA329_69 == BETWEEN_SYMBOL || LA329_69 == COLLATE_SYMBOL || LA329_69 == CROSS_SYMBOL || LA329_69 == FOR_SYMBOL || LA329_69 == FROM_SYMBOL || ((LA329_69 >= GROUP_SYMBOL) && (LA329_69 <= HAVING_SYMBOL)) || LA329_69 == INNER_SYMBOL || LA329_69 == INTO_SYMBOL || LA329_69 == IN_SYMBOL || LA329_69 == IS_SYMBOL || ((LA329_69 >= JOIN_SYMBOL) && (LA329_69 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_69 >= LEFT_SYMBOL) && (LA329_69 <= LIMIT_SYMBOL)) || LA329_69 == LOCK_SYMBOL || ((LA329_69 >= MOD_SYMBOL) && (LA329_69 <= NATURAL_SYMBOL)) || LA329_69 == NOT_SYMBOL || LA329_69 == ON_SYMBOL || LA329_69 == OPEN_PAR_SYMBOL || ((LA329_69 >= ORDER_SYMBOL) && (LA329_69 <= OR_SYMBOL)) || LA329_69 == PROCEDURE_SYMBOL || LA329_69 == REGEXP_SYMBOL || LA329_69 == RIGHT_SYMBOL || LA329_69 == SEMICOLON_SYMBOL || LA329_69 == SET_SYMBOL || LA329_69 == SINGLE_QUOTED_TEXT || LA329_69 == STRAIGHT_JOIN_SYMBOL || LA329_69 == UNION_SYMBOL || LA329_69 == USING_SYMBOL || LA329_69 == WHERE_SYMBOL || LA329_69 == WITH_SYMBOL || LA329_69 == XOR_SYMBOL || LA329_69 == AS_SYMBOL || ((LA329_69 >= BACK_TICK_QUOTED_ID) && (LA329_69 <= BITWISE_AND_OPERATOR)) || ((LA329_69 >= BITWISE_OR_OPERATOR) && (LA329_69 <= BITWISE_XOR_OPERATOR)) || ((LA329_69 >= CLOSE_CURLY_SYMBOL) && (LA329_69 <= CLOSE_PAR_SYMBOL)) || ((LA329_69 >= COMMA_SYMBOL) && (LA329_69 <= CONCAT_PIPES_SYMBOL)) || ((LA329_69 >= DIV_OPERATOR) && (LA329_69 <= DIV_SYMBOL)) || LA329_69 == DOUBLE_QUOTED_TEXT || LA329_69 == EQUAL_OPERATOR || ((LA329_69 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_69 <= GREATER_THAN_OPERATOR)) || ((LA329_69 >= HEX_NUMBER) && (LA329_69 <= IDENTIFIER)) || LA329_69 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_69 >= LESS_OR_EQUAL_OPERATOR) && (LA329_69 <= LESS_THAN_OPERATOR)) || LA329_69 == LOGICAL_AND_OPERATOR || LA329_69 == LOGICAL_OR_OPERATOR || ((LA329_69 >= MINUS_OPERATOR) && (LA329_69 <= MOD_OPERATOR)) || ((LA329_69 >= MULT_OPERATOR) && (LA329_69 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_69 == PLUS_OPERATOR || ((LA329_69 >= SHIFT_LEFT_OPERATOR) && (LA329_69 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 69;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MULTIPOINT_SYMBOL))
                {
                    {
                        int LA329_70 = LA(2);
                        if ( (LA329_70 == DOT_SYMBOL))
                        {
                            {
                                int LA329_1977 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 1977;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_70 == EOF || ((LA329_70 >= ACCOUNT_SYMBOL) && (LA329_70 <= YEAR_SYMBOL)) || LA329_70 == AND_SYMBOL || LA329_70 == BETWEEN_SYMBOL || LA329_70 == COLLATE_SYMBOL || LA329_70 == CROSS_SYMBOL || LA329_70 == FOR_SYMBOL || LA329_70 == FROM_SYMBOL || ((LA329_70 >= GROUP_SYMBOL) && (LA329_70 <= HAVING_SYMBOL)) || LA329_70 == INNER_SYMBOL || LA329_70 == INTO_SYMBOL || LA329_70 == IN_SYMBOL || LA329_70 == IS_SYMBOL || ((LA329_70 >= JOIN_SYMBOL) && (LA329_70 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_70 >= LEFT_SYMBOL) && (LA329_70 <= LIMIT_SYMBOL)) || LA329_70 == LOCK_SYMBOL || ((LA329_70 >= MOD_SYMBOL) && (LA329_70 <= NATURAL_SYMBOL)) || LA329_70 == NOT_SYMBOL || LA329_70 == ON_SYMBOL || LA329_70 == OPEN_PAR_SYMBOL || ((LA329_70 >= ORDER_SYMBOL) && (LA329_70 <= OR_SYMBOL)) || LA329_70 == PROCEDURE_SYMBOL || LA329_70 == REGEXP_SYMBOL || LA329_70 == RIGHT_SYMBOL || LA329_70 == SEMICOLON_SYMBOL || LA329_70 == SET_SYMBOL || LA329_70 == SINGLE_QUOTED_TEXT || LA329_70 == STRAIGHT_JOIN_SYMBOL || LA329_70 == UNION_SYMBOL || LA329_70 == USING_SYMBOL || LA329_70 == WHERE_SYMBOL || LA329_70 == WITH_SYMBOL || LA329_70 == XOR_SYMBOL || LA329_70 == AS_SYMBOL || ((LA329_70 >= BACK_TICK_QUOTED_ID) && (LA329_70 <= BITWISE_AND_OPERATOR)) || ((LA329_70 >= BITWISE_OR_OPERATOR) && (LA329_70 <= BITWISE_XOR_OPERATOR)) || ((LA329_70 >= CLOSE_CURLY_SYMBOL) && (LA329_70 <= CLOSE_PAR_SYMBOL)) || ((LA329_70 >= COMMA_SYMBOL) && (LA329_70 <= CONCAT_PIPES_SYMBOL)) || ((LA329_70 >= DIV_OPERATOR) && (LA329_70 <= DIV_SYMBOL)) || LA329_70 == DOUBLE_QUOTED_TEXT || LA329_70 == EQUAL_OPERATOR || ((LA329_70 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_70 <= GREATER_THAN_OPERATOR)) || ((LA329_70 >= HEX_NUMBER) && (LA329_70 <= IDENTIFIER)) || LA329_70 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_70 >= LESS_OR_EQUAL_OPERATOR) && (LA329_70 <= LESS_THAN_OPERATOR)) || LA329_70 == LOGICAL_AND_OPERATOR || LA329_70 == LOGICAL_OR_OPERATOR || ((LA329_70 >= MINUS_OPERATOR) && (LA329_70 <= MOD_OPERATOR)) || ((LA329_70 >= MULT_OPERATOR) && (LA329_70 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_70 == PLUS_OPERATOR || ((LA329_70 >= SHIFT_LEFT_OPERATOR) && (LA329_70 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 70;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == MULTIPOLYGON_SYMBOL))
                {
                    {
                        int LA329_71 = LA(2);
                        if ( (LA329_71 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2032 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2032;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_71 == EOF || ((LA329_71 >= ACCOUNT_SYMBOL) && (LA329_71 <= YEAR_SYMBOL)) || LA329_71 == AND_SYMBOL || LA329_71 == BETWEEN_SYMBOL || LA329_71 == COLLATE_SYMBOL || LA329_71 == CROSS_SYMBOL || LA329_71 == FOR_SYMBOL || LA329_71 == FROM_SYMBOL || ((LA329_71 >= GROUP_SYMBOL) && (LA329_71 <= HAVING_SYMBOL)) || LA329_71 == INNER_SYMBOL || LA329_71 == INTO_SYMBOL || LA329_71 == IN_SYMBOL || LA329_71 == IS_SYMBOL || ((LA329_71 >= JOIN_SYMBOL) && (LA329_71 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_71 >= LEFT_SYMBOL) && (LA329_71 <= LIMIT_SYMBOL)) || LA329_71 == LOCK_SYMBOL || ((LA329_71 >= MOD_SYMBOL) && (LA329_71 <= NATURAL_SYMBOL)) || LA329_71 == NOT_SYMBOL || LA329_71 == ON_SYMBOL || LA329_71 == OPEN_PAR_SYMBOL || ((LA329_71 >= ORDER_SYMBOL) && (LA329_71 <= OR_SYMBOL)) || LA329_71 == PROCEDURE_SYMBOL || LA329_71 == REGEXP_SYMBOL || LA329_71 == RIGHT_SYMBOL || LA329_71 == SEMICOLON_SYMBOL || LA329_71 == SET_SYMBOL || LA329_71 == SINGLE_QUOTED_TEXT || LA329_71 == STRAIGHT_JOIN_SYMBOL || LA329_71 == UNION_SYMBOL || LA329_71 == USING_SYMBOL || LA329_71 == WHERE_SYMBOL || LA329_71 == WITH_SYMBOL || LA329_71 == XOR_SYMBOL || LA329_71 == AS_SYMBOL || ((LA329_71 >= BACK_TICK_QUOTED_ID) && (LA329_71 <= BITWISE_AND_OPERATOR)) || ((LA329_71 >= BITWISE_OR_OPERATOR) && (LA329_71 <= BITWISE_XOR_OPERATOR)) || ((LA329_71 >= CLOSE_CURLY_SYMBOL) && (LA329_71 <= CLOSE_PAR_SYMBOL)) || ((LA329_71 >= COMMA_SYMBOL) && (LA329_71 <= CONCAT_PIPES_SYMBOL)) || ((LA329_71 >= DIV_OPERATOR) && (LA329_71 <= DIV_SYMBOL)) || LA329_71 == DOUBLE_QUOTED_TEXT || LA329_71 == EQUAL_OPERATOR || ((LA329_71 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_71 <= GREATER_THAN_OPERATOR)) || ((LA329_71 >= HEX_NUMBER) && (LA329_71 <= IDENTIFIER)) || LA329_71 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_71 >= LESS_OR_EQUAL_OPERATOR) && (LA329_71 <= LESS_THAN_OPERATOR)) || LA329_71 == LOGICAL_AND_OPERATOR || LA329_71 == LOGICAL_OR_OPERATOR || ((LA329_71 >= MINUS_OPERATOR) && (LA329_71 <= MOD_OPERATOR)) || ((LA329_71 >= MULT_OPERATOR) && (LA329_71 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_71 == PLUS_OPERATOR || ((LA329_71 >= SHIFT_LEFT_OPERATOR) && (LA329_71 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 71;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == POINT_SYMBOL))
                {
                    {
                        int LA329_72 = LA(2);
                        if ( (LA329_72 == EOF || ((LA329_72 >= ACCOUNT_SYMBOL) && (LA329_72 <= YEAR_SYMBOL)) || LA329_72 == AND_SYMBOL || LA329_72 == BETWEEN_SYMBOL || LA329_72 == COLLATE_SYMBOL || LA329_72 == CROSS_SYMBOL || LA329_72 == FOR_SYMBOL || LA329_72 == FROM_SYMBOL || ((LA329_72 >= GROUP_SYMBOL) && (LA329_72 <= HAVING_SYMBOL)) || LA329_72 == INNER_SYMBOL || LA329_72 == INTO_SYMBOL || LA329_72 == IN_SYMBOL || LA329_72 == IS_SYMBOL || ((LA329_72 >= JOIN_SYMBOL) && (LA329_72 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_72 >= LEFT_SYMBOL) && (LA329_72 <= LIMIT_SYMBOL)) || LA329_72 == LOCK_SYMBOL || ((LA329_72 >= MOD_SYMBOL) && (LA329_72 <= NATURAL_SYMBOL)) || LA329_72 == NOT_SYMBOL || LA329_72 == ON_SYMBOL || LA329_72 == OPEN_PAR_SYMBOL || ((LA329_72 >= ORDER_SYMBOL) && (LA329_72 <= OR_SYMBOL)) || LA329_72 == PROCEDURE_SYMBOL || LA329_72 == REGEXP_SYMBOL || LA329_72 == RIGHT_SYMBOL || LA329_72 == SEMICOLON_SYMBOL || LA329_72 == SET_SYMBOL || LA329_72 == SINGLE_QUOTED_TEXT || LA329_72 == STRAIGHT_JOIN_SYMBOL || LA329_72 == UNION_SYMBOL || LA329_72 == USING_SYMBOL || LA329_72 == WHERE_SYMBOL || LA329_72 == WITH_SYMBOL || LA329_72 == XOR_SYMBOL || LA329_72 == AS_SYMBOL || ((LA329_72 >= BACK_TICK_QUOTED_ID) && (LA329_72 <= BITWISE_AND_OPERATOR)) || ((LA329_72 >= BITWISE_OR_OPERATOR) && (LA329_72 <= BITWISE_XOR_OPERATOR)) || ((LA329_72 >= CLOSE_CURLY_SYMBOL) && (LA329_72 <= CLOSE_PAR_SYMBOL)) || ((LA329_72 >= COMMA_SYMBOL) && (LA329_72 <= CONCAT_PIPES_SYMBOL)) || ((LA329_72 >= DIV_OPERATOR) && (LA329_72 <= DIV_SYMBOL)) || LA329_72 == DOUBLE_QUOTED_TEXT || LA329_72 == EQUAL_OPERATOR || ((LA329_72 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_72 <= GREATER_THAN_OPERATOR)) || ((LA329_72 >= HEX_NUMBER) && (LA329_72 <= IDENTIFIER)) || LA329_72 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_72 >= LESS_OR_EQUAL_OPERATOR) && (LA329_72 <= LESS_THAN_OPERATOR)) || LA329_72 == LOGICAL_AND_OPERATOR || LA329_72 == LOGICAL_OR_OPERATOR || ((LA329_72 >= MINUS_OPERATOR) && (LA329_72 <= MOD_OPERATOR)) || ((LA329_72 >= MULT_OPERATOR) && (LA329_72 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_72 == PLUS_OPERATOR || ((LA329_72 >= SHIFT_LEFT_OPERATOR) && (LA329_72 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_72 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2088 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2088;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 72;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == POLYGON_SYMBOL))
                {
                    {
                        int LA329_73 = LA(2);
                        if ( (LA329_73 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2142 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2142;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_73 == EOF || ((LA329_73 >= ACCOUNT_SYMBOL) && (LA329_73 <= YEAR_SYMBOL)) || LA329_73 == AND_SYMBOL || LA329_73 == BETWEEN_SYMBOL || LA329_73 == COLLATE_SYMBOL || LA329_73 == CROSS_SYMBOL || LA329_73 == FOR_SYMBOL || LA329_73 == FROM_SYMBOL || ((LA329_73 >= GROUP_SYMBOL) && (LA329_73 <= HAVING_SYMBOL)) || LA329_73 == INNER_SYMBOL || LA329_73 == INTO_SYMBOL || LA329_73 == IN_SYMBOL || LA329_73 == IS_SYMBOL || ((LA329_73 >= JOIN_SYMBOL) && (LA329_73 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_73 >= LEFT_SYMBOL) && (LA329_73 <= LIMIT_SYMBOL)) || LA329_73 == LOCK_SYMBOL || ((LA329_73 >= MOD_SYMBOL) && (LA329_73 <= NATURAL_SYMBOL)) || LA329_73 == NOT_SYMBOL || LA329_73 == ON_SYMBOL || LA329_73 == OPEN_PAR_SYMBOL || ((LA329_73 >= ORDER_SYMBOL) && (LA329_73 <= OR_SYMBOL)) || LA329_73 == PROCEDURE_SYMBOL || LA329_73 == REGEXP_SYMBOL || LA329_73 == RIGHT_SYMBOL || LA329_73 == SEMICOLON_SYMBOL || LA329_73 == SET_SYMBOL || LA329_73 == SINGLE_QUOTED_TEXT || LA329_73 == STRAIGHT_JOIN_SYMBOL || LA329_73 == UNION_SYMBOL || LA329_73 == USING_SYMBOL || LA329_73 == WHERE_SYMBOL || LA329_73 == WITH_SYMBOL || LA329_73 == XOR_SYMBOL || LA329_73 == AS_SYMBOL || ((LA329_73 >= BACK_TICK_QUOTED_ID) && (LA329_73 <= BITWISE_AND_OPERATOR)) || ((LA329_73 >= BITWISE_OR_OPERATOR) && (LA329_73 <= BITWISE_XOR_OPERATOR)) || ((LA329_73 >= CLOSE_CURLY_SYMBOL) && (LA329_73 <= CLOSE_PAR_SYMBOL)) || ((LA329_73 >= COMMA_SYMBOL) && (LA329_73 <= CONCAT_PIPES_SYMBOL)) || ((LA329_73 >= DIV_OPERATOR) && (LA329_73 <= DIV_SYMBOL)) || LA329_73 == DOUBLE_QUOTED_TEXT || LA329_73 == EQUAL_OPERATOR || ((LA329_73 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_73 <= GREATER_THAN_OPERATOR)) || ((LA329_73 >= HEX_NUMBER) && (LA329_73 <= IDENTIFIER)) || LA329_73 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_73 >= LESS_OR_EQUAL_OPERATOR) && (LA329_73 <= LESS_THAN_OPERATOR)) || LA329_73 == LOGICAL_AND_OPERATOR || LA329_73 == LOGICAL_OR_OPERATOR || ((LA329_73 >= MINUS_OPERATOR) && (LA329_73 <= MOD_OPERATOR)) || ((LA329_73 >= MULT_OPERATOR) && (LA329_73 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_73 == PLUS_OPERATOR || ((LA329_73 >= SHIFT_LEFT_OPERATOR) && (LA329_73 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 73;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == AVG_SYMBOL))
                {
                    {
                        int LA329_74 = LA(2);
                        if ( (LA329_74 == EOF || ((LA329_74 >= ACCOUNT_SYMBOL) && (LA329_74 <= YEAR_SYMBOL)) || LA329_74 == AND_SYMBOL || LA329_74 == BETWEEN_SYMBOL || LA329_74 == COLLATE_SYMBOL || LA329_74 == CROSS_SYMBOL || LA329_74 == FOR_SYMBOL || LA329_74 == FROM_SYMBOL || ((LA329_74 >= GROUP_SYMBOL) && (LA329_74 <= HAVING_SYMBOL)) || LA329_74 == INNER_SYMBOL || LA329_74 == INTO_SYMBOL || LA329_74 == IN_SYMBOL || LA329_74 == IS_SYMBOL || ((LA329_74 >= JOIN_SYMBOL) && (LA329_74 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_74 >= LEFT_SYMBOL) && (LA329_74 <= LIMIT_SYMBOL)) || LA329_74 == LOCK_SYMBOL || ((LA329_74 >= MOD_SYMBOL) && (LA329_74 <= NATURAL_SYMBOL)) || LA329_74 == NOT_SYMBOL || LA329_74 == ON_SYMBOL || LA329_74 == OPEN_PAR_SYMBOL || ((LA329_74 >= ORDER_SYMBOL) && (LA329_74 <= OR_SYMBOL)) || LA329_74 == PROCEDURE_SYMBOL || LA329_74 == REGEXP_SYMBOL || LA329_74 == RIGHT_SYMBOL || LA329_74 == SEMICOLON_SYMBOL || LA329_74 == SET_SYMBOL || LA329_74 == SINGLE_QUOTED_TEXT || LA329_74 == STRAIGHT_JOIN_SYMBOL || LA329_74 == UNION_SYMBOL || LA329_74 == USING_SYMBOL || LA329_74 == WHERE_SYMBOL || LA329_74 == WITH_SYMBOL || LA329_74 == XOR_SYMBOL || LA329_74 == AS_SYMBOL || ((LA329_74 >= BACK_TICK_QUOTED_ID) && (LA329_74 <= BITWISE_AND_OPERATOR)) || ((LA329_74 >= BITWISE_OR_OPERATOR) && (LA329_74 <= BITWISE_XOR_OPERATOR)) || ((LA329_74 >= CLOSE_CURLY_SYMBOL) && (LA329_74 <= CLOSE_PAR_SYMBOL)) || ((LA329_74 >= COMMA_SYMBOL) && (LA329_74 <= CONCAT_PIPES_SYMBOL)) || ((LA329_74 >= DIV_OPERATOR) && (LA329_74 <= DIV_SYMBOL)) || LA329_74 == DOUBLE_QUOTED_TEXT || LA329_74 == EQUAL_OPERATOR || ((LA329_74 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_74 <= GREATER_THAN_OPERATOR)) || ((LA329_74 >= HEX_NUMBER) && (LA329_74 <= IDENTIFIER)) || LA329_74 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_74 >= LESS_OR_EQUAL_OPERATOR) && (LA329_74 <= LESS_THAN_OPERATOR)) || LA329_74 == LOGICAL_AND_OPERATOR || LA329_74 == LOGICAL_OR_OPERATOR || ((LA329_74 >= MINUS_OPERATOR) && (LA329_74 <= MOD_OPERATOR)) || ((LA329_74 >= MULT_OPERATOR) && (LA329_74 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_74 == PLUS_OPERATOR || ((LA329_74 >= SHIFT_LEFT_OPERATOR) && (LA329_74 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_74 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2198 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2198;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 74;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ACCOUNT_SYMBOL || ((LA329_0 >= ALWAYS_SYMBOL) && (LA329_0 <= CACHE_SYMBOL)) || ((LA329_0 >= CHECKSUM_SYMBOL) && (LA329_0 <= COMMIT_SYMBOL)) || ((LA329_0 >= DEALLOCATE_SYMBOL) && (LA329_0 <= FOLLOWS_SYMBOL)) || ((LA329_0 >= GROUP_REPLICATION_SYMBOL) && (LA329_0 <= STOP_SYMBOL)) || ((LA329_0 >= UNICODE_SYMBOL) && (LA329_0 <= XA_SYMBOL)) || LA329_0 == SHUTDOWN_SYMBOL))
                {
                    {
                        int LA329_75 = LA(2);
                        if ( (LA329_75 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2252 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2252;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_75 == EOF || ((LA329_75 >= ACCOUNT_SYMBOL) && (LA329_75 <= YEAR_SYMBOL)) || LA329_75 == AND_SYMBOL || LA329_75 == BETWEEN_SYMBOL || LA329_75 == COLLATE_SYMBOL || LA329_75 == CROSS_SYMBOL || LA329_75 == FOR_SYMBOL || LA329_75 == FROM_SYMBOL || ((LA329_75 >= GROUP_SYMBOL) && (LA329_75 <= HAVING_SYMBOL)) || LA329_75 == INNER_SYMBOL || LA329_75 == INTO_SYMBOL || LA329_75 == IN_SYMBOL || LA329_75 == IS_SYMBOL || ((LA329_75 >= JOIN_SYMBOL) && (LA329_75 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_75 >= LEFT_SYMBOL) && (LA329_75 <= LIMIT_SYMBOL)) || LA329_75 == LOCK_SYMBOL || ((LA329_75 >= MOD_SYMBOL) && (LA329_75 <= NATURAL_SYMBOL)) || LA329_75 == NOT_SYMBOL || LA329_75 == ON_SYMBOL || ((LA329_75 >= ORDER_SYMBOL) && (LA329_75 <= OR_SYMBOL)) || LA329_75 == PROCEDURE_SYMBOL || LA329_75 == REGEXP_SYMBOL || LA329_75 == RIGHT_SYMBOL || LA329_75 == SEMICOLON_SYMBOL || LA329_75 == SET_SYMBOL || LA329_75 == SINGLE_QUOTED_TEXT || LA329_75 == STRAIGHT_JOIN_SYMBOL || LA329_75 == UNION_SYMBOL || LA329_75 == USING_SYMBOL || LA329_75 == WHERE_SYMBOL || LA329_75 == WITH_SYMBOL || LA329_75 == XOR_SYMBOL || LA329_75 == AS_SYMBOL || ((LA329_75 >= BACK_TICK_QUOTED_ID) && (LA329_75 <= BITWISE_AND_OPERATOR)) || ((LA329_75 >= BITWISE_OR_OPERATOR) && (LA329_75 <= BITWISE_XOR_OPERATOR)) || ((LA329_75 >= CLOSE_CURLY_SYMBOL) && (LA329_75 <= CLOSE_PAR_SYMBOL)) || ((LA329_75 >= COMMA_SYMBOL) && (LA329_75 <= CONCAT_PIPES_SYMBOL)) || ((LA329_75 >= DIV_OPERATOR) && (LA329_75 <= DIV_SYMBOL)) || LA329_75 == DOUBLE_QUOTED_TEXT || LA329_75 == EQUAL_OPERATOR || ((LA329_75 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_75 <= GREATER_THAN_OPERATOR)) || ((LA329_75 >= HEX_NUMBER) && (LA329_75 <= IDENTIFIER)) || LA329_75 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_75 >= LESS_OR_EQUAL_OPERATOR) && (LA329_75 <= LESS_THAN_OPERATOR)) || LA329_75 == LOGICAL_AND_OPERATOR || LA329_75 == LOGICAL_OR_OPERATOR || ((LA329_75 >= MINUS_OPERATOR) && (LA329_75 <= MOD_OPERATOR)) || ((LA329_75 >= MULT_OPERATOR) && (LA329_75 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_75 == PLUS_OPERATOR || ((LA329_75 >= SHIFT_LEFT_OPERATOR) && (LA329_75 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 75;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ROW_SYMBOL))
                {
                    {
                        int LA329_76 = LA(2);
                        if ( (LA329_76 == EOF || ((LA329_76 >= ACCOUNT_SYMBOL) && (LA329_76 <= YEAR_SYMBOL)) || LA329_76 == AND_SYMBOL || LA329_76 == BETWEEN_SYMBOL || LA329_76 == COLLATE_SYMBOL || LA329_76 == CROSS_SYMBOL || LA329_76 == FOR_SYMBOL || LA329_76 == FROM_SYMBOL || ((LA329_76 >= GROUP_SYMBOL) && (LA329_76 <= HAVING_SYMBOL)) || LA329_76 == INNER_SYMBOL || LA329_76 == INTO_SYMBOL || LA329_76 == IN_SYMBOL || LA329_76 == IS_SYMBOL || ((LA329_76 >= JOIN_SYMBOL) && (LA329_76 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_76 >= LEFT_SYMBOL) && (LA329_76 <= LIMIT_SYMBOL)) || LA329_76 == LOCK_SYMBOL || ((LA329_76 >= MOD_SYMBOL) && (LA329_76 <= NATURAL_SYMBOL)) || LA329_76 == NOT_SYMBOL || LA329_76 == ON_SYMBOL || LA329_76 == OPEN_PAR_SYMBOL || ((LA329_76 >= ORDER_SYMBOL) && (LA329_76 <= OR_SYMBOL)) || LA329_76 == PROCEDURE_SYMBOL || LA329_76 == REGEXP_SYMBOL || LA329_76 == RIGHT_SYMBOL || LA329_76 == SEMICOLON_SYMBOL || LA329_76 == SET_SYMBOL || LA329_76 == SINGLE_QUOTED_TEXT || LA329_76 == STRAIGHT_JOIN_SYMBOL || LA329_76 == UNION_SYMBOL || LA329_76 == USING_SYMBOL || LA329_76 == WHERE_SYMBOL || LA329_76 == WITH_SYMBOL || LA329_76 == XOR_SYMBOL || LA329_76 == AS_SYMBOL || ((LA329_76 >= BACK_TICK_QUOTED_ID) && (LA329_76 <= BITWISE_AND_OPERATOR)) || ((LA329_76 >= BITWISE_OR_OPERATOR) && (LA329_76 <= BITWISE_XOR_OPERATOR)) || ((LA329_76 >= CLOSE_CURLY_SYMBOL) && (LA329_76 <= CLOSE_PAR_SYMBOL)) || ((LA329_76 >= COMMA_SYMBOL) && (LA329_76 <= CONCAT_PIPES_SYMBOL)) || ((LA329_76 >= DIV_OPERATOR) && (LA329_76 <= DIV_SYMBOL)) || LA329_76 == DOUBLE_QUOTED_TEXT || LA329_76 == EQUAL_OPERATOR || ((LA329_76 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_76 <= GREATER_THAN_OPERATOR)) || ((LA329_76 >= HEX_NUMBER) && (LA329_76 <= IDENTIFIER)) || LA329_76 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_76 >= LESS_OR_EQUAL_OPERATOR) && (LA329_76 <= LESS_THAN_OPERATOR)) || LA329_76 == LOGICAL_AND_OPERATOR || LA329_76 == LOGICAL_OR_OPERATOR || ((LA329_76 >= MINUS_OPERATOR) && (LA329_76 <= MOD_OPERATOR)) || ((LA329_76 >= MULT_OPERATOR) && (LA329_76 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_76 == PLUS_OPERATOR || ((LA329_76 >= SHIFT_LEFT_OPERATOR) && (LA329_76 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else if ( (LA329_76 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2307 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2307;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 76;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else if ( (LA329_0 == ACTION_SYMBOL || ((LA329_0 >= AFTER_SYMBOL) && (LA329_0 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA329_0 >= BINLOG_SYMBOL) && (LA329_0 <= CLASS_ORIGIN_SYMBOL)) || LA329_0 == CODE_SYMBOL || ((LA329_0 >= COLUMN_NAME_SYMBOL) && (LA329_0 <= DATETIME_SYMBOL)) || ((LA329_0 >= DEFAULT_AUTH_SYMBOL) && (LA329_0 <= GEOMETRY_SYMBOL)) || ((LA329_0 >= GRANTS_SYMBOL) && (LA329_0 <= HOSTS_SYMBOL)) || ((LA329_0 >= IDENTIFIED_SYMBOL) && (LA329_0 <= LEVEL_SYMBOL)) || ((LA329_0 >= LIST_SYMBOL) && (LA329_0 <= MESSAGE_TEXT_SYMBOL)) || LA329_0 == MIGRATE_SYMBOL || ((LA329_0 >= MIN_ROWS_SYMBOL) && (LA329_0 <= MODE_SYMBOL)) || ((LA329_0 >= MUTEX_SYMBOL) && (LA329_0 <= OFFSET_SYMBOL)) || ((LA329_0 >= ONE_SHOT_SYMBOL) && (LA329_0 <= PARTITIONS_SYMBOL)) || ((LA329_0 >= PHASE_SYMBOL) && (LA329_0 <= PLUGINS_SYMBOL)) || ((LA329_0 >= PRESERVE_SYMBOL) && (LA329_0 <= PROXY_SYMBOL)) || ((LA329_0 >= QUERY_SYMBOL) && (LA329_0 <= RETURNS_SYMBOL)) || ((LA329_0 >= ROLLUP_SYMBOL) && (LA329_0 <= ROWS_SYMBOL)) || LA329_0 == ROW_FORMAT_SYMBOL || ((LA329_0 >= RTREE_SYMBOL) && (LA329_0 <= SCHEMA_NAME_SYMBOL)) || ((LA329_0 >= SERIAL_SYMBOL) && (LA329_0 <= SHARE_SYMBOL)) || ((LA329_0 >= SLOW_SYMBOL) && (LA329_0 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA329_0 >= SUBJECT_SYMBOL) && (LA329_0 <= TRIGGERS_SYMBOL)) || ((LA329_0 >= TYPES_SYMBOL) && (LA329_0 <= USER_RESOURCES_SYMBOL)) || ((LA329_0 >= USE_FRM_SYMBOL) && (LA329_0 <= WAIT_SYMBOL)) || LA329_0 == WORK_SYMBOL || ((LA329_0 >= X509_SYMBOL) && (LA329_0 <= XML_SYMBOL))))
                {
                    {
                        int LA329_96 = LA(2);
                        if ( (LA329_96 == DOT_SYMBOL))
                        {
                            {
                                int LA329_2361 = LA(3);
                                if ( (synpred1_MySQLSimpleParser(ctx)))
                                {
                                    alt329=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt329=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 329;
                                    EXCEPTION->state        = 2361;


                                    goto ruleselect_itemEx;

                                }
                            }
                        }
                        else if ( (LA329_96 == EOF || ((LA329_96 >= ACCOUNT_SYMBOL) && (LA329_96 <= YEAR_SYMBOL)) || LA329_96 == AND_SYMBOL || LA329_96 == BETWEEN_SYMBOL || LA329_96 == COLLATE_SYMBOL || LA329_96 == CROSS_SYMBOL || LA329_96 == FOR_SYMBOL || LA329_96 == FROM_SYMBOL || ((LA329_96 >= GROUP_SYMBOL) && (LA329_96 <= HAVING_SYMBOL)) || LA329_96 == INNER_SYMBOL || LA329_96 == INTO_SYMBOL || LA329_96 == IN_SYMBOL || LA329_96 == IS_SYMBOL || ((LA329_96 >= JOIN_SYMBOL) && (LA329_96 <= JSON_SEPARATOR_SYMBOL)) || ((LA329_96 >= LEFT_SYMBOL) && (LA329_96 <= LIMIT_SYMBOL)) || LA329_96 == LOCK_SYMBOL || ((LA329_96 >= MOD_SYMBOL) && (LA329_96 <= NATURAL_SYMBOL)) || LA329_96 == NOT_SYMBOL || LA329_96 == ON_SYMBOL || ((LA329_96 >= ORDER_SYMBOL) && (LA329_96 <= OR_SYMBOL)) || LA329_96 == PROCEDURE_SYMBOL || LA329_96 == REGEXP_SYMBOL || LA329_96 == RIGHT_SYMBOL || LA329_96 == SEMICOLON_SYMBOL || LA329_96 == SET_SYMBOL || LA329_96 == SINGLE_QUOTED_TEXT || LA329_96 == STRAIGHT_JOIN_SYMBOL || LA329_96 == UNION_SYMBOL || LA329_96 == USING_SYMBOL || LA329_96 == WHERE_SYMBOL || LA329_96 == WITH_SYMBOL || LA329_96 == XOR_SYMBOL || LA329_96 == AS_SYMBOL || ((LA329_96 >= BACK_TICK_QUOTED_ID) && (LA329_96 <= BITWISE_AND_OPERATOR)) || ((LA329_96 >= BITWISE_OR_OPERATOR) && (LA329_96 <= BITWISE_XOR_OPERATOR)) || ((LA329_96 >= CLOSE_CURLY_SYMBOL) && (LA329_96 <= CLOSE_PAR_SYMBOL)) || ((LA329_96 >= COMMA_SYMBOL) && (LA329_96 <= CONCAT_PIPES_SYMBOL)) || ((LA329_96 >= DIV_OPERATOR) && (LA329_96 <= DIV_SYMBOL)) || LA329_96 == DOUBLE_QUOTED_TEXT || LA329_96 == EQUAL_OPERATOR || ((LA329_96 >= GREATER_OR_EQUAL_OPERATOR) && (LA329_96 <= GREATER_THAN_OPERATOR)) || ((LA329_96 >= HEX_NUMBER) && (LA329_96 <= IDENTIFIER)) || LA329_96 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA329_96 >= LESS_OR_EQUAL_OPERATOR) && (LA329_96 <= LESS_THAN_OPERATOR)) || LA329_96 == LOGICAL_AND_OPERATOR || LA329_96 == LOGICAL_OR_OPERATOR || ((LA329_96 >= MINUS_OPERATOR) && (LA329_96 <= MOD_OPERATOR)) || ((LA329_96 >= MULT_OPERATOR) && (LA329_96 <= NULL_SAFE_EQUAL_OPERATOR)) || LA329_96 == PLUS_OPERATOR || ((LA329_96 >= SHIFT_LEFT_OPERATOR) && (LA329_96 <= SHIFT_RIGHT_OPERATOR))))
                        {
                            alt329=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 329;
                            EXCEPTION->state        = 96;


                            goto ruleselect_itemEx;

                        }
                    }
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 329;
                    EXCEPTION->state        = 0;


                    goto ruleselect_itemEx;

                }
            }
            switch (alt329)
            {
        	case 1:
        	    // MySQLSimpleParser.g:864:2: ( table_wild )=> table_wild
        	    {
        	        FOLLOWPUSH(FOLLOW_table_wild_in_select_item4878);
        	        table_wild(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_itemEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:865:4: expression ( select_alias )?
        	    {
        	        FOLLOWPUSH(FOLLOW_expression_in_select_item4883);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_itemEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:865:15: ( select_alias )?
        	        {
        	            int alt328=2;
        	            {
        	                int LA328_0 = LA(1);
        	                if ( (((LA328_0 >= ACCOUNT_SYMBOL) && (LA328_0 <= YEAR_SYMBOL)) || LA328_0 == SINGLE_QUOTED_TEXT || LA328_0 == AS_SYMBOL || ((LA328_0 >= BACK_TICK_QUOTED_ID) && (LA328_0 <= BIN_NUMBER)) || LA328_0 == DOUBLE_QUOTED_TEXT || ((LA328_0 >= HEX_NUMBER) && (LA328_0 <= IDENTIFIER))))
        	                {
        	                    alt328=1;
        	                }
        	            }
        	            switch (alt328)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:865:15: select_alias
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_select_alias_in_select_item4885);
        	        	        select_alias(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleselect_itemEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_itemEx; /* Prevent compiler warnings */
    ruleselect_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_item */

/**
 * $ANTLR start select_alias
 * MySQLSimpleParser.g:868:1: select_alias : ( AS_SYMBOL )? ( identifier | text_string ) ;
 */
static void
select_alias(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:868:13: ( ( AS_SYMBOL )? ( identifier | text_string ) )
        // MySQLSimpleParser.g:869:2: ( AS_SYMBOL )? ( identifier | text_string )
        {
            // MySQLSimpleParser.g:869:2: ( AS_SYMBOL )?
            {
                int alt330=2;
                switch ( LA(1) )
                {
                    case AS_SYMBOL:
                    	{
                    		alt330=1;
                    	}
                        break;
                }

                switch (alt330)
                {
            	case 1:
            	    // MySQLSimpleParser.g:869:2: AS_SYMBOL
            	    {
            	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_select_alias4895);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_aliasEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:869:13: ( identifier | text_string )
            {
                int alt331=2;
                {
                    int LA331_0 = LA(1);
                    if ( (((LA331_0 >= ACCOUNT_SYMBOL) && (LA331_0 <= YEAR_SYMBOL)) || LA331_0 == BACK_TICK_QUOTED_ID || LA331_0 == DOUBLE_QUOTED_TEXT || LA331_0 == IDENTIFIER))
                    {
                        alt331=1;
                    }
                    else if ( (LA331_0 == SINGLE_QUOTED_TEXT || LA331_0 == BIN_NUMBER || LA331_0 == HEX_NUMBER))
                    {
                        alt331=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 331;
                        EXCEPTION->state        = 0;


                        goto ruleselect_aliasEx;

                    }
                }
                switch (alt331)
                {
            	case 1:
            	    // MySQLSimpleParser.g:869:14: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_select_alias4899);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_aliasEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:869:27: text_string
            	    {
            	        FOLLOWPUSH(FOLLOW_text_string_in_select_alias4903);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_aliasEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_aliasEx; /* Prevent compiler warnings */
    ruleselect_aliasEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_alias */

/**
 * $ANTLR start limit_clause
 * MySQLSimpleParser.g:872:1: limit_clause : LIMIT_SYMBOL limit_options ;
 */
static void
limit_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:872:13: ( LIMIT_SYMBOL limit_options )
        // MySQLSimpleParser.g:873:2: LIMIT_SYMBOL limit_options
        {
             MATCHT(LIMIT_SYMBOL, &FOLLOW_LIMIT_SYMBOL_in_limit_clause4923);
            if  (HASEXCEPTION())
            {
                goto rulelimit_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_limit_options_in_limit_clause4925);
            limit_options(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelimit_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelimit_clauseEx; /* Prevent compiler warnings */
    rulelimit_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end limit_clause */

/**
 * $ANTLR start simple_limit_clause
 * MySQLSimpleParser.g:876:1: simple_limit_clause : LIMIT_SYMBOL limit_option ;
 */
static void
simple_limit_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:876:20: ( LIMIT_SYMBOL limit_option )
        // MySQLSimpleParser.g:877:2: LIMIT_SYMBOL limit_option
        {
             MATCHT(LIMIT_SYMBOL, &FOLLOW_LIMIT_SYMBOL_in_simple_limit_clause4934);
            if  (HASEXCEPTION())
            {
                goto rulesimple_limit_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_limit_option_in_simple_limit_clause4936);
            limit_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesimple_limit_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesimple_limit_clauseEx; /* Prevent compiler warnings */
    rulesimple_limit_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end simple_limit_clause */

/**
 * $ANTLR start limit_options
 * MySQLSimpleParser.g:880:1: limit_options : limit_option ( options {greedy=true; } : ( COMMA_SYMBOL | OFFSET_SYMBOL ) limit_option )? ;
 */
static void
limit_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:880:14: ( limit_option ( options {greedy=true; } : ( COMMA_SYMBOL | OFFSET_SYMBOL ) limit_option )? )
        // MySQLSimpleParser.g:881:2: limit_option ( options {greedy=true; } : ( COMMA_SYMBOL | OFFSET_SYMBOL ) limit_option )?
        {
            FOLLOWPUSH(FOLLOW_limit_option_in_limit_options4945);
            limit_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelimit_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:881:15: ( options {greedy=true; } : ( COMMA_SYMBOL | OFFSET_SYMBOL ) limit_option )?
            {
                int alt332=2;
                switch ( LA(1) )
                {
                    case COMMA_SYMBOL:
                    	{
                    		{
                    		    int LA332_1 = LA(2);
                    		    if ( (LA332_1 == IDENTIFIER))
                    		    {
                    		        alt332=1;
                    		    }
                    		    else if ( (LA332_1 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        alt332=1;
                    		    }
                    		    else if ( (LA332_1 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        alt332=1;
                    		    }
                    		    else if ( (((LA332_1 >= ACTION_SYMBOL) && (LA332_1 <= SHARE_SYMBOL)) || ((LA332_1 >= SLOW_SYMBOL) && (LA332_1 <= YEAR_SYMBOL))))
                    		    {
                    		        alt332=1;
                    		    }
                    		    else if ( (((LA332_1 >= ACCOUNT_SYMBOL) && (LA332_1 <= XA_SYMBOL)) || LA332_1 == SHUTDOWN_SYMBOL || LA332_1 == PARAM_MARKER || LA332_1 == INT_NUMBER || LA332_1 == LONG_NUMBER || LA332_1 == ULONGLONG_NUMBER))
                    		    {
                    		        alt332=1;
                    		    }
                    		}
                    	}
                        break;
                    case OFFSET_SYMBOL:
                    	{
                    		alt332=1;
                    	}
                        break;
                }

                switch (alt332)
                {
            	case 1:
            	    // MySQLSimpleParser.g:881:45: ( COMMA_SYMBOL | OFFSET_SYMBOL ) limit_option
            	    {
            	        if ( LA(1) == OFFSET_SYMBOL || LA(1) == COMMA_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulelimit_optionsEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_limit_option_in_limit_options4969);
            	        limit_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelimit_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulelimit_optionsEx; /* Prevent compiler warnings */
    rulelimit_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end limit_options */

/**
 * $ANTLR start limit_option
 * MySQLSimpleParser.g:884:1: limit_option : ( identifier | PARAM_MARKER | ULONGLONG_NUMBER | LONG_NUMBER | INT_NUMBER );
 */
static void
limit_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:884:13: ( identifier | PARAM_MARKER | ULONGLONG_NUMBER | LONG_NUMBER | INT_NUMBER )

            ANTLR3_UINT32 alt333;

            alt333=5;

            {
                int LA333_0 = LA(1);
                if ( (((LA333_0 >= ACCOUNT_SYMBOL) && (LA333_0 <= YEAR_SYMBOL)) || LA333_0 == BACK_TICK_QUOTED_ID || LA333_0 == DOUBLE_QUOTED_TEXT || LA333_0 == IDENTIFIER))
                {
                    alt333=1;
                }
                else if ( (LA333_0 == PARAM_MARKER))
                {
                    alt333=2;
                }
                else if ( (LA333_0 == ULONGLONG_NUMBER))
                {
                    alt333=3;
                }
                else if ( (LA333_0 == LONG_NUMBER))
                {
                    alt333=4;
                }
                else if ( (LA333_0 == INT_NUMBER))
                {
                    alt333=5;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 333;
                    EXCEPTION->state        = 0;


                    goto rulelimit_optionEx;

                }
            }
            switch (alt333)
            {
        	case 1:
        	    // MySQLSimpleParser.g:885:2: identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_limit_option4980);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelimit_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:886:4: PARAM_MARKER
        	    {
        	         MATCHT(PARAM_MARKER, &FOLLOW_PARAM_MARKER_in_limit_option4985);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelimit_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:887:4: ULONGLONG_NUMBER
        	    {
        	         MATCHT(ULONGLONG_NUMBER, &FOLLOW_ULONGLONG_NUMBER_in_limit_option4990);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelimit_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:888:4: LONG_NUMBER
        	    {
        	         MATCHT(LONG_NUMBER, &FOLLOW_LONG_NUMBER_in_limit_option4995);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelimit_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:889:4: INT_NUMBER
        	    {
        	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_limit_option5000);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelimit_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelimit_optionEx; /* Prevent compiler warnings */
    rulelimit_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end limit_option */

/**
 * $ANTLR start into_clause
 * MySQLSimpleParser.g:892:1: into_clause : INTO_SYMBOL ( OUTFILE_SYMBOL string_literal ( charset_clause )? ( fields_clause )? ( lines_clause )? | DUMPFILE_SYMBOL string_literal | ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )* ) ;
 */
static void
into_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:892:12: ( INTO_SYMBOL ( OUTFILE_SYMBOL string_literal ( charset_clause )? ( fields_clause )? ( lines_clause )? | DUMPFILE_SYMBOL string_literal | ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )* ) )
        // MySQLSimpleParser.g:893:2: INTO_SYMBOL ( OUTFILE_SYMBOL string_literal ( charset_clause )? ( fields_clause )? ( lines_clause )? | DUMPFILE_SYMBOL string_literal | ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )* )
        {
             MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_into_clause5009);
            if  (HASEXCEPTION())
            {
                goto ruleinto_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:894:2: ( OUTFILE_SYMBOL string_literal ( charset_clause )? ( fields_clause )? ( lines_clause )? | DUMPFILE_SYMBOL string_literal | ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )* )
            {
                int alt342=3;
                {
                    int LA342_0 = LA(1);
                    if ( (LA342_0 == OUTFILE_SYMBOL))
                    {
                        alt342=1;
                    }
                    else if ( (LA342_0 == DUMPFILE_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case NCHAR_TEXT:
                        case SINGLE_QUOTED_TEXT:
                        case DOUBLE_QUOTED_TEXT:
                        case UNDERSCORE_CHARSET:
                        	{
                        		alt342=2;
                        	}
                            break;
                        case EOF:
                        case FOR_SYMBOL:
                        case FROM_SYMBOL:
                        case LOCK_SYMBOL:
                        case ON_SYMBOL:
                        case SEMICOLON_SYMBOL:
                        case UNION_SYMBOL:
                        case WITH_SYMBOL:
                        case CLOSE_PAR_SYMBOL:
                        case COMMA_SYMBOL:
                        	{
                        		alt342=3;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 342;
                            EXCEPTION->state        = 2;


                            goto ruleinto_clauseEx;

                        }

                    }
                    else if ( (((LA342_0 >= ACCOUNT_SYMBOL) && (LA342_0 <= DISK_SYMBOL)) || ((LA342_0 >= DUPLICATE_SYMBOL) && (LA342_0 <= YEAR_SYMBOL)) || LA342_0 == SINGLE_QUOTED_TEXT || ((LA342_0 >= AT_SIGN_SYMBOL) && (LA342_0 <= AT_TEXT_SUFFIX)) || LA342_0 == BACK_TICK_QUOTED_ID || LA342_0 == DOUBLE_QUOTED_TEXT || LA342_0 == IDENTIFIER))
                    {
                        alt342=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 342;
                        EXCEPTION->state        = 0;


                        goto ruleinto_clauseEx;

                    }
                }
                switch (alt342)
                {
            	case 1:
            	    // MySQLSimpleParser.g:895:3: OUTFILE_SYMBOL string_literal ( charset_clause )? ( fields_clause )? ( lines_clause )?
            	    {
            	         MATCHT(OUTFILE_SYMBOL, &FOLLOW_OUTFILE_SYMBOL_in_into_clause5016);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinto_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_into_clause5018);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinto_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:895:33: ( charset_clause )?
            	        {
            	            int alt334=2;
            	            switch ( LA(1) )
            	            {
            	                case CHARSET_SYMBOL:
            	                case CHAR_SYMBOL:
            	                	{
            	                		alt334=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt334)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:895:33: charset_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_charset_clause_in_into_clause5020);
            	        	        charset_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:895:49: ( fields_clause )?
            	        {
            	            int alt335=2;
            	            switch ( LA(1) )
            	            {
            	                case COLUMNS_SYMBOL:
            	                	{
            	                		alt335=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt335)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:895:49: fields_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_fields_clause_in_into_clause5023);
            	        	        fields_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:895:64: ( lines_clause )?
            	        {
            	            int alt336=2;
            	            switch ( LA(1) )
            	            {
            	                case LINES_SYMBOL:
            	                	{
            	                		alt336=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt336)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:895:64: lines_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_lines_clause_in_into_clause5026);
            	        	        lines_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:896:5: DUMPFILE_SYMBOL string_literal
            	    {
            	         MATCHT(DUMPFILE_SYMBOL, &FOLLOW_DUMPFILE_SYMBOL_in_into_clause5033);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinto_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_string_literal_in_into_clause5035);
            	        string_literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleinto_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:897:5: ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )*
            	    {
            	        // MySQLSimpleParser.g:897:5: ( AT_SIGN_SYMBOL )?
            	        {
            	            int alt337=2;
            	            switch ( LA(1) )
            	            {
            	                case AT_SIGN_SYMBOL:
            	                	{
            	                		alt337=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt337)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:897:5: AT_SIGN_SYMBOL
            	        	    {
            	        	         MATCHT(AT_SIGN_SYMBOL, &FOLLOW_AT_SIGN_SYMBOL_in_into_clause5041);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:897:21: ( text_or_identifier | AT_TEXT_SUFFIX )
            	        {
            	            int alt338=2;
            	            {
            	                int LA338_0 = LA(1);
            	                if ( (((LA338_0 >= ACCOUNT_SYMBOL) && (LA338_0 <= YEAR_SYMBOL)) || LA338_0 == SINGLE_QUOTED_TEXT || LA338_0 == BACK_TICK_QUOTED_ID || LA338_0 == DOUBLE_QUOTED_TEXT || LA338_0 == IDENTIFIER))
            	                {
            	                    alt338=1;
            	                }
            	                else if ( (LA338_0 == AT_TEXT_SUFFIX))
            	                {
            	                    alt338=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 338;
            	                    EXCEPTION->state        = 0;


            	                    goto ruleinto_clauseEx;

            	                }
            	            }
            	            switch (alt338)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:897:22: text_or_identifier
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_into_clause5045);
            	        	        text_or_identifier(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:897:43: AT_TEXT_SUFFIX
            	        	    {
            	        	         MATCHT(AT_TEXT_SUFFIX, &FOLLOW_AT_TEXT_SUFFIX_in_into_clause5049);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:897:59: ( COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX ) )*

            	        for (;;)
            	        {
            	            int alt341=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt341=1;
            	            	}
            	                break;

            	            }

            	            switch (alt341)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:897:60: COMMA_SYMBOL ( AT_SIGN_SYMBOL )? ( text_or_identifier | AT_TEXT_SUFFIX )
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_into_clause5053);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleinto_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:897:73: ( AT_SIGN_SYMBOL )?
            	        	        {
            	        	            int alt339=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case AT_SIGN_SYMBOL:
            	        	                	{
            	        	                		alt339=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt339)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:897:73: AT_SIGN_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(AT_SIGN_SYMBOL, &FOLLOW_AT_SIGN_SYMBOL_in_into_clause5055);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleinto_clauseEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:897:89: ( text_or_identifier | AT_TEXT_SUFFIX )
            	        	        {
            	        	            int alt340=2;
            	        	            {
            	        	                int LA340_0 = LA(1);
            	        	                if ( (((LA340_0 >= ACCOUNT_SYMBOL) && (LA340_0 <= YEAR_SYMBOL)) || LA340_0 == SINGLE_QUOTED_TEXT || LA340_0 == BACK_TICK_QUOTED_ID || LA340_0 == DOUBLE_QUOTED_TEXT || LA340_0 == IDENTIFIER))
            	        	                {
            	        	                    alt340=1;
            	        	                }
            	        	                else if ( (LA340_0 == AT_TEXT_SUFFIX))
            	        	                {
            	        	                    alt340=2;
            	        	                }
            	        	                else
            	        	                {
            	        	                    if (BACKTRACKING>0)
            	        	                    {
            	        	                        FAILEDFLAG = ANTLR3_TRUE;
            	        	                        return ;
            	        	                    }


            	        	                    CONSTRUCTEX();
            	        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	        	                    EXCEPTION->message      = (void *)"";
            	        	                    EXCEPTION->decisionNum  = 340;
            	        	                    EXCEPTION->state        = 0;


            	        	                    goto ruleinto_clauseEx;

            	        	                }
            	        	            }
            	        	            switch (alt340)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:897:90: text_or_identifier
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_into_clause5059);
            	        	        	        text_or_identifier(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleinto_clauseEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;
            	        	        	case 2:
            	        	        	    // MySQLSimpleParser.g:897:111: AT_TEXT_SUFFIX
            	        	        	    {
            	        	        	         MATCHT(AT_TEXT_SUFFIX, &FOLLOW_AT_TEXT_SUFFIX_in_into_clause5063);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleinto_clauseEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop341;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop341: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinto_clauseEx; /* Prevent compiler warnings */
    ruleinto_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end into_clause */

/**
 * $ANTLR start procedure_analyse_clause
 * MySQLSimpleParser.g:901:1: procedure_analyse_clause : PROCEDURE_SYMBOL ANALYSE_SYMBOL OPEN_PAR_SYMBOL ( INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? )? CLOSE_PAR_SYMBOL ;
 */
static void
procedure_analyse_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:901:25: ( PROCEDURE_SYMBOL ANALYSE_SYMBOL OPEN_PAR_SYMBOL ( INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:902:2: PROCEDURE_SYMBOL ANALYSE_SYMBOL OPEN_PAR_SYMBOL ( INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? )? CLOSE_PAR_SYMBOL
        {
             MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_procedure_analyse_clause5078);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_analyse_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ANALYSE_SYMBOL, &FOLLOW_ANALYSE_SYMBOL_in_procedure_analyse_clause5080);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_analyse_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_procedure_analyse_clause5082);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_analyse_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:902:50: ( INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? )?
            {
                int alt344=2;
                switch ( LA(1) )
                {
                    case INT_NUMBER:
                    	{
                    		alt344=1;
                    	}
                        break;
                }

                switch (alt344)
                {
            	case 1:
            	    // MySQLSimpleParser.g:902:51: INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )?
            	    {
            	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_procedure_analyse_clause5085);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprocedure_analyse_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:902:62: ( COMMA_SYMBOL INT_NUMBER )?
            	        {
            	            int alt343=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt343=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt343)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:902:63: COMMA_SYMBOL INT_NUMBER
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_procedure_analyse_clause5088);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleprocedure_analyse_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_procedure_analyse_clause5090);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleprocedure_analyse_clauseEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_procedure_analyse_clause5096);
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_analyse_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprocedure_analyse_clauseEx; /* Prevent compiler warnings */
    ruleprocedure_analyse_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end procedure_analyse_clause */

/**
 * $ANTLR start having_clause
 * MySQLSimpleParser.g:905:1: having_clause : HAVING_SYMBOL expression ;
 */
static void
having_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:905:14: ( HAVING_SYMBOL expression )
        // MySQLSimpleParser.g:906:2: HAVING_SYMBOL expression
        {
             MATCHT(HAVING_SYMBOL, &FOLLOW_HAVING_SYMBOL_in_having_clause5105);
            if  (HASEXCEPTION())
            {
                goto rulehaving_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_having_clause5107);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulehaving_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulehaving_clauseEx; /* Prevent compiler warnings */
    rulehaving_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end having_clause */

/**
 * $ANTLR start group_by_clause
 * MySQLSimpleParser.g:909:1: group_by_clause : GROUP_SYMBOL BY_SYMBOL expression_list_with_direction ( olap_option )? ;
 */
static void
group_by_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:909:16: ( GROUP_SYMBOL BY_SYMBOL expression_list_with_direction ( olap_option )? )
        // MySQLSimpleParser.g:910:2: GROUP_SYMBOL BY_SYMBOL expression_list_with_direction ( olap_option )?
        {
             MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_group_by_clause5116);
            if  (HASEXCEPTION())
            {
                goto rulegroup_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_group_by_clause5118);
            if  (HASEXCEPTION())
            {
                goto rulegroup_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_list_with_direction_in_group_by_clause5120);
            expression_list_with_direction(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegroup_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:910:56: ( olap_option )?
            {
                int alt345=2;
                switch ( LA(1) )
                {
                    case WITH_SYMBOL:
                    	{
                    		switch ( LA(2) )
                    		{
                    		    case CUBE_SYMBOL:
                    		    case ROLLUP_SYMBOL:
                    		    	{
                    		    		alt345=1;
                    		    	}
                    		        break;
                    		}

                    	}
                        break;
                }

                switch (alt345)
                {
            	case 1:
            	    // MySQLSimpleParser.g:910:56: olap_option
            	    {
            	        FOLLOWPUSH(FOLLOW_olap_option_in_group_by_clause5122);
            	        olap_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegroup_by_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulegroup_by_clauseEx; /* Prevent compiler warnings */
    rulegroup_by_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end group_by_clause */

/**
 * $ANTLR start olap_option
 * MySQLSimpleParser.g:913:1: olap_option : ( WITH_SYMBOL CUBE_SYMBOL | WITH_SYMBOL ROLLUP_SYMBOL );
 */
static void
olap_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:913:12: ( WITH_SYMBOL CUBE_SYMBOL | WITH_SYMBOL ROLLUP_SYMBOL )

            ANTLR3_UINT32 alt346;

            alt346=2;

            switch ( LA(1) )
            {
            case WITH_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case CUBE_SYMBOL:
            			{
            				alt346=1;
            			}
            		    break;
            		case ROLLUP_SYMBOL:
            			{
            				alt346=2;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 346;
            		    EXCEPTION->state        = 1;


            		    goto ruleolap_optionEx;

            		}

            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 346;
                EXCEPTION->state        = 0;


                goto ruleolap_optionEx;

            }

            switch (alt346)
            {
        	case 1:
        	    // MySQLSimpleParser.g:914:2: WITH_SYMBOL CUBE_SYMBOL
        	    {
        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_olap_option5132);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleolap_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CUBE_SYMBOL, &FOLLOW_CUBE_SYMBOL_in_olap_option5134);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleolap_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:915:4: WITH_SYMBOL ROLLUP_SYMBOL
        	    {
        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_olap_option5139);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleolap_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(ROLLUP_SYMBOL, &FOLLOW_ROLLUP_SYMBOL_in_olap_option5141);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleolap_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleolap_optionEx; /* Prevent compiler warnings */
    ruleolap_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end olap_option */

/**
 * $ANTLR start order_by_clause
 * MySQLSimpleParser.g:918:1: order_by_clause : ORDER_SYMBOL BY_SYMBOL expression_list_with_direction ;
 */
static void
order_by_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:918:16: ( ORDER_SYMBOL BY_SYMBOL expression_list_with_direction )
        // MySQLSimpleParser.g:919:2: ORDER_SYMBOL BY_SYMBOL expression_list_with_direction
        {
             MATCHT(ORDER_SYMBOL, &FOLLOW_ORDER_SYMBOL_in_order_by_clause5150);
            if  (HASEXCEPTION())
            {
                goto ruleorder_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_order_by_clause5152);
            if  (HASEXCEPTION())
            {
                goto ruleorder_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_list_with_direction_in_order_by_clause5154);
            expression_list_with_direction(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleorder_by_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleorder_by_clauseEx; /* Prevent compiler warnings */
    ruleorder_by_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end order_by_clause */

/**
 * $ANTLR start order_by_or_limit
 * MySQLSimpleParser.g:922:1: order_by_or_limit : ( order_by_clause ( limit_clause )? | limit_clause );
 */
static void
order_by_or_limit(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:922:18: ( order_by_clause ( limit_clause )? | limit_clause )

            ANTLR3_UINT32 alt348;

            alt348=2;

            switch ( LA(1) )
            {
            case ORDER_SYMBOL:
            	{
            		alt348=1;
            	}
                break;
            case LIMIT_SYMBOL:
            	{
            		alt348=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 348;
                EXCEPTION->state        = 0;


                goto ruleorder_by_or_limitEx;

            }

            switch (alt348)
            {
        	case 1:
        	    // MySQLSimpleParser.g:923:2: order_by_clause ( limit_clause )?
        	    {
        	        FOLLOWPUSH(FOLLOW_order_by_clause_in_order_by_or_limit5163);
        	        order_by_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleorder_by_or_limitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:923:18: ( limit_clause )?
        	        {
        	            int alt347=2;
        	            switch ( LA(1) )
        	            {
        	                case LIMIT_SYMBOL:
        	                	{
        	                		alt347=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt347)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:923:18: limit_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_order_by_or_limit5165);
        	        	        limit_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleorder_by_or_limitEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:924:4: limit_clause
        	    {
        	        FOLLOWPUSH(FOLLOW_limit_clause_in_order_by_or_limit5171);
        	        limit_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleorder_by_or_limitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleorder_by_or_limitEx; /* Prevent compiler warnings */
    ruleorder_by_or_limitEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end order_by_or_limit */

/**
 * $ANTLR start direction
 * MySQLSimpleParser.g:927:1: direction : ( ASC_SYMBOL | DESC_SYMBOL );
 */
static void
direction(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:927:10: ( ASC_SYMBOL | DESC_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == ASC_SYMBOL || LA(1) == DESC_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruledirectionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledirectionEx; /* Prevent compiler warnings */
    ruledirectionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end direction */

/**
 * $ANTLR start from_clause
 * MySQLSimpleParser.g:932:1: from_clause : FROM_SYMBOL table_reference_list ;
 */
static void
from_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:932:12: ( FROM_SYMBOL table_reference_list )
        // MySQLSimpleParser.g:933:2: FROM_SYMBOL table_reference_list
        {
             MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_from_clause5194);
            if  (HASEXCEPTION())
            {
                goto rulefrom_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_reference_list_in_from_clause5196);
            table_reference_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefrom_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefrom_clauseEx; /* Prevent compiler warnings */
    rulefrom_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end from_clause */

/**
 * $ANTLR start where_clause
 * MySQLSimpleParser.g:936:1: where_clause : WHERE_SYMBOL expression ;
 */
static void
where_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:936:13: ( WHERE_SYMBOL expression )
        // MySQLSimpleParser.g:937:2: WHERE_SYMBOL expression
        {
             MATCHT(WHERE_SYMBOL, &FOLLOW_WHERE_SYMBOL_in_where_clause5205);
            if  (HASEXCEPTION())
            {
                goto rulewhere_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_where_clause5207);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulewhere_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulewhere_clauseEx; /* Prevent compiler warnings */
    rulewhere_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end where_clause */

/**
 * $ANTLR start table_reference_list
 * MySQLSimpleParser.g:940:1: table_reference_list : ( join_table_list | DUAL_SYMBOL );
 */
static void
table_reference_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:940:21: ( join_table_list | DUAL_SYMBOL )

            ANTLR3_UINT32 alt349;

            alt349=2;

            {
                int LA349_0 = LA(1);
                if ( (((LA349_0 >= ACCOUNT_SYMBOL) && (LA349_0 <= YEAR_SYMBOL)) || ((LA349_0 >= OPEN_CURLY_SYMBOL) && (LA349_0 <= OPEN_PAR_SYMBOL)) || LA349_0 == SELECT_SYMBOL || LA349_0 == BACK_TICK_QUOTED_ID || LA349_0 == DOT_SYMBOL || LA349_0 == DOUBLE_QUOTED_TEXT || LA349_0 == IDENTIFIER))
                {
                    alt349=1;
                }
                else if ( (LA349_0 == DUAL_SYMBOL))
                {
                    alt349=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 349;
                    EXCEPTION->state        = 0;


                    goto ruletable_reference_listEx;

                }
            }
            switch (alt349)
            {
        	case 1:
        	    // MySQLSimpleParser.g:941:2: join_table_list
        	    {
        	        FOLLOWPUSH(FOLLOW_join_table_list_in_table_reference_list5216);
        	        join_table_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_reference_listEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:942:4: DUAL_SYMBOL
        	    {
        	         MATCHT(DUAL_SYMBOL, &FOLLOW_DUAL_SYMBOL_in_table_reference_list5221);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_reference_listEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletable_reference_listEx; /* Prevent compiler warnings */
    ruletable_reference_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_reference_list */

/**
 * $ANTLR start join_table_list
 * MySQLSimpleParser.g:945:1: join_table_list : escaped_table_reference ( options {greedy=true; } : COMMA_SYMBOL escaped_table_reference )* ;
 */
static void
join_table_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:945:16: ( escaped_table_reference ( options {greedy=true; } : COMMA_SYMBOL escaped_table_reference )* )
        // MySQLSimpleParser.g:946:3: escaped_table_reference ( options {greedy=true; } : COMMA_SYMBOL escaped_table_reference )*
        {
            FOLLOWPUSH(FOLLOW_escaped_table_reference_in_join_table_list5232);
            escaped_table_reference(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulejoin_table_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:946:27: ( options {greedy=true; } : COMMA_SYMBOL escaped_table_reference )*

            for (;;)
            {
                int alt350=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt350=1;
                	}
                    break;

                }

                switch (alt350)
                {
            	case 1:
            	    // MySQLSimpleParser.g:946:57: COMMA_SYMBOL escaped_table_reference
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_join_table_list5248);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulejoin_table_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_escaped_table_reference_in_join_table_list5250);
            	        escaped_table_reference(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulejoin_table_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop350;	/* break out of the loop */
            	    break;
                }
            }
            loop350: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulejoin_table_listEx; /* Prevent compiler warnings */
    rulejoin_table_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end join_table_list */

/**
 * $ANTLR start escaped_table_reference
 * MySQLSimpleParser.g:955:1: escaped_table_reference : ( table_reference | OPEN_CURLY_SYMBOL identifier table_reference CLOSE_CURLY_SYMBOL );
 */
static void
escaped_table_reference(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:955:24: ( table_reference | OPEN_CURLY_SYMBOL identifier table_reference CLOSE_CURLY_SYMBOL )

            ANTLR3_UINT32 alt351;

            alt351=2;

            {
                int LA351_0 = LA(1);
                if ( (((LA351_0 >= ACCOUNT_SYMBOL) && (LA351_0 <= YEAR_SYMBOL)) || LA351_0 == OPEN_PAR_SYMBOL || LA351_0 == SELECT_SYMBOL || LA351_0 == BACK_TICK_QUOTED_ID || LA351_0 == DOT_SYMBOL || LA351_0 == DOUBLE_QUOTED_TEXT || LA351_0 == IDENTIFIER))
                {
                    alt351=1;
                }
                else if ( (LA351_0 == OPEN_CURLY_SYMBOL))
                {
                    alt351=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 351;
                    EXCEPTION->state        = 0;


                    goto ruleescaped_table_referenceEx;

                }
            }
            switch (alt351)
            {
        	case 1:
        	    // MySQLSimpleParser.g:956:2: table_reference
        	    {
        	        FOLLOWPUSH(FOLLOW_table_reference_in_escaped_table_reference5267);
        	        table_reference(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleescaped_table_referenceEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:957:4: OPEN_CURLY_SYMBOL identifier table_reference CLOSE_CURLY_SYMBOL
        	    {
        	         MATCHT(OPEN_CURLY_SYMBOL, &FOLLOW_OPEN_CURLY_SYMBOL_in_escaped_table_reference5272);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleescaped_table_referenceEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_escaped_table_reference5274);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleescaped_table_referenceEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_reference_in_escaped_table_reference5276);
        	        table_reference(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleescaped_table_referenceEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_CURLY_SYMBOL, &FOLLOW_CLOSE_CURLY_SYMBOL_in_escaped_table_reference5278);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleescaped_table_referenceEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleescaped_table_referenceEx; /* Prevent compiler warnings */
    ruleescaped_table_referenceEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end escaped_table_reference */

/**
 * $ANTLR start table_reference
 * MySQLSimpleParser.g:960:1: table_reference : table_factor ( options {greedy=true; } : join )* ;
 */
static void
table_reference(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:960:16: ( table_factor ( options {greedy=true; } : join )* )
        // MySQLSimpleParser.g:961:2: table_factor ( options {greedy=true; } : join )*
        {
            FOLLOWPUSH(FOLLOW_table_factor_in_table_reference5288);
            table_factor(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_referenceEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:961:15: ( options {greedy=true; } : join )*

            for (;;)
            {
                int alt352=2;
                switch ( LA(1) )
                {
                case CROSS_SYMBOL:
                case INNER_SYMBOL:
                	{
                		alt352=1;
                	}
                    break;
                case JOIN_SYMBOL:
                	{
                		alt352=1;
                	}
                    break;
                case STRAIGHT_JOIN_SYMBOL:
                	{
                		alt352=1;
                	}
                    break;
                case LEFT_SYMBOL:
                case RIGHT_SYMBOL:
                	{
                		alt352=1;
                	}
                    break;
                case NATURAL_SYMBOL:
                	{
                		alt352=1;
                	}
                    break;

                }

                switch (alt352)
                {
            	case 1:
            	    // MySQLSimpleParser.g:961:45: join
            	    {
            	        FOLLOWPUSH(FOLLOW_join_in_table_reference5304);
            	        join(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_referenceEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop352;	/* break out of the loop */
            	    break;
                }
            }
            loop352: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_referenceEx; /* Prevent compiler warnings */
    ruletable_referenceEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_reference */

/**
 * $ANTLR start join
 * MySQLSimpleParser.g:964:1: join : join_table ;
 */
static void
join(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:964:5: ( join_table )
        // MySQLSimpleParser.g:965:2: join_table
        {
            FOLLOWPUSH(FOLLOW_join_table_in_join5315);
            join_table(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulejoinEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulejoinEx; /* Prevent compiler warnings */
    rulejoinEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end join */

/**
 * $ANTLR start table_factor
 * MySQLSimpleParser.g:968:1: table_factor : ( SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression | OPEN_PAR_SYMBOL select_table_factor_union CLOSE_PAR_SYMBOL ( table_alias )? | table_ref ( use_partition )? ( table_alias )? ( index_hint_list )? );
 */
static void
table_factor(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:968:13: ( SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression | OPEN_PAR_SYMBOL select_table_factor_union CLOSE_PAR_SYMBOL ( table_alias )? | table_ref ( use_partition )? ( table_alias )? ( index_hint_list )? )

            ANTLR3_UINT32 alt358;

            alt358=3;

            {
                int LA358_0 = LA(1);
                if ( (LA358_0 == SELECT_SYMBOL))
                {
                    alt358=1;
                }
                else if ( (LA358_0 == OPEN_PAR_SYMBOL))
                {
                    alt358=2;
                }
                else if ( (((LA358_0 >= ACCOUNT_SYMBOL) && (LA358_0 <= YEAR_SYMBOL)) || LA358_0 == BACK_TICK_QUOTED_ID || LA358_0 == DOT_SYMBOL || LA358_0 == DOUBLE_QUOTED_TEXT || LA358_0 == IDENTIFIER))
                {
                    alt358=3;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 358;
                    EXCEPTION->state        = 0;


                    goto ruletable_factorEx;

                }
            }
            switch (alt358)
            {
        	case 1:
        	    // MySQLSimpleParser.g:969:2: SELECT_SYMBOL ( options {greedy=true; } : select_option )* select_item_list table_expression
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_table_factor5324);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:969:16: ( options {greedy=true; } : select_option )*

        	        for (;;)
        	        {
        	            int alt353=2;
        	            switch ( LA(1) )
        	            {
        	            case SQL_BUFFER_RESULT_SYMBOL:
        	            	{
        	            		{
        	            		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
        	            		    */
        	            		    int LA353_96 = LA(2);
        	            		    if ( (LA353_96 == DOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MOD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MINUS_OPERATOR || LA353_96 == PLUS_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == NOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == SOUNDS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == IDENTIFIER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == BACK_TICK_QUOTED_ID))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == DOUBLE_QUOTED_TEXT))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == DATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ASCII_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == INTERVAL_SYMBOL || LA353_96 == NCHAR_TEXT || LA353_96 == NOT2_SYMBOL || LA353_96 == SINGLE_QUOTED_TEXT || LA353_96 == BITWISE_NOT_OPERATOR || LA353_96 == LOGICAL_NOT_OPERATOR || LA353_96 == UNDERSCORE_CHARSET))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == STRAIGHT_JOIN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == LEFT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == DECIMAL_NUMBER || LA353_96 == FLOAT_NUMBER || LA353_96 == HEX_NUMBER || LA353_96 == INT_NUMBER || LA353_96 == LONG_NUMBER || LA353_96 == ULONGLONG_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == TIMESTAMP_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == DAY_SYMBOL || LA353_96 == FALSE_SYMBOL || ((LA353_96 >= NULL2_SYMBOL) && (LA353_96 <= NULL_SYMBOL)) || LA353_96 == TRUE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == CHAR_SYMBOL || LA353_96 == CURRENT_USER_SYMBOL || LA353_96 == BIN_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == HOUR_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MINUTE_SYMBOL || LA353_96 == INSERT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == RIGHT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MONTH_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == SECOND_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == USER_SYMBOL || LA353_96 == TRIM_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == YEAR_SYMBOL || LA353_96 == VALUES_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ADDDATE_SYMBOL || LA353_96 == SUBDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == GET_FORMAT_SYMBOL || LA353_96 == CURDATE_SYMBOL || LA353_96 == CURTIME_SYMBOL || LA353_96 == DATE_ADD_SYMBOL || LA353_96 == DATE_SUB_SYMBOL || LA353_96 == EXTRACT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (((LA353_96 >= TIMESTAMP_ADD_SYMBOL) && (LA353_96 <= TIMESTAMP_DIFF_SYMBOL)) || LA353_96 == NOW_SYMBOL || LA353_96 == POSITION_SYMBOL || LA353_96 == SUBSTRING_SYMBOL || LA353_96 == SYSDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == COALESCE_SYMBOL || ((LA353_96 >= UTC_DATE_SYMBOL) && (LA353_96 <= UTC_TIME_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == CHARSET_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == FORMAT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == COLLATION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MICROSECOND_SYMBOL || LA353_96 == DATABASE_SYMBOL || LA353_96 == IF_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == TRUNCATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == OLD_PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MULT_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == QUARTER_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == REVERSE_SYMBOL || ((LA353_96 >= REPEAT_SYMBOL) && (LA353_96 <= REPLACE_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ROW_COUNT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == WEEK_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == CONTAINS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == WEIGHT_STRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == GEOMETRYCOLLECTION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == LINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MULTILINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MULTIPOINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MULTIPOLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == POINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == POLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == AVG_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ACCOUNT_SYMBOL || ((LA353_96 >= ALWAYS_SYMBOL) && (LA353_96 <= CACHE_SYMBOL)) || ((LA353_96 >= CHECKSUM_SYMBOL) && (LA353_96 <= COMMIT_SYMBOL)) || ((LA353_96 >= DEALLOCATE_SYMBOL) && (LA353_96 <= FOLLOWS_SYMBOL)) || ((LA353_96 >= GROUP_REPLICATION_SYMBOL) && (LA353_96 <= STOP_SYMBOL)) || ((LA353_96 >= UNICODE_SYMBOL) && (LA353_96 <= XA_SYMBOL)) || LA353_96 == SHUTDOWN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ROW_SYMBOL || LA353_96 == COUNT_SYMBOL || LA353_96 == EXISTS_SYMBOL || LA353_96 == GROUP_CONCAT_SYMBOL || LA353_96 == MAX_SYMBOL || LA353_96 == MIN_SYMBOL || LA353_96 == OPEN_PAR_SYMBOL || LA353_96 == PARAM_MARKER || LA353_96 == STDDEV_SAMP_SYMBOL || LA353_96 == STD_SYMBOL || LA353_96 == SUM_SYMBOL || LA353_96 == VARIANCE_SYMBOL || LA353_96 == VAR_SAMP_SYMBOL || ((LA353_96 >= AT_AT_SIGN_SYMBOL) && (LA353_96 <= AT_TEXT_SUFFIX)) || ((LA353_96 >= BIT_AND_SYMBOL) && (LA353_96 <= BIT_XOR_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == SQL_BUFFER_RESULT_SYMBOL || LA353_96 == BINARY_SYMBOL || ((LA353_96 >= CASE_SYMBOL) && (LA353_96 <= CAST_SYMBOL)) || LA353_96 == CONVERT_SYMBOL || LA353_96 == MATCH_SYMBOL || LA353_96 == OPEN_CURLY_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == SQL_NO_CACHE_SYMBOL || LA353_96 == ALL_SYMBOL || LA353_96 == DISTINCT_SYMBOL || LA353_96 == HIGH_PRIORITY_SYMBOL || ((LA353_96 >= SQL_BIG_RESULT_SYMBOL) && (LA353_96 <= SQL_SMALL_RESULT_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == SQL_CACHE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == MAX_STATEMENT_TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_96 == ACTION_SYMBOL || ((LA353_96 >= AFTER_SYMBOL) && (LA353_96 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA353_96 >= BINLOG_SYMBOL) && (LA353_96 <= CLASS_ORIGIN_SYMBOL)) || LA353_96 == CODE_SYMBOL || ((LA353_96 >= COLUMN_NAME_SYMBOL) && (LA353_96 <= DATETIME_SYMBOL)) || ((LA353_96 >= DEFAULT_AUTH_SYMBOL) && (LA353_96 <= GEOMETRY_SYMBOL)) || ((LA353_96 >= GRANTS_SYMBOL) && (LA353_96 <= HOSTS_SYMBOL)) || ((LA353_96 >= IDENTIFIED_SYMBOL) && (LA353_96 <= LEVEL_SYMBOL)) || ((LA353_96 >= LIST_SYMBOL) && (LA353_96 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA353_96 >= MAX_SIZE_SYMBOL) && (LA353_96 <= MESSAGE_TEXT_SYMBOL)) || LA353_96 == MIGRATE_SYMBOL || ((LA353_96 >= MIN_ROWS_SYMBOL) && (LA353_96 <= MODE_SYMBOL)) || ((LA353_96 >= MUTEX_SYMBOL) && (LA353_96 <= OFFSET_SYMBOL)) || ((LA353_96 >= ONE_SHOT_SYMBOL) && (LA353_96 <= PARTITIONS_SYMBOL)) || ((LA353_96 >= PHASE_SYMBOL) && (LA353_96 <= PLUGINS_SYMBOL)) || ((LA353_96 >= PRESERVE_SYMBOL) && (LA353_96 <= PROXY_SYMBOL)) || ((LA353_96 >= QUERY_SYMBOL) && (LA353_96 <= RETURNS_SYMBOL)) || ((LA353_96 >= ROLLUP_SYMBOL) && (LA353_96 <= ROWS_SYMBOL)) || LA353_96 == ROW_FORMAT_SYMBOL || ((LA353_96 >= RTREE_SYMBOL) && (LA353_96 <= SCHEMA_NAME_SYMBOL)) || ((LA353_96 >= SERIAL_SYMBOL) && (LA353_96 <= SHARE_SYMBOL)) || ((LA353_96 >= SLOW_SYMBOL) && (LA353_96 <= SNAPSHOT_SYMBOL)) || ((LA353_96 >= SOURCE_SYMBOL) && (LA353_96 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA353_96 >= SQL_THREAD_SYMBOL) && (LA353_96 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA353_96 >= SUBJECT_SYMBOL) && (LA353_96 <= TRIGGERS_SYMBOL)) || ((LA353_96 >= TYPES_SYMBOL) && (LA353_96 <= USER_RESOURCES_SYMBOL)) || ((LA353_96 >= USE_FRM_SYMBOL) && (LA353_96 <= WAIT_SYMBOL)) || LA353_96 == WORK_SYMBOL || ((LA353_96 >= X509_SYMBOL) && (LA353_96 <= XML_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }

        	            		}
        	            	}
        	                break;
        	            case SQL_NO_CACHE_SYMBOL:
        	            	{
        	            		{
        	            		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
        	            		    */
        	            		    int LA353_104 = LA(2);
        	            		    if ( (LA353_104 == DOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MOD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MINUS_OPERATOR || LA353_104 == PLUS_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == NOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == SOUNDS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == IDENTIFIER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == BACK_TICK_QUOTED_ID))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == DOUBLE_QUOTED_TEXT))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == DATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ASCII_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == INTERVAL_SYMBOL || LA353_104 == NCHAR_TEXT || LA353_104 == NOT2_SYMBOL || LA353_104 == SINGLE_QUOTED_TEXT || LA353_104 == BITWISE_NOT_OPERATOR || LA353_104 == LOGICAL_NOT_OPERATOR || LA353_104 == UNDERSCORE_CHARSET))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == STRAIGHT_JOIN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == LEFT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == DECIMAL_NUMBER || LA353_104 == FLOAT_NUMBER || LA353_104 == HEX_NUMBER || LA353_104 == INT_NUMBER || LA353_104 == LONG_NUMBER || LA353_104 == ULONGLONG_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == TIMESTAMP_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == DAY_SYMBOL || LA353_104 == FALSE_SYMBOL || ((LA353_104 >= NULL2_SYMBOL) && (LA353_104 <= NULL_SYMBOL)) || LA353_104 == TRUE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == CHAR_SYMBOL || LA353_104 == CURRENT_USER_SYMBOL || LA353_104 == BIN_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == HOUR_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MINUTE_SYMBOL || LA353_104 == INSERT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == RIGHT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MONTH_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == SECOND_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == USER_SYMBOL || LA353_104 == TRIM_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == YEAR_SYMBOL || LA353_104 == VALUES_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ADDDATE_SYMBOL || LA353_104 == SUBDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == GET_FORMAT_SYMBOL || LA353_104 == CURDATE_SYMBOL || LA353_104 == CURTIME_SYMBOL || LA353_104 == DATE_ADD_SYMBOL || LA353_104 == DATE_SUB_SYMBOL || LA353_104 == EXTRACT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (((LA353_104 >= TIMESTAMP_ADD_SYMBOL) && (LA353_104 <= TIMESTAMP_DIFF_SYMBOL)) || LA353_104 == NOW_SYMBOL || LA353_104 == POSITION_SYMBOL || LA353_104 == SUBSTRING_SYMBOL || LA353_104 == SYSDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == COALESCE_SYMBOL || ((LA353_104 >= UTC_DATE_SYMBOL) && (LA353_104 <= UTC_TIME_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == CHARSET_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == FORMAT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == COLLATION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MICROSECOND_SYMBOL || LA353_104 == DATABASE_SYMBOL || LA353_104 == IF_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == TRUNCATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == OLD_PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MULT_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == QUARTER_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == REVERSE_SYMBOL || ((LA353_104 >= REPEAT_SYMBOL) && (LA353_104 <= REPLACE_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ROW_COUNT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == WEEK_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == CONTAINS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == WEIGHT_STRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == GEOMETRYCOLLECTION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == LINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MULTILINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MULTIPOINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MULTIPOLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == POINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == POLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == AVG_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ACCOUNT_SYMBOL || ((LA353_104 >= ALWAYS_SYMBOL) && (LA353_104 <= CACHE_SYMBOL)) || ((LA353_104 >= CHECKSUM_SYMBOL) && (LA353_104 <= COMMIT_SYMBOL)) || ((LA353_104 >= DEALLOCATE_SYMBOL) && (LA353_104 <= FOLLOWS_SYMBOL)) || ((LA353_104 >= GROUP_REPLICATION_SYMBOL) && (LA353_104 <= STOP_SYMBOL)) || ((LA353_104 >= UNICODE_SYMBOL) && (LA353_104 <= XA_SYMBOL)) || LA353_104 == SHUTDOWN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ROW_SYMBOL || LA353_104 == COUNT_SYMBOL || LA353_104 == EXISTS_SYMBOL || LA353_104 == GROUP_CONCAT_SYMBOL || LA353_104 == MAX_SYMBOL || LA353_104 == MIN_SYMBOL || LA353_104 == OPEN_PAR_SYMBOL || LA353_104 == PARAM_MARKER || LA353_104 == STDDEV_SAMP_SYMBOL || LA353_104 == STD_SYMBOL || LA353_104 == SUM_SYMBOL || LA353_104 == VARIANCE_SYMBOL || LA353_104 == VAR_SAMP_SYMBOL || ((LA353_104 >= AT_AT_SIGN_SYMBOL) && (LA353_104 <= AT_TEXT_SUFFIX)) || ((LA353_104 >= BIT_AND_SYMBOL) && (LA353_104 <= BIT_XOR_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == SQL_BUFFER_RESULT_SYMBOL || LA353_104 == BINARY_SYMBOL || ((LA353_104 >= CASE_SYMBOL) && (LA353_104 <= CAST_SYMBOL)) || LA353_104 == CONVERT_SYMBOL || LA353_104 == MATCH_SYMBOL || LA353_104 == OPEN_CURLY_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == SQL_NO_CACHE_SYMBOL || LA353_104 == ALL_SYMBOL || LA353_104 == DISTINCT_SYMBOL || LA353_104 == HIGH_PRIORITY_SYMBOL || ((LA353_104 >= SQL_BIG_RESULT_SYMBOL) && (LA353_104 <= SQL_SMALL_RESULT_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == SQL_CACHE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == MAX_STATEMENT_TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_104 == ACTION_SYMBOL || ((LA353_104 >= AFTER_SYMBOL) && (LA353_104 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA353_104 >= BINLOG_SYMBOL) && (LA353_104 <= CLASS_ORIGIN_SYMBOL)) || LA353_104 == CODE_SYMBOL || ((LA353_104 >= COLUMN_NAME_SYMBOL) && (LA353_104 <= DATETIME_SYMBOL)) || ((LA353_104 >= DEFAULT_AUTH_SYMBOL) && (LA353_104 <= GEOMETRY_SYMBOL)) || ((LA353_104 >= GRANTS_SYMBOL) && (LA353_104 <= HOSTS_SYMBOL)) || ((LA353_104 >= IDENTIFIED_SYMBOL) && (LA353_104 <= LEVEL_SYMBOL)) || ((LA353_104 >= LIST_SYMBOL) && (LA353_104 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA353_104 >= MAX_SIZE_SYMBOL) && (LA353_104 <= MESSAGE_TEXT_SYMBOL)) || LA353_104 == MIGRATE_SYMBOL || ((LA353_104 >= MIN_ROWS_SYMBOL) && (LA353_104 <= MODE_SYMBOL)) || ((LA353_104 >= MUTEX_SYMBOL) && (LA353_104 <= OFFSET_SYMBOL)) || ((LA353_104 >= ONE_SHOT_SYMBOL) && (LA353_104 <= PARTITIONS_SYMBOL)) || ((LA353_104 >= PHASE_SYMBOL) && (LA353_104 <= PLUGINS_SYMBOL)) || ((LA353_104 >= PRESERVE_SYMBOL) && (LA353_104 <= PROXY_SYMBOL)) || ((LA353_104 >= QUERY_SYMBOL) && (LA353_104 <= RETURNS_SYMBOL)) || ((LA353_104 >= ROLLUP_SYMBOL) && (LA353_104 <= ROWS_SYMBOL)) || LA353_104 == ROW_FORMAT_SYMBOL || ((LA353_104 >= RTREE_SYMBOL) && (LA353_104 <= SCHEMA_NAME_SYMBOL)) || ((LA353_104 >= SERIAL_SYMBOL) && (LA353_104 <= SHARE_SYMBOL)) || ((LA353_104 >= SLOW_SYMBOL) && (LA353_104 <= SNAPSHOT_SYMBOL)) || ((LA353_104 >= SOURCE_SYMBOL) && (LA353_104 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA353_104 >= SQL_THREAD_SYMBOL) && (LA353_104 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA353_104 >= SUBJECT_SYMBOL) && (LA353_104 <= TRIGGERS_SYMBOL)) || ((LA353_104 >= TYPES_SYMBOL) && (LA353_104 <= USER_RESOURCES_SYMBOL)) || ((LA353_104 >= USE_FRM_SYMBOL) && (LA353_104 <= WAIT_SYMBOL)) || LA353_104 == WORK_SYMBOL || ((LA353_104 >= X509_SYMBOL) && (LA353_104 <= XML_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }

        	            		}
        	            	}
        	                break;
        	            case ALL_SYMBOL:
        	            case DISTINCT_SYMBOL:
        	            case HIGH_PRIORITY_SYMBOL:
        	            case SQL_BIG_RESULT_SYMBOL:
        	            case SQL_CALC_FOUND_ROWS_SYMBOL:
        	            case SQL_SMALL_RESULT_SYMBOL:
        	            case STRAIGHT_JOIN_SYMBOL:
        	            	{
        	            		alt353=1;
        	            	}
        	                break;
        	            case SQL_CACHE_SYMBOL:
        	            	{
        	            		{
        	            		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
        	            		    */
        	            		    int LA353_106 = LA(2);
        	            		    if ( (LA353_106 == DOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MOD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MINUS_OPERATOR || LA353_106 == PLUS_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == NOT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == SOUNDS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == IDENTIFIER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == BACK_TICK_QUOTED_ID))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == DOUBLE_QUOTED_TEXT))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == DATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ASCII_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == INTERVAL_SYMBOL || LA353_106 == NCHAR_TEXT || LA353_106 == NOT2_SYMBOL || LA353_106 == SINGLE_QUOTED_TEXT || LA353_106 == BITWISE_NOT_OPERATOR || LA353_106 == LOGICAL_NOT_OPERATOR || LA353_106 == UNDERSCORE_CHARSET))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == STRAIGHT_JOIN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == LEFT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == DECIMAL_NUMBER || LA353_106 == FLOAT_NUMBER || LA353_106 == HEX_NUMBER || LA353_106 == INT_NUMBER || LA353_106 == LONG_NUMBER || LA353_106 == ULONGLONG_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == TIMESTAMP_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == DAY_SYMBOL || LA353_106 == FALSE_SYMBOL || ((LA353_106 >= NULL2_SYMBOL) && (LA353_106 <= NULL_SYMBOL)) || LA353_106 == TRUE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == CHAR_SYMBOL || LA353_106 == CURRENT_USER_SYMBOL || LA353_106 == BIN_NUMBER))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == HOUR_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MINUTE_SYMBOL || LA353_106 == INSERT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == RIGHT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MONTH_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == SECOND_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == USER_SYMBOL || LA353_106 == TRIM_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == YEAR_SYMBOL || LA353_106 == VALUES_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ADDDATE_SYMBOL || LA353_106 == SUBDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == GET_FORMAT_SYMBOL || LA353_106 == CURDATE_SYMBOL || LA353_106 == CURTIME_SYMBOL || LA353_106 == DATE_ADD_SYMBOL || LA353_106 == DATE_SUB_SYMBOL || LA353_106 == EXTRACT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (((LA353_106 >= TIMESTAMP_ADD_SYMBOL) && (LA353_106 <= TIMESTAMP_DIFF_SYMBOL)) || LA353_106 == NOW_SYMBOL || LA353_106 == POSITION_SYMBOL || LA353_106 == SUBSTRING_SYMBOL || LA353_106 == SYSDATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == COALESCE_SYMBOL || ((LA353_106 >= UTC_DATE_SYMBOL) && (LA353_106 <= UTC_TIME_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == CHARSET_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == FORMAT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == COLLATION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MICROSECOND_SYMBOL || LA353_106 == DATABASE_SYMBOL || LA353_106 == IF_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == TRUNCATE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == OLD_PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MULT_OPERATOR))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == PASSWORD_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == QUARTER_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == REVERSE_SYMBOL || ((LA353_106 >= REPEAT_SYMBOL) && (LA353_106 <= REPLACE_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ROW_COUNT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == WEEK_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == CONTAINS_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == WEIGHT_STRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == GEOMETRYCOLLECTION_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == LINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MULTILINESTRING_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MULTIPOINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MULTIPOLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == POINT_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == POLYGON_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == AVG_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ACCOUNT_SYMBOL || ((LA353_106 >= ALWAYS_SYMBOL) && (LA353_106 <= CACHE_SYMBOL)) || ((LA353_106 >= CHECKSUM_SYMBOL) && (LA353_106 <= COMMIT_SYMBOL)) || ((LA353_106 >= DEALLOCATE_SYMBOL) && (LA353_106 <= FOLLOWS_SYMBOL)) || ((LA353_106 >= GROUP_REPLICATION_SYMBOL) && (LA353_106 <= STOP_SYMBOL)) || ((LA353_106 >= UNICODE_SYMBOL) && (LA353_106 <= XA_SYMBOL)) || LA353_106 == SHUTDOWN_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ROW_SYMBOL || LA353_106 == COUNT_SYMBOL || LA353_106 == EXISTS_SYMBOL || LA353_106 == GROUP_CONCAT_SYMBOL || LA353_106 == MAX_SYMBOL || LA353_106 == MIN_SYMBOL || LA353_106 == OPEN_PAR_SYMBOL || LA353_106 == PARAM_MARKER || LA353_106 == STDDEV_SAMP_SYMBOL || LA353_106 == STD_SYMBOL || LA353_106 == SUM_SYMBOL || LA353_106 == VARIANCE_SYMBOL || LA353_106 == VAR_SAMP_SYMBOL || ((LA353_106 >= AT_AT_SIGN_SYMBOL) && (LA353_106 <= AT_TEXT_SUFFIX)) || ((LA353_106 >= BIT_AND_SYMBOL) && (LA353_106 <= BIT_XOR_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == SQL_BUFFER_RESULT_SYMBOL || LA353_106 == BINARY_SYMBOL || ((LA353_106 >= CASE_SYMBOL) && (LA353_106 <= CAST_SYMBOL)) || LA353_106 == CONVERT_SYMBOL || LA353_106 == MATCH_SYMBOL || LA353_106 == OPEN_CURLY_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == SQL_NO_CACHE_SYMBOL || LA353_106 == ALL_SYMBOL || LA353_106 == DISTINCT_SYMBOL || LA353_106 == HIGH_PRIORITY_SYMBOL || ((LA353_106 >= SQL_BIG_RESULT_SYMBOL) && (LA353_106 <= SQL_SMALL_RESULT_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == SQL_CACHE_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == MAX_STATEMENT_TIME_SYMBOL))
        	            		    {
        	            		        alt353=1;
        	            		    }
        	            		    else if ( (LA353_106 == ACTION_SYMBOL || ((LA353_106 >= AFTER_SYMBOL) && (LA353_106 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA353_106 >= BINLOG_SYMBOL) && (LA353_106 <= CLASS_ORIGIN_SYMBOL)) || LA353_106 == CODE_SYMBOL || ((LA353_106 >= COLUMN_NAME_SYMBOL) && (LA353_106 <= DATETIME_SYMBOL)) || ((LA353_106 >= DEFAULT_AUTH_SYMBOL) && (LA353_106 <= GEOMETRY_SYMBOL)) || ((LA353_106 >= GRANTS_SYMBOL) && (LA353_106 <= HOSTS_SYMBOL)) || ((LA353_106 >= IDENTIFIED_SYMBOL) && (LA353_106 <= LEVEL_SYMBOL)) || ((LA353_106 >= LIST_SYMBOL) && (LA353_106 <= MAX_QUERIES_PER_HOUR_SYMBOL)) || ((LA353_106 >= MAX_SIZE_SYMBOL) && (LA353_106 <= MESSAGE_TEXT_SYMBOL)) || LA353_106 == MIGRATE_SYMBOL || ((LA353_106 >= MIN_ROWS_SYMBOL) && (LA353_106 <= MODE_SYMBOL)) || ((LA353_106 >= MUTEX_SYMBOL) && (LA353_106 <= OFFSET_SYMBOL)) || ((LA353_106 >= ONE_SHOT_SYMBOL) && (LA353_106 <= PARTITIONS_SYMBOL)) || ((LA353_106 >= PHASE_SYMBOL) && (LA353_106 <= PLUGINS_SYMBOL)) || ((LA353_106 >= PRESERVE_SYMBOL) && (LA353_106 <= PROXY_SYMBOL)) || ((LA353_106 >= QUERY_SYMBOL) && (LA353_106 <= RETURNS_SYMBOL)) || ((LA353_106 >= ROLLUP_SYMBOL) && (LA353_106 <= ROWS_SYMBOL)) || LA353_106 == ROW_FORMAT_SYMBOL || ((LA353_106 >= RTREE_SYMBOL) && (LA353_106 <= SCHEMA_NAME_SYMBOL)) || ((LA353_106 >= SERIAL_SYMBOL) && (LA353_106 <= SHARE_SYMBOL)) || ((LA353_106 >= SLOW_SYMBOL) && (LA353_106 <= SNAPSHOT_SYMBOL)) || ((LA353_106 >= SOURCE_SYMBOL) && (LA353_106 <= SQL_BEFORE_GTIDS_SYMBOL)) || ((LA353_106 >= SQL_THREAD_SYMBOL) && (LA353_106 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA353_106 >= SUBJECT_SYMBOL) && (LA353_106 <= TRIGGERS_SYMBOL)) || ((LA353_106 >= TYPES_SYMBOL) && (LA353_106 <= USER_RESOURCES_SYMBOL)) || ((LA353_106 >= USE_FRM_SYMBOL) && (LA353_106 <= WAIT_SYMBOL)) || LA353_106 == WORK_SYMBOL || ((LA353_106 >= X509_SYMBOL) && (LA353_106 <= XML_SYMBOL))))
        	            		    {
        	            		        alt353=1;
        	            		    }

        	            		}
        	            	}
        	                break;
        	            case MAX_STATEMENT_TIME_SYMBOL:
        	            	{
        	            		switch ( LA(2) )
        	            		{
        	            		case EQUAL_OPERATOR:
        	            			{
        	            				{
        	            				   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
        	            				    */
        	            				    int LA353_568 = LA(3);
        	            				    if ( ((SERVER_VERSION >= 50704 && SERVER_VERSION < 50708)))
        	            				    {
        	            				        alt353=1;
        	            				    }

        	            				}
        	            			}
        	            		    break;

        	            		}

        	            	}
        	                break;

        	            }

        	            switch (alt353)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:969:46: select_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_select_option_in_table_factor5340);
        	        	        select_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_factorEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop353;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop353: ; /* Jump out to here if this rule does not match */


        	        FOLLOWPUSH(FOLLOW_select_item_list_in_table_factor5344);
        	        select_item_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_expression_in_table_factor5346);
        	        table_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:970:4: OPEN_PAR_SYMBOL select_table_factor_union CLOSE_PAR_SYMBOL ( table_alias )?
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_table_factor5351);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_table_factor_union_in_table_factor5353);
        	        select_table_factor_union(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_table_factor5355);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:970:63: ( table_alias )?
        	        {
        	            int alt354=2;
        	            {
        	                int LA354_0 = LA(1);
        	                if ( (((LA354_0 >= ACCOUNT_SYMBOL) && (LA354_0 <= YEAR_SYMBOL)) || LA354_0 == AS_SYMBOL || LA354_0 == BACK_TICK_QUOTED_ID || LA354_0 == DOUBLE_QUOTED_TEXT || LA354_0 == EQUAL_OPERATOR || LA354_0 == IDENTIFIER))
        	                {
        	                    alt354=1;
        	                }
        	            }
        	            switch (alt354)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:970:63: table_alias
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_table_alias_in_table_factor5357);
        	        	        table_alias(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_factorEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:971:4: table_ref ( use_partition )? ( table_alias )? ( index_hint_list )?
        	    {
        	        FOLLOWPUSH(FOLLOW_table_ref_in_table_factor5363);
        	        table_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_factorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:971:14: ( use_partition )?
        	        {
        	            int alt355=2;
        	            switch ( LA(1) )
        	            {
        	                case PARTITION_SYMBOL:
        	                	{
        	                		{
        	                		    int LA355_1 = LA(2);
        	                		    if ( ((SERVER_VERSION >= 50602)) && (LA355_1 == OPEN_PAR_SYMBOL))
        	                		    {
        	                		        alt355=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	            }

        	            switch (alt355)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:971:14: use_partition
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_use_partition_in_table_factor5365);
        	        	        use_partition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_factorEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:971:29: ( table_alias )?
        	        {
        	            int alt356=2;
        	            {
        	                int LA356_0 = LA(1);
        	                if ( (((LA356_0 >= ACCOUNT_SYMBOL) && (LA356_0 <= YEAR_SYMBOL)) || LA356_0 == AS_SYMBOL || LA356_0 == BACK_TICK_QUOTED_ID || LA356_0 == DOUBLE_QUOTED_TEXT || LA356_0 == EQUAL_OPERATOR || LA356_0 == IDENTIFIER))
        	                {
        	                    alt356=1;
        	                }
        	            }
        	            switch (alt356)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:971:29: table_alias
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_table_alias_in_table_factor5368);
        	        	        table_alias(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_factorEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:971:42: ( index_hint_list )?
        	        {
        	            int alt357=2;
        	            switch ( LA(1) )
        	            {
        	                case FORCE_SYMBOL:
        	                case IGNORE_SYMBOL:
        	                case USE_SYMBOL:
        	                	{
        	                		alt357=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt357)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:971:42: index_hint_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_index_hint_list_in_table_factor5371);
        	        	        index_hint_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_factorEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletable_factorEx; /* Prevent compiler warnings */
    ruletable_factorEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_factor */

/**
 * $ANTLR start select_table_factor_union
 * MySQLSimpleParser.g:974:1: select_table_factor_union : ( table_reference_list ( order_by_or_limit )? ) ( UNION_SYMBOL ( union_option )? query_specification )* ;
 */
static void
select_table_factor_union(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:974:26: ( ( table_reference_list ( order_by_or_limit )? ) ( UNION_SYMBOL ( union_option )? query_specification )* )
        // MySQLSimpleParser.g:975:2: ( table_reference_list ( order_by_or_limit )? ) ( UNION_SYMBOL ( union_option )? query_specification )*
        {
            // MySQLSimpleParser.g:975:2: ( table_reference_list ( order_by_or_limit )? )
            // MySQLSimpleParser.g:975:3: table_reference_list ( order_by_or_limit )?
            {
                FOLLOWPUSH(FOLLOW_table_reference_list_in_select_table_factor_union5382);
                table_reference_list(ctx);

                FOLLOWPOP();
                if  (HASEXCEPTION())
                {
                    goto ruleselect_table_factor_unionEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


                // MySQLSimpleParser.g:975:24: ( order_by_or_limit )?
                {
                    int alt359=2;
                    switch ( LA(1) )
                    {
                        case LIMIT_SYMBOL:
                        case ORDER_SYMBOL:
                        	{
                        		alt359=1;
                        	}
                            break;
                    }

                    switch (alt359)
                    {
                	case 1:
                	    // MySQLSimpleParser.g:975:24: order_by_or_limit
                	    {
                	        FOLLOWPUSH(FOLLOW_order_by_or_limit_in_select_table_factor_union5384);
                	        order_by_or_limit(ctx);

                	        FOLLOWPOP();
                	        if  (HASEXCEPTION())
                	        {
                	            goto ruleselect_table_factor_unionEx;
                	        }
                	        if (HASFAILED())
                	        {
                	            return ;
                	        }


                	    }
                	    break;

                    }
                }

            }


            // MySQLSimpleParser.g:975:44: ( UNION_SYMBOL ( union_option )? query_specification )*

            for (;;)
            {
                int alt361=2;
                switch ( LA(1) )
                {
                case UNION_SYMBOL:
                	{
                		alt361=1;
                	}
                    break;

                }

                switch (alt361)
                {
            	case 1:
            	    // MySQLSimpleParser.g:975:45: UNION_SYMBOL ( union_option )? query_specification
            	    {
            	         MATCHT(UNION_SYMBOL, &FOLLOW_UNION_SYMBOL_in_select_table_factor_union5389);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_table_factor_unionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:975:58: ( union_option )?
            	        {
            	            int alt360=2;
            	            switch ( LA(1) )
            	            {
            	                case ALL_SYMBOL:
            	                case DISTINCT_SYMBOL:
            	                	{
            	                		alt360=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt360)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:975:58: union_option
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_union_option_in_select_table_factor_union5391);
            	        	        union_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleselect_table_factor_unionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_query_specification_in_select_table_factor_union5394);
            	        query_specification(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleselect_table_factor_unionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop361;	/* break out of the loop */
            	    break;
                }
            }
            loop361: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleselect_table_factor_unionEx; /* Prevent compiler warnings */
    ruleselect_table_factor_unionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_table_factor_union */

/**
 * $ANTLR start query_specification
 * MySQLSimpleParser.g:978:1: query_specification : ( SELECT_SYMBOL select_part2_derived table_expression | OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL ( order_by_or_limit )? );
 */
static void
query_specification(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:978:20: ( SELECT_SYMBOL select_part2_derived table_expression | OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL ( order_by_or_limit )? )

            ANTLR3_UINT32 alt363;

            alt363=2;

            switch ( LA(1) )
            {
            case SELECT_SYMBOL:
            	{
            		alt363=1;
            	}
                break;
            case OPEN_PAR_SYMBOL:
            	{
            		alt363=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 363;
                EXCEPTION->state        = 0;


                goto rulequery_specificationEx;

            }

            switch (alt363)
            {
        	case 1:
        	    // MySQLSimpleParser.g:979:2: SELECT_SYMBOL select_part2_derived table_expression
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_query_specification5405);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_part2_derived_in_query_specification5407);
        	        select_part2_derived(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_expression_in_query_specification5409);
        	        table_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:980:4: OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL ( order_by_or_limit )?
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_query_specification5414);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_paren_derived_in_query_specification5416);
        	        select_paren_derived(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_query_specification5418);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulequery_specificationEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:980:58: ( order_by_or_limit )?
        	        {
        	            int alt362=2;
        	            switch ( LA(1) )
        	            {
        	                case LIMIT_SYMBOL:
        	                case ORDER_SYMBOL:
        	                	{
        	                		alt362=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt362)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:980:58: order_by_or_limit
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_order_by_or_limit_in_query_specification5420);
        	        	        order_by_or_limit(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulequery_specificationEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulequery_specificationEx; /* Prevent compiler warnings */
    rulequery_specificationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end query_specification */

/**
 * $ANTLR start select_paren_derived
 * MySQLSimpleParser.g:983:1: select_paren_derived : ( SELECT_SYMBOL select_part2_derived table_expression | OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL );
 */
static void
select_paren_derived(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:983:21: ( SELECT_SYMBOL select_part2_derived table_expression | OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt364;

            alt364=2;

            switch ( LA(1) )
            {
            case SELECT_SYMBOL:
            	{
            		alt364=1;
            	}
                break;
            case OPEN_PAR_SYMBOL:
            	{
            		alt364=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 364;
                EXCEPTION->state        = 0;


                goto ruleselect_paren_derivedEx;

            }

            switch (alt364)
            {
        	case 1:
        	    // MySQLSimpleParser.g:984:2: SELECT_SYMBOL select_part2_derived table_expression
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_select_paren_derived5430);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_part2_derived_in_select_paren_derived5432);
        	        select_part2_derived(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_expression_in_select_paren_derived5434);
        	        table_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:985:4: OPEN_PAR_SYMBOL select_paren_derived CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_select_paren_derived5439);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_select_paren_derived_in_select_paren_derived5441);
        	        select_paren_derived(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_select_paren_derived5443);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_paren_derivedEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_paren_derivedEx; /* Prevent compiler warnings */
    ruleselect_paren_derivedEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_paren_derived */

/**
 * $ANTLR start join_table
 * MySQLSimpleParser.g:988:1: join_table : ( ( INNER_SYMBOL | CROSS_SYMBOL )? JOIN_SYMBOL table_reference ( options {greedy=true; } : join_condition )? | STRAIGHT_JOIN_SYMBOL table_factor ( options {greedy=true; } : ON_SYMBOL expression )? | ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? JOIN_SYMBOL table_reference join_condition | NATURAL_SYMBOL ( ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? )? JOIN_SYMBOL table_factor );
 */
static void
join_table(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:988:11: ( ( INNER_SYMBOL | CROSS_SYMBOL )? JOIN_SYMBOL table_reference ( options {greedy=true; } : join_condition )? | STRAIGHT_JOIN_SYMBOL table_factor ( options {greedy=true; } : ON_SYMBOL expression )? | ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? JOIN_SYMBOL table_reference join_condition | NATURAL_SYMBOL ( ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? )? JOIN_SYMBOL table_factor )

            ANTLR3_UINT32 alt371;

            alt371=4;

            switch ( LA(1) )
            {
            case CROSS_SYMBOL:
            case INNER_SYMBOL:
            case JOIN_SYMBOL:
            	{
            		alt371=1;
            	}
                break;
            case STRAIGHT_JOIN_SYMBOL:
            	{
            		alt371=2;
            	}
                break;
            case LEFT_SYMBOL:
            case RIGHT_SYMBOL:
            	{
            		alt371=3;
            	}
                break;
            case NATURAL_SYMBOL:
            	{
            		alt371=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 371;
                EXCEPTION->state        = 0;


                goto rulejoin_tableEx;

            }

            switch (alt371)
            {
        	case 1:
        	    // MySQLSimpleParser.g:989:2: ( INNER_SYMBOL | CROSS_SYMBOL )? JOIN_SYMBOL table_reference ( options {greedy=true; } : join_condition )?
        	    {
        	        // MySQLSimpleParser.g:989:2: ( INNER_SYMBOL | CROSS_SYMBOL )?
        	        {
        	            int alt365=2;
        	            switch ( LA(1) )
        	            {
        	                case CROSS_SYMBOL:
        	                case INNER_SYMBOL:
        	                	{
        	                		alt365=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt365)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:
        	        	    {
        	        	        if ( LA(1) == CROSS_SYMBOL || LA(1) == INNER_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulejoin_tableEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(JOIN_SYMBOL, &FOLLOW_JOIN_SYMBOL_in_join_table5462);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_reference_in_join_table5464);
        	        table_reference(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:989:61: ( options {greedy=true; } : join_condition )?
        	        {
        	            int alt366=2;
        	            switch ( LA(1) )
        	            {
        	                case ON_SYMBOL:
        	                	{
        	                		alt366=1;
        	                	}
        	                    break;
        	                case USING_SYMBOL:
        	                	{
        	                		alt366=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt366)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:989:89: join_condition
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_join_condition_in_join_table5478);
        	        	        join_condition(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulejoin_tableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:990:4: STRAIGHT_JOIN_SYMBOL table_factor ( options {greedy=true; } : ON_SYMBOL expression )?
        	    {
        	         MATCHT(STRAIGHT_JOIN_SYMBOL, &FOLLOW_STRAIGHT_JOIN_SYMBOL_in_join_table5485);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_factor_in_join_table5487);
        	        table_factor(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:990:38: ( options {greedy=true; } : ON_SYMBOL expression )?
        	        {
        	            int alt367=2;
        	            switch ( LA(1) )
        	            {
        	                case ON_SYMBOL:
        	                	{
        	                		alt367=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt367)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:990:66: ON_SYMBOL expression
        	        	    {
        	        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_join_table5501);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulejoin_tableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_join_table5503);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulejoin_tableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:991:4: ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? JOIN_SYMBOL table_reference join_condition
        	    {
        	        if ( LA(1) == LEFT_SYMBOL || LA(1) == RIGHT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulejoin_tableEx;
        	        }


        	        // MySQLSimpleParser.g:991:33: ( OUTER_SYMBOL )?
        	        {
        	            int alt368=2;
        	            switch ( LA(1) )
        	            {
        	                case OUTER_SYMBOL:
        	                	{
        	                		alt368=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt368)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:991:33: OUTER_SYMBOL
        	        	    {
        	        	         MATCHT(OUTER_SYMBOL, &FOLLOW_OUTER_SYMBOL_in_join_table5518);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulejoin_tableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(JOIN_SYMBOL, &FOLLOW_JOIN_SYMBOL_in_join_table5521);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_reference_in_join_table5523);
        	        table_reference(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_join_condition_in_join_table5525);
        	        join_condition(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:992:4: NATURAL_SYMBOL ( ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? )? JOIN_SYMBOL table_factor
        	    {
        	         MATCHT(NATURAL_SYMBOL, &FOLLOW_NATURAL_SYMBOL_in_join_table5530);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:992:19: ( ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )? )?
        	        {
        	            int alt370=2;
        	            switch ( LA(1) )
        	            {
        	                case LEFT_SYMBOL:
        	                case RIGHT_SYMBOL:
        	                	{
        	                		alt370=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt370)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:992:20: ( LEFT_SYMBOL | RIGHT_SYMBOL ) ( OUTER_SYMBOL )?
        	        	    {
        	        	        if ( LA(1) == LEFT_SYMBOL || LA(1) == RIGHT_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulejoin_tableEx;
        	        	        }


        	        	        // MySQLSimpleParser.g:992:49: ( OUTER_SYMBOL )?
        	        	        {
        	        	            int alt369=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case OUTER_SYMBOL:
        	        	                	{
        	        	                		alt369=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt369)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:992:49: OUTER_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(OUTER_SYMBOL, &FOLLOW_OUTER_SYMBOL_in_join_table5541);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulejoin_tableEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(JOIN_SYMBOL, &FOLLOW_JOIN_SYMBOL_in_join_table5546);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_factor_in_join_table5548);
        	        table_factor(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_tableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulejoin_tableEx; /* Prevent compiler warnings */
    rulejoin_tableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end join_table */

/**
 * $ANTLR start join_condition
 * MySQLSimpleParser.g:995:1: join_condition : ( ON_SYMBOL expression | USING_SYMBOL identifier_list_with_parentheses );
 */
static void
join_condition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:995:15: ( ON_SYMBOL expression | USING_SYMBOL identifier_list_with_parentheses )

            ANTLR3_UINT32 alt372;

            alt372=2;

            switch ( LA(1) )
            {
            case ON_SYMBOL:
            	{
            		alt372=1;
            	}
                break;
            case USING_SYMBOL:
            	{
            		alt372=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 372;
                EXCEPTION->state        = 0;


                goto rulejoin_conditionEx;

            }

            switch (alt372)
            {
        	case 1:
        	    // MySQLSimpleParser.g:996:2: ON_SYMBOL expression
        	    {
        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_join_condition5557);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_join_condition5559);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:997:4: USING_SYMBOL identifier_list_with_parentheses
        	    {
        	         MATCHT(USING_SYMBOL, &FOLLOW_USING_SYMBOL_in_join_condition5564);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_join_condition5566);
        	        identifier_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejoin_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulejoin_conditionEx; /* Prevent compiler warnings */
    rulejoin_conditionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end join_condition */

/**
 * $ANTLR start union_clause
 * MySQLSimpleParser.g:1000:1: union_clause : UNION_SYMBOL ( union_option )? select_statement ;
 */
static void
union_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1000:13: ( UNION_SYMBOL ( union_option )? select_statement )
        // MySQLSimpleParser.g:1001:3: UNION_SYMBOL ( union_option )? select_statement
        {
             MATCHT(UNION_SYMBOL, &FOLLOW_UNION_SYMBOL_in_union_clause5576);
            if  (HASEXCEPTION())
            {
                goto ruleunion_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1001:16: ( union_option )?
            {
                int alt373=2;
                switch ( LA(1) )
                {
                    case ALL_SYMBOL:
                    case DISTINCT_SYMBOL:
                    	{
                    		alt373=1;
                    	}
                        break;
                }

                switch (alt373)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1001:16: union_option
            	    {
            	        FOLLOWPUSH(FOLLOW_union_option_in_union_clause5578);
            	        union_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleunion_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_select_statement_in_union_clause5581);
            select_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleunion_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleunion_clauseEx; /* Prevent compiler warnings */
    ruleunion_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end union_clause */

/**
 * $ANTLR start union_option
 * MySQLSimpleParser.g:1004:1: union_option : ( DISTINCT_SYMBOL | ALL_SYMBOL );
 */
static void
union_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1004:13: ( DISTINCT_SYMBOL | ALL_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == ALL_SYMBOL || LA(1) == DISTINCT_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleunion_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleunion_optionEx; /* Prevent compiler warnings */
    ruleunion_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end union_option */

/**
 * $ANTLR start union_or_order_by_or_limit
 * MySQLSimpleParser.g:1009:1: union_or_order_by_or_limit : ( union_clause | order_by_or_limit );
 */
static void
union_or_order_by_or_limit(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1009:27: ( union_clause | order_by_or_limit )

            ANTLR3_UINT32 alt374;

            alt374=2;

            switch ( LA(1) )
            {
            case UNION_SYMBOL:
            	{
            		alt374=1;
            	}
                break;
            case LIMIT_SYMBOL:
            case ORDER_SYMBOL:
            	{
            		alt374=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 374;
                EXCEPTION->state        = 0;


                goto ruleunion_or_order_by_or_limitEx;

            }

            switch (alt374)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1010:2: union_clause
        	    {
        	        FOLLOWPUSH(FOLLOW_union_clause_in_union_or_order_by_or_limit5606);
        	        union_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunion_or_order_by_or_limitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1011:4: order_by_or_limit
        	    {
        	        FOLLOWPUSH(FOLLOW_order_by_or_limit_in_union_or_order_by_or_limit5611);
        	        order_by_or_limit(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunion_or_order_by_or_limitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleunion_or_order_by_or_limitEx; /* Prevent compiler warnings */
    ruleunion_or_order_by_or_limitEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end union_or_order_by_or_limit */

/**
 * $ANTLR start select_lock_type
 * MySQLSimpleParser.g:1014:1: select_lock_type : ( FOR_SYMBOL UPDATE_SYMBOL | LOCK_SYMBOL IN_SYMBOL SHARE_SYMBOL MODE_SYMBOL );
 */
static void
select_lock_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1014:17: ( FOR_SYMBOL UPDATE_SYMBOL | LOCK_SYMBOL IN_SYMBOL SHARE_SYMBOL MODE_SYMBOL )

            ANTLR3_UINT32 alt375;

            alt375=2;

            switch ( LA(1) )
            {
            case FOR_SYMBOL:
            	{
            		alt375=1;
            	}
                break;
            case LOCK_SYMBOL:
            	{
            		alt375=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 375;
                EXCEPTION->state        = 0;


                goto ruleselect_lock_typeEx;

            }

            switch (alt375)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1015:2: FOR_SYMBOL UPDATE_SYMBOL
        	    {
        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_select_lock_type5620);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_select_lock_type5622);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1016:4: LOCK_SYMBOL IN_SYMBOL SHARE_SYMBOL MODE_SYMBOL
        	    {
        	         MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_select_lock_type5627);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_select_lock_type5629);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SHARE_SYMBOL, &FOLLOW_SHARE_SYMBOL_in_select_lock_type5631);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(MODE_SYMBOL, &FOLLOW_MODE_SYMBOL_in_select_lock_type5633);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleselect_lock_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleselect_lock_typeEx; /* Prevent compiler warnings */
    ruleselect_lock_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end select_lock_type */

/**
 * $ANTLR start table_alias
 * MySQLSimpleParser.g:1019:1: table_alias : ( AS_SYMBOL | EQUAL_OPERATOR )? identifier ;
 */
static void
table_alias(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1019:12: ( ( AS_SYMBOL | EQUAL_OPERATOR )? identifier )
        // MySQLSimpleParser.g:1020:2: ( AS_SYMBOL | EQUAL_OPERATOR )? identifier
        {
            // MySQLSimpleParser.g:1020:2: ( AS_SYMBOL | EQUAL_OPERATOR )?
            {
                int alt376=2;
                switch ( LA(1) )
                {
                    case AS_SYMBOL:
                    case EQUAL_OPERATOR:
                    	{
                    		alt376=1;
                    	}
                        break;
                }

                switch (alt376)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == AS_SYMBOL || LA(1) == EQUAL_OPERATOR )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruletable_aliasEx;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_identifier_in_table_alias5651);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_aliasEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_aliasEx; /* Prevent compiler warnings */
    ruletable_aliasEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_alias */

/**
 * $ANTLR start index_hint_list
 * MySQLSimpleParser.g:1023:1: index_hint_list : index_hint ( COMMA_SYMBOL index_hint )* ;
 */
static void
index_hint_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1023:16: ( index_hint ( COMMA_SYMBOL index_hint )* )
        // MySQLSimpleParser.g:1024:2: index_hint ( COMMA_SYMBOL index_hint )*
        {
            FOLLOWPUSH(FOLLOW_index_hint_in_index_hint_list5660);
            index_hint(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_hint_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1024:13: ( COMMA_SYMBOL index_hint )*

            for (;;)
            {
                int alt377=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case FORCE_SYMBOL:
                		case IGNORE_SYMBOL:
                		case USE_SYMBOL:
                			{
                				alt377=1;
                			}
                		    break;

                		}

                	}
                    break;

                }

                switch (alt377)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1024:14: COMMA_SYMBOL index_hint
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_index_hint_list5663);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_hint_in_index_hint_list5665);
            	        index_hint(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop377;	/* break out of the loop */
            	    break;
                }
            }
            loop377: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_hint_listEx; /* Prevent compiler warnings */
    ruleindex_hint_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_hint_list */

/**
 * $ANTLR start index_hint
 * MySQLSimpleParser.g:1027:1: index_hint : ( index_hint_type key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL index_list CLOSE_PAR_SYMBOL | USE_SYMBOL key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL ( index_list )? CLOSE_PAR_SYMBOL );
 */
static void
index_hint(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1027:11: ( index_hint_type key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL index_list CLOSE_PAR_SYMBOL | USE_SYMBOL key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL ( index_list )? CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt381;

            alt381=2;

            switch ( LA(1) )
            {
            case FORCE_SYMBOL:
            case IGNORE_SYMBOL:
            	{
            		alt381=1;
            	}
                break;
            case USE_SYMBOL:
            	{
            		alt381=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 381;
                EXCEPTION->state        = 0;


                goto ruleindex_hintEx;

            }

            switch (alt381)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1028:2: index_hint_type key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL index_list CLOSE_PAR_SYMBOL
        	    {
        	        FOLLOWPUSH(FOLLOW_index_hint_type_in_index_hint5676);
        	        index_hint_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_key_or_index_in_index_hint5678);
        	        key_or_index(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1028:31: ( index_hint_clause )?
        	        {
        	            int alt378=2;
        	            switch ( LA(1) )
        	            {
        	                case FOR_SYMBOL:
        	                	{
        	                		alt378=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt378)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1028:31: index_hint_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_index_hint_clause_in_index_hint5680);
        	        	        index_hint_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleindex_hintEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5683);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_index_list_in_index_hint5685);
        	        index_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5687);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1029:4: USE_SYMBOL key_or_index ( index_hint_clause )? OPEN_PAR_SYMBOL ( index_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(USE_SYMBOL, &FOLLOW_USE_SYMBOL_in_index_hint5692);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_key_or_index_in_index_hint5694);
        	        key_or_index(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1029:28: ( index_hint_clause )?
        	        {
        	            int alt379=2;
        	            switch ( LA(1) )
        	            {
        	                case FOR_SYMBOL:
        	                	{
        	                		alt379=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt379)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1029:28: index_hint_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_index_hint_clause_in_index_hint5696);
        	        	        index_hint_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleindex_hintEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_index_hint5699);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1029:63: ( index_list )?
        	        {
        	            int alt380=2;
        	            {
        	                int LA380_0 = LA(1);
        	                if ( (((LA380_0 >= ACCOUNT_SYMBOL) && (LA380_0 <= YEAR_SYMBOL)) || LA380_0 == PRIMARY_SYMBOL || LA380_0 == BACK_TICK_QUOTED_ID || LA380_0 == DOUBLE_QUOTED_TEXT || LA380_0 == IDENTIFIER))
        	                {
        	                    alt380=1;
        	                }
        	            }
        	            switch (alt380)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1029:63: index_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_index_list_in_index_hint5701);
        	        	        index_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleindex_hintEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_index_hint5704);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_hintEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleindex_hintEx; /* Prevent compiler warnings */
    ruleindex_hintEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_hint */

/**
 * $ANTLR start index_hint_type
 * MySQLSimpleParser.g:1032:1: index_hint_type : ( FORCE_SYMBOL | IGNORE_SYMBOL );
 */
static void
index_hint_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1032:16: ( FORCE_SYMBOL | IGNORE_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == FORCE_SYMBOL || LA(1) == IGNORE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleindex_hint_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_hint_typeEx; /* Prevent compiler warnings */
    ruleindex_hint_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_hint_type */

/**
 * $ANTLR start key_or_index
 * MySQLSimpleParser.g:1037:1: key_or_index : ( KEY_SYMBOL | INDEX_SYMBOL );
 */
static void
key_or_index(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1037:13: ( KEY_SYMBOL | INDEX_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulekey_or_indexEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulekey_or_indexEx; /* Prevent compiler warnings */
    rulekey_or_indexEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end key_or_index */

/**
 * $ANTLR start index_hint_clause
 * MySQLSimpleParser.g:1042:1: index_hint_clause : FOR_SYMBOL ( JOIN_SYMBOL | ORDER_SYMBOL BY_SYMBOL | GROUP_SYMBOL BY_SYMBOL ) ;
 */
static void
index_hint_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1042:18: ( FOR_SYMBOL ( JOIN_SYMBOL | ORDER_SYMBOL BY_SYMBOL | GROUP_SYMBOL BY_SYMBOL ) )
        // MySQLSimpleParser.g:1043:2: FOR_SYMBOL ( JOIN_SYMBOL | ORDER_SYMBOL BY_SYMBOL | GROUP_SYMBOL BY_SYMBOL )
        {
             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_index_hint_clause5741);
            if  (HASEXCEPTION())
            {
                goto ruleindex_hint_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1043:13: ( JOIN_SYMBOL | ORDER_SYMBOL BY_SYMBOL | GROUP_SYMBOL BY_SYMBOL )
            {
                int alt382=3;
                switch ( LA(1) )
                {
                case JOIN_SYMBOL:
                	{
                		alt382=1;
                	}
                    break;
                case ORDER_SYMBOL:
                	{
                		alt382=2;
                	}
                    break;
                case GROUP_SYMBOL:
                	{
                		alt382=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 382;
                    EXCEPTION->state        = 0;


                    goto ruleindex_hint_clauseEx;

                }

                switch (alt382)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1043:14: JOIN_SYMBOL
            	    {
            	         MATCHT(JOIN_SYMBOL, &FOLLOW_JOIN_SYMBOL_in_index_hint_clause5744);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1043:28: ORDER_SYMBOL BY_SYMBOL
            	    {
            	         MATCHT(ORDER_SYMBOL, &FOLLOW_ORDER_SYMBOL_in_index_hint_clause5748);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_index_hint_clause5750);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1043:53: GROUP_SYMBOL BY_SYMBOL
            	    {
            	         MATCHT(GROUP_SYMBOL, &FOLLOW_GROUP_SYMBOL_in_index_hint_clause5754);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_index_hint_clause5756);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_hint_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_hint_clauseEx; /* Prevent compiler warnings */
    ruleindex_hint_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_hint_clause */

/**
 * $ANTLR start index_list
 * MySQLSimpleParser.g:1046:1: index_list : index_list_element ( COMMA_SYMBOL index_list_element )* ;
 */
static void
index_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1046:11: ( index_list_element ( COMMA_SYMBOL index_list_element )* )
        // MySQLSimpleParser.g:1047:2: index_list_element ( COMMA_SYMBOL index_list_element )*
        {
            FOLLOWPUSH(FOLLOW_index_list_element_in_index_list5766);
            index_list_element(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1047:21: ( COMMA_SYMBOL index_list_element )*

            for (;;)
            {
                int alt383=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt383=1;
                	}
                    break;

                }

                switch (alt383)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1047:22: COMMA_SYMBOL index_list_element
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_index_list5769);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_list_element_in_index_list5771);
            	        index_list_element(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop383;	/* break out of the loop */
            	    break;
                }
            }
            loop383: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_listEx; /* Prevent compiler warnings */
    ruleindex_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_list */

/**
 * $ANTLR start index_list_element
 * MySQLSimpleParser.g:1050:1: index_list_element : ( identifier | PRIMARY_SYMBOL );
 */
static void
index_list_element(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1050:19: ( identifier | PRIMARY_SYMBOL )

            ANTLR3_UINT32 alt384;

            alt384=2;

            {
                int LA384_0 = LA(1);
                if ( (((LA384_0 >= ACCOUNT_SYMBOL) && (LA384_0 <= YEAR_SYMBOL)) || LA384_0 == BACK_TICK_QUOTED_ID || LA384_0 == DOUBLE_QUOTED_TEXT || LA384_0 == IDENTIFIER))
                {
                    alt384=1;
                }
                else if ( (LA384_0 == PRIMARY_SYMBOL))
                {
                    alt384=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 384;
                    EXCEPTION->state        = 0;


                    goto ruleindex_list_elementEx;

                }
            }
            switch (alt384)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1051:2: identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_index_list_element5782);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1052:4: PRIMARY_SYMBOL
        	    {
        	         MATCHT(PRIMARY_SYMBOL, &FOLLOW_PRIMARY_SYMBOL_in_index_list_element5787);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleindex_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleindex_list_elementEx; /* Prevent compiler warnings */
    ruleindex_list_elementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_list_element */

/**
 * $ANTLR start update_statement
 * MySQLSimpleParser.g:1057:1: update_statement : UPDATE_SYMBOL ( LOW_PRIORITY_SYMBOL )? ( IGNORE_SYMBOL )? join_table_list SET_SYMBOL column_assignment_list_with_default ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? ;
 */
static void
update_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1057:17: ( UPDATE_SYMBOL ( LOW_PRIORITY_SYMBOL )? ( IGNORE_SYMBOL )? join_table_list SET_SYMBOL column_assignment_list_with_default ( where_clause )? ( order_by_clause )? ( simple_limit_clause )? )
        // MySQLSimpleParser.g:1058:2: UPDATE_SYMBOL ( LOW_PRIORITY_SYMBOL )? ( IGNORE_SYMBOL )? join_table_list SET_SYMBOL column_assignment_list_with_default ( where_clause )? ( order_by_clause )? ( simple_limit_clause )?
        {
             MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_update_statement5798);
            if  (HASEXCEPTION())
            {
                goto ruleupdate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1058:16: ( LOW_PRIORITY_SYMBOL )?
            {
                int alt385=2;
                switch ( LA(1) )
                {
                    case LOW_PRIORITY_SYMBOL:
                    	{
                    		alt385=1;
                    	}
                        break;
                }

                switch (alt385)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1058:16: LOW_PRIORITY_SYMBOL
            	    {
            	         MATCHT(LOW_PRIORITY_SYMBOL, &FOLLOW_LOW_PRIORITY_SYMBOL_in_update_statement5800);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleupdate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1058:37: ( IGNORE_SYMBOL )?
            {
                int alt386=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    	{
                    		alt386=1;
                    	}
                        break;
                }

                switch (alt386)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1058:37: IGNORE_SYMBOL
            	    {
            	         MATCHT(IGNORE_SYMBOL, &FOLLOW_IGNORE_SYMBOL_in_update_statement5803);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleupdate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_join_table_list_in_update_statement5806);
            join_table_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleupdate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_update_statement5810);
            if  (HASEXCEPTION())
            {
                goto ruleupdate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_column_assignment_list_with_default_in_update_statement5812);
            column_assignment_list_with_default(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleupdate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1059:50: ( where_clause )?
            {
                int alt387=2;
                switch ( LA(1) )
                {
                    case WHERE_SYMBOL:
                    	{
                    		alt387=1;
                    	}
                        break;
                }

                switch (alt387)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1059:50: where_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_where_clause_in_update_statement5814);
            	        where_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleupdate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1059:64: ( order_by_clause )?
            {
                int alt388=2;
                switch ( LA(1) )
                {
                    case ORDER_SYMBOL:
                    	{
                    		alt388=1;
                    	}
                        break;
                }

                switch (alt388)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1059:64: order_by_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_order_by_clause_in_update_statement5817);
            	        order_by_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleupdate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1059:81: ( simple_limit_clause )?
            {
                int alt389=2;
                switch ( LA(1) )
                {
                    case LIMIT_SYMBOL:
                    	{
                    		alt389=1;
                    	}
                        break;
                }

                switch (alt389)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1059:81: simple_limit_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_simple_limit_clause_in_update_statement5820);
            	        simple_limit_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleupdate_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleupdate_statementEx; /* Prevent compiler warnings */
    ruleupdate_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end update_statement */

/**
 * $ANTLR start transaction_or_locking_statement
 * MySQLSimpleParser.g:1064:1: transaction_or_locking_statement : ( transaction_statement | savepoint_statement | lock_statement | xa_statement );
 */
static void
transaction_or_locking_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1064:33: ( transaction_statement | savepoint_statement | lock_statement | xa_statement )

            ANTLR3_UINT32 alt390;

            alt390=4;

            switch ( LA(1) )
            {
            case COMMIT_SYMBOL:
            case ROLLBACK_SYMBOL:
            case START_SYMBOL:
            	{
            		alt390=1;
            	}
                break;
            case SAVEPOINT_SYMBOL:
            case RELEASE_SYMBOL:
            	{
            		alt390=2;
            	}
                break;
            case LOCK_SYMBOL:
            case UNLOCK_SYMBOL:
            	{
            		alt390=3;
            	}
                break;
            case XA_SYMBOL:
            	{
            		alt390=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 390;
                EXCEPTION->state        = 0;


                goto ruletransaction_or_locking_statementEx;

            }

            switch (alt390)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1065:2: transaction_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_transaction_statement_in_transaction_or_locking_statement5832);
        	        transaction_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_or_locking_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1066:4: savepoint_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_savepoint_statement_in_transaction_or_locking_statement5837);
        	        savepoint_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_or_locking_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1067:4: lock_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_lock_statement_in_transaction_or_locking_statement5842);
        	        lock_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_or_locking_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1068:4: xa_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_xa_statement_in_transaction_or_locking_statement5847);
        	        xa_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_or_locking_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletransaction_or_locking_statementEx; /* Prevent compiler warnings */
    ruletransaction_or_locking_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end transaction_or_locking_statement */

/**
 * $ANTLR start transaction_statement
 * MySQLSimpleParser.g:1071:1: transaction_statement : ( START_SYMBOL TRANSACTION_SYMBOL ( transaction_characteristic )* | COMMIT_SYMBOL ( WORK_SYMBOL )? ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | ROLLBACK_SYMBOL ( WORK_SYMBOL )? ( ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | TO_SYMBOL ( SAVEPOINT_SYMBOL )? identifier ) );
 */
static void
transaction_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1071:22: ( START_SYMBOL TRANSACTION_SYMBOL ( transaction_characteristic )* | COMMIT_SYMBOL ( WORK_SYMBOL )? ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | ROLLBACK_SYMBOL ( WORK_SYMBOL )? ( ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | TO_SYMBOL ( SAVEPOINT_SYMBOL )? identifier ) )

            ANTLR3_UINT32 alt404;

            alt404=3;

            switch ( LA(1) )
            {
            case START_SYMBOL:
            	{
            		alt404=1;
            	}
                break;
            case COMMIT_SYMBOL:
            	{
            		alt404=2;
            	}
                break;
            case ROLLBACK_SYMBOL:
            	{
            		alt404=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 404;
                EXCEPTION->state        = 0;


                goto ruletransaction_statementEx;

            }

            switch (alt404)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1072:2: START_SYMBOL TRANSACTION_SYMBOL ( transaction_characteristic )*
        	    {
        	         MATCHT(START_SYMBOL, &FOLLOW_START_SYMBOL_in_transaction_statement5856);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TRANSACTION_SYMBOL, &FOLLOW_TRANSACTION_SYMBOL_in_transaction_statement5858);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1072:34: ( transaction_characteristic )*

        	        for (;;)
        	        {
        	            int alt391=2;
        	            switch ( LA(1) )
        	            {
        	            case READ_SYMBOL:
        	            case WITH_SYMBOL:
        	            	{
        	            		alt391=1;
        	            	}
        	                break;

        	            }

        	            switch (alt391)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1072:34: transaction_characteristic
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_transaction_characteristic_in_transaction_statement5860);
        	        	        transaction_characteristic(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop391;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop391: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1073:4: COMMIT_SYMBOL ( WORK_SYMBOL )? ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )?
        	    {
        	         MATCHT(COMMIT_SYMBOL, &FOLLOW_COMMIT_SYMBOL_in_transaction_statement5866);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1073:18: ( WORK_SYMBOL )?
        	        {
        	            int alt392=2;
        	            switch ( LA(1) )
        	            {
        	                case WORK_SYMBOL:
        	                	{
        	                		alt392=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt392)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1073:18: WORK_SYMBOL
        	        	    {
        	        	         MATCHT(WORK_SYMBOL, &FOLLOW_WORK_SYMBOL_in_transaction_statement5868);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1073:31: ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )?
        	        {
        	            int alt394=2;
        	            switch ( LA(1) )
        	            {
        	                case AND_SYMBOL:
        	                	{
        	                		alt394=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt394)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1073:32: AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL
        	        	    {
        	        	         MATCHT(AND_SYMBOL, &FOLLOW_AND_SYMBOL_in_transaction_statement5872);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:1073:43: ( NO_SYMBOL )?
        	        	        {
        	        	            int alt393=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case NO_SYMBOL:
        	        	                	{
        	        	                		alt393=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt393)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1073:43: NO_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_transaction_statement5874);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	         MATCHT(CHAIN_SYMBOL, &FOLLOW_CHAIN_SYMBOL_in_transaction_statement5877);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1073:69: ( ( NO_SYMBOL )? RELEASE_SYMBOL )?
        	        {
        	            int alt396=2;
        	            switch ( LA(1) )
        	            {
        	                case NO_SYMBOL:
        	                case RELEASE_SYMBOL:
        	                	{
        	                		alt396=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt396)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1073:70: ( NO_SYMBOL )? RELEASE_SYMBOL
        	        	    {
        	        	        // MySQLSimpleParser.g:1073:70: ( NO_SYMBOL )?
        	        	        {
        	        	            int alt395=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case NO_SYMBOL:
        	        	                	{
        	        	                		alt395=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt395)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1073:70: NO_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_transaction_statement5882);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	         MATCHT(RELEASE_SYMBOL, &FOLLOW_RELEASE_SYMBOL_in_transaction_statement5885);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1074:4: ROLLBACK_SYMBOL ( WORK_SYMBOL )? ( ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | TO_SYMBOL ( SAVEPOINT_SYMBOL )? identifier )
        	    {
        	         MATCHT(ROLLBACK_SYMBOL, &FOLLOW_ROLLBACK_SYMBOL_in_transaction_statement5892);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1074:20: ( WORK_SYMBOL )?
        	        {
        	            int alt397=2;
        	            switch ( LA(1) )
        	            {
        	                case WORK_SYMBOL:
        	                	{
        	                		alt397=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt397)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1074:20: WORK_SYMBOL
        	        	    {
        	        	         MATCHT(WORK_SYMBOL, &FOLLOW_WORK_SYMBOL_in_transaction_statement5894);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1075:3: ( ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )? | TO_SYMBOL ( SAVEPOINT_SYMBOL )? identifier )
        	        {
        	            int alt403=2;
        	            switch ( LA(1) )
        	            {
        	            case EOF:
        	            case NO_SYMBOL:
        	            case AND_SYMBOL:
        	            case RELEASE_SYMBOL:
        	            case SEMICOLON_SYMBOL:
        	            	{
        	            		alt403=1;
        	            	}
        	                break;
        	            case TO_SYMBOL:
        	            	{
        	            		alt403=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 403;
        	                EXCEPTION->state        = 0;


        	                goto ruletransaction_statementEx;

        	            }

        	            switch (alt403)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1076:4: ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )? ( ( NO_SYMBOL )? RELEASE_SYMBOL )?
        	        	    {
        	        	        // MySQLSimpleParser.g:1076:4: ( AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL )?
        	        	        {
        	        	            int alt399=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case AND_SYMBOL:
        	        	                	{
        	        	                		alt399=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt399)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1076:5: AND_SYMBOL ( NO_SYMBOL )? CHAIN_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(AND_SYMBOL, &FOLLOW_AND_SYMBOL_in_transaction_statement5905);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        // MySQLSimpleParser.g:1076:16: ( NO_SYMBOL )?
        	        	        	        {
        	        	        	            int alt398=2;
        	        	        	            switch ( LA(1) )
        	        	        	            {
        	        	        	                case NO_SYMBOL:
        	        	        	                	{
        	        	        	                		alt398=1;
        	        	        	                	}
        	        	        	                    break;
        	        	        	            }

        	        	        	            switch (alt398)
        	        	        	            {
        	        	        	        	case 1:
        	        	        	        	    // MySQLSimpleParser.g:1076:16: NO_SYMBOL
        	        	        	        	    {
        	        	        	        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_transaction_statement5907);
        	        	        	        	        if  (HASEXCEPTION())
        	        	        	        	        {
        	        	        	        	            goto ruletransaction_statementEx;
        	        	        	        	        }
        	        	        	        	        if (HASFAILED())
        	        	        	        	        {
        	        	        	        	            return ;
        	        	        	        	        }


        	        	        	        	    }
        	        	        	        	    break;

        	        	        	            }
        	        	        	        }

        	        	        	         MATCHT(CHAIN_SYMBOL, &FOLLOW_CHAIN_SYMBOL_in_transaction_statement5910);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        // MySQLSimpleParser.g:1076:42: ( ( NO_SYMBOL )? RELEASE_SYMBOL )?
        	        	        {
        	        	            int alt401=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case NO_SYMBOL:
        	        	                case RELEASE_SYMBOL:
        	        	                	{
        	        	                		alt401=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt401)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1076:43: ( NO_SYMBOL )? RELEASE_SYMBOL
        	        	        	    {
        	        	        	        // MySQLSimpleParser.g:1076:43: ( NO_SYMBOL )?
        	        	        	        {
        	        	        	            int alt400=2;
        	        	        	            switch ( LA(1) )
        	        	        	            {
        	        	        	                case NO_SYMBOL:
        	        	        	                	{
        	        	        	                		alt400=1;
        	        	        	                	}
        	        	        	                    break;
        	        	        	            }

        	        	        	            switch (alt400)
        	        	        	            {
        	        	        	        	case 1:
        	        	        	        	    // MySQLSimpleParser.g:1076:43: NO_SYMBOL
        	        	        	        	    {
        	        	        	        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_transaction_statement5915);
        	        	        	        	        if  (HASEXCEPTION())
        	        	        	        	        {
        	        	        	        	            goto ruletransaction_statementEx;
        	        	        	        	        }
        	        	        	        	        if (HASFAILED())
        	        	        	        	        {
        	        	        	        	            return ;
        	        	        	        	        }


        	        	        	        	    }
        	        	        	        	    break;

        	        	        	            }
        	        	        	        }

        	        	        	         MATCHT(RELEASE_SYMBOL, &FOLLOW_RELEASE_SYMBOL_in_transaction_statement5918);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1077:6: TO_SYMBOL ( SAVEPOINT_SYMBOL )? identifier
        	        	    {
        	        	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_transaction_statement5927);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:1077:16: ( SAVEPOINT_SYMBOL )?
        	        	        {
        	        	            int alt402=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case SAVEPOINT_SYMBOL:
        	        	                	{
        	        	                		{
        	        	                		    int LA402_1 = LA(2);
        	        	                		    if ( (((LA402_1 >= ACCOUNT_SYMBOL) && (LA402_1 <= YEAR_SYMBOL)) || LA402_1 == BACK_TICK_QUOTED_ID || LA402_1 == DOUBLE_QUOTED_TEXT || LA402_1 == IDENTIFIER))
        	        	                		    {
        	        	                		        alt402=1;
        	        	                		    }
        	        	                		}
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt402)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1077:16: SAVEPOINT_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(SAVEPOINT_SYMBOL, &FOLLOW_SAVEPOINT_SYMBOL_in_transaction_statement5929);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruletransaction_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_identifier_in_transaction_statement5932);
        	        	        identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletransaction_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletransaction_statementEx; /* Prevent compiler warnings */
    ruletransaction_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end transaction_statement */

/**
 * $ANTLR start begin_work
 * MySQLSimpleParser.g:1085:1: begin_work : BEGIN_SYMBOL ( WORK_SYMBOL )? ;
 */
static void
begin_work(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1085:11: ( BEGIN_SYMBOL ( WORK_SYMBOL )? )
        // MySQLSimpleParser.g:1086:2: BEGIN_SYMBOL ( WORK_SYMBOL )?
        {
             MATCHT(BEGIN_SYMBOL, &FOLLOW_BEGIN_SYMBOL_in_begin_work5953);
            if  (HASEXCEPTION())
            {
                goto rulebegin_workEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1086:15: ( WORK_SYMBOL )?
            {
                int alt405=2;
                switch ( LA(1) )
                {
                    case WORK_SYMBOL:
                    	{
                    		alt405=1;
                    	}
                        break;
                }

                switch (alt405)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1086:15: WORK_SYMBOL
            	    {
            	         MATCHT(WORK_SYMBOL, &FOLLOW_WORK_SYMBOL_in_begin_work5955);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebegin_workEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulebegin_workEx; /* Prevent compiler warnings */
    rulebegin_workEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end begin_work */

/**
 * $ANTLR start transaction_characteristic
 * MySQLSimpleParser.g:1089:1: transaction_characteristic : ( WITH_SYMBOL CONSISTENT_SYMBOL SNAPSHOT_SYMBOL |{...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL ) );
 */
static void
transaction_characteristic(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1089:27: ( WITH_SYMBOL CONSISTENT_SYMBOL SNAPSHOT_SYMBOL |{...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL ) )

            ANTLR3_UINT32 alt406;

            alt406=2;

            switch ( LA(1) )
            {
            case WITH_SYMBOL:
            	{
            		alt406=1;
            	}
                break;
            case READ_SYMBOL:
            	{
            		alt406=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 406;
                EXCEPTION->state        = 0;


                goto ruletransaction_characteristicEx;

            }

            switch (alt406)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1090:2: WITH_SYMBOL CONSISTENT_SYMBOL SNAPSHOT_SYMBOL
        	    {
        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_transaction_characteristic5965);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CONSISTENT_SYMBOL, &FOLLOW_CONSISTENT_SYMBOL_in_transaction_characteristic5967);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SNAPSHOT_SYMBOL, &FOLLOW_SNAPSHOT_SYMBOL_in_transaction_characteristic5969);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1091:4: {...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL )
        	    {
        	        if ( !((SERVER_VERSION >= 50605)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50605";
        	                EXCEPTION->ruleName	 = (void *)"transaction_characteristic";


        	        }

        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_transaction_characteristic5976);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletransaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == ONLY_SYMBOL || LA(1) == WRITE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruletransaction_characteristicEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletransaction_characteristicEx; /* Prevent compiler warnings */
    ruletransaction_characteristicEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end transaction_characteristic */

/**
 * $ANTLR start set_transaction_characteristic
 * MySQLSimpleParser.g:1094:1: set_transaction_characteristic : ( ISOLATION_SYMBOL LEVEL_SYMBOL isolation_level |{...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL ) );
 */
static void
set_transaction_characteristic(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1094:31: ( ISOLATION_SYMBOL LEVEL_SYMBOL isolation_level |{...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL ) )

            ANTLR3_UINT32 alt407;

            alt407=2;

            switch ( LA(1) )
            {
            case ISOLATION_SYMBOL:
            	{
            		alt407=1;
            	}
                break;
            case READ_SYMBOL:
            	{
            		alt407=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 407;
                EXCEPTION->state        = 0;


                goto ruleset_transaction_characteristicEx;

            }

            switch (alt407)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1095:2: ISOLATION_SYMBOL LEVEL_SYMBOL isolation_level
        	    {
        	         MATCHT(ISOLATION_SYMBOL, &FOLLOW_ISOLATION_SYMBOL_in_set_transaction_characteristic5993);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_transaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(LEVEL_SYMBOL, &FOLLOW_LEVEL_SYMBOL_in_set_transaction_characteristic5995);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_transaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_isolation_level_in_set_transaction_characteristic5997);
        	        isolation_level(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_transaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1096:4: {...}? READ_SYMBOL ( WRITE_SYMBOL | ONLY_SYMBOL )
        	    {
        	        if ( !((SERVER_VERSION >= 50605)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50605";
        	                EXCEPTION->ruleName	 = (void *)"set_transaction_characteristic";


        	        }

        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_set_transaction_characteristic6004);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_transaction_characteristicEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == ONLY_SYMBOL || LA(1) == WRITE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleset_transaction_characteristicEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleset_transaction_characteristicEx; /* Prevent compiler warnings */
    ruleset_transaction_characteristicEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end set_transaction_characteristic */

/**
 * $ANTLR start isolation_level
 * MySQLSimpleParser.g:1099:1: isolation_level : ( REPEATABLE_SYMBOL READ_SYMBOL | READ_SYMBOL ( COMMITTED_SYMBOL | UNCOMMITTED_SYMBOL ) | SERIALIZABLE_SYMBOL );
 */
static void
isolation_level(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1099:16: ( REPEATABLE_SYMBOL READ_SYMBOL | READ_SYMBOL ( COMMITTED_SYMBOL | UNCOMMITTED_SYMBOL ) | SERIALIZABLE_SYMBOL )

            ANTLR3_UINT32 alt408;

            alt408=3;

            switch ( LA(1) )
            {
            case REPEATABLE_SYMBOL:
            	{
            		alt408=1;
            	}
                break;
            case READ_SYMBOL:
            	{
            		alt408=2;
            	}
                break;
            case SERIALIZABLE_SYMBOL:
            	{
            		alt408=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 408;
                EXCEPTION->state        = 0;


                goto ruleisolation_levelEx;

            }

            switch (alt408)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1100:2: REPEATABLE_SYMBOL READ_SYMBOL
        	    {
        	         MATCHT(REPEATABLE_SYMBOL, &FOLLOW_REPEATABLE_SYMBOL_in_isolation_level6021);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleisolation_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_isolation_level6023);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleisolation_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1101:4: READ_SYMBOL ( COMMITTED_SYMBOL | UNCOMMITTED_SYMBOL )
        	    {
        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_isolation_level6028);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleisolation_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == COMMITTED_SYMBOL || LA(1) == UNCOMMITTED_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleisolation_levelEx;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1102:4: SERIALIZABLE_SYMBOL
        	    {
        	         MATCHT(SERIALIZABLE_SYMBOL, &FOLLOW_SERIALIZABLE_SYMBOL_in_isolation_level6041);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleisolation_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleisolation_levelEx; /* Prevent compiler warnings */
    ruleisolation_levelEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end isolation_level */

/**
 * $ANTLR start savepoint_statement
 * MySQLSimpleParser.g:1105:1: savepoint_statement : ( SAVEPOINT_SYMBOL identifier | RELEASE_SYMBOL SAVEPOINT_SYMBOL identifier );
 */
static void
savepoint_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1105:20: ( SAVEPOINT_SYMBOL identifier | RELEASE_SYMBOL SAVEPOINT_SYMBOL identifier )

            ANTLR3_UINT32 alt409;

            alt409=2;

            switch ( LA(1) )
            {
            case SAVEPOINT_SYMBOL:
            	{
            		alt409=1;
            	}
                break;
            case RELEASE_SYMBOL:
            	{
            		alt409=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 409;
                EXCEPTION->state        = 0;


                goto rulesavepoint_statementEx;

            }

            switch (alt409)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1106:2: SAVEPOINT_SYMBOL identifier
        	    {
        	         MATCHT(SAVEPOINT_SYMBOL, &FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6050);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesavepoint_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_savepoint_statement6052);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesavepoint_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1107:4: RELEASE_SYMBOL SAVEPOINT_SYMBOL identifier
        	    {
        	         MATCHT(RELEASE_SYMBOL, &FOLLOW_RELEASE_SYMBOL_in_savepoint_statement6057);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesavepoint_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SAVEPOINT_SYMBOL, &FOLLOW_SAVEPOINT_SYMBOL_in_savepoint_statement6059);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesavepoint_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_savepoint_statement6061);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesavepoint_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulesavepoint_statementEx; /* Prevent compiler warnings */
    rulesavepoint_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end savepoint_statement */

/**
 * $ANTLR start lock_statement
 * MySQLSimpleParser.g:1110:1: lock_statement : ( LOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL ) lock_item ( COMMA_SYMBOL lock_item )* | UNLOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL ) );
 */
static void
lock_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1110:15: ( LOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL ) lock_item ( COMMA_SYMBOL lock_item )* | UNLOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL ) )

            ANTLR3_UINT32 alt411;

            alt411=2;

            switch ( LA(1) )
            {
            case LOCK_SYMBOL:
            	{
            		alt411=1;
            	}
                break;
            case UNLOCK_SYMBOL:
            	{
            		alt411=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 411;
                EXCEPTION->state        = 0;


                goto rulelock_statementEx;

            }

            switch (alt411)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1111:2: LOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL ) lock_item ( COMMA_SYMBOL lock_item )*
        	    {
        	         MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_lock_statement6070);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelock_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == TABLES_SYMBOL || LA(1) == TABLE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulelock_statementEx;
        	        }


        	        FOLLOWPUSH(FOLLOW_lock_item_in_lock_statement6080);
        	        lock_item(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelock_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1111:55: ( COMMA_SYMBOL lock_item )*

        	        for (;;)
        	        {
        	            int alt410=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt410=1;
        	            	}
        	                break;

        	            }

        	            switch (alt410)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1111:56: COMMA_SYMBOL lock_item
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_lock_statement6083);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelock_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_lock_item_in_lock_statement6085);
        	        	        lock_item(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelock_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop410;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop410: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1112:4: UNLOCK_SYMBOL ( TABLES_SYMBOL | TABLE_SYMBOL )
        	    {
        	         MATCHT(UNLOCK_SYMBOL, &FOLLOW_UNLOCK_SYMBOL_in_lock_statement6092);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelock_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == TABLES_SYMBOL || LA(1) == TABLE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulelock_statementEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelock_statementEx; /* Prevent compiler warnings */
    rulelock_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end lock_statement */

/**
 * $ANTLR start lock_item
 * MySQLSimpleParser.g:1115:1: lock_item : table_ref ( table_alias )? lock_option ;
 */
static void
lock_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1115:10: ( table_ref ( table_alias )? lock_option )
        // MySQLSimpleParser.g:1116:2: table_ref ( table_alias )? lock_option
        {
            FOLLOWPUSH(FOLLOW_table_ref_in_lock_item6109);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelock_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1116:12: ( table_alias )?
            {
                int alt412=2;
                {
                    int LA412_0 = LA(1);
                    if ( (((LA412_0 >= ACCOUNT_SYMBOL) && (LA412_0 <= YEAR_SYMBOL)) || LA412_0 == AS_SYMBOL || LA412_0 == BACK_TICK_QUOTED_ID || LA412_0 == DOUBLE_QUOTED_TEXT || LA412_0 == EQUAL_OPERATOR || LA412_0 == IDENTIFIER))
                    {
                        alt412=1;
                    }
                }
                switch (alt412)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1116:12: table_alias
            	    {
            	        FOLLOWPUSH(FOLLOW_table_alias_in_lock_item6111);
            	        table_alias(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelock_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_lock_option_in_lock_item6114);
            lock_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelock_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelock_itemEx; /* Prevent compiler warnings */
    rulelock_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end lock_item */

/**
 * $ANTLR start lock_option
 * MySQLSimpleParser.g:1119:1: lock_option : ( READ_SYMBOL ( LOCAL_SYMBOL )? | ( LOW_PRIORITY_SYMBOL )? WRITE_SYMBOL );
 */
static void
lock_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1119:12: ( READ_SYMBOL ( LOCAL_SYMBOL )? | ( LOW_PRIORITY_SYMBOL )? WRITE_SYMBOL )

            ANTLR3_UINT32 alt415;

            alt415=2;

            switch ( LA(1) )
            {
            case READ_SYMBOL:
            	{
            		alt415=1;
            	}
                break;
            case LOW_PRIORITY_SYMBOL:
            case WRITE_SYMBOL:
            	{
            		alt415=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 415;
                EXCEPTION->state        = 0;


                goto rulelock_optionEx;

            }

            switch (alt415)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1120:2: READ_SYMBOL ( LOCAL_SYMBOL )?
        	    {
        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_lock_option6123);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelock_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1120:14: ( LOCAL_SYMBOL )?
        	        {
        	            int alt413=2;
        	            switch ( LA(1) )
        	            {
        	                case LOCAL_SYMBOL:
        	                	{
        	                		alt413=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt413)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1120:14: LOCAL_SYMBOL
        	        	    {
        	        	         MATCHT(LOCAL_SYMBOL, &FOLLOW_LOCAL_SYMBOL_in_lock_option6125);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelock_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1121:4: ( LOW_PRIORITY_SYMBOL )? WRITE_SYMBOL
        	    {
        	        // MySQLSimpleParser.g:1121:4: ( LOW_PRIORITY_SYMBOL )?
        	        {
        	            int alt414=2;
        	            switch ( LA(1) )
        	            {
        	                case LOW_PRIORITY_SYMBOL:
        	                	{
        	                		alt414=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt414)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1121:4: LOW_PRIORITY_SYMBOL
        	        	    {
        	        	         MATCHT(LOW_PRIORITY_SYMBOL, &FOLLOW_LOW_PRIORITY_SYMBOL_in_lock_option6131);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulelock_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(WRITE_SYMBOL, &FOLLOW_WRITE_SYMBOL_in_lock_option6134);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelock_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelock_optionEx; /* Prevent compiler warnings */
    rulelock_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end lock_option */

/**
 * $ANTLR start xa_statement
 * MySQLSimpleParser.g:1124:1: xa_statement : XA_SYMBOL ( ( START_SYMBOL | BEGIN_SYMBOL ) xid ( JOIN_SYMBOL | RESUME_SYMBOL )? | END_SYMBOL xid ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )? | PREPARE_SYMBOL xid | COMMIT_SYMBOL xid ( ONE_SYMBOL PHASE_SYMBOL )? | ROLLBACK_SYMBOL xid | RECOVER_SYMBOL xa_convert ) ;
 */
static void
xa_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1124:13: ( XA_SYMBOL ( ( START_SYMBOL | BEGIN_SYMBOL ) xid ( JOIN_SYMBOL | RESUME_SYMBOL )? | END_SYMBOL xid ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )? | PREPARE_SYMBOL xid | COMMIT_SYMBOL xid ( ONE_SYMBOL PHASE_SYMBOL )? | ROLLBACK_SYMBOL xid | RECOVER_SYMBOL xa_convert ) )
        // MySQLSimpleParser.g:1125:2: XA_SYMBOL ( ( START_SYMBOL | BEGIN_SYMBOL ) xid ( JOIN_SYMBOL | RESUME_SYMBOL )? | END_SYMBOL xid ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )? | PREPARE_SYMBOL xid | COMMIT_SYMBOL xid ( ONE_SYMBOL PHASE_SYMBOL )? | ROLLBACK_SYMBOL xid | RECOVER_SYMBOL xa_convert )
        {
             MATCHT(XA_SYMBOL, &FOLLOW_XA_SYMBOL_in_xa_statement6144);
            if  (HASEXCEPTION())
            {
                goto rulexa_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1126:3: ( ( START_SYMBOL | BEGIN_SYMBOL ) xid ( JOIN_SYMBOL | RESUME_SYMBOL )? | END_SYMBOL xid ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )? | PREPARE_SYMBOL xid | COMMIT_SYMBOL xid ( ONE_SYMBOL PHASE_SYMBOL )? | ROLLBACK_SYMBOL xid | RECOVER_SYMBOL xa_convert )
            {
                int alt420=6;
                switch ( LA(1) )
                {
                case BEGIN_SYMBOL:
                case START_SYMBOL:
                	{
                		alt420=1;
                	}
                    break;
                case END_SYMBOL:
                	{
                		alt420=2;
                	}
                    break;
                case PREPARE_SYMBOL:
                	{
                		alt420=3;
                	}
                    break;
                case COMMIT_SYMBOL:
                	{
                		alt420=4;
                	}
                    break;
                case ROLLBACK_SYMBOL:
                	{
                		alt420=5;
                	}
                    break;
                case RECOVER_SYMBOL:
                	{
                		alt420=6;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 420;
                    EXCEPTION->state        = 0;


                    goto rulexa_statementEx;

                }

                switch (alt420)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1127:4: ( START_SYMBOL | BEGIN_SYMBOL ) xid ( JOIN_SYMBOL | RESUME_SYMBOL )?
            	    {
            	        if ( LA(1) == BEGIN_SYMBOL || LA(1) == START_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulexa_statementEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_xid_in_xa_statement6161);
            	        xid(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1127:38: ( JOIN_SYMBOL | RESUME_SYMBOL )?
            	        {
            	            int alt416=2;
            	            switch ( LA(1) )
            	            {
            	                case RESUME_SYMBOL:
            	                case JOIN_SYMBOL:
            	                	{
            	                		alt416=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt416)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == RESUME_SYMBOL || LA(1) == JOIN_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto rulexa_statementEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1128:6: END_SYMBOL xid ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )?
            	    {
            	         MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_xa_statement6177);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_xid_in_xa_statement6179);
            	        xid(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1128:21: ( SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )? )?
            	        {
            	            int alt418=2;
            	            switch ( LA(1) )
            	            {
            	                case SUSPEND_SYMBOL:
            	                	{
            	                		alt418=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt418)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1128:22: SUSPEND_SYMBOL ( FOR_SYMBOL MIGRATE_SYMBOL )?
            	        	    {
            	        	         MATCHT(SUSPEND_SYMBOL, &FOLLOW_SUSPEND_SYMBOL_in_xa_statement6182);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulexa_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:1128:37: ( FOR_SYMBOL MIGRATE_SYMBOL )?
            	        	        {
            	        	            int alt417=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case FOR_SYMBOL:
            	        	                	{
            	        	                		alt417=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt417)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:1128:38: FOR_SYMBOL MIGRATE_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_xa_statement6185);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulexa_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	         MATCHT(MIGRATE_SYMBOL, &FOLLOW_MIGRATE_SYMBOL_in_xa_statement6187);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulexa_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1129:6: PREPARE_SYMBOL xid
            	    {
            	         MATCHT(PREPARE_SYMBOL, &FOLLOW_PREPARE_SYMBOL_in_xa_statement6198);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_xid_in_xa_statement6200);
            	        xid(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:1130:6: COMMIT_SYMBOL xid ( ONE_SYMBOL PHASE_SYMBOL )?
            	    {
            	         MATCHT(COMMIT_SYMBOL, &FOLLOW_COMMIT_SYMBOL_in_xa_statement6207);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_xid_in_xa_statement6209);
            	        xid(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1130:24: ( ONE_SYMBOL PHASE_SYMBOL )?
            	        {
            	            int alt419=2;
            	            switch ( LA(1) )
            	            {
            	                case ONE_SYMBOL:
            	                	{
            	                		alt419=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt419)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1130:25: ONE_SYMBOL PHASE_SYMBOL
            	        	    {
            	        	         MATCHT(ONE_SYMBOL, &FOLLOW_ONE_SYMBOL_in_xa_statement6212);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulexa_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(PHASE_SYMBOL, &FOLLOW_PHASE_SYMBOL_in_xa_statement6214);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulexa_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:1131:6: ROLLBACK_SYMBOL xid
            	    {
            	         MATCHT(ROLLBACK_SYMBOL, &FOLLOW_ROLLBACK_SYMBOL_in_xa_statement6223);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_xid_in_xa_statement6225);
            	        xid(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:1132:6: RECOVER_SYMBOL xa_convert
            	    {
            	         MATCHT(RECOVER_SYMBOL, &FOLLOW_RECOVER_SYMBOL_in_xa_statement6232);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_xa_convert_in_xa_statement6234);
            	        xa_convert(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexa_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulexa_statementEx; /* Prevent compiler warnings */
    rulexa_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end xa_statement */

/**
 * $ANTLR start xa_convert
 * MySQLSimpleParser.g:1136:1: xa_convert : ({...}? ( CONVERT_SYMBOL XID_SYMBOL )? |);
 */
static void
xa_convert(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1136:11: ({...}? ( CONVERT_SYMBOL XID_SYMBOL )? |)

            ANTLR3_UINT32 alt422;

            alt422=2;

            switch ( LA(1) )
            {
            case CONVERT_SYMBOL:
            	{
            		alt422=1;
            	}
                break;
            case SEMICOLON_SYMBOL:
            	{
            		{
            		    int LA422_2 = LA(2);
            		    if ( ((SERVER_VERSION >= 50704)))
            		    {
            		        alt422=1;
            		    }
            		    else if ( (ANTLR3_TRUE))
            		    {
            		        alt422=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 422;
            		        EXCEPTION->state        = 2;


            		        goto rulexa_convertEx;

            		    }
            		}
            	}
                break;
            case EOF:
            	{
            		{
            		    int LA422_3 = LA(2);
            		    if ( ((SERVER_VERSION >= 50704)))
            		    {
            		        alt422=1;
            		    }
            		    else if ( (ANTLR3_TRUE))
            		    {
            		        alt422=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 422;
            		        EXCEPTION->state        = 3;


            		        goto rulexa_convertEx;

            		    }
            		}
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 422;
                EXCEPTION->state        = 0;


                goto rulexa_convertEx;

            }

            switch (alt422)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1137:2: {...}? ( CONVERT_SYMBOL XID_SYMBOL )?
        	    {
        	        if ( !((SERVER_VERSION >= 50704)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50704";
        	                EXCEPTION->ruleName	 = (void *)"xa_convert";


        	        }

        	        // MySQLSimpleParser.g:1137:29: ( CONVERT_SYMBOL XID_SYMBOL )?
        	        {
        	            int alt421=2;
        	            switch ( LA(1) )
        	            {
        	                case CONVERT_SYMBOL:
        	                	{
        	                		alt421=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt421)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1137:30: CONVERT_SYMBOL XID_SYMBOL
        	        	    {
        	        	         MATCHT(CONVERT_SYMBOL, &FOLLOW_CONVERT_SYMBOL_in_xa_convert6250);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulexa_convertEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(XID_SYMBOL, &FOLLOW_XID_SYMBOL_in_xa_convert6252);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulexa_convertEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1139:2: 
        	    {
        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulexa_convertEx; /* Prevent compiler warnings */
    rulexa_convertEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end xa_convert */

/**
 * $ANTLR start xid
 * MySQLSimpleParser.g:1141:1: xid : text_string ( COMMA_SYMBOL text_string ( COMMA_SYMBOL ulong_number )? )? ;
 */
static void
xid(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1141:4: ( text_string ( COMMA_SYMBOL text_string ( COMMA_SYMBOL ulong_number )? )? )
        // MySQLSimpleParser.g:1142:2: text_string ( COMMA_SYMBOL text_string ( COMMA_SYMBOL ulong_number )? )?
        {
            FOLLOWPUSH(FOLLOW_text_string_in_xid6268);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulexidEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1142:14: ( COMMA_SYMBOL text_string ( COMMA_SYMBOL ulong_number )? )?
            {
                int alt424=2;
                switch ( LA(1) )
                {
                    case COMMA_SYMBOL:
                    	{
                    		alt424=1;
                    	}
                        break;
                }

                switch (alt424)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1142:15: COMMA_SYMBOL text_string ( COMMA_SYMBOL ulong_number )?
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_xid6271);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexidEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_text_string_in_xid6273);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulexidEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1142:40: ( COMMA_SYMBOL ulong_number )?
            	        {
            	            int alt423=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt423=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt423)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1142:41: COMMA_SYMBOL ulong_number
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_xid6276);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulexidEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_xid6278);
            	        	        ulong_number(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulexidEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulexidEx; /* Prevent compiler warnings */
    rulexidEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end xid */

/**
 * $ANTLR start replication_statement
 * MySQLSimpleParser.g:1147:1: replication_statement : ( PURGE_SYMBOL ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL ( TO_SYMBOL string_literal | BEFORE_SYMBOL expression ) | change_master |{...}? change_replication | slave |{...}? replication_load |{...}? group_replication );
 */
static void
replication_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1147:22: ( PURGE_SYMBOL ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL ( TO_SYMBOL string_literal | BEFORE_SYMBOL expression ) | change_master |{...}? change_replication | slave |{...}? replication_load |{...}? group_replication )

            ANTLR3_UINT32 alt426;

            alt426=6;

            switch ( LA(1) )
            {
            case PURGE_SYMBOL:
            	{
            		alt426=1;
            	}
                break;
            case CHANGE_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case MASTER_SYMBOL:
            			{
            				alt426=2;
            			}
            		    break;
            		case REPLICATION_SYMBOL:
            			{
            				alt426=3;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 426;
            		    EXCEPTION->state        = 2;


            		    goto rulereplication_statementEx;

            		}

            	}
                break;
            case START_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case SLAVE_SYMBOL:
            			{
            				alt426=4;
            			}
            		    break;
            		case GROUP_REPLICATION_SYMBOL:
            			{
            				alt426=6;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 426;
            		    EXCEPTION->state        = 3;


            		    goto rulereplication_statementEx;

            		}

            	}
                break;
            case STOP_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case SLAVE_SYMBOL:
            			{
            				alt426=4;
            			}
            		    break;
            		case GROUP_REPLICATION_SYMBOL:
            			{
            				alt426=6;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 426;
            		    EXCEPTION->state        = 4;


            		    goto rulereplication_statementEx;

            		}

            	}
                break;
            case LOAD_SYMBOL:
            	{
            		alt426=5;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 426;
                EXCEPTION->state        = 0;


                goto rulereplication_statementEx;

            }

            switch (alt426)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1148:2: PURGE_SYMBOL ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL ( TO_SYMBOL string_literal | BEFORE_SYMBOL expression )
        	    {
        	         MATCHT(PURGE_SYMBOL, &FOLLOW_PURGE_SYMBOL_in_replication_statement6293);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == MASTER_SYMBOL || LA(1) == BINARY_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulereplication_statementEx;
        	        }


        	         MATCHT(LOGS_SYMBOL, &FOLLOW_LOGS_SYMBOL_in_replication_statement6303);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1148:59: ( TO_SYMBOL string_literal | BEFORE_SYMBOL expression )
        	        {
        	            int alt425=2;
        	            switch ( LA(1) )
        	            {
        	            case TO_SYMBOL:
        	            	{
        	            		alt425=1;
        	            	}
        	                break;
        	            case BEFORE_SYMBOL:
        	            	{
        	            		alt425=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 425;
        	                EXCEPTION->state        = 0;


        	                goto rulereplication_statementEx;

        	            }

        	            switch (alt425)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1148:60: TO_SYMBOL string_literal
        	        	    {
        	        	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_replication_statement6306);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereplication_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_string_literal_in_replication_statement6308);
        	        	        string_literal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereplication_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1148:87: BEFORE_SYMBOL expression
        	        	    {
        	        	         MATCHT(BEFORE_SYMBOL, &FOLLOW_BEFORE_SYMBOL_in_replication_statement6312);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereplication_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_replication_statement6314);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereplication_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1149:4: change_master
        	    {
        	        FOLLOWPUSH(FOLLOW_change_master_in_replication_statement6320);
        	        change_master(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1150:4: {...}? change_replication
        	    {
        	        if ( !((SERVER_VERSION >= 50700)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
        	                EXCEPTION->ruleName	 = (void *)"replication_statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_change_replication_in_replication_statement6327);
        	        change_replication(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1155:4: slave
        	    {
        	        FOLLOWPUSH(FOLLOW_slave_in_replication_statement6335);
        	        slave(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1156:4: {...}? replication_load
        	    {
        	        if ( !((SERVER_VERSION < 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50500";
        	                EXCEPTION->ruleName	 = (void *)"replication_statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_replication_load_in_replication_statement6342);
        	        replication_load(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1157:4: {...}? group_replication
        	    {
        	        if ( !((SERVER_VERSION > 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION > 50706";
        	                EXCEPTION->ruleName	 = (void *)"replication_statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_group_replication_in_replication_statement6349);
        	        group_replication(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereplication_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulereplication_statementEx; /* Prevent compiler warnings */
    rulereplication_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end replication_statement */

/**
 * $ANTLR start replication_load
 * MySQLSimpleParser.g:1160:1: replication_load : LOAD_SYMBOL ( DATA_SYMBOL | TABLE_SYMBOL table_ref ) FROM_SYMBOL MASTER_SYMBOL ;
 */
static void
replication_load(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1160:17: ( LOAD_SYMBOL ( DATA_SYMBOL | TABLE_SYMBOL table_ref ) FROM_SYMBOL MASTER_SYMBOL )
        // MySQLSimpleParser.g:1161:2: LOAD_SYMBOL ( DATA_SYMBOL | TABLE_SYMBOL table_ref ) FROM_SYMBOL MASTER_SYMBOL
        {
             MATCHT(LOAD_SYMBOL, &FOLLOW_LOAD_SYMBOL_in_replication_load6358);
            if  (HASEXCEPTION())
            {
                goto rulereplication_loadEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1161:14: ( DATA_SYMBOL | TABLE_SYMBOL table_ref )
            {
                int alt427=2;
                switch ( LA(1) )
                {
                case DATA_SYMBOL:
                	{
                		alt427=1;
                	}
                    break;
                case TABLE_SYMBOL:
                	{
                		alt427=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 427;
                    EXCEPTION->state        = 0;


                    goto rulereplication_loadEx;

                }

                switch (alt427)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1161:15: DATA_SYMBOL
            	    {
            	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_replication_load6361);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereplication_loadEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1161:29: TABLE_SYMBOL table_ref
            	    {
            	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_replication_load6365);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereplication_loadEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_replication_load6367);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereplication_loadEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_replication_load6370);
            if  (HASEXCEPTION())
            {
                goto rulereplication_loadEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(MASTER_SYMBOL, &FOLLOW_MASTER_SYMBOL_in_replication_load6372);
            if  (HASEXCEPTION())
            {
                goto rulereplication_loadEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulereplication_loadEx; /* Prevent compiler warnings */
    rulereplication_loadEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end replication_load */

/**
 * $ANTLR start change_master
 * MySQLSimpleParser.g:1164:1: change_master : CHANGE_SYMBOL MASTER_SYMBOL TO_SYMBOL change_master_options ( channel )? ;
 */
static void
change_master(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1164:14: ( CHANGE_SYMBOL MASTER_SYMBOL TO_SYMBOL change_master_options ( channel )? )
        // MySQLSimpleParser.g:1165:2: CHANGE_SYMBOL MASTER_SYMBOL TO_SYMBOL change_master_options ( channel )?
        {
             MATCHT(CHANGE_SYMBOL, &FOLLOW_CHANGE_SYMBOL_in_change_master6381);
            if  (HASEXCEPTION())
            {
                goto rulechange_masterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(MASTER_SYMBOL, &FOLLOW_MASTER_SYMBOL_in_change_master6383);
            if  (HASEXCEPTION())
            {
                goto rulechange_masterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_change_master6385);
            if  (HASEXCEPTION())
            {
                goto rulechange_masterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_change_master_options_in_change_master6387);
            change_master_options(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulechange_masterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1165:62: ( channel )?
            {
                int alt428=2;
                {
                    int LA428_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50706)) && (LA428_0 == FOR_SYMBOL))
                    {
                        alt428=1;
                    }
                }
                switch (alt428)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1165:62: channel
            	    {
            	        FOLLOWPUSH(FOLLOW_channel_in_change_master6389);
            	        channel(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulechange_masterEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulechange_masterEx; /* Prevent compiler warnings */
    rulechange_masterEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end change_master */

/**
 * $ANTLR start change_master_options
 * MySQLSimpleParser.g:1168:1: change_master_options : master_option ( COMMA_SYMBOL master_option )* ;
 */
static void
change_master_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1168:22: ( master_option ( COMMA_SYMBOL master_option )* )
        // MySQLSimpleParser.g:1169:2: master_option ( COMMA_SYMBOL master_option )*
        {
            FOLLOWPUSH(FOLLOW_master_option_in_change_master_options6399);
            master_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulechange_master_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1169:16: ( COMMA_SYMBOL master_option )*

            for (;;)
            {
                int alt429=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt429=1;
                	}
                    break;

                }

                switch (alt429)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1169:17: COMMA_SYMBOL master_option
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_change_master_options6402);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulechange_master_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_master_option_in_change_master_options6404);
            	        master_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulechange_master_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop429;	/* break out of the loop */
            	    break;
                }
            }
            loop429: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulechange_master_optionsEx; /* Prevent compiler warnings */
    rulechange_master_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end change_master_options */

/**
 * $ANTLR start master_option
 * MySQLSimpleParser.g:1172:1: master_option : ( MASTER_HOST_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_BIND_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_USER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_PASSWORD_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_PORT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_CONNECT_RETRY_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_RETRY_COUNT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_DELAY_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_CA_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CAPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CERT_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CIPHER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_KEY_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_CRL_SYMBOL EQUAL_OPERATOR string_literal | MASTER_SSL_CRLPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_HEARTBEAT_PERIOD_SYMBOL EQUAL_OPERATOR ulong_number | IGNORE_SERVER_IDS_SYMBOL EQUAL_OPERATOR server_id_list | MASTER_AUTO_POSITION_SYMBOL EQUAL_OPERATOR ulong_number | master_file_def );
 */
static void
master_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1172:14: ( MASTER_HOST_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_BIND_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_USER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_PASSWORD_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_PORT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_CONNECT_RETRY_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_RETRY_COUNT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_DELAY_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_CA_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CAPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CERT_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_CIPHER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_KEY_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL EQUAL_OPERATOR ulong_number | MASTER_SSL_CRL_SYMBOL EQUAL_OPERATOR string_literal | MASTER_SSL_CRLPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_HEARTBEAT_PERIOD_SYMBOL EQUAL_OPERATOR ulong_number | IGNORE_SERVER_IDS_SYMBOL EQUAL_OPERATOR server_id_list | MASTER_AUTO_POSITION_SYMBOL EQUAL_OPERATOR ulong_number | master_file_def )

            ANTLR3_UINT32 alt430;

            alt430=21;

            switch ( LA(1) )
            {
            case MASTER_HOST_SYMBOL:
            	{
            		alt430=1;
            	}
                break;
            case MASTER_BIND_SYMBOL:
            	{
            		alt430=2;
            	}
                break;
            case MASTER_USER_SYMBOL:
            	{
            		alt430=3;
            	}
                break;
            case MASTER_PASSWORD_SYMBOL:
            	{
            		alt430=4;
            	}
                break;
            case MASTER_PORT_SYMBOL:
            	{
            		alt430=5;
            	}
                break;
            case MASTER_CONNECT_RETRY_SYMBOL:
            	{
            		alt430=6;
            	}
                break;
            case MASTER_RETRY_COUNT_SYMBOL:
            	{
            		alt430=7;
            	}
                break;
            case MASTER_DELAY_SYMBOL:
            	{
            		alt430=8;
            	}
                break;
            case MASTER_SSL_SYMBOL:
            	{
            		alt430=9;
            	}
                break;
            case MASTER_SSL_CA_SYMBOL:
            	{
            		alt430=10;
            	}
                break;
            case MASTER_SSL_CAPATH_SYMBOL:
            	{
            		alt430=11;
            	}
                break;
            case MASTER_SSL_CERT_SYMBOL:
            	{
            		alt430=12;
            	}
                break;
            case MASTER_SSL_CIPHER_SYMBOL:
            	{
            		alt430=13;
            	}
                break;
            case MASTER_SSL_KEY_SYMBOL:
            	{
            		alt430=14;
            	}
                break;
            case MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL:
            	{
            		alt430=15;
            	}
                break;
            case MASTER_SSL_CRL_SYMBOL:
            	{
            		alt430=16;
            	}
                break;
            case MASTER_SSL_CRLPATH_SYMBOL:
            	{
            		alt430=17;
            	}
                break;
            case MASTER_HEARTBEAT_PERIOD_SYMBOL:
            	{
            		alt430=18;
            	}
                break;
            case IGNORE_SERVER_IDS_SYMBOL:
            	{
            		alt430=19;
            	}
                break;
            case MASTER_AUTO_POSITION_SYMBOL:
            	{
            		alt430=20;
            	}
                break;
            case MASTER_LOG_FILE_SYMBOL:
            case MASTER_LOG_POS_SYMBOL:
            case RELAY_LOG_FILE_SYMBOL:
            case RELAY_LOG_POS_SYMBOL:
            	{
            		alt430=21;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 430;
                EXCEPTION->state        = 0;


                goto rulemaster_optionEx;

            }

            switch (alt430)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1173:2: MASTER_HOST_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_HOST_SYMBOL, &FOLLOW_MASTER_HOST_SYMBOL_in_master_option6415);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6417);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6419);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1174:4: MASTER_BIND_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_BIND_SYMBOL, &FOLLOW_MASTER_BIND_SYMBOL_in_master_option6424);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6426);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6428);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1175:4: MASTER_USER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_USER_SYMBOL, &FOLLOW_MASTER_USER_SYMBOL_in_master_option6433);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6435);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6437);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1176:4: MASTER_PASSWORD_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_PASSWORD_SYMBOL, &FOLLOW_MASTER_PASSWORD_SYMBOL_in_master_option6442);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6444);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6446);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1177:4: MASTER_PORT_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_PORT_SYMBOL, &FOLLOW_MASTER_PORT_SYMBOL_in_master_option6451);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6453);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6455);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1178:4: MASTER_CONNECT_RETRY_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_CONNECT_RETRY_SYMBOL, &FOLLOW_MASTER_CONNECT_RETRY_SYMBOL_in_master_option6460);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6462);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6464);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1179:4: MASTER_RETRY_COUNT_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_RETRY_COUNT_SYMBOL, &FOLLOW_MASTER_RETRY_COUNT_SYMBOL_in_master_option6469);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6471);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6473);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1180:4: MASTER_DELAY_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_DELAY_SYMBOL, &FOLLOW_MASTER_DELAY_SYMBOL_in_master_option6478);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6480);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6482);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:1181:4: MASTER_SSL_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_SSL_SYMBOL, &FOLLOW_MASTER_SSL_SYMBOL_in_master_option6487);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6489);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6491);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:1182:4: MASTER_SSL_CA_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_CA_SYMBOL, &FOLLOW_MASTER_SSL_CA_SYMBOL_in_master_option6496);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6498);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6500);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:1183:4: MASTER_SSL_CAPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_CAPATH_SYMBOL, &FOLLOW_MASTER_SSL_CAPATH_SYMBOL_in_master_option6505);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6507);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6509);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:1184:4: MASTER_SSL_CERT_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_CERT_SYMBOL, &FOLLOW_MASTER_SSL_CERT_SYMBOL_in_master_option6514);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6516);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6518);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:1185:4: MASTER_SSL_CIPHER_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_CIPHER_SYMBOL, &FOLLOW_MASTER_SSL_CIPHER_SYMBOL_in_master_option6523);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6525);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6527);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:1186:4: MASTER_SSL_KEY_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_KEY_SYMBOL, &FOLLOW_MASTER_SSL_KEY_SYMBOL_in_master_option6532);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6534);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6536);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:1187:4: MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL, &FOLLOW_MASTER_SSL_VERIFY_SERVER_CERT_SYMBOL_in_master_option6541);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6543);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6545);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:1188:4: MASTER_SSL_CRL_SYMBOL EQUAL_OPERATOR string_literal
        	    {
        	         MATCHT(MASTER_SSL_CRL_SYMBOL, &FOLLOW_MASTER_SSL_CRL_SYMBOL_in_master_option6550);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6552);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_master_option6554);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:1189:4: MASTER_SSL_CRLPATH_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_SSL_CRLPATH_SYMBOL, &FOLLOW_MASTER_SSL_CRLPATH_SYMBOL_in_master_option6559);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6561);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_option6563);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:1190:4: MASTER_HEARTBEAT_PERIOD_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_HEARTBEAT_PERIOD_SYMBOL, &FOLLOW_MASTER_HEARTBEAT_PERIOD_SYMBOL_in_master_option6568);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6570);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6572);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:1191:4: IGNORE_SERVER_IDS_SYMBOL EQUAL_OPERATOR server_id_list
        	    {
        	         MATCHT(IGNORE_SERVER_IDS_SYMBOL, &FOLLOW_IGNORE_SERVER_IDS_SYMBOL_in_master_option6577);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6579);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_server_id_list_in_master_option6581);
        	        server_id_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:1192:4: MASTER_AUTO_POSITION_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(MASTER_AUTO_POSITION_SYMBOL, &FOLLOW_MASTER_AUTO_POSITION_SYMBOL_in_master_option6586);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_option6588);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_option6590);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:1193:4: master_file_def
        	    {
        	        FOLLOWPUSH(FOLLOW_master_file_def_in_master_option6595);
        	        master_file_def(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulemaster_optionEx; /* Prevent compiler warnings */
    rulemaster_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end master_option */

/**
 * $ANTLR start master_file_def
 * MySQLSimpleParser.g:1196:1: master_file_def : ( MASTER_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_LOG_POS_SYMBOL EQUAL_OPERATOR ulonglong_number | RELAY_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | RELAY_LOG_POS_SYMBOL EQUAL_OPERATOR ulong_number );
 */
static void
master_file_def(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1196:16: ( MASTER_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | MASTER_LOG_POS_SYMBOL EQUAL_OPERATOR ulonglong_number | RELAY_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak | RELAY_LOG_POS_SYMBOL EQUAL_OPERATOR ulong_number )

            ANTLR3_UINT32 alt431;

            alt431=4;

            switch ( LA(1) )
            {
            case MASTER_LOG_FILE_SYMBOL:
            	{
            		alt431=1;
            	}
                break;
            case MASTER_LOG_POS_SYMBOL:
            	{
            		alt431=2;
            	}
                break;
            case RELAY_LOG_FILE_SYMBOL:
            	{
            		alt431=3;
            	}
                break;
            case RELAY_LOG_POS_SYMBOL:
            	{
            		alt431=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 431;
                EXCEPTION->state        = 0;


                goto rulemaster_file_defEx;

            }

            switch (alt431)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1197:2: MASTER_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(MASTER_LOG_FILE_SYMBOL, &FOLLOW_MASTER_LOG_FILE_SYMBOL_in_master_file_def6604);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_file_def6606);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_file_def6608);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1198:4: MASTER_LOG_POS_SYMBOL EQUAL_OPERATOR ulonglong_number
        	    {
        	         MATCHT(MASTER_LOG_POS_SYMBOL, &FOLLOW_MASTER_LOG_POS_SYMBOL_in_master_file_def6613);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_file_def6615);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulonglong_number_in_master_file_def6617);
        	        ulonglong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1199:4: RELAY_LOG_FILE_SYMBOL EQUAL_OPERATOR text_string_no_linebreak
        	    {
        	         MATCHT(RELAY_LOG_FILE_SYMBOL, &FOLLOW_RELAY_LOG_FILE_SYMBOL_in_master_file_def6622);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_file_def6624);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_master_file_def6626);
        	        text_string_no_linebreak(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1200:4: RELAY_LOG_POS_SYMBOL EQUAL_OPERATOR ulong_number
        	    {
        	         MATCHT(RELAY_LOG_POS_SYMBOL, &FOLLOW_RELAY_LOG_POS_SYMBOL_in_master_file_def6631);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_master_file_def6633);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_master_file_def6635);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulemaster_file_defEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulemaster_file_defEx; /* Prevent compiler warnings */
    rulemaster_file_defEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end master_file_def */

/**
 * $ANTLR start server_id_list
 * MySQLSimpleParser.g:1203:1: server_id_list : OPEN_PAR_SYMBOL ( ulong_number ( COMMA_SYMBOL ulong_number )* )? CLOSE_PAR_SYMBOL ;
 */
static void
server_id_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1203:15: ( OPEN_PAR_SYMBOL ( ulong_number ( COMMA_SYMBOL ulong_number )* )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:1204:2: OPEN_PAR_SYMBOL ( ulong_number ( COMMA_SYMBOL ulong_number )* )? CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_server_id_list6644);
            if  (HASEXCEPTION())
            {
                goto ruleserver_id_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1204:18: ( ulong_number ( COMMA_SYMBOL ulong_number )* )?
            {
                int alt433=2;
                switch ( LA(1) )
                {
                    case DECIMAL_NUMBER:
                    case FLOAT_NUMBER:
                    case HEX_NUMBER:
                    case INT_NUMBER:
                    case LONG_NUMBER:
                    case ULONGLONG_NUMBER:
                    	{
                    		alt433=1;
                    	}
                        break;
                }

                switch (alt433)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1204:19: ulong_number ( COMMA_SYMBOL ulong_number )*
            	    {
            	        FOLLOWPUSH(FOLLOW_ulong_number_in_server_id_list6647);
            	        ulong_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleserver_id_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1204:32: ( COMMA_SYMBOL ulong_number )*

            	        for (;;)
            	        {
            	            int alt432=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt432=1;
            	            	}
            	                break;

            	            }

            	            switch (alt432)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1204:33: COMMA_SYMBOL ulong_number
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_server_id_list6650);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleserver_id_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_server_id_list6652);
            	        	        ulong_number(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleserver_id_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop432;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop432: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_server_id_list6658);
            if  (HASEXCEPTION())
            {
                goto ruleserver_id_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleserver_id_listEx; /* Prevent compiler warnings */
    ruleserver_id_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end server_id_list */

/**
 * $ANTLR start change_replication
 * MySQLSimpleParser.g:1207:1: change_replication : CHANGE_SYMBOL REPLICATION_SYMBOL FILTER_SYMBOL filter_definition ( COMMA_SYMBOL filter_definition )* ;
 */
static void
change_replication(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1207:19: ( CHANGE_SYMBOL REPLICATION_SYMBOL FILTER_SYMBOL filter_definition ( COMMA_SYMBOL filter_definition )* )
        // MySQLSimpleParser.g:1208:2: CHANGE_SYMBOL REPLICATION_SYMBOL FILTER_SYMBOL filter_definition ( COMMA_SYMBOL filter_definition )*
        {
             MATCHT(CHANGE_SYMBOL, &FOLLOW_CHANGE_SYMBOL_in_change_replication6667);
            if  (HASEXCEPTION())
            {
                goto rulechange_replicationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(REPLICATION_SYMBOL, &FOLLOW_REPLICATION_SYMBOL_in_change_replication6669);
            if  (HASEXCEPTION())
            {
                goto rulechange_replicationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FILTER_SYMBOL, &FOLLOW_FILTER_SYMBOL_in_change_replication6671);
            if  (HASEXCEPTION())
            {
                goto rulechange_replicationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_filter_definition_in_change_replication6673);
            filter_definition(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulechange_replicationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1208:67: ( COMMA_SYMBOL filter_definition )*

            for (;;)
            {
                int alt434=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt434=1;
                	}
                    break;

                }

                switch (alt434)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1208:68: COMMA_SYMBOL filter_definition
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_change_replication6676);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulechange_replicationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_filter_definition_in_change_replication6678);
            	        filter_definition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulechange_replicationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop434;	/* break out of the loop */
            	    break;
                }
            }
            loop434: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulechange_replicationEx; /* Prevent compiler warnings */
    rulechange_replicationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end change_replication */

/**
 * $ANTLR start filter_definition
 * MySQLSimpleParser.g:1211:1: filter_definition : ( REPLICATE_DO_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL | REPLICATE_IGNORE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL | REPLICATE_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL | REPLICATE_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL | REPLICATE_WILD_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL | REPLICATE_WILD_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL | REPLICATE_REWRITE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_pair_list )? CLOSE_PAR_SYMBOL );
 */
static void
filter_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1211:18: ( REPLICATE_DO_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL | REPLICATE_IGNORE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL | REPLICATE_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL | REPLICATE_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL | REPLICATE_WILD_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL | REPLICATE_WILD_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL | REPLICATE_REWRITE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_pair_list )? CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt442;

            alt442=7;

            switch ( LA(1) )
            {
            case REPLICATE_DO_DB_SYMBOL:
            	{
            		alt442=1;
            	}
                break;
            case REPLICATE_IGNORE_DB_SYMBOL:
            	{
            		alt442=2;
            	}
                break;
            case REPLICATE_DO_TABLE_SYMBOL:
            	{
            		alt442=3;
            	}
                break;
            case REPLICATE_IGNORE_TABLE_SYMBOL:
            	{
            		alt442=4;
            	}
                break;
            case REPLICATE_WILD_DO_TABLE_SYMBOL:
            	{
            		alt442=5;
            	}
                break;
            case REPLICATE_WILD_IGNORE_TABLE_SYMBOL:
            	{
            		alt442=6;
            	}
                break;
            case REPLICATE_REWRITE_DB_SYMBOL:
            	{
            		alt442=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 442;
                EXCEPTION->state        = 0;


                goto rulefilter_definitionEx;

            }

            switch (alt442)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1212:2: REPLICATE_DO_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_DO_DB_SYMBOL, &FOLLOW_REPLICATE_DO_DB_SYMBOL_in_filter_definition6689);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6691);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6693);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1212:56: ( filter_db_list )?
        	        {
        	            int alt435=2;
        	            {
        	                int LA435_0 = LA(1);
        	                if ( (((LA435_0 >= ACCOUNT_SYMBOL) && (LA435_0 <= YEAR_SYMBOL)) || LA435_0 == BACK_TICK_QUOTED_ID || LA435_0 == DOUBLE_QUOTED_TEXT || LA435_0 == IDENTIFIER))
        	                {
        	                    alt435=1;
        	                }
        	            }
        	            switch (alt435)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1212:56: filter_db_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_db_list_in_filter_definition6695);
        	        	        filter_db_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6698);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1213:4: REPLICATE_IGNORE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_IGNORE_DB_SYMBOL, &FOLLOW_REPLICATE_IGNORE_DB_SYMBOL_in_filter_definition6703);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6705);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6707);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1213:62: ( filter_db_list )?
        	        {
        	            int alt436=2;
        	            {
        	                int LA436_0 = LA(1);
        	                if ( (((LA436_0 >= ACCOUNT_SYMBOL) && (LA436_0 <= YEAR_SYMBOL)) || LA436_0 == BACK_TICK_QUOTED_ID || LA436_0 == DOUBLE_QUOTED_TEXT || LA436_0 == IDENTIFIER))
        	                {
        	                    alt436=1;
        	                }
        	            }
        	            switch (alt436)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1213:62: filter_db_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_db_list_in_filter_definition6709);
        	        	        filter_db_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6712);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1214:4: REPLICATE_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_DO_TABLE_SYMBOL, &FOLLOW_REPLICATE_DO_TABLE_SYMBOL_in_filter_definition6717);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6719);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6721);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1214:61: ( filter_table_list )?
        	        {
        	            int alt437=2;
        	            {
        	                int LA437_0 = LA(1);
        	                if ( (((LA437_0 >= ACCOUNT_SYMBOL) && (LA437_0 <= YEAR_SYMBOL)) || LA437_0 == BACK_TICK_QUOTED_ID || LA437_0 == DOUBLE_QUOTED_TEXT || LA437_0 == IDENTIFIER))
        	                {
        	                    alt437=1;
        	                }
        	            }
        	            switch (alt437)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1214:61: filter_table_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_table_list_in_filter_definition6723);
        	        	        filter_table_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6726);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1215:4: REPLICATE_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_table_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_IGNORE_TABLE_SYMBOL, &FOLLOW_REPLICATE_IGNORE_TABLE_SYMBOL_in_filter_definition6731);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6733);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6735);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1215:65: ( filter_table_list )?
        	        {
        	            int alt438=2;
        	            {
        	                int LA438_0 = LA(1);
        	                if ( (((LA438_0 >= ACCOUNT_SYMBOL) && (LA438_0 <= YEAR_SYMBOL)) || LA438_0 == BACK_TICK_QUOTED_ID || LA438_0 == DOUBLE_QUOTED_TEXT || LA438_0 == IDENTIFIER))
        	                {
        	                    alt438=1;
        	                }
        	            }
        	            switch (alt438)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1215:65: filter_table_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_table_list_in_filter_definition6737);
        	        	        filter_table_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6740);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1216:4: REPLICATE_WILD_DO_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_WILD_DO_TABLE_SYMBOL, &FOLLOW_REPLICATE_WILD_DO_TABLE_SYMBOL_in_filter_definition6745);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6747);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6749);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1216:66: ( filter_string_list )?
        	        {
        	            int alt439=2;
        	            switch ( LA(1) )
        	            {
        	                case SINGLE_QUOTED_TEXT:
        	                case BIN_NUMBER:
        	                case HEX_NUMBER:
        	                	{
        	                		alt439=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt439)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1216:66: filter_string_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_string_list_in_filter_definition6751);
        	        	        filter_string_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6754);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1217:4: REPLICATE_WILD_IGNORE_TABLE_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_string_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_WILD_IGNORE_TABLE_SYMBOL, &FOLLOW_REPLICATE_WILD_IGNORE_TABLE_SYMBOL_in_filter_definition6759);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6761);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6763);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1217:70: ( filter_string_list )?
        	        {
        	            int alt440=2;
        	            switch ( LA(1) )
        	            {
        	                case SINGLE_QUOTED_TEXT:
        	                case BIN_NUMBER:
        	                case HEX_NUMBER:
        	                	{
        	                		alt440=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt440)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1217:70: filter_string_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_string_list_in_filter_definition6765);
        	        	        filter_string_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6768);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1218:4: REPLICATE_REWRITE_DB_SYMBOL EQUAL_OPERATOR OPEN_PAR_SYMBOL ( filter_db_pair_list )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLICATE_REWRITE_DB_SYMBOL, &FOLLOW_REPLICATE_REWRITE_DB_SYMBOL_in_filter_definition6773);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_filter_definition6775);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_filter_definition6777);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1218:63: ( filter_db_pair_list )?
        	        {
        	            int alt441=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt441=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt441)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1218:63: filter_db_pair_list
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_filter_db_pair_list_in_filter_definition6779);
        	        	        filter_db_pair_list(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefilter_definitionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_filter_definition6782);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefilter_definitionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulefilter_definitionEx; /* Prevent compiler warnings */
    rulefilter_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_definition */

/**
 * $ANTLR start filter_db_list
 * MySQLSimpleParser.g:1221:1: filter_db_list : schema_ref ( COMMA_SYMBOL schema_ref )* ;
 */
static void
filter_db_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1221:15: ( schema_ref ( COMMA_SYMBOL schema_ref )* )
        // MySQLSimpleParser.g:1222:2: schema_ref ( COMMA_SYMBOL schema_ref )*
        {
            FOLLOWPUSH(FOLLOW_schema_ref_in_filter_db_list6791);
            schema_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_db_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1222:13: ( COMMA_SYMBOL schema_ref )*

            for (;;)
            {
                int alt443=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt443=1;
                	}
                    break;

                }

                switch (alt443)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1222:14: COMMA_SYMBOL schema_ref
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_filter_db_list6794);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_db_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_schema_ref_in_filter_db_list6796);
            	        schema_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_db_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop443;	/* break out of the loop */
            	    break;
                }
            }
            loop443: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_db_listEx; /* Prevent compiler warnings */
    rulefilter_db_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_db_list */

/**
 * $ANTLR start filter_table_list
 * MySQLSimpleParser.g:1225:1: filter_table_list : filter_table_ref ( COMMA_SYMBOL filter_table_ref )* ;
 */
static void
filter_table_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1225:18: ( filter_table_ref ( COMMA_SYMBOL filter_table_ref )* )
        // MySQLSimpleParser.g:1226:2: filter_table_ref ( COMMA_SYMBOL filter_table_ref )*
        {
            FOLLOWPUSH(FOLLOW_filter_table_ref_in_filter_table_list6807);
            filter_table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_table_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1226:19: ( COMMA_SYMBOL filter_table_ref )*

            for (;;)
            {
                int alt444=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt444=1;
                	}
                    break;

                }

                switch (alt444)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1226:20: COMMA_SYMBOL filter_table_ref
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_filter_table_list6810);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_table_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_filter_table_ref_in_filter_table_list6812);
            	        filter_table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_table_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop444;	/* break out of the loop */
            	    break;
                }
            }
            loop444: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_table_listEx; /* Prevent compiler warnings */
    rulefilter_table_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_table_list */

/**
 * $ANTLR start filter_string_list
 * MySQLSimpleParser.g:1229:1: filter_string_list : filter_wild_db_table_string ( COMMA_SYMBOL filter_wild_db_table_string )* ;
 */
static void
filter_string_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1229:19: ( filter_wild_db_table_string ( COMMA_SYMBOL filter_wild_db_table_string )* )
        // MySQLSimpleParser.g:1230:2: filter_wild_db_table_string ( COMMA_SYMBOL filter_wild_db_table_string )*
        {
            FOLLOWPUSH(FOLLOW_filter_wild_db_table_string_in_filter_string_list6823);
            filter_wild_db_table_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_string_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1230:30: ( COMMA_SYMBOL filter_wild_db_table_string )*

            for (;;)
            {
                int alt445=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt445=1;
                	}
                    break;

                }

                switch (alt445)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1230:31: COMMA_SYMBOL filter_wild_db_table_string
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_filter_string_list6826);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_string_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_filter_wild_db_table_string_in_filter_string_list6828);
            	        filter_wild_db_table_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_string_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop445;	/* break out of the loop */
            	    break;
                }
            }
            loop445: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_string_listEx; /* Prevent compiler warnings */
    rulefilter_string_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_string_list */

/**
 * $ANTLR start filter_wild_db_table_string
 * MySQLSimpleParser.g:1233:1: filter_wild_db_table_string : text_string_no_linebreak ;
 */
static void
filter_wild_db_table_string(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1233:28: ( text_string_no_linebreak )
        // MySQLSimpleParser.g:1234:2: text_string_no_linebreak
        {
            FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_filter_wild_db_table_string6839);
            text_string_no_linebreak(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_wild_db_table_stringEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_wild_db_table_stringEx; /* Prevent compiler warnings */
    rulefilter_wild_db_table_stringEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_wild_db_table_string */

/**
 * $ANTLR start filter_db_pair_list
 * MySQLSimpleParser.g:1237:1: filter_db_pair_list : schema_identifier_pair ( COMMA_SYMBOL schema_identifier_pair )* ;
 */
static void
filter_db_pair_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1237:20: ( schema_identifier_pair ( COMMA_SYMBOL schema_identifier_pair )* )
        // MySQLSimpleParser.g:1238:2: schema_identifier_pair ( COMMA_SYMBOL schema_identifier_pair )*
        {
            FOLLOWPUSH(FOLLOW_schema_identifier_pair_in_filter_db_pair_list6849);
            schema_identifier_pair(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_db_pair_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1238:25: ( COMMA_SYMBOL schema_identifier_pair )*

            for (;;)
            {
                int alt446=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt446=1;
                	}
                    break;

                }

                switch (alt446)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1238:26: COMMA_SYMBOL schema_identifier_pair
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_filter_db_pair_list6852);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_db_pair_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_schema_identifier_pair_in_filter_db_pair_list6854);
            	        schema_identifier_pair(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefilter_db_pair_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop446;	/* break out of the loop */
            	    break;
                }
            }
            loop446: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_db_pair_listEx; /* Prevent compiler warnings */
    rulefilter_db_pair_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_db_pair_list */

/**
 * $ANTLR start slave
 * MySQLSimpleParser.g:1241:1: slave : ( START_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( UNTIL_SYMBOL slave_until_options )? slave_connection_options ( channel )? | STOP_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( channel )? );
 */
static void
slave(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1241:6: ( START_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( UNTIL_SYMBOL slave_until_options )? slave_connection_options ( channel )? | STOP_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( channel )? )

            ANTLR3_UINT32 alt452;

            alt452=2;

            switch ( LA(1) )
            {
            case START_SYMBOL:
            	{
            		alt452=1;
            	}
                break;
            case STOP_SYMBOL:
            	{
            		alt452=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 452;
                EXCEPTION->state        = 0;


                goto ruleslaveEx;

            }

            switch (alt452)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1242:2: START_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( UNTIL_SYMBOL slave_until_options )? slave_connection_options ( channel )?
        	    {
        	         MATCHT(START_SYMBOL, &FOLLOW_START_SYMBOL_in_slave6865);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleslaveEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_slave6867);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleslaveEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1242:28: ( slave_thread_options )?
        	        {
        	            int alt447=2;
        	            switch ( LA(1) )
        	            {
        	                case RELAY_THREAD_SYMBOL:
        	                case SQL_THREAD_SYMBOL:
        	                	{
        	                		alt447=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt447)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1242:28: slave_thread_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_slave_thread_options_in_slave6869);
        	        	        slave_thread_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1242:50: ( UNTIL_SYMBOL slave_until_options )?
        	        {
        	            int alt448=2;
        	            switch ( LA(1) )
        	            {
        	                case UNTIL_SYMBOL:
        	                	{
        	                		alt448=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt448)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1242:51: UNTIL_SYMBOL slave_until_options
        	        	    {
        	        	         MATCHT(UNTIL_SYMBOL, &FOLLOW_UNTIL_SYMBOL_in_slave6873);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_slave_until_options_in_slave6875);
        	        	        slave_until_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_slave_connection_options_in_slave6879);
        	        slave_connection_options(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleslaveEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1242:111: ( channel )?
        	        {
        	            int alt449=2;
        	            {
        	                int LA449_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50706)) && (LA449_0 == FOR_SYMBOL))
        	                {
        	                    alt449=1;
        	                }
        	            }
        	            switch (alt449)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1242:111: channel
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_channel_in_slave6881);
        	        	        channel(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1243:4: STOP_SYMBOL SLAVE_SYMBOL ( slave_thread_options )? ( channel )?
        	    {
        	         MATCHT(STOP_SYMBOL, &FOLLOW_STOP_SYMBOL_in_slave6887);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleslaveEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_slave6889);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleslaveEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1243:29: ( slave_thread_options )?
        	        {
        	            int alt450=2;
        	            switch ( LA(1) )
        	            {
        	                case RELAY_THREAD_SYMBOL:
        	                case SQL_THREAD_SYMBOL:
        	                	{
        	                		alt450=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt450)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1243:29: slave_thread_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_slave_thread_options_in_slave6891);
        	        	        slave_thread_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1243:51: ( channel )?
        	        {
        	            int alt451=2;
        	            {
        	                int LA451_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50706)) && (LA451_0 == FOR_SYMBOL))
        	                {
        	                    alt451=1;
        	                }
        	            }
        	            switch (alt451)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1243:51: channel
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_channel_in_slave6894);
        	        	        channel(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslaveEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleslaveEx; /* Prevent compiler warnings */
    ruleslaveEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end slave */

/**
 * $ANTLR start slave_until_options
 * MySQLSimpleParser.g:1246:1: slave_until_options : ( master_file_def |{...}? ( SQL_BEFORE_GTIDS_SYMBOL | SQL_AFTER_GTIDS_SYMBOL ) EQUAL_OPERATOR text_string |{...}? SQL_AFTER_MTS_GAPS_SYMBOL ) ( COMMA_SYMBOL master_file_def )* ;
 */
static void
slave_until_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1246:20: ( ( master_file_def |{...}? ( SQL_BEFORE_GTIDS_SYMBOL | SQL_AFTER_GTIDS_SYMBOL ) EQUAL_OPERATOR text_string |{...}? SQL_AFTER_MTS_GAPS_SYMBOL ) ( COMMA_SYMBOL master_file_def )* )
        // MySQLSimpleParser.g:1247:2: ( master_file_def |{...}? ( SQL_BEFORE_GTIDS_SYMBOL | SQL_AFTER_GTIDS_SYMBOL ) EQUAL_OPERATOR text_string |{...}? SQL_AFTER_MTS_GAPS_SYMBOL ) ( COMMA_SYMBOL master_file_def )*
        {
            // MySQLSimpleParser.g:1247:2: ( master_file_def |{...}? ( SQL_BEFORE_GTIDS_SYMBOL | SQL_AFTER_GTIDS_SYMBOL ) EQUAL_OPERATOR text_string |{...}? SQL_AFTER_MTS_GAPS_SYMBOL )
            {
                int alt453=3;
                switch ( LA(1) )
                {
                case MASTER_LOG_FILE_SYMBOL:
                case MASTER_LOG_POS_SYMBOL:
                case RELAY_LOG_FILE_SYMBOL:
                case RELAY_LOG_POS_SYMBOL:
                	{
                		alt453=1;
                	}
                    break;
                case SQL_AFTER_GTIDS_SYMBOL:
                case SQL_BEFORE_GTIDS_SYMBOL:
                	{
                		alt453=2;
                	}
                    break;
                case SQL_AFTER_MTS_GAPS_SYMBOL:
                	{
                		alt453=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 453;
                    EXCEPTION->state        = 0;


                    goto ruleslave_until_optionsEx;

                }

                switch (alt453)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1248:3: master_file_def
            	    {
            	        FOLLOWPUSH(FOLLOW_master_file_def_in_slave_until_options6908);
            	        master_file_def(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1249:5: {...}? ( SQL_BEFORE_GTIDS_SYMBOL | SQL_AFTER_GTIDS_SYMBOL ) EQUAL_OPERATOR text_string
            	    {
            	        if ( !((SERVER_VERSION >= 50606)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50606";
            	                EXCEPTION->ruleName	 = (void *)"slave_until_options";


            	        }

            	        if ( LA(1) == SQL_AFTER_GTIDS_SYMBOL || LA(1) == SQL_BEFORE_GTIDS_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleslave_until_optionsEx;
            	        }


            	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_slave_until_options6924);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_text_string_in_slave_until_options6926);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1250:5: {...}? SQL_AFTER_MTS_GAPS_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION >= 50606)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50606";
            	                EXCEPTION->ruleName	 = (void *)"slave_until_options";


            	        }

            	         MATCHT(SQL_AFTER_MTS_GAPS_SYMBOL, &FOLLOW_SQL_AFTER_MTS_GAPS_SYMBOL_in_slave_until_options6934);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1252:2: ( COMMA_SYMBOL master_file_def )*

            for (;;)
            {
                int alt454=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt454=1;
                	}
                    break;

                }

                switch (alt454)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1252:3: COMMA_SYMBOL master_file_def
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_slave_until_options6941);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_master_file_def_in_slave_until_options6943);
            	        master_file_def(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_until_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop454;	/* break out of the loop */
            	    break;
                }
            }
            loop454: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleslave_until_optionsEx; /* Prevent compiler warnings */
    ruleslave_until_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end slave_until_options */

/**
 * $ANTLR start slave_connection_options
 * MySQLSimpleParser.g:1255:1: slave_connection_options : ({...}? ( USER_SYMBOL EQUAL_OPERATOR text_string )? ( PASSWORD_SYMBOL EQUAL_OPERATOR text_string )? ( DEFAULT_AUTH_SYMBOL EQUAL_OPERATOR text_string )? ( PLUGIN_DIR_SYMBOL EQUAL_OPERATOR text_string )? |);
 */
static void
slave_connection_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1255:25: ({...}? ( USER_SYMBOL EQUAL_OPERATOR text_string )? ( PASSWORD_SYMBOL EQUAL_OPERATOR text_string )? ( DEFAULT_AUTH_SYMBOL EQUAL_OPERATOR text_string )? ( PLUGIN_DIR_SYMBOL EQUAL_OPERATOR text_string )? |)

            ANTLR3_UINT32 alt459;

            alt459=2;

            {
                int LA459_0 = LA(1);
                if ( (LA459_0 == DEFAULT_AUTH_SYMBOL || LA459_0 == PASSWORD_SYMBOL || LA459_0 == PLUGIN_DIR_SYMBOL || LA459_0 == USER_SYMBOL))
                {
                    alt459=1;
                }
                else if ( ((SERVER_VERSION >= 50706)) && (LA459_0 == FOR_SYMBOL))
                {
                    {
                        int LA459_5 = LA(2);
                        if ( (( (( ((SERVER_VERSION >= 50604)) && ((SERVER_VERSION >= 50604)) )) && ((SERVER_VERSION >= 50706)) )))
                        {
                            alt459=1;
                        }
                        else if ( ((SERVER_VERSION >= 50706)))
                        {
                            alt459=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 459;
                            EXCEPTION->state        = 5;


                            goto ruleslave_connection_optionsEx;

                        }
                    }
                }
                else if ( (LA459_0 == SEMICOLON_SYMBOL))
                {
                    {
                        int LA459_6 = LA(2);
                        if ( ((SERVER_VERSION >= 50604)))
                        {
                            alt459=1;
                        }
                        else if ( (ANTLR3_TRUE))
                        {
                            alt459=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 459;
                            EXCEPTION->state        = 6;


                            goto ruleslave_connection_optionsEx;

                        }
                    }
                }
                else if ( (LA459_0 == EOF))
                {
                    {
                        int LA459_7 = LA(2);
                        if ( ((SERVER_VERSION >= 50604)))
                        {
                            alt459=1;
                        }
                        else if ( (ANTLR3_TRUE))
                        {
                            alt459=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 459;
                            EXCEPTION->state        = 7;


                            goto ruleslave_connection_optionsEx;

                        }
                    }
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 459;
                    EXCEPTION->state        = 0;


                    goto ruleslave_connection_optionsEx;

                }
            }
            switch (alt459)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1256:2: {...}? ( USER_SYMBOL EQUAL_OPERATOR text_string )? ( PASSWORD_SYMBOL EQUAL_OPERATOR text_string )? ( DEFAULT_AUTH_SYMBOL EQUAL_OPERATOR text_string )? ( PLUGIN_DIR_SYMBOL EQUAL_OPERATOR text_string )?
        	    {
        	        if ( !((SERVER_VERSION >= 50604)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50604";
        	                EXCEPTION->ruleName	 = (void *)"slave_connection_options";


        	        }

        	        // MySQLSimpleParser.g:1256:29: ( USER_SYMBOL EQUAL_OPERATOR text_string )?
        	        {
        	            int alt455=2;
        	            switch ( LA(1) )
        	            {
        	                case USER_SYMBOL:
        	                	{
        	                		alt455=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt455)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1256:30: USER_SYMBOL EQUAL_OPERATOR text_string
        	        	    {
        	        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_slave_connection_options6957);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6959);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_string_in_slave_connection_options6961);
        	        	        text_string(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1256:71: ( PASSWORD_SYMBOL EQUAL_OPERATOR text_string )?
        	        {
        	            int alt456=2;
        	            switch ( LA(1) )
        	            {
        	                case PASSWORD_SYMBOL:
        	                	{
        	                		alt456=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt456)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1256:72: PASSWORD_SYMBOL EQUAL_OPERATOR text_string
        	        	    {
        	        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_slave_connection_options6966);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6968);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_string_in_slave_connection_options6970);
        	        	        text_string(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1257:3: ( DEFAULT_AUTH_SYMBOL EQUAL_OPERATOR text_string )?
        	        {
        	            int alt457=2;
        	            switch ( LA(1) )
        	            {
        	                case DEFAULT_AUTH_SYMBOL:
        	                	{
        	                		alt457=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt457)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1257:4: DEFAULT_AUTH_SYMBOL EQUAL_OPERATOR text_string
        	        	    {
        	        	         MATCHT(DEFAULT_AUTH_SYMBOL, &FOLLOW_DEFAULT_AUTH_SYMBOL_in_slave_connection_options6977);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6979);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_string_in_slave_connection_options6981);
        	        	        text_string(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1257:53: ( PLUGIN_DIR_SYMBOL EQUAL_OPERATOR text_string )?
        	        {
        	            int alt458=2;
        	            switch ( LA(1) )
        	            {
        	                case PLUGIN_DIR_SYMBOL:
        	                	{
        	                		alt458=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt458)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1257:54: PLUGIN_DIR_SYMBOL EQUAL_OPERATOR text_string
        	        	    {
        	        	         MATCHT(PLUGIN_DIR_SYMBOL, &FOLLOW_PLUGIN_DIR_SYMBOL_in_slave_connection_options6986);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_slave_connection_options6988);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_string_in_slave_connection_options6990);
        	        	        text_string(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleslave_connection_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1259:2: 
        	    {
        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleslave_connection_optionsEx; /* Prevent compiler warnings */
    ruleslave_connection_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end slave_connection_options */

/**
 * $ANTLR start slave_thread_options
 * MySQLSimpleParser.g:1261:1: slave_thread_options : slave_thread_option ( COMMA_SYMBOL slave_thread_option )* ;
 */
static void
slave_thread_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1261:21: ( slave_thread_option ( COMMA_SYMBOL slave_thread_option )* )
        // MySQLSimpleParser.g:1262:2: slave_thread_option ( COMMA_SYMBOL slave_thread_option )*
        {
            FOLLOWPUSH(FOLLOW_slave_thread_option_in_slave_thread_options7006);
            slave_thread_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleslave_thread_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1262:22: ( COMMA_SYMBOL slave_thread_option )*

            for (;;)
            {
                int alt460=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt460=1;
                	}
                    break;

                }

                switch (alt460)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1262:23: COMMA_SYMBOL slave_thread_option
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_slave_thread_options7009);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_thread_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_slave_thread_option_in_slave_thread_options7011);
            	        slave_thread_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleslave_thread_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop460;	/* break out of the loop */
            	    break;
                }
            }
            loop460: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleslave_thread_optionsEx; /* Prevent compiler warnings */
    ruleslave_thread_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end slave_thread_options */

/**
 * $ANTLR start slave_thread_option
 * MySQLSimpleParser.g:1265:1: slave_thread_option : ( RELAY_THREAD_SYMBOL | SQL_THREAD_SYMBOL );
 */
static void
slave_thread_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1265:20: ( RELAY_THREAD_SYMBOL | SQL_THREAD_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == RELAY_THREAD_SYMBOL || LA(1) == SQL_THREAD_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleslave_thread_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleslave_thread_optionEx; /* Prevent compiler warnings */
    ruleslave_thread_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end slave_thread_option */

/**
 * $ANTLR start group_replication
 * MySQLSimpleParser.g:1269:1: group_replication : ( START_SYMBOL | STOP_SYMBOL ) GROUP_REPLICATION_SYMBOL ;
 */
static void
group_replication(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1269:18: ( ( START_SYMBOL | STOP_SYMBOL ) GROUP_REPLICATION_SYMBOL )
        // MySQLSimpleParser.g:1270:2: ( START_SYMBOL | STOP_SYMBOL ) GROUP_REPLICATION_SYMBOL
        {
            if ( ((LA(1) >= START_SYMBOL) && (LA(1) <= STOP_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulegroup_replicationEx;
            }


             MATCHT(GROUP_REPLICATION_SYMBOL, &FOLLOW_GROUP_REPLICATION_SYMBOL_in_group_replication7043);
            if  (HASEXCEPTION())
            {
                goto rulegroup_replicationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulegroup_replicationEx; /* Prevent compiler warnings */
    rulegroup_replicationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end group_replication */

/**
 * $ANTLR start prepared_statement
 * MySQLSimpleParser.g:1275:1: prepared_statement : ( PREPARE_SYMBOL identifier FROM_SYMBOL ( string_literal | user_variable ) | execute_statement | ( DEALLOCATE_SYMBOL | DROP_SYMBOL ) PREPARE_SYMBOL identifier );
 */
static void
prepared_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1275:19: ( PREPARE_SYMBOL identifier FROM_SYMBOL ( string_literal | user_variable ) | execute_statement | ( DEALLOCATE_SYMBOL | DROP_SYMBOL ) PREPARE_SYMBOL identifier )

            ANTLR3_UINT32 alt462;

            alt462=3;

            switch ( LA(1) )
            {
            case PREPARE_SYMBOL:
            	{
            		alt462=1;
            	}
                break;
            case EXECUTE_SYMBOL:
            	{
            		alt462=2;
            	}
                break;
            case DEALLOCATE_SYMBOL:
            case DROP_SYMBOL:
            	{
            		alt462=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 462;
                EXCEPTION->state        = 0;


                goto ruleprepared_statementEx;

            }

            switch (alt462)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1276:2: PREPARE_SYMBOL identifier FROM_SYMBOL ( string_literal | user_variable )
        	    {
        	         MATCHT(PREPARE_SYMBOL, &FOLLOW_PREPARE_SYMBOL_in_prepared_statement7054);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_prepared_statement7056);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_prepared_statement7058);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1276:40: ( string_literal | user_variable )
        	        {
        	            int alt461=2;
        	            switch ( LA(1) )
        	            {
        	            case NCHAR_TEXT:
        	            case SINGLE_QUOTED_TEXT:
        	            case DOUBLE_QUOTED_TEXT:
        	            case UNDERSCORE_CHARSET:
        	            	{
        	            		alt461=1;
        	            	}
        	                break;
        	            case AT_SIGN_SYMBOL:
        	            case AT_TEXT_SUFFIX:
        	            	{
        	            		alt461=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 461;
        	                EXCEPTION->state        = 0;


        	                goto ruleprepared_statementEx;

        	            }

        	            switch (alt461)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1276:41: string_literal
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_literal_in_prepared_statement7061);
        	        	        string_literal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprepared_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1276:58: user_variable
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_user_variable_in_prepared_statement7065);
        	        	        user_variable(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprepared_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1277:4: execute_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_execute_statement_in_prepared_statement7071);
        	        execute_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1278:4: ( DEALLOCATE_SYMBOL | DROP_SYMBOL ) PREPARE_SYMBOL identifier
        	    {
        	        if ( LA(1) == DEALLOCATE_SYMBOL || LA(1) == DROP_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleprepared_statementEx;
        	        }


        	         MATCHT(PREPARE_SYMBOL, &FOLLOW_PREPARE_SYMBOL_in_prepared_statement7084);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_prepared_statement7086);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprepared_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleprepared_statementEx; /* Prevent compiler warnings */
    ruleprepared_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end prepared_statement */

/**
 * $ANTLR start execute_statement
 * MySQLSimpleParser.g:1281:1: execute_statement : EXECUTE_SYMBOL identifier ( USING_SYMBOL execute_var_list )? ;
 */
static void
execute_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1281:18: ( EXECUTE_SYMBOL identifier ( USING_SYMBOL execute_var_list )? )
        // MySQLSimpleParser.g:1282:2: EXECUTE_SYMBOL identifier ( USING_SYMBOL execute_var_list )?
        {
             MATCHT(EXECUTE_SYMBOL, &FOLLOW_EXECUTE_SYMBOL_in_execute_statement7095);
            if  (HASEXCEPTION())
            {
                goto ruleexecute_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_execute_statement7097);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexecute_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1282:28: ( USING_SYMBOL execute_var_list )?
            {
                int alt463=2;
                switch ( LA(1) )
                {
                    case USING_SYMBOL:
                    	{
                    		alt463=1;
                    	}
                        break;
                }

                switch (alt463)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1282:29: USING_SYMBOL execute_var_list
            	    {
            	         MATCHT(USING_SYMBOL, &FOLLOW_USING_SYMBOL_in_execute_statement7100);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexecute_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_execute_var_list_in_execute_statement7102);
            	        execute_var_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexecute_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexecute_statementEx; /* Prevent compiler warnings */
    ruleexecute_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end execute_statement */

/**
 * $ANTLR start execute_var_list
 * MySQLSimpleParser.g:1285:1: execute_var_list : user_variable ( COMMA_SYMBOL user_variable )* ;
 */
static void
execute_var_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1285:17: ( user_variable ( COMMA_SYMBOL user_variable )* )
        // MySQLSimpleParser.g:1286:2: user_variable ( COMMA_SYMBOL user_variable )*
        {
            FOLLOWPUSH(FOLLOW_user_variable_in_execute_var_list7113);
            user_variable(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexecute_var_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1286:16: ( COMMA_SYMBOL user_variable )*

            for (;;)
            {
                int alt464=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt464=1;
                	}
                    break;

                }

                switch (alt464)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1286:17: COMMA_SYMBOL user_variable
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_execute_var_list7116);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexecute_var_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_variable_in_execute_var_list7118);
            	        user_variable(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexecute_var_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop464;	/* break out of the loop */
            	    break;
                }
            }
            loop464: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexecute_var_listEx; /* Prevent compiler warnings */
    ruleexecute_var_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end execute_var_list */

/**
 * $ANTLR start account_management_statement
 * MySQLSimpleParser.g:1291:1: account_management_statement : ({...}? alter_user | create_user | drop_user |{...}? grant_proxy | grant | rename_user | revoke_statement | set_password );
 */
static void
account_management_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1291:29: ({...}? alter_user | create_user | drop_user |{...}? grant_proxy | grant | rename_user | revoke_statement | set_password )

            ANTLR3_UINT32 alt465;

            alt465=8;

            switch ( LA(1) )
            {
            case ALTER_SYMBOL:
            	{
            		alt465=1;
            	}
                break;
            case CREATE_SYMBOL:
            	{
            		alt465=2;
            	}
                break;
            case DROP_SYMBOL:
            	{
            		alt465=3;
            	}
                break;
            case GRANT_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case PROXY_SYMBOL:
            			{
            				{
            				    int LA465_8 = LA(3);
            				    if ( ((SERVER_VERSION >= 50500)))
            				    {
            				        alt465=4;
            				    }
            				    else if ( (ANTLR3_TRUE))
            				    {
            				        alt465=5;
            				    }
            				    else
            				    {
            				        if (BACKTRACKING>0)
            				        {
            				            FAILEDFLAG = ANTLR3_TRUE;
            				            return ;
            				        }


            				        CONSTRUCTEX();
            				        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            				        EXCEPTION->message      = (void *)"";
            				        EXCEPTION->decisionNum  = 465;
            				        EXCEPTION->state        = 8;


            				        goto ruleaccount_management_statementEx;

            				    }
            				}
            			}
            		    break;
            		case EXECUTE_SYMBOL:
            		case EVENT_SYMBOL:
            		case FILE_SYMBOL:
            		case PROCESS_SYMBOL:
            		case RELOAD_SYMBOL:
            		case REPLICATION_SYMBOL:
            		case SHUTDOWN_SYMBOL:
            		case SUPER_SYMBOL:
            		case ALL_SYMBOL:
            		case ALTER_SYMBOL:
            		case CREATE_SYMBOL:
            		case DELETE_SYMBOL:
            		case DROP_SYMBOL:
            		case GRANT_SYMBOL:
            		case INDEX_SYMBOL:
            		case INSERT_SYMBOL:
            		case LOCK_SYMBOL:
            		case REFERENCES_SYMBOL:
            		case SELECT_SYMBOL:
            		case SHOW_SYMBOL:
            		case TRIGGER_SYMBOL:
            		case UPDATE_SYMBOL:
            		case USAGE_SYMBOL:
            			{
            				alt465=5;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 465;
            		    EXCEPTION->state        = 4;


            		    goto ruleaccount_management_statementEx;

            		}

            	}
                break;
            case RENAME_SYMBOL:
            	{
            		alt465=6;
            	}
                break;
            case REVOKE_SYMBOL:
            	{
            		alt465=7;
            	}
                break;
            case SET_SYMBOL:
            	{
            		alt465=8;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 465;
                EXCEPTION->state        = 0;


                goto ruleaccount_management_statementEx;

            }

            switch (alt465)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1292:2: {...}? alter_user
        	    {
        	        if ( !((SERVER_VERSION >= 50606)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50606";
        	                EXCEPTION->ruleName	 = (void *)"account_management_statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_alter_user_in_account_management_statement7133);
        	        alter_user(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1293:4: create_user
        	    {
        	        FOLLOWPUSH(FOLLOW_create_user_in_account_management_statement7138);
        	        create_user(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1294:4: drop_user
        	    {
        	        FOLLOWPUSH(FOLLOW_drop_user_in_account_management_statement7143);
        	        drop_user(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1295:4: {...}? grant_proxy
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"account_management_statement";


        	        }

        	        FOLLOWPUSH(FOLLOW_grant_proxy_in_account_management_statement7150);
        	        grant_proxy(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1296:4: grant
        	    {
        	        FOLLOWPUSH(FOLLOW_grant_in_account_management_statement7155);
        	        grant(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1297:4: rename_user
        	    {
        	        FOLLOWPUSH(FOLLOW_rename_user_in_account_management_statement7160);
        	        rename_user(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1298:4: revoke_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_revoke_statement_in_account_management_statement7165);
        	        revoke_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1299:4: set_password
        	    {
        	        FOLLOWPUSH(FOLLOW_set_password_in_account_management_statement7170);
        	        set_password(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_management_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleaccount_management_statementEx; /* Prevent compiler warnings */
    ruleaccount_management_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end account_management_statement */

/**
 * $ANTLR start alter_user
 * MySQLSimpleParser.g:1302:1: alter_user : ALTER_SYMBOL USER_SYMBOL ({...}? if_exists |) alter_user_tail ;
 */
static void
alter_user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1302:11: ( ALTER_SYMBOL USER_SYMBOL ({...}? if_exists |) alter_user_tail )
        // MySQLSimpleParser.g:1303:2: ALTER_SYMBOL USER_SYMBOL ({...}? if_exists |) alter_user_tail
        {
             MATCHT(ALTER_SYMBOL, &FOLLOW_ALTER_SYMBOL_in_alter_user7179);
            if  (HASEXCEPTION())
            {
                goto rulealter_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_alter_user7181);
            if  (HASEXCEPTION())
            {
                goto rulealter_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1303:27: ({...}? if_exists |)
            {
                int alt466=2;
                {
                    int LA466_0 = LA(1);
                    if ( (LA466_0 == IF_SYMBOL))
                    {
                        alt466=1;
                    }
                    else if ( (((LA466_0 >= ACCOUNT_SYMBOL) && (LA466_0 <= YEAR_SYMBOL)) || LA466_0 == CURRENT_USER_SYMBOL || LA466_0 == SINGLE_QUOTED_TEXT || LA466_0 == BACK_TICK_QUOTED_ID || LA466_0 == DOUBLE_QUOTED_TEXT || LA466_0 == IDENTIFIER))
                    {
                        alt466=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 466;
                        EXCEPTION->state        = 0;


                        goto rulealter_userEx;

                    }
                }
                switch (alt466)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1303:28: {...}? if_exists
            	    {
            	        if ( !((SERVER_VERSION >= 50706)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
            	                EXCEPTION->ruleName	 = (void *)"alter_user";


            	        }

            	        FOLLOWPUSH(FOLLOW_if_exists_in_alter_user7186);
            	        if_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealter_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1303:78: 
            	    {
            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_alter_user_tail_in_alter_user7193);
            alter_user_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealter_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_userEx; /* Prevent compiler warnings */
    rulealter_userEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_user */

/**
 * $ANTLR start alter_user_tail
 * MySQLSimpleParser.g:1306:1: alter_user_tail : ( grant_list create_user_tail |{...}? USER_SYMBOL parentheses IDENTIFIED_SYMBOL BY_SYMBOL text_string );
 */
static void
alter_user_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1306:16: ( grant_list create_user_tail |{...}? USER_SYMBOL parentheses IDENTIFIED_SYMBOL BY_SYMBOL text_string )

            ANTLR3_UINT32 alt467;

            alt467=2;

            {
                int LA467_0 = LA(1);
                if ( (((LA467_0 >= ACCOUNT_SYMBOL) && (LA467_0 <= USER_RESOURCES_SYMBOL)) || ((LA467_0 >= USE_FRM_SYMBOL) && (LA467_0 <= YEAR_SYMBOL)) || LA467_0 == CURRENT_USER_SYMBOL || LA467_0 == SINGLE_QUOTED_TEXT || LA467_0 == BACK_TICK_QUOTED_ID || LA467_0 == DOUBLE_QUOTED_TEXT || LA467_0 == IDENTIFIER))
                {
                    alt467=1;
                }
                else if ( (LA467_0 == USER_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case EOF:
                    case ACCOUNT_SYMBOL:
                    case IDENTIFIED_SYMBOL:
                    case PASSWORD_SYMBOL:
                    case REQUIRE_SYMBOL:
                    case SEMICOLON_SYMBOL:
                    case WITH_SYMBOL:
                    case AT_SIGN_SYMBOL:
                    case AT_TEXT_SUFFIX:
                    case COMMA_SYMBOL:
                    	{
                    		alt467=1;
                    	}
                        break;
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt467=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 467;
                        EXCEPTION->state        = 5;


                        goto rulealter_user_tailEx;

                    }

                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 467;
                    EXCEPTION->state        = 0;


                    goto rulealter_user_tailEx;

                }
            }
            switch (alt467)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1307:2: grant_list create_user_tail
        	    {
        	        FOLLOWPUSH(FOLLOW_grant_list_in_alter_user_tail7202);
        	        grant_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_create_user_tail_in_alter_user_tail7204);
        	        create_user_tail(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1308:4: {...}? USER_SYMBOL parentheses IDENTIFIED_SYMBOL BY_SYMBOL text_string
        	    {
        	        if ( !((SERVER_VERSION >= 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
        	                EXCEPTION->ruleName	 = (void *)"alter_user_tail";


        	        }

        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_alter_user_tail7211);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_parentheses_in_alter_user_tail7213);
        	        parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(IDENTIFIED_SYMBOL, &FOLLOW_IDENTIFIED_SYMBOL_in_alter_user_tail7215);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_alter_user_tail7217);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_alter_user_tail7219);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulealter_user_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulealter_user_tailEx; /* Prevent compiler warnings */
    rulealter_user_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_user_tail */

/**
 * $ANTLR start create_user
 * MySQLSimpleParser.g:1311:1: create_user : CREATE_SYMBOL USER_SYMBOL ({...}? if_not_exists |) grant_list create_user_tail ;
 */
static void
create_user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1311:12: ( CREATE_SYMBOL USER_SYMBOL ({...}? if_not_exists |) grant_list create_user_tail )
        // MySQLSimpleParser.g:1312:2: CREATE_SYMBOL USER_SYMBOL ({...}? if_not_exists |) grant_list create_user_tail
        {
             MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_create_user7228);
            if  (HASEXCEPTION())
            {
                goto rulecreate_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_create_user7230);
            if  (HASEXCEPTION())
            {
                goto rulecreate_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1312:28: ({...}? if_not_exists |)
            {
                int alt468=2;
                {
                    int LA468_0 = LA(1);
                    if ( (LA468_0 == IF_SYMBOL))
                    {
                        alt468=1;
                    }
                    else if ( (((LA468_0 >= ACCOUNT_SYMBOL) && (LA468_0 <= YEAR_SYMBOL)) || LA468_0 == CURRENT_USER_SYMBOL || LA468_0 == SINGLE_QUOTED_TEXT || LA468_0 == BACK_TICK_QUOTED_ID || LA468_0 == DOUBLE_QUOTED_TEXT || LA468_0 == IDENTIFIER))
                    {
                        alt468=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 468;
                        EXCEPTION->state        = 0;


                        goto rulecreate_userEx;

                    }
                }
                switch (alt468)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1312:29: {...}? if_not_exists
            	    {
            	        if ( !((SERVER_VERSION >= 50706)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
            	                EXCEPTION->ruleName	 = (void *)"create_user";


            	        }

            	        FOLLOWPUSH(FOLLOW_if_not_exists_in_create_user7235);
            	        if_not_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1312:83: 
            	    {
            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_grant_list_in_create_user7242);
            grant_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_create_user_tail_in_create_user7244);
            create_user_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_userEx; /* Prevent compiler warnings */
    rulecreate_userEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_user */

/**
 * $ANTLR start create_user_tail
 * MySQLSimpleParser.g:1315:1: create_user_tail : ({...}? ( require_clause )? ( connect_options )? ( account_lock_password_expire_options )? |);
 */
static void
create_user_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1315:17: ({...}? ( require_clause )? ( connect_options )? ( account_lock_password_expire_options )? |)

            ANTLR3_UINT32 alt472;

            alt472=2;

            switch ( LA(1) )
            {
            case ACCOUNT_SYMBOL:
            case PASSWORD_SYMBOL:
            case REQUIRE_SYMBOL:
            case WITH_SYMBOL:
            	{
            		alt472=1;
            	}
                break;
            case SEMICOLON_SYMBOL:
            	{
            		{
            		    int LA472_5 = LA(2);
            		    if ( ((SERVER_VERSION >= 50706)))
            		    {
            		        alt472=1;
            		    }
            		    else if ( (ANTLR3_TRUE))
            		    {
            		        alt472=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 472;
            		        EXCEPTION->state        = 5;


            		        goto rulecreate_user_tailEx;

            		    }
            		}
            	}
                break;
            case EOF:
            	{
            		{
            		    int LA472_6 = LA(2);
            		    if ( ((SERVER_VERSION >= 50706)))
            		    {
            		        alt472=1;
            		    }
            		    else if ( (ANTLR3_TRUE))
            		    {
            		        alt472=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 472;
            		        EXCEPTION->state        = 6;


            		        goto rulecreate_user_tailEx;

            		    }
            		}
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 472;
                EXCEPTION->state        = 0;


                goto rulecreate_user_tailEx;

            }

            switch (alt472)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1316:2: {...}? ( require_clause )? ( connect_options )? ( account_lock_password_expire_options )?
        	    {
        	        if ( !((SERVER_VERSION >= 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
        	                EXCEPTION->ruleName	 = (void *)"create_user_tail";


        	        }

        	        // MySQLSimpleParser.g:1316:29: ( require_clause )?
        	        {
        	            int alt469=2;
        	            switch ( LA(1) )
        	            {
        	                case REQUIRE_SYMBOL:
        	                	{
        	                		alt469=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt469)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1316:29: require_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_require_clause_in_create_user_tail7255);
        	        	        require_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_user_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1316:45: ( connect_options )?
        	        {
        	            int alt470=2;
        	            switch ( LA(1) )
        	            {
        	                case WITH_SYMBOL:
        	                	{
        	                		alt470=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt470)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1316:45: connect_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_connect_options_in_create_user_tail7258);
        	        	        connect_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_user_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1316:62: ( account_lock_password_expire_options )?
        	        {
        	            int alt471=2;
        	            switch ( LA(1) )
        	            {
        	                case ACCOUNT_SYMBOL:
        	                case PASSWORD_SYMBOL:
        	                	{
        	                		alt471=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt471)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1316:62: account_lock_password_expire_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_account_lock_password_expire_options_in_create_user_tail7261);
        	        	        account_lock_password_expire_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_user_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1318:2: 
        	    {
        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecreate_user_tailEx; /* Prevent compiler warnings */
    rulecreate_user_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_user_tail */

/**
 * $ANTLR start require_clause
 * MySQLSimpleParser.g:1320:1: require_clause : REQUIRE_SYMBOL ( require_list | SSL_SYMBOL | X509_SYMBOL | NONE_SYMBOL ) ;
 */
static void
require_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1320:15: ( REQUIRE_SYMBOL ( require_list | SSL_SYMBOL | X509_SYMBOL | NONE_SYMBOL ) )
        // MySQLSimpleParser.g:1321:2: REQUIRE_SYMBOL ( require_list | SSL_SYMBOL | X509_SYMBOL | NONE_SYMBOL )
        {
             MATCHT(REQUIRE_SYMBOL, &FOLLOW_REQUIRE_SYMBOL_in_require_clause7276);
            if  (HASEXCEPTION())
            {
                goto rulerequire_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1321:17: ( require_list | SSL_SYMBOL | X509_SYMBOL | NONE_SYMBOL )
            {
                int alt473=4;
                switch ( LA(1) )
                {
                case CIPHER_SYMBOL:
                case ISSUER_SYMBOL:
                case SUBJECT_SYMBOL:
                	{
                		alt473=1;
                	}
                    break;
                case SSL_SYMBOL:
                	{
                		alt473=2;
                	}
                    break;
                case X509_SYMBOL:
                	{
                		alt473=3;
                	}
                    break;
                case NONE_SYMBOL:
                	{
                		alt473=4;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 473;
                    EXCEPTION->state        = 0;


                    goto rulerequire_clauseEx;

                }

                switch (alt473)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1321:18: require_list
            	    {
            	        FOLLOWPUSH(FOLLOW_require_list_in_require_clause7279);
            	        require_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerequire_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1321:33: SSL_SYMBOL
            	    {
            	         MATCHT(SSL_SYMBOL, &FOLLOW_SSL_SYMBOL_in_require_clause7283);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerequire_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1321:46: X509_SYMBOL
            	    {
            	         MATCHT(X509_SYMBOL, &FOLLOW_X509_SYMBOL_in_require_clause7287);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerequire_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:1321:60: NONE_SYMBOL
            	    {
            	         MATCHT(NONE_SYMBOL, &FOLLOW_NONE_SYMBOL_in_require_clause7291);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerequire_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulerequire_clauseEx; /* Prevent compiler warnings */
    rulerequire_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end require_clause */

/**
 * $ANTLR start connect_options
 * MySQLSimpleParser.g:1324:1: connect_options : WITH_SYMBOL ( MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number )+ ;
 */
static void
connect_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1324:16: ( WITH_SYMBOL ( MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number )+ )
        // MySQLSimpleParser.g:1325:2: WITH_SYMBOL ( MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number )+
        {
             MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_connect_options7301);
            if  (HASEXCEPTION())
            {
                goto ruleconnect_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1326:2: ( MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number )+
            {
                int cnt474=0;

                for (;;)
                {
                    int alt474=5;
            	switch ( LA(1) )
            	{
            	case MAX_QUERIES_PER_HOUR_SYMBOL:
            		{
            			alt474=1;
            		}
            	    break;
            	case MAX_UPDATES_PER_HOUR_SYMBOL:
            		{
            			alt474=2;
            		}
            	    break;
            	case MAX_CONNECTIONS_PER_HOUR_SYMBOL:
            		{
            			alt474=3;
            		}
            	    break;
            	case MAX_USER_CONNECTIONS_SYMBOL:
            		{
            			alt474=4;
            		}
            	    break;

            	}

            	switch (alt474)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:1327:3: MAX_QUERIES_PER_HOUR_SYMBOL ulong_number
            	        {
            	             MATCHT(MAX_QUERIES_PER_HOUR_SYMBOL, &FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_connect_options7308);
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_ulong_number_in_connect_options7310);
            	            ulong_number(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;
            	    case 2:
            	        // MySQLSimpleParser.g:1328:5: MAX_UPDATES_PER_HOUR_SYMBOL ulong_number
            	        {
            	             MATCHT(MAX_UPDATES_PER_HOUR_SYMBOL, &FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_connect_options7316);
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_ulong_number_in_connect_options7318);
            	            ulong_number(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;
            	    case 3:
            	        // MySQLSimpleParser.g:1329:5: MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number
            	        {
            	             MATCHT(MAX_CONNECTIONS_PER_HOUR_SYMBOL, &FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_connect_options7324);
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_ulong_number_in_connect_options7326);
            	            ulong_number(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;
            	    case 4:
            	        // MySQLSimpleParser.g:1330:5: MAX_USER_CONNECTIONS_SYMBOL ulong_number
            	        {
            	             MATCHT(MAX_USER_CONNECTIONS_SYMBOL, &FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_connect_options7332);
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_ulong_number_in_connect_options7334);
            	            ulong_number(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleconnect_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt474 >= 1 )
            		{
            		    goto loop474;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto ruleconnect_optionsEx;
            	}
            	cnt474++;
                }
                loop474: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleconnect_optionsEx; /* Prevent compiler warnings */
    ruleconnect_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end connect_options */

/**
 * $ANTLR start account_lock_password_expire_options
 * MySQLSimpleParser.g:1334:1: account_lock_password_expire_options : ( ACCOUNT_SYMBOL ( LOCK_SYMBOL | UNLOCK_SYMBOL ) | PASSWORD_SYMBOL EXPIRE_SYMBOL ( INTERVAL_SYMBOL real_ulong_number DAY_SYMBOL | NEVER_SYMBOL | DEFAULT_SYMBOL ) );
 */
static void
account_lock_password_expire_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1334:37: ( ACCOUNT_SYMBOL ( LOCK_SYMBOL | UNLOCK_SYMBOL ) | PASSWORD_SYMBOL EXPIRE_SYMBOL ( INTERVAL_SYMBOL real_ulong_number DAY_SYMBOL | NEVER_SYMBOL | DEFAULT_SYMBOL ) )

            ANTLR3_UINT32 alt476;

            alt476=2;

            switch ( LA(1) )
            {
            case ACCOUNT_SYMBOL:
            	{
            		alt476=1;
            	}
                break;
            case PASSWORD_SYMBOL:
            	{
            		alt476=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 476;
                EXCEPTION->state        = 0;


                goto ruleaccount_lock_password_expire_optionsEx;

            }

            switch (alt476)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1335:2: ACCOUNT_SYMBOL ( LOCK_SYMBOL | UNLOCK_SYMBOL )
        	    {
        	         MATCHT(ACCOUNT_SYMBOL, &FOLLOW_ACCOUNT_SYMBOL_in_account_lock_password_expire_options7347);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == LOCK_SYMBOL || LA(1) == UNLOCK_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1336:4: PASSWORD_SYMBOL EXPIRE_SYMBOL ( INTERVAL_SYMBOL real_ulong_number DAY_SYMBOL | NEVER_SYMBOL | DEFAULT_SYMBOL )
        	    {
        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_account_lock_password_expire_options7360);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EXPIRE_SYMBOL, &FOLLOW_EXPIRE_SYMBOL_in_account_lock_password_expire_options7362);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1337:3: ( INTERVAL_SYMBOL real_ulong_number DAY_SYMBOL | NEVER_SYMBOL | DEFAULT_SYMBOL )
        	        {
        	            int alt475=3;
        	            switch ( LA(1) )
        	            {
        	            case INTERVAL_SYMBOL:
        	            	{
        	            		alt475=1;
        	            	}
        	                break;
        	            case NEVER_SYMBOL:
        	            	{
        	            		alt475=2;
        	            	}
        	                break;
        	            case DEFAULT_SYMBOL:
        	            	{
        	            		alt475=3;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 475;
        	                EXCEPTION->state        = 0;


        	                goto ruleaccount_lock_password_expire_optionsEx;

        	            }

        	            switch (alt475)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1338:4: INTERVAL_SYMBOL real_ulong_number DAY_SYMBOL
        	        	    {
        	        	         MATCHT(INTERVAL_SYMBOL, &FOLLOW_INTERVAL_SYMBOL_in_account_lock_password_expire_options7371);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_account_lock_password_expire_options7373);
        	        	        real_ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(DAY_SYMBOL, &FOLLOW_DAY_SYMBOL_in_account_lock_password_expire_options7375);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1339:6: NEVER_SYMBOL
        	        	    {
        	        	         MATCHT(NEVER_SYMBOL, &FOLLOW_NEVER_SYMBOL_in_account_lock_password_expire_options7382);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:1340:6: DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_account_lock_password_expire_options7389);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaccount_lock_password_expire_optionsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleaccount_lock_password_expire_optionsEx; /* Prevent compiler warnings */
    ruleaccount_lock_password_expire_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end account_lock_password_expire_options */

/**
 * $ANTLR start drop_user
 * MySQLSimpleParser.g:1344:1: drop_user : DROP_SYMBOL USER_SYMBOL ({...}? if_exists |) user_list ;
 */
static void
drop_user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1344:10: ( DROP_SYMBOL USER_SYMBOL ({...}? if_exists |) user_list )
        // MySQLSimpleParser.g:1345:2: DROP_SYMBOL USER_SYMBOL ({...}? if_exists |) user_list
        {
             MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_drop_user7403);
            if  (HASEXCEPTION())
            {
                goto ruledrop_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_drop_user7405);
            if  (HASEXCEPTION())
            {
                goto ruledrop_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1345:26: ({...}? if_exists |)
            {
                int alt477=2;
                {
                    int LA477_0 = LA(1);
                    if ( (LA477_0 == IF_SYMBOL))
                    {
                        alt477=1;
                    }
                    else if ( (((LA477_0 >= ACCOUNT_SYMBOL) && (LA477_0 <= YEAR_SYMBOL)) || LA477_0 == CURRENT_USER_SYMBOL || LA477_0 == SINGLE_QUOTED_TEXT || LA477_0 == BACK_TICK_QUOTED_ID || LA477_0 == DOUBLE_QUOTED_TEXT || LA477_0 == IDENTIFIER))
                    {
                        alt477=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 477;
                        EXCEPTION->state        = 0;


                        goto ruledrop_userEx;

                    }
                }
                switch (alt477)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1345:27: {...}? if_exists
            	    {
            	        if ( !((SERVER_VERSION >= 50706)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
            	                EXCEPTION->ruleName	 = (void *)"drop_user";


            	        }

            	        FOLLOWPUSH(FOLLOW_if_exists_in_drop_user7410);
            	        if_exists(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledrop_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1345:77: 
            	    {
            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_user_list_in_drop_user7417);
            user_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruledrop_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledrop_userEx; /* Prevent compiler warnings */
    ruledrop_userEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end drop_user */

/**
 * $ANTLR start parse_grant
 * MySQLSimpleParser.g:1348:1: parse_grant : grant EOF ;
 */
static void
parse_grant(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1348:12: ( grant EOF )
        // MySQLSimpleParser.g:1349:2: grant EOF
        {
            FOLLOWPUSH(FOLLOW_grant_in_parse_grant7427);
            grant(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleparse_grantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EOF, &FOLLOW_EOF_in_parse_grant7429);
            if  (HASEXCEPTION())
            {
                goto ruleparse_grantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleparse_grantEx; /* Prevent compiler warnings */
    ruleparse_grantEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end parse_grant */

/**
 * $ANTLR start grant
 * MySQLSimpleParser.g:1352:1: grant : GRANT_SYMBOL grant_privileges privilege_target TO_SYMBOL grant_list ( require_clause )? ( WITH_SYMBOL ( grant_option )+ )? ;
 */
static void
grant(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1352:6: ( GRANT_SYMBOL grant_privileges privilege_target TO_SYMBOL grant_list ( require_clause )? ( WITH_SYMBOL ( grant_option )+ )? )
        // MySQLSimpleParser.g:1353:2: GRANT_SYMBOL grant_privileges privilege_target TO_SYMBOL grant_list ( require_clause )? ( WITH_SYMBOL ( grant_option )+ )?
        {
             MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_grant7438);
            if  (HASEXCEPTION())
            {
                goto rulegrantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_grant_privileges_in_grant7440);
            grant_privileges(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_privilege_target_in_grant7442);
            privilege_target(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_grant7446);
            if  (HASEXCEPTION())
            {
                goto rulegrantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_grant_list_in_grant7448);
            grant_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrantEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1354:24: ( require_clause )?
            {
                int alt478=2;
                switch ( LA(1) )
                {
                    case REQUIRE_SYMBOL:
                    	{
                    		alt478=1;
                    	}
                        break;
                }

                switch (alt478)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1354:24: require_clause
            	    {
            	        FOLLOWPUSH(FOLLOW_require_clause_in_grant7450);
            	        require_clause(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrantEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1354:40: ( WITH_SYMBOL ( grant_option )+ )?
            {
                int alt480=2;
                switch ( LA(1) )
                {
                    case WITH_SYMBOL:
                    	{
                    		alt480=1;
                    	}
                        break;
                }

                switch (alt480)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1354:41: WITH_SYMBOL ( grant_option )+
            	    {
            	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_grant7454);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrantEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1354:53: ( grant_option )+
            	        {
            	            int cnt479=0;

            	            for (;;)
            	            {
            	                int alt479=2;
            	        	switch ( LA(1) )
            	        	{
            	        	case MAX_CONNECTIONS_PER_HOUR_SYMBOL:
            	        	case MAX_QUERIES_PER_HOUR_SYMBOL:
            	        	case MAX_UPDATES_PER_HOUR_SYMBOL:
            	        	case MAX_USER_CONNECTIONS_SYMBOL:
            	        	case GRANT_SYMBOL:
            	        		{
            	        			alt479=1;
            	        		}
            	        	    break;

            	        	}

            	        	switch (alt479)
            	        	{
            	        	    case 1:
            	        	        // MySQLSimpleParser.g:1354:53: grant_option
            	        	        {
            	        	            FOLLOWPUSH(FOLLOW_grant_option_in_grant7456);
            	        	            grant_option(ctx);

            	        	            FOLLOWPOP();
            	        	            if  (HASEXCEPTION())
            	        	            {
            	        	                goto rulegrantEx;
            	        	            }
            	        	            if (HASFAILED())
            	        	            {
            	        	                return ;
            	        	            }


            	        	        }
            	        	        break;

            	        	    default:

            	        		if ( cnt479 >= 1 )
            	        		{
            	        		    goto loop479;
            	        		}
            	        		if (BACKTRACKING>0)
            	        		{
            	        		    FAILEDFLAG = ANTLR3_TRUE;
            	        		    return ;
            	        		}

            	        		/* mismatchedSetEx()
            	        		 */
            	        		CONSTRUCTEX();
            	        		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            	        		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            	        		goto rulegrantEx;
            	        	}
            	        	cnt479++;
            	            }
            	            loop479: ;	/* Jump to here if this rule does not match */
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulegrantEx; /* Prevent compiler warnings */
    rulegrantEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant */

/**
 * $ANTLR start grant_proxy
 * MySQLSimpleParser.g:1357:1: grant_proxy : GRANT_SYMBOL PROXY_SYMBOL ON_SYMBOL grant_user TO_SYMBOL grant_user ( COMMA_SYMBOL grant_user )* ( WITH_SYMBOL GRANT_SYMBOL OPTION_SYMBOL )? ;
 */
static void
grant_proxy(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1357:12: ( GRANT_SYMBOL PROXY_SYMBOL ON_SYMBOL grant_user TO_SYMBOL grant_user ( COMMA_SYMBOL grant_user )* ( WITH_SYMBOL GRANT_SYMBOL OPTION_SYMBOL )? )
        // MySQLSimpleParser.g:1358:2: GRANT_SYMBOL PROXY_SYMBOL ON_SYMBOL grant_user TO_SYMBOL grant_user ( COMMA_SYMBOL grant_user )* ( WITH_SYMBOL GRANT_SYMBOL OPTION_SYMBOL )?
        {
             MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_grant_proxy7468);
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(PROXY_SYMBOL, &FOLLOW_PROXY_SYMBOL_in_grant_proxy7470);
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_grant_proxy7472);
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_grant_user_in_grant_proxy7474);
            grant_user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_grant_proxy7476);
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_grant_user_in_grant_proxy7478);
            grant_user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrant_proxyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1358:70: ( COMMA_SYMBOL grant_user )*

            for (;;)
            {
                int alt481=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt481=1;
                	}
                    break;

                }

                switch (alt481)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1358:71: COMMA_SYMBOL grant_user
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_grant_proxy7481);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_proxyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_grant_user_in_grant_proxy7483);
            	        grant_user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_proxyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop481;	/* break out of the loop */
            	    break;
                }
            }
            loop481: ; /* Jump out to here if this rule does not match */


            // MySQLSimpleParser.g:1359:3: ( WITH_SYMBOL GRANT_SYMBOL OPTION_SYMBOL )?
            {
                int alt482=2;
                switch ( LA(1) )
                {
                    case WITH_SYMBOL:
                    	{
                    		alt482=1;
                    	}
                        break;
                }

                switch (alt482)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1359:4: WITH_SYMBOL GRANT_SYMBOL OPTION_SYMBOL
            	    {
            	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_grant_proxy7490);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_proxyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_grant_proxy7492);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_proxyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_grant_proxy7494);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_proxyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulegrant_proxyEx; /* Prevent compiler warnings */
    rulegrant_proxyEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_proxy */

/**
 * $ANTLR start rename_user
 * MySQLSimpleParser.g:1362:1: rename_user : RENAME_SYMBOL USER_SYMBOL user TO_SYMBOL user ( COMMA_SYMBOL user TO_SYMBOL user )* ;
 */
static void
rename_user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1362:12: ( RENAME_SYMBOL USER_SYMBOL user TO_SYMBOL user ( COMMA_SYMBOL user TO_SYMBOL user )* )
        // MySQLSimpleParser.g:1363:2: RENAME_SYMBOL USER_SYMBOL user TO_SYMBOL user ( COMMA_SYMBOL user TO_SYMBOL user )*
        {
             MATCHT(RENAME_SYMBOL, &FOLLOW_RENAME_SYMBOL_in_rename_user7505);
            if  (HASEXCEPTION())
            {
                goto rulerename_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_rename_user7507);
            if  (HASEXCEPTION())
            {
                goto rulerename_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_user_in_rename_user7509);
            user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerename_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_rename_user7511);
            if  (HASEXCEPTION())
            {
                goto rulerename_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_user_in_rename_user7513);
            user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerename_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1363:48: ( COMMA_SYMBOL user TO_SYMBOL user )*

            for (;;)
            {
                int alt483=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt483=1;
                	}
                    break;

                }

                switch (alt483)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1363:49: COMMA_SYMBOL user TO_SYMBOL user
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_rename_user7516);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_in_rename_user7518);
            	        user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_rename_user7520);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_in_rename_user7522);
            	        user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerename_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop483;	/* break out of the loop */
            	    break;
                }
            }
            loop483: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulerename_userEx; /* Prevent compiler warnings */
    rulerename_userEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end rename_user */

/**
 * $ANTLR start revoke_statement
 * MySQLSimpleParser.g:1366:1: revoke_statement : REVOKE_SYMBOL ({...}? => ALL_SYMBOL ( PRIVILEGES_SYMBOL )? COMMA_SYMBOL GRANT_SYMBOL OPTION_SYMBOL FROM_SYMBOL user_list | grant_privileges privilege_target FROM_SYMBOL user_list |{...}? PROXY_SYMBOL ON_SYMBOL user FROM_SYMBOL user_list ) ;
 */
static void
revoke_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1366:17: ( REVOKE_SYMBOL ({...}? => ALL_SYMBOL ( PRIVILEGES_SYMBOL )? COMMA_SYMBOL GRANT_SYMBOL OPTION_SYMBOL FROM_SYMBOL user_list | grant_privileges privilege_target FROM_SYMBOL user_list |{...}? PROXY_SYMBOL ON_SYMBOL user FROM_SYMBOL user_list ) )
        // MySQLSimpleParser.g:1367:2: REVOKE_SYMBOL ({...}? => ALL_SYMBOL ( PRIVILEGES_SYMBOL )? COMMA_SYMBOL GRANT_SYMBOL OPTION_SYMBOL FROM_SYMBOL user_list | grant_privileges privilege_target FROM_SYMBOL user_list |{...}? PROXY_SYMBOL ON_SYMBOL user FROM_SYMBOL user_list )
        {
             MATCHT(REVOKE_SYMBOL, &FOLLOW_REVOKE_SYMBOL_in_revoke_statement7533);
            if  (HASEXCEPTION())
            {
                goto rulerevoke_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1368:2: ({...}? => ALL_SYMBOL ( PRIVILEGES_SYMBOL )? COMMA_SYMBOL GRANT_SYMBOL OPTION_SYMBOL FROM_SYMBOL user_list | grant_privileges privilege_target FROM_SYMBOL user_list |{...}? PROXY_SYMBOL ON_SYMBOL user FROM_SYMBOL user_list )
            {
                int alt485=3;
                switch ( LA(1) )
                {
                case ALL_SYMBOL:
                	{
                		{
                		    int LA485_1 = LA(2);
                		    if ( (LA485_1 == PRIVILEGES_SYMBOL))
                		    {
                		        {
                		            int LA485_25 = LA(3);
                		            if ( ((LA(1) == ALL_SYMBOL)))
                		            {
                		                alt485=1;
                		            }
                		            else if ( (ANTLR3_TRUE))
                		            {
                		                alt485=2;
                		            }
                		            else
                		            {
                		                if (BACKTRACKING>0)
                		                {
                		                    FAILEDFLAG = ANTLR3_TRUE;
                		                    return ;
                		                }


                		                CONSTRUCTEX();
                		                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		                EXCEPTION->message      = (void *)"";
                		                EXCEPTION->decisionNum  = 485;
                		                EXCEPTION->state        = 25;


                		                goto rulerevoke_statementEx;

                		            }
                		        }
                		    }
                		    else if ( ((LA(1) == ALL_SYMBOL)) && (LA485_1 == COMMA_SYMBOL))
                		    {
                		        alt485=1;
                		    }
                		    else if ( (LA485_1 == ON_SYMBOL))
                		    {
                		        alt485=2;
                		    }
                		    else
                		    {
                		        if (BACKTRACKING>0)
                		        {
                		            FAILEDFLAG = ANTLR3_TRUE;
                		            return ;
                		        }


                		        CONSTRUCTEX();
                		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		        EXCEPTION->message      = (void *)"";
                		        EXCEPTION->decisionNum  = 485;
                		        EXCEPTION->state        = 1;


                		        goto rulerevoke_statementEx;

                		    }
                		}
                	}
                    break;
                case EXECUTE_SYMBOL:
                case EVENT_SYMBOL:
                case FILE_SYMBOL:
                case PROCESS_SYMBOL:
                case RELOAD_SYMBOL:
                case REPLICATION_SYMBOL:
                case SHUTDOWN_SYMBOL:
                case SUPER_SYMBOL:
                case ALTER_SYMBOL:
                case CREATE_SYMBOL:
                case DELETE_SYMBOL:
                case DROP_SYMBOL:
                case GRANT_SYMBOL:
                case INDEX_SYMBOL:
                case INSERT_SYMBOL:
                case LOCK_SYMBOL:
                case REFERENCES_SYMBOL:
                case SELECT_SYMBOL:
                case SHOW_SYMBOL:
                case TRIGGER_SYMBOL:
                case UPDATE_SYMBOL:
                case USAGE_SYMBOL:
                	{
                		alt485=2;
                	}
                    break;
                case PROXY_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case ON_SYMBOL:
                			{
                				{
                				    int LA485_28 = LA(3);
                				    if ( (!( ((SERVER_VERSION >= 50500)) )))
                				    {
                				        alt485=2;
                				    }
                				    else if ( ((SERVER_VERSION >= 50500)))
                				    {
                				        alt485=3;
                				    }
                				    else
                				    {
                				        if (BACKTRACKING>0)
                				        {
                				            FAILEDFLAG = ANTLR3_TRUE;
                				            return ;
                				        }


                				        CONSTRUCTEX();
                				        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                				        EXCEPTION->message      = (void *)"";
                				        EXCEPTION->decisionNum  = 485;
                				        EXCEPTION->state        = 28;


                				        goto rulerevoke_statementEx;

                				    }
                				}
                			}
                		    break;
                		case COMMA_SYMBOL:
                			{
                				alt485=2;
                			}
                		    break;

                		default:
                		    if (BACKTRACKING>0)
                		    {
                		        FAILEDFLAG = ANTLR3_TRUE;
                		        return ;
                		    }

                		    CONSTRUCTEX();
                		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		    EXCEPTION->message      = (void *)"";
                		    EXCEPTION->decisionNum  = 485;
                		    EXCEPTION->state        = 14;


                		    goto rulerevoke_statementEx;

                		}

                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 485;
                    EXCEPTION->state        = 0;


                    goto rulerevoke_statementEx;

                }

                switch (alt485)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1369:3: {...}? => ALL_SYMBOL ( PRIVILEGES_SYMBOL )? COMMA_SYMBOL GRANT_SYMBOL OPTION_SYMBOL FROM_SYMBOL user_list
            	    {
            	        if ( !((LA(1) == ALL_SYMBOL)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"LA(1) == ALL_SYMBOL";
            	                EXCEPTION->ruleName	 = (void *)"revoke_statement";


            	        }

            	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_revoke_statement7544);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1369:40: ( PRIVILEGES_SYMBOL )?
            	        {
            	            int alt484=2;
            	            switch ( LA(1) )
            	            {
            	                case PRIVILEGES_SYMBOL:
            	                	{
            	                		alt484=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt484)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1369:40: PRIVILEGES_SYMBOL
            	        	    {
            	        	         MATCHT(PRIVILEGES_SYMBOL, &FOLLOW_PRIVILEGES_SYMBOL_in_revoke_statement7546);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulerevoke_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_revoke_statement7549);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_revoke_statement7551);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_revoke_statement7553);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_revoke_statement7555);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_list_in_revoke_statement7557);
            	        user_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1370:5: grant_privileges privilege_target FROM_SYMBOL user_list
            	    {
            	        FOLLOWPUSH(FOLLOW_grant_privileges_in_revoke_statement7563);
            	        grant_privileges(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_privilege_target_in_revoke_statement7565);
            	        privilege_target(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_revoke_statement7567);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_list_in_revoke_statement7569);
            	        user_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1371:5: {...}? PROXY_SYMBOL ON_SYMBOL user FROM_SYMBOL user_list
            	    {
            	        if ( !((SERVER_VERSION >= 50500)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
            	                EXCEPTION->ruleName	 = (void *)"revoke_statement";


            	        }

            	         MATCHT(PROXY_SYMBOL, &FOLLOW_PROXY_SYMBOL_in_revoke_statement7577);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_revoke_statement7579);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_in_revoke_statement7581);
            	        user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_revoke_statement7583);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_list_in_revoke_statement7585);
            	        user_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerevoke_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulerevoke_statementEx; /* Prevent compiler warnings */
    rulerevoke_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end revoke_statement */

/**
 * $ANTLR start privilege_target
 * MySQLSimpleParser.g:1375:1: privilege_target : ON_SYMBOL grant_object_type privilege_level ;
 */
static void
privilege_target(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1375:17: ( ON_SYMBOL grant_object_type privilege_level )
        // MySQLSimpleParser.g:1376:2: ON_SYMBOL grant_object_type privilege_level
        {
             MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_privilege_target7597);
            if  (HASEXCEPTION())
            {
                goto ruleprivilege_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_grant_object_type_in_privilege_target7599);
            grant_object_type(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprivilege_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_privilege_level_in_privilege_target7601);
            privilege_level(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprivilege_targetEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprivilege_targetEx; /* Prevent compiler warnings */
    ruleprivilege_targetEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end privilege_target */

/**
 * $ANTLR start set_password
 * MySQLSimpleParser.g:1379:1: set_password : SET_SYMBOL PASSWORD_SYMBOL ( FOR_SYMBOL user )? equal ( PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL | text_string ) ;
 */
static void
set_password(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1379:13: ( SET_SYMBOL PASSWORD_SYMBOL ( FOR_SYMBOL user )? equal ( PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL | text_string ) )
        // MySQLSimpleParser.g:1380:2: SET_SYMBOL PASSWORD_SYMBOL ( FOR_SYMBOL user )? equal ( PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL | text_string )
        {
             MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_set_password7610);
            if  (HASEXCEPTION())
            {
                goto ruleset_passwordEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_set_password7612);
            if  (HASEXCEPTION())
            {
                goto ruleset_passwordEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1380:29: ( FOR_SYMBOL user )?
            {
                int alt486=2;
                switch ( LA(1) )
                {
                    case FOR_SYMBOL:
                    	{
                    		alt486=1;
                    	}
                        break;
                }

                switch (alt486)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1380:30: FOR_SYMBOL user
            	    {
            	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_set_password7615);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_in_set_password7617);
            	        user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_equal_in_set_password7621);
            equal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleset_passwordEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1381:2: ( PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL | text_string )
            {
                int alt487=3;
                switch ( LA(1) )
                {
                case PASSWORD_SYMBOL:
                	{
                		alt487=1;
                	}
                    break;
                case OLD_PASSWORD_SYMBOL:
                	{
                		alt487=2;
                	}
                    break;
                case SINGLE_QUOTED_TEXT:
                case BIN_NUMBER:
                case HEX_NUMBER:
                	{
                		alt487=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 487;
                    EXCEPTION->state        = 0;


                    goto ruleset_passwordEx;

                }

                switch (alt487)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1382:3: PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_set_password7628);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_set_password7630);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_text_string_in_set_password7632);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7634);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1383:5: {...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL text_string CLOSE_PAR_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION < 50706)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50706";
            	                EXCEPTION->ruleName	 = (void *)"set_password";


            	        }

            	         MATCHT(OLD_PASSWORD_SYMBOL, &FOLLOW_OLD_PASSWORD_SYMBOL_in_set_password7642);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_set_password7644);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_text_string_in_set_password7646);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_set_password7648);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1384:5: text_string
            	    {
            	        FOLLOWPUSH(FOLLOW_text_string_in_set_password7654);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_passwordEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleset_passwordEx; /* Prevent compiler warnings */
    ruleset_passwordEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end set_password */

/**
 * $ANTLR start grant_object_type
 * MySQLSimpleParser.g:1388:1: grant_object_type : ( ( TABLE_SYMBOL )? | FUNCTION_SYMBOL | PROCEDURE_SYMBOL ) ;
 */
static void
grant_object_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1388:18: ( ( ( TABLE_SYMBOL )? | FUNCTION_SYMBOL | PROCEDURE_SYMBOL ) )
        // MySQLSimpleParser.g:1389:2: ( ( TABLE_SYMBOL )? | FUNCTION_SYMBOL | PROCEDURE_SYMBOL )
        {
            // MySQLSimpleParser.g:1389:2: ( ( TABLE_SYMBOL )? | FUNCTION_SYMBOL | PROCEDURE_SYMBOL )
            {
                int alt489=3;
                {
                    int LA489_0 = LA(1);
                    if ( (((LA489_0 >= ACCOUNT_SYMBOL) && (LA489_0 <= UDF_RETURNS_SYMBOL)) || ((LA489_0 >= UNCOMMITTED_SYMBOL) && (LA489_0 <= YEAR_SYMBOL)) || LA489_0 == TABLE_SYMBOL || LA489_0 == BACK_TICK_QUOTED_ID || LA489_0 == DOUBLE_QUOTED_TEXT || LA489_0 == IDENTIFIER || LA489_0 == MULT_OPERATOR))
                    {
                        alt489=1;
                    }
                    else if ( (LA489_0 == FUNCTION_SYMBOL))
                    {
                        {
                            int LA489_6 = LA(2);
                            if ( (LA489_6 == FROM_SYMBOL || LA489_6 == TO_SYMBOL || LA489_6 == DOT_SYMBOL))
                            {
                                alt489=1;
                            }
                            else if ( (((LA489_6 >= ACCOUNT_SYMBOL) && (LA489_6 <= YEAR_SYMBOL)) || LA489_6 == BACK_TICK_QUOTED_ID || LA489_6 == DOUBLE_QUOTED_TEXT || LA489_6 == IDENTIFIER || LA489_6 == MULT_OPERATOR))
                            {
                                alt489=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 489;
                                EXCEPTION->state        = 6;


                                goto rulegrant_object_typeEx;

                            }
                        }
                    }
                    else if ( (LA489_0 == PROCEDURE_SYMBOL))
                    {
                        alt489=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 489;
                        EXCEPTION->state        = 0;


                        goto rulegrant_object_typeEx;

                    }
                }
                switch (alt489)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1389:3: ( TABLE_SYMBOL )?
            	    {
            	        // MySQLSimpleParser.g:1389:3: ( TABLE_SYMBOL )?
            	        {
            	            int alt488=2;
            	            switch ( LA(1) )
            	            {
            	                case TABLE_SYMBOL:
            	                	{
            	                		alt488=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt488)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1389:3: TABLE_SYMBOL
            	        	    {
            	        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_grant_object_type7667);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulegrant_object_typeEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1389:19: FUNCTION_SYMBOL
            	    {
            	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_grant_object_type7672);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_object_typeEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1389:37: PROCEDURE_SYMBOL
            	    {
            	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_grant_object_type7676);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_object_typeEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulegrant_object_typeEx; /* Prevent compiler warnings */
    rulegrant_object_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_object_type */

/**
 * $ANTLR start grant_privileges
 * MySQLSimpleParser.g:1392:1: grant_privileges : ( ALL_SYMBOL ( PRIVILEGES_SYMBOL )? | privilege_type ( COMMA_SYMBOL privilege_type )* );
 */
static void
grant_privileges(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1392:17: ( ALL_SYMBOL ( PRIVILEGES_SYMBOL )? | privilege_type ( COMMA_SYMBOL privilege_type )* )

            ANTLR3_UINT32 alt492;

            alt492=2;

            switch ( LA(1) )
            {
            case ALL_SYMBOL:
            	{
            		alt492=1;
            	}
                break;
            case EXECUTE_SYMBOL:
            case EVENT_SYMBOL:
            case FILE_SYMBOL:
            case PROCESS_SYMBOL:
            case PROXY_SYMBOL:
            case RELOAD_SYMBOL:
            case REPLICATION_SYMBOL:
            case SHUTDOWN_SYMBOL:
            case SUPER_SYMBOL:
            case ALTER_SYMBOL:
            case CREATE_SYMBOL:
            case DELETE_SYMBOL:
            case DROP_SYMBOL:
            case GRANT_SYMBOL:
            case INDEX_SYMBOL:
            case INSERT_SYMBOL:
            case LOCK_SYMBOL:
            case REFERENCES_SYMBOL:
            case SELECT_SYMBOL:
            case SHOW_SYMBOL:
            case TRIGGER_SYMBOL:
            case UPDATE_SYMBOL:
            case USAGE_SYMBOL:
            	{
            		alt492=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 492;
                EXCEPTION->state        = 0;


                goto rulegrant_privilegesEx;

            }

            switch (alt492)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1393:2: ALL_SYMBOL ( PRIVILEGES_SYMBOL )?
        	    {
        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_grant_privileges7686);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_privilegesEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1393:13: ( PRIVILEGES_SYMBOL )?
        	        {
        	            int alt490=2;
        	            switch ( LA(1) )
        	            {
        	                case PRIVILEGES_SYMBOL:
        	                	{
        	                		alt490=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt490)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1393:13: PRIVILEGES_SYMBOL
        	        	    {
        	        	         MATCHT(PRIVILEGES_SYMBOL, &FOLLOW_PRIVILEGES_SYMBOL_in_grant_privileges7688);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegrant_privilegesEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1394:4: privilege_type ( COMMA_SYMBOL privilege_type )*
        	    {
        	        FOLLOWPUSH(FOLLOW_privilege_type_in_grant_privileges7694);
        	        privilege_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_privilegesEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1394:19: ( COMMA_SYMBOL privilege_type )*

        	        for (;;)
        	        {
        	            int alt491=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt491=1;
        	            	}
        	                break;

        	            }

        	            switch (alt491)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1394:20: COMMA_SYMBOL privilege_type
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_grant_privileges7697);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegrant_privilegesEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_privilege_type_in_grant_privileges7699);
        	        	        privilege_type(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegrant_privilegesEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop491;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop491: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulegrant_privilegesEx; /* Prevent compiler warnings */
    rulegrant_privilegesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_privileges */

/**
 * $ANTLR start privilege_type
 * MySQLSimpleParser.g:1397:1: privilege_type : ( ALTER_SYMBOL ( ROUTINE_SYMBOL )? | CREATE_SYMBOL ( ROUTINE_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL TABLES_SYMBOL | USER_SYMBOL | VIEW_SYMBOL )? | DELETE_SYMBOL | DROP_SYMBOL | EVENT_SYMBOL | EXECUTE_SYMBOL | FILE_SYMBOL | GRANT_SYMBOL OPTION_SYMBOL | INDEX_SYMBOL | INSERT_SYMBOL ( identifier_list_with_parentheses )? | LOCK_SYMBOL TABLES_SYMBOL | PROCESS_SYMBOL | PROXY_SYMBOL | REFERENCES_SYMBOL ( identifier_list_with_parentheses )? | RELOAD_SYMBOL | REPLICATION_SYMBOL ( CLIENT_SYMBOL | SLAVE_SYMBOL ) | SELECT_SYMBOL ( identifier_list_with_parentheses )? | SHOW_SYMBOL DATABASES_SYMBOL | SHOW_SYMBOL VIEW_SYMBOL | SHUTDOWN_SYMBOL | SUPER_SYMBOL | TRIGGER_SYMBOL | UPDATE_SYMBOL ( identifier_list_with_parentheses )? | USAGE_SYMBOL );
 */
static void
privilege_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1397:15: ( ALTER_SYMBOL ( ROUTINE_SYMBOL )? | CREATE_SYMBOL ( ROUTINE_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL TABLES_SYMBOL | USER_SYMBOL | VIEW_SYMBOL )? | DELETE_SYMBOL | DROP_SYMBOL | EVENT_SYMBOL | EXECUTE_SYMBOL | FILE_SYMBOL | GRANT_SYMBOL OPTION_SYMBOL | INDEX_SYMBOL | INSERT_SYMBOL ( identifier_list_with_parentheses )? | LOCK_SYMBOL TABLES_SYMBOL | PROCESS_SYMBOL | PROXY_SYMBOL | REFERENCES_SYMBOL ( identifier_list_with_parentheses )? | RELOAD_SYMBOL | REPLICATION_SYMBOL ( CLIENT_SYMBOL | SLAVE_SYMBOL ) | SELECT_SYMBOL ( identifier_list_with_parentheses )? | SHOW_SYMBOL DATABASES_SYMBOL | SHOW_SYMBOL VIEW_SYMBOL | SHUTDOWN_SYMBOL | SUPER_SYMBOL | TRIGGER_SYMBOL | UPDATE_SYMBOL ( identifier_list_with_parentheses )? | USAGE_SYMBOL )

            ANTLR3_UINT32 alt499;

            alt499=24;

            switch ( LA(1) )
            {
            case ALTER_SYMBOL:
            	{
            		alt499=1;
            	}
                break;
            case CREATE_SYMBOL:
            	{
            		alt499=2;
            	}
                break;
            case DELETE_SYMBOL:
            	{
            		alt499=3;
            	}
                break;
            case DROP_SYMBOL:
            	{
            		alt499=4;
            	}
                break;
            case EVENT_SYMBOL:
            	{
            		alt499=5;
            	}
                break;
            case EXECUTE_SYMBOL:
            	{
            		alt499=6;
            	}
                break;
            case FILE_SYMBOL:
            	{
            		alt499=7;
            	}
                break;
            case GRANT_SYMBOL:
            	{
            		alt499=8;
            	}
                break;
            case INDEX_SYMBOL:
            	{
            		alt499=9;
            	}
                break;
            case INSERT_SYMBOL:
            	{
            		alt499=10;
            	}
                break;
            case LOCK_SYMBOL:
            	{
            		alt499=11;
            	}
                break;
            case PROCESS_SYMBOL:
            	{
            		alt499=12;
            	}
                break;
            case PROXY_SYMBOL:
            	{
            		alt499=13;
            	}
                break;
            case REFERENCES_SYMBOL:
            	{
            		alt499=14;
            	}
                break;
            case RELOAD_SYMBOL:
            	{
            		alt499=15;
            	}
                break;
            case REPLICATION_SYMBOL:
            	{
            		alt499=16;
            	}
                break;
            case SELECT_SYMBOL:
            	{
            		alt499=17;
            	}
                break;
            case SHOW_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case DATABASES_SYMBOL:
            			{
            				alt499=18;
            			}
            		    break;
            		case VIEW_SYMBOL:
            			{
            				alt499=19;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 499;
            		    EXCEPTION->state        = 18;


            		    goto ruleprivilege_typeEx;

            		}

            	}
                break;
            case SHUTDOWN_SYMBOL:
            	{
            		alt499=20;
            	}
                break;
            case SUPER_SYMBOL:
            	{
            		alt499=21;
            	}
                break;
            case TRIGGER_SYMBOL:
            	{
            		alt499=22;
            	}
                break;
            case UPDATE_SYMBOL:
            	{
            		alt499=23;
            	}
                break;
            case USAGE_SYMBOL:
            	{
            		alt499=24;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 499;
                EXCEPTION->state        = 0;


                goto ruleprivilege_typeEx;

            }

            switch (alt499)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1398:2: ALTER_SYMBOL ( ROUTINE_SYMBOL )?
        	    {
        	         MATCHT(ALTER_SYMBOL, &FOLLOW_ALTER_SYMBOL_in_privilege_type7710);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1398:15: ( ROUTINE_SYMBOL )?
        	        {
        	            int alt493=2;
        	            switch ( LA(1) )
        	            {
        	                case ROUTINE_SYMBOL:
        	                	{
        	                		alt493=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt493)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1398:15: ROUTINE_SYMBOL
        	        	    {
        	        	         MATCHT(ROUTINE_SYMBOL, &FOLLOW_ROUTINE_SYMBOL_in_privilege_type7712);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1399:4: CREATE_SYMBOL ( ROUTINE_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL TABLES_SYMBOL | USER_SYMBOL | VIEW_SYMBOL )?
        	    {
        	         MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_privilege_type7718);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1399:18: ( ROUTINE_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL TABLES_SYMBOL | USER_SYMBOL | VIEW_SYMBOL )?
        	        {
        	            int alt494=6;
        	            switch ( LA(1) )
        	            {
        	                case ROUTINE_SYMBOL:
        	                	{
        	                		alt494=1;
        	                	}
        	                    break;
        	                case TABLESPACE_SYMBOL:
        	                	{
        	                		alt494=2;
        	                	}
        	                    break;
        	                case TEMPORARY_SYMBOL:
        	                	{
        	                		alt494=3;
        	                	}
        	                    break;
        	                case USER_SYMBOL:
        	                	{
        	                		alt494=4;
        	                	}
        	                    break;
        	                case VIEW_SYMBOL:
        	                	{
        	                		alt494=5;
        	                	}
        	                    break;
        	            }

        	            switch (alt494)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1399:19: ROUTINE_SYMBOL
        	        	    {
        	        	         MATCHT(ROUTINE_SYMBOL, &FOLLOW_ROUTINE_SYMBOL_in_privilege_type7721);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1399:36: TABLESPACE_SYMBOL
        	        	    {
        	        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_privilege_type7725);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:1399:56: TEMPORARY_SYMBOL TABLES_SYMBOL
        	        	    {
        	        	         MATCHT(TEMPORARY_SYMBOL, &FOLLOW_TEMPORARY_SYMBOL_in_privilege_type7729);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(TABLES_SYMBOL, &FOLLOW_TABLES_SYMBOL_in_privilege_type7731);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 4:
        	        	    // MySQLSimpleParser.g:1399:89: USER_SYMBOL
        	        	    {
        	        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_privilege_type7735);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 5:
        	        	    // MySQLSimpleParser.g:1399:103: VIEW_SYMBOL
        	        	    {
        	        	         MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_privilege_type7739);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1400:4: DELETE_SYMBOL
        	    {
        	         MATCHT(DELETE_SYMBOL, &FOLLOW_DELETE_SYMBOL_in_privilege_type7746);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1401:4: DROP_SYMBOL
        	    {
        	         MATCHT(DROP_SYMBOL, &FOLLOW_DROP_SYMBOL_in_privilege_type7751);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1402:4: EVENT_SYMBOL
        	    {
        	         MATCHT(EVENT_SYMBOL, &FOLLOW_EVENT_SYMBOL_in_privilege_type7756);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1403:4: EXECUTE_SYMBOL
        	    {
        	         MATCHT(EXECUTE_SYMBOL, &FOLLOW_EXECUTE_SYMBOL_in_privilege_type7761);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1404:4: FILE_SYMBOL
        	    {
        	         MATCHT(FILE_SYMBOL, &FOLLOW_FILE_SYMBOL_in_privilege_type7766);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1405:4: GRANT_SYMBOL OPTION_SYMBOL
        	    {
        	         MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_privilege_type7771);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_privilege_type7773);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:1406:4: INDEX_SYMBOL
        	    {
        	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_privilege_type7778);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:1407:4: INSERT_SYMBOL ( identifier_list_with_parentheses )?
        	    {
        	         MATCHT(INSERT_SYMBOL, &FOLLOW_INSERT_SYMBOL_in_privilege_type7783);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1407:18: ( identifier_list_with_parentheses )?
        	        {
        	            int alt495=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt495=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt495)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1407:18: identifier_list_with_parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_privilege_type7785);
        	        	        identifier_list_with_parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:1408:4: LOCK_SYMBOL TABLES_SYMBOL
        	    {
        	         MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_privilege_type7791);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLES_SYMBOL, &FOLLOW_TABLES_SYMBOL_in_privilege_type7793);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:1409:4: PROCESS_SYMBOL
        	    {
        	         MATCHT(PROCESS_SYMBOL, &FOLLOW_PROCESS_SYMBOL_in_privilege_type7798);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:1410:4: PROXY_SYMBOL
        	    {
        	         MATCHT(PROXY_SYMBOL, &FOLLOW_PROXY_SYMBOL_in_privilege_type7803);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:1411:4: REFERENCES_SYMBOL ( identifier_list_with_parentheses )?
        	    {
        	         MATCHT(REFERENCES_SYMBOL, &FOLLOW_REFERENCES_SYMBOL_in_privilege_type7808);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1411:22: ( identifier_list_with_parentheses )?
        	        {
        	            int alt496=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt496=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt496)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1411:22: identifier_list_with_parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_privilege_type7810);
        	        	        identifier_list_with_parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:1412:4: RELOAD_SYMBOL
        	    {
        	         MATCHT(RELOAD_SYMBOL, &FOLLOW_RELOAD_SYMBOL_in_privilege_type7816);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:1413:4: REPLICATION_SYMBOL ( CLIENT_SYMBOL | SLAVE_SYMBOL )
        	    {
        	         MATCHT(REPLICATION_SYMBOL, &FOLLOW_REPLICATION_SYMBOL_in_privilege_type7821);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == SLAVE_SYMBOL || LA(1) == CLIENT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleprivilege_typeEx;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:1414:4: SELECT_SYMBOL ( identifier_list_with_parentheses )?
        	    {
        	         MATCHT(SELECT_SYMBOL, &FOLLOW_SELECT_SYMBOL_in_privilege_type7834);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1414:18: ( identifier_list_with_parentheses )?
        	        {
        	            int alt497=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt497=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt497)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1414:18: identifier_list_with_parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_privilege_type7836);
        	        	        identifier_list_with_parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:1415:4: SHOW_SYMBOL DATABASES_SYMBOL
        	    {
        	         MATCHT(SHOW_SYMBOL, &FOLLOW_SHOW_SYMBOL_in_privilege_type7842);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DATABASES_SYMBOL, &FOLLOW_DATABASES_SYMBOL_in_privilege_type7844);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:1416:4: SHOW_SYMBOL VIEW_SYMBOL
        	    {
        	         MATCHT(SHOW_SYMBOL, &FOLLOW_SHOW_SYMBOL_in_privilege_type7849);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_privilege_type7851);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:1417:4: SHUTDOWN_SYMBOL
        	    {
        	         MATCHT(SHUTDOWN_SYMBOL, &FOLLOW_SHUTDOWN_SYMBOL_in_privilege_type7856);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:1418:4: SUPER_SYMBOL
        	    {
        	         MATCHT(SUPER_SYMBOL, &FOLLOW_SUPER_SYMBOL_in_privilege_type7861);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 22:
        	    // MySQLSimpleParser.g:1419:4: TRIGGER_SYMBOL
        	    {
        	         MATCHT(TRIGGER_SYMBOL, &FOLLOW_TRIGGER_SYMBOL_in_privilege_type7866);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 23:
        	    // MySQLSimpleParser.g:1420:4: UPDATE_SYMBOL ( identifier_list_with_parentheses )?
        	    {
        	         MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_privilege_type7871);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1420:18: ( identifier_list_with_parentheses )?
        	        {
        	            int alt498=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt498=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt498)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1420:18: identifier_list_with_parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_privilege_type7873);
        	        	        identifier_list_with_parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 24:
        	    // MySQLSimpleParser.g:1421:4: USAGE_SYMBOL
        	    {
        	         MATCHT(USAGE_SYMBOL, &FOLLOW_USAGE_SYMBOL_in_privilege_type7879);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleprivilege_typeEx; /* Prevent compiler warnings */
    ruleprivilege_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end privilege_type */

/**
 * $ANTLR start privilege_level
 * MySQLSimpleParser.g:1424:1: privilege_level : ( MULT_OPERATOR ( DOT_SYMBOL MULT_OPERATOR )? | identifier ( DOT_SYMBOL MULT_OPERATOR | dot_identifier )? );
 */
static void
privilege_level(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1424:16: ( MULT_OPERATOR ( DOT_SYMBOL MULT_OPERATOR )? | identifier ( DOT_SYMBOL MULT_OPERATOR | dot_identifier )? )

            ANTLR3_UINT32 alt502;

            alt502=2;

            {
                int LA502_0 = LA(1);
                if ( (LA502_0 == MULT_OPERATOR))
                {
                    alt502=1;
                }
                else if ( (((LA502_0 >= ACCOUNT_SYMBOL) && (LA502_0 <= YEAR_SYMBOL)) || LA502_0 == BACK_TICK_QUOTED_ID || LA502_0 == DOUBLE_QUOTED_TEXT || LA502_0 == IDENTIFIER))
                {
                    alt502=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 502;
                    EXCEPTION->state        = 0;


                    goto ruleprivilege_levelEx;

                }
            }
            switch (alt502)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1425:2: MULT_OPERATOR ( DOT_SYMBOL MULT_OPERATOR )?
        	    {
        	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_privilege_level7888);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1425:16: ( DOT_SYMBOL MULT_OPERATOR )?
        	        {
        	            int alt500=2;
        	            switch ( LA(1) )
        	            {
        	                case DOT_SYMBOL:
        	                	{
        	                		alt500=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt500)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1425:17: DOT_SYMBOL MULT_OPERATOR
        	        	    {
        	        	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_privilege_level7891);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_levelEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_privilege_level7893);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_levelEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1426:4: identifier ( DOT_SYMBOL MULT_OPERATOR | dot_identifier )?
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_privilege_level7900);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprivilege_levelEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1426:15: ( DOT_SYMBOL MULT_OPERATOR | dot_identifier )?
        	        {
        	            int alt501=3;
        	            switch ( LA(1) )
        	            {
        	                case DOT_SYMBOL:
        	                	{
        	                		{
        	                		    int LA501_1 = LA(2);
        	                		    if ( (LA501_1 == MULT_OPERATOR))
        	                		    {
        	                		        alt501=1;
        	                		    }
        	                		    else if ( (((LA501_1 >= ACCOUNT_SYMBOL) && (LA501_1 <= YEAR_SYMBOL)) || LA501_1 == BACK_TICK_QUOTED_ID || LA501_1 == DOUBLE_QUOTED_TEXT || LA501_1 == IDENTIFIER))
        	                		    {
        	                		        alt501=2;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	            }

        	            switch (alt501)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1426:16: DOT_SYMBOL MULT_OPERATOR
        	        	    {
        	        	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_privilege_level7903);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_levelEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_privilege_level7905);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_levelEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1426:43: dot_identifier
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_privilege_level7909);
        	        	        dot_identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleprivilege_levelEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleprivilege_levelEx; /* Prevent compiler warnings */
    ruleprivilege_levelEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end privilege_level */

/**
 * $ANTLR start require_list
 * MySQLSimpleParser.g:1429:1: require_list : require_list_element ( ( AND_SYMBOL )? require_list_element )* ;
 */
static void
require_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1429:13: ( require_list_element ( ( AND_SYMBOL )? require_list_element )* )
        // MySQLSimpleParser.g:1430:2: require_list_element ( ( AND_SYMBOL )? require_list_element )*
        {
            FOLLOWPUSH(FOLLOW_require_list_element_in_require_list7920);
            require_list_element(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerequire_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1430:23: ( ( AND_SYMBOL )? require_list_element )*

            for (;;)
            {
                int alt504=2;
                switch ( LA(1) )
                {
                case CIPHER_SYMBOL:
                case ISSUER_SYMBOL:
                case SUBJECT_SYMBOL:
                case AND_SYMBOL:
                	{
                		alt504=1;
                	}
                    break;

                }

                switch (alt504)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1430:24: ( AND_SYMBOL )? require_list_element
            	    {
            	        // MySQLSimpleParser.g:1430:24: ( AND_SYMBOL )?
            	        {
            	            int alt503=2;
            	            switch ( LA(1) )
            	            {
            	                case AND_SYMBOL:
            	                	{
            	                		alt503=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt503)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1430:24: AND_SYMBOL
            	        	    {
            	        	         MATCHT(AND_SYMBOL, &FOLLOW_AND_SYMBOL_in_require_list7923);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulerequire_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_require_list_element_in_require_list7926);
            	        require_list_element(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulerequire_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop504;	/* break out of the loop */
            	    break;
                }
            }
            loop504: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulerequire_listEx; /* Prevent compiler warnings */
    rulerequire_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end require_list */

/**
 * $ANTLR start require_list_element
 * MySQLSimpleParser.g:1433:1: require_list_element : ( CIPHER_SYMBOL text_string | ISSUER_SYMBOL text_string | SUBJECT_SYMBOL text_string );
 */
static void
require_list_element(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1433:21: ( CIPHER_SYMBOL text_string | ISSUER_SYMBOL text_string | SUBJECT_SYMBOL text_string )

            ANTLR3_UINT32 alt505;

            alt505=3;

            switch ( LA(1) )
            {
            case CIPHER_SYMBOL:
            	{
            		alt505=1;
            	}
                break;
            case ISSUER_SYMBOL:
            	{
            		alt505=2;
            	}
                break;
            case SUBJECT_SYMBOL:
            	{
            		alt505=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 505;
                EXCEPTION->state        = 0;


                goto rulerequire_list_elementEx;

            }

            switch (alt505)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1434:2: CIPHER_SYMBOL text_string
        	    {
        	         MATCHT(CIPHER_SYMBOL, &FOLLOW_CIPHER_SYMBOL_in_require_list_element7937);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_require_list_element7939);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1435:4: ISSUER_SYMBOL text_string
        	    {
        	         MATCHT(ISSUER_SYMBOL, &FOLLOW_ISSUER_SYMBOL_in_require_list_element7944);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_require_list_element7946);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1436:4: SUBJECT_SYMBOL text_string
        	    {
        	         MATCHT(SUBJECT_SYMBOL, &FOLLOW_SUBJECT_SYMBOL_in_require_list_element7951);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_require_list_element7953);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulerequire_list_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulerequire_list_elementEx; /* Prevent compiler warnings */
    rulerequire_list_elementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end require_list_element */

/**
 * $ANTLR start grant_option
 * MySQLSimpleParser.g:1439:1: grant_option : ( GRANT_SYMBOL OPTION_SYMBOL | MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number );
 */
static void
grant_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1439:13: ( GRANT_SYMBOL OPTION_SYMBOL | MAX_QUERIES_PER_HOUR_SYMBOL ulong_number | MAX_UPDATES_PER_HOUR_SYMBOL ulong_number | MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number | MAX_USER_CONNECTIONS_SYMBOL ulong_number )

            ANTLR3_UINT32 alt506;

            alt506=5;

            switch ( LA(1) )
            {
            case GRANT_SYMBOL:
            	{
            		alt506=1;
            	}
                break;
            case MAX_QUERIES_PER_HOUR_SYMBOL:
            	{
            		alt506=2;
            	}
                break;
            case MAX_UPDATES_PER_HOUR_SYMBOL:
            	{
            		alt506=3;
            	}
                break;
            case MAX_CONNECTIONS_PER_HOUR_SYMBOL:
            	{
            		alt506=4;
            	}
                break;
            case MAX_USER_CONNECTIONS_SYMBOL:
            	{
            		alt506=5;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 506;
                EXCEPTION->state        = 0;


                goto rulegrant_optionEx;

            }

            switch (alt506)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1440:2: GRANT_SYMBOL OPTION_SYMBOL
        	    {
        	         MATCHT(GRANT_SYMBOL, &FOLLOW_GRANT_SYMBOL_in_grant_option7962);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPTION_SYMBOL, &FOLLOW_OPTION_SYMBOL_in_grant_option7964);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1441:4: MAX_QUERIES_PER_HOUR_SYMBOL ulong_number
        	    {
        	         MATCHT(MAX_QUERIES_PER_HOUR_SYMBOL, &FOLLOW_MAX_QUERIES_PER_HOUR_SYMBOL_in_grant_option7969);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_grant_option7971);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1442:4: MAX_UPDATES_PER_HOUR_SYMBOL ulong_number
        	    {
        	         MATCHT(MAX_UPDATES_PER_HOUR_SYMBOL, &FOLLOW_MAX_UPDATES_PER_HOUR_SYMBOL_in_grant_option7976);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_grant_option7978);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1443:4: MAX_CONNECTIONS_PER_HOUR_SYMBOL ulong_number
        	    {
        	         MATCHT(MAX_CONNECTIONS_PER_HOUR_SYMBOL, &FOLLOW_MAX_CONNECTIONS_PER_HOUR_SYMBOL_in_grant_option7983);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_grant_option7985);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1444:4: MAX_USER_CONNECTIONS_SYMBOL ulong_number
        	    {
        	         MATCHT(MAX_USER_CONNECTIONS_SYMBOL, &FOLLOW_MAX_USER_CONNECTIONS_SYMBOL_in_grant_option7990);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_grant_option7992);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegrant_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulegrant_optionEx; /* Prevent compiler warnings */
    rulegrant_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_option */

/**
 * $ANTLR start table_administration_statement
 * MySQLSimpleParser.g:1449:1: table_administration_statement : ( ANALYZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list | CHECK_SYMBOL TABLE_SYMBOL table_ref_list ( check_option )* | CHECKSUM_SYMBOL TABLE_SYMBOL table_ref_list ( QUICK_SYMBOL | EXTENDED_SYMBOL )? | OPTIMIZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list | REPAIR_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list ( repair_option )* |{...}? BACKUP_SYMBOL TABLE_SYMBOL table_ref_list TO_SYMBOL string_literal |{...}? RESTORE_SYMBOL TABLE_SYMBOL table_ref_list FROM_SYMBOL string_literal );
 */
static void
table_administration_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1449:31: ( ANALYZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list | CHECK_SYMBOL TABLE_SYMBOL table_ref_list ( check_option )* | CHECKSUM_SYMBOL TABLE_SYMBOL table_ref_list ( QUICK_SYMBOL | EXTENDED_SYMBOL )? | OPTIMIZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list | REPAIR_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list ( repair_option )* |{...}? BACKUP_SYMBOL TABLE_SYMBOL table_ref_list TO_SYMBOL string_literal |{...}? RESTORE_SYMBOL TABLE_SYMBOL table_ref_list FROM_SYMBOL string_literal )

            ANTLR3_UINT32 alt513;

            alt513=7;

            switch ( LA(1) )
            {
            case ANALYZE_SYMBOL:
            	{
            		alt513=1;
            	}
                break;
            case CHECK_SYMBOL:
            	{
            		alt513=2;
            	}
                break;
            case CHECKSUM_SYMBOL:
            	{
            		alt513=3;
            	}
                break;
            case OPTIMIZE_SYMBOL:
            	{
            		alt513=4;
            	}
                break;
            case REPAIR_SYMBOL:
            	{
            		alt513=5;
            	}
                break;
            case BACKUP_SYMBOL:
            	{
            		alt513=6;
            	}
                break;
            case RESTORE_SYMBOL:
            	{
            		alt513=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 513;
                EXCEPTION->state        = 0;


                goto ruletable_administration_statementEx;

            }

            switch (alt513)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1450:2: ANALYZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list
        	    {
        	         MATCHT(ANALYZE_SYMBOL, &FOLLOW_ANALYZE_SYMBOL_in_table_administration_statement8003);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1450:17: ( no_write_to_bin_log )?
        	        {
        	            int alt507=2;
        	            {
        	                int LA507_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA507_0 == LOCAL_SYMBOL))
        	                {
        	                    alt507=1;
        	                }
        	                else if ( (LA507_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt507=1;
        	                }
        	            }
        	            switch (alt507)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1450:17: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_table_administration_statement8005);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_administration_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8008);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8010);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1451:4: CHECK_SYMBOL TABLE_SYMBOL table_ref_list ( check_option )*
        	    {
        	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_table_administration_statement8015);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8017);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8019);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1451:45: ( check_option )*

        	        for (;;)
        	        {
        	            int alt508=2;
        	            switch ( LA(1) )
        	            {
        	            case CHANGED_SYMBOL:
        	            case EXTENDED_SYMBOL:
        	            case FAST_SYMBOL:
        	            case MEDIUM_SYMBOL:
        	            case QUICK_SYMBOL:
        	            case FOR_SYMBOL:
        	            	{
        	            		alt508=1;
        	            	}
        	                break;

        	            }

        	            switch (alt508)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1451:45: check_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_check_option_in_table_administration_statement8021);
        	        	        check_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_administration_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop508;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop508: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1452:4: CHECKSUM_SYMBOL TABLE_SYMBOL table_ref_list ( QUICK_SYMBOL | EXTENDED_SYMBOL )?
        	    {
        	         MATCHT(CHECKSUM_SYMBOL, &FOLLOW_CHECKSUM_SYMBOL_in_table_administration_statement8027);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8029);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8031);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1452:48: ( QUICK_SYMBOL | EXTENDED_SYMBOL )?
        	        {
        	            int alt509=2;
        	            switch ( LA(1) )
        	            {
        	                case EXTENDED_SYMBOL:
        	                case QUICK_SYMBOL:
        	                	{
        	                		alt509=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt509)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:
        	        	    {
        	        	        if ( LA(1) == EXTENDED_SYMBOL || LA(1) == QUICK_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto ruletable_administration_statementEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1453:4: OPTIMIZE_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list
        	    {
        	         MATCHT(OPTIMIZE_SYMBOL, &FOLLOW_OPTIMIZE_SYMBOL_in_table_administration_statement8045);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1453:20: ( no_write_to_bin_log )?
        	        {
        	            int alt510=2;
        	            {
        	                int LA510_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA510_0 == LOCAL_SYMBOL))
        	                {
        	                    alt510=1;
        	                }
        	                else if ( (LA510_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt510=1;
        	                }
        	            }
        	            switch (alt510)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1453:20: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_table_administration_statement8047);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_administration_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8050);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8052);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1454:4: REPAIR_SYMBOL ( no_write_to_bin_log )? TABLE_SYMBOL table_ref_list ( repair_option )*
        	    {
        	         MATCHT(REPAIR_SYMBOL, &FOLLOW_REPAIR_SYMBOL_in_table_administration_statement8057);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1454:18: ( no_write_to_bin_log )?
        	        {
        	            int alt511=2;
        	            {
        	                int LA511_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA511_0 == LOCAL_SYMBOL))
        	                {
        	                    alt511=1;
        	                }
        	                else if ( (LA511_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt511=1;
        	                }
        	            }
        	            switch (alt511)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1454:18: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_table_administration_statement8059);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_administration_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8062);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8064);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1454:67: ( repair_option )*

        	        for (;;)
        	        {
        	            int alt512=2;
        	            switch ( LA(1) )
        	            {
        	            case EXTENDED_SYMBOL:
        	            case QUICK_SYMBOL:
        	            case USE_FRM_SYMBOL:
        	            	{
        	            		alt512=1;
        	            	}
        	                break;

        	            }

        	            switch (alt512)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1454:67: repair_option
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_repair_option_in_table_administration_statement8066);
        	        	        repair_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruletable_administration_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop512;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop512: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1455:4: {...}? BACKUP_SYMBOL TABLE_SYMBOL table_ref_list TO_SYMBOL string_literal
        	    {
        	        if ( !((SERVER_VERSION < 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50500";
        	                EXCEPTION->ruleName	 = (void *)"table_administration_statement";


        	        }

        	         MATCHT(BACKUP_SYMBOL, &FOLLOW_BACKUP_SYMBOL_in_table_administration_statement8074);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8076);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8078);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TO_SYMBOL, &FOLLOW_TO_SYMBOL_in_table_administration_statement8080);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_table_administration_statement8082);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1456:4: {...}? RESTORE_SYMBOL TABLE_SYMBOL table_ref_list FROM_SYMBOL string_literal
        	    {
        	        if ( !((SERVER_VERSION < 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50500";
        	                EXCEPTION->ruleName	 = (void *)"table_administration_statement";


        	        }

        	         MATCHT(RESTORE_SYMBOL, &FOLLOW_RESTORE_SYMBOL_in_table_administration_statement8089);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_table_administration_statement8091);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_table_administration_statement8093);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_table_administration_statement8095);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_table_administration_statement8097);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_administration_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletable_administration_statementEx; /* Prevent compiler warnings */
    ruletable_administration_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_administration_statement */

/**
 * $ANTLR start check_option
 * MySQLSimpleParser.g:1459:1: check_option : ( FOR_SYMBOL UPGRADE_SYMBOL | QUICK_SYMBOL | FAST_SYMBOL | MEDIUM_SYMBOL | EXTENDED_SYMBOL | CHANGED_SYMBOL );
 */
static void
check_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1459:13: ( FOR_SYMBOL UPGRADE_SYMBOL | QUICK_SYMBOL | FAST_SYMBOL | MEDIUM_SYMBOL | EXTENDED_SYMBOL | CHANGED_SYMBOL )

            ANTLR3_UINT32 alt514;

            alt514=6;

            switch ( LA(1) )
            {
            case FOR_SYMBOL:
            	{
            		alt514=1;
            	}
                break;
            case QUICK_SYMBOL:
            	{
            		alt514=2;
            	}
                break;
            case FAST_SYMBOL:
            	{
            		alt514=3;
            	}
                break;
            case MEDIUM_SYMBOL:
            	{
            		alt514=4;
            	}
                break;
            case EXTENDED_SYMBOL:
            	{
            		alt514=5;
            	}
                break;
            case CHANGED_SYMBOL:
            	{
            		alt514=6;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 514;
                EXCEPTION->state        = 0;


                goto rulecheck_optionEx;

            }

            switch (alt514)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1460:2: FOR_SYMBOL UPGRADE_SYMBOL
        	    {
        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_check_option8106);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(UPGRADE_SYMBOL, &FOLLOW_UPGRADE_SYMBOL_in_check_option8108);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1460:30: QUICK_SYMBOL
        	    {
        	         MATCHT(QUICK_SYMBOL, &FOLLOW_QUICK_SYMBOL_in_check_option8112);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1460:45: FAST_SYMBOL
        	    {
        	         MATCHT(FAST_SYMBOL, &FOLLOW_FAST_SYMBOL_in_check_option8116);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1460:59: MEDIUM_SYMBOL
        	    {
        	         MATCHT(MEDIUM_SYMBOL, &FOLLOW_MEDIUM_SYMBOL_in_check_option8120);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1460:75: EXTENDED_SYMBOL
        	    {
        	         MATCHT(EXTENDED_SYMBOL, &FOLLOW_EXTENDED_SYMBOL_in_check_option8124);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1460:93: CHANGED_SYMBOL
        	    {
        	         MATCHT(CHANGED_SYMBOL, &FOLLOW_CHANGED_SYMBOL_in_check_option8128);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecheck_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecheck_optionEx; /* Prevent compiler warnings */
    rulecheck_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end check_option */

/**
 * $ANTLR start repair_option
 * MySQLSimpleParser.g:1463:1: repair_option : ( QUICK_SYMBOL | EXTENDED_SYMBOL | USE_FRM_SYMBOL );
 */
static void
repair_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1463:14: ( QUICK_SYMBOL | EXTENDED_SYMBOL | USE_FRM_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == EXTENDED_SYMBOL || LA(1) == QUICK_SYMBOL || LA(1) == USE_FRM_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulerepair_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulerepair_optionEx; /* Prevent compiler warnings */
    rulerepair_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end repair_option */

/**
 * $ANTLR start install_uninstall_statment
 * MySQLSimpleParser.g:1469:1: install_uninstall_statment : ( INSTALL_SYMBOL PLUGIN_SYMBOL identifier SONAME_SYMBOL string_literal | UNINSTALL_SYMBOL PLUGIN_SYMBOL identifier );
 */
static void
install_uninstall_statment(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1469:27: ( INSTALL_SYMBOL PLUGIN_SYMBOL identifier SONAME_SYMBOL string_literal | UNINSTALL_SYMBOL PLUGIN_SYMBOL identifier )

            ANTLR3_UINT32 alt515;

            alt515=2;

            switch ( LA(1) )
            {
            case INSTALL_SYMBOL:
            	{
            		alt515=1;
            	}
                break;
            case UNINSTALL_SYMBOL:
            	{
            		alt515=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 515;
                EXCEPTION->state        = 0;


                goto ruleinstall_uninstall_statmentEx;

            }

            switch (alt515)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1470:2: INSTALL_SYMBOL PLUGIN_SYMBOL identifier SONAME_SYMBOL string_literal
        	    {
        	         MATCHT(INSTALL_SYMBOL, &FOLLOW_INSTALL_SYMBOL_in_install_uninstall_statment8156);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PLUGIN_SYMBOL, &FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8158);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_install_uninstall_statment8160);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SONAME_SYMBOL, &FOLLOW_SONAME_SYMBOL_in_install_uninstall_statment8162);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_install_uninstall_statment8164);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1471:4: UNINSTALL_SYMBOL PLUGIN_SYMBOL identifier
        	    {
        	         MATCHT(UNINSTALL_SYMBOL, &FOLLOW_UNINSTALL_SYMBOL_in_install_uninstall_statment8169);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PLUGIN_SYMBOL, &FOLLOW_PLUGIN_SYMBOL_in_install_uninstall_statment8171);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_install_uninstall_statment8173);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinstall_uninstall_statmentEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleinstall_uninstall_statmentEx; /* Prevent compiler warnings */
    ruleinstall_uninstall_statmentEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end install_uninstall_statment */

/**
 * $ANTLR start set_statement
 * MySQLSimpleParser.g:1476:1: set_statement : SET_SYMBOL ( options {k=3; } : ( option_type )? TRANSACTION_SYMBOL set_transaction_characteristic | ( ONE_SHOT_SYMBOL )? option_value_no_option_type ( COMMA_SYMBOL option_value_list )? | option_type option_value_following_option_type ( COMMA_SYMBOL option_value_list )? ) ;
 */
static void
set_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1476:14: ( SET_SYMBOL ( options {k=3; } : ( option_type )? TRANSACTION_SYMBOL set_transaction_characteristic | ( ONE_SHOT_SYMBOL )? option_value_no_option_type ( COMMA_SYMBOL option_value_list )? | option_type option_value_following_option_type ( COMMA_SYMBOL option_value_list )? ) )
        // MySQLSimpleParser.g:1477:2: SET_SYMBOL ( options {k=3; } : ( option_type )? TRANSACTION_SYMBOL set_transaction_characteristic | ( ONE_SHOT_SYMBOL )? option_value_no_option_type ( COMMA_SYMBOL option_value_list )? | option_type option_value_following_option_type ( COMMA_SYMBOL option_value_list )? )
        {
             MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_set_statement8184);
            if  (HASEXCEPTION())
            {
                goto ruleset_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1478:3: ( options {k=3; } : ( option_type )? TRANSACTION_SYMBOL set_transaction_characteristic | ( ONE_SHOT_SYMBOL )? option_value_no_option_type ( COMMA_SYMBOL option_value_list )? | option_type option_value_following_option_type ( COMMA_SYMBOL option_value_list )? )
            {
                int alt520=3;
                {
                    int LA520_0 = LA(1);
                    if ( (LA520_0 == GLOBAL_SYMBOL || LA520_0 == LOCAL_SYMBOL || LA520_0 == SESSION_SYMBOL))
                    {
                        {
                            int LA520_1 = LA(2);
                            if ( (LA520_1 == TRANSACTION_SYMBOL))
                            {
                                switch ( LA(3) )
                                {
                                case ISOLATION_SYMBOL:
                                case READ_SYMBOL:
                                	{
                                		alt520=1;
                                	}
                                    break;
                                case ASSIGN_OPERATOR:
                                case DOT_SYMBOL:
                                case EQUAL_OPERATOR:
                                	{
                                		alt520=3;
                                	}
                                    break;

                                default:
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }

                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 520;
                                    EXCEPTION->state        = 16;


                                    goto ruleset_statementEx;

                                }

                            }
                            else if ( (LA520_1 == ASSIGN_OPERATOR || LA520_1 == DOT_SYMBOL || LA520_1 == EQUAL_OPERATOR))
                            {
                                alt520=2;
                            }
                            else if ( (((LA520_1 >= ACCOUNT_SYMBOL) && (LA520_1 <= THAN_SYMBOL)) || ((LA520_1 >= TRIGGERS_SYMBOL) && (LA520_1 <= YEAR_SYMBOL)) || LA520_1 == DEFAULT_SYMBOL || LA520_1 == BACK_TICK_QUOTED_ID || LA520_1 == DOUBLE_QUOTED_TEXT || LA520_1 == IDENTIFIER))
                            {
                                alt520=3;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 520;
                                EXCEPTION->state        = 1;


                                goto ruleset_statementEx;

                            }
                        }
                    }
                    else if ( (LA520_0 == TRANSACTION_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case ISOLATION_SYMBOL:
                        case READ_SYMBOL:
                        	{
                        		alt520=1;
                        	}
                            break;
                        case ASSIGN_OPERATOR:
                        case DOT_SYMBOL:
                        case EQUAL_OPERATOR:
                        	{
                        		alt520=2;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 520;
                            EXCEPTION->state        = 2;


                            goto ruleset_statementEx;

                        }

                    }
                    else if ( (((LA520_0 >= ACCOUNT_SYMBOL) && (LA520_0 <= GRANTS_SYMBOL)) || ((LA520_0 >= HASH_SYMBOL) && (LA520_0 <= LIST_SYMBOL)) || ((LA520_0 >= LOCKS_SYMBOL) && (LA520_0 <= SERIALIZABLE_SYMBOL)) || ((LA520_0 >= SIMPLE_SYMBOL) && (LA520_0 <= THAN_SYMBOL)) || ((LA520_0 >= TRIGGERS_SYMBOL) && (LA520_0 <= YEAR_SYMBOL)) || LA520_0 == CHAR_SYMBOL || LA520_0 == DEFAULT_SYMBOL || ((LA520_0 >= AT_AT_SIGN_SYMBOL) && (LA520_0 <= AT_TEXT_SUFFIX)) || LA520_0 == BACK_TICK_QUOTED_ID || LA520_0 == DOUBLE_QUOTED_TEXT || LA520_0 == IDENTIFIER))
                    {
                        alt520=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 520;
                        EXCEPTION->state        = 0;


                        goto ruleset_statementEx;

                    }
                }
                switch (alt520)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1479:5: ( option_type )? TRANSACTION_SYMBOL set_transaction_characteristic
            	    {
            	        // MySQLSimpleParser.g:1479:5: ( option_type )?
            	        {
            	            int alt516=2;
            	            switch ( LA(1) )
            	            {
            	                case GLOBAL_SYMBOL:
            	                case LOCAL_SYMBOL:
            	                case SESSION_SYMBOL:
            	                	{
            	                		alt516=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt516)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1479:5: option_type
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_option_type_in_set_statement8206);
            	        	        option_type(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(TRANSACTION_SYMBOL, &FOLLOW_TRANSACTION_SYMBOL_in_set_statement8209);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_set_transaction_characteristic_in_set_statement8211);
            	        set_transaction_characteristic(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1481:6: ( ONE_SHOT_SYMBOL )? option_value_no_option_type ( COMMA_SYMBOL option_value_list )?
            	    {
            	        // MySQLSimpleParser.g:1481:6: ( ONE_SHOT_SYMBOL )?
            	        {
            	            int alt517=2;
            	            switch ( LA(1) )
            	            {
            	                case ONE_SHOT_SYMBOL:
            	                	{
            	                		{
            	                		    int LA517_1 = LA(2);
            	                		    if ( (((LA517_1 >= ACCOUNT_SYMBOL) && (LA517_1 <= YEAR_SYMBOL)) || LA517_1 == CHAR_SYMBOL || LA517_1 == DEFAULT_SYMBOL || ((LA517_1 >= AT_AT_SIGN_SYMBOL) && (LA517_1 <= AT_TEXT_SUFFIX)) || LA517_1 == BACK_TICK_QUOTED_ID || LA517_1 == DOUBLE_QUOTED_TEXT || LA517_1 == IDENTIFIER))
            	                		    {
            	                		        alt517=1;
            	                		    }
            	                		}
            	                	}
            	                    break;
            	            }

            	            switch (alt517)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1481:6: ONE_SHOT_SYMBOL
            	        	    {
            	        	         MATCHT(ONE_SHOT_SYMBOL, &FOLLOW_ONE_SHOT_SYMBOL_in_set_statement8222);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_option_value_no_option_type_in_set_statement8225);
            	        option_value_no_option_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1481:51: ( COMMA_SYMBOL option_value_list )?
            	        {
            	            int alt518=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt518=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt518)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1481:52: COMMA_SYMBOL option_value_list
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_set_statement8228);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_option_value_list_in_set_statement8230);
            	        	        option_value_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1482:6: option_type option_value_following_option_type ( COMMA_SYMBOL option_value_list )?
            	    {
            	        FOLLOWPUSH(FOLLOW_option_type_in_set_statement8239);
            	        option_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_option_value_following_option_type_in_set_statement8241);
            	        option_value_following_option_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleset_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1482:53: ( COMMA_SYMBOL option_value_list )?
            	        {
            	            int alt519=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt519=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt519)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1482:54: COMMA_SYMBOL option_value_list
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_set_statement8244);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_option_value_list_in_set_statement8246);
            	        	        option_value_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleset_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleset_statementEx; /* Prevent compiler warnings */
    ruleset_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end set_statement */

/**
 * $ANTLR start option_value_no_option_type
 * MySQLSimpleParser.g:1488:1: option_value_no_option_type : ({...}? NAMES_SYMBOL ( equal expression | charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? ) | variable_name equal set_expression_or_default | user_variable equal expression | system_variable equal set_expression_or_default | charset_clause );
 */
static void
option_value_no_option_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1488:28: ({...}? NAMES_SYMBOL ( equal expression | charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? ) | variable_name equal set_expression_or_default | user_variable equal expression | system_variable equal set_expression_or_default | charset_clause )

            ANTLR3_UINT32 alt523;

            alt523=5;

            {
                int LA523_0 = LA(1);
                if ( (LA523_0 == NAMES_SYMBOL))
                {
                    {
                        int LA523_1 = LA(2);
                        if ( (LA523_1 == ASSIGN_OPERATOR || LA523_1 == EQUAL_OPERATOR))
                        {
                            {
                                int LA523_13 = LA(3);
                                if ( ((LA(1) == NAMES_SYMBOL)))
                                {
                                    alt523=1;
                                }
                                else if ( (ANTLR3_TRUE))
                                {
                                    alt523=2;
                                }
                                else
                                {
                                    if (BACKTRACKING>0)
                                    {
                                        FAILEDFLAG = ANTLR3_TRUE;
                                        return ;
                                    }


                                    CONSTRUCTEX();
                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                    EXCEPTION->message      = (void *)"";
                                    EXCEPTION->decisionNum  = 523;
                                    EXCEPTION->state        = 13;


                                    goto ruleoption_value_no_option_typeEx;

                                }
                            }
                        }
                        else if ( (((LA523_1 >= ACCOUNT_SYMBOL) && (LA523_1 <= YEAR_SYMBOL)) || LA523_1 == BINARY_SYMBOL || LA523_1 == DEFAULT_SYMBOL || LA523_1 == SINGLE_QUOTED_TEXT || LA523_1 == BACK_TICK_QUOTED_ID || LA523_1 == DOUBLE_QUOTED_TEXT || LA523_1 == IDENTIFIER))
                        {
                            alt523=1;
                        }
                        else if ( (LA523_1 == DOT_SYMBOL))
                        {
                            alt523=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 523;
                            EXCEPTION->state        = 1;


                            goto ruleoption_value_no_option_typeEx;

                        }
                    }
                }
                else if ( (((LA523_0 >= ACCOUNT_SYMBOL) && (LA523_0 <= CACHE_SYMBOL)) || ((LA523_0 >= CHECKSUM_SYMBOL) && (LA523_0 <= NAME_SYMBOL)) || ((LA523_0 >= NATIONAL_SYMBOL) && (LA523_0 <= YEAR_SYMBOL)) || LA523_0 == DEFAULT_SYMBOL || LA523_0 == BACK_TICK_QUOTED_ID || LA523_0 == DOUBLE_QUOTED_TEXT || LA523_0 == IDENTIFIER))
                {
                    alt523=2;
                }
                else if ( (LA523_0 == CHARSET_SYMBOL))
                {
                    {
                        int LA523_6 = LA(2);
                        if ( (LA523_6 == ASSIGN_OPERATOR || LA523_6 == DOT_SYMBOL || LA523_6 == EQUAL_OPERATOR))
                        {
                            alt523=2;
                        }
                        else if ( (((LA523_6 >= ACCOUNT_SYMBOL) && (LA523_6 <= YEAR_SYMBOL)) || LA523_6 == BINARY_SYMBOL || LA523_6 == DEFAULT_SYMBOL || LA523_6 == SINGLE_QUOTED_TEXT || LA523_6 == BACK_TICK_QUOTED_ID || LA523_6 == DOUBLE_QUOTED_TEXT || LA523_6 == IDENTIFIER))
                        {
                            alt523=5;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 523;
                            EXCEPTION->state        = 6;


                            goto ruleoption_value_no_option_typeEx;

                        }
                    }
                }
                else if ( (((LA523_0 >= AT_SIGN_SYMBOL) && (LA523_0 <= AT_TEXT_SUFFIX))))
                {
                    alt523=3;
                }
                else if ( (LA523_0 == AT_AT_SIGN_SYMBOL))
                {
                    alt523=4;
                }
                else if ( (LA523_0 == CHAR_SYMBOL))
                {
                    alt523=5;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 523;
                    EXCEPTION->state        = 0;


                    goto ruleoption_value_no_option_typeEx;

                }
            }
            switch (alt523)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1489:2: {...}? NAMES_SYMBOL ( equal expression | charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? )
        	    {
        	        if ( !((LA(1) == NAMES_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == NAMES_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"option_value_no_option_type";


        	        }

        	         MATCHT(NAMES_SYMBOL, &FOLLOW_NAMES_SYMBOL_in_option_value_no_option_type8271);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1490:3: ( equal expression | charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )? )
        	        {
        	            int alt522=2;
        	            {
        	                int LA522_0 = LA(1);
        	                if ( (LA522_0 == ASSIGN_OPERATOR || LA522_0 == EQUAL_OPERATOR))
        	                {
        	                    alt522=1;
        	                }
        	                else if ( (((LA522_0 >= ACCOUNT_SYMBOL) && (LA522_0 <= YEAR_SYMBOL)) || LA522_0 == BINARY_SYMBOL || LA522_0 == DEFAULT_SYMBOL || LA522_0 == SINGLE_QUOTED_TEXT || LA522_0 == BACK_TICK_QUOTED_ID || LA522_0 == DOUBLE_QUOTED_TEXT || LA522_0 == IDENTIFIER))
        	                {
        	                    alt522=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 522;
        	                    EXCEPTION->state        = 0;


        	                    goto ruleoption_value_no_option_typeEx;

        	                }
        	            }
        	            switch (alt522)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1491:4: equal expression
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_equal_in_option_value_no_option_type8280);
        	        	        equal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleoption_value_no_option_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_option_value_no_option_type8282);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleoption_value_no_option_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1492:6: charset_name_or_default ( COLLATE_SYMBOL collation_name_or_default )?
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_charset_name_or_default_in_option_value_no_option_type8289);
        	        	        charset_name_or_default(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleoption_value_no_option_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:1492:30: ( COLLATE_SYMBOL collation_name_or_default )?
        	        	        {
        	        	            int alt521=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case COLLATE_SYMBOL:
        	        	                	{
        	        	                		alt521=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt521)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1492:31: COLLATE_SYMBOL collation_name_or_default
        	        	        	    {
        	        	        	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_option_value_no_option_type8292);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleoption_value_no_option_typeEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_collation_name_or_default_in_option_value_no_option_type8294);
        	        	        	        collation_name_or_default(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleoption_value_no_option_typeEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1494:4: variable_name equal set_expression_or_default
        	    {
        	        FOLLOWPUSH(FOLLOW_variable_name_in_option_value_no_option_type8305);
        	        variable_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_equal_in_option_value_no_option_type8307);
        	        equal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_set_expression_or_default_in_option_value_no_option_type8309);
        	        set_expression_or_default(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1495:4: user_variable equal expression
        	    {
        	        FOLLOWPUSH(FOLLOW_user_variable_in_option_value_no_option_type8314);
        	        user_variable(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_equal_in_option_value_no_option_type8316);
        	        equal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_option_value_no_option_type8318);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1496:4: system_variable equal set_expression_or_default
        	    {
        	        FOLLOWPUSH(FOLLOW_system_variable_in_option_value_no_option_type8323);
        	        system_variable(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_equal_in_option_value_no_option_type8325);
        	        equal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_set_expression_or_default_in_option_value_no_option_type8327);
        	        set_expression_or_default(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1497:4: charset_clause
        	    {
        	        FOLLOWPUSH(FOLLOW_charset_clause_in_option_value_no_option_type8332);
        	        charset_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_value_no_option_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleoption_value_no_option_typeEx; /* Prevent compiler warnings */
    ruleoption_value_no_option_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end option_value_no_option_type */

/**
 * $ANTLR start option_value_following_option_type
 * MySQLSimpleParser.g:1500:1: option_value_following_option_type : variable_name equal set_expression_or_default ;
 */
static void
option_value_following_option_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1500:35: ( variable_name equal set_expression_or_default )
        // MySQLSimpleParser.g:1501:2: variable_name equal set_expression_or_default
        {
            FOLLOWPUSH(FOLLOW_variable_name_in_option_value_following_option_type8341);
            variable_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleoption_value_following_option_typeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_equal_in_option_value_following_option_type8343);
            equal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleoption_value_following_option_typeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_set_expression_or_default_in_option_value_following_option_type8345);
            set_expression_or_default(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleoption_value_following_option_typeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleoption_value_following_option_typeEx; /* Prevent compiler warnings */
    ruleoption_value_following_option_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end option_value_following_option_type */

/**
 * $ANTLR start set_expression_or_default
 * MySQLSimpleParser.g:1504:1: set_expression_or_default : ( expression | DEFAULT_SYMBOL | ON_SYMBOL | ALL_SYMBOL | BINARY_SYMBOL );
 */
static void
set_expression_or_default(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1504:26: ( expression | DEFAULT_SYMBOL | ON_SYMBOL | ALL_SYMBOL | BINARY_SYMBOL )

            ANTLR3_UINT32 alt524;

            alt524=5;

            {
                int LA524_0 = LA(1);
                if ( (((LA524_0 >= ACCOUNT_SYMBOL) && (LA524_0 <= YEAR_SYMBOL)) || ((LA524_0 >= CASE_SYMBOL) && (LA524_0 <= CAST_SYMBOL)) || LA524_0 == CHAR_SYMBOL || ((LA524_0 >= CONVERT_SYMBOL) && (LA524_0 <= COUNT_SYMBOL)) || LA524_0 == CURDATE_SYMBOL || LA524_0 == CURRENT_USER_SYMBOL || LA524_0 == CURTIME_SYMBOL || LA524_0 == DATABASE_SYMBOL || LA524_0 == DATE_ADD_SYMBOL || LA524_0 == DATE_SUB_SYMBOL || LA524_0 == EXISTS_SYMBOL || ((LA524_0 >= EXTRACT_SYMBOL) && (LA524_0 <= FALSE_SYMBOL)) || LA524_0 == GROUP_CONCAT_SYMBOL || LA524_0 == IF_SYMBOL || LA524_0 == INSERT_SYMBOL || LA524_0 == INTERVAL_SYMBOL || LA524_0 == LEFT_SYMBOL || LA524_0 == MATCH_SYMBOL || LA524_0 == MAX_SYMBOL || LA524_0 == MIN_SYMBOL || LA524_0 == MOD_SYMBOL || LA524_0 == NCHAR_TEXT || ((LA524_0 >= NOT2_SYMBOL) && (LA524_0 <= NOW_SYMBOL)) || ((LA524_0 >= NULL2_SYMBOL) && (LA524_0 <= NULL_SYMBOL)) || ((LA524_0 >= OPEN_CURLY_SYMBOL) && (LA524_0 <= OPEN_PAR_SYMBOL)) || ((LA524_0 >= PARAM_MARKER) && (LA524_0 <= POSITION_SYMBOL)) || ((LA524_0 >= REPEAT_SYMBOL) && (LA524_0 <= REPLACE_SYMBOL)) || LA524_0 == RIGHT_SYMBOL || LA524_0 == SINGLE_QUOTED_TEXT || LA524_0 == STDDEV_SAMP_SYMBOL || LA524_0 == STD_SYMBOL || LA524_0 == SUBSTRING_SYMBOL || ((LA524_0 >= SUM_SYMBOL) && (LA524_0 <= SYSDATE_SYMBOL)) || ((LA524_0 >= TRIM_SYMBOL) && (LA524_0 <= TRUE_SYMBOL)) || ((LA524_0 >= UTC_DATE_SYMBOL) && (LA524_0 <= UTC_TIME_SYMBOL)) || LA524_0 == VALUES_SYMBOL || LA524_0 == VARIANCE_SYMBOL || LA524_0 == VAR_SAMP_SYMBOL || ((LA524_0 >= AT_AT_SIGN_SYMBOL) && (LA524_0 <= AT_TEXT_SUFFIX)) || ((LA524_0 >= BACK_TICK_QUOTED_ID) && (LA524_0 <= BIN_NUMBER)) || LA524_0 == BITWISE_NOT_OPERATOR || ((LA524_0 >= BIT_AND_SYMBOL) && (LA524_0 <= BIT_XOR_SYMBOL)) || LA524_0 == DECIMAL_NUMBER || LA524_0 == DOT_SYMBOL || LA524_0 == DOUBLE_QUOTED_TEXT || LA524_0 == FLOAT_NUMBER || ((LA524_0 >= HEX_NUMBER) && (LA524_0 <= IDENTIFIER)) || LA524_0 == INT_NUMBER || LA524_0 == LOGICAL_NOT_OPERATOR || ((LA524_0 >= LONG_NUMBER) && (LA524_0 <= MINUS_OPERATOR)) || LA524_0 == PLUS_OPERATOR || ((LA524_0 >= ULONGLONG_NUMBER) && (LA524_0 <= UNDERSCORE_CHARSET))))
                {
                    alt524=1;
                }
                else if ( (LA524_0 == BINARY_SYMBOL))
                {
                    {
                        int LA524_100 = LA(2);
                        if ( (((LA524_100 >= ACCOUNT_SYMBOL) && (LA524_100 <= YEAR_SYMBOL)) || LA524_100 == BINARY_SYMBOL || ((LA524_100 >= CASE_SYMBOL) && (LA524_100 <= CAST_SYMBOL)) || LA524_100 == CHAR_SYMBOL || ((LA524_100 >= CONVERT_SYMBOL) && (LA524_100 <= COUNT_SYMBOL)) || LA524_100 == CURDATE_SYMBOL || LA524_100 == CURRENT_USER_SYMBOL || LA524_100 == CURTIME_SYMBOL || LA524_100 == DATABASE_SYMBOL || LA524_100 == DATE_ADD_SYMBOL || LA524_100 == DATE_SUB_SYMBOL || LA524_100 == EXISTS_SYMBOL || ((LA524_100 >= EXTRACT_SYMBOL) && (LA524_100 <= FALSE_SYMBOL)) || LA524_100 == GROUP_CONCAT_SYMBOL || LA524_100 == IF_SYMBOL || LA524_100 == INSERT_SYMBOL || LA524_100 == LEFT_SYMBOL || LA524_100 == MATCH_SYMBOL || LA524_100 == MAX_SYMBOL || LA524_100 == MIN_SYMBOL || LA524_100 == MOD_SYMBOL || LA524_100 == NCHAR_TEXT || LA524_100 == NOW_SYMBOL || ((LA524_100 >= NULL2_SYMBOL) && (LA524_100 <= NULL_SYMBOL)) || ((LA524_100 >= OPEN_CURLY_SYMBOL) && (LA524_100 <= OPEN_PAR_SYMBOL)) || ((LA524_100 >= PARAM_MARKER) && (LA524_100 <= POSITION_SYMBOL)) || ((LA524_100 >= REPEAT_SYMBOL) && (LA524_100 <= REPLACE_SYMBOL)) || LA524_100 == RIGHT_SYMBOL || LA524_100 == SINGLE_QUOTED_TEXT || LA524_100 == STDDEV_SAMP_SYMBOL || LA524_100 == STD_SYMBOL || LA524_100 == SUBSTRING_SYMBOL || ((LA524_100 >= SUM_SYMBOL) && (LA524_100 <= SYSDATE_SYMBOL)) || ((LA524_100 >= TRIM_SYMBOL) && (LA524_100 <= TRUE_SYMBOL)) || ((LA524_100 >= UTC_DATE_SYMBOL) && (LA524_100 <= UTC_TIME_SYMBOL)) || LA524_100 == VALUES_SYMBOL || LA524_100 == VARIANCE_SYMBOL || LA524_100 == VAR_SAMP_SYMBOL || ((LA524_100 >= AT_AT_SIGN_SYMBOL) && (LA524_100 <= AT_TEXT_SUFFIX)) || ((LA524_100 >= BACK_TICK_QUOTED_ID) && (LA524_100 <= BIN_NUMBER)) || ((LA524_100 >= BIT_AND_SYMBOL) && (LA524_100 <= BIT_XOR_SYMBOL)) || LA524_100 == DECIMAL_NUMBER || LA524_100 == DOT_SYMBOL || LA524_100 == DOUBLE_QUOTED_TEXT || LA524_100 == FLOAT_NUMBER || ((LA524_100 >= HEX_NUMBER) && (LA524_100 <= IDENTIFIER)) || LA524_100 == INT_NUMBER || LA524_100 == LONG_NUMBER || ((LA524_100 >= ULONGLONG_NUMBER) && (LA524_100 <= UNDERSCORE_CHARSET))))
                        {
                            alt524=1;
                        }
                        else if ( (LA524_100 == EOF || LA524_100 == SEMICOLON_SYMBOL || LA524_100 == COMMA_SYMBOL))
                        {
                            alt524=5;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 524;
                            EXCEPTION->state        = 100;


                            goto ruleset_expression_or_defaultEx;

                        }
                    }
                }
                else if ( (LA524_0 == DEFAULT_SYMBOL))
                {
                    alt524=2;
                }
                else if ( (LA524_0 == ON_SYMBOL))
                {
                    alt524=3;
                }
                else if ( (LA524_0 == ALL_SYMBOL))
                {
                    alt524=4;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 524;
                    EXCEPTION->state        = 0;


                    goto ruleset_expression_or_defaultEx;

                }
            }
            switch (alt524)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1505:2: expression
        	    {
        	        FOLLOWPUSH(FOLLOW_expression_in_set_expression_or_default8355);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_expression_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1506:4: DEFAULT_SYMBOL
        	    {
        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_set_expression_or_default8360);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_expression_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1507:4: ON_SYMBOL
        	    {
        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_set_expression_or_default8365);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_expression_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1508:4: ALL_SYMBOL
        	    {
        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_set_expression_or_default8370);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_expression_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1509:4: BINARY_SYMBOL
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_set_expression_or_default8375);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleset_expression_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleset_expression_or_defaultEx; /* Prevent compiler warnings */
    ruleset_expression_or_defaultEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end set_expression_or_default */

/**
 * $ANTLR start option_value_list
 * MySQLSimpleParser.g:1512:1: option_value_list : option_value ( COMMA_SYMBOL option_value )* ;
 */
static void
option_value_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1512:18: ( option_value ( COMMA_SYMBOL option_value )* )
        // MySQLSimpleParser.g:1513:2: option_value ( COMMA_SYMBOL option_value )*
        {
            FOLLOWPUSH(FOLLOW_option_value_in_option_value_list8384);
            option_value(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleoption_value_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1513:15: ( COMMA_SYMBOL option_value )*

            for (;;)
            {
                int alt525=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt525=1;
                	}
                    break;

                }

                switch (alt525)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1513:16: COMMA_SYMBOL option_value
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_option_value_list8387);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleoption_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_option_value_in_option_value_list8389);
            	        option_value(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleoption_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop525;	/* break out of the loop */
            	    break;
                }
            }
            loop525: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleoption_value_listEx; /* Prevent compiler warnings */
    ruleoption_value_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end option_value_list */

/**
 * $ANTLR start option_value
 * MySQLSimpleParser.g:1516:1: option_value : ( option_type variable_name equal set_expression_or_default | option_value_no_option_type );
 */
static void
option_value(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1516:13: ( option_type variable_name equal set_expression_or_default | option_value_no_option_type )

            ANTLR3_UINT32 alt526;

            alt526=2;

            {
                int LA526_0 = LA(1);
                if ( (LA526_0 == GLOBAL_SYMBOL || LA526_0 == LOCAL_SYMBOL || LA526_0 == SESSION_SYMBOL))
                {
                    {
                        int LA526_1 = LA(2);
                        if ( (((LA526_1 >= ACCOUNT_SYMBOL) && (LA526_1 <= YEAR_SYMBOL)) || LA526_1 == DEFAULT_SYMBOL || LA526_1 == BACK_TICK_QUOTED_ID || LA526_1 == DOUBLE_QUOTED_TEXT || LA526_1 == IDENTIFIER))
                        {
                            alt526=1;
                        }
                        else if ( (LA526_1 == ASSIGN_OPERATOR || LA526_1 == DOT_SYMBOL || LA526_1 == EQUAL_OPERATOR))
                        {
                            alt526=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 526;
                            EXCEPTION->state        = 1;


                            goto ruleoption_valueEx;

                        }
                    }
                }
                else if ( (((LA526_0 >= ACCOUNT_SYMBOL) && (LA526_0 <= GRANTS_SYMBOL)) || ((LA526_0 >= HASH_SYMBOL) && (LA526_0 <= LIST_SYMBOL)) || ((LA526_0 >= LOCKS_SYMBOL) && (LA526_0 <= SERIALIZABLE_SYMBOL)) || ((LA526_0 >= SIMPLE_SYMBOL) && (LA526_0 <= YEAR_SYMBOL)) || LA526_0 == CHAR_SYMBOL || LA526_0 == DEFAULT_SYMBOL || ((LA526_0 >= AT_AT_SIGN_SYMBOL) && (LA526_0 <= AT_TEXT_SUFFIX)) || LA526_0 == BACK_TICK_QUOTED_ID || LA526_0 == DOUBLE_QUOTED_TEXT || LA526_0 == IDENTIFIER))
                {
                    alt526=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 526;
                    EXCEPTION->state        = 0;


                    goto ruleoption_valueEx;

                }
            }
            switch (alt526)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1517:2: option_type variable_name equal set_expression_or_default
        	    {
        	        FOLLOWPUSH(FOLLOW_option_type_in_option_value8400);
        	        option_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_valueEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_variable_name_in_option_value8402);
        	        variable_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_valueEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_equal_in_option_value8404);
        	        equal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_valueEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_set_expression_or_default_in_option_value8406);
        	        set_expression_or_default(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_valueEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1518:4: option_value_no_option_type
        	    {
        	        FOLLOWPUSH(FOLLOW_option_value_no_option_type_in_option_value8411);
        	        option_value_no_option_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleoption_valueEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleoption_valueEx; /* Prevent compiler warnings */
    ruleoption_valueEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end option_value */

/**
 * $ANTLR start show_statement
 * MySQLSimpleParser.g:1523:1: show_statement : SHOW_SYMBOL ({...}? AUTHORS_SYMBOL | DATABASES_SYMBOL ( like_or_where )? | ( FULL_SYMBOL )? TABLES_SYMBOL ( in_db )? ( like_or_where )? | ( FULL_SYMBOL )? TRIGGERS_SYMBOL ( in_db )? ( like_or_where )? | EVENTS_SYMBOL ( in_db )? ( like_or_where )? | TABLE_SYMBOL STATUS_SYMBOL ( in_db )? ( like_or_where )? | OPEN_SYMBOL TABLES_SYMBOL ( in_db )? ( like_or_where )? |{...}? PLUGIN_SYMBOL |{...}? PLUGINS_SYMBOL | ENGINE_SYMBOL engine_ref ( STATUS_SYMBOL | MUTEX_SYMBOL ) | ( FULL_SYMBOL )? COLUMNS_SYMBOL ( FROM_SYMBOL | IN_SYMBOL ) table_ref ( in_db )? ( like_or_where )? | ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL | SLAVE_SYMBOL ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? ) | ( BINLOG_SYMBOL | RELAYLOG_SYMBOL ) EVENTS_SYMBOL ( IN_SYMBOL text_string )? ( FROM_SYMBOL ulonglong_number )? ( limit_clause )? ( channel )? | ( INDEX_SYMBOL | INDEXES_SYMBOL | KEYS_SYMBOL ) from_or_in table_ref ( in_db )? ( where_clause )? | ( STORAGE_SYMBOL )? ENGINES_SYMBOL | PRIVILEGES_SYMBOL | COUNT_SYMBOL OPEN_PAR_SYMBOL MULT_OPERATOR CLOSE_PAR_SYMBOL ( WARNINGS_SYMBOL | ERRORS_SYMBOL ) | WARNINGS_SYMBOL ( limit_clause )? | ERRORS_SYMBOL ( limit_clause )? | PROFILES_SYMBOL | PROFILE_SYMBOL ( profile_type ( COMMA_SYMBOL profile_type )* )? ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )? ( limit_clause )? | ( option_type )? ( STATUS_SYMBOL | VARIABLES_SYMBOL ) ( like_or_where )? | ( FULL_SYMBOL )? PROCESSLIST_SYMBOL | charset ( like_or_where )? | COLLATION_SYMBOL ( like_or_where )? |{...}? CONTRIBUTORS_SYMBOL | GRANTS_SYMBOL ( FOR_SYMBOL user )? | MASTER_SYMBOL STATUS_SYMBOL | CREATE_SYMBOL ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user ) | PROCEDURE_SYMBOL STATUS_SYMBOL ( like_or_where )? | FUNCTION_SYMBOL STATUS_SYMBOL ( like_or_where )? | PROCEDURE_SYMBOL CODE_SYMBOL procedure_ref | FUNCTION_SYMBOL CODE_SYMBOL function_ref |{...}? INNODB_SYMBOL STATUS_SYMBOL ) ;
 */
static void
show_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1523:15: ( SHOW_SYMBOL ({...}? AUTHORS_SYMBOL | DATABASES_SYMBOL ( like_or_where )? | ( FULL_SYMBOL )? TABLES_SYMBOL ( in_db )? ( like_or_where )? | ( FULL_SYMBOL )? TRIGGERS_SYMBOL ( in_db )? ( like_or_where )? | EVENTS_SYMBOL ( in_db )? ( like_or_where )? | TABLE_SYMBOL STATUS_SYMBOL ( in_db )? ( like_or_where )? | OPEN_SYMBOL TABLES_SYMBOL ( in_db )? ( like_or_where )? |{...}? PLUGIN_SYMBOL |{...}? PLUGINS_SYMBOL | ENGINE_SYMBOL engine_ref ( STATUS_SYMBOL | MUTEX_SYMBOL ) | ( FULL_SYMBOL )? COLUMNS_SYMBOL ( FROM_SYMBOL | IN_SYMBOL ) table_ref ( in_db )? ( like_or_where )? | ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL | SLAVE_SYMBOL ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? ) | ( BINLOG_SYMBOL | RELAYLOG_SYMBOL ) EVENTS_SYMBOL ( IN_SYMBOL text_string )? ( FROM_SYMBOL ulonglong_number )? ( limit_clause )? ( channel )? | ( INDEX_SYMBOL | INDEXES_SYMBOL | KEYS_SYMBOL ) from_or_in table_ref ( in_db )? ( where_clause )? | ( STORAGE_SYMBOL )? ENGINES_SYMBOL | PRIVILEGES_SYMBOL | COUNT_SYMBOL OPEN_PAR_SYMBOL MULT_OPERATOR CLOSE_PAR_SYMBOL ( WARNINGS_SYMBOL | ERRORS_SYMBOL ) | WARNINGS_SYMBOL ( limit_clause )? | ERRORS_SYMBOL ( limit_clause )? | PROFILES_SYMBOL | PROFILE_SYMBOL ( profile_type ( COMMA_SYMBOL profile_type )* )? ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )? ( limit_clause )? | ( option_type )? ( STATUS_SYMBOL | VARIABLES_SYMBOL ) ( like_or_where )? | ( FULL_SYMBOL )? PROCESSLIST_SYMBOL | charset ( like_or_where )? | COLLATION_SYMBOL ( like_or_where )? |{...}? CONTRIBUTORS_SYMBOL | GRANTS_SYMBOL ( FOR_SYMBOL user )? | MASTER_SYMBOL STATUS_SYMBOL | CREATE_SYMBOL ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user ) | PROCEDURE_SYMBOL STATUS_SYMBOL ( like_or_where )? | FUNCTION_SYMBOL STATUS_SYMBOL ( like_or_where )? | PROCEDURE_SYMBOL CODE_SYMBOL procedure_ref | FUNCTION_SYMBOL CODE_SYMBOL function_ref |{...}? INNODB_SYMBOL STATUS_SYMBOL ) )
        // MySQLSimpleParser.g:1524:2: SHOW_SYMBOL ({...}? AUTHORS_SYMBOL | DATABASES_SYMBOL ( like_or_where )? | ( FULL_SYMBOL )? TABLES_SYMBOL ( in_db )? ( like_or_where )? | ( FULL_SYMBOL )? TRIGGERS_SYMBOL ( in_db )? ( like_or_where )? | EVENTS_SYMBOL ( in_db )? ( like_or_where )? | TABLE_SYMBOL STATUS_SYMBOL ( in_db )? ( like_or_where )? | OPEN_SYMBOL TABLES_SYMBOL ( in_db )? ( like_or_where )? |{...}? PLUGIN_SYMBOL |{...}? PLUGINS_SYMBOL | ENGINE_SYMBOL engine_ref ( STATUS_SYMBOL | MUTEX_SYMBOL ) | ( FULL_SYMBOL )? COLUMNS_SYMBOL ( FROM_SYMBOL | IN_SYMBOL ) table_ref ( in_db )? ( like_or_where )? | ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL | SLAVE_SYMBOL ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? ) | ( BINLOG_SYMBOL | RELAYLOG_SYMBOL ) EVENTS_SYMBOL ( IN_SYMBOL text_string )? ( FROM_SYMBOL ulonglong_number )? ( limit_clause )? ( channel )? | ( INDEX_SYMBOL | INDEXES_SYMBOL | KEYS_SYMBOL ) from_or_in table_ref ( in_db )? ( where_clause )? | ( STORAGE_SYMBOL )? ENGINES_SYMBOL | PRIVILEGES_SYMBOL | COUNT_SYMBOL OPEN_PAR_SYMBOL MULT_OPERATOR CLOSE_PAR_SYMBOL ( WARNINGS_SYMBOL | ERRORS_SYMBOL ) | WARNINGS_SYMBOL ( limit_clause )? | ERRORS_SYMBOL ( limit_clause )? | PROFILES_SYMBOL | PROFILE_SYMBOL ( profile_type ( COMMA_SYMBOL profile_type )* )? ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )? ( limit_clause )? | ( option_type )? ( STATUS_SYMBOL | VARIABLES_SYMBOL ) ( like_or_where )? | ( FULL_SYMBOL )? PROCESSLIST_SYMBOL | charset ( like_or_where )? | COLLATION_SYMBOL ( like_or_where )? |{...}? CONTRIBUTORS_SYMBOL | GRANTS_SYMBOL ( FOR_SYMBOL user )? | MASTER_SYMBOL STATUS_SYMBOL | CREATE_SYMBOL ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user ) | PROCEDURE_SYMBOL STATUS_SYMBOL ( like_or_where )? | FUNCTION_SYMBOL STATUS_SYMBOL ( like_or_where )? | PROCEDURE_SYMBOL CODE_SYMBOL procedure_ref | FUNCTION_SYMBOL CODE_SYMBOL function_ref |{...}? INNODB_SYMBOL STATUS_SYMBOL )
        {
             MATCHT(SHOW_SYMBOL, &FOLLOW_SHOW_SYMBOL_in_show_statement8422);
            if  (HASEXCEPTION())
            {
                goto ruleshow_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1525:2: ({...}? AUTHORS_SYMBOL | DATABASES_SYMBOL ( like_or_where )? | ( FULL_SYMBOL )? TABLES_SYMBOL ( in_db )? ( like_or_where )? | ( FULL_SYMBOL )? TRIGGERS_SYMBOL ( in_db )? ( like_or_where )? | EVENTS_SYMBOL ( in_db )? ( like_or_where )? | TABLE_SYMBOL STATUS_SYMBOL ( in_db )? ( like_or_where )? | OPEN_SYMBOL TABLES_SYMBOL ( in_db )? ( like_or_where )? |{...}? PLUGIN_SYMBOL |{...}? PLUGINS_SYMBOL | ENGINE_SYMBOL engine_ref ( STATUS_SYMBOL | MUTEX_SYMBOL ) | ( FULL_SYMBOL )? COLUMNS_SYMBOL ( FROM_SYMBOL | IN_SYMBOL ) table_ref ( in_db )? ( like_or_where )? | ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL | SLAVE_SYMBOL ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? ) | ( BINLOG_SYMBOL | RELAYLOG_SYMBOL ) EVENTS_SYMBOL ( IN_SYMBOL text_string )? ( FROM_SYMBOL ulonglong_number )? ( limit_clause )? ( channel )? | ( INDEX_SYMBOL | INDEXES_SYMBOL | KEYS_SYMBOL ) from_or_in table_ref ( in_db )? ( where_clause )? | ( STORAGE_SYMBOL )? ENGINES_SYMBOL | PRIVILEGES_SYMBOL | COUNT_SYMBOL OPEN_PAR_SYMBOL MULT_OPERATOR CLOSE_PAR_SYMBOL ( WARNINGS_SYMBOL | ERRORS_SYMBOL ) | WARNINGS_SYMBOL ( limit_clause )? | ERRORS_SYMBOL ( limit_clause )? | PROFILES_SYMBOL | PROFILE_SYMBOL ( profile_type ( COMMA_SYMBOL profile_type )* )? ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )? ( limit_clause )? | ( option_type )? ( STATUS_SYMBOL | VARIABLES_SYMBOL ) ( like_or_where )? | ( FULL_SYMBOL )? PROCESSLIST_SYMBOL | charset ( like_or_where )? | COLLATION_SYMBOL ( like_or_where )? |{...}? CONTRIBUTORS_SYMBOL | GRANTS_SYMBOL ( FOR_SYMBOL user )? | MASTER_SYMBOL STATUS_SYMBOL | CREATE_SYMBOL ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user ) | PROCEDURE_SYMBOL STATUS_SYMBOL ( like_or_where )? | FUNCTION_SYMBOL STATUS_SYMBOL ( like_or_where )? | PROCEDURE_SYMBOL CODE_SYMBOL procedure_ref | FUNCTION_SYMBOL CODE_SYMBOL function_ref |{...}? INNODB_SYMBOL STATUS_SYMBOL )
            {
                int alt568=35;
                {
                    int LA568_0 = LA(1);
                    if ( (LA568_0 == AUTHORS_SYMBOL))
                    {
                        alt568=1;
                    }
                    else if ( (LA568_0 == DATABASES_SYMBOL))
                    {
                        alt568=2;
                    }
                    else if ( (LA568_0 == FULL_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case TABLES_SYMBOL:
                        	{
                        		alt568=3;
                        	}
                            break;
                        case TRIGGERS_SYMBOL:
                        	{
                        		alt568=4;
                        	}
                            break;
                        case COLUMNS_SYMBOL:
                        	{
                        		alt568=11;
                        	}
                            break;
                        case PROCESSLIST_SYMBOL:
                        	{
                        		alt568=24;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 568;
                            EXCEPTION->state        = 3;


                            goto ruleshow_statementEx;

                        }

                    }
                    else if ( (LA568_0 == TABLES_SYMBOL))
                    {
                        alt568=3;
                    }
                    else if ( (LA568_0 == TRIGGERS_SYMBOL))
                    {
                        alt568=4;
                    }
                    else if ( (LA568_0 == EVENTS_SYMBOL))
                    {
                        alt568=5;
                    }
                    else if ( (LA568_0 == TABLE_SYMBOL))
                    {
                        alt568=6;
                    }
                    else if ( (LA568_0 == OPEN_SYMBOL))
                    {
                        alt568=7;
                    }
                    else if ( (LA568_0 == PLUGIN_SYMBOL))
                    {
                        alt568=8;
                    }
                    else if ( (LA568_0 == PLUGINS_SYMBOL))
                    {
                        alt568=9;
                    }
                    else if ( (LA568_0 == ENGINE_SYMBOL))
                    {
                        alt568=10;
                    }
                    else if ( (LA568_0 == COLUMNS_SYMBOL))
                    {
                        alt568=11;
                    }
                    else if ( (LA568_0 == MASTER_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case LOGS_SYMBOL:
                        	{
                        		alt568=12;
                        	}
                            break;
                        case STATUS_SYMBOL:
                        	{
                        		alt568=29;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 568;
                            EXCEPTION->state        = 13;


                            goto ruleshow_statementEx;

                        }

                    }
                    else if ( (LA568_0 == SLAVE_SYMBOL))
                    {
                        alt568=13;
                    }
                    else if ( (LA568_0 == BINLOG_SYMBOL || LA568_0 == RELAYLOG_SYMBOL))
                    {
                        alt568=14;
                    }
                    else if ( (LA568_0 == INDEXES_SYMBOL || LA568_0 == INDEX_SYMBOL || LA568_0 == KEYS_SYMBOL))
                    {
                        alt568=15;
                    }
                    else if ( (LA568_0 == ENGINES_SYMBOL || LA568_0 == STORAGE_SYMBOL))
                    {
                        alt568=16;
                    }
                    else if ( (LA568_0 == PRIVILEGES_SYMBOL))
                    {
                        alt568=17;
                    }
                    else if ( (LA568_0 == COUNT_SYMBOL))
                    {
                        alt568=18;
                    }
                    else if ( (LA568_0 == WARNINGS_SYMBOL))
                    {
                        alt568=19;
                    }
                    else if ( (LA568_0 == ERRORS_SYMBOL))
                    {
                        alt568=20;
                    }
                    else if ( (LA568_0 == PROFILES_SYMBOL))
                    {
                        alt568=21;
                    }
                    else if ( (LA568_0 == PROFILE_SYMBOL))
                    {
                        alt568=22;
                    }
                    else if ( (LA568_0 == GLOBAL_SYMBOL || LA568_0 == LOCAL_SYMBOL || LA568_0 == SESSION_SYMBOL || LA568_0 == STATUS_SYMBOL || LA568_0 == VARIABLES_SYMBOL))
                    {
                        alt568=23;
                    }
                    else if ( (LA568_0 == PROCESSLIST_SYMBOL))
                    {
                        alt568=24;
                    }
                    else if ( (LA568_0 == CHARSET_SYMBOL || LA568_0 == CHAR_SYMBOL))
                    {
                        alt568=25;
                    }
                    else if ( (LA568_0 == COLLATION_SYMBOL))
                    {
                        alt568=26;
                    }
                    else if ( (LA568_0 == CONTRIBUTORS_SYMBOL))
                    {
                        alt568=27;
                    }
                    else if ( (LA568_0 == GRANTS_SYMBOL))
                    {
                        alt568=28;
                    }
                    else if ( (LA568_0 == BINARY_SYMBOL))
                    {
                        alt568=12;
                    }
                    else if ( (LA568_0 == CREATE_SYMBOL))
                    {
                        alt568=30;
                    }
                    else if ( (LA568_0 == PROCEDURE_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case STATUS_SYMBOL:
                        	{
                        		alt568=31;
                        	}
                            break;
                        case CODE_SYMBOL:
                        	{
                        		alt568=33;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 568;
                            EXCEPTION->state        = 35;


                            goto ruleshow_statementEx;

                        }

                    }
                    else if ( (LA568_0 == FUNCTION_SYMBOL))
                    {
                        switch ( LA(2) )
                        {
                        case STATUS_SYMBOL:
                        	{
                        		alt568=32;
                        	}
                            break;
                        case CODE_SYMBOL:
                        	{
                        		alt568=34;
                        	}
                            break;

                        default:
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }

                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 568;
                            EXCEPTION->state        = 36;


                            goto ruleshow_statementEx;

                        }

                    }
                    else if ( (LA568_0 == INNODB_SYMBOL))
                    {
                        alt568=35;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 568;
                        EXCEPTION->state        = 0;


                        goto ruleshow_statementEx;

                    }
                }
                switch (alt568)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1526:3: {...}? AUTHORS_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION < 50700)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50700";
            	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        }

            	         MATCHT(AUTHORS_SYMBOL, &FOLLOW_AUTHORS_SYMBOL_in_show_statement8431);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1527:5: DATABASES_SYMBOL ( like_or_where )?
            	    {
            	         MATCHT(DATABASES_SYMBOL, &FOLLOW_DATABASES_SYMBOL_in_show_statement8437);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1527:22: ( like_or_where )?
            	        {
            	            int alt527=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt527=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt527)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1527:22: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8439);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1528:5: ( FULL_SYMBOL )? TABLES_SYMBOL ( in_db )? ( like_or_where )?
            	    {
            	        // MySQLSimpleParser.g:1528:5: ( FULL_SYMBOL )?
            	        {
            	            int alt528=2;
            	            switch ( LA(1) )
            	            {
            	                case FULL_SYMBOL:
            	                	{
            	                		alt528=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt528)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1528:5: FULL_SYMBOL
            	        	    {
            	        	         MATCHT(FULL_SYMBOL, &FOLLOW_FULL_SYMBOL_in_show_statement8446);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(TABLES_SYMBOL, &FOLLOW_TABLES_SYMBOL_in_show_statement8449);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1528:32: ( in_db )?
            	        {
            	            int alt529=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt529=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt529)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1528:32: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8451);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1528:39: ( like_or_where )?
            	        {
            	            int alt530=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt530=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt530)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1528:39: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8454);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:1529:5: ( FULL_SYMBOL )? TRIGGERS_SYMBOL ( in_db )? ( like_or_where )?
            	    {
            	        // MySQLSimpleParser.g:1529:5: ( FULL_SYMBOL )?
            	        {
            	            int alt531=2;
            	            switch ( LA(1) )
            	            {
            	                case FULL_SYMBOL:
            	                	{
            	                		alt531=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt531)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1529:5: FULL_SYMBOL
            	        	    {
            	        	         MATCHT(FULL_SYMBOL, &FOLLOW_FULL_SYMBOL_in_show_statement8461);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(TRIGGERS_SYMBOL, &FOLLOW_TRIGGERS_SYMBOL_in_show_statement8464);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1529:34: ( in_db )?
            	        {
            	            int alt532=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt532=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt532)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1529:34: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8466);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1529:41: ( like_or_where )?
            	        {
            	            int alt533=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt533=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt533)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1529:41: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8469);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:1530:5: EVENTS_SYMBOL ( in_db )? ( like_or_where )?
            	    {
            	         MATCHT(EVENTS_SYMBOL, &FOLLOW_EVENTS_SYMBOL_in_show_statement8476);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1530:19: ( in_db )?
            	        {
            	            int alt534=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt534=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt534)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1530:19: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8478);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1530:26: ( like_or_where )?
            	        {
            	            int alt535=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt535=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt535)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1530:26: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8481);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:1531:5: TABLE_SYMBOL STATUS_SYMBOL ( in_db )? ( like_or_where )?
            	    {
            	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_show_statement8488);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8490);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1531:32: ( in_db )?
            	        {
            	            int alt536=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt536=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt536)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1531:32: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8492);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1531:39: ( like_or_where )?
            	        {
            	            int alt537=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt537=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt537)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1531:39: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8495);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 7:
            	    // MySQLSimpleParser.g:1532:5: OPEN_SYMBOL TABLES_SYMBOL ( in_db )? ( like_or_where )?
            	    {
            	         MATCHT(OPEN_SYMBOL, &FOLLOW_OPEN_SYMBOL_in_show_statement8502);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(TABLES_SYMBOL, &FOLLOW_TABLES_SYMBOL_in_show_statement8504);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1532:31: ( in_db )?
            	        {
            	            int alt538=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt538=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt538)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1532:31: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8506);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1532:38: ( like_or_where )?
            	        {
            	            int alt539=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt539=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt539)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1532:38: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8509);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 8:
            	    // MySQLSimpleParser.g:1533:5: {...}? PLUGIN_SYMBOL
            	    {
            	        if ( !(((SERVER_VERSION >= 50105) && (SERVER_VERSION < 50500))) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"(SERVER_VERSION >= 50105) && (SERVER_VERSION < 50500)";
            	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        }

            	         MATCHT(PLUGIN_SYMBOL, &FOLLOW_PLUGIN_SYMBOL_in_show_statement8518);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 9:
            	    // MySQLSimpleParser.g:1534:5: {...}? PLUGINS_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION >= 50500)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
            	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        }

            	         MATCHT(PLUGINS_SYMBOL, &FOLLOW_PLUGINS_SYMBOL_in_show_statement8527);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 10:
            	    // MySQLSimpleParser.g:1535:5: ENGINE_SYMBOL engine_ref ( STATUS_SYMBOL | MUTEX_SYMBOL )
            	    {
            	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_show_statement8533);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_engine_ref_in_show_statement8535);
            	        engine_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == MUTEX_SYMBOL || LA(1) == STATUS_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	    }
            	    break;
            	case 11:
            	    // MySQLSimpleParser.g:1536:5: ( FULL_SYMBOL )? COLUMNS_SYMBOL ( FROM_SYMBOL | IN_SYMBOL ) table_ref ( in_db )? ( like_or_where )?
            	    {
            	        // MySQLSimpleParser.g:1536:5: ( FULL_SYMBOL )?
            	        {
            	            int alt540=2;
            	            switch ( LA(1) )
            	            {
            	                case FULL_SYMBOL:
            	                	{
            	                		alt540=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt540)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1536:5: FULL_SYMBOL
            	        	    {
            	        	         MATCHT(FULL_SYMBOL, &FOLLOW_FULL_SYMBOL_in_show_statement8549);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(COLUMNS_SYMBOL, &FOLLOW_COLUMNS_SYMBOL_in_show_statement8552);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == FROM_SYMBOL || LA(1) == IN_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_show_statement8562);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1536:69: ( in_db )?
            	        {
            	            int alt541=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt541=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt541)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1536:69: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8564);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1536:76: ( like_or_where )?
            	        {
            	            int alt542=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt542=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt542)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1536:76: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8567);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 12:
            	    // MySQLSimpleParser.g:1537:5: ( BINARY_SYMBOL | MASTER_SYMBOL ) LOGS_SYMBOL
            	    {
            	        if ( LA(1) == MASTER_SYMBOL || LA(1) == BINARY_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	         MATCHT(LOGS_SYMBOL, &FOLLOW_LOGS_SYMBOL_in_show_statement8582);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 13:
            	    // MySQLSimpleParser.g:1538:5: SLAVE_SYMBOL ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? )
            	    {
            	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_show_statement8588);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1539:4: ( HOSTS_SYMBOL | STATUS_SYMBOL non_blocking ( channel )? )
            	        {
            	            int alt544=2;
            	            switch ( LA(1) )
            	            {
            	            case HOSTS_SYMBOL:
            	            	{
            	            		alt544=1;
            	            	}
            	                break;
            	            case STATUS_SYMBOL:
            	            	{
            	            		alt544=2;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 544;
            	                EXCEPTION->state        = 0;


            	                goto ruleshow_statementEx;

            	            }

            	            switch (alt544)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1540:5: HOSTS_SYMBOL
            	        	    {
            	        	         MATCHT(HOSTS_SYMBOL, &FOLLOW_HOSTS_SYMBOL_in_show_statement8599);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:1541:7: STATUS_SYMBOL non_blocking ( channel )?
            	        	    {
            	        	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8607);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_non_blocking_in_show_statement8609);
            	        	        non_blocking(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:1541:34: ( channel )?
            	        	        {
            	        	            int alt543=2;
            	        	            {
            	        	                int LA543_0 = LA(1);
            	        	                if ( ((SERVER_VERSION >= 50706)) && (LA543_0 == FOR_SYMBOL))
            	        	                {
            	        	                    alt543=1;
            	        	                }
            	        	            }
            	        	            switch (alt543)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:1541:34: channel
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_channel_in_show_statement8611);
            	        	        	        channel(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleshow_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 14:
            	    // MySQLSimpleParser.g:1543:5: ( BINLOG_SYMBOL | RELAYLOG_SYMBOL ) EVENTS_SYMBOL ( IN_SYMBOL text_string )? ( FROM_SYMBOL ulonglong_number )? ( limit_clause )? ( channel )?
            	    {
            	        if ( LA(1) == BINLOG_SYMBOL || LA(1) == RELAYLOG_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	         MATCHT(EVENTS_SYMBOL, &FOLLOW_EVENTS_SYMBOL_in_show_statement8631);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1543:53: ( IN_SYMBOL text_string )?
            	        {
            	            int alt545=2;
            	            switch ( LA(1) )
            	            {
            	                case IN_SYMBOL:
            	                	{
            	                		alt545=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt545)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1543:54: IN_SYMBOL text_string
            	        	    {
            	        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_show_statement8634);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_text_string_in_show_statement8636);
            	        	        text_string(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1543:78: ( FROM_SYMBOL ulonglong_number )?
            	        {
            	            int alt546=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                	{
            	                		alt546=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt546)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1543:79: FROM_SYMBOL ulonglong_number
            	        	    {
            	        	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_show_statement8641);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_ulonglong_number_in_show_statement8643);
            	        	        ulonglong_number(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1543:110: ( limit_clause )?
            	        {
            	            int alt547=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                	{
            	                		alt547=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt547)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1543:110: limit_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_show_statement8647);
            	        	        limit_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1543:124: ( channel )?
            	        {
            	            int alt548=2;
            	            {
            	                int LA548_0 = LA(1);
            	                if ( ((SERVER_VERSION >= 50706)) && (LA548_0 == FOR_SYMBOL))
            	                {
            	                    alt548=1;
            	                }
            	            }
            	            switch (alt548)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1543:124: channel
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_channel_in_show_statement8650);
            	        	        channel(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 15:
            	    // MySQLSimpleParser.g:1544:5: ( INDEX_SYMBOL | INDEXES_SYMBOL | KEYS_SYMBOL ) from_or_in table_ref ( in_db )? ( where_clause )?
            	    {
            	        if ( LA(1) == INDEXES_SYMBOL || LA(1) == INDEX_SYMBOL || LA(1) == KEYS_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_from_or_in_in_show_statement8669);
            	        from_or_in(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_show_statement8671);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1544:72: ( in_db )?
            	        {
            	            int alt549=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                case IN_SYMBOL:
            	                	{
            	                		alt549=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt549)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1544:72: in_db
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_in_db_in_show_statement8673);
            	        	        in_db(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1544:79: ( where_clause )?
            	        {
            	            int alt550=2;
            	            switch ( LA(1) )
            	            {
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt550=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt550)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1544:79: where_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_where_clause_in_show_statement8676);
            	        	        where_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 16:
            	    // MySQLSimpleParser.g:1545:5: ( STORAGE_SYMBOL )? ENGINES_SYMBOL
            	    {
            	        // MySQLSimpleParser.g:1545:5: ( STORAGE_SYMBOL )?
            	        {
            	            int alt551=2;
            	            switch ( LA(1) )
            	            {
            	                case STORAGE_SYMBOL:
            	                	{
            	                		alt551=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt551)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1545:5: STORAGE_SYMBOL
            	        	    {
            	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_show_statement8683);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(ENGINES_SYMBOL, &FOLLOW_ENGINES_SYMBOL_in_show_statement8686);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 17:
            	    // MySQLSimpleParser.g:1546:5: PRIVILEGES_SYMBOL
            	    {
            	         MATCHT(PRIVILEGES_SYMBOL, &FOLLOW_PRIVILEGES_SYMBOL_in_show_statement8692);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 18:
            	    // MySQLSimpleParser.g:1547:5: COUNT_SYMBOL OPEN_PAR_SYMBOL MULT_OPERATOR CLOSE_PAR_SYMBOL ( WARNINGS_SYMBOL | ERRORS_SYMBOL )
            	    {
            	         MATCHT(COUNT_SYMBOL, &FOLLOW_COUNT_SYMBOL_in_show_statement8698);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_show_statement8700);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_show_statement8702);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_show_statement8704);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == ERRORS_SYMBOL || LA(1) == WARNINGS_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	    }
            	    break;
            	case 19:
            	    // MySQLSimpleParser.g:1548:5: WARNINGS_SYMBOL ( limit_clause )?
            	    {
            	         MATCHT(WARNINGS_SYMBOL, &FOLLOW_WARNINGS_SYMBOL_in_show_statement8718);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1548:21: ( limit_clause )?
            	        {
            	            int alt552=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                	{
            	                		alt552=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt552)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1548:21: limit_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_show_statement8720);
            	        	        limit_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 20:
            	    // MySQLSimpleParser.g:1549:5: ERRORS_SYMBOL ( limit_clause )?
            	    {
            	         MATCHT(ERRORS_SYMBOL, &FOLLOW_ERRORS_SYMBOL_in_show_statement8727);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1549:19: ( limit_clause )?
            	        {
            	            int alt553=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                	{
            	                		alt553=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt553)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1549:19: limit_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_show_statement8729);
            	        	        limit_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 21:
            	    // MySQLSimpleParser.g:1550:5: PROFILES_SYMBOL
            	    {
            	         MATCHT(PROFILES_SYMBOL, &FOLLOW_PROFILES_SYMBOL_in_show_statement8736);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 22:
            	    // MySQLSimpleParser.g:1551:5: PROFILE_SYMBOL ( profile_type ( COMMA_SYMBOL profile_type )* )? ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )? ( limit_clause )?
            	    {
            	         MATCHT(PROFILE_SYMBOL, &FOLLOW_PROFILE_SYMBOL_in_show_statement8742);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1551:20: ( profile_type ( COMMA_SYMBOL profile_type )* )?
            	        {
            	            int alt555=2;
            	            switch ( LA(1) )
            	            {
            	                case BLOCK_SYMBOL:
            	                case CONTEXT_SYMBOL:
            	                case CPU_SYMBOL:
            	                case IPC_SYMBOL:
            	                case MEMORY_SYMBOL:
            	                case PAGE_SYMBOL:
            	                case SOURCE_SYMBOL:
            	                case SWAPS_SYMBOL:
            	                case ALL_SYMBOL:
            	                	{
            	                		alt555=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt555)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1551:21: profile_type ( COMMA_SYMBOL profile_type )*
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_profile_type_in_show_statement8745);
            	        	        profile_type(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:1551:34: ( COMMA_SYMBOL profile_type )*

            	        	        for (;;)
            	        	        {
            	        	            int alt554=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case COMMA_SYMBOL:
            	        	            	{
            	        	            		alt554=1;
            	        	            	}
            	        	                break;

            	        	            }

            	        	            switch (alt554)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:1551:35: COMMA_SYMBOL profile_type
            	        	        	    {
            	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_show_statement8748);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleshow_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	        FOLLOWPUSH(FOLLOW_profile_type_in_show_statement8750);
            	        	        	        profile_type(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleshow_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	        	default:
            	        	        	    goto loop554;	/* break out of the loop */
            	        	        	    break;
            	        	            }
            	        	        }
            	        	        loop554: ; /* Jump out to here if this rule does not match */


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1551:65: ( FOR_SYMBOL QUERY_SYMBOL INT_NUMBER )?
            	        {
            	            int alt556=2;
            	            switch ( LA(1) )
            	            {
            	                case FOR_SYMBOL:
            	                	{
            	                		alt556=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt556)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1551:66: FOR_SYMBOL QUERY_SYMBOL INT_NUMBER
            	        	    {
            	        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_show_statement8757);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(QUERY_SYMBOL, &FOLLOW_QUERY_SYMBOL_in_show_statement8759);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_show_statement8761);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1551:103: ( limit_clause )?
            	        {
            	            int alt557=2;
            	            switch ( LA(1) )
            	            {
            	                case LIMIT_SYMBOL:
            	                	{
            	                		alt557=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt557)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1551:103: limit_clause
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_limit_clause_in_show_statement8765);
            	        	        limit_clause(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 23:
            	    // MySQLSimpleParser.g:1552:5: ( option_type )? ( STATUS_SYMBOL | VARIABLES_SYMBOL ) ( like_or_where )?
            	    {
            	        // MySQLSimpleParser.g:1552:5: ( option_type )?
            	        {
            	            int alt558=2;
            	            switch ( LA(1) )
            	            {
            	                case GLOBAL_SYMBOL:
            	                case LOCAL_SYMBOL:
            	                case SESSION_SYMBOL:
            	                	{
            	                		alt558=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt558)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1552:5: option_type
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_option_type_in_show_statement8772);
            	        	        option_type(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        if ( LA(1) == STATUS_SYMBOL || LA(1) == VARIABLES_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshow_statementEx;
            	        }


            	        // MySQLSimpleParser.g:1552:53: ( like_or_where )?
            	        {
            	            int alt559=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt559=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt559)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1552:53: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8783);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 24:
            	    // MySQLSimpleParser.g:1553:5: ( FULL_SYMBOL )? PROCESSLIST_SYMBOL
            	    {
            	        // MySQLSimpleParser.g:1553:5: ( FULL_SYMBOL )?
            	        {
            	            int alt560=2;
            	            switch ( LA(1) )
            	            {
            	                case FULL_SYMBOL:
            	                	{
            	                		alt560=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt560)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1553:5: FULL_SYMBOL
            	        	    {
            	        	         MATCHT(FULL_SYMBOL, &FOLLOW_FULL_SYMBOL_in_show_statement8790);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(PROCESSLIST_SYMBOL, &FOLLOW_PROCESSLIST_SYMBOL_in_show_statement8793);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 25:
            	    // MySQLSimpleParser.g:1554:5: charset ( like_or_where )?
            	    {
            	        FOLLOWPUSH(FOLLOW_charset_in_show_statement8799);
            	        charset(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1554:13: ( like_or_where )?
            	        {
            	            int alt561=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt561=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt561)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1554:13: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8801);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 26:
            	    // MySQLSimpleParser.g:1555:5: COLLATION_SYMBOL ( like_or_where )?
            	    {
            	         MATCHT(COLLATION_SYMBOL, &FOLLOW_COLLATION_SYMBOL_in_show_statement8808);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1555:22: ( like_or_where )?
            	        {
            	            int alt562=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt562=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt562)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1555:22: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8810);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 27:
            	    // MySQLSimpleParser.g:1556:5: {...}? CONTRIBUTORS_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION < 50700)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50700";
            	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        }

            	         MATCHT(CONTRIBUTORS_SYMBOL, &FOLLOW_CONTRIBUTORS_SYMBOL_in_show_statement8819);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 28:
            	    // MySQLSimpleParser.g:1557:5: GRANTS_SYMBOL ( FOR_SYMBOL user )?
            	    {
            	         MATCHT(GRANTS_SYMBOL, &FOLLOW_GRANTS_SYMBOL_in_show_statement8825);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1557:19: ( FOR_SYMBOL user )?
            	        {
            	            int alt563=2;
            	            switch ( LA(1) )
            	            {
            	                case FOR_SYMBOL:
            	                	{
            	                		alt563=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt563)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1557:20: FOR_SYMBOL user
            	        	    {
            	        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_show_statement8828);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_user_in_show_statement8830);
            	        	        user(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 29:
            	    // MySQLSimpleParser.g:1558:5: MASTER_SYMBOL STATUS_SYMBOL
            	    {
            	         MATCHT(MASTER_SYMBOL, &FOLLOW_MASTER_SYMBOL_in_show_statement8838);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8840);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 30:
            	    // MySQLSimpleParser.g:1559:5: CREATE_SYMBOL ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user )
            	    {
            	         MATCHT(CREATE_SYMBOL, &FOLLOW_CREATE_SYMBOL_in_show_statement8846);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1560:4: ( DATABASE_SYMBOL ( if_not_exists )? schema_ref | EVENT_SYMBOL event_ref | FUNCTION_SYMBOL function_ref | PROCEDURE_SYMBOL procedure_ref | TABLE_SYMBOL table_ref | TRIGGER_SYMBOL trigger_ref | VIEW_SYMBOL view_ref |{...}? USER_SYMBOL user )
            	        {
            	            int alt565=8;
            	            switch ( LA(1) )
            	            {
            	            case DATABASE_SYMBOL:
            	            	{
            	            		alt565=1;
            	            	}
            	                break;
            	            case EVENT_SYMBOL:
            	            	{
            	            		alt565=2;
            	            	}
            	                break;
            	            case FUNCTION_SYMBOL:
            	            	{
            	            		alt565=3;
            	            	}
            	                break;
            	            case PROCEDURE_SYMBOL:
            	            	{
            	            		alt565=4;
            	            	}
            	                break;
            	            case TABLE_SYMBOL:
            	            	{
            	            		alt565=5;
            	            	}
            	                break;
            	            case TRIGGER_SYMBOL:
            	            	{
            	            		alt565=6;
            	            	}
            	                break;
            	            case VIEW_SYMBOL:
            	            	{
            	            		alt565=7;
            	            	}
            	                break;
            	            case USER_SYMBOL:
            	            	{
            	            		alt565=8;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 565;
            	                EXCEPTION->state        = 0;


            	                goto ruleshow_statementEx;

            	            }

            	            switch (alt565)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1561:5: DATABASE_SYMBOL ( if_not_exists )? schema_ref
            	        	    {
            	        	         MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_show_statement8857);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:1561:21: ( if_not_exists )?
            	        	        {
            	        	            int alt564=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case IF_SYMBOL:
            	        	                	{
            	        	                		alt564=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt564)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:1561:21: if_not_exists
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_if_not_exists_in_show_statement8859);
            	        	        	        if_not_exists(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto ruleshow_statementEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_schema_ref_in_show_statement8862);
            	        	        schema_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:1562:7: EVENT_SYMBOL event_ref
            	        	    {
            	        	         MATCHT(EVENT_SYMBOL, &FOLLOW_EVENT_SYMBOL_in_show_statement8870);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_event_ref_in_show_statement8872);
            	        	        event_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 3:
            	        	    // MySQLSimpleParser.g:1563:7: FUNCTION_SYMBOL function_ref
            	        	    {
            	        	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_show_statement8880);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_function_ref_in_show_statement8882);
            	        	        function_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 4:
            	        	    // MySQLSimpleParser.g:1564:7: PROCEDURE_SYMBOL procedure_ref
            	        	    {
            	        	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_show_statement8890);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_procedure_ref_in_show_statement8892);
            	        	        procedure_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 5:
            	        	    // MySQLSimpleParser.g:1565:7: TABLE_SYMBOL table_ref
            	        	    {
            	        	         MATCHT(TABLE_SYMBOL, &FOLLOW_TABLE_SYMBOL_in_show_statement8900);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_table_ref_in_show_statement8902);
            	        	        table_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 6:
            	        	    // MySQLSimpleParser.g:1566:7: TRIGGER_SYMBOL trigger_ref
            	        	    {
            	        	         MATCHT(TRIGGER_SYMBOL, &FOLLOW_TRIGGER_SYMBOL_in_show_statement8910);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_trigger_ref_in_show_statement8912);
            	        	        trigger_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 7:
            	        	    // MySQLSimpleParser.g:1567:7: VIEW_SYMBOL view_ref
            	        	    {
            	        	         MATCHT(VIEW_SYMBOL, &FOLLOW_VIEW_SYMBOL_in_show_statement8920);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_view_ref_in_show_statement8922);
            	        	        view_ref(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 8:
            	        	    // MySQLSimpleParser.g:1568:7: {...}? USER_SYMBOL user
            	        	    {
            	        	        if ( !((SERVER_VERSION >= 50704)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50704";
            	        	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        	        }

            	        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_show_statement8932);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_user_in_show_statement8934);
            	        	        user(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 31:
            	    // MySQLSimpleParser.g:1570:5: PROCEDURE_SYMBOL STATUS_SYMBOL ( like_or_where )?
            	    {
            	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_show_statement8945);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8947);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1570:36: ( like_or_where )?
            	        {
            	            int alt566=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt566=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt566)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1570:36: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8949);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 32:
            	    // MySQLSimpleParser.g:1571:5: FUNCTION_SYMBOL STATUS_SYMBOL ( like_or_where )?
            	    {
            	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_show_statement8956);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8958);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1571:35: ( like_or_where )?
            	        {
            	            int alt567=2;
            	            switch ( LA(1) )
            	            {
            	                case LIKE_SYMBOL:
            	                case WHERE_SYMBOL:
            	                	{
            	                		alt567=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt567)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1571:35: like_or_where
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_like_or_where_in_show_statement8960);
            	        	        like_or_where(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleshow_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 33:
            	    // MySQLSimpleParser.g:1572:5: PROCEDURE_SYMBOL CODE_SYMBOL procedure_ref
            	    {
            	         MATCHT(PROCEDURE_SYMBOL, &FOLLOW_PROCEDURE_SYMBOL_in_show_statement8967);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CODE_SYMBOL, &FOLLOW_CODE_SYMBOL_in_show_statement8969);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_procedure_ref_in_show_statement8971);
            	        procedure_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 34:
            	    // MySQLSimpleParser.g:1573:5: FUNCTION_SYMBOL CODE_SYMBOL function_ref
            	    {
            	         MATCHT(FUNCTION_SYMBOL, &FOLLOW_FUNCTION_SYMBOL_in_show_statement8977);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CODE_SYMBOL, &FOLLOW_CODE_SYMBOL_in_show_statement8979);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_function_ref_in_show_statement8981);
            	        function_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 35:
            	    // MySQLSimpleParser.g:1574:5: {...}? INNODB_SYMBOL STATUS_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION < 50500)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50500";
            	                EXCEPTION->ruleName	 = (void *)"show_statement";


            	        }

            	         MATCHT(INNODB_SYMBOL, &FOLLOW_INNODB_SYMBOL_in_show_statement8989);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_show_statement8991);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshow_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleshow_statementEx; /* Prevent compiler warnings */
    ruleshow_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end show_statement */

/**
 * $ANTLR start non_blocking
 * MySQLSimpleParser.g:1578:1: non_blocking : ({...}? ( NONBLOCKING_SYMBOL )? |);
 */
static void
non_blocking(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1578:13: ({...}? ( NONBLOCKING_SYMBOL )? |)

            ANTLR3_UINT32 alt570;

            alt570=2;

            {
                int LA570_0 = LA(1);
                if ( (LA570_0 == NONBLOCKING_SYMBOL))
                {
                    alt570=1;
                }
                else if ( ((SERVER_VERSION >= 50706)) && (LA570_0 == FOR_SYMBOL))
                {
                    {
                        int LA570_2 = LA(2);
                        if ( (( (( ((SERVER_VERSION >= 50700 && SERVER_VERSION < 50706)) && ((SERVER_VERSION >= 50700 && SERVER_VERSION < 50706)) )) && ((SERVER_VERSION >= 50706)) )))
                        {
                            alt570=1;
                        }
                        else if ( ((SERVER_VERSION >= 50706)))
                        {
                            alt570=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 570;
                            EXCEPTION->state        = 2;


                            goto rulenon_blockingEx;

                        }
                    }
                }
                else if ( (LA570_0 == SEMICOLON_SYMBOL))
                {
                    {
                        int LA570_3 = LA(2);
                        if ( ((SERVER_VERSION >= 50700 && SERVER_VERSION < 50706)))
                        {
                            alt570=1;
                        }
                        else if ( (ANTLR3_TRUE))
                        {
                            alt570=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 570;
                            EXCEPTION->state        = 3;


                            goto rulenon_blockingEx;

                        }
                    }
                }
                else if ( (LA570_0 == EOF))
                {
                    {
                        int LA570_4 = LA(2);
                        if ( ((SERVER_VERSION >= 50700 && SERVER_VERSION < 50706)))
                        {
                            alt570=1;
                        }
                        else if ( (ANTLR3_TRUE))
                        {
                            alt570=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 570;
                            EXCEPTION->state        = 4;


                            goto rulenon_blockingEx;

                        }
                    }
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 570;
                    EXCEPTION->state        = 0;


                    goto rulenon_blockingEx;

                }
            }
            switch (alt570)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1579:2: {...}? ( NONBLOCKING_SYMBOL )?
        	    {
        	        if ( !((SERVER_VERSION >= 50700 && SERVER_VERSION < 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700 && SERVER_VERSION < 50706";
        	                EXCEPTION->ruleName	 = (void *)"non_blocking";


        	        }

        	        // MySQLSimpleParser.g:1579:55: ( NONBLOCKING_SYMBOL )?
        	        {
        	            int alt569=2;
        	            switch ( LA(1) )
        	            {
        	                case NONBLOCKING_SYMBOL:
        	                	{
        	                		alt569=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt569)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1579:55: NONBLOCKING_SYMBOL
        	        	    {
        	        	         MATCHT(NONBLOCKING_SYMBOL, &FOLLOW_NONBLOCKING_SYMBOL_in_non_blocking9006);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulenon_blockingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1581:2: 
        	    {
        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulenon_blockingEx; /* Prevent compiler warnings */
    rulenon_blockingEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end non_blocking */

/**
 * $ANTLR start from_or_in
 * MySQLSimpleParser.g:1583:1: from_or_in : ( FROM_SYMBOL | IN_SYMBOL );
 */
static void
from_or_in(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1583:11: ( FROM_SYMBOL | IN_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == FROM_SYMBOL || LA(1) == IN_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulefrom_or_inEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefrom_or_inEx; /* Prevent compiler warnings */
    rulefrom_or_inEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end from_or_in */

/**
 * $ANTLR start in_db
 * MySQLSimpleParser.g:1587:1: in_db : from_or_in identifier ;
 */
static void
in_db(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1587:6: ( from_or_in identifier )
        // MySQLSimpleParser.g:1588:2: from_or_in identifier
        {
            FOLLOWPUSH(FOLLOW_from_or_in_in_in_db9034);
            from_or_in(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulein_dbEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_in_db9036);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulein_dbEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulein_dbEx; /* Prevent compiler warnings */
    rulein_dbEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end in_db */

/**
 * $ANTLR start profile_type
 * MySQLSimpleParser.g:1591:1: profile_type : ( ALL_SYMBOL | BLOCK_SYMBOL IO_SYMBOL | CONTEXT_SYMBOL SWITCHES_SYMBOL | CPU_SYMBOL | IPC_SYMBOL | MEMORY_SYMBOL | PAGE_SYMBOL FAULTS_SYMBOL | SOURCE_SYMBOL | SWAPS_SYMBOL );
 */
static void
profile_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1591:13: ( ALL_SYMBOL | BLOCK_SYMBOL IO_SYMBOL | CONTEXT_SYMBOL SWITCHES_SYMBOL | CPU_SYMBOL | IPC_SYMBOL | MEMORY_SYMBOL | PAGE_SYMBOL FAULTS_SYMBOL | SOURCE_SYMBOL | SWAPS_SYMBOL )

            ANTLR3_UINT32 alt571;

            alt571=9;

            switch ( LA(1) )
            {
            case ALL_SYMBOL:
            	{
            		alt571=1;
            	}
                break;
            case BLOCK_SYMBOL:
            	{
            		alt571=2;
            	}
                break;
            case CONTEXT_SYMBOL:
            	{
            		alt571=3;
            	}
                break;
            case CPU_SYMBOL:
            	{
            		alt571=4;
            	}
                break;
            case IPC_SYMBOL:
            	{
            		alt571=5;
            	}
                break;
            case MEMORY_SYMBOL:
            	{
            		alt571=6;
            	}
                break;
            case PAGE_SYMBOL:
            	{
            		alt571=7;
            	}
                break;
            case SOURCE_SYMBOL:
            	{
            		alt571=8;
            	}
                break;
            case SWAPS_SYMBOL:
            	{
            		alt571=9;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 571;
                EXCEPTION->state        = 0;


                goto ruleprofile_typeEx;

            }

            switch (alt571)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1592:2: ALL_SYMBOL
        	    {
        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_profile_type9045);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1593:4: BLOCK_SYMBOL IO_SYMBOL
        	    {
        	         MATCHT(BLOCK_SYMBOL, &FOLLOW_BLOCK_SYMBOL_in_profile_type9050);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(IO_SYMBOL, &FOLLOW_IO_SYMBOL_in_profile_type9052);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1594:4: CONTEXT_SYMBOL SWITCHES_SYMBOL
        	    {
        	         MATCHT(CONTEXT_SYMBOL, &FOLLOW_CONTEXT_SYMBOL_in_profile_type9057);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SWITCHES_SYMBOL, &FOLLOW_SWITCHES_SYMBOL_in_profile_type9059);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1595:4: CPU_SYMBOL
        	    {
        	         MATCHT(CPU_SYMBOL, &FOLLOW_CPU_SYMBOL_in_profile_type9064);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1596:4: IPC_SYMBOL
        	    {
        	         MATCHT(IPC_SYMBOL, &FOLLOW_IPC_SYMBOL_in_profile_type9069);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1597:4: MEMORY_SYMBOL
        	    {
        	         MATCHT(MEMORY_SYMBOL, &FOLLOW_MEMORY_SYMBOL_in_profile_type9074);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1598:4: PAGE_SYMBOL FAULTS_SYMBOL
        	    {
        	         MATCHT(PAGE_SYMBOL, &FOLLOW_PAGE_SYMBOL_in_profile_type9079);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FAULTS_SYMBOL, &FOLLOW_FAULTS_SYMBOL_in_profile_type9081);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1599:4: SOURCE_SYMBOL
        	    {
        	         MATCHT(SOURCE_SYMBOL, &FOLLOW_SOURCE_SYMBOL_in_profile_type9086);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:1600:4: SWAPS_SYMBOL
        	    {
        	         MATCHT(SWAPS_SYMBOL, &FOLLOW_SWAPS_SYMBOL_in_profile_type9091);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleprofile_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleprofile_typeEx; /* Prevent compiler warnings */
    ruleprofile_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end profile_type */

/**
 * $ANTLR start other_administrative_statement
 * MySQLSimpleParser.g:1605:1: other_administrative_statement : ( BINLOG_SYMBOL string_literal | CACHE_SYMBOL INDEX_SYMBOL key_cache_list_or_parts IN_SYMBOL ( identifier | DEFAULT_SYMBOL ) | FLUSH_SYMBOL ( no_write_to_bin_log )? ( flush_tables | flush_option ( COMMA_SYMBOL flush_option )* ) | KILL_SYMBOL ( options {greedy=true; } : ( CONNECTION_SYMBOL | QUERY_SYMBOL ) )? expression | LOAD_SYMBOL INDEX_SYMBOL INTO_SYMBOL CACHE_SYMBOL load_table_index_list | RESET_SYMBOL reset_option ( COMMA_SYMBOL reset_option )* |{...}? SHUTDOWN_SYMBOL );
 */
static void
other_administrative_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1605:31: ( BINLOG_SYMBOL string_literal | CACHE_SYMBOL INDEX_SYMBOL key_cache_list_or_parts IN_SYMBOL ( identifier | DEFAULT_SYMBOL ) | FLUSH_SYMBOL ( no_write_to_bin_log )? ( flush_tables | flush_option ( COMMA_SYMBOL flush_option )* ) | KILL_SYMBOL ( options {greedy=true; } : ( CONNECTION_SYMBOL | QUERY_SYMBOL ) )? expression | LOAD_SYMBOL INDEX_SYMBOL INTO_SYMBOL CACHE_SYMBOL load_table_index_list | RESET_SYMBOL reset_option ( COMMA_SYMBOL reset_option )* |{...}? SHUTDOWN_SYMBOL )

            ANTLR3_UINT32 alt578;

            alt578=7;

            switch ( LA(1) )
            {
            case BINLOG_SYMBOL:
            	{
            		alt578=1;
            	}
                break;
            case CACHE_SYMBOL:
            	{
            		alt578=2;
            	}
                break;
            case FLUSH_SYMBOL:
            	{
            		alt578=3;
            	}
                break;
            case KILL_SYMBOL:
            	{
            		alt578=4;
            	}
                break;
            case LOAD_SYMBOL:
            	{
            		alt578=5;
            	}
                break;
            case RESET_SYMBOL:
            	{
            		alt578=6;
            	}
                break;
            case SHUTDOWN_SYMBOL:
            	{
            		alt578=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 578;
                EXCEPTION->state        = 0;


                goto ruleother_administrative_statementEx;

            }

            switch (alt578)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1606:2: BINLOG_SYMBOL string_literal
        	    {
        	         MATCHT(BINLOG_SYMBOL, &FOLLOW_BINLOG_SYMBOL_in_other_administrative_statement9102);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_other_administrative_statement9104);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1607:4: CACHE_SYMBOL INDEX_SYMBOL key_cache_list_or_parts IN_SYMBOL ( identifier | DEFAULT_SYMBOL )
        	    {
        	         MATCHT(CACHE_SYMBOL, &FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9109);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9111);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_key_cache_list_or_parts_in_other_administrative_statement9113);
        	        key_cache_list_or_parts(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_other_administrative_statement9115);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1607:64: ( identifier | DEFAULT_SYMBOL )
        	        {
        	            int alt572=2;
        	            {
        	                int LA572_0 = LA(1);
        	                if ( (((LA572_0 >= ACCOUNT_SYMBOL) && (LA572_0 <= YEAR_SYMBOL)) || LA572_0 == BACK_TICK_QUOTED_ID || LA572_0 == DOUBLE_QUOTED_TEXT || LA572_0 == IDENTIFIER))
        	                {
        	                    alt572=1;
        	                }
        	                else if ( (LA572_0 == DEFAULT_SYMBOL))
        	                {
        	                    alt572=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 572;
        	                    EXCEPTION->state        = 0;


        	                    goto ruleother_administrative_statementEx;

        	                }
        	            }
        	            switch (alt572)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1607:65: identifier
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_identifier_in_other_administrative_statement9118);
        	        	        identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1607:78: DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_other_administrative_statement9122);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1608:4: FLUSH_SYMBOL ( no_write_to_bin_log )? ( flush_tables | flush_option ( COMMA_SYMBOL flush_option )* )
        	    {
        	         MATCHT(FLUSH_SYMBOL, &FOLLOW_FLUSH_SYMBOL_in_other_administrative_statement9128);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1608:17: ( no_write_to_bin_log )?
        	        {
        	            int alt573=2;
        	            {
        	                int LA573_0 = LA(1);
        	                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA573_0 == LOCAL_SYMBOL))
        	                {
        	                    alt573=1;
        	                }
        	                else if ( (LA573_0 == NO_WRITE_TO_BINLOG_SYMBOL))
        	                {
        	                    alt573=1;
        	                }
        	            }
        	            switch (alt573)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1608:17: no_write_to_bin_log
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_no_write_to_bin_log_in_other_administrative_statement9130);
        	        	        no_write_to_bin_log(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1609:3: ( flush_tables | flush_option ( COMMA_SYMBOL flush_option )* )
        	        {
        	            int alt575=2;
        	            switch ( LA(1) )
        	            {
        	            case TABLES_SYMBOL:
        	            case TABLE_SYMBOL:
        	            	{
        	            		alt575=1;
        	            	}
        	                break;
        	            case DES_KEY_FILE_SYMBOL:
        	            case ENGINE_SYMBOL:
        	            case ERROR_SYMBOL:
        	            case GENERAL_SYMBOL:
        	            case HOSTS_SYMBOL:
        	            case LOGS_SYMBOL:
        	            case PRIVILEGES_SYMBOL:
        	            case QUERY_SYMBOL:
        	            case RELAY_SYMBOL:
        	            case SLOW_SYMBOL:
        	            case STATUS_SYMBOL:
        	            case USER_RESOURCES_SYMBOL:
        	            case BINARY_SYMBOL:
        	            case OPTIMIZER_COSTS_SYMBOL:
        	            	{
        	            		alt575=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 575;
        	                EXCEPTION->state        = 0;


        	                goto ruleother_administrative_statementEx;

        	            }

        	            switch (alt575)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1610:4: flush_tables
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_flush_tables_in_other_administrative_statement9140);
        	        	        flush_tables(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1611:6: flush_option ( COMMA_SYMBOL flush_option )*
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_flush_option_in_other_administrative_statement9147);
        	        	        flush_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:1611:19: ( COMMA_SYMBOL flush_option )*

        	        	        for (;;)
        	        	        {
        	        	            int alt574=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	            case COMMA_SYMBOL:
        	        	            	{
        	        	            		alt574=1;
        	        	            	}
        	        	                break;

        	        	            }

        	        	            switch (alt574)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1611:20: COMMA_SYMBOL flush_option
        	        	        	    {
        	        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9150);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleother_administrative_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_flush_option_in_other_administrative_statement9152);
        	        	        	        flush_option(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleother_administrative_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	        	default:
        	        	        	    goto loop574;	/* break out of the loop */
        	        	        	    break;
        	        	            }
        	        	        }
        	        	        loop574: ; /* Jump out to here if this rule does not match */


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1613:4: KILL_SYMBOL ( options {greedy=true; } : ( CONNECTION_SYMBOL | QUERY_SYMBOL ) )? expression
        	    {
        	         MATCHT(KILL_SYMBOL, &FOLLOW_KILL_SYMBOL_in_other_administrative_statement9163);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1613:17: ( options {greedy=true; } : ( CONNECTION_SYMBOL | QUERY_SYMBOL ) )?
        	        {
        	            int alt576=2;
        	            switch ( LA(1) )
        	            {
        	                case CONNECTION_SYMBOL:
        	                case QUERY_SYMBOL:
        	                	{
        	                		{
        	                		    int LA576_1 = LA(2);
        	                		    if ( (LA576_1 == NOT_SYMBOL))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		    else if ( (LA576_1 == MINUS_OPERATOR || LA576_1 == PLUS_OPERATOR))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		    else if ( (((LA576_1 >= ACCOUNT_SYMBOL) && (LA576_1 <= SNAPSHOT_SYMBOL)) || ((LA576_1 >= SOURCE_SYMBOL) && (LA576_1 <= YEAR_SYMBOL)) || LA576_1 == BINARY_SYMBOL || ((LA576_1 >= CASE_SYMBOL) && (LA576_1 <= CAST_SYMBOL)) || LA576_1 == CHAR_SYMBOL || ((LA576_1 >= CONVERT_SYMBOL) && (LA576_1 <= COUNT_SYMBOL)) || LA576_1 == CURDATE_SYMBOL || LA576_1 == CURRENT_USER_SYMBOL || LA576_1 == CURTIME_SYMBOL || LA576_1 == DATABASE_SYMBOL || LA576_1 == DATE_ADD_SYMBOL || LA576_1 == DATE_SUB_SYMBOL || LA576_1 == EXISTS_SYMBOL || ((LA576_1 >= EXTRACT_SYMBOL) && (LA576_1 <= FALSE_SYMBOL)) || LA576_1 == GROUP_CONCAT_SYMBOL || LA576_1 == IF_SYMBOL || LA576_1 == INSERT_SYMBOL || LA576_1 == INTERVAL_SYMBOL || LA576_1 == LEFT_SYMBOL || LA576_1 == MATCH_SYMBOL || LA576_1 == MAX_SYMBOL || LA576_1 == MIN_SYMBOL || LA576_1 == NCHAR_TEXT || LA576_1 == NOT2_SYMBOL || LA576_1 == NOW_SYMBOL || ((LA576_1 >= NULL2_SYMBOL) && (LA576_1 <= NULL_SYMBOL)) || ((LA576_1 >= OPEN_CURLY_SYMBOL) && (LA576_1 <= OPEN_PAR_SYMBOL)) || ((LA576_1 >= PARAM_MARKER) && (LA576_1 <= POSITION_SYMBOL)) || ((LA576_1 >= REPEAT_SYMBOL) && (LA576_1 <= REPLACE_SYMBOL)) || LA576_1 == RIGHT_SYMBOL || LA576_1 == SINGLE_QUOTED_TEXT || LA576_1 == STDDEV_SAMP_SYMBOL || LA576_1 == STD_SYMBOL || LA576_1 == SUBSTRING_SYMBOL || ((LA576_1 >= SUM_SYMBOL) && (LA576_1 <= SYSDATE_SYMBOL)) || ((LA576_1 >= TRIM_SYMBOL) && (LA576_1 <= TRUE_SYMBOL)) || ((LA576_1 >= UTC_DATE_SYMBOL) && (LA576_1 <= UTC_TIME_SYMBOL)) || LA576_1 == VALUES_SYMBOL || LA576_1 == VARIANCE_SYMBOL || LA576_1 == VAR_SAMP_SYMBOL || ((LA576_1 >= AT_AT_SIGN_SYMBOL) && (LA576_1 <= AT_TEXT_SUFFIX)) || ((LA576_1 >= BACK_TICK_QUOTED_ID) && (LA576_1 <= BIN_NUMBER)) || LA576_1 == BITWISE_NOT_OPERATOR || ((LA576_1 >= BIT_AND_SYMBOL) && (LA576_1 <= BIT_XOR_SYMBOL)) || LA576_1 == DECIMAL_NUMBER || LA576_1 == DOUBLE_QUOTED_TEXT || LA576_1 == FLOAT_NUMBER || ((LA576_1 >= HEX_NUMBER) && (LA576_1 <= IDENTIFIER)) || LA576_1 == INT_NUMBER || LA576_1 == LOGICAL_NOT_OPERATOR || LA576_1 == LONG_NUMBER || ((LA576_1 >= ULONGLONG_NUMBER) && (LA576_1 <= UNDERSCORE_CHARSET))))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		    else if ( (LA576_1 == MOD_SYMBOL))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		    else if ( (LA576_1 == DOT_SYMBOL))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		    else if ( (LA576_1 == SOUNDS_SYMBOL))
        	                		    {
        	                		        alt576=1;
        	                		    }
        	                		}
        	                	}
        	                    break;
        	            }

        	            switch (alt576)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1613:46: ( CONNECTION_SYMBOL | QUERY_SYMBOL )
        	        	    {
        	        	        if ( LA(1) == CONNECTION_SYMBOL || LA(1) == QUERY_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto ruleother_administrative_statementEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_expression_in_other_administrative_statement9189);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1614:4: LOAD_SYMBOL INDEX_SYMBOL INTO_SYMBOL CACHE_SYMBOL load_table_index_list
        	    {
        	         MATCHT(LOAD_SYMBOL, &FOLLOW_LOAD_SYMBOL_in_other_administrative_statement9194);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_other_administrative_statement9196);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_other_administrative_statement9198);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CACHE_SYMBOL, &FOLLOW_CACHE_SYMBOL_in_other_administrative_statement9200);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_load_table_index_list_in_other_administrative_statement9202);
        	        load_table_index_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1615:4: RESET_SYMBOL reset_option ( COMMA_SYMBOL reset_option )*
        	    {
        	         MATCHT(RESET_SYMBOL, &FOLLOW_RESET_SYMBOL_in_other_administrative_statement9207);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_reset_option_in_other_administrative_statement9209);
        	        reset_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1615:30: ( COMMA_SYMBOL reset_option )*

        	        for (;;)
        	        {
        	            int alt577=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt577=1;
        	            	}
        	                break;

        	            }

        	            switch (alt577)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1615:31: COMMA_SYMBOL reset_option
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_other_administrative_statement9212);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_reset_option_in_other_administrative_statement9214);
        	        	        reset_option(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleother_administrative_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop577;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop577: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1616:4: {...}? SHUTDOWN_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50709)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50709";
        	                EXCEPTION->ruleName	 = (void *)"other_administrative_statement";


        	        }

        	         MATCHT(SHUTDOWN_SYMBOL, &FOLLOW_SHUTDOWN_SYMBOL_in_other_administrative_statement9223);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleother_administrative_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleother_administrative_statementEx; /* Prevent compiler warnings */
    ruleother_administrative_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end other_administrative_statement */

/**
 * $ANTLR start key_cache_list_or_parts
 * MySQLSimpleParser.g:1619:1: key_cache_list_or_parts options {k=4; } : ( key_cache_list | assign_to_keycache_partition );
 */
static void
key_cache_list_or_parts(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1619:43: ( key_cache_list | assign_to_keycache_partition )

            ANTLR3_UINT32 alt579;

            alt579=2;

            {
                int LA579_0 = LA(1);
                if ( (LA579_0 == IDENTIFIER))
                {
                    switch ( LA(2) )
                    {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA579_7 = LA(3);
                    		    if ( (LA579_7 == IDENTIFIER))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 37;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_7 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 38;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_7 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 39;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_7 >= ACTION_SYMBOL) && (LA579_7 <= SHARE_SYMBOL)) || ((LA579_7 >= SLOW_SYMBOL) && (LA579_7 <= YEAR_SYMBOL))))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 40;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_7 >= ACCOUNT_SYMBOL) && (LA579_7 <= XA_SYMBOL)) || LA579_7 == SHUTDOWN_SYMBOL))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 41;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 579;
                    		        EXCEPTION->state        = 7;


                    		        goto rulekey_cache_list_or_partsEx;

                    		    }
                    		}
                    	}
                        break;
                    case INDEX_SYMBOL:
                    case IN_SYMBOL:
                    case KEY_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt579=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt579=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 579;
                        EXCEPTION->state        = 1;


                        goto rulekey_cache_list_or_partsEx;

                    }

                }
                else if ( (LA579_0 == BACK_TICK_QUOTED_ID))
                {
                    switch ( LA(2) )
                    {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA579_12 = LA(3);
                    		    if ( (LA579_12 == IDENTIFIER))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 42;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_12 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 43;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_12 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 44;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_12 >= ACTION_SYMBOL) && (LA579_12 <= SHARE_SYMBOL)) || ((LA579_12 >= SLOW_SYMBOL) && (LA579_12 <= YEAR_SYMBOL))))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 45;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_12 >= ACCOUNT_SYMBOL) && (LA579_12 <= XA_SYMBOL)) || LA579_12 == SHUTDOWN_SYMBOL))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 46;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 579;
                    		        EXCEPTION->state        = 12;


                    		        goto rulekey_cache_list_or_partsEx;

                    		    }
                    		}
                    	}
                        break;
                    case INDEX_SYMBOL:
                    case IN_SYMBOL:
                    case KEY_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt579=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt579=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 579;
                        EXCEPTION->state        = 2;


                        goto rulekey_cache_list_or_partsEx;

                    }

                }
                else if ( (LA579_0 == DOUBLE_QUOTED_TEXT))
                {
                    switch ( LA(2) )
                    {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA579_17 = LA(3);
                    		    if ( (LA579_17 == IDENTIFIER))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 47;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_17 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 48;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_17 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 49;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_17 >= ACTION_SYMBOL) && (LA579_17 <= SHARE_SYMBOL)) || ((LA579_17 >= SLOW_SYMBOL) && (LA579_17 <= YEAR_SYMBOL))))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 50;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_17 >= ACCOUNT_SYMBOL) && (LA579_17 <= XA_SYMBOL)) || LA579_17 == SHUTDOWN_SYMBOL))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 51;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 579;
                    		        EXCEPTION->state        = 17;


                    		        goto rulekey_cache_list_or_partsEx;

                    		    }
                    		}
                    	}
                        break;
                    case INDEX_SYMBOL:
                    case IN_SYMBOL:
                    case KEY_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt579=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt579=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 579;
                        EXCEPTION->state        = 3;


                        goto rulekey_cache_list_or_partsEx;

                    }

                }
                else if ( (((LA579_0 >= ACTION_SYMBOL) && (LA579_0 <= SHARE_SYMBOL)) || ((LA579_0 >= SLOW_SYMBOL) && (LA579_0 <= YEAR_SYMBOL))))
                {
                    switch ( LA(2) )
                    {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA579_22 = LA(3);
                    		    if ( (LA579_22 == IDENTIFIER))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 52;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_22 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 53;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_22 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 54;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_22 >= ACTION_SYMBOL) && (LA579_22 <= SHARE_SYMBOL)) || ((LA579_22 >= SLOW_SYMBOL) && (LA579_22 <= YEAR_SYMBOL))))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 55;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_22 >= ACCOUNT_SYMBOL) && (LA579_22 <= XA_SYMBOL)) || LA579_22 == SHUTDOWN_SYMBOL))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 56;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 579;
                    		        EXCEPTION->state        = 22;


                    		        goto rulekey_cache_list_or_partsEx;

                    		    }
                    		}
                    	}
                        break;
                    case INDEX_SYMBOL:
                    case IN_SYMBOL:
                    case KEY_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt579=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt579=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 579;
                        EXCEPTION->state        = 4;


                        goto rulekey_cache_list_or_partsEx;

                    }

                }
                else if ( (((LA579_0 >= ACCOUNT_SYMBOL) && (LA579_0 <= XA_SYMBOL)) || LA579_0 == SHUTDOWN_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA579_27 = LA(3);
                    		    if ( (LA579_27 == IDENTIFIER))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 57;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_27 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 58;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (LA579_27 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 59;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_27 >= ACTION_SYMBOL) && (LA579_27 <= SHARE_SYMBOL)) || ((LA579_27 >= SLOW_SYMBOL) && (LA579_27 <= YEAR_SYMBOL))))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 60;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else if ( (((LA579_27 >= ACCOUNT_SYMBOL) && (LA579_27 <= XA_SYMBOL)) || LA579_27 == SHUTDOWN_SYMBOL))
                    		    {
                    		        switch ( LA(4) )
                    		        {
                    		        case INDEX_SYMBOL:
                    		        case IN_SYMBOL:
                    		        case KEY_SYMBOL:
                    		        case COMMA_SYMBOL:
                    		        	{
                    		        		alt579=1;
                    		        	}
                    		            break;
                    		        case PARTITION_SYMBOL:
                    		        	{
                    		        		alt579=2;
                    		        	}
                    		            break;

                    		        default:
                    		            if (BACKTRACKING>0)
                    		            {
                    		                FAILEDFLAG = ANTLR3_TRUE;
                    		                return ;
                    		            }

                    		            CONSTRUCTEX();
                    		            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		            EXCEPTION->message      = (void *)"";
                    		            EXCEPTION->decisionNum  = 579;
                    		            EXCEPTION->state        = 61;


                    		            goto rulekey_cache_list_or_partsEx;

                    		        }

                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 579;
                    		        EXCEPTION->state        = 27;


                    		        goto rulekey_cache_list_or_partsEx;

                    		    }
                    		}
                    	}
                        break;
                    case INDEX_SYMBOL:
                    case IN_SYMBOL:
                    case KEY_SYMBOL:
                    case COMMA_SYMBOL:
                    	{
                    		alt579=1;
                    	}
                        break;
                    case PARTITION_SYMBOL:
                    	{
                    		alt579=2;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 579;
                        EXCEPTION->state        = 5;


                        goto rulekey_cache_list_or_partsEx;

                    }

                }
                else if ( (LA579_0 == DOT_SYMBOL))
                {
                    {
                        int LA579_6 = LA(2);
                        if ( (LA579_6 == IDENTIFIER))
                        {
                            switch ( LA(3) )
                            {
                            case INDEX_SYMBOL:
                            case IN_SYMBOL:
                            case KEY_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt579=1;
                            	}
                                break;
                            case PARTITION_SYMBOL:
                            	{
                            		alt579=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 579;
                                EXCEPTION->state        = 32;


                                goto rulekey_cache_list_or_partsEx;

                            }

                        }
                        else if ( (LA579_6 == BACK_TICK_QUOTED_ID))
                        {
                            switch ( LA(3) )
                            {
                            case INDEX_SYMBOL:
                            case IN_SYMBOL:
                            case KEY_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt579=1;
                            	}
                                break;
                            case PARTITION_SYMBOL:
                            	{
                            		alt579=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 579;
                                EXCEPTION->state        = 33;


                                goto rulekey_cache_list_or_partsEx;

                            }

                        }
                        else if ( (LA579_6 == DOUBLE_QUOTED_TEXT))
                        {
                            switch ( LA(3) )
                            {
                            case INDEX_SYMBOL:
                            case IN_SYMBOL:
                            case KEY_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt579=1;
                            	}
                                break;
                            case PARTITION_SYMBOL:
                            	{
                            		alt579=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 579;
                                EXCEPTION->state        = 34;


                                goto rulekey_cache_list_or_partsEx;

                            }

                        }
                        else if ( (((LA579_6 >= ACTION_SYMBOL) && (LA579_6 <= SHARE_SYMBOL)) || ((LA579_6 >= SLOW_SYMBOL) && (LA579_6 <= YEAR_SYMBOL))))
                        {
                            switch ( LA(3) )
                            {
                            case INDEX_SYMBOL:
                            case IN_SYMBOL:
                            case KEY_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt579=1;
                            	}
                                break;
                            case PARTITION_SYMBOL:
                            	{
                            		alt579=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 579;
                                EXCEPTION->state        = 35;


                                goto rulekey_cache_list_or_partsEx;

                            }

                        }
                        else if ( (((LA579_6 >= ACCOUNT_SYMBOL) && (LA579_6 <= XA_SYMBOL)) || LA579_6 == SHUTDOWN_SYMBOL))
                        {
                            switch ( LA(3) )
                            {
                            case INDEX_SYMBOL:
                            case IN_SYMBOL:
                            case KEY_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt579=1;
                            	}
                                break;
                            case PARTITION_SYMBOL:
                            	{
                            		alt579=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 579;
                                EXCEPTION->state        = 36;


                                goto rulekey_cache_list_or_partsEx;

                            }

                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 579;
                            EXCEPTION->state        = 6;


                            goto rulekey_cache_list_or_partsEx;

                        }
                    }
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 579;
                    EXCEPTION->state        = 0;


                    goto rulekey_cache_list_or_partsEx;

                }
            }
            switch (alt579)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1620:2: key_cache_list
        	    {
        	        FOLLOWPUSH(FOLLOW_key_cache_list_in_key_cache_list_or_parts9243);
        	        key_cache_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_cache_list_or_partsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1621:4: assign_to_keycache_partition
        	    {
        	        FOLLOWPUSH(FOLLOW_assign_to_keycache_partition_in_key_cache_list_or_parts9248);
        	        assign_to_keycache_partition(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_cache_list_or_partsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulekey_cache_list_or_partsEx; /* Prevent compiler warnings */
    rulekey_cache_list_or_partsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end key_cache_list_or_parts */

/**
 * $ANTLR start key_cache_list
 * MySQLSimpleParser.g:1624:1: key_cache_list : assign_to_keycache ( COMMA_SYMBOL assign_to_keycache )* ;
 */
static void
key_cache_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1624:15: ( assign_to_keycache ( COMMA_SYMBOL assign_to_keycache )* )
        // MySQLSimpleParser.g:1625:2: assign_to_keycache ( COMMA_SYMBOL assign_to_keycache )*
        {
            FOLLOWPUSH(FOLLOW_assign_to_keycache_in_key_cache_list9257);
            assign_to_keycache(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulekey_cache_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1625:21: ( COMMA_SYMBOL assign_to_keycache )*

            for (;;)
            {
                int alt580=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt580=1;
                	}
                    break;

                }

                switch (alt580)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1625:22: COMMA_SYMBOL assign_to_keycache
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_key_cache_list9260);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulekey_cache_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_assign_to_keycache_in_key_cache_list9262);
            	        assign_to_keycache(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulekey_cache_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop580;	/* break out of the loop */
            	    break;
                }
            }
            loop580: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulekey_cache_listEx; /* Prevent compiler warnings */
    rulekey_cache_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end key_cache_list */

/**
 * $ANTLR start assign_to_keycache
 * MySQLSimpleParser.g:1628:1: assign_to_keycache : table_ref ( cache_keys_spec )? ;
 */
static void
assign_to_keycache(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1628:19: ( table_ref ( cache_keys_spec )? )
        // MySQLSimpleParser.g:1629:2: table_ref ( cache_keys_spec )?
        {
            FOLLOWPUSH(FOLLOW_table_ref_in_assign_to_keycache9273);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleassign_to_keycacheEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1629:12: ( cache_keys_spec )?
            {
                int alt581=2;
                switch ( LA(1) )
                {
                    case INDEX_SYMBOL:
                    case KEY_SYMBOL:
                    	{
                    		alt581=1;
                    	}
                        break;
                }

                switch (alt581)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1629:12: cache_keys_spec
            	    {
            	        FOLLOWPUSH(FOLLOW_cache_keys_spec_in_assign_to_keycache9275);
            	        cache_keys_spec(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleassign_to_keycacheEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleassign_to_keycacheEx; /* Prevent compiler warnings */
    ruleassign_to_keycacheEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end assign_to_keycache */

/**
 * $ANTLR start assign_to_keycache_partition
 * MySQLSimpleParser.g:1632:1: assign_to_keycache_partition : table_ref PARTITION_SYMBOL OPEN_PAR_SYMBOL ( ALL_SYMBOL | identifier_list ) CLOSE_PAR_SYMBOL ( cache_keys_spec )? ;
 */
static void
assign_to_keycache_partition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1632:29: ( table_ref PARTITION_SYMBOL OPEN_PAR_SYMBOL ( ALL_SYMBOL | identifier_list ) CLOSE_PAR_SYMBOL ( cache_keys_spec )? )
        // MySQLSimpleParser.g:1633:2: table_ref PARTITION_SYMBOL OPEN_PAR_SYMBOL ( ALL_SYMBOL | identifier_list ) CLOSE_PAR_SYMBOL ( cache_keys_spec )?
        {
            FOLLOWPUSH(FOLLOW_table_ref_in_assign_to_keycache_partition9285);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleassign_to_keycache_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_assign_to_keycache_partition9287);
            if  (HASEXCEPTION())
            {
                goto ruleassign_to_keycache_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_assign_to_keycache_partition9289);
            if  (HASEXCEPTION())
            {
                goto ruleassign_to_keycache_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1633:45: ( ALL_SYMBOL | identifier_list )
            {
                int alt582=2;
                {
                    int LA582_0 = LA(1);
                    if ( (LA582_0 == ALL_SYMBOL))
                    {
                        alt582=1;
                    }
                    else if ( (((LA582_0 >= ACCOUNT_SYMBOL) && (LA582_0 <= YEAR_SYMBOL)) || LA582_0 == BACK_TICK_QUOTED_ID || LA582_0 == DOUBLE_QUOTED_TEXT || LA582_0 == IDENTIFIER))
                    {
                        alt582=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 582;
                        EXCEPTION->state        = 0;


                        goto ruleassign_to_keycache_partitionEx;

                    }
                }
                switch (alt582)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1633:46: ALL_SYMBOL
            	    {
            	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_assign_to_keycache_partition9292);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleassign_to_keycache_partitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1633:59: identifier_list
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_list_in_assign_to_keycache_partition9296);
            	        identifier_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleassign_to_keycache_partitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_assign_to_keycache_partition9299);
            if  (HASEXCEPTION())
            {
                goto ruleassign_to_keycache_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1633:93: ( cache_keys_spec )?
            {
                int alt583=2;
                switch ( LA(1) )
                {
                    case INDEX_SYMBOL:
                    case KEY_SYMBOL:
                    	{
                    		alt583=1;
                    	}
                        break;
                }

                switch (alt583)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1633:93: cache_keys_spec
            	    {
            	        FOLLOWPUSH(FOLLOW_cache_keys_spec_in_assign_to_keycache_partition9301);
            	        cache_keys_spec(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleassign_to_keycache_partitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleassign_to_keycache_partitionEx; /* Prevent compiler warnings */
    ruleassign_to_keycache_partitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end assign_to_keycache_partition */

/**
 * $ANTLR start cache_keys_spec
 * MySQLSimpleParser.g:1636:1: cache_keys_spec : ( KEY_SYMBOL | INDEX_SYMBOL ) OPEN_PAR_SYMBOL ( key_usage_element ( COMMA_SYMBOL key_usage_element )* )? CLOSE_PAR_SYMBOL ;
 */
static void
cache_keys_spec(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1636:16: ( ( KEY_SYMBOL | INDEX_SYMBOL ) OPEN_PAR_SYMBOL ( key_usage_element ( COMMA_SYMBOL key_usage_element )* )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:1637:2: ( KEY_SYMBOL | INDEX_SYMBOL ) OPEN_PAR_SYMBOL ( key_usage_element ( COMMA_SYMBOL key_usage_element )* )? CLOSE_PAR_SYMBOL
        {
            if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulecache_keys_specEx;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_cache_keys_spec9319);
            if  (HASEXCEPTION())
            {
                goto rulecache_keys_specEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1637:46: ( key_usage_element ( COMMA_SYMBOL key_usage_element )* )?
            {
                int alt585=2;
                {
                    int LA585_0 = LA(1);
                    if ( (((LA585_0 >= ACCOUNT_SYMBOL) && (LA585_0 <= YEAR_SYMBOL)) || LA585_0 == PRIMARY_SYMBOL || LA585_0 == BACK_TICK_QUOTED_ID || LA585_0 == DOUBLE_QUOTED_TEXT || LA585_0 == IDENTIFIER))
                    {
                        alt585=1;
                    }
                }
                switch (alt585)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1637:47: key_usage_element ( COMMA_SYMBOL key_usage_element )*
            	    {
            	        FOLLOWPUSH(FOLLOW_key_usage_element_in_cache_keys_spec9322);
            	        key_usage_element(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecache_keys_specEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1637:65: ( COMMA_SYMBOL key_usage_element )*

            	        for (;;)
            	        {
            	            int alt584=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt584=1;
            	            	}
            	                break;

            	            }

            	            switch (alt584)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1637:66: COMMA_SYMBOL key_usage_element
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_cache_keys_spec9325);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecache_keys_specEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_key_usage_element_in_cache_keys_spec9327);
            	        	        key_usage_element(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecache_keys_specEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop584;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop584: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_cache_keys_spec9333);
            if  (HASEXCEPTION())
            {
                goto rulecache_keys_specEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecache_keys_specEx; /* Prevent compiler warnings */
    rulecache_keys_specEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cache_keys_spec */

/**
 * $ANTLR start key_usage_element
 * MySQLSimpleParser.g:1640:1: key_usage_element : ( identifier | PRIMARY_SYMBOL );
 */
static void
key_usage_element(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1640:18: ( identifier | PRIMARY_SYMBOL )

            ANTLR3_UINT32 alt586;

            alt586=2;

            {
                int LA586_0 = LA(1);
                if ( (((LA586_0 >= ACCOUNT_SYMBOL) && (LA586_0 <= YEAR_SYMBOL)) || LA586_0 == BACK_TICK_QUOTED_ID || LA586_0 == DOUBLE_QUOTED_TEXT || LA586_0 == IDENTIFIER))
                {
                    alt586=1;
                }
                else if ( (LA586_0 == PRIMARY_SYMBOL))
                {
                    alt586=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 586;
                    EXCEPTION->state        = 0;


                    goto rulekey_usage_elementEx;

                }
            }
            switch (alt586)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1641:2: identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_key_usage_element9342);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_usage_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1642:4: PRIMARY_SYMBOL
        	    {
        	         MATCHT(PRIMARY_SYMBOL, &FOLLOW_PRIMARY_SYMBOL_in_key_usage_element9347);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekey_usage_elementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulekey_usage_elementEx; /* Prevent compiler warnings */
    rulekey_usage_elementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end key_usage_element */

/**
 * $ANTLR start flush_option
 * MySQLSimpleParser.g:1645:1: flush_option : ( DES_KEY_FILE_SYMBOL | HOSTS_SYMBOL | ( log_type )? LOGS_SYMBOL | RELAY_SYMBOL LOGS_SYMBOL ( channel )? | PRIVILEGES_SYMBOL | QUERY_SYMBOL CACHE_SYMBOL | STATUS_SYMBOL | USER_RESOURCES_SYMBOL |{...}? OPTIMIZER_COSTS_SYMBOL );
 */
static void
flush_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1645:13: ( DES_KEY_FILE_SYMBOL | HOSTS_SYMBOL | ( log_type )? LOGS_SYMBOL | RELAY_SYMBOL LOGS_SYMBOL ( channel )? | PRIVILEGES_SYMBOL | QUERY_SYMBOL CACHE_SYMBOL | STATUS_SYMBOL | USER_RESOURCES_SYMBOL |{...}? OPTIMIZER_COSTS_SYMBOL )

            ANTLR3_UINT32 alt589;

            alt589=9;

            switch ( LA(1) )
            {
            case DES_KEY_FILE_SYMBOL:
            	{
            		alt589=1;
            	}
                break;
            case HOSTS_SYMBOL:
            	{
            		alt589=2;
            	}
                break;
            case ENGINE_SYMBOL:
            case ERROR_SYMBOL:
            case GENERAL_SYMBOL:
            case LOGS_SYMBOL:
            case SLOW_SYMBOL:
            case BINARY_SYMBOL:
            	{
            		alt589=3;
            	}
                break;
            case RELAY_SYMBOL:
            	{
            		alt589=4;
            	}
                break;
            case PRIVILEGES_SYMBOL:
            	{
            		alt589=5;
            	}
                break;
            case QUERY_SYMBOL:
            	{
            		alt589=6;
            	}
                break;
            case STATUS_SYMBOL:
            	{
            		alt589=7;
            	}
                break;
            case USER_RESOURCES_SYMBOL:
            	{
            		alt589=8;
            	}
                break;
            case OPTIMIZER_COSTS_SYMBOL:
            	{
            		alt589=9;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 589;
                EXCEPTION->state        = 0;


                goto ruleflush_optionEx;

            }

            switch (alt589)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1646:2: DES_KEY_FILE_SYMBOL
        	    {
        	         MATCHT(DES_KEY_FILE_SYMBOL, &FOLLOW_DES_KEY_FILE_SYMBOL_in_flush_option9356);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1647:4: HOSTS_SYMBOL
        	    {
        	         MATCHT(HOSTS_SYMBOL, &FOLLOW_HOSTS_SYMBOL_in_flush_option9361);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1648:4: ( log_type )? LOGS_SYMBOL
        	    {
        	        // MySQLSimpleParser.g:1648:4: ( log_type )?
        	        {
        	            int alt587=2;
        	            switch ( LA(1) )
        	            {
        	                case ENGINE_SYMBOL:
        	                case ERROR_SYMBOL:
        	                case GENERAL_SYMBOL:
        	                case SLOW_SYMBOL:
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt587=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt587)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1648:4: log_type
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_log_type_in_flush_option9366);
        	        	        log_type(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleflush_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(LOGS_SYMBOL, &FOLLOW_LOGS_SYMBOL_in_flush_option9369);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1649:4: RELAY_SYMBOL LOGS_SYMBOL ( channel )?
        	    {
        	         MATCHT(RELAY_SYMBOL, &FOLLOW_RELAY_SYMBOL_in_flush_option9374);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(LOGS_SYMBOL, &FOLLOW_LOGS_SYMBOL_in_flush_option9376);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1649:29: ( channel )?
        	        {
        	            int alt588=2;
        	            {
        	                int LA588_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50706)) && (LA588_0 == FOR_SYMBOL))
        	                {
        	                    alt588=1;
        	                }
        	            }
        	            switch (alt588)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1649:29: channel
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_channel_in_flush_option9378);
        	        	        channel(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleflush_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1650:4: PRIVILEGES_SYMBOL
        	    {
        	         MATCHT(PRIVILEGES_SYMBOL, &FOLLOW_PRIVILEGES_SYMBOL_in_flush_option9384);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1651:4: QUERY_SYMBOL CACHE_SYMBOL
        	    {
        	         MATCHT(QUERY_SYMBOL, &FOLLOW_QUERY_SYMBOL_in_flush_option9389);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CACHE_SYMBOL, &FOLLOW_CACHE_SYMBOL_in_flush_option9391);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1652:4: STATUS_SYMBOL
        	    {
        	         MATCHT(STATUS_SYMBOL, &FOLLOW_STATUS_SYMBOL_in_flush_option9396);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1653:4: USER_RESOURCES_SYMBOL
        	    {
        	         MATCHT(USER_RESOURCES_SYMBOL, &FOLLOW_USER_RESOURCES_SYMBOL_in_flush_option9401);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:1654:4: {...}? OPTIMIZER_COSTS_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
        	                EXCEPTION->ruleName	 = (void *)"flush_option";


        	        }

        	         MATCHT(OPTIMIZER_COSTS_SYMBOL, &FOLLOW_OPTIMIZER_COSTS_SYMBOL_in_flush_option9408);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleflush_optionEx; /* Prevent compiler warnings */
    ruleflush_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end flush_option */

/**
 * $ANTLR start log_type
 * MySQLSimpleParser.g:1657:1: log_type : ( BINARY_SYMBOL | ENGINE_SYMBOL | ERROR_SYMBOL | GENERAL_SYMBOL | SLOW_SYMBOL );
 */
static void
log_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1657:9: ( BINARY_SYMBOL | ENGINE_SYMBOL | ERROR_SYMBOL | GENERAL_SYMBOL | SLOW_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == ENGINE_SYMBOL || LA(1) == ERROR_SYMBOL || LA(1) == GENERAL_SYMBOL || LA(1) == SLOW_SYMBOL || LA(1) == BINARY_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulelog_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelog_typeEx; /* Prevent compiler warnings */
    rulelog_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end log_type */

/**
 * $ANTLR start flush_tables
 * MySQLSimpleParser.g:1665:1: flush_tables : ( TABLES_SYMBOL | TABLE_SYMBOL ) ( WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL | identifier_list ( flush_tables_options )? )? ;
 */
static void
flush_tables(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1665:13: ( ( TABLES_SYMBOL | TABLE_SYMBOL ) ( WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL | identifier_list ( flush_tables_options )? )? )
        // MySQLSimpleParser.g:1666:2: ( TABLES_SYMBOL | TABLE_SYMBOL ) ( WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL | identifier_list ( flush_tables_options )? )?
        {
            if ( LA(1) == TABLES_SYMBOL || LA(1) == TABLE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleflush_tablesEx;
            }


            // MySQLSimpleParser.g:1667:2: ( WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL | identifier_list ( flush_tables_options )? )?
            {
                int alt591=3;
                {
                    int LA591_0 = LA(1);
                    if ( (LA591_0 == WITH_SYMBOL))
                    {
                        alt591=1;
                    }
                    else if ( (((LA591_0 >= ACCOUNT_SYMBOL) && (LA591_0 <= YEAR_SYMBOL)) || LA591_0 == BACK_TICK_QUOTED_ID || LA591_0 == DOUBLE_QUOTED_TEXT || LA591_0 == IDENTIFIER))
                    {
                        alt591=2;
                    }
                }
                switch (alt591)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1668:3: WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL
            	    {
            	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_flush_tables9459);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleflush_tablesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_flush_tables9461);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleflush_tablesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_flush_tables9463);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleflush_tablesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1669:5: identifier_list ( flush_tables_options )?
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_list_in_flush_tables9469);
            	        identifier_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleflush_tablesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1669:21: ( flush_tables_options )?
            	        {
            	            int alt590=2;
            	            switch ( LA(1) )
            	            {
            	                case FOR_SYMBOL:
            	                case WITH_SYMBOL:
            	                	{
            	                		alt590=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt590)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1669:21: flush_tables_options
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_flush_tables_options_in_flush_tables9471);
            	        	        flush_tables_options(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleflush_tablesEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleflush_tablesEx; /* Prevent compiler warnings */
    ruleflush_tablesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end flush_tables */

/**
 * $ANTLR start flush_tables_options
 * MySQLSimpleParser.g:1673:1: flush_tables_options : ({...}? FOR_SYMBOL EXPORT_SYMBOL | WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL );
 */
static void
flush_tables_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1673:21: ({...}? FOR_SYMBOL EXPORT_SYMBOL | WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL )

            ANTLR3_UINT32 alt592;

            alt592=2;

            switch ( LA(1) )
            {
            case FOR_SYMBOL:
            	{
            		alt592=1;
            	}
                break;
            case WITH_SYMBOL:
            	{
            		alt592=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 592;
                EXCEPTION->state        = 0;


                goto ruleflush_tables_optionsEx;

            }

            switch (alt592)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1674:2: {...}? FOR_SYMBOL EXPORT_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50606)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50606";
        	                EXCEPTION->ruleName	 = (void *)"flush_tables_options";


        	        }

        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_flush_tables_options9487);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_tables_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(EXPORT_SYMBOL, &FOLLOW_EXPORT_SYMBOL_in_flush_tables_options9489);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_tables_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1675:4: WITH_SYMBOL READ_SYMBOL LOCK_SYMBOL
        	    {
        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_flush_tables_options9494);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_tables_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(READ_SYMBOL, &FOLLOW_READ_SYMBOL_in_flush_tables_options9496);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_tables_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(LOCK_SYMBOL, &FOLLOW_LOCK_SYMBOL_in_flush_tables_options9498);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleflush_tables_optionsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleflush_tables_optionsEx; /* Prevent compiler warnings */
    ruleflush_tables_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end flush_tables_options */

/**
 * $ANTLR start load_table_index_list
 * MySQLSimpleParser.g:1678:1: load_table_index_list : table_ref ( load_table_index_partion )? ( ( INDEX_SYMBOL | KEY_SYMBOL )? identifier_list_with_parentheses )? ( IGNORE_SYMBOL LEAVES_SYMBOL )? ;
 */
static void
load_table_index_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1678:22: ( table_ref ( load_table_index_partion )? ( ( INDEX_SYMBOL | KEY_SYMBOL )? identifier_list_with_parentheses )? ( IGNORE_SYMBOL LEAVES_SYMBOL )? )
        // MySQLSimpleParser.g:1679:2: table_ref ( load_table_index_partion )? ( ( INDEX_SYMBOL | KEY_SYMBOL )? identifier_list_with_parentheses )? ( IGNORE_SYMBOL LEAVES_SYMBOL )?
        {
            FOLLOWPUSH(FOLLOW_table_ref_in_load_table_index_list9507);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleload_table_index_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1679:12: ( load_table_index_partion )?
            {
                int alt593=2;
                {
                    int LA593_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50500)) && (LA593_0 == PARTITION_SYMBOL))
                    {
                        alt593=1;
                    }
                }
                switch (alt593)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1679:12: load_table_index_partion
            	    {
            	        FOLLOWPUSH(FOLLOW_load_table_index_partion_in_load_table_index_list9509);
            	        load_table_index_partion(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_table_index_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1680:3: ( ( INDEX_SYMBOL | KEY_SYMBOL )? identifier_list_with_parentheses )?
            {
                int alt595=2;
                switch ( LA(1) )
                {
                    case INDEX_SYMBOL:
                    case KEY_SYMBOL:
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt595=1;
                    	}
                        break;
                }

                switch (alt595)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1680:4: ( INDEX_SYMBOL | KEY_SYMBOL )? identifier_list_with_parentheses
            	    {
            	        // MySQLSimpleParser.g:1680:4: ( INDEX_SYMBOL | KEY_SYMBOL )?
            	        {
            	            int alt594=2;
            	            switch ( LA(1) )
            	            {
            	                case INDEX_SYMBOL:
            	                case KEY_SYMBOL:
            	                	{
            	                		alt594=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt594)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:
            	        	    {
            	        	        if ( LA(1) == INDEX_SYMBOL || LA(1) == KEY_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto ruleload_table_index_listEx;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_load_table_index_list9524);
            	        identifier_list_with_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_table_index_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1680:68: ( IGNORE_SYMBOL LEAVES_SYMBOL )?
            {
                int alt596=2;
                switch ( LA(1) )
                {
                    case IGNORE_SYMBOL:
                    	{
                    		alt596=1;
                    	}
                        break;
                }

                switch (alt596)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1680:69: IGNORE_SYMBOL LEAVES_SYMBOL
            	    {
            	         MATCHT(IGNORE_SYMBOL, &FOLLOW_IGNORE_SYMBOL_in_load_table_index_list9529);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_table_index_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(LEAVES_SYMBOL, &FOLLOW_LEAVES_SYMBOL_in_load_table_index_list9531);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleload_table_index_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleload_table_index_listEx; /* Prevent compiler warnings */
    ruleload_table_index_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end load_table_index_list */

/**
 * $ANTLR start load_table_index_partion
 * MySQLSimpleParser.g:1683:1: load_table_index_partion :{...}? => ( PARTITION_SYMBOL OPEN_PAR_SYMBOL ( identifier_list | ALL_SYMBOL ) CLOSE_PAR_SYMBOL ) ;
 */
static void
load_table_index_partion(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1683:25: ({...}? => ( PARTITION_SYMBOL OPEN_PAR_SYMBOL ( identifier_list | ALL_SYMBOL ) CLOSE_PAR_SYMBOL ) )
        // MySQLSimpleParser.g:1684:2: {...}? => ( PARTITION_SYMBOL OPEN_PAR_SYMBOL ( identifier_list | ALL_SYMBOL ) CLOSE_PAR_SYMBOL )
        {
            if ( !((SERVER_VERSION >= 50500)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
                    EXCEPTION->ruleName	 = (void *)"load_table_index_partion";


            }

            // MySQLSimpleParser.g:1684:32: ( PARTITION_SYMBOL OPEN_PAR_SYMBOL ( identifier_list | ALL_SYMBOL ) CLOSE_PAR_SYMBOL )
            // MySQLSimpleParser.g:1684:33: PARTITION_SYMBOL OPEN_PAR_SYMBOL ( identifier_list | ALL_SYMBOL ) CLOSE_PAR_SYMBOL
            {
                 MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_load_table_index_partion9547);
                if  (HASEXCEPTION())
                {
                    goto ruleload_table_index_partionEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


                 MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_load_table_index_partion9549);
                if  (HASEXCEPTION())
                {
                    goto ruleload_table_index_partionEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


                // MySQLSimpleParser.g:1684:66: ( identifier_list | ALL_SYMBOL )
                {
                    int alt597=2;
                    {
                        int LA597_0 = LA(1);
                        if ( (((LA597_0 >= ACCOUNT_SYMBOL) && (LA597_0 <= YEAR_SYMBOL)) || LA597_0 == BACK_TICK_QUOTED_ID || LA597_0 == DOUBLE_QUOTED_TEXT || LA597_0 == IDENTIFIER))
                        {
                            alt597=1;
                        }
                        else if ( (LA597_0 == ALL_SYMBOL))
                        {
                            alt597=2;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 597;
                            EXCEPTION->state        = 0;


                            goto ruleload_table_index_partionEx;

                        }
                    }
                    switch (alt597)
                    {
                	case 1:
                	    // MySQLSimpleParser.g:1684:67: identifier_list
                	    {
                	        FOLLOWPUSH(FOLLOW_identifier_list_in_load_table_index_partion9552);
                	        identifier_list(ctx);

                	        FOLLOWPOP();
                	        if  (HASEXCEPTION())
                	        {
                	            goto ruleload_table_index_partionEx;
                	        }
                	        if (HASFAILED())
                	        {
                	            return ;
                	        }


                	    }
                	    break;
                	case 2:
                	    // MySQLSimpleParser.g:1684:85: ALL_SYMBOL
                	    {
                	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_load_table_index_partion9556);
                	        if  (HASEXCEPTION())
                	        {
                	            goto ruleload_table_index_partionEx;
                	        }
                	        if (HASFAILED())
                	        {
                	            return ;
                	        }


                	    }
                	    break;

                    }
                }

                 MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_load_table_index_partion9559);
                if  (HASEXCEPTION())
                {
                    goto ruleload_table_index_partionEx;
                }
                if (HASFAILED())
                {
                    return ;
                }


            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleload_table_index_partionEx; /* Prevent compiler warnings */
    ruleload_table_index_partionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end load_table_index_partion */

/**
 * $ANTLR start reset_option
 * MySQLSimpleParser.g:1687:1: reset_option : ( MASTER_SYMBOL | QUERY_SYMBOL CACHE_SYMBOL | SLAVE_SYMBOL ( ALL_SYMBOL )? ( channel )? );
 */
static void
reset_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1687:13: ( MASTER_SYMBOL | QUERY_SYMBOL CACHE_SYMBOL | SLAVE_SYMBOL ( ALL_SYMBOL )? ( channel )? )

            ANTLR3_UINT32 alt600;

            alt600=3;

            switch ( LA(1) )
            {
            case MASTER_SYMBOL:
            	{
            		alt600=1;
            	}
                break;
            case QUERY_SYMBOL:
            	{
            		alt600=2;
            	}
                break;
            case SLAVE_SYMBOL:
            	{
            		alt600=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 600;
                EXCEPTION->state        = 0;


                goto rulereset_optionEx;

            }

            switch (alt600)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1688:2: MASTER_SYMBOL
        	    {
        	         MATCHT(MASTER_SYMBOL, &FOLLOW_MASTER_SYMBOL_in_reset_option9569);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereset_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1689:4: QUERY_SYMBOL CACHE_SYMBOL
        	    {
        	         MATCHT(QUERY_SYMBOL, &FOLLOW_QUERY_SYMBOL_in_reset_option9574);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereset_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CACHE_SYMBOL, &FOLLOW_CACHE_SYMBOL_in_reset_option9576);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereset_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1690:4: SLAVE_SYMBOL ( ALL_SYMBOL )? ( channel )?
        	    {
        	         MATCHT(SLAVE_SYMBOL, &FOLLOW_SLAVE_SYMBOL_in_reset_option9581);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereset_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1690:17: ( ALL_SYMBOL )?
        	        {
        	            int alt598=2;
        	            switch ( LA(1) )
        	            {
        	                case ALL_SYMBOL:
        	                	{
        	                		alt598=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt598)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1690:17: ALL_SYMBOL
        	        	    {
        	        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_reset_option9583);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereset_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:1690:29: ( channel )?
        	        {
        	            int alt599=2;
        	            {
        	                int LA599_0 = LA(1);
        	                if ( ((SERVER_VERSION >= 50706)) && (LA599_0 == FOR_SYMBOL))
        	                {
        	                    alt599=1;
        	                }
        	            }
        	            switch (alt599)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1690:29: channel
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_channel_in_reset_option9586);
        	        	        channel(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereset_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulereset_optionEx; /* Prevent compiler warnings */
    rulereset_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end reset_option */

/**
 * $ANTLR start utility_statement
 * MySQLSimpleParser.g:1695:1: utility_statement : ( describe_command ( table_ref ( text_string | identifier )? | ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )? explainable_statement ) | HELP_SYMBOL text_or_identifier | use_command );
 */
static void
utility_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1695:18: ( describe_command ( table_ref ( text_string | identifier )? | ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )? explainable_statement ) | HELP_SYMBOL text_or_identifier | use_command )

            ANTLR3_UINT32 alt604;

            alt604=3;

            switch ( LA(1) )
            {
            case DESCRIBE_SYMBOL:
            case DESC_SYMBOL:
            	{
            		alt604=1;
            	}
                break;
            case HELP_SYMBOL:
            	{
            		alt604=2;
            	}
                break;
            case USE_SYMBOL:
            	{
            		alt604=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 604;
                EXCEPTION->state        = 0;


                goto ruleutility_statementEx;

            }

            switch (alt604)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1696:2: describe_command ( table_ref ( text_string | identifier )? | ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )? explainable_statement )
        	    {
        	        FOLLOWPUSH(FOLLOW_describe_command_in_utility_statement9598);
        	        describe_command(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleutility_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1697:3: ( table_ref ( text_string | identifier )? | ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )? explainable_statement )
        	        {
        	            int alt603=2;
        	            {
        	                int LA603_0 = LA(1);
        	                if ( (((LA603_0 >= ACCOUNT_SYMBOL) && (LA603_0 <= FOLLOWS_SYMBOL)) || ((LA603_0 >= GROUP_REPLICATION_SYMBOL) && (LA603_0 <= EXPORT_SYMBOL)) || ((LA603_0 >= EXTENT_SIZE_SYMBOL) && (LA603_0 <= PARTITIONING_SYMBOL)) || ((LA603_0 >= PASSWORD_SYMBOL) && (LA603_0 <= YEAR_SYMBOL)) || LA603_0 == BACK_TICK_QUOTED_ID || LA603_0 == DOT_SYMBOL || LA603_0 == DOUBLE_QUOTED_TEXT || LA603_0 == IDENTIFIER))
        	                {
        	                    alt603=1;
        	                }
        	                else if ( (LA603_0 == EXTENDED_SYMBOL))
        	                {
        	                    {
        	                        int LA603_4 = LA(2);
        	                        if ( (LA603_4 == EOF || ((LA603_4 >= ACCOUNT_SYMBOL) && (LA603_4 <= YEAR_SYMBOL)) || LA603_4 == SEMICOLON_SYMBOL || LA603_4 == SINGLE_QUOTED_TEXT || ((LA603_4 >= BACK_TICK_QUOTED_ID) && (LA603_4 <= BIN_NUMBER)) || LA603_4 == DOT_SYMBOL || LA603_4 == DOUBLE_QUOTED_TEXT || ((LA603_4 >= HEX_NUMBER) && (LA603_4 <= IDENTIFIER))))
        	                        {
        	                            alt603=1;
        	                        }
        	                        else if ( (LA603_4 == DELETE_SYMBOL || LA603_4 == FOR_SYMBOL || LA603_4 == INSERT_SYMBOL || LA603_4 == OPEN_PAR_SYMBOL || LA603_4 == REPLACE_SYMBOL || LA603_4 == SELECT_SYMBOL || LA603_4 == UPDATE_SYMBOL))
        	                        {
        	                            alt603=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 603;
        	                            EXCEPTION->state        = 4;


        	                            goto ruleutility_statementEx;

        	                        }
        	                    }
        	                }
        	                else if ( (LA603_0 == FORMAT_SYMBOL))
        	                {
        	                    {
        	                        int LA603_5 = LA(2);
        	                        if ( (LA603_5 == EQUAL_OPERATOR))
        	                        {
        	                            alt603=2;
        	                        }
        	                        else if ( (LA603_5 == EOF || ((LA603_5 >= ACCOUNT_SYMBOL) && (LA603_5 <= YEAR_SYMBOL)) || LA603_5 == SEMICOLON_SYMBOL || LA603_5 == SINGLE_QUOTED_TEXT || ((LA603_5 >= BACK_TICK_QUOTED_ID) && (LA603_5 <= BIN_NUMBER)) || LA603_5 == DOT_SYMBOL || LA603_5 == DOUBLE_QUOTED_TEXT || ((LA603_5 >= HEX_NUMBER) && (LA603_5 <= IDENTIFIER))))
        	                        {
        	                            alt603=1;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 603;
        	                            EXCEPTION->state        = 5;


        	                            goto ruleutility_statementEx;

        	                        }
        	                    }
        	                }
        	                else if ( (LA603_0 == PARTITIONS_SYMBOL))
        	                {
        	                    {
        	                        int LA603_7 = LA(2);
        	                        if ( (LA603_7 == EOF || ((LA603_7 >= ACCOUNT_SYMBOL) && (LA603_7 <= YEAR_SYMBOL)) || LA603_7 == SEMICOLON_SYMBOL || LA603_7 == SINGLE_QUOTED_TEXT || ((LA603_7 >= BACK_TICK_QUOTED_ID) && (LA603_7 <= BIN_NUMBER)) || LA603_7 == DOT_SYMBOL || LA603_7 == DOUBLE_QUOTED_TEXT || ((LA603_7 >= HEX_NUMBER) && (LA603_7 <= IDENTIFIER))))
        	                        {
        	                            alt603=1;
        	                        }
        	                        else if ( (LA603_7 == DELETE_SYMBOL || LA603_7 == FOR_SYMBOL || LA603_7 == INSERT_SYMBOL || LA603_7 == OPEN_PAR_SYMBOL || LA603_7 == REPLACE_SYMBOL || LA603_7 == SELECT_SYMBOL || LA603_7 == UPDATE_SYMBOL))
        	                        {
        	                            alt603=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 603;
        	                            EXCEPTION->state        = 7;


        	                            goto ruleutility_statementEx;

        	                        }
        	                    }
        	                }
        	                else if ( (LA603_0 == DELETE_SYMBOL || LA603_0 == FOR_SYMBOL || LA603_0 == INSERT_SYMBOL || LA603_0 == OPEN_PAR_SYMBOL || LA603_0 == REPLACE_SYMBOL || LA603_0 == SELECT_SYMBOL || LA603_0 == UPDATE_SYMBOL))
        	                {
        	                    alt603=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 603;
        	                    EXCEPTION->state        = 0;


        	                    goto ruleutility_statementEx;

        	                }
        	            }
        	            switch (alt603)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1698:4: table_ref ( text_string | identifier )?
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_table_ref_in_utility_statement9607);
        	        	        table_ref(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleutility_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:1698:14: ( text_string | identifier )?
        	        	        {
        	        	            int alt601=3;
        	        	            {
        	        	                int LA601_0 = LA(1);
        	        	                if ( (LA601_0 == SINGLE_QUOTED_TEXT || LA601_0 == BIN_NUMBER || LA601_0 == HEX_NUMBER))
        	        	                {
        	        	                    alt601=1;
        	        	                }
        	        	                else if ( (((LA601_0 >= ACCOUNT_SYMBOL) && (LA601_0 <= YEAR_SYMBOL)) || LA601_0 == BACK_TICK_QUOTED_ID || LA601_0 == DOUBLE_QUOTED_TEXT || LA601_0 == IDENTIFIER))
        	        	                {
        	        	                    alt601=2;
        	        	                }
        	        	            }
        	        	            switch (alt601)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1698:15: text_string
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_text_string_in_utility_statement9610);
        	        	        	        text_string(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;
        	        	        	case 2:
        	        	        	    // MySQLSimpleParser.g:1698:29: identifier
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_identifier_in_utility_statement9614);
        	        	        	        identifier(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1700:5: ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )? explainable_statement
        	        	    {
        	        	        // MySQLSimpleParser.g:1700:5: ( EXTENDED_SYMBOL |{...}? PARTITIONS_SYMBOL |{...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier )?
        	        	        {
        	        	            int alt602=4;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case EXTENDED_SYMBOL:
        	        	                	{
        	        	                		alt602=1;
        	        	                	}
        	        	                    break;
        	        	                case PARTITIONS_SYMBOL:
        	        	                	{
        	        	                		alt602=2;
        	        	                	}
        	        	                    break;
        	        	                case FORMAT_SYMBOL:
        	        	                	{
        	        	                		alt602=3;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt602)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1703:6: EXTENDED_SYMBOL
        	        	        	    {
        	        	        	         MATCHT(EXTENDED_SYMBOL, &FOLLOW_EXTENDED_SYMBOL_in_utility_statement9646);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;
        	        	        	case 2:
        	        	        	    // MySQLSimpleParser.g:1704:8: {...}? PARTITIONS_SYMBOL
        	        	        	    {
        	        	        	        if ( !((SERVER_VERSION >= 50105)) )
        	        	        	        {
        	        	        	            if (BACKTRACKING>0)
        	        	        	            {
        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	        	                return ;
        	        	        	            }

        	        	        	                CONSTRUCTEX();
        	        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50105";
        	        	        	                EXCEPTION->ruleName	 = (void *)"utility_statement";


        	        	        	        }

        	        	        	         MATCHT(PARTITIONS_SYMBOL, &FOLLOW_PARTITIONS_SYMBOL_in_utility_statement9658);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;
        	        	        	case 3:
        	        	        	    // MySQLSimpleParser.g:1705:8: {...}? FORMAT_SYMBOL EQUAL_OPERATOR text_or_identifier
        	        	        	    {
        	        	        	        if ( !((SERVER_VERSION >= 50605)) )
        	        	        	        {
        	        	        	            if (BACKTRACKING>0)
        	        	        	            {
        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	        	                return ;
        	        	        	            }

        	        	        	                CONSTRUCTEX();
        	        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50605";
        	        	        	                EXCEPTION->ruleName	 = (void *)"utility_statement";


        	        	        	        }

        	        	        	         MATCHT(FORMAT_SYMBOL, &FOLLOW_FORMAT_SYMBOL_in_utility_statement9670);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_utility_statement9672);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_utility_statement9674);
        	        	        	        text_or_identifier(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleutility_statementEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_explainable_statement_in_utility_statement9683);
        	        	        explainable_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleutility_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1708:4: HELP_SYMBOL text_or_identifier
        	    {
        	         MATCHT(HELP_SYMBOL, &FOLLOW_HELP_SYMBOL_in_utility_statement9692);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleutility_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_utility_statement9694);
        	        text_or_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleutility_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1709:4: use_command
        	    {
        	        FOLLOWPUSH(FOLLOW_use_command_in_utility_statement9699);
        	        use_command(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleutility_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleutility_statementEx; /* Prevent compiler warnings */
    ruleutility_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end utility_statement */

/**
 * $ANTLR start describe_command
 * MySQLSimpleParser.g:1712:1: describe_command : ( DESCRIBE_SYMBOL | DESC_SYMBOL );
 */
static void
describe_command(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1712:17: ( DESCRIBE_SYMBOL | DESC_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= DESCRIBE_SYMBOL) && (LA(1) <= DESC_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruledescribe_commandEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledescribe_commandEx; /* Prevent compiler warnings */
    ruledescribe_commandEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end describe_command */

/**
 * $ANTLR start explainable_statement
 * MySQLSimpleParser.g:1717:1: explainable_statement : ( select_statement |{...}? ( delete_statement | insert_statement | replace_statement | update_statement ) |{...}? FOR_SYMBOL CONNECTION_SYMBOL real_ulong_number );
 */
static void
explainable_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1717:22: ( select_statement |{...}? ( delete_statement | insert_statement | replace_statement | update_statement ) |{...}? FOR_SYMBOL CONNECTION_SYMBOL real_ulong_number )

            ANTLR3_UINT32 alt606;

            alt606=3;

            switch ( LA(1) )
            {
            case OPEN_PAR_SYMBOL:
            case SELECT_SYMBOL:
            	{
            		alt606=1;
            	}
                break;
            case DELETE_SYMBOL:
            case INSERT_SYMBOL:
            case REPLACE_SYMBOL:
            case UPDATE_SYMBOL:
            	{
            		alt606=2;
            	}
                break;
            case FOR_SYMBOL:
            	{
            		alt606=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 606;
                EXCEPTION->state        = 0;


                goto ruleexplainable_statementEx;

            }

            switch (alt606)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1718:2: select_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_select_statement_in_explainable_statement9722);
        	        select_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexplainable_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1719:4: {...}? ( delete_statement | insert_statement | replace_statement | update_statement )
        	    {
        	        if ( !((SERVER_VERSION >= 50603)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50603";
        	                EXCEPTION->ruleName	 = (void *)"explainable_statement";


        	        }

        	        // MySQLSimpleParser.g:1720:3: ( delete_statement | insert_statement | replace_statement | update_statement )
        	        {
        	            int alt605=4;
        	            switch ( LA(1) )
        	            {
        	            case DELETE_SYMBOL:
        	            	{
        	            		alt605=1;
        	            	}
        	                break;
        	            case INSERT_SYMBOL:
        	            	{
        	            		alt605=2;
        	            	}
        	                break;
        	            case REPLACE_SYMBOL:
        	            	{
        	            		alt605=3;
        	            	}
        	                break;
        	            case UPDATE_SYMBOL:
        	            	{
        	            		alt605=4;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 605;
        	                EXCEPTION->state        = 0;


        	                goto ruleexplainable_statementEx;

        	            }

        	            switch (alt605)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1721:4: delete_statement
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_delete_statement_in_explainable_statement9736);
        	        	        delete_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleexplainable_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1722:6: insert_statement
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_insert_statement_in_explainable_statement9743);
        	        	        insert_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleexplainable_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:1723:6: replace_statement
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_replace_statement_in_explainable_statement9750);
        	        	        replace_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleexplainable_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 4:
        	        	    // MySQLSimpleParser.g:1724:6: update_statement
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_update_statement_in_explainable_statement9757);
        	        	        update_statement(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleexplainable_statementEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1726:4: {...}? FOR_SYMBOL CONNECTION_SYMBOL real_ulong_number
        	    {
        	        if ( !((SERVER_VERSION >= 50700)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
        	                EXCEPTION->ruleName	 = (void *)"explainable_statement";


        	        }

        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_explainable_statement9768);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexplainable_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CONNECTION_SYMBOL, &FOLLOW_CONNECTION_SYMBOL_in_explainable_statement9770);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexplainable_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_explainable_statement9772);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexplainable_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleexplainable_statementEx; /* Prevent compiler warnings */
    ruleexplainable_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end explainable_statement */

/**
 * $ANTLR start use_command
 * MySQLSimpleParser.g:1729:1: use_command : USE_SYMBOL identifier ;
 */
static void
use_command(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1729:12: ( USE_SYMBOL identifier )
        // MySQLSimpleParser.g:1730:2: USE_SYMBOL identifier
        {
             MATCHT(USE_SYMBOL, &FOLLOW_USE_SYMBOL_in_use_command9781);
            if  (HASEXCEPTION())
            {
                goto ruleuse_commandEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_use_command9783);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleuse_commandEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleuse_commandEx; /* Prevent compiler warnings */
    ruleuse_commandEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end use_command */

/**
 * $ANTLR start expression
 * MySQLSimpleParser.g:1738:1: expression : logical_or_expression ;
 */
static void
expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1738:11: ( logical_or_expression )
        // MySQLSimpleParser.g:1739:2: logical_or_expression
        {
            FOLLOWPUSH(FOLLOW_logical_or_expression_in_expression9797);
            logical_or_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexpressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexpressionEx; /* Prevent compiler warnings */
    ruleexpressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end expression */

/**
 * $ANTLR start logical_or_expression
 * MySQLSimpleParser.g:1742:1: logical_or_expression : logical_xor_expression ( options {greedy=true; } : ( LOGICAL_OR_OPERATOR | OR_SYMBOL ) logical_xor_expression )* ;
 */
static void
logical_or_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1742:22: ( logical_xor_expression ( options {greedy=true; } : ( LOGICAL_OR_OPERATOR | OR_SYMBOL ) logical_xor_expression )* )
        // MySQLSimpleParser.g:1743:2: logical_xor_expression ( options {greedy=true; } : ( LOGICAL_OR_OPERATOR | OR_SYMBOL ) logical_xor_expression )*
        {
            FOLLOWPUSH(FOLLOW_logical_xor_expression_in_logical_or_expression9806);
            logical_xor_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelogical_or_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1743:25: ( options {greedy=true; } : ( LOGICAL_OR_OPERATOR | OR_SYMBOL ) logical_xor_expression )*

            for (;;)
            {
                int alt607=2;
                switch ( LA(1) )
                {
                case OR_SYMBOL:
                case LOGICAL_OR_OPERATOR:
                	{
                		alt607=1;
                	}
                    break;

                }

                switch (alt607)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1743:55: ( LOGICAL_OR_OPERATOR | OR_SYMBOL ) logical_xor_expression
            	    {
            	        if ( LA(1) == OR_SYMBOL || LA(1) == LOGICAL_OR_OPERATOR )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulelogical_or_expressionEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_logical_xor_expression_in_logical_or_expression9830);
            	        logical_xor_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelogical_or_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop607;	/* break out of the loop */
            	    break;
                }
            }
            loop607: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogical_or_expressionEx; /* Prevent compiler warnings */
    rulelogical_or_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logical_or_expression */

/**
 * $ANTLR start logical_xor_expression
 * MySQLSimpleParser.g:1746:1: logical_xor_expression : logical_and_expression ( options {greedy=true; } : XOR_SYMBOL logical_and_expression )* ;
 */
static void
logical_xor_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1746:23: ( logical_and_expression ( options {greedy=true; } : XOR_SYMBOL logical_and_expression )* )
        // MySQLSimpleParser.g:1747:2: logical_and_expression ( options {greedy=true; } : XOR_SYMBOL logical_and_expression )*
        {
            FOLLOWPUSH(FOLLOW_logical_and_expression_in_logical_xor_expression9841);
            logical_and_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelogical_xor_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1747:25: ( options {greedy=true; } : XOR_SYMBOL logical_and_expression )*

            for (;;)
            {
                int alt608=2;
                switch ( LA(1) )
                {
                case XOR_SYMBOL:
                	{
                		alt608=1;
                	}
                    break;

                }

                switch (alt608)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1747:55: XOR_SYMBOL logical_and_expression
            	    {
            	         MATCHT(XOR_SYMBOL, &FOLLOW_XOR_SYMBOL_in_logical_xor_expression9857);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelogical_xor_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_logical_and_expression_in_logical_xor_expression9859);
            	        logical_and_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelogical_xor_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop608;	/* break out of the loop */
            	    break;
                }
            }
            loop608: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogical_xor_expressionEx; /* Prevent compiler warnings */
    rulelogical_xor_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logical_xor_expression */

/**
 * $ANTLR start logical_and_expression
 * MySQLSimpleParser.g:1750:1: logical_and_expression : logical_not_expression ( options {greedy=true; } : ( LOGICAL_AND_OPERATOR | AND_SYMBOL ) logical_not_expression )* ;
 */
static void
logical_and_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1750:23: ( logical_not_expression ( options {greedy=true; } : ( LOGICAL_AND_OPERATOR | AND_SYMBOL ) logical_not_expression )* )
        // MySQLSimpleParser.g:1751:2: logical_not_expression ( options {greedy=true; } : ( LOGICAL_AND_OPERATOR | AND_SYMBOL ) logical_not_expression )*
        {
            FOLLOWPUSH(FOLLOW_logical_not_expression_in_logical_and_expression9870);
            logical_not_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelogical_and_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1751:25: ( options {greedy=true; } : ( LOGICAL_AND_OPERATOR | AND_SYMBOL ) logical_not_expression )*

            for (;;)
            {
                int alt609=2;
                switch ( LA(1) )
                {
                case AND_SYMBOL:
                	{
                		alt609=1;
                	}
                    break;
                case LOGICAL_AND_OPERATOR:
                	{
                		alt609=1;
                	}
                    break;

                }

                switch (alt609)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1751:55: ( LOGICAL_AND_OPERATOR | AND_SYMBOL ) logical_not_expression
            	    {
            	        if ( LA(1) == AND_SYMBOL || LA(1) == LOGICAL_AND_OPERATOR )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulelogical_and_expressionEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_logical_not_expression_in_logical_and_expression9894);
            	        logical_not_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelogical_and_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop609;	/* break out of the loop */
            	    break;
                }
            }
            loop609: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogical_and_expressionEx; /* Prevent compiler warnings */
    rulelogical_and_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logical_and_expression */

/**
 * $ANTLR start logical_not_expression
 * MySQLSimpleParser.g:1754:1: logical_not_expression : ( NOT_SYMBOL logical_not_expression | boolean_primary_expression );
 */
static void
logical_not_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1754:23: ( NOT_SYMBOL logical_not_expression | boolean_primary_expression )

            ANTLR3_UINT32 alt610;

            alt610=2;

            {
                int LA610_0 = LA(1);
                if ( (LA610_0 == NOT_SYMBOL))
                {
                    alt610=1;
                }
                else if ( (((LA610_0 >= ACCOUNT_SYMBOL) && (LA610_0 <= YEAR_SYMBOL)) || LA610_0 == BINARY_SYMBOL || ((LA610_0 >= CASE_SYMBOL) && (LA610_0 <= CAST_SYMBOL)) || LA610_0 == CHAR_SYMBOL || ((LA610_0 >= CONVERT_SYMBOL) && (LA610_0 <= COUNT_SYMBOL)) || LA610_0 == CURDATE_SYMBOL || LA610_0 == CURRENT_USER_SYMBOL || LA610_0 == CURTIME_SYMBOL || LA610_0 == DATABASE_SYMBOL || LA610_0 == DATE_ADD_SYMBOL || LA610_0 == DATE_SUB_SYMBOL || LA610_0 == EXISTS_SYMBOL || ((LA610_0 >= EXTRACT_SYMBOL) && (LA610_0 <= FALSE_SYMBOL)) || LA610_0 == GROUP_CONCAT_SYMBOL || LA610_0 == IF_SYMBOL || LA610_0 == INSERT_SYMBOL || LA610_0 == INTERVAL_SYMBOL || LA610_0 == LEFT_SYMBOL || LA610_0 == MATCH_SYMBOL || LA610_0 == MAX_SYMBOL || LA610_0 == MIN_SYMBOL || LA610_0 == MOD_SYMBOL || LA610_0 == NCHAR_TEXT || LA610_0 == NOT2_SYMBOL || LA610_0 == NOW_SYMBOL || ((LA610_0 >= NULL2_SYMBOL) && (LA610_0 <= NULL_SYMBOL)) || ((LA610_0 >= OPEN_CURLY_SYMBOL) && (LA610_0 <= OPEN_PAR_SYMBOL)) || ((LA610_0 >= PARAM_MARKER) && (LA610_0 <= POSITION_SYMBOL)) || ((LA610_0 >= REPEAT_SYMBOL) && (LA610_0 <= REPLACE_SYMBOL)) || LA610_0 == RIGHT_SYMBOL || LA610_0 == SINGLE_QUOTED_TEXT || LA610_0 == STDDEV_SAMP_SYMBOL || LA610_0 == STD_SYMBOL || LA610_0 == SUBSTRING_SYMBOL || ((LA610_0 >= SUM_SYMBOL) && (LA610_0 <= SYSDATE_SYMBOL)) || ((LA610_0 >= TRIM_SYMBOL) && (LA610_0 <= TRUE_SYMBOL)) || ((LA610_0 >= UTC_DATE_SYMBOL) && (LA610_0 <= UTC_TIME_SYMBOL)) || LA610_0 == VALUES_SYMBOL || LA610_0 == VARIANCE_SYMBOL || LA610_0 == VAR_SAMP_SYMBOL || ((LA610_0 >= AT_AT_SIGN_SYMBOL) && (LA610_0 <= AT_TEXT_SUFFIX)) || ((LA610_0 >= BACK_TICK_QUOTED_ID) && (LA610_0 <= BIN_NUMBER)) || LA610_0 == BITWISE_NOT_OPERATOR || ((LA610_0 >= BIT_AND_SYMBOL) && (LA610_0 <= BIT_XOR_SYMBOL)) || LA610_0 == DECIMAL_NUMBER || LA610_0 == DOT_SYMBOL || LA610_0 == DOUBLE_QUOTED_TEXT || LA610_0 == FLOAT_NUMBER || ((LA610_0 >= HEX_NUMBER) && (LA610_0 <= IDENTIFIER)) || LA610_0 == INT_NUMBER || LA610_0 == LOGICAL_NOT_OPERATOR || ((LA610_0 >= LONG_NUMBER) && (LA610_0 <= MINUS_OPERATOR)) || LA610_0 == PLUS_OPERATOR || ((LA610_0 >= ULONGLONG_NUMBER) && (LA610_0 <= UNDERSCORE_CHARSET))))
                {
                    alt610=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 610;
                    EXCEPTION->state        = 0;


                    goto rulelogical_not_expressionEx;

                }
            }
            switch (alt610)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1755:2: NOT_SYMBOL logical_not_expression
        	    {
        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_logical_not_expression9905);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogical_not_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_logical_not_expression_in_logical_not_expression9907);
        	        logical_not_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogical_not_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1756:4: boolean_primary_expression
        	    {
        	        FOLLOWPUSH(FOLLOW_boolean_primary_expression_in_logical_not_expression9912);
        	        boolean_primary_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelogical_not_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelogical_not_expressionEx; /* Prevent compiler warnings */
    rulelogical_not_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logical_not_expression */

/**
 * $ANTLR start boolean_primary_expression
 * MySQLSimpleParser.g:1759:1: boolean_primary_expression : predicate ( options {greedy=true; } : comparison_operator ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate ) | ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+ )* ;
 */
static void
boolean_primary_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1759:27: ( predicate ( options {greedy=true; } : comparison_operator ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate ) | ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+ )* )
        // MySQLSimpleParser.g:1760:2: predicate ( options {greedy=true; } : comparison_operator ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate ) | ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+ )*
        {
            FOLLOWPUSH(FOLLOW_predicate_in_boolean_primary_expression9921);
            predicate(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleboolean_primary_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1761:3: ( options {greedy=true; } : comparison_operator ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate ) | ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+ )*

            for (;;)
            {
                int alt615=3;
                switch ( LA(1) )
                {
                case EQUAL_OPERATOR:
                	{
                		alt615=1;
                	}
                    break;
                case IS_SYMBOL:
                	{
                		alt615=2;
                	}
                    break;
                case GREATER_OR_EQUAL_OPERATOR:
                case GREATER_THAN_OPERATOR:
                case LESS_OR_EQUAL_OPERATOR:
                case LESS_THAN_OPERATOR:
                case NOT_EQUAL2_OPERATOR:
                case NOT_EQUAL_OPERATOR:
                case NULL_SAFE_EQUAL_OPERATOR:
                	{
                		alt615=1;
                	}
                    break;

                }

                switch (alt615)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1762:4: comparison_operator ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate )
            	    {
            	        FOLLOWPUSH(FOLLOW_comparison_operator_in_boolean_primary_expression9942);
            	        comparison_operator(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleboolean_primary_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1763:4: ({...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery | predicate )
            	        {
            	            int alt611=2;
            	            {
            	                int LA611_0 = LA(1);
            	                if ( (LA611_0 == ANY_SYMBOL))
            	                {
            	                    {
            	                        int LA611_1 = LA(2);
            	                        if ( (LA611_1 == OPEN_PAR_SYMBOL))
            	                        {
            	                            alt611=1;
            	                        }
            	                        else if ( (LA611_1 == EOF || ((LA611_1 >= ACCOUNT_SYMBOL) && (LA611_1 <= YEAR_SYMBOL)) || ((LA611_1 >= AND_SYMBOL) && (LA611_1 <= ASC_SYMBOL)) || LA611_1 == BETWEEN_SYMBOL || LA611_1 == COLLATE_SYMBOL || LA611_1 == CROSS_SYMBOL || ((LA611_1 >= DAY_HOUR_SYMBOL) && (LA611_1 <= DAY_SECOND_SYMBOL)) || LA611_1 == DESC_SYMBOL || LA611_1 == ELSE_SYMBOL || ((LA611_1 >= FOR_SYMBOL) && (LA611_1 <= FROM_SYMBOL)) || ((LA611_1 >= GROUP_SYMBOL) && (LA611_1 <= HAVING_SYMBOL)) || ((LA611_1 >= HOUR_MICROSECOND_SYMBOL) && (LA611_1 <= HOUR_SECOND_SYMBOL)) || LA611_1 == INNER_SYMBOL || LA611_1 == INTO_SYMBOL || LA611_1 == IN_SYMBOL || LA611_1 == IS_SYMBOL || ((LA611_1 >= JOIN_SYMBOL) && (LA611_1 <= JSON_SEPARATOR_SYMBOL)) || ((LA611_1 >= LEFT_SYMBOL) && (LA611_1 <= LIMIT_SYMBOL)) || LA611_1 == LOCK_SYMBOL || ((LA611_1 >= MINUTE_MICROSECOND_SYMBOL) && (LA611_1 <= MINUTE_SECOND_SYMBOL)) || ((LA611_1 >= MOD_SYMBOL) && (LA611_1 <= NATURAL_SYMBOL)) || LA611_1 == NOT_SYMBOL || LA611_1 == ON_SYMBOL || ((LA611_1 >= ORDER_SYMBOL) && (LA611_1 <= OR_SYMBOL)) || LA611_1 == PROCEDURE_SYMBOL || LA611_1 == REGEXP_SYMBOL || LA611_1 == RENAME_SYMBOL || LA611_1 == RIGHT_SYMBOL || LA611_1 == SECOND_MICROSECOND_SYMBOL || LA611_1 == SEMICOLON_SYMBOL || LA611_1 == SEPARATOR_SYMBOL || LA611_1 == SET_SYMBOL || LA611_1 == SINGLE_QUOTED_TEXT || LA611_1 == STRAIGHT_JOIN_SYMBOL || LA611_1 == THEN_SYMBOL || LA611_1 == UNION_SYMBOL || LA611_1 == USING_SYMBOL || ((LA611_1 >= WHEN_SYMBOL) && (LA611_1 <= WHERE_SYMBOL)) || LA611_1 == WITH_SYMBOL || ((LA611_1 >= XOR_SYMBOL) && (LA611_1 <= YEAR_MONTH_SYMBOL)) || ((LA611_1 >= AS_SYMBOL) && (LA611_1 <= AT_TEXT_SUFFIX)) || ((LA611_1 >= BACK_TICK_QUOTED_ID) && (LA611_1 <= BITWISE_AND_OPERATOR)) || ((LA611_1 >= BITWISE_OR_OPERATOR) && (LA611_1 <= BITWISE_XOR_OPERATOR)) || ((LA611_1 >= CLOSE_CURLY_SYMBOL) && (LA611_1 <= CLOSE_PAR_SYMBOL)) || ((LA611_1 >= COMMA_SYMBOL) && (LA611_1 <= CONCAT_PIPES_SYMBOL)) || ((LA611_1 >= DIV_OPERATOR) && (LA611_1 <= DIV_SYMBOL)) || LA611_1 == DOT_SYMBOL || LA611_1 == DOUBLE_QUOTED_TEXT || LA611_1 == EQUAL_OPERATOR || ((LA611_1 >= GREATER_OR_EQUAL_OPERATOR) && (LA611_1 <= GREATER_THAN_OPERATOR)) || ((LA611_1 >= HEX_NUMBER) && (LA611_1 <= IDENTIFIER)) || LA611_1 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA611_1 >= LESS_OR_EQUAL_OPERATOR) && (LA611_1 <= LESS_THAN_OPERATOR)) || LA611_1 == LOGICAL_AND_OPERATOR || LA611_1 == LOGICAL_OR_OPERATOR || ((LA611_1 >= MINUS_OPERATOR) && (LA611_1 <= MOD_OPERATOR)) || ((LA611_1 >= MULT_OPERATOR) && (LA611_1 <= NULL_SAFE_EQUAL_OPERATOR)) || LA611_1 == PLUS_OPERATOR || ((LA611_1 >= SHIFT_LEFT_OPERATOR) && (LA611_1 <= SHIFT_RIGHT_OPERATOR))))
            	                        {
            	                            alt611=2;
            	                        }
            	                        else
            	                        {
            	                            if (BACKTRACKING>0)
            	                            {
            	                                FAILEDFLAG = ANTLR3_TRUE;
            	                                return ;
            	                            }


            	                            CONSTRUCTEX();
            	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                            EXCEPTION->message      = (void *)"";
            	                            EXCEPTION->decisionNum  = 611;
            	                            EXCEPTION->state        = 1;


            	                            goto ruleboolean_primary_expressionEx;

            	                        }
            	                    }
            	                }
            	                else if ( (((LA611_0 >= ACCOUNT_SYMBOL) && (LA611_0 <= ANALYZE_SYMBOL)) || ((LA611_0 >= AT_SYMBOL) && (LA611_0 <= YEAR_SYMBOL)) || LA611_0 == BINARY_SYMBOL || ((LA611_0 >= CASE_SYMBOL) && (LA611_0 <= CAST_SYMBOL)) || LA611_0 == CHAR_SYMBOL || ((LA611_0 >= CONVERT_SYMBOL) && (LA611_0 <= COUNT_SYMBOL)) || LA611_0 == CURDATE_SYMBOL || LA611_0 == CURRENT_USER_SYMBOL || LA611_0 == CURTIME_SYMBOL || LA611_0 == DATABASE_SYMBOL || LA611_0 == DATE_ADD_SYMBOL || LA611_0 == DATE_SUB_SYMBOL || LA611_0 == EXISTS_SYMBOL || ((LA611_0 >= EXTRACT_SYMBOL) && (LA611_0 <= FALSE_SYMBOL)) || LA611_0 == GROUP_CONCAT_SYMBOL || LA611_0 == IF_SYMBOL || LA611_0 == INSERT_SYMBOL || LA611_0 == INTERVAL_SYMBOL || LA611_0 == LEFT_SYMBOL || LA611_0 == MATCH_SYMBOL || LA611_0 == MAX_SYMBOL || LA611_0 == MIN_SYMBOL || LA611_0 == MOD_SYMBOL || LA611_0 == NCHAR_TEXT || LA611_0 == NOT2_SYMBOL || LA611_0 == NOW_SYMBOL || ((LA611_0 >= NULL2_SYMBOL) && (LA611_0 <= NULL_SYMBOL)) || ((LA611_0 >= OPEN_CURLY_SYMBOL) && (LA611_0 <= OPEN_PAR_SYMBOL)) || ((LA611_0 >= PARAM_MARKER) && (LA611_0 <= POSITION_SYMBOL)) || ((LA611_0 >= REPEAT_SYMBOL) && (LA611_0 <= REPLACE_SYMBOL)) || LA611_0 == RIGHT_SYMBOL || LA611_0 == SINGLE_QUOTED_TEXT || LA611_0 == STDDEV_SAMP_SYMBOL || LA611_0 == STD_SYMBOL || LA611_0 == SUBSTRING_SYMBOL || ((LA611_0 >= SUM_SYMBOL) && (LA611_0 <= SYSDATE_SYMBOL)) || ((LA611_0 >= TRIM_SYMBOL) && (LA611_0 <= TRUE_SYMBOL)) || ((LA611_0 >= UTC_DATE_SYMBOL) && (LA611_0 <= UTC_TIME_SYMBOL)) || LA611_0 == VALUES_SYMBOL || LA611_0 == VARIANCE_SYMBOL || LA611_0 == VAR_SAMP_SYMBOL || ((LA611_0 >= AT_AT_SIGN_SYMBOL) && (LA611_0 <= AT_TEXT_SUFFIX)) || ((LA611_0 >= BACK_TICK_QUOTED_ID) && (LA611_0 <= BIN_NUMBER)) || LA611_0 == BITWISE_NOT_OPERATOR || ((LA611_0 >= BIT_AND_SYMBOL) && (LA611_0 <= BIT_XOR_SYMBOL)) || LA611_0 == DECIMAL_NUMBER || LA611_0 == DOT_SYMBOL || LA611_0 == DOUBLE_QUOTED_TEXT || LA611_0 == FLOAT_NUMBER || ((LA611_0 >= HEX_NUMBER) && (LA611_0 <= IDENTIFIER)) || LA611_0 == INT_NUMBER || LA611_0 == LOGICAL_NOT_OPERATOR || ((LA611_0 >= LONG_NUMBER) && (LA611_0 <= MINUS_OPERATOR)) || LA611_0 == PLUS_OPERATOR || ((LA611_0 >= ULONGLONG_NUMBER) && (LA611_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt611=2;
            	                }
            	                else if ( (LA611_0 == ALL_SYMBOL))
            	                {
            	                    alt611=1;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 611;
            	                    EXCEPTION->state        = 0;


            	                    goto ruleboolean_primary_expressionEx;

            	                }
            	            }
            	            switch (alt611)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1764:5: {...}? ( ALL_SYMBOL | ANY_SYMBOL ) subquery
            	        	    {
            	        	        if ( !((LA(2) == OPEN_PAR_SYMBOL)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"LA(2) == OPEN_PAR_SYMBOL";
            	        	                EXCEPTION->ruleName	 = (void *)"boolean_primary_expression";


            	        	        }

            	        	        if ( LA(1) == ANY_SYMBOL || LA(1) == ALL_SYMBOL )
            	        	        {
            	        	            CONSUME();
            	        	            PERRORRECOVERY=ANTLR3_FALSE;
            	        	            FAILEDFLAG=ANTLR3_FALSE;

            	        	        }
            	        	        else
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	            CONSTRUCTEX();
            	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	        	            EXCEPTION->expectingSet = NULL;

            	        	            goto ruleboolean_primary_expressionEx;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_subquery_in_boolean_primary_expression9963);
            	        	        subquery(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleboolean_primary_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:1765:7: predicate
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_predicate_in_boolean_primary_expression9971);
            	        	        predicate(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleboolean_primary_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1767:9: ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+
            	    {
            	        // MySQLSimpleParser.g:1767:9: ( options {greedy=true; } : IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL ) )+
            	        {
            	            int cnt614=0;

            	            for (;;)
            	            {
            	                int alt614=2;
            	        	switch ( LA(1) )
            	        	{
            	        	case IS_SYMBOL:
            	        		{
            	        			alt614=1;
            	        		}
            	        	    break;

            	        	}

            	        	switch (alt614)
            	        	{
            	        	    case 1:
            	        	        // MySQLSimpleParser.g:1767:39: IS_SYMBOL ( NOT_SYMBOL )? ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL )
            	        	        {
            	        	             MATCHT(IS_SYMBOL, &FOLLOW_IS_SYMBOL_in_boolean_primary_expression10000);
            	        	            if  (HASEXCEPTION())
            	        	            {
            	        	                goto ruleboolean_primary_expressionEx;
            	        	            }
            	        	            if (HASFAILED())
            	        	            {
            	        	                return ;
            	        	            }


            	        	            // MySQLSimpleParser.g:1767:49: ( NOT_SYMBOL )?
            	        	            {
            	        	                int alt612=2;
            	        	                switch ( LA(1) )
            	        	                {
            	        	                    case NOT_SYMBOL:
            	        	                    	{
            	        	                    		alt612=1;
            	        	                    	}
            	        	                        break;
            	        	                }

            	        	                switch (alt612)
            	        	                {
            	        	            	case 1:
            	        	            	    // MySQLSimpleParser.g:1767:49: NOT_SYMBOL
            	        	            	    {
            	        	            	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_boolean_primary_expression10002);
            	        	            	        if  (HASEXCEPTION())
            	        	            	        {
            	        	            	            goto ruleboolean_primary_expressionEx;
            	        	            	        }
            	        	            	        if (HASFAILED())
            	        	            	        {
            	        	            	            return ;
            	        	            	        }


            	        	            	    }
            	        	            	    break;

            	        	                }
            	        	            }

            	        	            // MySQLSimpleParser.g:1767:61: ( null_literal | FALSE_SYMBOL | TRUE_SYMBOL | UNKNOWN_SYMBOL )
            	        	            {
            	        	                int alt613=4;
            	        	                switch ( LA(1) )
            	        	                {
            	        	                case NULL2_SYMBOL:
            	        	                case NULL_SYMBOL:
            	        	                	{
            	        	                		alt613=1;
            	        	                	}
            	        	                    break;
            	        	                case FALSE_SYMBOL:
            	        	                	{
            	        	                		alt613=2;
            	        	                	}
            	        	                    break;
            	        	                case TRUE_SYMBOL:
            	        	                	{
            	        	                		alt613=3;
            	        	                	}
            	        	                    break;
            	        	                case UNKNOWN_SYMBOL:
            	        	                	{
            	        	                		alt613=4;
            	        	                	}
            	        	                    break;

            	        	                default:
            	        	                    if (BACKTRACKING>0)
            	        	                    {
            	        	                        FAILEDFLAG = ANTLR3_TRUE;
            	        	                        return ;
            	        	                    }

            	        	                    CONSTRUCTEX();
            	        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	        	                    EXCEPTION->message      = (void *)"";
            	        	                    EXCEPTION->decisionNum  = 613;
            	        	                    EXCEPTION->state        = 0;


            	        	                    goto ruleboolean_primary_expressionEx;

            	        	                }

            	        	                switch (alt613)
            	        	                {
            	        	            	case 1:
            	        	            	    // MySQLSimpleParser.g:1767:63: null_literal
            	        	            	    {
            	        	            	        FOLLOWPUSH(FOLLOW_null_literal_in_boolean_primary_expression10007);
            	        	            	        null_literal(ctx);

            	        	            	        FOLLOWPOP();
            	        	            	        if  (HASEXCEPTION())
            	        	            	        {
            	        	            	            goto ruleboolean_primary_expressionEx;
            	        	            	        }
            	        	            	        if (HASFAILED())
            	        	            	        {
            	        	            	            return ;
            	        	            	        }


            	        	            	    }
            	        	            	    break;
            	        	            	case 2:
            	        	            	    // MySQLSimpleParser.g:1767:78: FALSE_SYMBOL
            	        	            	    {
            	        	            	         MATCHT(FALSE_SYMBOL, &FOLLOW_FALSE_SYMBOL_in_boolean_primary_expression10011);
            	        	            	        if  (HASEXCEPTION())
            	        	            	        {
            	        	            	            goto ruleboolean_primary_expressionEx;
            	        	            	        }
            	        	            	        if (HASFAILED())
            	        	            	        {
            	        	            	            return ;
            	        	            	        }


            	        	            	    }
            	        	            	    break;
            	        	            	case 3:
            	        	            	    // MySQLSimpleParser.g:1767:93: TRUE_SYMBOL
            	        	            	    {
            	        	            	         MATCHT(TRUE_SYMBOL, &FOLLOW_TRUE_SYMBOL_in_boolean_primary_expression10015);
            	        	            	        if  (HASEXCEPTION())
            	        	            	        {
            	        	            	            goto ruleboolean_primary_expressionEx;
            	        	            	        }
            	        	            	        if (HASFAILED())
            	        	            	        {
            	        	            	            return ;
            	        	            	        }


            	        	            	    }
            	        	            	    break;
            	        	            	case 4:
            	        	            	    // MySQLSimpleParser.g:1767:107: UNKNOWN_SYMBOL
            	        	            	    {
            	        	            	         MATCHT(UNKNOWN_SYMBOL, &FOLLOW_UNKNOWN_SYMBOL_in_boolean_primary_expression10019);
            	        	            	        if  (HASEXCEPTION())
            	        	            	        {
            	        	            	            goto ruleboolean_primary_expressionEx;
            	        	            	        }
            	        	            	        if (HASFAILED())
            	        	            	        {
            	        	            	            return ;
            	        	            	        }


            	        	            	    }
            	        	            	    break;

            	        	                }
            	        	            }

            	        	        }
            	        	        break;

            	        	    default:

            	        		if ( cnt614 >= 1 )
            	        		{
            	        		    goto loop614;
            	        		}
            	        		if (BACKTRACKING>0)
            	        		{
            	        		    FAILEDFLAG = ANTLR3_TRUE;
            	        		    return ;
            	        		}

            	        		/* mismatchedSetEx()
            	        		 */
            	        		CONSTRUCTEX();
            	        		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            	        		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            	        		goto ruleboolean_primary_expressionEx;
            	        	}
            	        	cnt614++;
            	            }
            	            loop614: ;	/* Jump to here if this rule does not match */
            	        }

            	    }
            	    break;

            	default:
            	    goto loop615;	/* break out of the loop */
            	    break;
                }
            }
            loop615: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleboolean_primary_expressionEx; /* Prevent compiler warnings */
    ruleboolean_primary_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end boolean_primary_expression */

/**
 * $ANTLR start predicate
 * MySQLSimpleParser.g:1771:1: predicate : bitwise_or_expression ( options {greedy=true; } : ( NOT_SYMBOL )? ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in ) | SOUNDS_SYMBOL LIKE_SYMBOL bitwise_or_expression )? ;
 */
static void
predicate(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1771:10: ( bitwise_or_expression ( options {greedy=true; } : ( NOT_SYMBOL )? ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in ) | SOUNDS_SYMBOL LIKE_SYMBOL bitwise_or_expression )? )
        // MySQLSimpleParser.g:1772:2: bitwise_or_expression ( options {greedy=true; } : ( NOT_SYMBOL )? ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in ) | SOUNDS_SYMBOL LIKE_SYMBOL bitwise_or_expression )?
        {
            FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_predicate10036);
            bitwise_or_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulepredicateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1773:3: ( options {greedy=true; } : ( NOT_SYMBOL )? ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in ) | SOUNDS_SYMBOL LIKE_SYMBOL bitwise_or_expression )?
            {
                int alt619=3;
                switch ( LA(1) )
                {
                    case NOT_SYMBOL:
                    	{
                    		alt619=1;
                    	}
                        break;
                    case BETWEEN_SYMBOL:
                    	{
                    		alt619=1;
                    	}
                        break;
                    case LIKE_SYMBOL:
                    	{
                    		alt619=1;
                    	}
                        break;
                    case REGEXP_SYMBOL:
                    	{
                    		alt619=1;
                    	}
                        break;
                    case IN_SYMBOL:
                    	{
                    		alt619=1;
                    	}
                        break;
                    case SOUNDS_SYMBOL:
                    	{
                    		alt619=2;
                    	}
                        break;
                }

                switch (alt619)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1774:4: ( NOT_SYMBOL )? ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in )
            	    {
            	        // MySQLSimpleParser.g:1774:4: ( NOT_SYMBOL )?
            	        {
            	            int alt616=2;
            	            switch ( LA(1) )
            	            {
            	                case NOT_SYMBOL:
            	                	{
            	                		alt616=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt616)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1774:4: NOT_SYMBOL
            	        	    {
            	        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_predicate10057);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:1775:5: ( BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate | LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )? | REGEXP_SYMBOL bitwise_or_expression | IN_SYMBOL predicate_in )
            	        {
            	            int alt618=4;
            	            switch ( LA(1) )
            	            {
            	            case BETWEEN_SYMBOL:
            	            	{
            	            		alt618=1;
            	            	}
            	                break;
            	            case LIKE_SYMBOL:
            	            	{
            	            		alt618=2;
            	            	}
            	                break;
            	            case REGEXP_SYMBOL:
            	            	{
            	            		alt618=3;
            	            	}
            	                break;
            	            case IN_SYMBOL:
            	            	{
            	            		alt618=4;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 618;
            	                EXCEPTION->state        = 0;


            	                goto rulepredicateEx;

            	            }

            	            switch (alt618)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1776:6: BETWEEN_SYMBOL bitwise_or_expression AND_SYMBOL predicate
            	        	    {
            	        	         MATCHT(BETWEEN_SYMBOL, &FOLLOW_BETWEEN_SYMBOL_in_predicate10071);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_predicate10073);
            	        	        bitwise_or_expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(AND_SYMBOL, &FOLLOW_AND_SYMBOL_in_predicate10075);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_predicate_in_predicate10077);
            	        	        predicate(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:1777:8: LIKE_SYMBOL unary_expression ( options {greedy=true; } : ESCAPE_SYMBOL primary )?
            	        	    {
            	        	         MATCHT(LIKE_SYMBOL, &FOLLOW_LIKE_SYMBOL_in_predicate10086);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_unary_expression_in_predicate10088);
            	        	        unary_expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:1777:37: ( options {greedy=true; } : ESCAPE_SYMBOL primary )?
            	        	        {
            	        	            int alt617=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case ESCAPE_SYMBOL:
            	        	                	{
            	        	                		alt617=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt617)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:1777:67: ESCAPE_SYMBOL primary
            	        	        	    {
            	        	        	         MATCHT(ESCAPE_SYMBOL, &FOLLOW_ESCAPE_SYMBOL_in_predicate10104);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepredicateEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	        FOLLOWPUSH(FOLLOW_primary_in_predicate10106);
            	        	        	        primary(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepredicateEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;
            	        	case 3:
            	        	    // MySQLSimpleParser.g:1778:8: REGEXP_SYMBOL bitwise_or_expression
            	        	    {
            	        	         MATCHT(REGEXP_SYMBOL, &FOLLOW_REGEXP_SYMBOL_in_predicate10117);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_predicate10119);
            	        	        bitwise_or_expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 4:
            	        	    // MySQLSimpleParser.g:1779:8: IN_SYMBOL predicate_in
            	        	    {
            	        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_predicate10128);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_predicate_in_in_predicate10130);
            	        	        predicate_in(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepredicateEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1781:9: SOUNDS_SYMBOL LIKE_SYMBOL bitwise_or_expression
            	    {
            	         MATCHT(SOUNDS_SYMBOL, &FOLLOW_SOUNDS_SYMBOL_in_predicate10149);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepredicateEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(LIKE_SYMBOL, &FOLLOW_LIKE_SYMBOL_in_predicate10151);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepredicateEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_predicate10153);
            	        bitwise_or_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepredicateEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulepredicateEx; /* Prevent compiler warnings */
    rulepredicateEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end predicate */

/**
 * $ANTLR start predicate_in
 * MySQLSimpleParser.g:1786:1: predicate_in : ({...}? => subquery | expression_list_with_parentheses );
 */
static void
predicate_in(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1786:13: ({...}? => subquery | expression_list_with_parentheses )

            ANTLR3_UINT32 alt620;

            alt620=2;

            switch ( LA(1) )
            {
            case OPEN_PAR_SYMBOL:
            	{
            		{
            		    int LA620_1 = LA(2);
            		    if ( ((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)) && (LA620_1 == SELECT_SYMBOL))
            		    {
            		        alt620=1;
            		    }
            		    else if ( (LA620_1 == OPEN_PAR_SYMBOL))
            		    {
            		        {
            		            int LA620_3 = LA(3);
            		            if ( ((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)))
            		            {
            		                alt620=1;
            		            }
            		            else if ( (ANTLR3_TRUE))
            		            {
            		                alt620=2;
            		            }
            		            else
            		            {
            		                if (BACKTRACKING>0)
            		                {
            		                    FAILEDFLAG = ANTLR3_TRUE;
            		                    return ;
            		                }


            		                CONSTRUCTEX();
            		                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		                EXCEPTION->message      = (void *)"";
            		                EXCEPTION->decisionNum  = 620;
            		                EXCEPTION->state        = 3;


            		                goto rulepredicate_inEx;

            		            }
            		        }
            		    }
            		    else if ( (((LA620_1 >= ACCOUNT_SYMBOL) && (LA620_1 <= YEAR_SYMBOL)) || LA620_1 == BINARY_SYMBOL || ((LA620_1 >= CASE_SYMBOL) && (LA620_1 <= CAST_SYMBOL)) || LA620_1 == CHAR_SYMBOL || ((LA620_1 >= CONVERT_SYMBOL) && (LA620_1 <= COUNT_SYMBOL)) || LA620_1 == CURDATE_SYMBOL || LA620_1 == CURRENT_USER_SYMBOL || LA620_1 == CURTIME_SYMBOL || LA620_1 == DATABASE_SYMBOL || LA620_1 == DATE_ADD_SYMBOL || LA620_1 == DATE_SUB_SYMBOL || LA620_1 == EXISTS_SYMBOL || ((LA620_1 >= EXTRACT_SYMBOL) && (LA620_1 <= FALSE_SYMBOL)) || LA620_1 == GROUP_CONCAT_SYMBOL || LA620_1 == IF_SYMBOL || LA620_1 == INSERT_SYMBOL || LA620_1 == INTERVAL_SYMBOL || LA620_1 == LEFT_SYMBOL || LA620_1 == MATCH_SYMBOL || LA620_1 == MAX_SYMBOL || LA620_1 == MIN_SYMBOL || LA620_1 == MOD_SYMBOL || LA620_1 == NCHAR_TEXT || ((LA620_1 >= NOT2_SYMBOL) && (LA620_1 <= NOW_SYMBOL)) || ((LA620_1 >= NULL2_SYMBOL) && (LA620_1 <= NULL_SYMBOL)) || LA620_1 == OPEN_CURLY_SYMBOL || ((LA620_1 >= PARAM_MARKER) && (LA620_1 <= POSITION_SYMBOL)) || ((LA620_1 >= REPEAT_SYMBOL) && (LA620_1 <= REPLACE_SYMBOL)) || LA620_1 == RIGHT_SYMBOL || LA620_1 == SINGLE_QUOTED_TEXT || LA620_1 == STDDEV_SAMP_SYMBOL || LA620_1 == STD_SYMBOL || LA620_1 == SUBSTRING_SYMBOL || ((LA620_1 >= SUM_SYMBOL) && (LA620_1 <= SYSDATE_SYMBOL)) || ((LA620_1 >= TRIM_SYMBOL) && (LA620_1 <= TRUE_SYMBOL)) || ((LA620_1 >= UTC_DATE_SYMBOL) && (LA620_1 <= UTC_TIME_SYMBOL)) || LA620_1 == VALUES_SYMBOL || LA620_1 == VARIANCE_SYMBOL || LA620_1 == VAR_SAMP_SYMBOL || ((LA620_1 >= AT_AT_SIGN_SYMBOL) && (LA620_1 <= AT_TEXT_SUFFIX)) || ((LA620_1 >= BACK_TICK_QUOTED_ID) && (LA620_1 <= BIN_NUMBER)) || LA620_1 == BITWISE_NOT_OPERATOR || ((LA620_1 >= BIT_AND_SYMBOL) && (LA620_1 <= BIT_XOR_SYMBOL)) || LA620_1 == DECIMAL_NUMBER || LA620_1 == DOT_SYMBOL || LA620_1 == DOUBLE_QUOTED_TEXT || LA620_1 == FLOAT_NUMBER || ((LA620_1 >= HEX_NUMBER) && (LA620_1 <= IDENTIFIER)) || LA620_1 == INT_NUMBER || LA620_1 == LOGICAL_NOT_OPERATOR || ((LA620_1 >= LONG_NUMBER) && (LA620_1 <= MINUS_OPERATOR)) || LA620_1 == PLUS_OPERATOR || ((LA620_1 >= ULONGLONG_NUMBER) && (LA620_1 <= UNDERSCORE_CHARSET))))
            		    {
            		        alt620=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 620;
            		        EXCEPTION->state        = 1;


            		        goto rulepredicate_inEx;

            		    }
            		}
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 620;
                EXCEPTION->state        = 0;


                goto rulepredicate_inEx;

            }

            switch (alt620)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1787:2: {...}? => subquery
        	    {
        	        if ( !((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"predicate_in";


        	        }

        	        FOLLOWPUSH(FOLLOW_subquery_in_predicate_in10172);
        	        subquery(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepredicate_inEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1788:4: expression_list_with_parentheses
        	    {
        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_predicate_in10177);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepredicate_inEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulepredicate_inEx; /* Prevent compiler warnings */
    rulepredicate_inEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end predicate_in */

/**
 * $ANTLR start bitwise_or_expression
 * MySQLSimpleParser.g:1791:1: bitwise_or_expression : bitwise_and_expression ( options {greedy=true; } : BITWISE_OR_OPERATOR bitwise_and_expression )* ;
 */
static void
bitwise_or_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1791:22: ( bitwise_and_expression ( options {greedy=true; } : BITWISE_OR_OPERATOR bitwise_and_expression )* )
        // MySQLSimpleParser.g:1792:2: bitwise_and_expression ( options {greedy=true; } : BITWISE_OR_OPERATOR bitwise_and_expression )*
        {
            FOLLOWPUSH(FOLLOW_bitwise_and_expression_in_bitwise_or_expression10186);
            bitwise_and_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulebitwise_or_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1792:25: ( options {greedy=true; } : BITWISE_OR_OPERATOR bitwise_and_expression )*

            for (;;)
            {
                int alt621=2;
                switch ( LA(1) )
                {
                case BITWISE_OR_OPERATOR:
                	{
                		alt621=1;
                	}
                    break;

                }

                switch (alt621)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1792:55: BITWISE_OR_OPERATOR bitwise_and_expression
            	    {
            	         MATCHT(BITWISE_OR_OPERATOR, &FOLLOW_BITWISE_OR_OPERATOR_in_bitwise_or_expression10202);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_or_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_bitwise_and_expression_in_bitwise_or_expression10204);
            	        bitwise_and_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_or_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop621;	/* break out of the loop */
            	    break;
                }
            }
            loop621: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulebitwise_or_expressionEx; /* Prevent compiler warnings */
    rulebitwise_or_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end bitwise_or_expression */

/**
 * $ANTLR start bitwise_and_expression
 * MySQLSimpleParser.g:1795:1: bitwise_and_expression : shift_expression ( options {greedy=true; } : BITWISE_AND_OPERATOR shift_expression )* ;
 */
static void
bitwise_and_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1795:23: ( shift_expression ( options {greedy=true; } : BITWISE_AND_OPERATOR shift_expression )* )
        // MySQLSimpleParser.g:1796:2: shift_expression ( options {greedy=true; } : BITWISE_AND_OPERATOR shift_expression )*
        {
            FOLLOWPUSH(FOLLOW_shift_expression_in_bitwise_and_expression10215);
            shift_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulebitwise_and_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1796:19: ( options {greedy=true; } : BITWISE_AND_OPERATOR shift_expression )*

            for (;;)
            {
                int alt622=2;
                switch ( LA(1) )
                {
                case BITWISE_AND_OPERATOR:
                	{
                		alt622=1;
                	}
                    break;

                }

                switch (alt622)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1796:49: BITWISE_AND_OPERATOR shift_expression
            	    {
            	         MATCHT(BITWISE_AND_OPERATOR, &FOLLOW_BITWISE_AND_OPERATOR_in_bitwise_and_expression10231);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_and_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_shift_expression_in_bitwise_and_expression10233);
            	        shift_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_and_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop622;	/* break out of the loop */
            	    break;
                }
            }
            loop622: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulebitwise_and_expressionEx; /* Prevent compiler warnings */
    rulebitwise_and_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end bitwise_and_expression */

/**
 * $ANTLR start shift_expression
 * MySQLSimpleParser.g:1799:1: shift_expression : additive_expression ( options {greedy=true; } : ( SHIFT_LEFT_OPERATOR | SHIFT_RIGHT_OPERATOR ) additive_expression )* ;
 */
static void
shift_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1799:17: ( additive_expression ( options {greedy=true; } : ( SHIFT_LEFT_OPERATOR | SHIFT_RIGHT_OPERATOR ) additive_expression )* )
        // MySQLSimpleParser.g:1800:2: additive_expression ( options {greedy=true; } : ( SHIFT_LEFT_OPERATOR | SHIFT_RIGHT_OPERATOR ) additive_expression )*
        {
            FOLLOWPUSH(FOLLOW_additive_expression_in_shift_expression10244);
            additive_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleshift_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1800:22: ( options {greedy=true; } : ( SHIFT_LEFT_OPERATOR | SHIFT_RIGHT_OPERATOR ) additive_expression )*

            for (;;)
            {
                int alt623=2;
                switch ( LA(1) )
                {
                case SHIFT_LEFT_OPERATOR:
                case SHIFT_RIGHT_OPERATOR:
                	{
                		alt623=1;
                	}
                    break;

                }

                switch (alt623)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1800:52: ( SHIFT_LEFT_OPERATOR | SHIFT_RIGHT_OPERATOR ) additive_expression
            	    {
            	        if ( ((LA(1) >= SHIFT_LEFT_OPERATOR) && (LA(1) <= SHIFT_RIGHT_OPERATOR)) )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleshift_expressionEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_additive_expression_in_shift_expression10268);
            	        additive_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleshift_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop623;	/* break out of the loop */
            	    break;
                }
            }
            loop623: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleshift_expressionEx; /* Prevent compiler warnings */
    ruleshift_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end shift_expression */

/**
 * $ANTLR start additive_expression
 * MySQLSimpleParser.g:1803:1: additive_expression : multiplicative_expression ( options {greedy=true; } : ( PLUS_OPERATOR | MINUS_OPERATOR ) multiplicative_expression )* ;
 */
static void
additive_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1803:20: ( multiplicative_expression ( options {greedy=true; } : ( PLUS_OPERATOR | MINUS_OPERATOR ) multiplicative_expression )* )
        // MySQLSimpleParser.g:1804:2: multiplicative_expression ( options {greedy=true; } : ( PLUS_OPERATOR | MINUS_OPERATOR ) multiplicative_expression )*
        {
            FOLLOWPUSH(FOLLOW_multiplicative_expression_in_additive_expression10279);
            multiplicative_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleadditive_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1804:28: ( options {greedy=true; } : ( PLUS_OPERATOR | MINUS_OPERATOR ) multiplicative_expression )*

            for (;;)
            {
                int alt624=2;
                switch ( LA(1) )
                {
                case MINUS_OPERATOR:
                case PLUS_OPERATOR:
                	{
                		alt624=1;
                	}
                    break;

                }

                switch (alt624)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1804:57: ( PLUS_OPERATOR | MINUS_OPERATOR ) multiplicative_expression
            	    {
            	        if ( LA(1) == MINUS_OPERATOR || LA(1) == PLUS_OPERATOR )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleadditive_expressionEx;
            	        }


            	        FOLLOWPUSH(FOLLOW_multiplicative_expression_in_additive_expression10303);
            	        multiplicative_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleadditive_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop624;	/* break out of the loop */
            	    break;
                }
            }
            loop624: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleadditive_expressionEx; /* Prevent compiler warnings */
    ruleadditive_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end additive_expression */

/**
 * $ANTLR start multiplicative_expression
 * MySQLSimpleParser.g:1807:1: multiplicative_expression : bitwise_xor_expression ( options {greedy=true; } : multiplication_operator bitwise_xor_expression )* ;
 */
static void
multiplicative_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1807:26: ( bitwise_xor_expression ( options {greedy=true; } : multiplication_operator bitwise_xor_expression )* )
        // MySQLSimpleParser.g:1808:2: bitwise_xor_expression ( options {greedy=true; } : multiplication_operator bitwise_xor_expression )*
        {
            FOLLOWPUSH(FOLLOW_bitwise_xor_expression_in_multiplicative_expression10314);
            bitwise_xor_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulemultiplicative_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1808:25: ( options {greedy=true; } : multiplication_operator bitwise_xor_expression )*

            for (;;)
            {
                int alt625=2;
                switch ( LA(1) )
                {
                case MOD_SYMBOL:
                case DIV_OPERATOR:
                case DIV_SYMBOL:
                case MOD_OPERATOR:
                case MULT_OPERATOR:
                	{
                		alt625=1;
                	}
                    break;

                }

                switch (alt625)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1808:55: multiplication_operator bitwise_xor_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_multiplication_operator_in_multiplicative_expression10330);
            	        multiplication_operator(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulemultiplicative_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_bitwise_xor_expression_in_multiplicative_expression10332);
            	        bitwise_xor_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulemultiplicative_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop625;	/* break out of the loop */
            	    break;
                }
            }
            loop625: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulemultiplicative_expressionEx; /* Prevent compiler warnings */
    rulemultiplicative_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end multiplicative_expression */

/**
 * $ANTLR start bitwise_xor_expression
 * MySQLSimpleParser.g:1811:1: bitwise_xor_expression : concat_expression ( options {greedy=true; } : BITWISE_XOR_OPERATOR concat_expression )* ;
 */
static void
bitwise_xor_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1811:23: ( concat_expression ( options {greedy=true; } : BITWISE_XOR_OPERATOR concat_expression )* )
        // MySQLSimpleParser.g:1812:2: concat_expression ( options {greedy=true; } : BITWISE_XOR_OPERATOR concat_expression )*
        {
            FOLLOWPUSH(FOLLOW_concat_expression_in_bitwise_xor_expression10343);
            concat_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulebitwise_xor_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1812:20: ( options {greedy=true; } : BITWISE_XOR_OPERATOR concat_expression )*

            for (;;)
            {
                int alt626=2;
                switch ( LA(1) )
                {
                case BITWISE_XOR_OPERATOR:
                	{
                		alt626=1;
                	}
                    break;

                }

                switch (alt626)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1812:50: BITWISE_XOR_OPERATOR concat_expression
            	    {
            	         MATCHT(BITWISE_XOR_OPERATOR, &FOLLOW_BITWISE_XOR_OPERATOR_in_bitwise_xor_expression10359);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_xor_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_concat_expression_in_bitwise_xor_expression10361);
            	        concat_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebitwise_xor_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop626;	/* break out of the loop */
            	    break;
                }
            }
            loop626: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulebitwise_xor_expressionEx; /* Prevent compiler warnings */
    rulebitwise_xor_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end bitwise_xor_expression */

/**
 * $ANTLR start concat_expression
 * MySQLSimpleParser.g:1816:1: concat_expression : unary_expression ( options {greedy=true; } : CONCAT_PIPES_SYMBOL unary_expression )* ;
 */
static void
concat_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1816:18: ( unary_expression ( options {greedy=true; } : CONCAT_PIPES_SYMBOL unary_expression )* )
        // MySQLSimpleParser.g:1817:2: unary_expression ( options {greedy=true; } : CONCAT_PIPES_SYMBOL unary_expression )*
        {
            FOLLOWPUSH(FOLLOW_unary_expression_in_concat_expression10373);
            unary_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleconcat_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1817:19: ( options {greedy=true; } : CONCAT_PIPES_SYMBOL unary_expression )*

            for (;;)
            {
                int alt627=2;
                switch ( LA(1) )
                {
                case CONCAT_PIPES_SYMBOL:
                	{
                		alt627=1;
                	}
                    break;

                }

                switch (alt627)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1817:49: CONCAT_PIPES_SYMBOL unary_expression
            	    {
            	         MATCHT(CONCAT_PIPES_SYMBOL, &FOLLOW_CONCAT_PIPES_SYMBOL_in_concat_expression10389);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleconcat_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_unary_expression_in_concat_expression10391);
            	        unary_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleconcat_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop627;	/* break out of the loop */
            	    break;
                }
            }
            loop627: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleconcat_expressionEx; /* Prevent compiler warnings */
    ruleconcat_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end concat_expression */

/**
 * $ANTLR start unary_expression
 * MySQLSimpleParser.g:1820:1: unary_expression : ( ( PLUS_OPERATOR | MINUS_OPERATOR | BITWISE_NOT_OPERATOR ) unary_expression | not_expression );
 */
static void
unary_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1820:17: ( ( PLUS_OPERATOR | MINUS_OPERATOR | BITWISE_NOT_OPERATOR ) unary_expression | not_expression )

            ANTLR3_UINT32 alt628;

            alt628=2;

            {
                int LA628_0 = LA(1);
                if ( (LA628_0 == BITWISE_NOT_OPERATOR || LA628_0 == MINUS_OPERATOR || LA628_0 == PLUS_OPERATOR))
                {
                    alt628=1;
                }
                else if ( (((LA628_0 >= ACCOUNT_SYMBOL) && (LA628_0 <= YEAR_SYMBOL)) || LA628_0 == BINARY_SYMBOL || ((LA628_0 >= CASE_SYMBOL) && (LA628_0 <= CAST_SYMBOL)) || LA628_0 == CHAR_SYMBOL || ((LA628_0 >= CONVERT_SYMBOL) && (LA628_0 <= COUNT_SYMBOL)) || LA628_0 == CURDATE_SYMBOL || LA628_0 == CURRENT_USER_SYMBOL || LA628_0 == CURTIME_SYMBOL || LA628_0 == DATABASE_SYMBOL || LA628_0 == DATE_ADD_SYMBOL || LA628_0 == DATE_SUB_SYMBOL || LA628_0 == EXISTS_SYMBOL || ((LA628_0 >= EXTRACT_SYMBOL) && (LA628_0 <= FALSE_SYMBOL)) || LA628_0 == GROUP_CONCAT_SYMBOL || LA628_0 == IF_SYMBOL || LA628_0 == INSERT_SYMBOL || LA628_0 == INTERVAL_SYMBOL || LA628_0 == LEFT_SYMBOL || LA628_0 == MATCH_SYMBOL || LA628_0 == MAX_SYMBOL || LA628_0 == MIN_SYMBOL || LA628_0 == MOD_SYMBOL || LA628_0 == NCHAR_TEXT || LA628_0 == NOT2_SYMBOL || LA628_0 == NOW_SYMBOL || ((LA628_0 >= NULL2_SYMBOL) && (LA628_0 <= NULL_SYMBOL)) || ((LA628_0 >= OPEN_CURLY_SYMBOL) && (LA628_0 <= OPEN_PAR_SYMBOL)) || ((LA628_0 >= PARAM_MARKER) && (LA628_0 <= POSITION_SYMBOL)) || ((LA628_0 >= REPEAT_SYMBOL) && (LA628_0 <= REPLACE_SYMBOL)) || LA628_0 == RIGHT_SYMBOL || LA628_0 == SINGLE_QUOTED_TEXT || LA628_0 == STDDEV_SAMP_SYMBOL || LA628_0 == STD_SYMBOL || LA628_0 == SUBSTRING_SYMBOL || ((LA628_0 >= SUM_SYMBOL) && (LA628_0 <= SYSDATE_SYMBOL)) || ((LA628_0 >= TRIM_SYMBOL) && (LA628_0 <= TRUE_SYMBOL)) || ((LA628_0 >= UTC_DATE_SYMBOL) && (LA628_0 <= UTC_TIME_SYMBOL)) || LA628_0 == VALUES_SYMBOL || LA628_0 == VARIANCE_SYMBOL || LA628_0 == VAR_SAMP_SYMBOL || ((LA628_0 >= AT_AT_SIGN_SYMBOL) && (LA628_0 <= AT_TEXT_SUFFIX)) || ((LA628_0 >= BACK_TICK_QUOTED_ID) && (LA628_0 <= BIN_NUMBER)) || ((LA628_0 >= BIT_AND_SYMBOL) && (LA628_0 <= BIT_XOR_SYMBOL)) || LA628_0 == DECIMAL_NUMBER || LA628_0 == DOT_SYMBOL || LA628_0 == DOUBLE_QUOTED_TEXT || LA628_0 == FLOAT_NUMBER || ((LA628_0 >= HEX_NUMBER) && (LA628_0 <= IDENTIFIER)) || LA628_0 == INT_NUMBER || LA628_0 == LOGICAL_NOT_OPERATOR || LA628_0 == LONG_NUMBER || ((LA628_0 >= ULONGLONG_NUMBER) && (LA628_0 <= UNDERSCORE_CHARSET))))
                {
                    alt628=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 628;
                    EXCEPTION->state        = 0;


                    goto ruleunary_expressionEx;

                }
            }
            switch (alt628)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1821:2: ( PLUS_OPERATOR | MINUS_OPERATOR | BITWISE_NOT_OPERATOR ) unary_expression
        	    {
        	        if ( LA(1) == BITWISE_NOT_OPERATOR || LA(1) == MINUS_OPERATOR || LA(1) == PLUS_OPERATOR )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleunary_expressionEx;
        	        }


        	        FOLLOWPUSH(FOLLOW_unary_expression_in_unary_expression10414);
        	        unary_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunary_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1822:4: not_expression
        	    {
        	        FOLLOWPUSH(FOLLOW_not_expression_in_unary_expression10419);
        	        not_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunary_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleunary_expressionEx; /* Prevent compiler warnings */
    ruleunary_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end unary_expression */

/**
 * $ANTLR start not_expression
 * MySQLSimpleParser.g:1825:1: not_expression : ( not2_rule )? interval_expression ;
 */
static void
not_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1825:15: ( ( not2_rule )? interval_expression )
        // MySQLSimpleParser.g:1826:2: ( not2_rule )? interval_expression
        {
            // MySQLSimpleParser.g:1826:2: ( not2_rule )?
            {
                int alt629=2;
                switch ( LA(1) )
                {
                    case NOT2_SYMBOL:
                    case LOGICAL_NOT_OPERATOR:
                    	{
                    		alt629=1;
                    	}
                        break;
                }

                switch (alt629)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1826:2: not2_rule
            	    {
            	        FOLLOWPUSH(FOLLOW_not2_rule_in_not_expression10428);
            	        not2_rule(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulenot_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_interval_expression_in_not_expression10431);
            interval_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulenot_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulenot_expressionEx; /* Prevent compiler warnings */
    rulenot_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end not_expression */

/**
 * $ANTLR start interval_expression
 * MySQLSimpleParser.g:1829:1: interval_expression : ( INTERVAL_SYMBOL ( ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )=> interval_function | interval_time_span ) | primary );
 */
static void
interval_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1829:20: ( INTERVAL_SYMBOL ( ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )=> interval_function | interval_time_span ) | primary )

            ANTLR3_UINT32 alt631;

            alt631=2;

            {
                int LA631_0 = LA(1);
                if ( (LA631_0 == INTERVAL_SYMBOL))
                {
                    alt631=1;
                }
                else if ( (((LA631_0 >= ACCOUNT_SYMBOL) && (LA631_0 <= YEAR_SYMBOL)) || LA631_0 == BINARY_SYMBOL || ((LA631_0 >= CASE_SYMBOL) && (LA631_0 <= CAST_SYMBOL)) || LA631_0 == CHAR_SYMBOL || ((LA631_0 >= CONVERT_SYMBOL) && (LA631_0 <= COUNT_SYMBOL)) || LA631_0 == CURDATE_SYMBOL || LA631_0 == CURRENT_USER_SYMBOL || LA631_0 == CURTIME_SYMBOL || LA631_0 == DATABASE_SYMBOL || LA631_0 == DATE_ADD_SYMBOL || LA631_0 == DATE_SUB_SYMBOL || LA631_0 == EXISTS_SYMBOL || ((LA631_0 >= EXTRACT_SYMBOL) && (LA631_0 <= FALSE_SYMBOL)) || LA631_0 == GROUP_CONCAT_SYMBOL || LA631_0 == IF_SYMBOL || LA631_0 == INSERT_SYMBOL || LA631_0 == LEFT_SYMBOL || LA631_0 == MATCH_SYMBOL || LA631_0 == MAX_SYMBOL || LA631_0 == MIN_SYMBOL || LA631_0 == MOD_SYMBOL || LA631_0 == NCHAR_TEXT || LA631_0 == NOW_SYMBOL || ((LA631_0 >= NULL2_SYMBOL) && (LA631_0 <= NULL_SYMBOL)) || ((LA631_0 >= OPEN_CURLY_SYMBOL) && (LA631_0 <= OPEN_PAR_SYMBOL)) || ((LA631_0 >= PARAM_MARKER) && (LA631_0 <= POSITION_SYMBOL)) || ((LA631_0 >= REPEAT_SYMBOL) && (LA631_0 <= REPLACE_SYMBOL)) || LA631_0 == RIGHT_SYMBOL || LA631_0 == SINGLE_QUOTED_TEXT || LA631_0 == STDDEV_SAMP_SYMBOL || LA631_0 == STD_SYMBOL || LA631_0 == SUBSTRING_SYMBOL || ((LA631_0 >= SUM_SYMBOL) && (LA631_0 <= SYSDATE_SYMBOL)) || ((LA631_0 >= TRIM_SYMBOL) && (LA631_0 <= TRUE_SYMBOL)) || ((LA631_0 >= UTC_DATE_SYMBOL) && (LA631_0 <= UTC_TIME_SYMBOL)) || LA631_0 == VALUES_SYMBOL || LA631_0 == VARIANCE_SYMBOL || LA631_0 == VAR_SAMP_SYMBOL || ((LA631_0 >= AT_AT_SIGN_SYMBOL) && (LA631_0 <= AT_TEXT_SUFFIX)) || ((LA631_0 >= BACK_TICK_QUOTED_ID) && (LA631_0 <= BIN_NUMBER)) || ((LA631_0 >= BIT_AND_SYMBOL) && (LA631_0 <= BIT_XOR_SYMBOL)) || LA631_0 == DECIMAL_NUMBER || LA631_0 == DOT_SYMBOL || LA631_0 == DOUBLE_QUOTED_TEXT || LA631_0 == FLOAT_NUMBER || ((LA631_0 >= HEX_NUMBER) && (LA631_0 <= IDENTIFIER)) || LA631_0 == INT_NUMBER || LA631_0 == LONG_NUMBER || ((LA631_0 >= ULONGLONG_NUMBER) && (LA631_0 <= UNDERSCORE_CHARSET))))
                {
                    alt631=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 631;
                    EXCEPTION->state        = 0;


                    goto ruleinterval_expressionEx;

                }
            }
            switch (alt631)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1830:2: INTERVAL_SYMBOL ( ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )=> interval_function | interval_time_span )
        	    {
        	         MATCHT(INTERVAL_SYMBOL, &FOLLOW_INTERVAL_SYMBOL_in_interval_expression10440);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1831:3: ( ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )=> interval_function | interval_time_span )
        	        {
        	            int alt630=2;
        	            {
        	                int LA630_0 = LA(1);
        	                if ( (LA630_0 == OPEN_PAR_SYMBOL))
        	                {
        	                    {
        	                        int LA630_1 = LA(2);
        	                        if ( (LA630_1 == NOT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_103 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 103;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BITWISE_NOT_OPERATOR || LA630_1 == MINUS_OPERATOR || LA630_1 == PLUS_OPERATOR))
        	                        {
        	                            {
        	                                int LA630_104 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 104;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == NOT2_SYMBOL || LA630_1 == LOGICAL_NOT_OPERATOR))
        	                        {
        	                            {
        	                                int LA630_105 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 105;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == INTERVAL_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_106 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 106;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == NCHAR_TEXT))
        	                        {
        	                            {
        	                                int LA630_107 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 107;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == UNDERSCORE_CHARSET))
        	                        {
        	                            {
        	                                int LA630_108 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 108;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SINGLE_QUOTED_TEXT))
        	                        {
        	                            {
        	                                int LA630_109 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 109;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DOUBLE_QUOTED_TEXT))
        	                        {
        	                            {
        	                                int LA630_110 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 110;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DECIMAL_NUMBER || LA630_1 == FLOAT_NUMBER || LA630_1 == INT_NUMBER || LA630_1 == LONG_NUMBER || LA630_1 == ULONGLONG_NUMBER))
        	                        {
        	                            {
        	                                int LA630_111 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 111;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_112 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 112;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == TIME_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_113 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 113;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == TIMESTAMP_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_114 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 114;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (((LA630_1 >= NULL2_SYMBOL) && (LA630_1 <= NULL_SYMBOL))))
        	                        {
        	                            {
        	                                int LA630_115 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 115;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == FALSE_SYMBOL || LA630_1 == TRUE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_116 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 116;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == HEX_NUMBER))
        	                        {
        	                            {
        	                                int LA630_117 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 117;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BIN_NUMBER))
        	                        {
        	                            {
        	                                int LA630_118 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 118;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == IDENTIFIER))
        	                        {
        	                            {
        	                                int LA630_119 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 119;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BACK_TICK_QUOTED_ID))
        	                        {
        	                            {
        	                                int LA630_120 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 120;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CHAR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_121 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 121;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CURRENT_USER_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_122 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 122;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DAY_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_123 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 123;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == HOUR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_124 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 124;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == INSERT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_125 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 125;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == LEFT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_126 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 126;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MINUTE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_127 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 127;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MONTH_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_128 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 128;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == RIGHT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_129 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 129;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SECOND_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_130 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 130;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == TRIM_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_131 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 131;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == USER_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_132 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 132;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == VALUES_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_133 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 133;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == YEAR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_134 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 134;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ADDDATE_SYMBOL || LA630_1 == SUBDATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_135 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 135;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CURDATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_136 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 136;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CURTIME_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_137 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 137;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DATE_ADD_SYMBOL || LA630_1 == DATE_SUB_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_138 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 138;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == EXTRACT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_139 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 139;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == GET_FORMAT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_140 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 140;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == NOW_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_141 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 141;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == POSITION_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_142 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 142;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SUBSTRING_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_143 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 143;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SYSDATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_144 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 144;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (((LA630_1 >= TIMESTAMP_ADD_SYMBOL) && (LA630_1 <= TIMESTAMP_DIFF_SYMBOL))))
        	                        {
        	                            {
        	                                int LA630_145 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 145;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == UTC_DATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_146 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 146;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == UTC_TIME_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_147 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 147;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == UTC_TIMESTAMP_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_148 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 148;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ASCII_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_149 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 149;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CHARSET_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_150 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 150;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == COALESCE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_151 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 151;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == COLLATION_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_152 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 152;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DATABASE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_153 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 153;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == IF_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_154 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 154;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == FORMAT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_155 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 155;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MICROSECOND_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_156 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 156;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MOD_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_157 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 157;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == OLD_PASSWORD_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_158 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 158;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == PASSWORD_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_159 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 159;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == QUARTER_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_160 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 160;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == REPEAT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_161 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 161;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == REPLACE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_162 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 162;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == REVERSE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_163 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 163;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ROW_COUNT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_164 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 164;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == TRUNCATE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_165 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 165;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == WEEK_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_166 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 166;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == WEIGHT_STRING_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_167 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 167;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == GEOMETRYCOLLECTION_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_168 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 168;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == LINESTRING_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_169 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 169;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MULTILINESTRING_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_170 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 170;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MULTIPOINT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_171 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 171;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MULTIPOLYGON_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_172 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 172;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == POINT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_173 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 173;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == POLYGON_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_174 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 174;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CONTAINS_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_175 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 175;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == AVG_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_176 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 176;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BIT_AND_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_177 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 177;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BIT_OR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_178 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 178;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BIT_XOR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_179 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 179;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == COUNT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_180 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 180;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MIN_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_181 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 181;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MAX_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_182 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 182;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == STD_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_183 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 183;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == VARIANCE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_184 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 184;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == STDDEV_SAMP_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_185 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 185;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == VAR_SAMP_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_186 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 186;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SUM_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_187 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 187;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == GROUP_CONCAT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_188 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 188;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == DOT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_189 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 189;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ROW_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_190 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 190;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ACCOUNT_SYMBOL || ((LA630_1 >= ALWAYS_SYMBOL) && (LA630_1 <= CACHE_SYMBOL)) || ((LA630_1 >= CHECKSUM_SYMBOL) && (LA630_1 <= COMMIT_SYMBOL)) || ((LA630_1 >= DEALLOCATE_SYMBOL) && (LA630_1 <= FOLLOWS_SYMBOL)) || ((LA630_1 >= GROUP_REPLICATION_SYMBOL) && (LA630_1 <= STOP_SYMBOL)) || ((LA630_1 >= UNICODE_SYMBOL) && (LA630_1 <= XA_SYMBOL)) || LA630_1 == SHUTDOWN_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_191 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 191;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == PARAM_MARKER))
        	                        {
        	                            {
        	                                int LA630_192 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 192;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == AT_SIGN_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_193 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 193;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == AT_TEXT_SUFFIX))
        	                        {
        	                            {
        	                                int LA630_194 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 194;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == AT_AT_SIGN_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_195 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 195;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == EXISTS_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_196 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 196;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == OPEN_PAR_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_197 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 197;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == ACTION_SYMBOL || ((LA630_1 >= AFTER_SYMBOL) && (LA630_1 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA630_1 >= BINLOG_SYMBOL) && (LA630_1 <= CLASS_ORIGIN_SYMBOL)) || LA630_1 == CODE_SYMBOL || ((LA630_1 >= COLUMN_NAME_SYMBOL) && (LA630_1 <= DATETIME_SYMBOL)) || ((LA630_1 >= DEFAULT_AUTH_SYMBOL) && (LA630_1 <= GEOMETRY_SYMBOL)) || ((LA630_1 >= GRANTS_SYMBOL) && (LA630_1 <= HOSTS_SYMBOL)) || ((LA630_1 >= IDENTIFIED_SYMBOL) && (LA630_1 <= LEVEL_SYMBOL)) || ((LA630_1 >= LIST_SYMBOL) && (LA630_1 <= MESSAGE_TEXT_SYMBOL)) || LA630_1 == MIGRATE_SYMBOL || ((LA630_1 >= MIN_ROWS_SYMBOL) && (LA630_1 <= MODE_SYMBOL)) || ((LA630_1 >= MUTEX_SYMBOL) && (LA630_1 <= OFFSET_SYMBOL)) || ((LA630_1 >= ONE_SHOT_SYMBOL) && (LA630_1 <= PARTITIONS_SYMBOL)) || ((LA630_1 >= PHASE_SYMBOL) && (LA630_1 <= PLUGINS_SYMBOL)) || ((LA630_1 >= PRESERVE_SYMBOL) && (LA630_1 <= PROXY_SYMBOL)) || ((LA630_1 >= QUERY_SYMBOL) && (LA630_1 <= RETURNS_SYMBOL)) || ((LA630_1 >= ROLLUP_SYMBOL) && (LA630_1 <= ROWS_SYMBOL)) || LA630_1 == ROW_FORMAT_SYMBOL || ((LA630_1 >= RTREE_SYMBOL) && (LA630_1 <= SCHEMA_NAME_SYMBOL)) || ((LA630_1 >= SERIAL_SYMBOL) && (LA630_1 <= SHARE_SYMBOL)) || ((LA630_1 >= SLOW_SYMBOL) && (LA630_1 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA630_1 >= SUBJECT_SYMBOL) && (LA630_1 <= TRIGGERS_SYMBOL)) || ((LA630_1 >= TYPES_SYMBOL) && (LA630_1 <= USER_RESOURCES_SYMBOL)) || ((LA630_1 >= USE_FRM_SYMBOL) && (LA630_1 <= WAIT_SYMBOL)) || LA630_1 == WORK_SYMBOL || ((LA630_1 >= X509_SYMBOL) && (LA630_1 <= XML_SYMBOL))))
        	                        {
        	                            {
        	                                int LA630_198 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 198;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == OPEN_CURLY_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_199 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 199;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == MATCH_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_200 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 200;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CASE_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_201 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 201;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == BINARY_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_202 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 202;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CAST_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_203 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 203;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == CONVERT_SYMBOL))
        	                        {
        	                            {
        	                                int LA630_204 = LA(3);
        	                                if ( (synpred2_MySQLSimpleParser(ctx)))
        	                                {
        	                                    alt630=1;
        	                                }
        	                                else if ( (ANTLR3_TRUE))
        	                                {
        	                                    alt630=2;
        	                                }
        	                                else
        	                                {
        	                                    if (BACKTRACKING>0)
        	                                    {
        	                                        FAILEDFLAG = ANTLR3_TRUE;
        	                                        return ;
        	                                    }


        	                                    CONSTRUCTEX();
        	                                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                                    EXCEPTION->message      = (void *)"";
        	                                    EXCEPTION->decisionNum  = 630;
        	                                    EXCEPTION->state        = 204;


        	                                    goto ruleinterval_expressionEx;

        	                                }
        	                            }
        	                        }
        	                        else if ( (LA630_1 == SELECT_SYMBOL))
        	                        {
        	                            alt630=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 630;
        	                            EXCEPTION->state        = 1;


        	                            goto ruleinterval_expressionEx;

        	                        }
        	                    }
        	                }
        	                else if ( (((LA630_0 >= ACCOUNT_SYMBOL) && (LA630_0 <= YEAR_SYMBOL)) || LA630_0 == BINARY_SYMBOL || ((LA630_0 >= CASE_SYMBOL) && (LA630_0 <= CAST_SYMBOL)) || LA630_0 == CHAR_SYMBOL || ((LA630_0 >= CONVERT_SYMBOL) && (LA630_0 <= COUNT_SYMBOL)) || LA630_0 == CURDATE_SYMBOL || LA630_0 == CURRENT_USER_SYMBOL || LA630_0 == CURTIME_SYMBOL || LA630_0 == DATABASE_SYMBOL || LA630_0 == DATE_ADD_SYMBOL || LA630_0 == DATE_SUB_SYMBOL || LA630_0 == EXISTS_SYMBOL || ((LA630_0 >= EXTRACT_SYMBOL) && (LA630_0 <= FALSE_SYMBOL)) || LA630_0 == GROUP_CONCAT_SYMBOL || LA630_0 == IF_SYMBOL || LA630_0 == INSERT_SYMBOL || LA630_0 == INTERVAL_SYMBOL || LA630_0 == LEFT_SYMBOL || LA630_0 == MATCH_SYMBOL || LA630_0 == MAX_SYMBOL || LA630_0 == MIN_SYMBOL || LA630_0 == MOD_SYMBOL || LA630_0 == NCHAR_TEXT || ((LA630_0 >= NOT2_SYMBOL) && (LA630_0 <= NOW_SYMBOL)) || ((LA630_0 >= NULL2_SYMBOL) && (LA630_0 <= NULL_SYMBOL)) || LA630_0 == OPEN_CURLY_SYMBOL || ((LA630_0 >= PARAM_MARKER) && (LA630_0 <= POSITION_SYMBOL)) || ((LA630_0 >= REPEAT_SYMBOL) && (LA630_0 <= REPLACE_SYMBOL)) || LA630_0 == RIGHT_SYMBOL || LA630_0 == SINGLE_QUOTED_TEXT || LA630_0 == STDDEV_SAMP_SYMBOL || LA630_0 == STD_SYMBOL || LA630_0 == SUBSTRING_SYMBOL || ((LA630_0 >= SUM_SYMBOL) && (LA630_0 <= SYSDATE_SYMBOL)) || ((LA630_0 >= TRIM_SYMBOL) && (LA630_0 <= TRUE_SYMBOL)) || ((LA630_0 >= UTC_DATE_SYMBOL) && (LA630_0 <= UTC_TIME_SYMBOL)) || LA630_0 == VALUES_SYMBOL || LA630_0 == VARIANCE_SYMBOL || LA630_0 == VAR_SAMP_SYMBOL || ((LA630_0 >= AT_AT_SIGN_SYMBOL) && (LA630_0 <= AT_TEXT_SUFFIX)) || ((LA630_0 >= BACK_TICK_QUOTED_ID) && (LA630_0 <= BIN_NUMBER)) || LA630_0 == BITWISE_NOT_OPERATOR || ((LA630_0 >= BIT_AND_SYMBOL) && (LA630_0 <= BIT_XOR_SYMBOL)) || LA630_0 == DECIMAL_NUMBER || LA630_0 == DOT_SYMBOL || LA630_0 == DOUBLE_QUOTED_TEXT || LA630_0 == FLOAT_NUMBER || ((LA630_0 >= HEX_NUMBER) && (LA630_0 <= IDENTIFIER)) || LA630_0 == INT_NUMBER || LA630_0 == LOGICAL_NOT_OPERATOR || ((LA630_0 >= LONG_NUMBER) && (LA630_0 <= MINUS_OPERATOR)) || LA630_0 == PLUS_OPERATOR || ((LA630_0 >= ULONGLONG_NUMBER) && (LA630_0 <= UNDERSCORE_CHARSET))))
        	                {
        	                    alt630=2;
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 630;
        	                    EXCEPTION->state        = 0;


        	                    goto ruleinterval_expressionEx;

        	                }
        	            }
        	            switch (alt630)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1835:4: ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )=> interval_function
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_interval_function_in_interval_expression10471);
        	        	        interval_function(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinterval_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1836:6: interval_time_span
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_interval_time_span_in_interval_expression10478);
        	        	        interval_time_span(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleinterval_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1838:4: primary
        	    {
        	        FOLLOWPUSH(FOLLOW_primary_in_interval_expression10487);
        	        primary(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleinterval_expressionEx; /* Prevent compiler warnings */
    ruleinterval_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval_expression */

/**
 * $ANTLR start interval_function
 * MySQLSimpleParser.g:1841:1: interval_function : OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL ;
 */
static void
interval_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1841:18: ( OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:1842:2: OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_interval_function10496);
            if  (HASEXCEPTION())
            {
                goto ruleinterval_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_interval_function10498);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleinterval_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1842:29: ( COMMA_SYMBOL expression )+
            {
                int cnt632=0;

                for (;;)
                {
                    int alt632=2;
            	switch ( LA(1) )
            	{
            	case COMMA_SYMBOL:
            		{
            			alt632=1;
            		}
            	    break;

            	}

            	switch (alt632)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:1842:30: COMMA_SYMBOL expression
            	        {
            	             MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_interval_function10501);
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleinterval_functionEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_expression_in_interval_function10503);
            	            expression(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto ruleinterval_functionEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt632 >= 1 )
            		{
            		    goto loop632;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto ruleinterval_functionEx;
            	}
            	cnt632++;
                }
                loop632: ;	/* Jump to here if this rule does not match */
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_interval_function10507);
            if  (HASEXCEPTION())
            {
                goto ruleinterval_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinterval_functionEx; /* Prevent compiler warnings */
    ruleinterval_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval_function */

/**
 * $ANTLR start interval_time_span
 * MySQLSimpleParser.g:1845:1: interval_time_span : interval ;
 */
static void
interval_time_span(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1845:19: ( interval )
        // MySQLSimpleParser.g:1846:2: interval
        {
            FOLLOWPUSH(FOLLOW_interval_in_interval_time_span10516);
            interval(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleinterval_time_spanEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinterval_time_spanEx; /* Prevent compiler warnings */
    ruleinterval_time_spanEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval_time_span */

/**
 * $ANTLR start primary
 * MySQLSimpleParser.g:1849:1: primary : ( options {k=4; } : literal | function_call | runtime_function_call | column_ref ( jsonOperator )? | PARAM_MARKER | variable | EXISTS_SYMBOL subquery | expression_with_nested_parentheses | ROW_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL | OPEN_CURLY_SYMBOL identifier expression CLOSE_CURLY_SYMBOL | match_expression | case_expression | cast_expression ) ( options {greedy=true; } : COLLATE_SYMBOL collation_name )* ;
 */
static void
primary(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1849:8: ( ( options {k=4; } : literal | function_call | runtime_function_call | column_ref ( jsonOperator )? | PARAM_MARKER | variable | EXISTS_SYMBOL subquery | expression_with_nested_parentheses | ROW_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL | OPEN_CURLY_SYMBOL identifier expression CLOSE_CURLY_SYMBOL | match_expression | case_expression | cast_expression ) ( options {greedy=true; } : COLLATE_SYMBOL collation_name )* )
        // MySQLSimpleParser.g:1850:5: ( options {k=4; } : literal | function_call | runtime_function_call | column_ref ( jsonOperator )? | PARAM_MARKER | variable | EXISTS_SYMBOL subquery | expression_with_nested_parentheses | ROW_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL | OPEN_CURLY_SYMBOL identifier expression CLOSE_CURLY_SYMBOL | match_expression | case_expression | cast_expression ) ( options {greedy=true; } : COLLATE_SYMBOL collation_name )*
        {
            // MySQLSimpleParser.g:1850:5: ( options {k=4; } : literal | function_call | runtime_function_call | column_ref ( jsonOperator )? | PARAM_MARKER | variable | EXISTS_SYMBOL subquery | expression_with_nested_parentheses | ROW_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL | OPEN_CURLY_SYMBOL identifier expression CLOSE_CURLY_SYMBOL | match_expression | case_expression | cast_expression )
            {
                int alt635=13;
                {
                    int LA635_0 = LA(1);
                    if ( (LA635_0 == FALSE_SYMBOL || LA635_0 == NCHAR_TEXT || ((LA635_0 >= NULL2_SYMBOL) && (LA635_0 <= NULL_SYMBOL)) || LA635_0 == SINGLE_QUOTED_TEXT || LA635_0 == TRUE_SYMBOL || LA635_0 == BIN_NUMBER || LA635_0 == DECIMAL_NUMBER || LA635_0 == FLOAT_NUMBER || LA635_0 == HEX_NUMBER || LA635_0 == INT_NUMBER || LA635_0 == LONG_NUMBER || ((LA635_0 >= ULONGLONG_NUMBER) && (LA635_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt635=1;
                    }
                    else if ( (LA635_0 == DOUBLE_QUOTED_TEXT))
                    {
                        {
                            int LA635_4 = LA(2);
                            if ( (LA635_4 == OPEN_PAR_SYMBOL || LA635_4 == DOT_SYMBOL))
                            {
                                alt635=2;
                            }
                            else if ( ((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
                            {
                                alt635=1;
                            }
                            else if ( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 4;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == DATE_SYMBOL))
                    {
                        {
                            int LA635_6 = LA(2);
                            if ( (LA635_6 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA635_187 = LA(3);
                                    if ( ((LA(1) == DATE_SYMBOL || LA(1) == TIME_SYMBOL || LA(1) == TIMESTAMP_SYMBOL)))
                                    {
                                        alt635=1;
                                    }
                                    else if ( (ANTLR3_TRUE))
                                    {
                                        alt635=4;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 635;
                                        EXCEPTION->state        = 187;


                                        goto ruleprimaryEx;

                                    }
                                }
                            }
                            else if ( (LA635_6 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_6 == EOF || ((LA635_6 >= ACCOUNT_SYMBOL) && (LA635_6 <= YEAR_SYMBOL)) || ((LA635_6 >= AND_SYMBOL) && (LA635_6 <= ASC_SYMBOL)) || LA635_6 == BETWEEN_SYMBOL || LA635_6 == COLLATE_SYMBOL || LA635_6 == CROSS_SYMBOL || ((LA635_6 >= DAY_HOUR_SYMBOL) && (LA635_6 <= DAY_SECOND_SYMBOL)) || LA635_6 == DESC_SYMBOL || LA635_6 == ELSE_SYMBOL || ((LA635_6 >= FOR_SYMBOL) && (LA635_6 <= FROM_SYMBOL)) || ((LA635_6 >= GROUP_SYMBOL) && (LA635_6 <= HAVING_SYMBOL)) || ((LA635_6 >= HOUR_MICROSECOND_SYMBOL) && (LA635_6 <= HOUR_SECOND_SYMBOL)) || LA635_6 == INNER_SYMBOL || LA635_6 == INTO_SYMBOL || LA635_6 == IN_SYMBOL || LA635_6 == IS_SYMBOL || ((LA635_6 >= JOIN_SYMBOL) && (LA635_6 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_6 >= LEFT_SYMBOL) && (LA635_6 <= LIMIT_SYMBOL)) || LA635_6 == LOCK_SYMBOL || ((LA635_6 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_6 <= MINUTE_SECOND_SYMBOL)) || ((LA635_6 >= MOD_SYMBOL) && (LA635_6 <= NATURAL_SYMBOL)) || LA635_6 == NOT_SYMBOL || LA635_6 == ON_SYMBOL || ((LA635_6 >= ORDER_SYMBOL) && (LA635_6 <= OR_SYMBOL)) || LA635_6 == PROCEDURE_SYMBOL || LA635_6 == REGEXP_SYMBOL || LA635_6 == RENAME_SYMBOL || LA635_6 == RIGHT_SYMBOL || LA635_6 == SECOND_MICROSECOND_SYMBOL || LA635_6 == SEMICOLON_SYMBOL || LA635_6 == SEPARATOR_SYMBOL || LA635_6 == SET_SYMBOL || LA635_6 == STRAIGHT_JOIN_SYMBOL || LA635_6 == THEN_SYMBOL || LA635_6 == UNION_SYMBOL || LA635_6 == USING_SYMBOL || ((LA635_6 >= WHEN_SYMBOL) && (LA635_6 <= WHERE_SYMBOL)) || LA635_6 == WITH_SYMBOL || ((LA635_6 >= XOR_SYMBOL) && (LA635_6 <= YEAR_MONTH_SYMBOL)) || ((LA635_6 >= AS_SYMBOL) && (LA635_6 <= AT_TEXT_SUFFIX)) || ((LA635_6 >= BACK_TICK_QUOTED_ID) && (LA635_6 <= BITWISE_AND_OPERATOR)) || ((LA635_6 >= BITWISE_OR_OPERATOR) && (LA635_6 <= BITWISE_XOR_OPERATOR)) || ((LA635_6 >= CLOSE_CURLY_SYMBOL) && (LA635_6 <= CLOSE_PAR_SYMBOL)) || ((LA635_6 >= COMMA_SYMBOL) && (LA635_6 <= CONCAT_PIPES_SYMBOL)) || ((LA635_6 >= DIV_OPERATOR) && (LA635_6 <= DIV_SYMBOL)) || LA635_6 == DOT_SYMBOL || LA635_6 == DOUBLE_QUOTED_TEXT || LA635_6 == EQUAL_OPERATOR || ((LA635_6 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_6 <= GREATER_THAN_OPERATOR)) || ((LA635_6 >= HEX_NUMBER) && (LA635_6 <= IDENTIFIER)) || LA635_6 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_6 >= LESS_OR_EQUAL_OPERATOR) && (LA635_6 <= LESS_THAN_OPERATOR)) || LA635_6 == LOGICAL_AND_OPERATOR || LA635_6 == LOGICAL_OR_OPERATOR || ((LA635_6 >= MINUS_OPERATOR) && (LA635_6 <= MOD_OPERATOR)) || ((LA635_6 >= MULT_OPERATOR) && (LA635_6 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_6 == PLUS_OPERATOR || ((LA635_6 >= SHIFT_LEFT_OPERATOR) && (LA635_6 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 6;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == TIME_SYMBOL))
                    {
                        {
                            int LA635_7 = LA(2);
                            if ( (LA635_7 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA635_275 = LA(3);
                                    if ( ((LA(1) == DATE_SYMBOL || LA(1) == TIME_SYMBOL || LA(1) == TIMESTAMP_SYMBOL)))
                                    {
                                        alt635=1;
                                    }
                                    else if ( (ANTLR3_TRUE))
                                    {
                                        alt635=4;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 635;
                                        EXCEPTION->state        = 275;


                                        goto ruleprimaryEx;

                                    }
                                }
                            }
                            else if ( (LA635_7 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_7 == EOF || ((LA635_7 >= ACCOUNT_SYMBOL) && (LA635_7 <= YEAR_SYMBOL)) || ((LA635_7 >= AND_SYMBOL) && (LA635_7 <= ASC_SYMBOL)) || LA635_7 == BETWEEN_SYMBOL || LA635_7 == COLLATE_SYMBOL || LA635_7 == CROSS_SYMBOL || ((LA635_7 >= DAY_HOUR_SYMBOL) && (LA635_7 <= DAY_SECOND_SYMBOL)) || LA635_7 == DESC_SYMBOL || LA635_7 == ELSE_SYMBOL || ((LA635_7 >= FOR_SYMBOL) && (LA635_7 <= FROM_SYMBOL)) || ((LA635_7 >= GROUP_SYMBOL) && (LA635_7 <= HAVING_SYMBOL)) || ((LA635_7 >= HOUR_MICROSECOND_SYMBOL) && (LA635_7 <= HOUR_SECOND_SYMBOL)) || LA635_7 == INNER_SYMBOL || LA635_7 == INTO_SYMBOL || LA635_7 == IN_SYMBOL || LA635_7 == IS_SYMBOL || ((LA635_7 >= JOIN_SYMBOL) && (LA635_7 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_7 >= LEFT_SYMBOL) && (LA635_7 <= LIMIT_SYMBOL)) || LA635_7 == LOCK_SYMBOL || ((LA635_7 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_7 <= MINUTE_SECOND_SYMBOL)) || ((LA635_7 >= MOD_SYMBOL) && (LA635_7 <= NATURAL_SYMBOL)) || LA635_7 == NOT_SYMBOL || LA635_7 == ON_SYMBOL || ((LA635_7 >= ORDER_SYMBOL) && (LA635_7 <= OR_SYMBOL)) || LA635_7 == PROCEDURE_SYMBOL || LA635_7 == REGEXP_SYMBOL || LA635_7 == RENAME_SYMBOL || LA635_7 == RIGHT_SYMBOL || LA635_7 == SECOND_MICROSECOND_SYMBOL || LA635_7 == SEMICOLON_SYMBOL || LA635_7 == SEPARATOR_SYMBOL || LA635_7 == SET_SYMBOL || LA635_7 == STRAIGHT_JOIN_SYMBOL || LA635_7 == THEN_SYMBOL || LA635_7 == UNION_SYMBOL || LA635_7 == USING_SYMBOL || ((LA635_7 >= WHEN_SYMBOL) && (LA635_7 <= WHERE_SYMBOL)) || LA635_7 == WITH_SYMBOL || ((LA635_7 >= XOR_SYMBOL) && (LA635_7 <= YEAR_MONTH_SYMBOL)) || ((LA635_7 >= AS_SYMBOL) && (LA635_7 <= AT_TEXT_SUFFIX)) || ((LA635_7 >= BACK_TICK_QUOTED_ID) && (LA635_7 <= BITWISE_AND_OPERATOR)) || ((LA635_7 >= BITWISE_OR_OPERATOR) && (LA635_7 <= BITWISE_XOR_OPERATOR)) || ((LA635_7 >= CLOSE_CURLY_SYMBOL) && (LA635_7 <= CLOSE_PAR_SYMBOL)) || ((LA635_7 >= COMMA_SYMBOL) && (LA635_7 <= CONCAT_PIPES_SYMBOL)) || ((LA635_7 >= DIV_OPERATOR) && (LA635_7 <= DIV_SYMBOL)) || LA635_7 == DOT_SYMBOL || LA635_7 == DOUBLE_QUOTED_TEXT || LA635_7 == EQUAL_OPERATOR || ((LA635_7 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_7 <= GREATER_THAN_OPERATOR)) || ((LA635_7 >= HEX_NUMBER) && (LA635_7 <= IDENTIFIER)) || LA635_7 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_7 >= LESS_OR_EQUAL_OPERATOR) && (LA635_7 <= LESS_THAN_OPERATOR)) || LA635_7 == LOGICAL_AND_OPERATOR || LA635_7 == LOGICAL_OR_OPERATOR || ((LA635_7 >= MINUS_OPERATOR) && (LA635_7 <= MOD_OPERATOR)) || ((LA635_7 >= MULT_OPERATOR) && (LA635_7 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_7 == PLUS_OPERATOR || ((LA635_7 >= SHIFT_LEFT_OPERATOR) && (LA635_7 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 7;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == TIMESTAMP_SYMBOL))
                    {
                        {
                            int LA635_8 = LA(2);
                            if ( (LA635_8 == SINGLE_QUOTED_TEXT))
                            {
                                {
                                    int LA635_363 = LA(3);
                                    if ( ((LA(1) == DATE_SYMBOL || LA(1) == TIME_SYMBOL || LA(1) == TIMESTAMP_SYMBOL)))
                                    {
                                        alt635=1;
                                    }
                                    else if ( (ANTLR3_TRUE))
                                    {
                                        alt635=4;
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 635;
                                        EXCEPTION->state        = 363;


                                        goto ruleprimaryEx;

                                    }
                                }
                            }
                            else if ( (LA635_8 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_8 == EOF || ((LA635_8 >= ACCOUNT_SYMBOL) && (LA635_8 <= YEAR_SYMBOL)) || ((LA635_8 >= AND_SYMBOL) && (LA635_8 <= ASC_SYMBOL)) || LA635_8 == BETWEEN_SYMBOL || LA635_8 == COLLATE_SYMBOL || LA635_8 == CROSS_SYMBOL || ((LA635_8 >= DAY_HOUR_SYMBOL) && (LA635_8 <= DAY_SECOND_SYMBOL)) || LA635_8 == DESC_SYMBOL || LA635_8 == ELSE_SYMBOL || ((LA635_8 >= FOR_SYMBOL) && (LA635_8 <= FROM_SYMBOL)) || ((LA635_8 >= GROUP_SYMBOL) && (LA635_8 <= HAVING_SYMBOL)) || ((LA635_8 >= HOUR_MICROSECOND_SYMBOL) && (LA635_8 <= HOUR_SECOND_SYMBOL)) || LA635_8 == INNER_SYMBOL || LA635_8 == INTO_SYMBOL || LA635_8 == IN_SYMBOL || LA635_8 == IS_SYMBOL || ((LA635_8 >= JOIN_SYMBOL) && (LA635_8 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_8 >= LEFT_SYMBOL) && (LA635_8 <= LIMIT_SYMBOL)) || LA635_8 == LOCK_SYMBOL || ((LA635_8 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_8 <= MINUTE_SECOND_SYMBOL)) || ((LA635_8 >= MOD_SYMBOL) && (LA635_8 <= NATURAL_SYMBOL)) || LA635_8 == NOT_SYMBOL || LA635_8 == ON_SYMBOL || ((LA635_8 >= ORDER_SYMBOL) && (LA635_8 <= OR_SYMBOL)) || LA635_8 == PROCEDURE_SYMBOL || LA635_8 == REGEXP_SYMBOL || LA635_8 == RENAME_SYMBOL || LA635_8 == RIGHT_SYMBOL || LA635_8 == SECOND_MICROSECOND_SYMBOL || LA635_8 == SEMICOLON_SYMBOL || LA635_8 == SEPARATOR_SYMBOL || LA635_8 == SET_SYMBOL || LA635_8 == STRAIGHT_JOIN_SYMBOL || LA635_8 == THEN_SYMBOL || LA635_8 == UNION_SYMBOL || LA635_8 == USING_SYMBOL || ((LA635_8 >= WHEN_SYMBOL) && (LA635_8 <= WHERE_SYMBOL)) || LA635_8 == WITH_SYMBOL || ((LA635_8 >= XOR_SYMBOL) && (LA635_8 <= YEAR_MONTH_SYMBOL)) || ((LA635_8 >= AS_SYMBOL) && (LA635_8 <= AT_TEXT_SUFFIX)) || ((LA635_8 >= BACK_TICK_QUOTED_ID) && (LA635_8 <= BITWISE_AND_OPERATOR)) || ((LA635_8 >= BITWISE_OR_OPERATOR) && (LA635_8 <= BITWISE_XOR_OPERATOR)) || ((LA635_8 >= CLOSE_CURLY_SYMBOL) && (LA635_8 <= CLOSE_PAR_SYMBOL)) || ((LA635_8 >= COMMA_SYMBOL) && (LA635_8 <= CONCAT_PIPES_SYMBOL)) || ((LA635_8 >= DIV_OPERATOR) && (LA635_8 <= DIV_SYMBOL)) || LA635_8 == DOT_SYMBOL || LA635_8 == DOUBLE_QUOTED_TEXT || LA635_8 == EQUAL_OPERATOR || ((LA635_8 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_8 <= GREATER_THAN_OPERATOR)) || ((LA635_8 >= HEX_NUMBER) && (LA635_8 <= IDENTIFIER)) || LA635_8 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_8 >= LESS_OR_EQUAL_OPERATOR) && (LA635_8 <= LESS_THAN_OPERATOR)) || LA635_8 == LOGICAL_AND_OPERATOR || LA635_8 == LOGICAL_OR_OPERATOR || ((LA635_8 >= MINUS_OPERATOR) && (LA635_8 <= MOD_OPERATOR)) || ((LA635_8 >= MULT_OPERATOR) && (LA635_8 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_8 == PLUS_OPERATOR || ((LA635_8 >= SHIFT_LEFT_OPERATOR) && (LA635_8 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 8;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == IDENTIFIER))
                    {
                        {
                            int LA635_13 = LA(2);
                            if ( (LA635_13 == OPEN_PAR_SYMBOL))
                            {
                                alt635=2;
                            }
                            else if ( (LA635_13 == DOT_SYMBOL))
                            {
                                {
                                    int LA635_452 = LA(3);
                                    if ( (LA635_452 == IDENTIFIER))
                                    {
                                        {
                                            int LA635_3832 = LA(4);
                                            if ( (LA635_3832 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3832 == EOF || ((LA635_3832 >= ACCOUNT_SYMBOL) && (LA635_3832 <= YEAR_SYMBOL)) || ((LA635_3832 >= AND_SYMBOL) && (LA635_3832 <= ASC_SYMBOL)) || LA635_3832 == BETWEEN_SYMBOL || LA635_3832 == COLLATE_SYMBOL || LA635_3832 == CROSS_SYMBOL || ((LA635_3832 >= DAY_HOUR_SYMBOL) && (LA635_3832 <= DAY_SECOND_SYMBOL)) || LA635_3832 == DESC_SYMBOL || LA635_3832 == ELSE_SYMBOL || ((LA635_3832 >= FOR_SYMBOL) && (LA635_3832 <= FROM_SYMBOL)) || ((LA635_3832 >= GROUP_SYMBOL) && (LA635_3832 <= HAVING_SYMBOL)) || ((LA635_3832 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3832 <= HOUR_SECOND_SYMBOL)) || LA635_3832 == INNER_SYMBOL || LA635_3832 == INTO_SYMBOL || LA635_3832 == IN_SYMBOL || LA635_3832 == IS_SYMBOL || ((LA635_3832 >= JOIN_SYMBOL) && (LA635_3832 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3832 >= LEFT_SYMBOL) && (LA635_3832 <= LIMIT_SYMBOL)) || LA635_3832 == LOCK_SYMBOL || ((LA635_3832 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3832 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3832 >= MOD_SYMBOL) && (LA635_3832 <= NATURAL_SYMBOL)) || LA635_3832 == NOT_SYMBOL || LA635_3832 == ON_SYMBOL || ((LA635_3832 >= ORDER_SYMBOL) && (LA635_3832 <= OR_SYMBOL)) || LA635_3832 == PROCEDURE_SYMBOL || LA635_3832 == REGEXP_SYMBOL || LA635_3832 == RENAME_SYMBOL || LA635_3832 == RIGHT_SYMBOL || LA635_3832 == SECOND_MICROSECOND_SYMBOL || LA635_3832 == SEMICOLON_SYMBOL || LA635_3832 == SEPARATOR_SYMBOL || LA635_3832 == SET_SYMBOL || LA635_3832 == SINGLE_QUOTED_TEXT || LA635_3832 == STRAIGHT_JOIN_SYMBOL || LA635_3832 == THEN_SYMBOL || LA635_3832 == UNION_SYMBOL || LA635_3832 == USING_SYMBOL || ((LA635_3832 >= WHEN_SYMBOL) && (LA635_3832 <= WHERE_SYMBOL)) || LA635_3832 == WITH_SYMBOL || ((LA635_3832 >= XOR_SYMBOL) && (LA635_3832 <= YEAR_MONTH_SYMBOL)) || ((LA635_3832 >= AS_SYMBOL) && (LA635_3832 <= AT_TEXT_SUFFIX)) || ((LA635_3832 >= BACK_TICK_QUOTED_ID) && (LA635_3832 <= BITWISE_AND_OPERATOR)) || ((LA635_3832 >= BITWISE_OR_OPERATOR) && (LA635_3832 <= BITWISE_XOR_OPERATOR)) || ((LA635_3832 >= CLOSE_CURLY_SYMBOL) && (LA635_3832 <= CLOSE_PAR_SYMBOL)) || ((LA635_3832 >= COMMA_SYMBOL) && (LA635_3832 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3832 >= DIV_OPERATOR) && (LA635_3832 <= DIV_SYMBOL)) || LA635_3832 == DOT_SYMBOL || LA635_3832 == DOUBLE_QUOTED_TEXT || LA635_3832 == EQUAL_OPERATOR || ((LA635_3832 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3832 <= GREATER_THAN_OPERATOR)) || ((LA635_3832 >= HEX_NUMBER) && (LA635_3832 <= IDENTIFIER)) || LA635_3832 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3832 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3832 <= LESS_THAN_OPERATOR)) || LA635_3832 == LOGICAL_AND_OPERATOR || LA635_3832 == LOGICAL_OR_OPERATOR || ((LA635_3832 >= MINUS_OPERATOR) && (LA635_3832 <= MOD_OPERATOR)) || ((LA635_3832 >= MULT_OPERATOR) && (LA635_3832 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3832 == PLUS_OPERATOR || ((LA635_3832 >= SHIFT_LEFT_OPERATOR) && (LA635_3832 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3832;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (LA635_452 == BACK_TICK_QUOTED_ID))
                                    {
                                        {
                                            int LA635_3833 = LA(4);
                                            if ( (LA635_3833 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3833 == EOF || ((LA635_3833 >= ACCOUNT_SYMBOL) && (LA635_3833 <= YEAR_SYMBOL)) || ((LA635_3833 >= AND_SYMBOL) && (LA635_3833 <= ASC_SYMBOL)) || LA635_3833 == BETWEEN_SYMBOL || LA635_3833 == COLLATE_SYMBOL || LA635_3833 == CROSS_SYMBOL || ((LA635_3833 >= DAY_HOUR_SYMBOL) && (LA635_3833 <= DAY_SECOND_SYMBOL)) || LA635_3833 == DESC_SYMBOL || LA635_3833 == ELSE_SYMBOL || ((LA635_3833 >= FOR_SYMBOL) && (LA635_3833 <= FROM_SYMBOL)) || ((LA635_3833 >= GROUP_SYMBOL) && (LA635_3833 <= HAVING_SYMBOL)) || ((LA635_3833 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3833 <= HOUR_SECOND_SYMBOL)) || LA635_3833 == INNER_SYMBOL || LA635_3833 == INTO_SYMBOL || LA635_3833 == IN_SYMBOL || LA635_3833 == IS_SYMBOL || ((LA635_3833 >= JOIN_SYMBOL) && (LA635_3833 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3833 >= LEFT_SYMBOL) && (LA635_3833 <= LIMIT_SYMBOL)) || LA635_3833 == LOCK_SYMBOL || ((LA635_3833 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3833 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3833 >= MOD_SYMBOL) && (LA635_3833 <= NATURAL_SYMBOL)) || LA635_3833 == NOT_SYMBOL || LA635_3833 == ON_SYMBOL || ((LA635_3833 >= ORDER_SYMBOL) && (LA635_3833 <= OR_SYMBOL)) || LA635_3833 == PROCEDURE_SYMBOL || LA635_3833 == REGEXP_SYMBOL || LA635_3833 == RENAME_SYMBOL || LA635_3833 == RIGHT_SYMBOL || LA635_3833 == SECOND_MICROSECOND_SYMBOL || LA635_3833 == SEMICOLON_SYMBOL || LA635_3833 == SEPARATOR_SYMBOL || LA635_3833 == SET_SYMBOL || LA635_3833 == SINGLE_QUOTED_TEXT || LA635_3833 == STRAIGHT_JOIN_SYMBOL || LA635_3833 == THEN_SYMBOL || LA635_3833 == UNION_SYMBOL || LA635_3833 == USING_SYMBOL || ((LA635_3833 >= WHEN_SYMBOL) && (LA635_3833 <= WHERE_SYMBOL)) || LA635_3833 == WITH_SYMBOL || ((LA635_3833 >= XOR_SYMBOL) && (LA635_3833 <= YEAR_MONTH_SYMBOL)) || ((LA635_3833 >= AS_SYMBOL) && (LA635_3833 <= AT_TEXT_SUFFIX)) || ((LA635_3833 >= BACK_TICK_QUOTED_ID) && (LA635_3833 <= BITWISE_AND_OPERATOR)) || ((LA635_3833 >= BITWISE_OR_OPERATOR) && (LA635_3833 <= BITWISE_XOR_OPERATOR)) || ((LA635_3833 >= CLOSE_CURLY_SYMBOL) && (LA635_3833 <= CLOSE_PAR_SYMBOL)) || ((LA635_3833 >= COMMA_SYMBOL) && (LA635_3833 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3833 >= DIV_OPERATOR) && (LA635_3833 <= DIV_SYMBOL)) || LA635_3833 == DOT_SYMBOL || LA635_3833 == DOUBLE_QUOTED_TEXT || LA635_3833 == EQUAL_OPERATOR || ((LA635_3833 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3833 <= GREATER_THAN_OPERATOR)) || ((LA635_3833 >= HEX_NUMBER) && (LA635_3833 <= IDENTIFIER)) || LA635_3833 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3833 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3833 <= LESS_THAN_OPERATOR)) || LA635_3833 == LOGICAL_AND_OPERATOR || LA635_3833 == LOGICAL_OR_OPERATOR || ((LA635_3833 >= MINUS_OPERATOR) && (LA635_3833 <= MOD_OPERATOR)) || ((LA635_3833 >= MULT_OPERATOR) && (LA635_3833 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3833 == PLUS_OPERATOR || ((LA635_3833 >= SHIFT_LEFT_OPERATOR) && (LA635_3833 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3833;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (LA635_452 == DOUBLE_QUOTED_TEXT))
                                    {
                                        {
                                            int LA635_3834 = LA(4);
                                            if ( (LA635_3834 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3834 == EOF || ((LA635_3834 >= ACCOUNT_SYMBOL) && (LA635_3834 <= YEAR_SYMBOL)) || ((LA635_3834 >= AND_SYMBOL) && (LA635_3834 <= ASC_SYMBOL)) || LA635_3834 == BETWEEN_SYMBOL || LA635_3834 == COLLATE_SYMBOL || LA635_3834 == CROSS_SYMBOL || ((LA635_3834 >= DAY_HOUR_SYMBOL) && (LA635_3834 <= DAY_SECOND_SYMBOL)) || LA635_3834 == DESC_SYMBOL || LA635_3834 == ELSE_SYMBOL || ((LA635_3834 >= FOR_SYMBOL) && (LA635_3834 <= FROM_SYMBOL)) || ((LA635_3834 >= GROUP_SYMBOL) && (LA635_3834 <= HAVING_SYMBOL)) || ((LA635_3834 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3834 <= HOUR_SECOND_SYMBOL)) || LA635_3834 == INNER_SYMBOL || LA635_3834 == INTO_SYMBOL || LA635_3834 == IN_SYMBOL || LA635_3834 == IS_SYMBOL || ((LA635_3834 >= JOIN_SYMBOL) && (LA635_3834 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3834 >= LEFT_SYMBOL) && (LA635_3834 <= LIMIT_SYMBOL)) || LA635_3834 == LOCK_SYMBOL || ((LA635_3834 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3834 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3834 >= MOD_SYMBOL) && (LA635_3834 <= NATURAL_SYMBOL)) || LA635_3834 == NOT_SYMBOL || LA635_3834 == ON_SYMBOL || ((LA635_3834 >= ORDER_SYMBOL) && (LA635_3834 <= OR_SYMBOL)) || LA635_3834 == PROCEDURE_SYMBOL || LA635_3834 == REGEXP_SYMBOL || LA635_3834 == RENAME_SYMBOL || LA635_3834 == RIGHT_SYMBOL || LA635_3834 == SECOND_MICROSECOND_SYMBOL || LA635_3834 == SEMICOLON_SYMBOL || LA635_3834 == SEPARATOR_SYMBOL || LA635_3834 == SET_SYMBOL || LA635_3834 == SINGLE_QUOTED_TEXT || LA635_3834 == STRAIGHT_JOIN_SYMBOL || LA635_3834 == THEN_SYMBOL || LA635_3834 == UNION_SYMBOL || LA635_3834 == USING_SYMBOL || ((LA635_3834 >= WHEN_SYMBOL) && (LA635_3834 <= WHERE_SYMBOL)) || LA635_3834 == WITH_SYMBOL || ((LA635_3834 >= XOR_SYMBOL) && (LA635_3834 <= YEAR_MONTH_SYMBOL)) || ((LA635_3834 >= AS_SYMBOL) && (LA635_3834 <= AT_TEXT_SUFFIX)) || ((LA635_3834 >= BACK_TICK_QUOTED_ID) && (LA635_3834 <= BITWISE_AND_OPERATOR)) || ((LA635_3834 >= BITWISE_OR_OPERATOR) && (LA635_3834 <= BITWISE_XOR_OPERATOR)) || ((LA635_3834 >= CLOSE_CURLY_SYMBOL) && (LA635_3834 <= CLOSE_PAR_SYMBOL)) || ((LA635_3834 >= COMMA_SYMBOL) && (LA635_3834 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3834 >= DIV_OPERATOR) && (LA635_3834 <= DIV_SYMBOL)) || LA635_3834 == DOT_SYMBOL || LA635_3834 == DOUBLE_QUOTED_TEXT || LA635_3834 == EQUAL_OPERATOR || ((LA635_3834 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3834 <= GREATER_THAN_OPERATOR)) || ((LA635_3834 >= HEX_NUMBER) && (LA635_3834 <= IDENTIFIER)) || LA635_3834 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3834 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3834 <= LESS_THAN_OPERATOR)) || LA635_3834 == LOGICAL_AND_OPERATOR || LA635_3834 == LOGICAL_OR_OPERATOR || ((LA635_3834 >= MINUS_OPERATOR) && (LA635_3834 <= MOD_OPERATOR)) || ((LA635_3834 >= MULT_OPERATOR) && (LA635_3834 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3834 == PLUS_OPERATOR || ((LA635_3834 >= SHIFT_LEFT_OPERATOR) && (LA635_3834 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3834;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (((LA635_452 >= ACTION_SYMBOL) && (LA635_452 <= SHARE_SYMBOL)) || ((LA635_452 >= SLOW_SYMBOL) && (LA635_452 <= YEAR_SYMBOL))))
                                    {
                                        {
                                            int LA635_3835 = LA(4);
                                            if ( (LA635_3835 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3835 == EOF || ((LA635_3835 >= ACCOUNT_SYMBOL) && (LA635_3835 <= YEAR_SYMBOL)) || ((LA635_3835 >= AND_SYMBOL) && (LA635_3835 <= ASC_SYMBOL)) || LA635_3835 == BETWEEN_SYMBOL || LA635_3835 == COLLATE_SYMBOL || LA635_3835 == CROSS_SYMBOL || ((LA635_3835 >= DAY_HOUR_SYMBOL) && (LA635_3835 <= DAY_SECOND_SYMBOL)) || LA635_3835 == DESC_SYMBOL || LA635_3835 == ELSE_SYMBOL || ((LA635_3835 >= FOR_SYMBOL) && (LA635_3835 <= FROM_SYMBOL)) || ((LA635_3835 >= GROUP_SYMBOL) && (LA635_3835 <= HAVING_SYMBOL)) || ((LA635_3835 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3835 <= HOUR_SECOND_SYMBOL)) || LA635_3835 == INNER_SYMBOL || LA635_3835 == INTO_SYMBOL || LA635_3835 == IN_SYMBOL || LA635_3835 == IS_SYMBOL || ((LA635_3835 >= JOIN_SYMBOL) && (LA635_3835 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3835 >= LEFT_SYMBOL) && (LA635_3835 <= LIMIT_SYMBOL)) || LA635_3835 == LOCK_SYMBOL || ((LA635_3835 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3835 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3835 >= MOD_SYMBOL) && (LA635_3835 <= NATURAL_SYMBOL)) || LA635_3835 == NOT_SYMBOL || LA635_3835 == ON_SYMBOL || ((LA635_3835 >= ORDER_SYMBOL) && (LA635_3835 <= OR_SYMBOL)) || LA635_3835 == PROCEDURE_SYMBOL || LA635_3835 == REGEXP_SYMBOL || LA635_3835 == RENAME_SYMBOL || LA635_3835 == RIGHT_SYMBOL || LA635_3835 == SECOND_MICROSECOND_SYMBOL || LA635_3835 == SEMICOLON_SYMBOL || LA635_3835 == SEPARATOR_SYMBOL || LA635_3835 == SET_SYMBOL || LA635_3835 == SINGLE_QUOTED_TEXT || LA635_3835 == STRAIGHT_JOIN_SYMBOL || LA635_3835 == THEN_SYMBOL || LA635_3835 == UNION_SYMBOL || LA635_3835 == USING_SYMBOL || ((LA635_3835 >= WHEN_SYMBOL) && (LA635_3835 <= WHERE_SYMBOL)) || LA635_3835 == WITH_SYMBOL || ((LA635_3835 >= XOR_SYMBOL) && (LA635_3835 <= YEAR_MONTH_SYMBOL)) || ((LA635_3835 >= AS_SYMBOL) && (LA635_3835 <= AT_TEXT_SUFFIX)) || ((LA635_3835 >= BACK_TICK_QUOTED_ID) && (LA635_3835 <= BITWISE_AND_OPERATOR)) || ((LA635_3835 >= BITWISE_OR_OPERATOR) && (LA635_3835 <= BITWISE_XOR_OPERATOR)) || ((LA635_3835 >= CLOSE_CURLY_SYMBOL) && (LA635_3835 <= CLOSE_PAR_SYMBOL)) || ((LA635_3835 >= COMMA_SYMBOL) && (LA635_3835 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3835 >= DIV_OPERATOR) && (LA635_3835 <= DIV_SYMBOL)) || LA635_3835 == DOT_SYMBOL || LA635_3835 == DOUBLE_QUOTED_TEXT || LA635_3835 == EQUAL_OPERATOR || ((LA635_3835 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3835 <= GREATER_THAN_OPERATOR)) || ((LA635_3835 >= HEX_NUMBER) && (LA635_3835 <= IDENTIFIER)) || LA635_3835 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3835 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3835 <= LESS_THAN_OPERATOR)) || LA635_3835 == LOGICAL_AND_OPERATOR || LA635_3835 == LOGICAL_OR_OPERATOR || ((LA635_3835 >= MINUS_OPERATOR) && (LA635_3835 <= MOD_OPERATOR)) || ((LA635_3835 >= MULT_OPERATOR) && (LA635_3835 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3835 == PLUS_OPERATOR || ((LA635_3835 >= SHIFT_LEFT_OPERATOR) && (LA635_3835 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3835;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (((LA635_452 >= ACCOUNT_SYMBOL) && (LA635_452 <= XA_SYMBOL)) || LA635_452 == SHUTDOWN_SYMBOL))
                                    {
                                        {
                                            int LA635_3836 = LA(4);
                                            if ( (LA635_3836 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3836 == EOF || ((LA635_3836 >= ACCOUNT_SYMBOL) && (LA635_3836 <= YEAR_SYMBOL)) || ((LA635_3836 >= AND_SYMBOL) && (LA635_3836 <= ASC_SYMBOL)) || LA635_3836 == BETWEEN_SYMBOL || LA635_3836 == COLLATE_SYMBOL || LA635_3836 == CROSS_SYMBOL || ((LA635_3836 >= DAY_HOUR_SYMBOL) && (LA635_3836 <= DAY_SECOND_SYMBOL)) || LA635_3836 == DESC_SYMBOL || LA635_3836 == ELSE_SYMBOL || ((LA635_3836 >= FOR_SYMBOL) && (LA635_3836 <= FROM_SYMBOL)) || ((LA635_3836 >= GROUP_SYMBOL) && (LA635_3836 <= HAVING_SYMBOL)) || ((LA635_3836 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3836 <= HOUR_SECOND_SYMBOL)) || LA635_3836 == INNER_SYMBOL || LA635_3836 == INTO_SYMBOL || LA635_3836 == IN_SYMBOL || LA635_3836 == IS_SYMBOL || ((LA635_3836 >= JOIN_SYMBOL) && (LA635_3836 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3836 >= LEFT_SYMBOL) && (LA635_3836 <= LIMIT_SYMBOL)) || LA635_3836 == LOCK_SYMBOL || ((LA635_3836 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3836 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3836 >= MOD_SYMBOL) && (LA635_3836 <= NATURAL_SYMBOL)) || LA635_3836 == NOT_SYMBOL || LA635_3836 == ON_SYMBOL || ((LA635_3836 >= ORDER_SYMBOL) && (LA635_3836 <= OR_SYMBOL)) || LA635_3836 == PROCEDURE_SYMBOL || LA635_3836 == REGEXP_SYMBOL || LA635_3836 == RENAME_SYMBOL || LA635_3836 == RIGHT_SYMBOL || LA635_3836 == SECOND_MICROSECOND_SYMBOL || LA635_3836 == SEMICOLON_SYMBOL || LA635_3836 == SEPARATOR_SYMBOL || LA635_3836 == SET_SYMBOL || LA635_3836 == SINGLE_QUOTED_TEXT || LA635_3836 == STRAIGHT_JOIN_SYMBOL || LA635_3836 == THEN_SYMBOL || LA635_3836 == UNION_SYMBOL || LA635_3836 == USING_SYMBOL || ((LA635_3836 >= WHEN_SYMBOL) && (LA635_3836 <= WHERE_SYMBOL)) || LA635_3836 == WITH_SYMBOL || ((LA635_3836 >= XOR_SYMBOL) && (LA635_3836 <= YEAR_MONTH_SYMBOL)) || ((LA635_3836 >= AS_SYMBOL) && (LA635_3836 <= AT_TEXT_SUFFIX)) || ((LA635_3836 >= BACK_TICK_QUOTED_ID) && (LA635_3836 <= BITWISE_AND_OPERATOR)) || ((LA635_3836 >= BITWISE_OR_OPERATOR) && (LA635_3836 <= BITWISE_XOR_OPERATOR)) || ((LA635_3836 >= CLOSE_CURLY_SYMBOL) && (LA635_3836 <= CLOSE_PAR_SYMBOL)) || ((LA635_3836 >= COMMA_SYMBOL) && (LA635_3836 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3836 >= DIV_OPERATOR) && (LA635_3836 <= DIV_SYMBOL)) || LA635_3836 == DOT_SYMBOL || LA635_3836 == DOUBLE_QUOTED_TEXT || LA635_3836 == EQUAL_OPERATOR || ((LA635_3836 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3836 <= GREATER_THAN_OPERATOR)) || ((LA635_3836 >= HEX_NUMBER) && (LA635_3836 <= IDENTIFIER)) || LA635_3836 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3836 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3836 <= LESS_THAN_OPERATOR)) || LA635_3836 == LOGICAL_AND_OPERATOR || LA635_3836 == LOGICAL_OR_OPERATOR || ((LA635_3836 >= MINUS_OPERATOR) && (LA635_3836 <= MOD_OPERATOR)) || ((LA635_3836 >= MULT_OPERATOR) && (LA635_3836 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3836 == PLUS_OPERATOR || ((LA635_3836 >= SHIFT_LEFT_OPERATOR) && (LA635_3836 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3836;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 635;
                                        EXCEPTION->state        = 452;


                                        goto ruleprimaryEx;

                                    }
                                }
                            }
                            else if ( (LA635_13 == EOF || ((LA635_13 >= ACCOUNT_SYMBOL) && (LA635_13 <= YEAR_SYMBOL)) || ((LA635_13 >= AND_SYMBOL) && (LA635_13 <= ASC_SYMBOL)) || LA635_13 == BETWEEN_SYMBOL || LA635_13 == COLLATE_SYMBOL || LA635_13 == CROSS_SYMBOL || ((LA635_13 >= DAY_HOUR_SYMBOL) && (LA635_13 <= DAY_SECOND_SYMBOL)) || LA635_13 == DESC_SYMBOL || LA635_13 == ELSE_SYMBOL || ((LA635_13 >= FOR_SYMBOL) && (LA635_13 <= FROM_SYMBOL)) || ((LA635_13 >= GROUP_SYMBOL) && (LA635_13 <= HAVING_SYMBOL)) || ((LA635_13 >= HOUR_MICROSECOND_SYMBOL) && (LA635_13 <= HOUR_SECOND_SYMBOL)) || LA635_13 == INNER_SYMBOL || LA635_13 == INTO_SYMBOL || LA635_13 == IN_SYMBOL || LA635_13 == IS_SYMBOL || ((LA635_13 >= JOIN_SYMBOL) && (LA635_13 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_13 >= LEFT_SYMBOL) && (LA635_13 <= LIMIT_SYMBOL)) || LA635_13 == LOCK_SYMBOL || ((LA635_13 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_13 <= MINUTE_SECOND_SYMBOL)) || ((LA635_13 >= MOD_SYMBOL) && (LA635_13 <= NATURAL_SYMBOL)) || LA635_13 == NOT_SYMBOL || LA635_13 == ON_SYMBOL || ((LA635_13 >= ORDER_SYMBOL) && (LA635_13 <= OR_SYMBOL)) || LA635_13 == PROCEDURE_SYMBOL || LA635_13 == REGEXP_SYMBOL || LA635_13 == RENAME_SYMBOL || LA635_13 == RIGHT_SYMBOL || LA635_13 == SECOND_MICROSECOND_SYMBOL || LA635_13 == SEMICOLON_SYMBOL || LA635_13 == SEPARATOR_SYMBOL || LA635_13 == SET_SYMBOL || LA635_13 == SINGLE_QUOTED_TEXT || LA635_13 == STRAIGHT_JOIN_SYMBOL || LA635_13 == THEN_SYMBOL || LA635_13 == UNION_SYMBOL || LA635_13 == USING_SYMBOL || ((LA635_13 >= WHEN_SYMBOL) && (LA635_13 <= WHERE_SYMBOL)) || LA635_13 == WITH_SYMBOL || ((LA635_13 >= XOR_SYMBOL) && (LA635_13 <= YEAR_MONTH_SYMBOL)) || ((LA635_13 >= AS_SYMBOL) && (LA635_13 <= AT_TEXT_SUFFIX)) || ((LA635_13 >= BACK_TICK_QUOTED_ID) && (LA635_13 <= BITWISE_AND_OPERATOR)) || ((LA635_13 >= BITWISE_OR_OPERATOR) && (LA635_13 <= BITWISE_XOR_OPERATOR)) || ((LA635_13 >= CLOSE_CURLY_SYMBOL) && (LA635_13 <= CLOSE_PAR_SYMBOL)) || ((LA635_13 >= COMMA_SYMBOL) && (LA635_13 <= CONCAT_PIPES_SYMBOL)) || ((LA635_13 >= DIV_OPERATOR) && (LA635_13 <= DIV_SYMBOL)) || LA635_13 == DOUBLE_QUOTED_TEXT || LA635_13 == EQUAL_OPERATOR || ((LA635_13 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_13 <= GREATER_THAN_OPERATOR)) || ((LA635_13 >= HEX_NUMBER) && (LA635_13 <= IDENTIFIER)) || LA635_13 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_13 >= LESS_OR_EQUAL_OPERATOR) && (LA635_13 <= LESS_THAN_OPERATOR)) || LA635_13 == LOGICAL_AND_OPERATOR || LA635_13 == LOGICAL_OR_OPERATOR || ((LA635_13 >= MINUS_OPERATOR) && (LA635_13 <= MOD_OPERATOR)) || ((LA635_13 >= MULT_OPERATOR) && (LA635_13 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_13 == PLUS_OPERATOR || ((LA635_13 >= SHIFT_LEFT_OPERATOR) && (LA635_13 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 13;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == BACK_TICK_QUOTED_ID))
                    {
                        {
                            int LA635_14 = LA(2);
                            if ( (LA635_14 == OPEN_PAR_SYMBOL))
                            {
                                alt635=2;
                            }
                            else if ( (LA635_14 == DOT_SYMBOL))
                            {
                                {
                                    int LA635_539 = LA(3);
                                    if ( (LA635_539 == IDENTIFIER))
                                    {
                                        {
                                            int LA635_3837 = LA(4);
                                            if ( (LA635_3837 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3837 == EOF || ((LA635_3837 >= ACCOUNT_SYMBOL) && (LA635_3837 <= YEAR_SYMBOL)) || ((LA635_3837 >= AND_SYMBOL) && (LA635_3837 <= ASC_SYMBOL)) || LA635_3837 == BETWEEN_SYMBOL || LA635_3837 == COLLATE_SYMBOL || LA635_3837 == CROSS_SYMBOL || ((LA635_3837 >= DAY_HOUR_SYMBOL) && (LA635_3837 <= DAY_SECOND_SYMBOL)) || LA635_3837 == DESC_SYMBOL || LA635_3837 == ELSE_SYMBOL || ((LA635_3837 >= FOR_SYMBOL) && (LA635_3837 <= FROM_SYMBOL)) || ((LA635_3837 >= GROUP_SYMBOL) && (LA635_3837 <= HAVING_SYMBOL)) || ((LA635_3837 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3837 <= HOUR_SECOND_SYMBOL)) || LA635_3837 == INNER_SYMBOL || LA635_3837 == INTO_SYMBOL || LA635_3837 == IN_SYMBOL || LA635_3837 == IS_SYMBOL || ((LA635_3837 >= JOIN_SYMBOL) && (LA635_3837 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3837 >= LEFT_SYMBOL) && (LA635_3837 <= LIMIT_SYMBOL)) || LA635_3837 == LOCK_SYMBOL || ((LA635_3837 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3837 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3837 >= MOD_SYMBOL) && (LA635_3837 <= NATURAL_SYMBOL)) || LA635_3837 == NOT_SYMBOL || LA635_3837 == ON_SYMBOL || ((LA635_3837 >= ORDER_SYMBOL) && (LA635_3837 <= OR_SYMBOL)) || LA635_3837 == PROCEDURE_SYMBOL || LA635_3837 == REGEXP_SYMBOL || LA635_3837 == RENAME_SYMBOL || LA635_3837 == RIGHT_SYMBOL || LA635_3837 == SECOND_MICROSECOND_SYMBOL || LA635_3837 == SEMICOLON_SYMBOL || LA635_3837 == SEPARATOR_SYMBOL || LA635_3837 == SET_SYMBOL || LA635_3837 == SINGLE_QUOTED_TEXT || LA635_3837 == STRAIGHT_JOIN_SYMBOL || LA635_3837 == THEN_SYMBOL || LA635_3837 == UNION_SYMBOL || LA635_3837 == USING_SYMBOL || ((LA635_3837 >= WHEN_SYMBOL) && (LA635_3837 <= WHERE_SYMBOL)) || LA635_3837 == WITH_SYMBOL || ((LA635_3837 >= XOR_SYMBOL) && (LA635_3837 <= YEAR_MONTH_SYMBOL)) || ((LA635_3837 >= AS_SYMBOL) && (LA635_3837 <= AT_TEXT_SUFFIX)) || ((LA635_3837 >= BACK_TICK_QUOTED_ID) && (LA635_3837 <= BITWISE_AND_OPERATOR)) || ((LA635_3837 >= BITWISE_OR_OPERATOR) && (LA635_3837 <= BITWISE_XOR_OPERATOR)) || ((LA635_3837 >= CLOSE_CURLY_SYMBOL) && (LA635_3837 <= CLOSE_PAR_SYMBOL)) || ((LA635_3837 >= COMMA_SYMBOL) && (LA635_3837 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3837 >= DIV_OPERATOR) && (LA635_3837 <= DIV_SYMBOL)) || LA635_3837 == DOT_SYMBOL || LA635_3837 == DOUBLE_QUOTED_TEXT || LA635_3837 == EQUAL_OPERATOR || ((LA635_3837 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3837 <= GREATER_THAN_OPERATOR)) || ((LA635_3837 >= HEX_NUMBER) && (LA635_3837 <= IDENTIFIER)) || LA635_3837 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3837 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3837 <= LESS_THAN_OPERATOR)) || LA635_3837 == LOGICAL_AND_OPERATOR || LA635_3837 == LOGICAL_OR_OPERATOR || ((LA635_3837 >= MINUS_OPERATOR) && (LA635_3837 <= MOD_OPERATOR)) || ((LA635_3837 >= MULT_OPERATOR) && (LA635_3837 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3837 == PLUS_OPERATOR || ((LA635_3837 >= SHIFT_LEFT_OPERATOR) && (LA635_3837 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3837;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (LA635_539 == BACK_TICK_QUOTED_ID))
                                    {
                                        {
                                            int LA635_3838 = LA(4);
                                            if ( (LA635_3838 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3838 == EOF || ((LA635_3838 >= ACCOUNT_SYMBOL) && (LA635_3838 <= YEAR_SYMBOL)) || ((LA635_3838 >= AND_SYMBOL) && (LA635_3838 <= ASC_SYMBOL)) || LA635_3838 == BETWEEN_SYMBOL || LA635_3838 == COLLATE_SYMBOL || LA635_3838 == CROSS_SYMBOL || ((LA635_3838 >= DAY_HOUR_SYMBOL) && (LA635_3838 <= DAY_SECOND_SYMBOL)) || LA635_3838 == DESC_SYMBOL || LA635_3838 == ELSE_SYMBOL || ((LA635_3838 >= FOR_SYMBOL) && (LA635_3838 <= FROM_SYMBOL)) || ((LA635_3838 >= GROUP_SYMBOL) && (LA635_3838 <= HAVING_SYMBOL)) || ((LA635_3838 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3838 <= HOUR_SECOND_SYMBOL)) || LA635_3838 == INNER_SYMBOL || LA635_3838 == INTO_SYMBOL || LA635_3838 == IN_SYMBOL || LA635_3838 == IS_SYMBOL || ((LA635_3838 >= JOIN_SYMBOL) && (LA635_3838 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3838 >= LEFT_SYMBOL) && (LA635_3838 <= LIMIT_SYMBOL)) || LA635_3838 == LOCK_SYMBOL || ((LA635_3838 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3838 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3838 >= MOD_SYMBOL) && (LA635_3838 <= NATURAL_SYMBOL)) || LA635_3838 == NOT_SYMBOL || LA635_3838 == ON_SYMBOL || ((LA635_3838 >= ORDER_SYMBOL) && (LA635_3838 <= OR_SYMBOL)) || LA635_3838 == PROCEDURE_SYMBOL || LA635_3838 == REGEXP_SYMBOL || LA635_3838 == RENAME_SYMBOL || LA635_3838 == RIGHT_SYMBOL || LA635_3838 == SECOND_MICROSECOND_SYMBOL || LA635_3838 == SEMICOLON_SYMBOL || LA635_3838 == SEPARATOR_SYMBOL || LA635_3838 == SET_SYMBOL || LA635_3838 == SINGLE_QUOTED_TEXT || LA635_3838 == STRAIGHT_JOIN_SYMBOL || LA635_3838 == THEN_SYMBOL || LA635_3838 == UNION_SYMBOL || LA635_3838 == USING_SYMBOL || ((LA635_3838 >= WHEN_SYMBOL) && (LA635_3838 <= WHERE_SYMBOL)) || LA635_3838 == WITH_SYMBOL || ((LA635_3838 >= XOR_SYMBOL) && (LA635_3838 <= YEAR_MONTH_SYMBOL)) || ((LA635_3838 >= AS_SYMBOL) && (LA635_3838 <= AT_TEXT_SUFFIX)) || ((LA635_3838 >= BACK_TICK_QUOTED_ID) && (LA635_3838 <= BITWISE_AND_OPERATOR)) || ((LA635_3838 >= BITWISE_OR_OPERATOR) && (LA635_3838 <= BITWISE_XOR_OPERATOR)) || ((LA635_3838 >= CLOSE_CURLY_SYMBOL) && (LA635_3838 <= CLOSE_PAR_SYMBOL)) || ((LA635_3838 >= COMMA_SYMBOL) && (LA635_3838 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3838 >= DIV_OPERATOR) && (LA635_3838 <= DIV_SYMBOL)) || LA635_3838 == DOT_SYMBOL || LA635_3838 == DOUBLE_QUOTED_TEXT || LA635_3838 == EQUAL_OPERATOR || ((LA635_3838 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3838 <= GREATER_THAN_OPERATOR)) || ((LA635_3838 >= HEX_NUMBER) && (LA635_3838 <= IDENTIFIER)) || LA635_3838 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3838 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3838 <= LESS_THAN_OPERATOR)) || LA635_3838 == LOGICAL_AND_OPERATOR || LA635_3838 == LOGICAL_OR_OPERATOR || ((LA635_3838 >= MINUS_OPERATOR) && (LA635_3838 <= MOD_OPERATOR)) || ((LA635_3838 >= MULT_OPERATOR) && (LA635_3838 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3838 == PLUS_OPERATOR || ((LA635_3838 >= SHIFT_LEFT_OPERATOR) && (LA635_3838 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3838;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (LA635_539 == DOUBLE_QUOTED_TEXT))
                                    {
                                        {
                                            int LA635_3839 = LA(4);
                                            if ( (LA635_3839 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3839 == EOF || ((LA635_3839 >= ACCOUNT_SYMBOL) && (LA635_3839 <= YEAR_SYMBOL)) || ((LA635_3839 >= AND_SYMBOL) && (LA635_3839 <= ASC_SYMBOL)) || LA635_3839 == BETWEEN_SYMBOL || LA635_3839 == COLLATE_SYMBOL || LA635_3839 == CROSS_SYMBOL || ((LA635_3839 >= DAY_HOUR_SYMBOL) && (LA635_3839 <= DAY_SECOND_SYMBOL)) || LA635_3839 == DESC_SYMBOL || LA635_3839 == ELSE_SYMBOL || ((LA635_3839 >= FOR_SYMBOL) && (LA635_3839 <= FROM_SYMBOL)) || ((LA635_3839 >= GROUP_SYMBOL) && (LA635_3839 <= HAVING_SYMBOL)) || ((LA635_3839 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3839 <= HOUR_SECOND_SYMBOL)) || LA635_3839 == INNER_SYMBOL || LA635_3839 == INTO_SYMBOL || LA635_3839 == IN_SYMBOL || LA635_3839 == IS_SYMBOL || ((LA635_3839 >= JOIN_SYMBOL) && (LA635_3839 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3839 >= LEFT_SYMBOL) && (LA635_3839 <= LIMIT_SYMBOL)) || LA635_3839 == LOCK_SYMBOL || ((LA635_3839 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3839 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3839 >= MOD_SYMBOL) && (LA635_3839 <= NATURAL_SYMBOL)) || LA635_3839 == NOT_SYMBOL || LA635_3839 == ON_SYMBOL || ((LA635_3839 >= ORDER_SYMBOL) && (LA635_3839 <= OR_SYMBOL)) || LA635_3839 == PROCEDURE_SYMBOL || LA635_3839 == REGEXP_SYMBOL || LA635_3839 == RENAME_SYMBOL || LA635_3839 == RIGHT_SYMBOL || LA635_3839 == SECOND_MICROSECOND_SYMBOL || LA635_3839 == SEMICOLON_SYMBOL || LA635_3839 == SEPARATOR_SYMBOL || LA635_3839 == SET_SYMBOL || LA635_3839 == SINGLE_QUOTED_TEXT || LA635_3839 == STRAIGHT_JOIN_SYMBOL || LA635_3839 == THEN_SYMBOL || LA635_3839 == UNION_SYMBOL || LA635_3839 == USING_SYMBOL || ((LA635_3839 >= WHEN_SYMBOL) && (LA635_3839 <= WHERE_SYMBOL)) || LA635_3839 == WITH_SYMBOL || ((LA635_3839 >= XOR_SYMBOL) && (LA635_3839 <= YEAR_MONTH_SYMBOL)) || ((LA635_3839 >= AS_SYMBOL) && (LA635_3839 <= AT_TEXT_SUFFIX)) || ((LA635_3839 >= BACK_TICK_QUOTED_ID) && (LA635_3839 <= BITWISE_AND_OPERATOR)) || ((LA635_3839 >= BITWISE_OR_OPERATOR) && (LA635_3839 <= BITWISE_XOR_OPERATOR)) || ((LA635_3839 >= CLOSE_CURLY_SYMBOL) && (LA635_3839 <= CLOSE_PAR_SYMBOL)) || ((LA635_3839 >= COMMA_SYMBOL) && (LA635_3839 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3839 >= DIV_OPERATOR) && (LA635_3839 <= DIV_SYMBOL)) || LA635_3839 == DOT_SYMBOL || LA635_3839 == DOUBLE_QUOTED_TEXT || LA635_3839 == EQUAL_OPERATOR || ((LA635_3839 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3839 <= GREATER_THAN_OPERATOR)) || ((LA635_3839 >= HEX_NUMBER) && (LA635_3839 <= IDENTIFIER)) || LA635_3839 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3839 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3839 <= LESS_THAN_OPERATOR)) || LA635_3839 == LOGICAL_AND_OPERATOR || LA635_3839 == LOGICAL_OR_OPERATOR || ((LA635_3839 >= MINUS_OPERATOR) && (LA635_3839 <= MOD_OPERATOR)) || ((LA635_3839 >= MULT_OPERATOR) && (LA635_3839 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3839 == PLUS_OPERATOR || ((LA635_3839 >= SHIFT_LEFT_OPERATOR) && (LA635_3839 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3839;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (((LA635_539 >= ACTION_SYMBOL) && (LA635_539 <= SHARE_SYMBOL)) || ((LA635_539 >= SLOW_SYMBOL) && (LA635_539 <= YEAR_SYMBOL))))
                                    {
                                        {
                                            int LA635_3840 = LA(4);
                                            if ( (LA635_3840 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3840 == EOF || ((LA635_3840 >= ACCOUNT_SYMBOL) && (LA635_3840 <= YEAR_SYMBOL)) || ((LA635_3840 >= AND_SYMBOL) && (LA635_3840 <= ASC_SYMBOL)) || LA635_3840 == BETWEEN_SYMBOL || LA635_3840 == COLLATE_SYMBOL || LA635_3840 == CROSS_SYMBOL || ((LA635_3840 >= DAY_HOUR_SYMBOL) && (LA635_3840 <= DAY_SECOND_SYMBOL)) || LA635_3840 == DESC_SYMBOL || LA635_3840 == ELSE_SYMBOL || ((LA635_3840 >= FOR_SYMBOL) && (LA635_3840 <= FROM_SYMBOL)) || ((LA635_3840 >= GROUP_SYMBOL) && (LA635_3840 <= HAVING_SYMBOL)) || ((LA635_3840 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3840 <= HOUR_SECOND_SYMBOL)) || LA635_3840 == INNER_SYMBOL || LA635_3840 == INTO_SYMBOL || LA635_3840 == IN_SYMBOL || LA635_3840 == IS_SYMBOL || ((LA635_3840 >= JOIN_SYMBOL) && (LA635_3840 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3840 >= LEFT_SYMBOL) && (LA635_3840 <= LIMIT_SYMBOL)) || LA635_3840 == LOCK_SYMBOL || ((LA635_3840 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3840 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3840 >= MOD_SYMBOL) && (LA635_3840 <= NATURAL_SYMBOL)) || LA635_3840 == NOT_SYMBOL || LA635_3840 == ON_SYMBOL || ((LA635_3840 >= ORDER_SYMBOL) && (LA635_3840 <= OR_SYMBOL)) || LA635_3840 == PROCEDURE_SYMBOL || LA635_3840 == REGEXP_SYMBOL || LA635_3840 == RENAME_SYMBOL || LA635_3840 == RIGHT_SYMBOL || LA635_3840 == SECOND_MICROSECOND_SYMBOL || LA635_3840 == SEMICOLON_SYMBOL || LA635_3840 == SEPARATOR_SYMBOL || LA635_3840 == SET_SYMBOL || LA635_3840 == SINGLE_QUOTED_TEXT || LA635_3840 == STRAIGHT_JOIN_SYMBOL || LA635_3840 == THEN_SYMBOL || LA635_3840 == UNION_SYMBOL || LA635_3840 == USING_SYMBOL || ((LA635_3840 >= WHEN_SYMBOL) && (LA635_3840 <= WHERE_SYMBOL)) || LA635_3840 == WITH_SYMBOL || ((LA635_3840 >= XOR_SYMBOL) && (LA635_3840 <= YEAR_MONTH_SYMBOL)) || ((LA635_3840 >= AS_SYMBOL) && (LA635_3840 <= AT_TEXT_SUFFIX)) || ((LA635_3840 >= BACK_TICK_QUOTED_ID) && (LA635_3840 <= BITWISE_AND_OPERATOR)) || ((LA635_3840 >= BITWISE_OR_OPERATOR) && (LA635_3840 <= BITWISE_XOR_OPERATOR)) || ((LA635_3840 >= CLOSE_CURLY_SYMBOL) && (LA635_3840 <= CLOSE_PAR_SYMBOL)) || ((LA635_3840 >= COMMA_SYMBOL) && (LA635_3840 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3840 >= DIV_OPERATOR) && (LA635_3840 <= DIV_SYMBOL)) || LA635_3840 == DOT_SYMBOL || LA635_3840 == DOUBLE_QUOTED_TEXT || LA635_3840 == EQUAL_OPERATOR || ((LA635_3840 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3840 <= GREATER_THAN_OPERATOR)) || ((LA635_3840 >= HEX_NUMBER) && (LA635_3840 <= IDENTIFIER)) || LA635_3840 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3840 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3840 <= LESS_THAN_OPERATOR)) || LA635_3840 == LOGICAL_AND_OPERATOR || LA635_3840 == LOGICAL_OR_OPERATOR || ((LA635_3840 >= MINUS_OPERATOR) && (LA635_3840 <= MOD_OPERATOR)) || ((LA635_3840 >= MULT_OPERATOR) && (LA635_3840 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3840 == PLUS_OPERATOR || ((LA635_3840 >= SHIFT_LEFT_OPERATOR) && (LA635_3840 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3840;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else if ( (((LA635_539 >= ACCOUNT_SYMBOL) && (LA635_539 <= XA_SYMBOL)) || LA635_539 == SHUTDOWN_SYMBOL))
                                    {
                                        {
                                            int LA635_3841 = LA(4);
                                            if ( (LA635_3841 == OPEN_PAR_SYMBOL))
                                            {
                                                alt635=2;
                                            }
                                            else if ( (LA635_3841 == EOF || ((LA635_3841 >= ACCOUNT_SYMBOL) && (LA635_3841 <= YEAR_SYMBOL)) || ((LA635_3841 >= AND_SYMBOL) && (LA635_3841 <= ASC_SYMBOL)) || LA635_3841 == BETWEEN_SYMBOL || LA635_3841 == COLLATE_SYMBOL || LA635_3841 == CROSS_SYMBOL || ((LA635_3841 >= DAY_HOUR_SYMBOL) && (LA635_3841 <= DAY_SECOND_SYMBOL)) || LA635_3841 == DESC_SYMBOL || LA635_3841 == ELSE_SYMBOL || ((LA635_3841 >= FOR_SYMBOL) && (LA635_3841 <= FROM_SYMBOL)) || ((LA635_3841 >= GROUP_SYMBOL) && (LA635_3841 <= HAVING_SYMBOL)) || ((LA635_3841 >= HOUR_MICROSECOND_SYMBOL) && (LA635_3841 <= HOUR_SECOND_SYMBOL)) || LA635_3841 == INNER_SYMBOL || LA635_3841 == INTO_SYMBOL || LA635_3841 == IN_SYMBOL || LA635_3841 == IS_SYMBOL || ((LA635_3841 >= JOIN_SYMBOL) && (LA635_3841 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_3841 >= LEFT_SYMBOL) && (LA635_3841 <= LIMIT_SYMBOL)) || LA635_3841 == LOCK_SYMBOL || ((LA635_3841 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_3841 <= MINUTE_SECOND_SYMBOL)) || ((LA635_3841 >= MOD_SYMBOL) && (LA635_3841 <= NATURAL_SYMBOL)) || LA635_3841 == NOT_SYMBOL || LA635_3841 == ON_SYMBOL || ((LA635_3841 >= ORDER_SYMBOL) && (LA635_3841 <= OR_SYMBOL)) || LA635_3841 == PROCEDURE_SYMBOL || LA635_3841 == REGEXP_SYMBOL || LA635_3841 == RENAME_SYMBOL || LA635_3841 == RIGHT_SYMBOL || LA635_3841 == SECOND_MICROSECOND_SYMBOL || LA635_3841 == SEMICOLON_SYMBOL || LA635_3841 == SEPARATOR_SYMBOL || LA635_3841 == SET_SYMBOL || LA635_3841 == SINGLE_QUOTED_TEXT || LA635_3841 == STRAIGHT_JOIN_SYMBOL || LA635_3841 == THEN_SYMBOL || LA635_3841 == UNION_SYMBOL || LA635_3841 == USING_SYMBOL || ((LA635_3841 >= WHEN_SYMBOL) && (LA635_3841 <= WHERE_SYMBOL)) || LA635_3841 == WITH_SYMBOL || ((LA635_3841 >= XOR_SYMBOL) && (LA635_3841 <= YEAR_MONTH_SYMBOL)) || ((LA635_3841 >= AS_SYMBOL) && (LA635_3841 <= AT_TEXT_SUFFIX)) || ((LA635_3841 >= BACK_TICK_QUOTED_ID) && (LA635_3841 <= BITWISE_AND_OPERATOR)) || ((LA635_3841 >= BITWISE_OR_OPERATOR) && (LA635_3841 <= BITWISE_XOR_OPERATOR)) || ((LA635_3841 >= CLOSE_CURLY_SYMBOL) && (LA635_3841 <= CLOSE_PAR_SYMBOL)) || ((LA635_3841 >= COMMA_SYMBOL) && (LA635_3841 <= CONCAT_PIPES_SYMBOL)) || ((LA635_3841 >= DIV_OPERATOR) && (LA635_3841 <= DIV_SYMBOL)) || LA635_3841 == DOT_SYMBOL || LA635_3841 == DOUBLE_QUOTED_TEXT || LA635_3841 == EQUAL_OPERATOR || ((LA635_3841 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_3841 <= GREATER_THAN_OPERATOR)) || ((LA635_3841 >= HEX_NUMBER) && (LA635_3841 <= IDENTIFIER)) || LA635_3841 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_3841 >= LESS_OR_EQUAL_OPERATOR) && (LA635_3841 <= LESS_THAN_OPERATOR)) || LA635_3841 == LOGICAL_AND_OPERATOR || LA635_3841 == LOGICAL_OR_OPERATOR || ((LA635_3841 >= MINUS_OPERATOR) && (LA635_3841 <= MOD_OPERATOR)) || ((LA635_3841 >= MULT_OPERATOR) && (LA635_3841 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_3841 == PLUS_OPERATOR || ((LA635_3841 >= SHIFT_LEFT_OPERATOR) && (LA635_3841 <= SHIFT_RIGHT_OPERATOR))))
                                            {
                                                alt635=4;
                                            }
                                            else
                                            {
                                                if (BACKTRACKING>0)
                                                {
                                                    FAILEDFLAG = ANTLR3_TRUE;
                                                    return ;
                                                }


                                                CONSTRUCTEX();
                                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                                EXCEPTION->message      = (void *)"";
                                                EXCEPTION->decisionNum  = 635;
                                                EXCEPTION->state        = 3841;


                                                goto ruleprimaryEx;

                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (BACKTRACKING>0)
                                        {
                                            FAILEDFLAG = ANTLR3_TRUE;
                                            return ;
                                        }


                                        CONSTRUCTEX();
                                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                        EXCEPTION->message      = (void *)"";
                                        EXCEPTION->decisionNum  = 635;
                                        EXCEPTION->state        = 539;


                                        goto ruleprimaryEx;

                                    }
                                }
                            }
                            else if ( (LA635_14 == EOF || ((LA635_14 >= ACCOUNT_SYMBOL) && (LA635_14 <= YEAR_SYMBOL)) || ((LA635_14 >= AND_SYMBOL) && (LA635_14 <= ASC_SYMBOL)) || LA635_14 == BETWEEN_SYMBOL || LA635_14 == COLLATE_SYMBOL || LA635_14 == CROSS_SYMBOL || ((LA635_14 >= DAY_HOUR_SYMBOL) && (LA635_14 <= DAY_SECOND_SYMBOL)) || LA635_14 == DESC_SYMBOL || LA635_14 == ELSE_SYMBOL || ((LA635_14 >= FOR_SYMBOL) && (LA635_14 <= FROM_SYMBOL)) || ((LA635_14 >= GROUP_SYMBOL) && (LA635_14 <= HAVING_SYMBOL)) || ((LA635_14 >= HOUR_MICROSECOND_SYMBOL) && (LA635_14 <= HOUR_SECOND_SYMBOL)) || LA635_14 == INNER_SYMBOL || LA635_14 == INTO_SYMBOL || LA635_14 == IN_SYMBOL || LA635_14 == IS_SYMBOL || ((LA635_14 >= JOIN_SYMBOL) && (LA635_14 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_14 >= LEFT_SYMBOL) && (LA635_14 <= LIMIT_SYMBOL)) || LA635_14 == LOCK_SYMBOL || ((LA635_14 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_14 <= MINUTE_SECOND_SYMBOL)) || ((LA635_14 >= MOD_SYMBOL) && (LA635_14 <= NATURAL_SYMBOL)) || LA635_14 == NOT_SYMBOL || LA635_14 == ON_SYMBOL || ((LA635_14 >= ORDER_SYMBOL) && (LA635_14 <= OR_SYMBOL)) || LA635_14 == PROCEDURE_SYMBOL || LA635_14 == REGEXP_SYMBOL || LA635_14 == RENAME_SYMBOL || LA635_14 == RIGHT_SYMBOL || LA635_14 == SECOND_MICROSECOND_SYMBOL || LA635_14 == SEMICOLON_SYMBOL || LA635_14 == SEPARATOR_SYMBOL || LA635_14 == SET_SYMBOL || LA635_14 == SINGLE_QUOTED_TEXT || LA635_14 == STRAIGHT_JOIN_SYMBOL || LA635_14 == THEN_SYMBOL || LA635_14 == UNION_SYMBOL || LA635_14 == USING_SYMBOL || ((LA635_14 >= WHEN_SYMBOL) && (LA635_14 <= WHERE_SYMBOL)) || LA635_14 == WITH_SYMBOL || ((LA635_14 >= XOR_SYMBOL) && (LA635_14 <= YEAR_MONTH_SYMBOL)) || ((LA635_14 >= AS_SYMBOL) && (LA635_14 <= AT_TEXT_SUFFIX)) || ((LA635_14 >= BACK_TICK_QUOTED_ID) && (LA635_14 <= BITWISE_AND_OPERATOR)) || ((LA635_14 >= BITWISE_OR_OPERATOR) && (LA635_14 <= BITWISE_XOR_OPERATOR)) || ((LA635_14 >= CLOSE_CURLY_SYMBOL) && (LA635_14 <= CLOSE_PAR_SYMBOL)) || ((LA635_14 >= COMMA_SYMBOL) && (LA635_14 <= CONCAT_PIPES_SYMBOL)) || ((LA635_14 >= DIV_OPERATOR) && (LA635_14 <= DIV_SYMBOL)) || LA635_14 == DOUBLE_QUOTED_TEXT || LA635_14 == EQUAL_OPERATOR || ((LA635_14 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_14 <= GREATER_THAN_OPERATOR)) || ((LA635_14 >= HEX_NUMBER) && (LA635_14 <= IDENTIFIER)) || LA635_14 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_14 >= LESS_OR_EQUAL_OPERATOR) && (LA635_14 <= LESS_THAN_OPERATOR)) || LA635_14 == LOGICAL_AND_OPERATOR || LA635_14 == LOGICAL_OR_OPERATOR || ((LA635_14 >= MINUS_OPERATOR) && (LA635_14 <= MOD_OPERATOR)) || ((LA635_14 >= MULT_OPERATOR) && (LA635_14 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_14 == PLUS_OPERATOR || ((LA635_14 >= SHIFT_LEFT_OPERATOR) && (LA635_14 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 14;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == CHAR_SYMBOL || LA635_0 == COUNT_SYMBOL || LA635_0 == CURDATE_SYMBOL || LA635_0 == CURRENT_USER_SYMBOL || LA635_0 == CURTIME_SYMBOL || LA635_0 == DATABASE_SYMBOL || LA635_0 == DATE_ADD_SYMBOL || LA635_0 == DATE_SUB_SYMBOL || LA635_0 == EXTRACT_SYMBOL || LA635_0 == GROUP_CONCAT_SYMBOL || LA635_0 == IF_SYMBOL || LA635_0 == INSERT_SYMBOL || LA635_0 == LEFT_SYMBOL || LA635_0 == MAX_SYMBOL || LA635_0 == MIN_SYMBOL || LA635_0 == MOD_SYMBOL || LA635_0 == NOW_SYMBOL || LA635_0 == POSITION_SYMBOL || ((LA635_0 >= REPEAT_SYMBOL) && (LA635_0 <= REPLACE_SYMBOL)) || LA635_0 == RIGHT_SYMBOL || LA635_0 == STDDEV_SAMP_SYMBOL || LA635_0 == STD_SYMBOL || LA635_0 == SUBSTRING_SYMBOL || ((LA635_0 >= SUM_SYMBOL) && (LA635_0 <= SYSDATE_SYMBOL)) || LA635_0 == TRIM_SYMBOL || ((LA635_0 >= UTC_DATE_SYMBOL) && (LA635_0 <= UTC_TIME_SYMBOL)) || LA635_0 == VALUES_SYMBOL || LA635_0 == VARIANCE_SYMBOL || LA635_0 == VAR_SAMP_SYMBOL || ((LA635_0 >= BIT_AND_SYMBOL) && (LA635_0 <= BIT_XOR_SYMBOL))))
                    {
                        alt635=3;
                    }
                    else if ( (LA635_0 == DAY_SYMBOL))
                    {
                        {
                            int LA635_17 = LA(2);
                            if ( (LA635_17 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_17 == EOF || ((LA635_17 >= ACCOUNT_SYMBOL) && (LA635_17 <= YEAR_SYMBOL)) || ((LA635_17 >= AND_SYMBOL) && (LA635_17 <= ASC_SYMBOL)) || LA635_17 == BETWEEN_SYMBOL || LA635_17 == COLLATE_SYMBOL || LA635_17 == CROSS_SYMBOL || ((LA635_17 >= DAY_HOUR_SYMBOL) && (LA635_17 <= DAY_SECOND_SYMBOL)) || LA635_17 == DESC_SYMBOL || LA635_17 == ELSE_SYMBOL || ((LA635_17 >= FOR_SYMBOL) && (LA635_17 <= FROM_SYMBOL)) || ((LA635_17 >= GROUP_SYMBOL) && (LA635_17 <= HAVING_SYMBOL)) || ((LA635_17 >= HOUR_MICROSECOND_SYMBOL) && (LA635_17 <= HOUR_SECOND_SYMBOL)) || LA635_17 == INNER_SYMBOL || LA635_17 == INTO_SYMBOL || LA635_17 == IN_SYMBOL || LA635_17 == IS_SYMBOL || ((LA635_17 >= JOIN_SYMBOL) && (LA635_17 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_17 >= LEFT_SYMBOL) && (LA635_17 <= LIMIT_SYMBOL)) || LA635_17 == LOCK_SYMBOL || ((LA635_17 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_17 <= MINUTE_SECOND_SYMBOL)) || ((LA635_17 >= MOD_SYMBOL) && (LA635_17 <= NATURAL_SYMBOL)) || LA635_17 == NOT_SYMBOL || LA635_17 == ON_SYMBOL || ((LA635_17 >= ORDER_SYMBOL) && (LA635_17 <= OR_SYMBOL)) || LA635_17 == PROCEDURE_SYMBOL || LA635_17 == REGEXP_SYMBOL || LA635_17 == RENAME_SYMBOL || LA635_17 == RIGHT_SYMBOL || LA635_17 == SECOND_MICROSECOND_SYMBOL || LA635_17 == SEMICOLON_SYMBOL || LA635_17 == SEPARATOR_SYMBOL || LA635_17 == SET_SYMBOL || LA635_17 == SINGLE_QUOTED_TEXT || LA635_17 == STRAIGHT_JOIN_SYMBOL || LA635_17 == THEN_SYMBOL || LA635_17 == UNION_SYMBOL || LA635_17 == USING_SYMBOL || ((LA635_17 >= WHEN_SYMBOL) && (LA635_17 <= WHERE_SYMBOL)) || LA635_17 == WITH_SYMBOL || ((LA635_17 >= XOR_SYMBOL) && (LA635_17 <= YEAR_MONTH_SYMBOL)) || ((LA635_17 >= AS_SYMBOL) && (LA635_17 <= AT_TEXT_SUFFIX)) || ((LA635_17 >= BACK_TICK_QUOTED_ID) && (LA635_17 <= BITWISE_AND_OPERATOR)) || ((LA635_17 >= BITWISE_OR_OPERATOR) && (LA635_17 <= BITWISE_XOR_OPERATOR)) || ((LA635_17 >= CLOSE_CURLY_SYMBOL) && (LA635_17 <= CLOSE_PAR_SYMBOL)) || ((LA635_17 >= COMMA_SYMBOL) && (LA635_17 <= CONCAT_PIPES_SYMBOL)) || ((LA635_17 >= DIV_OPERATOR) && (LA635_17 <= DIV_SYMBOL)) || LA635_17 == DOT_SYMBOL || LA635_17 == DOUBLE_QUOTED_TEXT || LA635_17 == EQUAL_OPERATOR || ((LA635_17 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_17 <= GREATER_THAN_OPERATOR)) || ((LA635_17 >= HEX_NUMBER) && (LA635_17 <= IDENTIFIER)) || LA635_17 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_17 >= LESS_OR_EQUAL_OPERATOR) && (LA635_17 <= LESS_THAN_OPERATOR)) || LA635_17 == LOGICAL_AND_OPERATOR || LA635_17 == LOGICAL_OR_OPERATOR || ((LA635_17 >= MINUS_OPERATOR) && (LA635_17 <= MOD_OPERATOR)) || ((LA635_17 >= MULT_OPERATOR) && (LA635_17 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_17 == PLUS_OPERATOR || ((LA635_17 >= SHIFT_LEFT_OPERATOR) && (LA635_17 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 17;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == HOUR_SYMBOL))
                    {
                        {
                            int LA635_18 = LA(2);
                            if ( (LA635_18 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_18 == EOF || ((LA635_18 >= ACCOUNT_SYMBOL) && (LA635_18 <= YEAR_SYMBOL)) || ((LA635_18 >= AND_SYMBOL) && (LA635_18 <= ASC_SYMBOL)) || LA635_18 == BETWEEN_SYMBOL || LA635_18 == COLLATE_SYMBOL || LA635_18 == CROSS_SYMBOL || ((LA635_18 >= DAY_HOUR_SYMBOL) && (LA635_18 <= DAY_SECOND_SYMBOL)) || LA635_18 == DESC_SYMBOL || LA635_18 == ELSE_SYMBOL || ((LA635_18 >= FOR_SYMBOL) && (LA635_18 <= FROM_SYMBOL)) || ((LA635_18 >= GROUP_SYMBOL) && (LA635_18 <= HAVING_SYMBOL)) || ((LA635_18 >= HOUR_MICROSECOND_SYMBOL) && (LA635_18 <= HOUR_SECOND_SYMBOL)) || LA635_18 == INNER_SYMBOL || LA635_18 == INTO_SYMBOL || LA635_18 == IN_SYMBOL || LA635_18 == IS_SYMBOL || ((LA635_18 >= JOIN_SYMBOL) && (LA635_18 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_18 >= LEFT_SYMBOL) && (LA635_18 <= LIMIT_SYMBOL)) || LA635_18 == LOCK_SYMBOL || ((LA635_18 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_18 <= MINUTE_SECOND_SYMBOL)) || ((LA635_18 >= MOD_SYMBOL) && (LA635_18 <= NATURAL_SYMBOL)) || LA635_18 == NOT_SYMBOL || LA635_18 == ON_SYMBOL || ((LA635_18 >= ORDER_SYMBOL) && (LA635_18 <= OR_SYMBOL)) || LA635_18 == PROCEDURE_SYMBOL || LA635_18 == REGEXP_SYMBOL || LA635_18 == RENAME_SYMBOL || LA635_18 == RIGHT_SYMBOL || LA635_18 == SECOND_MICROSECOND_SYMBOL || LA635_18 == SEMICOLON_SYMBOL || LA635_18 == SEPARATOR_SYMBOL || LA635_18 == SET_SYMBOL || LA635_18 == SINGLE_QUOTED_TEXT || LA635_18 == STRAIGHT_JOIN_SYMBOL || LA635_18 == THEN_SYMBOL || LA635_18 == UNION_SYMBOL || LA635_18 == USING_SYMBOL || ((LA635_18 >= WHEN_SYMBOL) && (LA635_18 <= WHERE_SYMBOL)) || LA635_18 == WITH_SYMBOL || ((LA635_18 >= XOR_SYMBOL) && (LA635_18 <= YEAR_MONTH_SYMBOL)) || ((LA635_18 >= AS_SYMBOL) && (LA635_18 <= AT_TEXT_SUFFIX)) || ((LA635_18 >= BACK_TICK_QUOTED_ID) && (LA635_18 <= BITWISE_AND_OPERATOR)) || ((LA635_18 >= BITWISE_OR_OPERATOR) && (LA635_18 <= BITWISE_XOR_OPERATOR)) || ((LA635_18 >= CLOSE_CURLY_SYMBOL) && (LA635_18 <= CLOSE_PAR_SYMBOL)) || ((LA635_18 >= COMMA_SYMBOL) && (LA635_18 <= CONCAT_PIPES_SYMBOL)) || ((LA635_18 >= DIV_OPERATOR) && (LA635_18 <= DIV_SYMBOL)) || LA635_18 == DOT_SYMBOL || LA635_18 == DOUBLE_QUOTED_TEXT || LA635_18 == EQUAL_OPERATOR || ((LA635_18 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_18 <= GREATER_THAN_OPERATOR)) || ((LA635_18 >= HEX_NUMBER) && (LA635_18 <= IDENTIFIER)) || LA635_18 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_18 >= LESS_OR_EQUAL_OPERATOR) && (LA635_18 <= LESS_THAN_OPERATOR)) || LA635_18 == LOGICAL_AND_OPERATOR || LA635_18 == LOGICAL_OR_OPERATOR || ((LA635_18 >= MINUS_OPERATOR) && (LA635_18 <= MOD_OPERATOR)) || ((LA635_18 >= MULT_OPERATOR) && (LA635_18 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_18 == PLUS_OPERATOR || ((LA635_18 >= SHIFT_LEFT_OPERATOR) && (LA635_18 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 18;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MINUTE_SYMBOL))
                    {
                        {
                            int LA635_21 = LA(2);
                            if ( (LA635_21 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_21 == EOF || ((LA635_21 >= ACCOUNT_SYMBOL) && (LA635_21 <= YEAR_SYMBOL)) || ((LA635_21 >= AND_SYMBOL) && (LA635_21 <= ASC_SYMBOL)) || LA635_21 == BETWEEN_SYMBOL || LA635_21 == COLLATE_SYMBOL || LA635_21 == CROSS_SYMBOL || ((LA635_21 >= DAY_HOUR_SYMBOL) && (LA635_21 <= DAY_SECOND_SYMBOL)) || LA635_21 == DESC_SYMBOL || LA635_21 == ELSE_SYMBOL || ((LA635_21 >= FOR_SYMBOL) && (LA635_21 <= FROM_SYMBOL)) || ((LA635_21 >= GROUP_SYMBOL) && (LA635_21 <= HAVING_SYMBOL)) || ((LA635_21 >= HOUR_MICROSECOND_SYMBOL) && (LA635_21 <= HOUR_SECOND_SYMBOL)) || LA635_21 == INNER_SYMBOL || LA635_21 == INTO_SYMBOL || LA635_21 == IN_SYMBOL || LA635_21 == IS_SYMBOL || ((LA635_21 >= JOIN_SYMBOL) && (LA635_21 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_21 >= LEFT_SYMBOL) && (LA635_21 <= LIMIT_SYMBOL)) || LA635_21 == LOCK_SYMBOL || ((LA635_21 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_21 <= MINUTE_SECOND_SYMBOL)) || ((LA635_21 >= MOD_SYMBOL) && (LA635_21 <= NATURAL_SYMBOL)) || LA635_21 == NOT_SYMBOL || LA635_21 == ON_SYMBOL || ((LA635_21 >= ORDER_SYMBOL) && (LA635_21 <= OR_SYMBOL)) || LA635_21 == PROCEDURE_SYMBOL || LA635_21 == REGEXP_SYMBOL || LA635_21 == RENAME_SYMBOL || LA635_21 == RIGHT_SYMBOL || LA635_21 == SECOND_MICROSECOND_SYMBOL || LA635_21 == SEMICOLON_SYMBOL || LA635_21 == SEPARATOR_SYMBOL || LA635_21 == SET_SYMBOL || LA635_21 == SINGLE_QUOTED_TEXT || LA635_21 == STRAIGHT_JOIN_SYMBOL || LA635_21 == THEN_SYMBOL || LA635_21 == UNION_SYMBOL || LA635_21 == USING_SYMBOL || ((LA635_21 >= WHEN_SYMBOL) && (LA635_21 <= WHERE_SYMBOL)) || LA635_21 == WITH_SYMBOL || ((LA635_21 >= XOR_SYMBOL) && (LA635_21 <= YEAR_MONTH_SYMBOL)) || ((LA635_21 >= AS_SYMBOL) && (LA635_21 <= AT_TEXT_SUFFIX)) || ((LA635_21 >= BACK_TICK_QUOTED_ID) && (LA635_21 <= BITWISE_AND_OPERATOR)) || ((LA635_21 >= BITWISE_OR_OPERATOR) && (LA635_21 <= BITWISE_XOR_OPERATOR)) || ((LA635_21 >= CLOSE_CURLY_SYMBOL) && (LA635_21 <= CLOSE_PAR_SYMBOL)) || ((LA635_21 >= COMMA_SYMBOL) && (LA635_21 <= CONCAT_PIPES_SYMBOL)) || ((LA635_21 >= DIV_OPERATOR) && (LA635_21 <= DIV_SYMBOL)) || LA635_21 == DOT_SYMBOL || LA635_21 == DOUBLE_QUOTED_TEXT || LA635_21 == EQUAL_OPERATOR || ((LA635_21 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_21 <= GREATER_THAN_OPERATOR)) || ((LA635_21 >= HEX_NUMBER) && (LA635_21 <= IDENTIFIER)) || LA635_21 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_21 >= LESS_OR_EQUAL_OPERATOR) && (LA635_21 <= LESS_THAN_OPERATOR)) || LA635_21 == LOGICAL_AND_OPERATOR || LA635_21 == LOGICAL_OR_OPERATOR || ((LA635_21 >= MINUS_OPERATOR) && (LA635_21 <= MOD_OPERATOR)) || ((LA635_21 >= MULT_OPERATOR) && (LA635_21 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_21 == PLUS_OPERATOR || ((LA635_21 >= SHIFT_LEFT_OPERATOR) && (LA635_21 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 21;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MONTH_SYMBOL))
                    {
                        {
                            int LA635_22 = LA(2);
                            if ( (LA635_22 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_22 == EOF || ((LA635_22 >= ACCOUNT_SYMBOL) && (LA635_22 <= YEAR_SYMBOL)) || ((LA635_22 >= AND_SYMBOL) && (LA635_22 <= ASC_SYMBOL)) || LA635_22 == BETWEEN_SYMBOL || LA635_22 == COLLATE_SYMBOL || LA635_22 == CROSS_SYMBOL || ((LA635_22 >= DAY_HOUR_SYMBOL) && (LA635_22 <= DAY_SECOND_SYMBOL)) || LA635_22 == DESC_SYMBOL || LA635_22 == ELSE_SYMBOL || ((LA635_22 >= FOR_SYMBOL) && (LA635_22 <= FROM_SYMBOL)) || ((LA635_22 >= GROUP_SYMBOL) && (LA635_22 <= HAVING_SYMBOL)) || ((LA635_22 >= HOUR_MICROSECOND_SYMBOL) && (LA635_22 <= HOUR_SECOND_SYMBOL)) || LA635_22 == INNER_SYMBOL || LA635_22 == INTO_SYMBOL || LA635_22 == IN_SYMBOL || LA635_22 == IS_SYMBOL || ((LA635_22 >= JOIN_SYMBOL) && (LA635_22 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_22 >= LEFT_SYMBOL) && (LA635_22 <= LIMIT_SYMBOL)) || LA635_22 == LOCK_SYMBOL || ((LA635_22 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_22 <= MINUTE_SECOND_SYMBOL)) || ((LA635_22 >= MOD_SYMBOL) && (LA635_22 <= NATURAL_SYMBOL)) || LA635_22 == NOT_SYMBOL || LA635_22 == ON_SYMBOL || ((LA635_22 >= ORDER_SYMBOL) && (LA635_22 <= OR_SYMBOL)) || LA635_22 == PROCEDURE_SYMBOL || LA635_22 == REGEXP_SYMBOL || LA635_22 == RENAME_SYMBOL || LA635_22 == RIGHT_SYMBOL || LA635_22 == SECOND_MICROSECOND_SYMBOL || LA635_22 == SEMICOLON_SYMBOL || LA635_22 == SEPARATOR_SYMBOL || LA635_22 == SET_SYMBOL || LA635_22 == SINGLE_QUOTED_TEXT || LA635_22 == STRAIGHT_JOIN_SYMBOL || LA635_22 == THEN_SYMBOL || LA635_22 == UNION_SYMBOL || LA635_22 == USING_SYMBOL || ((LA635_22 >= WHEN_SYMBOL) && (LA635_22 <= WHERE_SYMBOL)) || LA635_22 == WITH_SYMBOL || ((LA635_22 >= XOR_SYMBOL) && (LA635_22 <= YEAR_MONTH_SYMBOL)) || ((LA635_22 >= AS_SYMBOL) && (LA635_22 <= AT_TEXT_SUFFIX)) || ((LA635_22 >= BACK_TICK_QUOTED_ID) && (LA635_22 <= BITWISE_AND_OPERATOR)) || ((LA635_22 >= BITWISE_OR_OPERATOR) && (LA635_22 <= BITWISE_XOR_OPERATOR)) || ((LA635_22 >= CLOSE_CURLY_SYMBOL) && (LA635_22 <= CLOSE_PAR_SYMBOL)) || ((LA635_22 >= COMMA_SYMBOL) && (LA635_22 <= CONCAT_PIPES_SYMBOL)) || ((LA635_22 >= DIV_OPERATOR) && (LA635_22 <= DIV_SYMBOL)) || LA635_22 == DOT_SYMBOL || LA635_22 == DOUBLE_QUOTED_TEXT || LA635_22 == EQUAL_OPERATOR || ((LA635_22 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_22 <= GREATER_THAN_OPERATOR)) || ((LA635_22 >= HEX_NUMBER) && (LA635_22 <= IDENTIFIER)) || LA635_22 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_22 >= LESS_OR_EQUAL_OPERATOR) && (LA635_22 <= LESS_THAN_OPERATOR)) || LA635_22 == LOGICAL_AND_OPERATOR || LA635_22 == LOGICAL_OR_OPERATOR || ((LA635_22 >= MINUS_OPERATOR) && (LA635_22 <= MOD_OPERATOR)) || ((LA635_22 >= MULT_OPERATOR) && (LA635_22 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_22 == PLUS_OPERATOR || ((LA635_22 >= SHIFT_LEFT_OPERATOR) && (LA635_22 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 22;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == SECOND_SYMBOL))
                    {
                        {
                            int LA635_24 = LA(2);
                            if ( (LA635_24 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_24 == EOF || ((LA635_24 >= ACCOUNT_SYMBOL) && (LA635_24 <= YEAR_SYMBOL)) || ((LA635_24 >= AND_SYMBOL) && (LA635_24 <= ASC_SYMBOL)) || LA635_24 == BETWEEN_SYMBOL || LA635_24 == COLLATE_SYMBOL || LA635_24 == CROSS_SYMBOL || ((LA635_24 >= DAY_HOUR_SYMBOL) && (LA635_24 <= DAY_SECOND_SYMBOL)) || LA635_24 == DESC_SYMBOL || LA635_24 == ELSE_SYMBOL || ((LA635_24 >= FOR_SYMBOL) && (LA635_24 <= FROM_SYMBOL)) || ((LA635_24 >= GROUP_SYMBOL) && (LA635_24 <= HAVING_SYMBOL)) || ((LA635_24 >= HOUR_MICROSECOND_SYMBOL) && (LA635_24 <= HOUR_SECOND_SYMBOL)) || LA635_24 == INNER_SYMBOL || LA635_24 == INTO_SYMBOL || LA635_24 == IN_SYMBOL || LA635_24 == IS_SYMBOL || ((LA635_24 >= JOIN_SYMBOL) && (LA635_24 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_24 >= LEFT_SYMBOL) && (LA635_24 <= LIMIT_SYMBOL)) || LA635_24 == LOCK_SYMBOL || ((LA635_24 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_24 <= MINUTE_SECOND_SYMBOL)) || ((LA635_24 >= MOD_SYMBOL) && (LA635_24 <= NATURAL_SYMBOL)) || LA635_24 == NOT_SYMBOL || LA635_24 == ON_SYMBOL || ((LA635_24 >= ORDER_SYMBOL) && (LA635_24 <= OR_SYMBOL)) || LA635_24 == PROCEDURE_SYMBOL || LA635_24 == REGEXP_SYMBOL || LA635_24 == RENAME_SYMBOL || LA635_24 == RIGHT_SYMBOL || LA635_24 == SECOND_MICROSECOND_SYMBOL || LA635_24 == SEMICOLON_SYMBOL || LA635_24 == SEPARATOR_SYMBOL || LA635_24 == SET_SYMBOL || LA635_24 == SINGLE_QUOTED_TEXT || LA635_24 == STRAIGHT_JOIN_SYMBOL || LA635_24 == THEN_SYMBOL || LA635_24 == UNION_SYMBOL || LA635_24 == USING_SYMBOL || ((LA635_24 >= WHEN_SYMBOL) && (LA635_24 <= WHERE_SYMBOL)) || LA635_24 == WITH_SYMBOL || ((LA635_24 >= XOR_SYMBOL) && (LA635_24 <= YEAR_MONTH_SYMBOL)) || ((LA635_24 >= AS_SYMBOL) && (LA635_24 <= AT_TEXT_SUFFIX)) || ((LA635_24 >= BACK_TICK_QUOTED_ID) && (LA635_24 <= BITWISE_AND_OPERATOR)) || ((LA635_24 >= BITWISE_OR_OPERATOR) && (LA635_24 <= BITWISE_XOR_OPERATOR)) || ((LA635_24 >= CLOSE_CURLY_SYMBOL) && (LA635_24 <= CLOSE_PAR_SYMBOL)) || ((LA635_24 >= COMMA_SYMBOL) && (LA635_24 <= CONCAT_PIPES_SYMBOL)) || ((LA635_24 >= DIV_OPERATOR) && (LA635_24 <= DIV_SYMBOL)) || LA635_24 == DOT_SYMBOL || LA635_24 == DOUBLE_QUOTED_TEXT || LA635_24 == EQUAL_OPERATOR || ((LA635_24 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_24 <= GREATER_THAN_OPERATOR)) || ((LA635_24 >= HEX_NUMBER) && (LA635_24 <= IDENTIFIER)) || LA635_24 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_24 >= LESS_OR_EQUAL_OPERATOR) && (LA635_24 <= LESS_THAN_OPERATOR)) || LA635_24 == LOGICAL_AND_OPERATOR || LA635_24 == LOGICAL_OR_OPERATOR || ((LA635_24 >= MINUS_OPERATOR) && (LA635_24 <= MOD_OPERATOR)) || ((LA635_24 >= MULT_OPERATOR) && (LA635_24 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_24 == PLUS_OPERATOR || ((LA635_24 >= SHIFT_LEFT_OPERATOR) && (LA635_24 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 24;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == USER_SYMBOL))
                    {
                        {
                            int LA635_26 = LA(2);
                            if ( (LA635_26 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_26 == EOF || ((LA635_26 >= ACCOUNT_SYMBOL) && (LA635_26 <= YEAR_SYMBOL)) || ((LA635_26 >= AND_SYMBOL) && (LA635_26 <= ASC_SYMBOL)) || LA635_26 == BETWEEN_SYMBOL || LA635_26 == COLLATE_SYMBOL || LA635_26 == CROSS_SYMBOL || ((LA635_26 >= DAY_HOUR_SYMBOL) && (LA635_26 <= DAY_SECOND_SYMBOL)) || LA635_26 == DESC_SYMBOL || LA635_26 == ELSE_SYMBOL || ((LA635_26 >= FOR_SYMBOL) && (LA635_26 <= FROM_SYMBOL)) || ((LA635_26 >= GROUP_SYMBOL) && (LA635_26 <= HAVING_SYMBOL)) || ((LA635_26 >= HOUR_MICROSECOND_SYMBOL) && (LA635_26 <= HOUR_SECOND_SYMBOL)) || LA635_26 == INNER_SYMBOL || LA635_26 == INTO_SYMBOL || LA635_26 == IN_SYMBOL || LA635_26 == IS_SYMBOL || ((LA635_26 >= JOIN_SYMBOL) && (LA635_26 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_26 >= LEFT_SYMBOL) && (LA635_26 <= LIMIT_SYMBOL)) || LA635_26 == LOCK_SYMBOL || ((LA635_26 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_26 <= MINUTE_SECOND_SYMBOL)) || ((LA635_26 >= MOD_SYMBOL) && (LA635_26 <= NATURAL_SYMBOL)) || LA635_26 == NOT_SYMBOL || LA635_26 == ON_SYMBOL || ((LA635_26 >= ORDER_SYMBOL) && (LA635_26 <= OR_SYMBOL)) || LA635_26 == PROCEDURE_SYMBOL || LA635_26 == REGEXP_SYMBOL || LA635_26 == RENAME_SYMBOL || LA635_26 == RIGHT_SYMBOL || LA635_26 == SECOND_MICROSECOND_SYMBOL || LA635_26 == SEMICOLON_SYMBOL || LA635_26 == SEPARATOR_SYMBOL || LA635_26 == SET_SYMBOL || LA635_26 == SINGLE_QUOTED_TEXT || LA635_26 == STRAIGHT_JOIN_SYMBOL || LA635_26 == THEN_SYMBOL || LA635_26 == UNION_SYMBOL || LA635_26 == USING_SYMBOL || ((LA635_26 >= WHEN_SYMBOL) && (LA635_26 <= WHERE_SYMBOL)) || LA635_26 == WITH_SYMBOL || ((LA635_26 >= XOR_SYMBOL) && (LA635_26 <= YEAR_MONTH_SYMBOL)) || ((LA635_26 >= AS_SYMBOL) && (LA635_26 <= AT_TEXT_SUFFIX)) || ((LA635_26 >= BACK_TICK_QUOTED_ID) && (LA635_26 <= BITWISE_AND_OPERATOR)) || ((LA635_26 >= BITWISE_OR_OPERATOR) && (LA635_26 <= BITWISE_XOR_OPERATOR)) || ((LA635_26 >= CLOSE_CURLY_SYMBOL) && (LA635_26 <= CLOSE_PAR_SYMBOL)) || ((LA635_26 >= COMMA_SYMBOL) && (LA635_26 <= CONCAT_PIPES_SYMBOL)) || ((LA635_26 >= DIV_OPERATOR) && (LA635_26 <= DIV_SYMBOL)) || LA635_26 == DOT_SYMBOL || LA635_26 == DOUBLE_QUOTED_TEXT || LA635_26 == EQUAL_OPERATOR || ((LA635_26 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_26 <= GREATER_THAN_OPERATOR)) || ((LA635_26 >= HEX_NUMBER) && (LA635_26 <= IDENTIFIER)) || LA635_26 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_26 >= LESS_OR_EQUAL_OPERATOR) && (LA635_26 <= LESS_THAN_OPERATOR)) || LA635_26 == LOGICAL_AND_OPERATOR || LA635_26 == LOGICAL_OR_OPERATOR || ((LA635_26 >= MINUS_OPERATOR) && (LA635_26 <= MOD_OPERATOR)) || ((LA635_26 >= MULT_OPERATOR) && (LA635_26 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_26 == PLUS_OPERATOR || ((LA635_26 >= SHIFT_LEFT_OPERATOR) && (LA635_26 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 26;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == YEAR_SYMBOL))
                    {
                        {
                            int LA635_28 = LA(2);
                            if ( (LA635_28 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_28 == EOF || ((LA635_28 >= ACCOUNT_SYMBOL) && (LA635_28 <= YEAR_SYMBOL)) || ((LA635_28 >= AND_SYMBOL) && (LA635_28 <= ASC_SYMBOL)) || LA635_28 == BETWEEN_SYMBOL || LA635_28 == COLLATE_SYMBOL || LA635_28 == CROSS_SYMBOL || ((LA635_28 >= DAY_HOUR_SYMBOL) && (LA635_28 <= DAY_SECOND_SYMBOL)) || LA635_28 == DESC_SYMBOL || LA635_28 == ELSE_SYMBOL || ((LA635_28 >= FOR_SYMBOL) && (LA635_28 <= FROM_SYMBOL)) || ((LA635_28 >= GROUP_SYMBOL) && (LA635_28 <= HAVING_SYMBOL)) || ((LA635_28 >= HOUR_MICROSECOND_SYMBOL) && (LA635_28 <= HOUR_SECOND_SYMBOL)) || LA635_28 == INNER_SYMBOL || LA635_28 == INTO_SYMBOL || LA635_28 == IN_SYMBOL || LA635_28 == IS_SYMBOL || ((LA635_28 >= JOIN_SYMBOL) && (LA635_28 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_28 >= LEFT_SYMBOL) && (LA635_28 <= LIMIT_SYMBOL)) || LA635_28 == LOCK_SYMBOL || ((LA635_28 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_28 <= MINUTE_SECOND_SYMBOL)) || ((LA635_28 >= MOD_SYMBOL) && (LA635_28 <= NATURAL_SYMBOL)) || LA635_28 == NOT_SYMBOL || LA635_28 == ON_SYMBOL || ((LA635_28 >= ORDER_SYMBOL) && (LA635_28 <= OR_SYMBOL)) || LA635_28 == PROCEDURE_SYMBOL || LA635_28 == REGEXP_SYMBOL || LA635_28 == RENAME_SYMBOL || LA635_28 == RIGHT_SYMBOL || LA635_28 == SECOND_MICROSECOND_SYMBOL || LA635_28 == SEMICOLON_SYMBOL || LA635_28 == SEPARATOR_SYMBOL || LA635_28 == SET_SYMBOL || LA635_28 == SINGLE_QUOTED_TEXT || LA635_28 == STRAIGHT_JOIN_SYMBOL || LA635_28 == THEN_SYMBOL || LA635_28 == UNION_SYMBOL || LA635_28 == USING_SYMBOL || ((LA635_28 >= WHEN_SYMBOL) && (LA635_28 <= WHERE_SYMBOL)) || LA635_28 == WITH_SYMBOL || ((LA635_28 >= XOR_SYMBOL) && (LA635_28 <= YEAR_MONTH_SYMBOL)) || ((LA635_28 >= AS_SYMBOL) && (LA635_28 <= AT_TEXT_SUFFIX)) || ((LA635_28 >= BACK_TICK_QUOTED_ID) && (LA635_28 <= BITWISE_AND_OPERATOR)) || ((LA635_28 >= BITWISE_OR_OPERATOR) && (LA635_28 <= BITWISE_XOR_OPERATOR)) || ((LA635_28 >= CLOSE_CURLY_SYMBOL) && (LA635_28 <= CLOSE_PAR_SYMBOL)) || ((LA635_28 >= COMMA_SYMBOL) && (LA635_28 <= CONCAT_PIPES_SYMBOL)) || ((LA635_28 >= DIV_OPERATOR) && (LA635_28 <= DIV_SYMBOL)) || LA635_28 == DOT_SYMBOL || LA635_28 == DOUBLE_QUOTED_TEXT || LA635_28 == EQUAL_OPERATOR || ((LA635_28 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_28 <= GREATER_THAN_OPERATOR)) || ((LA635_28 >= HEX_NUMBER) && (LA635_28 <= IDENTIFIER)) || LA635_28 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_28 >= LESS_OR_EQUAL_OPERATOR) && (LA635_28 <= LESS_THAN_OPERATOR)) || LA635_28 == LOGICAL_AND_OPERATOR || LA635_28 == LOGICAL_OR_OPERATOR || ((LA635_28 >= MINUS_OPERATOR) && (LA635_28 <= MOD_OPERATOR)) || ((LA635_28 >= MULT_OPERATOR) && (LA635_28 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_28 == PLUS_OPERATOR || ((LA635_28 >= SHIFT_LEFT_OPERATOR) && (LA635_28 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 28;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == ADDDATE_SYMBOL || LA635_0 == SUBDATE_SYMBOL))
                    {
                        {
                            int LA635_29 = LA(2);
                            if ( (LA635_29 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_29 == EOF || ((LA635_29 >= ACCOUNT_SYMBOL) && (LA635_29 <= YEAR_SYMBOL)) || ((LA635_29 >= AND_SYMBOL) && (LA635_29 <= ASC_SYMBOL)) || LA635_29 == BETWEEN_SYMBOL || LA635_29 == COLLATE_SYMBOL || LA635_29 == CROSS_SYMBOL || ((LA635_29 >= DAY_HOUR_SYMBOL) && (LA635_29 <= DAY_SECOND_SYMBOL)) || LA635_29 == DESC_SYMBOL || LA635_29 == ELSE_SYMBOL || ((LA635_29 >= FOR_SYMBOL) && (LA635_29 <= FROM_SYMBOL)) || ((LA635_29 >= GROUP_SYMBOL) && (LA635_29 <= HAVING_SYMBOL)) || ((LA635_29 >= HOUR_MICROSECOND_SYMBOL) && (LA635_29 <= HOUR_SECOND_SYMBOL)) || LA635_29 == INNER_SYMBOL || LA635_29 == INTO_SYMBOL || LA635_29 == IN_SYMBOL || LA635_29 == IS_SYMBOL || ((LA635_29 >= JOIN_SYMBOL) && (LA635_29 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_29 >= LEFT_SYMBOL) && (LA635_29 <= LIMIT_SYMBOL)) || LA635_29 == LOCK_SYMBOL || ((LA635_29 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_29 <= MINUTE_SECOND_SYMBOL)) || ((LA635_29 >= MOD_SYMBOL) && (LA635_29 <= NATURAL_SYMBOL)) || LA635_29 == NOT_SYMBOL || LA635_29 == ON_SYMBOL || ((LA635_29 >= ORDER_SYMBOL) && (LA635_29 <= OR_SYMBOL)) || LA635_29 == PROCEDURE_SYMBOL || LA635_29 == REGEXP_SYMBOL || LA635_29 == RENAME_SYMBOL || LA635_29 == RIGHT_SYMBOL || LA635_29 == SECOND_MICROSECOND_SYMBOL || LA635_29 == SEMICOLON_SYMBOL || LA635_29 == SEPARATOR_SYMBOL || LA635_29 == SET_SYMBOL || LA635_29 == SINGLE_QUOTED_TEXT || LA635_29 == STRAIGHT_JOIN_SYMBOL || LA635_29 == THEN_SYMBOL || LA635_29 == UNION_SYMBOL || LA635_29 == USING_SYMBOL || ((LA635_29 >= WHEN_SYMBOL) && (LA635_29 <= WHERE_SYMBOL)) || LA635_29 == WITH_SYMBOL || ((LA635_29 >= XOR_SYMBOL) && (LA635_29 <= YEAR_MONTH_SYMBOL)) || ((LA635_29 >= AS_SYMBOL) && (LA635_29 <= AT_TEXT_SUFFIX)) || ((LA635_29 >= BACK_TICK_QUOTED_ID) && (LA635_29 <= BITWISE_AND_OPERATOR)) || ((LA635_29 >= BITWISE_OR_OPERATOR) && (LA635_29 <= BITWISE_XOR_OPERATOR)) || ((LA635_29 >= CLOSE_CURLY_SYMBOL) && (LA635_29 <= CLOSE_PAR_SYMBOL)) || ((LA635_29 >= COMMA_SYMBOL) && (LA635_29 <= CONCAT_PIPES_SYMBOL)) || ((LA635_29 >= DIV_OPERATOR) && (LA635_29 <= DIV_SYMBOL)) || LA635_29 == DOT_SYMBOL || LA635_29 == DOUBLE_QUOTED_TEXT || LA635_29 == EQUAL_OPERATOR || ((LA635_29 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_29 <= GREATER_THAN_OPERATOR)) || ((LA635_29 >= HEX_NUMBER) && (LA635_29 <= IDENTIFIER)) || LA635_29 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_29 >= LESS_OR_EQUAL_OPERATOR) && (LA635_29 <= LESS_THAN_OPERATOR)) || LA635_29 == LOGICAL_AND_OPERATOR || LA635_29 == LOGICAL_OR_OPERATOR || ((LA635_29 >= MINUS_OPERATOR) && (LA635_29 <= MOD_OPERATOR)) || ((LA635_29 >= MULT_OPERATOR) && (LA635_29 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_29 == PLUS_OPERATOR || ((LA635_29 >= SHIFT_LEFT_OPERATOR) && (LA635_29 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 29;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == GET_FORMAT_SYMBOL))
                    {
                        {
                            int LA635_34 = LA(2);
                            if ( (LA635_34 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_34 == EOF || ((LA635_34 >= ACCOUNT_SYMBOL) && (LA635_34 <= YEAR_SYMBOL)) || ((LA635_34 >= AND_SYMBOL) && (LA635_34 <= ASC_SYMBOL)) || LA635_34 == BETWEEN_SYMBOL || LA635_34 == COLLATE_SYMBOL || LA635_34 == CROSS_SYMBOL || ((LA635_34 >= DAY_HOUR_SYMBOL) && (LA635_34 <= DAY_SECOND_SYMBOL)) || LA635_34 == DESC_SYMBOL || LA635_34 == ELSE_SYMBOL || ((LA635_34 >= FOR_SYMBOL) && (LA635_34 <= FROM_SYMBOL)) || ((LA635_34 >= GROUP_SYMBOL) && (LA635_34 <= HAVING_SYMBOL)) || ((LA635_34 >= HOUR_MICROSECOND_SYMBOL) && (LA635_34 <= HOUR_SECOND_SYMBOL)) || LA635_34 == INNER_SYMBOL || LA635_34 == INTO_SYMBOL || LA635_34 == IN_SYMBOL || LA635_34 == IS_SYMBOL || ((LA635_34 >= JOIN_SYMBOL) && (LA635_34 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_34 >= LEFT_SYMBOL) && (LA635_34 <= LIMIT_SYMBOL)) || LA635_34 == LOCK_SYMBOL || ((LA635_34 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_34 <= MINUTE_SECOND_SYMBOL)) || ((LA635_34 >= MOD_SYMBOL) && (LA635_34 <= NATURAL_SYMBOL)) || LA635_34 == NOT_SYMBOL || LA635_34 == ON_SYMBOL || ((LA635_34 >= ORDER_SYMBOL) && (LA635_34 <= OR_SYMBOL)) || LA635_34 == PROCEDURE_SYMBOL || LA635_34 == REGEXP_SYMBOL || LA635_34 == RENAME_SYMBOL || LA635_34 == RIGHT_SYMBOL || LA635_34 == SECOND_MICROSECOND_SYMBOL || LA635_34 == SEMICOLON_SYMBOL || LA635_34 == SEPARATOR_SYMBOL || LA635_34 == SET_SYMBOL || LA635_34 == SINGLE_QUOTED_TEXT || LA635_34 == STRAIGHT_JOIN_SYMBOL || LA635_34 == THEN_SYMBOL || LA635_34 == UNION_SYMBOL || LA635_34 == USING_SYMBOL || ((LA635_34 >= WHEN_SYMBOL) && (LA635_34 <= WHERE_SYMBOL)) || LA635_34 == WITH_SYMBOL || ((LA635_34 >= XOR_SYMBOL) && (LA635_34 <= YEAR_MONTH_SYMBOL)) || ((LA635_34 >= AS_SYMBOL) && (LA635_34 <= AT_TEXT_SUFFIX)) || ((LA635_34 >= BACK_TICK_QUOTED_ID) && (LA635_34 <= BITWISE_AND_OPERATOR)) || ((LA635_34 >= BITWISE_OR_OPERATOR) && (LA635_34 <= BITWISE_XOR_OPERATOR)) || ((LA635_34 >= CLOSE_CURLY_SYMBOL) && (LA635_34 <= CLOSE_PAR_SYMBOL)) || ((LA635_34 >= COMMA_SYMBOL) && (LA635_34 <= CONCAT_PIPES_SYMBOL)) || ((LA635_34 >= DIV_OPERATOR) && (LA635_34 <= DIV_SYMBOL)) || LA635_34 == DOT_SYMBOL || LA635_34 == DOUBLE_QUOTED_TEXT || LA635_34 == EQUAL_OPERATOR || ((LA635_34 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_34 <= GREATER_THAN_OPERATOR)) || ((LA635_34 >= HEX_NUMBER) && (LA635_34 <= IDENTIFIER)) || LA635_34 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_34 >= LESS_OR_EQUAL_OPERATOR) && (LA635_34 <= LESS_THAN_OPERATOR)) || LA635_34 == LOGICAL_AND_OPERATOR || LA635_34 == LOGICAL_OR_OPERATOR || ((LA635_34 >= MINUS_OPERATOR) && (LA635_34 <= MOD_OPERATOR)) || ((LA635_34 >= MULT_OPERATOR) && (LA635_34 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_34 == PLUS_OPERATOR || ((LA635_34 >= SHIFT_LEFT_OPERATOR) && (LA635_34 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 34;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (((LA635_0 >= TIMESTAMP_ADD_SYMBOL) && (LA635_0 <= TIMESTAMP_DIFF_SYMBOL))))
                    {
                        {
                            int LA635_39 = LA(2);
                            if ( (LA635_39 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_39 == EOF || ((LA635_39 >= ACCOUNT_SYMBOL) && (LA635_39 <= YEAR_SYMBOL)) || ((LA635_39 >= AND_SYMBOL) && (LA635_39 <= ASC_SYMBOL)) || LA635_39 == BETWEEN_SYMBOL || LA635_39 == COLLATE_SYMBOL || LA635_39 == CROSS_SYMBOL || ((LA635_39 >= DAY_HOUR_SYMBOL) && (LA635_39 <= DAY_SECOND_SYMBOL)) || LA635_39 == DESC_SYMBOL || LA635_39 == ELSE_SYMBOL || ((LA635_39 >= FOR_SYMBOL) && (LA635_39 <= FROM_SYMBOL)) || ((LA635_39 >= GROUP_SYMBOL) && (LA635_39 <= HAVING_SYMBOL)) || ((LA635_39 >= HOUR_MICROSECOND_SYMBOL) && (LA635_39 <= HOUR_SECOND_SYMBOL)) || LA635_39 == INNER_SYMBOL || LA635_39 == INTO_SYMBOL || LA635_39 == IN_SYMBOL || LA635_39 == IS_SYMBOL || ((LA635_39 >= JOIN_SYMBOL) && (LA635_39 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_39 >= LEFT_SYMBOL) && (LA635_39 <= LIMIT_SYMBOL)) || LA635_39 == LOCK_SYMBOL || ((LA635_39 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_39 <= MINUTE_SECOND_SYMBOL)) || ((LA635_39 >= MOD_SYMBOL) && (LA635_39 <= NATURAL_SYMBOL)) || LA635_39 == NOT_SYMBOL || LA635_39 == ON_SYMBOL || ((LA635_39 >= ORDER_SYMBOL) && (LA635_39 <= OR_SYMBOL)) || LA635_39 == PROCEDURE_SYMBOL || LA635_39 == REGEXP_SYMBOL || LA635_39 == RENAME_SYMBOL || LA635_39 == RIGHT_SYMBOL || LA635_39 == SECOND_MICROSECOND_SYMBOL || LA635_39 == SEMICOLON_SYMBOL || LA635_39 == SEPARATOR_SYMBOL || LA635_39 == SET_SYMBOL || LA635_39 == SINGLE_QUOTED_TEXT || LA635_39 == STRAIGHT_JOIN_SYMBOL || LA635_39 == THEN_SYMBOL || LA635_39 == UNION_SYMBOL || LA635_39 == USING_SYMBOL || ((LA635_39 >= WHEN_SYMBOL) && (LA635_39 <= WHERE_SYMBOL)) || LA635_39 == WITH_SYMBOL || ((LA635_39 >= XOR_SYMBOL) && (LA635_39 <= YEAR_MONTH_SYMBOL)) || ((LA635_39 >= AS_SYMBOL) && (LA635_39 <= AT_TEXT_SUFFIX)) || ((LA635_39 >= BACK_TICK_QUOTED_ID) && (LA635_39 <= BITWISE_AND_OPERATOR)) || ((LA635_39 >= BITWISE_OR_OPERATOR) && (LA635_39 <= BITWISE_XOR_OPERATOR)) || ((LA635_39 >= CLOSE_CURLY_SYMBOL) && (LA635_39 <= CLOSE_PAR_SYMBOL)) || ((LA635_39 >= COMMA_SYMBOL) && (LA635_39 <= CONCAT_PIPES_SYMBOL)) || ((LA635_39 >= DIV_OPERATOR) && (LA635_39 <= DIV_SYMBOL)) || LA635_39 == DOT_SYMBOL || LA635_39 == DOUBLE_QUOTED_TEXT || LA635_39 == EQUAL_OPERATOR || ((LA635_39 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_39 <= GREATER_THAN_OPERATOR)) || ((LA635_39 >= HEX_NUMBER) && (LA635_39 <= IDENTIFIER)) || LA635_39 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_39 >= LESS_OR_EQUAL_OPERATOR) && (LA635_39 <= LESS_THAN_OPERATOR)) || LA635_39 == LOGICAL_AND_OPERATOR || LA635_39 == LOGICAL_OR_OPERATOR || ((LA635_39 >= MINUS_OPERATOR) && (LA635_39 <= MOD_OPERATOR)) || ((LA635_39 >= MULT_OPERATOR) && (LA635_39 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_39 == PLUS_OPERATOR || ((LA635_39 >= SHIFT_LEFT_OPERATOR) && (LA635_39 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 39;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == ASCII_SYMBOL))
                    {
                        {
                            int LA635_43 = LA(2);
                            if ( (LA635_43 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_43 == EOF || ((LA635_43 >= ACCOUNT_SYMBOL) && (LA635_43 <= YEAR_SYMBOL)) || ((LA635_43 >= AND_SYMBOL) && (LA635_43 <= ASC_SYMBOL)) || LA635_43 == BETWEEN_SYMBOL || LA635_43 == COLLATE_SYMBOL || LA635_43 == CROSS_SYMBOL || ((LA635_43 >= DAY_HOUR_SYMBOL) && (LA635_43 <= DAY_SECOND_SYMBOL)) || LA635_43 == DESC_SYMBOL || LA635_43 == ELSE_SYMBOL || ((LA635_43 >= FOR_SYMBOL) && (LA635_43 <= FROM_SYMBOL)) || ((LA635_43 >= GROUP_SYMBOL) && (LA635_43 <= HAVING_SYMBOL)) || ((LA635_43 >= HOUR_MICROSECOND_SYMBOL) && (LA635_43 <= HOUR_SECOND_SYMBOL)) || LA635_43 == INNER_SYMBOL || LA635_43 == INTO_SYMBOL || LA635_43 == IN_SYMBOL || LA635_43 == IS_SYMBOL || ((LA635_43 >= JOIN_SYMBOL) && (LA635_43 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_43 >= LEFT_SYMBOL) && (LA635_43 <= LIMIT_SYMBOL)) || LA635_43 == LOCK_SYMBOL || ((LA635_43 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_43 <= MINUTE_SECOND_SYMBOL)) || ((LA635_43 >= MOD_SYMBOL) && (LA635_43 <= NATURAL_SYMBOL)) || LA635_43 == NOT_SYMBOL || LA635_43 == ON_SYMBOL || ((LA635_43 >= ORDER_SYMBOL) && (LA635_43 <= OR_SYMBOL)) || LA635_43 == PROCEDURE_SYMBOL || LA635_43 == REGEXP_SYMBOL || LA635_43 == RENAME_SYMBOL || LA635_43 == RIGHT_SYMBOL || LA635_43 == SECOND_MICROSECOND_SYMBOL || LA635_43 == SEMICOLON_SYMBOL || LA635_43 == SEPARATOR_SYMBOL || LA635_43 == SET_SYMBOL || LA635_43 == SINGLE_QUOTED_TEXT || LA635_43 == STRAIGHT_JOIN_SYMBOL || LA635_43 == THEN_SYMBOL || LA635_43 == UNION_SYMBOL || LA635_43 == USING_SYMBOL || ((LA635_43 >= WHEN_SYMBOL) && (LA635_43 <= WHERE_SYMBOL)) || LA635_43 == WITH_SYMBOL || ((LA635_43 >= XOR_SYMBOL) && (LA635_43 <= YEAR_MONTH_SYMBOL)) || ((LA635_43 >= AS_SYMBOL) && (LA635_43 <= AT_TEXT_SUFFIX)) || ((LA635_43 >= BACK_TICK_QUOTED_ID) && (LA635_43 <= BITWISE_AND_OPERATOR)) || ((LA635_43 >= BITWISE_OR_OPERATOR) && (LA635_43 <= BITWISE_XOR_OPERATOR)) || ((LA635_43 >= CLOSE_CURLY_SYMBOL) && (LA635_43 <= CLOSE_PAR_SYMBOL)) || ((LA635_43 >= COMMA_SYMBOL) && (LA635_43 <= CONCAT_PIPES_SYMBOL)) || ((LA635_43 >= DIV_OPERATOR) && (LA635_43 <= DIV_SYMBOL)) || LA635_43 == DOT_SYMBOL || LA635_43 == DOUBLE_QUOTED_TEXT || LA635_43 == EQUAL_OPERATOR || ((LA635_43 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_43 <= GREATER_THAN_OPERATOR)) || ((LA635_43 >= HEX_NUMBER) && (LA635_43 <= IDENTIFIER)) || LA635_43 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_43 >= LESS_OR_EQUAL_OPERATOR) && (LA635_43 <= LESS_THAN_OPERATOR)) || LA635_43 == LOGICAL_AND_OPERATOR || LA635_43 == LOGICAL_OR_OPERATOR || ((LA635_43 >= MINUS_OPERATOR) && (LA635_43 <= MOD_OPERATOR)) || ((LA635_43 >= MULT_OPERATOR) && (LA635_43 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_43 == PLUS_OPERATOR || ((LA635_43 >= SHIFT_LEFT_OPERATOR) && (LA635_43 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 43;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == CHARSET_SYMBOL))
                    {
                        {
                            int LA635_44 = LA(2);
                            if ( (LA635_44 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_44 == EOF || ((LA635_44 >= ACCOUNT_SYMBOL) && (LA635_44 <= YEAR_SYMBOL)) || ((LA635_44 >= AND_SYMBOL) && (LA635_44 <= ASC_SYMBOL)) || LA635_44 == BETWEEN_SYMBOL || LA635_44 == COLLATE_SYMBOL || LA635_44 == CROSS_SYMBOL || ((LA635_44 >= DAY_HOUR_SYMBOL) && (LA635_44 <= DAY_SECOND_SYMBOL)) || LA635_44 == DESC_SYMBOL || LA635_44 == ELSE_SYMBOL || ((LA635_44 >= FOR_SYMBOL) && (LA635_44 <= FROM_SYMBOL)) || ((LA635_44 >= GROUP_SYMBOL) && (LA635_44 <= HAVING_SYMBOL)) || ((LA635_44 >= HOUR_MICROSECOND_SYMBOL) && (LA635_44 <= HOUR_SECOND_SYMBOL)) || LA635_44 == INNER_SYMBOL || LA635_44 == INTO_SYMBOL || LA635_44 == IN_SYMBOL || LA635_44 == IS_SYMBOL || ((LA635_44 >= JOIN_SYMBOL) && (LA635_44 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_44 >= LEFT_SYMBOL) && (LA635_44 <= LIMIT_SYMBOL)) || LA635_44 == LOCK_SYMBOL || ((LA635_44 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_44 <= MINUTE_SECOND_SYMBOL)) || ((LA635_44 >= MOD_SYMBOL) && (LA635_44 <= NATURAL_SYMBOL)) || LA635_44 == NOT_SYMBOL || LA635_44 == ON_SYMBOL || ((LA635_44 >= ORDER_SYMBOL) && (LA635_44 <= OR_SYMBOL)) || LA635_44 == PROCEDURE_SYMBOL || LA635_44 == REGEXP_SYMBOL || LA635_44 == RENAME_SYMBOL || LA635_44 == RIGHT_SYMBOL || LA635_44 == SECOND_MICROSECOND_SYMBOL || LA635_44 == SEMICOLON_SYMBOL || LA635_44 == SEPARATOR_SYMBOL || LA635_44 == SET_SYMBOL || LA635_44 == SINGLE_QUOTED_TEXT || LA635_44 == STRAIGHT_JOIN_SYMBOL || LA635_44 == THEN_SYMBOL || LA635_44 == UNION_SYMBOL || LA635_44 == USING_SYMBOL || ((LA635_44 >= WHEN_SYMBOL) && (LA635_44 <= WHERE_SYMBOL)) || LA635_44 == WITH_SYMBOL || ((LA635_44 >= XOR_SYMBOL) && (LA635_44 <= YEAR_MONTH_SYMBOL)) || ((LA635_44 >= AS_SYMBOL) && (LA635_44 <= AT_TEXT_SUFFIX)) || ((LA635_44 >= BACK_TICK_QUOTED_ID) && (LA635_44 <= BITWISE_AND_OPERATOR)) || ((LA635_44 >= BITWISE_OR_OPERATOR) && (LA635_44 <= BITWISE_XOR_OPERATOR)) || ((LA635_44 >= CLOSE_CURLY_SYMBOL) && (LA635_44 <= CLOSE_PAR_SYMBOL)) || ((LA635_44 >= COMMA_SYMBOL) && (LA635_44 <= CONCAT_PIPES_SYMBOL)) || ((LA635_44 >= DIV_OPERATOR) && (LA635_44 <= DIV_SYMBOL)) || LA635_44 == DOT_SYMBOL || LA635_44 == DOUBLE_QUOTED_TEXT || LA635_44 == EQUAL_OPERATOR || ((LA635_44 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_44 <= GREATER_THAN_OPERATOR)) || ((LA635_44 >= HEX_NUMBER) && (LA635_44 <= IDENTIFIER)) || LA635_44 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_44 >= LESS_OR_EQUAL_OPERATOR) && (LA635_44 <= LESS_THAN_OPERATOR)) || LA635_44 == LOGICAL_AND_OPERATOR || LA635_44 == LOGICAL_OR_OPERATOR || ((LA635_44 >= MINUS_OPERATOR) && (LA635_44 <= MOD_OPERATOR)) || ((LA635_44 >= MULT_OPERATOR) && (LA635_44 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_44 == PLUS_OPERATOR || ((LA635_44 >= SHIFT_LEFT_OPERATOR) && (LA635_44 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 44;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == COALESCE_SYMBOL))
                    {
                        {
                            int LA635_45 = LA(2);
                            if ( (LA635_45 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_45 == EOF || ((LA635_45 >= ACCOUNT_SYMBOL) && (LA635_45 <= YEAR_SYMBOL)) || ((LA635_45 >= AND_SYMBOL) && (LA635_45 <= ASC_SYMBOL)) || LA635_45 == BETWEEN_SYMBOL || LA635_45 == COLLATE_SYMBOL || LA635_45 == CROSS_SYMBOL || ((LA635_45 >= DAY_HOUR_SYMBOL) && (LA635_45 <= DAY_SECOND_SYMBOL)) || LA635_45 == DESC_SYMBOL || LA635_45 == ELSE_SYMBOL || ((LA635_45 >= FOR_SYMBOL) && (LA635_45 <= FROM_SYMBOL)) || ((LA635_45 >= GROUP_SYMBOL) && (LA635_45 <= HAVING_SYMBOL)) || ((LA635_45 >= HOUR_MICROSECOND_SYMBOL) && (LA635_45 <= HOUR_SECOND_SYMBOL)) || LA635_45 == INNER_SYMBOL || LA635_45 == INTO_SYMBOL || LA635_45 == IN_SYMBOL || LA635_45 == IS_SYMBOL || ((LA635_45 >= JOIN_SYMBOL) && (LA635_45 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_45 >= LEFT_SYMBOL) && (LA635_45 <= LIMIT_SYMBOL)) || LA635_45 == LOCK_SYMBOL || ((LA635_45 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_45 <= MINUTE_SECOND_SYMBOL)) || ((LA635_45 >= MOD_SYMBOL) && (LA635_45 <= NATURAL_SYMBOL)) || LA635_45 == NOT_SYMBOL || LA635_45 == ON_SYMBOL || ((LA635_45 >= ORDER_SYMBOL) && (LA635_45 <= OR_SYMBOL)) || LA635_45 == PROCEDURE_SYMBOL || LA635_45 == REGEXP_SYMBOL || LA635_45 == RENAME_SYMBOL || LA635_45 == RIGHT_SYMBOL || LA635_45 == SECOND_MICROSECOND_SYMBOL || LA635_45 == SEMICOLON_SYMBOL || LA635_45 == SEPARATOR_SYMBOL || LA635_45 == SET_SYMBOL || LA635_45 == SINGLE_QUOTED_TEXT || LA635_45 == STRAIGHT_JOIN_SYMBOL || LA635_45 == THEN_SYMBOL || LA635_45 == UNION_SYMBOL || LA635_45 == USING_SYMBOL || ((LA635_45 >= WHEN_SYMBOL) && (LA635_45 <= WHERE_SYMBOL)) || LA635_45 == WITH_SYMBOL || ((LA635_45 >= XOR_SYMBOL) && (LA635_45 <= YEAR_MONTH_SYMBOL)) || ((LA635_45 >= AS_SYMBOL) && (LA635_45 <= AT_TEXT_SUFFIX)) || ((LA635_45 >= BACK_TICK_QUOTED_ID) && (LA635_45 <= BITWISE_AND_OPERATOR)) || ((LA635_45 >= BITWISE_OR_OPERATOR) && (LA635_45 <= BITWISE_XOR_OPERATOR)) || ((LA635_45 >= CLOSE_CURLY_SYMBOL) && (LA635_45 <= CLOSE_PAR_SYMBOL)) || ((LA635_45 >= COMMA_SYMBOL) && (LA635_45 <= CONCAT_PIPES_SYMBOL)) || ((LA635_45 >= DIV_OPERATOR) && (LA635_45 <= DIV_SYMBOL)) || LA635_45 == DOT_SYMBOL || LA635_45 == DOUBLE_QUOTED_TEXT || LA635_45 == EQUAL_OPERATOR || ((LA635_45 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_45 <= GREATER_THAN_OPERATOR)) || ((LA635_45 >= HEX_NUMBER) && (LA635_45 <= IDENTIFIER)) || LA635_45 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_45 >= LESS_OR_EQUAL_OPERATOR) && (LA635_45 <= LESS_THAN_OPERATOR)) || LA635_45 == LOGICAL_AND_OPERATOR || LA635_45 == LOGICAL_OR_OPERATOR || ((LA635_45 >= MINUS_OPERATOR) && (LA635_45 <= MOD_OPERATOR)) || ((LA635_45 >= MULT_OPERATOR) && (LA635_45 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_45 == PLUS_OPERATOR || ((LA635_45 >= SHIFT_LEFT_OPERATOR) && (LA635_45 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 45;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == COLLATION_SYMBOL))
                    {
                        {
                            int LA635_46 = LA(2);
                            if ( (LA635_46 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_46 == EOF || ((LA635_46 >= ACCOUNT_SYMBOL) && (LA635_46 <= YEAR_SYMBOL)) || ((LA635_46 >= AND_SYMBOL) && (LA635_46 <= ASC_SYMBOL)) || LA635_46 == BETWEEN_SYMBOL || LA635_46 == COLLATE_SYMBOL || LA635_46 == CROSS_SYMBOL || ((LA635_46 >= DAY_HOUR_SYMBOL) && (LA635_46 <= DAY_SECOND_SYMBOL)) || LA635_46 == DESC_SYMBOL || LA635_46 == ELSE_SYMBOL || ((LA635_46 >= FOR_SYMBOL) && (LA635_46 <= FROM_SYMBOL)) || ((LA635_46 >= GROUP_SYMBOL) && (LA635_46 <= HAVING_SYMBOL)) || ((LA635_46 >= HOUR_MICROSECOND_SYMBOL) && (LA635_46 <= HOUR_SECOND_SYMBOL)) || LA635_46 == INNER_SYMBOL || LA635_46 == INTO_SYMBOL || LA635_46 == IN_SYMBOL || LA635_46 == IS_SYMBOL || ((LA635_46 >= JOIN_SYMBOL) && (LA635_46 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_46 >= LEFT_SYMBOL) && (LA635_46 <= LIMIT_SYMBOL)) || LA635_46 == LOCK_SYMBOL || ((LA635_46 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_46 <= MINUTE_SECOND_SYMBOL)) || ((LA635_46 >= MOD_SYMBOL) && (LA635_46 <= NATURAL_SYMBOL)) || LA635_46 == NOT_SYMBOL || LA635_46 == ON_SYMBOL || ((LA635_46 >= ORDER_SYMBOL) && (LA635_46 <= OR_SYMBOL)) || LA635_46 == PROCEDURE_SYMBOL || LA635_46 == REGEXP_SYMBOL || LA635_46 == RENAME_SYMBOL || LA635_46 == RIGHT_SYMBOL || LA635_46 == SECOND_MICROSECOND_SYMBOL || LA635_46 == SEMICOLON_SYMBOL || LA635_46 == SEPARATOR_SYMBOL || LA635_46 == SET_SYMBOL || LA635_46 == SINGLE_QUOTED_TEXT || LA635_46 == STRAIGHT_JOIN_SYMBOL || LA635_46 == THEN_SYMBOL || LA635_46 == UNION_SYMBOL || LA635_46 == USING_SYMBOL || ((LA635_46 >= WHEN_SYMBOL) && (LA635_46 <= WHERE_SYMBOL)) || LA635_46 == WITH_SYMBOL || ((LA635_46 >= XOR_SYMBOL) && (LA635_46 <= YEAR_MONTH_SYMBOL)) || ((LA635_46 >= AS_SYMBOL) && (LA635_46 <= AT_TEXT_SUFFIX)) || ((LA635_46 >= BACK_TICK_QUOTED_ID) && (LA635_46 <= BITWISE_AND_OPERATOR)) || ((LA635_46 >= BITWISE_OR_OPERATOR) && (LA635_46 <= BITWISE_XOR_OPERATOR)) || ((LA635_46 >= CLOSE_CURLY_SYMBOL) && (LA635_46 <= CLOSE_PAR_SYMBOL)) || ((LA635_46 >= COMMA_SYMBOL) && (LA635_46 <= CONCAT_PIPES_SYMBOL)) || ((LA635_46 >= DIV_OPERATOR) && (LA635_46 <= DIV_SYMBOL)) || LA635_46 == DOT_SYMBOL || LA635_46 == DOUBLE_QUOTED_TEXT || LA635_46 == EQUAL_OPERATOR || ((LA635_46 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_46 <= GREATER_THAN_OPERATOR)) || ((LA635_46 >= HEX_NUMBER) && (LA635_46 <= IDENTIFIER)) || LA635_46 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_46 >= LESS_OR_EQUAL_OPERATOR) && (LA635_46 <= LESS_THAN_OPERATOR)) || LA635_46 == LOGICAL_AND_OPERATOR || LA635_46 == LOGICAL_OR_OPERATOR || ((LA635_46 >= MINUS_OPERATOR) && (LA635_46 <= MOD_OPERATOR)) || ((LA635_46 >= MULT_OPERATOR) && (LA635_46 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_46 == PLUS_OPERATOR || ((LA635_46 >= SHIFT_LEFT_OPERATOR) && (LA635_46 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 46;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == FORMAT_SYMBOL))
                    {
                        {
                            int LA635_49 = LA(2);
                            if ( (LA635_49 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_49 == EOF || ((LA635_49 >= ACCOUNT_SYMBOL) && (LA635_49 <= YEAR_SYMBOL)) || ((LA635_49 >= AND_SYMBOL) && (LA635_49 <= ASC_SYMBOL)) || LA635_49 == BETWEEN_SYMBOL || LA635_49 == COLLATE_SYMBOL || LA635_49 == CROSS_SYMBOL || ((LA635_49 >= DAY_HOUR_SYMBOL) && (LA635_49 <= DAY_SECOND_SYMBOL)) || LA635_49 == DESC_SYMBOL || LA635_49 == ELSE_SYMBOL || ((LA635_49 >= FOR_SYMBOL) && (LA635_49 <= FROM_SYMBOL)) || ((LA635_49 >= GROUP_SYMBOL) && (LA635_49 <= HAVING_SYMBOL)) || ((LA635_49 >= HOUR_MICROSECOND_SYMBOL) && (LA635_49 <= HOUR_SECOND_SYMBOL)) || LA635_49 == INNER_SYMBOL || LA635_49 == INTO_SYMBOL || LA635_49 == IN_SYMBOL || LA635_49 == IS_SYMBOL || ((LA635_49 >= JOIN_SYMBOL) && (LA635_49 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_49 >= LEFT_SYMBOL) && (LA635_49 <= LIMIT_SYMBOL)) || LA635_49 == LOCK_SYMBOL || ((LA635_49 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_49 <= MINUTE_SECOND_SYMBOL)) || ((LA635_49 >= MOD_SYMBOL) && (LA635_49 <= NATURAL_SYMBOL)) || LA635_49 == NOT_SYMBOL || LA635_49 == ON_SYMBOL || ((LA635_49 >= ORDER_SYMBOL) && (LA635_49 <= OR_SYMBOL)) || LA635_49 == PROCEDURE_SYMBOL || LA635_49 == REGEXP_SYMBOL || LA635_49 == RENAME_SYMBOL || LA635_49 == RIGHT_SYMBOL || LA635_49 == SECOND_MICROSECOND_SYMBOL || LA635_49 == SEMICOLON_SYMBOL || LA635_49 == SEPARATOR_SYMBOL || LA635_49 == SET_SYMBOL || LA635_49 == SINGLE_QUOTED_TEXT || LA635_49 == STRAIGHT_JOIN_SYMBOL || LA635_49 == THEN_SYMBOL || LA635_49 == UNION_SYMBOL || LA635_49 == USING_SYMBOL || ((LA635_49 >= WHEN_SYMBOL) && (LA635_49 <= WHERE_SYMBOL)) || LA635_49 == WITH_SYMBOL || ((LA635_49 >= XOR_SYMBOL) && (LA635_49 <= YEAR_MONTH_SYMBOL)) || ((LA635_49 >= AS_SYMBOL) && (LA635_49 <= AT_TEXT_SUFFIX)) || ((LA635_49 >= BACK_TICK_QUOTED_ID) && (LA635_49 <= BITWISE_AND_OPERATOR)) || ((LA635_49 >= BITWISE_OR_OPERATOR) && (LA635_49 <= BITWISE_XOR_OPERATOR)) || ((LA635_49 >= CLOSE_CURLY_SYMBOL) && (LA635_49 <= CLOSE_PAR_SYMBOL)) || ((LA635_49 >= COMMA_SYMBOL) && (LA635_49 <= CONCAT_PIPES_SYMBOL)) || ((LA635_49 >= DIV_OPERATOR) && (LA635_49 <= DIV_SYMBOL)) || LA635_49 == DOT_SYMBOL || LA635_49 == DOUBLE_QUOTED_TEXT || LA635_49 == EQUAL_OPERATOR || ((LA635_49 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_49 <= GREATER_THAN_OPERATOR)) || ((LA635_49 >= HEX_NUMBER) && (LA635_49 <= IDENTIFIER)) || LA635_49 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_49 >= LESS_OR_EQUAL_OPERATOR) && (LA635_49 <= LESS_THAN_OPERATOR)) || LA635_49 == LOGICAL_AND_OPERATOR || LA635_49 == LOGICAL_OR_OPERATOR || ((LA635_49 >= MINUS_OPERATOR) && (LA635_49 <= MOD_OPERATOR)) || ((LA635_49 >= MULT_OPERATOR) && (LA635_49 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_49 == PLUS_OPERATOR || ((LA635_49 >= SHIFT_LEFT_OPERATOR) && (LA635_49 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 49;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MICROSECOND_SYMBOL))
                    {
                        {
                            int LA635_50 = LA(2);
                            if ( (LA635_50 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_50 == EOF || ((LA635_50 >= ACCOUNT_SYMBOL) && (LA635_50 <= YEAR_SYMBOL)) || ((LA635_50 >= AND_SYMBOL) && (LA635_50 <= ASC_SYMBOL)) || LA635_50 == BETWEEN_SYMBOL || LA635_50 == COLLATE_SYMBOL || LA635_50 == CROSS_SYMBOL || ((LA635_50 >= DAY_HOUR_SYMBOL) && (LA635_50 <= DAY_SECOND_SYMBOL)) || LA635_50 == DESC_SYMBOL || LA635_50 == ELSE_SYMBOL || ((LA635_50 >= FOR_SYMBOL) && (LA635_50 <= FROM_SYMBOL)) || ((LA635_50 >= GROUP_SYMBOL) && (LA635_50 <= HAVING_SYMBOL)) || ((LA635_50 >= HOUR_MICROSECOND_SYMBOL) && (LA635_50 <= HOUR_SECOND_SYMBOL)) || LA635_50 == INNER_SYMBOL || LA635_50 == INTO_SYMBOL || LA635_50 == IN_SYMBOL || LA635_50 == IS_SYMBOL || ((LA635_50 >= JOIN_SYMBOL) && (LA635_50 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_50 >= LEFT_SYMBOL) && (LA635_50 <= LIMIT_SYMBOL)) || LA635_50 == LOCK_SYMBOL || ((LA635_50 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_50 <= MINUTE_SECOND_SYMBOL)) || ((LA635_50 >= MOD_SYMBOL) && (LA635_50 <= NATURAL_SYMBOL)) || LA635_50 == NOT_SYMBOL || LA635_50 == ON_SYMBOL || ((LA635_50 >= ORDER_SYMBOL) && (LA635_50 <= OR_SYMBOL)) || LA635_50 == PROCEDURE_SYMBOL || LA635_50 == REGEXP_SYMBOL || LA635_50 == RENAME_SYMBOL || LA635_50 == RIGHT_SYMBOL || LA635_50 == SECOND_MICROSECOND_SYMBOL || LA635_50 == SEMICOLON_SYMBOL || LA635_50 == SEPARATOR_SYMBOL || LA635_50 == SET_SYMBOL || LA635_50 == SINGLE_QUOTED_TEXT || LA635_50 == STRAIGHT_JOIN_SYMBOL || LA635_50 == THEN_SYMBOL || LA635_50 == UNION_SYMBOL || LA635_50 == USING_SYMBOL || ((LA635_50 >= WHEN_SYMBOL) && (LA635_50 <= WHERE_SYMBOL)) || LA635_50 == WITH_SYMBOL || ((LA635_50 >= XOR_SYMBOL) && (LA635_50 <= YEAR_MONTH_SYMBOL)) || ((LA635_50 >= AS_SYMBOL) && (LA635_50 <= AT_TEXT_SUFFIX)) || ((LA635_50 >= BACK_TICK_QUOTED_ID) && (LA635_50 <= BITWISE_AND_OPERATOR)) || ((LA635_50 >= BITWISE_OR_OPERATOR) && (LA635_50 <= BITWISE_XOR_OPERATOR)) || ((LA635_50 >= CLOSE_CURLY_SYMBOL) && (LA635_50 <= CLOSE_PAR_SYMBOL)) || ((LA635_50 >= COMMA_SYMBOL) && (LA635_50 <= CONCAT_PIPES_SYMBOL)) || ((LA635_50 >= DIV_OPERATOR) && (LA635_50 <= DIV_SYMBOL)) || LA635_50 == DOT_SYMBOL || LA635_50 == DOUBLE_QUOTED_TEXT || LA635_50 == EQUAL_OPERATOR || ((LA635_50 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_50 <= GREATER_THAN_OPERATOR)) || ((LA635_50 >= HEX_NUMBER) && (LA635_50 <= IDENTIFIER)) || LA635_50 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_50 >= LESS_OR_EQUAL_OPERATOR) && (LA635_50 <= LESS_THAN_OPERATOR)) || LA635_50 == LOGICAL_AND_OPERATOR || LA635_50 == LOGICAL_OR_OPERATOR || ((LA635_50 >= MINUS_OPERATOR) && (LA635_50 <= MOD_OPERATOR)) || ((LA635_50 >= MULT_OPERATOR) && (LA635_50 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_50 == PLUS_OPERATOR || ((LA635_50 >= SHIFT_LEFT_OPERATOR) && (LA635_50 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 50;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == OLD_PASSWORD_SYMBOL))
                    {
                        {
                            int LA635_52 = LA(2);
                            if ( (LA635_52 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_52 == EOF || ((LA635_52 >= ACCOUNT_SYMBOL) && (LA635_52 <= YEAR_SYMBOL)) || ((LA635_52 >= AND_SYMBOL) && (LA635_52 <= ASC_SYMBOL)) || LA635_52 == BETWEEN_SYMBOL || LA635_52 == COLLATE_SYMBOL || LA635_52 == CROSS_SYMBOL || ((LA635_52 >= DAY_HOUR_SYMBOL) && (LA635_52 <= DAY_SECOND_SYMBOL)) || LA635_52 == DESC_SYMBOL || LA635_52 == ELSE_SYMBOL || ((LA635_52 >= FOR_SYMBOL) && (LA635_52 <= FROM_SYMBOL)) || ((LA635_52 >= GROUP_SYMBOL) && (LA635_52 <= HAVING_SYMBOL)) || ((LA635_52 >= HOUR_MICROSECOND_SYMBOL) && (LA635_52 <= HOUR_SECOND_SYMBOL)) || LA635_52 == INNER_SYMBOL || LA635_52 == INTO_SYMBOL || LA635_52 == IN_SYMBOL || LA635_52 == IS_SYMBOL || ((LA635_52 >= JOIN_SYMBOL) && (LA635_52 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_52 >= LEFT_SYMBOL) && (LA635_52 <= LIMIT_SYMBOL)) || LA635_52 == LOCK_SYMBOL || ((LA635_52 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_52 <= MINUTE_SECOND_SYMBOL)) || ((LA635_52 >= MOD_SYMBOL) && (LA635_52 <= NATURAL_SYMBOL)) || LA635_52 == NOT_SYMBOL || LA635_52 == ON_SYMBOL || ((LA635_52 >= ORDER_SYMBOL) && (LA635_52 <= OR_SYMBOL)) || LA635_52 == PROCEDURE_SYMBOL || LA635_52 == REGEXP_SYMBOL || LA635_52 == RENAME_SYMBOL || LA635_52 == RIGHT_SYMBOL || LA635_52 == SECOND_MICROSECOND_SYMBOL || LA635_52 == SEMICOLON_SYMBOL || LA635_52 == SEPARATOR_SYMBOL || LA635_52 == SET_SYMBOL || LA635_52 == SINGLE_QUOTED_TEXT || LA635_52 == STRAIGHT_JOIN_SYMBOL || LA635_52 == THEN_SYMBOL || LA635_52 == UNION_SYMBOL || LA635_52 == USING_SYMBOL || ((LA635_52 >= WHEN_SYMBOL) && (LA635_52 <= WHERE_SYMBOL)) || LA635_52 == WITH_SYMBOL || ((LA635_52 >= XOR_SYMBOL) && (LA635_52 <= YEAR_MONTH_SYMBOL)) || ((LA635_52 >= AS_SYMBOL) && (LA635_52 <= AT_TEXT_SUFFIX)) || ((LA635_52 >= BACK_TICK_QUOTED_ID) && (LA635_52 <= BITWISE_AND_OPERATOR)) || ((LA635_52 >= BITWISE_OR_OPERATOR) && (LA635_52 <= BITWISE_XOR_OPERATOR)) || ((LA635_52 >= CLOSE_CURLY_SYMBOL) && (LA635_52 <= CLOSE_PAR_SYMBOL)) || ((LA635_52 >= COMMA_SYMBOL) && (LA635_52 <= CONCAT_PIPES_SYMBOL)) || ((LA635_52 >= DIV_OPERATOR) && (LA635_52 <= DIV_SYMBOL)) || LA635_52 == DOT_SYMBOL || LA635_52 == DOUBLE_QUOTED_TEXT || LA635_52 == EQUAL_OPERATOR || ((LA635_52 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_52 <= GREATER_THAN_OPERATOR)) || ((LA635_52 >= HEX_NUMBER) && (LA635_52 <= IDENTIFIER)) || LA635_52 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_52 >= LESS_OR_EQUAL_OPERATOR) && (LA635_52 <= LESS_THAN_OPERATOR)) || LA635_52 == LOGICAL_AND_OPERATOR || LA635_52 == LOGICAL_OR_OPERATOR || ((LA635_52 >= MINUS_OPERATOR) && (LA635_52 <= MOD_OPERATOR)) || ((LA635_52 >= MULT_OPERATOR) && (LA635_52 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_52 == PLUS_OPERATOR || ((LA635_52 >= SHIFT_LEFT_OPERATOR) && (LA635_52 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 52;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == PASSWORD_SYMBOL))
                    {
                        {
                            int LA635_53 = LA(2);
                            if ( (LA635_53 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_53 == EOF || ((LA635_53 >= ACCOUNT_SYMBOL) && (LA635_53 <= YEAR_SYMBOL)) || ((LA635_53 >= AND_SYMBOL) && (LA635_53 <= ASC_SYMBOL)) || LA635_53 == BETWEEN_SYMBOL || LA635_53 == COLLATE_SYMBOL || LA635_53 == CROSS_SYMBOL || ((LA635_53 >= DAY_HOUR_SYMBOL) && (LA635_53 <= DAY_SECOND_SYMBOL)) || LA635_53 == DESC_SYMBOL || LA635_53 == ELSE_SYMBOL || ((LA635_53 >= FOR_SYMBOL) && (LA635_53 <= FROM_SYMBOL)) || ((LA635_53 >= GROUP_SYMBOL) && (LA635_53 <= HAVING_SYMBOL)) || ((LA635_53 >= HOUR_MICROSECOND_SYMBOL) && (LA635_53 <= HOUR_SECOND_SYMBOL)) || LA635_53 == INNER_SYMBOL || LA635_53 == INTO_SYMBOL || LA635_53 == IN_SYMBOL || LA635_53 == IS_SYMBOL || ((LA635_53 >= JOIN_SYMBOL) && (LA635_53 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_53 >= LEFT_SYMBOL) && (LA635_53 <= LIMIT_SYMBOL)) || LA635_53 == LOCK_SYMBOL || ((LA635_53 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_53 <= MINUTE_SECOND_SYMBOL)) || ((LA635_53 >= MOD_SYMBOL) && (LA635_53 <= NATURAL_SYMBOL)) || LA635_53 == NOT_SYMBOL || LA635_53 == ON_SYMBOL || ((LA635_53 >= ORDER_SYMBOL) && (LA635_53 <= OR_SYMBOL)) || LA635_53 == PROCEDURE_SYMBOL || LA635_53 == REGEXP_SYMBOL || LA635_53 == RENAME_SYMBOL || LA635_53 == RIGHT_SYMBOL || LA635_53 == SECOND_MICROSECOND_SYMBOL || LA635_53 == SEMICOLON_SYMBOL || LA635_53 == SEPARATOR_SYMBOL || LA635_53 == SET_SYMBOL || LA635_53 == SINGLE_QUOTED_TEXT || LA635_53 == STRAIGHT_JOIN_SYMBOL || LA635_53 == THEN_SYMBOL || LA635_53 == UNION_SYMBOL || LA635_53 == USING_SYMBOL || ((LA635_53 >= WHEN_SYMBOL) && (LA635_53 <= WHERE_SYMBOL)) || LA635_53 == WITH_SYMBOL || ((LA635_53 >= XOR_SYMBOL) && (LA635_53 <= YEAR_MONTH_SYMBOL)) || ((LA635_53 >= AS_SYMBOL) && (LA635_53 <= AT_TEXT_SUFFIX)) || ((LA635_53 >= BACK_TICK_QUOTED_ID) && (LA635_53 <= BITWISE_AND_OPERATOR)) || ((LA635_53 >= BITWISE_OR_OPERATOR) && (LA635_53 <= BITWISE_XOR_OPERATOR)) || ((LA635_53 >= CLOSE_CURLY_SYMBOL) && (LA635_53 <= CLOSE_PAR_SYMBOL)) || ((LA635_53 >= COMMA_SYMBOL) && (LA635_53 <= CONCAT_PIPES_SYMBOL)) || ((LA635_53 >= DIV_OPERATOR) && (LA635_53 <= DIV_SYMBOL)) || LA635_53 == DOT_SYMBOL || LA635_53 == DOUBLE_QUOTED_TEXT || LA635_53 == EQUAL_OPERATOR || ((LA635_53 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_53 <= GREATER_THAN_OPERATOR)) || ((LA635_53 >= HEX_NUMBER) && (LA635_53 <= IDENTIFIER)) || LA635_53 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_53 >= LESS_OR_EQUAL_OPERATOR) && (LA635_53 <= LESS_THAN_OPERATOR)) || LA635_53 == LOGICAL_AND_OPERATOR || LA635_53 == LOGICAL_OR_OPERATOR || ((LA635_53 >= MINUS_OPERATOR) && (LA635_53 <= MOD_OPERATOR)) || ((LA635_53 >= MULT_OPERATOR) && (LA635_53 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_53 == PLUS_OPERATOR || ((LA635_53 >= SHIFT_LEFT_OPERATOR) && (LA635_53 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 53;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == QUARTER_SYMBOL))
                    {
                        {
                            int LA635_54 = LA(2);
                            if ( (LA635_54 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_54 == EOF || ((LA635_54 >= ACCOUNT_SYMBOL) && (LA635_54 <= YEAR_SYMBOL)) || ((LA635_54 >= AND_SYMBOL) && (LA635_54 <= ASC_SYMBOL)) || LA635_54 == BETWEEN_SYMBOL || LA635_54 == COLLATE_SYMBOL || LA635_54 == CROSS_SYMBOL || ((LA635_54 >= DAY_HOUR_SYMBOL) && (LA635_54 <= DAY_SECOND_SYMBOL)) || LA635_54 == DESC_SYMBOL || LA635_54 == ELSE_SYMBOL || ((LA635_54 >= FOR_SYMBOL) && (LA635_54 <= FROM_SYMBOL)) || ((LA635_54 >= GROUP_SYMBOL) && (LA635_54 <= HAVING_SYMBOL)) || ((LA635_54 >= HOUR_MICROSECOND_SYMBOL) && (LA635_54 <= HOUR_SECOND_SYMBOL)) || LA635_54 == INNER_SYMBOL || LA635_54 == INTO_SYMBOL || LA635_54 == IN_SYMBOL || LA635_54 == IS_SYMBOL || ((LA635_54 >= JOIN_SYMBOL) && (LA635_54 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_54 >= LEFT_SYMBOL) && (LA635_54 <= LIMIT_SYMBOL)) || LA635_54 == LOCK_SYMBOL || ((LA635_54 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_54 <= MINUTE_SECOND_SYMBOL)) || ((LA635_54 >= MOD_SYMBOL) && (LA635_54 <= NATURAL_SYMBOL)) || LA635_54 == NOT_SYMBOL || LA635_54 == ON_SYMBOL || ((LA635_54 >= ORDER_SYMBOL) && (LA635_54 <= OR_SYMBOL)) || LA635_54 == PROCEDURE_SYMBOL || LA635_54 == REGEXP_SYMBOL || LA635_54 == RENAME_SYMBOL || LA635_54 == RIGHT_SYMBOL || LA635_54 == SECOND_MICROSECOND_SYMBOL || LA635_54 == SEMICOLON_SYMBOL || LA635_54 == SEPARATOR_SYMBOL || LA635_54 == SET_SYMBOL || LA635_54 == SINGLE_QUOTED_TEXT || LA635_54 == STRAIGHT_JOIN_SYMBOL || LA635_54 == THEN_SYMBOL || LA635_54 == UNION_SYMBOL || LA635_54 == USING_SYMBOL || ((LA635_54 >= WHEN_SYMBOL) && (LA635_54 <= WHERE_SYMBOL)) || LA635_54 == WITH_SYMBOL || ((LA635_54 >= XOR_SYMBOL) && (LA635_54 <= YEAR_MONTH_SYMBOL)) || ((LA635_54 >= AS_SYMBOL) && (LA635_54 <= AT_TEXT_SUFFIX)) || ((LA635_54 >= BACK_TICK_QUOTED_ID) && (LA635_54 <= BITWISE_AND_OPERATOR)) || ((LA635_54 >= BITWISE_OR_OPERATOR) && (LA635_54 <= BITWISE_XOR_OPERATOR)) || ((LA635_54 >= CLOSE_CURLY_SYMBOL) && (LA635_54 <= CLOSE_PAR_SYMBOL)) || ((LA635_54 >= COMMA_SYMBOL) && (LA635_54 <= CONCAT_PIPES_SYMBOL)) || ((LA635_54 >= DIV_OPERATOR) && (LA635_54 <= DIV_SYMBOL)) || LA635_54 == DOT_SYMBOL || LA635_54 == DOUBLE_QUOTED_TEXT || LA635_54 == EQUAL_OPERATOR || ((LA635_54 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_54 <= GREATER_THAN_OPERATOR)) || ((LA635_54 >= HEX_NUMBER) && (LA635_54 <= IDENTIFIER)) || LA635_54 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_54 >= LESS_OR_EQUAL_OPERATOR) && (LA635_54 <= LESS_THAN_OPERATOR)) || LA635_54 == LOGICAL_AND_OPERATOR || LA635_54 == LOGICAL_OR_OPERATOR || ((LA635_54 >= MINUS_OPERATOR) && (LA635_54 <= MOD_OPERATOR)) || ((LA635_54 >= MULT_OPERATOR) && (LA635_54 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_54 == PLUS_OPERATOR || ((LA635_54 >= SHIFT_LEFT_OPERATOR) && (LA635_54 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 54;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == REVERSE_SYMBOL))
                    {
                        {
                            int LA635_57 = LA(2);
                            if ( (LA635_57 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_57 == EOF || ((LA635_57 >= ACCOUNT_SYMBOL) && (LA635_57 <= YEAR_SYMBOL)) || ((LA635_57 >= AND_SYMBOL) && (LA635_57 <= ASC_SYMBOL)) || LA635_57 == BETWEEN_SYMBOL || LA635_57 == COLLATE_SYMBOL || LA635_57 == CROSS_SYMBOL || ((LA635_57 >= DAY_HOUR_SYMBOL) && (LA635_57 <= DAY_SECOND_SYMBOL)) || LA635_57 == DESC_SYMBOL || LA635_57 == ELSE_SYMBOL || ((LA635_57 >= FOR_SYMBOL) && (LA635_57 <= FROM_SYMBOL)) || ((LA635_57 >= GROUP_SYMBOL) && (LA635_57 <= HAVING_SYMBOL)) || ((LA635_57 >= HOUR_MICROSECOND_SYMBOL) && (LA635_57 <= HOUR_SECOND_SYMBOL)) || LA635_57 == INNER_SYMBOL || LA635_57 == INTO_SYMBOL || LA635_57 == IN_SYMBOL || LA635_57 == IS_SYMBOL || ((LA635_57 >= JOIN_SYMBOL) && (LA635_57 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_57 >= LEFT_SYMBOL) && (LA635_57 <= LIMIT_SYMBOL)) || LA635_57 == LOCK_SYMBOL || ((LA635_57 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_57 <= MINUTE_SECOND_SYMBOL)) || ((LA635_57 >= MOD_SYMBOL) && (LA635_57 <= NATURAL_SYMBOL)) || LA635_57 == NOT_SYMBOL || LA635_57 == ON_SYMBOL || ((LA635_57 >= ORDER_SYMBOL) && (LA635_57 <= OR_SYMBOL)) || LA635_57 == PROCEDURE_SYMBOL || LA635_57 == REGEXP_SYMBOL || LA635_57 == RENAME_SYMBOL || LA635_57 == RIGHT_SYMBOL || LA635_57 == SECOND_MICROSECOND_SYMBOL || LA635_57 == SEMICOLON_SYMBOL || LA635_57 == SEPARATOR_SYMBOL || LA635_57 == SET_SYMBOL || LA635_57 == SINGLE_QUOTED_TEXT || LA635_57 == STRAIGHT_JOIN_SYMBOL || LA635_57 == THEN_SYMBOL || LA635_57 == UNION_SYMBOL || LA635_57 == USING_SYMBOL || ((LA635_57 >= WHEN_SYMBOL) && (LA635_57 <= WHERE_SYMBOL)) || LA635_57 == WITH_SYMBOL || ((LA635_57 >= XOR_SYMBOL) && (LA635_57 <= YEAR_MONTH_SYMBOL)) || ((LA635_57 >= AS_SYMBOL) && (LA635_57 <= AT_TEXT_SUFFIX)) || ((LA635_57 >= BACK_TICK_QUOTED_ID) && (LA635_57 <= BITWISE_AND_OPERATOR)) || ((LA635_57 >= BITWISE_OR_OPERATOR) && (LA635_57 <= BITWISE_XOR_OPERATOR)) || ((LA635_57 >= CLOSE_CURLY_SYMBOL) && (LA635_57 <= CLOSE_PAR_SYMBOL)) || ((LA635_57 >= COMMA_SYMBOL) && (LA635_57 <= CONCAT_PIPES_SYMBOL)) || ((LA635_57 >= DIV_OPERATOR) && (LA635_57 <= DIV_SYMBOL)) || LA635_57 == DOT_SYMBOL || LA635_57 == DOUBLE_QUOTED_TEXT || LA635_57 == EQUAL_OPERATOR || ((LA635_57 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_57 <= GREATER_THAN_OPERATOR)) || ((LA635_57 >= HEX_NUMBER) && (LA635_57 <= IDENTIFIER)) || LA635_57 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_57 >= LESS_OR_EQUAL_OPERATOR) && (LA635_57 <= LESS_THAN_OPERATOR)) || LA635_57 == LOGICAL_AND_OPERATOR || LA635_57 == LOGICAL_OR_OPERATOR || ((LA635_57 >= MINUS_OPERATOR) && (LA635_57 <= MOD_OPERATOR)) || ((LA635_57 >= MULT_OPERATOR) && (LA635_57 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_57 == PLUS_OPERATOR || ((LA635_57 >= SHIFT_LEFT_OPERATOR) && (LA635_57 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 57;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == ROW_COUNT_SYMBOL))
                    {
                        {
                            int LA635_58 = LA(2);
                            if ( (LA635_58 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_58 == EOF || ((LA635_58 >= ACCOUNT_SYMBOL) && (LA635_58 <= YEAR_SYMBOL)) || ((LA635_58 >= AND_SYMBOL) && (LA635_58 <= ASC_SYMBOL)) || LA635_58 == BETWEEN_SYMBOL || LA635_58 == COLLATE_SYMBOL || LA635_58 == CROSS_SYMBOL || ((LA635_58 >= DAY_HOUR_SYMBOL) && (LA635_58 <= DAY_SECOND_SYMBOL)) || LA635_58 == DESC_SYMBOL || LA635_58 == ELSE_SYMBOL || ((LA635_58 >= FOR_SYMBOL) && (LA635_58 <= FROM_SYMBOL)) || ((LA635_58 >= GROUP_SYMBOL) && (LA635_58 <= HAVING_SYMBOL)) || ((LA635_58 >= HOUR_MICROSECOND_SYMBOL) && (LA635_58 <= HOUR_SECOND_SYMBOL)) || LA635_58 == INNER_SYMBOL || LA635_58 == INTO_SYMBOL || LA635_58 == IN_SYMBOL || LA635_58 == IS_SYMBOL || ((LA635_58 >= JOIN_SYMBOL) && (LA635_58 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_58 >= LEFT_SYMBOL) && (LA635_58 <= LIMIT_SYMBOL)) || LA635_58 == LOCK_SYMBOL || ((LA635_58 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_58 <= MINUTE_SECOND_SYMBOL)) || ((LA635_58 >= MOD_SYMBOL) && (LA635_58 <= NATURAL_SYMBOL)) || LA635_58 == NOT_SYMBOL || LA635_58 == ON_SYMBOL || ((LA635_58 >= ORDER_SYMBOL) && (LA635_58 <= OR_SYMBOL)) || LA635_58 == PROCEDURE_SYMBOL || LA635_58 == REGEXP_SYMBOL || LA635_58 == RENAME_SYMBOL || LA635_58 == RIGHT_SYMBOL || LA635_58 == SECOND_MICROSECOND_SYMBOL || LA635_58 == SEMICOLON_SYMBOL || LA635_58 == SEPARATOR_SYMBOL || LA635_58 == SET_SYMBOL || LA635_58 == SINGLE_QUOTED_TEXT || LA635_58 == STRAIGHT_JOIN_SYMBOL || LA635_58 == THEN_SYMBOL || LA635_58 == UNION_SYMBOL || LA635_58 == USING_SYMBOL || ((LA635_58 >= WHEN_SYMBOL) && (LA635_58 <= WHERE_SYMBOL)) || LA635_58 == WITH_SYMBOL || ((LA635_58 >= XOR_SYMBOL) && (LA635_58 <= YEAR_MONTH_SYMBOL)) || ((LA635_58 >= AS_SYMBOL) && (LA635_58 <= AT_TEXT_SUFFIX)) || ((LA635_58 >= BACK_TICK_QUOTED_ID) && (LA635_58 <= BITWISE_AND_OPERATOR)) || ((LA635_58 >= BITWISE_OR_OPERATOR) && (LA635_58 <= BITWISE_XOR_OPERATOR)) || ((LA635_58 >= CLOSE_CURLY_SYMBOL) && (LA635_58 <= CLOSE_PAR_SYMBOL)) || ((LA635_58 >= COMMA_SYMBOL) && (LA635_58 <= CONCAT_PIPES_SYMBOL)) || ((LA635_58 >= DIV_OPERATOR) && (LA635_58 <= DIV_SYMBOL)) || LA635_58 == DOT_SYMBOL || LA635_58 == DOUBLE_QUOTED_TEXT || LA635_58 == EQUAL_OPERATOR || ((LA635_58 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_58 <= GREATER_THAN_OPERATOR)) || ((LA635_58 >= HEX_NUMBER) && (LA635_58 <= IDENTIFIER)) || LA635_58 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_58 >= LESS_OR_EQUAL_OPERATOR) && (LA635_58 <= LESS_THAN_OPERATOR)) || LA635_58 == LOGICAL_AND_OPERATOR || LA635_58 == LOGICAL_OR_OPERATOR || ((LA635_58 >= MINUS_OPERATOR) && (LA635_58 <= MOD_OPERATOR)) || ((LA635_58 >= MULT_OPERATOR) && (LA635_58 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_58 == PLUS_OPERATOR || ((LA635_58 >= SHIFT_LEFT_OPERATOR) && (LA635_58 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 58;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == TRUNCATE_SYMBOL))
                    {
                        {
                            int LA635_59 = LA(2);
                            if ( (LA635_59 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_59 == EOF || ((LA635_59 >= ACCOUNT_SYMBOL) && (LA635_59 <= YEAR_SYMBOL)) || ((LA635_59 >= AND_SYMBOL) && (LA635_59 <= ASC_SYMBOL)) || LA635_59 == BETWEEN_SYMBOL || LA635_59 == COLLATE_SYMBOL || LA635_59 == CROSS_SYMBOL || ((LA635_59 >= DAY_HOUR_SYMBOL) && (LA635_59 <= DAY_SECOND_SYMBOL)) || LA635_59 == DESC_SYMBOL || LA635_59 == ELSE_SYMBOL || ((LA635_59 >= FOR_SYMBOL) && (LA635_59 <= FROM_SYMBOL)) || ((LA635_59 >= GROUP_SYMBOL) && (LA635_59 <= HAVING_SYMBOL)) || ((LA635_59 >= HOUR_MICROSECOND_SYMBOL) && (LA635_59 <= HOUR_SECOND_SYMBOL)) || LA635_59 == INNER_SYMBOL || LA635_59 == INTO_SYMBOL || LA635_59 == IN_SYMBOL || LA635_59 == IS_SYMBOL || ((LA635_59 >= JOIN_SYMBOL) && (LA635_59 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_59 >= LEFT_SYMBOL) && (LA635_59 <= LIMIT_SYMBOL)) || LA635_59 == LOCK_SYMBOL || ((LA635_59 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_59 <= MINUTE_SECOND_SYMBOL)) || ((LA635_59 >= MOD_SYMBOL) && (LA635_59 <= NATURAL_SYMBOL)) || LA635_59 == NOT_SYMBOL || LA635_59 == ON_SYMBOL || ((LA635_59 >= ORDER_SYMBOL) && (LA635_59 <= OR_SYMBOL)) || LA635_59 == PROCEDURE_SYMBOL || LA635_59 == REGEXP_SYMBOL || LA635_59 == RENAME_SYMBOL || LA635_59 == RIGHT_SYMBOL || LA635_59 == SECOND_MICROSECOND_SYMBOL || LA635_59 == SEMICOLON_SYMBOL || LA635_59 == SEPARATOR_SYMBOL || LA635_59 == SET_SYMBOL || LA635_59 == SINGLE_QUOTED_TEXT || LA635_59 == STRAIGHT_JOIN_SYMBOL || LA635_59 == THEN_SYMBOL || LA635_59 == UNION_SYMBOL || LA635_59 == USING_SYMBOL || ((LA635_59 >= WHEN_SYMBOL) && (LA635_59 <= WHERE_SYMBOL)) || LA635_59 == WITH_SYMBOL || ((LA635_59 >= XOR_SYMBOL) && (LA635_59 <= YEAR_MONTH_SYMBOL)) || ((LA635_59 >= AS_SYMBOL) && (LA635_59 <= AT_TEXT_SUFFIX)) || ((LA635_59 >= BACK_TICK_QUOTED_ID) && (LA635_59 <= BITWISE_AND_OPERATOR)) || ((LA635_59 >= BITWISE_OR_OPERATOR) && (LA635_59 <= BITWISE_XOR_OPERATOR)) || ((LA635_59 >= CLOSE_CURLY_SYMBOL) && (LA635_59 <= CLOSE_PAR_SYMBOL)) || ((LA635_59 >= COMMA_SYMBOL) && (LA635_59 <= CONCAT_PIPES_SYMBOL)) || ((LA635_59 >= DIV_OPERATOR) && (LA635_59 <= DIV_SYMBOL)) || LA635_59 == DOT_SYMBOL || LA635_59 == DOUBLE_QUOTED_TEXT || LA635_59 == EQUAL_OPERATOR || ((LA635_59 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_59 <= GREATER_THAN_OPERATOR)) || ((LA635_59 >= HEX_NUMBER) && (LA635_59 <= IDENTIFIER)) || LA635_59 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_59 >= LESS_OR_EQUAL_OPERATOR) && (LA635_59 <= LESS_THAN_OPERATOR)) || LA635_59 == LOGICAL_AND_OPERATOR || LA635_59 == LOGICAL_OR_OPERATOR || ((LA635_59 >= MINUS_OPERATOR) && (LA635_59 <= MOD_OPERATOR)) || ((LA635_59 >= MULT_OPERATOR) && (LA635_59 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_59 == PLUS_OPERATOR || ((LA635_59 >= SHIFT_LEFT_OPERATOR) && (LA635_59 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 59;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == WEEK_SYMBOL))
                    {
                        {
                            int LA635_60 = LA(2);
                            if ( (LA635_60 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_60 == EOF || ((LA635_60 >= ACCOUNT_SYMBOL) && (LA635_60 <= YEAR_SYMBOL)) || ((LA635_60 >= AND_SYMBOL) && (LA635_60 <= ASC_SYMBOL)) || LA635_60 == BETWEEN_SYMBOL || LA635_60 == COLLATE_SYMBOL || LA635_60 == CROSS_SYMBOL || ((LA635_60 >= DAY_HOUR_SYMBOL) && (LA635_60 <= DAY_SECOND_SYMBOL)) || LA635_60 == DESC_SYMBOL || LA635_60 == ELSE_SYMBOL || ((LA635_60 >= FOR_SYMBOL) && (LA635_60 <= FROM_SYMBOL)) || ((LA635_60 >= GROUP_SYMBOL) && (LA635_60 <= HAVING_SYMBOL)) || ((LA635_60 >= HOUR_MICROSECOND_SYMBOL) && (LA635_60 <= HOUR_SECOND_SYMBOL)) || LA635_60 == INNER_SYMBOL || LA635_60 == INTO_SYMBOL || LA635_60 == IN_SYMBOL || LA635_60 == IS_SYMBOL || ((LA635_60 >= JOIN_SYMBOL) && (LA635_60 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_60 >= LEFT_SYMBOL) && (LA635_60 <= LIMIT_SYMBOL)) || LA635_60 == LOCK_SYMBOL || ((LA635_60 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_60 <= MINUTE_SECOND_SYMBOL)) || ((LA635_60 >= MOD_SYMBOL) && (LA635_60 <= NATURAL_SYMBOL)) || LA635_60 == NOT_SYMBOL || LA635_60 == ON_SYMBOL || ((LA635_60 >= ORDER_SYMBOL) && (LA635_60 <= OR_SYMBOL)) || LA635_60 == PROCEDURE_SYMBOL || LA635_60 == REGEXP_SYMBOL || LA635_60 == RENAME_SYMBOL || LA635_60 == RIGHT_SYMBOL || LA635_60 == SECOND_MICROSECOND_SYMBOL || LA635_60 == SEMICOLON_SYMBOL || LA635_60 == SEPARATOR_SYMBOL || LA635_60 == SET_SYMBOL || LA635_60 == SINGLE_QUOTED_TEXT || LA635_60 == STRAIGHT_JOIN_SYMBOL || LA635_60 == THEN_SYMBOL || LA635_60 == UNION_SYMBOL || LA635_60 == USING_SYMBOL || ((LA635_60 >= WHEN_SYMBOL) && (LA635_60 <= WHERE_SYMBOL)) || LA635_60 == WITH_SYMBOL || ((LA635_60 >= XOR_SYMBOL) && (LA635_60 <= YEAR_MONTH_SYMBOL)) || ((LA635_60 >= AS_SYMBOL) && (LA635_60 <= AT_TEXT_SUFFIX)) || ((LA635_60 >= BACK_TICK_QUOTED_ID) && (LA635_60 <= BITWISE_AND_OPERATOR)) || ((LA635_60 >= BITWISE_OR_OPERATOR) && (LA635_60 <= BITWISE_XOR_OPERATOR)) || ((LA635_60 >= CLOSE_CURLY_SYMBOL) && (LA635_60 <= CLOSE_PAR_SYMBOL)) || ((LA635_60 >= COMMA_SYMBOL) && (LA635_60 <= CONCAT_PIPES_SYMBOL)) || ((LA635_60 >= DIV_OPERATOR) && (LA635_60 <= DIV_SYMBOL)) || LA635_60 == DOT_SYMBOL || LA635_60 == DOUBLE_QUOTED_TEXT || LA635_60 == EQUAL_OPERATOR || ((LA635_60 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_60 <= GREATER_THAN_OPERATOR)) || ((LA635_60 >= HEX_NUMBER) && (LA635_60 <= IDENTIFIER)) || LA635_60 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_60 >= LESS_OR_EQUAL_OPERATOR) && (LA635_60 <= LESS_THAN_OPERATOR)) || LA635_60 == LOGICAL_AND_OPERATOR || LA635_60 == LOGICAL_OR_OPERATOR || ((LA635_60 >= MINUS_OPERATOR) && (LA635_60 <= MOD_OPERATOR)) || ((LA635_60 >= MULT_OPERATOR) && (LA635_60 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_60 == PLUS_OPERATOR || ((LA635_60 >= SHIFT_LEFT_OPERATOR) && (LA635_60 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 60;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == WEIGHT_STRING_SYMBOL))
                    {
                        {
                            int LA635_61 = LA(2);
                            if ( (LA635_61 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_61 == EOF || ((LA635_61 >= ACCOUNT_SYMBOL) && (LA635_61 <= YEAR_SYMBOL)) || ((LA635_61 >= AND_SYMBOL) && (LA635_61 <= ASC_SYMBOL)) || LA635_61 == BETWEEN_SYMBOL || LA635_61 == COLLATE_SYMBOL || LA635_61 == CROSS_SYMBOL || ((LA635_61 >= DAY_HOUR_SYMBOL) && (LA635_61 <= DAY_SECOND_SYMBOL)) || LA635_61 == DESC_SYMBOL || LA635_61 == ELSE_SYMBOL || ((LA635_61 >= FOR_SYMBOL) && (LA635_61 <= FROM_SYMBOL)) || ((LA635_61 >= GROUP_SYMBOL) && (LA635_61 <= HAVING_SYMBOL)) || ((LA635_61 >= HOUR_MICROSECOND_SYMBOL) && (LA635_61 <= HOUR_SECOND_SYMBOL)) || LA635_61 == INNER_SYMBOL || LA635_61 == INTO_SYMBOL || LA635_61 == IN_SYMBOL || LA635_61 == IS_SYMBOL || ((LA635_61 >= JOIN_SYMBOL) && (LA635_61 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_61 >= LEFT_SYMBOL) && (LA635_61 <= LIMIT_SYMBOL)) || LA635_61 == LOCK_SYMBOL || ((LA635_61 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_61 <= MINUTE_SECOND_SYMBOL)) || ((LA635_61 >= MOD_SYMBOL) && (LA635_61 <= NATURAL_SYMBOL)) || LA635_61 == NOT_SYMBOL || LA635_61 == ON_SYMBOL || ((LA635_61 >= ORDER_SYMBOL) && (LA635_61 <= OR_SYMBOL)) || LA635_61 == PROCEDURE_SYMBOL || LA635_61 == REGEXP_SYMBOL || LA635_61 == RENAME_SYMBOL || LA635_61 == RIGHT_SYMBOL || LA635_61 == SECOND_MICROSECOND_SYMBOL || LA635_61 == SEMICOLON_SYMBOL || LA635_61 == SEPARATOR_SYMBOL || LA635_61 == SET_SYMBOL || LA635_61 == SINGLE_QUOTED_TEXT || LA635_61 == STRAIGHT_JOIN_SYMBOL || LA635_61 == THEN_SYMBOL || LA635_61 == UNION_SYMBOL || LA635_61 == USING_SYMBOL || ((LA635_61 >= WHEN_SYMBOL) && (LA635_61 <= WHERE_SYMBOL)) || LA635_61 == WITH_SYMBOL || ((LA635_61 >= XOR_SYMBOL) && (LA635_61 <= YEAR_MONTH_SYMBOL)) || ((LA635_61 >= AS_SYMBOL) && (LA635_61 <= AT_TEXT_SUFFIX)) || ((LA635_61 >= BACK_TICK_QUOTED_ID) && (LA635_61 <= BITWISE_AND_OPERATOR)) || ((LA635_61 >= BITWISE_OR_OPERATOR) && (LA635_61 <= BITWISE_XOR_OPERATOR)) || ((LA635_61 >= CLOSE_CURLY_SYMBOL) && (LA635_61 <= CLOSE_PAR_SYMBOL)) || ((LA635_61 >= COMMA_SYMBOL) && (LA635_61 <= CONCAT_PIPES_SYMBOL)) || ((LA635_61 >= DIV_OPERATOR) && (LA635_61 <= DIV_SYMBOL)) || LA635_61 == DOT_SYMBOL || LA635_61 == DOUBLE_QUOTED_TEXT || LA635_61 == EQUAL_OPERATOR || ((LA635_61 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_61 <= GREATER_THAN_OPERATOR)) || ((LA635_61 >= HEX_NUMBER) && (LA635_61 <= IDENTIFIER)) || LA635_61 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_61 >= LESS_OR_EQUAL_OPERATOR) && (LA635_61 <= LESS_THAN_OPERATOR)) || LA635_61 == LOGICAL_AND_OPERATOR || LA635_61 == LOGICAL_OR_OPERATOR || ((LA635_61 >= MINUS_OPERATOR) && (LA635_61 <= MOD_OPERATOR)) || ((LA635_61 >= MULT_OPERATOR) && (LA635_61 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_61 == PLUS_OPERATOR || ((LA635_61 >= SHIFT_LEFT_OPERATOR) && (LA635_61 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 61;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == GEOMETRYCOLLECTION_SYMBOL))
                    {
                        {
                            int LA635_62 = LA(2);
                            if ( (LA635_62 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_62 == EOF || ((LA635_62 >= ACCOUNT_SYMBOL) && (LA635_62 <= YEAR_SYMBOL)) || ((LA635_62 >= AND_SYMBOL) && (LA635_62 <= ASC_SYMBOL)) || LA635_62 == BETWEEN_SYMBOL || LA635_62 == COLLATE_SYMBOL || LA635_62 == CROSS_SYMBOL || ((LA635_62 >= DAY_HOUR_SYMBOL) && (LA635_62 <= DAY_SECOND_SYMBOL)) || LA635_62 == DESC_SYMBOL || LA635_62 == ELSE_SYMBOL || ((LA635_62 >= FOR_SYMBOL) && (LA635_62 <= FROM_SYMBOL)) || ((LA635_62 >= GROUP_SYMBOL) && (LA635_62 <= HAVING_SYMBOL)) || ((LA635_62 >= HOUR_MICROSECOND_SYMBOL) && (LA635_62 <= HOUR_SECOND_SYMBOL)) || LA635_62 == INNER_SYMBOL || LA635_62 == INTO_SYMBOL || LA635_62 == IN_SYMBOL || LA635_62 == IS_SYMBOL || ((LA635_62 >= JOIN_SYMBOL) && (LA635_62 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_62 >= LEFT_SYMBOL) && (LA635_62 <= LIMIT_SYMBOL)) || LA635_62 == LOCK_SYMBOL || ((LA635_62 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_62 <= MINUTE_SECOND_SYMBOL)) || ((LA635_62 >= MOD_SYMBOL) && (LA635_62 <= NATURAL_SYMBOL)) || LA635_62 == NOT_SYMBOL || LA635_62 == ON_SYMBOL || ((LA635_62 >= ORDER_SYMBOL) && (LA635_62 <= OR_SYMBOL)) || LA635_62 == PROCEDURE_SYMBOL || LA635_62 == REGEXP_SYMBOL || LA635_62 == RENAME_SYMBOL || LA635_62 == RIGHT_SYMBOL || LA635_62 == SECOND_MICROSECOND_SYMBOL || LA635_62 == SEMICOLON_SYMBOL || LA635_62 == SEPARATOR_SYMBOL || LA635_62 == SET_SYMBOL || LA635_62 == SINGLE_QUOTED_TEXT || LA635_62 == STRAIGHT_JOIN_SYMBOL || LA635_62 == THEN_SYMBOL || LA635_62 == UNION_SYMBOL || LA635_62 == USING_SYMBOL || ((LA635_62 >= WHEN_SYMBOL) && (LA635_62 <= WHERE_SYMBOL)) || LA635_62 == WITH_SYMBOL || ((LA635_62 >= XOR_SYMBOL) && (LA635_62 <= YEAR_MONTH_SYMBOL)) || ((LA635_62 >= AS_SYMBOL) && (LA635_62 <= AT_TEXT_SUFFIX)) || ((LA635_62 >= BACK_TICK_QUOTED_ID) && (LA635_62 <= BITWISE_AND_OPERATOR)) || ((LA635_62 >= BITWISE_OR_OPERATOR) && (LA635_62 <= BITWISE_XOR_OPERATOR)) || ((LA635_62 >= CLOSE_CURLY_SYMBOL) && (LA635_62 <= CLOSE_PAR_SYMBOL)) || ((LA635_62 >= COMMA_SYMBOL) && (LA635_62 <= CONCAT_PIPES_SYMBOL)) || ((LA635_62 >= DIV_OPERATOR) && (LA635_62 <= DIV_SYMBOL)) || LA635_62 == DOT_SYMBOL || LA635_62 == DOUBLE_QUOTED_TEXT || LA635_62 == EQUAL_OPERATOR || ((LA635_62 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_62 <= GREATER_THAN_OPERATOR)) || ((LA635_62 >= HEX_NUMBER) && (LA635_62 <= IDENTIFIER)) || LA635_62 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_62 >= LESS_OR_EQUAL_OPERATOR) && (LA635_62 <= LESS_THAN_OPERATOR)) || LA635_62 == LOGICAL_AND_OPERATOR || LA635_62 == LOGICAL_OR_OPERATOR || ((LA635_62 >= MINUS_OPERATOR) && (LA635_62 <= MOD_OPERATOR)) || ((LA635_62 >= MULT_OPERATOR) && (LA635_62 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_62 == PLUS_OPERATOR || ((LA635_62 >= SHIFT_LEFT_OPERATOR) && (LA635_62 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 62;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == LINESTRING_SYMBOL))
                    {
                        {
                            int LA635_63 = LA(2);
                            if ( (LA635_63 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_63 == EOF || ((LA635_63 >= ACCOUNT_SYMBOL) && (LA635_63 <= YEAR_SYMBOL)) || ((LA635_63 >= AND_SYMBOL) && (LA635_63 <= ASC_SYMBOL)) || LA635_63 == BETWEEN_SYMBOL || LA635_63 == COLLATE_SYMBOL || LA635_63 == CROSS_SYMBOL || ((LA635_63 >= DAY_HOUR_SYMBOL) && (LA635_63 <= DAY_SECOND_SYMBOL)) || LA635_63 == DESC_SYMBOL || LA635_63 == ELSE_SYMBOL || ((LA635_63 >= FOR_SYMBOL) && (LA635_63 <= FROM_SYMBOL)) || ((LA635_63 >= GROUP_SYMBOL) && (LA635_63 <= HAVING_SYMBOL)) || ((LA635_63 >= HOUR_MICROSECOND_SYMBOL) && (LA635_63 <= HOUR_SECOND_SYMBOL)) || LA635_63 == INNER_SYMBOL || LA635_63 == INTO_SYMBOL || LA635_63 == IN_SYMBOL || LA635_63 == IS_SYMBOL || ((LA635_63 >= JOIN_SYMBOL) && (LA635_63 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_63 >= LEFT_SYMBOL) && (LA635_63 <= LIMIT_SYMBOL)) || LA635_63 == LOCK_SYMBOL || ((LA635_63 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_63 <= MINUTE_SECOND_SYMBOL)) || ((LA635_63 >= MOD_SYMBOL) && (LA635_63 <= NATURAL_SYMBOL)) || LA635_63 == NOT_SYMBOL || LA635_63 == ON_SYMBOL || ((LA635_63 >= ORDER_SYMBOL) && (LA635_63 <= OR_SYMBOL)) || LA635_63 == PROCEDURE_SYMBOL || LA635_63 == REGEXP_SYMBOL || LA635_63 == RENAME_SYMBOL || LA635_63 == RIGHT_SYMBOL || LA635_63 == SECOND_MICROSECOND_SYMBOL || LA635_63 == SEMICOLON_SYMBOL || LA635_63 == SEPARATOR_SYMBOL || LA635_63 == SET_SYMBOL || LA635_63 == SINGLE_QUOTED_TEXT || LA635_63 == STRAIGHT_JOIN_SYMBOL || LA635_63 == THEN_SYMBOL || LA635_63 == UNION_SYMBOL || LA635_63 == USING_SYMBOL || ((LA635_63 >= WHEN_SYMBOL) && (LA635_63 <= WHERE_SYMBOL)) || LA635_63 == WITH_SYMBOL || ((LA635_63 >= XOR_SYMBOL) && (LA635_63 <= YEAR_MONTH_SYMBOL)) || ((LA635_63 >= AS_SYMBOL) && (LA635_63 <= AT_TEXT_SUFFIX)) || ((LA635_63 >= BACK_TICK_QUOTED_ID) && (LA635_63 <= BITWISE_AND_OPERATOR)) || ((LA635_63 >= BITWISE_OR_OPERATOR) && (LA635_63 <= BITWISE_XOR_OPERATOR)) || ((LA635_63 >= CLOSE_CURLY_SYMBOL) && (LA635_63 <= CLOSE_PAR_SYMBOL)) || ((LA635_63 >= COMMA_SYMBOL) && (LA635_63 <= CONCAT_PIPES_SYMBOL)) || ((LA635_63 >= DIV_OPERATOR) && (LA635_63 <= DIV_SYMBOL)) || LA635_63 == DOT_SYMBOL || LA635_63 == DOUBLE_QUOTED_TEXT || LA635_63 == EQUAL_OPERATOR || ((LA635_63 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_63 <= GREATER_THAN_OPERATOR)) || ((LA635_63 >= HEX_NUMBER) && (LA635_63 <= IDENTIFIER)) || LA635_63 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_63 >= LESS_OR_EQUAL_OPERATOR) && (LA635_63 <= LESS_THAN_OPERATOR)) || LA635_63 == LOGICAL_AND_OPERATOR || LA635_63 == LOGICAL_OR_OPERATOR || ((LA635_63 >= MINUS_OPERATOR) && (LA635_63 <= MOD_OPERATOR)) || ((LA635_63 >= MULT_OPERATOR) && (LA635_63 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_63 == PLUS_OPERATOR || ((LA635_63 >= SHIFT_LEFT_OPERATOR) && (LA635_63 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 63;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MULTILINESTRING_SYMBOL))
                    {
                        {
                            int LA635_64 = LA(2);
                            if ( (LA635_64 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_64 == EOF || ((LA635_64 >= ACCOUNT_SYMBOL) && (LA635_64 <= YEAR_SYMBOL)) || ((LA635_64 >= AND_SYMBOL) && (LA635_64 <= ASC_SYMBOL)) || LA635_64 == BETWEEN_SYMBOL || LA635_64 == COLLATE_SYMBOL || LA635_64 == CROSS_SYMBOL || ((LA635_64 >= DAY_HOUR_SYMBOL) && (LA635_64 <= DAY_SECOND_SYMBOL)) || LA635_64 == DESC_SYMBOL || LA635_64 == ELSE_SYMBOL || ((LA635_64 >= FOR_SYMBOL) && (LA635_64 <= FROM_SYMBOL)) || ((LA635_64 >= GROUP_SYMBOL) && (LA635_64 <= HAVING_SYMBOL)) || ((LA635_64 >= HOUR_MICROSECOND_SYMBOL) && (LA635_64 <= HOUR_SECOND_SYMBOL)) || LA635_64 == INNER_SYMBOL || LA635_64 == INTO_SYMBOL || LA635_64 == IN_SYMBOL || LA635_64 == IS_SYMBOL || ((LA635_64 >= JOIN_SYMBOL) && (LA635_64 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_64 >= LEFT_SYMBOL) && (LA635_64 <= LIMIT_SYMBOL)) || LA635_64 == LOCK_SYMBOL || ((LA635_64 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_64 <= MINUTE_SECOND_SYMBOL)) || ((LA635_64 >= MOD_SYMBOL) && (LA635_64 <= NATURAL_SYMBOL)) || LA635_64 == NOT_SYMBOL || LA635_64 == ON_SYMBOL || ((LA635_64 >= ORDER_SYMBOL) && (LA635_64 <= OR_SYMBOL)) || LA635_64 == PROCEDURE_SYMBOL || LA635_64 == REGEXP_SYMBOL || LA635_64 == RENAME_SYMBOL || LA635_64 == RIGHT_SYMBOL || LA635_64 == SECOND_MICROSECOND_SYMBOL || LA635_64 == SEMICOLON_SYMBOL || LA635_64 == SEPARATOR_SYMBOL || LA635_64 == SET_SYMBOL || LA635_64 == SINGLE_QUOTED_TEXT || LA635_64 == STRAIGHT_JOIN_SYMBOL || LA635_64 == THEN_SYMBOL || LA635_64 == UNION_SYMBOL || LA635_64 == USING_SYMBOL || ((LA635_64 >= WHEN_SYMBOL) && (LA635_64 <= WHERE_SYMBOL)) || LA635_64 == WITH_SYMBOL || ((LA635_64 >= XOR_SYMBOL) && (LA635_64 <= YEAR_MONTH_SYMBOL)) || ((LA635_64 >= AS_SYMBOL) && (LA635_64 <= AT_TEXT_SUFFIX)) || ((LA635_64 >= BACK_TICK_QUOTED_ID) && (LA635_64 <= BITWISE_AND_OPERATOR)) || ((LA635_64 >= BITWISE_OR_OPERATOR) && (LA635_64 <= BITWISE_XOR_OPERATOR)) || ((LA635_64 >= CLOSE_CURLY_SYMBOL) && (LA635_64 <= CLOSE_PAR_SYMBOL)) || ((LA635_64 >= COMMA_SYMBOL) && (LA635_64 <= CONCAT_PIPES_SYMBOL)) || ((LA635_64 >= DIV_OPERATOR) && (LA635_64 <= DIV_SYMBOL)) || LA635_64 == DOT_SYMBOL || LA635_64 == DOUBLE_QUOTED_TEXT || LA635_64 == EQUAL_OPERATOR || ((LA635_64 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_64 <= GREATER_THAN_OPERATOR)) || ((LA635_64 >= HEX_NUMBER) && (LA635_64 <= IDENTIFIER)) || LA635_64 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_64 >= LESS_OR_EQUAL_OPERATOR) && (LA635_64 <= LESS_THAN_OPERATOR)) || LA635_64 == LOGICAL_AND_OPERATOR || LA635_64 == LOGICAL_OR_OPERATOR || ((LA635_64 >= MINUS_OPERATOR) && (LA635_64 <= MOD_OPERATOR)) || ((LA635_64 >= MULT_OPERATOR) && (LA635_64 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_64 == PLUS_OPERATOR || ((LA635_64 >= SHIFT_LEFT_OPERATOR) && (LA635_64 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 64;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MULTIPOINT_SYMBOL))
                    {
                        {
                            int LA635_65 = LA(2);
                            if ( (LA635_65 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_65 == EOF || ((LA635_65 >= ACCOUNT_SYMBOL) && (LA635_65 <= YEAR_SYMBOL)) || ((LA635_65 >= AND_SYMBOL) && (LA635_65 <= ASC_SYMBOL)) || LA635_65 == BETWEEN_SYMBOL || LA635_65 == COLLATE_SYMBOL || LA635_65 == CROSS_SYMBOL || ((LA635_65 >= DAY_HOUR_SYMBOL) && (LA635_65 <= DAY_SECOND_SYMBOL)) || LA635_65 == DESC_SYMBOL || LA635_65 == ELSE_SYMBOL || ((LA635_65 >= FOR_SYMBOL) && (LA635_65 <= FROM_SYMBOL)) || ((LA635_65 >= GROUP_SYMBOL) && (LA635_65 <= HAVING_SYMBOL)) || ((LA635_65 >= HOUR_MICROSECOND_SYMBOL) && (LA635_65 <= HOUR_SECOND_SYMBOL)) || LA635_65 == INNER_SYMBOL || LA635_65 == INTO_SYMBOL || LA635_65 == IN_SYMBOL || LA635_65 == IS_SYMBOL || ((LA635_65 >= JOIN_SYMBOL) && (LA635_65 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_65 >= LEFT_SYMBOL) && (LA635_65 <= LIMIT_SYMBOL)) || LA635_65 == LOCK_SYMBOL || ((LA635_65 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_65 <= MINUTE_SECOND_SYMBOL)) || ((LA635_65 >= MOD_SYMBOL) && (LA635_65 <= NATURAL_SYMBOL)) || LA635_65 == NOT_SYMBOL || LA635_65 == ON_SYMBOL || ((LA635_65 >= ORDER_SYMBOL) && (LA635_65 <= OR_SYMBOL)) || LA635_65 == PROCEDURE_SYMBOL || LA635_65 == REGEXP_SYMBOL || LA635_65 == RENAME_SYMBOL || LA635_65 == RIGHT_SYMBOL || LA635_65 == SECOND_MICROSECOND_SYMBOL || LA635_65 == SEMICOLON_SYMBOL || LA635_65 == SEPARATOR_SYMBOL || LA635_65 == SET_SYMBOL || LA635_65 == SINGLE_QUOTED_TEXT || LA635_65 == STRAIGHT_JOIN_SYMBOL || LA635_65 == THEN_SYMBOL || LA635_65 == UNION_SYMBOL || LA635_65 == USING_SYMBOL || ((LA635_65 >= WHEN_SYMBOL) && (LA635_65 <= WHERE_SYMBOL)) || LA635_65 == WITH_SYMBOL || ((LA635_65 >= XOR_SYMBOL) && (LA635_65 <= YEAR_MONTH_SYMBOL)) || ((LA635_65 >= AS_SYMBOL) && (LA635_65 <= AT_TEXT_SUFFIX)) || ((LA635_65 >= BACK_TICK_QUOTED_ID) && (LA635_65 <= BITWISE_AND_OPERATOR)) || ((LA635_65 >= BITWISE_OR_OPERATOR) && (LA635_65 <= BITWISE_XOR_OPERATOR)) || ((LA635_65 >= CLOSE_CURLY_SYMBOL) && (LA635_65 <= CLOSE_PAR_SYMBOL)) || ((LA635_65 >= COMMA_SYMBOL) && (LA635_65 <= CONCAT_PIPES_SYMBOL)) || ((LA635_65 >= DIV_OPERATOR) && (LA635_65 <= DIV_SYMBOL)) || LA635_65 == DOT_SYMBOL || LA635_65 == DOUBLE_QUOTED_TEXT || LA635_65 == EQUAL_OPERATOR || ((LA635_65 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_65 <= GREATER_THAN_OPERATOR)) || ((LA635_65 >= HEX_NUMBER) && (LA635_65 <= IDENTIFIER)) || LA635_65 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_65 >= LESS_OR_EQUAL_OPERATOR) && (LA635_65 <= LESS_THAN_OPERATOR)) || LA635_65 == LOGICAL_AND_OPERATOR || LA635_65 == LOGICAL_OR_OPERATOR || ((LA635_65 >= MINUS_OPERATOR) && (LA635_65 <= MOD_OPERATOR)) || ((LA635_65 >= MULT_OPERATOR) && (LA635_65 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_65 == PLUS_OPERATOR || ((LA635_65 >= SHIFT_LEFT_OPERATOR) && (LA635_65 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 65;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == MULTIPOLYGON_SYMBOL))
                    {
                        {
                            int LA635_66 = LA(2);
                            if ( (LA635_66 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_66 == EOF || ((LA635_66 >= ACCOUNT_SYMBOL) && (LA635_66 <= YEAR_SYMBOL)) || ((LA635_66 >= AND_SYMBOL) && (LA635_66 <= ASC_SYMBOL)) || LA635_66 == BETWEEN_SYMBOL || LA635_66 == COLLATE_SYMBOL || LA635_66 == CROSS_SYMBOL || ((LA635_66 >= DAY_HOUR_SYMBOL) && (LA635_66 <= DAY_SECOND_SYMBOL)) || LA635_66 == DESC_SYMBOL || LA635_66 == ELSE_SYMBOL || ((LA635_66 >= FOR_SYMBOL) && (LA635_66 <= FROM_SYMBOL)) || ((LA635_66 >= GROUP_SYMBOL) && (LA635_66 <= HAVING_SYMBOL)) || ((LA635_66 >= HOUR_MICROSECOND_SYMBOL) && (LA635_66 <= HOUR_SECOND_SYMBOL)) || LA635_66 == INNER_SYMBOL || LA635_66 == INTO_SYMBOL || LA635_66 == IN_SYMBOL || LA635_66 == IS_SYMBOL || ((LA635_66 >= JOIN_SYMBOL) && (LA635_66 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_66 >= LEFT_SYMBOL) && (LA635_66 <= LIMIT_SYMBOL)) || LA635_66 == LOCK_SYMBOL || ((LA635_66 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_66 <= MINUTE_SECOND_SYMBOL)) || ((LA635_66 >= MOD_SYMBOL) && (LA635_66 <= NATURAL_SYMBOL)) || LA635_66 == NOT_SYMBOL || LA635_66 == ON_SYMBOL || ((LA635_66 >= ORDER_SYMBOL) && (LA635_66 <= OR_SYMBOL)) || LA635_66 == PROCEDURE_SYMBOL || LA635_66 == REGEXP_SYMBOL || LA635_66 == RENAME_SYMBOL || LA635_66 == RIGHT_SYMBOL || LA635_66 == SECOND_MICROSECOND_SYMBOL || LA635_66 == SEMICOLON_SYMBOL || LA635_66 == SEPARATOR_SYMBOL || LA635_66 == SET_SYMBOL || LA635_66 == SINGLE_QUOTED_TEXT || LA635_66 == STRAIGHT_JOIN_SYMBOL || LA635_66 == THEN_SYMBOL || LA635_66 == UNION_SYMBOL || LA635_66 == USING_SYMBOL || ((LA635_66 >= WHEN_SYMBOL) && (LA635_66 <= WHERE_SYMBOL)) || LA635_66 == WITH_SYMBOL || ((LA635_66 >= XOR_SYMBOL) && (LA635_66 <= YEAR_MONTH_SYMBOL)) || ((LA635_66 >= AS_SYMBOL) && (LA635_66 <= AT_TEXT_SUFFIX)) || ((LA635_66 >= BACK_TICK_QUOTED_ID) && (LA635_66 <= BITWISE_AND_OPERATOR)) || ((LA635_66 >= BITWISE_OR_OPERATOR) && (LA635_66 <= BITWISE_XOR_OPERATOR)) || ((LA635_66 >= CLOSE_CURLY_SYMBOL) && (LA635_66 <= CLOSE_PAR_SYMBOL)) || ((LA635_66 >= COMMA_SYMBOL) && (LA635_66 <= CONCAT_PIPES_SYMBOL)) || ((LA635_66 >= DIV_OPERATOR) && (LA635_66 <= DIV_SYMBOL)) || LA635_66 == DOT_SYMBOL || LA635_66 == DOUBLE_QUOTED_TEXT || LA635_66 == EQUAL_OPERATOR || ((LA635_66 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_66 <= GREATER_THAN_OPERATOR)) || ((LA635_66 >= HEX_NUMBER) && (LA635_66 <= IDENTIFIER)) || LA635_66 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_66 >= LESS_OR_EQUAL_OPERATOR) && (LA635_66 <= LESS_THAN_OPERATOR)) || LA635_66 == LOGICAL_AND_OPERATOR || LA635_66 == LOGICAL_OR_OPERATOR || ((LA635_66 >= MINUS_OPERATOR) && (LA635_66 <= MOD_OPERATOR)) || ((LA635_66 >= MULT_OPERATOR) && (LA635_66 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_66 == PLUS_OPERATOR || ((LA635_66 >= SHIFT_LEFT_OPERATOR) && (LA635_66 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 66;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == POINT_SYMBOL))
                    {
                        {
                            int LA635_67 = LA(2);
                            if ( (LA635_67 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_67 == EOF || ((LA635_67 >= ACCOUNT_SYMBOL) && (LA635_67 <= YEAR_SYMBOL)) || ((LA635_67 >= AND_SYMBOL) && (LA635_67 <= ASC_SYMBOL)) || LA635_67 == BETWEEN_SYMBOL || LA635_67 == COLLATE_SYMBOL || LA635_67 == CROSS_SYMBOL || ((LA635_67 >= DAY_HOUR_SYMBOL) && (LA635_67 <= DAY_SECOND_SYMBOL)) || LA635_67 == DESC_SYMBOL || LA635_67 == ELSE_SYMBOL || ((LA635_67 >= FOR_SYMBOL) && (LA635_67 <= FROM_SYMBOL)) || ((LA635_67 >= GROUP_SYMBOL) && (LA635_67 <= HAVING_SYMBOL)) || ((LA635_67 >= HOUR_MICROSECOND_SYMBOL) && (LA635_67 <= HOUR_SECOND_SYMBOL)) || LA635_67 == INNER_SYMBOL || LA635_67 == INTO_SYMBOL || LA635_67 == IN_SYMBOL || LA635_67 == IS_SYMBOL || ((LA635_67 >= JOIN_SYMBOL) && (LA635_67 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_67 >= LEFT_SYMBOL) && (LA635_67 <= LIMIT_SYMBOL)) || LA635_67 == LOCK_SYMBOL || ((LA635_67 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_67 <= MINUTE_SECOND_SYMBOL)) || ((LA635_67 >= MOD_SYMBOL) && (LA635_67 <= NATURAL_SYMBOL)) || LA635_67 == NOT_SYMBOL || LA635_67 == ON_SYMBOL || ((LA635_67 >= ORDER_SYMBOL) && (LA635_67 <= OR_SYMBOL)) || LA635_67 == PROCEDURE_SYMBOL || LA635_67 == REGEXP_SYMBOL || LA635_67 == RENAME_SYMBOL || LA635_67 == RIGHT_SYMBOL || LA635_67 == SECOND_MICROSECOND_SYMBOL || LA635_67 == SEMICOLON_SYMBOL || LA635_67 == SEPARATOR_SYMBOL || LA635_67 == SET_SYMBOL || LA635_67 == SINGLE_QUOTED_TEXT || LA635_67 == STRAIGHT_JOIN_SYMBOL || LA635_67 == THEN_SYMBOL || LA635_67 == UNION_SYMBOL || LA635_67 == USING_SYMBOL || ((LA635_67 >= WHEN_SYMBOL) && (LA635_67 <= WHERE_SYMBOL)) || LA635_67 == WITH_SYMBOL || ((LA635_67 >= XOR_SYMBOL) && (LA635_67 <= YEAR_MONTH_SYMBOL)) || ((LA635_67 >= AS_SYMBOL) && (LA635_67 <= AT_TEXT_SUFFIX)) || ((LA635_67 >= BACK_TICK_QUOTED_ID) && (LA635_67 <= BITWISE_AND_OPERATOR)) || ((LA635_67 >= BITWISE_OR_OPERATOR) && (LA635_67 <= BITWISE_XOR_OPERATOR)) || ((LA635_67 >= CLOSE_CURLY_SYMBOL) && (LA635_67 <= CLOSE_PAR_SYMBOL)) || ((LA635_67 >= COMMA_SYMBOL) && (LA635_67 <= CONCAT_PIPES_SYMBOL)) || ((LA635_67 >= DIV_OPERATOR) && (LA635_67 <= DIV_SYMBOL)) || LA635_67 == DOT_SYMBOL || LA635_67 == DOUBLE_QUOTED_TEXT || LA635_67 == EQUAL_OPERATOR || ((LA635_67 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_67 <= GREATER_THAN_OPERATOR)) || ((LA635_67 >= HEX_NUMBER) && (LA635_67 <= IDENTIFIER)) || LA635_67 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_67 >= LESS_OR_EQUAL_OPERATOR) && (LA635_67 <= LESS_THAN_OPERATOR)) || LA635_67 == LOGICAL_AND_OPERATOR || LA635_67 == LOGICAL_OR_OPERATOR || ((LA635_67 >= MINUS_OPERATOR) && (LA635_67 <= MOD_OPERATOR)) || ((LA635_67 >= MULT_OPERATOR) && (LA635_67 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_67 == PLUS_OPERATOR || ((LA635_67 >= SHIFT_LEFT_OPERATOR) && (LA635_67 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 67;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == POLYGON_SYMBOL))
                    {
                        {
                            int LA635_68 = LA(2);
                            if ( (LA635_68 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_68 == EOF || ((LA635_68 >= ACCOUNT_SYMBOL) && (LA635_68 <= YEAR_SYMBOL)) || ((LA635_68 >= AND_SYMBOL) && (LA635_68 <= ASC_SYMBOL)) || LA635_68 == BETWEEN_SYMBOL || LA635_68 == COLLATE_SYMBOL || LA635_68 == CROSS_SYMBOL || ((LA635_68 >= DAY_HOUR_SYMBOL) && (LA635_68 <= DAY_SECOND_SYMBOL)) || LA635_68 == DESC_SYMBOL || LA635_68 == ELSE_SYMBOL || ((LA635_68 >= FOR_SYMBOL) && (LA635_68 <= FROM_SYMBOL)) || ((LA635_68 >= GROUP_SYMBOL) && (LA635_68 <= HAVING_SYMBOL)) || ((LA635_68 >= HOUR_MICROSECOND_SYMBOL) && (LA635_68 <= HOUR_SECOND_SYMBOL)) || LA635_68 == INNER_SYMBOL || LA635_68 == INTO_SYMBOL || LA635_68 == IN_SYMBOL || LA635_68 == IS_SYMBOL || ((LA635_68 >= JOIN_SYMBOL) && (LA635_68 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_68 >= LEFT_SYMBOL) && (LA635_68 <= LIMIT_SYMBOL)) || LA635_68 == LOCK_SYMBOL || ((LA635_68 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_68 <= MINUTE_SECOND_SYMBOL)) || ((LA635_68 >= MOD_SYMBOL) && (LA635_68 <= NATURAL_SYMBOL)) || LA635_68 == NOT_SYMBOL || LA635_68 == ON_SYMBOL || ((LA635_68 >= ORDER_SYMBOL) && (LA635_68 <= OR_SYMBOL)) || LA635_68 == PROCEDURE_SYMBOL || LA635_68 == REGEXP_SYMBOL || LA635_68 == RENAME_SYMBOL || LA635_68 == RIGHT_SYMBOL || LA635_68 == SECOND_MICROSECOND_SYMBOL || LA635_68 == SEMICOLON_SYMBOL || LA635_68 == SEPARATOR_SYMBOL || LA635_68 == SET_SYMBOL || LA635_68 == SINGLE_QUOTED_TEXT || LA635_68 == STRAIGHT_JOIN_SYMBOL || LA635_68 == THEN_SYMBOL || LA635_68 == UNION_SYMBOL || LA635_68 == USING_SYMBOL || ((LA635_68 >= WHEN_SYMBOL) && (LA635_68 <= WHERE_SYMBOL)) || LA635_68 == WITH_SYMBOL || ((LA635_68 >= XOR_SYMBOL) && (LA635_68 <= YEAR_MONTH_SYMBOL)) || ((LA635_68 >= AS_SYMBOL) && (LA635_68 <= AT_TEXT_SUFFIX)) || ((LA635_68 >= BACK_TICK_QUOTED_ID) && (LA635_68 <= BITWISE_AND_OPERATOR)) || ((LA635_68 >= BITWISE_OR_OPERATOR) && (LA635_68 <= BITWISE_XOR_OPERATOR)) || ((LA635_68 >= CLOSE_CURLY_SYMBOL) && (LA635_68 <= CLOSE_PAR_SYMBOL)) || ((LA635_68 >= COMMA_SYMBOL) && (LA635_68 <= CONCAT_PIPES_SYMBOL)) || ((LA635_68 >= DIV_OPERATOR) && (LA635_68 <= DIV_SYMBOL)) || LA635_68 == DOT_SYMBOL || LA635_68 == DOUBLE_QUOTED_TEXT || LA635_68 == EQUAL_OPERATOR || ((LA635_68 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_68 <= GREATER_THAN_OPERATOR)) || ((LA635_68 >= HEX_NUMBER) && (LA635_68 <= IDENTIFIER)) || LA635_68 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_68 >= LESS_OR_EQUAL_OPERATOR) && (LA635_68 <= LESS_THAN_OPERATOR)) || LA635_68 == LOGICAL_AND_OPERATOR || LA635_68 == LOGICAL_OR_OPERATOR || ((LA635_68 >= MINUS_OPERATOR) && (LA635_68 <= MOD_OPERATOR)) || ((LA635_68 >= MULT_OPERATOR) && (LA635_68 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_68 == PLUS_OPERATOR || ((LA635_68 >= SHIFT_LEFT_OPERATOR) && (LA635_68 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 68;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == CONTAINS_SYMBOL))
                    {
                        {
                            int LA635_69 = LA(2);
                            if ( (LA635_69 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_69 == EOF || ((LA635_69 >= ACCOUNT_SYMBOL) && (LA635_69 <= YEAR_SYMBOL)) || ((LA635_69 >= AND_SYMBOL) && (LA635_69 <= ASC_SYMBOL)) || LA635_69 == BETWEEN_SYMBOL || LA635_69 == COLLATE_SYMBOL || LA635_69 == CROSS_SYMBOL || ((LA635_69 >= DAY_HOUR_SYMBOL) && (LA635_69 <= DAY_SECOND_SYMBOL)) || LA635_69 == DESC_SYMBOL || LA635_69 == ELSE_SYMBOL || ((LA635_69 >= FOR_SYMBOL) && (LA635_69 <= FROM_SYMBOL)) || ((LA635_69 >= GROUP_SYMBOL) && (LA635_69 <= HAVING_SYMBOL)) || ((LA635_69 >= HOUR_MICROSECOND_SYMBOL) && (LA635_69 <= HOUR_SECOND_SYMBOL)) || LA635_69 == INNER_SYMBOL || LA635_69 == INTO_SYMBOL || LA635_69 == IN_SYMBOL || LA635_69 == IS_SYMBOL || ((LA635_69 >= JOIN_SYMBOL) && (LA635_69 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_69 >= LEFT_SYMBOL) && (LA635_69 <= LIMIT_SYMBOL)) || LA635_69 == LOCK_SYMBOL || ((LA635_69 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_69 <= MINUTE_SECOND_SYMBOL)) || ((LA635_69 >= MOD_SYMBOL) && (LA635_69 <= NATURAL_SYMBOL)) || LA635_69 == NOT_SYMBOL || LA635_69 == ON_SYMBOL || ((LA635_69 >= ORDER_SYMBOL) && (LA635_69 <= OR_SYMBOL)) || LA635_69 == PROCEDURE_SYMBOL || LA635_69 == REGEXP_SYMBOL || LA635_69 == RENAME_SYMBOL || LA635_69 == RIGHT_SYMBOL || LA635_69 == SECOND_MICROSECOND_SYMBOL || LA635_69 == SEMICOLON_SYMBOL || LA635_69 == SEPARATOR_SYMBOL || LA635_69 == SET_SYMBOL || LA635_69 == SINGLE_QUOTED_TEXT || LA635_69 == STRAIGHT_JOIN_SYMBOL || LA635_69 == THEN_SYMBOL || LA635_69 == UNION_SYMBOL || LA635_69 == USING_SYMBOL || ((LA635_69 >= WHEN_SYMBOL) && (LA635_69 <= WHERE_SYMBOL)) || LA635_69 == WITH_SYMBOL || ((LA635_69 >= XOR_SYMBOL) && (LA635_69 <= YEAR_MONTH_SYMBOL)) || ((LA635_69 >= AS_SYMBOL) && (LA635_69 <= AT_TEXT_SUFFIX)) || ((LA635_69 >= BACK_TICK_QUOTED_ID) && (LA635_69 <= BITWISE_AND_OPERATOR)) || ((LA635_69 >= BITWISE_OR_OPERATOR) && (LA635_69 <= BITWISE_XOR_OPERATOR)) || ((LA635_69 >= CLOSE_CURLY_SYMBOL) && (LA635_69 <= CLOSE_PAR_SYMBOL)) || ((LA635_69 >= COMMA_SYMBOL) && (LA635_69 <= CONCAT_PIPES_SYMBOL)) || ((LA635_69 >= DIV_OPERATOR) && (LA635_69 <= DIV_SYMBOL)) || LA635_69 == DOT_SYMBOL || LA635_69 == DOUBLE_QUOTED_TEXT || LA635_69 == EQUAL_OPERATOR || ((LA635_69 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_69 <= GREATER_THAN_OPERATOR)) || ((LA635_69 >= HEX_NUMBER) && (LA635_69 <= IDENTIFIER)) || LA635_69 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_69 >= LESS_OR_EQUAL_OPERATOR) && (LA635_69 <= LESS_THAN_OPERATOR)) || LA635_69 == LOGICAL_AND_OPERATOR || LA635_69 == LOGICAL_OR_OPERATOR || ((LA635_69 >= MINUS_OPERATOR) && (LA635_69 <= MOD_OPERATOR)) || ((LA635_69 >= MULT_OPERATOR) && (LA635_69 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_69 == PLUS_OPERATOR || ((LA635_69 >= SHIFT_LEFT_OPERATOR) && (LA635_69 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 69;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == AVG_SYMBOL))
                    {
                        {
                            int LA635_70 = LA(2);
                            if ( (LA635_70 == OPEN_PAR_SYMBOL))
                            {
                                alt635=3;
                            }
                            else if ( (LA635_70 == EOF || ((LA635_70 >= ACCOUNT_SYMBOL) && (LA635_70 <= YEAR_SYMBOL)) || ((LA635_70 >= AND_SYMBOL) && (LA635_70 <= ASC_SYMBOL)) || LA635_70 == BETWEEN_SYMBOL || LA635_70 == COLLATE_SYMBOL || LA635_70 == CROSS_SYMBOL || ((LA635_70 >= DAY_HOUR_SYMBOL) && (LA635_70 <= DAY_SECOND_SYMBOL)) || LA635_70 == DESC_SYMBOL || LA635_70 == ELSE_SYMBOL || ((LA635_70 >= FOR_SYMBOL) && (LA635_70 <= FROM_SYMBOL)) || ((LA635_70 >= GROUP_SYMBOL) && (LA635_70 <= HAVING_SYMBOL)) || ((LA635_70 >= HOUR_MICROSECOND_SYMBOL) && (LA635_70 <= HOUR_SECOND_SYMBOL)) || LA635_70 == INNER_SYMBOL || LA635_70 == INTO_SYMBOL || LA635_70 == IN_SYMBOL || LA635_70 == IS_SYMBOL || ((LA635_70 >= JOIN_SYMBOL) && (LA635_70 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_70 >= LEFT_SYMBOL) && (LA635_70 <= LIMIT_SYMBOL)) || LA635_70 == LOCK_SYMBOL || ((LA635_70 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_70 <= MINUTE_SECOND_SYMBOL)) || ((LA635_70 >= MOD_SYMBOL) && (LA635_70 <= NATURAL_SYMBOL)) || LA635_70 == NOT_SYMBOL || LA635_70 == ON_SYMBOL || ((LA635_70 >= ORDER_SYMBOL) && (LA635_70 <= OR_SYMBOL)) || LA635_70 == PROCEDURE_SYMBOL || LA635_70 == REGEXP_SYMBOL || LA635_70 == RENAME_SYMBOL || LA635_70 == RIGHT_SYMBOL || LA635_70 == SECOND_MICROSECOND_SYMBOL || LA635_70 == SEMICOLON_SYMBOL || LA635_70 == SEPARATOR_SYMBOL || LA635_70 == SET_SYMBOL || LA635_70 == SINGLE_QUOTED_TEXT || LA635_70 == STRAIGHT_JOIN_SYMBOL || LA635_70 == THEN_SYMBOL || LA635_70 == UNION_SYMBOL || LA635_70 == USING_SYMBOL || ((LA635_70 >= WHEN_SYMBOL) && (LA635_70 <= WHERE_SYMBOL)) || LA635_70 == WITH_SYMBOL || ((LA635_70 >= XOR_SYMBOL) && (LA635_70 <= YEAR_MONTH_SYMBOL)) || ((LA635_70 >= AS_SYMBOL) && (LA635_70 <= AT_TEXT_SUFFIX)) || ((LA635_70 >= BACK_TICK_QUOTED_ID) && (LA635_70 <= BITWISE_AND_OPERATOR)) || ((LA635_70 >= BITWISE_OR_OPERATOR) && (LA635_70 <= BITWISE_XOR_OPERATOR)) || ((LA635_70 >= CLOSE_CURLY_SYMBOL) && (LA635_70 <= CLOSE_PAR_SYMBOL)) || ((LA635_70 >= COMMA_SYMBOL) && (LA635_70 <= CONCAT_PIPES_SYMBOL)) || ((LA635_70 >= DIV_OPERATOR) && (LA635_70 <= DIV_SYMBOL)) || LA635_70 == DOT_SYMBOL || LA635_70 == DOUBLE_QUOTED_TEXT || LA635_70 == EQUAL_OPERATOR || ((LA635_70 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_70 <= GREATER_THAN_OPERATOR)) || ((LA635_70 >= HEX_NUMBER) && (LA635_70 <= IDENTIFIER)) || LA635_70 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_70 >= LESS_OR_EQUAL_OPERATOR) && (LA635_70 <= LESS_THAN_OPERATOR)) || LA635_70 == LOGICAL_AND_OPERATOR || LA635_70 == LOGICAL_OR_OPERATOR || ((LA635_70 >= MINUS_OPERATOR) && (LA635_70 <= MOD_OPERATOR)) || ((LA635_70 >= MULT_OPERATOR) && (LA635_70 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_70 == PLUS_OPERATOR || ((LA635_70 >= SHIFT_LEFT_OPERATOR) && (LA635_70 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 70;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == ACCOUNT_SYMBOL || ((LA635_0 >= ALWAYS_SYMBOL) && (LA635_0 <= CACHE_SYMBOL)) || ((LA635_0 >= CHECKSUM_SYMBOL) && (LA635_0 <= COMMIT_SYMBOL)) || ((LA635_0 >= DEALLOCATE_SYMBOL) && (LA635_0 <= FOLLOWS_SYMBOL)) || ((LA635_0 >= GROUP_REPLICATION_SYMBOL) && (LA635_0 <= STOP_SYMBOL)) || ((LA635_0 >= UNICODE_SYMBOL) && (LA635_0 <= ACTION_SYMBOL)) || ((LA635_0 >= AFTER_SYMBOL) && (LA635_0 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA635_0 >= BINLOG_SYMBOL) && (LA635_0 <= CLASS_ORIGIN_SYMBOL)) || LA635_0 == CODE_SYMBOL || ((LA635_0 >= COLUMN_NAME_SYMBOL) && (LA635_0 <= DATETIME_SYMBOL)) || ((LA635_0 >= DEFAULT_AUTH_SYMBOL) && (LA635_0 <= GEOMETRY_SYMBOL)) || ((LA635_0 >= GRANTS_SYMBOL) && (LA635_0 <= HOSTS_SYMBOL)) || ((LA635_0 >= IDENTIFIED_SYMBOL) && (LA635_0 <= LEVEL_SYMBOL)) || ((LA635_0 >= LIST_SYMBOL) && (LA635_0 <= MESSAGE_TEXT_SYMBOL)) || LA635_0 == MIGRATE_SYMBOL || ((LA635_0 >= MIN_ROWS_SYMBOL) && (LA635_0 <= MODE_SYMBOL)) || ((LA635_0 >= MUTEX_SYMBOL) && (LA635_0 <= OFFSET_SYMBOL)) || ((LA635_0 >= ONE_SHOT_SYMBOL) && (LA635_0 <= PARTITIONS_SYMBOL)) || ((LA635_0 >= PHASE_SYMBOL) && (LA635_0 <= PLUGINS_SYMBOL)) || ((LA635_0 >= PRESERVE_SYMBOL) && (LA635_0 <= PROXY_SYMBOL)) || ((LA635_0 >= QUERY_SYMBOL) && (LA635_0 <= RETURNS_SYMBOL)) || ((LA635_0 >= ROLLUP_SYMBOL) && (LA635_0 <= ROWS_SYMBOL)) || LA635_0 == ROW_FORMAT_SYMBOL || ((LA635_0 >= RTREE_SYMBOL) && (LA635_0 <= SCHEMA_NAME_SYMBOL)) || ((LA635_0 >= SERIAL_SYMBOL) && (LA635_0 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA635_0 >= SUBJECT_SYMBOL) && (LA635_0 <= TRIGGERS_SYMBOL)) || ((LA635_0 >= TYPES_SYMBOL) && (LA635_0 <= USER_RESOURCES_SYMBOL)) || ((LA635_0 >= USE_FRM_SYMBOL) && (LA635_0 <= WAIT_SYMBOL)) || LA635_0 == WORK_SYMBOL || ((LA635_0 >= X509_SYMBOL) && (LA635_0 <= XML_SYMBOL)) || LA635_0 == DOT_SYMBOL))
                    {
                        alt635=4;
                    }
                    else if ( (LA635_0 == ROW_SYMBOL))
                    {
                        {
                            int LA635_84 = LA(2);
                            if ( (LA635_84 == OPEN_PAR_SYMBOL))
                            {
                                alt635=9;
                            }
                            else if ( (LA635_84 == EOF || ((LA635_84 >= ACCOUNT_SYMBOL) && (LA635_84 <= YEAR_SYMBOL)) || ((LA635_84 >= AND_SYMBOL) && (LA635_84 <= ASC_SYMBOL)) || LA635_84 == BETWEEN_SYMBOL || LA635_84 == COLLATE_SYMBOL || LA635_84 == CROSS_SYMBOL || ((LA635_84 >= DAY_HOUR_SYMBOL) && (LA635_84 <= DAY_SECOND_SYMBOL)) || LA635_84 == DESC_SYMBOL || LA635_84 == ELSE_SYMBOL || ((LA635_84 >= FOR_SYMBOL) && (LA635_84 <= FROM_SYMBOL)) || ((LA635_84 >= GROUP_SYMBOL) && (LA635_84 <= HAVING_SYMBOL)) || ((LA635_84 >= HOUR_MICROSECOND_SYMBOL) && (LA635_84 <= HOUR_SECOND_SYMBOL)) || LA635_84 == INNER_SYMBOL || LA635_84 == INTO_SYMBOL || LA635_84 == IN_SYMBOL || LA635_84 == IS_SYMBOL || ((LA635_84 >= JOIN_SYMBOL) && (LA635_84 <= JSON_SEPARATOR_SYMBOL)) || ((LA635_84 >= LEFT_SYMBOL) && (LA635_84 <= LIMIT_SYMBOL)) || LA635_84 == LOCK_SYMBOL || ((LA635_84 >= MINUTE_MICROSECOND_SYMBOL) && (LA635_84 <= MINUTE_SECOND_SYMBOL)) || ((LA635_84 >= MOD_SYMBOL) && (LA635_84 <= NATURAL_SYMBOL)) || LA635_84 == NOT_SYMBOL || LA635_84 == ON_SYMBOL || ((LA635_84 >= ORDER_SYMBOL) && (LA635_84 <= OR_SYMBOL)) || LA635_84 == PROCEDURE_SYMBOL || LA635_84 == REGEXP_SYMBOL || LA635_84 == RENAME_SYMBOL || LA635_84 == RIGHT_SYMBOL || LA635_84 == SECOND_MICROSECOND_SYMBOL || LA635_84 == SEMICOLON_SYMBOL || LA635_84 == SEPARATOR_SYMBOL || LA635_84 == SET_SYMBOL || LA635_84 == SINGLE_QUOTED_TEXT || LA635_84 == STRAIGHT_JOIN_SYMBOL || LA635_84 == THEN_SYMBOL || LA635_84 == UNION_SYMBOL || LA635_84 == USING_SYMBOL || ((LA635_84 >= WHEN_SYMBOL) && (LA635_84 <= WHERE_SYMBOL)) || LA635_84 == WITH_SYMBOL || ((LA635_84 >= XOR_SYMBOL) && (LA635_84 <= YEAR_MONTH_SYMBOL)) || ((LA635_84 >= AS_SYMBOL) && (LA635_84 <= AT_TEXT_SUFFIX)) || ((LA635_84 >= BACK_TICK_QUOTED_ID) && (LA635_84 <= BITWISE_AND_OPERATOR)) || ((LA635_84 >= BITWISE_OR_OPERATOR) && (LA635_84 <= BITWISE_XOR_OPERATOR)) || ((LA635_84 >= CLOSE_CURLY_SYMBOL) && (LA635_84 <= CLOSE_PAR_SYMBOL)) || ((LA635_84 >= COMMA_SYMBOL) && (LA635_84 <= CONCAT_PIPES_SYMBOL)) || ((LA635_84 >= DIV_OPERATOR) && (LA635_84 <= DIV_SYMBOL)) || LA635_84 == DOT_SYMBOL || LA635_84 == DOUBLE_QUOTED_TEXT || LA635_84 == EQUAL_OPERATOR || ((LA635_84 >= GREATER_OR_EQUAL_OPERATOR) && (LA635_84 <= GREATER_THAN_OPERATOR)) || ((LA635_84 >= HEX_NUMBER) && (LA635_84 <= IDENTIFIER)) || LA635_84 == JSON_UNQUOTED_SEPARATOR_SYMBOL || ((LA635_84 >= LESS_OR_EQUAL_OPERATOR) && (LA635_84 <= LESS_THAN_OPERATOR)) || LA635_84 == LOGICAL_AND_OPERATOR || LA635_84 == LOGICAL_OR_OPERATOR || ((LA635_84 >= MINUS_OPERATOR) && (LA635_84 <= MOD_OPERATOR)) || ((LA635_84 >= MULT_OPERATOR) && (LA635_84 <= NULL_SAFE_EQUAL_OPERATOR)) || LA635_84 == PLUS_OPERATOR || ((LA635_84 >= SHIFT_LEFT_OPERATOR) && (LA635_84 <= SHIFT_RIGHT_OPERATOR))))
                            {
                                alt635=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 635;
                                EXCEPTION->state        = 84;


                                goto ruleprimaryEx;

                            }
                        }
                    }
                    else if ( (LA635_0 == PARAM_MARKER))
                    {
                        alt635=5;
                    }
                    else if ( (((LA635_0 >= AT_AT_SIGN_SYMBOL) && (LA635_0 <= AT_TEXT_SUFFIX))))
                    {
                        alt635=6;
                    }
                    else if ( (LA635_0 == EXISTS_SYMBOL))
                    {
                        alt635=7;
                    }
                    else if ( (LA635_0 == OPEN_PAR_SYMBOL))
                    {
                        alt635=8;
                    }
                    else if ( (LA635_0 == OPEN_CURLY_SYMBOL))
                    {
                        alt635=10;
                    }
                    else if ( (LA635_0 == MATCH_SYMBOL))
                    {
                        alt635=11;
                    }
                    else if ( (LA635_0 == CASE_SYMBOL))
                    {
                        alt635=12;
                    }
                    else if ( (LA635_0 == BINARY_SYMBOL || LA635_0 == CAST_SYMBOL || LA635_0 == CONVERT_SYMBOL))
                    {
                        alt635=13;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 635;
                        EXCEPTION->state        = 0;


                        goto ruleprimaryEx;

                    }
                }
                switch (alt635)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1851:3: literal
            	    {
            	        FOLLOWPUSH(FOLLOW_literal_in_primary10544);
            	        literal(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1852:5: function_call
            	    {
            	        FOLLOWPUSH(FOLLOW_function_call_in_primary10550);
            	        function_call(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:1853:5: runtime_function_call
            	    {
            	        FOLLOWPUSH(FOLLOW_runtime_function_call_in_primary10556);
            	        runtime_function_call(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:1854:5: column_ref ( jsonOperator )?
            	    {
            	        FOLLOWPUSH(FOLLOW_column_ref_in_primary10563);
            	        column_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1854:16: ( jsonOperator )?
            	        {
            	            int alt633=2;
            	            switch ( LA(1) )
            	            {
            	                case JSON_SEPARATOR_SYMBOL:
            	                case JSON_UNQUOTED_SEPARATOR_SYMBOL:
            	                	{
            	                		alt633=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt633)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1854:16: jsonOperator
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_jsonOperator_in_primary10565);
            	        	        jsonOperator(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleprimaryEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 5:
            	    // MySQLSimpleParser.g:1855:5: PARAM_MARKER
            	    {
            	         MATCHT(PARAM_MARKER, &FOLLOW_PARAM_MARKER_in_primary10572);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 6:
            	    // MySQLSimpleParser.g:1856:5: variable
            	    {
            	        FOLLOWPUSH(FOLLOW_variable_in_primary10578);
            	        variable(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 7:
            	    // MySQLSimpleParser.g:1857:5: EXISTS_SYMBOL subquery
            	    {
            	         MATCHT(EXISTS_SYMBOL, &FOLLOW_EXISTS_SYMBOL_in_primary10584);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_subquery_in_primary10586);
            	        subquery(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 8:
            	    // MySQLSimpleParser.g:1858:5: expression_with_nested_parentheses
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_with_nested_parentheses_in_primary10592);
            	        expression_with_nested_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 9:
            	    // MySQLSimpleParser.g:1859:5: ROW_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )+ CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(ROW_SYMBOL, &FOLLOW_ROW_SYMBOL_in_primary10598);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_primary10600);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_primary10602);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1859:43: ( COMMA_SYMBOL expression )+
            	        {
            	            int cnt634=0;

            	            for (;;)
            	            {
            	                int alt634=2;
            	        	switch ( LA(1) )
            	        	{
            	        	case COMMA_SYMBOL:
            	        		{
            	        			alt634=1;
            	        		}
            	        	    break;

            	        	}

            	        	switch (alt634)
            	        	{
            	        	    case 1:
            	        	        // MySQLSimpleParser.g:1859:44: COMMA_SYMBOL expression
            	        	        {
            	        	             MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_primary10605);
            	        	            if  (HASEXCEPTION())
            	        	            {
            	        	                goto ruleprimaryEx;
            	        	            }
            	        	            if (HASFAILED())
            	        	            {
            	        	                return ;
            	        	            }


            	        	            FOLLOWPUSH(FOLLOW_expression_in_primary10607);
            	        	            expression(ctx);

            	        	            FOLLOWPOP();
            	        	            if  (HASEXCEPTION())
            	        	            {
            	        	                goto ruleprimaryEx;
            	        	            }
            	        	            if (HASFAILED())
            	        	            {
            	        	                return ;
            	        	            }


            	        	        }
            	        	        break;

            	        	    default:

            	        		if ( cnt634 >= 1 )
            	        		{
            	        		    goto loop634;
            	        		}
            	        		if (BACKTRACKING>0)
            	        		{
            	        		    FAILEDFLAG = ANTLR3_TRUE;
            	        		    return ;
            	        		}

            	        		/* mismatchedSetEx()
            	        		 */
            	        		CONSTRUCTEX();
            	        		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            	        		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            	        		goto ruleprimaryEx;
            	        	}
            	        	cnt634++;
            	            }
            	            loop634: ;	/* Jump to here if this rule does not match */
            	        }

            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_primary10611);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 10:
            	    // MySQLSimpleParser.g:1860:5: OPEN_CURLY_SYMBOL identifier expression CLOSE_CURLY_SYMBOL
            	    {
            	         MATCHT(OPEN_CURLY_SYMBOL, &FOLLOW_OPEN_CURLY_SYMBOL_in_primary10617);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_identifier_in_primary10619);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_primary10621);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_CURLY_SYMBOL, &FOLLOW_CLOSE_CURLY_SYMBOL_in_primary10623);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 11:
            	    // MySQLSimpleParser.g:1861:5: match_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_match_expression_in_primary10629);
            	        match_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 12:
            	    // MySQLSimpleParser.g:1862:5: case_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_case_expression_in_primary10635);
            	        case_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 13:
            	    // MySQLSimpleParser.g:1863:5: cast_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_cast_expression_in_primary10641);
            	        cast_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:1866:2: ( options {greedy=true; } : COLLATE_SYMBOL collation_name )*

            for (;;)
            {
                int alt636=2;
                switch ( LA(1) )
                {
                case COLLATE_SYMBOL:
                	{
                		alt636=1;
                	}
                    break;

                }

                switch (alt636)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1866:32: COLLATE_SYMBOL collation_name
            	    {
            	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_primary10663);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_collation_name_in_primary10665);
            	        collation_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleprimaryEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop636;	/* break out of the loop */
            	    break;
                }
            }
            loop636: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprimaryEx; /* Prevent compiler warnings */
    ruleprimaryEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end primary */

/**
 * $ANTLR start jsonOperator
 * MySQLSimpleParser.g:1869:1: jsonOperator : ({...}? JSON_SEPARATOR_SYMBOL text_string |{...}? JSON_UNQUOTED_SEPARATOR_SYMBOL text_string );
 */
static void
jsonOperator(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1869:13: ({...}? JSON_SEPARATOR_SYMBOL text_string |{...}? JSON_UNQUOTED_SEPARATOR_SYMBOL text_string )

            ANTLR3_UINT32 alt637;

            alt637=2;

            switch ( LA(1) )
            {
            case JSON_SEPARATOR_SYMBOL:
            	{
            		alt637=1;
            	}
                break;
            case JSON_UNQUOTED_SEPARATOR_SYMBOL:
            	{
            		alt637=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 637;
                EXCEPTION->state        = 0;


                goto rulejsonOperatorEx;

            }

            switch (alt637)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1870:3: {...}? JSON_SEPARATOR_SYMBOL text_string
        	    {
        	        if ( !((SERVER_VERSION >= 50708)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50708";
        	                EXCEPTION->ruleName	 = (void *)"jsonOperator";


        	        }

        	         MATCHT(JSON_SEPARATOR_SYMBOL, &FOLLOW_JSON_SEPARATOR_SYMBOL_in_jsonOperator10679);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejsonOperatorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_jsonOperator10681);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejsonOperatorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1871:5: {...}? JSON_UNQUOTED_SEPARATOR_SYMBOL text_string
        	    {
        	        if ( !((SERVER_VERSION >= 50713)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50713";
        	                EXCEPTION->ruleName	 = (void *)"jsonOperator";


        	        }

        	         MATCHT(JSON_UNQUOTED_SEPARATOR_SYMBOL, &FOLLOW_JSON_UNQUOTED_SEPARATOR_SYMBOL_in_jsonOperator10689);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejsonOperatorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_jsonOperator10691);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulejsonOperatorEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulejsonOperatorEx; /* Prevent compiler warnings */
    rulejsonOperatorEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end jsonOperator */

/**
 * $ANTLR start expression_with_nested_parentheses
 * MySQLSimpleParser.g:1876:1: expression_with_nested_parentheses : ({...}? subquery | expression_list_with_parentheses );
 */
static void
expression_with_nested_parentheses(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1876:35: ({...}? subquery | expression_list_with_parentheses )

            ANTLR3_UINT32 alt638;

            alt638=2;

            switch ( LA(1) )
            {
            case OPEN_PAR_SYMBOL:
            	{
            		{
            		    int LA638_1 = LA(2);
            		    if ( (LA638_1 == SELECT_SYMBOL))
            		    {
            		        alt638=1;
            		    }
            		    else if ( (LA638_1 == OPEN_PAR_SYMBOL))
            		    {
            		        {
            		            int LA638_3 = LA(3);
            		            if ( ((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)))
            		            {
            		                alt638=1;
            		            }
            		            else if ( (ANTLR3_TRUE))
            		            {
            		                alt638=2;
            		            }
            		            else
            		            {
            		                if (BACKTRACKING>0)
            		                {
            		                    FAILEDFLAG = ANTLR3_TRUE;
            		                    return ;
            		                }


            		                CONSTRUCTEX();
            		                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		                EXCEPTION->message      = (void *)"";
            		                EXCEPTION->decisionNum  = 638;
            		                EXCEPTION->state        = 3;


            		                goto ruleexpression_with_nested_parenthesesEx;

            		            }
            		        }
            		    }
            		    else if ( (((LA638_1 >= ACCOUNT_SYMBOL) && (LA638_1 <= YEAR_SYMBOL)) || LA638_1 == BINARY_SYMBOL || ((LA638_1 >= CASE_SYMBOL) && (LA638_1 <= CAST_SYMBOL)) || LA638_1 == CHAR_SYMBOL || ((LA638_1 >= CONVERT_SYMBOL) && (LA638_1 <= COUNT_SYMBOL)) || LA638_1 == CURDATE_SYMBOL || LA638_1 == CURRENT_USER_SYMBOL || LA638_1 == CURTIME_SYMBOL || LA638_1 == DATABASE_SYMBOL || LA638_1 == DATE_ADD_SYMBOL || LA638_1 == DATE_SUB_SYMBOL || LA638_1 == EXISTS_SYMBOL || ((LA638_1 >= EXTRACT_SYMBOL) && (LA638_1 <= FALSE_SYMBOL)) || LA638_1 == GROUP_CONCAT_SYMBOL || LA638_1 == IF_SYMBOL || LA638_1 == INSERT_SYMBOL || LA638_1 == INTERVAL_SYMBOL || LA638_1 == LEFT_SYMBOL || LA638_1 == MATCH_SYMBOL || LA638_1 == MAX_SYMBOL || LA638_1 == MIN_SYMBOL || LA638_1 == MOD_SYMBOL || LA638_1 == NCHAR_TEXT || ((LA638_1 >= NOT2_SYMBOL) && (LA638_1 <= NOW_SYMBOL)) || ((LA638_1 >= NULL2_SYMBOL) && (LA638_1 <= NULL_SYMBOL)) || LA638_1 == OPEN_CURLY_SYMBOL || ((LA638_1 >= PARAM_MARKER) && (LA638_1 <= POSITION_SYMBOL)) || ((LA638_1 >= REPEAT_SYMBOL) && (LA638_1 <= REPLACE_SYMBOL)) || LA638_1 == RIGHT_SYMBOL || LA638_1 == SINGLE_QUOTED_TEXT || LA638_1 == STDDEV_SAMP_SYMBOL || LA638_1 == STD_SYMBOL || LA638_1 == SUBSTRING_SYMBOL || ((LA638_1 >= SUM_SYMBOL) && (LA638_1 <= SYSDATE_SYMBOL)) || ((LA638_1 >= TRIM_SYMBOL) && (LA638_1 <= TRUE_SYMBOL)) || ((LA638_1 >= UTC_DATE_SYMBOL) && (LA638_1 <= UTC_TIME_SYMBOL)) || LA638_1 == VALUES_SYMBOL || LA638_1 == VARIANCE_SYMBOL || LA638_1 == VAR_SAMP_SYMBOL || ((LA638_1 >= AT_AT_SIGN_SYMBOL) && (LA638_1 <= AT_TEXT_SUFFIX)) || ((LA638_1 >= BACK_TICK_QUOTED_ID) && (LA638_1 <= BIN_NUMBER)) || LA638_1 == BITWISE_NOT_OPERATOR || ((LA638_1 >= BIT_AND_SYMBOL) && (LA638_1 <= BIT_XOR_SYMBOL)) || LA638_1 == DECIMAL_NUMBER || LA638_1 == DOT_SYMBOL || LA638_1 == DOUBLE_QUOTED_TEXT || LA638_1 == FLOAT_NUMBER || ((LA638_1 >= HEX_NUMBER) && (LA638_1 <= IDENTIFIER)) || LA638_1 == INT_NUMBER || LA638_1 == LOGICAL_NOT_OPERATOR || ((LA638_1 >= LONG_NUMBER) && (LA638_1 <= MINUS_OPERATOR)) || LA638_1 == PLUS_OPERATOR || ((LA638_1 >= ULONGLONG_NUMBER) && (LA638_1 <= UNDERSCORE_CHARSET))))
            		    {
            		        alt638=2;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 638;
            		        EXCEPTION->state        = 1;


            		        goto ruleexpression_with_nested_parenthesesEx;

            		    }
            		}
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 638;
                EXCEPTION->state        = 0;


                goto ruleexpression_with_nested_parenthesesEx;

            }

            switch (alt638)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1877:2: {...}? subquery
        	    {
        	        if ( !((LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == OPEN_PAR_SYMBOL && LA(2) == SELECT_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"expression_with_nested_parentheses";


        	        }

        	        FOLLOWPUSH(FOLLOW_subquery_in_expression_with_nested_parentheses10704);
        	        subquery(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexpression_with_nested_parenthesesEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1878:4: expression_list_with_parentheses
        	    {
        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_expression_with_nested_parentheses10709);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleexpression_with_nested_parenthesesEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleexpression_with_nested_parenthesesEx; /* Prevent compiler warnings */
    ruleexpression_with_nested_parenthesesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end expression_with_nested_parentheses */

/**
 * $ANTLR start comparison_operator
 * MySQLSimpleParser.g:1881:1: comparison_operator : ( EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | LESS_THAN_OPERATOR | NOT_EQUAL_OPERATOR | NOT_EQUAL2_OPERATOR | NULL_SAFE_EQUAL_OPERATOR );
 */
static void
comparison_operator(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1881:20: ( EQUAL_OPERATOR | GREATER_OR_EQUAL_OPERATOR | GREATER_THAN_OPERATOR | LESS_OR_EQUAL_OPERATOR | LESS_THAN_OPERATOR | NOT_EQUAL_OPERATOR | NOT_EQUAL2_OPERATOR | NULL_SAFE_EQUAL_OPERATOR )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == EQUAL_OPERATOR || ((LA(1) >= GREATER_OR_EQUAL_OPERATOR) && (LA(1) <= GREATER_THAN_OPERATOR)) || ((LA(1) >= LESS_OR_EQUAL_OPERATOR) && (LA(1) <= LESS_THAN_OPERATOR)) || ((LA(1) >= NOT_EQUAL2_OPERATOR) && (LA(1) <= NULL_SAFE_EQUAL_OPERATOR)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulecomparison_operatorEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecomparison_operatorEx; /* Prevent compiler warnings */
    rulecomparison_operatorEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end comparison_operator */

/**
 * $ANTLR start multiplication_operator
 * MySQLSimpleParser.g:1892:1: multiplication_operator : ( MULT_OPERATOR | DIV_OPERATOR | DIV_SYMBOL | MOD_OPERATOR | MOD_SYMBOL );
 */
static void
multiplication_operator(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1892:24: ( MULT_OPERATOR | DIV_OPERATOR | DIV_SYMBOL | MOD_OPERATOR | MOD_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == MOD_SYMBOL || ((LA(1) >= DIV_OPERATOR) && (LA(1) <= DIV_SYMBOL)) || LA(1) == MOD_OPERATOR || LA(1) == MULT_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulemultiplication_operatorEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulemultiplication_operatorEx; /* Prevent compiler warnings */
    rulemultiplication_operatorEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end multiplication_operator */

/**
 * $ANTLR start runtime_function_call
 * MySQLSimpleParser.g:1896:1: runtime_function_call : runtime_function_call_expression ;
 */
static void
runtime_function_call(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1896:22: ( runtime_function_call_expression )
        // MySQLSimpleParser.g:1897:2: runtime_function_call_expression
        {
            FOLLOWPUSH(FOLLOW_runtime_function_call_expression_in_runtime_function_call10787);
            runtime_function_call_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleruntime_function_callEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleruntime_function_callEx; /* Prevent compiler warnings */
    ruleruntime_function_callEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end runtime_function_call */

/**
 * $ANTLR start runtime_function_call_expression
 * MySQLSimpleParser.g:1900:1: runtime_function_call_expression : ( CHAR_SYMBOL OPEN_PAR_SYMBOL expression_list ( USING_SYMBOL charset_name )? CLOSE_PAR_SYMBOL | CURRENT_USER_SYMBOL ( parentheses )? | DATE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | DAY_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | HOUR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | INSERT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | LEFT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | MINUTE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | MONTH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | RIGHT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | SECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | TIME_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | TIMESTAMP_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL | trim_function | USER_SYMBOL parentheses | VALUES_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | YEAR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( ADDDATE_SYMBOL | SUBDATE_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | CURDATE_SYMBOL ( parentheses )? | CURTIME_SYMBOL ( time_function_parameters )? | ( DATE_ADD_SYMBOL | DATE_SUB_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL INTERVAL_SYMBOL interval CLOSE_PAR_SYMBOL | EXTRACT_SYMBOL OPEN_PAR_SYMBOL interval_unit FROM_SYMBOL expression CLOSE_PAR_SYMBOL | GET_FORMAT_SYMBOL OPEN_PAR_SYMBOL date_time_type COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | NOW_SYMBOL ( time_function_parameters )? | POSITION_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression IN_SYMBOL expression CLOSE_PAR_SYMBOL | substring_function | SYSDATE_SYMBOL ( time_function_parameters )? | ( TIMESTAMP_ADD_SYMBOL | TIMESTAMP_DIFF_SYMBOL ) OPEN_PAR_SYMBOL interval_timestamp_unit COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | UTC_DATE_SYMBOL ( parentheses )? | UTC_TIME_SYMBOL ( time_function_parameters )? | UTC_TIMESTAMP_SYMBOL ( time_function_parameters )? | ASCII_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | CHARSET_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | COALESCE_SYMBOL expression_list_with_parentheses | COLLATION_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | DATABASE_SYMBOL parentheses | IF_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | FORMAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL | MICROSECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | MOD_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL string_literal CLOSE_PAR_SYMBOL | PASSWORD_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | QUARTER_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | REPEAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | REPLACE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | REVERSE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ROW_COUNT_SYMBOL parentheses | TRUNCATE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | WEEK_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL |{...}? WEIGHT_STRING_SYMBOL OPEN_PAR_SYMBOL expression ( ( AS_SYMBOL CHAR_SYMBOL field_length )? ( weight_string_levels )? | AS_SYMBOL BINARY_SYMBOL field_length | COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number ) CLOSE_PAR_SYMBOL | geometry_function | aggregate_function );
 */
static void
runtime_function_call_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:1900:33: ( CHAR_SYMBOL OPEN_PAR_SYMBOL expression_list ( USING_SYMBOL charset_name )? CLOSE_PAR_SYMBOL | CURRENT_USER_SYMBOL ( parentheses )? | DATE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | DAY_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | HOUR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | INSERT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | LEFT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | MINUTE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | MONTH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | RIGHT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | SECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | TIME_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | TIMESTAMP_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL | trim_function | USER_SYMBOL parentheses | VALUES_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | YEAR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( ADDDATE_SYMBOL | SUBDATE_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | CURDATE_SYMBOL ( parentheses )? | CURTIME_SYMBOL ( time_function_parameters )? | ( DATE_ADD_SYMBOL | DATE_SUB_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL INTERVAL_SYMBOL interval CLOSE_PAR_SYMBOL | EXTRACT_SYMBOL OPEN_PAR_SYMBOL interval_unit FROM_SYMBOL expression CLOSE_PAR_SYMBOL | GET_FORMAT_SYMBOL OPEN_PAR_SYMBOL date_time_type COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | NOW_SYMBOL ( time_function_parameters )? | POSITION_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression IN_SYMBOL expression CLOSE_PAR_SYMBOL | substring_function | SYSDATE_SYMBOL ( time_function_parameters )? | ( TIMESTAMP_ADD_SYMBOL | TIMESTAMP_DIFF_SYMBOL ) OPEN_PAR_SYMBOL interval_timestamp_unit COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | UTC_DATE_SYMBOL ( parentheses )? | UTC_TIME_SYMBOL ( time_function_parameters )? | UTC_TIMESTAMP_SYMBOL ( time_function_parameters )? | ASCII_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | CHARSET_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | COALESCE_SYMBOL expression_list_with_parentheses | COLLATION_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | DATABASE_SYMBOL parentheses | IF_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | FORMAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL | MICROSECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | MOD_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL string_literal CLOSE_PAR_SYMBOL | PASSWORD_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | QUARTER_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | REPEAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | REPLACE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | REVERSE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ROW_COUNT_SYMBOL parentheses | TRUNCATE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | WEEK_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL |{...}? WEIGHT_STRING_SYMBOL OPEN_PAR_SYMBOL expression ( ( AS_SYMBOL CHAR_SYMBOL field_length )? ( weight_string_levels )? | AS_SYMBOL BINARY_SYMBOL field_length | COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number ) CLOSE_PAR_SYMBOL | geometry_function | aggregate_function )

            ANTLR3_UINT32 alt654;

            alt654=52;

            {
                int LA654_0 = LA(1);
                if ( (LA654_0 == CHAR_SYMBOL))
                {
                    alt654=1;
                }
                else if ( (LA654_0 == CURRENT_USER_SYMBOL))
                {
                    alt654=2;
                }
                else if ( (LA654_0 == DATE_SYMBOL))
                {
                    alt654=3;
                }
                else if ( (LA654_0 == DAY_SYMBOL))
                {
                    alt654=4;
                }
                else if ( (LA654_0 == HOUR_SYMBOL))
                {
                    alt654=5;
                }
                else if ( (LA654_0 == INSERT_SYMBOL))
                {
                    alt654=6;
                }
                else if ( (LA654_0 == LEFT_SYMBOL))
                {
                    alt654=7;
                }
                else if ( (LA654_0 == MINUTE_SYMBOL))
                {
                    alt654=8;
                }
                else if ( (LA654_0 == MONTH_SYMBOL))
                {
                    alt654=9;
                }
                else if ( (LA654_0 == RIGHT_SYMBOL))
                {
                    alt654=10;
                }
                else if ( (LA654_0 == SECOND_SYMBOL))
                {
                    alt654=11;
                }
                else if ( (LA654_0 == TIME_SYMBOL))
                {
                    alt654=12;
                }
                else if ( (LA654_0 == TIMESTAMP_SYMBOL))
                {
                    alt654=13;
                }
                else if ( (LA654_0 == TRIM_SYMBOL))
                {
                    alt654=14;
                }
                else if ( (LA654_0 == USER_SYMBOL))
                {
                    alt654=15;
                }
                else if ( (LA654_0 == VALUES_SYMBOL))
                {
                    alt654=16;
                }
                else if ( (LA654_0 == YEAR_SYMBOL))
                {
                    alt654=17;
                }
                else if ( (LA654_0 == ADDDATE_SYMBOL || LA654_0 == SUBDATE_SYMBOL))
                {
                    alt654=18;
                }
                else if ( (LA654_0 == CURDATE_SYMBOL))
                {
                    alt654=19;
                }
                else if ( (LA654_0 == CURTIME_SYMBOL))
                {
                    alt654=20;
                }
                else if ( (LA654_0 == DATE_ADD_SYMBOL || LA654_0 == DATE_SUB_SYMBOL))
                {
                    alt654=21;
                }
                else if ( (LA654_0 == EXTRACT_SYMBOL))
                {
                    alt654=22;
                }
                else if ( (LA654_0 == GET_FORMAT_SYMBOL))
                {
                    alt654=23;
                }
                else if ( (LA654_0 == NOW_SYMBOL))
                {
                    alt654=24;
                }
                else if ( (LA654_0 == POSITION_SYMBOL))
                {
                    alt654=25;
                }
                else if ( (LA654_0 == SUBSTRING_SYMBOL))
                {
                    alt654=26;
                }
                else if ( (LA654_0 == SYSDATE_SYMBOL))
                {
                    alt654=27;
                }
                else if ( (((LA654_0 >= TIMESTAMP_ADD_SYMBOL) && (LA654_0 <= TIMESTAMP_DIFF_SYMBOL))))
                {
                    alt654=28;
                }
                else if ( (LA654_0 == UTC_DATE_SYMBOL))
                {
                    alt654=29;
                }
                else if ( (LA654_0 == UTC_TIME_SYMBOL))
                {
                    alt654=30;
                }
                else if ( (LA654_0 == UTC_TIMESTAMP_SYMBOL))
                {
                    alt654=31;
                }
                else if ( (LA654_0 == ASCII_SYMBOL))
                {
                    alt654=32;
                }
                else if ( (LA654_0 == CHARSET_SYMBOL))
                {
                    alt654=33;
                }
                else if ( (LA654_0 == COALESCE_SYMBOL))
                {
                    alt654=34;
                }
                else if ( (LA654_0 == COLLATION_SYMBOL))
                {
                    alt654=35;
                }
                else if ( (LA654_0 == DATABASE_SYMBOL))
                {
                    alt654=36;
                }
                else if ( (LA654_0 == IF_SYMBOL))
                {
                    alt654=37;
                }
                else if ( (LA654_0 == FORMAT_SYMBOL))
                {
                    alt654=38;
                }
                else if ( (LA654_0 == MICROSECOND_SYMBOL))
                {
                    alt654=39;
                }
                else if ( (LA654_0 == MOD_SYMBOL))
                {
                    alt654=40;
                }
                else if ( (LA654_0 == OLD_PASSWORD_SYMBOL))
                {
                    alt654=41;
                }
                else if ( (LA654_0 == PASSWORD_SYMBOL))
                {
                    alt654=42;
                }
                else if ( (LA654_0 == QUARTER_SYMBOL))
                {
                    alt654=43;
                }
                else if ( (LA654_0 == REPEAT_SYMBOL))
                {
                    alt654=44;
                }
                else if ( (LA654_0 == REPLACE_SYMBOL))
                {
                    alt654=45;
                }
                else if ( (LA654_0 == REVERSE_SYMBOL))
                {
                    alt654=46;
                }
                else if ( (LA654_0 == ROW_COUNT_SYMBOL))
                {
                    alt654=47;
                }
                else if ( (LA654_0 == TRUNCATE_SYMBOL))
                {
                    alt654=48;
                }
                else if ( (LA654_0 == WEEK_SYMBOL))
                {
                    alt654=49;
                }
                else if ( (LA654_0 == WEIGHT_STRING_SYMBOL))
                {
                    alt654=50;
                }
                else if ( (LA654_0 == CONTAINS_SYMBOL || LA654_0 == GEOMETRYCOLLECTION_SYMBOL || LA654_0 == LINESTRING_SYMBOL || ((LA654_0 >= MULTILINESTRING_SYMBOL) && (LA654_0 <= MULTIPOLYGON_SYMBOL)) || ((LA654_0 >= POINT_SYMBOL) && (LA654_0 <= POLYGON_SYMBOL))))
                {
                    alt654=51;
                }
                else if ( (LA654_0 == AVG_SYMBOL || LA654_0 == COUNT_SYMBOL || LA654_0 == GROUP_CONCAT_SYMBOL || LA654_0 == MAX_SYMBOL || LA654_0 == MIN_SYMBOL || LA654_0 == STDDEV_SAMP_SYMBOL || LA654_0 == STD_SYMBOL || LA654_0 == SUM_SYMBOL || LA654_0 == VARIANCE_SYMBOL || LA654_0 == VAR_SAMP_SYMBOL || ((LA654_0 >= BIT_AND_SYMBOL) && (LA654_0 <= BIT_XOR_SYMBOL))))
                {
                    alt654=52;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 654;
                    EXCEPTION->state        = 0;


                    goto ruleruntime_function_call_expressionEx;

                }
            }
            switch (alt654)
            {
        	case 1:
        	    // MySQLSimpleParser.g:1902:2: CHAR_SYMBOL OPEN_PAR_SYMBOL expression_list ( USING_SYMBOL charset_name )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression10798);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10800);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_in_runtime_function_call_expression10802);
        	        expression_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1902:46: ( USING_SYMBOL charset_name )?
        	        {
        	            int alt639=2;
        	            switch ( LA(1) )
        	            {
        	                case USING_SYMBOL:
        	                	{
        	                		alt639=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt639)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1902:47: USING_SYMBOL charset_name
        	        	    {
        	        	         MATCHT(USING_SYMBOL, &FOLLOW_USING_SYMBOL_in_runtime_function_call_expression10805);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_charset_name_in_runtime_function_call_expression10807);
        	        	        charset_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10811);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:1903:4: CURRENT_USER_SYMBOL ( parentheses )?
        	    {
        	         MATCHT(CURRENT_USER_SYMBOL, &FOLLOW_CURRENT_USER_SYMBOL_in_runtime_function_call_expression10816);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1903:24: ( parentheses )?
        	        {
        	            int alt640=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt640=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt640)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1903:24: parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression10818);
        	        	        parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:1904:4: DATE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(DATE_SYMBOL, &FOLLOW_DATE_SYMBOL_in_runtime_function_call_expression10824);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10826);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10828);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10830);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:1905:4: DAY_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(DAY_SYMBOL, &FOLLOW_DAY_SYMBOL_in_runtime_function_call_expression10835);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10837);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10839);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10841);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:1906:4: HOUR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(HOUR_SYMBOL, &FOLLOW_HOUR_SYMBOL_in_runtime_function_call_expression10846);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10848);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10850);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10852);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:1907:4: INSERT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(INSERT_SYMBOL, &FOLLOW_INSERT_SYMBOL_in_runtime_function_call_expression10857);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10859);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10861);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10863);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10865);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10867);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10869);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10871);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10873);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10875);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:1909:4: LEFT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(LEFT_SYMBOL, &FOLLOW_LEFT_SYMBOL_in_runtime_function_call_expression10882);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10884);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10886);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10888);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10890);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10892);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:1910:4: MINUTE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MINUTE_SYMBOL, &FOLLOW_MINUTE_SYMBOL_in_runtime_function_call_expression10897);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10899);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10901);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10903);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:1911:4: MONTH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MONTH_SYMBOL, &FOLLOW_MONTH_SYMBOL_in_runtime_function_call_expression10908);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10910);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10912);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10914);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:1912:4: RIGHT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(RIGHT_SYMBOL, &FOLLOW_RIGHT_SYMBOL_in_runtime_function_call_expression10919);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10921);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10923);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10925);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10927);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10929);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:1913:4: SECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(SECOND_SYMBOL, &FOLLOW_SECOND_SYMBOL_in_runtime_function_call_expression10934);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10936);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10938);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10940);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:1914:4: TIME_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(TIME_SYMBOL, &FOLLOW_TIME_SYMBOL_in_runtime_function_call_expression10945);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10947);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10949);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10951);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:1915:4: TIMESTAMP_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(TIMESTAMP_SYMBOL, &FOLLOW_TIMESTAMP_SYMBOL_in_runtime_function_call_expression10956);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10958);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10960);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1915:48: ( COMMA_SYMBOL expression )?
        	        {
        	            int alt641=2;
        	            switch ( LA(1) )
        	            {
        	                case COMMA_SYMBOL:
        	                	{
        	                		alt641=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt641)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1915:49: COMMA_SYMBOL expression
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression10963);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10965);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10969);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:1916:4: trim_function
        	    {
        	        FOLLOWPUSH(FOLLOW_trim_function_in_runtime_function_call_expression10974);
        	        trim_function(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:1917:4: USER_SYMBOL parentheses
        	    {
        	         MATCHT(USER_SYMBOL, &FOLLOW_USER_SYMBOL_in_runtime_function_call_expression10979);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression10981);
        	        parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:1918:4: VALUES_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(VALUES_SYMBOL, &FOLLOW_VALUES_SYMBOL_in_runtime_function_call_expression10986);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10988);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression10990);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression10992);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:1919:4: YEAR_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(YEAR_SYMBOL, &FOLLOW_YEAR_SYMBOL_in_runtime_function_call_expression10997);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression10999);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11001);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11003);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:1922:4: ( ADDDATE_SYMBOL | SUBDATE_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	        if ( LA(1) == ADDDATE_SYMBOL || LA(1) == SUBDATE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleruntime_function_call_expressionEx;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11019);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11021);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11023);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11025);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11027);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:1923:4: CURDATE_SYMBOL ( parentheses )?
        	    {
        	         MATCHT(CURDATE_SYMBOL, &FOLLOW_CURDATE_SYMBOL_in_runtime_function_call_expression11032);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1923:19: ( parentheses )?
        	        {
        	            int alt642=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt642=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt642)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1923:19: parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression11034);
        	        	        parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:1924:4: CURTIME_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(CURTIME_SYMBOL, &FOLLOW_CURTIME_SYMBOL_in_runtime_function_call_expression11040);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1924:19: ( time_function_parameters )?
        	        {
        	            int alt643=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt643=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt643)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1924:19: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_runtime_function_call_expression11042);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:1925:4: ( DATE_ADD_SYMBOL | DATE_SUB_SYMBOL ) OPEN_PAR_SYMBOL expression COMMA_SYMBOL INTERVAL_SYMBOL interval CLOSE_PAR_SYMBOL
        	    {
        	        if ( LA(1) == DATE_ADD_SYMBOL || LA(1) == DATE_SUB_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleruntime_function_call_expressionEx;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11056);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11058);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11060);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(INTERVAL_SYMBOL, &FOLLOW_INTERVAL_SYMBOL_in_runtime_function_call_expression11062);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_interval_in_runtime_function_call_expression11064);
        	        interval(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11066);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 22:
        	    // MySQLSimpleParser.g:1926:4: EXTRACT_SYMBOL OPEN_PAR_SYMBOL interval_unit FROM_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(EXTRACT_SYMBOL, &FOLLOW_EXTRACT_SYMBOL_in_runtime_function_call_expression11071);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11073);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_interval_unit_in_runtime_function_call_expression11075);
        	        interval_unit(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_runtime_function_call_expression11077);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11079);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11081);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 23:
        	    // MySQLSimpleParser.g:1927:4: GET_FORMAT_SYMBOL OPEN_PAR_SYMBOL date_time_type COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(GET_FORMAT_SYMBOL, &FOLLOW_GET_FORMAT_SYMBOL_in_runtime_function_call_expression11086);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11088);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_date_time_type_in_runtime_function_call_expression11090);
        	        date_time_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11093);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11095);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11097);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 24:
        	    // MySQLSimpleParser.g:1928:4: NOW_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(NOW_SYMBOL, &FOLLOW_NOW_SYMBOL_in_runtime_function_call_expression11102);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1928:15: ( time_function_parameters )?
        	        {
        	            int alt644=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt644=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt644)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1928:15: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_runtime_function_call_expression11104);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 25:
        	    // MySQLSimpleParser.g:1929:4: POSITION_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression IN_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(POSITION_SYMBOL, &FOLLOW_POSITION_SYMBOL_in_runtime_function_call_expression11110);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11112);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_runtime_function_call_expression11114);
        	        bitwise_or_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_runtime_function_call_expression11116);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11118);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11120);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 26:
        	    // MySQLSimpleParser.g:1930:4: substring_function
        	    {
        	        FOLLOWPUSH(FOLLOW_substring_function_in_runtime_function_call_expression11125);
        	        substring_function(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 27:
        	    // MySQLSimpleParser.g:1931:4: SYSDATE_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(SYSDATE_SYMBOL, &FOLLOW_SYSDATE_SYMBOL_in_runtime_function_call_expression11130);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1931:19: ( time_function_parameters )?
        	        {
        	            int alt645=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt645=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt645)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1931:19: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_runtime_function_call_expression11132);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 28:
        	    // MySQLSimpleParser.g:1932:4: ( TIMESTAMP_ADD_SYMBOL | TIMESTAMP_DIFF_SYMBOL ) OPEN_PAR_SYMBOL interval_timestamp_unit COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	        if ( ((LA(1) >= TIMESTAMP_ADD_SYMBOL) && (LA(1) <= TIMESTAMP_DIFF_SYMBOL)) )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleruntime_function_call_expressionEx;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11146);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_interval_timestamp_unit_in_runtime_function_call_expression11148);
        	        interval_timestamp_unit(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11150);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11152);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11154);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11156);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11158);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 29:
        	    // MySQLSimpleParser.g:1933:4: UTC_DATE_SYMBOL ( parentheses )?
        	    {
        	         MATCHT(UTC_DATE_SYMBOL, &FOLLOW_UTC_DATE_SYMBOL_in_runtime_function_call_expression11163);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1933:20: ( parentheses )?
        	        {
        	            int alt646=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt646=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt646)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1933:20: parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression11165);
        	        	        parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 30:
        	    // MySQLSimpleParser.g:1934:4: UTC_TIME_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(UTC_TIME_SYMBOL, &FOLLOW_UTC_TIME_SYMBOL_in_runtime_function_call_expression11171);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1934:20: ( time_function_parameters )?
        	        {
        	            int alt647=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt647=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt647)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1934:20: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_runtime_function_call_expression11173);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 31:
        	    // MySQLSimpleParser.g:1935:4: UTC_TIMESTAMP_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(UTC_TIMESTAMP_SYMBOL, &FOLLOW_UTC_TIMESTAMP_SYMBOL_in_runtime_function_call_expression11179);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1935:25: ( time_function_parameters )?
        	        {
        	            int alt648=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt648=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt648)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1935:25: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_runtime_function_call_expression11181);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 32:
        	    // MySQLSimpleParser.g:1938:4: ASCII_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(ASCII_SYMBOL, &FOLLOW_ASCII_SYMBOL_in_runtime_function_call_expression11190);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11192);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11194);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11196);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 33:
        	    // MySQLSimpleParser.g:1939:4: CHARSET_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(CHARSET_SYMBOL, &FOLLOW_CHARSET_SYMBOL_in_runtime_function_call_expression11201);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11203);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11205);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11207);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 34:
        	    // MySQLSimpleParser.g:1940:4: COALESCE_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(COALESCE_SYMBOL, &FOLLOW_COALESCE_SYMBOL_in_runtime_function_call_expression11212);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_runtime_function_call_expression11214);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 35:
        	    // MySQLSimpleParser.g:1941:4: COLLATION_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(COLLATION_SYMBOL, &FOLLOW_COLLATION_SYMBOL_in_runtime_function_call_expression11219);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11221);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11223);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11225);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 36:
        	    // MySQLSimpleParser.g:1942:4: DATABASE_SYMBOL parentheses
        	    {
        	         MATCHT(DATABASE_SYMBOL, &FOLLOW_DATABASE_SYMBOL_in_runtime_function_call_expression11230);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression11232);
        	        parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 37:
        	    // MySQLSimpleParser.g:1943:4: IF_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(IF_SYMBOL, &FOLLOW_IF_SYMBOL_in_runtime_function_call_expression11237);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11239);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11241);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11243);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11245);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11247);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11249);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11251);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 38:
        	    // MySQLSimpleParser.g:1944:4: FORMAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(FORMAT_SYMBOL, &FOLLOW_FORMAT_SYMBOL_in_runtime_function_call_expression11256);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11258);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11260);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11262);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11264);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1944:69: ( COMMA_SYMBOL expression )?
        	        {
        	            int alt649=2;
        	            switch ( LA(1) )
        	            {
        	                case COMMA_SYMBOL:
        	                	{
        	                		alt649=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt649)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1944:70: COMMA_SYMBOL expression
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11267);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11269);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11273);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 39:
        	    // MySQLSimpleParser.g:1945:4: MICROSECOND_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MICROSECOND_SYMBOL, &FOLLOW_MICROSECOND_SYMBOL_in_runtime_function_call_expression11278);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11280);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11282);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11284);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 40:
        	    // MySQLSimpleParser.g:1946:4: MOD_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MOD_SYMBOL, &FOLLOW_MOD_SYMBOL_in_runtime_function_call_expression11289);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11291);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11293);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11295);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11297);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11299);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 41:
        	    // MySQLSimpleParser.g:1947:4: {...}? OLD_PASSWORD_SYMBOL OPEN_PAR_SYMBOL string_literal CLOSE_PAR_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION < 50607)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50607";
        	                EXCEPTION->ruleName	 = (void *)"runtime_function_call_expression";


        	        }

        	         MATCHT(OLD_PASSWORD_SYMBOL, &FOLLOW_OLD_PASSWORD_SYMBOL_in_runtime_function_call_expression11306);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11308);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_runtime_function_call_expression11310);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11312);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 42:
        	    // MySQLSimpleParser.g:1948:4: PASSWORD_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_runtime_function_call_expression11317);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11319);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11321);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11323);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 43:
        	    // MySQLSimpleParser.g:1949:4: QUARTER_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(QUARTER_SYMBOL, &FOLLOW_QUARTER_SYMBOL_in_runtime_function_call_expression11328);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11330);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11332);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11334);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 44:
        	    // MySQLSimpleParser.g:1950:4: REPEAT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPEAT_SYMBOL, &FOLLOW_REPEAT_SYMBOL_in_runtime_function_call_expression11339);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11341);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11343);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11345);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11347);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11349);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 45:
        	    // MySQLSimpleParser.g:1951:4: REPLACE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REPLACE_SYMBOL, &FOLLOW_REPLACE_SYMBOL_in_runtime_function_call_expression11354);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11356);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11358);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11360);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11362);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11364);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11366);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11368);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 46:
        	    // MySQLSimpleParser.g:1952:4: REVERSE_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(REVERSE_SYMBOL, &FOLLOW_REVERSE_SYMBOL_in_runtime_function_call_expression11373);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11375);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11377);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11379);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 47:
        	    // MySQLSimpleParser.g:1953:4: ROW_COUNT_SYMBOL parentheses
        	    {
        	         MATCHT(ROW_COUNT_SYMBOL, &FOLLOW_ROW_COUNT_SYMBOL_in_runtime_function_call_expression11384);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_parentheses_in_runtime_function_call_expression11386);
        	        parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 48:
        	    // MySQLSimpleParser.g:1954:4: TRUNCATE_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(TRUNCATE_SYMBOL, &FOLLOW_TRUNCATE_SYMBOL_in_runtime_function_call_expression11391);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11393);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11395);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11397);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11399);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11401);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 49:
        	    // MySQLSimpleParser.g:1955:4: WEEK_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(WEEK_SYMBOL, &FOLLOW_WEEK_SYMBOL_in_runtime_function_call_expression11406);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11408);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11410);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1955:43: ( COMMA_SYMBOL expression )?
        	        {
        	            int alt650=2;
        	            switch ( LA(1) )
        	            {
        	                case COMMA_SYMBOL:
        	                	{
        	                		alt650=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt650)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1955:44: COMMA_SYMBOL expression
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11413);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11415);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11419);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 50:
        	    // MySQLSimpleParser.g:1956:4: {...}? WEIGHT_STRING_SYMBOL OPEN_PAR_SYMBOL expression ( ( AS_SYMBOL CHAR_SYMBOL field_length )? ( weight_string_levels )? | AS_SYMBOL BINARY_SYMBOL field_length | COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number ) CLOSE_PAR_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50600)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
        	                EXCEPTION->ruleName	 = (void *)"runtime_function_call_expression";


        	        }

        	         MATCHT(WEIGHT_STRING_SYMBOL, &FOLLOW_WEIGHT_STRING_SYMBOL_in_runtime_function_call_expression11426);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_runtime_function_call_expression11428);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_runtime_function_call_expression11430);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:1957:3: ( ( AS_SYMBOL CHAR_SYMBOL field_length )? ( weight_string_levels )? | AS_SYMBOL BINARY_SYMBOL field_length | COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number )
        	        {
        	            int alt653=3;
        	            switch ( LA(1) )
        	            {
        	            case AS_SYMBOL:
        	            	{
        	            		switch ( LA(2) )
        	            		{
        	            		case CHAR_SYMBOL:
        	            			{
        	            				alt653=1;
        	            			}
        	            		    break;
        	            		case BINARY_SYMBOL:
        	            			{
        	            				alt653=2;
        	            			}
        	            		    break;

        	            		default:
        	            		    if (BACKTRACKING>0)
        	            		    {
        	            		        FAILEDFLAG = ANTLR3_TRUE;
        	            		        return ;
        	            		    }

        	            		    CONSTRUCTEX();
        	            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	            		    EXCEPTION->message      = (void *)"";
        	            		    EXCEPTION->decisionNum  = 653;
        	            		    EXCEPTION->state        = 1;


        	            		    goto ruleruntime_function_call_expressionEx;

        	            		}

        	            	}
        	                break;
        	            case LEVEL_SYMBOL:
        	            case CLOSE_PAR_SYMBOL:
        	            	{
        	            		alt653=1;
        	            	}
        	                break;
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt653=3;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 653;
        	                EXCEPTION->state        = 0;


        	                goto ruleruntime_function_call_expressionEx;

        	            }

        	            switch (alt653)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:1958:4: ( AS_SYMBOL CHAR_SYMBOL field_length )? ( weight_string_levels )?
        	        	    {
        	        	        // MySQLSimpleParser.g:1958:4: ( AS_SYMBOL CHAR_SYMBOL field_length )?
        	        	        {
        	        	            int alt651=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case AS_SYMBOL:
        	        	                	{
        	        	                		alt651=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt651)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1958:5: AS_SYMBOL CHAR_SYMBOL field_length
        	        	        	    {
        	        	        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11440);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_runtime_function_call_expression11442);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	        FOLLOWPUSH(FOLLOW_field_length_in_runtime_function_call_expression11444);
        	        	        	        field_length(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        // MySQLSimpleParser.g:1958:42: ( weight_string_levels )?
        	        	        {
        	        	            int alt652=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case LEVEL_SYMBOL:
        	        	                	{
        	        	                		alt652=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt652)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:1958:42: weight_string_levels
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_weight_string_levels_in_runtime_function_call_expression11448);
        	        	        	        weight_string_levels(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:1959:6: AS_SYMBOL BINARY_SYMBOL field_length
        	        	    {
        	        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_runtime_function_call_expression11456);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_runtime_function_call_expression11458);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_field_length_in_runtime_function_call_expression11460);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:1960:6: COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number COMMA_SYMBOL ulong_number
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11467);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_runtime_function_call_expression11469);
        	        	        ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11471);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_runtime_function_call_expression11473);
        	        	        ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_runtime_function_call_expression11475);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_runtime_function_call_expression11477);
        	        	        ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleruntime_function_call_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_runtime_function_call_expression11485);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 51:
        	    // MySQLSimpleParser.g:1963:4: geometry_function
        	    {
        	        FOLLOWPUSH(FOLLOW_geometry_function_in_runtime_function_call_expression11490);
        	        geometry_function(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 52:
        	    // MySQLSimpleParser.g:1964:4: aggregate_function
        	    {
        	        FOLLOWPUSH(FOLLOW_aggregate_function_in_runtime_function_call_expression11495);
        	        aggregate_function(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleruntime_function_call_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleruntime_function_call_expressionEx; /* Prevent compiler warnings */
    ruleruntime_function_call_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end runtime_function_call_expression */

/**
 * $ANTLR start time_function_parameters
 * MySQLSimpleParser.g:1967:1: time_function_parameters : OPEN_PAR_SYMBOL ( fractional_precision )? CLOSE_PAR_SYMBOL ;
 */
static void
time_function_parameters(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1967:25: ( OPEN_PAR_SYMBOL ( fractional_precision )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:1968:2: OPEN_PAR_SYMBOL ( fractional_precision )? CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_time_function_parameters11504);
            if  (HASEXCEPTION())
            {
                goto ruletime_function_parametersEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1968:18: ( fractional_precision )?
            {
                int alt655=2;
                {
                    int LA655_0 = LA(1);
                    if ( ((SERVER_VERSION >= 50604)) && (LA655_0 == INT_NUMBER))
                    {
                        alt655=1;
                    }
                }
                switch (alt655)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1968:18: fractional_precision
            	    {
            	        FOLLOWPUSH(FOLLOW_fractional_precision_in_time_function_parameters11506);
            	        fractional_precision(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletime_function_parametersEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_time_function_parameters11509);
            if  (HASEXCEPTION())
            {
                goto ruletime_function_parametersEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletime_function_parametersEx; /* Prevent compiler warnings */
    ruletime_function_parametersEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end time_function_parameters */

/**
 * $ANTLR start fractional_precision
 * MySQLSimpleParser.g:1971:1: fractional_precision :{...}? => INT_NUMBER ;
 */
static void
fractional_precision(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1971:21: ({...}? => INT_NUMBER )
        // MySQLSimpleParser.g:1972:2: {...}? => INT_NUMBER
        {
            if ( !((SERVER_VERSION >= 50604)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50604";
                    EXCEPTION->ruleName	 = (void *)"fractional_precision";


            }

             MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_fractional_precision11522);
            if  (HASEXCEPTION())
            {
                goto rulefractional_precisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefractional_precisionEx; /* Prevent compiler warnings */
    rulefractional_precisionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end fractional_precision */

/**
 * $ANTLR start weight_string_levels
 * MySQLSimpleParser.g:1975:1: weight_string_levels : LEVEL_SYMBOL ( real_ulong_number MINUS_OPERATOR real_ulong_number | weight_string_level_list_item ( COMMA_SYMBOL weight_string_level_list_item )* ) ;
 */
static void
weight_string_levels(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1975:21: ( LEVEL_SYMBOL ( real_ulong_number MINUS_OPERATOR real_ulong_number | weight_string_level_list_item ( COMMA_SYMBOL weight_string_level_list_item )* ) )
        // MySQLSimpleParser.g:1976:2: LEVEL_SYMBOL ( real_ulong_number MINUS_OPERATOR real_ulong_number | weight_string_level_list_item ( COMMA_SYMBOL weight_string_level_list_item )* )
        {
             MATCHT(LEVEL_SYMBOL, &FOLLOW_LEVEL_SYMBOL_in_weight_string_levels11531);
            if  (HASEXCEPTION())
            {
                goto ruleweight_string_levelsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1977:2: ( real_ulong_number MINUS_OPERATOR real_ulong_number | weight_string_level_list_item ( COMMA_SYMBOL weight_string_level_list_item )* )
            {
                int alt657=2;
                switch ( LA(1) )
                {
                case HEX_NUMBER:
                case INT_NUMBER:
                case LONG_NUMBER:
                case ULONGLONG_NUMBER:
                	{
                		switch ( LA(2) )
                		{
                		case MINUS_OPERATOR:
                			{
                				alt657=1;
                			}
                		    break;
                		case REVERSE_SYMBOL:
                		case ASC_SYMBOL:
                		case DESC_SYMBOL:
                		case CLOSE_PAR_SYMBOL:
                		case COMMA_SYMBOL:
                			{
                				alt657=2;
                			}
                		    break;

                		default:
                		    if (BACKTRACKING>0)
                		    {
                		        FAILEDFLAG = ANTLR3_TRUE;
                		        return ;
                		    }

                		    CONSTRUCTEX();
                		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		    EXCEPTION->message      = (void *)"";
                		    EXCEPTION->decisionNum  = 657;
                		    EXCEPTION->state        = 1;


                		    goto ruleweight_string_levelsEx;

                		}

                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 657;
                    EXCEPTION->state        = 0;


                    goto ruleweight_string_levelsEx;

                }

                switch (alt657)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1978:3: real_ulong_number MINUS_OPERATOR real_ulong_number
            	    {
            	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_weight_string_levels11538);
            	        real_ulong_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleweight_string_levelsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MINUS_OPERATOR, &FOLLOW_MINUS_OPERATOR_in_weight_string_levels11540);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleweight_string_levelsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_weight_string_levels11542);
            	        real_ulong_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleweight_string_levelsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1979:5: weight_string_level_list_item ( COMMA_SYMBOL weight_string_level_list_item )*
            	    {
            	        FOLLOWPUSH(FOLLOW_weight_string_level_list_item_in_weight_string_levels11548);
            	        weight_string_level_list_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleweight_string_levelsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:1979:35: ( COMMA_SYMBOL weight_string_level_list_item )*

            	        for (;;)
            	        {
            	            int alt656=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt656=1;
            	            	}
            	                break;

            	            }

            	            switch (alt656)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1979:36: COMMA_SYMBOL weight_string_level_list_item
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_weight_string_levels11551);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleweight_string_levelsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_weight_string_level_list_item_in_weight_string_levels11553);
            	        	        weight_string_level_list_item(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleweight_string_levelsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop656;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop656: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleweight_string_levelsEx; /* Prevent compiler warnings */
    ruleweight_string_levelsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end weight_string_levels */

/**
 * $ANTLR start weight_string_level_list_item
 * MySQLSimpleParser.g:1983:1: weight_string_level_list_item : real_ulong_number ( ( ASC_SYMBOL | DESC_SYMBOL ) ( REVERSE_SYMBOL )? | REVERSE_SYMBOL )? ;
 */
static void
weight_string_level_list_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1983:30: ( real_ulong_number ( ( ASC_SYMBOL | DESC_SYMBOL ) ( REVERSE_SYMBOL )? | REVERSE_SYMBOL )? )
        // MySQLSimpleParser.g:1984:2: real_ulong_number ( ( ASC_SYMBOL | DESC_SYMBOL ) ( REVERSE_SYMBOL )? | REVERSE_SYMBOL )?
        {
            FOLLOWPUSH(FOLLOW_real_ulong_number_in_weight_string_level_list_item11567);
            real_ulong_number(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleweight_string_level_list_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:1985:2: ( ( ASC_SYMBOL | DESC_SYMBOL ) ( REVERSE_SYMBOL )? | REVERSE_SYMBOL )?
            {
                int alt659=3;
                switch ( LA(1) )
                {
                    case ASC_SYMBOL:
                    case DESC_SYMBOL:
                    	{
                    		alt659=1;
                    	}
                        break;
                    case REVERSE_SYMBOL:
                    	{
                    		alt659=2;
                    	}
                        break;
                }

                switch (alt659)
                {
            	case 1:
            	    // MySQLSimpleParser.g:1986:3: ( ASC_SYMBOL | DESC_SYMBOL ) ( REVERSE_SYMBOL )?
            	    {
            	        if ( LA(1) == ASC_SYMBOL || LA(1) == DESC_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleweight_string_level_list_itemEx;
            	        }


            	        // MySQLSimpleParser.g:1986:30: ( REVERSE_SYMBOL )?
            	        {
            	            int alt658=2;
            	            switch ( LA(1) )
            	            {
            	                case REVERSE_SYMBOL:
            	                	{
            	                		alt658=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt658)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:1986:30: REVERSE_SYMBOL
            	        	    {
            	        	         MATCHT(REVERSE_SYMBOL, &FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11582);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleweight_string_level_list_itemEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:1987:5: REVERSE_SYMBOL
            	    {
            	         MATCHT(REVERSE_SYMBOL, &FOLLOW_REVERSE_SYMBOL_in_weight_string_level_list_item11589);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleweight_string_level_list_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleweight_string_level_list_itemEx; /* Prevent compiler warnings */
    ruleweight_string_level_list_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end weight_string_level_list_item */

/**
 * $ANTLR start date_time_type
 * MySQLSimpleParser.g:1991:1: date_time_type : ( DATE_SYMBOL | TIME_SYMBOL | DATETIME_SYMBOL | TIMESTAMP_SYMBOL );
 */
static void
date_time_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1991:15: ( DATE_SYMBOL | TIME_SYMBOL | DATETIME_SYMBOL | TIMESTAMP_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= DATETIME_SYMBOL) && (LA(1) <= DATE_SYMBOL)) || LA(1) == TIMESTAMP_SYMBOL || LA(1) == TIME_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruledate_time_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledate_time_typeEx; /* Prevent compiler warnings */
    ruledate_time_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end date_time_type */

/**
 * $ANTLR start trim_function
 * MySQLSimpleParser.g:1998:1: trim_function : TRIM_SYMBOL OPEN_PAR_SYMBOL ( expression ( FROM_SYMBOL expression )? | LEADING_SYMBOL ( expression )? FROM_SYMBOL expression | TRAILING_SYMBOL ( expression )? FROM_SYMBOL expression | BOTH_SYMBOL ( expression )? FROM_SYMBOL expression ) CLOSE_PAR_SYMBOL ;
 */
static void
trim_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:1998:14: ( TRIM_SYMBOL OPEN_PAR_SYMBOL ( expression ( FROM_SYMBOL expression )? | LEADING_SYMBOL ( expression )? FROM_SYMBOL expression | TRAILING_SYMBOL ( expression )? FROM_SYMBOL expression | BOTH_SYMBOL ( expression )? FROM_SYMBOL expression ) CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:1999:2: TRIM_SYMBOL OPEN_PAR_SYMBOL ( expression ( FROM_SYMBOL expression )? | LEADING_SYMBOL ( expression )? FROM_SYMBOL expression | TRAILING_SYMBOL ( expression )? FROM_SYMBOL expression | BOTH_SYMBOL ( expression )? FROM_SYMBOL expression ) CLOSE_PAR_SYMBOL
        {
             MATCHT(TRIM_SYMBOL, &FOLLOW_TRIM_SYMBOL_in_trim_function11626);
            if  (HASEXCEPTION())
            {
                goto ruletrim_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_trim_function11628);
            if  (HASEXCEPTION())
            {
                goto ruletrim_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2000:2: ( expression ( FROM_SYMBOL expression )? | LEADING_SYMBOL ( expression )? FROM_SYMBOL expression | TRAILING_SYMBOL ( expression )? FROM_SYMBOL expression | BOTH_SYMBOL ( expression )? FROM_SYMBOL expression )
            {
                int alt664=4;
                {
                    int LA664_0 = LA(1);
                    if ( (((LA664_0 >= ACCOUNT_SYMBOL) && (LA664_0 <= YEAR_SYMBOL)) || LA664_0 == BINARY_SYMBOL || ((LA664_0 >= CASE_SYMBOL) && (LA664_0 <= CAST_SYMBOL)) || LA664_0 == CHAR_SYMBOL || ((LA664_0 >= CONVERT_SYMBOL) && (LA664_0 <= COUNT_SYMBOL)) || LA664_0 == CURDATE_SYMBOL || LA664_0 == CURRENT_USER_SYMBOL || LA664_0 == CURTIME_SYMBOL || LA664_0 == DATABASE_SYMBOL || LA664_0 == DATE_ADD_SYMBOL || LA664_0 == DATE_SUB_SYMBOL || LA664_0 == EXISTS_SYMBOL || ((LA664_0 >= EXTRACT_SYMBOL) && (LA664_0 <= FALSE_SYMBOL)) || LA664_0 == GROUP_CONCAT_SYMBOL || LA664_0 == IF_SYMBOL || LA664_0 == INSERT_SYMBOL || LA664_0 == INTERVAL_SYMBOL || LA664_0 == LEFT_SYMBOL || LA664_0 == MATCH_SYMBOL || LA664_0 == MAX_SYMBOL || LA664_0 == MIN_SYMBOL || LA664_0 == MOD_SYMBOL || LA664_0 == NCHAR_TEXT || ((LA664_0 >= NOT2_SYMBOL) && (LA664_0 <= NOW_SYMBOL)) || ((LA664_0 >= NULL2_SYMBOL) && (LA664_0 <= NULL_SYMBOL)) || ((LA664_0 >= OPEN_CURLY_SYMBOL) && (LA664_0 <= OPEN_PAR_SYMBOL)) || ((LA664_0 >= PARAM_MARKER) && (LA664_0 <= POSITION_SYMBOL)) || ((LA664_0 >= REPEAT_SYMBOL) && (LA664_0 <= REPLACE_SYMBOL)) || LA664_0 == RIGHT_SYMBOL || LA664_0 == SINGLE_QUOTED_TEXT || LA664_0 == STDDEV_SAMP_SYMBOL || LA664_0 == STD_SYMBOL || LA664_0 == SUBSTRING_SYMBOL || ((LA664_0 >= SUM_SYMBOL) && (LA664_0 <= SYSDATE_SYMBOL)) || ((LA664_0 >= TRIM_SYMBOL) && (LA664_0 <= TRUE_SYMBOL)) || ((LA664_0 >= UTC_DATE_SYMBOL) && (LA664_0 <= UTC_TIME_SYMBOL)) || LA664_0 == VALUES_SYMBOL || LA664_0 == VARIANCE_SYMBOL || LA664_0 == VAR_SAMP_SYMBOL || ((LA664_0 >= AT_AT_SIGN_SYMBOL) && (LA664_0 <= AT_TEXT_SUFFIX)) || ((LA664_0 >= BACK_TICK_QUOTED_ID) && (LA664_0 <= BIN_NUMBER)) || LA664_0 == BITWISE_NOT_OPERATOR || ((LA664_0 >= BIT_AND_SYMBOL) && (LA664_0 <= BIT_XOR_SYMBOL)) || LA664_0 == DECIMAL_NUMBER || LA664_0 == DOT_SYMBOL || LA664_0 == DOUBLE_QUOTED_TEXT || LA664_0 == FLOAT_NUMBER || ((LA664_0 >= HEX_NUMBER) && (LA664_0 <= IDENTIFIER)) || LA664_0 == INT_NUMBER || LA664_0 == LOGICAL_NOT_OPERATOR || ((LA664_0 >= LONG_NUMBER) && (LA664_0 <= MINUS_OPERATOR)) || LA664_0 == PLUS_OPERATOR || ((LA664_0 >= ULONGLONG_NUMBER) && (LA664_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt664=1;
                    }
                    else if ( (LA664_0 == LEADING_SYMBOL))
                    {
                        alt664=2;
                    }
                    else if ( (LA664_0 == TRAILING_SYMBOL))
                    {
                        alt664=3;
                    }
                    else if ( (LA664_0 == BOTH_SYMBOL))
                    {
                        alt664=4;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 664;
                        EXCEPTION->state        = 0;


                        goto ruletrim_functionEx;

                    }
                }
                switch (alt664)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2001:3: expression ( FROM_SYMBOL expression )?
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11635);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2001:14: ( FROM_SYMBOL expression )?
            	        {
            	            int alt660=2;
            	            switch ( LA(1) )
            	            {
            	                case FROM_SYMBOL:
            	                	{
            	                		alt660=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt660)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2001:15: FROM_SYMBOL expression
            	        	    {
            	        	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_trim_function11638);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletrim_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11640);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletrim_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2002:5: LEADING_SYMBOL ( expression )? FROM_SYMBOL expression
            	    {
            	         MATCHT(LEADING_SYMBOL, &FOLLOW_LEADING_SYMBOL_in_trim_function11648);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2002:20: ( expression )?
            	        {
            	            int alt661=2;
            	            {
            	                int LA661_0 = LA(1);
            	                if ( (((LA661_0 >= ACCOUNT_SYMBOL) && (LA661_0 <= YEAR_SYMBOL)) || LA661_0 == BINARY_SYMBOL || ((LA661_0 >= CASE_SYMBOL) && (LA661_0 <= CAST_SYMBOL)) || LA661_0 == CHAR_SYMBOL || ((LA661_0 >= CONVERT_SYMBOL) && (LA661_0 <= COUNT_SYMBOL)) || LA661_0 == CURDATE_SYMBOL || LA661_0 == CURRENT_USER_SYMBOL || LA661_0 == CURTIME_SYMBOL || LA661_0 == DATABASE_SYMBOL || LA661_0 == DATE_ADD_SYMBOL || LA661_0 == DATE_SUB_SYMBOL || LA661_0 == EXISTS_SYMBOL || ((LA661_0 >= EXTRACT_SYMBOL) && (LA661_0 <= FALSE_SYMBOL)) || LA661_0 == GROUP_CONCAT_SYMBOL || LA661_0 == IF_SYMBOL || LA661_0 == INSERT_SYMBOL || LA661_0 == INTERVAL_SYMBOL || LA661_0 == LEFT_SYMBOL || LA661_0 == MATCH_SYMBOL || LA661_0 == MAX_SYMBOL || LA661_0 == MIN_SYMBOL || LA661_0 == MOD_SYMBOL || LA661_0 == NCHAR_TEXT || ((LA661_0 >= NOT2_SYMBOL) && (LA661_0 <= NOW_SYMBOL)) || ((LA661_0 >= NULL2_SYMBOL) && (LA661_0 <= NULL_SYMBOL)) || ((LA661_0 >= OPEN_CURLY_SYMBOL) && (LA661_0 <= OPEN_PAR_SYMBOL)) || ((LA661_0 >= PARAM_MARKER) && (LA661_0 <= POSITION_SYMBOL)) || ((LA661_0 >= REPEAT_SYMBOL) && (LA661_0 <= REPLACE_SYMBOL)) || LA661_0 == RIGHT_SYMBOL || LA661_0 == SINGLE_QUOTED_TEXT || LA661_0 == STDDEV_SAMP_SYMBOL || LA661_0 == STD_SYMBOL || LA661_0 == SUBSTRING_SYMBOL || ((LA661_0 >= SUM_SYMBOL) && (LA661_0 <= SYSDATE_SYMBOL)) || ((LA661_0 >= TRIM_SYMBOL) && (LA661_0 <= TRUE_SYMBOL)) || ((LA661_0 >= UTC_DATE_SYMBOL) && (LA661_0 <= UTC_TIME_SYMBOL)) || LA661_0 == VALUES_SYMBOL || LA661_0 == VARIANCE_SYMBOL || LA661_0 == VAR_SAMP_SYMBOL || ((LA661_0 >= AT_AT_SIGN_SYMBOL) && (LA661_0 <= AT_TEXT_SUFFIX)) || ((LA661_0 >= BACK_TICK_QUOTED_ID) && (LA661_0 <= BIN_NUMBER)) || LA661_0 == BITWISE_NOT_OPERATOR || ((LA661_0 >= BIT_AND_SYMBOL) && (LA661_0 <= BIT_XOR_SYMBOL)) || LA661_0 == DECIMAL_NUMBER || LA661_0 == DOT_SYMBOL || LA661_0 == DOUBLE_QUOTED_TEXT || LA661_0 == FLOAT_NUMBER || ((LA661_0 >= HEX_NUMBER) && (LA661_0 <= IDENTIFIER)) || LA661_0 == INT_NUMBER || LA661_0 == LOGICAL_NOT_OPERATOR || ((LA661_0 >= LONG_NUMBER) && (LA661_0 <= MINUS_OPERATOR)) || LA661_0 == PLUS_OPERATOR || ((LA661_0 >= ULONGLONG_NUMBER) && (LA661_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt661=1;
            	                }
            	            }
            	            switch (alt661)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2002:20: expression
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11650);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletrim_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_trim_function11653);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11655);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:2003:5: TRAILING_SYMBOL ( expression )? FROM_SYMBOL expression
            	    {
            	         MATCHT(TRAILING_SYMBOL, &FOLLOW_TRAILING_SYMBOL_in_trim_function11661);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2003:21: ( expression )?
            	        {
            	            int alt662=2;
            	            {
            	                int LA662_0 = LA(1);
            	                if ( (((LA662_0 >= ACCOUNT_SYMBOL) && (LA662_0 <= YEAR_SYMBOL)) || LA662_0 == BINARY_SYMBOL || ((LA662_0 >= CASE_SYMBOL) && (LA662_0 <= CAST_SYMBOL)) || LA662_0 == CHAR_SYMBOL || ((LA662_0 >= CONVERT_SYMBOL) && (LA662_0 <= COUNT_SYMBOL)) || LA662_0 == CURDATE_SYMBOL || LA662_0 == CURRENT_USER_SYMBOL || LA662_0 == CURTIME_SYMBOL || LA662_0 == DATABASE_SYMBOL || LA662_0 == DATE_ADD_SYMBOL || LA662_0 == DATE_SUB_SYMBOL || LA662_0 == EXISTS_SYMBOL || ((LA662_0 >= EXTRACT_SYMBOL) && (LA662_0 <= FALSE_SYMBOL)) || LA662_0 == GROUP_CONCAT_SYMBOL || LA662_0 == IF_SYMBOL || LA662_0 == INSERT_SYMBOL || LA662_0 == INTERVAL_SYMBOL || LA662_0 == LEFT_SYMBOL || LA662_0 == MATCH_SYMBOL || LA662_0 == MAX_SYMBOL || LA662_0 == MIN_SYMBOL || LA662_0 == MOD_SYMBOL || LA662_0 == NCHAR_TEXT || ((LA662_0 >= NOT2_SYMBOL) && (LA662_0 <= NOW_SYMBOL)) || ((LA662_0 >= NULL2_SYMBOL) && (LA662_0 <= NULL_SYMBOL)) || ((LA662_0 >= OPEN_CURLY_SYMBOL) && (LA662_0 <= OPEN_PAR_SYMBOL)) || ((LA662_0 >= PARAM_MARKER) && (LA662_0 <= POSITION_SYMBOL)) || ((LA662_0 >= REPEAT_SYMBOL) && (LA662_0 <= REPLACE_SYMBOL)) || LA662_0 == RIGHT_SYMBOL || LA662_0 == SINGLE_QUOTED_TEXT || LA662_0 == STDDEV_SAMP_SYMBOL || LA662_0 == STD_SYMBOL || LA662_0 == SUBSTRING_SYMBOL || ((LA662_0 >= SUM_SYMBOL) && (LA662_0 <= SYSDATE_SYMBOL)) || ((LA662_0 >= TRIM_SYMBOL) && (LA662_0 <= TRUE_SYMBOL)) || ((LA662_0 >= UTC_DATE_SYMBOL) && (LA662_0 <= UTC_TIME_SYMBOL)) || LA662_0 == VALUES_SYMBOL || LA662_0 == VARIANCE_SYMBOL || LA662_0 == VAR_SAMP_SYMBOL || ((LA662_0 >= AT_AT_SIGN_SYMBOL) && (LA662_0 <= AT_TEXT_SUFFIX)) || ((LA662_0 >= BACK_TICK_QUOTED_ID) && (LA662_0 <= BIN_NUMBER)) || LA662_0 == BITWISE_NOT_OPERATOR || ((LA662_0 >= BIT_AND_SYMBOL) && (LA662_0 <= BIT_XOR_SYMBOL)) || LA662_0 == DECIMAL_NUMBER || LA662_0 == DOT_SYMBOL || LA662_0 == DOUBLE_QUOTED_TEXT || LA662_0 == FLOAT_NUMBER || ((LA662_0 >= HEX_NUMBER) && (LA662_0 <= IDENTIFIER)) || LA662_0 == INT_NUMBER || LA662_0 == LOGICAL_NOT_OPERATOR || ((LA662_0 >= LONG_NUMBER) && (LA662_0 <= MINUS_OPERATOR)) || LA662_0 == PLUS_OPERATOR || ((LA662_0 >= ULONGLONG_NUMBER) && (LA662_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt662=1;
            	                }
            	            }
            	            switch (alt662)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2003:21: expression
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11663);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletrim_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_trim_function11666);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11668);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:2004:5: BOTH_SYMBOL ( expression )? FROM_SYMBOL expression
            	    {
            	         MATCHT(BOTH_SYMBOL, &FOLLOW_BOTH_SYMBOL_in_trim_function11674);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2004:17: ( expression )?
            	        {
            	            int alt663=2;
            	            {
            	                int LA663_0 = LA(1);
            	                if ( (((LA663_0 >= ACCOUNT_SYMBOL) && (LA663_0 <= YEAR_SYMBOL)) || LA663_0 == BINARY_SYMBOL || ((LA663_0 >= CASE_SYMBOL) && (LA663_0 <= CAST_SYMBOL)) || LA663_0 == CHAR_SYMBOL || ((LA663_0 >= CONVERT_SYMBOL) && (LA663_0 <= COUNT_SYMBOL)) || LA663_0 == CURDATE_SYMBOL || LA663_0 == CURRENT_USER_SYMBOL || LA663_0 == CURTIME_SYMBOL || LA663_0 == DATABASE_SYMBOL || LA663_0 == DATE_ADD_SYMBOL || LA663_0 == DATE_SUB_SYMBOL || LA663_0 == EXISTS_SYMBOL || ((LA663_0 >= EXTRACT_SYMBOL) && (LA663_0 <= FALSE_SYMBOL)) || LA663_0 == GROUP_CONCAT_SYMBOL || LA663_0 == IF_SYMBOL || LA663_0 == INSERT_SYMBOL || LA663_0 == INTERVAL_SYMBOL || LA663_0 == LEFT_SYMBOL || LA663_0 == MATCH_SYMBOL || LA663_0 == MAX_SYMBOL || LA663_0 == MIN_SYMBOL || LA663_0 == MOD_SYMBOL || LA663_0 == NCHAR_TEXT || ((LA663_0 >= NOT2_SYMBOL) && (LA663_0 <= NOW_SYMBOL)) || ((LA663_0 >= NULL2_SYMBOL) && (LA663_0 <= NULL_SYMBOL)) || ((LA663_0 >= OPEN_CURLY_SYMBOL) && (LA663_0 <= OPEN_PAR_SYMBOL)) || ((LA663_0 >= PARAM_MARKER) && (LA663_0 <= POSITION_SYMBOL)) || ((LA663_0 >= REPEAT_SYMBOL) && (LA663_0 <= REPLACE_SYMBOL)) || LA663_0 == RIGHT_SYMBOL || LA663_0 == SINGLE_QUOTED_TEXT || LA663_0 == STDDEV_SAMP_SYMBOL || LA663_0 == STD_SYMBOL || LA663_0 == SUBSTRING_SYMBOL || ((LA663_0 >= SUM_SYMBOL) && (LA663_0 <= SYSDATE_SYMBOL)) || ((LA663_0 >= TRIM_SYMBOL) && (LA663_0 <= TRUE_SYMBOL)) || ((LA663_0 >= UTC_DATE_SYMBOL) && (LA663_0 <= UTC_TIME_SYMBOL)) || LA663_0 == VALUES_SYMBOL || LA663_0 == VARIANCE_SYMBOL || LA663_0 == VAR_SAMP_SYMBOL || ((LA663_0 >= AT_AT_SIGN_SYMBOL) && (LA663_0 <= AT_TEXT_SUFFIX)) || ((LA663_0 >= BACK_TICK_QUOTED_ID) && (LA663_0 <= BIN_NUMBER)) || LA663_0 == BITWISE_NOT_OPERATOR || ((LA663_0 >= BIT_AND_SYMBOL) && (LA663_0 <= BIT_XOR_SYMBOL)) || LA663_0 == DECIMAL_NUMBER || LA663_0 == DOT_SYMBOL || LA663_0 == DOUBLE_QUOTED_TEXT || LA663_0 == FLOAT_NUMBER || ((LA663_0 >= HEX_NUMBER) && (LA663_0 <= IDENTIFIER)) || LA663_0 == INT_NUMBER || LA663_0 == LOGICAL_NOT_OPERATOR || ((LA663_0 >= LONG_NUMBER) && (LA663_0 <= MINUS_OPERATOR)) || LA663_0 == PLUS_OPERATOR || ((LA663_0 >= ULONGLONG_NUMBER) && (LA663_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt663=1;
            	                }
            	            }
            	            switch (alt663)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2004:17: expression
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11676);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletrim_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_trim_function11679);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_trim_function11681);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletrim_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_trim_function11687);
            if  (HASEXCEPTION())
            {
                goto ruletrim_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletrim_functionEx; /* Prevent compiler warnings */
    ruletrim_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end trim_function */

/**
 * $ANTLR start substring_function
 * MySQLSimpleParser.g:2009:1: substring_function : SUBSTRING_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? | FROM_SYMBOL expression ( FOR_SYMBOL expression )? ) CLOSE_PAR_SYMBOL ;
 */
static void
substring_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2009:19: ( SUBSTRING_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? | FROM_SYMBOL expression ( FOR_SYMBOL expression )? ) CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2010:2: SUBSTRING_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? | FROM_SYMBOL expression ( FOR_SYMBOL expression )? ) CLOSE_PAR_SYMBOL
        {
             MATCHT(SUBSTRING_SYMBOL, &FOLLOW_SUBSTRING_SYMBOL_in_substring_function11696);
            if  (HASEXCEPTION())
            {
                goto rulesubstring_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_substring_function11698);
            if  (HASEXCEPTION())
            {
                goto rulesubstring_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_substring_function11700);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesubstring_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2011:2: ( COMMA_SYMBOL expression ( COMMA_SYMBOL expression )? | FROM_SYMBOL expression ( FOR_SYMBOL expression )? )
            {
                int alt667=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt667=1;
                	}
                    break;
                case FROM_SYMBOL:
                	{
                		alt667=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 667;
                    EXCEPTION->state        = 0;


                    goto rulesubstring_functionEx;

                }

                switch (alt667)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2012:3: COMMA_SYMBOL expression ( COMMA_SYMBOL expression )?
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_substring_function11707);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesubstring_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_substring_function11709);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesubstring_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2012:27: ( COMMA_SYMBOL expression )?
            	        {
            	            int alt665=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt665=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt665)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2012:28: COMMA_SYMBOL expression
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_substring_function11712);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesubstring_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_expression_in_substring_function11714);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesubstring_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2013:5: FROM_SYMBOL expression ( FOR_SYMBOL expression )?
            	    {
            	         MATCHT(FROM_SYMBOL, &FOLLOW_FROM_SYMBOL_in_substring_function11722);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesubstring_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_substring_function11724);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesubstring_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2013:28: ( FOR_SYMBOL expression )?
            	        {
            	            int alt666=2;
            	            switch ( LA(1) )
            	            {
            	                case FOR_SYMBOL:
            	                	{
            	                		alt666=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt666)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2013:29: FOR_SYMBOL expression
            	        	    {
            	        	         MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_substring_function11727);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesubstring_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_expression_in_substring_function11729);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesubstring_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_substring_function11737);
            if  (HASEXCEPTION())
            {
                goto rulesubstring_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesubstring_functionEx; /* Prevent compiler warnings */
    rulesubstring_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end substring_function */

/**
 * $ANTLR start geometry_function
 * MySQLSimpleParser.g:2018:1: geometry_function : ( GEOMETRYCOLLECTION_SYMBOL optional_expression_list_with_parentheses | LINESTRING_SYMBOL expression_list_with_parentheses | MULTILINESTRING_SYMBOL expression_list_with_parentheses | MULTIPOINT_SYMBOL expression_list_with_parentheses | MULTIPOLYGON_SYMBOL expression_list_with_parentheses | POINT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | POLYGON_SYMBOL expression_list_with_parentheses |{...}? CONTAINS_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL );
 */
static void
geometry_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2018:18: ( GEOMETRYCOLLECTION_SYMBOL optional_expression_list_with_parentheses | LINESTRING_SYMBOL expression_list_with_parentheses | MULTILINESTRING_SYMBOL expression_list_with_parentheses | MULTIPOINT_SYMBOL expression_list_with_parentheses | MULTIPOLYGON_SYMBOL expression_list_with_parentheses | POINT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL | POLYGON_SYMBOL expression_list_with_parentheses |{...}? CONTAINS_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt668;

            alt668=8;

            switch ( LA(1) )
            {
            case GEOMETRYCOLLECTION_SYMBOL:
            	{
            		alt668=1;
            	}
                break;
            case LINESTRING_SYMBOL:
            	{
            		alt668=2;
            	}
                break;
            case MULTILINESTRING_SYMBOL:
            	{
            		alt668=3;
            	}
                break;
            case MULTIPOINT_SYMBOL:
            	{
            		alt668=4;
            	}
                break;
            case MULTIPOLYGON_SYMBOL:
            	{
            		alt668=5;
            	}
                break;
            case POINT_SYMBOL:
            	{
            		alt668=6;
            	}
                break;
            case POLYGON_SYMBOL:
            	{
            		alt668=7;
            	}
                break;
            case CONTAINS_SYMBOL:
            	{
            		alt668=8;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 668;
                EXCEPTION->state        = 0;


                goto rulegeometry_functionEx;

            }

            switch (alt668)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2019:2: GEOMETRYCOLLECTION_SYMBOL optional_expression_list_with_parentheses
        	    {
        	         MATCHT(GEOMETRYCOLLECTION_SYMBOL, &FOLLOW_GEOMETRYCOLLECTION_SYMBOL_in_geometry_function11747);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_optional_expression_list_with_parentheses_in_geometry_function11749);
        	        optional_expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2020:4: LINESTRING_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(LINESTRING_SYMBOL, &FOLLOW_LINESTRING_SYMBOL_in_geometry_function11754);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_geometry_function11756);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2021:4: MULTILINESTRING_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(MULTILINESTRING_SYMBOL, &FOLLOW_MULTILINESTRING_SYMBOL_in_geometry_function11761);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_geometry_function11763);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2022:4: MULTIPOINT_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(MULTIPOINT_SYMBOL, &FOLLOW_MULTIPOINT_SYMBOL_in_geometry_function11768);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_geometry_function11770);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2023:4: MULTIPOLYGON_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(MULTIPOLYGON_SYMBOL, &FOLLOW_MULTIPOLYGON_SYMBOL_in_geometry_function11775);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_geometry_function11777);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2024:4: POINT_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(POINT_SYMBOL, &FOLLOW_POINT_SYMBOL_in_geometry_function11782);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11784);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_geometry_function11786);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_geometry_function11788);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_geometry_function11790);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11792);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2025:4: POLYGON_SYMBOL expression_list_with_parentheses
        	    {
        	         MATCHT(POLYGON_SYMBOL, &FOLLOW_POLYGON_SYMBOL_in_geometry_function11797);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_list_with_parentheses_in_geometry_function11799);
        	        expression_list_with_parentheses(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2026:4: {...}? CONTAINS_SYMBOL OPEN_PAR_SYMBOL expression COMMA_SYMBOL expression CLOSE_PAR_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION < 50706)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50706";
        	                EXCEPTION->ruleName	 = (void *)"geometry_function";


        	        }

        	         MATCHT(CONTAINS_SYMBOL, &FOLLOW_CONTAINS_SYMBOL_in_geometry_function11806);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_geometry_function11808);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_geometry_function11810);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_geometry_function11812);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_geometry_function11814);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_geometry_function11816);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegeometry_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulegeometry_functionEx; /* Prevent compiler warnings */
    rulegeometry_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end geometry_function */

/**
 * $ANTLR start aggregate_function
 * MySQLSimpleParser.g:2029:1: aggregate_function : ( AVG_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_AND_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_OR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_XOR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | count_function | MIN_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | MAX_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | STD_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | VARIANCE_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | STDDEV_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | VAR_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | SUM_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | GROUP_CONCAT_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? expression_list ( order_by_clause )? ( SEPARATOR_SYMBOL text_string )? CLOSE_PAR_SYMBOL );
 */
static void
aggregate_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2029:19: ( AVG_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_AND_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_OR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | BIT_XOR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | count_function | MIN_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | MAX_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | STD_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | VARIANCE_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | STDDEV_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | VAR_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL | SUM_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL | GROUP_CONCAT_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? expression_list ( order_by_clause )? ( SEPARATOR_SYMBOL text_string )? CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt676;

            alt676=13;

            switch ( LA(1) )
            {
            case AVG_SYMBOL:
            	{
            		alt676=1;
            	}
                break;
            case BIT_AND_SYMBOL:
            	{
            		alt676=2;
            	}
                break;
            case BIT_OR_SYMBOL:
            	{
            		alt676=3;
            	}
                break;
            case BIT_XOR_SYMBOL:
            	{
            		alt676=4;
            	}
                break;
            case COUNT_SYMBOL:
            	{
            		alt676=5;
            	}
                break;
            case MIN_SYMBOL:
            	{
            		alt676=6;
            	}
                break;
            case MAX_SYMBOL:
            	{
            		alt676=7;
            	}
                break;
            case STD_SYMBOL:
            	{
            		alt676=8;
            	}
                break;
            case VARIANCE_SYMBOL:
            	{
            		alt676=9;
            	}
                break;
            case STDDEV_SAMP_SYMBOL:
            	{
            		alt676=10;
            	}
                break;
            case VAR_SAMP_SYMBOL:
            	{
            		alt676=11;
            	}
                break;
            case SUM_SYMBOL:
            	{
            		alt676=12;
            	}
                break;
            case GROUP_CONCAT_SYMBOL:
            	{
            		alt676=13;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 676;
                EXCEPTION->state        = 0;


                goto ruleaggregate_functionEx;

            }

            switch (alt676)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2030:2: AVG_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(AVG_SYMBOL, &FOLLOW_AVG_SYMBOL_in_aggregate_function11825);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11827);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2030:29: ( DISTINCT_SYMBOL )?
        	        {
        	            int alt669=2;
        	            switch ( LA(1) )
        	            {
        	                case DISTINCT_SYMBOL:
        	                	{
        	                		alt669=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt669)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2030:29: DISTINCT_SYMBOL
        	        	    {
        	        	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11829);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11832);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11834);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2031:4: BIT_AND_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(BIT_AND_SYMBOL, &FOLLOW_BIT_AND_SYMBOL_in_aggregate_function11839);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11841);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11843);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11845);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2032:4: BIT_OR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(BIT_OR_SYMBOL, &FOLLOW_BIT_OR_SYMBOL_in_aggregate_function11850);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11852);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11854);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11856);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2033:4: BIT_XOR_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(BIT_XOR_SYMBOL, &FOLLOW_BIT_XOR_SYMBOL_in_aggregate_function11861);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11863);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11865);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11867);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2034:4: count_function
        	    {
        	        FOLLOWPUSH(FOLLOW_count_function_in_aggregate_function11872);
        	        count_function(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2035:4: MIN_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MIN_SYMBOL, &FOLLOW_MIN_SYMBOL_in_aggregate_function11877);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11879);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2035:31: ( DISTINCT_SYMBOL )?
        	        {
        	            int alt670=2;
        	            switch ( LA(1) )
        	            {
        	                case DISTINCT_SYMBOL:
        	                	{
        	                		alt670=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt670)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2035:31: DISTINCT_SYMBOL
        	        	    {
        	        	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11881);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11884);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11886);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2036:4: MAX_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(MAX_SYMBOL, &FOLLOW_MAX_SYMBOL_in_aggregate_function11891);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11893);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2036:31: ( DISTINCT_SYMBOL )?
        	        {
        	            int alt671=2;
        	            switch ( LA(1) )
        	            {
        	                case DISTINCT_SYMBOL:
        	                	{
        	                		alt671=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt671)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2036:31: DISTINCT_SYMBOL
        	        	    {
        	        	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11895);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11898);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11900);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2037:4: STD_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(STD_SYMBOL, &FOLLOW_STD_SYMBOL_in_aggregate_function11905);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11907);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11909);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11911);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2038:4: VARIANCE_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(VARIANCE_SYMBOL, &FOLLOW_VARIANCE_SYMBOL_in_aggregate_function11916);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11918);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11920);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11922);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2039:4: STDDEV_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(STDDEV_SAMP_SYMBOL, &FOLLOW_STDDEV_SAMP_SYMBOL_in_aggregate_function11927);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11929);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11931);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11933);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2040:4: VAR_SAMP_SYMBOL OPEN_PAR_SYMBOL in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(VAR_SAMP_SYMBOL, &FOLLOW_VAR_SAMP_SYMBOL_in_aggregate_function11938);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11940);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11942);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11944);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2041:4: SUM_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? in_aggregate_expression CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(SUM_SYMBOL, &FOLLOW_SUM_SYMBOL_in_aggregate_function11949);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11951);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2041:31: ( DISTINCT_SYMBOL )?
        	        {
        	            int alt672=2;
        	            switch ( LA(1) )
        	            {
        	                case DISTINCT_SYMBOL:
        	                	{
        	                		alt672=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt672)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2041:31: DISTINCT_SYMBOL
        	        	    {
        	        	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11953);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_aggregate_function11956);
        	        in_aggregate_expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11958);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:2042:4: GROUP_CONCAT_SYMBOL OPEN_PAR_SYMBOL ( DISTINCT_SYMBOL )? expression_list ( order_by_clause )? ( SEPARATOR_SYMBOL text_string )? CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(GROUP_CONCAT_SYMBOL, &FOLLOW_GROUP_CONCAT_SYMBOL_in_aggregate_function11963);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_aggregate_function11965);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2042:40: ( DISTINCT_SYMBOL )?
        	        {
        	            int alt673=2;
        	            switch ( LA(1) )
        	            {
        	                case DISTINCT_SYMBOL:
        	                	{
        	                		alt673=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt673)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2042:40: DISTINCT_SYMBOL
        	        	    {
        	        	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_aggregate_function11967);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_expression_list_in_aggregate_function11970);
        	        expression_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2042:73: ( order_by_clause )?
        	        {
        	            int alt674=2;
        	            switch ( LA(1) )
        	            {
        	                case ORDER_SYMBOL:
        	                	{
        	                		alt674=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt674)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2042:73: order_by_clause
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_order_by_clause_in_aggregate_function11972);
        	        	        order_by_clause(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2042:90: ( SEPARATOR_SYMBOL text_string )?
        	        {
        	            int alt675=2;
        	            switch ( LA(1) )
        	            {
        	                case SEPARATOR_SYMBOL:
        	                	{
        	                		alt675=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt675)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2042:91: SEPARATOR_SYMBOL text_string
        	        	    {
        	        	         MATCHT(SEPARATOR_SYMBOL, &FOLLOW_SEPARATOR_SYMBOL_in_aggregate_function11976);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_string_in_aggregate_function11978);
        	        	        text_string(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleaggregate_functionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_aggregate_function11982);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleaggregate_functionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleaggregate_functionEx; /* Prevent compiler warnings */
    ruleaggregate_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end aggregate_function */

/**
 * $ANTLR start in_aggregate_expression
 * MySQLSimpleParser.g:2045:1: in_aggregate_expression : ( ALL_SYMBOL )? expression ;
 */
static void
in_aggregate_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2045:24: ( ( ALL_SYMBOL )? expression )
        // MySQLSimpleParser.g:2046:2: ( ALL_SYMBOL )? expression
        {
            // MySQLSimpleParser.g:2046:2: ( ALL_SYMBOL )?
            {
                int alt677=2;
                switch ( LA(1) )
                {
                    case ALL_SYMBOL:
                    	{
                    		alt677=1;
                    	}
                        break;
                }

                switch (alt677)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2046:2: ALL_SYMBOL
            	    {
            	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_in_aggregate_expression11991);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulein_aggregate_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_expression_in_in_aggregate_expression11994);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulein_aggregate_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulein_aggregate_expressionEx; /* Prevent compiler warnings */
    rulein_aggregate_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end in_aggregate_expression */

/**
 * $ANTLR start count_function
 * MySQLSimpleParser.g:2049:1: count_function : COUNT_SYMBOL OPEN_PAR_SYMBOL ( ( ALL_SYMBOL )? MULT_OPERATOR | in_aggregate_expression | DISTINCT_SYMBOL expression_list ) CLOSE_PAR_SYMBOL ;
 */
static void
count_function(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2049:15: ( COUNT_SYMBOL OPEN_PAR_SYMBOL ( ( ALL_SYMBOL )? MULT_OPERATOR | in_aggregate_expression | DISTINCT_SYMBOL expression_list ) CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2050:2: COUNT_SYMBOL OPEN_PAR_SYMBOL ( ( ALL_SYMBOL )? MULT_OPERATOR | in_aggregate_expression | DISTINCT_SYMBOL expression_list ) CLOSE_PAR_SYMBOL
        {
             MATCHT(COUNT_SYMBOL, &FOLLOW_COUNT_SYMBOL_in_count_function12003);
            if  (HASEXCEPTION())
            {
                goto rulecount_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_count_function12005);
            if  (HASEXCEPTION())
            {
                goto rulecount_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2051:2: ( ( ALL_SYMBOL )? MULT_OPERATOR | in_aggregate_expression | DISTINCT_SYMBOL expression_list )
            {
                int alt679=3;
                {
                    int LA679_0 = LA(1);
                    if ( (LA679_0 == ALL_SYMBOL))
                    {
                        {
                            int LA679_1 = LA(2);
                            if ( (LA679_1 == MULT_OPERATOR))
                            {
                                alt679=1;
                            }
                            else if ( (((LA679_1 >= ACCOUNT_SYMBOL) && (LA679_1 <= YEAR_SYMBOL)) || LA679_1 == BINARY_SYMBOL || ((LA679_1 >= CASE_SYMBOL) && (LA679_1 <= CAST_SYMBOL)) || LA679_1 == CHAR_SYMBOL || ((LA679_1 >= CONVERT_SYMBOL) && (LA679_1 <= COUNT_SYMBOL)) || LA679_1 == CURDATE_SYMBOL || LA679_1 == CURRENT_USER_SYMBOL || LA679_1 == CURTIME_SYMBOL || LA679_1 == DATABASE_SYMBOL || LA679_1 == DATE_ADD_SYMBOL || LA679_1 == DATE_SUB_SYMBOL || LA679_1 == EXISTS_SYMBOL || ((LA679_1 >= EXTRACT_SYMBOL) && (LA679_1 <= FALSE_SYMBOL)) || LA679_1 == GROUP_CONCAT_SYMBOL || LA679_1 == IF_SYMBOL || LA679_1 == INSERT_SYMBOL || LA679_1 == INTERVAL_SYMBOL || LA679_1 == LEFT_SYMBOL || LA679_1 == MATCH_SYMBOL || LA679_1 == MAX_SYMBOL || LA679_1 == MIN_SYMBOL || LA679_1 == MOD_SYMBOL || LA679_1 == NCHAR_TEXT || ((LA679_1 >= NOT2_SYMBOL) && (LA679_1 <= NOW_SYMBOL)) || ((LA679_1 >= NULL2_SYMBOL) && (LA679_1 <= NULL_SYMBOL)) || ((LA679_1 >= OPEN_CURLY_SYMBOL) && (LA679_1 <= OPEN_PAR_SYMBOL)) || ((LA679_1 >= PARAM_MARKER) && (LA679_1 <= POSITION_SYMBOL)) || ((LA679_1 >= REPEAT_SYMBOL) && (LA679_1 <= REPLACE_SYMBOL)) || LA679_1 == RIGHT_SYMBOL || LA679_1 == SINGLE_QUOTED_TEXT || LA679_1 == STDDEV_SAMP_SYMBOL || LA679_1 == STD_SYMBOL || LA679_1 == SUBSTRING_SYMBOL || ((LA679_1 >= SUM_SYMBOL) && (LA679_1 <= SYSDATE_SYMBOL)) || ((LA679_1 >= TRIM_SYMBOL) && (LA679_1 <= TRUE_SYMBOL)) || ((LA679_1 >= UTC_DATE_SYMBOL) && (LA679_1 <= UTC_TIME_SYMBOL)) || LA679_1 == VALUES_SYMBOL || LA679_1 == VARIANCE_SYMBOL || LA679_1 == VAR_SAMP_SYMBOL || ((LA679_1 >= AT_AT_SIGN_SYMBOL) && (LA679_1 <= AT_TEXT_SUFFIX)) || ((LA679_1 >= BACK_TICK_QUOTED_ID) && (LA679_1 <= BIN_NUMBER)) || LA679_1 == BITWISE_NOT_OPERATOR || ((LA679_1 >= BIT_AND_SYMBOL) && (LA679_1 <= BIT_XOR_SYMBOL)) || LA679_1 == DECIMAL_NUMBER || LA679_1 == DOT_SYMBOL || LA679_1 == DOUBLE_QUOTED_TEXT || LA679_1 == FLOAT_NUMBER || ((LA679_1 >= HEX_NUMBER) && (LA679_1 <= IDENTIFIER)) || LA679_1 == INT_NUMBER || LA679_1 == LOGICAL_NOT_OPERATOR || ((LA679_1 >= LONG_NUMBER) && (LA679_1 <= MINUS_OPERATOR)) || LA679_1 == PLUS_OPERATOR || ((LA679_1 >= ULONGLONG_NUMBER) && (LA679_1 <= UNDERSCORE_CHARSET))))
                            {
                                alt679=2;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 679;
                                EXCEPTION->state        = 1;


                                goto rulecount_functionEx;

                            }
                        }
                    }
                    else if ( (LA679_0 == MULT_OPERATOR))
                    {
                        alt679=1;
                    }
                    else if ( (((LA679_0 >= ACCOUNT_SYMBOL) && (LA679_0 <= YEAR_SYMBOL)) || LA679_0 == BINARY_SYMBOL || ((LA679_0 >= CASE_SYMBOL) && (LA679_0 <= CAST_SYMBOL)) || LA679_0 == CHAR_SYMBOL || ((LA679_0 >= CONVERT_SYMBOL) && (LA679_0 <= COUNT_SYMBOL)) || LA679_0 == CURDATE_SYMBOL || LA679_0 == CURRENT_USER_SYMBOL || LA679_0 == CURTIME_SYMBOL || LA679_0 == DATABASE_SYMBOL || LA679_0 == DATE_ADD_SYMBOL || LA679_0 == DATE_SUB_SYMBOL || LA679_0 == EXISTS_SYMBOL || ((LA679_0 >= EXTRACT_SYMBOL) && (LA679_0 <= FALSE_SYMBOL)) || LA679_0 == GROUP_CONCAT_SYMBOL || LA679_0 == IF_SYMBOL || LA679_0 == INSERT_SYMBOL || LA679_0 == INTERVAL_SYMBOL || LA679_0 == LEFT_SYMBOL || LA679_0 == MATCH_SYMBOL || LA679_0 == MAX_SYMBOL || LA679_0 == MIN_SYMBOL || LA679_0 == MOD_SYMBOL || LA679_0 == NCHAR_TEXT || ((LA679_0 >= NOT2_SYMBOL) && (LA679_0 <= NOW_SYMBOL)) || ((LA679_0 >= NULL2_SYMBOL) && (LA679_0 <= NULL_SYMBOL)) || ((LA679_0 >= OPEN_CURLY_SYMBOL) && (LA679_0 <= OPEN_PAR_SYMBOL)) || ((LA679_0 >= PARAM_MARKER) && (LA679_0 <= POSITION_SYMBOL)) || ((LA679_0 >= REPEAT_SYMBOL) && (LA679_0 <= REPLACE_SYMBOL)) || LA679_0 == RIGHT_SYMBOL || LA679_0 == SINGLE_QUOTED_TEXT || LA679_0 == STDDEV_SAMP_SYMBOL || LA679_0 == STD_SYMBOL || LA679_0 == SUBSTRING_SYMBOL || ((LA679_0 >= SUM_SYMBOL) && (LA679_0 <= SYSDATE_SYMBOL)) || ((LA679_0 >= TRIM_SYMBOL) && (LA679_0 <= TRUE_SYMBOL)) || ((LA679_0 >= UTC_DATE_SYMBOL) && (LA679_0 <= UTC_TIME_SYMBOL)) || LA679_0 == VALUES_SYMBOL || LA679_0 == VARIANCE_SYMBOL || LA679_0 == VAR_SAMP_SYMBOL || ((LA679_0 >= AT_AT_SIGN_SYMBOL) && (LA679_0 <= AT_TEXT_SUFFIX)) || ((LA679_0 >= BACK_TICK_QUOTED_ID) && (LA679_0 <= BIN_NUMBER)) || LA679_0 == BITWISE_NOT_OPERATOR || ((LA679_0 >= BIT_AND_SYMBOL) && (LA679_0 <= BIT_XOR_SYMBOL)) || LA679_0 == DECIMAL_NUMBER || LA679_0 == DOT_SYMBOL || LA679_0 == DOUBLE_QUOTED_TEXT || LA679_0 == FLOAT_NUMBER || ((LA679_0 >= HEX_NUMBER) && (LA679_0 <= IDENTIFIER)) || LA679_0 == INT_NUMBER || LA679_0 == LOGICAL_NOT_OPERATOR || ((LA679_0 >= LONG_NUMBER) && (LA679_0 <= MINUS_OPERATOR)) || LA679_0 == PLUS_OPERATOR || ((LA679_0 >= ULONGLONG_NUMBER) && (LA679_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt679=2;
                    }
                    else if ( (LA679_0 == DISTINCT_SYMBOL))
                    {
                        alt679=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 679;
                        EXCEPTION->state        = 0;


                        goto rulecount_functionEx;

                    }
                }
                switch (alt679)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2052:3: ( ALL_SYMBOL )? MULT_OPERATOR
            	    {
            	        // MySQLSimpleParser.g:2052:3: ( ALL_SYMBOL )?
            	        {
            	            int alt678=2;
            	            switch ( LA(1) )
            	            {
            	                case ALL_SYMBOL:
            	                	{
            	                		alt678=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt678)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2052:3: ALL_SYMBOL
            	        	    {
            	        	         MATCHT(ALL_SYMBOL, &FOLLOW_ALL_SYMBOL_in_count_function12012);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecount_functionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_count_function12015);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecount_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2053:5: in_aggregate_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_in_aggregate_expression_in_count_function12021);
            	        in_aggregate_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecount_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:2054:5: DISTINCT_SYMBOL expression_list
            	    {
            	         MATCHT(DISTINCT_SYMBOL, &FOLLOW_DISTINCT_SYMBOL_in_count_function12027);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecount_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_list_in_count_function12029);
            	        expression_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecount_functionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_count_function12035);
            if  (HASEXCEPTION())
            {
                goto rulecount_functionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecount_functionEx; /* Prevent compiler warnings */
    rulecount_functionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end count_function */

/**
 * $ANTLR start function_call
 * MySQLSimpleParser.g:2059:1: function_call : function_call_expression ;
 */
static void
function_call(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2059:14: ( function_call_expression )
        // MySQLSimpleParser.g:2060:2: function_call_expression
        {
            FOLLOWPUSH(FOLLOW_function_call_expression_in_function_call12044);
            function_call_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_callEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefunction_callEx; /* Prevent compiler warnings */
    rulefunction_callEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_call */

/**
 * $ANTLR start function_call_expression
 * MySQLSimpleParser.g:2063:1: function_call_expression : pure_identifier ( OPEN_PAR_SYMBOL ( aliased_expression_list )? CLOSE_PAR_SYMBOL | dot_identifier OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL ) ;
 */
static void
function_call_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2063:25: ( pure_identifier ( OPEN_PAR_SYMBOL ( aliased_expression_list )? CLOSE_PAR_SYMBOL | dot_identifier OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL ) )
        // MySQLSimpleParser.g:2064:2: pure_identifier ( OPEN_PAR_SYMBOL ( aliased_expression_list )? CLOSE_PAR_SYMBOL | dot_identifier OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL )
        {
            FOLLOWPUSH(FOLLOW_pure_identifier_in_function_call_expression12053);
            pure_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_call_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2065:2: ( OPEN_PAR_SYMBOL ( aliased_expression_list )? CLOSE_PAR_SYMBOL | dot_identifier OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL )
            {
                int alt682=2;
                switch ( LA(1) )
                {
                case OPEN_PAR_SYMBOL:
                	{
                		alt682=1;
                	}
                    break;
                case DOT_SYMBOL:
                	{
                		alt682=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 682;
                    EXCEPTION->state        = 0;


                    goto rulefunction_call_expressionEx;

                }

                switch (alt682)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2066:3: OPEN_PAR_SYMBOL ( aliased_expression_list )? CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12060);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefunction_call_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2066:19: ( aliased_expression_list )?
            	        {
            	            int alt680=2;
            	            {
            	                int LA680_0 = LA(1);
            	                if ( (((LA680_0 >= ACCOUNT_SYMBOL) && (LA680_0 <= YEAR_SYMBOL)) || LA680_0 == BINARY_SYMBOL || ((LA680_0 >= CASE_SYMBOL) && (LA680_0 <= CAST_SYMBOL)) || LA680_0 == CHAR_SYMBOL || ((LA680_0 >= CONVERT_SYMBOL) && (LA680_0 <= COUNT_SYMBOL)) || LA680_0 == CURDATE_SYMBOL || LA680_0 == CURRENT_USER_SYMBOL || LA680_0 == CURTIME_SYMBOL || LA680_0 == DATABASE_SYMBOL || LA680_0 == DATE_ADD_SYMBOL || LA680_0 == DATE_SUB_SYMBOL || LA680_0 == EXISTS_SYMBOL || ((LA680_0 >= EXTRACT_SYMBOL) && (LA680_0 <= FALSE_SYMBOL)) || LA680_0 == GROUP_CONCAT_SYMBOL || LA680_0 == IF_SYMBOL || LA680_0 == INSERT_SYMBOL || LA680_0 == INTERVAL_SYMBOL || LA680_0 == LEFT_SYMBOL || LA680_0 == MATCH_SYMBOL || LA680_0 == MAX_SYMBOL || LA680_0 == MIN_SYMBOL || LA680_0 == MOD_SYMBOL || LA680_0 == NCHAR_TEXT || ((LA680_0 >= NOT2_SYMBOL) && (LA680_0 <= NOW_SYMBOL)) || ((LA680_0 >= NULL2_SYMBOL) && (LA680_0 <= NULL_SYMBOL)) || ((LA680_0 >= OPEN_CURLY_SYMBOL) && (LA680_0 <= OPEN_PAR_SYMBOL)) || ((LA680_0 >= PARAM_MARKER) && (LA680_0 <= POSITION_SYMBOL)) || ((LA680_0 >= REPEAT_SYMBOL) && (LA680_0 <= REPLACE_SYMBOL)) || LA680_0 == RIGHT_SYMBOL || LA680_0 == SINGLE_QUOTED_TEXT || LA680_0 == STDDEV_SAMP_SYMBOL || LA680_0 == STD_SYMBOL || LA680_0 == SUBSTRING_SYMBOL || ((LA680_0 >= SUM_SYMBOL) && (LA680_0 <= SYSDATE_SYMBOL)) || ((LA680_0 >= TRIM_SYMBOL) && (LA680_0 <= TRUE_SYMBOL)) || ((LA680_0 >= UTC_DATE_SYMBOL) && (LA680_0 <= UTC_TIME_SYMBOL)) || LA680_0 == VALUES_SYMBOL || LA680_0 == VARIANCE_SYMBOL || LA680_0 == VAR_SAMP_SYMBOL || ((LA680_0 >= AT_AT_SIGN_SYMBOL) && (LA680_0 <= AT_TEXT_SUFFIX)) || ((LA680_0 >= BACK_TICK_QUOTED_ID) && (LA680_0 <= BIN_NUMBER)) || LA680_0 == BITWISE_NOT_OPERATOR || ((LA680_0 >= BIT_AND_SYMBOL) && (LA680_0 <= BIT_XOR_SYMBOL)) || LA680_0 == DECIMAL_NUMBER || LA680_0 == DOT_SYMBOL || LA680_0 == DOUBLE_QUOTED_TEXT || LA680_0 == FLOAT_NUMBER || ((LA680_0 >= HEX_NUMBER) && (LA680_0 <= IDENTIFIER)) || LA680_0 == INT_NUMBER || LA680_0 == LOGICAL_NOT_OPERATOR || ((LA680_0 >= LONG_NUMBER) && (LA680_0 <= MINUS_OPERATOR)) || LA680_0 == PLUS_OPERATOR || ((LA680_0 >= ULONGLONG_NUMBER) && (LA680_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt680=1;
            	                }
            	            }
            	            switch (alt680)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2066:19: aliased_expression_list
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_aliased_expression_list_in_function_call_expression12062);
            	        	        aliased_expression_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulefunction_call_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12065);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefunction_call_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2067:5: dot_identifier OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL
            	    {
            	        FOLLOWPUSH(FOLLOW_dot_identifier_in_function_call_expression12072);
            	        dot_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefunction_call_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_function_call_expression12074);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefunction_call_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2067:36: ( expression_list )?
            	        {
            	            int alt681=2;
            	            {
            	                int LA681_0 = LA(1);
            	                if ( (((LA681_0 >= ACCOUNT_SYMBOL) && (LA681_0 <= YEAR_SYMBOL)) || LA681_0 == BINARY_SYMBOL || ((LA681_0 >= CASE_SYMBOL) && (LA681_0 <= CAST_SYMBOL)) || LA681_0 == CHAR_SYMBOL || ((LA681_0 >= CONVERT_SYMBOL) && (LA681_0 <= COUNT_SYMBOL)) || LA681_0 == CURDATE_SYMBOL || LA681_0 == CURRENT_USER_SYMBOL || LA681_0 == CURTIME_SYMBOL || LA681_0 == DATABASE_SYMBOL || LA681_0 == DATE_ADD_SYMBOL || LA681_0 == DATE_SUB_SYMBOL || LA681_0 == EXISTS_SYMBOL || ((LA681_0 >= EXTRACT_SYMBOL) && (LA681_0 <= FALSE_SYMBOL)) || LA681_0 == GROUP_CONCAT_SYMBOL || LA681_0 == IF_SYMBOL || LA681_0 == INSERT_SYMBOL || LA681_0 == INTERVAL_SYMBOL || LA681_0 == LEFT_SYMBOL || LA681_0 == MATCH_SYMBOL || LA681_0 == MAX_SYMBOL || LA681_0 == MIN_SYMBOL || LA681_0 == MOD_SYMBOL || LA681_0 == NCHAR_TEXT || ((LA681_0 >= NOT2_SYMBOL) && (LA681_0 <= NOW_SYMBOL)) || ((LA681_0 >= NULL2_SYMBOL) && (LA681_0 <= NULL_SYMBOL)) || ((LA681_0 >= OPEN_CURLY_SYMBOL) && (LA681_0 <= OPEN_PAR_SYMBOL)) || ((LA681_0 >= PARAM_MARKER) && (LA681_0 <= POSITION_SYMBOL)) || ((LA681_0 >= REPEAT_SYMBOL) && (LA681_0 <= REPLACE_SYMBOL)) || LA681_0 == RIGHT_SYMBOL || LA681_0 == SINGLE_QUOTED_TEXT || LA681_0 == STDDEV_SAMP_SYMBOL || LA681_0 == STD_SYMBOL || LA681_0 == SUBSTRING_SYMBOL || ((LA681_0 >= SUM_SYMBOL) && (LA681_0 <= SYSDATE_SYMBOL)) || ((LA681_0 >= TRIM_SYMBOL) && (LA681_0 <= TRUE_SYMBOL)) || ((LA681_0 >= UTC_DATE_SYMBOL) && (LA681_0 <= UTC_TIME_SYMBOL)) || LA681_0 == VALUES_SYMBOL || LA681_0 == VARIANCE_SYMBOL || LA681_0 == VAR_SAMP_SYMBOL || ((LA681_0 >= AT_AT_SIGN_SYMBOL) && (LA681_0 <= AT_TEXT_SUFFIX)) || ((LA681_0 >= BACK_TICK_QUOTED_ID) && (LA681_0 <= BIN_NUMBER)) || LA681_0 == BITWISE_NOT_OPERATOR || ((LA681_0 >= BIT_AND_SYMBOL) && (LA681_0 <= BIT_XOR_SYMBOL)) || LA681_0 == DECIMAL_NUMBER || LA681_0 == DOT_SYMBOL || LA681_0 == DOUBLE_QUOTED_TEXT || LA681_0 == FLOAT_NUMBER || ((LA681_0 >= HEX_NUMBER) && (LA681_0 <= IDENTIFIER)) || LA681_0 == INT_NUMBER || LA681_0 == LOGICAL_NOT_OPERATOR || ((LA681_0 >= LONG_NUMBER) && (LA681_0 <= MINUS_OPERATOR)) || LA681_0 == PLUS_OPERATOR || ((LA681_0 >= ULONGLONG_NUMBER) && (LA681_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt681=1;
            	                }
            	            }
            	            switch (alt681)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2067:36: expression_list
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_list_in_function_call_expression12076);
            	        	        expression_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulefunction_call_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_function_call_expression12079);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefunction_call_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulefunction_call_expressionEx; /* Prevent compiler warnings */
    rulefunction_call_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_call_expression */

/**
 * $ANTLR start aliased_expression_list
 * MySQLSimpleParser.g:2071:1: aliased_expression_list : aliased_expression ( COMMA_SYMBOL aliased_expression )* ;
 */
static void
aliased_expression_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2071:24: ( aliased_expression ( COMMA_SYMBOL aliased_expression )* )
        // MySQLSimpleParser.g:2072:2: aliased_expression ( COMMA_SYMBOL aliased_expression )*
        {
            FOLLOWPUSH(FOLLOW_aliased_expression_in_aliased_expression_list12092);
            aliased_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealiased_expression_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2072:21: ( COMMA_SYMBOL aliased_expression )*

            for (;;)
            {
                int alt683=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt683=1;
                	}
                    break;

                }

                switch (alt683)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2072:22: COMMA_SYMBOL aliased_expression
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_aliased_expression_list12095);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealiased_expression_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_aliased_expression_in_aliased_expression_list12097);
            	        aliased_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealiased_expression_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop683;	/* break out of the loop */
            	    break;
                }
            }
            loop683: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulealiased_expression_listEx; /* Prevent compiler warnings */
    rulealiased_expression_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end aliased_expression_list */

/**
 * $ANTLR start aliased_expression
 * MySQLSimpleParser.g:2075:1: aliased_expression : expression ( select_alias )? ;
 */
static void
aliased_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2075:19: ( expression ( select_alias )? )
        // MySQLSimpleParser.g:2076:2: expression ( select_alias )?
        {
            FOLLOWPUSH(FOLLOW_expression_in_aliased_expression12108);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulealiased_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2076:13: ( select_alias )?
            {
                int alt684=2;
                {
                    int LA684_0 = LA(1);
                    if ( (((LA684_0 >= ACCOUNT_SYMBOL) && (LA684_0 <= YEAR_SYMBOL)) || LA684_0 == SINGLE_QUOTED_TEXT || LA684_0 == AS_SYMBOL || ((LA684_0 >= BACK_TICK_QUOTED_ID) && (LA684_0 <= BIN_NUMBER)) || LA684_0 == DOUBLE_QUOTED_TEXT || ((LA684_0 >= HEX_NUMBER) && (LA684_0 <= IDENTIFIER))))
                    {
                        alt684=1;
                    }
                }
                switch (alt684)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2076:13: select_alias
            	    {
            	        FOLLOWPUSH(FOLLOW_select_alias_in_aliased_expression12110);
            	        select_alias(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulealiased_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealiased_expressionEx; /* Prevent compiler warnings */
    rulealiased_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end aliased_expression */

/**
 * $ANTLR start variable
 * MySQLSimpleParser.g:2079:1: variable options {greedy=true; } : ( user_variable ( ASSIGN_OPERATOR expression )? | system_variable );
 */
static void
variable(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2079:36: ( user_variable ( ASSIGN_OPERATOR expression )? | system_variable )

            ANTLR3_UINT32 alt686;

            alt686=2;

            switch ( LA(1) )
            {
            case AT_SIGN_SYMBOL:
            case AT_TEXT_SUFFIX:
            	{
            		alt686=1;
            	}
                break;
            case AT_AT_SIGN_SYMBOL:
            	{
            		alt686=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 686;
                EXCEPTION->state        = 0;


                goto rulevariableEx;

            }

            switch (alt686)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2080:2: user_variable ( ASSIGN_OPERATOR expression )?
        	    {
        	        FOLLOWPUSH(FOLLOW_user_variable_in_variable12131);
        	        user_variable(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevariableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2080:16: ( ASSIGN_OPERATOR expression )?
        	        {
        	            int alt685=2;
        	            switch ( LA(1) )
        	            {
        	                case ASSIGN_OPERATOR:
        	                	{
        	                		alt685=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt685)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2080:17: ASSIGN_OPERATOR expression
        	        	    {
        	        	         MATCHT(ASSIGN_OPERATOR, &FOLLOW_ASSIGN_OPERATOR_in_variable12134);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulevariableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_variable12136);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulevariableEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2081:4: system_variable
        	    {
        	        FOLLOWPUSH(FOLLOW_system_variable_in_variable12143);
        	        system_variable(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevariableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulevariableEx; /* Prevent compiler warnings */
    rulevariableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end variable */

/**
 * $ANTLR start user_variable
 * MySQLSimpleParser.g:2084:1: user_variable : ( ( AT_SIGN_SYMBOL text_or_identifier ) | AT_TEXT_SUFFIX );
 */
static void
user_variable(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2084:14: ( ( AT_SIGN_SYMBOL text_or_identifier ) | AT_TEXT_SUFFIX )

            ANTLR3_UINT32 alt687;

            alt687=2;

            switch ( LA(1) )
            {
            case AT_SIGN_SYMBOL:
            	{
            		alt687=1;
            	}
                break;
            case AT_TEXT_SUFFIX:
            	{
            		alt687=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 687;
                EXCEPTION->state        = 0;


                goto ruleuser_variableEx;

            }

            switch (alt687)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2085:2: ( AT_SIGN_SYMBOL text_or_identifier )
        	    {
        	        // MySQLSimpleParser.g:2085:2: ( AT_SIGN_SYMBOL text_or_identifier )
        	        // MySQLSimpleParser.g:2085:3: AT_SIGN_SYMBOL text_or_identifier
        	        {
        	             MATCHT(AT_SIGN_SYMBOL, &FOLLOW_AT_SIGN_SYMBOL_in_user_variable12153);
        	            if  (HASEXCEPTION())
        	            {
        	                goto ruleuser_variableEx;
        	            }
        	            if (HASFAILED())
        	            {
        	                return ;
        	            }


        	            FOLLOWPUSH(FOLLOW_text_or_identifier_in_user_variable12155);
        	            text_or_identifier(ctx);

        	            FOLLOWPOP();
        	            if  (HASEXCEPTION())
        	            {
        	                goto ruleuser_variableEx;
        	            }
        	            if (HASFAILED())
        	            {
        	                return ;
        	            }


        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2086:4: AT_TEXT_SUFFIX
        	    {
        	         MATCHT(AT_TEXT_SUFFIX, &FOLLOW_AT_TEXT_SUFFIX_in_user_variable12161);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleuser_variableEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleuser_variableEx; /* Prevent compiler warnings */
    ruleuser_variableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end user_variable */

/**
 * $ANTLR start system_variable
 * MySQLSimpleParser.g:2090:1: system_variable : AT_AT_SIGN_SYMBOL ( options {greedy=true; } : option_type DOT_SYMBOL )? variable_name ;
 */
static void
system_variable(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2090:16: ( AT_AT_SIGN_SYMBOL ( options {greedy=true; } : option_type DOT_SYMBOL )? variable_name )
        // MySQLSimpleParser.g:2091:2: AT_AT_SIGN_SYMBOL ( options {greedy=true; } : option_type DOT_SYMBOL )? variable_name
        {
             MATCHT(AT_AT_SIGN_SYMBOL, &FOLLOW_AT_AT_SIGN_SYMBOL_in_system_variable12171);
            if  (HASEXCEPTION())
            {
                goto rulesystem_variableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2091:20: ( options {greedy=true; } : option_type DOT_SYMBOL )?
            {
                int alt688=2;
                switch ( LA(1) )
                {
                    case GLOBAL_SYMBOL:
                    case LOCAL_SYMBOL:
                    case SESSION_SYMBOL:
                    	{
                    		switch ( LA(2) )
                    		{
                    		    case DOT_SYMBOL:
                    		    	{
                    		    		alt688=1;
                    		    	}
                    		        break;
                    		}

                    	}
                        break;
                }

                switch (alt688)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2091:50: option_type DOT_SYMBOL
            	    {
            	        FOLLOWPUSH(FOLLOW_option_type_in_system_variable12187);
            	        option_type(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesystem_variableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_system_variable12189);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesystem_variableEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_variable_name_in_system_variable12193);
            variable_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesystem_variableEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesystem_variableEx; /* Prevent compiler warnings */
    rulesystem_variableEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end system_variable */

/**
 * $ANTLR start variable_name
 * MySQLSimpleParser.g:2094:1: variable_name : ( identifier ( dot_identifier )? | DEFAULT_SYMBOL dot_identifier );
 */
static void
variable_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2094:14: ( identifier ( dot_identifier )? | DEFAULT_SYMBOL dot_identifier )

            ANTLR3_UINT32 alt690;

            alt690=2;

            {
                int LA690_0 = LA(1);
                if ( (((LA690_0 >= ACCOUNT_SYMBOL) && (LA690_0 <= YEAR_SYMBOL)) || LA690_0 == BACK_TICK_QUOTED_ID || LA690_0 == DOUBLE_QUOTED_TEXT || LA690_0 == IDENTIFIER))
                {
                    alt690=1;
                }
                else if ( (LA690_0 == DEFAULT_SYMBOL))
                {
                    alt690=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 690;
                    EXCEPTION->state        = 0;


                    goto rulevariable_nameEx;

                }
            }
            switch (alt690)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2095:2: identifier ( dot_identifier )?
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_variable_name12202);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevariable_nameEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2095:13: ( dot_identifier )?
        	        {
        	            int alt689=2;
        	            switch ( LA(1) )
        	            {
        	                case DOT_SYMBOL:
        	                	{
        	                		alt689=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt689)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2095:13: dot_identifier
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_variable_name12204);
        	        	        dot_identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulevariable_nameEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2096:4: DEFAULT_SYMBOL dot_identifier
        	    {
        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_variable_name12214);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevariable_nameEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_variable_name12216);
        	        dot_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevariable_nameEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulevariable_nameEx; /* Prevent compiler warnings */
    rulevariable_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end variable_name */

/**
 * $ANTLR start match_expression
 * MySQLSimpleParser.g:2099:1: match_expression : MATCH_SYMBOL OPEN_PAR_SYMBOL field_name_list CLOSE_PAR_SYMBOL AGAINST_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression ( IN_SYMBOL BOOLEAN_SYMBOL MODE_SYMBOL | ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )? ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )? ) CLOSE_PAR_SYMBOL ;
 */
static void
match_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2099:17: ( MATCH_SYMBOL OPEN_PAR_SYMBOL field_name_list CLOSE_PAR_SYMBOL AGAINST_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression ( IN_SYMBOL BOOLEAN_SYMBOL MODE_SYMBOL | ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )? ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )? ) CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2100:2: MATCH_SYMBOL OPEN_PAR_SYMBOL field_name_list CLOSE_PAR_SYMBOL AGAINST_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression ( IN_SYMBOL BOOLEAN_SYMBOL MODE_SYMBOL | ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )? ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )? ) CLOSE_PAR_SYMBOL
        {
             MATCHT(MATCH_SYMBOL, &FOLLOW_MATCH_SYMBOL_in_match_expression12225);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12227);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_field_name_list_in_match_expression12229);
            field_name_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12231);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(AGAINST_SYMBOL, &FOLLOW_AGAINST_SYMBOL_in_match_expression12233);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_match_expression12235);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_match_expression12237);
            bitwise_or_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2101:3: ( IN_SYMBOL BOOLEAN_SYMBOL MODE_SYMBOL | ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )? ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )? )
            {
                int alt693=2;
                switch ( LA(1) )
                {
                case IN_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case BOOLEAN_SYMBOL:
                			{
                				alt693=1;
                			}
                		    break;
                		case NATURAL_SYMBOL:
                			{
                				alt693=2;
                			}
                		    break;

                		default:
                		    if (BACKTRACKING>0)
                		    {
                		        FAILEDFLAG = ANTLR3_TRUE;
                		        return ;
                		    }

                		    CONSTRUCTEX();
                		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		    EXCEPTION->message      = (void *)"";
                		    EXCEPTION->decisionNum  = 693;
                		    EXCEPTION->state        = 1;


                		    goto rulematch_expressionEx;

                		}

                	}
                    break;
                case WITH_SYMBOL:
                case CLOSE_PAR_SYMBOL:
                	{
                		alt693=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 693;
                    EXCEPTION->state        = 0;


                    goto rulematch_expressionEx;

                }

                switch (alt693)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2102:4: IN_SYMBOL BOOLEAN_SYMBOL MODE_SYMBOL
            	    {
            	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_match_expression12246);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulematch_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(BOOLEAN_SYMBOL, &FOLLOW_BOOLEAN_SYMBOL_in_match_expression12248);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulematch_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MODE_SYMBOL, &FOLLOW_MODE_SYMBOL_in_match_expression12250);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulematch_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2103:6: ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )? ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )?
            	    {
            	        // MySQLSimpleParser.g:2103:6: ( IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL )?
            	        {
            	            int alt691=2;
            	            switch ( LA(1) )
            	            {
            	                case IN_SYMBOL:
            	                	{
            	                		alt691=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt691)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2103:7: IN_SYMBOL NATURAL_SYMBOL LANGUAGE_SYMBOL MODE_SYMBOL
            	        	    {
            	        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_match_expression12258);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(NATURAL_SYMBOL, &FOLLOW_NATURAL_SYMBOL_in_match_expression12260);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(LANGUAGE_SYMBOL, &FOLLOW_LANGUAGE_SYMBOL_in_match_expression12262);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(MODE_SYMBOL, &FOLLOW_MODE_SYMBOL_in_match_expression12264);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:2103:62: ( WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL )?
            	        {
            	            int alt692=2;
            	            switch ( LA(1) )
            	            {
            	                case WITH_SYMBOL:
            	                	{
            	                		alt692=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt692)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2103:63: WITH_SYMBOL QUERY_SYMBOL EXPANSION_SYMBOL
            	        	    {
            	        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_match_expression12269);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(QUERY_SYMBOL, &FOLLOW_QUERY_SYMBOL_in_match_expression12271);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(EXPANSION_SYMBOL, &FOLLOW_EXPANSION_SYMBOL_in_match_expression12273);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulematch_expressionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_match_expression12283);
            if  (HASEXCEPTION())
            {
                goto rulematch_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulematch_expressionEx; /* Prevent compiler warnings */
    rulematch_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end match_expression */

/**
 * $ANTLR start case_expression
 * MySQLSimpleParser.g:2109:1: case_expression : CASE_SYMBOL ( expression )? ( when_expression then_expression )+ ( else_expression )? END_SYMBOL ;
 */
static void
case_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2109:16: ( CASE_SYMBOL ( expression )? ( when_expression then_expression )+ ( else_expression )? END_SYMBOL )
        // MySQLSimpleParser.g:2110:2: CASE_SYMBOL ( expression )? ( when_expression then_expression )+ ( else_expression )? END_SYMBOL
        {
             MATCHT(CASE_SYMBOL, &FOLLOW_CASE_SYMBOL_in_case_expression12293);
            if  (HASEXCEPTION())
            {
                goto rulecase_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2110:14: ( expression )?
            {
                int alt694=2;
                {
                    int LA694_0 = LA(1);
                    if ( (((LA694_0 >= ACCOUNT_SYMBOL) && (LA694_0 <= YEAR_SYMBOL)) || LA694_0 == BINARY_SYMBOL || ((LA694_0 >= CASE_SYMBOL) && (LA694_0 <= CAST_SYMBOL)) || LA694_0 == CHAR_SYMBOL || ((LA694_0 >= CONVERT_SYMBOL) && (LA694_0 <= COUNT_SYMBOL)) || LA694_0 == CURDATE_SYMBOL || LA694_0 == CURRENT_USER_SYMBOL || LA694_0 == CURTIME_SYMBOL || LA694_0 == DATABASE_SYMBOL || LA694_0 == DATE_ADD_SYMBOL || LA694_0 == DATE_SUB_SYMBOL || LA694_0 == EXISTS_SYMBOL || ((LA694_0 >= EXTRACT_SYMBOL) && (LA694_0 <= FALSE_SYMBOL)) || LA694_0 == GROUP_CONCAT_SYMBOL || LA694_0 == IF_SYMBOL || LA694_0 == INSERT_SYMBOL || LA694_0 == INTERVAL_SYMBOL || LA694_0 == LEFT_SYMBOL || LA694_0 == MATCH_SYMBOL || LA694_0 == MAX_SYMBOL || LA694_0 == MIN_SYMBOL || LA694_0 == MOD_SYMBOL || LA694_0 == NCHAR_TEXT || ((LA694_0 >= NOT2_SYMBOL) && (LA694_0 <= NOW_SYMBOL)) || ((LA694_0 >= NULL2_SYMBOL) && (LA694_0 <= NULL_SYMBOL)) || ((LA694_0 >= OPEN_CURLY_SYMBOL) && (LA694_0 <= OPEN_PAR_SYMBOL)) || ((LA694_0 >= PARAM_MARKER) && (LA694_0 <= POSITION_SYMBOL)) || ((LA694_0 >= REPEAT_SYMBOL) && (LA694_0 <= REPLACE_SYMBOL)) || LA694_0 == RIGHT_SYMBOL || LA694_0 == SINGLE_QUOTED_TEXT || LA694_0 == STDDEV_SAMP_SYMBOL || LA694_0 == STD_SYMBOL || LA694_0 == SUBSTRING_SYMBOL || ((LA694_0 >= SUM_SYMBOL) && (LA694_0 <= SYSDATE_SYMBOL)) || ((LA694_0 >= TRIM_SYMBOL) && (LA694_0 <= TRUE_SYMBOL)) || ((LA694_0 >= UTC_DATE_SYMBOL) && (LA694_0 <= UTC_TIME_SYMBOL)) || LA694_0 == VALUES_SYMBOL || LA694_0 == VARIANCE_SYMBOL || LA694_0 == VAR_SAMP_SYMBOL || ((LA694_0 >= AT_AT_SIGN_SYMBOL) && (LA694_0 <= AT_TEXT_SUFFIX)) || ((LA694_0 >= BACK_TICK_QUOTED_ID) && (LA694_0 <= BIN_NUMBER)) || LA694_0 == BITWISE_NOT_OPERATOR || ((LA694_0 >= BIT_AND_SYMBOL) && (LA694_0 <= BIT_XOR_SYMBOL)) || LA694_0 == DECIMAL_NUMBER || LA694_0 == DOT_SYMBOL || LA694_0 == DOUBLE_QUOTED_TEXT || LA694_0 == FLOAT_NUMBER || ((LA694_0 >= HEX_NUMBER) && (LA694_0 <= IDENTIFIER)) || LA694_0 == INT_NUMBER || LA694_0 == LOGICAL_NOT_OPERATOR || ((LA694_0 >= LONG_NUMBER) && (LA694_0 <= MINUS_OPERATOR)) || LA694_0 == PLUS_OPERATOR || ((LA694_0 >= ULONGLONG_NUMBER) && (LA694_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt694=1;
                    }
                }
                switch (alt694)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2110:14: expression
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_case_expression12295);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecase_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2110:26: ( when_expression then_expression )+
            {
                int cnt695=0;

                for (;;)
                {
                    int alt695=2;
            	switch ( LA(1) )
            	{
            	case WHEN_SYMBOL:
            		{
            			alt695=1;
            		}
            	    break;

            	}

            	switch (alt695)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2110:27: when_expression then_expression
            	        {
            	            FOLLOWPUSH(FOLLOW_when_expression_in_case_expression12299);
            	            when_expression(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecase_expressionEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_then_expression_in_case_expression12301);
            	            then_expression(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecase_expressionEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt695 >= 1 )
            		{
            		    goto loop695;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulecase_expressionEx;
            	}
            	cnt695++;
                }
                loop695: ;	/* Jump to here if this rule does not match */
            }

            // MySQLSimpleParser.g:2110:61: ( else_expression )?
            {
                int alt696=2;
                switch ( LA(1) )
                {
                    case ELSE_SYMBOL:
                    	{
                    		alt696=1;
                    	}
                        break;
                }

                switch (alt696)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2110:61: else_expression
            	    {
            	        FOLLOWPUSH(FOLLOW_else_expression_in_case_expression12305);
            	        else_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecase_expressionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_case_expression12308);
            if  (HASEXCEPTION())
            {
                goto rulecase_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecase_expressionEx; /* Prevent compiler warnings */
    rulecase_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end case_expression */

/**
 * $ANTLR start when_expression
 * MySQLSimpleParser.g:2113:1: when_expression : WHEN_SYMBOL expression ;
 */
static void
when_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2113:16: ( WHEN_SYMBOL expression )
        // MySQLSimpleParser.g:2114:2: WHEN_SYMBOL expression
        {
             MATCHT(WHEN_SYMBOL, &FOLLOW_WHEN_SYMBOL_in_when_expression12317);
            if  (HASEXCEPTION())
            {
                goto rulewhen_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_when_expression12319);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulewhen_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulewhen_expressionEx; /* Prevent compiler warnings */
    rulewhen_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end when_expression */

/**
 * $ANTLR start then_expression
 * MySQLSimpleParser.g:2117:1: then_expression : THEN_SYMBOL expression ;
 */
static void
then_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2117:16: ( THEN_SYMBOL expression )
        // MySQLSimpleParser.g:2118:2: THEN_SYMBOL expression
        {
             MATCHT(THEN_SYMBOL, &FOLLOW_THEN_SYMBOL_in_then_expression12328);
            if  (HASEXCEPTION())
            {
                goto rulethen_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_then_expression12330);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulethen_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulethen_expressionEx; /* Prevent compiler warnings */
    rulethen_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end then_expression */

/**
 * $ANTLR start else_expression
 * MySQLSimpleParser.g:2121:1: else_expression : ELSE_SYMBOL expression ;
 */
static void
else_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2121:16: ( ELSE_SYMBOL expression )
        // MySQLSimpleParser.g:2122:2: ELSE_SYMBOL expression
        {
             MATCHT(ELSE_SYMBOL, &FOLLOW_ELSE_SYMBOL_in_else_expression12339);
            if  (HASEXCEPTION())
            {
                goto ruleelse_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_else_expression12341);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleelse_expressionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleelse_expressionEx; /* Prevent compiler warnings */
    ruleelse_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end else_expression */

/**
 * $ANTLR start cast_expression
 * MySQLSimpleParser.g:2125:1: cast_expression : ( BINARY_SYMBOL primary | CAST_SYMBOL OPEN_PAR_SYMBOL expression AS_SYMBOL cast_type CLOSE_PAR_SYMBOL | CONVERT_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL cast_type | USING_SYMBOL charset_name ) CLOSE_PAR_SYMBOL );
 */
static void
cast_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2125:16: ( BINARY_SYMBOL primary | CAST_SYMBOL OPEN_PAR_SYMBOL expression AS_SYMBOL cast_type CLOSE_PAR_SYMBOL | CONVERT_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL cast_type | USING_SYMBOL charset_name ) CLOSE_PAR_SYMBOL )

            ANTLR3_UINT32 alt698;

            alt698=3;

            switch ( LA(1) )
            {
            case BINARY_SYMBOL:
            	{
            		alt698=1;
            	}
                break;
            case CAST_SYMBOL:
            	{
            		alt698=2;
            	}
                break;
            case CONVERT_SYMBOL:
            	{
            		alt698=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 698;
                EXCEPTION->state        = 0;


                goto rulecast_expressionEx;

            }

            switch (alt698)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2126:2: BINARY_SYMBOL primary
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_cast_expression12350);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_primary_in_cast_expression12352);
        	        primary(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2127:4: CAST_SYMBOL OPEN_PAR_SYMBOL expression AS_SYMBOL cast_type CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(CAST_SYMBOL, &FOLLOW_CAST_SYMBOL_in_cast_expression12357);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12359);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_cast_expression12361);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_cast_expression12363);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_cast_type_in_cast_expression12365);
        	        cast_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12367);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2128:4: CONVERT_SYMBOL OPEN_PAR_SYMBOL expression ( COMMA_SYMBOL cast_type | USING_SYMBOL charset_name ) CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(CONVERT_SYMBOL, &FOLLOW_CONVERT_SYMBOL_in_cast_expression12372);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_cast_expression12374);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_cast_expression12376);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2128:46: ( COMMA_SYMBOL cast_type | USING_SYMBOL charset_name )
        	        {
        	            int alt697=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMA_SYMBOL:
        	            	{
        	            		alt697=1;
        	            	}
        	                break;
        	            case USING_SYMBOL:
        	            	{
        	            		alt697=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 697;
        	                EXCEPTION->state        = 0;


        	                goto rulecast_expressionEx;

        	            }

        	            switch (alt697)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2128:47: COMMA_SYMBOL cast_type
        	        	    {
        	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_cast_expression12379);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_cast_type_in_cast_expression12381);
        	        	        cast_type(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2128:72: USING_SYMBOL charset_name
        	        	    {
        	        	         MATCHT(USING_SYMBOL, &FOLLOW_USING_SYMBOL_in_cast_expression12385);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_charset_name_in_cast_expression12387);
        	        	        charset_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_expressionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_cast_expression12390);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecast_expressionEx; /* Prevent compiler warnings */
    rulecast_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cast_expression */

/**
 * $ANTLR start cast_type
 * MySQLSimpleParser.g:2131:1: cast_type : ( BINARY_SYMBOL ( field_length )? | CHAR_SYMBOL ( field_length )? ( encoding )? | NCHAR_SYMBOL ( field_length )? | SIGNED_SYMBOL ( INT_SYMBOL )? | UNSIGNED_SYMBOL ( INT_SYMBOL )? | DATE_SYMBOL | TIME_SYMBOL ( type_datetime_precision )? | DATETIME_SYMBOL ( type_datetime_precision )? | DECIMAL_SYMBOL ( float_options )? |{...}? JSON_SYMBOL );
 */
static void
cast_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2131:10: ( BINARY_SYMBOL ( field_length )? | CHAR_SYMBOL ( field_length )? ( encoding )? | NCHAR_SYMBOL ( field_length )? | SIGNED_SYMBOL ( INT_SYMBOL )? | UNSIGNED_SYMBOL ( INT_SYMBOL )? | DATE_SYMBOL | TIME_SYMBOL ( type_datetime_precision )? | DATETIME_SYMBOL ( type_datetime_precision )? | DECIMAL_SYMBOL ( float_options )? |{...}? JSON_SYMBOL )

            ANTLR3_UINT32 alt708;

            alt708=10;

            switch ( LA(1) )
            {
            case BINARY_SYMBOL:
            	{
            		alt708=1;
            	}
                break;
            case CHAR_SYMBOL:
            	{
            		alt708=2;
            	}
                break;
            case NCHAR_SYMBOL:
            	{
            		alt708=3;
            	}
                break;
            case SIGNED_SYMBOL:
            	{
            		alt708=4;
            	}
                break;
            case UNSIGNED_SYMBOL:
            	{
            		alt708=5;
            	}
                break;
            case DATE_SYMBOL:
            	{
            		alt708=6;
            	}
                break;
            case TIME_SYMBOL:
            	{
            		alt708=7;
            	}
                break;
            case DATETIME_SYMBOL:
            	{
            		alt708=8;
            	}
                break;
            case DECIMAL_SYMBOL:
            	{
            		alt708=9;
            	}
                break;
            case JSON_SYMBOL:
            	{
            		alt708=10;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 708;
                EXCEPTION->state        = 0;


                goto rulecast_typeEx;

            }

            switch (alt708)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2132:2: BINARY_SYMBOL ( field_length )?
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_cast_type12399);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2132:16: ( field_length )?
        	        {
        	            int alt699=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt699=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt699)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2132:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_cast_type12401);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2133:4: CHAR_SYMBOL ( field_length )? ( encoding )?
        	    {
        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_cast_type12407);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2133:16: ( field_length )?
        	        {
        	            int alt700=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt700=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt700)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2133:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_cast_type12409);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2133:30: ( encoding )?
        	        {
        	            int alt701=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt701=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt701)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2133:30: encoding
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_encoding_in_cast_type12412);
        	        	        encoding(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2134:4: NCHAR_SYMBOL ( field_length )?
        	    {
        	         MATCHT(NCHAR_SYMBOL, &FOLLOW_NCHAR_SYMBOL_in_cast_type12418);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2134:17: ( field_length )?
        	        {
        	            int alt702=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt702=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt702)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2134:17: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_cast_type12420);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2135:4: SIGNED_SYMBOL ( INT_SYMBOL )?
        	    {
        	         MATCHT(SIGNED_SYMBOL, &FOLLOW_SIGNED_SYMBOL_in_cast_type12426);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2135:18: ( INT_SYMBOL )?
        	        {
        	            int alt703=2;
        	            switch ( LA(1) )
        	            {
        	                case INT_SYMBOL:
        	                	{
        	                		alt703=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt703)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2135:18: INT_SYMBOL
        	        	    {
        	        	         MATCHT(INT_SYMBOL, &FOLLOW_INT_SYMBOL_in_cast_type12428);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2136:4: UNSIGNED_SYMBOL ( INT_SYMBOL )?
        	    {
        	         MATCHT(UNSIGNED_SYMBOL, &FOLLOW_UNSIGNED_SYMBOL_in_cast_type12434);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2136:20: ( INT_SYMBOL )?
        	        {
        	            int alt704=2;
        	            switch ( LA(1) )
        	            {
        	                case INT_SYMBOL:
        	                	{
        	                		alt704=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt704)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2136:20: INT_SYMBOL
        	        	    {
        	        	         MATCHT(INT_SYMBOL, &FOLLOW_INT_SYMBOL_in_cast_type12436);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2137:4: DATE_SYMBOL
        	    {
        	         MATCHT(DATE_SYMBOL, &FOLLOW_DATE_SYMBOL_in_cast_type12442);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2138:4: TIME_SYMBOL ( type_datetime_precision )?
        	    {
        	         MATCHT(TIME_SYMBOL, &FOLLOW_TIME_SYMBOL_in_cast_type12447);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2138:16: ( type_datetime_precision )?
        	        {
        	            int alt705=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt705=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt705)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2138:16: type_datetime_precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_type_datetime_precision_in_cast_type12449);
        	        	        type_datetime_precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2139:4: DATETIME_SYMBOL ( type_datetime_precision )?
        	    {
        	         MATCHT(DATETIME_SYMBOL, &FOLLOW_DATETIME_SYMBOL_in_cast_type12455);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2139:20: ( type_datetime_precision )?
        	        {
        	            int alt706=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt706=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt706)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2139:20: type_datetime_precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_type_datetime_precision_in_cast_type12457);
        	        	        type_datetime_precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2140:4: DECIMAL_SYMBOL ( float_options )?
        	    {
        	         MATCHT(DECIMAL_SYMBOL, &FOLLOW_DECIMAL_SYMBOL_in_cast_type12463);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2140:19: ( float_options )?
        	        {
        	            int alt707=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt707=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt707)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2140:19: float_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_float_options_in_cast_type12465);
        	        	        float_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecast_typeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2141:4: {...}? JSON_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50708)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50708";
        	                EXCEPTION->ruleName	 = (void *)"cast_type";


        	        }

        	         MATCHT(JSON_SYMBOL, &FOLLOW_JSON_SYMBOL_in_cast_type12473);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecast_typeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecast_typeEx; /* Prevent compiler warnings */
    rulecast_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cast_type */

/**
 * $ANTLR start encoding
 * MySQLSimpleParser.g:2144:1: encoding : ( ASCII_SYMBOL ( BINARY_SYMBOL )? | BINARY_SYMBOL ( ASCII_SYMBOL | UNICODE_SYMBOL | charset charset_name )? | UNICODE_SYMBOL ( BINARY_SYMBOL )? | BYTE_SYMBOL | charset charset_name ( BINARY_SYMBOL )? );
 */
static void
encoding(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2144:9: ( ASCII_SYMBOL ( BINARY_SYMBOL )? | BINARY_SYMBOL ( ASCII_SYMBOL | UNICODE_SYMBOL | charset charset_name )? | UNICODE_SYMBOL ( BINARY_SYMBOL )? | BYTE_SYMBOL | charset charset_name ( BINARY_SYMBOL )? )

            ANTLR3_UINT32 alt713;

            alt713=5;

            switch ( LA(1) )
            {
            case ASCII_SYMBOL:
            	{
            		alt713=1;
            	}
                break;
            case BINARY_SYMBOL:
            	{
            		alt713=2;
            	}
                break;
            case UNICODE_SYMBOL:
            	{
            		alt713=3;
            	}
                break;
            case BYTE_SYMBOL:
            	{
            		alt713=4;
            	}
                break;
            case CHARSET_SYMBOL:
            case CHAR_SYMBOL:
            	{
            		alt713=5;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 713;
                EXCEPTION->state        = 0;


                goto ruleencodingEx;

            }

            switch (alt713)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2145:2: ASCII_SYMBOL ( BINARY_SYMBOL )?
        	    {
        	         MATCHT(ASCII_SYMBOL, &FOLLOW_ASCII_SYMBOL_in_encoding12482);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2145:15: ( BINARY_SYMBOL )?
        	        {
        	            int alt709=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt709=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt709)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2145:15: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_encoding12484);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2146:4: BINARY_SYMBOL ( ASCII_SYMBOL | UNICODE_SYMBOL | charset charset_name )?
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_encoding12490);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2146:18: ( ASCII_SYMBOL | UNICODE_SYMBOL | charset charset_name )?
        	        {
        	            int alt710=4;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                	{
        	                		alt710=1;
        	                	}
        	                    break;
        	                case UNICODE_SYMBOL:
        	                	{
        	                		alt710=2;
        	                	}
        	                    break;
        	                case CHARSET_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt710=3;
        	                	}
        	                    break;
        	            }

        	            switch (alt710)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2146:19: ASCII_SYMBOL
        	        	    {
        	        	         MATCHT(ASCII_SYMBOL, &FOLLOW_ASCII_SYMBOL_in_encoding12493);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2146:34: UNICODE_SYMBOL
        	        	    {
        	        	         MATCHT(UNICODE_SYMBOL, &FOLLOW_UNICODE_SYMBOL_in_encoding12497);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 3:
        	        	    // MySQLSimpleParser.g:2146:51: charset charset_name
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_charset_in_encoding12501);
        	        	        charset(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_charset_name_in_encoding12503);
        	        	        charset_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2147:4: UNICODE_SYMBOL ( BINARY_SYMBOL )?
        	    {
        	         MATCHT(UNICODE_SYMBOL, &FOLLOW_UNICODE_SYMBOL_in_encoding12510);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2147:19: ( BINARY_SYMBOL )?
        	        {
        	            int alt711=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt711=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt711)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2147:19: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_encoding12512);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2148:4: BYTE_SYMBOL
        	    {
        	         MATCHT(BYTE_SYMBOL, &FOLLOW_BYTE_SYMBOL_in_encoding12518);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2149:4: charset charset_name ( BINARY_SYMBOL )?
        	    {
        	        FOLLOWPUSH(FOLLOW_charset_in_encoding12523);
        	        charset(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_charset_name_in_encoding12525);
        	        charset_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleencodingEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2149:25: ( BINARY_SYMBOL )?
        	        {
        	            int alt712=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt712=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt712)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2149:25: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_encoding12527);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleencodingEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleencodingEx; /* Prevent compiler warnings */
    ruleencodingEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end encoding */

/**
 * $ANTLR start charset
 * MySQLSimpleParser.g:2152:1: charset : ( CHAR_SYMBOL SET_SYMBOL | CHARSET_SYMBOL );
 */
static void
charset(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2152:8: ( CHAR_SYMBOL SET_SYMBOL | CHARSET_SYMBOL )

            ANTLR3_UINT32 alt714;

            alt714=2;

            switch ( LA(1) )
            {
            case CHAR_SYMBOL:
            	{
            		alt714=1;
            	}
                break;
            case CHARSET_SYMBOL:
            	{
            		alt714=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 714;
                EXCEPTION->state        = 0;


                goto rulecharsetEx;

            }

            switch (alt714)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2153:2: CHAR_SYMBOL SET_SYMBOL
        	    {
        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_charset12537);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharsetEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_charset12539);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharsetEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2153:27: CHARSET_SYMBOL
        	    {
        	         MATCHT(CHARSET_SYMBOL, &FOLLOW_CHARSET_SYMBOL_in_charset12543);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharsetEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecharsetEx; /* Prevent compiler warnings */
    rulecharsetEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end charset */

/**
 * $ANTLR start not_rule
 * MySQLSimpleParser.g:2156:1: not_rule : ( NOT_SYMBOL | NOT2_SYMBOL );
 */
static void
not_rule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2156:9: ( NOT_SYMBOL | NOT2_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= NOT2_SYMBOL) && (LA(1) <= NOT_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulenot_ruleEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulenot_ruleEx; /* Prevent compiler warnings */
    rulenot_ruleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end not_rule */

/**
 * $ANTLR start not2_rule
 * MySQLSimpleParser.g:2161:1: not2_rule : ( LOGICAL_NOT_OPERATOR | NOT2_SYMBOL );
 */
static void
not2_rule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2161:10: ( LOGICAL_NOT_OPERATOR | NOT2_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == NOT2_SYMBOL || LA(1) == LOGICAL_NOT_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulenot2_ruleEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulenot2_ruleEx; /* Prevent compiler warnings */
    rulenot2_ruleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end not2_rule */

/**
 * $ANTLR start or_rule
 * MySQLSimpleParser.g:2166:1: or_rule : ( OR_SYMBOL | LOGICAL_OR_OPERATOR );
 */
static void
or_rule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2166:8: ( OR_SYMBOL | LOGICAL_OR_OPERATOR )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == OR_SYMBOL || LA(1) == LOGICAL_OR_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleor_ruleEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleor_ruleEx; /* Prevent compiler warnings */
    ruleor_ruleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end or_rule */

/**
 * $ANTLR start xor_rule
 * MySQLSimpleParser.g:2171:1: xor_rule : ( XOR_SYMBOL | BITWISE_XOR_OPERATOR );
 */
static void
xor_rule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2171:9: ( XOR_SYMBOL | BITWISE_XOR_OPERATOR )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == XOR_SYMBOL || LA(1) == BITWISE_XOR_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulexor_ruleEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulexor_ruleEx; /* Prevent compiler warnings */
    rulexor_ruleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end xor_rule */

/**
 * $ANTLR start and_rule
 * MySQLSimpleParser.g:2176:1: and_rule : ( AND_SYMBOL | LOGICAL_AND_OPERATOR );
 */
static void
and_rule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2176:9: ( AND_SYMBOL | LOGICAL_AND_OPERATOR )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == AND_SYMBOL || LA(1) == LOGICAL_AND_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleand_ruleEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleand_ruleEx; /* Prevent compiler warnings */
    ruleand_ruleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end and_rule */

/**
 * $ANTLR start interval
 * MySQLSimpleParser.g:2181:1: interval : expression interval_unit ;
 */
static void
interval(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2181:9: ( expression interval_unit )
        // MySQLSimpleParser.g:2182:3: expression interval_unit
        {
            FOLLOWPUSH(FOLLOW_expression_in_interval12624);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleintervalEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_interval_unit_in_interval12626);
            interval_unit(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleintervalEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleintervalEx; /* Prevent compiler warnings */
    ruleintervalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval */

/**
 * $ANTLR start interval_unit
 * MySQLSimpleParser.g:2186:1: interval_unit : ( interval_timestamp_unit | SECOND_MICROSECOND_SYMBOL | MINUTE_MICROSECOND_SYMBOL | MINUTE_SECOND_SYMBOL | HOUR_MICROSECOND_SYMBOL | HOUR_SECOND_SYMBOL | HOUR_MINUTE_SYMBOL | DAY_MICROSECOND_SYMBOL | DAY_SECOND_SYMBOL | DAY_MINUTE_SYMBOL | DAY_HOUR_SYMBOL | YEAR_MONTH_SYMBOL );
 */
static void
interval_unit(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2186:14: ( interval_timestamp_unit | SECOND_MICROSECOND_SYMBOL | MINUTE_MICROSECOND_SYMBOL | MINUTE_SECOND_SYMBOL | HOUR_MICROSECOND_SYMBOL | HOUR_SECOND_SYMBOL | HOUR_MINUTE_SYMBOL | DAY_MICROSECOND_SYMBOL | DAY_SECOND_SYMBOL | DAY_MINUTE_SYMBOL | DAY_HOUR_SYMBOL | YEAR_MONTH_SYMBOL )

            ANTLR3_UINT32 alt715;

            alt715=12;

            switch ( LA(1) )
            {
            case DAY_SYMBOL:
            case HOUR_SYMBOL:
            case MICROSECOND_SYMBOL:
            case MINUTE_SYMBOL:
            case MONTH_SYMBOL:
            case QUARTER_SYMBOL:
            case SECOND_SYMBOL:
            case WEEK_SYMBOL:
            case YEAR_SYMBOL:
            case FRAC_SECOND_SYMBOL:
            	{
            		alt715=1;
            	}
                break;
            case SECOND_MICROSECOND_SYMBOL:
            	{
            		alt715=2;
            	}
                break;
            case MINUTE_MICROSECOND_SYMBOL:
            	{
            		alt715=3;
            	}
                break;
            case MINUTE_SECOND_SYMBOL:
            	{
            		alt715=4;
            	}
                break;
            case HOUR_MICROSECOND_SYMBOL:
            	{
            		alt715=5;
            	}
                break;
            case HOUR_SECOND_SYMBOL:
            	{
            		alt715=6;
            	}
                break;
            case HOUR_MINUTE_SYMBOL:
            	{
            		alt715=7;
            	}
                break;
            case DAY_MICROSECOND_SYMBOL:
            	{
            		alt715=8;
            	}
                break;
            case DAY_SECOND_SYMBOL:
            	{
            		alt715=9;
            	}
                break;
            case DAY_MINUTE_SYMBOL:
            	{
            		alt715=10;
            	}
                break;
            case DAY_HOUR_SYMBOL:
            	{
            		alt715=11;
            	}
                break;
            case YEAR_MONTH_SYMBOL:
            	{
            		alt715=12;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 715;
                EXCEPTION->state        = 0;


                goto ruleinterval_unitEx;

            }

            switch (alt715)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2187:2: interval_timestamp_unit
        	    {
        	        FOLLOWPUSH(FOLLOW_interval_timestamp_unit_in_interval_unit12636);
        	        interval_timestamp_unit(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2188:4: SECOND_MICROSECOND_SYMBOL
        	    {
        	         MATCHT(SECOND_MICROSECOND_SYMBOL, &FOLLOW_SECOND_MICROSECOND_SYMBOL_in_interval_unit12641);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2189:4: MINUTE_MICROSECOND_SYMBOL
        	    {
        	         MATCHT(MINUTE_MICROSECOND_SYMBOL, &FOLLOW_MINUTE_MICROSECOND_SYMBOL_in_interval_unit12646);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2190:4: MINUTE_SECOND_SYMBOL
        	    {
        	         MATCHT(MINUTE_SECOND_SYMBOL, &FOLLOW_MINUTE_SECOND_SYMBOL_in_interval_unit12651);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2191:4: HOUR_MICROSECOND_SYMBOL
        	    {
        	         MATCHT(HOUR_MICROSECOND_SYMBOL, &FOLLOW_HOUR_MICROSECOND_SYMBOL_in_interval_unit12656);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2192:4: HOUR_SECOND_SYMBOL
        	    {
        	         MATCHT(HOUR_SECOND_SYMBOL, &FOLLOW_HOUR_SECOND_SYMBOL_in_interval_unit12661);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2193:4: HOUR_MINUTE_SYMBOL
        	    {
        	         MATCHT(HOUR_MINUTE_SYMBOL, &FOLLOW_HOUR_MINUTE_SYMBOL_in_interval_unit12666);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2194:4: DAY_MICROSECOND_SYMBOL
        	    {
        	         MATCHT(DAY_MICROSECOND_SYMBOL, &FOLLOW_DAY_MICROSECOND_SYMBOL_in_interval_unit12671);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2195:4: DAY_SECOND_SYMBOL
        	    {
        	         MATCHT(DAY_SECOND_SYMBOL, &FOLLOW_DAY_SECOND_SYMBOL_in_interval_unit12676);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2196:4: DAY_MINUTE_SYMBOL
        	    {
        	         MATCHT(DAY_MINUTE_SYMBOL, &FOLLOW_DAY_MINUTE_SYMBOL_in_interval_unit12681);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2197:4: DAY_HOUR_SYMBOL
        	    {
        	         MATCHT(DAY_HOUR_SYMBOL, &FOLLOW_DAY_HOUR_SYMBOL_in_interval_unit12686);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2198:4: YEAR_MONTH_SYMBOL
        	    {
        	         MATCHT(YEAR_MONTH_SYMBOL, &FOLLOW_YEAR_MONTH_SYMBOL_in_interval_unit12691);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleinterval_unitEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleinterval_unitEx; /* Prevent compiler warnings */
    ruleinterval_unitEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval_unit */

/**
 * $ANTLR start interval_timestamp_unit
 * MySQLSimpleParser.g:2202:1: interval_timestamp_unit : ( MICROSECOND_SYMBOL | FRAC_SECOND_SYMBOL | SECOND_SYMBOL | MINUTE_SYMBOL | HOUR_SYMBOL | DAY_SYMBOL | WEEK_SYMBOL | MONTH_SYMBOL | QUARTER_SYMBOL | YEAR_SYMBOL );
 */
static void
interval_timestamp_unit(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2202:24: ( MICROSECOND_SYMBOL | FRAC_SECOND_SYMBOL | SECOND_SYMBOL | MINUTE_SYMBOL | HOUR_SYMBOL | DAY_SYMBOL | WEEK_SYMBOL | MONTH_SYMBOL | QUARTER_SYMBOL | YEAR_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == DAY_SYMBOL || LA(1) == HOUR_SYMBOL || LA(1) == MICROSECOND_SYMBOL || LA(1) == MINUTE_SYMBOL || LA(1) == MONTH_SYMBOL || LA(1) == QUARTER_SYMBOL || LA(1) == SECOND_SYMBOL || LA(1) == WEEK_SYMBOL || LA(1) == YEAR_SYMBOL || LA(1) == FRAC_SECOND_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleinterval_timestamp_unitEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinterval_timestamp_unitEx; /* Prevent compiler warnings */
    ruleinterval_timestamp_unitEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end interval_timestamp_unit */

/**
 * $ANTLR start expression_list_with_parentheses
 * MySQLSimpleParser.g:2215:1: expression_list_with_parentheses : OPEN_PAR_SYMBOL expression_list CLOSE_PAR_SYMBOL ;
 */
static void
expression_list_with_parentheses(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2215:33: ( OPEN_PAR_SYMBOL expression_list CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2216:2: OPEN_PAR_SYMBOL expression_list CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_expression_list_with_parentheses12759);
            if  (HASEXCEPTION())
            {
                goto ruleexpression_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_list_in_expression_list_with_parentheses12761);
            expression_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexpression_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_expression_list_with_parentheses12763);
            if  (HASEXCEPTION())
            {
                goto ruleexpression_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexpression_list_with_parenthesesEx; /* Prevent compiler warnings */
    ruleexpression_list_with_parenthesesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end expression_list_with_parentheses */

/**
 * $ANTLR start optional_expression_list_with_parentheses
 * MySQLSimpleParser.g:2219:1: optional_expression_list_with_parentheses : OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL ;
 */
static void
optional_expression_list_with_parentheses(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2219:42: ( OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2220:2: OPEN_PAR_SYMBOL ( expression_list )? CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_optional_expression_list_with_parentheses12772);
            if  (HASEXCEPTION())
            {
                goto ruleoptional_expression_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2220:18: ( expression_list )?
            {
                int alt716=2;
                {
                    int LA716_0 = LA(1);
                    if ( (((LA716_0 >= ACCOUNT_SYMBOL) && (LA716_0 <= YEAR_SYMBOL)) || LA716_0 == BINARY_SYMBOL || ((LA716_0 >= CASE_SYMBOL) && (LA716_0 <= CAST_SYMBOL)) || LA716_0 == CHAR_SYMBOL || ((LA716_0 >= CONVERT_SYMBOL) && (LA716_0 <= COUNT_SYMBOL)) || LA716_0 == CURDATE_SYMBOL || LA716_0 == CURRENT_USER_SYMBOL || LA716_0 == CURTIME_SYMBOL || LA716_0 == DATABASE_SYMBOL || LA716_0 == DATE_ADD_SYMBOL || LA716_0 == DATE_SUB_SYMBOL || LA716_0 == EXISTS_SYMBOL || ((LA716_0 >= EXTRACT_SYMBOL) && (LA716_0 <= FALSE_SYMBOL)) || LA716_0 == GROUP_CONCAT_SYMBOL || LA716_0 == IF_SYMBOL || LA716_0 == INSERT_SYMBOL || LA716_0 == INTERVAL_SYMBOL || LA716_0 == LEFT_SYMBOL || LA716_0 == MATCH_SYMBOL || LA716_0 == MAX_SYMBOL || LA716_0 == MIN_SYMBOL || LA716_0 == MOD_SYMBOL || LA716_0 == NCHAR_TEXT || ((LA716_0 >= NOT2_SYMBOL) && (LA716_0 <= NOW_SYMBOL)) || ((LA716_0 >= NULL2_SYMBOL) && (LA716_0 <= NULL_SYMBOL)) || ((LA716_0 >= OPEN_CURLY_SYMBOL) && (LA716_0 <= OPEN_PAR_SYMBOL)) || ((LA716_0 >= PARAM_MARKER) && (LA716_0 <= POSITION_SYMBOL)) || ((LA716_0 >= REPEAT_SYMBOL) && (LA716_0 <= REPLACE_SYMBOL)) || LA716_0 == RIGHT_SYMBOL || LA716_0 == SINGLE_QUOTED_TEXT || LA716_0 == STDDEV_SAMP_SYMBOL || LA716_0 == STD_SYMBOL || LA716_0 == SUBSTRING_SYMBOL || ((LA716_0 >= SUM_SYMBOL) && (LA716_0 <= SYSDATE_SYMBOL)) || ((LA716_0 >= TRIM_SYMBOL) && (LA716_0 <= TRUE_SYMBOL)) || ((LA716_0 >= UTC_DATE_SYMBOL) && (LA716_0 <= UTC_TIME_SYMBOL)) || LA716_0 == VALUES_SYMBOL || LA716_0 == VARIANCE_SYMBOL || LA716_0 == VAR_SAMP_SYMBOL || ((LA716_0 >= AT_AT_SIGN_SYMBOL) && (LA716_0 <= AT_TEXT_SUFFIX)) || ((LA716_0 >= BACK_TICK_QUOTED_ID) && (LA716_0 <= BIN_NUMBER)) || LA716_0 == BITWISE_NOT_OPERATOR || ((LA716_0 >= BIT_AND_SYMBOL) && (LA716_0 <= BIT_XOR_SYMBOL)) || LA716_0 == DECIMAL_NUMBER || LA716_0 == DOT_SYMBOL || LA716_0 == DOUBLE_QUOTED_TEXT || LA716_0 == FLOAT_NUMBER || ((LA716_0 >= HEX_NUMBER) && (LA716_0 <= IDENTIFIER)) || LA716_0 == INT_NUMBER || LA716_0 == LOGICAL_NOT_OPERATOR || ((LA716_0 >= LONG_NUMBER) && (LA716_0 <= MINUS_OPERATOR)) || LA716_0 == PLUS_OPERATOR || ((LA716_0 >= ULONGLONG_NUMBER) && (LA716_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt716=1;
                    }
                }
                switch (alt716)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2220:18: expression_list
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_list_in_optional_expression_list_with_parentheses12774);
            	        expression_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleoptional_expression_list_with_parenthesesEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_optional_expression_list_with_parentheses12777);
            if  (HASEXCEPTION())
            {
                goto ruleoptional_expression_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleoptional_expression_list_with_parenthesesEx; /* Prevent compiler warnings */
    ruleoptional_expression_list_with_parenthesesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end optional_expression_list_with_parentheses */

/**
 * $ANTLR start expression_list
 * MySQLSimpleParser.g:2223:1: expression_list : expression ( COMMA_SYMBOL expression )* ;
 */
static void
expression_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2223:16: ( expression ( COMMA_SYMBOL expression )* )
        // MySQLSimpleParser.g:2224:2: expression ( COMMA_SYMBOL expression )*
        {
            FOLLOWPUSH(FOLLOW_expression_in_expression_list12786);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexpression_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2224:13: ( COMMA_SYMBOL expression )*

            for (;;)
            {
                int alt717=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt717=1;
                	}
                    break;

                }

                switch (alt717)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2224:14: COMMA_SYMBOL expression
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_expression_list12789);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexpression_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_expression_list12791);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexpression_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop717;	/* break out of the loop */
            	    break;
                }
            }
            loop717: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexpression_listEx; /* Prevent compiler warnings */
    ruleexpression_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end expression_list */

/**
 * $ANTLR start expression_list_with_direction
 * MySQLSimpleParser.g:2227:1: expression_list_with_direction : expression ( direction )? ( options {greedy=true; } : COMMA_SYMBOL expression ( direction )? )* ;
 */
static void
expression_list_with_direction(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2227:31: ( expression ( direction )? ( options {greedy=true; } : COMMA_SYMBOL expression ( direction )? )* )
        // MySQLSimpleParser.g:2228:2: expression ( direction )? ( options {greedy=true; } : COMMA_SYMBOL expression ( direction )? )*
        {
            FOLLOWPUSH(FOLLOW_expression_in_expression_list_with_direction12802);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleexpression_list_with_directionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2228:13: ( direction )?
            {
                int alt718=2;
                switch ( LA(1) )
                {
                    case ASC_SYMBOL:
                    case DESC_SYMBOL:
                    	{
                    		alt718=1;
                    	}
                        break;
                }

                switch (alt718)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2228:13: direction
            	    {
            	        FOLLOWPUSH(FOLLOW_direction_in_expression_list_with_direction12804);
            	        direction(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexpression_list_with_directionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2228:24: ( options {greedy=true; } : COMMA_SYMBOL expression ( direction )? )*

            for (;;)
            {
                int alt720=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		{
                		   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
                		    */
                		    int LA720_19 = LA(2);
                		    if ( (LA720_19 == OPEN_PAR_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == IDENTIFIER))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == BACK_TICK_QUOTED_ID))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == DOUBLE_QUOTED_TEXT))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == DATE_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ASCII_SYMBOL || LA720_19 == INTERVAL_SYMBOL || LA720_19 == NCHAR_TEXT || ((LA720_19 >= NOT2_SYMBOL) && (LA720_19 <= NOT_SYMBOL)) || LA720_19 == SINGLE_QUOTED_TEXT || LA720_19 == BITWISE_NOT_OPERATOR || LA720_19 == DECIMAL_NUMBER || LA720_19 == FLOAT_NUMBER || LA720_19 == INT_NUMBER || LA720_19 == LOGICAL_NOT_OPERATOR || ((LA720_19 >= LONG_NUMBER) && (LA720_19 <= MINUS_OPERATOR)) || LA720_19 == PLUS_OPERATOR || ((LA720_19 >= ULONGLONG_NUMBER) && (LA720_19 <= UNDERSCORE_CHARSET))))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == DOT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == OPEN_CURLY_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == TIME_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == TIMESTAMP_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == DAY_SYMBOL || LA720_19 == CHAR_SYMBOL || LA720_19 == CURRENT_USER_SYMBOL || LA720_19 == FALSE_SYMBOL || ((LA720_19 >= NULL2_SYMBOL) && (LA720_19 <= NULL_SYMBOL)) || LA720_19 == TRUE_SYMBOL || LA720_19 == BIN_NUMBER || LA720_19 == HEX_NUMBER))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == HOUR_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MINUTE_SYMBOL || LA720_19 == INSERT_SYMBOL || LA720_19 == LEFT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MONTH_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == SECOND_SYMBOL || LA720_19 == RIGHT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == USER_SYMBOL || LA720_19 == TRIM_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == YEAR_SYMBOL || LA720_19 == VALUES_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ADDDATE_SYMBOL || LA720_19 == SUBDATE_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == GET_FORMAT_SYMBOL || LA720_19 == CURDATE_SYMBOL || LA720_19 == CURTIME_SYMBOL || LA720_19 == DATE_ADD_SYMBOL || LA720_19 == DATE_SUB_SYMBOL || LA720_19 == EXTRACT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (((LA720_19 >= TIMESTAMP_ADD_SYMBOL) && (LA720_19 <= TIMESTAMP_DIFF_SYMBOL)) || LA720_19 == NOW_SYMBOL || LA720_19 == POSITION_SYMBOL || LA720_19 == SUBSTRING_SYMBOL || LA720_19 == SYSDATE_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == COALESCE_SYMBOL || ((LA720_19 >= UTC_DATE_SYMBOL) && (LA720_19 <= UTC_TIME_SYMBOL))))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == CHARSET_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == FORMAT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == COLLATION_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MICROSECOND_SYMBOL || LA720_19 == DATABASE_SYMBOL || LA720_19 == IF_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == TRUNCATE_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == OLD_PASSWORD_SYMBOL || LA720_19 == MOD_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == PASSWORD_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == QUARTER_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == REVERSE_SYMBOL || ((LA720_19 >= REPEAT_SYMBOL) && (LA720_19 <= REPLACE_SYMBOL))))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ROW_COUNT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == WEEK_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == CONTAINS_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == WEIGHT_STRING_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == GEOMETRYCOLLECTION_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == LINESTRING_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MULTILINESTRING_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MULTIPOINT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == MULTIPOLYGON_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == POINT_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == POLYGON_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == AVG_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ACCOUNT_SYMBOL || ((LA720_19 >= ALWAYS_SYMBOL) && (LA720_19 <= CACHE_SYMBOL)) || ((LA720_19 >= CHECKSUM_SYMBOL) && (LA720_19 <= COMMIT_SYMBOL)) || ((LA720_19 >= DEALLOCATE_SYMBOL) && (LA720_19 <= FOLLOWS_SYMBOL)) || ((LA720_19 >= GROUP_REPLICATION_SYMBOL) && (LA720_19 <= STOP_SYMBOL)) || ((LA720_19 >= UNICODE_SYMBOL) && (LA720_19 <= XA_SYMBOL)) || LA720_19 == SHUTDOWN_SYMBOL))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ROW_SYMBOL || LA720_19 == COUNT_SYMBOL || LA720_19 == EXISTS_SYMBOL || LA720_19 == GROUP_CONCAT_SYMBOL || LA720_19 == MAX_SYMBOL || LA720_19 == MIN_SYMBOL || LA720_19 == PARAM_MARKER || LA720_19 == STDDEV_SAMP_SYMBOL || LA720_19 == STD_SYMBOL || LA720_19 == SUM_SYMBOL || LA720_19 == VARIANCE_SYMBOL || LA720_19 == VAR_SAMP_SYMBOL || ((LA720_19 >= AT_AT_SIGN_SYMBOL) && (LA720_19 <= AT_TEXT_SUFFIX)) || ((LA720_19 >= BIT_AND_SYMBOL) && (LA720_19 <= BIT_XOR_SYMBOL))))
                		    {
                		        alt720=1;
                		    }
                		    else if ( (LA720_19 == ACTION_SYMBOL || ((LA720_19 >= AFTER_SYMBOL) && (LA720_19 <= AVG_ROW_LENGTH_SYMBOL)) || ((LA720_19 >= BINLOG_SYMBOL) && (LA720_19 <= CLASS_ORIGIN_SYMBOL)) || LA720_19 == CODE_SYMBOL || ((LA720_19 >= COLUMN_NAME_SYMBOL) && (LA720_19 <= DATETIME_SYMBOL)) || ((LA720_19 >= DEFAULT_AUTH_SYMBOL) && (LA720_19 <= GEOMETRY_SYMBOL)) || ((LA720_19 >= GRANTS_SYMBOL) && (LA720_19 <= HOSTS_SYMBOL)) || ((LA720_19 >= IDENTIFIED_SYMBOL) && (LA720_19 <= LEVEL_SYMBOL)) || ((LA720_19 >= LIST_SYMBOL) && (LA720_19 <= MESSAGE_TEXT_SYMBOL)) || LA720_19 == MIGRATE_SYMBOL || ((LA720_19 >= MIN_ROWS_SYMBOL) && (LA720_19 <= MODE_SYMBOL)) || ((LA720_19 >= MUTEX_SYMBOL) && (LA720_19 <= OFFSET_SYMBOL)) || ((LA720_19 >= ONE_SHOT_SYMBOL) && (LA720_19 <= PARTITIONS_SYMBOL)) || ((LA720_19 >= PHASE_SYMBOL) && (LA720_19 <= PLUGINS_SYMBOL)) || ((LA720_19 >= PRESERVE_SYMBOL) && (LA720_19 <= PROXY_SYMBOL)) || ((LA720_19 >= QUERY_SYMBOL) && (LA720_19 <= RETURNS_SYMBOL)) || ((LA720_19 >= ROLLUP_SYMBOL) && (LA720_19 <= ROWS_SYMBOL)) || LA720_19 == ROW_FORMAT_SYMBOL || ((LA720_19 >= RTREE_SYMBOL) && (LA720_19 <= SCHEMA_NAME_SYMBOL)) || ((LA720_19 >= SERIAL_SYMBOL) && (LA720_19 <= SHARE_SYMBOL)) || ((LA720_19 >= SLOW_SYMBOL) && (LA720_19 <= SUBCLASS_ORIGIN_SYMBOL)) || ((LA720_19 >= SUBJECT_SYMBOL) && (LA720_19 <= TRIGGERS_SYMBOL)) || ((LA720_19 >= TYPES_SYMBOL) && (LA720_19 <= USER_RESOURCES_SYMBOL)) || ((LA720_19 >= USE_FRM_SYMBOL) && (LA720_19 <= WAIT_SYMBOL)) || LA720_19 == WORK_SYMBOL || ((LA720_19 >= X509_SYMBOL) && (LA720_19 <= XML_SYMBOL)) || LA720_19 == BINARY_SYMBOL || ((LA720_19 >= CASE_SYMBOL) && (LA720_19 <= CAST_SYMBOL)) || LA720_19 == CONVERT_SYMBOL || LA720_19 == MATCH_SYMBOL))
                		    {
                		        alt720=1;
                		    }

                		}
                	}
                    break;

                }

                switch (alt720)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2228:54: COMMA_SYMBOL expression ( direction )?
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_expression_list_with_direction12821);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexpression_list_with_directionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_expression_list_with_direction12823);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleexpression_list_with_directionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2228:78: ( direction )?
            	        {
            	            int alt719=2;
            	            switch ( LA(1) )
            	            {
            	                case ASC_SYMBOL:
            	                case DESC_SYMBOL:
            	                	{
            	                		alt719=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt719)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2228:78: direction
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_direction_in_expression_list_with_direction12825);
            	        	        direction(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleexpression_list_with_directionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

            	default:
            	    goto loop720;	/* break out of the loop */
            	    break;
                }
            }
            loop720: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleexpression_list_with_directionEx; /* Prevent compiler warnings */
    ruleexpression_list_with_directionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end expression_list_with_direction */

/**
 * $ANTLR start channel
 * MySQLSimpleParser.g:2231:1: channel :{...}? => FOR_SYMBOL CHANNEL_SYMBOL text_string_no_linebreak ;
 */
static void
channel(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2231:8: ({...}? => FOR_SYMBOL CHANNEL_SYMBOL text_string_no_linebreak )
        // MySQLSimpleParser.g:2232:2: {...}? => FOR_SYMBOL CHANNEL_SYMBOL text_string_no_linebreak
        {
            if ( !((SERVER_VERSION >= 50706)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
                    EXCEPTION->ruleName	 = (void *)"channel";


            }

             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_channel12841);
            if  (HASEXCEPTION())
            {
                goto rulechannelEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CHANNEL_SYMBOL, &FOLLOW_CHANNEL_SYMBOL_in_channel12843);
            if  (HASEXCEPTION())
            {
                goto rulechannelEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_string_no_linebreak_in_channel12845);
            text_string_no_linebreak(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulechannelEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulechannelEx; /* Prevent compiler warnings */
    rulechannelEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end channel */

/**
 * $ANTLR start compound_statement
 * MySQLSimpleParser.g:2239:1: compound_statement options {k=3; } : ( statement | return_statement | if_statement | case_statement | labeled_block | unlabeled_block | labeled_control | unlabeled_control | leave_statement | iterate_statement | cursor_open | cursor_fetch | cursor_close );
 */
static void
compound_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2239:38: ( statement | return_statement | if_statement | case_statement | labeled_block | unlabeled_block | labeled_control | unlabeled_control | leave_statement | iterate_statement | cursor_open | cursor_fetch | cursor_close )

            ANTLR3_UINT32 alt721;

            alt721=13;

            {
                int LA721_0 = LA(1);
                if ( (LA721_0 == DO_SYMBOL || LA721_0 == HANDLER_SYMBOL || LA721_0 == TRUNCATE_SYMBOL || LA721_0 == ALTER_SYMBOL || LA721_0 == CALL_SYMBOL || LA721_0 == CREATE_SYMBOL || LA721_0 == DELETE_SYMBOL || LA721_0 == DROP_SYMBOL || LA721_0 == INSERT_SYMBOL || LA721_0 == LOAD_SYMBOL || LA721_0 == OPEN_PAR_SYMBOL || LA721_0 == RENAME_SYMBOL || LA721_0 == REPLACE_SYMBOL || LA721_0 == SELECT_SYMBOL || LA721_0 == UPDATE_SYMBOL))
                {
                    alt721=1;
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA721_0 == PARTITION_SYMBOL))
                {
                    alt721=1;
                }
                else if ( (LA721_0 == BACKUP_SYMBOL || LA721_0 == CACHE_SYMBOL || LA721_0 == CHECKSUM_SYMBOL || LA721_0 == COMMIT_SYMBOL || LA721_0 == DEALLOCATE_SYMBOL || ((LA721_0 >= EXECUTE_SYMBOL) && (LA721_0 <= FLUSH_SYMBOL)) || LA721_0 == HELP_SYMBOL || LA721_0 == INSTALL_SYMBOL || LA721_0 == PREPARE_SYMBOL || ((LA721_0 >= REPAIR_SYMBOL) && (LA721_0 <= SAVEPOINT_SYMBOL)) || ((LA721_0 >= START_SYMBOL) && (LA721_0 <= STOP_SYMBOL)) || LA721_0 == UNINSTALL_SYMBOL || LA721_0 == XA_SYMBOL || LA721_0 == SHUTDOWN_SYMBOL || LA721_0 == CHANGE_SYMBOL || LA721_0 == CHECK_SYMBOL || ((LA721_0 >= DESCRIBE_SYMBOL) && (LA721_0 <= DESC_SYMBOL)) || ((LA721_0 >= GET_SYMBOL) && (LA721_0 <= GRANT_SYMBOL)) || LA721_0 == KILL_SYMBOL || LA721_0 == LOCK_SYMBOL || LA721_0 == OPTIMIZE_SYMBOL || LA721_0 == PURGE_SYMBOL || LA721_0 == RELEASE_SYMBOL || LA721_0 == RESIGNAL_SYMBOL || LA721_0 == REVOKE_SYMBOL || LA721_0 == SET_SYMBOL || ((LA721_0 >= SHOW_SYMBOL) && (LA721_0 <= SIGNAL_SYMBOL)) || LA721_0 == UNLOCK_SYMBOL || LA721_0 == USE_SYMBOL))
                {
                    alt721=1;
                }
                else if ( (LA721_0 == ANALYZE_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case LOCAL_SYMBOL:
                    case NO_WRITE_TO_BINLOG_SYMBOL:
                    case TABLE_SYMBOL:
                    	{
                    		alt721=1;
                    	}
                        break;
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 75;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 34;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (LA721_0 == BINLOG_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case NCHAR_TEXT:
                    case SINGLE_QUOTED_TEXT:
                    case DOUBLE_QUOTED_TEXT:
                    case UNDERSCORE_CHARSET:
                    	{
                    		alt721=1;
                    	}
                        break;
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 80;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 44;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (LA721_0 == RETURN_SYMBOL))
                {
                    alt721=2;
                }
                else if ( (LA721_0 == IF_SYMBOL))
                {
                    alt721=3;
                }
                else if ( (LA721_0 == CASE_SYMBOL))
                {
                    alt721=4;
                }
                else if ( (LA721_0 == IDENTIFIER))
                {
                    switch ( LA(2) )
                    {
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 81;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 59;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (LA721_0 == BACK_TICK_QUOTED_ID))
                {
                    switch ( LA(2) )
                    {
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 82;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 60;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (LA721_0 == DOUBLE_QUOTED_TEXT))
                {
                    switch ( LA(2) )
                    {
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 83;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 61;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (((LA721_0 >= ACTION_SYMBOL) && (LA721_0 <= ALGORITHM_SYMBOL)) || ((LA721_0 >= ANY_SYMBOL) && (LA721_0 <= AVG_SYMBOL)) || ((LA721_0 >= BIT_SYMBOL) && (LA721_0 <= SHARE_SYMBOL)) || ((LA721_0 >= SLOW_SYMBOL) && (LA721_0 <= YEAR_SYMBOL))))
                {
                    switch ( LA(2) )
                    {
                    case COLON_SYMBOL:
                    	{
                    		switch ( LA(3) )
                    		{
                    		case BEGIN_SYMBOL:
                    			{
                    				alt721=5;
                    			}
                    		    break;
                    		case LOOP_SYMBOL:
                    		case REPEAT_SYMBOL:
                    		case WHILE_SYMBOL:
                    			{
                    				alt721=7;
                    			}
                    		    break;

                    		default:
                    		    if (BACKTRACKING>0)
                    		    {
                    		        FAILEDFLAG = ANTLR3_TRUE;
                    		        return ;
                    		    }

                    		    CONSTRUCTEX();
                    		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		    EXCEPTION->message      = (void *)"";
                    		    EXCEPTION->decisionNum  = 721;
                    		    EXCEPTION->state        = 84;


                    		    goto rulecompound_statementEx;

                    		}

                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 721;
                        EXCEPTION->state        = 62;


                        goto rulecompound_statementEx;

                    }

                }
                else if ( (LA721_0 == BEGIN_SYMBOL))
                {
                    alt721=6;
                }
                else if ( (LA721_0 == LOOP_SYMBOL || LA721_0 == REPEAT_SYMBOL || LA721_0 == WHILE_SYMBOL))
                {
                    alt721=8;
                }
                else if ( (LA721_0 == LEAVE_SYMBOL))
                {
                    alt721=9;
                }
                else if ( (LA721_0 == ITERATE_SYMBOL))
                {
                    alt721=10;
                }
                else if ( (LA721_0 == OPEN_SYMBOL))
                {
                    alt721=11;
                }
                else if ( (LA721_0 == FETCH_SYMBOL))
                {
                    alt721=12;
                }
                else if ( (LA721_0 == CLOSE_SYMBOL))
                {
                    alt721=13;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 721;
                    EXCEPTION->state        = 0;


                    goto rulecompound_statementEx;

                }
            }
            switch (alt721)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2240:2: statement
        	    {
        	        FOLLOWPUSH(FOLLOW_statement_in_compound_statement12869);
        	        statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2241:4: return_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_return_statement_in_compound_statement12874);
        	        return_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2242:4: if_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_if_statement_in_compound_statement12879);
        	        if_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2243:4: case_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_case_statement_in_compound_statement12884);
        	        case_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2244:4: labeled_block
        	    {
        	        FOLLOWPUSH(FOLLOW_labeled_block_in_compound_statement12889);
        	        labeled_block(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2245:4: unlabeled_block
        	    {
        	        FOLLOWPUSH(FOLLOW_unlabeled_block_in_compound_statement12894);
        	        unlabeled_block(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2246:4: labeled_control
        	    {
        	        FOLLOWPUSH(FOLLOW_labeled_control_in_compound_statement12899);
        	        labeled_control(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2247:4: unlabeled_control
        	    {
        	        FOLLOWPUSH(FOLLOW_unlabeled_control_in_compound_statement12904);
        	        unlabeled_control(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2248:4: leave_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_leave_statement_in_compound_statement12909);
        	        leave_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2249:4: iterate_statement
        	    {
        	        FOLLOWPUSH(FOLLOW_iterate_statement_in_compound_statement12914);
        	        iterate_statement(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2251:4: cursor_open
        	    {
        	        FOLLOWPUSH(FOLLOW_cursor_open_in_compound_statement12921);
        	        cursor_open(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2252:4: cursor_fetch
        	    {
        	        FOLLOWPUSH(FOLLOW_cursor_fetch_in_compound_statement12926);
        	        cursor_fetch(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:2253:4: cursor_close
        	    {
        	        FOLLOWPUSH(FOLLOW_cursor_close_in_compound_statement12931);
        	        cursor_close(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecompound_statementEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecompound_statementEx; /* Prevent compiler warnings */
    rulecompound_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end compound_statement */

/**
 * $ANTLR start return_statement
 * MySQLSimpleParser.g:2256:1: return_statement : RETURN_SYMBOL expression ;
 */
static void
return_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2256:17: ( RETURN_SYMBOL expression )
        // MySQLSimpleParser.g:2257:2: RETURN_SYMBOL expression
        {
             MATCHT(RETURN_SYMBOL, &FOLLOW_RETURN_SYMBOL_in_return_statement12940);
            if  (HASEXCEPTION())
            {
                goto rulereturn_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_return_statement12942);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulereturn_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulereturn_statementEx; /* Prevent compiler warnings */
    rulereturn_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end return_statement */

/**
 * $ANTLR start if_statement
 * MySQLSimpleParser.g:2260:1: if_statement : IF_SYMBOL if_body END_SYMBOL IF_SYMBOL ;
 */
static void
if_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2260:13: ( IF_SYMBOL if_body END_SYMBOL IF_SYMBOL )
        // MySQLSimpleParser.g:2261:2: IF_SYMBOL if_body END_SYMBOL IF_SYMBOL
        {
             MATCHT(IF_SYMBOL, &FOLLOW_IF_SYMBOL_in_if_statement12951);
            if  (HASEXCEPTION())
            {
                goto ruleif_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_if_body_in_if_statement12953);
            if_body(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleif_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_if_statement12955);
            if  (HASEXCEPTION())
            {
                goto ruleif_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(IF_SYMBOL, &FOLLOW_IF_SYMBOL_in_if_statement12957);
            if  (HASEXCEPTION())
            {
                goto ruleif_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleif_statementEx; /* Prevent compiler warnings */
    ruleif_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end if_statement */

/**
 * $ANTLR start if_body
 * MySQLSimpleParser.g:2264:1: if_body : expression then_statement ( ELSEIF_SYMBOL if_body | ELSE_SYMBOL compound_statement_list )? ;
 */
static void
if_body(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2264:8: ( expression then_statement ( ELSEIF_SYMBOL if_body | ELSE_SYMBOL compound_statement_list )? )
        // MySQLSimpleParser.g:2265:2: expression then_statement ( ELSEIF_SYMBOL if_body | ELSE_SYMBOL compound_statement_list )?
        {
            FOLLOWPUSH(FOLLOW_expression_in_if_body12967);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleif_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_then_statement_in_if_body12969);
            then_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleif_bodyEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2265:28: ( ELSEIF_SYMBOL if_body | ELSE_SYMBOL compound_statement_list )?
            {
                int alt722=3;
                switch ( LA(1) )
                {
                    case ELSEIF_SYMBOL:
                    	{
                    		alt722=1;
                    	}
                        break;
                    case ELSE_SYMBOL:
                    	{
                    		alt722=2;
                    	}
                        break;
                }

                switch (alt722)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2265:29: ELSEIF_SYMBOL if_body
            	    {
            	         MATCHT(ELSEIF_SYMBOL, &FOLLOW_ELSEIF_SYMBOL_in_if_body12972);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleif_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_if_body_in_if_body12974);
            	        if_body(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleif_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2265:53: ELSE_SYMBOL compound_statement_list
            	    {
            	         MATCHT(ELSE_SYMBOL, &FOLLOW_ELSE_SYMBOL_in_if_body12978);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleif_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_compound_statement_list_in_if_body12980);
            	        compound_statement_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleif_bodyEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleif_bodyEx; /* Prevent compiler warnings */
    ruleif_bodyEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end if_body */

/**
 * $ANTLR start then_statement
 * MySQLSimpleParser.g:2268:1: then_statement : THEN_SYMBOL compound_statement_list ;
 */
static void
then_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2268:15: ( THEN_SYMBOL compound_statement_list )
        // MySQLSimpleParser.g:2269:2: THEN_SYMBOL compound_statement_list
        {
             MATCHT(THEN_SYMBOL, &FOLLOW_THEN_SYMBOL_in_then_statement12991);
            if  (HASEXCEPTION())
            {
                goto rulethen_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_list_in_then_statement12993);
            compound_statement_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulethen_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulethen_statementEx; /* Prevent compiler warnings */
    rulethen_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end then_statement */

/**
 * $ANTLR start compound_statement_list
 * MySQLSimpleParser.g:2272:1: compound_statement_list : ( compound_statement SEMICOLON_SYMBOL )+ ;
 */
static void
compound_statement_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2272:24: ( ( compound_statement SEMICOLON_SYMBOL )+ )
        // MySQLSimpleParser.g:2273:2: ( compound_statement SEMICOLON_SYMBOL )+
        {
            // MySQLSimpleParser.g:2273:2: ( compound_statement SEMICOLON_SYMBOL )+
            {
                int cnt723=0;

                for (;;)
                {
                    int alt723=2;
            	{
            	   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
            	    */
            	    int LA723_0 = LA(1);
            	    if ( (LA723_0 == UNTIL_SYMBOL))
            	    {
            	        switch ( LA(2) )
            	        {
            	        case COLON_SYMBOL:
            	        	{
            	        		alt723=1;
            	        	}
            	            break;

            	        }

            	    }
            	    else if ( (LA723_0 == DO_SYMBOL || LA723_0 == HANDLER_SYMBOL || LA723_0 == TRUNCATE_SYMBOL || LA723_0 == ALTER_SYMBOL || LA723_0 == CALL_SYMBOL || LA723_0 == CREATE_SYMBOL || LA723_0 == DELETE_SYMBOL || LA723_0 == DROP_SYMBOL || LA723_0 == INSERT_SYMBOL || LA723_0 == LOAD_SYMBOL || LA723_0 == OPEN_PAR_SYMBOL || LA723_0 == RENAME_SYMBOL || LA723_0 == REPLACE_SYMBOL || LA723_0 == SELECT_SYMBOL || LA723_0 == UPDATE_SYMBOL))
            	    {
            	        alt723=1;
            	    }
            	    else if ( ((SERVER_VERSION >= 50100)) && (LA723_0 == PARTITION_SYMBOL))
            	    {
            	        alt723=1;
            	    }
            	    else if ( (((LA723_0 >= BACKUP_SYMBOL) && (LA723_0 <= BEGIN_SYMBOL)) || LA723_0 == CACHE_SYMBOL || ((LA723_0 >= CHECKSUM_SYMBOL) && (LA723_0 <= CLOSE_SYMBOL)) || LA723_0 == COMMIT_SYMBOL || LA723_0 == DEALLOCATE_SYMBOL || ((LA723_0 >= EXECUTE_SYMBOL) && (LA723_0 <= FLUSH_SYMBOL)) || LA723_0 == HELP_SYMBOL || LA723_0 == INSTALL_SYMBOL || LA723_0 == OPEN_SYMBOL || LA723_0 == PREPARE_SYMBOL || ((LA723_0 >= REPAIR_SYMBOL) && (LA723_0 <= SAVEPOINT_SYMBOL)) || ((LA723_0 >= START_SYMBOL) && (LA723_0 <= STOP_SYMBOL)) || LA723_0 == UNINSTALL_SYMBOL || ((LA723_0 >= XA_SYMBOL) && (LA723_0 <= UNKNOWN_SYMBOL)) || ((LA723_0 >= USER_RESOURCES_SYMBOL) && (LA723_0 <= YEAR_SYMBOL)) || LA723_0 == CASE_SYMBOL || LA723_0 == CHANGE_SYMBOL || LA723_0 == CHECK_SYMBOL || ((LA723_0 >= DESCRIBE_SYMBOL) && (LA723_0 <= DESC_SYMBOL)) || LA723_0 == FETCH_SYMBOL || ((LA723_0 >= GET_SYMBOL) && (LA723_0 <= GRANT_SYMBOL)) || LA723_0 == IF_SYMBOL || LA723_0 == ITERATE_SYMBOL || LA723_0 == KILL_SYMBOL || LA723_0 == LEAVE_SYMBOL || LA723_0 == LOCK_SYMBOL || LA723_0 == LOOP_SYMBOL || LA723_0 == OPTIMIZE_SYMBOL || LA723_0 == PURGE_SYMBOL || LA723_0 == RELEASE_SYMBOL || LA723_0 == REPEAT_SYMBOL || LA723_0 == RESIGNAL_SYMBOL || ((LA723_0 >= RETURN_SYMBOL) && (LA723_0 <= REVOKE_SYMBOL)) || LA723_0 == SET_SYMBOL || ((LA723_0 >= SHOW_SYMBOL) && (LA723_0 <= SIGNAL_SYMBOL)) || LA723_0 == UNLOCK_SYMBOL || LA723_0 == USE_SYMBOL || LA723_0 == WHILE_SYMBOL || LA723_0 == BACK_TICK_QUOTED_ID || LA723_0 == DOUBLE_QUOTED_TEXT || LA723_0 == IDENTIFIER))
            	    {
            	        alt723=1;
            	    }

            	}
            	switch (alt723)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2273:3: compound_statement SEMICOLON_SYMBOL
            	        {
            	            FOLLOWPUSH(FOLLOW_compound_statement_in_compound_statement_list13003);
            	            compound_statement(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecompound_statement_listEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	             MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_compound_statement_list13005);
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecompound_statement_listEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt723 >= 1 )
            		{
            		    goto loop723;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulecompound_statement_listEx;
            	}
            	cnt723++;
                }
                loop723: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecompound_statement_listEx; /* Prevent compiler warnings */
    rulecompound_statement_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end compound_statement_list */

/**
 * $ANTLR start case_statement
 * MySQLSimpleParser.g:2277:1: case_statement : CASE_SYMBOL ( expression )? ( when_expression then_statement )+ ( else_statement )? END_SYMBOL CASE_SYMBOL ;
 */
static void
case_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2277:15: ( CASE_SYMBOL ( expression )? ( when_expression then_statement )+ ( else_statement )? END_SYMBOL CASE_SYMBOL )
        // MySQLSimpleParser.g:2278:2: CASE_SYMBOL ( expression )? ( when_expression then_statement )+ ( else_statement )? END_SYMBOL CASE_SYMBOL
        {
             MATCHT(CASE_SYMBOL, &FOLLOW_CASE_SYMBOL_in_case_statement13017);
            if  (HASEXCEPTION())
            {
                goto rulecase_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2278:14: ( expression )?
            {
                int alt724=2;
                {
                    int LA724_0 = LA(1);
                    if ( (((LA724_0 >= ACCOUNT_SYMBOL) && (LA724_0 <= YEAR_SYMBOL)) || LA724_0 == BINARY_SYMBOL || ((LA724_0 >= CASE_SYMBOL) && (LA724_0 <= CAST_SYMBOL)) || LA724_0 == CHAR_SYMBOL || ((LA724_0 >= CONVERT_SYMBOL) && (LA724_0 <= COUNT_SYMBOL)) || LA724_0 == CURDATE_SYMBOL || LA724_0 == CURRENT_USER_SYMBOL || LA724_0 == CURTIME_SYMBOL || LA724_0 == DATABASE_SYMBOL || LA724_0 == DATE_ADD_SYMBOL || LA724_0 == DATE_SUB_SYMBOL || LA724_0 == EXISTS_SYMBOL || ((LA724_0 >= EXTRACT_SYMBOL) && (LA724_0 <= FALSE_SYMBOL)) || LA724_0 == GROUP_CONCAT_SYMBOL || LA724_0 == IF_SYMBOL || LA724_0 == INSERT_SYMBOL || LA724_0 == INTERVAL_SYMBOL || LA724_0 == LEFT_SYMBOL || LA724_0 == MATCH_SYMBOL || LA724_0 == MAX_SYMBOL || LA724_0 == MIN_SYMBOL || LA724_0 == MOD_SYMBOL || LA724_0 == NCHAR_TEXT || ((LA724_0 >= NOT2_SYMBOL) && (LA724_0 <= NOW_SYMBOL)) || ((LA724_0 >= NULL2_SYMBOL) && (LA724_0 <= NULL_SYMBOL)) || ((LA724_0 >= OPEN_CURLY_SYMBOL) && (LA724_0 <= OPEN_PAR_SYMBOL)) || ((LA724_0 >= PARAM_MARKER) && (LA724_0 <= POSITION_SYMBOL)) || ((LA724_0 >= REPEAT_SYMBOL) && (LA724_0 <= REPLACE_SYMBOL)) || LA724_0 == RIGHT_SYMBOL || LA724_0 == SINGLE_QUOTED_TEXT || LA724_0 == STDDEV_SAMP_SYMBOL || LA724_0 == STD_SYMBOL || LA724_0 == SUBSTRING_SYMBOL || ((LA724_0 >= SUM_SYMBOL) && (LA724_0 <= SYSDATE_SYMBOL)) || ((LA724_0 >= TRIM_SYMBOL) && (LA724_0 <= TRUE_SYMBOL)) || ((LA724_0 >= UTC_DATE_SYMBOL) && (LA724_0 <= UTC_TIME_SYMBOL)) || LA724_0 == VALUES_SYMBOL || LA724_0 == VARIANCE_SYMBOL || LA724_0 == VAR_SAMP_SYMBOL || ((LA724_0 >= AT_AT_SIGN_SYMBOL) && (LA724_0 <= AT_TEXT_SUFFIX)) || ((LA724_0 >= BACK_TICK_QUOTED_ID) && (LA724_0 <= BIN_NUMBER)) || LA724_0 == BITWISE_NOT_OPERATOR || ((LA724_0 >= BIT_AND_SYMBOL) && (LA724_0 <= BIT_XOR_SYMBOL)) || LA724_0 == DECIMAL_NUMBER || LA724_0 == DOT_SYMBOL || LA724_0 == DOUBLE_QUOTED_TEXT || LA724_0 == FLOAT_NUMBER || ((LA724_0 >= HEX_NUMBER) && (LA724_0 <= IDENTIFIER)) || LA724_0 == INT_NUMBER || LA724_0 == LOGICAL_NOT_OPERATOR || ((LA724_0 >= LONG_NUMBER) && (LA724_0 <= MINUS_OPERATOR)) || LA724_0 == PLUS_OPERATOR || ((LA724_0 >= ULONGLONG_NUMBER) && (LA724_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt724=1;
                    }
                }
                switch (alt724)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2278:14: expression
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_case_statement13019);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecase_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2278:26: ( when_expression then_statement )+
            {
                int cnt725=0;

                for (;;)
                {
                    int alt725=2;
            	switch ( LA(1) )
            	{
            	case WHEN_SYMBOL:
            		{
            			alt725=1;
            		}
            	    break;

            	}

            	switch (alt725)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2278:27: when_expression then_statement
            	        {
            	            FOLLOWPUSH(FOLLOW_when_expression_in_case_statement13023);
            	            when_expression(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecase_statementEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	            FOLLOWPUSH(FOLLOW_then_statement_in_case_statement13025);
            	            then_statement(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulecase_statementEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt725 >= 1 )
            		{
            		    goto loop725;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulecase_statementEx;
            	}
            	cnt725++;
                }
                loop725: ;	/* Jump to here if this rule does not match */
            }

            // MySQLSimpleParser.g:2278:60: ( else_statement )?
            {
                int alt726=2;
                switch ( LA(1) )
                {
                    case ELSE_SYMBOL:
                    	{
                    		alt726=1;
                    	}
                        break;
                }

                switch (alt726)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2278:60: else_statement
            	    {
            	        FOLLOWPUSH(FOLLOW_else_statement_in_case_statement13029);
            	        else_statement(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecase_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_case_statement13032);
            if  (HASEXCEPTION())
            {
                goto rulecase_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CASE_SYMBOL, &FOLLOW_CASE_SYMBOL_in_case_statement13034);
            if  (HASEXCEPTION())
            {
                goto rulecase_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecase_statementEx; /* Prevent compiler warnings */
    rulecase_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end case_statement */

/**
 * $ANTLR start else_statement
 * MySQLSimpleParser.g:2281:1: else_statement : ELSE_SYMBOL compound_statement_list ;
 */
static void
else_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2281:15: ( ELSE_SYMBOL compound_statement_list )
        // MySQLSimpleParser.g:2282:2: ELSE_SYMBOL compound_statement_list
        {
             MATCHT(ELSE_SYMBOL, &FOLLOW_ELSE_SYMBOL_in_else_statement13043);
            if  (HASEXCEPTION())
            {
                goto ruleelse_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_list_in_else_statement13045);
            compound_statement_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleelse_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleelse_statementEx; /* Prevent compiler warnings */
    ruleelse_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end else_statement */

/**
 * $ANTLR start labeled_block
 * MySQLSimpleParser.g:2285:1: labeled_block : label begin_end_block ( label_identifier )? ;
 */
static void
labeled_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2285:14: ( label begin_end_block ( label_identifier )? )
        // MySQLSimpleParser.g:2286:2: label begin_end_block ( label_identifier )?
        {
            FOLLOWPUSH(FOLLOW_label_in_labeled_block13054);
            label(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelabeled_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_begin_end_block_in_labeled_block13056);
            begin_end_block(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelabeled_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2286:24: ( label_identifier )?
            {
                int alt727=2;
                {
                    int LA727_0 = LA(1);
                    if ( (((LA727_0 >= ACTION_SYMBOL) && (LA727_0 <= SHARE_SYMBOL)) || ((LA727_0 >= SLOW_SYMBOL) && (LA727_0 <= YEAR_SYMBOL)) || LA727_0 == BACK_TICK_QUOTED_ID || LA727_0 == DOUBLE_QUOTED_TEXT || LA727_0 == IDENTIFIER))
                    {
                        alt727=1;
                    }
                }
                switch (alt727)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2286:24: label_identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_label_identifier_in_labeled_block13058);
            	        label_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelabeled_blockEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulelabeled_blockEx; /* Prevent compiler warnings */
    rulelabeled_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end labeled_block */

/**
 * $ANTLR start unlabeled_block
 * MySQLSimpleParser.g:2289:1: unlabeled_block : begin_end_block ;
 */
static void
unlabeled_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2289:16: ( begin_end_block )
        // MySQLSimpleParser.g:2290:2: begin_end_block
        {
            FOLLOWPUSH(FOLLOW_begin_end_block_in_unlabeled_block13069);
            begin_end_block(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleunlabeled_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleunlabeled_blockEx; /* Prevent compiler warnings */
    ruleunlabeled_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end unlabeled_block */

/**
 * $ANTLR start label
 * MySQLSimpleParser.g:2293:1: label : label_identifier COLON_SYMBOL ;
 */
static void
label(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2293:6: ( label_identifier COLON_SYMBOL )
        // MySQLSimpleParser.g:2295:2: label_identifier COLON_SYMBOL
        {
            FOLLOWPUSH(FOLLOW_label_identifier_in_label13081);
            label_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelabelEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(COLON_SYMBOL, &FOLLOW_COLON_SYMBOL_in_label13083);
            if  (HASEXCEPTION())
            {
                goto rulelabelEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelabelEx; /* Prevent compiler warnings */
    rulelabelEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end label */

/**
 * $ANTLR start label_identifier
 * MySQLSimpleParser.g:2298:1: label_identifier : ( pure_identifier | keyword_sp );
 */
static void
label_identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2298:17: ( pure_identifier | keyword_sp )

            ANTLR3_UINT32 alt728;

            alt728=2;

            {
                int LA728_0 = LA(1);
                if ( (LA728_0 == BACK_TICK_QUOTED_ID || LA728_0 == DOUBLE_QUOTED_TEXT || LA728_0 == IDENTIFIER))
                {
                    alt728=1;
                }
                else if ( (((LA728_0 >= ACTION_SYMBOL) && (LA728_0 <= SHARE_SYMBOL)) || ((LA728_0 >= SLOW_SYMBOL) && (LA728_0 <= YEAR_SYMBOL))))
                {
                    alt728=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 728;
                    EXCEPTION->state        = 0;


                    goto rulelabel_identifierEx;

                }
            }
            switch (alt728)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2299:2: pure_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_pure_identifier_in_label_identifier13092);
        	        pure_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelabel_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2299:20: keyword_sp
        	    {
        	        FOLLOWPUSH(FOLLOW_keyword_sp_in_label_identifier13096);
        	        keyword_sp(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelabel_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelabel_identifierEx; /* Prevent compiler warnings */
    rulelabel_identifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end label_identifier */

/**
 * $ANTLR start begin_end_block
 * MySQLSimpleParser.g:2302:1: begin_end_block : BEGIN_SYMBOL ( sp_declarations )? ( compound_statement_list )? END_SYMBOL ;
 */
static void
begin_end_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2302:16: ( BEGIN_SYMBOL ( sp_declarations )? ( compound_statement_list )? END_SYMBOL )
        // MySQLSimpleParser.g:2303:2: BEGIN_SYMBOL ( sp_declarations )? ( compound_statement_list )? END_SYMBOL
        {
             MATCHT(BEGIN_SYMBOL, &FOLLOW_BEGIN_SYMBOL_in_begin_end_block13106);
            if  (HASEXCEPTION())
            {
                goto rulebegin_end_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2303:15: ( sp_declarations )?
            {
                int alt729=2;
                switch ( LA(1) )
                {
                    case DECLARE_SYMBOL:
                    	{
                    		alt729=1;
                    	}
                        break;
                }

                switch (alt729)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2303:15: sp_declarations
            	    {
            	        FOLLOWPUSH(FOLLOW_sp_declarations_in_begin_end_block13108);
            	        sp_declarations(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebegin_end_blockEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2303:32: ( compound_statement_list )?
            {
                int alt730=2;
                {
                    int LA730_0 = LA(1);
                    if ( (LA730_0 == DO_SYMBOL || LA730_0 == HANDLER_SYMBOL || LA730_0 == TRUNCATE_SYMBOL || LA730_0 == ALTER_SYMBOL || LA730_0 == CALL_SYMBOL || LA730_0 == CREATE_SYMBOL || LA730_0 == DELETE_SYMBOL || LA730_0 == DROP_SYMBOL || LA730_0 == INSERT_SYMBOL || LA730_0 == LOAD_SYMBOL || LA730_0 == OPEN_PAR_SYMBOL || LA730_0 == RENAME_SYMBOL || LA730_0 == REPLACE_SYMBOL || LA730_0 == SELECT_SYMBOL || LA730_0 == UPDATE_SYMBOL))
                    {
                        alt730=1;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA730_0 == PARTITION_SYMBOL))
                    {
                        alt730=1;
                    }
                    else if ( (((LA730_0 >= BACKUP_SYMBOL) && (LA730_0 <= BEGIN_SYMBOL)) || LA730_0 == CACHE_SYMBOL || ((LA730_0 >= CHECKSUM_SYMBOL) && (LA730_0 <= CLOSE_SYMBOL)) || LA730_0 == COMMIT_SYMBOL || LA730_0 == DEALLOCATE_SYMBOL || ((LA730_0 >= EXECUTE_SYMBOL) && (LA730_0 <= FLUSH_SYMBOL)) || LA730_0 == HELP_SYMBOL || LA730_0 == INSTALL_SYMBOL || LA730_0 == OPEN_SYMBOL || LA730_0 == PREPARE_SYMBOL || ((LA730_0 >= REPAIR_SYMBOL) && (LA730_0 <= SAVEPOINT_SYMBOL)) || ((LA730_0 >= START_SYMBOL) && (LA730_0 <= STOP_SYMBOL)) || LA730_0 == UNINSTALL_SYMBOL || ((LA730_0 >= XA_SYMBOL) && (LA730_0 <= YEAR_SYMBOL)) || LA730_0 == CASE_SYMBOL || LA730_0 == CHANGE_SYMBOL || LA730_0 == CHECK_SYMBOL || ((LA730_0 >= DESCRIBE_SYMBOL) && (LA730_0 <= DESC_SYMBOL)) || LA730_0 == FETCH_SYMBOL || ((LA730_0 >= GET_SYMBOL) && (LA730_0 <= GRANT_SYMBOL)) || LA730_0 == IF_SYMBOL || LA730_0 == ITERATE_SYMBOL || LA730_0 == KILL_SYMBOL || LA730_0 == LEAVE_SYMBOL || LA730_0 == LOCK_SYMBOL || LA730_0 == LOOP_SYMBOL || LA730_0 == OPTIMIZE_SYMBOL || LA730_0 == PURGE_SYMBOL || LA730_0 == RELEASE_SYMBOL || LA730_0 == REPEAT_SYMBOL || LA730_0 == RESIGNAL_SYMBOL || ((LA730_0 >= RETURN_SYMBOL) && (LA730_0 <= REVOKE_SYMBOL)) || LA730_0 == SET_SYMBOL || ((LA730_0 >= SHOW_SYMBOL) && (LA730_0 <= SIGNAL_SYMBOL)) || LA730_0 == UNLOCK_SYMBOL || LA730_0 == USE_SYMBOL || LA730_0 == WHILE_SYMBOL || LA730_0 == BACK_TICK_QUOTED_ID || LA730_0 == DOUBLE_QUOTED_TEXT || LA730_0 == IDENTIFIER))
                    {
                        alt730=1;
                    }
                }
                switch (alt730)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2303:32: compound_statement_list
            	    {
            	        FOLLOWPUSH(FOLLOW_compound_statement_list_in_begin_end_block13111);
            	        compound_statement_list(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulebegin_end_blockEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_begin_end_block13114);
            if  (HASEXCEPTION())
            {
                goto rulebegin_end_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulebegin_end_blockEx; /* Prevent compiler warnings */
    rulebegin_end_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end begin_end_block */

/**
 * $ANTLR start labeled_control
 * MySQLSimpleParser.g:2306:1: labeled_control : label unlabeled_control ( label_identifier )? ;
 */
static void
labeled_control(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2306:16: ( label unlabeled_control ( label_identifier )? )
        // MySQLSimpleParser.g:2307:2: label unlabeled_control ( label_identifier )?
        {
            FOLLOWPUSH(FOLLOW_label_in_labeled_control13123);
            label(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelabeled_controlEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_unlabeled_control_in_labeled_control13125);
            unlabeled_control(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelabeled_controlEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2307:26: ( label_identifier )?
            {
                int alt731=2;
                {
                    int LA731_0 = LA(1);
                    if ( (((LA731_0 >= ACTION_SYMBOL) && (LA731_0 <= SHARE_SYMBOL)) || ((LA731_0 >= SLOW_SYMBOL) && (LA731_0 <= YEAR_SYMBOL)) || LA731_0 == BACK_TICK_QUOTED_ID || LA731_0 == DOUBLE_QUOTED_TEXT || LA731_0 == IDENTIFIER))
                    {
                        alt731=1;
                    }
                }
                switch (alt731)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2307:26: label_identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_label_identifier_in_labeled_control13127);
            	        label_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulelabeled_controlEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulelabeled_controlEx; /* Prevent compiler warnings */
    rulelabeled_controlEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end labeled_control */

/**
 * $ANTLR start unlabeled_control
 * MySQLSimpleParser.g:2310:1: unlabeled_control : ( loop_block | while_do_block | repeat_until_block );
 */
static void
unlabeled_control(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2310:18: ( loop_block | while_do_block | repeat_until_block )

            ANTLR3_UINT32 alt732;

            alt732=3;

            switch ( LA(1) )
            {
            case LOOP_SYMBOL:
            	{
            		alt732=1;
            	}
                break;
            case WHILE_SYMBOL:
            	{
            		alt732=2;
            	}
                break;
            case REPEAT_SYMBOL:
            	{
            		alt732=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 732;
                EXCEPTION->state        = 0;


                goto ruleunlabeled_controlEx;

            }

            switch (alt732)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2311:2: loop_block
        	    {
        	        FOLLOWPUSH(FOLLOW_loop_block_in_unlabeled_control13137);
        	        loop_block(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunlabeled_controlEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2312:4: while_do_block
        	    {
        	        FOLLOWPUSH(FOLLOW_while_do_block_in_unlabeled_control13142);
        	        while_do_block(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunlabeled_controlEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2313:4: repeat_until_block
        	    {
        	        FOLLOWPUSH(FOLLOW_repeat_until_block_in_unlabeled_control13147);
        	        repeat_until_block(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunlabeled_controlEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleunlabeled_controlEx; /* Prevent compiler warnings */
    ruleunlabeled_controlEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end unlabeled_control */

/**
 * $ANTLR start loop_block
 * MySQLSimpleParser.g:2316:1: loop_block : LOOP_SYMBOL compound_statement_list END_SYMBOL LOOP_SYMBOL ;
 */
static void
loop_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2316:11: ( LOOP_SYMBOL compound_statement_list END_SYMBOL LOOP_SYMBOL )
        // MySQLSimpleParser.g:2317:2: LOOP_SYMBOL compound_statement_list END_SYMBOL LOOP_SYMBOL
        {
             MATCHT(LOOP_SYMBOL, &FOLLOW_LOOP_SYMBOL_in_loop_block13156);
            if  (HASEXCEPTION())
            {
                goto ruleloop_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_list_in_loop_block13158);
            compound_statement_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleloop_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_loop_block13160);
            if  (HASEXCEPTION())
            {
                goto ruleloop_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(LOOP_SYMBOL, &FOLLOW_LOOP_SYMBOL_in_loop_block13162);
            if  (HASEXCEPTION())
            {
                goto ruleloop_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleloop_blockEx; /* Prevent compiler warnings */
    ruleloop_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end loop_block */

/**
 * $ANTLR start while_do_block
 * MySQLSimpleParser.g:2320:1: while_do_block : WHILE_SYMBOL expression DO_SYMBOL compound_statement_list END_SYMBOL WHILE_SYMBOL ;
 */
static void
while_do_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2320:15: ( WHILE_SYMBOL expression DO_SYMBOL compound_statement_list END_SYMBOL WHILE_SYMBOL )
        // MySQLSimpleParser.g:2321:2: WHILE_SYMBOL expression DO_SYMBOL compound_statement_list END_SYMBOL WHILE_SYMBOL
        {
             MATCHT(WHILE_SYMBOL, &FOLLOW_WHILE_SYMBOL_in_while_do_block13171);
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_while_do_block13173);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(DO_SYMBOL, &FOLLOW_DO_SYMBOL_in_while_do_block13175);
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_list_in_while_do_block13177);
            compound_statement_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_while_do_block13179);
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(WHILE_SYMBOL, &FOLLOW_WHILE_SYMBOL_in_while_do_block13181);
            if  (HASEXCEPTION())
            {
                goto rulewhile_do_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulewhile_do_blockEx; /* Prevent compiler warnings */
    rulewhile_do_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end while_do_block */

/**
 * $ANTLR start repeat_until_block
 * MySQLSimpleParser.g:2324:1: repeat_until_block : REPEAT_SYMBOL compound_statement_list UNTIL_SYMBOL expression END_SYMBOL REPEAT_SYMBOL ;
 */
static void
repeat_until_block(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2324:19: ( REPEAT_SYMBOL compound_statement_list UNTIL_SYMBOL expression END_SYMBOL REPEAT_SYMBOL )
        // MySQLSimpleParser.g:2325:2: REPEAT_SYMBOL compound_statement_list UNTIL_SYMBOL expression END_SYMBOL REPEAT_SYMBOL
        {
             MATCHT(REPEAT_SYMBOL, &FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13190);
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_compound_statement_list_in_repeat_until_block13192);
            compound_statement_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(UNTIL_SYMBOL, &FOLLOW_UNTIL_SYMBOL_in_repeat_until_block13194);
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_expression_in_repeat_until_block13196);
            expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(END_SYMBOL, &FOLLOW_END_SYMBOL_in_repeat_until_block13198);
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(REPEAT_SYMBOL, &FOLLOW_REPEAT_SYMBOL_in_repeat_until_block13200);
            if  (HASEXCEPTION())
            {
                goto rulerepeat_until_blockEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulerepeat_until_blockEx; /* Prevent compiler warnings */
    rulerepeat_until_blockEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end repeat_until_block */

/**
 * $ANTLR start sp_declarations
 * MySQLSimpleParser.g:2328:1: sp_declarations : ( sp_declaration SEMICOLON_SYMBOL )+ ;
 */
static void
sp_declarations(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2328:16: ( ( sp_declaration SEMICOLON_SYMBOL )+ )
        // MySQLSimpleParser.g:2329:2: ( sp_declaration SEMICOLON_SYMBOL )+
        {
            // MySQLSimpleParser.g:2329:2: ( sp_declaration SEMICOLON_SYMBOL )+
            {
                int cnt733=0;

                for (;;)
                {
                    int alt733=2;
            	switch ( LA(1) )
            	{
            	case DECLARE_SYMBOL:
            		{
            			alt733=1;
            		}
            	    break;

            	}

            	switch (alt733)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2329:3: sp_declaration SEMICOLON_SYMBOL
            	        {
            	            FOLLOWPUSH(FOLLOW_sp_declaration_in_sp_declarations13210);
            	            sp_declaration(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulesp_declarationsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	             MATCHT(SEMICOLON_SYMBOL, &FOLLOW_SEMICOLON_SYMBOL_in_sp_declarations13212);
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulesp_declarationsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt733 >= 1 )
            		{
            		    goto loop733;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulesp_declarationsEx;
            	}
            	cnt733++;
                }
                loop733: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulesp_declarationsEx; /* Prevent compiler warnings */
    rulesp_declarationsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end sp_declarations */

/**
 * $ANTLR start sp_declaration
 * MySQLSimpleParser.g:2332:1: sp_declaration : DECLARE_SYMBOL ( variable_declaration | condition_declaration | handler_declaration | cursor_declaration ) ;
 */
static void
sp_declaration(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2332:15: ( DECLARE_SYMBOL ( variable_declaration | condition_declaration | handler_declaration | cursor_declaration ) )
        // MySQLSimpleParser.g:2333:2: DECLARE_SYMBOL ( variable_declaration | condition_declaration | handler_declaration | cursor_declaration )
        {
             MATCHT(DECLARE_SYMBOL, &FOLLOW_DECLARE_SYMBOL_in_sp_declaration13223);
            if  (HASEXCEPTION())
            {
                goto rulesp_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2334:2: ( variable_declaration | condition_declaration | handler_declaration | cursor_declaration )
            {
                int alt734=4;
                {
                    int LA734_0 = LA(1);
                    if ( (LA734_0 == IDENTIFIER))
                    {
                        {
                            int LA734_1 = LA(2);
                            if ( (LA734_1 == BIT_SYMBOL || ((LA734_1 >= BOOL_SYMBOL) && (LA734_1 <= BOOLEAN_SYMBOL)) || ((LA734_1 >= DATETIME_SYMBOL) && (LA734_1 <= DATE_SYMBOL)) || LA734_1 == ENUM_SYMBOL || LA734_1 == FIXED_SYMBOL || ((LA734_1 >= GEOMETRY_SYMBOL) && (LA734_1 <= GEOMETRYCOLLECTION_SYMBOL)) || LA734_1 == JSON_SYMBOL || LA734_1 == LINESTRING_SYMBOL || ((LA734_1 >= MULTILINESTRING_SYMBOL) && (LA734_1 <= MULTIPOLYGON_SYMBOL)) || ((LA734_1 >= NATIONAL_SYMBOL) && (LA734_1 <= NCHAR_SYMBOL)) || LA734_1 == NVARCHAR_SYMBOL || ((LA734_1 >= POINT_SYMBOL) && (LA734_1 <= POLYGON_SYMBOL)) || LA734_1 == SERIAL_SYMBOL || LA734_1 == TEXT_SYMBOL || LA734_1 == TIMESTAMP_SYMBOL || LA734_1 == TIME_SYMBOL || LA734_1 == YEAR_SYMBOL || ((LA734_1 >= BIGINT_SYMBOL) && (LA734_1 <= BINARY_SYMBOL)) || LA734_1 == BLOB_SYMBOL || LA734_1 == CHAR_SYMBOL || LA734_1 == DECIMAL_SYMBOL || LA734_1 == FLOAT_SYMBOL || LA734_1 == INT_SYMBOL || ((LA734_1 >= LONGBLOB_SYMBOL) && (LA734_1 <= LONGTEXT_SYMBOL)) || LA734_1 == LONG_SYMBOL || ((LA734_1 >= MEDIUMBLOB_SYMBOL) && (LA734_1 <= MEDIUMTEXT_SYMBOL)) || LA734_1 == NUMERIC_SYMBOL || LA734_1 == REAL_SYMBOL || LA734_1 == SET_SYMBOL || LA734_1 == SMALLINT_SYMBOL || ((LA734_1 >= TINYBLOB_SYMBOL) && (LA734_1 <= TINYTEXT_SYMBOL)) || LA734_1 == VARBINARY_SYMBOL || LA734_1 == VARCHAR_SYMBOL || LA734_1 == COMMA_SYMBOL || LA734_1 == DOUBLE_SYMBOL))
                            {
                                alt734=1;
                            }
                            else if ( (LA734_1 == CONDITION_SYMBOL))
                            {
                                alt734=2;
                            }
                            else if ( (LA734_1 == CURSOR_SYMBOL))
                            {
                                alt734=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 734;
                                EXCEPTION->state        = 1;


                                goto rulesp_declarationEx;

                            }
                        }
                    }
                    else if ( (LA734_0 == BACK_TICK_QUOTED_ID))
                    {
                        {
                            int LA734_2 = LA(2);
                            if ( (LA734_2 == BIT_SYMBOL || ((LA734_2 >= BOOL_SYMBOL) && (LA734_2 <= BOOLEAN_SYMBOL)) || ((LA734_2 >= DATETIME_SYMBOL) && (LA734_2 <= DATE_SYMBOL)) || LA734_2 == ENUM_SYMBOL || LA734_2 == FIXED_SYMBOL || ((LA734_2 >= GEOMETRY_SYMBOL) && (LA734_2 <= GEOMETRYCOLLECTION_SYMBOL)) || LA734_2 == JSON_SYMBOL || LA734_2 == LINESTRING_SYMBOL || ((LA734_2 >= MULTILINESTRING_SYMBOL) && (LA734_2 <= MULTIPOLYGON_SYMBOL)) || ((LA734_2 >= NATIONAL_SYMBOL) && (LA734_2 <= NCHAR_SYMBOL)) || LA734_2 == NVARCHAR_SYMBOL || ((LA734_2 >= POINT_SYMBOL) && (LA734_2 <= POLYGON_SYMBOL)) || LA734_2 == SERIAL_SYMBOL || LA734_2 == TEXT_SYMBOL || LA734_2 == TIMESTAMP_SYMBOL || LA734_2 == TIME_SYMBOL || LA734_2 == YEAR_SYMBOL || ((LA734_2 >= BIGINT_SYMBOL) && (LA734_2 <= BINARY_SYMBOL)) || LA734_2 == BLOB_SYMBOL || LA734_2 == CHAR_SYMBOL || LA734_2 == DECIMAL_SYMBOL || LA734_2 == FLOAT_SYMBOL || LA734_2 == INT_SYMBOL || ((LA734_2 >= LONGBLOB_SYMBOL) && (LA734_2 <= LONGTEXT_SYMBOL)) || LA734_2 == LONG_SYMBOL || ((LA734_2 >= MEDIUMBLOB_SYMBOL) && (LA734_2 <= MEDIUMTEXT_SYMBOL)) || LA734_2 == NUMERIC_SYMBOL || LA734_2 == REAL_SYMBOL || LA734_2 == SET_SYMBOL || LA734_2 == SMALLINT_SYMBOL || ((LA734_2 >= TINYBLOB_SYMBOL) && (LA734_2 <= TINYTEXT_SYMBOL)) || LA734_2 == VARBINARY_SYMBOL || LA734_2 == VARCHAR_SYMBOL || LA734_2 == COMMA_SYMBOL || LA734_2 == DOUBLE_SYMBOL))
                            {
                                alt734=1;
                            }
                            else if ( (LA734_2 == CONDITION_SYMBOL))
                            {
                                alt734=2;
                            }
                            else if ( (LA734_2 == CURSOR_SYMBOL))
                            {
                                alt734=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 734;
                                EXCEPTION->state        = 2;


                                goto rulesp_declarationEx;

                            }
                        }
                    }
                    else if ( (LA734_0 == DOUBLE_QUOTED_TEXT))
                    {
                        {
                            int LA734_3 = LA(2);
                            if ( (LA734_3 == BIT_SYMBOL || ((LA734_3 >= BOOL_SYMBOL) && (LA734_3 <= BOOLEAN_SYMBOL)) || ((LA734_3 >= DATETIME_SYMBOL) && (LA734_3 <= DATE_SYMBOL)) || LA734_3 == ENUM_SYMBOL || LA734_3 == FIXED_SYMBOL || ((LA734_3 >= GEOMETRY_SYMBOL) && (LA734_3 <= GEOMETRYCOLLECTION_SYMBOL)) || LA734_3 == JSON_SYMBOL || LA734_3 == LINESTRING_SYMBOL || ((LA734_3 >= MULTILINESTRING_SYMBOL) && (LA734_3 <= MULTIPOLYGON_SYMBOL)) || ((LA734_3 >= NATIONAL_SYMBOL) && (LA734_3 <= NCHAR_SYMBOL)) || LA734_3 == NVARCHAR_SYMBOL || ((LA734_3 >= POINT_SYMBOL) && (LA734_3 <= POLYGON_SYMBOL)) || LA734_3 == SERIAL_SYMBOL || LA734_3 == TEXT_SYMBOL || LA734_3 == TIMESTAMP_SYMBOL || LA734_3 == TIME_SYMBOL || LA734_3 == YEAR_SYMBOL || ((LA734_3 >= BIGINT_SYMBOL) && (LA734_3 <= BINARY_SYMBOL)) || LA734_3 == BLOB_SYMBOL || LA734_3 == CHAR_SYMBOL || LA734_3 == DECIMAL_SYMBOL || LA734_3 == FLOAT_SYMBOL || LA734_3 == INT_SYMBOL || ((LA734_3 >= LONGBLOB_SYMBOL) && (LA734_3 <= LONGTEXT_SYMBOL)) || LA734_3 == LONG_SYMBOL || ((LA734_3 >= MEDIUMBLOB_SYMBOL) && (LA734_3 <= MEDIUMTEXT_SYMBOL)) || LA734_3 == NUMERIC_SYMBOL || LA734_3 == REAL_SYMBOL || LA734_3 == SET_SYMBOL || LA734_3 == SMALLINT_SYMBOL || ((LA734_3 >= TINYBLOB_SYMBOL) && (LA734_3 <= TINYTEXT_SYMBOL)) || LA734_3 == VARBINARY_SYMBOL || LA734_3 == VARCHAR_SYMBOL || LA734_3 == COMMA_SYMBOL || LA734_3 == DOUBLE_SYMBOL))
                            {
                                alt734=1;
                            }
                            else if ( (LA734_3 == CONDITION_SYMBOL))
                            {
                                alt734=2;
                            }
                            else if ( (LA734_3 == CURSOR_SYMBOL))
                            {
                                alt734=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 734;
                                EXCEPTION->state        = 3;


                                goto rulesp_declarationEx;

                            }
                        }
                    }
                    else if ( (((LA734_0 >= ACTION_SYMBOL) && (LA734_0 <= SHARE_SYMBOL)) || ((LA734_0 >= SLOW_SYMBOL) && (LA734_0 <= YEAR_SYMBOL))))
                    {
                        {
                            int LA734_4 = LA(2);
                            if ( (LA734_4 == BIT_SYMBOL || ((LA734_4 >= BOOL_SYMBOL) && (LA734_4 <= BOOLEAN_SYMBOL)) || ((LA734_4 >= DATETIME_SYMBOL) && (LA734_4 <= DATE_SYMBOL)) || LA734_4 == ENUM_SYMBOL || LA734_4 == FIXED_SYMBOL || ((LA734_4 >= GEOMETRY_SYMBOL) && (LA734_4 <= GEOMETRYCOLLECTION_SYMBOL)) || LA734_4 == JSON_SYMBOL || LA734_4 == LINESTRING_SYMBOL || ((LA734_4 >= MULTILINESTRING_SYMBOL) && (LA734_4 <= MULTIPOLYGON_SYMBOL)) || ((LA734_4 >= NATIONAL_SYMBOL) && (LA734_4 <= NCHAR_SYMBOL)) || LA734_4 == NVARCHAR_SYMBOL || ((LA734_4 >= POINT_SYMBOL) && (LA734_4 <= POLYGON_SYMBOL)) || LA734_4 == SERIAL_SYMBOL || LA734_4 == TEXT_SYMBOL || LA734_4 == TIMESTAMP_SYMBOL || LA734_4 == TIME_SYMBOL || LA734_4 == YEAR_SYMBOL || ((LA734_4 >= BIGINT_SYMBOL) && (LA734_4 <= BINARY_SYMBOL)) || LA734_4 == BLOB_SYMBOL || LA734_4 == CHAR_SYMBOL || LA734_4 == DECIMAL_SYMBOL || LA734_4 == FLOAT_SYMBOL || LA734_4 == INT_SYMBOL || ((LA734_4 >= LONGBLOB_SYMBOL) && (LA734_4 <= LONGTEXT_SYMBOL)) || LA734_4 == LONG_SYMBOL || ((LA734_4 >= MEDIUMBLOB_SYMBOL) && (LA734_4 <= MEDIUMTEXT_SYMBOL)) || LA734_4 == NUMERIC_SYMBOL || LA734_4 == REAL_SYMBOL || LA734_4 == SET_SYMBOL || LA734_4 == SMALLINT_SYMBOL || ((LA734_4 >= TINYBLOB_SYMBOL) && (LA734_4 <= TINYTEXT_SYMBOL)) || LA734_4 == VARBINARY_SYMBOL || LA734_4 == VARCHAR_SYMBOL || LA734_4 == COMMA_SYMBOL || LA734_4 == DOUBLE_SYMBOL))
                            {
                                alt734=1;
                            }
                            else if ( (LA734_4 == CONDITION_SYMBOL))
                            {
                                alt734=2;
                            }
                            else if ( (LA734_4 == CURSOR_SYMBOL))
                            {
                                alt734=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 734;
                                EXCEPTION->state        = 4;


                                goto rulesp_declarationEx;

                            }
                        }
                    }
                    else if ( (((LA734_0 >= ACCOUNT_SYMBOL) && (LA734_0 <= XA_SYMBOL)) || LA734_0 == SHUTDOWN_SYMBOL))
                    {
                        {
                            int LA734_5 = LA(2);
                            if ( (LA734_5 == BIT_SYMBOL || ((LA734_5 >= BOOL_SYMBOL) && (LA734_5 <= BOOLEAN_SYMBOL)) || ((LA734_5 >= DATETIME_SYMBOL) && (LA734_5 <= DATE_SYMBOL)) || LA734_5 == ENUM_SYMBOL || LA734_5 == FIXED_SYMBOL || ((LA734_5 >= GEOMETRY_SYMBOL) && (LA734_5 <= GEOMETRYCOLLECTION_SYMBOL)) || LA734_5 == JSON_SYMBOL || LA734_5 == LINESTRING_SYMBOL || ((LA734_5 >= MULTILINESTRING_SYMBOL) && (LA734_5 <= MULTIPOLYGON_SYMBOL)) || ((LA734_5 >= NATIONAL_SYMBOL) && (LA734_5 <= NCHAR_SYMBOL)) || LA734_5 == NVARCHAR_SYMBOL || ((LA734_5 >= POINT_SYMBOL) && (LA734_5 <= POLYGON_SYMBOL)) || LA734_5 == SERIAL_SYMBOL || LA734_5 == TEXT_SYMBOL || LA734_5 == TIMESTAMP_SYMBOL || LA734_5 == TIME_SYMBOL || LA734_5 == YEAR_SYMBOL || ((LA734_5 >= BIGINT_SYMBOL) && (LA734_5 <= BINARY_SYMBOL)) || LA734_5 == BLOB_SYMBOL || LA734_5 == CHAR_SYMBOL || LA734_5 == DECIMAL_SYMBOL || LA734_5 == FLOAT_SYMBOL || LA734_5 == INT_SYMBOL || ((LA734_5 >= LONGBLOB_SYMBOL) && (LA734_5 <= LONGTEXT_SYMBOL)) || LA734_5 == LONG_SYMBOL || ((LA734_5 >= MEDIUMBLOB_SYMBOL) && (LA734_5 <= MEDIUMTEXT_SYMBOL)) || LA734_5 == NUMERIC_SYMBOL || LA734_5 == REAL_SYMBOL || LA734_5 == SET_SYMBOL || LA734_5 == SMALLINT_SYMBOL || ((LA734_5 >= TINYBLOB_SYMBOL) && (LA734_5 <= TINYTEXT_SYMBOL)) || LA734_5 == VARBINARY_SYMBOL || LA734_5 == VARCHAR_SYMBOL || LA734_5 == COMMA_SYMBOL || LA734_5 == DOUBLE_SYMBOL))
                            {
                                alt734=1;
                            }
                            else if ( (LA734_5 == CONDITION_SYMBOL))
                            {
                                alt734=2;
                            }
                            else if ( (LA734_5 == CURSOR_SYMBOL))
                            {
                                alt734=4;
                            }
                            else
                            {
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }


                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 734;
                                EXCEPTION->state        = 5;


                                goto rulesp_declarationEx;

                            }
                        }
                    }
                    else if ( (LA734_0 == CONTINUE_SYMBOL || LA734_0 == EXIT_SYMBOL || LA734_0 == UNDO_SYMBOL))
                    {
                        alt734=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 734;
                        EXCEPTION->state        = 0;


                        goto rulesp_declarationEx;

                    }
                }
                switch (alt734)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2335:3: variable_declaration
            	    {
            	        FOLLOWPUSH(FOLLOW_variable_declaration_in_sp_declaration13230);
            	        variable_declaration(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesp_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2336:5: condition_declaration
            	    {
            	        FOLLOWPUSH(FOLLOW_condition_declaration_in_sp_declaration13236);
            	        condition_declaration(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesp_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:2337:5: handler_declaration
            	    {
            	        FOLLOWPUSH(FOLLOW_handler_declaration_in_sp_declaration13242);
            	        handler_declaration(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesp_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 4:
            	    // MySQLSimpleParser.g:2338:5: cursor_declaration
            	    {
            	        FOLLOWPUSH(FOLLOW_cursor_declaration_in_sp_declaration13248);
            	        cursor_declaration(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesp_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulesp_declarationEx; /* Prevent compiler warnings */
    rulesp_declarationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end sp_declaration */

/**
 * $ANTLR start variable_declaration
 * MySQLSimpleParser.g:2342:1: variable_declaration : identifier_list data_type ( COLLATE_SYMBOL collation_name_or_default )? ( DEFAULT_SYMBOL expression )? ;
 */
static void
variable_declaration(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2342:21: ( identifier_list data_type ( COLLATE_SYMBOL collation_name_or_default )? ( DEFAULT_SYMBOL expression )? )
        // MySQLSimpleParser.g:2343:2: identifier_list data_type ( COLLATE_SYMBOL collation_name_or_default )? ( DEFAULT_SYMBOL expression )?
        {
            FOLLOWPUSH(FOLLOW_identifier_list_in_variable_declaration13261);
            identifier_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulevariable_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_data_type_in_variable_declaration13263);
            data_type(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulevariable_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2343:28: ( COLLATE_SYMBOL collation_name_or_default )?
            {
                int alt735=2;
                switch ( LA(1) )
                {
                    case COLLATE_SYMBOL:
                    	{
                    		alt735=1;
                    	}
                        break;
                }

                switch (alt735)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2343:29: COLLATE_SYMBOL collation_name_or_default
            	    {
            	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_variable_declaration13266);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevariable_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_collation_name_or_default_in_variable_declaration13268);
            	        collation_name_or_default(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevariable_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2343:72: ( DEFAULT_SYMBOL expression )?
            {
                int alt736=2;
                switch ( LA(1) )
                {
                    case DEFAULT_SYMBOL:
                    	{
                    		alt736=1;
                    	}
                        break;
                }

                switch (alt736)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2343:73: DEFAULT_SYMBOL expression
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_variable_declaration13273);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevariable_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_variable_declaration13275);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulevariable_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulevariable_declarationEx; /* Prevent compiler warnings */
    rulevariable_declarationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end variable_declaration */

/**
 * $ANTLR start condition_declaration
 * MySQLSimpleParser.g:2346:1: condition_declaration : identifier CONDITION_SYMBOL FOR_SYMBOL sp_condition ;
 */
static void
condition_declaration(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2346:22: ( identifier CONDITION_SYMBOL FOR_SYMBOL sp_condition )
        // MySQLSimpleParser.g:2347:2: identifier CONDITION_SYMBOL FOR_SYMBOL sp_condition
        {
            FOLLOWPUSH(FOLLOW_identifier_in_condition_declaration13286);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecondition_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CONDITION_SYMBOL, &FOLLOW_CONDITION_SYMBOL_in_condition_declaration13288);
            if  (HASEXCEPTION())
            {
                goto rulecondition_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_condition_declaration13290);
            if  (HASEXCEPTION())
            {
                goto rulecondition_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_sp_condition_in_condition_declaration13292);
            sp_condition(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecondition_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecondition_declarationEx; /* Prevent compiler warnings */
    rulecondition_declarationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end condition_declaration */

/**
 * $ANTLR start sp_condition
 * MySQLSimpleParser.g:2350:1: sp_condition : ( ulong_number | sqlstate );
 */
static void
sp_condition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2350:13: ( ulong_number | sqlstate )

            ANTLR3_UINT32 alt737;

            alt737=2;

            switch ( LA(1) )
            {
            case DECIMAL_NUMBER:
            case FLOAT_NUMBER:
            case HEX_NUMBER:
            case INT_NUMBER:
            case LONG_NUMBER:
            case ULONGLONG_NUMBER:
            	{
            		alt737=1;
            	}
                break;
            case SQLSTATE_SYMBOL:
            	{
            		alt737=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 737;
                EXCEPTION->state        = 0;


                goto rulesp_conditionEx;

            }

            switch (alt737)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2351:2: ulong_number
        	    {
        	        FOLLOWPUSH(FOLLOW_ulong_number_in_sp_condition13301);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesp_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2352:4: sqlstate
        	    {
        	        FOLLOWPUSH(FOLLOW_sqlstate_in_sp_condition13306);
        	        sqlstate(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesp_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulesp_conditionEx; /* Prevent compiler warnings */
    rulesp_conditionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end sp_condition */

/**
 * $ANTLR start sqlstate
 * MySQLSimpleParser.g:2355:1: sqlstate : SQLSTATE_SYMBOL ( VALUE_SYMBOL )? string_literal ;
 */
static void
sqlstate(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2355:9: ( SQLSTATE_SYMBOL ( VALUE_SYMBOL )? string_literal )
        // MySQLSimpleParser.g:2356:2: SQLSTATE_SYMBOL ( VALUE_SYMBOL )? string_literal
        {
             MATCHT(SQLSTATE_SYMBOL, &FOLLOW_SQLSTATE_SYMBOL_in_sqlstate13315);
            if  (HASEXCEPTION())
            {
                goto rulesqlstateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2356:18: ( VALUE_SYMBOL )?
            {
                int alt738=2;
                switch ( LA(1) )
                {
                    case VALUE_SYMBOL:
                    	{
                    		alt738=1;
                    	}
                        break;
                }

                switch (alt738)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2356:18: VALUE_SYMBOL
            	    {
            	         MATCHT(VALUE_SYMBOL, &FOLLOW_VALUE_SYMBOL_in_sqlstate13317);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesqlstateEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_string_literal_in_sqlstate13320);
            string_literal(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesqlstateEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesqlstateEx; /* Prevent compiler warnings */
    rulesqlstateEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end sqlstate */

/**
 * $ANTLR start handler_declaration
 * MySQLSimpleParser.g:2359:1: handler_declaration : ( CONTINUE_SYMBOL | EXIT_SYMBOL | UNDO_SYMBOL ) HANDLER_SYMBOL FOR_SYMBOL handler_condition ( COMMA_SYMBOL handler_condition )* compound_statement ;
 */
static void
handler_declaration(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2359:20: ( ( CONTINUE_SYMBOL | EXIT_SYMBOL | UNDO_SYMBOL ) HANDLER_SYMBOL FOR_SYMBOL handler_condition ( COMMA_SYMBOL handler_condition )* compound_statement )
        // MySQLSimpleParser.g:2360:2: ( CONTINUE_SYMBOL | EXIT_SYMBOL | UNDO_SYMBOL ) HANDLER_SYMBOL FOR_SYMBOL handler_condition ( COMMA_SYMBOL handler_condition )* compound_statement
        {
            if ( LA(1) == CONTINUE_SYMBOL || LA(1) == EXIT_SYMBOL || LA(1) == UNDO_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulehandler_declarationEx;
            }


             MATCHT(HANDLER_SYMBOL, &FOLLOW_HANDLER_SYMBOL_in_handler_declaration13341);
            if  (HASEXCEPTION())
            {
                goto rulehandler_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_handler_declaration13345);
            if  (HASEXCEPTION())
            {
                goto rulehandler_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_handler_condition_in_handler_declaration13347);
            handler_condition(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulehandler_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2361:32: ( COMMA_SYMBOL handler_condition )*

            for (;;)
            {
                int alt739=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt739=1;
                	}
                    break;

                }

                switch (alt739)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2361:33: COMMA_SYMBOL handler_condition
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_handler_declaration13350);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulehandler_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_handler_condition_in_handler_declaration13352);
            	        handler_condition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulehandler_declarationEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop739;	/* break out of the loop */
            	    break;
                }
            }
            loop739: ; /* Jump out to here if this rule does not match */


            FOLLOWPUSH(FOLLOW_compound_statement_in_handler_declaration13356);
            compound_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulehandler_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulehandler_declarationEx; /* Prevent compiler warnings */
    rulehandler_declarationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end handler_declaration */

/**
 * $ANTLR start handler_condition
 * MySQLSimpleParser.g:2364:1: handler_condition : ( sp_condition | identifier | SQLWARNING_SYMBOL | not_rule FOUND_SYMBOL | SQLEXCEPTION_SYMBOL );
 */
static void
handler_condition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2364:18: ( sp_condition | identifier | SQLWARNING_SYMBOL | not_rule FOUND_SYMBOL | SQLEXCEPTION_SYMBOL )

            ANTLR3_UINT32 alt740;

            alt740=5;

            {
                int LA740_0 = LA(1);
                if ( (LA740_0 == SQLSTATE_SYMBOL || LA740_0 == DECIMAL_NUMBER || LA740_0 == FLOAT_NUMBER || LA740_0 == HEX_NUMBER || LA740_0 == INT_NUMBER || LA740_0 == LONG_NUMBER || LA740_0 == ULONGLONG_NUMBER))
                {
                    alt740=1;
                }
                else if ( (((LA740_0 >= ACCOUNT_SYMBOL) && (LA740_0 <= YEAR_SYMBOL)) || LA740_0 == BACK_TICK_QUOTED_ID || LA740_0 == DOUBLE_QUOTED_TEXT || LA740_0 == IDENTIFIER))
                {
                    alt740=2;
                }
                else if ( (LA740_0 == SQLWARNING_SYMBOL))
                {
                    alt740=3;
                }
                else if ( (((LA740_0 >= NOT2_SYMBOL) && (LA740_0 <= NOT_SYMBOL))))
                {
                    alt740=4;
                }
                else if ( (LA740_0 == SQLEXCEPTION_SYMBOL))
                {
                    alt740=5;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 740;
                    EXCEPTION->state        = 0;


                    goto rulehandler_conditionEx;

                }
            }
            switch (alt740)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2365:2: sp_condition
        	    {
        	        FOLLOWPUSH(FOLLOW_sp_condition_in_handler_condition13365);
        	        sp_condition(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2366:4: identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_handler_condition13370);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2367:4: SQLWARNING_SYMBOL
        	    {
        	         MATCHT(SQLWARNING_SYMBOL, &FOLLOW_SQLWARNING_SYMBOL_in_handler_condition13375);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2368:4: not_rule FOUND_SYMBOL
        	    {
        	        FOLLOWPUSH(FOLLOW_not_rule_in_handler_condition13380);
        	        not_rule(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(FOUND_SYMBOL, &FOLLOW_FOUND_SYMBOL_in_handler_condition13382);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2369:4: SQLEXCEPTION_SYMBOL
        	    {
        	         MATCHT(SQLEXCEPTION_SYMBOL, &FOLLOW_SQLEXCEPTION_SYMBOL_in_handler_condition13387);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulehandler_conditionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulehandler_conditionEx; /* Prevent compiler warnings */
    rulehandler_conditionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end handler_condition */

/**
 * $ANTLR start cursor_declaration
 * MySQLSimpleParser.g:2372:1: cursor_declaration : identifier CURSOR_SYMBOL FOR_SYMBOL select_statement ;
 */
static void
cursor_declaration(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2372:19: ( identifier CURSOR_SYMBOL FOR_SYMBOL select_statement )
        // MySQLSimpleParser.g:2373:2: identifier CURSOR_SYMBOL FOR_SYMBOL select_statement
        {
            FOLLOWPUSH(FOLLOW_identifier_in_cursor_declaration13396);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CURSOR_SYMBOL, &FOLLOW_CURSOR_SYMBOL_in_cursor_declaration13398);
            if  (HASEXCEPTION())
            {
                goto rulecursor_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(FOR_SYMBOL, &FOLLOW_FOR_SYMBOL_in_cursor_declaration13400);
            if  (HASEXCEPTION())
            {
                goto rulecursor_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_select_statement_in_cursor_declaration13402);
            select_statement(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_declarationEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecursor_declarationEx; /* Prevent compiler warnings */
    rulecursor_declarationEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cursor_declaration */

/**
 * $ANTLR start iterate_statement
 * MySQLSimpleParser.g:2376:1: iterate_statement : ITERATE_SYMBOL label_identifier ;
 */
static void
iterate_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2376:18: ( ITERATE_SYMBOL label_identifier )
        // MySQLSimpleParser.g:2377:2: ITERATE_SYMBOL label_identifier
        {
             MATCHT(ITERATE_SYMBOL, &FOLLOW_ITERATE_SYMBOL_in_iterate_statement13411);
            if  (HASEXCEPTION())
            {
                goto ruleiterate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_label_identifier_in_iterate_statement13413);
            label_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleiterate_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleiterate_statementEx; /* Prevent compiler warnings */
    ruleiterate_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end iterate_statement */

/**
 * $ANTLR start leave_statement
 * MySQLSimpleParser.g:2380:1: leave_statement : LEAVE_SYMBOL label_identifier ;
 */
static void
leave_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2380:16: ( LEAVE_SYMBOL label_identifier )
        // MySQLSimpleParser.g:2381:2: LEAVE_SYMBOL label_identifier
        {
             MATCHT(LEAVE_SYMBOL, &FOLLOW_LEAVE_SYMBOL_in_leave_statement13422);
            if  (HASEXCEPTION())
            {
                goto ruleleave_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_label_identifier_in_leave_statement13424);
            label_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleleave_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleleave_statementEx; /* Prevent compiler warnings */
    ruleleave_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end leave_statement */

/**
 * $ANTLR start get_diagnostics
 * MySQLSimpleParser.g:2384:1: get_diagnostics : GET_SYMBOL ( CURRENT_SYMBOL |{...}? STACKED_SYMBOL )? DIAGNOSTICS_SYMBOL ( statement_information_item ( COMMA_SYMBOL statement_information_item )* | CONDITION_SYMBOL signal_allowed_expression condition_information_item ( COMMA_SYMBOL condition_information_item )* ) ;
 */
static void
get_diagnostics(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2384:16: ( GET_SYMBOL ( CURRENT_SYMBOL |{...}? STACKED_SYMBOL )? DIAGNOSTICS_SYMBOL ( statement_information_item ( COMMA_SYMBOL statement_information_item )* | CONDITION_SYMBOL signal_allowed_expression condition_information_item ( COMMA_SYMBOL condition_information_item )* ) )
        // MySQLSimpleParser.g:2385:2: GET_SYMBOL ( CURRENT_SYMBOL |{...}? STACKED_SYMBOL )? DIAGNOSTICS_SYMBOL ( statement_information_item ( COMMA_SYMBOL statement_information_item )* | CONDITION_SYMBOL signal_allowed_expression condition_information_item ( COMMA_SYMBOL condition_information_item )* )
        {
             MATCHT(GET_SYMBOL, &FOLLOW_GET_SYMBOL_in_get_diagnostics13433);
            if  (HASEXCEPTION())
            {
                goto ruleget_diagnosticsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2386:2: ( CURRENT_SYMBOL |{...}? STACKED_SYMBOL )?
            {
                int alt741=3;
                switch ( LA(1) )
                {
                    case CURRENT_SYMBOL:
                    	{
                    		alt741=1;
                    	}
                        break;
                    case STACKED_SYMBOL:
                    	{
                    		alt741=2;
                    	}
                        break;
                }

                switch (alt741)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2387:3: CURRENT_SYMBOL
            	    {
            	         MATCHT(CURRENT_SYMBOL, &FOLLOW_CURRENT_SYMBOL_in_get_diagnostics13440);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2388:5: {...}? STACKED_SYMBOL
            	    {
            	        if ( !((SERVER_VERSION >= 50700)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
            	                EXCEPTION->ruleName	 = (void *)"get_diagnostics";


            	        }

            	         MATCHT(STACKED_SYMBOL, &FOLLOW_STACKED_SYMBOL_in_get_diagnostics13448);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(DIAGNOSTICS_SYMBOL, &FOLLOW_DIAGNOSTICS_SYMBOL_in_get_diagnostics13454);
            if  (HASEXCEPTION())
            {
                goto ruleget_diagnosticsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2390:2: ( statement_information_item ( COMMA_SYMBOL statement_information_item )* | CONDITION_SYMBOL signal_allowed_expression condition_information_item ( COMMA_SYMBOL condition_information_item )* )
            {
                int alt744=2;
                {
                    int LA744_0 = LA(1);
                    if ( (((LA744_0 >= ACCOUNT_SYMBOL) && (LA744_0 <= YEAR_SYMBOL)) || ((LA744_0 >= AT_AT_SIGN_SYMBOL) && (LA744_0 <= AT_TEXT_SUFFIX)) || LA744_0 == BACK_TICK_QUOTED_ID || LA744_0 == DOUBLE_QUOTED_TEXT || LA744_0 == IDENTIFIER))
                    {
                        alt744=1;
                    }
                    else if ( (LA744_0 == CONDITION_SYMBOL))
                    {
                        alt744=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 744;
                        EXCEPTION->state        = 0;


                        goto ruleget_diagnosticsEx;

                    }
                }
                switch (alt744)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2391:3: statement_information_item ( COMMA_SYMBOL statement_information_item )*
            	    {
            	        FOLLOWPUSH(FOLLOW_statement_information_item_in_get_diagnostics13461);
            	        statement_information_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2391:30: ( COMMA_SYMBOL statement_information_item )*

            	        for (;;)
            	        {
            	            int alt742=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt742=1;
            	            	}
            	                break;

            	            }

            	            switch (alt742)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2391:31: COMMA_SYMBOL statement_information_item
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_get_diagnostics13464);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleget_diagnosticsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_statement_information_item_in_get_diagnostics13466);
            	        	        statement_information_item(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleget_diagnosticsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop742;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop742: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2392:5: CONDITION_SYMBOL signal_allowed_expression condition_information_item ( COMMA_SYMBOL condition_information_item )*
            	    {
            	         MATCHT(CONDITION_SYMBOL, &FOLLOW_CONDITION_SYMBOL_in_get_diagnostics13474);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_signal_allowed_expression_in_get_diagnostics13476);
            	        signal_allowed_expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_condition_information_item_in_get_diagnostics13478);
            	        condition_information_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleget_diagnosticsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2392:75: ( COMMA_SYMBOL condition_information_item )*

            	        for (;;)
            	        {
            	            int alt743=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt743=1;
            	            	}
            	                break;

            	            }

            	            switch (alt743)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2392:76: COMMA_SYMBOL condition_information_item
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_get_diagnostics13481);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleget_diagnosticsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_condition_information_item_in_get_diagnostics13483);
            	        	        condition_information_item(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleget_diagnosticsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop743;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop743: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleget_diagnosticsEx; /* Prevent compiler warnings */
    ruleget_diagnosticsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end get_diagnostics */

/**
 * $ANTLR start signal_allowed_expression
 * MySQLSimpleParser.g:2397:1: signal_allowed_expression : ( literal | variable | qualified_identifier );
 */
static void
signal_allowed_expression(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2397:26: ( literal | variable | qualified_identifier )

            ANTLR3_UINT32 alt745;

            alt745=3;

            {
                int LA745_0 = LA(1);
                if ( (LA745_0 == FALSE_SYMBOL || LA745_0 == NCHAR_TEXT || ((LA745_0 >= NULL2_SYMBOL) && (LA745_0 <= NULL_SYMBOL)) || LA745_0 == SINGLE_QUOTED_TEXT || LA745_0 == TRUE_SYMBOL || LA745_0 == BIN_NUMBER || LA745_0 == DECIMAL_NUMBER || LA745_0 == FLOAT_NUMBER || LA745_0 == HEX_NUMBER || LA745_0 == INT_NUMBER || LA745_0 == LONG_NUMBER || ((LA745_0 >= ULONGLONG_NUMBER) && (LA745_0 <= UNDERSCORE_CHARSET))))
                {
                    alt745=1;
                }
                else if ( (LA745_0 == DOUBLE_QUOTED_TEXT))
                {
                    {
                        int LA745_4 = LA(2);
                        if ( ((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
                        {
                            alt745=1;
                        }
                        else if ( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
                        {
                            alt745=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 745;
                            EXCEPTION->state        = 4;


                            goto rulesignal_allowed_expressionEx;

                        }
                    }
                }
                else if ( (LA745_0 == DATE_SYMBOL))
                {
                    {
                        int LA745_6 = LA(2);
                        if ( (LA745_6 == SINGLE_QUOTED_TEXT))
                        {
                            alt745=1;
                        }
                        else if ( (LA745_6 == EOF || ((LA745_6 >= ACCOUNT_SYMBOL) && (LA745_6 <= YEAR_SYMBOL)) || LA745_6 == SEMICOLON_SYMBOL || ((LA745_6 >= AT_AT_SIGN_SYMBOL) && (LA745_6 <= AT_TEXT_SUFFIX)) || LA745_6 == BACK_TICK_QUOTED_ID || LA745_6 == COMMA_SYMBOL || LA745_6 == DOT_SYMBOL || LA745_6 == DOUBLE_QUOTED_TEXT || LA745_6 == IDENTIFIER))
                        {
                            alt745=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 745;
                            EXCEPTION->state        = 6;


                            goto rulesignal_allowed_expressionEx;

                        }
                    }
                }
                else if ( (LA745_0 == TIME_SYMBOL))
                {
                    {
                        int LA745_7 = LA(2);
                        if ( (LA745_7 == SINGLE_QUOTED_TEXT))
                        {
                            alt745=1;
                        }
                        else if ( (LA745_7 == EOF || ((LA745_7 >= ACCOUNT_SYMBOL) && (LA745_7 <= YEAR_SYMBOL)) || LA745_7 == SEMICOLON_SYMBOL || ((LA745_7 >= AT_AT_SIGN_SYMBOL) && (LA745_7 <= AT_TEXT_SUFFIX)) || LA745_7 == BACK_TICK_QUOTED_ID || LA745_7 == COMMA_SYMBOL || LA745_7 == DOT_SYMBOL || LA745_7 == DOUBLE_QUOTED_TEXT || LA745_7 == IDENTIFIER))
                        {
                            alt745=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 745;
                            EXCEPTION->state        = 7;


                            goto rulesignal_allowed_expressionEx;

                        }
                    }
                }
                else if ( (LA745_0 == TIMESTAMP_SYMBOL))
                {
                    {
                        int LA745_8 = LA(2);
                        if ( (LA745_8 == SINGLE_QUOTED_TEXT))
                        {
                            alt745=1;
                        }
                        else if ( (LA745_8 == EOF || ((LA745_8 >= ACCOUNT_SYMBOL) && (LA745_8 <= YEAR_SYMBOL)) || LA745_8 == SEMICOLON_SYMBOL || ((LA745_8 >= AT_AT_SIGN_SYMBOL) && (LA745_8 <= AT_TEXT_SUFFIX)) || LA745_8 == BACK_TICK_QUOTED_ID || LA745_8 == COMMA_SYMBOL || LA745_8 == DOT_SYMBOL || LA745_8 == DOUBLE_QUOTED_TEXT || LA745_8 == IDENTIFIER))
                        {
                            alt745=3;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 745;
                            EXCEPTION->state        = 8;


                            goto rulesignal_allowed_expressionEx;

                        }
                    }
                }
                else if ( (((LA745_0 >= AT_AT_SIGN_SYMBOL) && (LA745_0 <= AT_TEXT_SUFFIX))))
                {
                    alt745=2;
                }
                else if ( (((LA745_0 >= ACCOUNT_SYMBOL) && (LA745_0 <= DATETIME_SYMBOL)) || ((LA745_0 >= DAY_SYMBOL) && (LA745_0 <= TRIGGERS_SYMBOL)) || ((LA745_0 >= TIMESTAMP_ADD_SYMBOL) && (LA745_0 <= TIMESTAMP_DIFF_SYMBOL)) || ((LA745_0 >= TYPES_SYMBOL) && (LA745_0 <= YEAR_SYMBOL)) || LA745_0 == BACK_TICK_QUOTED_ID || LA745_0 == IDENTIFIER))
                {
                    alt745=3;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 745;
                    EXCEPTION->state        = 0;


                    goto rulesignal_allowed_expressionEx;

                }
            }
            switch (alt745)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2398:2: literal
        	    {
        	        FOLLOWPUSH(FOLLOW_literal_in_signal_allowed_expression13498);
        	        literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesignal_allowed_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2399:4: variable
        	    {
        	        FOLLOWPUSH(FOLLOW_variable_in_signal_allowed_expression13503);
        	        variable(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesignal_allowed_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2400:4: qualified_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_qualified_identifier_in_signal_allowed_expression13508);
        	        qualified_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesignal_allowed_expressionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulesignal_allowed_expressionEx; /* Prevent compiler warnings */
    rulesignal_allowed_expressionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end signal_allowed_expression */

/**
 * $ANTLR start statement_information_item
 * MySQLSimpleParser.g:2403:1: statement_information_item : ( variable | identifier ) EQUAL_OPERATOR ( NUMBER_SYMBOL | ROW_COUNT_SYMBOL ) ;
 */
static void
statement_information_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2403:27: ( ( variable | identifier ) EQUAL_OPERATOR ( NUMBER_SYMBOL | ROW_COUNT_SYMBOL ) )
        // MySQLSimpleParser.g:2404:2: ( variable | identifier ) EQUAL_OPERATOR ( NUMBER_SYMBOL | ROW_COUNT_SYMBOL )
        {
            // MySQLSimpleParser.g:2404:2: ( variable | identifier )
            {
                int alt746=2;
                {
                    int LA746_0 = LA(1);
                    if ( (((LA746_0 >= AT_AT_SIGN_SYMBOL) && (LA746_0 <= AT_TEXT_SUFFIX))))
                    {
                        alt746=1;
                    }
                    else if ( (((LA746_0 >= ACCOUNT_SYMBOL) && (LA746_0 <= YEAR_SYMBOL)) || LA746_0 == BACK_TICK_QUOTED_ID || LA746_0 == DOUBLE_QUOTED_TEXT || LA746_0 == IDENTIFIER))
                    {
                        alt746=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 746;
                        EXCEPTION->state        = 0;


                        goto rulestatement_information_itemEx;

                    }
                }
                switch (alt746)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2404:3: variable
            	    {
            	        FOLLOWPUSH(FOLLOW_variable_in_statement_information_item13518);
            	        variable(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulestatement_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2404:14: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_statement_information_item13522);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulestatement_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_statement_information_item13525);
            if  (HASEXCEPTION())
            {
                goto rulestatement_information_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            if ( LA(1) == NUMBER_SYMBOL || LA(1) == ROW_COUNT_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulestatement_information_itemEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulestatement_information_itemEx; /* Prevent compiler warnings */
    rulestatement_information_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end statement_information_item */

/**
 * $ANTLR start condition_information_item
 * MySQLSimpleParser.g:2407:1: condition_information_item : ( variable | identifier ) EQUAL_OPERATOR ( signal_information_item_name | RETURNED_SQLSTATE_SYMBOL ) ;
 */
static void
condition_information_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2407:27: ( ( variable | identifier ) EQUAL_OPERATOR ( signal_information_item_name | RETURNED_SQLSTATE_SYMBOL ) )
        // MySQLSimpleParser.g:2408:2: ( variable | identifier ) EQUAL_OPERATOR ( signal_information_item_name | RETURNED_SQLSTATE_SYMBOL )
        {
            // MySQLSimpleParser.g:2408:2: ( variable | identifier )
            {
                int alt747=2;
                {
                    int LA747_0 = LA(1);
                    if ( (((LA747_0 >= AT_AT_SIGN_SYMBOL) && (LA747_0 <= AT_TEXT_SUFFIX))))
                    {
                        alt747=1;
                    }
                    else if ( (((LA747_0 >= ACCOUNT_SYMBOL) && (LA747_0 <= YEAR_SYMBOL)) || LA747_0 == BACK_TICK_QUOTED_ID || LA747_0 == DOUBLE_QUOTED_TEXT || LA747_0 == IDENTIFIER))
                    {
                        alt747=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 747;
                        EXCEPTION->state        = 0;


                        goto rulecondition_information_itemEx;

                    }
                }
                switch (alt747)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2408:3: variable
            	    {
            	        FOLLOWPUSH(FOLLOW_variable_in_condition_information_item13543);
            	        variable(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecondition_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2408:14: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_condition_information_item13547);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecondition_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_condition_information_item13550);
            if  (HASEXCEPTION())
            {
                goto rulecondition_information_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2408:41: ( signal_information_item_name | RETURNED_SQLSTATE_SYMBOL )
            {
                int alt748=2;
                switch ( LA(1) )
                {
                case CATALOG_NAME_SYMBOL:
                case CLASS_ORIGIN_SYMBOL:
                case COLUMN_NAME_SYMBOL:
                case CONSTRAINT_CATALOG_SYMBOL:
                case CONSTRAINT_SCHEMA_SYMBOL:
                case CONSTRAINT_NAME_SYMBOL:
                case CURSOR_NAME_SYMBOL:
                case MESSAGE_TEXT_SYMBOL:
                case MYSQL_ERRNO_SYMBOL:
                case SCHEMA_NAME_SYMBOL:
                case SUBCLASS_ORIGIN_SYMBOL:
                case TABLE_NAME_SYMBOL:
                	{
                		alt748=1;
                	}
                    break;
                case RETURNED_SQLSTATE_SYMBOL:
                	{
                		alt748=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 748;
                    EXCEPTION->state        = 0;


                    goto rulecondition_information_itemEx;

                }

                switch (alt748)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2408:42: signal_information_item_name
            	    {
            	        FOLLOWPUSH(FOLLOW_signal_information_item_name_in_condition_information_item13553);
            	        signal_information_item_name(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecondition_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2408:73: RETURNED_SQLSTATE_SYMBOL
            	    {
            	         MATCHT(RETURNED_SQLSTATE_SYMBOL, &FOLLOW_RETURNED_SQLSTATE_SYMBOL_in_condition_information_item13557);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecondition_information_itemEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecondition_information_itemEx; /* Prevent compiler warnings */
    rulecondition_information_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end condition_information_item */

/**
 * $ANTLR start signal_information_item_name
 * MySQLSimpleParser.g:2411:1: signal_information_item_name : ( CLASS_ORIGIN_SYMBOL | SUBCLASS_ORIGIN_SYMBOL | CONSTRAINT_CATALOG_SYMBOL | CONSTRAINT_SCHEMA_SYMBOL | CONSTRAINT_NAME_SYMBOL | CATALOG_NAME_SYMBOL | SCHEMA_NAME_SYMBOL | TABLE_NAME_SYMBOL | COLUMN_NAME_SYMBOL | CURSOR_NAME_SYMBOL | MESSAGE_TEXT_SYMBOL | MYSQL_ERRNO_SYMBOL );
 */
static void
signal_information_item_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2411:29: ( CLASS_ORIGIN_SYMBOL | SUBCLASS_ORIGIN_SYMBOL | CONSTRAINT_CATALOG_SYMBOL | CONSTRAINT_SCHEMA_SYMBOL | CONSTRAINT_NAME_SYMBOL | CATALOG_NAME_SYMBOL | SCHEMA_NAME_SYMBOL | TABLE_NAME_SYMBOL | COLUMN_NAME_SYMBOL | CURSOR_NAME_SYMBOL | MESSAGE_TEXT_SYMBOL | MYSQL_ERRNO_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == CATALOG_NAME_SYMBOL || LA(1) == CLASS_ORIGIN_SYMBOL || LA(1) == COLUMN_NAME_SYMBOL || ((LA(1) >= CONSTRAINT_CATALOG_SYMBOL) && (LA(1) <= CONSTRAINT_NAME_SYMBOL)) || LA(1) == CURSOR_NAME_SYMBOL || LA(1) == MESSAGE_TEXT_SYMBOL || LA(1) == MYSQL_ERRNO_SYMBOL || LA(1) == SCHEMA_NAME_SYMBOL || LA(1) == SUBCLASS_ORIGIN_SYMBOL || LA(1) == TABLE_NAME_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulesignal_information_item_nameEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesignal_information_item_nameEx; /* Prevent compiler warnings */
    rulesignal_information_item_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end signal_information_item_name */

/**
 * $ANTLR start signal_statement
 * MySQLSimpleParser.g:2426:1: signal_statement : SIGNAL_SYMBOL ( identifier | sqlstate ) ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )? ;
 */
static void
signal_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2426:17: ( SIGNAL_SYMBOL ( identifier | sqlstate ) ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )? )
        // MySQLSimpleParser.g:2427:2: SIGNAL_SYMBOL ( identifier | sqlstate ) ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )?
        {
             MATCHT(SIGNAL_SYMBOL, &FOLLOW_SIGNAL_SYMBOL_in_signal_statement13631);
            if  (HASEXCEPTION())
            {
                goto rulesignal_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2427:16: ( identifier | sqlstate )
            {
                int alt749=2;
                {
                    int LA749_0 = LA(1);
                    if ( (((LA749_0 >= ACCOUNT_SYMBOL) && (LA749_0 <= YEAR_SYMBOL)) || LA749_0 == BACK_TICK_QUOTED_ID || LA749_0 == DOUBLE_QUOTED_TEXT || LA749_0 == IDENTIFIER))
                    {
                        alt749=1;
                    }
                    else if ( (LA749_0 == SQLSTATE_SYMBOL))
                    {
                        alt749=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 749;
                        EXCEPTION->state        = 0;


                        goto rulesignal_statementEx;

                    }
                }
                switch (alt749)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2427:17: identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_identifier_in_signal_statement13634);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2427:30: sqlstate
            	    {
            	        FOLLOWPUSH(FOLLOW_sqlstate_in_signal_statement13638);
            	        sqlstate(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2427:40: ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )?
            {
                int alt751=2;
                switch ( LA(1) )
                {
                    case SET_SYMBOL:
                    	{
                    		alt751=1;
                    	}
                        break;
                }

                switch (alt751)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2427:41: SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )*
            	    {
            	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_signal_statement13642);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_signal_information_item_in_signal_statement13644);
            	        signal_information_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2427:76: ( COMMA_SYMBOL signal_information_item )*

            	        for (;;)
            	        {
            	            int alt750=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt750=1;
            	            	}
            	                break;

            	            }

            	            switch (alt750)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2427:77: COMMA_SYMBOL signal_information_item
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_signal_statement13647);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesignal_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_signal_information_item_in_signal_statement13649);
            	        	        signal_information_item(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulesignal_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop750;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop750: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulesignal_statementEx; /* Prevent compiler warnings */
    rulesignal_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end signal_statement */

/**
 * $ANTLR start resignal_statement
 * MySQLSimpleParser.g:2430:1: resignal_statement : RESIGNAL_SYMBOL ( SQLSTATE_SYMBOL ( VALUE_SYMBOL )? text_or_identifier )? ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )? ;
 */
static void
resignal_statement(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2430:19: ( RESIGNAL_SYMBOL ( SQLSTATE_SYMBOL ( VALUE_SYMBOL )? text_or_identifier )? ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )? )
        // MySQLSimpleParser.g:2431:2: RESIGNAL_SYMBOL ( SQLSTATE_SYMBOL ( VALUE_SYMBOL )? text_or_identifier )? ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )?
        {
             MATCHT(RESIGNAL_SYMBOL, &FOLLOW_RESIGNAL_SYMBOL_in_resignal_statement13662);
            if  (HASEXCEPTION())
            {
                goto ruleresignal_statementEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2431:18: ( SQLSTATE_SYMBOL ( VALUE_SYMBOL )? text_or_identifier )?
            {
                int alt753=2;
                switch ( LA(1) )
                {
                    case SQLSTATE_SYMBOL:
                    	{
                    		alt753=1;
                    	}
                        break;
                }

                switch (alt753)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2431:19: SQLSTATE_SYMBOL ( VALUE_SYMBOL )? text_or_identifier
            	    {
            	         MATCHT(SQLSTATE_SYMBOL, &FOLLOW_SQLSTATE_SYMBOL_in_resignal_statement13665);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleresignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2431:35: ( VALUE_SYMBOL )?
            	        {
            	            int alt752=2;
            	            switch ( LA(1) )
            	            {
            	                case VALUE_SYMBOL:
            	                	{
            	                		{
            	                		    int LA752_1 = LA(2);
            	                		    if ( (((LA752_1 >= ACCOUNT_SYMBOL) && (LA752_1 <= YEAR_SYMBOL)) || LA752_1 == SINGLE_QUOTED_TEXT || LA752_1 == BACK_TICK_QUOTED_ID || LA752_1 == DOUBLE_QUOTED_TEXT || LA752_1 == IDENTIFIER))
            	                		    {
            	                		        alt752=1;
            	                		    }
            	                		}
            	                	}
            	                    break;
            	            }

            	            switch (alt752)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2431:35: VALUE_SYMBOL
            	        	    {
            	        	         MATCHT(VALUE_SYMBOL, &FOLLOW_VALUE_SYMBOL_in_resignal_statement13667);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleresignal_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_resignal_statement13670);
            	        text_or_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleresignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2432:3: ( SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )* )?
            {
                int alt755=2;
                switch ( LA(1) )
                {
                    case SET_SYMBOL:
                    	{
                    		alt755=1;
                    	}
                        break;
                }

                switch (alt755)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2432:4: SET_SYMBOL signal_information_item ( COMMA_SYMBOL signal_information_item )*
            	    {
            	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_resignal_statement13677);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleresignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_signal_information_item_in_resignal_statement13679);
            	        signal_information_item(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleresignal_statementEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2432:39: ( COMMA_SYMBOL signal_information_item )*

            	        for (;;)
            	        {
            	            int alt754=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt754=1;
            	            	}
            	                break;

            	            }

            	            switch (alt754)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2432:40: COMMA_SYMBOL signal_information_item
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_resignal_statement13682);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleresignal_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_signal_information_item_in_resignal_statement13684);
            	        	        signal_information_item(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruleresignal_statementEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop754;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop754: ; /* Jump out to here if this rule does not match */


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleresignal_statementEx; /* Prevent compiler warnings */
    ruleresignal_statementEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end resignal_statement */

/**
 * $ANTLR start signal_information_item
 * MySQLSimpleParser.g:2435:1: signal_information_item : signal_information_item_name EQUAL_OPERATOR signal_allowed_expression ;
 */
static void
signal_information_item(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2435:24: ( signal_information_item_name EQUAL_OPERATOR signal_allowed_expression )
        // MySQLSimpleParser.g:2436:2: signal_information_item_name EQUAL_OPERATOR signal_allowed_expression
        {
            FOLLOWPUSH(FOLLOW_signal_information_item_name_in_signal_information_item13697);
            signal_information_item_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesignal_information_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_signal_information_item13699);
            if  (HASEXCEPTION())
            {
                goto rulesignal_information_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_signal_allowed_expression_in_signal_information_item13701);
            signal_allowed_expression(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesignal_information_itemEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesignal_information_itemEx; /* Prevent compiler warnings */
    rulesignal_information_itemEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end signal_information_item */

/**
 * $ANTLR start cursor_open
 * MySQLSimpleParser.g:2439:1: cursor_open : OPEN_SYMBOL identifier ;
 */
static void
cursor_open(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2439:12: ( OPEN_SYMBOL identifier )
        // MySQLSimpleParser.g:2440:2: OPEN_SYMBOL identifier
        {
             MATCHT(OPEN_SYMBOL, &FOLLOW_OPEN_SYMBOL_in_cursor_open13710);
            if  (HASEXCEPTION())
            {
                goto rulecursor_openEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_cursor_open13712);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_openEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecursor_openEx; /* Prevent compiler warnings */
    rulecursor_openEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cursor_open */

/**
 * $ANTLR start cursor_close
 * MySQLSimpleParser.g:2443:1: cursor_close : CLOSE_SYMBOL identifier ;
 */
static void
cursor_close(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2443:13: ( CLOSE_SYMBOL identifier )
        // MySQLSimpleParser.g:2444:2: CLOSE_SYMBOL identifier
        {
             MATCHT(CLOSE_SYMBOL, &FOLLOW_CLOSE_SYMBOL_in_cursor_close13721);
            if  (HASEXCEPTION())
            {
                goto rulecursor_closeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_cursor_close13723);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_closeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecursor_closeEx; /* Prevent compiler warnings */
    rulecursor_closeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cursor_close */

/**
 * $ANTLR start cursor_fetch
 * MySQLSimpleParser.g:2447:1: cursor_fetch : FETCH_SYMBOL identifier INTO_SYMBOL identifier_list ;
 */
static void
cursor_fetch(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2447:13: ( FETCH_SYMBOL identifier INTO_SYMBOL identifier_list )
        // MySQLSimpleParser.g:2448:2: FETCH_SYMBOL identifier INTO_SYMBOL identifier_list
        {
             MATCHT(FETCH_SYMBOL, &FOLLOW_FETCH_SYMBOL_in_cursor_fetch13732);
            if  (HASEXCEPTION())
            {
                goto rulecursor_fetchEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_cursor_fetch13734);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_fetchEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(INTO_SYMBOL, &FOLLOW_INTO_SYMBOL_in_cursor_fetch13736);
            if  (HASEXCEPTION())
            {
                goto rulecursor_fetchEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_list_in_cursor_fetch13738);
            identifier_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecursor_fetchEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecursor_fetchEx; /* Prevent compiler warnings */
    rulecursor_fetchEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end cursor_fetch */

/**
 * $ANTLR start schedule
 * MySQLSimpleParser.g:2454:1: schedule : ( AT_SYMBOL expression | EVERY_SYMBOL interval ( STARTS_SYMBOL expression )? ( ENDS_SYMBOL expression )? );
 */
static void
schedule(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2454:9: ( AT_SYMBOL expression | EVERY_SYMBOL interval ( STARTS_SYMBOL expression )? ( ENDS_SYMBOL expression )? )

            ANTLR3_UINT32 alt758;

            alt758=2;

            switch ( LA(1) )
            {
            case AT_SYMBOL:
            	{
            		alt758=1;
            	}
                break;
            case EVERY_SYMBOL:
            	{
            		alt758=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 758;
                EXCEPTION->state        = 0;


                goto rulescheduleEx;

            }

            switch (alt758)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2455:2: AT_SYMBOL expression
        	    {
        	         MATCHT(AT_SYMBOL, &FOLLOW_AT_SYMBOL_in_schedule13750);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulescheduleEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_schedule13752);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulescheduleEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2456:4: EVERY_SYMBOL interval ( STARTS_SYMBOL expression )? ( ENDS_SYMBOL expression )?
        	    {
        	         MATCHT(EVERY_SYMBOL, &FOLLOW_EVERY_SYMBOL_in_schedule13757);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulescheduleEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_interval_in_schedule13759);
        	        interval(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulescheduleEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2456:26: ( STARTS_SYMBOL expression )?
        	        {
        	            int alt756=2;
        	            switch ( LA(1) )
        	            {
        	                case STARTS_SYMBOL:
        	                	{
        	                		alt756=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt756)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2456:27: STARTS_SYMBOL expression
        	        	    {
        	        	         MATCHT(STARTS_SYMBOL, &FOLLOW_STARTS_SYMBOL_in_schedule13762);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulescheduleEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_schedule13764);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulescheduleEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2456:54: ( ENDS_SYMBOL expression )?
        	        {
        	            int alt757=2;
        	            switch ( LA(1) )
        	            {
        	                case ENDS_SYMBOL:
        	                	{
        	                		alt757=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt757)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2456:55: ENDS_SYMBOL expression
        	        	    {
        	        	         MATCHT(ENDS_SYMBOL, &FOLLOW_ENDS_SYMBOL_in_schedule13769);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulescheduleEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_expression_in_schedule13771);
        	        	        expression(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulescheduleEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulescheduleEx; /* Prevent compiler warnings */
    rulescheduleEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end schedule */

/**
 * $ANTLR start database_option
 * MySQLSimpleParser.g:2459:1: database_option : ( DEFAULT_SYMBOL )? ( charset ( EQUAL_OPERATOR )? charset_name_or_default | COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default ) ;
 */
static void
database_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2459:16: ( ( DEFAULT_SYMBOL )? ( charset ( EQUAL_OPERATOR )? charset_name_or_default | COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default ) )
        // MySQLSimpleParser.g:2460:2: ( DEFAULT_SYMBOL )? ( charset ( EQUAL_OPERATOR )? charset_name_or_default | COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default )
        {
            // MySQLSimpleParser.g:2460:2: ( DEFAULT_SYMBOL )?
            {
                int alt759=2;
                switch ( LA(1) )
                {
                    case DEFAULT_SYMBOL:
                    	{
                    		alt759=1;
                    	}
                        break;
                }

                switch (alt759)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2460:2: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_database_option13782);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledatabase_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2461:3: ( charset ( EQUAL_OPERATOR )? charset_name_or_default | COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default )
            {
                int alt762=2;
                switch ( LA(1) )
                {
                case CHARSET_SYMBOL:
                case CHAR_SYMBOL:
                	{
                		alt762=1;
                	}
                    break;
                case COLLATE_SYMBOL:
                	{
                		alt762=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 762;
                    EXCEPTION->state        = 0;


                    goto ruledatabase_optionEx;

                }

                switch (alt762)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2462:4: charset ( EQUAL_OPERATOR )? charset_name_or_default
            	    {
            	        FOLLOWPUSH(FOLLOW_charset_in_database_option13792);
            	        charset(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledatabase_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2462:12: ( EQUAL_OPERATOR )?
            	        {
            	            int alt760=2;
            	            switch ( LA(1) )
            	            {
            	                case EQUAL_OPERATOR:
            	                	{
            	                		alt760=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt760)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2462:12: EQUAL_OPERATOR
            	        	    {
            	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_database_option13794);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledatabase_optionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_charset_name_or_default_in_database_option13797);
            	        charset_name_or_default(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledatabase_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2463:6: COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default
            	    {
            	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_database_option13804);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledatabase_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2463:21: ( EQUAL_OPERATOR )?
            	        {
            	            int alt761=2;
            	            switch ( LA(1) )
            	            {
            	                case EQUAL_OPERATOR:
            	                	{
            	                		alt761=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt761)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2463:21: EQUAL_OPERATOR
            	        	    {
            	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_database_option13806);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruledatabase_optionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_collation_name_or_default_in_database_option13809);
            	        collation_name_or_default(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruledatabase_optionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruledatabase_optionEx; /* Prevent compiler warnings */
    ruledatabase_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end database_option */

/**
 * $ANTLR start column_definition
 * MySQLSimpleParser.g:2467:1: column_definition : field_spec ( reference_definition | ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )? ) ;
 */
static void
column_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2467:18: ( field_spec ( reference_definition | ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )? ) )
        // MySQLSimpleParser.g:2468:2: field_spec ( reference_definition | ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )? )
        {
            FOLLOWPUSH(FOLLOW_field_spec_in_column_definition13822);
            field_spec(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2468:13: ( reference_definition | ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )? )
            {
                int alt764=2;
                {
                    int LA764_0 = LA(1);
                    if ( (LA764_0 == REFERENCES_SYMBOL))
                    {
                        alt764=1;
                    }
                    else if ( (LA764_0 == AFTER_SYMBOL || LA764_0 == FIRST_SYMBOL || LA764_0 == CHECK_SYMBOL || LA764_0 == COMMA_SYMBOL))
                    {
                        alt764=2;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA764_0 == PARTITION_SYMBOL))
                    {
                        alt764=2;
                    }
                    else if ( (LA764_0 == EOF || LA764_0 == SEMICOLON_SYMBOL))
                    {
                        alt764=2;
                    }
                    else if ( ((SERVER_VERSION >= 50100)) && (LA764_0 == REMOVE_SYMBOL))
                    {
                        alt764=2;
                    }
                    else if ( (LA764_0 == CLOSE_PAR_SYMBOL))
                    {
                        alt764=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 764;
                        EXCEPTION->state        = 0;


                        goto rulecolumn_definitionEx;

                    }
                }
                switch (alt764)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2468:14: reference_definition
            	    {
            	        FOLLOWPUSH(FOLLOW_reference_definition_in_column_definition13825);
            	        reference_definition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2468:37: ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )?
            	    {
            	        // MySQLSimpleParser.g:2468:37: ( CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL )?
            	        {
            	            int alt763=2;
            	            switch ( LA(1) )
            	            {
            	                case CHECK_SYMBOL:
            	                	{
            	                		alt763=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt763)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2468:38: CHECK_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
            	        	    {
            	        	         MATCHT(CHECK_SYMBOL, &FOLLOW_CHECK_SYMBOL_in_column_definition13830);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_column_definition13832);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_expression_in_column_definition13834);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_column_definition13836);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_definitionEx; /* Prevent compiler warnings */
    rulecolumn_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_definition */

/**
 * $ANTLR start field_spec
 * MySQLSimpleParser.g:2471:1: field_spec : column_name field_def ;
 */
static void
field_spec(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2471:11: ( column_name field_def )
        // MySQLSimpleParser.g:2472:2: column_name field_def
        {
            FOLLOWPUSH(FOLLOW_column_name_in_field_spec13848);
            column_name(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefield_specEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_field_def_in_field_spec13850);
            field_def(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefield_specEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_specEx; /* Prevent compiler warnings */
    rulefield_specEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_spec */

/**
 * $ANTLR start field_def
 * MySQLSimpleParser.g:2475:1: field_def : data_type field_def_tail ;
 */
static void
field_def(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2475:10: ( data_type field_def_tail )
        // MySQLSimpleParser.g:2476:2: data_type field_def_tail
        {
            FOLLOWPUSH(FOLLOW_data_type_in_field_def13859);
            data_type(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefield_defEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_field_def_tail_in_field_def13861);
            field_def_tail(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefield_defEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_defEx; /* Prevent compiler warnings */
    rulefield_defEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_def */

/**
 * $ANTLR start field_def_tail
 * MySQLSimpleParser.g:2479:1: field_def_tail options {k=3; } : ( ( attribute )* |{...}? ( COLLATE_SYMBOL collation_name )? ( GENERATED_SYMBOL ALWAYS_SYMBOL )? AS_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL ( VIRTUAL_SYMBOL | STORED_SYMBOL )? ( gcol_attribute )* );
 */
static void
field_def_tail(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2479:34: ( ( attribute )* |{...}? ( COLLATE_SYMBOL collation_name )? ( GENERATED_SYMBOL ALWAYS_SYMBOL )? AS_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL ( VIRTUAL_SYMBOL | STORED_SYMBOL )? ( gcol_attribute )* )

            ANTLR3_UINT32 alt770;

            alt770=2;

            {
                int LA770_0 = LA(1);
                if ( (LA770_0 == COMMENT_SYMBOL || LA770_0 == AFTER_SYMBOL || LA770_0 == AUTO_INCREMENT_SYMBOL || LA770_0 == COLUMN_FORMAT_SYMBOL || LA770_0 == FIRST_SYMBOL || LA770_0 == SERIAL_SYMBOL || LA770_0 == STORAGE_SYMBOL || LA770_0 == DEFAULT_SYMBOL || LA770_0 == KEY_SYMBOL || LA770_0 == NOT_SYMBOL || ((LA770_0 >= NULL2_SYMBOL) && (LA770_0 <= NULL_SYMBOL)) || LA770_0 == ON_SYMBOL || LA770_0 == PRIMARY_SYMBOL || LA770_0 == UNIQUE_SYMBOL || LA770_0 == COMMA_SYMBOL))
                {
                    alt770=1;
                }
                else if ( (LA770_0 == COLLATE_SYMBOL))
                {
                    {
                        int LA770_11 = LA(2);
                        if ( (LA770_11 == SINGLE_QUOTED_TEXT))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 26;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else if ( (LA770_11 == IDENTIFIER))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 27;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else if ( (LA770_11 == BACK_TICK_QUOTED_ID))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 28;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else if ( (LA770_11 == DOUBLE_QUOTED_TEXT))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 29;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else if ( (((LA770_11 >= ACTION_SYMBOL) && (LA770_11 <= SHARE_SYMBOL)) || ((LA770_11 >= SLOW_SYMBOL) && (LA770_11 <= YEAR_SYMBOL))))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 30;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else if ( (((LA770_11 >= ACCOUNT_SYMBOL) && (LA770_11 <= XA_SYMBOL)) || LA770_11 == SHUTDOWN_SYMBOL))
                        {
                            switch ( LA(3) )
                            {
                            case EOF:
                            case COMMENT_SYMBOL:
                            case PARTITION_SYMBOL:
                            case REMOVE_SYMBOL:
                            case AFTER_SYMBOL:
                            case AUTO_INCREMENT_SYMBOL:
                            case COLUMN_FORMAT_SYMBOL:
                            case FIRST_SYMBOL:
                            case SERIAL_SYMBOL:
                            case STORAGE_SYMBOL:
                            case CHECK_SYMBOL:
                            case COLLATE_SYMBOL:
                            case DEFAULT_SYMBOL:
                            case KEY_SYMBOL:
                            case NOT_SYMBOL:
                            case NULL2_SYMBOL:
                            case NULL_SYMBOL:
                            case ON_SYMBOL:
                            case PRIMARY_SYMBOL:
                            case REFERENCES_SYMBOL:
                            case SEMICOLON_SYMBOL:
                            case UNIQUE_SYMBOL:
                            case CLOSE_PAR_SYMBOL:
                            case COMMA_SYMBOL:
                            	{
                            		alt770=1;
                            	}
                                break;
                            case GENERATED_SYMBOL:
                            case AS_SYMBOL:
                            	{
                            		alt770=2;
                            	}
                                break;

                            default:
                                if (BACKTRACKING>0)
                                {
                                    FAILEDFLAG = ANTLR3_TRUE;
                                    return ;
                                }

                                CONSTRUCTEX();
                                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                                EXCEPTION->message      = (void *)"";
                                EXCEPTION->decisionNum  = 770;
                                EXCEPTION->state        = 31;


                                goto rulefield_def_tailEx;

                            }

                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 770;
                            EXCEPTION->state        = 11;


                            goto rulefield_def_tailEx;

                        }
                    }
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA770_0 == PARTITION_SYMBOL))
                {
                    alt770=1;
                }
                else if ( (LA770_0 == EOF || LA770_0 == SEMICOLON_SYMBOL))
                {
                    alt770=1;
                }
                else if ( ((SERVER_VERSION >= 50100)) && (LA770_0 == REMOVE_SYMBOL))
                {
                    alt770=1;
                }
                else if ( (LA770_0 == CHECK_SYMBOL || LA770_0 == REFERENCES_SYMBOL || LA770_0 == CLOSE_PAR_SYMBOL))
                {
                    alt770=1;
                }
                else if ( (LA770_0 == GENERATED_SYMBOL || LA770_0 == AS_SYMBOL))
                {
                    alt770=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 770;
                    EXCEPTION->state        = 0;


                    goto rulefield_def_tailEx;

                }
            }
            switch (alt770)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2480:2: ( attribute )*
        	    {
        	        // MySQLSimpleParser.g:2480:2: ( attribute )*

        	        for (;;)
        	        {
        	            int alt765=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMENT_SYMBOL:
        	            case AUTO_INCREMENT_SYMBOL:
        	            case COLUMN_FORMAT_SYMBOL:
        	            case SERIAL_SYMBOL:
        	            case STORAGE_SYMBOL:
        	            case COLLATE_SYMBOL:
        	            case DEFAULT_SYMBOL:
        	            case KEY_SYMBOL:
        	            case NOT_SYMBOL:
        	            case NULL2_SYMBOL:
        	            case NULL_SYMBOL:
        	            case ON_SYMBOL:
        	            case PRIMARY_SYMBOL:
        	            case UNIQUE_SYMBOL:
        	            	{
        	            		alt765=1;
        	            	}
        	                break;

        	            }

        	            switch (alt765)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2480:2: attribute
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_attribute_in_field_def_tail13881);
        	        	        attribute(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop765;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop765: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2481:4: {...}? ( COLLATE_SYMBOL collation_name )? ( GENERATED_SYMBOL ALWAYS_SYMBOL )? AS_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL ( VIRTUAL_SYMBOL | STORED_SYMBOL )? ( gcol_attribute )*
        	    {
        	        if ( !((SERVER_VERSION >= 50707)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50707";
        	                EXCEPTION->ruleName	 = (void *)"field_def_tail";


        	        }

        	        // MySQLSimpleParser.g:2481:31: ( COLLATE_SYMBOL collation_name )?
        	        {
        	            int alt766=2;
        	            switch ( LA(1) )
        	            {
        	                case COLLATE_SYMBOL:
        	                	{
        	                		alt766=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt766)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2481:32: COLLATE_SYMBOL collation_name
        	        	    {
        	        	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_field_def_tail13890);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_collation_name_in_field_def_tail13892);
        	        	        collation_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2481:64: ( GENERATED_SYMBOL ALWAYS_SYMBOL )?
        	        {
        	            int alt767=2;
        	            switch ( LA(1) )
        	            {
        	                case GENERATED_SYMBOL:
        	                	{
        	                		alt767=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt767)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2481:65: GENERATED_SYMBOL ALWAYS_SYMBOL
        	        	    {
        	        	         MATCHT(GENERATED_SYMBOL, &FOLLOW_GENERATED_SYMBOL_in_field_def_tail13897);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	         MATCHT(ALWAYS_SYMBOL, &FOLLOW_ALWAYS_SYMBOL_in_field_def_tail13899);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_field_def_tail13903);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_def_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_field_def_tail13907);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_def_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_expression_in_field_def_tail13909);
        	        expression(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_def_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_field_def_tail13911);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_def_tailEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2482:47: ( VIRTUAL_SYMBOL | STORED_SYMBOL )?
        	        {
        	            int alt768=2;
        	            switch ( LA(1) )
        	            {
        	                case STORED_SYMBOL:
        	                case VIRTUAL_SYMBOL:
        	                	{
        	                		alt768=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt768)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:
        	        	    {
        	        	        if ( LA(1) == STORED_SYMBOL || LA(1) == VIRTUAL_SYMBOL )
        	        	        {
        	        	            CONSUME();
        	        	            PERRORRECOVERY=ANTLR3_FALSE;
        	        	            FAILEDFLAG=ANTLR3_FALSE;

        	        	        }
        	        	        else
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	            CONSTRUCTEX();
        	        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	        	            EXCEPTION->expectingSet = NULL;

        	        	            goto rulefield_def_tailEx;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2482:81: ( gcol_attribute )*

        	        for (;;)
        	        {
        	            int alt769=2;
        	            switch ( LA(1) )
        	            {
        	            case COMMENT_SYMBOL:
        	            case KEY_SYMBOL:
        	            case NOT2_SYMBOL:
        	            case NOT_SYMBOL:
        	            case NULL_SYMBOL:
        	            case PRIMARY_SYMBOL:
        	            case UNIQUE_SYMBOL:
        	            	{
        	            		alt769=1;
        	            	}
        	                break;

        	            }

        	            switch (alt769)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2482:81: gcol_attribute
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_gcol_attribute_in_field_def_tail13922);
        	        	        gcol_attribute(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_def_tailEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	        	default:
        	        	    goto loop769;	/* break out of the loop */
        	        	    break;
        	            }
        	        }
        	        loop769: ; /* Jump out to here if this rule does not match */


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulefield_def_tailEx; /* Prevent compiler warnings */
    rulefield_def_tailEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_def_tail */

/**
 * $ANTLR start attribute
 * MySQLSimpleParser.g:2485:1: attribute : ( ( NOT_SYMBOL )? null_literal | DEFAULT_SYMBOL ( signed_literal | NOW_SYMBOL ( time_function_parameters )? ) | ON_SYMBOL UPDATE_SYMBOL NOW_SYMBOL ( time_function_parameters )? | AUTO_INCREMENT_SYMBOL | SERIAL_SYMBOL DEFAULT_SYMBOL VALUE_SYMBOL |{...}? UNIQUE_SYMBOL | ( PRIMARY_SYMBOL | UNIQUE_SYMBOL ) KEY_SYMBOL | KEY_SYMBOL | COMMENT_SYMBOL string_literal | COLLATE_SYMBOL collation_name | COLUMN_FORMAT_SYMBOL ( FIXED_SYMBOL | DYNAMIC_SYMBOL | DEFAULT_SYMBOL ) | STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL | DEFAULT_SYMBOL ) );
 */
static void
attribute(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2485:10: ( ( NOT_SYMBOL )? null_literal | DEFAULT_SYMBOL ( signed_literal | NOW_SYMBOL ( time_function_parameters )? ) | ON_SYMBOL UPDATE_SYMBOL NOW_SYMBOL ( time_function_parameters )? | AUTO_INCREMENT_SYMBOL | SERIAL_SYMBOL DEFAULT_SYMBOL VALUE_SYMBOL |{...}? UNIQUE_SYMBOL | ( PRIMARY_SYMBOL | UNIQUE_SYMBOL ) KEY_SYMBOL | KEY_SYMBOL | COMMENT_SYMBOL string_literal | COLLATE_SYMBOL collation_name | COLUMN_FORMAT_SYMBOL ( FIXED_SYMBOL | DYNAMIC_SYMBOL | DEFAULT_SYMBOL ) | STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL | DEFAULT_SYMBOL ) )

            ANTLR3_UINT32 alt775;

            alt775=12;

            switch ( LA(1) )
            {
            case NOT_SYMBOL:
            case NULL2_SYMBOL:
            case NULL_SYMBOL:
            	{
            		alt775=1;
            	}
                break;
            case DEFAULT_SYMBOL:
            	{
            		alt775=2;
            	}
                break;
            case ON_SYMBOL:
            	{
            		alt775=3;
            	}
                break;
            case AUTO_INCREMENT_SYMBOL:
            	{
            		alt775=4;
            	}
                break;
            case SERIAL_SYMBOL:
            	{
            		alt775=5;
            	}
                break;
            case UNIQUE_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case KEY_SYMBOL:
            			{
            				{
            				    int LA775_14 = LA(3);
            				    if ( ((LA(2) != KEY_SYMBOL)))
            				    {
            				        alt775=6;
            				    }
            				    else if ( (ANTLR3_TRUE))
            				    {
            				        alt775=7;
            				    }
            				    else
            				    {
            				        if (BACKTRACKING>0)
            				        {
            				            FAILEDFLAG = ANTLR3_TRUE;
            				            return ;
            				        }


            				        CONSTRUCTEX();
            				        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            				        EXCEPTION->message      = (void *)"";
            				        EXCEPTION->decisionNum  = 775;
            				        EXCEPTION->state        = 14;


            				        goto ruleattributeEx;

            				    }
            				}
            			}
            		    break;
            		case EOF:
            		case COMMENT_SYMBOL:
            		case PARTITION_SYMBOL:
            		case REMOVE_SYMBOL:
            		case AFTER_SYMBOL:
            		case AUTO_INCREMENT_SYMBOL:
            		case COLUMN_FORMAT_SYMBOL:
            		case FIRST_SYMBOL:
            		case SERIAL_SYMBOL:
            		case STORAGE_SYMBOL:
            		case CHECK_SYMBOL:
            		case COLLATE_SYMBOL:
            		case DEFAULT_SYMBOL:
            		case NOT_SYMBOL:
            		case NULL2_SYMBOL:
            		case NULL_SYMBOL:
            		case ON_SYMBOL:
            		case PRIMARY_SYMBOL:
            		case REFERENCES_SYMBOL:
            		case SEMICOLON_SYMBOL:
            		case UNIQUE_SYMBOL:
            		case CLOSE_PAR_SYMBOL:
            		case COMMA_SYMBOL:
            			{
            				alt775=6;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 775;
            		    EXCEPTION->state        = 7;


            		    goto ruleattributeEx;

            		}

            	}
                break;
            case PRIMARY_SYMBOL:
            	{
            		alt775=7;
            	}
                break;
            case KEY_SYMBOL:
            	{
            		alt775=8;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt775=9;
            	}
                break;
            case COLLATE_SYMBOL:
            	{
            		alt775=10;
            	}
                break;
            case COLUMN_FORMAT_SYMBOL:
            	{
            		alt775=11;
            	}
                break;
            case STORAGE_SYMBOL:
            	{
            		alt775=12;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 775;
                EXCEPTION->state        = 0;


                goto ruleattributeEx;

            }

            switch (alt775)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2486:2: ( NOT_SYMBOL )? null_literal
        	    {
        	        // MySQLSimpleParser.g:2486:2: ( NOT_SYMBOL )?
        	        {
        	            int alt771=2;
        	            switch ( LA(1) )
        	            {
        	                case NOT_SYMBOL:
        	                	{
        	                		alt771=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt771)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2486:2: NOT_SYMBOL
        	        	    {
        	        	         MATCHT(NOT_SYMBOL, &FOLLOW_NOT_SYMBOL_in_attribute13932);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleattributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_null_literal_in_attribute13935);
        	        null_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2487:4: DEFAULT_SYMBOL ( signed_literal | NOW_SYMBOL ( time_function_parameters )? )
        	    {
        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_attribute13940);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2487:19: ( signed_literal | NOW_SYMBOL ( time_function_parameters )? )
        	        {
        	            int alt773=2;
        	            switch ( LA(1) )
        	            {
        	            case DATE_SYMBOL:
        	            case TIMESTAMP_SYMBOL:
        	            case TIME_SYMBOL:
        	            case FALSE_SYMBOL:
        	            case NCHAR_TEXT:
        	            case NULL2_SYMBOL:
        	            case NULL_SYMBOL:
        	            case SINGLE_QUOTED_TEXT:
        	            case TRUE_SYMBOL:
        	            case BIN_NUMBER:
        	            case DECIMAL_NUMBER:
        	            case DOUBLE_QUOTED_TEXT:
        	            case FLOAT_NUMBER:
        	            case HEX_NUMBER:
        	            case INT_NUMBER:
        	            case LONG_NUMBER:
        	            case MINUS_OPERATOR:
        	            case PLUS_OPERATOR:
        	            case ULONGLONG_NUMBER:
        	            case UNDERSCORE_CHARSET:
        	            	{
        	            		alt773=1;
        	            	}
        	                break;
        	            case NOW_SYMBOL:
        	            	{
        	            		alt773=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 773;
        	                EXCEPTION->state        = 0;


        	                goto ruleattributeEx;

        	            }

        	            switch (alt773)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2487:20: signed_literal
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_signed_literal_in_attribute13943);
        	        	        signed_literal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleattributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2487:37: NOW_SYMBOL ( time_function_parameters )?
        	        	    {
        	        	         MATCHT(NOW_SYMBOL, &FOLLOW_NOW_SYMBOL_in_attribute13947);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleattributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:2487:48: ( time_function_parameters )?
        	        	        {
        	        	            int alt772=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case OPEN_PAR_SYMBOL:
        	        	                	{
        	        	                		alt772=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt772)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:2487:48: time_function_parameters
        	        	        	    {
        	        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_attribute13949);
        	        	        	        time_function_parameters(ctx);

        	        	        	        FOLLOWPOP();
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto ruleattributeEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2488:4: ON_SYMBOL UPDATE_SYMBOL NOW_SYMBOL ( time_function_parameters )?
        	    {
        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_attribute13956);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_attribute13958);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(NOW_SYMBOL, &FOLLOW_NOW_SYMBOL_in_attribute13960);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2488:39: ( time_function_parameters )?
        	        {
        	            int alt774=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt774=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt774)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2488:39: time_function_parameters
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_time_function_parameters_in_attribute13962);
        	        	        time_function_parameters(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleattributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2489:4: AUTO_INCREMENT_SYMBOL
        	    {
        	         MATCHT(AUTO_INCREMENT_SYMBOL, &FOLLOW_AUTO_INCREMENT_SYMBOL_in_attribute13968);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2490:4: SERIAL_SYMBOL DEFAULT_SYMBOL VALUE_SYMBOL
        	    {
        	         MATCHT(SERIAL_SYMBOL, &FOLLOW_SERIAL_SYMBOL_in_attribute13973);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_attribute13975);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VALUE_SYMBOL, &FOLLOW_VALUE_SYMBOL_in_attribute13977);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2491:4: {...}? UNIQUE_SYMBOL
        	    {
        	        if ( !((LA(2) != KEY_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(2) != KEY_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"attribute";


        	        }

        	         MATCHT(UNIQUE_SYMBOL, &FOLLOW_UNIQUE_SYMBOL_in_attribute13984);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2492:4: ( PRIMARY_SYMBOL | UNIQUE_SYMBOL ) KEY_SYMBOL
        	    {
        	        if ( LA(1) == PRIMARY_SYMBOL || LA(1) == UNIQUE_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleattributeEx;
        	        }


        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_attribute13997);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2493:4: KEY_SYMBOL
        	    {
        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_attribute14002);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2494:4: COMMENT_SYMBOL string_literal
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_attribute14007);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_attribute14009);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2495:4: COLLATE_SYMBOL collation_name
        	    {
        	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_attribute14014);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_collation_name_in_attribute14016);
        	        collation_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2496:4: COLUMN_FORMAT_SYMBOL ( FIXED_SYMBOL | DYNAMIC_SYMBOL | DEFAULT_SYMBOL )
        	    {
        	         MATCHT(COLUMN_FORMAT_SYMBOL, &FOLLOW_COLUMN_FORMAT_SYMBOL_in_attribute14021);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == DYNAMIC_SYMBOL || LA(1) == FIXED_SYMBOL || LA(1) == DEFAULT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleattributeEx;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2497:4: STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL | DEFAULT_SYMBOL )
        	    {
        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_attribute14038);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleattributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == DISK_SYMBOL || LA(1) == MEMORY_SYMBOL || LA(1) == DEFAULT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto ruleattributeEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleattributeEx; /* Prevent compiler warnings */
    ruleattributeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end attribute */

/**
 * $ANTLR start gcol_attribute
 * MySQLSimpleParser.g:2501:1: gcol_attribute : ( UNIQUE_SYMBOL ( options {greedy=true; } : KEY_SYMBOL )? | COMMENT_SYMBOL text_string | ( not_rule )? NULL_SYMBOL | ( PRIMARY_SYMBOL )? KEY_SYMBOL );
 */
static void
gcol_attribute(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2501:15: ( UNIQUE_SYMBOL ( options {greedy=true; } : KEY_SYMBOL )? | COMMENT_SYMBOL text_string | ( not_rule )? NULL_SYMBOL | ( PRIMARY_SYMBOL )? KEY_SYMBOL )

            ANTLR3_UINT32 alt779;

            alt779=4;

            switch ( LA(1) )
            {
            case UNIQUE_SYMBOL:
            	{
            		alt779=1;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt779=2;
            	}
                break;
            case NOT2_SYMBOL:
            case NOT_SYMBOL:
            case NULL_SYMBOL:
            	{
            		alt779=3;
            	}
                break;
            case KEY_SYMBOL:
            case PRIMARY_SYMBOL:
            	{
            		alt779=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 779;
                EXCEPTION->state        = 0;


                goto rulegcol_attributeEx;

            }

            switch (alt779)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2502:2: UNIQUE_SYMBOL ( options {greedy=true; } : KEY_SYMBOL )?
        	    {
        	         MATCHT(UNIQUE_SYMBOL, &FOLLOW_UNIQUE_SYMBOL_in_gcol_attribute14060);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegcol_attributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2502:16: ( options {greedy=true; } : KEY_SYMBOL )?
        	        {
        	            int alt776=2;
        	            switch ( LA(1) )
        	            {
        	                case KEY_SYMBOL:
        	                	{
        	                		alt776=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt776)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2502:45: KEY_SYMBOL
        	        	    {
        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_gcol_attribute14075);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegcol_attributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2503:4: COMMENT_SYMBOL text_string
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_gcol_attribute14083);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegcol_attributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_gcol_attribute14085);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegcol_attributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2504:4: ( not_rule )? NULL_SYMBOL
        	    {
        	        // MySQLSimpleParser.g:2504:4: ( not_rule )?
        	        {
        	            int alt777=2;
        	            switch ( LA(1) )
        	            {
        	                case NOT2_SYMBOL:
        	                case NOT_SYMBOL:
        	                	{
        	                		alt777=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt777)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2504:4: not_rule
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_not_rule_in_gcol_attribute14090);
        	        	        not_rule(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegcol_attributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(NULL_SYMBOL, &FOLLOW_NULL_SYMBOL_in_gcol_attribute14093);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegcol_attributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2505:4: ( PRIMARY_SYMBOL )? KEY_SYMBOL
        	    {
        	        // MySQLSimpleParser.g:2505:4: ( PRIMARY_SYMBOL )?
        	        {
        	            int alt778=2;
        	            switch ( LA(1) )
        	            {
        	                case PRIMARY_SYMBOL:
        	                	{
        	                		alt778=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt778)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2505:4: PRIMARY_SYMBOL
        	        	    {
        	        	         MATCHT(PRIMARY_SYMBOL, &FOLLOW_PRIMARY_SYMBOL_in_gcol_attribute14098);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulegcol_attributeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_gcol_attribute14101);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulegcol_attributeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulegcol_attributeEx; /* Prevent compiler warnings */
    rulegcol_attributeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end gcol_attribute */

/**
 * $ANTLR start reference_definition
 * MySQLSimpleParser.g:2514:1: reference_definition : REFERENCES_SYMBOL table_ref ( index_columns )? ( MATCH_SYMBOL ( FULL_SYMBOL | PARTIAL_SYMBOL | SIMPLE_SYMBOL ) )? ( ON_SYMBOL DELETE_SYMBOL reference_option ( ON_SYMBOL UPDATE_SYMBOL reference_option )? | ON_SYMBOL UPDATE_SYMBOL reference_option ( ON_SYMBOL DELETE_SYMBOL reference_option )? )? ;
 */
static void
reference_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2514:21: ( REFERENCES_SYMBOL table_ref ( index_columns )? ( MATCH_SYMBOL ( FULL_SYMBOL | PARTIAL_SYMBOL | SIMPLE_SYMBOL ) )? ( ON_SYMBOL DELETE_SYMBOL reference_option ( ON_SYMBOL UPDATE_SYMBOL reference_option )? | ON_SYMBOL UPDATE_SYMBOL reference_option ( ON_SYMBOL DELETE_SYMBOL reference_option )? )? )
        // MySQLSimpleParser.g:2515:2: REFERENCES_SYMBOL table_ref ( index_columns )? ( MATCH_SYMBOL ( FULL_SYMBOL | PARTIAL_SYMBOL | SIMPLE_SYMBOL ) )? ( ON_SYMBOL DELETE_SYMBOL reference_option ( ON_SYMBOL UPDATE_SYMBOL reference_option )? | ON_SYMBOL UPDATE_SYMBOL reference_option ( ON_SYMBOL DELETE_SYMBOL reference_option )? )?
        {
             MATCHT(REFERENCES_SYMBOL, &FOLLOW_REFERENCES_SYMBOL_in_reference_definition14113);
            if  (HASEXCEPTION())
            {
                goto rulereference_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_table_ref_in_reference_definition14115);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulereference_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2515:30: ( index_columns )?
            {
                int alt780=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt780=1;
                    	}
                        break;
                }

                switch (alt780)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2515:30: index_columns
            	    {
            	        FOLLOWPUSH(FOLLOW_index_columns_in_reference_definition14117);
            	        index_columns(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2518:3: ( MATCH_SYMBOL ( FULL_SYMBOL | PARTIAL_SYMBOL | SIMPLE_SYMBOL ) )?
            {
                int alt781=2;
                switch ( LA(1) )
                {
                    case MATCH_SYMBOL:
                    	{
                    		alt781=1;
                    	}
                        break;
                }

                switch (alt781)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2518:4: MATCH_SYMBOL ( FULL_SYMBOL | PARTIAL_SYMBOL | SIMPLE_SYMBOL )
            	    {
            	         MATCHT(MATCH_SYMBOL, &FOLLOW_MATCH_SYMBOL_in_reference_definition14129);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        if ( LA(1) == FULL_SYMBOL || LA(1) == PARTIAL_SYMBOL || LA(1) == SIMPLE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulereference_definitionEx;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2519:3: ( ON_SYMBOL DELETE_SYMBOL reference_option ( ON_SYMBOL UPDATE_SYMBOL reference_option )? | ON_SYMBOL UPDATE_SYMBOL reference_option ( ON_SYMBOL DELETE_SYMBOL reference_option )? )?
            {
                int alt784=3;
                switch ( LA(1) )
                {
                    case ON_SYMBOL:
                    	{
                    		switch ( LA(2) )
                    		{
                    		    case DELETE_SYMBOL:
                    		    	{
                    		    		alt784=1;
                    		    	}
                    		        break;
                    		    case UPDATE_SYMBOL:
                    		    	{
                    		    		alt784=2;
                    		    	}
                    		        break;
                    		}

                    	}
                        break;
                }

                switch (alt784)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2520:4: ON_SYMBOL DELETE_SYMBOL reference_option ( ON_SYMBOL UPDATE_SYMBOL reference_option )?
            	    {
            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_reference_definition14152);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DELETE_SYMBOL, &FOLLOW_DELETE_SYMBOL_in_reference_definition14154);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_reference_option_in_reference_definition14156);
            	        reference_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2520:45: ( ON_SYMBOL UPDATE_SYMBOL reference_option )?
            	        {
            	            int alt782=2;
            	            switch ( LA(1) )
            	            {
            	                case ON_SYMBOL:
            	                	{
            	                		alt782=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt782)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2520:46: ON_SYMBOL UPDATE_SYMBOL reference_option
            	        	    {
            	        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_reference_definition14159);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_reference_definition14161);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_reference_option_in_reference_definition14163);
            	        	        reference_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2521:6: ON_SYMBOL UPDATE_SYMBOL reference_option ( ON_SYMBOL DELETE_SYMBOL reference_option )?
            	    {
            	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_reference_definition14172);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(UPDATE_SYMBOL, &FOLLOW_UPDATE_SYMBOL_in_reference_definition14174);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_reference_option_in_reference_definition14176);
            	        reference_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulereference_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2521:47: ( ON_SYMBOL DELETE_SYMBOL reference_option )?
            	        {
            	            int alt783=2;
            	            switch ( LA(1) )
            	            {
            	                case ON_SYMBOL:
            	                	{
            	                		alt783=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt783)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2521:48: ON_SYMBOL DELETE_SYMBOL reference_option
            	        	    {
            	        	         MATCHT(ON_SYMBOL, &FOLLOW_ON_SYMBOL_in_reference_definition14179);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(DELETE_SYMBOL, &FOLLOW_DELETE_SYMBOL_in_reference_definition14181);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_reference_option_in_reference_definition14183);
            	        	        reference_option(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulereference_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulereference_definitionEx; /* Prevent compiler warnings */
    rulereference_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end reference_definition */

/**
 * $ANTLR start index_columns
 * MySQLSimpleParser.g:2525:1: index_columns : OPEN_PAR_SYMBOL index_column ( COMMA_SYMBOL index_column )* CLOSE_PAR_SYMBOL ;
 */
static void
index_columns(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2525:14: ( OPEN_PAR_SYMBOL index_column ( COMMA_SYMBOL index_column )* CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2526:2: OPEN_PAR_SYMBOL index_column ( COMMA_SYMBOL index_column )* CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_index_columns14199);
            if  (HASEXCEPTION())
            {
                goto ruleindex_columnsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_index_column_in_index_columns14201);
            index_column(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_columnsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2526:31: ( COMMA_SYMBOL index_column )*

            for (;;)
            {
                int alt785=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt785=1;
                	}
                    break;

                }

                switch (alt785)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2526:32: COMMA_SYMBOL index_column
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_index_columns14204);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_columnsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_index_column_in_index_columns14206);
            	        index_column(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_columnsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop785;	/* break out of the loop */
            	    break;
                }
            }
            loop785: ; /* Jump out to here if this rule does not match */


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_index_columns14210);
            if  (HASEXCEPTION())
            {
                goto ruleindex_columnsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_columnsEx; /* Prevent compiler warnings */
    ruleindex_columnsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_columns */

/**
 * $ANTLR start index_column
 * MySQLSimpleParser.g:2529:1: index_column : identifier ( field_length )? ( direction )? ;
 */
static void
index_column(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2529:13: ( identifier ( field_length )? ( direction )? )
        // MySQLSimpleParser.g:2530:2: identifier ( field_length )? ( direction )?
        {
            FOLLOWPUSH(FOLLOW_identifier_in_index_column14219);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_columnEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2530:13: ( field_length )?
            {
                int alt786=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt786=1;
                    	}
                        break;
                }

                switch (alt786)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2530:13: field_length
            	    {
            	        FOLLOWPUSH(FOLLOW_field_length_in_index_column14221);
            	        field_length(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_columnEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2530:27: ( direction )?
            {
                int alt787=2;
                switch ( LA(1) )
                {
                    case ASC_SYMBOL:
                    case DESC_SYMBOL:
                    	{
                    		alt787=1;
                    	}
                        break;
                }

                switch (alt787)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2530:27: direction
            	    {
            	        FOLLOWPUSH(FOLLOW_direction_in_index_column14224);
            	        direction(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleindex_columnEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_columnEx; /* Prevent compiler warnings */
    ruleindex_columnEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_column */

/**
 * $ANTLR start index_type
 * MySQLSimpleParser.g:2533:1: index_type : ( USING_SYMBOL | TYPE_SYMBOL ) ( BTREE_SYMBOL | RTREE_SYMBOL | HASH_SYMBOL ) ;
 */
static void
index_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2533:11: ( ( USING_SYMBOL | TYPE_SYMBOL ) ( BTREE_SYMBOL | RTREE_SYMBOL | HASH_SYMBOL ) )
        // MySQLSimpleParser.g:2534:2: ( USING_SYMBOL | TYPE_SYMBOL ) ( BTREE_SYMBOL | RTREE_SYMBOL | HASH_SYMBOL )
        {
            if ( LA(1) == TYPE_SYMBOL || LA(1) == USING_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleindex_typeEx;
            }


            if ( LA(1) == BTREE_SYMBOL || LA(1) == HASH_SYMBOL || LA(1) == RTREE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleindex_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_typeEx; /* Prevent compiler warnings */
    ruleindex_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_type */

/**
 * $ANTLR start normal_index_option
 * MySQLSimpleParser.g:2537:1: normal_index_option : ( index_type | all_key_option );
 */
static void
normal_index_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2537:20: ( index_type | all_key_option )

            ANTLR3_UINT32 alt788;

            alt788=2;

            switch ( LA(1) )
            {
            case TYPE_SYMBOL:
            case USING_SYMBOL:
            	{
            		alt788=1;
            	}
                break;
            case COMMENT_SYMBOL:
            case KEY_BLOCK_SIZE_SYMBOL:
            	{
            		alt788=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 788;
                EXCEPTION->state        = 0;


                goto rulenormal_index_optionEx;

            }

            switch (alt788)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2538:2: index_type
        	    {
        	        FOLLOWPUSH(FOLLOW_index_type_in_normal_index_option14261);
        	        index_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenormal_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2539:4: all_key_option
        	    {
        	        FOLLOWPUSH(FOLLOW_all_key_option_in_normal_index_option14266);
        	        all_key_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenormal_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulenormal_index_optionEx; /* Prevent compiler warnings */
    rulenormal_index_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end normal_index_option */

/**
 * $ANTLR start fulltext_index_option
 * MySQLSimpleParser.g:2542:1: fulltext_index_option : ( WITH_SYMBOL PARSER_SYMBOL identifier | all_key_option );
 */
static void
fulltext_index_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2542:22: ( WITH_SYMBOL PARSER_SYMBOL identifier | all_key_option )

            ANTLR3_UINT32 alt789;

            alt789=2;

            switch ( LA(1) )
            {
            case WITH_SYMBOL:
            	{
            		alt789=1;
            	}
                break;
            case COMMENT_SYMBOL:
            case KEY_BLOCK_SIZE_SYMBOL:
            	{
            		alt789=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 789;
                EXCEPTION->state        = 0;


                goto rulefulltext_index_optionEx;

            }

            switch (alt789)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2543:2: WITH_SYMBOL PARSER_SYMBOL identifier
        	    {
        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_fulltext_index_option14275);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefulltext_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(PARSER_SYMBOL, &FOLLOW_PARSER_SYMBOL_in_fulltext_index_option14277);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefulltext_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_identifier_in_fulltext_index_option14279);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefulltext_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2544:4: all_key_option
        	    {
        	        FOLLOWPUSH(FOLLOW_all_key_option_in_fulltext_index_option14284);
        	        all_key_option(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefulltext_index_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulefulltext_index_optionEx; /* Prevent compiler warnings */
    rulefulltext_index_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end fulltext_index_option */

/**
 * $ANTLR start spatial_index_option
 * MySQLSimpleParser.g:2547:1: spatial_index_option : all_key_option ;
 */
static void
spatial_index_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2547:21: ( all_key_option )
        // MySQLSimpleParser.g:2548:2: all_key_option
        {
            FOLLOWPUSH(FOLLOW_all_key_option_in_spatial_index_option14293);
            all_key_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulespatial_index_optionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulespatial_index_optionEx; /* Prevent compiler warnings */
    rulespatial_index_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end spatial_index_option */

/**
 * $ANTLR start all_key_option
 * MySQLSimpleParser.g:2551:1: all_key_option : ( KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number |{...}? COMMENT_SYMBOL string_literal );
 */
static void
all_key_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2551:15: ( KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number |{...}? COMMENT_SYMBOL string_literal )

            ANTLR3_UINT32 alt791;

            alt791=2;

            switch ( LA(1) )
            {
            case KEY_BLOCK_SIZE_SYMBOL:
            	{
            		alt791=1;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt791=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 791;
                EXCEPTION->state        = 0;


                goto ruleall_key_optionEx;

            }

            switch (alt791)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2552:2: KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number
        	    {
        	         MATCHT(KEY_BLOCK_SIZE_SYMBOL, &FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_all_key_option14302);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_key_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2552:24: ( EQUAL_OPERATOR )?
        	        {
        	            int alt790=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt790=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt790)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2552:24: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_all_key_option14304);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleall_key_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulong_number_in_all_key_option14307);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_key_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2553:4: {...}? COMMENT_SYMBOL string_literal
        	    {
        	        if ( !((SERVER_VERSION >= 50600)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
        	                EXCEPTION->ruleName	 = (void *)"all_key_option";


        	        }

        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_all_key_option14314);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_key_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_literal_in_all_key_option14316);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleall_key_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleall_key_optionEx; /* Prevent compiler warnings */
    ruleall_key_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end all_key_option */

/**
 * $ANTLR start reference_option
 * MySQLSimpleParser.g:2556:1: reference_option : ( RESTRICT_SYMBOL | CASCADE_SYMBOL | SET_SYMBOL null_literal | NO_SYMBOL ACTION_SYMBOL );
 */
static void
reference_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2556:17: ( RESTRICT_SYMBOL | CASCADE_SYMBOL | SET_SYMBOL null_literal | NO_SYMBOL ACTION_SYMBOL )

            ANTLR3_UINT32 alt792;

            alt792=4;

            switch ( LA(1) )
            {
            case RESTRICT_SYMBOL:
            	{
            		alt792=1;
            	}
                break;
            case CASCADE_SYMBOL:
            	{
            		alt792=2;
            	}
                break;
            case SET_SYMBOL:
            	{
            		alt792=3;
            	}
                break;
            case NO_SYMBOL:
            	{
            		alt792=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 792;
                EXCEPTION->state        = 0;


                goto rulereference_optionEx;

            }

            switch (alt792)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2557:2: RESTRICT_SYMBOL
        	    {
        	         MATCHT(RESTRICT_SYMBOL, &FOLLOW_RESTRICT_SYMBOL_in_reference_option14325);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2557:20: CASCADE_SYMBOL
        	    {
        	         MATCHT(CASCADE_SYMBOL, &FOLLOW_CASCADE_SYMBOL_in_reference_option14329);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2557:37: SET_SYMBOL null_literal
        	    {
        	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_reference_option14333);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_null_literal_in_reference_option14335);
        	        null_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2557:63: NO_SYMBOL ACTION_SYMBOL
        	    {
        	         MATCHT(NO_SYMBOL, &FOLLOW_NO_SYMBOL_in_reference_option14339);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(ACTION_SYMBOL, &FOLLOW_ACTION_SYMBOL_in_reference_option14341);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereference_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulereference_optionEx; /* Prevent compiler warnings */
    rulereference_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end reference_option */

/**
 * $ANTLR start data_type
 * MySQLSimpleParser.g:2560:1: data_type : data_type_elements ;
 */
static void
data_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2560:10: ( data_type_elements )
        // MySQLSimpleParser.g:2561:2: data_type_elements
        {
            FOLLOWPUSH(FOLLOW_data_type_elements_in_data_type14350);
            data_type_elements(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruledata_typeEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledata_typeEx; /* Prevent compiler warnings */
    ruledata_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end data_type */

/**
 * $ANTLR start data_type_definition
 * MySQLSimpleParser.g:2564:1: data_type_definition : data_type_elements EOF ;
 */
static void
data_type_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2564:21: ( data_type_elements EOF )
        // MySQLSimpleParser.g:2565:2: data_type_elements EOF
        {
            FOLLOWPUSH(FOLLOW_data_type_elements_in_data_type_definition14360);
            data_type_elements(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruledata_type_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EOF, &FOLLOW_EOF_in_data_type_definition14362);
            if  (HASEXCEPTION())
            {
                goto ruledata_type_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledata_type_definitionEx; /* Prevent compiler warnings */
    ruledata_type_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end data_type_definition */

/**
 * $ANTLR start data_type_elements
 * MySQLSimpleParser.g:2568:1: data_type_elements : ( integer_type ( field_length )? ( field_options )? | real_literal ( precision )? ( field_options )? | FLOAT_SYMBOL ( float_options )? ( field_options )? | BIT_SYMBOL ( field_length )? | BOOL_SYMBOL | BOOLEAN_SYMBOL | CHAR_SYMBOL ( field_length )? ( string_binary )? | nchar_literal ( field_length )? ( BINARY_SYMBOL )? | BINARY_SYMBOL ( field_length )? | varchar_literal field_length ( string_binary )? | nvarchar_literal field_length ( BINARY_SYMBOL )? | VARBINARY_SYMBOL field_length | YEAR_SYMBOL ( field_length )? ( field_options )? | DATE_SYMBOL | TIME_SYMBOL ( type_datetime_precision )? | TIMESTAMP_SYMBOL ( type_datetime_precision )? | DATETIME_SYMBOL ( type_datetime_precision )? | TINYBLOB_SYMBOL | BLOB_SYMBOL ( field_length )? | MEDIUMBLOB_SYMBOL | LONGBLOB_SYMBOL | LONG_SYMBOL VARBINARY_SYMBOL | LONG_SYMBOL ( varchar_literal )? ( string_binary )? | TINYTEXT_SYMBOL ( string_binary )? | TEXT_SYMBOL ( field_length )? ( string_binary )? | MEDIUMTEXT_SYMBOL ( string_binary )? | LONGTEXT_SYMBOL ( string_binary )? | DECIMAL_SYMBOL ( float_options )? ( field_options )? | NUMERIC_SYMBOL ( float_options )? ( field_options )? | FIXED_SYMBOL ( float_options )? ( field_options )? | ENUM_SYMBOL string_list ( string_binary )? | SET_SYMBOL string_list ( string_binary )? | SERIAL_SYMBOL |{...}? JSON_SYMBOL | spatial_type );
 */
static void
data_type_elements(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2568:19: ( integer_type ( field_length )? ( field_options )? | real_literal ( precision )? ( field_options )? | FLOAT_SYMBOL ( float_options )? ( field_options )? | BIT_SYMBOL ( field_length )? | BOOL_SYMBOL | BOOLEAN_SYMBOL | CHAR_SYMBOL ( field_length )? ( string_binary )? | nchar_literal ( field_length )? ( BINARY_SYMBOL )? | BINARY_SYMBOL ( field_length )? | varchar_literal field_length ( string_binary )? | nvarchar_literal field_length ( BINARY_SYMBOL )? | VARBINARY_SYMBOL field_length | YEAR_SYMBOL ( field_length )? ( field_options )? | DATE_SYMBOL | TIME_SYMBOL ( type_datetime_precision )? | TIMESTAMP_SYMBOL ( type_datetime_precision )? | DATETIME_SYMBOL ( type_datetime_precision )? | TINYBLOB_SYMBOL | BLOB_SYMBOL ( field_length )? | MEDIUMBLOB_SYMBOL | LONGBLOB_SYMBOL | LONG_SYMBOL VARBINARY_SYMBOL | LONG_SYMBOL ( varchar_literal )? ( string_binary )? | TINYTEXT_SYMBOL ( string_binary )? | TEXT_SYMBOL ( field_length )? ( string_binary )? | MEDIUMTEXT_SYMBOL ( string_binary )? | LONGTEXT_SYMBOL ( string_binary )? | DECIMAL_SYMBOL ( float_options )? ( field_options )? | NUMERIC_SYMBOL ( float_options )? ( field_options )? | FIXED_SYMBOL ( float_options )? ( field_options )? | ENUM_SYMBOL string_list ( string_binary )? | SET_SYMBOL string_list ( string_binary )? | SERIAL_SYMBOL |{...}? JSON_SYMBOL | spatial_type )

            ANTLR3_UINT32 alt828;

            alt828=35;

            {
                int LA828_0 = LA(1);
                if ( (LA828_0 == BIGINT_SYMBOL || LA828_0 == INT_SYMBOL || LA828_0 == MEDIUMINT_SYMBOL || LA828_0 == SMALLINT_SYMBOL || LA828_0 == TINYINT_SYMBOL))
                {
                    alt828=1;
                }
                else if ( (LA828_0 == REAL_SYMBOL || LA828_0 == DOUBLE_SYMBOL))
                {
                    alt828=2;
                }
                else if ( (LA828_0 == FLOAT_SYMBOL))
                {
                    alt828=3;
                }
                else if ( (LA828_0 == BIT_SYMBOL))
                {
                    alt828=4;
                }
                else if ( (LA828_0 == BOOL_SYMBOL))
                {
                    alt828=5;
                }
                else if ( (LA828_0 == BOOLEAN_SYMBOL))
                {
                    alt828=6;
                }
                else if ( (LA828_0 == CHAR_SYMBOL))
                {
                    {
                        int LA828_8 = LA(2);
                        if ( (LA828_8 == VARYING_SYMBOL))
                        {
                            alt828=10;
                        }
                        else if ( (LA828_8 == EOF || LA828_8 == ASCII_SYMBOL || ((LA828_8 >= BACKUP_SYMBOL) && (LA828_8 <= DO_SYMBOL)) || ((LA828_8 >= EXECUTE_SYMBOL) && (LA828_8 <= FLUSH_SYMBOL)) || ((LA828_8 >= HANDLER_SYMBOL) && (LA828_8 <= HELP_SYMBOL)) || ((LA828_8 >= INSTALL_SYMBOL) && (LA828_8 <= OPEN_SYMBOL)) || LA828_8 == PARTITION_SYMBOL || ((LA828_8 >= PREPARE_SYMBOL) && (LA828_8 <= SAVEPOINT_SYMBOL)) || ((LA828_8 >= START_SYMBOL) && (LA828_8 <= UNINSTALL_SYMBOL)) || ((LA828_8 >= XA_SYMBOL) && (LA828_8 <= YEAR_SYMBOL)) || LA828_8 == ALTER_SYMBOL || LA828_8 == BINARY_SYMBOL || LA828_8 == CALL_SYMBOL || LA828_8 == CASE_SYMBOL || LA828_8 == CHANGE_SYMBOL || ((LA828_8 >= CHAR_SYMBOL) && (LA828_8 <= COLLATE_SYMBOL)) || LA828_8 == CREATE_SYMBOL || LA828_8 == DEFAULT_SYMBOL || ((LA828_8 >= DELETE_SYMBOL) && (LA828_8 <= DETERMINISTIC_SYMBOL)) || LA828_8 == DROP_SYMBOL || LA828_8 == FETCH_SYMBOL || ((LA828_8 >= GENERATED_SYMBOL) && (LA828_8 <= GRANT_SYMBOL)) || LA828_8 == IF_SYMBOL || LA828_8 == INSERT_SYMBOL || LA828_8 == ITERATE_SYMBOL || ((LA828_8 >= KEY_SYMBOL) && (LA828_8 <= KILL_SYMBOL)) || LA828_8 == LEAVE_SYMBOL || LA828_8 == LOAD_SYMBOL || LA828_8 == LOCK_SYMBOL || LA828_8 == LOOP_SYMBOL || LA828_8 == MODIFIES_SYMBOL || LA828_8 == NOT_SYMBOL || ((LA828_8 >= NULL2_SYMBOL) && (LA828_8 <= NULL_SYMBOL)) || LA828_8 == ON_SYMBOL || LA828_8 == OPEN_PAR_SYMBOL || LA828_8 == OPTIMIZE_SYMBOL || LA828_8 == PRIMARY_SYMBOL || LA828_8 == PURGE_SYMBOL || LA828_8 == READS_SYMBOL || LA828_8 == REFERENCES_SYMBOL || ((LA828_8 >= RELEASE_SYMBOL) && (LA828_8 <= REPLACE_SYMBOL)) || LA828_8 == RESIGNAL_SYMBOL || ((LA828_8 >= RETURN_SYMBOL) && (LA828_8 <= REVOKE_SYMBOL)) || ((LA828_8 >= SELECT_SYMBOL) && (LA828_8 <= SEMICOLON_SYMBOL)) || LA828_8 == SET_SYMBOL || ((LA828_8 >= SHOW_SYMBOL) && (LA828_8 <= SIGNAL_SYMBOL)) || LA828_8 == SQL_SYMBOL || ((LA828_8 >= UNIQUE_SYMBOL) && (LA828_8 <= UNLOCK_SYMBOL)) || LA828_8 == UPDATE_SYMBOL || LA828_8 == USE_SYMBOL || LA828_8 == WHILE_SYMBOL || LA828_8 == AS_SYMBOL || LA828_8 == BACK_TICK_QUOTED_ID || LA828_8 == CLOSE_PAR_SYMBOL || LA828_8 == COMMA_SYMBOL || LA828_8 == DOUBLE_QUOTED_TEXT || LA828_8 == IDENTIFIER))
                        {
                            alt828=7;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 828;
                            EXCEPTION->state        = 8;


                            goto ruledata_type_elementsEx;

                        }
                    }
                }
                else if ( (LA828_0 == NCHAR_SYMBOL))
                {
                    {
                        int LA828_9 = LA(2);
                        if ( (LA828_9 == VARCHAR_SYMBOL || LA828_9 == VARYING_SYMBOL))
                        {
                            alt828=11;
                        }
                        else if ( (LA828_9 == EOF || ((LA828_9 >= BACKUP_SYMBOL) && (LA828_9 <= BEGIN_SYMBOL)) || LA828_9 == CACHE_SYMBOL || ((LA828_9 >= CHECKSUM_SYMBOL) && (LA828_9 <= DO_SYMBOL)) || ((LA828_9 >= EXECUTE_SYMBOL) && (LA828_9 <= FLUSH_SYMBOL)) || ((LA828_9 >= HANDLER_SYMBOL) && (LA828_9 <= HELP_SYMBOL)) || ((LA828_9 >= INSTALL_SYMBOL) && (LA828_9 <= OPEN_SYMBOL)) || LA828_9 == PARTITION_SYMBOL || ((LA828_9 >= PREPARE_SYMBOL) && (LA828_9 <= SAVEPOINT_SYMBOL)) || ((LA828_9 >= START_SYMBOL) && (LA828_9 <= TRUNCATE_SYMBOL)) || LA828_9 == UNINSTALL_SYMBOL || ((LA828_9 >= XA_SYMBOL) && (LA828_9 <= YEAR_SYMBOL)) || LA828_9 == ALTER_SYMBOL || LA828_9 == BINARY_SYMBOL || LA828_9 == CALL_SYMBOL || LA828_9 == CASE_SYMBOL || LA828_9 == CHANGE_SYMBOL || ((LA828_9 >= CHECK_SYMBOL) && (LA828_9 <= COLLATE_SYMBOL)) || LA828_9 == CREATE_SYMBOL || LA828_9 == DEFAULT_SYMBOL || ((LA828_9 >= DELETE_SYMBOL) && (LA828_9 <= DETERMINISTIC_SYMBOL)) || LA828_9 == DROP_SYMBOL || LA828_9 == FETCH_SYMBOL || ((LA828_9 >= GENERATED_SYMBOL) && (LA828_9 <= GRANT_SYMBOL)) || LA828_9 == IF_SYMBOL || LA828_9 == INSERT_SYMBOL || LA828_9 == ITERATE_SYMBOL || ((LA828_9 >= KEY_SYMBOL) && (LA828_9 <= KILL_SYMBOL)) || LA828_9 == LEAVE_SYMBOL || LA828_9 == LOAD_SYMBOL || LA828_9 == LOCK_SYMBOL || LA828_9 == LOOP_SYMBOL || LA828_9 == MODIFIES_SYMBOL || LA828_9 == NOT_SYMBOL || ((LA828_9 >= NULL2_SYMBOL) && (LA828_9 <= NULL_SYMBOL)) || LA828_9 == ON_SYMBOL || LA828_9 == OPEN_PAR_SYMBOL || LA828_9 == OPTIMIZE_SYMBOL || LA828_9 == PRIMARY_SYMBOL || LA828_9 == PURGE_SYMBOL || LA828_9 == READS_SYMBOL || LA828_9 == REFERENCES_SYMBOL || ((LA828_9 >= RELEASE_SYMBOL) && (LA828_9 <= REPLACE_SYMBOL)) || LA828_9 == RESIGNAL_SYMBOL || ((LA828_9 >= RETURN_SYMBOL) && (LA828_9 <= REVOKE_SYMBOL)) || ((LA828_9 >= SELECT_SYMBOL) && (LA828_9 <= SEMICOLON_SYMBOL)) || LA828_9 == SET_SYMBOL || ((LA828_9 >= SHOW_SYMBOL) && (LA828_9 <= SIGNAL_SYMBOL)) || LA828_9 == SQL_SYMBOL || ((LA828_9 >= UNIQUE_SYMBOL) && (LA828_9 <= UNLOCK_SYMBOL)) || LA828_9 == UPDATE_SYMBOL || LA828_9 == USE_SYMBOL || LA828_9 == WHILE_SYMBOL || LA828_9 == AS_SYMBOL || LA828_9 == BACK_TICK_QUOTED_ID || LA828_9 == CLOSE_PAR_SYMBOL || LA828_9 == COMMA_SYMBOL || LA828_9 == DOUBLE_QUOTED_TEXT || LA828_9 == IDENTIFIER))
                        {
                            alt828=8;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 828;
                            EXCEPTION->state        = 9;


                            goto ruledata_type_elementsEx;

                        }
                    }
                }
                else if ( (LA828_0 == NATIONAL_SYMBOL))
                {
                    switch ( LA(2) )
                    {
                    case CHAR_SYMBOL:
                    	{
                    		{
                    		    int LA828_249 = LA(3);
                    		    if ( ((LA(3) != VARYING_SYMBOL)))
                    		    {
                    		        alt828=8;
                    		    }
                    		    else if ( (ANTLR3_TRUE))
                    		    {
                    		        alt828=11;
                    		    }
                    		    else
                    		    {
                    		        if (BACKTRACKING>0)
                    		        {
                    		            FAILEDFLAG = ANTLR3_TRUE;
                    		            return ;
                    		        }


                    		        CONSTRUCTEX();
                    		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    		        EXCEPTION->message      = (void *)"";
                    		        EXCEPTION->decisionNum  = 828;
                    		        EXCEPTION->state        = 249;


                    		        goto ruledata_type_elementsEx;

                    		    }
                    		}
                    	}
                        break;
                    case VARCHAR_SYMBOL:
                    	{
                    		alt828=11;
                    	}
                        break;

                    default:
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }

                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 828;
                        EXCEPTION->state        = 10;


                        goto ruledata_type_elementsEx;

                    }

                }
                else if ( (LA828_0 == BINARY_SYMBOL))
                {
                    alt828=9;
                }
                else if ( (LA828_0 == VARCHAR_SYMBOL))
                {
                    alt828=10;
                }
                else if ( (LA828_0 == NVARCHAR_SYMBOL))
                {
                    alt828=11;
                }
                else if ( (LA828_0 == VARBINARY_SYMBOL))
                {
                    alt828=12;
                }
                else if ( (LA828_0 == YEAR_SYMBOL))
                {
                    alt828=13;
                }
                else if ( (LA828_0 == DATE_SYMBOL))
                {
                    alt828=14;
                }
                else if ( (LA828_0 == TIME_SYMBOL))
                {
                    alt828=15;
                }
                else if ( (LA828_0 == TIMESTAMP_SYMBOL))
                {
                    alt828=16;
                }
                else if ( (LA828_0 == DATETIME_SYMBOL))
                {
                    alt828=17;
                }
                else if ( (LA828_0 == TINYBLOB_SYMBOL))
                {
                    alt828=18;
                }
                else if ( (LA828_0 == BLOB_SYMBOL))
                {
                    alt828=19;
                }
                else if ( (LA828_0 == MEDIUMBLOB_SYMBOL))
                {
                    alt828=20;
                }
                else if ( (LA828_0 == LONGBLOB_SYMBOL))
                {
                    alt828=21;
                }
                else if ( (LA828_0 == LONG_SYMBOL))
                {
                    {
                        int LA828_24 = LA(2);
                        if ( (LA828_24 == VARBINARY_SYMBOL))
                        {
                            alt828=22;
                        }
                        else if ( (LA828_24 == EOF || LA828_24 == ASCII_SYMBOL || ((LA828_24 >= BACKUP_SYMBOL) && (LA828_24 <= DO_SYMBOL)) || ((LA828_24 >= EXECUTE_SYMBOL) && (LA828_24 <= FLUSH_SYMBOL)) || ((LA828_24 >= HANDLER_SYMBOL) && (LA828_24 <= HELP_SYMBOL)) || ((LA828_24 >= INSTALL_SYMBOL) && (LA828_24 <= OPEN_SYMBOL)) || LA828_24 == PARTITION_SYMBOL || ((LA828_24 >= PREPARE_SYMBOL) && (LA828_24 <= SAVEPOINT_SYMBOL)) || ((LA828_24 >= START_SYMBOL) && (LA828_24 <= UNINSTALL_SYMBOL)) || ((LA828_24 >= XA_SYMBOL) && (LA828_24 <= YEAR_SYMBOL)) || LA828_24 == ALTER_SYMBOL || LA828_24 == BINARY_SYMBOL || LA828_24 == CALL_SYMBOL || LA828_24 == CASE_SYMBOL || LA828_24 == CHANGE_SYMBOL || ((LA828_24 >= CHAR_SYMBOL) && (LA828_24 <= COLLATE_SYMBOL)) || LA828_24 == CREATE_SYMBOL || LA828_24 == DEFAULT_SYMBOL || ((LA828_24 >= DELETE_SYMBOL) && (LA828_24 <= DETERMINISTIC_SYMBOL)) || LA828_24 == DROP_SYMBOL || LA828_24 == FETCH_SYMBOL || ((LA828_24 >= GENERATED_SYMBOL) && (LA828_24 <= GRANT_SYMBOL)) || LA828_24 == IF_SYMBOL || LA828_24 == INSERT_SYMBOL || LA828_24 == ITERATE_SYMBOL || ((LA828_24 >= KEY_SYMBOL) && (LA828_24 <= KILL_SYMBOL)) || LA828_24 == LEAVE_SYMBOL || LA828_24 == LOAD_SYMBOL || LA828_24 == LOCK_SYMBOL || LA828_24 == LOOP_SYMBOL || LA828_24 == MODIFIES_SYMBOL || LA828_24 == NOT_SYMBOL || ((LA828_24 >= NULL2_SYMBOL) && (LA828_24 <= NULL_SYMBOL)) || LA828_24 == ON_SYMBOL || LA828_24 == OPEN_PAR_SYMBOL || LA828_24 == OPTIMIZE_SYMBOL || LA828_24 == PRIMARY_SYMBOL || LA828_24 == PURGE_SYMBOL || LA828_24 == READS_SYMBOL || LA828_24 == REFERENCES_SYMBOL || ((LA828_24 >= RELEASE_SYMBOL) && (LA828_24 <= REPLACE_SYMBOL)) || LA828_24 == RESIGNAL_SYMBOL || ((LA828_24 >= RETURN_SYMBOL) && (LA828_24 <= REVOKE_SYMBOL)) || ((LA828_24 >= SELECT_SYMBOL) && (LA828_24 <= SEMICOLON_SYMBOL)) || LA828_24 == SET_SYMBOL || ((LA828_24 >= SHOW_SYMBOL) && (LA828_24 <= SIGNAL_SYMBOL)) || LA828_24 == SQL_SYMBOL || ((LA828_24 >= UNIQUE_SYMBOL) && (LA828_24 <= UNLOCK_SYMBOL)) || LA828_24 == UPDATE_SYMBOL || LA828_24 == USE_SYMBOL || LA828_24 == VARCHAR_SYMBOL || LA828_24 == WHILE_SYMBOL || LA828_24 == AS_SYMBOL || LA828_24 == BACK_TICK_QUOTED_ID || LA828_24 == CLOSE_PAR_SYMBOL || LA828_24 == COMMA_SYMBOL || LA828_24 == DOUBLE_QUOTED_TEXT || LA828_24 == IDENTIFIER))
                        {
                            alt828=23;
                        }
                        else
                        {
                            if (BACKTRACKING>0)
                            {
                                FAILEDFLAG = ANTLR3_TRUE;
                                return ;
                            }


                            CONSTRUCTEX();
                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                            EXCEPTION->message      = (void *)"";
                            EXCEPTION->decisionNum  = 828;
                            EXCEPTION->state        = 24;


                            goto ruledata_type_elementsEx;

                        }
                    }
                }
                else if ( (LA828_0 == TINYTEXT_SYMBOL))
                {
                    alt828=24;
                }
                else if ( (LA828_0 == TEXT_SYMBOL))
                {
                    alt828=25;
                }
                else if ( (LA828_0 == MEDIUMTEXT_SYMBOL))
                {
                    alt828=26;
                }
                else if ( (LA828_0 == LONGTEXT_SYMBOL))
                {
                    alt828=27;
                }
                else if ( (LA828_0 == DECIMAL_SYMBOL))
                {
                    alt828=28;
                }
                else if ( (LA828_0 == NUMERIC_SYMBOL))
                {
                    alt828=29;
                }
                else if ( (LA828_0 == FIXED_SYMBOL))
                {
                    alt828=30;
                }
                else if ( (LA828_0 == ENUM_SYMBOL))
                {
                    alt828=31;
                }
                else if ( (LA828_0 == SET_SYMBOL))
                {
                    alt828=32;
                }
                else if ( (LA828_0 == SERIAL_SYMBOL))
                {
                    alt828=33;
                }
                else if ( (LA828_0 == JSON_SYMBOL))
                {
                    alt828=34;
                }
                else if ( (((LA828_0 >= GEOMETRY_SYMBOL) && (LA828_0 <= GEOMETRYCOLLECTION_SYMBOL)) || LA828_0 == LINESTRING_SYMBOL || ((LA828_0 >= MULTILINESTRING_SYMBOL) && (LA828_0 <= MULTIPOLYGON_SYMBOL)) || ((LA828_0 >= POINT_SYMBOL) && (LA828_0 <= POLYGON_SYMBOL))))
                {
                    alt828=35;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 828;
                    EXCEPTION->state        = 0;


                    goto ruledata_type_elementsEx;

                }
            }
            switch (alt828)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2569:2: integer_type ( field_length )? ( field_options )?
        	    {
        	        FOLLOWPUSH(FOLLOW_integer_type_in_data_type_elements14371);
        	        integer_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2569:15: ( field_length )?
        	        {
        	            int alt793=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt793=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt793)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2569:15: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14373);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2569:29: ( field_options )?
        	        {
        	            int alt794=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt794=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt794)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2569:29: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14376);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2571:4: real_literal ( precision )? ( field_options )?
        	    {
        	        FOLLOWPUSH(FOLLOW_real_literal_in_data_type_elements14383);
        	        real_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2571:17: ( precision )?
        	        {
        	            int alt795=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt795=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt795)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2571:17: precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_precision_in_data_type_elements14385);
        	        	        precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2571:28: ( field_options )?
        	        {
        	            int alt796=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt796=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt796)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2571:28: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14388);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2572:4: FLOAT_SYMBOL ( float_options )? ( field_options )?
        	    {
        	         MATCHT(FLOAT_SYMBOL, &FOLLOW_FLOAT_SYMBOL_in_data_type_elements14394);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2572:17: ( float_options )?
        	        {
        	            int alt797=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt797=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt797)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2572:17: float_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_float_options_in_data_type_elements14396);
        	        	        float_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2572:32: ( field_options )?
        	        {
        	            int alt798=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt798=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt798)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2572:32: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14399);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2574:4: BIT_SYMBOL ( field_length )?
        	    {
        	         MATCHT(BIT_SYMBOL, &FOLLOW_BIT_SYMBOL_in_data_type_elements14407);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2574:15: ( field_length )?
        	        {
        	            int alt799=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt799=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt799)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2574:15: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14409);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2575:4: BOOL_SYMBOL
        	    {
        	         MATCHT(BOOL_SYMBOL, &FOLLOW_BOOL_SYMBOL_in_data_type_elements14415);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2576:4: BOOLEAN_SYMBOL
        	    {
        	         MATCHT(BOOLEAN_SYMBOL, &FOLLOW_BOOLEAN_SYMBOL_in_data_type_elements14420);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2578:4: CHAR_SYMBOL ( field_length )? ( string_binary )?
        	    {
        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_data_type_elements14427);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2578:16: ( field_length )?
        	        {
        	            int alt800=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt800=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt800)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2578:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14429);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2578:30: ( string_binary )?
        	        {
        	            int alt801=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt801=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt801)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2578:30: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14432);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2579:4: nchar_literal ( field_length )? ( BINARY_SYMBOL )?
        	    {
        	        FOLLOWPUSH(FOLLOW_nchar_literal_in_data_type_elements14438);
        	        nchar_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2579:18: ( field_length )?
        	        {
        	            int alt802=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt802=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt802)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2579:18: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14440);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2579:32: ( BINARY_SYMBOL )?
        	        {
        	            int alt803=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt803=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt803)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2579:32: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_data_type_elements14443);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2580:4: BINARY_SYMBOL ( field_length )?
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_data_type_elements14449);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2580:18: ( field_length )?
        	        {
        	            int alt804=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt804=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt804)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2580:18: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14451);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2581:4: varchar_literal field_length ( string_binary )?
        	    {
        	        FOLLOWPUSH(FOLLOW_varchar_literal_in_data_type_elements14457);
        	        varchar_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14459);
        	        field_length(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2581:33: ( string_binary )?
        	        {
        	            int alt805=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt805=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt805)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2581:33: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14461);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2582:4: nvarchar_literal field_length ( BINARY_SYMBOL )?
        	    {
        	        FOLLOWPUSH(FOLLOW_nvarchar_literal_in_data_type_elements14467);
        	        nvarchar_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14469);
        	        field_length(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2582:34: ( BINARY_SYMBOL )?
        	        {
        	            int alt806=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt806=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt806)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2582:34: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_data_type_elements14471);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2584:4: VARBINARY_SYMBOL field_length
        	    {
        	         MATCHT(VARBINARY_SYMBOL, &FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14478);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14480);
        	        field_length(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:2586:4: YEAR_SYMBOL ( field_length )? ( field_options )?
        	    {
        	         MATCHT(YEAR_SYMBOL, &FOLLOW_YEAR_SYMBOL_in_data_type_elements14486);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2586:16: ( field_length )?
        	        {
        	            int alt807=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt807=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt807)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2586:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14488);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2586:30: ( field_options )?
        	        {
        	            int alt808=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt808=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt808)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2586:30: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14491);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:2587:4: DATE_SYMBOL
        	    {
        	         MATCHT(DATE_SYMBOL, &FOLLOW_DATE_SYMBOL_in_data_type_elements14497);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:2588:4: TIME_SYMBOL ( type_datetime_precision )?
        	    {
        	         MATCHT(TIME_SYMBOL, &FOLLOW_TIME_SYMBOL_in_data_type_elements14502);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2588:16: ( type_datetime_precision )?
        	        {
        	            int alt809=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt809=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt809)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2588:16: type_datetime_precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_type_datetime_precision_in_data_type_elements14504);
        	        	        type_datetime_precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:2589:4: TIMESTAMP_SYMBOL ( type_datetime_precision )?
        	    {
        	         MATCHT(TIMESTAMP_SYMBOL, &FOLLOW_TIMESTAMP_SYMBOL_in_data_type_elements14510);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2589:21: ( type_datetime_precision )?
        	        {
        	            int alt810=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt810=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt810)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2589:21: type_datetime_precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_type_datetime_precision_in_data_type_elements14512);
        	        	        type_datetime_precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:2590:4: DATETIME_SYMBOL ( type_datetime_precision )?
        	    {
        	         MATCHT(DATETIME_SYMBOL, &FOLLOW_DATETIME_SYMBOL_in_data_type_elements14518);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2590:20: ( type_datetime_precision )?
        	        {
        	            int alt811=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt811=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt811)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2590:20: type_datetime_precision
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_type_datetime_precision_in_data_type_elements14520);
        	        	        type_datetime_precision(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:2592:4: TINYBLOB_SYMBOL
        	    {
        	         MATCHT(TINYBLOB_SYMBOL, &FOLLOW_TINYBLOB_SYMBOL_in_data_type_elements14527);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:2593:4: BLOB_SYMBOL ( field_length )?
        	    {
        	         MATCHT(BLOB_SYMBOL, &FOLLOW_BLOB_SYMBOL_in_data_type_elements14532);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2593:16: ( field_length )?
        	        {
        	            int alt812=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt812=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt812)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2593:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14534);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:2594:4: MEDIUMBLOB_SYMBOL
        	    {
        	         MATCHT(MEDIUMBLOB_SYMBOL, &FOLLOW_MEDIUMBLOB_SYMBOL_in_data_type_elements14540);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:2595:4: LONGBLOB_SYMBOL
        	    {
        	         MATCHT(LONGBLOB_SYMBOL, &FOLLOW_LONGBLOB_SYMBOL_in_data_type_elements14545);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 22:
        	    // MySQLSimpleParser.g:2596:4: LONG_SYMBOL VARBINARY_SYMBOL
        	    {
        	         MATCHT(LONG_SYMBOL, &FOLLOW_LONG_SYMBOL_in_data_type_elements14550);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARBINARY_SYMBOL, &FOLLOW_VARBINARY_SYMBOL_in_data_type_elements14552);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 23:
        	    // MySQLSimpleParser.g:2597:4: LONG_SYMBOL ( varchar_literal )? ( string_binary )?
        	    {
        	         MATCHT(LONG_SYMBOL, &FOLLOW_LONG_SYMBOL_in_data_type_elements14557);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2597:16: ( varchar_literal )?
        	        {
        	            int alt813=2;
        	            switch ( LA(1) )
        	            {
        	                case CHAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case VARYING_SYMBOL:
        	                		    	{
        	                		    		alt813=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	                case VARCHAR_SYMBOL:
        	                	{
        	                		alt813=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt813)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2597:16: varchar_literal
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_varchar_literal_in_data_type_elements14559);
        	        	        varchar_literal(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2597:33: ( string_binary )?
        	        {
        	            int alt814=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt814=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt814)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2597:33: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14562);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 24:
        	    // MySQLSimpleParser.g:2599:4: TINYTEXT_SYMBOL ( string_binary )?
        	    {
        	         MATCHT(TINYTEXT_SYMBOL, &FOLLOW_TINYTEXT_SYMBOL_in_data_type_elements14569);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2599:20: ( string_binary )?
        	        {
        	            int alt815=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt815=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt815)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2599:20: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14571);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 25:
        	    // MySQLSimpleParser.g:2600:4: TEXT_SYMBOL ( field_length )? ( string_binary )?
        	    {
        	         MATCHT(TEXT_SYMBOL, &FOLLOW_TEXT_SYMBOL_in_data_type_elements14577);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2600:16: ( field_length )?
        	        {
        	            int alt816=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case DECIMAL_NUMBER:
        	                		    case INT_NUMBER:
        	                		    case LONG_NUMBER:
        	                		    case ULONGLONG_NUMBER:
        	                		    	{
        	                		    		alt816=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt816)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2600:16: field_length
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_length_in_data_type_elements14579);
        	        	        field_length(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2600:30: ( string_binary )?
        	        {
        	            int alt817=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt817=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt817)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2600:30: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14582);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 26:
        	    // MySQLSimpleParser.g:2601:4: MEDIUMTEXT_SYMBOL ( string_binary )?
        	    {
        	         MATCHT(MEDIUMTEXT_SYMBOL, &FOLLOW_MEDIUMTEXT_SYMBOL_in_data_type_elements14588);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2601:22: ( string_binary )?
        	        {
        	            int alt818=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt818=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt818)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2601:22: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14590);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 27:
        	    // MySQLSimpleParser.g:2602:4: LONGTEXT_SYMBOL ( string_binary )?
        	    {
        	         MATCHT(LONGTEXT_SYMBOL, &FOLLOW_LONGTEXT_SYMBOL_in_data_type_elements14596);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2602:20: ( string_binary )?
        	        {
        	            int alt819=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt819=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt819)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2602:20: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14598);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 28:
        	    // MySQLSimpleParser.g:2604:4: DECIMAL_SYMBOL ( float_options )? ( field_options )?
        	    {
        	         MATCHT(DECIMAL_SYMBOL, &FOLLOW_DECIMAL_SYMBOL_in_data_type_elements14606);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2604:19: ( float_options )?
        	        {
        	            int alt820=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt820=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt820)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2604:19: float_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_float_options_in_data_type_elements14608);
        	        	        float_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2604:34: ( field_options )?
        	        {
        	            int alt821=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt821=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt821)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2604:34: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14611);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 29:
        	    // MySQLSimpleParser.g:2605:4: NUMERIC_SYMBOL ( float_options )? ( field_options )?
        	    {
        	         MATCHT(NUMERIC_SYMBOL, &FOLLOW_NUMERIC_SYMBOL_in_data_type_elements14617);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2605:19: ( float_options )?
        	        {
        	            int alt822=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt822=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt822)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2605:19: float_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_float_options_in_data_type_elements14619);
        	        	        float_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2605:34: ( field_options )?
        	        {
        	            int alt823=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt823=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt823)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2605:34: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14622);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 30:
        	    // MySQLSimpleParser.g:2606:4: FIXED_SYMBOL ( float_options )? ( field_options )?
        	    {
        	         MATCHT(FIXED_SYMBOL, &FOLLOW_FIXED_SYMBOL_in_data_type_elements14628);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2606:17: ( float_options )?
        	        {
        	            int alt824=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		switch ( LA(2) )
        	                		{
        	                		    case INT_NUMBER:
        	                		    	{
        	                		    		alt824=1;
        	                		    	}
        	                		        break;
        	                		}

        	                	}
        	                    break;
        	            }

        	            switch (alt824)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2606:17: float_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_float_options_in_data_type_elements14630);
        	        	        float_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2606:32: ( field_options )?
        	        {
        	            int alt825=2;
        	            switch ( LA(1) )
        	            {
        	                case SIGNED_SYMBOL:
        	                case UNSIGNED_SYMBOL:
        	                case ZEROFILL_SYMBOL:
        	                	{
        	                		alt825=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt825)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2606:32: field_options
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_field_options_in_data_type_elements14633);
        	        	        field_options(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 31:
        	    // MySQLSimpleParser.g:2608:4: ENUM_SYMBOL string_list ( string_binary )?
        	    {
        	         MATCHT(ENUM_SYMBOL, &FOLLOW_ENUM_SYMBOL_in_data_type_elements14641);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_list_in_data_type_elements14643);
        	        string_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2608:28: ( string_binary )?
        	        {
        	            int alt826=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt826=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt826)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2608:28: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14645);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 32:
        	    // MySQLSimpleParser.g:2609:4: SET_SYMBOL string_list ( string_binary )?
        	    {
        	         MATCHT(SET_SYMBOL, &FOLLOW_SET_SYMBOL_in_data_type_elements14651);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_string_list_in_data_type_elements14653);
        	        string_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2609:27: ( string_binary )?
        	        {
        	            int alt827=2;
        	            switch ( LA(1) )
        	            {
        	                case ASCII_SYMBOL:
        	                case BYTE_SYMBOL:
        	                case CHARSET_SYMBOL:
        	                case UNICODE_SYMBOL:
        	                case BINARY_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt827=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt827)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2609:27: string_binary
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_string_binary_in_data_type_elements14655);
        	        	        string_binary(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruledata_type_elementsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 33:
        	    // MySQLSimpleParser.g:2610:4: SERIAL_SYMBOL
        	    {
        	         MATCHT(SERIAL_SYMBOL, &FOLLOW_SERIAL_SYMBOL_in_data_type_elements14661);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 34:
        	    // MySQLSimpleParser.g:2611:4: {...}? JSON_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50707)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50707";
        	                EXCEPTION->ruleName	 = (void *)"data_type_elements";


        	        }

        	         MATCHT(JSON_SYMBOL, &FOLLOW_JSON_SYMBOL_in_data_type_elements14668);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 35:
        	    // MySQLSimpleParser.g:2612:4: spatial_type
        	    {
        	        FOLLOWPUSH(FOLLOW_spatial_type_in_data_type_elements14673);
        	        spatial_type(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruledata_type_elementsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruledata_type_elementsEx; /* Prevent compiler warnings */
    ruledata_type_elementsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end data_type_elements */

/**
 * $ANTLR start field_length
 * MySQLSimpleParser.g:2615:1: field_length : OPEN_PAR_SYMBOL ( real_ulonglong_number | DECIMAL_NUMBER ) CLOSE_PAR_SYMBOL ;
 */
static void
field_length(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2615:13: ( OPEN_PAR_SYMBOL ( real_ulonglong_number | DECIMAL_NUMBER ) CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2616:2: OPEN_PAR_SYMBOL ( real_ulonglong_number | DECIMAL_NUMBER ) CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_field_length14682);
            if  (HASEXCEPTION())
            {
                goto rulefield_lengthEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2616:18: ( real_ulonglong_number | DECIMAL_NUMBER )
            {
                int alt829=2;
                switch ( LA(1) )
                {
                case INT_NUMBER:
                case LONG_NUMBER:
                case ULONGLONG_NUMBER:
                	{
                		alt829=1;
                	}
                    break;
                case DECIMAL_NUMBER:
                	{
                		alt829=2;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 829;
                    EXCEPTION->state        = 0;


                    goto rulefield_lengthEx;

                }

                switch (alt829)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2616:19: real_ulonglong_number
            	    {
            	        FOLLOWPUSH(FOLLOW_real_ulonglong_number_in_field_length14685);
            	        real_ulonglong_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_lengthEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2616:43: DECIMAL_NUMBER
            	    {
            	         MATCHT(DECIMAL_NUMBER, &FOLLOW_DECIMAL_NUMBER_in_field_length14689);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_lengthEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_field_length14692);
            if  (HASEXCEPTION())
            {
                goto rulefield_lengthEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_lengthEx; /* Prevent compiler warnings */
    rulefield_lengthEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_length */

/**
 * $ANTLR start field_options
 * MySQLSimpleParser.g:2619:1: field_options : ( SIGNED_SYMBOL | UNSIGNED_SYMBOL | ZEROFILL_SYMBOL )+ ;
 */
static void
field_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2619:14: ( ( SIGNED_SYMBOL | UNSIGNED_SYMBOL | ZEROFILL_SYMBOL )+ )
        // MySQLSimpleParser.g:2620:2: ( SIGNED_SYMBOL | UNSIGNED_SYMBOL | ZEROFILL_SYMBOL )+
        {
            // MySQLSimpleParser.g:2620:2: ( SIGNED_SYMBOL | UNSIGNED_SYMBOL | ZEROFILL_SYMBOL )+
            {
                int cnt830=0;

                for (;;)
                {
                    int alt830=2;
            	switch ( LA(1) )
            	{
            	case SIGNED_SYMBOL:
            	case UNSIGNED_SYMBOL:
            	case ZEROFILL_SYMBOL:
            		{
            			alt830=1;
            		}
            	    break;

            	}

            	switch (alt830)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:
            	        {
            	            if ( LA(1) == SIGNED_SYMBOL || LA(1) == UNSIGNED_SYMBOL || LA(1) == ZEROFILL_SYMBOL )
            	            {
            	                CONSUME();
            	                PERRORRECOVERY=ANTLR3_FALSE;
            	                FAILEDFLAG=ANTLR3_FALSE;

            	            }
            	            else
            	            {
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	                EXCEPTION->expectingSet = NULL;

            	                goto rulefield_optionsEx;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt830 >= 1 )
            		{
            		    goto loop830;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulefield_optionsEx;
            	}
            	cnt830++;
                }
                loop830: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_optionsEx; /* Prevent compiler warnings */
    rulefield_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_options */

/**
 * $ANTLR start string_binary
 * MySQLSimpleParser.g:2623:1: string_binary : ( ascii | unicode | BYTE_SYMBOL | charset charset_name ( BINARY_SYMBOL )? | BINARY_SYMBOL ( charset charset_name )? );
 */
static void
string_binary(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2623:14: ( ascii | unicode | BYTE_SYMBOL | charset charset_name ( BINARY_SYMBOL )? | BINARY_SYMBOL ( charset charset_name )? )

            ANTLR3_UINT32 alt833;

            alt833=5;

            switch ( LA(1) )
            {
            case ASCII_SYMBOL:
            	{
            		alt833=1;
            	}
                break;
            case BINARY_SYMBOL:
            	{
            		{
            		    int LA833_2 = LA(2);
            		    if ( (LA833_2 == ASCII_SYMBOL))
            		    {
            		        alt833=1;
            		    }
            		    else if ( (LA833_2 == UNICODE_SYMBOL))
            		    {
            		        alt833=2;
            		    }
            		    else if ( (LA833_2 == EOF || ((LA833_2 >= BACKUP_SYMBOL) && (LA833_2 <= BEGIN_SYMBOL)) || ((LA833_2 >= CACHE_SYMBOL) && (LA833_2 <= DO_SYMBOL)) || ((LA833_2 >= EXECUTE_SYMBOL) && (LA833_2 <= FLUSH_SYMBOL)) || ((LA833_2 >= HANDLER_SYMBOL) && (LA833_2 <= HELP_SYMBOL)) || ((LA833_2 >= INSTALL_SYMBOL) && (LA833_2 <= OPEN_SYMBOL)) || LA833_2 == PARTITION_SYMBOL || ((LA833_2 >= PREPARE_SYMBOL) && (LA833_2 <= SAVEPOINT_SYMBOL)) || ((LA833_2 >= START_SYMBOL) && (LA833_2 <= TRUNCATE_SYMBOL)) || LA833_2 == UNINSTALL_SYMBOL || ((LA833_2 >= XA_SYMBOL) && (LA833_2 <= YEAR_SYMBOL)) || LA833_2 == ALTER_SYMBOL || LA833_2 == CALL_SYMBOL || LA833_2 == CASE_SYMBOL || LA833_2 == CHANGE_SYMBOL || ((LA833_2 >= CHAR_SYMBOL) && (LA833_2 <= COLLATE_SYMBOL)) || LA833_2 == CREATE_SYMBOL || LA833_2 == DEFAULT_SYMBOL || ((LA833_2 >= DELETE_SYMBOL) && (LA833_2 <= DETERMINISTIC_SYMBOL)) || LA833_2 == DROP_SYMBOL || LA833_2 == FETCH_SYMBOL || ((LA833_2 >= GENERATED_SYMBOL) && (LA833_2 <= GRANT_SYMBOL)) || LA833_2 == IF_SYMBOL || LA833_2 == INSERT_SYMBOL || LA833_2 == ITERATE_SYMBOL || ((LA833_2 >= KEY_SYMBOL) && (LA833_2 <= KILL_SYMBOL)) || LA833_2 == LEAVE_SYMBOL || LA833_2 == LOAD_SYMBOL || LA833_2 == LOCK_SYMBOL || LA833_2 == LOOP_SYMBOL || LA833_2 == MODIFIES_SYMBOL || LA833_2 == NOT_SYMBOL || ((LA833_2 >= NULL2_SYMBOL) && (LA833_2 <= NULL_SYMBOL)) || LA833_2 == ON_SYMBOL || LA833_2 == OPEN_PAR_SYMBOL || LA833_2 == OPTIMIZE_SYMBOL || LA833_2 == PRIMARY_SYMBOL || LA833_2 == PURGE_SYMBOL || LA833_2 == READS_SYMBOL || LA833_2 == REFERENCES_SYMBOL || ((LA833_2 >= RELEASE_SYMBOL) && (LA833_2 <= REPLACE_SYMBOL)) || LA833_2 == RESIGNAL_SYMBOL || ((LA833_2 >= RETURN_SYMBOL) && (LA833_2 <= REVOKE_SYMBOL)) || ((LA833_2 >= SELECT_SYMBOL) && (LA833_2 <= SEMICOLON_SYMBOL)) || LA833_2 == SET_SYMBOL || ((LA833_2 >= SHOW_SYMBOL) && (LA833_2 <= SIGNAL_SYMBOL)) || LA833_2 == SQL_SYMBOL || ((LA833_2 >= UNIQUE_SYMBOL) && (LA833_2 <= UNLOCK_SYMBOL)) || LA833_2 == UPDATE_SYMBOL || LA833_2 == USE_SYMBOL || LA833_2 == WHILE_SYMBOL || LA833_2 == AS_SYMBOL || LA833_2 == BACK_TICK_QUOTED_ID || LA833_2 == CLOSE_PAR_SYMBOL || LA833_2 == COMMA_SYMBOL || LA833_2 == DOUBLE_QUOTED_TEXT || LA833_2 == IDENTIFIER))
            		    {
            		        alt833=5;
            		    }
            		    else
            		    {
            		        if (BACKTRACKING>0)
            		        {
            		            FAILEDFLAG = ANTLR3_TRUE;
            		            return ;
            		        }


            		        CONSTRUCTEX();
            		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		        EXCEPTION->message      = (void *)"";
            		        EXCEPTION->decisionNum  = 833;
            		        EXCEPTION->state        = 2;


            		        goto rulestring_binaryEx;

            		    }
            		}
            	}
                break;
            case UNICODE_SYMBOL:
            	{
            		alt833=2;
            	}
                break;
            case BYTE_SYMBOL:
            	{
            		alt833=3;
            	}
                break;
            case CHARSET_SYMBOL:
            case CHAR_SYMBOL:
            	{
            		alt833=4;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 833;
                EXCEPTION->state        = 0;


                goto rulestring_binaryEx;

            }

            switch (alt833)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2624:2: ascii
        	    {
        	        FOLLOWPUSH(FOLLOW_ascii_in_string_binary14721);
        	        ascii(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2625:4: unicode
        	    {
        	        FOLLOWPUSH(FOLLOW_unicode_in_string_binary14726);
        	        unicode(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2626:4: BYTE_SYMBOL
        	    {
        	         MATCHT(BYTE_SYMBOL, &FOLLOW_BYTE_SYMBOL_in_string_binary14731);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2627:4: charset charset_name ( BINARY_SYMBOL )?
        	    {
        	        FOLLOWPUSH(FOLLOW_charset_in_string_binary14736);
        	        charset(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_charset_name_in_string_binary14738);
        	        charset_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2627:25: ( BINARY_SYMBOL )?
        	        {
        	            int alt831=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt831=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt831)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2627:25: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_string_binary14740);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulestring_binaryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2628:4: BINARY_SYMBOL ( charset charset_name )?
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_string_binary14746);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestring_binaryEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2628:18: ( charset charset_name )?
        	        {
        	            int alt832=2;
        	            switch ( LA(1) )
        	            {
        	                case CHARSET_SYMBOL:
        	                case CHAR_SYMBOL:
        	                	{
        	                		alt832=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt832)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2628:19: charset charset_name
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_charset_in_string_binary14749);
        	        	        charset(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulestring_binaryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_charset_name_in_string_binary14751);
        	        	        charset_name(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulestring_binaryEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulestring_binaryEx; /* Prevent compiler warnings */
    rulestring_binaryEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end string_binary */

/**
 * $ANTLR start ascii
 * MySQLSimpleParser.g:2631:1: ascii : ( ASCII_SYMBOL ( BINARY_SYMBOL )? |{...}? BINARY_SYMBOL ASCII_SYMBOL );
 */
static void
ascii(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2631:6: ( ASCII_SYMBOL ( BINARY_SYMBOL )? |{...}? BINARY_SYMBOL ASCII_SYMBOL )

            ANTLR3_UINT32 alt835;

            alt835=2;

            switch ( LA(1) )
            {
            case ASCII_SYMBOL:
            	{
            		alt835=1;
            	}
                break;
            case BINARY_SYMBOL:
            	{
            		alt835=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 835;
                EXCEPTION->state        = 0;


                goto ruleasciiEx;

            }

            switch (alt835)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2632:2: ASCII_SYMBOL ( BINARY_SYMBOL )?
        	    {
        	         MATCHT(ASCII_SYMBOL, &FOLLOW_ASCII_SYMBOL_in_ascii14762);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleasciiEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2632:15: ( BINARY_SYMBOL )?
        	        {
        	            int alt834=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt834=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt834)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2632:15: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_ascii14764);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleasciiEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2633:4: {...}? BINARY_SYMBOL ASCII_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"ascii";


        	        }

        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_ascii14772);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleasciiEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(ASCII_SYMBOL, &FOLLOW_ASCII_SYMBOL_in_ascii14774);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleasciiEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleasciiEx; /* Prevent compiler warnings */
    ruleasciiEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end ascii */

/**
 * $ANTLR start unicode
 * MySQLSimpleParser.g:2636:1: unicode : ( UNICODE_SYMBOL ( BINARY_SYMBOL )? |{...}? BINARY_SYMBOL UNICODE_SYMBOL );
 */
static void
unicode(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2636:8: ( UNICODE_SYMBOL ( BINARY_SYMBOL )? |{...}? BINARY_SYMBOL UNICODE_SYMBOL )

            ANTLR3_UINT32 alt837;

            alt837=2;

            switch ( LA(1) )
            {
            case UNICODE_SYMBOL:
            	{
            		alt837=1;
            	}
                break;
            case BINARY_SYMBOL:
            	{
            		alt837=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 837;
                EXCEPTION->state        = 0;


                goto ruleunicodeEx;

            }

            switch (alt837)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2637:2: UNICODE_SYMBOL ( BINARY_SYMBOL )?
        	    {
        	         MATCHT(UNICODE_SYMBOL, &FOLLOW_UNICODE_SYMBOL_in_unicode14783);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunicodeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2637:17: ( BINARY_SYMBOL )?
        	        {
        	            int alt836=2;
        	            switch ( LA(1) )
        	            {
        	                case BINARY_SYMBOL:
        	                	{
        	                		alt836=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt836)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2637:17: BINARY_SYMBOL
        	        	    {
        	        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_unicode14785);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleunicodeEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2638:4: {...}? BINARY_SYMBOL UNICODE_SYMBOL
        	    {
        	        if ( !((SERVER_VERSION >= 50500)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
        	                EXCEPTION->ruleName	 = (void *)"unicode";


        	        }

        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_unicode14793);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunicodeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(UNICODE_SYMBOL, &FOLLOW_UNICODE_SYMBOL_in_unicode14795);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleunicodeEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleunicodeEx; /* Prevent compiler warnings */
    ruleunicodeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end unicode */

/**
 * $ANTLR start type_datetime_precision
 * MySQLSimpleParser.g:2641:1: type_datetime_precision :{...}? OPEN_PAR_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL ;
 */
static void
type_datetime_precision(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2641:24: ({...}? OPEN_PAR_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2642:2: {...}? OPEN_PAR_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL
        {
            if ( !((SERVER_VERSION >= 50600)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
                    EXCEPTION->ruleName	 = (void *)"type_datetime_precision";


            }

             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_type_datetime_precision14806);
            if  (HASEXCEPTION())
            {
                goto ruletype_datetime_precisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_type_datetime_precision14808);
            if  (HASEXCEPTION())
            {
                goto ruletype_datetime_precisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_type_datetime_precision14810);
            if  (HASEXCEPTION())
            {
                goto ruletype_datetime_precisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletype_datetime_precisionEx; /* Prevent compiler warnings */
    ruletype_datetime_precisionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end type_datetime_precision */

/**
 * $ANTLR start charset_name
 * MySQLSimpleParser.g:2645:1: charset_name : ( text_or_identifier | BINARY_SYMBOL );
 */
static void
charset_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2645:13: ( text_or_identifier | BINARY_SYMBOL )

            ANTLR3_UINT32 alt838;

            alt838=2;

            {
                int LA838_0 = LA(1);
                if ( (((LA838_0 >= ACCOUNT_SYMBOL) && (LA838_0 <= YEAR_SYMBOL)) || LA838_0 == SINGLE_QUOTED_TEXT || LA838_0 == BACK_TICK_QUOTED_ID || LA838_0 == DOUBLE_QUOTED_TEXT || LA838_0 == IDENTIFIER))
                {
                    alt838=1;
                }
                else if ( (LA838_0 == BINARY_SYMBOL))
                {
                    alt838=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 838;
                    EXCEPTION->state        = 0;


                    goto rulecharset_nameEx;

                }
            }
            switch (alt838)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2646:2: text_or_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_charset_name14819);
        	        text_or_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharset_nameEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2647:4: BINARY_SYMBOL
        	    {
        	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_charset_name14824);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharset_nameEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecharset_nameEx; /* Prevent compiler warnings */
    rulecharset_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end charset_name */

/**
 * $ANTLR start charset_name_or_default
 * MySQLSimpleParser.g:2650:1: charset_name_or_default : ( charset_name | DEFAULT_SYMBOL );
 */
static void
charset_name_or_default(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2650:24: ( charset_name | DEFAULT_SYMBOL )

            ANTLR3_UINT32 alt839;

            alt839=2;

            {
                int LA839_0 = LA(1);
                if ( (((LA839_0 >= ACCOUNT_SYMBOL) && (LA839_0 <= YEAR_SYMBOL)) || LA839_0 == BINARY_SYMBOL || LA839_0 == SINGLE_QUOTED_TEXT || LA839_0 == BACK_TICK_QUOTED_ID || LA839_0 == DOUBLE_QUOTED_TEXT || LA839_0 == IDENTIFIER))
                {
                    alt839=1;
                }
                else if ( (LA839_0 == DEFAULT_SYMBOL))
                {
                    alt839=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 839;
                    EXCEPTION->state        = 0;


                    goto rulecharset_name_or_defaultEx;

                }
            }
            switch (alt839)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2651:2: charset_name
        	    {
        	        FOLLOWPUSH(FOLLOW_charset_name_in_charset_name_or_default14833);
        	        charset_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharset_name_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2652:4: DEFAULT_SYMBOL
        	    {
        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_charset_name_or_default14838);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecharset_name_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecharset_name_or_defaultEx; /* Prevent compiler warnings */
    rulecharset_name_or_defaultEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end charset_name_or_default */

/**
 * $ANTLR start collation_name
 * MySQLSimpleParser.g:2655:1: collation_name : text_or_identifier ;
 */
static void
collation_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2655:15: ( text_or_identifier )
        // MySQLSimpleParser.g:2656:2: text_or_identifier
        {
            FOLLOWPUSH(FOLLOW_text_or_identifier_in_collation_name14847);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecollation_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecollation_nameEx; /* Prevent compiler warnings */
    rulecollation_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end collation_name */

/**
 * $ANTLR start collation_name_or_default
 * MySQLSimpleParser.g:2659:1: collation_name_or_default : ( collation_name | DEFAULT_SYMBOL );
 */
static void
collation_name_or_default(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2659:26: ( collation_name | DEFAULT_SYMBOL )

            ANTLR3_UINT32 alt840;

            alt840=2;

            {
                int LA840_0 = LA(1);
                if ( (((LA840_0 >= ACCOUNT_SYMBOL) && (LA840_0 <= YEAR_SYMBOL)) || LA840_0 == SINGLE_QUOTED_TEXT || LA840_0 == BACK_TICK_QUOTED_ID || LA840_0 == DOUBLE_QUOTED_TEXT || LA840_0 == IDENTIFIER))
                {
                    alt840=1;
                }
                else if ( (LA840_0 == DEFAULT_SYMBOL))
                {
                    alt840=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 840;
                    EXCEPTION->state        = 0;


                    goto rulecollation_name_or_defaultEx;

                }
            }
            switch (alt840)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2660:2: collation_name
        	    {
        	        FOLLOWPUSH(FOLLOW_collation_name_in_collation_name_or_default14856);
        	        collation_name(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecollation_name_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2661:4: DEFAULT_SYMBOL
        	    {
        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_collation_name_or_default14861);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecollation_name_or_defaultEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecollation_name_or_defaultEx; /* Prevent compiler warnings */
    rulecollation_name_or_defaultEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end collation_name_or_default */

/**
 * $ANTLR start spatial_type
 * MySQLSimpleParser.g:2664:1: spatial_type : ( GEOMETRY_SYMBOL | GEOMETRYCOLLECTION_SYMBOL | POINT_SYMBOL | MULTIPOINT_SYMBOL | LINESTRING_SYMBOL | MULTILINESTRING_SYMBOL | POLYGON_SYMBOL | MULTIPOLYGON_SYMBOL );
 */
static void
spatial_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2664:13: ( GEOMETRY_SYMBOL | GEOMETRYCOLLECTION_SYMBOL | POINT_SYMBOL | MULTIPOINT_SYMBOL | LINESTRING_SYMBOL | MULTILINESTRING_SYMBOL | POLYGON_SYMBOL | MULTIPOLYGON_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= GEOMETRY_SYMBOL) && (LA(1) <= GEOMETRYCOLLECTION_SYMBOL)) || LA(1) == LINESTRING_SYMBOL || ((LA(1) >= MULTILINESTRING_SYMBOL) && (LA(1) <= MULTIPOLYGON_SYMBOL)) || ((LA(1) >= POINT_SYMBOL) && (LA(1) <= POLYGON_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulespatial_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulespatial_typeEx; /* Prevent compiler warnings */
    rulespatial_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end spatial_type */

/**
 * $ANTLR start alter_table_options
 * MySQLSimpleParser.g:2675:1: alter_table_options : ( create_table_option )+ ;
 */
static void
alter_table_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2675:20: ( ( create_table_option )+ )
        // MySQLSimpleParser.g:2676:2: ( create_table_option )+
        {
            // MySQLSimpleParser.g:2676:2: ( create_table_option )+
            {
                int cnt841=0;

                for (;;)
                {
                    int alt841=2;
            	switch ( LA(1) )
            	{
            	case CHARSET_SYMBOL:
            	case CHECKSUM_SYMBOL:
            	case COMMENT_SYMBOL:
            	case AUTO_INCREMENT_SYMBOL:
            	case AVG_ROW_LENGTH_SYMBOL:
            	case COMPRESSION_SYMBOL:
            	case CONNECTION_SYMBOL:
            	case DATA_SYMBOL:
            	case DELAY_KEY_WRITE_SYMBOL:
            	case ENGINE_SYMBOL:
            	case INSERT_METHOD_SYMBOL:
            	case KEY_BLOCK_SIZE_SYMBOL:
            	case MAX_ROWS_SYMBOL:
            	case MIN_ROWS_SYMBOL:
            	case PACK_KEYS_SYMBOL:
            	case PASSWORD_SYMBOL:
            	case ROW_FORMAT_SYMBOL:
            	case STATS_AUTO_RECALC_SYMBOL:
            	case STATS_PERSISTENT_SYMBOL:
            	case STATS_SAMPLE_PAGES_SYMBOL:
            	case STORAGE_SYMBOL:
            	case TABLE_CHECKSUM_SYMBOL:
            	case TABLESPACE_SYMBOL:
            	case TYPE_SYMBOL:
            	case CHAR_SYMBOL:
            	case COLLATE_SYMBOL:
            	case DEFAULT_SYMBOL:
            	case INDEX_SYMBOL:
            	case UNION_SYMBOL:
            		{
            			alt841=1;
            		}
            	    break;

            	}

            	switch (alt841)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2676:2: create_table_option
            	        {
            	            FOLLOWPUSH(FOLLOW_create_table_option_in_alter_table_options14914);
            	            create_table_option(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulealter_table_optionsEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt841 >= 1 )
            		{
            		    goto loop841;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulealter_table_optionsEx;
            	}
            	cnt841++;
                }
                loop841: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulealter_table_optionsEx; /* Prevent compiler warnings */
    rulealter_table_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end alter_table_options */

/**
 * $ANTLR start create_table_options
 * MySQLSimpleParser.g:2679:1: create_table_options : create_table_option ( ( COMMA_SYMBOL )? create_table_option )* ;
 */
static void
create_table_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2679:21: ( create_table_option ( ( COMMA_SYMBOL )? create_table_option )* )
        // MySQLSimpleParser.g:2680:2: create_table_option ( ( COMMA_SYMBOL )? create_table_option )*
        {
            FOLLOWPUSH(FOLLOW_create_table_option_in_create_table_options14924);
            create_table_option(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecreate_table_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2680:22: ( ( COMMA_SYMBOL )? create_table_option )*

            for (;;)
            {
                int alt843=2;
                switch ( LA(1) )
                {
                case CHARSET_SYMBOL:
                case CHECKSUM_SYMBOL:
                case COMMENT_SYMBOL:
                case AUTO_INCREMENT_SYMBOL:
                case AVG_ROW_LENGTH_SYMBOL:
                case COMPRESSION_SYMBOL:
                case CONNECTION_SYMBOL:
                case DATA_SYMBOL:
                case DELAY_KEY_WRITE_SYMBOL:
                case ENGINE_SYMBOL:
                case INSERT_METHOD_SYMBOL:
                case KEY_BLOCK_SIZE_SYMBOL:
                case MAX_ROWS_SYMBOL:
                case MIN_ROWS_SYMBOL:
                case PACK_KEYS_SYMBOL:
                case PASSWORD_SYMBOL:
                case ROW_FORMAT_SYMBOL:
                case STATS_AUTO_RECALC_SYMBOL:
                case STATS_PERSISTENT_SYMBOL:
                case STATS_SAMPLE_PAGES_SYMBOL:
                case STORAGE_SYMBOL:
                case TABLE_CHECKSUM_SYMBOL:
                case TABLESPACE_SYMBOL:
                case TYPE_SYMBOL:
                case CHAR_SYMBOL:
                case COLLATE_SYMBOL:
                case DEFAULT_SYMBOL:
                case INDEX_SYMBOL:
                case UNION_SYMBOL:
                case COMMA_SYMBOL:
                	{
                		alt843=1;
                	}
                    break;

                }

                switch (alt843)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2680:23: ( COMMA_SYMBOL )? create_table_option
            	    {
            	        // MySQLSimpleParser.g:2680:23: ( COMMA_SYMBOL )?
            	        {
            	            int alt842=2;
            	            switch ( LA(1) )
            	            {
            	                case COMMA_SYMBOL:
            	                	{
            	                		alt842=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt842)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2680:23: COMMA_SYMBOL
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_create_table_options14927);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecreate_table_optionsEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_create_table_option_in_create_table_options14930);
            	        create_table_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecreate_table_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop843;	/* break out of the loop */
            	    break;
                }
            }
            loop843: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecreate_table_optionsEx; /* Prevent compiler warnings */
    rulecreate_table_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_table_options */

/**
 * $ANTLR start create_table_option
 * MySQLSimpleParser.g:2683:1: create_table_option : ( ( ENGINE_SYMBOL |{...}? TYPE_SYMBOL ) ( EQUAL_OPERATOR )? engine_ref | MAX_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | MIN_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | AVG_ROW_LENGTH_SYMBOL ( EQUAL_OPERATOR )? ulong_number | PASSWORD_SYMBOL ( EQUAL_OPERATOR )? text_string | COMMENT_SYMBOL ( EQUAL_OPERATOR )? text_string | COMPRESSION_SYMBOL ( EQUAL_OPERATOR )? text_string | AUTO_INCREMENT_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | PACK_KEYS_SYMBOL ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL ) |{...}? ( STATS_AUTO_RECALC_SYMBOL | STATS_PERSISTENT_SYMBOL | STATS_SAMPLE_PAGES_SYMBOL ) ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL ) | ( CHECKSUM_SYMBOL | TABLE_CHECKSUM_SYMBOL ) ( EQUAL_OPERATOR )? ulong_number | DELAY_KEY_WRITE_SYMBOL ( EQUAL_OPERATOR )? ulong_number | ROW_FORMAT_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | DYNAMIC_SYMBOL | FIXED_SYMBOL | COMPRESSED_SYMBOL | REDUNDANT_SYMBOL | COMPACT_SYMBOL ) | UNION_SYMBOL ( EQUAL_OPERATOR )? OPEN_PAR_SYMBOL table_ref_list CLOSE_PAR_SYMBOL | ( DEFAULT_SYMBOL )? ( COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default | charset ( EQUAL_OPERATOR )? charset_name_or_default ) | INSERT_METHOD_SYMBOL ( EQUAL_OPERATOR )? ( NO_SYMBOL | FIRST_SYMBOL | LAST_SYMBOL ) | DATA_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string | INDEX_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string | TABLESPACE_SYMBOL ({...}? ( EQUAL_OPERATOR )? |) identifier | STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL ) | CONNECTION_SYMBOL ( EQUAL_OPERATOR )? text_string | KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number );
 */
static void
create_table_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2683:20: ( ( ENGINE_SYMBOL |{...}? TYPE_SYMBOL ) ( EQUAL_OPERATOR )? engine_ref | MAX_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | MIN_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | AVG_ROW_LENGTH_SYMBOL ( EQUAL_OPERATOR )? ulong_number | PASSWORD_SYMBOL ( EQUAL_OPERATOR )? text_string | COMMENT_SYMBOL ( EQUAL_OPERATOR )? text_string | COMPRESSION_SYMBOL ( EQUAL_OPERATOR )? text_string | AUTO_INCREMENT_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number | PACK_KEYS_SYMBOL ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL ) |{...}? ( STATS_AUTO_RECALC_SYMBOL | STATS_PERSISTENT_SYMBOL | STATS_SAMPLE_PAGES_SYMBOL ) ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL ) | ( CHECKSUM_SYMBOL | TABLE_CHECKSUM_SYMBOL ) ( EQUAL_OPERATOR )? ulong_number | DELAY_KEY_WRITE_SYMBOL ( EQUAL_OPERATOR )? ulong_number | ROW_FORMAT_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | DYNAMIC_SYMBOL | FIXED_SYMBOL | COMPRESSED_SYMBOL | REDUNDANT_SYMBOL | COMPACT_SYMBOL ) | UNION_SYMBOL ( EQUAL_OPERATOR )? OPEN_PAR_SYMBOL table_ref_list CLOSE_PAR_SYMBOL | ( DEFAULT_SYMBOL )? ( COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default | charset ( EQUAL_OPERATOR )? charset_name_or_default ) | INSERT_METHOD_SYMBOL ( EQUAL_OPERATOR )? ( NO_SYMBOL | FIRST_SYMBOL | LAST_SYMBOL ) | DATA_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string | INDEX_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string | TABLESPACE_SYMBOL ({...}? ( EQUAL_OPERATOR )? |) identifier | STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL ) | CONNECTION_SYMBOL ( EQUAL_OPERATOR )? text_string | KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number )

            ANTLR3_UINT32 alt872;

            alt872=22;

            switch ( LA(1) )
            {
            case ENGINE_SYMBOL:
            case TYPE_SYMBOL:
            	{
            		alt872=1;
            	}
                break;
            case MAX_ROWS_SYMBOL:
            	{
            		alt872=2;
            	}
                break;
            case MIN_ROWS_SYMBOL:
            	{
            		alt872=3;
            	}
                break;
            case AVG_ROW_LENGTH_SYMBOL:
            	{
            		alt872=4;
            	}
                break;
            case PASSWORD_SYMBOL:
            	{
            		alt872=5;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt872=6;
            	}
                break;
            case COMPRESSION_SYMBOL:
            	{
            		alt872=7;
            	}
                break;
            case AUTO_INCREMENT_SYMBOL:
            	{
            		alt872=8;
            	}
                break;
            case PACK_KEYS_SYMBOL:
            	{
            		alt872=9;
            	}
                break;
            case STATS_AUTO_RECALC_SYMBOL:
            case STATS_PERSISTENT_SYMBOL:
            case STATS_SAMPLE_PAGES_SYMBOL:
            	{
            		alt872=10;
            	}
                break;
            case CHECKSUM_SYMBOL:
            case TABLE_CHECKSUM_SYMBOL:
            	{
            		alt872=11;
            	}
                break;
            case DELAY_KEY_WRITE_SYMBOL:
            	{
            		alt872=12;
            	}
                break;
            case ROW_FORMAT_SYMBOL:
            	{
            		alt872=13;
            	}
                break;
            case UNION_SYMBOL:
            	{
            		alt872=14;
            	}
                break;
            case CHARSET_SYMBOL:
            case CHAR_SYMBOL:
            case COLLATE_SYMBOL:
            case DEFAULT_SYMBOL:
            	{
            		alt872=15;
            	}
                break;
            case INSERT_METHOD_SYMBOL:
            	{
            		alt872=16;
            	}
                break;
            case DATA_SYMBOL:
            	{
            		alt872=17;
            	}
                break;
            case INDEX_SYMBOL:
            	{
            		alt872=18;
            	}
                break;
            case TABLESPACE_SYMBOL:
            	{
            		alt872=19;
            	}
                break;
            case STORAGE_SYMBOL:
            	{
            		alt872=20;
            	}
                break;
            case CONNECTION_SYMBOL:
            	{
            		alt872=21;
            	}
                break;
            case KEY_BLOCK_SIZE_SYMBOL:
            	{
            		alt872=22;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 872;
                EXCEPTION->state        = 0;


                goto rulecreate_table_optionEx;

            }

            switch (alt872)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2684:2: ( ENGINE_SYMBOL |{...}? TYPE_SYMBOL ) ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:2684:2: ( ENGINE_SYMBOL |{...}? TYPE_SYMBOL )
        	        {
        	            int alt844=2;
        	            switch ( LA(1) )
        	            {
        	            case ENGINE_SYMBOL:
        	            	{
        	            		alt844=1;
        	            	}
        	                break;
        	            case TYPE_SYMBOL:
        	            	{
        	            		alt844=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 844;
        	                EXCEPTION->state        = 0;


        	                goto rulecreate_table_optionEx;

        	            }

        	            switch (alt844)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2684:3: ENGINE_SYMBOL
        	        	    {
        	        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_create_table_option14943);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2684:19: {...}? TYPE_SYMBOL
        	        	    {
        	        	        if ( !((SERVER_VERSION < 50500)) )
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	                CONSTRUCTEX();
        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION < 50500";
        	        	                EXCEPTION->ruleName	 = (void *)"create_table_option";


        	        	        }

        	        	         MATCHT(TYPE_SYMBOL, &FOLLOW_TYPE_SYMBOL_in_create_table_option14949);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2684:58: ( EQUAL_OPERATOR )?
        	        {
        	            int alt845=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt845=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt845)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2684:58: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option14952);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_create_table_option14955);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2685:4: MAX_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number
        	    {
        	         MATCHT(MAX_ROWS_SYMBOL, &FOLLOW_MAX_ROWS_SYMBOL_in_create_table_option14960);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2685:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt846=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt846=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt846)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2685:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option14962);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulonglong_number_in_create_table_option14965);
        	        ulonglong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2686:4: MIN_ROWS_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number
        	    {
        	         MATCHT(MIN_ROWS_SYMBOL, &FOLLOW_MIN_ROWS_SYMBOL_in_create_table_option14970);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2686:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt847=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt847=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt847)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2686:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option14972);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulonglong_number_in_create_table_option14975);
        	        ulonglong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2687:4: AVG_ROW_LENGTH_SYMBOL ( EQUAL_OPERATOR )? ulong_number
        	    {
        	         MATCHT(AVG_ROW_LENGTH_SYMBOL, &FOLLOW_AVG_ROW_LENGTH_SYMBOL_in_create_table_option14980);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2687:26: ( EQUAL_OPERATOR )?
        	        {
        	            int alt848=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt848=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt848)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2687:26: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option14982);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option14985);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2688:4: PASSWORD_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_create_table_option14990);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2688:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt849=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt849=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt849)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2688:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option14992);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option14995);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2689:4: COMMENT_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_create_table_option15000);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2689:19: ( EQUAL_OPERATOR )?
        	        {
        	            int alt850=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt850=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt850)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2689:19: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15002);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option15005);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:2690:4: COMPRESSION_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(COMPRESSION_SYMBOL, &FOLLOW_COMPRESSION_SYMBOL_in_create_table_option15010);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2690:23: ( EQUAL_OPERATOR )?
        	        {
        	            int alt851=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt851=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt851)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2690:23: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15012);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option15015);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 8:
        	    // MySQLSimpleParser.g:2691:4: AUTO_INCREMENT_SYMBOL ( EQUAL_OPERATOR )? ulonglong_number
        	    {
        	         MATCHT(AUTO_INCREMENT_SYMBOL, &FOLLOW_AUTO_INCREMENT_SYMBOL_in_create_table_option15020);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2691:26: ( EQUAL_OPERATOR )?
        	        {
        	            int alt852=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt852=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt852)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2691:26: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15022);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulonglong_number_in_create_table_option15025);
        	        ulonglong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 9:
        	    // MySQLSimpleParser.g:2692:4: PACK_KEYS_SYMBOL ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL )
        	    {
        	         MATCHT(PACK_KEYS_SYMBOL, &FOLLOW_PACK_KEYS_SYMBOL_in_create_table_option15030);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2692:21: ( EQUAL_OPERATOR )?
        	        {
        	            int alt853=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt853=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt853)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2692:21: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15032);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2692:37: ( ulong_number | DEFAULT_SYMBOL )
        	        {
        	            int alt854=2;
        	            switch ( LA(1) )
        	            {
        	            case DECIMAL_NUMBER:
        	            case FLOAT_NUMBER:
        	            case HEX_NUMBER:
        	            case INT_NUMBER:
        	            case LONG_NUMBER:
        	            case ULONGLONG_NUMBER:
        	            	{
        	            		alt854=1;
        	            	}
        	                break;
        	            case DEFAULT_SYMBOL:
        	            	{
        	            		alt854=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 854;
        	                EXCEPTION->state        = 0;


        	                goto rulecreate_table_optionEx;

        	            }

        	            switch (alt854)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2692:38: ulong_number
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option15036);
        	        	        ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2692:53: DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_create_table_option15040);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 10:
        	    // MySQLSimpleParser.g:2693:4: {...}? ( STATS_AUTO_RECALC_SYMBOL | STATS_PERSISTENT_SYMBOL | STATS_SAMPLE_PAGES_SYMBOL ) ( EQUAL_OPERATOR )? ( ulong_number | DEFAULT_SYMBOL )
        	    {
        	        if ( !((SERVER_VERSION >= 50600)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
        	                EXCEPTION->ruleName	 = (void *)"create_table_option";


        	        }

        	        if ( ((LA(1) >= STATS_AUTO_RECALC_SYMBOL) && (LA(1) <= STATS_SAMPLE_PAGES_SYMBOL)) )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulecreate_table_optionEx;
        	        }


        	        // MySQLSimpleParser.g:2694:3: ( EQUAL_OPERATOR )?
        	        {
        	            int alt855=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt855=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt855)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2694:3: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15062);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2694:19: ( ulong_number | DEFAULT_SYMBOL )
        	        {
        	            int alt856=2;
        	            switch ( LA(1) )
        	            {
        	            case DECIMAL_NUMBER:
        	            case FLOAT_NUMBER:
        	            case HEX_NUMBER:
        	            case INT_NUMBER:
        	            case LONG_NUMBER:
        	            case ULONGLONG_NUMBER:
        	            	{
        	            		alt856=1;
        	            	}
        	                break;
        	            case DEFAULT_SYMBOL:
        	            	{
        	            		alt856=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 856;
        	                EXCEPTION->state        = 0;


        	                goto rulecreate_table_optionEx;

        	            }

        	            switch (alt856)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2694:20: ulong_number
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option15066);
        	        	        ulong_number(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2694:35: DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_create_table_option15070);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 11:
        	    // MySQLSimpleParser.g:2695:4: ( CHECKSUM_SYMBOL | TABLE_CHECKSUM_SYMBOL ) ( EQUAL_OPERATOR )? ulong_number
        	    {
        	        if ( LA(1) == CHECKSUM_SYMBOL || LA(1) == TABLE_CHECKSUM_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulecreate_table_optionEx;
        	        }


        	        // MySQLSimpleParser.g:2695:46: ( EQUAL_OPERATOR )?
        	        {
        	            int alt857=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt857=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt857)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2695:46: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15084);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option15087);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 12:
        	    // MySQLSimpleParser.g:2696:4: DELAY_KEY_WRITE_SYMBOL ( EQUAL_OPERATOR )? ulong_number
        	    {
        	         MATCHT(DELAY_KEY_WRITE_SYMBOL, &FOLLOW_DELAY_KEY_WRITE_SYMBOL_in_create_table_option15092);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2696:27: ( EQUAL_OPERATOR )?
        	        {
        	            int alt858=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt858=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt858)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2696:27: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15094);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option15097);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 13:
        	    // MySQLSimpleParser.g:2697:4: ROW_FORMAT_SYMBOL ( EQUAL_OPERATOR )? ( DEFAULT_SYMBOL | DYNAMIC_SYMBOL | FIXED_SYMBOL | COMPRESSED_SYMBOL | REDUNDANT_SYMBOL | COMPACT_SYMBOL )
        	    {
        	         MATCHT(ROW_FORMAT_SYMBOL, &FOLLOW_ROW_FORMAT_SYMBOL_in_create_table_option15102);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2697:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt859=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt859=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt859)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2697:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15104);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        if ( LA(1) == COMPACT_SYMBOL || LA(1) == COMPRESSED_SYMBOL || LA(1) == DYNAMIC_SYMBOL || LA(1) == FIXED_SYMBOL || LA(1) == REDUNDANT_SYMBOL || LA(1) == DEFAULT_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulecreate_table_optionEx;
        	        }


        	    }
        	    break;
        	case 14:
        	    // MySQLSimpleParser.g:2698:4: UNION_SYMBOL ( EQUAL_OPERATOR )? OPEN_PAR_SYMBOL table_ref_list CLOSE_PAR_SYMBOL
        	    {
        	         MATCHT(UNION_SYMBOL, &FOLLOW_UNION_SYMBOL_in_create_table_option15134);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2698:17: ( EQUAL_OPERATOR )?
        	        {
        	            int alt860=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt860=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt860)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2698:17: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15136);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_create_table_option15139);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_table_ref_list_in_create_table_option15141);
        	        table_ref_list(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_create_table_option15143);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 15:
        	    // MySQLSimpleParser.g:2699:4: ( DEFAULT_SYMBOL )? ( COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default | charset ( EQUAL_OPERATOR )? charset_name_or_default )
        	    {
        	        // MySQLSimpleParser.g:2699:4: ( DEFAULT_SYMBOL )?
        	        {
        	            int alt861=2;
        	            switch ( LA(1) )
        	            {
        	                case DEFAULT_SYMBOL:
        	                	{
        	                		alt861=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt861)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2699:4: DEFAULT_SYMBOL
        	        	    {
        	        	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_create_table_option15148);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:2700:3: ( COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default | charset ( EQUAL_OPERATOR )? charset_name_or_default )
        	        {
        	            int alt864=2;
        	            switch ( LA(1) )
        	            {
        	            case COLLATE_SYMBOL:
        	            	{
        	            		alt864=1;
        	            	}
        	                break;
        	            case CHARSET_SYMBOL:
        	            case CHAR_SYMBOL:
        	            	{
        	            		alt864=2;
        	            	}
        	                break;

        	            default:
        	                if (BACKTRACKING>0)
        	                {
        	                    FAILEDFLAG = ANTLR3_TRUE;
        	                    return ;
        	                }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                EXCEPTION->message      = (void *)"";
        	                EXCEPTION->decisionNum  = 864;
        	                EXCEPTION->state        = 0;


        	                goto rulecreate_table_optionEx;

        	            }

        	            switch (alt864)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2701:4: COLLATE_SYMBOL ( EQUAL_OPERATOR )? collation_name_or_default
        	        	    {
        	        	         MATCHT(COLLATE_SYMBOL, &FOLLOW_COLLATE_SYMBOL_in_create_table_option15158);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:2701:19: ( EQUAL_OPERATOR )?
        	        	        {
        	        	            int alt862=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case EQUAL_OPERATOR:
        	        	                	{
        	        	                		alt862=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt862)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:2701:19: EQUAL_OPERATOR
        	        	        	    {
        	        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15160);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulecreate_table_optionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_collation_name_or_default_in_create_table_option15163);
        	        	        collation_name_or_default(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2702:6: charset ( EQUAL_OPERATOR )? charset_name_or_default
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_charset_in_create_table_option15170);
        	        	        charset(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        // MySQLSimpleParser.g:2702:14: ( EQUAL_OPERATOR )?
        	        	        {
        	        	            int alt863=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case EQUAL_OPERATOR:
        	        	                	{
        	        	                		alt863=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt863)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:2702:14: EQUAL_OPERATOR
        	        	        	    {
        	        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15172);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulecreate_table_optionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	        FOLLOWPUSH(FOLLOW_charset_name_or_default_in_create_table_option15175);
        	        	        charset_name_or_default(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 16:
        	    // MySQLSimpleParser.g:2705:4: INSERT_METHOD_SYMBOL ( EQUAL_OPERATOR )? ( NO_SYMBOL | FIRST_SYMBOL | LAST_SYMBOL )
        	    {
        	         MATCHT(INSERT_METHOD_SYMBOL, &FOLLOW_INSERT_METHOD_SYMBOL_in_create_table_option15185);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2705:25: ( EQUAL_OPERATOR )?
        	        {
        	            int alt865=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt865=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt865)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2705:25: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15187);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        if ( LA(1) == NO_SYMBOL || LA(1) == FIRST_SYMBOL || LA(1) == LAST_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulecreate_table_optionEx;
        	        }


        	    }
        	    break;
        	case 17:
        	    // MySQLSimpleParser.g:2706:4: DATA_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(DATA_SYMBOL, &FOLLOW_DATA_SYMBOL_in_create_table_option15205);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DIRECTORY_SYMBOL, &FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15207);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2706:33: ( EQUAL_OPERATOR )?
        	        {
        	            int alt866=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt866=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt866)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2706:33: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15209);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option15212);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 18:
        	    // MySQLSimpleParser.g:2707:4: INDEX_SYMBOL DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(INDEX_SYMBOL, &FOLLOW_INDEX_SYMBOL_in_create_table_option15217);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(DIRECTORY_SYMBOL, &FOLLOW_DIRECTORY_SYMBOL_in_create_table_option15219);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2707:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt867=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt867=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt867)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2707:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15221);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option15224);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 19:
        	    // MySQLSimpleParser.g:2708:4: TABLESPACE_SYMBOL ({...}? ( EQUAL_OPERATOR )? |) identifier
        	    {
        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_create_table_option15229);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2708:22: ({...}? ( EQUAL_OPERATOR )? |)
        	        {
        	            int alt869=2;
        	            {
        	                int LA869_0 = LA(1);
        	                if ( (LA869_0 == EQUAL_OPERATOR))
        	                {
        	                    alt869=1;
        	                }
        	                else if ( (LA869_0 == IDENTIFIER))
        	                {
        	                    {
        	                        int LA869_2 = LA(2);
        	                        if ( ((SERVER_VERSION >= 50707)))
        	                        {
        	                            alt869=1;
        	                        }
        	                        else if ( (ANTLR3_TRUE))
        	                        {
        	                            alt869=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 869;
        	                            EXCEPTION->state        = 2;


        	                            goto rulecreate_table_optionEx;

        	                        }
        	                    }
        	                }
        	                else if ( (LA869_0 == BACK_TICK_QUOTED_ID))
        	                {
        	                    {
        	                        int LA869_3 = LA(2);
        	                        if ( ((SERVER_VERSION >= 50707)))
        	                        {
        	                            alt869=1;
        	                        }
        	                        else if ( (ANTLR3_TRUE))
        	                        {
        	                            alt869=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 869;
        	                            EXCEPTION->state        = 3;


        	                            goto rulecreate_table_optionEx;

        	                        }
        	                    }
        	                }
        	                else if ( (LA869_0 == DOUBLE_QUOTED_TEXT))
        	                {
        	                    {
        	                        int LA869_4 = LA(2);
        	                        if ( (( (( ((SERVER_VERSION >= 50707)) && ((SERVER_VERSION >= 50707)) )) && ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )))
        	                        {
        	                            alt869=1;
        	                        }
        	                        else if ( ((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
        	                        {
        	                            alt869=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 869;
        	                            EXCEPTION->state        = 4;


        	                            goto rulecreate_table_optionEx;

        	                        }
        	                    }
        	                }
        	                else if ( (((LA869_0 >= ACTION_SYMBOL) && (LA869_0 <= SHARE_SYMBOL)) || ((LA869_0 >= SLOW_SYMBOL) && (LA869_0 <= YEAR_SYMBOL))))
        	                {
        	                    {
        	                        int LA869_5 = LA(2);
        	                        if ( ((SERVER_VERSION >= 50707)))
        	                        {
        	                            alt869=1;
        	                        }
        	                        else if ( (ANTLR3_TRUE))
        	                        {
        	                            alt869=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 869;
        	                            EXCEPTION->state        = 5;


        	                            goto rulecreate_table_optionEx;

        	                        }
        	                    }
        	                }
        	                else if ( (((LA869_0 >= ACCOUNT_SYMBOL) && (LA869_0 <= XA_SYMBOL)) || LA869_0 == SHUTDOWN_SYMBOL))
        	                {
        	                    {
        	                        int LA869_6 = LA(2);
        	                        if ( ((SERVER_VERSION >= 50707)))
        	                        {
        	                            alt869=1;
        	                        }
        	                        else if ( (ANTLR3_TRUE))
        	                        {
        	                            alt869=2;
        	                        }
        	                        else
        	                        {
        	                            if (BACKTRACKING>0)
        	                            {
        	                                FAILEDFLAG = ANTLR3_TRUE;
        	                                return ;
        	                            }


        	                            CONSTRUCTEX();
        	                            EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                            EXCEPTION->message      = (void *)"";
        	                            EXCEPTION->decisionNum  = 869;
        	                            EXCEPTION->state        = 6;


        	                            goto rulecreate_table_optionEx;

        	                        }
        	                    }
        	                }
        	                else
        	                {
        	                    if (BACKTRACKING>0)
        	                    {
        	                        FAILEDFLAG = ANTLR3_TRUE;
        	                        return ;
        	                    }


        	                    CONSTRUCTEX();
        	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
        	                    EXCEPTION->message      = (void *)"";
        	                    EXCEPTION->decisionNum  = 869;
        	                    EXCEPTION->state        = 0;


        	                    goto rulecreate_table_optionEx;

        	                }
        	            }
        	            switch (alt869)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2708:23: {...}? ( EQUAL_OPERATOR )?
        	        	    {
        	        	        if ( !((SERVER_VERSION >= 50707)) )
        	        	        {
        	        	            if (BACKTRACKING>0)
        	        	            {
        	        	                FAILEDFLAG = ANTLR3_TRUE;
        	        	                return ;
        	        	            }

        	        	                CONSTRUCTEX();
        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50707";
        	        	                EXCEPTION->ruleName	 = (void *)"create_table_option";


        	        	        }

        	        	        // MySQLSimpleParser.g:2708:50: ( EQUAL_OPERATOR )?
        	        	        {
        	        	            int alt868=2;
        	        	            switch ( LA(1) )
        	        	            {
        	        	                case EQUAL_OPERATOR:
        	        	                	{
        	        	                		alt868=1;
        	        	                	}
        	        	                    break;
        	        	            }

        	        	            switch (alt868)
        	        	            {
        	        	        	case 1:
        	        	        	    // MySQLSimpleParser.g:2708:50: EQUAL_OPERATOR
        	        	        	    {
        	        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15234);
        	        	        	        if  (HASEXCEPTION())
        	        	        	        {
        	        	        	            goto rulecreate_table_optionEx;
        	        	        	        }
        	        	        	        if (HASFAILED())
        	        	        	        {
        	        	        	            return ;
        	        	        	        }


        	        	        	    }
        	        	        	    break;

        	        	            }
        	        	        }

        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2708:80: 
        	        	    {
        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_identifier_in_create_table_option15243);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 20:
        	    // MySQLSimpleParser.g:2709:4: STORAGE_SYMBOL ( DISK_SYMBOL | MEMORY_SYMBOL )
        	    {
        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_create_table_option15248);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        if ( LA(1) == DISK_SYMBOL || LA(1) == MEMORY_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulecreate_table_optionEx;
        	        }


        	    }
        	    break;
        	case 21:
        	    // MySQLSimpleParser.g:2710:4: CONNECTION_SYMBOL ( EQUAL_OPERATOR )? text_string
        	    {
        	         MATCHT(CONNECTION_SYMBOL, &FOLLOW_CONNECTION_SYMBOL_in_create_table_option15261);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2710:22: ( EQUAL_OPERATOR )?
        	        {
        	            int alt870=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt870=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt870)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2710:22: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15263);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_text_string_in_create_table_option15266);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 22:
        	    // MySQLSimpleParser.g:2711:4: KEY_BLOCK_SIZE_SYMBOL ( EQUAL_OPERATOR )? ulong_number
        	    {
        	         MATCHT(KEY_BLOCK_SIZE_SYMBOL, &FOLLOW_KEY_BLOCK_SIZE_SYMBOL_in_create_table_option15271);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2711:26: ( EQUAL_OPERATOR )?
        	        {
        	            int alt871=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt871=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt871)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2711:26: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_create_table_option15273);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecreate_table_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_ulong_number_in_create_table_option15276);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecreate_table_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecreate_table_optionEx; /* Prevent compiler warnings */
    rulecreate_table_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end create_table_option */

/**
 * $ANTLR start partitioning
 * MySQLSimpleParser.g:2715:1: partitioning :{...}? => PARTITION_SYMBOL BY_SYMBOL ( ( LINEAR_SYMBOL )? HASH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( LINEAR_SYMBOL )? KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses | ( RANGE_SYMBOL | LIST_SYMBOL ) ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses ) ) ( PARTITIONS_SYMBOL real_ulong_number )? ( SUBPARTITION_SYMBOL BY_SYMBOL ( LINEAR_SYMBOL )? ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )? )? ( partition_definitions )? ;
 */
static void
partitioning(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2715:13: ({...}? => PARTITION_SYMBOL BY_SYMBOL ( ( LINEAR_SYMBOL )? HASH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( LINEAR_SYMBOL )? KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses | ( RANGE_SYMBOL | LIST_SYMBOL ) ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses ) ) ( PARTITIONS_SYMBOL real_ulong_number )? ( SUBPARTITION_SYMBOL BY_SYMBOL ( LINEAR_SYMBOL )? ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )? )? ( partition_definitions )? )
        // MySQLSimpleParser.g:2716:2: {...}? => PARTITION_SYMBOL BY_SYMBOL ( ( LINEAR_SYMBOL )? HASH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( LINEAR_SYMBOL )? KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses | ( RANGE_SYMBOL | LIST_SYMBOL ) ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses ) ) ( PARTITIONS_SYMBOL real_ulong_number )? ( SUBPARTITION_SYMBOL BY_SYMBOL ( LINEAR_SYMBOL )? ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )? )? ( partition_definitions )?
        {
            if ( !((SERVER_VERSION >= 50100)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50100";
                    EXCEPTION->ruleName	 = (void *)"partitioning";


            }

             MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_partitioning15290);
            if  (HASEXCEPTION())
            {
                goto rulepartitioningEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_partitioning15292);
            if  (HASEXCEPTION())
            {
                goto rulepartitioningEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2717:2: ( ( LINEAR_SYMBOL )? HASH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL | ( LINEAR_SYMBOL )? KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses | ( RANGE_SYMBOL | LIST_SYMBOL ) ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses ) )
            {
                int alt877=3;
                switch ( LA(1) )
                {
                case LINEAR_SYMBOL:
                	{
                		switch ( LA(2) )
                		{
                		case HASH_SYMBOL:
                			{
                				alt877=1;
                			}
                		    break;
                		case KEY_SYMBOL:
                			{
                				alt877=2;
                			}
                		    break;

                		default:
                		    if (BACKTRACKING>0)
                		    {
                		        FAILEDFLAG = ANTLR3_TRUE;
                		        return ;
                		    }

                		    CONSTRUCTEX();
                		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		    EXCEPTION->message      = (void *)"";
                		    EXCEPTION->decisionNum  = 877;
                		    EXCEPTION->state        = 1;


                		    goto rulepartitioningEx;

                		}

                	}
                    break;
                case HASH_SYMBOL:
                	{
                		alt877=1;
                	}
                    break;
                case KEY_SYMBOL:
                	{
                		alt877=2;
                	}
                    break;
                case LIST_SYMBOL:
                case RANGE_SYMBOL:
                	{
                		alt877=3;
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 877;
                    EXCEPTION->state        = 0;


                    goto rulepartitioningEx;

                }

                switch (alt877)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2718:3: ( LINEAR_SYMBOL )? HASH_SYMBOL OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
            	    {
            	        // MySQLSimpleParser.g:2718:3: ( LINEAR_SYMBOL )?
            	        {
            	            int alt873=2;
            	            switch ( LA(1) )
            	            {
            	                case LINEAR_SYMBOL:
            	                	{
            	                		alt873=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt873)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2718:3: LINEAR_SYMBOL
            	        	    {
            	        	         MATCHT(LINEAR_SYMBOL, &FOLLOW_LINEAR_SYMBOL_in_partitioning15299);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(HASH_SYMBOL, &FOLLOW_HASH_SYMBOL_in_partitioning15302);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15304);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_expression_in_partitioning15306);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15308);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2719:5: ( LINEAR_SYMBOL )? KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses
            	    {
            	        // MySQLSimpleParser.g:2719:5: ( LINEAR_SYMBOL )?
            	        {
            	            int alt874=2;
            	            switch ( LA(1) )
            	            {
            	                case LINEAR_SYMBOL:
            	                	{
            	                		alt874=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt874)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2719:5: LINEAR_SYMBOL
            	        	    {
            	        	         MATCHT(LINEAR_SYMBOL, &FOLLOW_LINEAR_SYMBOL_in_partitioning15314);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_partitioning15317);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2719:31: ( partition_key_algorithm )?
            	        {
            	            int alt875=2;
            	            switch ( LA(1) )
            	            {
            	                case ALGORITHM_SYMBOL:
            	                	{
            	                		alt875=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt875)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2719:31: partition_key_algorithm
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_partition_key_algorithm_in_partitioning15319);
            	        	        partition_key_algorithm(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_partitioning15322);
            	        identifier_list_with_parentheses(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:2720:5: ( RANGE_SYMBOL | LIST_SYMBOL ) ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses )
            	    {
            	        if ( LA(1) == LIST_SYMBOL || LA(1) == RANGE_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto rulepartitioningEx;
            	        }


            	        // MySQLSimpleParser.g:2721:4: ( OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL |{...}? COLUMNS_SYMBOL identifier_list_with_parentheses )
            	        {
            	            int alt876=2;
            	            switch ( LA(1) )
            	            {
            	            case OPEN_PAR_SYMBOL:
            	            	{
            	            		alt876=1;
            	            	}
            	                break;
            	            case COLUMNS_SYMBOL:
            	            	{
            	            		alt876=2;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 876;
            	                EXCEPTION->state        = 0;


            	                goto rulepartitioningEx;

            	            }

            	            switch (alt876)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2722:5: OPEN_PAR_SYMBOL expression CLOSE_PAR_SYMBOL
            	        	    {
            	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15346);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_expression_in_partitioning15348);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15350);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:2723:7: {...}? COLUMNS_SYMBOL identifier_list_with_parentheses
            	        	    {
            	        	        if ( !((SERVER_VERSION >= 50500)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50500";
            	        	                EXCEPTION->ruleName	 = (void *)"partitioning";


            	        	        }

            	        	         MATCHT(COLUMNS_SYMBOL, &FOLLOW_COLUMNS_SYMBOL_in_partitioning15360);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_partitioning15362);
            	        	        identifier_list_with_parentheses(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2726:2: ( PARTITIONS_SYMBOL real_ulong_number )?
            {
                int alt878=2;
                switch ( LA(1) )
                {
                    case PARTITIONS_SYMBOL:
                    	{
                    		alt878=1;
                    	}
                        break;
                }

                switch (alt878)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2726:3: PARTITIONS_SYMBOL real_ulong_number
            	    {
            	         MATCHT(PARTITIONS_SYMBOL, &FOLLOW_PARTITIONS_SYMBOL_in_partitioning15374);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_partitioning15376);
            	        real_ulong_number(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2727:2: ( SUBPARTITION_SYMBOL BY_SYMBOL ( LINEAR_SYMBOL )? ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )? )?
            {
                int alt884=2;
                switch ( LA(1) )
                {
                    case SUBPARTITION_SYMBOL:
                    	{
                    		alt884=1;
                    	}
                        break;
                }

                switch (alt884)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2728:3: SUBPARTITION_SYMBOL BY_SYMBOL ( LINEAR_SYMBOL )? ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )?
            	    {
            	         MATCHT(SUBPARTITION_SYMBOL, &FOLLOW_SUBPARTITION_SYMBOL_in_partitioning15385);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_partitioning15387);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2728:33: ( LINEAR_SYMBOL )?
            	        {
            	            int alt879=2;
            	            switch ( LA(1) )
            	            {
            	                case LINEAR_SYMBOL:
            	                	{
            	                		alt879=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt879)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2728:33: LINEAR_SYMBOL
            	        	    {
            	        	         MATCHT(LINEAR_SYMBOL, &FOLLOW_LINEAR_SYMBOL_in_partitioning15389);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartitioningEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	        // MySQLSimpleParser.g:2729:3: ( ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )? )?
            	        {
            	            int alt883=2;
            	            switch ( LA(1) )
            	            {
            	                case HASH_SYMBOL:
            	                case KEY_SYMBOL:
            	                	{
            	                		alt883=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt883)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2730:4: ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses ) ( SUBPARTITIONS_SYMBOL real_ulong_number )?
            	        	    {
            	        	        // MySQLSimpleParser.g:2730:4: ( HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL | KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses )
            	        	        {
            	        	            int alt881=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case HASH_SYMBOL:
            	        	            	{
            	        	            		alt881=1;
            	        	            	}
            	        	                break;
            	        	            case KEY_SYMBOL:
            	        	            	{
            	        	            		alt881=2;
            	        	            	}
            	        	                break;

            	        	            default:
            	        	                if (BACKTRACKING>0)
            	        	                {
            	        	                    FAILEDFLAG = ANTLR3_TRUE;
            	        	                    return ;
            	        	                }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"";
            	        	                EXCEPTION->decisionNum  = 881;
            	        	                EXCEPTION->state        = 0;


            	        	                goto rulepartitioningEx;

            	        	            }

            	        	            switch (alt881)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:2731:6: HASH_SYMBOL OPEN_PAR_SYMBOL bitwise_or_expression CLOSE_PAR_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(HASH_SYMBOL, &FOLLOW_HASH_SYMBOL_in_partitioning15406);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partitioning15408);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	        FOLLOWPUSH(FOLLOW_bitwise_or_expression_in_partitioning15410);
            	        	        	        bitwise_or_expression(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partitioning15412);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;
            	        	        	case 2:
            	        	        	    // MySQLSimpleParser.g:2732:7: KEY_SYMBOL ( partition_key_algorithm )? identifier_list_with_parentheses
            	        	        	    {
            	        	        	         MATCHT(KEY_SYMBOL, &FOLLOW_KEY_SYMBOL_in_partitioning15420);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	        // MySQLSimpleParser.g:2732:18: ( partition_key_algorithm )?
            	        	        	        {
            	        	        	            int alt880=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	                case ALGORITHM_SYMBOL:
            	        	        	                	{
            	        	        	                		alt880=1;
            	        	        	                	}
            	        	        	                    break;
            	        	        	            }

            	        	        	            switch (alt880)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:2732:18: partition_key_algorithm
            	        	        	        	    {
            	        	        	        	        FOLLOWPUSH(FOLLOW_partition_key_algorithm_in_partitioning15422);
            	        	        	        	        partition_key_algorithm(ctx);

            	        	        	        	        FOLLOWPOP();
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto rulepartitioningEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_partitioning15425);
            	        	        	        identifier_list_with_parentheses(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        // MySQLSimpleParser.g:2734:4: ( SUBPARTITIONS_SYMBOL real_ulong_number )?
            	        	        {
            	        	            int alt882=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case SUBPARTITIONS_SYMBOL:
            	        	                	{
            	        	                		alt882=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt882)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:2734:5: SUBPARTITIONS_SYMBOL real_ulong_number
            	        	        	    {
            	        	        	         MATCHT(SUBPARTITIONS_SYMBOL, &FOLLOW_SUBPARTITIONS_SYMBOL_in_partitioning15436);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_partitioning15438);
            	        	        	        real_ulong_number(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartitioningEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2737:5: ( partition_definitions )?
            {
                int alt885=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		switch ( LA(2) )
                    		{
                    		    case PARTITION_SYMBOL:
                    		    	{
                    		    		alt885=1;
                    		    	}
                    		        break;
                    		}

                    	}
                        break;
                }

                switch (alt885)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2737:5: partition_definitions
            	    {
            	        FOLLOWPUSH(FOLLOW_partition_definitions_in_partitioning15458);
            	        partition_definitions(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartitioningEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartitioningEx; /* Prevent compiler warnings */
    rulepartitioningEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partitioning */

/**
 * $ANTLR start partition_key_algorithm
 * MySQLSimpleParser.g:2740:1: partition_key_algorithm :{...}? ALGORITHM_SYMBOL EQUAL_OPERATOR real_ulong_number ;
 */
static void
partition_key_algorithm(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2740:24: ({...}? ALGORITHM_SYMBOL EQUAL_OPERATOR real_ulong_number )
        // MySQLSimpleParser.g:2741:2: {...}? ALGORITHM_SYMBOL EQUAL_OPERATOR real_ulong_number
        {
            if ( !((SERVER_VERSION >= 50700)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50700";
                    EXCEPTION->ruleName	 = (void *)"partition_key_algorithm";


            }

             MATCHT(ALGORITHM_SYMBOL, &FOLLOW_ALGORITHM_SYMBOL_in_partition_key_algorithm15471);
            if  (HASEXCEPTION())
            {
                goto rulepartition_key_algorithmEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_key_algorithm15473);
            if  (HASEXCEPTION())
            {
                goto rulepartition_key_algorithmEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_real_ulong_number_in_partition_key_algorithm15475);
            real_ulong_number(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulepartition_key_algorithmEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartition_key_algorithmEx; /* Prevent compiler warnings */
    rulepartition_key_algorithmEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_key_algorithm */

/**
 * $ANTLR start partition_definitions
 * MySQLSimpleParser.g:2744:1: partition_definitions : OPEN_PAR_SYMBOL partition_definition ( COMMA_SYMBOL partition_definition )* CLOSE_PAR_SYMBOL ;
 */
static void
partition_definitions(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2744:22: ( OPEN_PAR_SYMBOL partition_definition ( COMMA_SYMBOL partition_definition )* CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2745:2: OPEN_PAR_SYMBOL partition_definition ( COMMA_SYMBOL partition_definition )* CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partition_definitions15484);
            if  (HASEXCEPTION())
            {
                goto rulepartition_definitionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_partition_definition_in_partition_definitions15486);
            partition_definition(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulepartition_definitionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2745:39: ( COMMA_SYMBOL partition_definition )*

            for (;;)
            {
                int alt886=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt886=1;
                	}
                    break;

                }

                switch (alt886)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2745:40: COMMA_SYMBOL partition_definition
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_partition_definitions15489);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_partition_definition_in_partition_definitions15491);
            	        partition_definition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop886;	/* break out of the loop */
            	    break;
                }
            }
            loop886: ; /* Jump out to here if this rule does not match */


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definitions15495);
            if  (HASEXCEPTION())
            {
                goto rulepartition_definitionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartition_definitionsEx; /* Prevent compiler warnings */
    rulepartition_definitionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_definitions */

/**
 * $ANTLR start partition_definition
 * MySQLSimpleParser.g:2748:1: partition_definition : PARTITION_SYMBOL identifier ( VALUES_SYMBOL ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list ) )? ( partition_option )* ( OPEN_PAR_SYMBOL subpartition_definition ( COMMA_SYMBOL subpartition_definition )* CLOSE_PAR_SYMBOL )? ;
 */
static void
partition_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2748:21: ( PARTITION_SYMBOL identifier ( VALUES_SYMBOL ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list ) )? ( partition_option )* ( OPEN_PAR_SYMBOL subpartition_definition ( COMMA_SYMBOL subpartition_definition )* CLOSE_PAR_SYMBOL )? )
        // MySQLSimpleParser.g:2749:2: PARTITION_SYMBOL identifier ( VALUES_SYMBOL ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list ) )? ( partition_option )* ( OPEN_PAR_SYMBOL subpartition_definition ( COMMA_SYMBOL subpartition_definition )* CLOSE_PAR_SYMBOL )?
        {
             MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_partition_definition15504);
            if  (HASEXCEPTION())
            {
                goto rulepartition_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_partition_definition15506);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulepartition_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2750:2: ( VALUES_SYMBOL ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list ) )?
            {
                int alt889=2;
                switch ( LA(1) )
                {
                    case VALUES_SYMBOL:
                    	{
                    		alt889=1;
                    	}
                        break;
                }

                switch (alt889)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2751:3: VALUES_SYMBOL ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list )
            	    {
            	         MATCHT(VALUES_SYMBOL, &FOLLOW_VALUES_SYMBOL_in_partition_definition15513);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2752:3: ( LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL ) | IN_SYMBOL partition_value_list )
            	        {
            	            int alt888=2;
            	            switch ( LA(1) )
            	            {
            	            case LESS_SYMBOL:
            	            	{
            	            		alt888=1;
            	            	}
            	                break;
            	            case IN_SYMBOL:
            	            	{
            	            		alt888=2;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 888;
            	                EXCEPTION->state        = 0;


            	                goto rulepartition_definitionEx;

            	            }

            	            switch (alt888)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2753:4: LESS_SYMBOL THAN_SYMBOL ( partition_value_list | MAXVALUE_SYMBOL )
            	        	    {
            	        	         MATCHT(LESS_SYMBOL, &FOLLOW_LESS_SYMBOL_in_partition_definition15522);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(THAN_SYMBOL, &FOLLOW_THAN_SYMBOL_in_partition_definition15524);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:2753:28: ( partition_value_list | MAXVALUE_SYMBOL )
            	        	        {
            	        	            int alt887=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	            case OPEN_PAR_SYMBOL:
            	        	            	{
            	        	            		alt887=1;
            	        	            	}
            	        	                break;
            	        	            case MAXVALUE_SYMBOL:
            	        	            	{
            	        	            		alt887=2;
            	        	            	}
            	        	                break;

            	        	            default:
            	        	                if (BACKTRACKING>0)
            	        	                {
            	        	                    FAILEDFLAG = ANTLR3_TRUE;
            	        	                    return ;
            	        	                }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"";
            	        	                EXCEPTION->decisionNum  = 887;
            	        	                EXCEPTION->state        = 0;


            	        	                goto rulepartition_definitionEx;

            	        	            }

            	        	            switch (alt887)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:2753:29: partition_value_list
            	        	        	    {
            	        	        	        FOLLOWPUSH(FOLLOW_partition_value_list_in_partition_definition15527);
            	        	        	        partition_value_list(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartition_definitionEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;
            	        	        	case 2:
            	        	        	    // MySQLSimpleParser.g:2753:52: MAXVALUE_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(MAXVALUE_SYMBOL, &FOLLOW_MAXVALUE_SYMBOL_in_partition_definition15531);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulepartition_definitionEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:2754:6: IN_SYMBOL partition_value_list
            	        	    {
            	        	         MATCHT(IN_SYMBOL, &FOLLOW_IN_SYMBOL_in_partition_definition15539);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_partition_value_list_in_partition_definition15541);
            	        	        partition_value_list(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2757:2: ( partition_option )*

            for (;;)
            {
                int alt890=2;
                switch ( LA(1) )
                {
                case COMMENT_SYMBOL:
                case DATA_SYMBOL:
                case ENGINE_SYMBOL:
                case MAX_ROWS_SYMBOL:
                case MIN_ROWS_SYMBOL:
                case NODEGROUP_SYMBOL:
                case STORAGE_SYMBOL:
                case TABLESPACE_SYMBOL:
                case INDEX_SYMBOL:
                	{
                		alt890=1;
                	}
                    break;

                }

                switch (alt890)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2757:3: partition_option
            	    {
            	        FOLLOWPUSH(FOLLOW_partition_option_in_partition_definition15553);
            	        partition_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop890;	/* break out of the loop */
            	    break;
                }
            }
            loop890: ; /* Jump out to here if this rule does not match */


            // MySQLSimpleParser.g:2758:2: ( OPEN_PAR_SYMBOL subpartition_definition ( COMMA_SYMBOL subpartition_definition )* CLOSE_PAR_SYMBOL )?
            {
                int alt892=2;
                switch ( LA(1) )
                {
                    case OPEN_PAR_SYMBOL:
                    	{
                    		alt892=1;
                    	}
                        break;
                }

                switch (alt892)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2758:3: OPEN_PAR_SYMBOL subpartition_definition ( COMMA_SYMBOL subpartition_definition )* CLOSE_PAR_SYMBOL
            	    {
            	         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partition_definition15559);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_subpartition_definition_in_partition_definition15561);
            	        subpartition_definition(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2758:43: ( COMMA_SYMBOL subpartition_definition )*

            	        for (;;)
            	        {
            	            int alt891=2;
            	            switch ( LA(1) )
            	            {
            	            case COMMA_SYMBOL:
            	            	{
            	            		alt891=1;
            	            	}
            	                break;

            	            }

            	            switch (alt891)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2758:44: COMMA_SYMBOL subpartition_definition
            	        	    {
            	        	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_partition_definition15564);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_subpartition_definition_in_partition_definition15566);
            	        	        subpartition_definition(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_definitionEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	        	default:
            	        	    goto loop891;	/* break out of the loop */
            	        	    break;
            	            }
            	        }
            	        loop891: ; /* Jump out to here if this rule does not match */


            	         MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partition_definition15570);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartition_definitionEx; /* Prevent compiler warnings */
    rulepartition_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_definition */

/**
 * $ANTLR start partition_option
 * MySQLSimpleParser.g:2761:1: partition_option : ( TABLESPACE_SYMBOL ( EQUAL_OPERATOR )? identifier | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( MAX_ROWS_SYMBOL | MIN_ROWS_SYMBOL ) ( EQUAL_OPERATOR )? real_ulong_number | ( DATA_SYMBOL | INDEX_SYMBOL ) DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? string_literal | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal );
 */
static void
partition_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2761:17: ( TABLESPACE_SYMBOL ( EQUAL_OPERATOR )? identifier | ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref | NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number | ( MAX_ROWS_SYMBOL | MIN_ROWS_SYMBOL ) ( EQUAL_OPERATOR )? real_ulong_number | ( DATA_SYMBOL | INDEX_SYMBOL ) DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? string_literal | COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal )

            ANTLR3_UINT32 alt900;

            alt900=6;

            switch ( LA(1) )
            {
            case TABLESPACE_SYMBOL:
            	{
            		alt900=1;
            	}
                break;
            case ENGINE_SYMBOL:
            case STORAGE_SYMBOL:
            	{
            		alt900=2;
            	}
                break;
            case NODEGROUP_SYMBOL:
            	{
            		alt900=3;
            	}
                break;
            case MAX_ROWS_SYMBOL:
            case MIN_ROWS_SYMBOL:
            	{
            		alt900=4;
            	}
                break;
            case DATA_SYMBOL:
            case INDEX_SYMBOL:
            	{
            		alt900=5;
            	}
                break;
            case COMMENT_SYMBOL:
            	{
            		alt900=6;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 900;
                EXCEPTION->state        = 0;


                goto rulepartition_optionEx;

            }

            switch (alt900)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2762:2: TABLESPACE_SYMBOL ( EQUAL_OPERATOR )? identifier
        	    {
        	         MATCHT(TABLESPACE_SYMBOL, &FOLLOW_TABLESPACE_SYMBOL_in_partition_option15581);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2762:20: ( EQUAL_OPERATOR )?
        	        {
        	            int alt893=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt893=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt893)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2762:20: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15583);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_identifier_in_partition_option15586);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2763:4: ( STORAGE_SYMBOL )? ENGINE_SYMBOL ( EQUAL_OPERATOR )? engine_ref
        	    {
        	        // MySQLSimpleParser.g:2763:4: ( STORAGE_SYMBOL )?
        	        {
        	            int alt894=2;
        	            switch ( LA(1) )
        	            {
        	                case STORAGE_SYMBOL:
        	                	{
        	                		alt894=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt894)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2763:4: STORAGE_SYMBOL
        	        	    {
        	        	         MATCHT(STORAGE_SYMBOL, &FOLLOW_STORAGE_SYMBOL_in_partition_option15591);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENGINE_SYMBOL, &FOLLOW_ENGINE_SYMBOL_in_partition_option15594);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2763:34: ( EQUAL_OPERATOR )?
        	        {
        	            int alt895=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt895=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt895)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2763:34: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15596);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_engine_ref_in_partition_option15599);
        	        engine_ref(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2764:4: NODEGROUP_SYMBOL ( EQUAL_OPERATOR )? real_ulong_number
        	    {
        	         MATCHT(NODEGROUP_SYMBOL, &FOLLOW_NODEGROUP_SYMBOL_in_partition_option15604);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2764:21: ( EQUAL_OPERATOR )?
        	        {
        	            int alt896=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt896=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt896)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2764:21: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15606);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_partition_option15609);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:2765:4: ( MAX_ROWS_SYMBOL | MIN_ROWS_SYMBOL ) ( EQUAL_OPERATOR )? real_ulong_number
        	    {
        	        if ( LA(1) == MAX_ROWS_SYMBOL || LA(1) == MIN_ROWS_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulepartition_optionEx;
        	        }


        	        // MySQLSimpleParser.g:2765:40: ( EQUAL_OPERATOR )?
        	        {
        	            int alt897=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt897=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt897)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2765:40: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15622);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_real_ulong_number_in_partition_option15625);
        	        real_ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:2766:4: ( DATA_SYMBOL | INDEX_SYMBOL ) DIRECTORY_SYMBOL ( EQUAL_OPERATOR )? string_literal
        	    {
        	        if ( LA(1) == DATA_SYMBOL || LA(1) == INDEX_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulepartition_optionEx;
        	        }


        	         MATCHT(DIRECTORY_SYMBOL, &FOLLOW_DIRECTORY_SYMBOL_in_partition_option15638);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2766:50: ( EQUAL_OPERATOR )?
        	        {
        	            int alt898=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt898=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt898)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2766:50: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15640);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_string_literal_in_partition_option15643);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:2767:4: COMMENT_SYMBOL ( EQUAL_OPERATOR )? string_literal
        	    {
        	         MATCHT(COMMENT_SYMBOL, &FOLLOW_COMMENT_SYMBOL_in_partition_option15648);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2767:19: ( EQUAL_OPERATOR )?
        	        {
        	            int alt899=2;
        	            switch ( LA(1) )
        	            {
        	                case EQUAL_OPERATOR:
        	                	{
        	                		alt899=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt899)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2767:19: EQUAL_OPERATOR
        	        	    {
        	        	         MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_partition_option15650);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulepartition_optionEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        FOLLOWPUSH(FOLLOW_string_literal_in_partition_option15653);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepartition_optionEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulepartition_optionEx; /* Prevent compiler warnings */
    rulepartition_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_option */

/**
 * $ANTLR start subpartition_definition
 * MySQLSimpleParser.g:2770:1: subpartition_definition : SUBPARTITION_SYMBOL identifier ( partition_option )* ;
 */
static void
subpartition_definition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2770:24: ( SUBPARTITION_SYMBOL identifier ( partition_option )* )
        // MySQLSimpleParser.g:2771:2: SUBPARTITION_SYMBOL identifier ( partition_option )*
        {
             MATCHT(SUBPARTITION_SYMBOL, &FOLLOW_SUBPARTITION_SYMBOL_in_subpartition_definition15662);
            if  (HASEXCEPTION())
            {
                goto rulesubpartition_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_subpartition_definition15664);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulesubpartition_definitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2771:33: ( partition_option )*

            for (;;)
            {
                int alt901=2;
                switch ( LA(1) )
                {
                case COMMENT_SYMBOL:
                case DATA_SYMBOL:
                case ENGINE_SYMBOL:
                case MAX_ROWS_SYMBOL:
                case MIN_ROWS_SYMBOL:
                case NODEGROUP_SYMBOL:
                case STORAGE_SYMBOL:
                case TABLESPACE_SYMBOL:
                case INDEX_SYMBOL:
                	{
                		alt901=1;
                	}
                    break;

                }

                switch (alt901)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2771:34: partition_option
            	    {
            	        FOLLOWPUSH(FOLLOW_partition_option_in_subpartition_definition15667);
            	        partition_option(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulesubpartition_definitionEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop901;	/* break out of the loop */
            	    break;
                }
            }
            loop901: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulesubpartition_definitionEx; /* Prevent compiler warnings */
    rulesubpartition_definitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end subpartition_definition */

/**
 * $ANTLR start partition_value_list
 * MySQLSimpleParser.g:2774:1: partition_value_list : OPEN_PAR_SYMBOL ( expression | MAXVALUE_SYMBOL ) ( COMMA_SYMBOL ( expression | MAXVALUE_SYMBOL ) )* CLOSE_PAR_SYMBOL ;
 */
static void
partition_value_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2774:21: ( OPEN_PAR_SYMBOL ( expression | MAXVALUE_SYMBOL ) ( COMMA_SYMBOL ( expression | MAXVALUE_SYMBOL ) )* CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2775:2: OPEN_PAR_SYMBOL ( expression | MAXVALUE_SYMBOL ) ( COMMA_SYMBOL ( expression | MAXVALUE_SYMBOL ) )* CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_partition_value_list15678);
            if  (HASEXCEPTION())
            {
                goto rulepartition_value_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2775:18: ( expression | MAXVALUE_SYMBOL )
            {
                int alt902=2;
                {
                    int LA902_0 = LA(1);
                    if ( (((LA902_0 >= ACCOUNT_SYMBOL) && (LA902_0 <= YEAR_SYMBOL)) || LA902_0 == BINARY_SYMBOL || ((LA902_0 >= CASE_SYMBOL) && (LA902_0 <= CAST_SYMBOL)) || LA902_0 == CHAR_SYMBOL || ((LA902_0 >= CONVERT_SYMBOL) && (LA902_0 <= COUNT_SYMBOL)) || LA902_0 == CURDATE_SYMBOL || LA902_0 == CURRENT_USER_SYMBOL || LA902_0 == CURTIME_SYMBOL || LA902_0 == DATABASE_SYMBOL || LA902_0 == DATE_ADD_SYMBOL || LA902_0 == DATE_SUB_SYMBOL || LA902_0 == EXISTS_SYMBOL || ((LA902_0 >= EXTRACT_SYMBOL) && (LA902_0 <= FALSE_SYMBOL)) || LA902_0 == GROUP_CONCAT_SYMBOL || LA902_0 == IF_SYMBOL || LA902_0 == INSERT_SYMBOL || LA902_0 == INTERVAL_SYMBOL || LA902_0 == LEFT_SYMBOL || LA902_0 == MATCH_SYMBOL || LA902_0 == MAX_SYMBOL || LA902_0 == MIN_SYMBOL || LA902_0 == MOD_SYMBOL || LA902_0 == NCHAR_TEXT || ((LA902_0 >= NOT2_SYMBOL) && (LA902_0 <= NOW_SYMBOL)) || ((LA902_0 >= NULL2_SYMBOL) && (LA902_0 <= NULL_SYMBOL)) || ((LA902_0 >= OPEN_CURLY_SYMBOL) && (LA902_0 <= OPEN_PAR_SYMBOL)) || ((LA902_0 >= PARAM_MARKER) && (LA902_0 <= POSITION_SYMBOL)) || ((LA902_0 >= REPEAT_SYMBOL) && (LA902_0 <= REPLACE_SYMBOL)) || LA902_0 == RIGHT_SYMBOL || LA902_0 == SINGLE_QUOTED_TEXT || LA902_0 == STDDEV_SAMP_SYMBOL || LA902_0 == STD_SYMBOL || LA902_0 == SUBSTRING_SYMBOL || ((LA902_0 >= SUM_SYMBOL) && (LA902_0 <= SYSDATE_SYMBOL)) || ((LA902_0 >= TRIM_SYMBOL) && (LA902_0 <= TRUE_SYMBOL)) || ((LA902_0 >= UTC_DATE_SYMBOL) && (LA902_0 <= UTC_TIME_SYMBOL)) || LA902_0 == VALUES_SYMBOL || LA902_0 == VARIANCE_SYMBOL || LA902_0 == VAR_SAMP_SYMBOL || ((LA902_0 >= AT_AT_SIGN_SYMBOL) && (LA902_0 <= AT_TEXT_SUFFIX)) || ((LA902_0 >= BACK_TICK_QUOTED_ID) && (LA902_0 <= BIN_NUMBER)) || LA902_0 == BITWISE_NOT_OPERATOR || ((LA902_0 >= BIT_AND_SYMBOL) && (LA902_0 <= BIT_XOR_SYMBOL)) || LA902_0 == DECIMAL_NUMBER || LA902_0 == DOT_SYMBOL || LA902_0 == DOUBLE_QUOTED_TEXT || LA902_0 == FLOAT_NUMBER || ((LA902_0 >= HEX_NUMBER) && (LA902_0 <= IDENTIFIER)) || LA902_0 == INT_NUMBER || LA902_0 == LOGICAL_NOT_OPERATOR || ((LA902_0 >= LONG_NUMBER) && (LA902_0 <= MINUS_OPERATOR)) || LA902_0 == PLUS_OPERATOR || ((LA902_0 >= ULONGLONG_NUMBER) && (LA902_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt902=1;
                    }
                    else if ( (LA902_0 == MAXVALUE_SYMBOL))
                    {
                        alt902=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 902;
                        EXCEPTION->state        = 0;


                        goto rulepartition_value_listEx;

                    }
                }
                switch (alt902)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2775:19: expression
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_partition_value_list15681);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2775:32: MAXVALUE_SYMBOL
            	    {
            	         MATCHT(MAXVALUE_SYMBOL, &FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15685);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

            // MySQLSimpleParser.g:2775:49: ( COMMA_SYMBOL ( expression | MAXVALUE_SYMBOL ) )*

            for (;;)
            {
                int alt904=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt904=1;
                	}
                    break;

                }

                switch (alt904)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2775:50: COMMA_SYMBOL ( expression | MAXVALUE_SYMBOL )
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_partition_value_list15689);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulepartition_value_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2775:63: ( expression | MAXVALUE_SYMBOL )
            	        {
            	            int alt903=2;
            	            {
            	                int LA903_0 = LA(1);
            	                if ( (((LA903_0 >= ACCOUNT_SYMBOL) && (LA903_0 <= YEAR_SYMBOL)) || LA903_0 == BINARY_SYMBOL || ((LA903_0 >= CASE_SYMBOL) && (LA903_0 <= CAST_SYMBOL)) || LA903_0 == CHAR_SYMBOL || ((LA903_0 >= CONVERT_SYMBOL) && (LA903_0 <= COUNT_SYMBOL)) || LA903_0 == CURDATE_SYMBOL || LA903_0 == CURRENT_USER_SYMBOL || LA903_0 == CURTIME_SYMBOL || LA903_0 == DATABASE_SYMBOL || LA903_0 == DATE_ADD_SYMBOL || LA903_0 == DATE_SUB_SYMBOL || LA903_0 == EXISTS_SYMBOL || ((LA903_0 >= EXTRACT_SYMBOL) && (LA903_0 <= FALSE_SYMBOL)) || LA903_0 == GROUP_CONCAT_SYMBOL || LA903_0 == IF_SYMBOL || LA903_0 == INSERT_SYMBOL || LA903_0 == INTERVAL_SYMBOL || LA903_0 == LEFT_SYMBOL || LA903_0 == MATCH_SYMBOL || LA903_0 == MAX_SYMBOL || LA903_0 == MIN_SYMBOL || LA903_0 == MOD_SYMBOL || LA903_0 == NCHAR_TEXT || ((LA903_0 >= NOT2_SYMBOL) && (LA903_0 <= NOW_SYMBOL)) || ((LA903_0 >= NULL2_SYMBOL) && (LA903_0 <= NULL_SYMBOL)) || ((LA903_0 >= OPEN_CURLY_SYMBOL) && (LA903_0 <= OPEN_PAR_SYMBOL)) || ((LA903_0 >= PARAM_MARKER) && (LA903_0 <= POSITION_SYMBOL)) || ((LA903_0 >= REPEAT_SYMBOL) && (LA903_0 <= REPLACE_SYMBOL)) || LA903_0 == RIGHT_SYMBOL || LA903_0 == SINGLE_QUOTED_TEXT || LA903_0 == STDDEV_SAMP_SYMBOL || LA903_0 == STD_SYMBOL || LA903_0 == SUBSTRING_SYMBOL || ((LA903_0 >= SUM_SYMBOL) && (LA903_0 <= SYSDATE_SYMBOL)) || ((LA903_0 >= TRIM_SYMBOL) && (LA903_0 <= TRUE_SYMBOL)) || ((LA903_0 >= UTC_DATE_SYMBOL) && (LA903_0 <= UTC_TIME_SYMBOL)) || LA903_0 == VALUES_SYMBOL || LA903_0 == VARIANCE_SYMBOL || LA903_0 == VAR_SAMP_SYMBOL || ((LA903_0 >= AT_AT_SIGN_SYMBOL) && (LA903_0 <= AT_TEXT_SUFFIX)) || ((LA903_0 >= BACK_TICK_QUOTED_ID) && (LA903_0 <= BIN_NUMBER)) || LA903_0 == BITWISE_NOT_OPERATOR || ((LA903_0 >= BIT_AND_SYMBOL) && (LA903_0 <= BIT_XOR_SYMBOL)) || LA903_0 == DECIMAL_NUMBER || LA903_0 == DOT_SYMBOL || LA903_0 == DOUBLE_QUOTED_TEXT || LA903_0 == FLOAT_NUMBER || ((LA903_0 >= HEX_NUMBER) && (LA903_0 <= IDENTIFIER)) || LA903_0 == INT_NUMBER || LA903_0 == LOGICAL_NOT_OPERATOR || ((LA903_0 >= LONG_NUMBER) && (LA903_0 <= MINUS_OPERATOR)) || LA903_0 == PLUS_OPERATOR || ((LA903_0 >= ULONGLONG_NUMBER) && (LA903_0 <= UNDERSCORE_CHARSET))))
            	                {
            	                    alt903=1;
            	                }
            	                else if ( (LA903_0 == MAXVALUE_SYMBOL))
            	                {
            	                    alt903=2;
            	                }
            	                else
            	                {
            	                    if (BACKTRACKING>0)
            	                    {
            	                        FAILEDFLAG = ANTLR3_TRUE;
            	                        return ;
            	                    }


            	                    CONSTRUCTEX();
            	                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                    EXCEPTION->message      = (void *)"";
            	                    EXCEPTION->decisionNum  = 903;
            	                    EXCEPTION->state        = 0;


            	                    goto rulepartition_value_listEx;

            	                }
            	            }
            	            switch (alt903)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2775:64: expression
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_expression_in_partition_value_list15692);
            	        	        expression(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_value_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:2775:77: MAXVALUE_SYMBOL
            	        	    {
            	        	         MATCHT(MAXVALUE_SYMBOL, &FOLLOW_MAXVALUE_SYMBOL_in_partition_value_list15696);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulepartition_value_listEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

            	default:
            	    goto loop904;	/* break out of the loop */
            	    break;
                }
            }
            loop904: ; /* Jump out to here if this rule does not match */


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_partition_value_list15701);
            if  (HASEXCEPTION())
            {
                goto rulepartition_value_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulepartition_value_listEx; /* Prevent compiler warnings */
    rulepartition_value_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end partition_value_list */

/**
 * $ANTLR start definer_clause
 * MySQLSimpleParser.g:2778:1: definer_clause : DEFINER_SYMBOL EQUAL_OPERATOR user ;
 */
static void
definer_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2778:15: ( DEFINER_SYMBOL EQUAL_OPERATOR user )
        // MySQLSimpleParser.g:2779:2: DEFINER_SYMBOL EQUAL_OPERATOR user
        {
             MATCHT(DEFINER_SYMBOL, &FOLLOW_DEFINER_SYMBOL_in_definer_clause15710);
            if  (HASEXCEPTION())
            {
                goto ruledefiner_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_definer_clause15712);
            if  (HASEXCEPTION())
            {
                goto ruledefiner_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_user_in_definer_clause15714);
            user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruledefiner_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledefiner_clauseEx; /* Prevent compiler warnings */
    ruledefiner_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end definer_clause */

/**
 * $ANTLR start if_exists
 * MySQLSimpleParser.g:2782:1: if_exists : IF_SYMBOL EXISTS_SYMBOL ;
 */
static void
if_exists(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2782:10: ( IF_SYMBOL EXISTS_SYMBOL )
        // MySQLSimpleParser.g:2783:2: IF_SYMBOL EXISTS_SYMBOL
        {
             MATCHT(IF_SYMBOL, &FOLLOW_IF_SYMBOL_in_if_exists15723);
            if  (HASEXCEPTION())
            {
                goto ruleif_existsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EXISTS_SYMBOL, &FOLLOW_EXISTS_SYMBOL_in_if_exists15725);
            if  (HASEXCEPTION())
            {
                goto ruleif_existsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleif_existsEx; /* Prevent compiler warnings */
    ruleif_existsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end if_exists */

/**
 * $ANTLR start if_not_exists
 * MySQLSimpleParser.g:2786:1: if_not_exists : IF_SYMBOL not_rule EXISTS_SYMBOL ;
 */
static void
if_not_exists(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2786:14: ( IF_SYMBOL not_rule EXISTS_SYMBOL )
        // MySQLSimpleParser.g:2787:2: IF_SYMBOL not_rule EXISTS_SYMBOL
        {
             MATCHT(IF_SYMBOL, &FOLLOW_IF_SYMBOL_in_if_not_exists15734);
            if  (HASEXCEPTION())
            {
                goto ruleif_not_existsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_not_rule_in_if_not_exists15736);
            not_rule(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleif_not_existsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EXISTS_SYMBOL, &FOLLOW_EXISTS_SYMBOL_in_if_not_exists15738);
            if  (HASEXCEPTION())
            {
                goto ruleif_not_existsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleif_not_existsEx; /* Prevent compiler warnings */
    ruleif_not_existsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end if_not_exists */

/**
 * $ANTLR start procedure_parameter
 * MySQLSimpleParser.g:2790:1: procedure_parameter : ( IN_SYMBOL | OUT_SYMBOL | INOUT_SYMBOL )? function_parameter ;
 */
static void
procedure_parameter(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2790:20: ( ( IN_SYMBOL | OUT_SYMBOL | INOUT_SYMBOL )? function_parameter )
        // MySQLSimpleParser.g:2791:2: ( IN_SYMBOL | OUT_SYMBOL | INOUT_SYMBOL )? function_parameter
        {
            // MySQLSimpleParser.g:2791:2: ( IN_SYMBOL | OUT_SYMBOL | INOUT_SYMBOL )?
            {
                int alt905=2;
                switch ( LA(1) )
                {
                    case INOUT_SYMBOL:
                    case IN_SYMBOL:
                    case OUT_SYMBOL:
                    	{
                    		alt905=1;
                    	}
                        break;
                }

                switch (alt905)
                {
            	case 1:
            	    // MySQLSimpleParser.g:
            	    {
            	        if ( LA(1) == INOUT_SYMBOL || LA(1) == IN_SYMBOL || LA(1) == OUT_SYMBOL )
            	        {
            	            CONSUME();
            	            PERRORRECOVERY=ANTLR3_FALSE;
            	            FAILEDFLAG=ANTLR3_FALSE;

            	        }
            	        else
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	            CONSTRUCTEX();
            	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
            	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
            	            EXCEPTION->expectingSet = NULL;

            	            goto ruleprocedure_parameterEx;
            	        }


            	    }
            	    break;

                }
            }

            FOLLOWPUSH(FOLLOW_function_parameter_in_procedure_parameter15760);
            function_parameter(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_parameterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprocedure_parameterEx; /* Prevent compiler warnings */
    ruleprocedure_parameterEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end procedure_parameter */

/**
 * $ANTLR start function_parameter
 * MySQLSimpleParser.g:2794:1: function_parameter : identifier data_type ;
 */
static void
function_parameter(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2794:19: ( identifier data_type )
        // MySQLSimpleParser.g:2795:2: identifier data_type
        {
            FOLLOWPUSH(FOLLOW_identifier_in_function_parameter15769);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_parameterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_data_type_in_function_parameter15771);
            data_type(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_parameterEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefunction_parameterEx; /* Prevent compiler warnings */
    rulefunction_parameterEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_parameter */

/**
 * $ANTLR start schema_identifier_pair
 * MySQLSimpleParser.g:2798:1: schema_identifier_pair : OPEN_PAR_SYMBOL schema_ref COMMA_SYMBOL schema_ref CLOSE_PAR_SYMBOL ;
 */
static void
schema_identifier_pair(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2798:23: ( OPEN_PAR_SYMBOL schema_ref COMMA_SYMBOL schema_ref CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:2799:2: OPEN_PAR_SYMBOL schema_ref COMMA_SYMBOL schema_ref CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_schema_identifier_pair15780);
            if  (HASEXCEPTION())
            {
                goto ruleschema_identifier_pairEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_schema_ref_in_schema_identifier_pair15782);
            schema_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleschema_identifier_pairEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_schema_identifier_pair15784);
            if  (HASEXCEPTION())
            {
                goto ruleschema_identifier_pairEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_schema_ref_in_schema_identifier_pair15786);
            schema_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleschema_identifier_pairEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_schema_identifier_pair15788);
            if  (HASEXCEPTION())
            {
                goto ruleschema_identifier_pairEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleschema_identifier_pairEx; /* Prevent compiler warnings */
    ruleschema_identifier_pairEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end schema_identifier_pair */

/**
 * $ANTLR start view_identifier_list
 * MySQLSimpleParser.g:2802:1: view_identifier_list : view_ref ( COMMA_SYMBOL view_ref )* ;
 */
static void
view_identifier_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2802:21: ( view_ref ( COMMA_SYMBOL view_ref )* )
        // MySQLSimpleParser.g:2803:2: view_ref ( COMMA_SYMBOL view_ref )*
        {
            FOLLOWPUSH(FOLLOW_view_ref_in_view_identifier_list15797);
            view_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleview_identifier_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2803:11: ( COMMA_SYMBOL view_ref )*

            for (;;)
            {
                int alt906=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt906=1;
                	}
                    break;

                }

                switch (alt906)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2803:12: COMMA_SYMBOL view_ref
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_view_identifier_list15800);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleview_identifier_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_view_ref_in_view_identifier_list15802);
            	        view_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleview_identifier_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop906;	/* break out of the loop */
            	    break;
                }
            }
            loop906: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleview_identifier_listEx; /* Prevent compiler warnings */
    ruleview_identifier_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end view_identifier_list */

/**
 * $ANTLR start field_name_list
 * MySQLSimpleParser.g:2806:1: field_name_list : column_ref ( COMMA_SYMBOL column_ref )* ;
 */
static void
field_name_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2806:16: ( column_ref ( COMMA_SYMBOL column_ref )* )
        // MySQLSimpleParser.g:2807:2: column_ref ( COMMA_SYMBOL column_ref )*
        {
            FOLLOWPUSH(FOLLOW_column_ref_in_field_name_list15820);
            column_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefield_name_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2807:13: ( COMMA_SYMBOL column_ref )*

            for (;;)
            {
                int alt907=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt907=1;
                	}
                    break;

                }

                switch (alt907)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2807:14: COMMA_SYMBOL column_ref
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_field_name_list15823);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_name_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_column_ref_in_field_name_list15825);
            	        column_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefield_name_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop907;	/* break out of the loop */
            	    break;
                }
            }
            loop907: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefield_name_listEx; /* Prevent compiler warnings */
    rulefield_name_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_name_list */

/**
 * $ANTLR start column_assignment_list_with_default
 * MySQLSimpleParser.g:2810:1: column_assignment_list_with_default : column_assignment_with_default ( COMMA_SYMBOL column_assignment_with_default )* ;
 */
static void
column_assignment_list_with_default(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2810:36: ( column_assignment_with_default ( COMMA_SYMBOL column_assignment_with_default )* )
        // MySQLSimpleParser.g:2811:2: column_assignment_with_default ( COMMA_SYMBOL column_assignment_with_default )*
        {
            FOLLOWPUSH(FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15836);
            column_assignment_with_default(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_assignment_list_with_defaultEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2811:33: ( COMMA_SYMBOL column_assignment_with_default )*

            for (;;)
            {
                int alt908=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt908=1;
                	}
                    break;

                }

                switch (alt908)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2811:34: COMMA_SYMBOL column_assignment_with_default
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_column_assignment_list_with_default15839);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_assignment_list_with_defaultEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_column_assignment_with_default_in_column_assignment_list_with_default15841);
            	        column_assignment_with_default(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_assignment_list_with_defaultEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop908;	/* break out of the loop */
            	    break;
                }
            }
            loop908: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_assignment_list_with_defaultEx; /* Prevent compiler warnings */
    rulecolumn_assignment_list_with_defaultEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_assignment_list_with_default */

/**
 * $ANTLR start column_assignment_with_default
 * MySQLSimpleParser.g:2814:1: column_assignment_with_default : column_ref EQUAL_OPERATOR ( expression | DEFAULT_SYMBOL ) ;
 */
static void
column_assignment_with_default(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2814:31: ( column_ref EQUAL_OPERATOR ( expression | DEFAULT_SYMBOL ) )
        // MySQLSimpleParser.g:2815:2: column_ref EQUAL_OPERATOR ( expression | DEFAULT_SYMBOL )
        {
            FOLLOWPUSH(FOLLOW_column_ref_in_column_assignment_with_default15852);
            column_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_assignment_with_defaultEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(EQUAL_OPERATOR, &FOLLOW_EQUAL_OPERATOR_in_column_assignment_with_default15854);
            if  (HASEXCEPTION())
            {
                goto rulecolumn_assignment_with_defaultEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2815:28: ( expression | DEFAULT_SYMBOL )
            {
                int alt909=2;
                {
                    int LA909_0 = LA(1);
                    if ( (((LA909_0 >= ACCOUNT_SYMBOL) && (LA909_0 <= YEAR_SYMBOL)) || LA909_0 == BINARY_SYMBOL || ((LA909_0 >= CASE_SYMBOL) && (LA909_0 <= CAST_SYMBOL)) || LA909_0 == CHAR_SYMBOL || ((LA909_0 >= CONVERT_SYMBOL) && (LA909_0 <= COUNT_SYMBOL)) || LA909_0 == CURDATE_SYMBOL || LA909_0 == CURRENT_USER_SYMBOL || LA909_0 == CURTIME_SYMBOL || LA909_0 == DATABASE_SYMBOL || LA909_0 == DATE_ADD_SYMBOL || LA909_0 == DATE_SUB_SYMBOL || LA909_0 == EXISTS_SYMBOL || ((LA909_0 >= EXTRACT_SYMBOL) && (LA909_0 <= FALSE_SYMBOL)) || LA909_0 == GROUP_CONCAT_SYMBOL || LA909_0 == IF_SYMBOL || LA909_0 == INSERT_SYMBOL || LA909_0 == INTERVAL_SYMBOL || LA909_0 == LEFT_SYMBOL || LA909_0 == MATCH_SYMBOL || LA909_0 == MAX_SYMBOL || LA909_0 == MIN_SYMBOL || LA909_0 == MOD_SYMBOL || LA909_0 == NCHAR_TEXT || ((LA909_0 >= NOT2_SYMBOL) && (LA909_0 <= NOW_SYMBOL)) || ((LA909_0 >= NULL2_SYMBOL) && (LA909_0 <= NULL_SYMBOL)) || ((LA909_0 >= OPEN_CURLY_SYMBOL) && (LA909_0 <= OPEN_PAR_SYMBOL)) || ((LA909_0 >= PARAM_MARKER) && (LA909_0 <= POSITION_SYMBOL)) || ((LA909_0 >= REPEAT_SYMBOL) && (LA909_0 <= REPLACE_SYMBOL)) || LA909_0 == RIGHT_SYMBOL || LA909_0 == SINGLE_QUOTED_TEXT || LA909_0 == STDDEV_SAMP_SYMBOL || LA909_0 == STD_SYMBOL || LA909_0 == SUBSTRING_SYMBOL || ((LA909_0 >= SUM_SYMBOL) && (LA909_0 <= SYSDATE_SYMBOL)) || ((LA909_0 >= TRIM_SYMBOL) && (LA909_0 <= TRUE_SYMBOL)) || ((LA909_0 >= UTC_DATE_SYMBOL) && (LA909_0 <= UTC_TIME_SYMBOL)) || LA909_0 == VALUES_SYMBOL || LA909_0 == VARIANCE_SYMBOL || LA909_0 == VAR_SAMP_SYMBOL || ((LA909_0 >= AT_AT_SIGN_SYMBOL) && (LA909_0 <= AT_TEXT_SUFFIX)) || ((LA909_0 >= BACK_TICK_QUOTED_ID) && (LA909_0 <= BIN_NUMBER)) || LA909_0 == BITWISE_NOT_OPERATOR || ((LA909_0 >= BIT_AND_SYMBOL) && (LA909_0 <= BIT_XOR_SYMBOL)) || LA909_0 == DECIMAL_NUMBER || LA909_0 == DOT_SYMBOL || LA909_0 == DOUBLE_QUOTED_TEXT || LA909_0 == FLOAT_NUMBER || ((LA909_0 >= HEX_NUMBER) && (LA909_0 <= IDENTIFIER)) || LA909_0 == INT_NUMBER || LA909_0 == LOGICAL_NOT_OPERATOR || ((LA909_0 >= LONG_NUMBER) && (LA909_0 <= MINUS_OPERATOR)) || LA909_0 == PLUS_OPERATOR || ((LA909_0 >= ULONGLONG_NUMBER) && (LA909_0 <= UNDERSCORE_CHARSET))))
                    {
                        alt909=1;
                    }
                    else if ( (LA909_0 == DEFAULT_SYMBOL))
                    {
                        alt909=2;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 909;
                        EXCEPTION->state        = 0;


                        goto rulecolumn_assignment_with_defaultEx;

                    }
                }
                switch (alt909)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2815:29: expression
            	    {
            	        FOLLOWPUSH(FOLLOW_expression_in_column_assignment_with_default15857);
            	        expression(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_assignment_with_defaultEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2815:42: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_column_assignment_with_default15861);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_assignment_with_defaultEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_assignment_with_defaultEx; /* Prevent compiler warnings */
    rulecolumn_assignment_with_defaultEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_assignment_with_default */

/**
 * $ANTLR start charset_clause
 * MySQLSimpleParser.g:2818:1: charset_clause : charset ( text_or_identifier | DEFAULT_SYMBOL | BINARY_SYMBOL ) ;
 */
static void
charset_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2818:15: ( charset ( text_or_identifier | DEFAULT_SYMBOL | BINARY_SYMBOL ) )
        // MySQLSimpleParser.g:2819:2: charset ( text_or_identifier | DEFAULT_SYMBOL | BINARY_SYMBOL )
        {
            FOLLOWPUSH(FOLLOW_charset_in_charset_clause15871);
            charset(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecharset_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2819:10: ( text_or_identifier | DEFAULT_SYMBOL | BINARY_SYMBOL )
            {
                int alt910=3;
                {
                    int LA910_0 = LA(1);
                    if ( (((LA910_0 >= ACCOUNT_SYMBOL) && (LA910_0 <= YEAR_SYMBOL)) || LA910_0 == SINGLE_QUOTED_TEXT || LA910_0 == BACK_TICK_QUOTED_ID || LA910_0 == DOUBLE_QUOTED_TEXT || LA910_0 == IDENTIFIER))
                    {
                        alt910=1;
                    }
                    else if ( (LA910_0 == DEFAULT_SYMBOL))
                    {
                        alt910=2;
                    }
                    else if ( (LA910_0 == BINARY_SYMBOL))
                    {
                        alt910=3;
                    }
                    else
                    {
                        if (BACKTRACKING>0)
                        {
                            FAILEDFLAG = ANTLR3_TRUE;
                            return ;
                        }


                        CONSTRUCTEX();
                        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                        EXCEPTION->message      = (void *)"";
                        EXCEPTION->decisionNum  = 910;
                        EXCEPTION->state        = 0;


                        goto rulecharset_clauseEx;

                    }
                }
                switch (alt910)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2819:11: text_or_identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_charset_clause15874);
            	        text_or_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecharset_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2819:32: DEFAULT_SYMBOL
            	    {
            	         MATCHT(DEFAULT_SYMBOL, &FOLLOW_DEFAULT_SYMBOL_in_charset_clause15878);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecharset_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 3:
            	    // MySQLSimpleParser.g:2819:49: BINARY_SYMBOL
            	    {
            	         MATCHT(BINARY_SYMBOL, &FOLLOW_BINARY_SYMBOL_in_charset_clause15882);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecharset_clauseEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecharset_clauseEx; /* Prevent compiler warnings */
    rulecharset_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end charset_clause */

/**
 * $ANTLR start fields_clause
 * MySQLSimpleParser.g:2822:1: fields_clause : COLUMNS_SYMBOL ( field_term )+ ;
 */
static void
fields_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2822:14: ( COLUMNS_SYMBOL ( field_term )+ )
        // MySQLSimpleParser.g:2823:2: COLUMNS_SYMBOL ( field_term )+
        {
             MATCHT(COLUMNS_SYMBOL, &FOLLOW_COLUMNS_SYMBOL_in_fields_clause15892);
            if  (HASEXCEPTION())
            {
                goto rulefields_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2823:17: ( field_term )+
            {
                int cnt911=0;

                for (;;)
                {
                    int alt911=2;
            	switch ( LA(1) )
            	{
            	case ENCLOSED_SYMBOL:
            	case ESCAPED_SYMBOL:
            	case OPTIONALLY_SYMBOL:
            	case TERMINATED_SYMBOL:
            		{
            			alt911=1;
            		}
            	    break;

            	}

            	switch (alt911)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2823:17: field_term
            	        {
            	            FOLLOWPUSH(FOLLOW_field_term_in_fields_clause15894);
            	            field_term(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulefields_clauseEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt911 >= 1 )
            		{
            		    goto loop911;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulefields_clauseEx;
            	}
            	cnt911++;
                }
                loop911: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulefields_clauseEx; /* Prevent compiler warnings */
    rulefields_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end fields_clause */

/**
 * $ANTLR start field_term
 * MySQLSimpleParser.g:2826:1: field_term : ( TERMINATED_SYMBOL BY_SYMBOL text_string | ( OPTIONALLY_SYMBOL )? ENCLOSED_SYMBOL BY_SYMBOL text_string | ESCAPED_SYMBOL BY_SYMBOL text_string );
 */
static void
field_term(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2826:11: ( TERMINATED_SYMBOL BY_SYMBOL text_string | ( OPTIONALLY_SYMBOL )? ENCLOSED_SYMBOL BY_SYMBOL text_string | ESCAPED_SYMBOL BY_SYMBOL text_string )

            ANTLR3_UINT32 alt913;

            alt913=3;

            switch ( LA(1) )
            {
            case TERMINATED_SYMBOL:
            	{
            		alt913=1;
            	}
                break;
            case ENCLOSED_SYMBOL:
            case OPTIONALLY_SYMBOL:
            	{
            		alt913=2;
            	}
                break;
            case ESCAPED_SYMBOL:
            	{
            		alt913=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 913;
                EXCEPTION->state        = 0;


                goto rulefield_termEx;

            }

            switch (alt913)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2827:2: TERMINATED_SYMBOL BY_SYMBOL text_string
        	    {
        	         MATCHT(TERMINATED_SYMBOL, &FOLLOW_TERMINATED_SYMBOL_in_field_term15904);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_field_term15906);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_field_term15908);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2828:4: ( OPTIONALLY_SYMBOL )? ENCLOSED_SYMBOL BY_SYMBOL text_string
        	    {
        	        // MySQLSimpleParser.g:2828:4: ( OPTIONALLY_SYMBOL )?
        	        {
        	            int alt912=2;
        	            switch ( LA(1) )
        	            {
        	                case OPTIONALLY_SYMBOL:
        	                	{
        	                		alt912=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt912)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2828:4: OPTIONALLY_SYMBOL
        	        	    {
        	        	         MATCHT(OPTIONALLY_SYMBOL, &FOLLOW_OPTIONALLY_SYMBOL_in_field_term15913);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulefield_termEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(ENCLOSED_SYMBOL, &FOLLOW_ENCLOSED_SYMBOL_in_field_term15916);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_field_term15918);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_field_term15920);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:2829:4: ESCAPED_SYMBOL BY_SYMBOL text_string
        	    {
        	         MATCHT(ESCAPED_SYMBOL, &FOLLOW_ESCAPED_SYMBOL_in_field_term15925);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_field_term15927);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_text_string_in_field_term15929);
        	        text_string(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulefield_termEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulefield_termEx; /* Prevent compiler warnings */
    rulefield_termEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end field_term */

/**
 * $ANTLR start lines_clause
 * MySQLSimpleParser.g:2832:1: lines_clause : LINES_SYMBOL ( line_term )+ ;
 */
static void
lines_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2832:13: ( LINES_SYMBOL ( line_term )+ )
        // MySQLSimpleParser.g:2833:2: LINES_SYMBOL ( line_term )+
        {
             MATCHT(LINES_SYMBOL, &FOLLOW_LINES_SYMBOL_in_lines_clause15938);
            if  (HASEXCEPTION())
            {
                goto rulelines_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2833:15: ( line_term )+
            {
                int cnt914=0;

                for (;;)
                {
                    int alt914=2;
            	switch ( LA(1) )
            	{
            	case STARTING_SYMBOL:
            	case TERMINATED_SYMBOL:
            		{
            			alt914=1;
            		}
            	    break;

            	}

            	switch (alt914)
            	{
            	    case 1:
            	        // MySQLSimpleParser.g:2833:15: line_term
            	        {
            	            FOLLOWPUSH(FOLLOW_line_term_in_lines_clause15940);
            	            line_term(ctx);

            	            FOLLOWPOP();
            	            if  (HASEXCEPTION())
            	            {
            	                goto rulelines_clauseEx;
            	            }
            	            if (HASFAILED())
            	            {
            	                return ;
            	            }


            	        }
            	        break;

            	    default:

            		if ( cnt914 >= 1 )
            		{
            		    goto loop914;
            		}
            		if (BACKTRACKING>0)
            		{
            		    FAILEDFLAG = ANTLR3_TRUE;
            		    return ;
            		}

            		/* mismatchedSetEx()
            		 */
            		CONSTRUCTEX();
            		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
            		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


            		goto rulelines_clauseEx;
            	}
            	cnt914++;
                }
                loop914: ;	/* Jump to here if this rule does not match */
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulelines_clauseEx; /* Prevent compiler warnings */
    rulelines_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end lines_clause */

/**
 * $ANTLR start line_term
 * MySQLSimpleParser.g:2836:1: line_term : ( TERMINATED_SYMBOL | STARTING_SYMBOL ) BY_SYMBOL text_string ;
 */
static void
line_term(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2836:10: ( ( TERMINATED_SYMBOL | STARTING_SYMBOL ) BY_SYMBOL text_string )
        // MySQLSimpleParser.g:2837:2: ( TERMINATED_SYMBOL | STARTING_SYMBOL ) BY_SYMBOL text_string
        {
            if ( LA(1) == STARTING_SYMBOL || LA(1) == TERMINATED_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleline_termEx;
            }


             MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_line_term15958);
            if  (HASEXCEPTION())
            {
                goto ruleline_termEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_string_in_line_term15960);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleline_termEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleline_termEx; /* Prevent compiler warnings */
    ruleline_termEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end line_term */

/**
 * $ANTLR start user_list
 * MySQLSimpleParser.g:2840:1: user_list : user ( COMMA_SYMBOL user )* ;
 */
static void
user_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2840:10: ( user ( COMMA_SYMBOL user )* )
        // MySQLSimpleParser.g:2841:2: user ( COMMA_SYMBOL user )*
        {
            FOLLOWPUSH(FOLLOW_user_in_user_list15969);
            user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleuser_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2841:7: ( COMMA_SYMBOL user )*

            for (;;)
            {
                int alt915=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt915=1;
                	}
                    break;

                }

                switch (alt915)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2841:8: COMMA_SYMBOL user
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_user_list15972);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleuser_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_user_in_user_list15974);
            	        user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleuser_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop915;	/* break out of the loop */
            	    break;
                }
            }
            loop915: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleuser_listEx; /* Prevent compiler warnings */
    ruleuser_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end user_list */

/**
 * $ANTLR start grant_list
 * MySQLSimpleParser.g:2844:1: grant_list : grant_user ( COMMA_SYMBOL grant_user )* ;
 */
static void
grant_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2844:11: ( grant_user ( COMMA_SYMBOL grant_user )* )
        // MySQLSimpleParser.g:2845:2: grant_user ( COMMA_SYMBOL grant_user )*
        {
            FOLLOWPUSH(FOLLOW_grant_user_in_grant_list15985);
            grant_user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrant_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2845:13: ( COMMA_SYMBOL grant_user )*

            for (;;)
            {
                int alt916=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt916=1;
                	}
                    break;

                }

                switch (alt916)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2845:14: COMMA_SYMBOL grant_user
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_grant_list15988);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_grant_user_in_grant_list15990);
            	        grant_user(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop916;	/* break out of the loop */
            	    break;
                }
            }
            loop916: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto rulegrant_listEx; /* Prevent compiler warnings */
    rulegrant_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_list */

/**
 * $ANTLR start grant_user
 * MySQLSimpleParser.g:2848:1: grant_user : user ( IDENTIFIED_SYMBOL ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? ) )? ;
 */
static void
grant_user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2848:11: ( user ( IDENTIFIED_SYMBOL ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? ) )? )
        // MySQLSimpleParser.g:2849:2: user ( IDENTIFIED_SYMBOL ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? ) )?
        {
            FOLLOWPUSH(FOLLOW_user_in_grant_user16001);
            user(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulegrant_userEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2850:2: ( IDENTIFIED_SYMBOL ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? ) )?
            {
                int alt921=2;
                switch ( LA(1) )
                {
                    case IDENTIFIED_SYMBOL:
                    	{
                    		alt921=1;
                    	}
                        break;
                }

                switch (alt921)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2851:3: IDENTIFIED_SYMBOL ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? )
            	    {
            	         MATCHT(IDENTIFIED_SYMBOL, &FOLLOW_IDENTIFIED_SYMBOL_in_grant_user16008);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulegrant_userEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2852:3: ( BY_SYMBOL ( PASSWORD_SYMBOL )? text_string |{...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )? )
            	        {
            	            int alt920=2;
            	            switch ( LA(1) )
            	            {
            	            case BY_SYMBOL:
            	            	{
            	            		alt920=1;
            	            	}
            	                break;
            	            case WITH_SYMBOL:
            	            	{
            	            		alt920=2;
            	            	}
            	                break;

            	            default:
            	                if (BACKTRACKING>0)
            	                {
            	                    FAILEDFLAG = ANTLR3_TRUE;
            	                    return ;
            	                }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	                EXCEPTION->message      = (void *)"";
            	                EXCEPTION->decisionNum  = 920;
            	                EXCEPTION->state        = 0;


            	                goto rulegrant_userEx;

            	            }

            	            switch (alt920)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2853:4: BY_SYMBOL ( PASSWORD_SYMBOL )? text_string
            	        	    {
            	        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_grant_user16017);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulegrant_userEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:2853:14: ( PASSWORD_SYMBOL )?
            	        	        {
            	        	            int alt917=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case PASSWORD_SYMBOL:
            	        	                	{
            	        	                		alt917=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt917)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:2853:14: PASSWORD_SYMBOL
            	        	        	    {
            	        	        	         MATCHT(PASSWORD_SYMBOL, &FOLLOW_PASSWORD_SYMBOL_in_grant_user16019);
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulegrant_userEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	        FOLLOWPUSH(FOLLOW_text_string_in_grant_user16022);
            	        	        text_string(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulegrant_userEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:2854:6: {...}? WITH_SYMBOL text_or_identifier ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )?
            	        	    {
            	        	        if ( !((SERVER_VERSION >= 50600)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50600";
            	        	                EXCEPTION->ruleName	 = (void *)"grant_user";


            	        	        }

            	        	         MATCHT(WITH_SYMBOL, &FOLLOW_WITH_SYMBOL_in_grant_user16031);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulegrant_userEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_grant_user16033);
            	        	        text_or_identifier(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulegrant_userEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	        // MySQLSimpleParser.g:2854:64: ( ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string )?
            	        	        {
            	        	            int alt919=2;
            	        	            switch ( LA(1) )
            	        	            {
            	        	                case BY_SYMBOL:
            	        	                case AS_SYMBOL:
            	        	                	{
            	        	                		alt919=1;
            	        	                	}
            	        	                    break;
            	        	            }

            	        	            switch (alt919)
            	        	            {
            	        	        	case 1:
            	        	        	    // MySQLSimpleParser.g:2855:5: ( AS_SYMBOL |{...}? BY_SYMBOL ) text_string
            	        	        	    {
            	        	        	        // MySQLSimpleParser.g:2855:5: ( AS_SYMBOL |{...}? BY_SYMBOL )
            	        	        	        {
            	        	        	            int alt918=2;
            	        	        	            switch ( LA(1) )
            	        	        	            {
            	        	        	            case AS_SYMBOL:
            	        	        	            	{
            	        	        	            		alt918=1;
            	        	        	            	}
            	        	        	                break;
            	        	        	            case BY_SYMBOL:
            	        	        	            	{
            	        	        	            		alt918=2;
            	        	        	            	}
            	        	        	                break;

            	        	        	            default:
            	        	        	                if (BACKTRACKING>0)
            	        	        	                {
            	        	        	                    FAILEDFLAG = ANTLR3_TRUE;
            	        	        	                    return ;
            	        	        	                }

            	        	        	                CONSTRUCTEX();
            	        	        	                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            	        	        	                EXCEPTION->message      = (void *)"";
            	        	        	                EXCEPTION->decisionNum  = 918;
            	        	        	                EXCEPTION->state        = 0;


            	        	        	                goto rulegrant_userEx;

            	        	        	            }

            	        	        	            switch (alt918)
            	        	        	            {
            	        	        	        	case 1:
            	        	        	        	    // MySQLSimpleParser.g:2855:6: AS_SYMBOL
            	        	        	        	    {
            	        	        	        	         MATCHT(AS_SYMBOL, &FOLLOW_AS_SYMBOL_in_grant_user16042);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto rulegrant_userEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;
            	        	        	        	case 2:
            	        	        	        	    // MySQLSimpleParser.g:2855:18: {...}? BY_SYMBOL
            	        	        	        	    {
            	        	        	        	        if ( !((SERVER_VERSION >= 50706)) )
            	        	        	        	        {
            	        	        	        	            if (BACKTRACKING>0)
            	        	        	        	            {
            	        	        	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	        	        	                return ;
            	        	        	        	            }

            	        	        	        	                CONSTRUCTEX();
            	        	        	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	        	        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50706";
            	        	        	        	                EXCEPTION->ruleName	 = (void *)"grant_user";


            	        	        	        	        }

            	        	        	        	         MATCHT(BY_SYMBOL, &FOLLOW_BY_SYMBOL_in_grant_user16048);
            	        	        	        	        if  (HASEXCEPTION())
            	        	        	        	        {
            	        	        	        	            goto rulegrant_userEx;
            	        	        	        	        }
            	        	        	        	        if (HASFAILED())
            	        	        	        	        {
            	        	        	        	            return ;
            	        	        	        	        }


            	        	        	        	    }
            	        	        	        	    break;

            	        	        	            }
            	        	        	        }

            	        	        	        FOLLOWPUSH(FOLLOW_text_string_in_grant_user16051);
            	        	        	        text_string(ctx);

            	        	        	        FOLLOWPOP();
            	        	        	        if  (HASEXCEPTION())
            	        	        	        {
            	        	        	            goto rulegrant_userEx;
            	        	        	        }
            	        	        	        if (HASFAILED())
            	        	        	        {
            	        	        	            return ;
            	        	        	        }


            	        	        	    }
            	        	        	    break;

            	        	            }
            	        	        }

            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulegrant_userEx; /* Prevent compiler warnings */
    rulegrant_userEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end grant_user */

/**
 * $ANTLR start user
 * MySQLSimpleParser.g:2860:1: user : ( text_or_identifier ( AT_SIGN_SYMBOL text_or_identifier | AT_TEXT_SUFFIX )? | CURRENT_USER_SYMBOL ( parentheses )? );
 */
static void
user(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2860:5: ( text_or_identifier ( AT_SIGN_SYMBOL text_or_identifier | AT_TEXT_SUFFIX )? | CURRENT_USER_SYMBOL ( parentheses )? )

            ANTLR3_UINT32 alt924;

            alt924=2;

            {
                int LA924_0 = LA(1);
                if ( (((LA924_0 >= ACCOUNT_SYMBOL) && (LA924_0 <= YEAR_SYMBOL)) || LA924_0 == SINGLE_QUOTED_TEXT || LA924_0 == BACK_TICK_QUOTED_ID || LA924_0 == DOUBLE_QUOTED_TEXT || LA924_0 == IDENTIFIER))
                {
                    alt924=1;
                }
                else if ( (LA924_0 == CURRENT_USER_SYMBOL))
                {
                    alt924=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 924;
                    EXCEPTION->state        = 0;


                    goto ruleuserEx;

                }
            }
            switch (alt924)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2861:2: text_or_identifier ( AT_SIGN_SYMBOL text_or_identifier | AT_TEXT_SUFFIX )?
        	    {
        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_user16070);
        	        text_or_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleuserEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2861:21: ( AT_SIGN_SYMBOL text_or_identifier | AT_TEXT_SUFFIX )?
        	        {
        	            int alt922=3;
        	            switch ( LA(1) )
        	            {
        	                case AT_SIGN_SYMBOL:
        	                	{
        	                		alt922=1;
        	                	}
        	                    break;
        	                case AT_TEXT_SUFFIX:
        	                	{
        	                		alt922=2;
        	                	}
        	                    break;
        	            }

        	            switch (alt922)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2861:22: AT_SIGN_SYMBOL text_or_identifier
        	        	    {
        	        	         MATCHT(AT_SIGN_SYMBOL, &FOLLOW_AT_SIGN_SYMBOL_in_user16073);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleuserEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	        FOLLOWPUSH(FOLLOW_text_or_identifier_in_user16075);
        	        	        text_or_identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleuserEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;
        	        	case 2:
        	        	    // MySQLSimpleParser.g:2861:58: AT_TEXT_SUFFIX
        	        	    {
        	        	         MATCHT(AT_TEXT_SUFFIX, &FOLLOW_AT_TEXT_SUFFIX_in_user16079);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleuserEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2862:4: CURRENT_USER_SYMBOL ( parentheses )?
        	    {
        	         MATCHT(CURRENT_USER_SYMBOL, &FOLLOW_CURRENT_USER_SYMBOL_in_user16086);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleuserEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2862:24: ( parentheses )?
        	        {
        	            int alt923=2;
        	            switch ( LA(1) )
        	            {
        	                case OPEN_PAR_SYMBOL:
        	                	{
        	                		alt923=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt923)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2862:24: parentheses
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_parentheses_in_user16088);
        	        	        parentheses(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleuserEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleuserEx; /* Prevent compiler warnings */
    ruleuserEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end user */

/**
 * $ANTLR start like_clause
 * MySQLSimpleParser.g:2865:1: like_clause : LIKE_SYMBOL text_string ;
 */
static void
like_clause(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2865:12: ( LIKE_SYMBOL text_string )
        // MySQLSimpleParser.g:2866:2: LIKE_SYMBOL text_string
        {
             MATCHT(LIKE_SYMBOL, &FOLLOW_LIKE_SYMBOL_in_like_clause16098);
            if  (HASEXCEPTION())
            {
                goto rulelike_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_string_in_like_clause16100);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelike_clauseEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelike_clauseEx; /* Prevent compiler warnings */
    rulelike_clauseEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end like_clause */

/**
 * $ANTLR start like_or_where
 * MySQLSimpleParser.g:2869:1: like_or_where : ( like_clause | where_clause );
 */
static void
like_or_where(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2869:14: ( like_clause | where_clause )

            ANTLR3_UINT32 alt925;

            alt925=2;

            switch ( LA(1) )
            {
            case LIKE_SYMBOL:
            	{
            		alt925=1;
            	}
                break;
            case WHERE_SYMBOL:
            	{
            		alt925=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 925;
                EXCEPTION->state        = 0;


                goto rulelike_or_whereEx;

            }

            switch (alt925)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2870:2: like_clause
        	    {
        	        FOLLOWPUSH(FOLLOW_like_clause_in_like_or_where16109);
        	        like_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelike_or_whereEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2870:16: where_clause
        	    {
        	        FOLLOWPUSH(FOLLOW_where_clause_in_like_or_where16113);
        	        where_clause(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulelike_or_whereEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulelike_or_whereEx; /* Prevent compiler warnings */
    rulelike_or_whereEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end like_or_where */

/**
 * $ANTLR start online_option
 * MySQLSimpleParser.g:2873:1: online_option :{...}? ( ONLINE_SYMBOL | OFFLINE_SYMBOL ) ;
 */
static void
online_option(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2873:14: ({...}? ( ONLINE_SYMBOL | OFFLINE_SYMBOL ) )
        // MySQLSimpleParser.g:2874:2: {...}? ( ONLINE_SYMBOL | OFFLINE_SYMBOL )
        {
            if ( !((SERVER_VERSION < 50600)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION < 50600";
                    EXCEPTION->ruleName	 = (void *)"online_option";


            }

            if ( ((LA(1) >= OFFLINE_SYMBOL) && (LA(1) <= ONLINE_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleonline_optionEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleonline_optionEx; /* Prevent compiler warnings */
    ruleonline_optionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end online_option */

/**
 * $ANTLR start no_write_to_bin_log
 * MySQLSimpleParser.g:2877:1: no_write_to_bin_log : ({...}? => LOCAL_SYMBOL | NO_WRITE_TO_BINLOG_SYMBOL );
 */
static void
no_write_to_bin_log(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2877:20: ({...}? => LOCAL_SYMBOL | NO_WRITE_TO_BINLOG_SYMBOL )

            ANTLR3_UINT32 alt926;

            alt926=2;

            {
                int LA926_0 = LA(1);
                if ( ((LA(1) == LOCAL_SYMBOL)) && (LA926_0 == LOCAL_SYMBOL))
                {
                    alt926=1;
                }
                else if ( (LA926_0 == NO_WRITE_TO_BINLOG_SYMBOL))
                {
                    alt926=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 926;
                    EXCEPTION->state        = 0;


                    goto ruleno_write_to_bin_logEx;

                }
            }
            switch (alt926)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2878:2: {...}? => LOCAL_SYMBOL
        	    {
        	        if ( !((LA(1) == LOCAL_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == LOCAL_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"no_write_to_bin_log";


        	        }

        	         MATCHT(LOCAL_SYMBOL, &FOLLOW_LOCAL_SYMBOL_in_no_write_to_bin_log16143);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleno_write_to_bin_logEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2879:4: NO_WRITE_TO_BINLOG_SYMBOL
        	    {
        	         MATCHT(NO_WRITE_TO_BINLOG_SYMBOL, &FOLLOW_NO_WRITE_TO_BINLOG_SYMBOL_in_no_write_to_bin_log16149);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleno_write_to_bin_logEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleno_write_to_bin_logEx; /* Prevent compiler warnings */
    ruleno_write_to_bin_logEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end no_write_to_bin_log */

/**
 * $ANTLR start use_partition
 * MySQLSimpleParser.g:2882:1: use_partition :{...}? => PARTITION_SYMBOL identifier_list_with_parentheses ;
 */
static void
use_partition(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2882:14: ({...}? => PARTITION_SYMBOL identifier_list_with_parentheses )
        // MySQLSimpleParser.g:2883:2: {...}? => PARTITION_SYMBOL identifier_list_with_parentheses
        {
            if ( !((SERVER_VERSION >= 50602)) )
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
                    EXCEPTION->message      = (void *)"SERVER_VERSION >= 50602";
                    EXCEPTION->ruleName	 = (void *)"use_partition";


            }

             MATCHT(PARTITION_SYMBOL, &FOLLOW_PARTITION_SYMBOL_in_use_partition16162);
            if  (HASEXCEPTION())
            {
                goto ruleuse_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_list_with_parentheses_in_use_partition16164);
            identifier_list_with_parentheses(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleuse_partitionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleuse_partitionEx; /* Prevent compiler warnings */
    ruleuse_partitionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end use_partition */

/**
 * $ANTLR start column_name
 * MySQLSimpleParser.g:2894:1: column_name : identifier ;
 */
static void
column_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2894:12: ( identifier )
        // MySQLSimpleParser.g:2895:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_column_name16181);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_nameEx; /* Prevent compiler warnings */
    rulecolumn_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_name */

/**
 * $ANTLR start column_ref
 * MySQLSimpleParser.g:2898:1: column_ref : column_ref_variants ;
 */
static void
column_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2898:11: ( column_ref_variants )
        // MySQLSimpleParser.g:2899:2: column_ref_variants
        {
            FOLLOWPUSH(FOLLOW_column_ref_variants_in_column_ref16190);
            column_ref_variants(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_refEx; /* Prevent compiler warnings */
    rulecolumn_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_ref */

/**
 * $ANTLR start column_ref_variants
 * MySQLSimpleParser.g:2903:1: column_ref_variants : ( dot_identifier | qualified_identifier ( options {greedy=true; } : dot_identifier )? );
 */
static void
column_ref_variants(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:2903:20: ( dot_identifier | qualified_identifier ( options {greedy=true; } : dot_identifier )? )

            ANTLR3_UINT32 alt928;

            alt928=2;

            {
                int LA928_0 = LA(1);
                if ( (LA928_0 == DOT_SYMBOL))
                {
                    alt928=1;
                }
                else if ( (((LA928_0 >= ACCOUNT_SYMBOL) && (LA928_0 <= YEAR_SYMBOL)) || LA928_0 == BACK_TICK_QUOTED_ID || LA928_0 == DOUBLE_QUOTED_TEXT || LA928_0 == IDENTIFIER))
                {
                    alt928=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 928;
                    EXCEPTION->state        = 0;


                    goto rulecolumn_ref_variantsEx;

                }
            }
            switch (alt928)
            {
        	case 1:
        	    // MySQLSimpleParser.g:2904:2: dot_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_column_ref_variants16200);
        	        dot_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecolumn_ref_variantsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:2905:4: qualified_identifier ( options {greedy=true; } : dot_identifier )?
        	    {
        	        FOLLOWPUSH(FOLLOW_qualified_identifier_in_column_ref_variants16205);
        	        qualified_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulecolumn_ref_variantsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:2905:25: ( options {greedy=true; } : dot_identifier )?
        	        {
        	            int alt927=2;
        	            switch ( LA(1) )
        	            {
        	                case DOT_SYMBOL:
        	                	{
        	                		alt927=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt927)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:2905:54: dot_identifier
        	        	    {
        	        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_column_ref_variants16220);
        	        	        dot_identifier(ctx);

        	        	        FOLLOWPOP();
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulecolumn_ref_variantsEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_ref_variantsEx; /* Prevent compiler warnings */
    rulecolumn_ref_variantsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_ref_variants */

/**
 * $ANTLR start column_internal_ref
 * MySQLSimpleParser.g:2908:1: column_internal_ref : column_ref_variants ;
 */
static void
column_internal_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2908:20: ( column_ref_variants )
        // MySQLSimpleParser.g:2909:2: column_ref_variants
        {
            FOLLOWPUSH(FOLLOW_column_ref_variants_in_column_internal_ref16231);
            column_ref_variants(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_internal_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_internal_refEx; /* Prevent compiler warnings */
    rulecolumn_internal_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_internal_ref */

/**
 * $ANTLR start column_ref_with_wildcard
 * MySQLSimpleParser.g:2912:1: column_ref_with_wildcard : column_ref_with_wildcard2 ;
 */
static void
column_ref_with_wildcard(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2912:25: ( column_ref_with_wildcard2 )
        // MySQLSimpleParser.g:2913:2: column_ref_with_wildcard2
        {
            FOLLOWPUSH(FOLLOW_column_ref_with_wildcard2_in_column_ref_with_wildcard16240);
            column_ref_with_wildcard2(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_ref_with_wildcardEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_ref_with_wildcardEx; /* Prevent compiler warnings */
    rulecolumn_ref_with_wildcardEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_ref_with_wildcard */

/**
 * $ANTLR start column_ref_with_wildcard2
 * MySQLSimpleParser.g:2916:1: column_ref_with_wildcard2 : identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )? )? ;
 */
static void
column_ref_with_wildcard2(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2916:26: ( identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )? )? )
        // MySQLSimpleParser.g:2917:2: identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )? )?
        {
            FOLLOWPUSH(FOLLOW_identifier_in_column_ref_with_wildcard216249);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulecolumn_ref_with_wildcard2Ex;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2918:2: ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )? )?
            {
                int alt930=3;
                switch ( LA(1) )
                {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA930_1 = LA(2);
                    		    if ( (LA930_1 == MULT_OPERATOR))
                    		    {
                    		        alt930=1;
                    		    }
                    		    else if ( (((LA930_1 >= ACCOUNT_SYMBOL) && (LA930_1 <= YEAR_SYMBOL)) || LA930_1 == BACK_TICK_QUOTED_ID || LA930_1 == DOUBLE_QUOTED_TEXT || LA930_1 == IDENTIFIER))
                    		    {
                    		        alt930=2;
                    		    }
                    		}
                    	}
                        break;
                }

                switch (alt930)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2919:3: {...}? DOT_SYMBOL MULT_OPERATOR
            	    {
            	        if ( !((LA(2) ==  MULT_OPERATOR)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"LA(2) ==  MULT_OPERATOR";
            	                EXCEPTION->ruleName	 = (void *)"column_ref_with_wildcard2";


            	        }

            	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216258);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_ref_with_wildcard2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216260);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_ref_with_wildcard2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2920:5: dot_identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )?
            	    {
            	        FOLLOWPUSH(FOLLOW_dot_identifier_in_column_ref_with_wildcard216266);
            	        dot_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulecolumn_ref_with_wildcard2Ex;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:2921:3: ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier )?
            	        {
            	            int alt929=3;
            	            switch ( LA(1) )
            	            {
            	                case DOT_SYMBOL:
            	                	{
            	                		{
            	                		    int LA929_1 = LA(2);
            	                		    if ( (LA929_1 == MULT_OPERATOR))
            	                		    {
            	                		        alt929=1;
            	                		    }
            	                		    else if ( (((LA929_1 >= ACCOUNT_SYMBOL) && (LA929_1 <= YEAR_SYMBOL)) || LA929_1 == BACK_TICK_QUOTED_ID || LA929_1 == DOUBLE_QUOTED_TEXT || LA929_1 == IDENTIFIER))
            	                		    {
            	                		        alt929=2;
            	                		    }
            	                		}
            	                	}
            	                    break;
            	            }

            	            switch (alt929)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:2922:4: {...}? DOT_SYMBOL MULT_OPERATOR
            	        	    {
            	        	        if ( !((LA(2) ==  MULT_OPERATOR)) )
            	        	        {
            	        	            if (BACKTRACKING>0)
            	        	            {
            	        	                FAILEDFLAG = ANTLR3_TRUE;
            	        	                return ;
            	        	            }

            	        	                CONSTRUCTEX();
            	        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	        	                EXCEPTION->message      = (void *)"LA(2) ==  MULT_OPERATOR";
            	        	                EXCEPTION->ruleName	 = (void *)"column_ref_with_wildcard2";


            	        	        }

            	        	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_column_ref_with_wildcard216277);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_ref_with_wildcard2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_column_ref_with_wildcard216279);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_ref_with_wildcard2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;
            	        	case 2:
            	        	    // MySQLSimpleParser.g:2923:6: dot_identifier
            	        	    {
            	        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_column_ref_with_wildcard216286);
            	        	        dot_identifier(ctx);

            	        	        FOLLOWPOP();
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto rulecolumn_ref_with_wildcard2Ex;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulecolumn_ref_with_wildcard2Ex; /* Prevent compiler warnings */
    rulecolumn_ref_with_wildcard2Ex: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end column_ref_with_wildcard2 */

/**
 * $ANTLR start index_name
 * MySQLSimpleParser.g:2928:1: index_name : identifier ;
 */
static void
index_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2928:11: ( identifier )
        // MySQLSimpleParser.g:2929:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_index_name16304);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_nameEx; /* Prevent compiler warnings */
    ruleindex_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_name */

/**
 * $ANTLR start index_ref
 * MySQLSimpleParser.g:2932:1: index_ref : identifier ;
 */
static void
index_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2932:10: ( identifier )
        // MySQLSimpleParser.g:2933:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_index_ref16313);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleindex_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleindex_refEx; /* Prevent compiler warnings */
    ruleindex_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end index_ref */

/**
 * $ANTLR start table_wild
 * MySQLSimpleParser.g:2936:1: table_wild : identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier DOT_SYMBOL MULT_OPERATOR ) ;
 */
static void
table_wild(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2936:11: ( identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier DOT_SYMBOL MULT_OPERATOR ) )
        // MySQLSimpleParser.g:2937:2: identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier DOT_SYMBOL MULT_OPERATOR )
        {
            FOLLOWPUSH(FOLLOW_identifier_in_table_wild16322);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_wildEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:2938:2: ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier DOT_SYMBOL MULT_OPERATOR )
            {
                int alt931=2;
                switch ( LA(1) )
                {
                case DOT_SYMBOL:
                	{
                		{
                		    int LA931_1 = LA(2);
                		    if ( (LA931_1 == MULT_OPERATOR))
                		    {
                		        alt931=1;
                		    }
                		    else if ( (((LA931_1 >= ACCOUNT_SYMBOL) && (LA931_1 <= YEAR_SYMBOL)) || LA931_1 == BACK_TICK_QUOTED_ID || LA931_1 == DOUBLE_QUOTED_TEXT || LA931_1 == IDENTIFIER))
                		    {
                		        alt931=2;
                		    }
                		    else
                		    {
                		        if (BACKTRACKING>0)
                		        {
                		            FAILEDFLAG = ANTLR3_TRUE;
                		            return ;
                		        }


                		        CONSTRUCTEX();
                		        EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                		        EXCEPTION->message      = (void *)"";
                		        EXCEPTION->decisionNum  = 931;
                		        EXCEPTION->state        = 1;


                		        goto ruletable_wildEx;

                		    }
                		}
                	}
                    break;

                default:
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }

                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 931;
                    EXCEPTION->state        = 0;


                    goto ruletable_wildEx;

                }

                switch (alt931)
                {
            	case 1:
            	    // MySQLSimpleParser.g:2939:3: {...}? DOT_SYMBOL MULT_OPERATOR
            	    {
            	        if ( !((LA(2) ==  MULT_OPERATOR)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"LA(2) ==  MULT_OPERATOR";
            	                EXCEPTION->ruleName	 = (void *)"table_wild";


            	        }

            	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_table_wild16331);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_wildEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_table_wild16333);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_wildEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:2940:5: dot_identifier DOT_SYMBOL MULT_OPERATOR
            	    {
            	        FOLLOWPUSH(FOLLOW_dot_identifier_in_table_wild16339);
            	        dot_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_wildEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_table_wild16341);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_wildEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_table_wild16343);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_wildEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_wildEx; /* Prevent compiler warnings */
    ruletable_wildEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_wild */

/**
 * $ANTLR start schema_name
 * MySQLSimpleParser.g:2944:1: schema_name : identifier ;
 */
static void
schema_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2944:12: ( identifier )
        // MySQLSimpleParser.g:2945:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_schema_name16362);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleschema_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleschema_nameEx; /* Prevent compiler warnings */
    ruleschema_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end schema_name */

/**
 * $ANTLR start schema_ref
 * MySQLSimpleParser.g:2948:1: schema_ref : identifier ;
 */
static void
schema_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2948:11: ( identifier )
        // MySQLSimpleParser.g:2949:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_schema_ref16371);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleschema_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleschema_refEx; /* Prevent compiler warnings */
    ruleschema_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end schema_ref */

/**
 * $ANTLR start procedure_name
 * MySQLSimpleParser.g:2952:1: procedure_name : qualified_identifier ;
 */
static void
procedure_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2952:15: ( qualified_identifier )
        // MySQLSimpleParser.g:2953:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_procedure_name16380);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprocedure_nameEx; /* Prevent compiler warnings */
    ruleprocedure_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end procedure_name */

/**
 * $ANTLR start procedure_ref
 * MySQLSimpleParser.g:2956:1: procedure_ref : qualified_identifier ;
 */
static void
procedure_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2956:14: ( qualified_identifier )
        // MySQLSimpleParser.g:2957:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_procedure_ref16389);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleprocedure_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprocedure_refEx; /* Prevent compiler warnings */
    ruleprocedure_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end procedure_ref */

/**
 * $ANTLR start function_name
 * MySQLSimpleParser.g:2960:1: function_name : qualified_identifier ;
 */
static void
function_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2960:14: ( qualified_identifier )
        // MySQLSimpleParser.g:2961:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_function_name16398);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefunction_nameEx; /* Prevent compiler warnings */
    rulefunction_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_name */

/**
 * $ANTLR start function_ref
 * MySQLSimpleParser.g:2964:1: function_ref : qualified_identifier ;
 */
static void
function_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2964:13: ( qualified_identifier )
        // MySQLSimpleParser.g:2965:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_function_ref16407);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefunction_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefunction_refEx; /* Prevent compiler warnings */
    rulefunction_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end function_ref */

/**
 * $ANTLR start trigger_name
 * MySQLSimpleParser.g:2968:1: trigger_name : qualified_identifier ;
 */
static void
trigger_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2968:13: ( qualified_identifier )
        // MySQLSimpleParser.g:2969:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_trigger_name16416);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletrigger_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletrigger_nameEx; /* Prevent compiler warnings */
    ruletrigger_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end trigger_name */

/**
 * $ANTLR start trigger_ref
 * MySQLSimpleParser.g:2972:1: trigger_ref : qualified_identifier ;
 */
static void
trigger_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2972:12: ( qualified_identifier )
        // MySQLSimpleParser.g:2973:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_trigger_ref16425);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletrigger_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletrigger_refEx; /* Prevent compiler warnings */
    ruletrigger_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end trigger_ref */

/**
 * $ANTLR start view_name
 * MySQLSimpleParser.g:2976:1: view_name : table_name_variants ;
 */
static void
view_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2976:10: ( table_name_variants )
        // MySQLSimpleParser.g:2977:2: table_name_variants
        {
            FOLLOWPUSH(FOLLOW_table_name_variants_in_view_name16434);
            table_name_variants(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleview_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleview_nameEx; /* Prevent compiler warnings */
    ruleview_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end view_name */

/**
 * $ANTLR start view_ref
 * MySQLSimpleParser.g:2980:1: view_ref : qualified_identifier ;
 */
static void
view_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2980:9: ( qualified_identifier )
        // MySQLSimpleParser.g:2981:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_view_ref16443);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleview_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleview_refEx; /* Prevent compiler warnings */
    ruleview_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end view_ref */

/**
 * $ANTLR start tablespace_name
 * MySQLSimpleParser.g:2984:1: tablespace_name : identifier ;
 */
static void
tablespace_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2984:16: ( identifier )
        // MySQLSimpleParser.g:2985:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_tablespace_name16452);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletablespace_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletablespace_nameEx; /* Prevent compiler warnings */
    ruletablespace_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end tablespace_name */

/**
 * $ANTLR start tablespace_ref
 * MySQLSimpleParser.g:2988:1: tablespace_ref : identifier ;
 */
static void
tablespace_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2988:15: ( identifier )
        // MySQLSimpleParser.g:2989:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_tablespace_ref16461);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletablespace_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletablespace_refEx; /* Prevent compiler warnings */
    ruletablespace_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end tablespace_ref */

/**
 * $ANTLR start logfile_group_name
 * MySQLSimpleParser.g:2992:1: logfile_group_name : identifier ;
 */
static void
logfile_group_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2992:19: ( identifier )
        // MySQLSimpleParser.g:2993:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_logfile_group_name16470);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelogfile_group_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogfile_group_nameEx; /* Prevent compiler warnings */
    rulelogfile_group_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logfile_group_name */

/**
 * $ANTLR start logfile_group_ref
 * MySQLSimpleParser.g:2996:1: logfile_group_ref : identifier ;
 */
static void
logfile_group_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:2996:18: ( identifier )
        // MySQLSimpleParser.g:2997:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_logfile_group_ref16479);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulelogfile_group_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulelogfile_group_refEx; /* Prevent compiler warnings */
    rulelogfile_group_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end logfile_group_ref */

/**
 * $ANTLR start event_name
 * MySQLSimpleParser.g:3000:1: event_name : qualified_identifier ;
 */
static void
event_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3000:11: ( qualified_identifier )
        // MySQLSimpleParser.g:3001:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_event_name16488);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleevent_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleevent_nameEx; /* Prevent compiler warnings */
    ruleevent_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end event_name */

/**
 * $ANTLR start event_ref
 * MySQLSimpleParser.g:3004:1: event_ref : qualified_identifier ;
 */
static void
event_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3004:10: ( qualified_identifier )
        // MySQLSimpleParser.g:3005:2: qualified_identifier
        {
            FOLLOWPUSH(FOLLOW_qualified_identifier_in_event_ref16497);
            qualified_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleevent_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleevent_refEx; /* Prevent compiler warnings */
    ruleevent_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end event_ref */

/**
 * $ANTLR start udf_name
 * MySQLSimpleParser.g:3008:1: udf_name : identifier ;
 */
static void
udf_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3008:9: ( identifier )
        // MySQLSimpleParser.g:3009:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_udf_name16507);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleudf_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleudf_nameEx; /* Prevent compiler warnings */
    ruleudf_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end udf_name */

/**
 * $ANTLR start server_name
 * MySQLSimpleParser.g:3012:1: server_name : text_or_identifier ;
 */
static void
server_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3012:12: ( text_or_identifier )
        // MySQLSimpleParser.g:3013:2: text_or_identifier
        {
            FOLLOWPUSH(FOLLOW_text_or_identifier_in_server_name16516);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleserver_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleserver_nameEx; /* Prevent compiler warnings */
    ruleserver_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end server_name */

/**
 * $ANTLR start server_ref
 * MySQLSimpleParser.g:3016:1: server_ref : text_or_identifier ;
 */
static void
server_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3016:11: ( text_or_identifier )
        // MySQLSimpleParser.g:3017:2: text_or_identifier
        {
            FOLLOWPUSH(FOLLOW_text_or_identifier_in_server_ref16525);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleserver_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleserver_refEx; /* Prevent compiler warnings */
    ruleserver_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end server_ref */

/**
 * $ANTLR start engine_ref
 * MySQLSimpleParser.g:3020:1: engine_ref : text_or_identifier ;
 */
static void
engine_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3020:11: ( text_or_identifier )
        // MySQLSimpleParser.g:3021:2: text_or_identifier
        {
            FOLLOWPUSH(FOLLOW_text_or_identifier_in_engine_ref16534);
            text_or_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleengine_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleengine_refEx; /* Prevent compiler warnings */
    ruleengine_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end engine_ref */

/**
 * $ANTLR start table_name
 * MySQLSimpleParser.g:3024:1: table_name : table_name_variants ;
 */
static void
table_name(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3024:11: ( table_name_variants )
        // MySQLSimpleParser.g:3025:2: table_name_variants
        {
            FOLLOWPUSH(FOLLOW_table_name_variants_in_table_name16543);
            table_name_variants(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_nameEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_nameEx; /* Prevent compiler warnings */
    ruletable_nameEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_name */

/**
 * $ANTLR start filter_table_ref
 * MySQLSimpleParser.g:3028:1: filter_table_ref : identifier dot_identifier ;
 */
static void
filter_table_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3028:17: ( identifier dot_identifier )
        // MySQLSimpleParser.g:3029:2: identifier dot_identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_filter_table_ref16553);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_table_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_dot_identifier_in_filter_table_ref16555);
            dot_identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulefilter_table_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefilter_table_refEx; /* Prevent compiler warnings */
    rulefilter_table_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end filter_table_ref */

/**
 * $ANTLR start table_ref_with_wildcard
 * MySQLSimpleParser.g:3032:1: table_ref_with_wildcard : identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ( DOT_SYMBOL MULT_OPERATOR )? )? ;
 */
static void
table_ref_with_wildcard(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3032:24: ( identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ( DOT_SYMBOL MULT_OPERATOR )? )? )
        // MySQLSimpleParser.g:3033:2: identifier ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ( DOT_SYMBOL MULT_OPERATOR )? )?
        {
            FOLLOWPUSH(FOLLOW_identifier_in_table_ref_with_wildcard16564);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_ref_with_wildcardEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3034:2: ({...}? DOT_SYMBOL MULT_OPERATOR | dot_identifier ( DOT_SYMBOL MULT_OPERATOR )? )?
            {
                int alt933=3;
                switch ( LA(1) )
                {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA933_1 = LA(2);
                    		    if ( (LA933_1 == MULT_OPERATOR))
                    		    {
                    		        alt933=1;
                    		    }
                    		    else if ( (((LA933_1 >= ACCOUNT_SYMBOL) && (LA933_1 <= YEAR_SYMBOL)) || LA933_1 == BACK_TICK_QUOTED_ID || LA933_1 == DOUBLE_QUOTED_TEXT || LA933_1 == IDENTIFIER))
                    		    {
                    		        alt933=2;
                    		    }
                    		}
                    	}
                        break;
                }

                switch (alt933)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3035:3: {...}? DOT_SYMBOL MULT_OPERATOR
            	    {
            	        if ( !((LA(2) == MULT_OPERATOR)) )
            	        {
            	            if (BACKTRACKING>0)
            	            {
            	                FAILEDFLAG = ANTLR3_TRUE;
            	                return ;
            	            }

            	                CONSTRUCTEX();
            	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
            	                EXCEPTION->message      = (void *)"LA(2) == MULT_OPERATOR";
            	                EXCEPTION->ruleName	 = (void *)"table_ref_with_wildcard";


            	        }

            	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16573);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_with_wildcardEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16575);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_with_wildcardEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;
            	case 2:
            	    // MySQLSimpleParser.g:3036:5: dot_identifier ( DOT_SYMBOL MULT_OPERATOR )?
            	    {
            	        FOLLOWPUSH(FOLLOW_dot_identifier_in_table_ref_with_wildcard16581);
            	        dot_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_with_wildcardEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        // MySQLSimpleParser.g:3036:20: ( DOT_SYMBOL MULT_OPERATOR )?
            	        {
            	            int alt932=2;
            	            switch ( LA(1) )
            	            {
            	                case DOT_SYMBOL:
            	                	{
            	                		alt932=1;
            	                	}
            	                    break;
            	            }

            	            switch (alt932)
            	            {
            	        	case 1:
            	        	    // MySQLSimpleParser.g:3036:21: DOT_SYMBOL MULT_OPERATOR
            	        	    {
            	        	         MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_table_ref_with_wildcard16584);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletable_ref_with_wildcardEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	         MATCHT(MULT_OPERATOR, &FOLLOW_MULT_OPERATOR_in_table_ref_with_wildcard16586);
            	        	        if  (HASEXCEPTION())
            	        	        {
            	        	            goto ruletable_ref_with_wildcardEx;
            	        	        }
            	        	        if (HASFAILED())
            	        	        {
            	        	            return ;
            	        	        }


            	        	    }
            	        	    break;

            	            }
            	        }

            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_ref_with_wildcardEx; /* Prevent compiler warnings */
    ruletable_ref_with_wildcardEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_ref_with_wildcard */

/**
 * $ANTLR start table_ref
 * MySQLSimpleParser.g:3040:1: table_ref : table_name_variants ;
 */
static void
table_ref(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3040:10: ( table_name_variants )
        // MySQLSimpleParser.g:3041:2: table_name_variants
        {
            FOLLOWPUSH(FOLLOW_table_name_variants_in_table_ref16601);
            table_name_variants(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_refEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_refEx; /* Prevent compiler warnings */
    ruletable_refEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_ref */

/**
 * $ANTLR start table_ref_no_db
 * MySQLSimpleParser.g:3044:1: table_ref_no_db : identifier ;
 */
static void
table_ref_no_db(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3044:16: ( identifier )
        // MySQLSimpleParser.g:3045:2: identifier
        {
            FOLLOWPUSH(FOLLOW_identifier_in_table_ref_no_db16610);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_ref_no_dbEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_ref_no_dbEx; /* Prevent compiler warnings */
    ruletable_ref_no_dbEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_ref_no_db */

/**
 * $ANTLR start table_name_variants
 * MySQLSimpleParser.g:3048:1: table_name_variants : ( qualified_identifier | dot_identifier );
 */
static void
table_name_variants(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3048:20: ( qualified_identifier | dot_identifier )

            ANTLR3_UINT32 alt934;

            alt934=2;

            {
                int LA934_0 = LA(1);
                if ( (((LA934_0 >= ACCOUNT_SYMBOL) && (LA934_0 <= YEAR_SYMBOL)) || LA934_0 == BACK_TICK_QUOTED_ID || LA934_0 == DOUBLE_QUOTED_TEXT || LA934_0 == IDENTIFIER))
                {
                    alt934=1;
                }
                else if ( (LA934_0 == DOT_SYMBOL))
                {
                    alt934=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 934;
                    EXCEPTION->state        = 0;


                    goto ruletable_name_variantsEx;

                }
            }
            switch (alt934)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3049:2: qualified_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_qualified_identifier_in_table_name_variants16619);
        	        qualified_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_name_variantsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3050:4: dot_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_dot_identifier_in_table_name_variants16624);
        	        dot_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletable_name_variantsEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletable_name_variantsEx; /* Prevent compiler warnings */
    ruletable_name_variantsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_name_variants */

/**
 * $ANTLR start table_ref_list
 * MySQLSimpleParser.g:3053:1: table_ref_list : table_ref ( COMMA_SYMBOL table_ref )* ;
 */
static void
table_ref_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3053:15: ( table_ref ( COMMA_SYMBOL table_ref )* )
        // MySQLSimpleParser.g:3054:2: table_ref ( COMMA_SYMBOL table_ref )*
        {
            FOLLOWPUSH(FOLLOW_table_ref_in_table_ref_list16633);
            table_ref(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_ref_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3054:12: ( COMMA_SYMBOL table_ref )*

            for (;;)
            {
                int alt935=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt935=1;
                	}
                    break;

                }

                switch (alt935)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3054:13: COMMA_SYMBOL table_ref
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_table_ref_list16636);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_in_table_ref_list16638);
            	        table_ref(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop935;	/* break out of the loop */
            	    break;
                }
            }
            loop935: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_ref_listEx; /* Prevent compiler warnings */
    ruletable_ref_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_ref_list */

/**
 * $ANTLR start table_ref_list_with_wildcard
 * MySQLSimpleParser.g:3057:1: table_ref_list_with_wildcard : table_ref_with_wildcard ( COMMA_SYMBOL table_ref_with_wildcard )* ;
 */
static void
table_ref_list_with_wildcard(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3057:29: ( table_ref_with_wildcard ( COMMA_SYMBOL table_ref_with_wildcard )* )
        // MySQLSimpleParser.g:3058:2: table_ref_with_wildcard ( COMMA_SYMBOL table_ref_with_wildcard )*
        {
            FOLLOWPUSH(FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16656);
            table_ref_with_wildcard(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletable_ref_list_with_wildcardEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3058:26: ( COMMA_SYMBOL table_ref_with_wildcard )*

            for (;;)
            {
                int alt936=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt936=1;
                	}
                    break;

                }

                switch (alt936)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3058:27: COMMA_SYMBOL table_ref_with_wildcard
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_table_ref_list_with_wildcard16659);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_list_with_wildcardEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_table_ref_with_wildcard_in_table_ref_list_with_wildcard16661);
            	        table_ref_with_wildcard(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruletable_ref_list_with_wildcardEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop936;	/* break out of the loop */
            	    break;
                }
            }
            loop936: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletable_ref_list_with_wildcardEx; /* Prevent compiler warnings */
    ruletable_ref_list_with_wildcardEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end table_ref_list_with_wildcard */

/**
 * $ANTLR start pure_identifier
 * MySQLSimpleParser.g:3064:1: pure_identifier : ( IDENTIFIER | BACK_TICK_QUOTED_ID |{...}? DOUBLE_QUOTED_TEXT );
 */
static void
pure_identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3064:16: ( IDENTIFIER | BACK_TICK_QUOTED_ID |{...}? DOUBLE_QUOTED_TEXT )

            ANTLR3_UINT32 alt937;

            alt937=3;

            switch ( LA(1) )
            {
            case IDENTIFIER:
            	{
            		alt937=1;
            	}
                break;
            case BACK_TICK_QUOTED_ID:
            	{
            		alt937=2;
            	}
                break;
            case DOUBLE_QUOTED_TEXT:
            	{
            		alt937=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 937;
                EXCEPTION->state        = 0;


                goto rulepure_identifierEx;

            }

            switch (alt937)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3065:2: IDENTIFIER
        	    {
        	         MATCHT(IDENTIFIER, &FOLLOW_IDENTIFIER_in_pure_identifier16675);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepure_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3066:4: BACK_TICK_QUOTED_ID
        	    {
        	         MATCHT(BACK_TICK_QUOTED_ID, &FOLLOW_BACK_TICK_QUOTED_ID_in_pure_identifier16680);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepure_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:3067:4: {...}? DOUBLE_QUOTED_TEXT
        	    {
        	        if ( !((SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)";
        	                EXCEPTION->ruleName	 = (void *)"pure_identifier";


        	        }

        	         MATCHT(DOUBLE_QUOTED_TEXT, &FOLLOW_DOUBLE_QUOTED_TEXT_in_pure_identifier16687);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulepure_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulepure_identifierEx; /* Prevent compiler warnings */
    rulepure_identifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end pure_identifier */

/**
 * $ANTLR start identifier
 * MySQLSimpleParser.g:3071:1: identifier : ( pure_identifier | keyword );
 */
static void
identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3071:11: ( pure_identifier | keyword )

            ANTLR3_UINT32 alt938;

            alt938=2;

            {
                int LA938_0 = LA(1);
                if ( (LA938_0 == BACK_TICK_QUOTED_ID || LA938_0 == DOUBLE_QUOTED_TEXT || LA938_0 == IDENTIFIER))
                {
                    alt938=1;
                }
                else if ( (((LA938_0 >= ACCOUNT_SYMBOL) && (LA938_0 <= YEAR_SYMBOL))))
                {
                    alt938=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 938;
                    EXCEPTION->state        = 0;


                    goto ruleidentifierEx;

                }
            }
            switch (alt938)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3072:2: pure_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_pure_identifier_in_identifier16697);
        	        pure_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleidentifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3073:4: keyword
        	    {
        	        FOLLOWPUSH(FOLLOW_keyword_in_identifier16702);
        	        keyword(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleidentifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleidentifierEx; /* Prevent compiler warnings */
    ruleidentifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end identifier */

/**
 * $ANTLR start identifier_list
 * MySQLSimpleParser.g:3076:1: identifier_list : identifier ( options {greedy=true; } : COMMA_SYMBOL identifier )* ;
 */
static void
identifier_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3076:16: ( identifier ( options {greedy=true; } : COMMA_SYMBOL identifier )* )
        // MySQLSimpleParser.g:3077:2: identifier ( options {greedy=true; } : COMMA_SYMBOL identifier )*
        {
            FOLLOWPUSH(FOLLOW_identifier_in_identifier_list16711);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleidentifier_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3077:13: ( options {greedy=true; } : COMMA_SYMBOL identifier )*

            for (;;)
            {
                int alt939=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt939=1;
                	}
                    break;

                }

                switch (alt939)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3077:43: COMMA_SYMBOL identifier
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_identifier_list16727);
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleidentifier_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_identifier_in_identifier_list16729);
            	        identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto ruleidentifier_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop939;	/* break out of the loop */
            	    break;
                }
            }
            loop939: ; /* Jump out to here if this rule does not match */


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleidentifier_listEx; /* Prevent compiler warnings */
    ruleidentifier_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end identifier_list */

/**
 * $ANTLR start identifier_list_with_parentheses
 * MySQLSimpleParser.g:3080:1: identifier_list_with_parentheses : OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL ;
 */
static void
identifier_list_with_parentheses(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3080:33: ( OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:3081:2: OPEN_PAR_SYMBOL identifier_list CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_identifier_list_with_parentheses16740);
            if  (HASEXCEPTION())
            {
                goto ruleidentifier_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_list_in_identifier_list_with_parentheses16742);
            identifier_list(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruleidentifier_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_identifier_list_with_parentheses16744);
            if  (HASEXCEPTION())
            {
                goto ruleidentifier_list_with_parenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleidentifier_list_with_parenthesesEx; /* Prevent compiler warnings */
    ruleidentifier_list_with_parenthesesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end identifier_list_with_parentheses */

/**
 * $ANTLR start qualified_identifier
 * MySQLSimpleParser.g:3084:1: qualified_identifier : identifier ( options {greedy=true; } : dot_identifier )? ;
 */
static void
qualified_identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3084:21: ( identifier ( options {greedy=true; } : dot_identifier )? )
        // MySQLSimpleParser.g:3085:2: identifier ( options {greedy=true; } : dot_identifier )?
        {
            FOLLOWPUSH(FOLLOW_identifier_in_qualified_identifier16753);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulequalified_identifierEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3085:13: ( options {greedy=true; } : dot_identifier )?
            {
                int alt940=2;
                switch ( LA(1) )
                {
                    case DOT_SYMBOL:
                    	{
                    		{
                    		    int LA940_1 = LA(2);
                    		    if ( (LA940_1 == IDENTIFIER))
                    		    {
                    		        alt940=1;
                    		    }
                    		    else if ( (LA940_1 == BACK_TICK_QUOTED_ID))
                    		    {
                    		        alt940=1;
                    		    }
                    		    else if ( (LA940_1 == DOUBLE_QUOTED_TEXT))
                    		    {
                    		        alt940=1;
                    		    }
                    		    else if ( (((LA940_1 >= ACTION_SYMBOL) && (LA940_1 <= SHARE_SYMBOL)) || ((LA940_1 >= SLOW_SYMBOL) && (LA940_1 <= YEAR_SYMBOL))))
                    		    {
                    		        alt940=1;
                    		    }
                    		    else if ( (((LA940_1 >= ACCOUNT_SYMBOL) && (LA940_1 <= XA_SYMBOL)) || LA940_1 == SHUTDOWN_SYMBOL))
                    		    {
                    		        alt940=1;
                    		    }
                    		}
                    	}
                        break;
                }

                switch (alt940)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3085:42: dot_identifier
            	    {
            	        FOLLOWPUSH(FOLLOW_dot_identifier_in_qualified_identifier16768);
            	        dot_identifier(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulequalified_identifierEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

        }

    }

    // This is where rules clean up and exit
    //
    goto rulequalified_identifierEx; /* Prevent compiler warnings */
    rulequalified_identifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end qualified_identifier */

/**
 * $ANTLR start dot_identifier
 * MySQLSimpleParser.g:3090:1: dot_identifier : DOT_SYMBOL identifier ;
 */
static void
dot_identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3090:15: ( DOT_SYMBOL identifier )
        // MySQLSimpleParser.g:3091:2: DOT_SYMBOL identifier
        {
             MATCHT(DOT_SYMBOL, &FOLLOW_DOT_SYMBOL_in_dot_identifier16781);
            if  (HASEXCEPTION())
            {
                goto ruledot_identifierEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_identifier_in_dot_identifier16783);
            identifier(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruledot_identifierEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruledot_identifierEx; /* Prevent compiler warnings */
    ruledot_identifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end dot_identifier */

/**
 * $ANTLR start ulong_number
 * MySQLSimpleParser.g:3095:1: ulong_number : ( INT_NUMBER | HEX_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER );
 */
static void
ulong_number(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3095:13: ( INT_NUMBER | HEX_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == DECIMAL_NUMBER || LA(1) == FLOAT_NUMBER || LA(1) == HEX_NUMBER || LA(1) == INT_NUMBER || LA(1) == LONG_NUMBER || LA(1) == ULONGLONG_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleulong_numberEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleulong_numberEx; /* Prevent compiler warnings */
    ruleulong_numberEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end ulong_number */

/**
 * $ANTLR start real_ulong_number
 * MySQLSimpleParser.g:3104:1: real_ulong_number : ( INT_NUMBER | HEX_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER );
 */
static void
real_ulong_number(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3104:18: ( INT_NUMBER | HEX_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == HEX_NUMBER || LA(1) == INT_NUMBER || LA(1) == LONG_NUMBER || LA(1) == ULONGLONG_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulereal_ulong_numberEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulereal_ulong_numberEx; /* Prevent compiler warnings */
    rulereal_ulong_numberEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end real_ulong_number */

/**
 * $ANTLR start ulonglong_number
 * MySQLSimpleParser.g:3111:1: ulonglong_number : ( INT_NUMBER | ULONGLONG_NUMBER | LONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER );
 */
static void
ulonglong_number(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3111:17: ( INT_NUMBER | ULONGLONG_NUMBER | LONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == DECIMAL_NUMBER || LA(1) == FLOAT_NUMBER || LA(1) == INT_NUMBER || LA(1) == LONG_NUMBER || LA(1) == ULONGLONG_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleulonglong_numberEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleulonglong_numberEx; /* Prevent compiler warnings */
    ruleulonglong_numberEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end ulonglong_number */

/**
 * $ANTLR start real_ulonglong_number
 * MySQLSimpleParser.g:3119:1: real_ulonglong_number : ( INT_NUMBER | ULONGLONG_NUMBER | LONG_NUMBER );
 */
static void
real_ulonglong_number(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3119:22: ( INT_NUMBER | ULONGLONG_NUMBER | LONG_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == INT_NUMBER || LA(1) == LONG_NUMBER || LA(1) == ULONGLONG_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulereal_ulonglong_numberEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulereal_ulonglong_numberEx; /* Prevent compiler warnings */
    rulereal_ulonglong_numberEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end real_ulonglong_number */

/**
 * $ANTLR start string_list
 * MySQLSimpleParser.g:3125:1: string_list : OPEN_PAR_SYMBOL text_string ( COMMA_SYMBOL text_string )* CLOSE_PAR_SYMBOL ;
 */
static void
string_list(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3125:12: ( OPEN_PAR_SYMBOL text_string ( COMMA_SYMBOL text_string )* CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:3126:2: OPEN_PAR_SYMBOL text_string ( COMMA_SYMBOL text_string )* CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_string_list16901);
            if  (HASEXCEPTION())
            {
                goto rulestring_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            FOLLOWPUSH(FOLLOW_text_string_in_string_list16903);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulestring_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3126:30: ( COMMA_SYMBOL text_string )*

            for (;;)
            {
                int alt941=2;
                switch ( LA(1) )
                {
                case COMMA_SYMBOL:
                	{
                		alt941=1;
                	}
                    break;

                }

                switch (alt941)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3126:31: COMMA_SYMBOL text_string
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_string_list16906);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulestring_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	        FOLLOWPUSH(FOLLOW_text_string_in_string_list16908);
            	        text_string(ctx);

            	        FOLLOWPOP();
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulestring_listEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

            	default:
            	    goto loop941;	/* break out of the loop */
            	    break;
                }
            }
            loop941: ; /* Jump out to here if this rule does not match */


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_string_list16912);
            if  (HASEXCEPTION())
            {
                goto rulestring_listEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulestring_listEx; /* Prevent compiler warnings */
    rulestring_listEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end string_list */

/**
 * $ANTLR start text_string
 * MySQLSimpleParser.g:3129:1: text_string : ( SINGLE_QUOTED_TEXT | HEX_NUMBER | BIN_NUMBER );
 */
static void
text_string(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3129:12: ( SINGLE_QUOTED_TEXT | HEX_NUMBER | BIN_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == SINGLE_QUOTED_TEXT || LA(1) == BIN_NUMBER || LA(1) == HEX_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruletext_stringEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletext_stringEx; /* Prevent compiler warnings */
    ruletext_stringEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end text_string */

/**
 * $ANTLR start text_string_no_linebreak
 * MySQLSimpleParser.g:3137:1: text_string_no_linebreak : text_string ;
 */
static void
text_string_no_linebreak(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3137:25: ( text_string )
        // MySQLSimpleParser.g:3138:2: text_string
        {
            FOLLOWPUSH(FOLLOW_text_string_in_text_string_no_linebreak16942);
            text_string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto ruletext_string_no_linebreakEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruletext_string_no_linebreakEx; /* Prevent compiler warnings */
    ruletext_string_no_linebreakEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end text_string_no_linebreak */

/**
 * $ANTLR start literal
 * MySQLSimpleParser.g:3141:1: literal : ( string_literal | num_literal |{...}? temporal_literal | null_literal | bool_literal | ( UNDERSCORE_CHARSET )? HEX_NUMBER |{...}? ( UNDERSCORE_CHARSET )? BIN_NUMBER );
 */
static void
literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3141:8: ( string_literal | num_literal |{...}? temporal_literal | null_literal | bool_literal | ( UNDERSCORE_CHARSET )? HEX_NUMBER |{...}? ( UNDERSCORE_CHARSET )? BIN_NUMBER )

            ANTLR3_UINT32 alt944;

            alt944=7;

            switch ( LA(1) )
            {
            case NCHAR_TEXT:
            case SINGLE_QUOTED_TEXT:
            case DOUBLE_QUOTED_TEXT:
            	{
            		alt944=1;
            	}
                break;
            case UNDERSCORE_CHARSET:
            	{
            		switch ( LA(2) )
            		{
            		case SINGLE_QUOTED_TEXT:
            		case DOUBLE_QUOTED_TEXT:
            			{
            				alt944=1;
            			}
            		    break;
            		case HEX_NUMBER:
            			{
            				alt944=6;
            			}
            		    break;
            		case BIN_NUMBER:
            			{
            				alt944=7;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 944;
            		    EXCEPTION->state        = 2;


            		    goto ruleliteralEx;

            		}

            	}
                break;
            case DECIMAL_NUMBER:
            case FLOAT_NUMBER:
            case INT_NUMBER:
            case LONG_NUMBER:
            case ULONGLONG_NUMBER:
            	{
            		alt944=2;
            	}
                break;
            case DATE_SYMBOL:
            case TIMESTAMP_SYMBOL:
            case TIME_SYMBOL:
            	{
            		alt944=3;
            	}
                break;
            case NULL2_SYMBOL:
            case NULL_SYMBOL:
            	{
            		alt944=4;
            	}
                break;
            case FALSE_SYMBOL:
            case TRUE_SYMBOL:
            	{
            		alt944=5;
            	}
                break;
            case HEX_NUMBER:
            	{
            		alt944=6;
            	}
                break;
            case BIN_NUMBER:
            	{
            		alt944=7;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 944;
                EXCEPTION->state        = 0;


                goto ruleliteralEx;

            }

            switch (alt944)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3142:2: string_literal
        	    {
        	        FOLLOWPUSH(FOLLOW_string_literal_in_literal16951);
        	        string_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3143:4: num_literal
        	    {
        	        FOLLOWPUSH(FOLLOW_num_literal_in_literal16956);
        	        num_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:3145:4: {...}? temporal_literal
        	    {
        	        if ( !((LA(1) == DATE_SYMBOL || LA(1) == TIME_SYMBOL || LA(1) == TIMESTAMP_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(1) == DATE_SYMBOL || LA(1) == TIME_SYMBOL || LA(1) == TIMESTAMP_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"literal";


        	        }

        	        FOLLOWPUSH(FOLLOW_temporal_literal_in_literal16968);
        	        temporal_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:3146:4: null_literal
        	    {
        	        FOLLOWPUSH(FOLLOW_null_literal_in_literal16973);
        	        null_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:3147:4: bool_literal
        	    {
        	        FOLLOWPUSH(FOLLOW_bool_literal_in_literal16978);
        	        bool_literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 6:
        	    // MySQLSimpleParser.g:3148:4: ( UNDERSCORE_CHARSET )? HEX_NUMBER
        	    {
        	        // MySQLSimpleParser.g:3148:4: ( UNDERSCORE_CHARSET )?
        	        {
        	            int alt942=2;
        	            switch ( LA(1) )
        	            {
        	                case UNDERSCORE_CHARSET:
        	                	{
        	                		alt942=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt942)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:3148:4: UNDERSCORE_CHARSET
        	        	    {
        	        	         MATCHT(UNDERSCORE_CHARSET, &FOLLOW_UNDERSCORE_CHARSET_in_literal16983);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleliteralEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(HEX_NUMBER, &FOLLOW_HEX_NUMBER_in_literal16986);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 7:
        	    // MySQLSimpleParser.g:3149:4: {...}? ( UNDERSCORE_CHARSET )? BIN_NUMBER
        	    {
        	        if ( !((SERVER_VERSION >= 50003)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"SERVER_VERSION >= 50003";
        	                EXCEPTION->ruleName	 = (void *)"literal";


        	        }

        	        // MySQLSimpleParser.g:3149:31: ( UNDERSCORE_CHARSET )?
        	        {
        	            int alt943=2;
        	            switch ( LA(1) )
        	            {
        	                case UNDERSCORE_CHARSET:
        	                	{
        	                		alt943=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt943)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:3149:31: UNDERSCORE_CHARSET
        	        	    {
        	        	         MATCHT(UNDERSCORE_CHARSET, &FOLLOW_UNDERSCORE_CHARSET_in_literal16993);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto ruleliteralEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	         MATCHT(BIN_NUMBER, &FOLLOW_BIN_NUMBER_in_literal16996);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruleliteralEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruleliteralEx; /* Prevent compiler warnings */
    ruleliteralEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end literal */

/**
 * $ANTLR start signed_literal
 * MySQLSimpleParser.g:3152:1: signed_literal : ( literal | PLUS_OPERATOR ulong_number | MINUS_OPERATOR ulong_number );
 */
static void
signed_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3152:15: ( literal | PLUS_OPERATOR ulong_number | MINUS_OPERATOR ulong_number )

            ANTLR3_UINT32 alt945;

            alt945=3;

            switch ( LA(1) )
            {
            case DATE_SYMBOL:
            case TIMESTAMP_SYMBOL:
            case TIME_SYMBOL:
            case FALSE_SYMBOL:
            case NCHAR_TEXT:
            case NULL2_SYMBOL:
            case NULL_SYMBOL:
            case SINGLE_QUOTED_TEXT:
            case TRUE_SYMBOL:
            case BIN_NUMBER:
            case DECIMAL_NUMBER:
            case DOUBLE_QUOTED_TEXT:
            case FLOAT_NUMBER:
            case HEX_NUMBER:
            case INT_NUMBER:
            case LONG_NUMBER:
            case ULONGLONG_NUMBER:
            case UNDERSCORE_CHARSET:
            	{
            		alt945=1;
            	}
                break;
            case PLUS_OPERATOR:
            	{
            		alt945=2;
            	}
                break;
            case MINUS_OPERATOR:
            	{
            		alt945=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 945;
                EXCEPTION->state        = 0;


                goto rulesigned_literalEx;

            }

            switch (alt945)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3153:2: literal
        	    {
        	        FOLLOWPUSH(FOLLOW_literal_in_signed_literal17006);
        	        literal(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesigned_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3154:4: PLUS_OPERATOR ulong_number
        	    {
        	         MATCHT(PLUS_OPERATOR, &FOLLOW_PLUS_OPERATOR_in_signed_literal17011);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesigned_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_signed_literal17013);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesigned_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:3155:4: MINUS_OPERATOR ulong_number
        	    {
        	         MATCHT(MINUS_OPERATOR, &FOLLOW_MINUS_OPERATOR_in_signed_literal17018);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesigned_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        FOLLOWPUSH(FOLLOW_ulong_number_in_signed_literal17020);
        	        ulong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesigned_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulesigned_literalEx; /* Prevent compiler warnings */
    rulesigned_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end signed_literal */

/**
 * $ANTLR start string_literal
 * MySQLSimpleParser.g:3160:1: string_literal : string ;
 */
static void
string_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3160:15: ( string )
        // MySQLSimpleParser.g:3161:2: string
        {
            FOLLOWPUSH(FOLLOW_string_in_string_literal17031);
            string(ctx);

            FOLLOWPOP();
            if  (HASEXCEPTION())
            {
                goto rulestring_literalEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulestring_literalEx; /* Prevent compiler warnings */
    rulestring_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end string_literal */

/**
 * $ANTLR start string
 * MySQLSimpleParser.g:3164:1: string : ( NCHAR_TEXT | ( UNDERSCORE_CHARSET )? ( options {greedy=true; } : SINGLE_QUOTED_TEXT |{...}? DOUBLE_QUOTED_TEXT )+ );
 */
static void
string(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3164:7: ( NCHAR_TEXT | ( UNDERSCORE_CHARSET )? ( options {greedy=true; } : SINGLE_QUOTED_TEXT |{...}? DOUBLE_QUOTED_TEXT )+ )

            ANTLR3_UINT32 alt948;

            alt948=2;

            switch ( LA(1) )
            {
            case NCHAR_TEXT:
            	{
            		alt948=1;
            	}
                break;
            case SINGLE_QUOTED_TEXT:
            case DOUBLE_QUOTED_TEXT:
            case UNDERSCORE_CHARSET:
            	{
            		alt948=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 948;
                EXCEPTION->state        = 0;


                goto rulestringEx;

            }

            switch (alt948)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3165:2: NCHAR_TEXT
        	    {
        	         MATCHT(NCHAR_TEXT, &FOLLOW_NCHAR_TEXT_in_string17040);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulestringEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3166:4: ( UNDERSCORE_CHARSET )? ( options {greedy=true; } : SINGLE_QUOTED_TEXT |{...}? DOUBLE_QUOTED_TEXT )+
        	    {
        	        // MySQLSimpleParser.g:3166:4: ( UNDERSCORE_CHARSET )?
        	        {
        	            int alt946=2;
        	            switch ( LA(1) )
        	            {
        	                case UNDERSCORE_CHARSET:
        	                	{
        	                		alt946=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt946)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:3166:4: UNDERSCORE_CHARSET
        	        	    {
        	        	         MATCHT(UNDERSCORE_CHARSET, &FOLLOW_UNDERSCORE_CHARSET_in_string17045);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulestringEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	        // MySQLSimpleParser.g:3166:24: ( options {greedy=true; } : SINGLE_QUOTED_TEXT |{...}? DOUBLE_QUOTED_TEXT )+
        	        {
        	            int cnt947=0;

        	            for (;;)
        	            {
        	                int alt947=3;
        	        	switch ( LA(1) )
        	        	{
        	        	case DOUBLE_QUOTED_TEXT:
        	        		{
        	        			{
        	        			   /* dfaLoopbackState(k,edges,eotPredictsAlt,description,stateNumber,semPredState)
        	        			    */
        	        			    int LA947_70 = LA(2);
        	        			    if ( ((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))))
        	        			    {
        	        			        alt947=2;
        	        			    }

        	        			}
        	        		}
        	        	    break;
        	        	case SINGLE_QUOTED_TEXT:
        	        		{
        	        			alt947=1;
        	        		}
        	        	    break;

        	        	}

        	        	switch (alt947)
        	        	{
        	        	    case 1:
        	        	        // MySQLSimpleParser.g:3166:54: SINGLE_QUOTED_TEXT
        	        	        {
        	        	             MATCHT(SINGLE_QUOTED_TEXT, &FOLLOW_SINGLE_QUOTED_TEXT_in_string17062);
        	        	            if  (HASEXCEPTION())
        	        	            {
        	        	                goto rulestringEx;
        	        	            }
        	        	            if (HASFAILED())
        	        	            {
        	        	                return ;
        	        	            }


        	        	        }
        	        	        break;
        	        	    case 2:
        	        	        // MySQLSimpleParser.g:3166:75: {...}? DOUBLE_QUOTED_TEXT
        	        	        {
        	        	            if ( !((!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES))) )
        	        	            {
        	        	                if (BACKTRACKING>0)
        	        	                {
        	        	                    FAILEDFLAG = ANTLR3_TRUE;
        	        	                    return ;
        	        	                }

        	        	                    CONSTRUCTEX();
        	        	                    EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	        	                    EXCEPTION->message      = (void *)"!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)";
        	        	                    EXCEPTION->ruleName	 = (void *)"string";


        	        	            }

        	        	             MATCHT(DOUBLE_QUOTED_TEXT, &FOLLOW_DOUBLE_QUOTED_TEXT_in_string17068);
        	        	            if  (HASEXCEPTION())
        	        	            {
        	        	                goto rulestringEx;
        	        	            }
        	        	            if (HASFAILED())
        	        	            {
        	        	                return ;
        	        	            }


        	        	        }
        	        	        break;

        	        	    default:

        	        		if ( cnt947 >= 1 )
        	        		{
        	        		    goto loop947;
        	        		}
        	        		if (BACKTRACKING>0)
        	        		{
        	        		    FAILEDFLAG = ANTLR3_TRUE;
        	        		    return ;
        	        		}

        	        		/* mismatchedSetEx()
        	        		 */
        	        		CONSTRUCTEX();
        	        		EXCEPTION->type = ANTLR3_EARLY_EXIT_EXCEPTION;
        	        		EXCEPTION->name = (void *)ANTLR3_EARLY_EXIT_NAME;


        	        		goto rulestringEx;
        	        	}
        	        	cnt947++;
        	            }
        	            loop947: ;	/* Jump to here if this rule does not match */
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulestringEx; /* Prevent compiler warnings */
    rulestringEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end string */

/**
 * $ANTLR start num_literal
 * MySQLSimpleParser.g:3169:1: num_literal : ( INT_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER );
 */
static void
num_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3169:12: ( INT_NUMBER | LONG_NUMBER | ULONGLONG_NUMBER | DECIMAL_NUMBER | FLOAT_NUMBER )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == DECIMAL_NUMBER || LA(1) == FLOAT_NUMBER || LA(1) == INT_NUMBER || LA(1) == LONG_NUMBER || LA(1) == ULONGLONG_NUMBER )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulenum_literalEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulenum_literalEx; /* Prevent compiler warnings */
    rulenum_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end num_literal */

/**
 * $ANTLR start bool_literal
 * MySQLSimpleParser.g:3177:1: bool_literal : ( TRUE_SYMBOL | FALSE_SYMBOL );
 */
static void
bool_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3177:13: ( TRUE_SYMBOL | FALSE_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == FALSE_SYMBOL || LA(1) == TRUE_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulebool_literalEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulebool_literalEx; /* Prevent compiler warnings */
    rulebool_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end bool_literal */

/**
 * $ANTLR start null_literal
 * MySQLSimpleParser.g:3182:1: null_literal : ( NULL_SYMBOL | NULL2_SYMBOL );
 */
static void
null_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3182:13: ( NULL_SYMBOL | NULL2_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= NULL2_SYMBOL) && (LA(1) <= NULL_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulenull_literalEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulenull_literalEx; /* Prevent compiler warnings */
    rulenull_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end null_literal */

/**
 * $ANTLR start temporal_literal
 * MySQLSimpleParser.g:3187:1: temporal_literal : ( DATE_SYMBOL SINGLE_QUOTED_TEXT | TIME_SYMBOL SINGLE_QUOTED_TEXT | TIMESTAMP_SYMBOL SINGLE_QUOTED_TEXT );
 */
static void
temporal_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3187:17: ( DATE_SYMBOL SINGLE_QUOTED_TEXT | TIME_SYMBOL SINGLE_QUOTED_TEXT | TIMESTAMP_SYMBOL SINGLE_QUOTED_TEXT )

            ANTLR3_UINT32 alt949;

            alt949=3;

            switch ( LA(1) )
            {
            case DATE_SYMBOL:
            	{
            		alt949=1;
            	}
                break;
            case TIME_SYMBOL:
            	{
            		alt949=2;
            	}
                break;
            case TIMESTAMP_SYMBOL:
            	{
            		alt949=3;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 949;
                EXCEPTION->state        = 0;


                goto ruletemporal_literalEx;

            }

            switch (alt949)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3188:2: DATE_SYMBOL SINGLE_QUOTED_TEXT
        	    {
        	         MATCHT(DATE_SYMBOL, &FOLLOW_DATE_SYMBOL_in_temporal_literal17137);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SINGLE_QUOTED_TEXT, &FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17139);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3189:4: TIME_SYMBOL SINGLE_QUOTED_TEXT
        	    {
        	         MATCHT(TIME_SYMBOL, &FOLLOW_TIME_SYMBOL_in_temporal_literal17144);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SINGLE_QUOTED_TEXT, &FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17146);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:3190:4: TIMESTAMP_SYMBOL SINGLE_QUOTED_TEXT
        	    {
        	         MATCHT(TIMESTAMP_SYMBOL, &FOLLOW_TIMESTAMP_SYMBOL_in_temporal_literal17151);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(SINGLE_QUOTED_TEXT, &FOLLOW_SINGLE_QUOTED_TEXT_in_temporal_literal17153);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletemporal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletemporal_literalEx; /* Prevent compiler warnings */
    ruletemporal_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end temporal_literal */

/**
 * $ANTLR start integer_type
 * MySQLSimpleParser.g:3195:1: integer_type : ( INT_SYMBOL | TINYINT_SYMBOL | SMALLINT_SYMBOL | MEDIUMINT_SYMBOL | BIGINT_SYMBOL );
 */
static void
integer_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3195:13: ( INT_SYMBOL | TINYINT_SYMBOL | SMALLINT_SYMBOL | MEDIUMINT_SYMBOL | BIGINT_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == BIGINT_SYMBOL || LA(1) == INT_SYMBOL || LA(1) == MEDIUMINT_SYMBOL || LA(1) == SMALLINT_SYMBOL || LA(1) == TINYINT_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleinteger_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleinteger_typeEx; /* Prevent compiler warnings */
    ruleinteger_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end integer_type */

/**
 * $ANTLR start real_literal
 * MySQLSimpleParser.g:3203:1: real_literal : ( REAL_SYMBOL | DOUBLE_SYMBOL ( PRECISION_SYMBOL )? );
 */
static void
real_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3203:13: ( REAL_SYMBOL | DOUBLE_SYMBOL ( PRECISION_SYMBOL )? )

            ANTLR3_UINT32 alt951;

            alt951=2;

            switch ( LA(1) )
            {
            case REAL_SYMBOL:
            	{
            		alt951=1;
            	}
                break;
            case DOUBLE_SYMBOL:
            	{
            		alt951=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 951;
                EXCEPTION->state        = 0;


                goto rulereal_literalEx;

            }

            switch (alt951)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3204:2: REAL_SYMBOL
        	    {
        	         MATCHT(REAL_SYMBOL, &FOLLOW_REAL_SYMBOL_in_real_literal17193);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3204:16: DOUBLE_SYMBOL ( PRECISION_SYMBOL )?
        	    {
        	         MATCHT(DOUBLE_SYMBOL, &FOLLOW_DOUBLE_SYMBOL_in_real_literal17197);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulereal_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	        // MySQLSimpleParser.g:3204:30: ( PRECISION_SYMBOL )?
        	        {
        	            int alt950=2;
        	            switch ( LA(1) )
        	            {
        	                case PRECISION_SYMBOL:
        	                	{
        	                		alt950=1;
        	                	}
        	                    break;
        	            }

        	            switch (alt950)
        	            {
        	        	case 1:
        	        	    // MySQLSimpleParser.g:3204:30: PRECISION_SYMBOL
        	        	    {
        	        	         MATCHT(PRECISION_SYMBOL, &FOLLOW_PRECISION_SYMBOL_in_real_literal17199);
        	        	        if  (HASEXCEPTION())
        	        	        {
        	        	            goto rulereal_literalEx;
        	        	        }
        	        	        if (HASFAILED())
        	        	        {
        	        	            return ;
        	        	        }


        	        	    }
        	        	    break;

        	            }
        	        }

        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulereal_literalEx; /* Prevent compiler warnings */
    rulereal_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end real_literal */

/**
 * $ANTLR start float_options
 * MySQLSimpleParser.g:3207:1: float_options : OPEN_PAR_SYMBOL INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? CLOSE_PAR_SYMBOL ;
 */
static void
float_options(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3207:14: ( OPEN_PAR_SYMBOL INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:3209:2: OPEN_PAR_SYMBOL INT_NUMBER ( COMMA_SYMBOL INT_NUMBER )? CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_float_options17211);
            if  (HASEXCEPTION())
            {
                goto rulefloat_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_float_options17213);
            if  (HASEXCEPTION())
            {
                goto rulefloat_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


            // MySQLSimpleParser.g:3209:29: ( COMMA_SYMBOL INT_NUMBER )?
            {
                int alt952=2;
                switch ( LA(1) )
                {
                    case COMMA_SYMBOL:
                    	{
                    		alt952=1;
                    	}
                        break;
                }

                switch (alt952)
                {
            	case 1:
            	    // MySQLSimpleParser.g:3209:30: COMMA_SYMBOL INT_NUMBER
            	    {
            	         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_float_options17216);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefloat_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	         MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_float_options17218);
            	        if  (HASEXCEPTION())
            	        {
            	            goto rulefloat_optionsEx;
            	        }
            	        if (HASFAILED())
            	        {
            	            return ;
            	        }


            	    }
            	    break;

                }
            }

             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_float_options17222);
            if  (HASEXCEPTION())
            {
                goto rulefloat_optionsEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulefloat_optionsEx; /* Prevent compiler warnings */
    rulefloat_optionsEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end float_options */

/**
 * $ANTLR start precision
 * MySQLSimpleParser.g:3212:1: precision : OPEN_PAR_SYMBOL INT_NUMBER COMMA_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL ;
 */
static void
precision(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3212:10: ( OPEN_PAR_SYMBOL INT_NUMBER COMMA_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:3213:2: OPEN_PAR_SYMBOL INT_NUMBER COMMA_SYMBOL INT_NUMBER CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_precision17231);
            if  (HASEXCEPTION())
            {
                goto ruleprecisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_precision17233);
            if  (HASEXCEPTION())
            {
                goto ruleprecisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_precision17235);
            if  (HASEXCEPTION())
            {
                goto ruleprecisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(INT_NUMBER, &FOLLOW_INT_NUMBER_in_precision17237);
            if  (HASEXCEPTION())
            {
                goto ruleprecisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_precision17239);
            if  (HASEXCEPTION())
            {
                goto ruleprecisionEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleprecisionEx; /* Prevent compiler warnings */
    ruleprecisionEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end precision */

/**
 * $ANTLR start nchar_literal
 * MySQLSimpleParser.g:3216:1: nchar_literal : ( NCHAR_SYMBOL |{...}? NATIONAL_SYMBOL CHAR_SYMBOL );
 */
static void
nchar_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3216:14: ( NCHAR_SYMBOL |{...}? NATIONAL_SYMBOL CHAR_SYMBOL )

            ANTLR3_UINT32 alt953;

            alt953=2;

            switch ( LA(1) )
            {
            case NCHAR_SYMBOL:
            	{
            		alt953=1;
            	}
                break;
            case NATIONAL_SYMBOL:
            	{
            		alt953=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 953;
                EXCEPTION->state        = 0;


                goto rulenchar_literalEx;

            }

            switch (alt953)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3217:2: NCHAR_SYMBOL
        	    {
        	         MATCHT(NCHAR_SYMBOL, &FOLLOW_NCHAR_SYMBOL_in_nchar_literal17248);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3218:4: {...}? NATIONAL_SYMBOL CHAR_SYMBOL
        	    {
        	        if ( !((LA(3) != VARYING_SYMBOL)) )
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	                CONSTRUCTEX();
        	                EXCEPTION->type         = ANTLR3_FAILED_PREDICATE_EXCEPTION;
        	                EXCEPTION->message      = (void *)"LA(3) != VARYING_SYMBOL";
        	                EXCEPTION->ruleName	 = (void *)"nchar_literal";


        	        }

        	         MATCHT(NATIONAL_SYMBOL, &FOLLOW_NATIONAL_SYMBOL_in_nchar_literal17255);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_nchar_literal17257);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulenchar_literalEx; /* Prevent compiler warnings */
    rulenchar_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end nchar_literal */

/**
 * $ANTLR start varchar_literal
 * MySQLSimpleParser.g:3221:1: varchar_literal : ( CHAR_SYMBOL VARYING_SYMBOL | VARCHAR_SYMBOL );
 */
static void
varchar_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3221:16: ( CHAR_SYMBOL VARYING_SYMBOL | VARCHAR_SYMBOL )

            ANTLR3_UINT32 alt954;

            alt954=2;

            switch ( LA(1) )
            {
            case CHAR_SYMBOL:
            	{
            		alt954=1;
            	}
                break;
            case VARCHAR_SYMBOL:
            	{
            		alt954=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 954;
                EXCEPTION->state        = 0;


                goto rulevarchar_literalEx;

            }

            switch (alt954)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3222:2: CHAR_SYMBOL VARYING_SYMBOL
        	    {
        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_varchar_literal17267);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARYING_SYMBOL, &FOLLOW_VARYING_SYMBOL_in_varchar_literal17269);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3223:4: VARCHAR_SYMBOL
        	    {
        	         MATCHT(VARCHAR_SYMBOL, &FOLLOW_VARCHAR_SYMBOL_in_varchar_literal17274);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulevarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulevarchar_literalEx; /* Prevent compiler warnings */
    rulevarchar_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end varchar_literal */

/**
 * $ANTLR start nvarchar_literal
 * MySQLSimpleParser.g:3226:1: nvarchar_literal : ( NATIONAL_SYMBOL VARCHAR_SYMBOL | NVARCHAR_SYMBOL | NCHAR_SYMBOL VARCHAR_SYMBOL | NATIONAL_SYMBOL CHAR_SYMBOL VARYING_SYMBOL | NCHAR_SYMBOL VARYING_SYMBOL );
 */
static void
nvarchar_literal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3226:17: ( NATIONAL_SYMBOL VARCHAR_SYMBOL | NVARCHAR_SYMBOL | NCHAR_SYMBOL VARCHAR_SYMBOL | NATIONAL_SYMBOL CHAR_SYMBOL VARYING_SYMBOL | NCHAR_SYMBOL VARYING_SYMBOL )

            ANTLR3_UINT32 alt955;

            alt955=5;

            switch ( LA(1) )
            {
            case NATIONAL_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case VARCHAR_SYMBOL:
            			{
            				alt955=1;
            			}
            		    break;
            		case CHAR_SYMBOL:
            			{
            				alt955=4;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 955;
            		    EXCEPTION->state        = 1;


            		    goto rulenvarchar_literalEx;

            		}

            	}
                break;
            case NVARCHAR_SYMBOL:
            	{
            		alt955=2;
            	}
                break;
            case NCHAR_SYMBOL:
            	{
            		switch ( LA(2) )
            		{
            		case VARCHAR_SYMBOL:
            			{
            				alt955=3;
            			}
            		    break;
            		case VARYING_SYMBOL:
            			{
            				alt955=5;
            			}
            		    break;

            		default:
            		    if (BACKTRACKING>0)
            		    {
            		        FAILEDFLAG = ANTLR3_TRUE;
            		        return ;
            		    }

            		    CONSTRUCTEX();
            		    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
            		    EXCEPTION->message      = (void *)"";
            		    EXCEPTION->decisionNum  = 955;
            		    EXCEPTION->state        = 3;


            		    goto rulenvarchar_literalEx;

            		}

            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 955;
                EXCEPTION->state        = 0;


                goto rulenvarchar_literalEx;

            }

            switch (alt955)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3227:2: NATIONAL_SYMBOL VARCHAR_SYMBOL
        	    {
        	         MATCHT(NATIONAL_SYMBOL, &FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17283);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARCHAR_SYMBOL, &FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17285);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3228:4: NVARCHAR_SYMBOL
        	    {
        	         MATCHT(NVARCHAR_SYMBOL, &FOLLOW_NVARCHAR_SYMBOL_in_nvarchar_literal17290);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 3:
        	    // MySQLSimpleParser.g:3229:4: NCHAR_SYMBOL VARCHAR_SYMBOL
        	    {
        	         MATCHT(NCHAR_SYMBOL, &FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17295);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARCHAR_SYMBOL, &FOLLOW_VARCHAR_SYMBOL_in_nvarchar_literal17297);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 4:
        	    // MySQLSimpleParser.g:3230:4: NATIONAL_SYMBOL CHAR_SYMBOL VARYING_SYMBOL
        	    {
        	         MATCHT(NATIONAL_SYMBOL, &FOLLOW_NATIONAL_SYMBOL_in_nvarchar_literal17302);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(CHAR_SYMBOL, &FOLLOW_CHAR_SYMBOL_in_nvarchar_literal17304);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARYING_SYMBOL, &FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17306);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 5:
        	    // MySQLSimpleParser.g:3231:4: NCHAR_SYMBOL VARYING_SYMBOL
        	    {
        	         MATCHT(NCHAR_SYMBOL, &FOLLOW_NCHAR_SYMBOL_in_nvarchar_literal17311);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	         MATCHT(VARYING_SYMBOL, &FOLLOW_VARYING_SYMBOL_in_nvarchar_literal17313);
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulenvarchar_literalEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulenvarchar_literalEx; /* Prevent compiler warnings */
    rulenvarchar_literalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end nvarchar_literal */

/**
 * $ANTLR start text_or_identifier
 * MySQLSimpleParser.g:3234:1: text_or_identifier : ( SINGLE_QUOTED_TEXT | identifier );
 */
static void
text_or_identifier(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3234:19: ( SINGLE_QUOTED_TEXT | identifier )

            ANTLR3_UINT32 alt956;

            alt956=2;

            {
                int LA956_0 = LA(1);
                if ( (LA956_0 == SINGLE_QUOTED_TEXT))
                {
                    alt956=1;
                }
                else if ( (((LA956_0 >= ACCOUNT_SYMBOL) && (LA956_0 <= YEAR_SYMBOL)) || LA956_0 == BACK_TICK_QUOTED_ID || LA956_0 == DOUBLE_QUOTED_TEXT || LA956_0 == IDENTIFIER))
                {
                    alt956=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 956;
                    EXCEPTION->state        = 0;


                    goto ruletext_or_identifierEx;

                }
            }
            switch (alt956)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3235:2: SINGLE_QUOTED_TEXT
        	    {
        	         MATCHT(SINGLE_QUOTED_TEXT, &FOLLOW_SINGLE_QUOTED_TEXT_in_text_or_identifier17322);
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletext_or_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3236:4: identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_identifier_in_text_or_identifier17327);
        	        identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto ruletext_or_identifierEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto ruletext_or_identifierEx; /* Prevent compiler warnings */
    ruletext_or_identifierEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end text_or_identifier */

/**
 * $ANTLR start size_number
 * MySQLSimpleParser.g:3240:1: size_number : ( real_ulonglong_number | pure_identifier );
 */
static void
size_number(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3240:12: ( real_ulonglong_number | pure_identifier )

            ANTLR3_UINT32 alt957;

            alt957=2;

            switch ( LA(1) )
            {
            case INT_NUMBER:
            case LONG_NUMBER:
            case ULONGLONG_NUMBER:
            	{
            		alt957=1;
            	}
                break;
            case BACK_TICK_QUOTED_ID:
            case DOUBLE_QUOTED_TEXT:
            case IDENTIFIER:
            	{
            		alt957=2;
            	}
                break;

            default:
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                EXCEPTION->message      = (void *)"";
                EXCEPTION->decisionNum  = 957;
                EXCEPTION->state        = 0;


                goto rulesize_numberEx;

            }

            switch (alt957)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3241:2: real_ulonglong_number
        	    {
        	        FOLLOWPUSH(FOLLOW_real_ulonglong_number_in_size_number17338);
        	        real_ulonglong_number(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesize_numberEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3242:4: pure_identifier
        	    {
        	        FOLLOWPUSH(FOLLOW_pure_identifier_in_size_number17343);
        	        pure_identifier(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulesize_numberEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulesize_numberEx; /* Prevent compiler warnings */
    rulesize_numberEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end size_number */

/**
 * $ANTLR start parentheses
 * MySQLSimpleParser.g:3245:1: parentheses : OPEN_PAR_SYMBOL CLOSE_PAR_SYMBOL ;
 */
static void
parentheses(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3245:12: ( OPEN_PAR_SYMBOL CLOSE_PAR_SYMBOL )
        // MySQLSimpleParser.g:3246:2: OPEN_PAR_SYMBOL CLOSE_PAR_SYMBOL
        {
             MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_parentheses17353);
            if  (HASEXCEPTION())
            {
                goto ruleparenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


             MATCHT(CLOSE_PAR_SYMBOL, &FOLLOW_CLOSE_PAR_SYMBOL_in_parentheses17355);
            if  (HASEXCEPTION())
            {
                goto ruleparenthesesEx;
            }
            if (HASFAILED())
            {
                return ;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleparenthesesEx; /* Prevent compiler warnings */
    ruleparenthesesEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end parentheses */

/**
 * $ANTLR start equal
 * MySQLSimpleParser.g:3249:1: equal : ( EQUAL_OPERATOR | ASSIGN_OPERATOR );
 */
static void
equal(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3249:6: ( EQUAL_OPERATOR | ASSIGN_OPERATOR )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == ASSIGN_OPERATOR || LA(1) == EQUAL_OPERATOR )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleequalEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleequalEx; /* Prevent compiler warnings */
    ruleequalEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end equal */

/**
 * $ANTLR start option_type
 * MySQLSimpleParser.g:3253:1: option_type : ( GLOBAL_SYMBOL | LOCAL_SYMBOL | SESSION_SYMBOL );
 */
static void
option_type(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3253:12: ( GLOBAL_SYMBOL | LOCAL_SYMBOL | SESSION_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( LA(1) == GLOBAL_SYMBOL || LA(1) == LOCAL_SYMBOL || LA(1) == SESSION_SYMBOL )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto ruleoption_typeEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto ruleoption_typeEx; /* Prevent compiler warnings */
    ruleoption_typeEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end option_type */

/**
 * $ANTLR start keyword
 * MySQLSimpleParser.g:3260:1: keyword : ( keyword_sp | ( ACCOUNT_SYMBOL | ASCII_SYMBOL | ALWAYS_SYMBOL | BACKUP_SYMBOL | BEGIN_SYMBOL | BYTE_SYMBOL | CACHE_SYMBOL | CHARSET_SYMBOL | CHECKSUM_SYMBOL | CLOSE_SYMBOL | COMMENT_SYMBOL | COMMIT_SYMBOL | CONTAINS_SYMBOL | DEALLOCATE_SYMBOL | DO_SYMBOL | END_SYMBOL | EXECUTE_SYMBOL | FLUSH_SYMBOL | FOLLOWS_SYMBOL | FORMAT_SYMBOL | GROUP_REPLICATION_SYMBOL | HANDLER_SYMBOL | HELP_SYMBOL | HOST_SYMBOL | INSTALL_SYMBOL | LANGUAGE_SYMBOL | NO_SYMBOL | OPEN_SYMBOL | OPTIONS_SYMBOL | OWNER_SYMBOL | PARSER_SYMBOL | PARTITION_SYMBOL | PORT_SYMBOL | PRECEDES_SYMBOL | PREPARE_SYMBOL | REMOVE_SYMBOL | REPAIR_SYMBOL | RESET_SYMBOL | RESTORE_SYMBOL | ROLLBACK_SYMBOL | SAVEPOINT_SYMBOL | SECURITY_SYMBOL | SERVER_SYMBOL | SHUTDOWN_SYMBOL | SIGNED_SYMBOL | SLAVE_SYMBOL | SOCKET_SYMBOL | SONAME_SYMBOL | START_SYMBOL | STOP_SYMBOL | TRUNCATE_SYMBOL | UNICODE_SYMBOL | UNINSTALL_SYMBOL | UPGRADE_SYMBOL | WRAPPER_SYMBOL | XA_SYMBOL ) );
 */
static void
keyword(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        {
            //  MySQLSimpleParser.g:3260:8: ( keyword_sp | ( ACCOUNT_SYMBOL | ASCII_SYMBOL | ALWAYS_SYMBOL | BACKUP_SYMBOL | BEGIN_SYMBOL | BYTE_SYMBOL | CACHE_SYMBOL | CHARSET_SYMBOL | CHECKSUM_SYMBOL | CLOSE_SYMBOL | COMMENT_SYMBOL | COMMIT_SYMBOL | CONTAINS_SYMBOL | DEALLOCATE_SYMBOL | DO_SYMBOL | END_SYMBOL | EXECUTE_SYMBOL | FLUSH_SYMBOL | FOLLOWS_SYMBOL | FORMAT_SYMBOL | GROUP_REPLICATION_SYMBOL | HANDLER_SYMBOL | HELP_SYMBOL | HOST_SYMBOL | INSTALL_SYMBOL | LANGUAGE_SYMBOL | NO_SYMBOL | OPEN_SYMBOL | OPTIONS_SYMBOL | OWNER_SYMBOL | PARSER_SYMBOL | PARTITION_SYMBOL | PORT_SYMBOL | PRECEDES_SYMBOL | PREPARE_SYMBOL | REMOVE_SYMBOL | REPAIR_SYMBOL | RESET_SYMBOL | RESTORE_SYMBOL | ROLLBACK_SYMBOL | SAVEPOINT_SYMBOL | SECURITY_SYMBOL | SERVER_SYMBOL | SHUTDOWN_SYMBOL | SIGNED_SYMBOL | SLAVE_SYMBOL | SOCKET_SYMBOL | SONAME_SYMBOL | START_SYMBOL | STOP_SYMBOL | TRUNCATE_SYMBOL | UNICODE_SYMBOL | UNINSTALL_SYMBOL | UPGRADE_SYMBOL | WRAPPER_SYMBOL | XA_SYMBOL ) )

            ANTLR3_UINT32 alt958;

            alt958=2;

            {
                int LA958_0 = LA(1);
                if ( (((LA958_0 >= ACTION_SYMBOL) && (LA958_0 <= SHARE_SYMBOL)) || ((LA958_0 >= SLOW_SYMBOL) && (LA958_0 <= YEAR_SYMBOL))))
                {
                    alt958=1;
                }
                else if ( (((LA958_0 >= ACCOUNT_SYMBOL) && (LA958_0 <= XA_SYMBOL)) || LA958_0 == SHUTDOWN_SYMBOL))
                {
                    alt958=2;
                }
                else
                {
                    if (BACKTRACKING>0)
                    {
                        FAILEDFLAG = ANTLR3_TRUE;
                        return ;
                    }


                    CONSTRUCTEX();
                    EXCEPTION->type         = ANTLR3_NO_VIABLE_ALT_EXCEPTION;
                    EXCEPTION->message      = (void *)"";
                    EXCEPTION->decisionNum  = 958;
                    EXCEPTION->state        = 0;


                    goto rulekeywordEx;

                }
            }
            switch (alt958)
            {
        	case 1:
        	    // MySQLSimpleParser.g:3261:2: keyword_sp
        	    {
        	        FOLLOWPUSH(FOLLOW_keyword_sp_in_keyword17397);
        	        keyword_sp(ctx);

        	        FOLLOWPOP();
        	        if  (HASEXCEPTION())
        	        {
        	            goto rulekeywordEx;
        	        }
        	        if (HASFAILED())
        	        {
        	            return ;
        	        }


        	    }
        	    break;
        	case 2:
        	    // MySQLSimpleParser.g:3262:4: ( ACCOUNT_SYMBOL | ASCII_SYMBOL | ALWAYS_SYMBOL | BACKUP_SYMBOL | BEGIN_SYMBOL | BYTE_SYMBOL | CACHE_SYMBOL | CHARSET_SYMBOL | CHECKSUM_SYMBOL | CLOSE_SYMBOL | COMMENT_SYMBOL | COMMIT_SYMBOL | CONTAINS_SYMBOL | DEALLOCATE_SYMBOL | DO_SYMBOL | END_SYMBOL | EXECUTE_SYMBOL | FLUSH_SYMBOL | FOLLOWS_SYMBOL | FORMAT_SYMBOL | GROUP_REPLICATION_SYMBOL | HANDLER_SYMBOL | HELP_SYMBOL | HOST_SYMBOL | INSTALL_SYMBOL | LANGUAGE_SYMBOL | NO_SYMBOL | OPEN_SYMBOL | OPTIONS_SYMBOL | OWNER_SYMBOL | PARSER_SYMBOL | PARTITION_SYMBOL | PORT_SYMBOL | PRECEDES_SYMBOL | PREPARE_SYMBOL | REMOVE_SYMBOL | REPAIR_SYMBOL | RESET_SYMBOL | RESTORE_SYMBOL | ROLLBACK_SYMBOL | SAVEPOINT_SYMBOL | SECURITY_SYMBOL | SERVER_SYMBOL | SHUTDOWN_SYMBOL | SIGNED_SYMBOL | SLAVE_SYMBOL | SOCKET_SYMBOL | SONAME_SYMBOL | START_SYMBOL | STOP_SYMBOL | TRUNCATE_SYMBOL | UNICODE_SYMBOL | UNINSTALL_SYMBOL | UPGRADE_SYMBOL | WRAPPER_SYMBOL | XA_SYMBOL )
        	    {
        	        if ( ((LA(1) >= ACCOUNT_SYMBOL) && (LA(1) <= XA_SYMBOL)) || LA(1) == SHUTDOWN_SYMBOL )
        	        {
        	            CONSUME();
        	            PERRORRECOVERY=ANTLR3_FALSE;
        	            FAILEDFLAG=ANTLR3_FALSE;

        	        }
        	        else
        	        {
        	            if (BACKTRACKING>0)
        	            {
        	                FAILEDFLAG = ANTLR3_TRUE;
        	                return ;
        	            }

        	            CONSTRUCTEX();
        	            EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
        	            EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
        	            EXCEPTION->expectingSet = NULL;

        	            goto rulekeywordEx;
        	        }


        	    }
        	    break;

            }
        }
    }

    // This is where rules clean up and exit
    //
    goto rulekeywordEx; /* Prevent compiler warnings */
    rulekeywordEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end keyword */

/**
 * $ANTLR start keyword_sp
 * MySQLSimpleParser.g:3329:1: keyword_sp : ( ACTION_SYMBOL | ADDDATE_SYMBOL | AFTER_SYMBOL | AGAINST_SYMBOL | AGGREGATE_SYMBOL | ALGORITHM_SYMBOL | ANALYZE_SYMBOL | ANY_SYMBOL | AT_SYMBOL | AUTHORS_SYMBOL | AUTO_INCREMENT_SYMBOL | AUTOEXTEND_SIZE_SYMBOL | AVG_ROW_LENGTH_SYMBOL | AVG_SYMBOL | BINLOG_SYMBOL | BIT_SYMBOL | BLOCK_SYMBOL | BOOL_SYMBOL | BOOLEAN_SYMBOL | BTREE_SYMBOL | CASCADED_SYMBOL | CATALOG_NAME_SYMBOL | CHAIN_SYMBOL | CHANGED_SYMBOL | CHANNEL_SYMBOL | CIPHER_SYMBOL | CLIENT_SYMBOL | CLASS_ORIGIN_SYMBOL | COALESCE_SYMBOL | CODE_SYMBOL | COLLATION_SYMBOL | COLUMN_NAME_SYMBOL | COLUMN_FORMAT_SYMBOL | COLUMNS_SYMBOL | COMMITTED_SYMBOL | COMPACT_SYMBOL | COMPLETION_SYMBOL | COMPRESSED_SYMBOL | COMPRESSION_SYMBOL | CONCURRENT_SYMBOL | CONNECTION_SYMBOL | CONSISTENT_SYMBOL | CONSTRAINT_CATALOG_SYMBOL | CONSTRAINT_SCHEMA_SYMBOL | CONSTRAINT_NAME_SYMBOL | CONTEXT_SYMBOL | CONTRIBUTORS_SYMBOL | CPU_SYMBOL | CUBE_SYMBOL | CURRENT_SYMBOL | CURSOR_NAME_SYMBOL | DATA_SYMBOL | DATAFILE_SYMBOL | DATETIME_SYMBOL | DATE_SYMBOL | DAY_SYMBOL | DEFAULT_AUTH_SYMBOL | DEFINER_SYMBOL | DELAY_KEY_WRITE_SYMBOL | DES_KEY_FILE_SYMBOL | DIAGNOSTICS_SYMBOL | DIRECTORY_SYMBOL | DISABLE_SYMBOL | DISCARD_SYMBOL | DISK_SYMBOL | DUMPFILE_SYMBOL | DUPLICATE_SYMBOL | DYNAMIC_SYMBOL | ENDS_SYMBOL | ENUM_SYMBOL | ENGINE_SYMBOL | ENGINES_SYMBOL | ERROR_SYMBOL | ERRORS_SYMBOL | ESCAPE_SYMBOL | EVENT_SYMBOL | EVENTS_SYMBOL | EVERY_SYMBOL | EXPANSION_SYMBOL | EXPORT_SYMBOL | EXTENDED_SYMBOL | EXTENT_SIZE_SYMBOL | FAULTS_SYMBOL | FAST_SYMBOL | FOUND_SYMBOL | ENABLE_SYMBOL | FULL_SYMBOL | FILE_SYMBOL | FILE_BLOCK_SIZE_SYMBOL | FILTER_SYMBOL | FIRST_SYMBOL | FIXED_SYMBOL | GENERAL_SYMBOL | GEOMETRY_SYMBOL | GEOMETRYCOLLECTION_SYMBOL | GET_FORMAT_SYMBOL | GRANTS_SYMBOL | GLOBAL_SYMBOL | HASH_SYMBOL | HOSTS_SYMBOL | HOUR_SYMBOL | IDENTIFIED_SYMBOL | IGNORE_SERVER_IDS_SYMBOL | INVOKER_SYMBOL | IMPORT_SYMBOL | INDEXES_SYMBOL | INITIAL_SIZE_SYMBOL | INNODB_SYMBOL | IO_SYMBOL | IPC_SYMBOL | ISOLATION_SYMBOL | ISSUER_SYMBOL | INSERT_METHOD_SYMBOL | JSON_SYMBOL | KEY_BLOCK_SIZE_SYMBOL | LAST_SYMBOL | LEAVES_SYMBOL | LESS_SYMBOL | LEVEL_SYMBOL | LINESTRING_SYMBOL | LIST_SYMBOL | LOCAL_SYMBOL | LOCKS_SYMBOL | LOGFILE_SYMBOL | LOGS_SYMBOL | MAX_ROWS_SYMBOL | MASTER_SYMBOL | MASTER_HEARTBEAT_PERIOD_SYMBOL | MASTER_HOST_SYMBOL | MASTER_PORT_SYMBOL | MASTER_LOG_FILE_SYMBOL | MASTER_LOG_POS_SYMBOL | MASTER_USER_SYMBOL | MASTER_PASSWORD_SYMBOL | MASTER_SERVER_ID_SYMBOL | MASTER_CONNECT_RETRY_SYMBOL | MASTER_RETRY_COUNT_SYMBOL | MASTER_DELAY_SYMBOL | MASTER_SSL_SYMBOL | MASTER_SSL_CA_SYMBOL | MASTER_SSL_CAPATH_SYMBOL | MASTER_SSL_CERT_SYMBOL | MASTER_SSL_CIPHER_SYMBOL | MASTER_SSL_CRL_SYMBOL | MASTER_SSL_CRLPATH_SYMBOL | MASTER_SSL_KEY_SYMBOL | MASTER_AUTO_POSITION_SYMBOL | MAX_CONNECTIONS_PER_HOUR_SYMBOL | MAX_QUERIES_PER_HOUR_SYMBOL | MAX_STATEMENT_TIME_SYMBOL | MAX_SIZE_SYMBOL | MAX_UPDATES_PER_HOUR_SYMBOL | MAX_USER_CONNECTIONS_SYMBOL | MEDIUM_SYMBOL | MEMORY_SYMBOL | MERGE_SYMBOL | MESSAGE_TEXT_SYMBOL | MICROSECOND_SYMBOL | MIGRATE_SYMBOL | MINUTE_SYMBOL | MIN_ROWS_SYMBOL | MODIFY_SYMBOL | MODE_SYMBOL | MONTH_SYMBOL | MULTILINESTRING_SYMBOL | MULTIPOINT_SYMBOL | MULTIPOLYGON_SYMBOL | MUTEX_SYMBOL | MYSQL_ERRNO_SYMBOL | NAME_SYMBOL | NAMES_SYMBOL | NATIONAL_SYMBOL | NCHAR_SYMBOL | NDBCLUSTER_SYMBOL | NEVER_SYMBOL | NEXT_SYMBOL | NEW_SYMBOL | NO_WAIT_SYMBOL | NODEGROUP_SYMBOL | NONE_SYMBOL | NUMBER_SYMBOL | NVARCHAR_SYMBOL | OFFSET_SYMBOL | OLD_PASSWORD_SYMBOL | ONE_SHOT_SYMBOL | ONE_SYMBOL | PACK_KEYS_SYMBOL | PAGE_SYMBOL | PARTIAL_SYMBOL | PARTITIONING_SYMBOL | PARTITIONS_SYMBOL | PASSWORD_SYMBOL | PHASE_SYMBOL | PLUGIN_DIR_SYMBOL | PLUGIN_SYMBOL | PLUGINS_SYMBOL | POINT_SYMBOL | POLYGON_SYMBOL | PRESERVE_SYMBOL | PREV_SYMBOL | PRIVILEGES_SYMBOL | PROCESS_SYMBOL | PROCESSLIST_SYMBOL | PROFILE_SYMBOL | PROFILES_SYMBOL | PROXY_SYMBOL | QUARTER_SYMBOL | QUERY_SYMBOL | QUICK_SYMBOL | READ_ONLY_SYMBOL | REBUILD_SYMBOL | RECOVER_SYMBOL | REDO_BUFFER_SIZE_SYMBOL | REDOFILE_SYMBOL | REDUNDANT_SYMBOL | RELAY_SYMBOL | RELAYLOG_SYMBOL | RELAY_LOG_FILE_SYMBOL | RELAY_LOG_POS_SYMBOL | RELAY_THREAD_SYMBOL | RELOAD_SYMBOL | REORGANIZE_SYMBOL | REPEATABLE_SYMBOL | REPLICATION_SYMBOL | REPLICATE_DO_DB_SYMBOL | REPLICATE_IGNORE_DB_SYMBOL | REPLICATE_DO_TABLE_SYMBOL | REPLICATE_IGNORE_TABLE_SYMBOL | REPLICATE_WILD_DO_TABLE_SYMBOL | REPLICATE_WILD_IGNORE_TABLE_SYMBOL | REPLICATE_REWRITE_DB_SYMBOL | RESUME_SYMBOL | RETURNED_SQLSTATE_SYMBOL | RETURNS_SYMBOL | REVERSE_SYMBOL | ROLLUP_SYMBOL | ROUTINE_SYMBOL | ROWS_SYMBOL | ROW_COUNT_SYMBOL | ROW_FORMAT_SYMBOL | ROW_SYMBOL | RTREE_SYMBOL | SCHEDULE_SYMBOL | SCHEMA_NAME_SYMBOL | SECOND_SYMBOL | SERIAL_SYMBOL | SERIALIZABLE_SYMBOL | SESSION_SYMBOL | SIMPLE_SYMBOL | SHARE_SYMBOL | SLOW_SYMBOL | SNAPSHOT_SYMBOL | SOUNDS_SYMBOL | SOURCE_SYMBOL | SQL_AFTER_GTIDS_SYMBOL | SQL_AFTER_MTS_GAPS_SYMBOL | SQL_BEFORE_GTIDS_SYMBOL | SQL_CACHE_SYMBOL | SQL_BUFFER_RESULT_SYMBOL | SQL_NO_CACHE_SYMBOL | SQL_THREAD_SYMBOL | STACKED_SYMBOL | STARTS_SYMBOL | STATS_AUTO_RECALC_SYMBOL | STATS_PERSISTENT_SYMBOL | STATS_SAMPLE_PAGES_SYMBOL | STATUS_SYMBOL | STORAGE_SYMBOL | STRING_SYMBOL | SUBCLASS_ORIGIN_SYMBOL | SUBDATE_SYMBOL | SUBJECT_SYMBOL | SUBPARTITION_SYMBOL | SUBPARTITIONS_SYMBOL | SUPER_SYMBOL | SUSPEND_SYMBOL | SWAPS_SYMBOL | SWITCHES_SYMBOL | TABLE_NAME_SYMBOL | TABLES_SYMBOL | TABLE_CHECKSUM_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL | TEMPTABLE_SYMBOL | TEXT_SYMBOL | THAN_SYMBOL | TRANSACTION_SYMBOL | TRIGGERS_SYMBOL | TIMESTAMP_SYMBOL | TIMESTAMP_ADD_SYMBOL | TIMESTAMP_DIFF_SYMBOL | TIME_SYMBOL | TYPES_SYMBOL | TYPE_SYMBOL | UDF_RETURNS_SYMBOL | FUNCTION_SYMBOL | UNCOMMITTED_SYMBOL | UNDEFINED_SYMBOL | UNDO_BUFFER_SIZE_SYMBOL | UNDOFILE_SYMBOL | UNKNOWN_SYMBOL | UNTIL_SYMBOL | USER_RESOURCES_SYMBOL | USER_SYMBOL | USE_FRM_SYMBOL | VARIABLES_SYMBOL | VIEW_SYMBOL | VALUE_SYMBOL | WARNINGS_SYMBOL | WAIT_SYMBOL | WEEK_SYMBOL | WORK_SYMBOL | WEIGHT_STRING_SYMBOL | X509_SYMBOL | XID_SYMBOL | XML_SYMBOL | YEAR_SYMBOL );
 */
static void
keyword_sp(pMySQLSimpleParser ctx)
{
    /* Initialize rule variables
     */

    {
        // MySQLSimpleParser.g:3329:11: ( ACTION_SYMBOL | ADDDATE_SYMBOL | AFTER_SYMBOL | AGAINST_SYMBOL | AGGREGATE_SYMBOL | ALGORITHM_SYMBOL | ANALYZE_SYMBOL | ANY_SYMBOL | AT_SYMBOL | AUTHORS_SYMBOL | AUTO_INCREMENT_SYMBOL | AUTOEXTEND_SIZE_SYMBOL | AVG_ROW_LENGTH_SYMBOL | AVG_SYMBOL | BINLOG_SYMBOL | BIT_SYMBOL | BLOCK_SYMBOL | BOOL_SYMBOL | BOOLEAN_SYMBOL | BTREE_SYMBOL | CASCADED_SYMBOL | CATALOG_NAME_SYMBOL | CHAIN_SYMBOL | CHANGED_SYMBOL | CHANNEL_SYMBOL | CIPHER_SYMBOL | CLIENT_SYMBOL | CLASS_ORIGIN_SYMBOL | COALESCE_SYMBOL | CODE_SYMBOL | COLLATION_SYMBOL | COLUMN_NAME_SYMBOL | COLUMN_FORMAT_SYMBOL | COLUMNS_SYMBOL | COMMITTED_SYMBOL | COMPACT_SYMBOL | COMPLETION_SYMBOL | COMPRESSED_SYMBOL | COMPRESSION_SYMBOL | CONCURRENT_SYMBOL | CONNECTION_SYMBOL | CONSISTENT_SYMBOL | CONSTRAINT_CATALOG_SYMBOL | CONSTRAINT_SCHEMA_SYMBOL | CONSTRAINT_NAME_SYMBOL | CONTEXT_SYMBOL | CONTRIBUTORS_SYMBOL | CPU_SYMBOL | CUBE_SYMBOL | CURRENT_SYMBOL | CURSOR_NAME_SYMBOL | DATA_SYMBOL | DATAFILE_SYMBOL | DATETIME_SYMBOL | DATE_SYMBOL | DAY_SYMBOL | DEFAULT_AUTH_SYMBOL | DEFINER_SYMBOL | DELAY_KEY_WRITE_SYMBOL | DES_KEY_FILE_SYMBOL | DIAGNOSTICS_SYMBOL | DIRECTORY_SYMBOL | DISABLE_SYMBOL | DISCARD_SYMBOL | DISK_SYMBOL | DUMPFILE_SYMBOL | DUPLICATE_SYMBOL | DYNAMIC_SYMBOL | ENDS_SYMBOL | ENUM_SYMBOL | ENGINE_SYMBOL | ENGINES_SYMBOL | ERROR_SYMBOL | ERRORS_SYMBOL | ESCAPE_SYMBOL | EVENT_SYMBOL | EVENTS_SYMBOL | EVERY_SYMBOL | EXPANSION_SYMBOL | EXPORT_SYMBOL | EXTENDED_SYMBOL | EXTENT_SIZE_SYMBOL | FAULTS_SYMBOL | FAST_SYMBOL | FOUND_SYMBOL | ENABLE_SYMBOL | FULL_SYMBOL | FILE_SYMBOL | FILE_BLOCK_SIZE_SYMBOL | FILTER_SYMBOL | FIRST_SYMBOL | FIXED_SYMBOL | GENERAL_SYMBOL | GEOMETRY_SYMBOL | GEOMETRYCOLLECTION_SYMBOL | GET_FORMAT_SYMBOL | GRANTS_SYMBOL | GLOBAL_SYMBOL | HASH_SYMBOL | HOSTS_SYMBOL | HOUR_SYMBOL | IDENTIFIED_SYMBOL | IGNORE_SERVER_IDS_SYMBOL | INVOKER_SYMBOL | IMPORT_SYMBOL | INDEXES_SYMBOL | INITIAL_SIZE_SYMBOL | INNODB_SYMBOL | IO_SYMBOL | IPC_SYMBOL | ISOLATION_SYMBOL | ISSUER_SYMBOL | INSERT_METHOD_SYMBOL | JSON_SYMBOL | KEY_BLOCK_SIZE_SYMBOL | LAST_SYMBOL | LEAVES_SYMBOL | LESS_SYMBOL | LEVEL_SYMBOL | LINESTRING_SYMBOL | LIST_SYMBOL | LOCAL_SYMBOL | LOCKS_SYMBOL | LOGFILE_SYMBOL | LOGS_SYMBOL | MAX_ROWS_SYMBOL | MASTER_SYMBOL | MASTER_HEARTBEAT_PERIOD_SYMBOL | MASTER_HOST_SYMBOL | MASTER_PORT_SYMBOL | MASTER_LOG_FILE_SYMBOL | MASTER_LOG_POS_SYMBOL | MASTER_USER_SYMBOL | MASTER_PASSWORD_SYMBOL | MASTER_SERVER_ID_SYMBOL | MASTER_CONNECT_RETRY_SYMBOL | MASTER_RETRY_COUNT_SYMBOL | MASTER_DELAY_SYMBOL | MASTER_SSL_SYMBOL | MASTER_SSL_CA_SYMBOL | MASTER_SSL_CAPATH_SYMBOL | MASTER_SSL_CERT_SYMBOL | MASTER_SSL_CIPHER_SYMBOL | MASTER_SSL_CRL_SYMBOL | MASTER_SSL_CRLPATH_SYMBOL | MASTER_SSL_KEY_SYMBOL | MASTER_AUTO_POSITION_SYMBOL | MAX_CONNECTIONS_PER_HOUR_SYMBOL | MAX_QUERIES_PER_HOUR_SYMBOL | MAX_STATEMENT_TIME_SYMBOL | MAX_SIZE_SYMBOL | MAX_UPDATES_PER_HOUR_SYMBOL | MAX_USER_CONNECTIONS_SYMBOL | MEDIUM_SYMBOL | MEMORY_SYMBOL | MERGE_SYMBOL | MESSAGE_TEXT_SYMBOL | MICROSECOND_SYMBOL | MIGRATE_SYMBOL | MINUTE_SYMBOL | MIN_ROWS_SYMBOL | MODIFY_SYMBOL | MODE_SYMBOL | MONTH_SYMBOL | MULTILINESTRING_SYMBOL | MULTIPOINT_SYMBOL | MULTIPOLYGON_SYMBOL | MUTEX_SYMBOL | MYSQL_ERRNO_SYMBOL | NAME_SYMBOL | NAMES_SYMBOL | NATIONAL_SYMBOL | NCHAR_SYMBOL | NDBCLUSTER_SYMBOL | NEVER_SYMBOL | NEXT_SYMBOL | NEW_SYMBOL | NO_WAIT_SYMBOL | NODEGROUP_SYMBOL | NONE_SYMBOL | NUMBER_SYMBOL | NVARCHAR_SYMBOL | OFFSET_SYMBOL | OLD_PASSWORD_SYMBOL | ONE_SHOT_SYMBOL | ONE_SYMBOL | PACK_KEYS_SYMBOL | PAGE_SYMBOL | PARTIAL_SYMBOL | PARTITIONING_SYMBOL | PARTITIONS_SYMBOL | PASSWORD_SYMBOL | PHASE_SYMBOL | PLUGIN_DIR_SYMBOL | PLUGIN_SYMBOL | PLUGINS_SYMBOL | POINT_SYMBOL | POLYGON_SYMBOL | PRESERVE_SYMBOL | PREV_SYMBOL | PRIVILEGES_SYMBOL | PROCESS_SYMBOL | PROCESSLIST_SYMBOL | PROFILE_SYMBOL | PROFILES_SYMBOL | PROXY_SYMBOL | QUARTER_SYMBOL | QUERY_SYMBOL | QUICK_SYMBOL | READ_ONLY_SYMBOL | REBUILD_SYMBOL | RECOVER_SYMBOL | REDO_BUFFER_SIZE_SYMBOL | REDOFILE_SYMBOL | REDUNDANT_SYMBOL | RELAY_SYMBOL | RELAYLOG_SYMBOL | RELAY_LOG_FILE_SYMBOL | RELAY_LOG_POS_SYMBOL | RELAY_THREAD_SYMBOL | RELOAD_SYMBOL | REORGANIZE_SYMBOL | REPEATABLE_SYMBOL | REPLICATION_SYMBOL | REPLICATE_DO_DB_SYMBOL | REPLICATE_IGNORE_DB_SYMBOL | REPLICATE_DO_TABLE_SYMBOL | REPLICATE_IGNORE_TABLE_SYMBOL | REPLICATE_WILD_DO_TABLE_SYMBOL | REPLICATE_WILD_IGNORE_TABLE_SYMBOL | REPLICATE_REWRITE_DB_SYMBOL | RESUME_SYMBOL | RETURNED_SQLSTATE_SYMBOL | RETURNS_SYMBOL | REVERSE_SYMBOL | ROLLUP_SYMBOL | ROUTINE_SYMBOL | ROWS_SYMBOL | ROW_COUNT_SYMBOL | ROW_FORMAT_SYMBOL | ROW_SYMBOL | RTREE_SYMBOL | SCHEDULE_SYMBOL | SCHEMA_NAME_SYMBOL | SECOND_SYMBOL | SERIAL_SYMBOL | SERIALIZABLE_SYMBOL | SESSION_SYMBOL | SIMPLE_SYMBOL | SHARE_SYMBOL | SLOW_SYMBOL | SNAPSHOT_SYMBOL | SOUNDS_SYMBOL | SOURCE_SYMBOL | SQL_AFTER_GTIDS_SYMBOL | SQL_AFTER_MTS_GAPS_SYMBOL | SQL_BEFORE_GTIDS_SYMBOL | SQL_CACHE_SYMBOL | SQL_BUFFER_RESULT_SYMBOL | SQL_NO_CACHE_SYMBOL | SQL_THREAD_SYMBOL | STACKED_SYMBOL | STARTS_SYMBOL | STATS_AUTO_RECALC_SYMBOL | STATS_PERSISTENT_SYMBOL | STATS_SAMPLE_PAGES_SYMBOL | STATUS_SYMBOL | STORAGE_SYMBOL | STRING_SYMBOL | SUBCLASS_ORIGIN_SYMBOL | SUBDATE_SYMBOL | SUBJECT_SYMBOL | SUBPARTITION_SYMBOL | SUBPARTITIONS_SYMBOL | SUPER_SYMBOL | SUSPEND_SYMBOL | SWAPS_SYMBOL | SWITCHES_SYMBOL | TABLE_NAME_SYMBOL | TABLES_SYMBOL | TABLE_CHECKSUM_SYMBOL | TABLESPACE_SYMBOL | TEMPORARY_SYMBOL | TEMPTABLE_SYMBOL | TEXT_SYMBOL | THAN_SYMBOL | TRANSACTION_SYMBOL | TRIGGERS_SYMBOL | TIMESTAMP_SYMBOL | TIMESTAMP_ADD_SYMBOL | TIMESTAMP_DIFF_SYMBOL | TIME_SYMBOL | TYPES_SYMBOL | TYPE_SYMBOL | UDF_RETURNS_SYMBOL | FUNCTION_SYMBOL | UNCOMMITTED_SYMBOL | UNDEFINED_SYMBOL | UNDO_BUFFER_SIZE_SYMBOL | UNDOFILE_SYMBOL | UNKNOWN_SYMBOL | UNTIL_SYMBOL | USER_RESOURCES_SYMBOL | USER_SYMBOL | USE_FRM_SYMBOL | VARIABLES_SYMBOL | VIEW_SYMBOL | VALUE_SYMBOL | WARNINGS_SYMBOL | WAIT_SYMBOL | WEEK_SYMBOL | WORK_SYMBOL | WEIGHT_STRING_SYMBOL | X509_SYMBOL | XID_SYMBOL | XML_SYMBOL | YEAR_SYMBOL )
        // MySQLSimpleParser.g:
        {
            if ( ((LA(1) >= ACTION_SYMBOL) && (LA(1) <= SHARE_SYMBOL)) || ((LA(1) >= SLOW_SYMBOL) && (LA(1) <= YEAR_SYMBOL)) )
            {
                CONSUME();
                PERRORRECOVERY=ANTLR3_FALSE;
                FAILEDFLAG=ANTLR3_FALSE;

            }
            else
            {
                if (BACKTRACKING>0)
                {
                    FAILEDFLAG = ANTLR3_TRUE;
                    return ;
                }

                CONSTRUCTEX();
                EXCEPTION->type         = ANTLR3_MISMATCHED_SET_EXCEPTION;
                EXCEPTION->name         = (void *)ANTLR3_MISMATCHED_SET_NAME;
                EXCEPTION->expectingSet = NULL;

                goto rulekeyword_spEx;
            }


        }

    }

    // This is where rules clean up and exit
    //
    goto rulekeyword_spEx; /* Prevent compiler warnings */
    rulekeyword_spEx: ;

            if (HASEXCEPTION())
            {
                PREPORTERROR();
                PRECOVER();
            }
    return ;
}
/* $ANTLR end keyword_sp */

// $ANTLR start synpred1_MySQLSimpleParser
static void synpred1_MySQLSimpleParser_fragment(pMySQLSimpleParser ctx )
{
    // MySQLSimpleParser.g:864:2: ( table_wild )
    // MySQLSimpleParser.g:864:3: table_wild
    {
        FOLLOWPUSH(FOLLOW_table_wild_in_synpred1_MySQLSimpleParser4873);
        table_wild(ctx);

        FOLLOWPOP();
        if  (HASEXCEPTION())
        {
            goto rulesynpred1_MySQLSimpleParserEx;
        }
        if (HASFAILED())
        {
            return ;
        }


    }

// This is where rules clean up and exit
//
goto rulesynpred1_MySQLSimpleParserEx; /* Prevent compiler warnings */
rulesynpred1_MySQLSimpleParserEx: ;

}
// $ANTLR end synpred1_MySQLSimpleParser

// $ANTLR start synpred2_MySQLSimpleParser
static void synpred2_MySQLSimpleParser_fragment(pMySQLSimpleParser ctx )
{
    // MySQLSimpleParser.g:1835:4: ( OPEN_PAR_SYMBOL expression COMMA_SYMBOL )
    // MySQLSimpleParser.g:1835:5: OPEN_PAR_SYMBOL expression COMMA_SYMBOL
    {
         MATCHT(OPEN_PAR_SYMBOL, &FOLLOW_OPEN_PAR_SYMBOL_in_synpred2_MySQLSimpleParser10462);
        if  (HASEXCEPTION())
        {
            goto rulesynpred2_MySQLSimpleParserEx;
        }
        if (HASFAILED())
        {
            return ;
        }


        FOLLOWPUSH(FOLLOW_expression_in_synpred2_MySQLSimpleParser10464);
        expression(ctx);

        FOLLOWPOP();
        if  (HASEXCEPTION())
        {
            goto rulesynpred2_MySQLSimpleParserEx;
        }
        if (HASFAILED())
        {
            return ;
        }


         MATCHT(COMMA_SYMBOL, &FOLLOW_COMMA_SYMBOL_in_synpred2_MySQLSimpleParser10466);
        if  (HASEXCEPTION())
        {
            goto rulesynpred2_MySQLSimpleParserEx;
        }
        if (HASFAILED())
        {
            return ;
        }


    }

// This is where rules clean up and exit
//
goto rulesynpred2_MySQLSimpleParserEx; /* Prevent compiler warnings */
rulesynpred2_MySQLSimpleParserEx: ;

}
// $ANTLR end synpred2_MySQLSimpleParser
/* End of parsing rules
 * ==============================================
 */

/* ==============================================
 * Syntactic predicates
 */
static ANTLR3_BOOLEAN synpred2_MySQLSimpleParser(pMySQLSimpleParser ctx)
{
    ANTLR3_MARKER   start;
    ANTLR3_BOOLEAN  success;

    BACKTRACKING++;
    start	= MARK();
    synpred2_MySQLSimpleParser_fragment(ctx);	    // can never throw exception
    success	= !(FAILEDFLAG);
    REWIND(start);
    BACKTRACKING--;
    FAILEDFLAG	= ANTLR3_FALSE;
    return success;
}
static ANTLR3_BOOLEAN synpred1_MySQLSimpleParser(pMySQLSimpleParser ctx)
{
    ANTLR3_MARKER   start;
    ANTLR3_BOOLEAN  success;

    BACKTRACKING++;
    start	= MARK();
    synpred1_MySQLSimpleParser_fragment(ctx);	    // can never throw exception
    success	= !(FAILEDFLAG);
    REWIND(start);
    BACKTRACKING--;
    FAILEDFLAG	= ANTLR3_FALSE;
    return success;
}

/* End of syntactic predicates
 * ==============================================
 */






/* End of code
 * =============================================================================
 */
